#ifdef REV_INFO
static char rcsid[] = "$Id: Clipboard.cc,v 1.2 1992/12/10 16:24:12 jojic Exp $";
#endif
/*****************************************************************************
MODULE:  GoDraw
FILE:    Clipboard.cc
AUTHOR:  Aline Jacomet
DATE:    November 1992

DESCRIPTION:
    See Clipboard.h description.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include <stdio.h>

#include <Go/Go.h>
#include <Go/GoRoot.h>
#include <Go/BGoUIEd.h>

#include "Clipboard.h"


typedef struct _UndoReparentStruct {
    Go*              go;
    Go*              parent;
    Go*              next;
    GoDrawClipboard* cb;
} UndoReparentStruct;

typedef struct _UndoStruct {
    Go*              go;
    GoDrawClipboard* cb;
} UndoStruct;



                          /******************
			   * PUBLIC METHODS *
			   ******************/


 GoDrawClipboard::GoDrawClipboard(GUndoManager* undoMgr)
{
    undo_mgr = undoMgr;
}

void GoDrawClipboard::copy(GoRoot* goroot)
{
    reset_buffer();
    goroot->forEverySelectedSubtree(cb_copy_proc, this);
}

void GoDrawClipboard::paste(GoRoot* goroot)
{
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
    bgoui->cancelGrabbers();
    
    bgoui->unselectAll();
    paste_recur ((Go*) buffer.first(), bgoui->currentParent());
    goroot->flush();

    extern void Move (void*, void*, char*);
    Move (NULL, NULL, NULL);
}


void GoDrawClipboard::cut(GoRoot* goroot)
{
    reset_buffer();
    undo_mgr->openMark();
    goroot->forEverySelectedSubtree(cb_cut_proc, this);
    undo_mgr->closeMark();
}


                          /*******************
			   * PRIVATE METHODS *
			   *******************/


void GoDrawClipboard::reset_buffer()
{
    for (Go* tmp = (Go*)buffer.first(); tmp; tmp = (Go*)buffer.next())
	tmp->destroy();
    buffer.clear();
}

void GoDrawClipboard::paste_recur (Go* go, Go* current_parent)
{
    if (go) {
	paste_recur((Go*)buffer.next(), current_parent);

	Go* cpy = go->dup();
	cpy->reparent(current_parent);
	cpy->select();
	UndoReparentStruct* str = (UndoReparentStruct*)
	    GMALLOC(sizeof(UndoReparentStruct));
	str->go = cpy;
	str->next = cpy->next();
	str->parent = cpy->parent();
	str->cb = this;
	undo_mgr->push(cb_undo_create, str, (GUndoProc)free);
    }
}



                          /*********************
			   * PRIVATE FUNCTIONS *
			   *********************/



void cb_copy_proc (void* cb, Go* go)
{
    Go* cpy = go->dup();
    if (cpy) {
	cpy->setCoordsys(go->coordsys());
	((GoDrawClipboard*)cb)->buffer.add(cpy);
    }
}


void cb_cut_proc(void* cb, Go* go)
{
    UndoReparentStruct* str = (UndoReparentStruct*)
	    GMALLOC(sizeof(UndoReparentStruct));
    str->go = go;
    str->parent = go->parent();
    str->next = go->next();
    str->cb = (GoDrawClipboard*)cb;
    ((GoDrawClipboard*)cb)->undo_mgr->push(cb_undo_destroy, str,
					   (GUndoProc)free);
    go->reparent(NULL);
    ((GoDrawClipboard*)cb)->buffer.add(go);
}


void cb_undo_unselect_all(void* cb, Go* go)
{
    UndoStruct* str = (UndoStruct*)GMALLOC(sizeof(UndoStruct));
    str->go = go;
    str->cb = (GoDrawClipboard*)cb;
    str->cb->undo_mgr->push(cb_undo_unselect_one, str, (GUndoProc)free);
}


void cb_undo_create(void* s)
{
    UndoReparentStruct* str = (UndoReparentStruct*)
	GMALLOC(sizeof(UndoReparentStruct));
    str->parent = ((UndoReparentStruct*)s)->go->parent();
    str->next = ((UndoReparentStruct*)s)->go->next();
    str->go = ((UndoReparentStruct*)s)->go;
    str->cb = ((UndoReparentStruct*)s)->cb;
    str->cb->undo_mgr->push(cb_undo_destroy, str, (GUndoProc)free);
    str->go->reparent(NULL);
    GFREE(s);
}


void cb_undo_destroy(void* s)
{
    UndoReparentStruct* str = (UndoReparentStruct*)
	GMALLOC(sizeof(UndoReparentStruct));
    str->parent = NULL;
    str->next = NULL;
    str->go = ((UndoReparentStruct*)s)->go;
    str->cb = ((UndoReparentStruct*)s)->cb;
    str->cb->undo_mgr->push(cb_undo_create, str, (GUndoProc)free);
    str->go->reparent(((UndoReparentStruct*)s)->parent,
		      ((UndoReparentStruct*)s)->next);
    GFREE(s);
}


void cb_undo_select_one(void* s)
{
    UndoStruct* str = (UndoStruct*)GMALLOC(sizeof(UndoStruct));
    str->go = ((UndoStruct*)s)->go;
    str->cb = ((UndoStruct*)s)->cb;
    str->go->unselect();
    str->cb->undo_mgr->push(cb_undo_unselect_one, str, (GUndoProc)free);
}

void cb_undo_unselect_one(void* s)
{
    UndoStruct* str = (UndoStruct*)GMALLOC(sizeof(UndoStruct));
    str->go = ((UndoStruct*)s)->go;
    str->cb = ((UndoStruct*)s)->cb;
    str->go->select();
    str->cb->undo_mgr->push(cb_undo_select_one, str, (GUndoProc)free);
    GFREE(s);
}
