/* $Id: Clipboard.h,v 1.1 1992/11/30 11:50:51 jacomet Exp $ */
/*****************************************************************************
MODULE:  GoDraw
FILE:    Clipboard.h
AUTHOR:  Aline Jacomet
DATE:    November 1992

DESCRIPTION:
    Defines the clipboard object used by GoDraw in the free version.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#ifndef Clipboard_h
#define Clipboard_h

#include <Go/Tools/Set.h>
#include <Go/Tools/Undo.h>

extern class Go;
extern class GoRoot;

class GoDrawClipboard
{
    friend void cb_copy_proc(void*, Go*);
    friend void cb_cut_proc(void*, Go*);
    friend void cb_undo_unselect_all(void*, Go*);
    friend void cb_undo_create(void*);
    friend void cb_undo_destroy(void*);
    friend void cb_undo_unselect_one(void*);
    friend void cb_undo_select_one(void*);
    
  public:
    GoDrawClipboard(GUndoManager* undoMgr);
    void cut(GoRoot* goroot);
    void copy(GoRoot* goroot);
    void paste(GoRoot* goroot);

  private:
    GUndoManager* undo_mgr;
    GSet buffer;

    void reset_buffer();
    void paste_recur(Go*, Go*);
};

#endif /* Clipboard_h */
