#ifdef REV_INFO
static char rcsid[] = "$Id: allsheet.cc,v 1.1.1.1 1992/03/18 09:03:48 uep Exp $";
#endif
/*****************************************************************************
MODULE:  Go Editor
FILE:    allsheet.cc
AUTHOR:  Aline Jacomet
DATE:    May 22th 1991

DESCRIPTION:
    

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include <string.h>

#include <Go/Tools/Alloc.h>

#include "classes.h"
#include "main.h"

#include <Xm/TextF.h>
#include <Xm/Text.h>
#include <Xm/Form.h>
#include <Xm/PushBG.h>
#include <Xm/SeparatoG.h>
#include <Xm/LabelG.h>
#include <Xm/ToggleBG.h>
#include <Xm/RowColumn.h>


/***
 * widget function
 ***/
static Widget CreateAllDialog(Widget);
static void CreatePenColors(Widget);
static void CreateLineWidth(Widget);
static void CreateLineStyle(Widget);
static void CreateJoinStyle(Widget);
static void CreateCapStyle(Widget);
static void CreateBrushColors(Widget);
static void CreateFillRule(Widget);
static void CreateTextColors(Widget);
static void CreateFont(Widget);

void MapAllAttrib(void*, void*, caddr_t);


static void ModifyFillRule(Widget, int ls, caddr_t);
static void ModifyJoinStyle(Widget, int ls, caddr_t);
static void ModifyCapStyle(Widget, int ls, caddr_t);
static void ModifyLineStyle(Widget, int ls, caddr_t);
static void ModifyAllSheet();
static void ReinitAllSheet();
static void ReinitToggleField(Widget, caddr_t, XmToggleButtonCallbackStruct*);
static void Verify(Widget, caddr_t, caddr_t);



/***
 *  Static Variables
 ***/
static Widget allDialog;
static Widget lineStyle_menu;
static Widget capStyle_menu;
static Widget joinStyle_menu;
static Widget lineWidth;
static Widget line_wid[3];
static Widget cap_wid[4];
static Widget join_wid[3];
static Widget pen_color_label;
static Widget pen_color_list;
static Widget altColor_label;
static Widget altColor_list;
static Widget lineWidth_label;
static Widget joinStyle_label;
static Widget lineStyle_label;
static Widget capStyle_label;
static Widget fillRule_label;

static Widget brush_color_label;
static Widget brush_color_list;
static Widget fillRule_menu;
static Widget rule_wid[2];

static Widget text_color_label;
static Widget text_color_list;
static Widget font_label;
static Widget font_list;

/* Validation toggles (when modifying Gos) : one per attribute subwidget */

Widget penColorToggle;
Widget altColorToggle;
Widget brushColorToggle;
Widget textColorToggle;
Widget fontToggle;
Widget lineWidthToggle;
Widget lineStyleToggle;
Widget joinStyleToggle;
Widget capStyleToggle;
Widget fillRuleToggle;

static Widget valid_toggles[10];
static int num_valid_toggles = 0;


/***
 *  Global output attributes
 ***/
Widget edit_pen_color = NULL;
Widget edit_pen_alt_color = NULL;

Widget edit_brush_color = NULL;

Widget edit_text_color = NULL;
Widget edit_text_font = NULL;

extern Widget penTextEdit;
extern Widget brushTextEdit;
extern Widget textBrushTextEdit;

extern int WhichAction;
extern int WhichObject;
extern Boolean MapEditAllAttrib;

extern Boolean allMapped;

extern PenAttributes reinit_pen;
extern BrushAttributes reinit_brush;
extern TextAttributes reinit_text;
extern PenAttributes pen;
extern BrushAttributes brush;
extern TextAttributes text;
extern AllAttributes all_attrib;



/*
 * EXTERN FUNCTIONS
 */

// in file namedattrib.cc;
extern void ModifyColor(Widget, char*);
extern void UnmanageWidget(Widget, Widget, caddr_t);

// in file editattrib.cc;
extern void ManageValidationToggles(Boolean, Widget*, int);
extern void SensitiveEditButtons(Widget, caddr_t, caddr_t);
extern void ValidFunction(Widget, Widget, caddr_t);

// in file colorsel.cc;
extern void MapColorList(Widget, Widget, caddr_t);

// in file help.cc;
extern void MapHelpFile(void*, char*, void*);

// in file fontsel.cc;
extern void MapFontList(Widget, int, caddr_t);

// in file pensheet.cc;
extern void ModifyLineWidth(Widget, int, caddr_t);
extern int GetItem(Widget*, int, Widget);

// in file goAttrib.cc;
extern void ModifyGoCapStyle(int);
extern void ModifyGoJoinStyle(int);
extern void ModifyGoLineStyle(int);
extern void ModifyGoFillRule(int);
extern void ModifyGoEditPenSheet();
extern void ModifyGoEditBrushSheet();
extern void ModifyGoEditTextSheet();
extern void ModifyGoFont(char*);






/***
 * Redo textfield translations for line width.
 ***/

char *LineWidthEventBindings1 =
"Shift<Key>osfQuickPaste:	do-quick-action()\n\
 Meta <Key>osfQuickPaste:	quick-cut-set()\n\
 Alt <Key>osfQuickPaste:	quick-cut-set()\n\
 <Key>osfQuickPaste:		quick-copy-set()\n\
 Meta <Key>osfPrimaryPaste:	cut-primary()\n\
 Alt <Key>osfPrimaryPaste:	cut-primary()\n\
 <Key>osfPrimaryPaste:		copy-primary()\n\
 Meta <Key>osfCut:		cut-primary()\n\
 Alt <Key>osfCut:		cut-primary()\n\
 <Key>osfCut:			cut-clipboard()\n\
 <Key>osfPaste:			paste-clipboard()\n\
 Meta <Key>osfCopy:		copy-primary()\n\
 Alt <Key>osfCopy:		copy-primary()\n\
 <Key>osfCopy:			copy-clipboard()\n\
 Shift <Key>osfBeginLine:	beginning-of-line(extend)\n\
 <Key>osfBeginLine:		beginning-of-line()\n\
 Shift <Key>osfEndLine:		end-of-line(extend)\n\
 <Key>osfEndLine:		end-of-line()\n\
 Ctrl <Key>osfPageUp:		page-left()\n\
 Ctrl <Key>osfPageDown:		page-right()\n\
 <Key>osfClear:			clear-selection()\n\
 ~Meta ~Alt <Key>osfBackSpace:	delete-previous-character()\n\
 Shift Meta<Key>osfDelete:	cut-primary()\n\
 Shift Alt<Key>osfDelete:	cut-primary()\n\
 Shift ~Ctrl ~Meta ~Alt <Key>osfDelete:		cut-clipboard()\n\
 ~Shift Ctrl ~Meta ~Alt <Key>osfDelete:		delete-to-end-of-line()\n\
 ~Shift ~Ctrl ~Meta ~Alt <Key>osfDelete:	delete-next-character()\n";


char *LineWidthEventBindings2 =
"Ctrl Meta<Key>osfInsert:	copy-primary()\n\
 Ctrl Alt<Key>osfInsert:	copy-primary()\n\
 Shift ~Ctrl ~Meta ~Alt <Key>osfInsert:		paste-clipboard()\n\
 ~Shift Ctrl ~Meta ~Alt <Key>osfInsert:		copy-clipboard()\n\
 ~Shift <Key>osfSelect:		set-anchor()\n\
 Shift ~Ctrl <Key>osfSelect:	key-select()\n\
 <Key>osfActivate:		activate()\n\
 <Key>osfAddMode:		toggle-add-mode()\n\
 <Key>osfHelp:			Help()\n\
 <Key>osfCancel:		process-cancel()\n\
 Shift Ctrl <Key>osfLeft:	backward-word(extend)\n\
 Ctrl <Key>osfLeft:		backward-word()\n\
 Shift <Key>osfLeft:		key-select(left)\n\
 <Key>osfLeft:			backward-character()\n\
 Shift Ctrl <Key>osfRight:	forward-word(extend)\n\
 Ctrl <Key>osfRight:		forward-word()\n\
 Shift <Key>osfRight:		key-select(right)\n\
 <Key>osfRight:			forward-character()\n\
 <Key>osfUp:			traverse-prev()\n\
 <Key>osfDown:			traverse-next()\n\
 Ctrl ~Meta ~Alt <Key>slash:	select-all()\n\
 Ctrl ~Meta ~Alt <Key>backslash:	deselect-all()\n\
 Shift ~Meta ~Alt <Key>Tab:	prev-tab-group()\n\
 ~Meta ~Alt <Key>Tab:		next-tab-group()\n";

char *LineWidthEventBindings3 =
 "~Shift ~Meta ~Alt <Key>Return:	activate()\n\
 Ctrl ~Shift ~Meta ~Alt <Key>space:	set-anchor()\n\
 Shift ~Ctrl ~Meta ~Alt <Key>space:	key-select()\n\
~Ctrl ~Meta ~Alt <Key>0:	self-insert()\n\
 ~Ctrl ~Meta ~Alt <Key>1:	self-insert()\n\
 ~Ctrl ~Meta ~Alt <Key>2:	self-insert()\n\
 ~Ctrl ~Meta ~Alt <Key>3:	self-insert()\n\
 ~Ctrl ~Meta ~Alt <Key>4:	self-insert()\n\
 ~Ctrl ~Meta ~Alt <Key>5:	self-insert()\n\
 ~Ctrl ~Meta ~Alt <Key>6:	self-insert()\n\
 ~Ctrl ~Meta ~Alt <Key>7:	self-insert()\n\
 ~Ctrl ~Meta ~Alt <Key>8:	self-insert()\n\
 ~Ctrl ~Meta ~Alt <Key>9:	self-insert()\n\
 ~Ctrl ~Meta ~Alt <Key>period:	self-insert()\n\
 ~Ctrl  Shift ~Meta ~Alt<Btn1Down>:	extend-start()\n\
  Ctrl ~Shift ~Meta ~Alt<Btn1Down>:	move-destination()\n\
 ~Ctrl ~Shift ~Meta ~Alt<Btn1Down>:	grab-focus()\n\
 ~Ctrl ~Meta ~Alt<Btn1Motion>:		extend-adjust()\n\
 ~Ctrl ~Meta ~Alt<Btn1Up>:		extend-end()\n\
 <FocusIn>:				focusIn()\n\
 <FocusOut>:				focusOut()\n\
 <Enter>:				enter()\n\
 <Leave>:				leave()\n\
 <Unmap>:				unmap()";




/**
FUNCTION
    ModifyEditColor()
DESCRIPTION
    Modifies colors of gos used in the attributes graphic widget.
**/
static void ModifyEditColor(Widget w, int wc, caddr_t)
{
    WhichAttributes = EDIT;
    WhichColor = wc;
    char* color = XmTextFieldGetString(w);
    ModifyColor(w, color);
    GFREE (color);
}



/**
FUNCTION
    MapAllAttrib()
DESCRIPTION
    Maps and creates, if necessary, the all attributes editor.
**/
void MapAllAttrib(void*, void* parent, caddr_t)
{
    Boolean create = (allDialog == NULL);

    if (!create && XtIsManaged(allDialog))
	return;
    if (create) {
	XDefineCursor(XtDisplay((Widget)graphic), XtWindow((Widget)graphic),
		      clock_cursor);
	XDefineCursor(XtDisplay((Widget)main_window),
		      XtWindow((Widget)main_window), clock_cursor);
	XFlush(XtDisplay((Widget)graphic));
	allDialog = CreateAllDialog((Widget)parent);
    }

    if (WhichAction == MODIFY)
	ManageValidationToggles(True, valid_toggles, num_valid_toggles);
    else
	ManageValidationToggles(False, valid_toggles, num_valid_toggles);

    ReinitAllSheet();
    XtManageChild(allDialog);
    MapEditAllAttrib = True;
    
    if (create) {
	XFlush(XtDisplay((Widget)graphic));
	XUndefineCursor(XtDisplay((Widget)graphic), XtWindow((Widget)graphic));
	XUndefineCursor(XtDisplay((Widget)main_window),
			XtWindow((Widget)main_window));
    }
    allMapped = True;
}


/**
FUNCTION
    CreateAllDialog()
DESCRIPTION
    Creates the all attributes editor.
**/
static Widget CreateAllDialog(Widget parent)
{
    Widget w;
    Arg args[20];

    XmString str_null = XmStringCreate("", XmSTRING_DEFAULT_CHARSET);
    int n = 0;
    int nb = 0;
    XtSetArg(args[n], XmNautoUnmanage, False); n++;
    XtSetArg(args[n], XmNresizePolicy, XmRESIZE_NONE); n++;
    Widget ps = XmCreateFormDialog(parent, "AllDialog", args, n);
    XtAddCallback(ps, XmNhelpCallback, (XtCallbackProc)MapHelpFile,
		  (XtPointer)ALL_ATTRIB_FILE);

    /* Apply, Reinit, Cancel buttons */
    w = XmCreatePushButtonGadget(ps, "Apply", NULL, 0);
    XtManageChild(w);
    XtAddCallback(w, XmNactivateCallback,
		  (XtCallbackProc)SensitiveEditButtons, NULL);
    XtAddCallback(w, XmNactivateCallback, (XtCallbackProc)Verify,
		  (XtPointer)ps);
    XtAddCallback(w, XmNhelpCallback, (XtCallbackProc)MapHelpFile,
		  (XtPointer)ALL_ATTRIB_FILE);

    w = XmCreatePushButtonGadget(ps, "Reinit", NULL, 0);
    XtManageChild(w);
    XtAddCallback(w, XmNactivateCallback, (XtCallbackProc)ReinitAllSheet,
		  NULL);
    XtAddCallback(w, XmNhelpCallback, (XtCallbackProc)MapHelpFile,
		  (XtPointer)ALL_ATTRIB_FILE);
    
    w = XmCreatePushButtonGadget(ps, "Cancel", NULL, 0);
    XtManageChild(w);
    XtAddCallback(w, XmNactivateCallback,
		  (XtCallbackProc)SensitiveEditButtons, NULL);
    XtAddCallback(w, XmNactivateCallback,
		  (XtCallbackProc)ModifyAllSheet, NULL);
    XtAddCallback(w, XmNactivateCallback, (XtCallbackProc) UnmanageWidget,
		  (XtPointer)ps);
    XtAddCallback(w, XmNhelpCallback, (XtCallbackProc)MapHelpFile,
		  (XtPointer)ALL_ATTRIB_FILE);

    /* labels and separators */
    w = XmCreateSeparatorGadget(ps, "Separator1", NULL, 0);
    XtManageChild(w);
    XtAddCallback(w, XmNhelpCallback, (XtCallbackProc)MapHelpFile,
		  (XtPointer)ALL_ATTRIB_FILE);
    w = XmCreateSeparatorGadget(ps, "Separator2", NULL, 0);
    XtManageChild(w);
    XtAddCallback(w, XmNhelpCallback, (XtCallbackProc)MapHelpFile,
		  (XtPointer)ALL_ATTRIB_FILE);
    w = XmCreateSeparatorGadget(ps, "Separator3", NULL, 0);
    XtManageChild(w);
    XtAddCallback(w, XmNhelpCallback, (XtCallbackProc)MapHelpFile,
		  (XtPointer)ALL_ATTRIB_FILE);
    w = XmCreateLabelGadget(ps, "OutlineLabel", NULL, 0);
    XtManageChild(w);
    XtAddCallback(w, XmNhelpCallback, (XtCallbackProc)MapHelpFile,
		  (XtPointer)ALL_ATTRIB_FILE);
    w = XmCreateLabelGadget(ps, "BackgroundLabel", NULL, 0);
    XtManageChild(w);
    XtAddCallback(w, XmNhelpCallback, (XtCallbackProc)MapHelpFile,
		  (XtPointer)ALL_ATTRIB_FILE);
    w = XmCreateLabelGadget(ps, "TextLabel", NULL, 0);
    XtManageChild(w);
    XtAddCallback(w, XmNhelpCallback, (XtCallbackProc)MapHelpFile,
		  (XtPointer)ALL_ATTRIB_FILE);
    

    n = 0;
    XtSetArg(args[n], XmNset, TRUE); n++;

    /*  pen color and  pen alternate color */
    XtSetArg(args[n], XmNmarginWidth, 0); n++;
    XtSetArg(args[n], XmNmarginHeight, 0); n++;
    XtSetArg(args[n], XmNindicatorType, XmONE_OF_MANY); n++;
    XtSetArg(args[n], XmNlabelString, str_null); n++;
    penColorToggle = valid_toggles[nb] = 
	XmCreateToggleButtonGadget(ps, "ValidPenColor", args, n);
    XtAddCallback(penColorToggle, XmNvalueChangedCallback,
		  (XtCallbackProc) ReinitToggleField, NULL);
    XtAddCallback(penColorToggle, XmNhelpCallback, (XtCallbackProc)MapHelpFile,
		  (XtPointer)ALL_ATTRIB_FILE);
    nb++;
    n = 1;
    XtSetArg(args[n], XmNmarginWidth, 0); n++;
    XtSetArg(args[n], XmNmarginHeight, 0); n++;
    XtSetArg(args[n], XmNindicatorType, XmONE_OF_MANY); n++;
    XtSetArg(args[n], XmNlabelString, str_null); n++;
    altColorToggle = valid_toggles[nb] = 
	XmCreateToggleButtonGadget(ps, "ValidAltColor", args, n);
    XtAddCallback(altColorToggle, XmNvalueChangedCallback,
		  (XtCallbackProc) ReinitToggleField, NULL);
    XtAddCallback(altColorToggle, XmNhelpCallback, (XtCallbackProc)MapHelpFile,
		  (XtPointer)ALL_ATTRIB_FILE);
    nb++;
    CreatePenColors(ps);

    /* pen line width */
    n = 1;
    XtSetArg(args[n], XmNmarginWidth, 0); n++;
    XtSetArg(args[n], XmNmarginHeight, 0); n++;
    XtSetArg(args[n], XmNindicatorType, XmONE_OF_MANY); n++;
    XtSetArg(args[n], XmNlabelString, str_null); n++;
    lineWidthToggle = valid_toggles[nb] = 
	XmCreateToggleButtonGadget(ps, "ValidLineWidth", args, n);
    XtAddCallback(lineWidthToggle, XmNvalueChangedCallback,
		  (XtCallbackProc) ReinitToggleField, NULL);
    XtAddCallback(lineWidthToggle, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)ALL_ATTRIB_FILE);
    nb++;
    CreateLineWidth(ps);

    /* pen line style */
    n = 1;
    XtSetArg(args[n], XmNmarginWidth, 0); n++;
    XtSetArg(args[n], XmNmarginHeight, 0); n++;
    XtSetArg(args[n], XmNindicatorType, XmONE_OF_MANY); n++;
    XtSetArg(args[n], XmNlabelString, str_null); n++;
    lineStyleToggle = valid_toggles[nb] = 
	XmCreateToggleButtonGadget(ps, "ValidLineStyle", args, n);
    XtAddCallback(lineStyleToggle, XmNvalueChangedCallback,
		  (XtCallbackProc)ReinitToggleField, NULL);
    XtAddCallback(lineStyleToggle, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)ALL_ATTRIB_FILE);
    nb++;
    CreateLineStyle(ps);

    /* pen cap style */
    n = 1;
    XtSetArg(args[n], XmNmarginWidth, 0); n++;
    XtSetArg(args[n], XmNmarginHeight, 0); n++;
    XtSetArg(args[n], XmNindicatorType, XmONE_OF_MANY); n++;
    XtSetArg(args[n], XmNlabelString, str_null); n++;
    capStyleToggle = valid_toggles[nb] = 
	XmCreateToggleButtonGadget(ps, "ValidCapStyle", args, n);
    XtAddCallback(capStyleToggle, XmNvalueChangedCallback,
		  (XtCallbackProc) ReinitToggleField, NULL);
    XtAddCallback(capStyleToggle, XmNhelpCallback, (XtCallbackProc)MapHelpFile,
		  (XtPointer)ALL_ATTRIB_FILE);
    nb++;
    CreateCapStyle(ps);

    /* pen join style */
    n = 1;
    XtSetArg(args[n], XmNmarginWidth, 0); n++;
    XtSetArg(args[n], XmNmarginHeight, 0); n++;
    XtSetArg(args[n], XmNindicatorType, XmONE_OF_MANY); n++;
    XtSetArg(args[n], XmNlabelString, str_null); n++;
    joinStyleToggle = valid_toggles[nb] = 
	XmCreateToggleButtonGadget(ps, "ValidJoinStyle", args, n);
    XtAddCallback(joinStyleToggle, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)ALL_ATTRIB_FILE);
    XtAddCallback(joinStyleToggle, XmNvalueChangedCallback,
		  (XtCallbackProc) ReinitToggleField, NULL);
    nb++;
    CreateJoinStyle(ps);

    /* brush color */
    n = 1;
    XtSetArg(args[n], XmNmarginWidth, 0); n++;
    XtSetArg(args[n], XmNmarginHeight, 0); n++;
    XtSetArg(args[n], XmNindicatorType, XmONE_OF_MANY); n++;
    XtSetArg(args[n], XmNlabelString, str_null); n++;
    brushColorToggle = valid_toggles[nb] = 
	XmCreateToggleButtonGadget(ps, "ValidBrushColor", args, n);
    XtAddCallback(brushColorToggle, XmNvalueChangedCallback,
		  (XtCallbackProc) ReinitToggleField, NULL);
    XtAddCallback(brushColorToggle, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)ALL_ATTRIB_FILE);
    nb++;
    CreateBrushColors(ps);

    /* brush fill rule */
    n = 1;
    XtSetArg(args[n], XmNmarginWidth, 0); n++;
    XtSetArg(args[n], XmNmarginHeight, 0); n++;
    XtSetArg(args[n], XmNindicatorType, XmONE_OF_MANY); n++;
    XtSetArg(args[n], XmNlabelString, str_null); n++;
    fillRuleToggle = valid_toggles[nb] = 
	XmCreateToggleButtonGadget(ps, "ValidFillRule", args, n);
    XtAddCallback(fillRuleToggle, XmNvalueChangedCallback,
		  (XtCallbackProc) ReinitToggleField, NULL);
    XtAddCallback(fillRuleToggle, XmNhelpCallback, (XtCallbackProc)MapHelpFile,
		  (XtPointer)ALL_ATTRIB_FILE);
    nb++;
    CreateFillRule(ps);

    /* textbrush color */
    n = 1;
    XtSetArg(args[n], XmNmarginWidth, 0); n++;
    XtSetArg(args[n], XmNmarginHeight, 0); n++;
    XtSetArg(args[n], XmNindicatorType, XmONE_OF_MANY); n++;
    XtSetArg(args[n], XmNlabelString, str_null); n++;
    textColorToggle = valid_toggles[nb] = 
	XmCreateToggleButtonGadget(ps, "ValidTextColor", args, n);
    XtAddCallback(textColorToggle, XmNvalueChangedCallback,
		  (XtCallbackProc) ReinitToggleField, NULL);
    XtAddCallback(textColorToggle, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)ALL_ATTRIB_FILE);
    nb++;
    CreateTextColors(ps);

    /* textbrush font */
    n = 1;
    XtSetArg(args[n], XmNmarginWidth, 0); n++;
    XtSetArg(args[n], XmNmarginHeight, 0); n++;
    XtSetArg(args[n], XmNindicatorType, XmONE_OF_MANY); n++;
    XtSetArg(args[n], XmNlabelString, str_null); n++;
    fontToggle = valid_toggles[nb] = 
	XmCreateToggleButtonGadget(ps, "ValidFont", args, n);
    XtAddCallback(fontToggle, XmNvalueChangedCallback,
		  (XtCallbackProc) ReinitToggleField, NULL);
    XtAddCallback(fontToggle, XmNhelpCallback, (XtCallbackProc)MapHelpFile,
		  (XtPointer)ALL_ATTRIB_FILE);
    nb++;
    CreateFont(ps);

    num_valid_toggles = nb;

    XmStringFree(str_null);
    
    /* attach widgets */
    n = 0;
    XtSetArg(args[n], XmNleftAttachment, XmATTACH_OPPOSITE_WIDGET); n++;
    XtSetArg(args[n], XmNleftWidget, pen_color_label); n++;
    XtSetValues(joinStyle_label, args, n);
    XtSetValues(altColor_label, args, n);
    XtSetValues(brush_color_label, args, n);
    XtSetValues(text_color_label, args, n);
    XtSetValues(font_label, args, n);
    n = 0;
    XtSetArg(args[n], XmNleftAttachment, XmATTACH_OPPOSITE_WIDGET); n++;
    XtSetArg(args[n], XmNleftWidget, pen_color_list); n++;
    XtSetValues(joinStyle_menu, args, n);
    XtSetValues(altColor_list, args, n);
    XtSetValues(brush_color_list, args, n);
    XtSetValues(text_color_list, args, n);
    XtSetValues(font_list, args, n);
    n = 0;
    XtSetArg(args[n], XmNrightAttachment, XmATTACH_OPPOSITE_WIDGET); n++;
    XtSetArg(args[n], XmNrightWidget, edit_pen_color); n++;
    XtSetValues(edit_pen_alt_color, args, n);
    XtSetValues(edit_brush_color, args, n);
    XtSetValues(edit_text_color, args, n);
    n = 0;
    XtSetArg(args[n], XmNleftAttachment, XmATTACH_OPPOSITE_WIDGET); n++;
    XtSetArg(args[n], XmNleftWidget, lineWidth_label); n++;
    XtSetValues(lineStyle_label, args, n);
    XtSetValues(capStyle_label, args, n);
    XtSetValues(fillRule_label, args, n);
    n = 0;
    XtSetArg(args[n], XmNleftAttachment, XmATTACH_OPPOSITE_WIDGET); n++;
    XtSetArg(args[n], XmNleftWidget, lineWidth); n++;
    XtSetValues(lineStyle_menu, args, n);
    XtSetValues(capStyle_menu, args, n);
    XtSetValues(fillRule_menu, args, n);

    return ps;
}


static void CreatePenColors(Widget parent)
{
    pen_color_label = XmCreateLabelGadget(parent, "PenColorLabel", NULL, 0);
    XtManageChild(pen_color_label);
    XtAddCallback(pen_color_label, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)ALL_ATTRIB_FILE);
    Arg args[2];
    int n = 0;
    XtSetArg(args[n], XmNmarginWidth, 5); n++;
    XtSetArg(args[n], XmNmarginHeight, 5); n++;
    pen_color_list = XmCreatePushButtonGadget(parent, "PenColorList", args, n);
    XtManageChild(pen_color_list);
    XtAddCallback(pen_color_list, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)ALL_ATTRIB_FILE);
    edit_pen_color = XmCreateTextField(parent, "PenColorName", NULL, 0);
    XtManageChild(edit_pen_color);
    XtAddCallback(pen_color_list, XmNactivateCallback,
		  (XtCallbackProc)MapColorList, (XtPointer)edit_pen_color);
    XtAddCallback(edit_pen_color, XmNactivateCallback,
		  (XtCallbackProc) ModifyEditColor, PEN_COLOR);
    XtAddCallback(edit_pen_color, XmNactivateCallback,
		  (XtCallbackProc)ValidFunction, (XtPointer)penColorToggle);
    XtAddCallback(edit_pen_color, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)ALL_ATTRIB_FILE);

    altColor_label = XmCreateLabelGadget(parent, "PenAltColorLabel", NULL, 0);
    XtManageChild(altColor_label);
    XtAddCallback(altColor_label, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)ALL_ATTRIB_FILE);
    n = 0;
    XtSetArg(args[n], XmNmarginWidth, 5); n++;
    XtSetArg(args[n], XmNmarginHeight, 5); n++;
    altColor_list = XmCreatePushButtonGadget(parent, "PenAltColorList", args,
					     n);
    XtManageChild(altColor_list);
    XtAddCallback(altColor_list, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)ALL_ATTRIB_FILE);
    edit_pen_alt_color = XmCreateTextField(parent, "PenAltColorName", NULL, 0);
    XtManageChild(edit_pen_alt_color);
    XtAddCallback(altColor_list, XmNactivateCallback,
		  (XtCallbackProc)MapColorList, (XtPointer)edit_pen_alt_color);
    XtAddCallback(edit_pen_alt_color, XmNactivateCallback,
		  (XtCallbackProc) ModifyEditColor, (XtPointer) PEN_ALT_COLOR);
    XtAddCallback(edit_pen_alt_color, XmNactivateCallback,
		  (XtCallbackProc) ValidFunction, (XtPointer)altColorToggle);
    XtAddCallback(edit_pen_alt_color, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)ALL_ATTRIB_FILE);
}


static void CreateLineWidth(Widget parent)
{
    char* line_width_event_bindings =
	(char *) GMALLOC(strlen(LineWidthEventBindings1) +
			  strlen(LineWidthEventBindings2) +
			  strlen(LineWidthEventBindings3) + 1);

    strcpy(line_width_event_bindings, LineWidthEventBindings1);
    strcat(line_width_event_bindings, LineWidthEventBindings2);
    strcat(line_width_event_bindings, LineWidthEventBindings3);
    XtTranslations line_width_translations =
	XtParseTranslationTable(line_width_event_bindings);
    GFREE(line_width_event_bindings);
    
    lineWidth_label = XmCreateLabelGadget(parent, "LineWidthLabel",
						 NULL, 0);
    XtManageChild(lineWidth_label);
    XtAddCallback(lineWidth_label, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)ALL_ATTRIB_FILE);

    Arg args[1];
    int n = 0;
    XtSetArg(args[n], XmNtranslations, line_width_translations); n++;
    lineWidth = XmCreateTextField(parent, "LineWidth", args, n);
    XtManageChild(lineWidth);
    XtAddCallback(lineWidth, XmNactivateCallback,
		  (XtCallbackProc)ModifyLineWidth, EDIT);
    XtAddCallback(lineWidth, XmNactivateCallback,
		  (XtCallbackProc)ValidFunction, (XtPointer)lineWidthToggle);
    XtAddCallback(lineWidth, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)ALL_ATTRIB_FILE);
}


static void CreateLineStyle(Widget parent)
{
    lineStyle_label = XmCreateLabelGadget(parent, "LineStyleLabel",
						 NULL,0);
    XtManageChild(lineStyle_label);
    XtAddCallback(lineStyle_label, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)ALL_ATTRIB_FILE);
    Widget lineStyle_pull = XmCreatePulldownMenu(parent, "LineStylePull",
						 NULL, 0);
    Arg args[5];
    int n = 0;
    XtSetArg(args[n], XmNsubMenuId, lineStyle_pull); n++;
    XtSetArg(args[n], XmNmarginWidth, 0); n++;
    XtSetArg(args[n], XmNmarginHeight, 0); n++;
    lineStyle_menu = XmCreateOptionMenu(parent, "LineStyleMenu", args, n);
    XtManageChild(lineStyle_menu);
    n = 0;
    XtSetArg(args[n], XmNlabelPixmap, lineSolidIcon); n++;
    XtSetArg(args[n], XmNlabelType, XmPIXMAP); n++;
    line_wid[LINESOLID] = XmCreatePushButtonGadget(lineStyle_pull, "lineSolid",
					     args, n);
    XtAddCallback(line_wid[LINESOLID], XmNactivateCallback,
		  (XtCallbackProc)ModifyLineStyle, (XtPointer)LINESOLID);
    XtAddCallback(line_wid[LINESOLID], XmNactivateCallback,
		  (XtCallbackProc)ValidFunction, (XtPointer)lineStyleToggle);
    XtAddCallback(line_wid[LINESOLID], XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)ALL_ATTRIB_FILE);
    n = 0;
    XtSetArg(args[n], XmNlabelPixmap, lineOnOffDashIcon); n++;
    XtSetArg(args[n], XmNlabelType, XmPIXMAP); n++;
    line_wid[LINEONOFF] = XmCreatePushButtonGadget(lineStyle_pull,
						   "lineOnOffDash", args, n);
    XtAddCallback(line_wid[LINEONOFF], XmNactivateCallback,
		  (XtCallbackProc)ModifyLineStyle, (XtPointer)LINEONOFF);
    XtAddCallback(line_wid[LINEONOFF], XmNactivateCallback,
		  (XtCallbackProc)ValidFunction, (XtPointer)lineStyleToggle);
    XtAddCallback(line_wid[LINEONOFF], XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)ALL_ATTRIB_FILE);
    n = 0;
    XtSetArg(args[n], XmNlabelPixmap, lineDoubleDashIcon); n++;
    XtSetArg(args[n], XmNlabelType, XmPIXMAP); n++;
    line_wid[LINEDOUBLE] = XmCreatePushButtonGadget(lineStyle_pull,
						    "lineDoubleDash", args, n);
    XtAddCallback(line_wid[LINEDOUBLE], XmNactivateCallback,
		  (XtCallbackProc)ModifyLineStyle, (XtPointer)LINEDOUBLE);
    XtAddCallback(line_wid[LINEDOUBLE], XmNactivateCallback,
		  (XtCallbackProc)ValidFunction, (XtPointer)lineStyleToggle);
    XtAddCallback(line_wid[LINEDOUBLE], XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)ALL_ATTRIB_FILE);
    XtManageChildren(line_wid, 3);
}


static void CreateCapStyle(Widget parent)
{
    capStyle_label = XmCreateLabelGadget(parent, "CapStyleLabel",
						NULL, 0);
    XtAddCallback(capStyle_label, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)ALL_ATTRIB_FILE);
    XtManageChild(capStyle_label);
    Widget capStyle_pull = XmCreatePulldownMenu(parent, "CapStylePull",
						NULL, 0);
    Arg args[5];
    int n = 0;
    XtSetArg(args[n], XmNsubMenuId, capStyle_pull); n++;
    XtSetArg(args[n], XmNmarginWidth, 0); n++;
    XtSetArg(args[n], XmNmarginHeight, 0); n++;
    capStyle_menu = XmCreateOptionMenu(parent, "CapStyleMenu", args, n);
    XtManageChild(capStyle_menu);
    cap_wid[0] = NULL;
    cap_wid[1] = XmCreatePushButtonGadget(capStyle_pull, "capButt",
						NULL, 0);
    XtAddCallback(cap_wid[1], XmNactivateCallback,
		  (XtCallbackProc)ModifyCapStyle, (XtPointer)CAPBUTT);
    XtAddCallback(cap_wid[1], XmNactivateCallback,
		  (XtCallbackProc)ValidFunction, (XtPointer)capStyleToggle);
    XtAddCallback(cap_wid[1], XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)ALL_ATTRIB_FILE);
    XtManageChild(cap_wid[1]);
    cap_wid[2] = XmCreatePushButtonGadget(capStyle_pull, "capRound",
						 NULL, 0);
    XtAddCallback(cap_wid[2], XmNactivateCallback,
		  (XtCallbackProc)ModifyCapStyle, (XtPointer)CAPROUND);
    XtAddCallback(cap_wid[2], XmNactivateCallback,
		  (XtCallbackProc)ValidFunction, (XtPointer)capStyleToggle);
    XtAddCallback(cap_wid[2], XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)ALL_ATTRIB_FILE);
    XtManageChild(cap_wid[2]);
    cap_wid[3] = XmCreatePushButtonGadget(capStyle_pull,
						   "capProjecting", NULL, 0);
    XtAddCallback(cap_wid[3], XmNactivateCallback,
		  (XtCallbackProc)ModifyCapStyle, (XtPointer)CAPPROJECT);
    XtAddCallback(cap_wid[3], XmNactivateCallback,
		  (XtCallbackProc)ValidFunction, (XtPointer)capStyleToggle);
    XtAddCallback(cap_wid[3], XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)ALL_ATTRIB_FILE);
    XtManageChild(cap_wid[3]);
}


static void CreateJoinStyle(Widget parent)
{
    joinStyle_label = XmCreateLabelGadget(parent, "JoinStyleLabel", NULL, 0);
    XtManageChild(joinStyle_label);
    XtAddCallback(joinStyle_label, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)ALL_ATTRIB_FILE);
    Widget joinStyle_pull = XmCreatePulldownMenu(parent, "JoinStylePull",
						 NULL, 0);
    Arg args[5];
    int n = 0;
    XtSetArg(args[n], XmNsubMenuId, joinStyle_pull); n++;
    XtSetArg(args[n], XmNmarginWidth, 0); n++;
    XtSetArg(args[n], XmNmarginHeight, 0); n++;
    joinStyle_menu = XmCreateOptionMenu(parent, "JoinStyleMenu", args, n);
    XtManageChild(joinStyle_menu);
    n = 0;
    XtSetArg(args[n], XmNlabelPixmap, joinMiterIcon); n++;
    XtSetArg(args[n], XmNlabelType, XmPIXMAP); n++;
    join_wid[JOINMITER] = XmCreatePushButtonGadget(joinStyle_pull,
						   "joinMitter", args, n);
    XtAddCallback(join_wid[JOINMITER], XmNactivateCallback,
		  (XtCallbackProc)ModifyJoinStyle, (XtPointer)JOINMITER);
    XtAddCallback(join_wid[JOINMITER], XmNactivateCallback,
		  (XtCallbackProc)ValidFunction, (XtPointer)joinStyleToggle);
    XtAddCallback(join_wid[JOINMITER], XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)ALL_ATTRIB_FILE);
    n = 0;
    XtSetArg(args[n], XmNlabelPixmap, joinRoundIcon); n++;
    XtSetArg(args[n], XmNlabelType, XmPIXMAP); n++;
    join_wid[JOINROUND] = XmCreatePushButtonGadget(joinStyle_pull, "joinRound",
						   args, n);
    XtAddCallback(join_wid[JOINROUND], XmNactivateCallback,
		  (XtCallbackProc)ModifyJoinStyle, (XtPointer)JOINROUND);
    XtAddCallback(join_wid[JOINROUND], XmNactivateCallback,
		  (XtCallbackProc)ValidFunction, (XtPointer)joinStyleToggle);
    XtAddCallback(join_wid[JOINROUND], XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)ALL_ATTRIB_FILE);
    n = 0;
    XtSetArg(args[n], XmNlabelPixmap, joinBevelIcon); n++;
    XtSetArg(args[n], XmNlabelType, XmPIXMAP); n++;
    join_wid[JOINBEVEL] = XmCreatePushButtonGadget(joinStyle_pull, "joinBevel",
						   args, n);
    XtAddCallback(join_wid[JOINBEVEL], XmNactivateCallback,
		  (XtCallbackProc)ModifyJoinStyle, (XtPointer)JOINBEVEL);
    XtAddCallback(join_wid[JOINBEVEL], XmNactivateCallback,
		  (XtCallbackProc)ValidFunction, (XtPointer)joinStyleToggle);
    XtAddCallback(join_wid[JOINBEVEL], XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)ALL_ATTRIB_FILE);
    XtManageChildren(join_wid, 3);
}


static void CreateBrushColors(Widget parent)
{
    brush_color_label = XmCreateLabelGadget(parent, "BrushColorLabel", NULL,
					    0);
    XtAddCallback(brush_color_label, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)ALL_ATTRIB_FILE);
    XtManageChild(brush_color_label);
    Arg args[2];
    int n = 0;
    XtSetArg(args[n], XmNmarginWidth, 5); n++;
    XtSetArg(args[n], XmNmarginHeight, 5); n++;
    brush_color_list = XmCreatePushButtonGadget(parent, "BrushColorList",
						args, n);
    XtAddCallback(brush_color_list, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)ALL_ATTRIB_FILE);
    XtManageChild(brush_color_list);
    edit_brush_color = XmCreateTextField(parent, "BrushColorName", NULL, 0);
    XtManageChild(edit_brush_color);
    XtAddCallback(brush_color_list, XmNactivateCallback,
		  (XtCallbackProc)MapColorList, (XtPointer)edit_brush_color);
    XtAddCallback(edit_brush_color, XmNactivateCallback,
		  (XtCallbackProc) ModifyEditColor, (XtPointer) BRUSH_COLOR);
    XtAddCallback(edit_brush_color, XmNactivateCallback,
		  (XtCallbackProc) ValidFunction, (XtPointer) brushColorToggle);
    XtAddCallback(edit_brush_color, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)ALL_ATTRIB_FILE);
}


static void CreateFillRule(Widget parent)
{
    fillRule_label = XmCreateLabelGadget(parent, "FillRuleLabel",
						NULL, 0);
    XtAddCallback(fillRule_label, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)ALL_ATTRIB_FILE);
    XtManageChild(fillRule_label);
    Widget fillRule_pull = XmCreatePulldownMenu(parent, "FillRulePull",
						NULL, 0);
    Arg args[5];
    int n = 0;
    XtSetArg(args[n], XmNsubMenuId, fillRule_pull); n++;
    XtSetArg(args[n], XmNmarginWidth, 0); n++;
    XtSetArg(args[n], XmNmarginHeight, 0); n++;
    fillRule_menu = XmCreateOptionMenu(parent, "FillRuleMenu", args, n);
    XtManageChild(fillRule_menu);
    n = 0;
    XtSetArg(args[n], XmNlabelPixmap, evenOddRuleIcon); n++;
    XtSetArg(args[n], XmNlabelType, XmPIXMAP); n++;
    rule_wid[EVENODDRULE] = XmCreatePushButtonGadget(fillRule_pull,
						     "evenOddRule", args, n);
    XtAddCallback(rule_wid[EVENODDRULE], XmNactivateCallback,
		  (XtCallbackProc)ModifyFillRule,(XtPointer) EVENODDRULE);
    XtAddCallback(rule_wid[EVENODDRULE], XmNactivateCallback,
		  (XtCallbackProc)ValidFunction, (XtPointer)fillRuleToggle);
    XtAddCallback(rule_wid[EVENODDRULE], XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)ALL_ATTRIB_FILE);
    n = 0;
    XtSetArg(args[n], XmNlabelPixmap, windingRuleIcon); n++;
    XtSetArg(args[n], XmNlabelType, XmPIXMAP); n++;
    rule_wid[WINDINGRULE] = XmCreatePushButtonGadget(fillRule_pull,
						     "windingRule", args, n);
    XtAddCallback(rule_wid[WINDINGRULE], XmNactivateCallback,
		  (XtCallbackProc)ModifyFillRule, (XtPointer)WINDINGRULE);
    XtAddCallback(rule_wid[WINDINGRULE], XmNactivateCallback,
		  (XtCallbackProc)ValidFunction, (XtPointer)fillRuleToggle);
    XtAddCallback(rule_wid[WINDINGRULE], XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)ALL_ATTRIB_FILE);
    XtManageChildren(rule_wid, 2);
}


static void CreateTextColors(Widget parent)
{
    text_color_label = XmCreateLabelGadget(parent, "TextColorLabel", NULL, 0);
    XtAddCallback(text_color_label, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)ALL_ATTRIB_FILE);
    XtManageChild(text_color_label);
    Arg args[2];
    int n = 0;
    XtSetArg(args[n], XmNmarginWidth, 5); n++;
    XtSetArg(args[n], XmNmarginHeight, 5); n++;
    text_color_list = XmCreatePushButtonGadget(parent, "TextColorList",
					       args, n);
    XtAddCallback(text_color_list, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)ALL_ATTRIB_FILE);
    XtManageChild(text_color_list);
    edit_text_color = XmCreateTextField(parent, "TextColorName", NULL, 0);
    XtManageChild(edit_text_color);
    XtAddCallback(text_color_list, XmNactivateCallback,
		  (XtCallbackProc)MapColorList, (XtPointer)edit_text_color);
    XtAddCallback(edit_text_color, XmNactivateCallback,
		  (XtCallbackProc) ModifyEditColor, (XtPointer) TEXT_COLOR);
    XtAddCallback(edit_text_color, XmNactivateCallback,
		  (XtCallbackProc) ValidFunction, (XtPointer) textColorToggle);
    XtAddCallback(edit_text_color, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)ALL_ATTRIB_FILE);
}


static void CreateFont(Widget parent)
{
    font_label = XmCreateLabelGadget(parent, "FontLabel", NULL, 0);
    XtAddCallback(font_label, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)ALL_ATTRIB_FILE);
    XtManageChild(font_label);
    Arg args[2];
    int n = 0;
    XtSetArg(args[n], XmNmarginWidth, 5); n++;
    XtSetArg(args[n], XmNmarginHeight, 5); n++;
    font_list = XmCreatePushButtonGadget(parent, "FontList", args, n);
    XtAddCallback(font_list, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)ALL_ATTRIB_FILE);
    XtAddCallback(font_list, XmNactivateCallback,
		  (XtCallbackProc)MapFontList, (XtPointer)EDIT);
    XtManageChild(font_list);
    
    // Create XmText because of a XmTextField bug when string is too long;
    n = 0;
    XtSetArg(args[n], XmNeditable, False); n++;
    edit_text_font = XmCreateText(parent, "FontName", args, n);
    XtAddCallback(edit_text_font, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)ALL_ATTRIB_FILE);
    XtManageChild(edit_text_font);
}





/**
FUNCTION
    ModifyLineStyle()
DESCRIPTION
    
**/
static void ModifyLineStyle(Widget, int ls, caddr_t)
{
    WhichAttributes = EDIT;
    ModifyGoLineStyle(ls);
}



/**
FUNCTION
    ModifyCapStyle()
DESCRIPTION
    
**/
static void ModifyCapStyle(Widget, int ls, caddr_t)
{
    WhichAttributes = EDIT;
    ModifyGoCapStyle(ls);
}



/**
FUNCTION
    ModifyJoinStyle()
DESCRIPTION
    
**/
static void ModifyJoinStyle(Widget, int ls, caddr_t)
{
    WhichAttributes = EDIT;
    ModifyGoJoinStyle(ls);
}



/**
FUNCTION
    ModifyFillRule()
DESCRIPTION
    
**/
static void ModifyFillRule(Widget, int ls, caddr_t)
{
    WhichAttributes = EDIT;
    ModifyGoFillRule(ls);
}



/**
FUNCTION
    ModifyAllSheet()
DESCRIPTION
    
**/
static void ModifyAllSheet()
{
    WhichAttributes = EDIT;
    ModifyGoEditPenSheet();
    ModifyGoEditBrushSheet();
    ModifyGoEditTextSheet();

}



/*****************************************************************************/


/**
FUNCTION
    ReinitAllSheet()
DESCRIPTION
    
**/
static void ReinitAllSheet()
{
    WhichAttributes = EDIT;

    XmTextFieldSetString(lineWidth, pen.LineWidth);
    XmTextFieldSetString(edit_pen_color, pen.Color);
    XmTextFieldSetString(edit_pen_alt_color, pen.AltColor);
    
    Arg arg;
    XtSetArg(arg, XmNmenuHistory, line_wid[pen.LineStyle]);
    XtSetValues(lineStyle_menu, &arg, 1);
    
    XtSetArg(arg, XmNmenuHistory, cap_wid[pen.CapStyle]);
    XtSetValues(capStyle_menu, &arg, 1);
    
    XtSetArg(arg, XmNmenuHistory, join_wid[pen.JoinStyle]);
    XtSetValues(joinStyle_menu, &arg, 1);

    XmTextFieldSetString(edit_brush_color, brush.Color);
    
    XtSetArg(arg, XmNmenuHistory, rule_wid[brush.FillRule]);
    XtSetValues(fillRule_menu, &arg, 1);

    XmTextFieldSetString(edit_text_color, text.Color);
    XmTextSetString(edit_text_font, text.Font);

    if (WhichObject & PEN) {
	XmToggleButtonGadgetSetState(penColorToggle, True, False);
	XmToggleButtonGadgetSetState(altColorToggle, True, False);
	XmToggleButtonGadgetSetState(joinStyleToggle, True, False);
	XmToggleButtonGadgetSetState(lineWidthToggle, True, False);
	XmToggleButtonGadgetSetState(lineStyleToggle, True, False);
	XmToggleButtonGadgetSetState(capStyleToggle, True, False);
    } else {
	XmToggleButtonGadgetSetState(penColorToggle, False, False);
	XmToggleButtonGadgetSetState(altColorToggle, False, False);
	XmToggleButtonGadgetSetState(joinStyleToggle, False, False);
	XmToggleButtonGadgetSetState(lineWidthToggle, False, False);
	XmToggleButtonGadgetSetState(lineStyleToggle, False, False);
	XmToggleButtonGadgetSetState(capStyleToggle, False, False);
    }
    if (WhichObject & BRUSH) {
	XmToggleButtonGadgetSetState(brushColorToggle, True, False);
	XmToggleButtonGadgetSetState(fillRuleToggle, True, False);
    } else {
	XmToggleButtonGadgetSetState(brushColorToggle, False, False);
	XmToggleButtonGadgetSetState(fillRuleToggle, False, False);
    }
    if (WhichObject & TEXTBRUSH) {
	XmToggleButtonGadgetSetState(textColorToggle, True, False);
	XmToggleButtonGadgetSetState(fontToggle, True, False);
    } else {
	XmToggleButtonGadgetSetState(textColorToggle, False, False);
	XmToggleButtonGadgetSetState(fontToggle, False, False);
    }

    WhichObject = NONE;
    ModifyGoEditPenSheet();
    ModifyGoEditBrushSheet();
    ModifyGoEditTextSheet();

}


/**
FUNCTION
    ReinitAll()
DESCRIPTION
    
**/
void ReinitAll()
{
    pen = all_attrib.pen;
    brush = all_attrib.brush;
    text = all_attrib.text;
    if (allDialog)
	ReinitAllSheet();
}


/**
FUNCTION
    ReinitToggleField()
DESCRIPTION
    
**/
static void ReinitToggleField(Widget w, caddr_t,
			      XmToggleButtonCallbackStruct* data)
{
    if (! data->set) {
	Arg arg;
	if (w == penColorToggle) {
	    XmTextFieldSetString(edit_pen_color, pen.Color);
	    ModifyEditColor(edit_pen_color, PEN_COLOR, NULL);
	} else
	if (w == altColorToggle) {
	    XmTextFieldSetString(edit_pen_alt_color, pen.AltColor);
	    ModifyEditColor(edit_pen_alt_color, PEN_ALT_COLOR, NULL);
	} else
	if (w == lineWidthToggle) {
	    XmTextFieldSetString(lineWidth, pen.LineWidth);
	    ModifyLineWidth(lineWidth, EDIT, NULL);
	} else
	if (w == lineStyleToggle) {
	    XtSetArg(arg, XmNmenuHistory, line_wid[pen.LineStyle]);
	    XtSetValues(lineStyle_menu, &arg, 1);
	    ModifyLineStyle(NULL, pen.LineStyle, NULL);
	} else
	if (w == joinStyleToggle) {
	    XtSetArg(arg, XmNmenuHistory, join_wid[pen.JoinStyle]);
	    XtSetValues(joinStyle_menu, &arg, 1);
	    ModifyJoinStyle(NULL, pen.JoinStyle, NULL);
	} else
	if (w == capStyleToggle) {
	    XtSetArg(arg, XmNmenuHistory, cap_wid[pen.CapStyle]);
	    XtSetValues(capStyle_menu, &arg, 1);
	    ModifyCapStyle(NULL, pen.CapStyle, NULL);
	} else
	if (w == brushColorToggle) {
	    XmTextFieldSetString(edit_brush_color, brush.Color);
	    ModifyEditColor(edit_brush_color, BRUSH_COLOR, NULL);
	} else
	if (w == fillRuleToggle) {
	    XtSetArg(arg, XmNmenuHistory, rule_wid[brush.FillRule]);
	    XtSetValues(fillRule_menu, &arg, 1);
	    ModifyFillRule(NULL, brush.FillRule, NULL);
	} else
	if (w == textColorToggle) {
	    XmTextFieldSetString(edit_text_color, text.Color);
	    ModifyEditColor(edit_text_color, TEXT_COLOR, NULL);
	} else
	if (w == fontToggle) {
	    XmTextSetString(edit_text_font, text.Font);
	    ModifyGoFont(text.Font);
	}
    }
}


/**
FUNCTION
    Verify()
DESCRIPTION
    
**/
static void Verify(Widget, caddr_t parent, caddr_t)
{
    Arg arg;
    Widget w;
    WhichAttributes = EDIT;

    reinit_pen = pen;
    reinit_brush = brush;
    reinit_text = text;
    
    XtSetArg(arg, XmNmenuHistory, &w);
    XtGetValues(lineStyle_menu, &arg, 1);
    pen.LineStyle = GetItem(line_wid, 3, w);

    XtSetArg(arg, XmNmenuHistory, &w);
    XtGetValues(capStyle_menu, &arg, 1);
    pen.CapStyle = GetItem(cap_wid, 4, w);

    XtSetArg(arg, XmNmenuHistory, &w);
    XtGetValues(joinStyle_menu, &arg, 1);
    pen.JoinStyle = GetItem(join_wid, 3, w);

    GFREE(pen.LineWidth);
    pen.LineWidth = XmTextFieldGetString(lineWidth);

    GFREE(pen.Color);
    pen.Color = XmTextFieldGetString(edit_pen_color);

    GFREE(pen.AltColor);
    pen.AltColor = XmTextFieldGetString(edit_pen_alt_color);

    XtSetArg(arg, XmNmenuHistory, &w);
    XtGetValues(fillRule_menu, &arg, 1);
    brush.FillRule = GetItem(rule_wid, 2, w);

    GFREE(brush.Color);
    brush.Color = XmTextFieldGetString(edit_brush_color);

    GFREE(text.Color);
    text.Color = XmTextFieldGetString(edit_text_color);

    GFREE(text.Font);
    text.Font = XmTextGetString(edit_text_font);

    if (pen != reinit_pen) {
	GFREE(pen.PenText);
	pen.PenText = (char*)GMALLOC(strlen(UNNAMED) + 1);
	strcpy(pen.PenText, UNNAMED);
	XmTextFieldSetString(penTextEdit, UNNAMED);
    }
    if (brush != reinit_brush) {
	GFREE(brush.BrushText);
	brush.BrushText = (char*)GMALLOC(strlen(UNNAMED) + 1);
	strcpy(brush.BrushText, UNNAMED);
	XmTextFieldSetString(brushTextEdit, UNNAMED);
    }
    if (text != reinit_text) {
	GFREE(text.TextBrushText);
	text.TextBrushText = (char*)GMALLOC(strlen(UNNAMED) + 1);
	strcpy(text.TextBrushText, UNNAMED);
	XmTextFieldSetString(textBrushTextEdit, UNNAMED);
    }

    XtUnmanageChild((Widget)parent);
}

