#ifdef REV_INFO
static char rcsid[] = "$Id: brushsheet.cc,v 1.3 1992/12/10 16:24:19 jojic Exp $";
#endif
/*****************************************************************************
MODULE:  Go Editor
FILE:    brushsheet.cc
AUTHOR:  Aline Jacomet
DATE:    May 22th 1991

DESCRIPTION:
    

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include <Go/Tools/StringAssoc.h>
#include <Go/Tools/Alloc.h>

#include <Xm/Form.h>
#include <Xm/PushBG.h>
#include <Xm/SeparatoG.h>
#include <Xm/LabelG.h>
#include <Xm/TextF.h>
#include <Xm/RowColumn.h>

#include "classes.h"
#include "main.h"




/***
 * widget function
 ***/
static Widget CreateBrushDialog(Widget);

void MapBrushAttrib(void*, void*, caddr_t);

static void ModifyFillRule(Widget, int ls, caddr_t);
static void ModifyBrushSheet();
static void ReinitBrushSheet(Widget, Boolean, caddr_t);
static void Verify(Widget, caddr_t, caddr_t);
static void NamedChooseGo(Widget, caddr_t, caddr_t);
void VerifyBrush(Widget, caddr_t, caddr_t);


/***
 *  Static Variables
 ***/
static Widget brushDialog;
static Widget name;
static Widget color_label;
static Widget color_list;
static Widget fillRule_menu;
static Widget rule_wid[2];



/***
 *  Global output attributes
 ***/
BrushAttributes named_brush;

Widget brush_color = NULL;

Boolean brushMapped = False;

extern Widget brushText;

extern GStringAssoc NamedBrush;
extern AllAttributes all_attrib;



/*
 * EXTERN FUNCTIONS
 */

// in file namedattrib.cc;
extern void SensitiveButtons(Widget, Boolean*, caddr_t);
extern void UnmanageWidget(Widget, Widget, caddr_t);
extern void ModifyNamedColor(Widget, int, caddr_t);

// in file colorsel.cc;
extern void MapColorList(Widget, Widget, caddr_t);

// in file goAttrib.cc;
extern void ModifyGoFillRule(int);
extern void ModifyGoNamedBrushSheet();
extern void ModifyGoNamedBrush();
extern void CreateGoNamedBrush();
extern void ChooseGo(int, void (*)(int, class Go*), int);

// in file message.cc;
extern void ObjectName(void*);
extern void AcknowledgeBrushName(void*);

// in file pensheet.cc;
extern int GetItem(Widget*, int, Widget);

// in file help.cc;
extern void MapHelpFile(void*, char*, void*);



/**
FUNCTION
    MapBrushAttrib()
DESCRIPTION
    Maps and creates, if necessary, the brush attributes editor.
**/
void MapBrushAttrib(void*, void* parent, caddr_t)
{
    Boolean create = (brushDialog == NULL);

    if (!create && XtIsManaged(brushDialog))
	return;
    if (create) {
	XDefineCursor(XtDisplay((Widget)graphic), XtWindow((Widget)graphic),
		      clock_cursor);
	XDefineCursor(XtDisplay((Widget)main_window),
		      XtWindow((Widget)main_window), clock_cursor);
	XFlush(XtDisplay((Widget)graphic));
	brushDialog = CreateBrushDialog((Widget)parent);
    }

    ReinitBrushSheet(NULL, TRUE, NULL);
    XtManageChild(brushDialog);
    
    if (create) {
	XFlush(XtDisplay((Widget)graphic));
	XUndefineCursor(XtDisplay((Widget)graphic), XtWindow((Widget)graphic));
	XUndefineCursor(XtDisplay((Widget)main_window),
			XtWindow((Widget)main_window));
    }
    brushMapped = True;
}


/**
FUNCTION
    CreateBrushDialog()
DESCRIPTION
    Creates the brush attributes editor.
**/
static Widget CreateBrushDialog(Widget parent)
{
    Widget w;
    Arg args[20];
    int n = 0;
    XtSetArg(args[n], XmNautoUnmanage, False); n++;
    XtSetArg(args[n], XmNresizePolicy, XmRESIZE_NONE); n++;
    Widget ps = XmCreateFormDialog(parent, "BrushDialog", args, n);
    XtAddCallback(ps, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)BRUSH_ATTRIB_FILE);

    /* Apply, Reinit, Cancel buttons */
    w = XmCreatePushButtonGadget(ps, "Apply", NULL, 0);
    XtManageChild(w);
    XtAddCallback(w, XmNactivateCallback,
		  (XtCallbackProc)SensitiveButtons, (XtPointer)&brushMapped);
    XtAddCallback(w, XmNactivateCallback, (XtCallbackProc)Verify,
		  (XtPointer)ps);
    XtAddCallback(w, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)BRUSH_ATTRIB_FILE);

    w = XmCreatePushButtonGadget(ps, "Reinit", NULL, 0);
    XtManageChild(w);
    XtAddCallback(w, XmNactivateCallback, (XtCallbackProc)ReinitBrushSheet,
		  (XtPointer) TRUE);
    XtAddCallback(w, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)BRUSH_ATTRIB_FILE);
    
    w = XmCreatePushButtonGadget(ps, "Cancel", NULL, 0);
    XtManageChild(w);
    XtAddCallback(w, XmNactivateCallback,
		  (XtCallbackProc)SensitiveButtons, (XtPointer)&brushMapped);
    XtAddCallback(w, XmNactivateCallback,
		  (XtCallbackProc) ModifyBrushSheet, NULL);
    XtAddCallback(w, XmNactivateCallback, (XtCallbackProc) UnmanageWidget,
		  (XtPointer) ps);
    XtAddCallback(w, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)BRUSH_ATTRIB_FILE);

    /* click button */
    w = XmCreatePushButtonGadget(ps, "ClickButton", NULL, 0);
    XtManageChild(w);
    XtAddCallback(w, XmNactivateCallback, (XtCallbackProc)NamedChooseGo, NULL);
    XtAddCallback(w, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)BRUSH_ATTRIB_FILE);

    /* label and separators */
    w = XmCreateSeparatorGadget(ps, "Separator1", NULL, 0);
    XtAddCallback(w, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)BRUSH_ATTRIB_FILE);
    XtManageChild(w);
    w = XmCreateSeparatorGadget(ps, "Separator2", NULL, 0);
    XtAddCallback(w, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)BRUSH_ATTRIB_FILE);
    XtManageChild(w);
    w = XmCreateLabelGadget(ps, "BrushLabel", NULL, 0);
    XtManageChild(w);
    XtAddCallback(w, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)BRUSH_ATTRIB_FILE);
    
    /* name */
    Widget nameLabel = XmCreateLabelGadget(ps, "NameLabel", NULL, 0);
    XtAddCallback(nameLabel, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)BRUSH_ATTRIB_FILE);
    XtManageChild(nameLabel);

    name = XmCreateTextField(ps, "Name", NULL, 0);
    XtAddCallback(name, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)BRUSH_ATTRIB_FILE);
    XtManageChild(name);

    /* color */
    color_label = XmCreateLabelGadget(ps, "ColorLabel", NULL, 0);
    XtAddCallback(color_label, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)BRUSH_ATTRIB_FILE);
    XtManageChild(color_label);
    n = 0;
    XtSetArg(args[n], XmNmarginWidth, 5); n++;
    XtSetArg(args[n], XmNmarginHeight, 5); n++;
    color_list = XmCreatePushButtonGadget(ps, "ColorList", args, n);
    XtAddCallback(color_list, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)BRUSH_ATTRIB_FILE);
    XtManageChild(color_list);
    brush_color = XmCreateTextField(ps, "ColorName", NULL, 0);
    XtManageChild(brush_color);
    XtAddCallback(color_list, XmNactivateCallback,
		  (XtCallbackProc)MapColorList, (XtPointer)brush_color);
    XtAddCallback(brush_color, XmNactivateCallback,
		  (XtCallbackProc) ModifyNamedColor, (XtPointer) BRUSH_COLOR);
    XtAddCallback(brush_color, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)BRUSH_ATTRIB_FILE);

    /* fill rule */
    Widget fillRule_label = XmCreateLabelGadget(ps, "FillRuleLabel", NULL, 0);
    XtAddCallback(fillRule_label, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)BRUSH_ATTRIB_FILE);
    XtManageChild(fillRule_label);
    Widget fillRule_pull = XmCreatePulldownMenu(ps, "FillRulePull", NULL, 0);
    n = 0;
    XtSetArg(args[n], XmNsubMenuId, fillRule_pull); n++;
    XtSetArg(args[n], XmNmarginWidth, 0); n++;
    XtSetArg(args[n], XmNmarginHeight, 0); n++;
    fillRule_menu = XmCreateOptionMenu(ps, "FillRuleMenu", args, n);
    XtManageChild(fillRule_menu);
    n = 0;
    XtSetArg(args[n], XmNlabelPixmap, evenOddRuleIcon); n++;
    XtSetArg(args[n], XmNlabelType, XmPIXMAP); n++;
    rule_wid[EVENODDRULE] = XmCreatePushButtonGadget(fillRule_pull,
						     "evenOddRule", args, n);
    XtAddCallback(rule_wid[EVENODDRULE], XmNactivateCallback,
		  (XtCallbackProc)ModifyFillRule,(XtPointer) EVENODDRULE);
    XtAddCallback(rule_wid[EVENODDRULE], XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)BRUSH_ATTRIB_FILE);
    n = 0;
    XtSetArg(args[n], XmNlabelPixmap, windingRuleIcon); n++;
    XtSetArg(args[n], XmNlabelType, XmPIXMAP); n++;
    rule_wid[WINDINGRULE] = XmCreatePushButtonGadget(fillRule_pull,
						     "windingRule", args, n);
    XtAddCallback(rule_wid[WINDINGRULE], XmNactivateCallback,
		  (XtCallbackProc)ModifyFillRule, (XtPointer)WINDINGRULE);
    XtAddCallback(rule_wid[WINDINGRULE], XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)BRUSH_ATTRIB_FILE);
    XtManageChildren(rule_wid, 2);

    
    /* attach widgets */
    n = 0;
    XtSetArg(args[n], XmNleftAttachment, XmATTACH_OPPOSITE_WIDGET); n++;
    XtSetArg(args[n], XmNleftWidget, nameLabel); n++;
    XtSetValues(color_label, args, n);
    n = 0;
    XtSetArg(args[n], XmNleftAttachment, XmATTACH_OPPOSITE_WIDGET); n++;
    XtSetArg(args[n], XmNleftWidget, name); n++;
    XtSetValues(color_list, args, n);
    n = 0;
    XtSetArg(args[n], XmNrightAttachment, XmATTACH_OPPOSITE_WIDGET); n++;
    XtSetArg(args[n], XmNrightWidget, brush_color); n++;
    XtSetValues(name, args, n);

    return ps;
}





/**
FUNCTION
    ModifyFillRule()
DESCRIPTION
    
**/
static void ModifyFillRule(Widget, int ls, caddr_t)
{
    WhichAttributes = NAMED;
    ModifyGoFillRule(ls);
}



/**
FUNCTION
    ModifyBrushSheet()
DESCRIPTION
    
**/
static void ModifyBrushSheet()
{
    WhichAttributes = NAMED;
    ModifyGoNamedBrushSheet();
}



/*****************************************************************************/


/**
FUNCTION
    ReinitBrushSheet()
DESCRIPTION
    
**/
static void ReinitBrushSheet(Widget, Boolean reinit_name, caddr_t)
{
    if (reinit_name)
        XmTextFieldSetString(name, named_brush.BrushText);
    XmTextFieldSetString(brush_color, named_brush.Color);
    
    Arg arg;
    XtSetArg(arg, XmNmenuHistory, rule_wid[named_brush.FillRule]);
    XtSetValues(fillRule_menu, &arg, 1);

    WhichAttributes = NAMED;
    ModifyGoNamedBrushSheet();
}


/**
FUNCTION
    ReinitBrush()
DESCRIPTION
    
**/
void ReinitBrush()
{
    named_brush = all_attrib.brush;
    if (brushDialog)
	ReinitBrushSheet(NULL, TRUE, NULL);
}


/**
FUNCTION
    Verify()
DESCRIPTION
    
**/
static void Verify(Widget, caddr_t parent, caddr_t)
{
    WhichAttributes = NAMED;
    char *name_str = XmTextFieldGetString(name);
    if (name_str[0] == '\0' ||
	strcmp(name_str, UNNAMED) == 0) {
	GFREE(name_str);
	ObjectName(brushDialog);
	return;
    }
    if (NamedBrush.get(name_str)) {
	AcknowledgeBrushName(parent);
	GFREE(name_str);
	return;
    }
    VerifyBrush(NULL, parent, NULL);
    GFREE(name_str);
}


void VerifyBrush(Widget, caddr_t parent, caddr_t)
{
    Arg arg;
    Widget w;

    char *name_str = XmTextFieldGetString(name);

    XtSetArg(arg, XmNmenuHistory, &w);
    XtGetValues(fillRule_menu, &arg, 1);
    named_brush.FillRule = GetItem(rule_wid, 2, w);

    GFREE(named_brush.Color);
    named_brush.Color = XmTextFieldGetString(brush_color);

    if (strcmp(name_str, UNNAMED) != 0) {
	GFREE(named_brush.BrushText);
	named_brush.BrushText = (char*)GMALLOC(strlen(name_str) + 1);
	strcpy(named_brush.BrushText, name_str);
	if (NamedBrush.get(name_str))
	    ModifyGoNamedBrush();
	else
	    CreateGoNamedBrush();
	XmTextFieldSetString(brushText, named_brush.BrushText);
    }

    if (strcmp(all_attrib.BrushText, name_str) == 0) {
	/*
	 * This brush is used as current attributes ==> I have to
	 * modify all_attrib.brush;
	 */
	all_attrib.brush = named_brush;
    }
    GFREE(name_str);

    XtUnmanageChild((Widget)parent);
}



/**
FUNCTION
    NamedChooseGo()
DESCRIPTION
    
**/
static void NamedChooseGo(Widget, caddr_t, caddr_t)
{
    WhichAttributes = NAMED;
    ChooseGo(BRUSH, NULL, 0);
    ReinitBrushSheet(NULL, FALSE, NULL);
}

