#ifdef REV_INFO
static char rcsid[] = "$Id: classes.cc,v 1.1.1.1 1992/03/18 09:04:06 uep Exp $";
#endif
/*****************************************************************************
MODULE:  Go Editor
FILE:    classes.cc
AUTHOR:  Aline Jacomet
DATE:    May 31th 1991

DESCRIPTION:
    

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include <string.h>

#include <Go/Tools/Alloc.h>

#include "classes.h"



/**
FUNCTION
    AllAttributes()
DESCRIPTION
    constructor of AllAttributes class.
**/
AllAttributes::AllAttributes()
{
    BackgroundIsSet = '\001';
    OutlineIsSet = '\001';
    RoundedIsSet = '\000';
    PenText = (char*)GMALLOC(strlen(UNNAMED) + 1);
    strcpy(PenText, UNNAMED);
    BrushText = (char*)GMALLOC(strlen(UNNAMED) + 1);
    strcpy(BrushText, UNNAMED);
    TextBrushText = (char*)GMALLOC(strlen(UNNAMED) + 1);
    strcpy(TextBrushText, UNNAMED);
}


/**
FUNCTION
    ~AllAttributes()
DESCRIPTION
    destructor of AllAttributes class.
**/
 AllAttributes::~AllAttributes()
{
    GFREE(PenText);
    GFREE(BrushText);
    GFREE(TextBrushText);
}


/**
FUNCTION
    PenAttributes()
DESCRIPTION
    constructor of PenAttributes class.
**/
 PenAttributes::PenAttributes() : Rgb_Color(0, 0, 0), Rgb_AltColor(0, 0, 0)
{
    PenText = (char*)GMALLOC(strlen(UNNAMED) + 1);
    strcpy(PenText, UNNAMED);

    LineWidth = (char*)GMALLOC(strlen("0.0") + 1);
    strcpy(LineWidth, "0.0");

    Color = (char*)GMALLOC(strlen("black") + 1);
    strcpy(Color, "black");

    AltColor = (char*)GMALLOC(strlen("black") + 1);
    strcpy(AltColor, "black");

    LineStyle = LINESOLID;
    CapStyle = CAPBUTT;
    JoinStyle = JOINMITER;
}


/**
FUNCTION
    ~PenAttributes()
DESCRIPTION
    destructor of PenAttributes class.
**/
 PenAttributes::~PenAttributes()
{
    GFREE(PenText);
    GFREE(LineWidth);
    GFREE(Color);
    GFREE(AltColor);
}


/**
FUNCTION
    operator =
DESCRIPTION
    Operates = between two PenAttributes instances.
**/
void PenAttributes::operator = (const PenAttributes& p)
{
    GFREE(PenText);
    PenText = (char*)GMALLOC(strlen(p.PenText) + 1);
    strcpy(PenText, p.PenText);

    GFREE(LineWidth);
    LineWidth = (char*)GMALLOC(strlen(p.LineWidth) + 1);
    strcpy(LineWidth, p.LineWidth);

    LineStyle = p.LineStyle;
    CapStyle = p.CapStyle;
    JoinStyle = p.JoinStyle;

    GFREE(Color);
    Color = (char*)GMALLOC(strlen(p.Color) + 1);
    strcpy(Color, p.Color);

    Rgb_Color.red = p.Rgb_Color.red;
    Rgb_Color.green = p.Rgb_Color.green;
    Rgb_Color.blue = p.Rgb_Color.blue;

    GFREE(AltColor);
    AltColor = (char*)GMALLOC(strlen(p.AltColor) + 1);
    strcpy(AltColor, p.AltColor);

    Rgb_AltColor.red = p.Rgb_AltColor.red;
    Rgb_AltColor.green = p.Rgb_AltColor.green;
    Rgb_AltColor.blue = p.Rgb_AltColor.blue;
}


/**
FUNCTION
    operator !=
DESCRIPTION
    Operates != between two PenAttributes instances.
**/
unsigned char PenAttributes::operator != (const PenAttributes& p)
{
    if (strcmp(PenText, p.PenText) != 0)
	return '\001';
    if (strcmp(Color, p.Color) != 0)
	return '\001';
    if (strcmp(AltColor, p.AltColor) != 0)
	return '\001';
    if (LineStyle != p.LineStyle)
	return '\001';
    if (strcmp(LineWidth, p.LineWidth) != 0)
	return '\001';
    if (CapStyle != p.CapStyle)
	return '\001';
    if (JoinStyle != p.JoinStyle)
	return '\001';
    return '\000';
}


/**
FUNCTION
    BrushAttributes()
DESCRIPTION
    constructor of BrushAttributes class.
**/
 BrushAttributes::BrushAttributes() : Rgb_Color(GMAX_RGB_VALUE, GMAX_RGB_VALUE,
						GMAX_RGB_VALUE)
{
    BrushText = (char*)GMALLOC(strlen(UNNAMED) + 1);
    strcpy(BrushText, UNNAMED);
    FillRule = EVENODDRULE;
    Color = (char*)GMALLOC(strlen("white") + 1);
    strcpy(Color, "white");
}


/**
FUNCTION
    ~BrushAttributes()
DESCRIPTION
    destructor of BrushAttributes class.
**/
 BrushAttributes::~BrushAttributes()
{
    GFREE(BrushText);
    GFREE(Color);
}


/**
FUNCTION
    operator =
DESCRIPTION
    Operates = between two BrushAttributes instances.
**/
void BrushAttributes::operator = (const BrushAttributes& b)
{
    GFREE(BrushText);
    BrushText = (char*)GMALLOC(strlen(b.BrushText) + 1);
    strcpy(BrushText, b.BrushText);

    FillRule = b.FillRule;

    GFREE(Color);
    Color = (char*)GMALLOC(strlen(b.Color) + 1);
    strcpy(Color, b.Color);

    Rgb_Color.red = b.Rgb_Color.red;
    Rgb_Color.green = b.Rgb_Color.green;
    Rgb_Color.blue = b.Rgb_Color.blue;
}


/**
FUNCTION
    operator !=
DESCRIPTION
    Operates != between two BrushAttributes instances.
**/
unsigned char BrushAttributes::operator != (const BrushAttributes& b)
{
    if (strcmp(BrushText, b.BrushText) != 0)
	return '\001';
    if (strcmp(Color, b.Color) != 0)
	return '\001';
    if (FillRule != b.FillRule)
	return '\001';
    return '\000';
}


/**
FUNCTION
    TextAttributes()
DESCRIPTION
    constructor of Textttributes class.
**/
 TextAttributes::TextAttributes() : Rgb_Color(0, 0, 0)
{
    TextBrushText = (char*)GMALLOC(strlen(UNNAMED) + 1);
    strcpy(TextBrushText, UNNAMED);

    Font = (char*)GMALLOC(1);
    Font[0] = '\0';

    Color = (char*)GMALLOC(strlen("black") + 1);
    strcpy(Color, "black");
}



/**
FUNCTION
    ~TextAttributes()
DESCRIPTION
    destructor of TextAttributes class.
**/
 TextAttributes::~TextAttributes()
{
    GFREE(TextBrushText);
    GFREE(Font);
    GFREE(Color);
}


/**
FUNCTION
    operator =
DESCRIPTION
    Operates = between two TextAttributes instances.
**/
void TextAttributes::operator = (const TextAttributes& b)
{
    GFREE(TextBrushText);
    TextBrushText = (char*)GMALLOC(strlen(b.TextBrushText) + 1);
    strcpy(TextBrushText, b.TextBrushText);

    GFREE(Font);
    Font = (char*)GMALLOC(strlen(b.Font) + 1);
    strcpy(Font, b.Font);

    GFREE(Color);
    Color = (char*)GMALLOC(strlen(b.Color) + 1);
    strcpy(Color, b.Color);

    Rgb_Color.red = b.Rgb_Color.red;
    Rgb_Color.green = b.Rgb_Color.green;
    Rgb_Color.blue = b.Rgb_Color.blue;
}


/**
FUNCTION
    operator !=
DESCRIPTION
    Operates != between two TextAttributes instances.
**/
unsigned char TextAttributes::operator != (const TextAttributes& b)
{
    if (strcmp(TextBrushText, b.TextBrushText) != 0)
	return '\001';
    if (strcmp(Color, b.Color) != 0)
	return '\001';
    if (strcmp(Font, b.Font) != 0)
	return '\001';
    return '\000';
}

