#ifdef REV_INFO
static char rcsid[] = "$Id: colorsel.cc,v 1.2 1992/12/10 16:24:20 jojic Exp $";
#endif
/*****************************************************************************
MODULE:  Go Editor
FILE:    colorsel.cc
AUTHOR:  Aline Jacomet
DATE:    May 24th 1991

DESCRIPTION:
    Color Management

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include <stdio.h>

#include <Go/Tools/Set.h>
#include <Go/Tools/Rgb.h>
#include <Go/Tools/Alloc.h>

#include <Xm/TextF.h>
#include <Xm/ToggleBG.h>
#include <Xm/SelectioB.h>

#include "classes.h"
#include "main.h"


/***
 *  Global
 ***/
extern Widget pen_color;
extern Widget pen_alt_color;
extern Widget brush_color;
extern Widget text_color;
extern Widget edit_pen_color;
extern Widget edit_pen_alt_color;
extern Widget edit_brush_color;
extern Widget edit_text_color;
extern Widget penColorToggle;
extern Widget altColorToggle;
extern Widget brushColorToggle;
extern Widget textColorToggle;


extern int WhichAction;

/***
 *  Static Variables
 ***/
static Widget colorboxDialog;



/*
 * EXTERN FUNCTIONS
 */

// in file goAttrib.cc;
extern void ModifyGoColor(GRgb*, char*);

// in file message.cc;
extern void NotInList(void*, void*, caddr_t);
extern void Error(void*);



static void Apply(Widget w, caddr_t, XmSelectionBoxCallbackStruct* cb)
{
    char* color;
    XmStringGetLtoR(cb->value, XmSTRING_DEFAULT_CHARSET, &color);
    
    XColor exact_rgb, real_rgb;
    Display* dpy = XtDisplay(w);

    XLookupColor(dpy, DefaultColormap(dpy, DefaultScreen(dpy)), color,
		 &exact_rgb, &real_rgb);

    GRgb rgb(real_rgb.red, real_rgb.green, real_rgb.blue);
    switch(WhichColor) {
      case PEN_COLOR:
	if (WhichAttributes == NAMED)
	    XmTextFieldSetString(pen_color, color);
	else {
	    if (WhichAction != FAST_MODIFY) {
		XmTextFieldSetString(edit_pen_color, color);
		if (WhichAction == MODIFY)
		    XmToggleButtonGadgetSetState(penColorToggle, True,  False);
	    }
	}
	ModifyGoColor(&rgb, color);
	break;
      case PEN_ALT_COLOR:
	if (WhichAttributes == NAMED)
	    XmTextFieldSetString(pen_alt_color, color);
	else {
	    if (WhichAction != FAST_MODIFY) {
		XmTextFieldSetString(edit_pen_alt_color, color);
		if (WhichAction == MODIFY)
		    XmToggleButtonGadgetSetState(altColorToggle, True,  False);
	    }
	}
	ModifyGoColor(&rgb, color);
	break;
      case BRUSH_COLOR:
	if (WhichAttributes == NAMED)
	    XmTextFieldSetString(brush_color, color);
	else {
	    if (WhichAction != FAST_MODIFY) {
		XmTextFieldSetString(edit_brush_color, color);
		if (WhichAction == MODIFY)
		    XmToggleButtonGadgetSetState(brushColorToggle, True,
						 False);
	    }
	}
	ModifyGoColor(&rgb, color);
	break;
      case TEXT_COLOR:
	if (WhichAttributes == NAMED)
	    XmTextFieldSetString(text_color, color);
	else {
	    if (WhichAction != FAST_MODIFY) {
		XmTextFieldSetString(edit_text_color, color);
		if (WhichAction == MODIFY)
		    XmToggleButtonGadgetSetState(textColorToggle, True,
						 False);
	    }
	}
	ModifyGoColor(&rgb, color);
	break;
    }
    GFREE(color);
}



static void CreateSelectionDialog()
{
    Arg args[2];

    int n = 0;
    XtSetArg(args[n], XmNmustMatch, True); n++;
    XtSetArg(args[n], XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL); n++;
    colorboxDialog = XmCreateSelectionDialog((Widget)main_window,
					       "ColorBox", args, n);
    XtAddCallback(colorboxDialog, XmNokCallback,
		  (XtCallbackProc)Apply, NULL);
    XtAddCallback(colorboxDialog, XmNnoMatchCallback,
		  (XtCallbackProc)NotInList, (XtPointer)main_window);
    Widget wid = XmSelectionBoxGetChild(colorboxDialog,
					XmDIALOG_APPLY_BUTTON);
    XtUnmanageChild(wid);
    wid = XmSelectionBoxGetChild(colorboxDialog, XmDIALOG_HELP_BUTTON);
    XtUnmanageChild(wid);

    FILE* file = fopen("/usr/lib/X11/rgb.txt", "r");
    if (file) {
	char tmp1[5], tmp2[5], tmp3[5];
	char buf1[50], buf2[20];
	buf2[0] = '\0';
	GSet list;
	while(fscanf(file, "%s%s%s%s%[^\n]",
		     tmp1, tmp2, tmp3, buf1, buf2) != EOF) {
	    strcat(buf1, buf2);
	    list.add((const void*)XmStringCreate(buf1,
						 XmSTRING_DEFAULT_CHARSET));
	    buf2[0] = '\0';
	}
	int nb;
	const void** tmp = list.allSet(nb);
	XtSetArg(args[0], XmNlistItems, (XmStringTable)tmp);
	XtSetArg(args[1], XmNlistItemCount, nb);
	XtSetValues(colorboxDialog, args, 2);
	fclose(file);
    } else {
	Error(main_window);
    }
    
}



/**
FUNCTION
    MapColorList()
DESCRIPTION
    Creates if necessary, and maps the color selection box.
**/
void MapColorList(Widget, Widget parent, caddr_t)
{
    Boolean create = (colorboxDialog == NULL);

    if (create) {
	XDefineCursor(XtDisplay((Widget)graphic), XtWindow((Widget)graphic),
		      clock_cursor);
	XDefineCursor(XtDisplay((Widget)main_window),
		      XtWindow((Widget)main_window), clock_cursor);
	XFlush(XtDisplay((Widget)graphic));
	CreateSelectionDialog();
    }

    if (parent == pen_color) {
	WhichColor = PEN_COLOR;
	WhichAttributes = NAMED;
    } else
	if (parent == pen_alt_color) {
	    WhichColor = PEN_ALT_COLOR;
	    WhichAttributes = NAMED;
	} else
	    if (parent == brush_color) {
		WhichColor = BRUSH_COLOR;
		WhichAttributes = NAMED;
	    } else
		if (parent == text_color) {
		    WhichColor = TEXT_COLOR;
		    WhichAttributes = NAMED;
		} else
		    if (parent == edit_pen_color) {
			WhichColor = PEN_COLOR;
			WhichAttributes = EDIT;
		    } else
			if (parent == edit_pen_alt_color) {
			    WhichColor = PEN_ALT_COLOR;
			    WhichAttributes = EDIT;
			} else
			    if (parent == edit_brush_color) {
				WhichColor = BRUSH_COLOR;
				WhichAttributes = EDIT;
			    } else
				if (parent == edit_text_color) {
				    WhichColor = TEXT_COLOR;
				    WhichAttributes = EDIT;
		}

    if (create) {
	XFlush(XtDisplay((Widget)graphic));
	XUndefineCursor(XtDisplay((Widget)graphic), XtWindow((Widget)graphic));
	XUndefineCursor(XtDisplay((Widget)main_window),
			XtWindow((Widget)main_window));
    }
    
    XtManageChild(colorboxDialog);
}


