#ifdef REV_INFO
static char rcsid[] = "$Id: curvemenu.cc,v 1.1.1.1 1992/03/18 09:04:40 uep Exp $";
#endif
/*****************************************************************************
MODULE:  Go Editor
FILE:    curvemenu.cc
AUTHOR:  Patricia Langle and Aline Jacomet
DATE:    May 14th 1991

DESCRIPTION:
    menu for curves

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include "main.h"

#include <Xm/PushBG.h>
#include <Xm/RowColumn.h>


void* curveBtn;


/*
 * EXTERN FUNCTIONS
 */

// in file goCreation.cc;
extern void CreateBezier(void*, void*, caddr_t);
extern void CreateBSpline(void*, void*, caddr_t);
extern void CreateBetaSpline(void*, void*, caddr_t);
extern void CreateCardinal(void*, void*, caddr_t);
extern void CreateTauSpline(void*, void*, caddr_t);

// in file help.cc;
extern void MapHelpFile(void*, char*, void*);



/**
FUNCTION
    InitCurvesEntries()
DESCRIPTION
    Creates curves creation menu and adds callbacks.
**/
void InitCurveEntries(Widget parent)
{
    XmString str = XmStringCreate("Bezier", XmSTRING_DEFAULT_CHARSET);
    Arg args[2];
    XtSetArg(args[0], XmNtraversalOn, False);
    XtSetArg(args[1], XmNlabelString, str);
    Widget button = XmCreatePushButtonGadget(parent, "bezier", args, 2);
    XtAddCallback (button, XmNactivateCallback,
		   (XtCallbackProc)CreateBezier, NULL);
    XtManageChild(button);
    XmStringFree(str);

    str = XmStringCreate("B-Spline", XmSTRING_DEFAULT_CHARSET);
    XtSetArg(args[1], XmNlabelString, str);
    button = XmCreatePushButtonGadget(parent, "bspline", args, 2);
    XtAddCallback (button, XmNactivateCallback,
		   (XtCallbackProc)CreateBSpline, NULL);
    XtManageChild(button);
    XmStringFree(str);

    str = XmStringCreate("Cardinal-Spline", XmSTRING_DEFAULT_CHARSET);
    XtSetArg(args[1], XmNlabelString, str);
    button = XmCreatePushButtonGadget(parent, "cardinal", args, 2);
    XtAddCallback (button, XmNactivateCallback,
		   (XtCallbackProc)CreateCardinal, NULL);
    XtManageChild(button);
    XmStringFree(str);

    str = XmStringCreate("Beta-Spline", XmSTRING_DEFAULT_CHARSET);
    XtSetArg(args[1], XmNlabelString, str);
    button = XmCreatePushButtonGadget(parent, "beta", args, 2);
    XtAddCallback (button, XmNactivateCallback,
		   (XtCallbackProc)CreateBetaSpline, NULL);
    XtManageChild(button);
    XmStringFree(str);

    str = XmStringCreate("Tau-Spline", XmSTRING_DEFAULT_CHARSET);
    XtSetArg(args[1], XmNlabelString, str);
    button = XmCreatePushButtonGadget(parent, "tau", args, 2);
    XtAddCallback (button, XmNactivateCallback,
		   (XtCallbackProc)CreateTauSpline, NULL);
    XtManageChild(button);
    XmStringFree(str);
}


/**
FUNCTION
    PostCurveMenu()
DESCRIPTION
    Displays menu at the right position.
**/
void PostCurveMenu(Widget w, Widget curve_popup, XmPushButtonCallbackStruct*)
{
    Arg args[10];
    Position root_x,root_y;
    XtTranslateCoords(w, 0, 0, &root_x, &root_y);
    int n = 0;
    XtSetArg(args[n], XmNx, root_x);  n++;
    XtSetArg(args[n], XmNy, root_y);  n++;
    XtSetValues(curve_popup, args, n);
    XtManageChild (curve_popup);
}


/**
FUNCTION
    CreateCurveButton()
DESCRIPTION
    Creates menu popup used to create curves.
**/
Widget CreateCurveButton(Widget parent)
{
    Arg args[4];
    XtSetArg(args[0], XmNtraversalOn, False);
    XtSetArg(args[1], XmNwhichButton, 1);
    Widget curve_popup = XmCreatePopupMenu(parent, "CurvePopup", args, 2);
    InitCurveEntries(curve_popup);
    int n = 0;
    XtSetArg(args[n], XmNlabelPixmap, curveIcon); n++;
    XtSetArg(args[n], XmNtraversalOn, False); n++;
    XtSetArg(args[n], XmNlabelType, XmPIXMAP); n++;
    XtSetArg(args[n], XmNlabelInsensitivePixmap, iCurveIcon); n++;
    curveBtn =  XmCreatePushButtonGadget(parent, "Curves", args, n);
    XtAddCallback ((Widget)curveBtn, XmNarmCallback,
		   (XtCallbackProc)PostCurveMenu, (XtPointer)curve_popup);
    XtAddCallback ((Widget)curveBtn, XmNhelpCallback,
		   (XtCallbackProc)MapHelpFile, CURVE_CR_FILE);
    return (Widget)curveBtn;
}

