#ifdef REV_INFO
static char rcsid[] = "$Id: editattrib.cc,v 1.3 1992/12/10 16:24:24 jojic Exp $";
#endif
/*****************************************************************************
MODULE:  Go Editor
FILE:    editattrib.cc
AUTHOR:  Aline Jacomet
DATE:    May 21th 1991

DESCRIPTION:
    

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include <Go/Tools/Rgb.h>
#include <Go/Tools/Alloc.h>

#include <string.h>

#include "classes.h"
#include "main.h"

#include <Xm/Form.h>
#include <Xm/PushBG.h>
#include <Xm/ToggleBG.h>
#include <Xm/Frame.h>
#include <Xm/RowColumn.h>
#include <Xm/CascadeBG.h>
#include <Xm/Label.h>
#include <Xm/LabelG.h>
#include <Xm/TextF.h>

#include <Xb/Graphic.h>


/***
 * widget creation
 ***/
static Widget CreateGCDialog(Widget);
static void CreateOutlineWid(Widget);
static void CreateBackgroundWid(Widget);
static void CreateRoundedWid(Widget);
static void CreatePenNameWid(Widget);
static void CreateBrushNameWid(Widget);
static void CreateTextBrushNameWid(Widget);
static void CreateClickMenu(Widget);

/***
 * feedback on the local graphic widget gcDraw
 ***/
static void ShowBackground(Widget, caddr_t, XmToggleButtonCallbackStruct*);
static void ShowOutline(Widget, void*, XmToggleButtonCallbackStruct*);
static void ShowRounded(Widget, void*, XmToggleButtonCallbackStruct*);


/***
 * reinit functions
 ***/
static void ReinitSheet();


/***
 * miscellaneous
 ***/
static void Apply();
static void PrepareGCDialog(Widget);
static void SetDefaultAttributes();
static void InitDialog();
static void UpdateSelectedObjects();

void ValidFunction(Widget, Widget, caddr_t);
void ManageValidationToggles(Boolean, Widget*, int);
static void EditChooseGo(Widget, int, caddr_t);

/***
 *  Static Variables
 ***/
static Widget editGcDialog;
static Widget editGcDraw;
static Widget editGcFrame;



/* Attributes subwidgets */
static Widget backgroundWid;
static Widget outlineWid;
static Widget roundedWid;
static Widget editWid;
static Widget penLabel;
static Widget brushLabel;
static Widget textBrushLabel;
static Widget penList;
static Widget brushList;
static Widget textBrushList;

/* Validation toggles (when modifying Gos) : one per attribute subwidget */
static Widget backgroundToggle;
static Widget outlineToggle;
static Widget roundedToggle;

static Widget apply;
static Widget cancel;
static Widget reinit;
static Widget menu_click;

static Widget valid_toggles[5];
static int num_valid_toggles = 0;


/****
 * Global
 ****/

PenAttributes   reinit_pen;
BrushAttributes reinit_brush;
TextAttributes  reinit_text;

PenAttributes   pen;
BrushAttributes brush;
TextAttributes  text;

AllAttributes all_attrib;

Widget penTextEdit;
Widget brushTextEdit;
Widget textBrushTextEdit;

Boolean allMapped = False;

int WhichColor;
int WhichAttributes;
int WhichAction;
int WhichObject;

Boolean MapEditAllAttrib;

extern Widget penColorToggle;
extern Widget altColorToggle;
extern Widget joinStyleToggle;
extern Widget lineStyleToggle;
extern Widget capStyleToggle;
extern Widget lineWidthToggle;
extern Widget brushColorToggle;
extern Widget fillRuleToggle;
extern Widget textColorToggle;
extern Widget fontToggle;

extern void* modif_entry;
extern void* current_entry;
extern void* pen_modif_entry;
extern void* brush_modif_entry;
extern void* text_modif_entry;
extern void* line_modif_entry;
extern void* font_modif_entry;



/*
 * EXTERN FUNCTION
 */

// in file goTools.cc;
extern int  GetNumMainSelectedObjects();

// in file message.cc;
extern void NoSelectedObject(void*);

// in help.cc;
extern void MapHelpFile(void*, char*, void*);

// in file editattrib.cc;
extern void SetSensitive(Widget, Widget);

// in file allsheet.cc;
extern void MapAllAttrib(void*, void*, caddr_t);

// in file selectionbox.cc;
extern void SelectNamedPen(void*, int, caddr_t);    
extern void SelectNamedBrush(void*, int, caddr_t);    
extern void SelectNamedText(void*, int, caddr_t);    

// in file namedattrib.cc;
extern void UnmanageWidget(Widget, Widget, caddr_t);
extern void VerifFunction(Widget, caddr_t, caddr_t);

// in file goAttrib.cc;
extern int HaveTheySamePen(int*, int*);
extern int HaveTheySameBrush(int*);
extern int HaveTheySameText();
extern void ModifyGoEditPenSheet();
extern void ModifyGoEditBrushSheet();
extern void ModifyGoEditTextSheet();
extern void ModifyGoRounded(int);
extern void ModifyGoOutline(int);
extern void ModifyGoBackground(int);
extern void ModifyGoPen(char*);
extern void ModifyGoJoinStyle(int);
extern void ModifyGoLineStyle(int);
extern void ModifyGoLineWidth(double);
extern void ModifyGoCapStyle(int);
extern void ModifyGoText(char*);
extern void ModifyGoFont(char*);
extern void ChooseGo(int, void (*)(int, class Go*), int);
extern void ModifyGoBrush(char*);
extern void ModifyGoFillRule(int);
extern void ModifyGoColor(GRgb*, char*);
extern GRgb  HexaStringToRgb(const char*);

// in file goInit.cc;
extern void InitEditGo(void*);



/**
FUNCTION
   UnsensitiveButtons()
DESCRIPTION
    The three buttons (Apply, Cancel, Reinit) and the menu click of the
    editGcDialog are not sensitive
**/
static void UnsensitiveButtons(Widget, caddr_t, caddr_t)
{
    Arg arg;
    XtSetArg(arg, XmNsensitive, False);
    XtSetValues(apply, &arg, 1);
    XtSetValues(cancel, &arg, 1);
    XtSetValues(reinit, &arg, 1);
    XtSetValues(menu_click, &arg, 1);
}



/**
FUNCTION
   SensitiveEditButtons()
DESCRIPTION
    The three buttons (Apply, Cancel, Reinit) and the menu click of the
    editGcDialog are sensitive
**/
void SensitiveEditButtons(Widget, caddr_t, caddr_t)
{
    allMapped = False;
    Arg arg;
    XtSetArg(arg, XmNsensitive, True);
    XtSetValues(apply, &arg, 1);
    XtSetValues(cancel, &arg, 1);
    XtSetValues(reinit, &arg, 1);
    XtSetValues(menu_click, &arg, 1);
}





/**
FUNCTION
    CurrentDialog()
DESCRIPTION
    Display the PropertySheet to update the current attributes
    (defaultAttributes)
    (Validation toggles are not managed)
**/
void CurrentDialog(void*, void* main_window, caddr_t)
{
    Boolean create = (editGcDialog == NULL);
    Arg arg;

    XtSetArg(arg, XmNsensitive, FALSE);
    XtSetValues((Widget)current_entry, &arg, 1);
    XtSetValues((Widget)modif_entry, &arg, 1);
    XtSetValues((Widget)pen_modif_entry, &arg, 1);
    XtSetValues((Widget)line_modif_entry, &arg, 1);
    XtSetValues((Widget)brush_modif_entry, &arg, 1);
    XtSetValues((Widget)text_modif_entry, &arg, 1);
    XtSetValues((Widget)font_modif_entry, &arg, 1);

    if (create) {
	XDefineCursor(XtDisplay((Widget)graphic), XtWindow((Widget)graphic),
		      clock_cursor);
	XDefineCursor(XtDisplay((Widget)main_window),
		      XtWindow((Widget)main_window), clock_cursor);
	XFlush(XtDisplay((Widget)graphic));
	editGcDialog = CreateGCDialog((Widget)main_window);
    } else
	ManageValidationToggles(False, valid_toggles, num_valid_toggles);

    
    WhichAttributes = EDIT;
    WhichAction = CURRENT;
    WhichObject = NONE;
    MapEditAllAttrib = False;
    XtManageChild(editGcDialog);

    if (create) {
	PrepareGCDialog(editGcDraw);
	XFlush(XtDisplay((Widget)graphic));
	XUndefineCursor(XtDisplay((Widget)graphic), XtWindow((Widget)graphic));
	XUndefineCursor(XtDisplay((Widget)main_window),
			XtWindow((Widget)main_window));
    }
    InitDialog();
}


/**
FUNCTION
    ModifyDialog()
DESCRIPTION
    Display the PropertySheet to modify the selected objects GC 
    (Validation toggles are managed)
**/
void ModifyDialog(void*, void* main_window, caddr_t)
{
    Arg arg;

    if (! GetNumMainSelectedObjects()) {
	NoSelectedObject(main_window);
	return;
    }
    
    Boolean create = (editGcDialog == NULL);

    XtSetArg(arg, XmNsensitive, FALSE);
    XtSetValues((Widget)current_entry, &arg, 1);
    XtSetValues((Widget)modif_entry, &arg, 1);
    XtSetValues((Widget)pen_modif_entry, &arg, 1);
    XtSetValues((Widget)line_modif_entry, &arg, 1);
    XtSetValues((Widget)brush_modif_entry, &arg, 1);
    XtSetValues((Widget)text_modif_entry, &arg, 1);
    XtSetValues((Widget)font_modif_entry, &arg, 1);

    if (create) {
	XDefineCursor(XtDisplay((Widget)graphic), XtWindow((Widget)graphic),
		      clock_cursor);
	XDefineCursor(XtDisplay((Widget)main_window),
		      XtWindow((Widget)main_window), clock_cursor);
	XFlush(XtDisplay((Widget)graphic));
	editGcDialog = CreateGCDialog((Widget)main_window);
    }
    ManageValidationToggles(True, valid_toggles, num_valid_toggles);
    
    WhichAttributes = EDIT;
    WhichAction = MODIFY;
    WhichObject = NONE;
    MapEditAllAttrib = False;

    XtManageChild(editGcDialog);
    
    if (create) {
	PrepareGCDialog(editGcDraw);
	XFlush(XtDisplay((Widget)graphic));
	XUndefineCursor(XtDisplay((Widget)graphic), XtWindow((Widget)graphic));
	XUndefineCursor(XtDisplay((Widget)main_window),
			XtWindow((Widget)main_window));
    }

    InitDialog();
}



/**
FUNCTION
    CreateGCDialog()
DESCRIPTION
    Create the PropertySheet used to manage the GCs.
    It is the same box to update the current GC, to modify the GC of the 
    selected objects or to edit named pens, brushes and textbrushes.
    In the second case, toggle buttons permit to valid each entry of the 
    PropertySheet.
**/
static Widget CreateGCDialog(Widget main_window)
{
    Widget ps;
    int n = 0;
    int nb = 0;
    Arg args[20];


    n = 0;
    XtSetArg(args[n], XmNresizePolicy, XmRESIZE_NONE); n++;
    XtSetArg(args[n], XmNautoUnmanage, False); n++;
    ps = XmCreateFormDialog(main_window, "EditGCDialog", args, n);
    XtAddCallback(ps, XmNhelpCallback, (XtCallbackProc)MapHelpFile,
		  (XtPointer)EDIT_ATTRIB_FILE);


    /* Apply, Reinit, Cancel buttons */
    apply = XmCreatePushButtonGadget(ps, "Apply", NULL, 0);
    XtManageChild(apply);
    XtAddCallback(apply, XmNactivateCallback,
		  (XtCallbackProc)SetSensitive, (XtPointer)current_entry);
    XtAddCallback(apply, XmNactivateCallback,
		  (XtCallbackProc)SetSensitive, (XtPointer)modif_entry);
    XtAddCallback(apply, XmNactivateCallback,
		  (XtCallbackProc)SetSensitive, (XtPointer)pen_modif_entry);
    XtAddCallback(apply, XmNactivateCallback,
		  (XtCallbackProc)SetSensitive, (XtPointer)line_modif_entry);
    XtAddCallback(apply, XmNactivateCallback,
		  (XtCallbackProc)SetSensitive, (XtPointer)brush_modif_entry);
    XtAddCallback(apply, XmNactivateCallback,
		  (XtCallbackProc)SetSensitive, (XtPointer)text_modif_entry);
    XtAddCallback(apply, XmNactivateCallback,
		  (XtCallbackProc)SetSensitive, (XtPointer)font_modif_entry);
    XtAddCallback(apply, XmNactivateCallback, (XtCallbackProc)Apply, NULL);
    XtAddCallback(apply, XmNactivateCallback, (XtCallbackProc)UnmanageWidget,
		  (XtPointer)ps);
    XtAddCallback(apply, XmNhelpCallback, (XtCallbackProc)MapHelpFile,
		  (XtPointer)EDIT_ATTRIB_FILE);
    
    reinit = XmCreatePushButtonGadget(ps, "Reinit", NULL, 0);
    XtManageChild(reinit);
    XtAddCallback(reinit, XmNactivateCallback,
		  (XtCallbackProc)ReinitSheet, NULL);
    XtAddCallback(reinit, XmNhelpCallback, (XtCallbackProc)MapHelpFile,
		  (XtPointer)EDIT_ATTRIB_FILE);
    
    cancel = XmCreatePushButtonGadget(ps, "Cancel", NULL, 0);
    XtManageChild(cancel);
    XtAddCallback(cancel, XmNactivateCallback, (XtCallbackProc)SetSensitive,
		  (XtPointer)current_entry);
    XtAddCallback(cancel, XmNactivateCallback, (XtCallbackProc)SetSensitive,
		  (XtPointer)modif_entry);
    XtAddCallback(cancel, XmNactivateCallback,
		  (XtCallbackProc)SetSensitive, (XtPointer)pen_modif_entry);
    XtAddCallback(cancel, XmNactivateCallback,
		  (XtCallbackProc)SetSensitive, (XtPointer)line_modif_entry);
    XtAddCallback(cancel, XmNactivateCallback,
		  (XtCallbackProc)SetSensitive, (XtPointer)brush_modif_entry);
    XtAddCallback(cancel, XmNactivateCallback,
		  (XtCallbackProc)SetSensitive, (XtPointer)text_modif_entry);
    XtAddCallback(cancel, XmNactivateCallback,
		  (XtCallbackProc)SetSensitive, (XtPointer)font_modif_entry);
    XtAddCallback(cancel, XmNactivateCallback, (XtCallbackProc)UnmanageWidget,
		  (XtPointer)ps);
    XtAddCallback(cancel, XmNhelpCallback, (XtCallbackProc)MapHelpFile,
		  (XtPointer)EDIT_ATTRIB_FILE);
    
    XmString str_null = XmStringCreate("", XmSTRING_DEFAULT_CHARSET);
    n = 0;
    XtSetArg(args[n], XmNset, TRUE); n++;
    XtSetArg(args[n], XmNindicatorType, XmONE_OF_MANY); n++;
    XtSetArg(args[n], XmNlabelString, str_null); n++;

    /* background toggle and valid_toggle */
    backgroundToggle = valid_toggles[nb] = 
	XmCreateToggleButtonGadget(ps, "ValidBackground", args, n);
    XtAddCallback(backgroundToggle, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)EDIT_ATTRIB_FILE);
    CreateBackgroundWid(ps);
    nb++;

    /* outline toggle and valid_toggle */
    outlineToggle = valid_toggles[nb] = 
	XmCreateToggleButtonGadget(ps, "ValidOutline", args, n);
    XtAddCallback(outlineToggle, XmNhelpCallback, (XtCallbackProc)MapHelpFile,
		  (XtPointer)EDIT_ATTRIB_FILE);
    CreateOutlineWid(ps);
    nb++;

    /* rounded toggle and valid_toggle */
    roundedToggle = valid_toggles[nb] = 
	XmCreateToggleButtonGadget(ps, "ValidRounded", args, n);
    XtAddCallback(roundedToggle, XmNhelpCallback, (XtCallbackProc)MapHelpFile,
		  (XtPointer)EDIT_ATTRIB_FILE);
    CreateRoundedWid(ps);
    nb++;

    XmStringFree(str_null);
    
    /* edit attributes */
    n = 0;
    XtSetArg(args[n], XmNmarginWidth, 5); n++;
    XtSetArg(args[n], XmNmarginHeight, 5); n++;
    editWid = XmCreatePushButtonGadget(ps, "EditButton", args, n);
    XtManageChild(editWid);
    XtAddCallback(editWid, XmNactivateCallback,
		  (XtCallbackProc) MapAllAttrib, (XtPointer) main_window);
    XtAddCallback(editWid, XmNactivateCallback,
		  (XtCallbackProc) UnsensitiveButtons, NULL);
    XtAddCallback(editWid, XmNhelpCallback, (XtCallbackProc)MapHelpFile,
		  (XtPointer)EDIT_ATTRIB_FILE);
    
    /* click menu */
    CreateClickMenu(ps);

    /* pen text */
    CreatePenNameWid(ps);

    /* brush text */
    CreateBrushNameWid(ps);

    /* textbrush text */
    CreateTextBrushNameWid(ps);

    /* frame and its child, editGcDraw widget */
    editGcFrame = XmCreateFrame(ps, "EditGCFrame", NULL, 0);
    XtManageChild(editGcFrame);
    XtAddCallback(editGcFrame, XmNhelpCallback, (XtCallbackProc)MapHelpFile,
		  (XtPointer)EDIT_ATTRIB_FILE);
    editGcDraw = XbCreateGraphic(editGcFrame, "EditGCDraw", NULL, 0);
    XtManageChild(editGcDraw);
    XtAddCallback(editGcDraw, XmNhelpCallback, (XtCallbackProc)MapHelpFile,
		  (XtPointer)EDIT_ATTRIB_FILE);

    num_valid_toggles = nb;

    return(ps);
}


/**
FUNCTION
    CreateOutlineWid()
DESCRIPTION
    Creation of toggle button "with outline".
**/
static void CreateOutlineWid(Widget parent)
{
    Arg args[5];
    int n = 0;
    XtSetArg(args[n], XmNindicatorType, XmONE_OF_MANY); n++;
    outlineWid = XmCreateToggleButtonGadget(parent, "Outline", args, n);
    XtAddCallback(outlineWid, XmNvalueChangedCallback,
		  (XtCallbackProc)ShowOutline, NULL);
    XtAddCallback(outlineWid, XmNvalueChangedCallback,
		  (XtCallbackProc)ValidFunction, (XtPointer)outlineToggle);
    XtAddCallback(outlineWid, XmNhelpCallback, (XtCallbackProc)MapHelpFile,
		  (XtPointer)EDIT_ATTRIB_FILE);
    XtManageChild(outlineWid);
}


/**
FUNCTION
    CreateBackgroundWid()
DESCRIPTION
    Creation of toggle button "with background".
**/
static void CreateBackgroundWid(Widget parent)
{
    Arg args[5];
    int n = 0;
    XtSetArg(args[n], XmNindicatorType, XmONE_OF_MANY); n++;
    backgroundWid = XmCreateToggleButtonGadget(parent, "Background", args, n);
    XtAddCallback(backgroundWid, XmNvalueChangedCallback,
		  (XtCallbackProc)ShowBackground,NULL);
    XtAddCallback(backgroundWid, XmNvalueChangedCallback,
		  (XtCallbackProc)ValidFunction, (XtPointer)backgroundToggle);
    XtAddCallback(backgroundWid, XmNhelpCallback, (XtCallbackProc)MapHelpFile,
		  (XtPointer)EDIT_ATTRIB_FILE);
    XtManageChild(backgroundWid);
}


/**
FUNCTION
    CreateRoundedWid()
DESCRIPTION
    Creation of toggle button "rounded".
**/
static void CreateRoundedWid(Widget parent)
{
    Arg args[5];
    int n = 0;
    XtSetArg(args[n], XmNindicatorType, XmONE_OF_MANY); n++;
    roundedWid = XmCreateToggleButtonGadget(parent, "Rounded", args, n);
    XtAddCallback(roundedWid, XmNvalueChangedCallback,
		  (XtCallbackProc)ShowRounded,NULL);
    XtAddCallback(roundedWid, XmNvalueChangedCallback,
		  (XtCallbackProc)ValidFunction, (XtPointer)roundedToggle);
    XtAddCallback(roundedWid, XmNhelpCallback, (XtCallbackProc)MapHelpFile,
		  (XtPointer)EDIT_ATTRIB_FILE);
    XtManageChild(roundedWid);
}



static char* menuname[] = {
    "ClickCB", "PenClick", "BrushClick", "TextBrushClick"
};


/**
FUNCTION
    CreateClickMenu()
DESCRIPTION
    Creation of popup menu used to click on gos.
**/
static void CreateClickMenu(Widget parent)
{
    Arg args[2];
    int n = 0;
    XtSetArg(args[n], XmNmarginWidth, 0); n++;
    XtSetArg(args[n], XmNmarginHeight, 0); n++;
    Widget w = XmCreateMenuBar(parent, "ClickPopup", args, n);
    XtManageChild(w);
    int calldata[3];
    calldata[0] = PEN;
    calldata[1] = BRUSH;
    calldata[2] = TEXTBRUSH;
    Widget sub = XmCreatePulldownMenu(w, "sub", NULL, 0);
    
    XtSetArg(args[0], XmNsubMenuId, sub);
    menu_click = XmCreateCascadeButtonGadget(w, menuname[0], args, 1);
    XtAddCallback(menu_click, XmNhelpCallback, (XtCallbackProc)MapHelpFile,
		  (XtPointer)EDIT_ATTRIB_FILE);
    XtManageChild(menu_click);
    
    Widget button;
    for (int i = 1; i < 4; i++) {
	button = XmCreatePushButtonGadget(sub, menuname[i], NULL, 0);
	XtAddCallback(button, XmNactivateCallback,
		      (XtCallbackProc)EditChooseGo,
		      (XtPointer)calldata[i - 1]);
	XtManageChild(button);
    }
}
    

/**
FUNCTION
    CreatePenNameWid()
DESCRIPTION
    Creation of text and label used to edit named pen attributes.
**/
static void CreatePenNameWid(Widget parent)
{
    penLabel = XmCreateLabelGadget(parent, "PenLabel", NULL, 0);
    XtAddCallback(penLabel, XmNhelpCallback, (XtCallbackProc)MapHelpFile,
		  (XtPointer)EDIT_ATTRIB_FILE);
    XtManageChild(penLabel);

    Arg args[2];
    int n = 0;
    XtSetArg(args[n], XmNmarginWidth, 5); n++;
    XtSetArg(args[n], XmNmarginHeight, 5); n++;
    penList = XmCreatePushButtonGadget(parent, "PenList", args, n);
    XtManageChild(penList);
    XtAddCallback(penList, XmNactivateCallback,
		  (XtCallbackProc) SelectNamedPen, EDIT);
    XtAddCallback(penList, XmNhelpCallback, (XtCallbackProc)MapHelpFile,
		  (XtPointer)EDIT_ATTRIB_FILE);

    penTextEdit = XmCreateTextField(parent, "PenText", NULL, 0);
    XtManageChild(penTextEdit);
    XtAddCallback(penTextEdit, XmNactivateCallback,
		  (XtCallbackProc) VerifFunction, NULL);
    XtAddCallback(penTextEdit, XmNhelpCallback, (XtCallbackProc)MapHelpFile,
		  (XtPointer)EDIT_ATTRIB_FILE);
}


/**
FUNCTION
    CreateBrushNameWid()
DESCRIPTION
    Creation of text and label used to edit named brush attributes.
**/
static void CreateBrushNameWid(Widget parent)
{
    brushLabel = XmCreateLabelGadget(parent, "BrushLabel", NULL, 0);
    XtManageChild(brushLabel);
    XtAddCallback(brushLabel, XmNhelpCallback, (XtCallbackProc)MapHelpFile,
		  (XtPointer)EDIT_ATTRIB_FILE);

    Arg args[2];
    int n = 0;
    XtSetArg(args[n], XmNmarginWidth, 5); n++;
    XtSetArg(args[n], XmNmarginHeight, 5); n++;
    brushList = XmCreatePushButtonGadget(parent, "BrushList", args, n);
    XtManageChild(brushList);
    XtAddCallback(brushList, XmNactivateCallback,
		  (XtCallbackProc) SelectNamedBrush, EDIT);
    XtAddCallback(brushList, XmNhelpCallback, (XtCallbackProc)MapHelpFile,
		  (XtPointer)EDIT_ATTRIB_FILE);

    brushTextEdit = XmCreateTextField(parent, "BrushText", NULL, 0);
    XtManageChild(brushTextEdit);
    XtAddCallback(brushTextEdit, XmNactivateCallback,
		  (XtCallbackProc) VerifFunction, NULL);
    XtAddCallback(brushTextEdit, XmNhelpCallback, (XtCallbackProc)MapHelpFile,
		  (XtPointer)EDIT_ATTRIB_FILE);
}



/**
FUNCTION
    CreateTextBrushNameWid()
DESCRIPTION
    Creation of text and label used to edit named textbrush attributes.
**/
static void CreateTextBrushNameWid(Widget parent)
{
    textBrushLabel = XmCreateLabelGadget(parent, "TextBrushLabel", NULL, 0);
    XtManageChild(textBrushLabel);
    XtAddCallback(textBrushLabel, XmNhelpCallback, (XtCallbackProc)MapHelpFile,
		  (XtPointer)EDIT_ATTRIB_FILE);

    Arg args[2];
    int n = 0;
    XtSetArg(args[n], XmNmarginWidth, 5); n++;
    XtSetArg(args[n], XmNmarginHeight, 5); n++;
    textBrushList = XmCreatePushButtonGadget(parent, "TextBrushList", args, n);
    XtManageChild(textBrushList);
    XtAddCallback(textBrushList, XmNactivateCallback,
		  (XtCallbackProc) SelectNamedText, EDIT);
    XtAddCallback(textBrushList, XmNhelpCallback, (XtCallbackProc)MapHelpFile,
		  (XtPointer)EDIT_ATTRIB_FILE);

    textBrushTextEdit = XmCreateTextField(parent, "TextBrushText", NULL, 0);
    XtManageChild(textBrushTextEdit);
    XtAddCallback(textBrushTextEdit, XmNactivateCallback,
		  (XtCallbackProc) VerifFunction, NULL);
    XtAddCallback(textBrushTextEdit, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)EDIT_ATTRIB_FILE);
}



/**
FUNCTION
    Apply()
DESCRIPTION
    Is called when "Apply" button is pushed.
**/
static void Apply()
{
    if (WhichAction == CURRENT)
	SetDefaultAttributes();
    else
	if (WhichAction == MODIFY)
	    UpdateSelectedObjects();
}



    

/*---------------------------------------------------------------------------*/


/**
FUNCTION
    InitDialog()
DESCRIPTION
    Displays the gos in the attributes graphic widget with the right attributes
    Initializes global variables.
**/
static void InitDialog()
{
    WhichAttributes = EDIT;
    int round = (int)all_attrib.RoundedIsSet;
    int outline = (int)all_attrib.OutlineIsSet;
    int background = (int)all_attrib.BackgroundIsSet;
    pen = all_attrib.pen;
    brush = all_attrib.brush;
    text = all_attrib.text;
    reinit_pen = all_attrib.pen;
    reinit_brush = all_attrib.brush;
    reinit_text = all_attrib.text;

    if (WhichAction == MODIFY) {
	// Verify if selected objects have same attributes and in this case ;
	// initialize with these attributes;
	HaveTheySamePen(&outline, &round);
	HaveTheySameBrush(&background);
	HaveTheySameText();
    }
    
    ModifyGoEditPenSheet();
    ModifyGoEditBrushSheet();
    ModifyGoEditTextSheet();
    
    if (WhichAction == MODIFY) {
	XmToggleButtonGadgetSetState(backgroundToggle, False, False);
	XmToggleButtonGadgetSetState(outlineToggle, False, False);
	XmToggleButtonGadgetSetState(roundedToggle, False, False);
    }
    XmTextFieldSetString(penTextEdit, pen.PenText);
    XmTextFieldSetString(brushTextEdit, brush.BrushText);
    XmTextFieldSetString(textBrushTextEdit, text.TextBrushText);
    XmToggleButtonGadgetSetState(backgroundWid, background, False);
    ModifyGoBackground(background);
    XmToggleButtonGadgetSetState(outlineWid, outline, False);
    ModifyGoOutline(outline);
    XmToggleButtonGadgetSetState(roundedWid, round, False);
    ModifyGoRounded(round);

}


/**
FUNCTION
    ValidFunction()
DESCRIPTION
    Sets validation toggles to True when a modification is done.
**/
void ValidFunction(Widget, Widget toggle, caddr_t)
{
    if (WhichAction == MODIFY) 
	if (! XmToggleButtonGadgetGetState(toggle))
	    XmToggleButtonGadgetSetState(toggle, True, False);
}



/**
FUNCTION
    ManageValidationToggles()
DESCRIPTION
    Manages or unmanages validation toggles (used in the modify editGcDialog)
**/
void ManageValidationToggles(Boolean state, Widget* tab, int nb)
{
    int i;
    for (i = 0; i < nb; i++)
	XmToggleButtonGadgetSetState(tab[i], ! state, True);
    if (state)
	XtManageChildren(tab, nb);
    else
	XtUnmanageChildren(tab, nb);
}



/**
FUNCTION
    SetSensitive()
DESCRIPTION
    
**/
void SetSensitive(Widget, Widget target)
{
    Arg arg;
    XtSetArg(arg, XmNsensitive, True);
    XtSetValues(target, &arg, 1);
}


/**
FUNCTION
    SetUnsensitive()
DESCRIPTION
    
**/
void SetUnsensitive(Widget, Widget target)
{
    Arg arg;
    XtSetArg(arg, XmNsensitive, False);
    XtSetValues(target, &arg, 1);
}


/**
FUNCTION
    ShowBackground()
DESCRIPTION
    Displays background of gos used in the attributes graphic widget.
**/
static void ShowBackground(Widget, caddr_t, XmToggleButtonCallbackStruct* cd)
{
    WhichAttributes = EDIT;
    ModifyGoBackground((int)cd->set);
}


/**
FUNCTION
    ShowOutline()
DESCRIPTION
    Displays outline of gos used in the attributes graphic widget.
**/
static void ShowOutline(Widget, void*, XmToggleButtonCallbackStruct* cd)
{
    WhichAttributes = EDIT;
    ModifyGoOutline((int)cd->set);
}


/**
FUNCTION
    ShowRounded()
DESCRIPTION
    GoGeometric are rounded in the attributes widget.
**/
static void ShowRounded(Widget, void*, XmToggleButtonCallbackStruct* cd)
{
    WhichAttributes = EDIT;
    ModifyGoRounded((int)cd->set);
}


/**
FUNCTION
    SetDefaultAttributes()
DESCRIPTION
     This function permits to update the defaultAttributes (current GC) from
     the Property Sheet.
**/
static void SetDefaultAttributes()
{
    all_attrib.BackgroundIsSet = XmToggleButtonGadgetGetState(backgroundWid);
    all_attrib.OutlineIsSet = XmToggleButtonGadgetGetState(outlineWid);
    all_attrib.RoundedIsSet = XmToggleButtonGadgetGetState(roundedWid);
    GFREE(all_attrib.PenText);
    all_attrib.PenText = XmTextFieldGetString(penTextEdit);
    GFREE(all_attrib.BrushText);
    all_attrib.BrushText = XmTextFieldGetString(brushTextEdit);
    GFREE(all_attrib.TextBrushText);
    all_attrib.TextBrushText = XmTextFieldGetString(textBrushTextEdit);
    all_attrib.pen = pen;
    all_attrib.brush = brush;
    all_attrib.text = text;
}


/**
FUNCTION
    PrepareGCDialog()
DESCRIPTION
    Creates and initializes gos used in the attributes graphic widget.
**/
static void PrepareGCDialog(Widget drawWid)
{
    void* interf = (void*)XbGraphicGoInterf((XbGraphicWidget)drawWid);
    InitEditGo(interf);
}



/**
FUNCTION
    UpdateSelectedObjects()
DESCRIPTION
    Modifies selected gos in the main graphic widget. 
**/
static void UpdateSelectedObjects()
{
    Display* dpy = XtDisplay((Widget)main_window);
    XColor exact_rgb, real_rgb;
    GRgb rgb;
    
    WhichAttributes = MAIN;
    
    if (XmToggleButtonGadgetGetState(backgroundToggle))
	ModifyGoBackground((int) XmToggleButtonGadgetGetState(backgroundWid));
    if (XmToggleButtonGadgetGetState(outlineToggle))
	ModifyGoOutline((int) XmToggleButtonGadgetGetState(outlineWid));
    if (XmToggleButtonGadgetGetState(roundedToggle))
	ModifyGoRounded((int) XmToggleButtonGadgetGetState(roundedWid));

    // pen management;
    char* name = XmTextFieldGetString(penTextEdit);
    if (strcmp(name, UNNAMED) != 0)
	// a named pen is used;
	ModifyGoPen(name);
    else {
	if (WhichObject & PEN) {
	    // the user clicked on a Go to choose a pen;
	    WhichColor = PEN_COLOR;
	    if (pen.Color[0] != '#') {
		XLookupColor(dpy, DefaultColormap(dpy, DefaultScreen(dpy)),
			     pen.Color, &exact_rgb, &real_rgb);
		rgb.red = real_rgb.red;
		rgb.green = real_rgb.green;
		rgb.blue = real_rgb.blue;
	    } else
		rgb = HexaStringToRgb(pen.Color);
	    ModifyGoColor(&rgb, pen.Color);
	    WhichColor = PEN_ALT_COLOR;
	    if (pen.AltColor[0] != '#') {
		XLookupColor(dpy, DefaultColormap(dpy, DefaultScreen(dpy)),
			     pen.AltColor, &exact_rgb, &real_rgb);
		rgb.red = real_rgb.red;
		rgb.green = real_rgb.green;
		rgb.blue = real_rgb.blue;
	    } else
		rgb = HexaStringToRgb(pen.AltColor);
	    ModifyGoColor(&rgb, pen.AltColor);
	    ModifyGoJoinStyle(pen.JoinStyle);
	    ModifyGoLineStyle(pen.LineStyle);
	    ModifyGoCapStyle(pen.CapStyle);
	    ModifyGoLineWidth(atof(pen.LineWidth));
	} else {
	    if (MapEditAllAttrib) {
		// the user clicked on "Edit unnamed attributes";
		if (XmToggleButtonGadgetGetState(penColorToggle)) {
		    WhichColor = PEN_COLOR;
		    if (pen.Color[0] != '#') {
			XLookupColor(dpy, DefaultColormap(dpy, DefaultScreen(dpy)),
				     pen.Color, &exact_rgb, &real_rgb);
			rgb.red = real_rgb.red;
			rgb.green = real_rgb.green;
			rgb.blue = real_rgb.blue;
		    } else
			rgb = HexaStringToRgb(pen.Color);
		    ModifyGoColor(&rgb, pen.Color);
		}
		if (XmToggleButtonGadgetGetState(altColorToggle)) {
		    WhichColor = PEN_ALT_COLOR;
		    if (pen.AltColor[0] != '#') {
			XLookupColor(dpy, DefaultColormap(dpy,
							  DefaultScreen(dpy)),
				     pen.AltColor, &exact_rgb, &real_rgb);
			rgb.red = real_rgb.red;
			rgb.green = real_rgb.green;
			rgb.blue = real_rgb.blue;
		    } else
			rgb = HexaStringToRgb(pen.AltColor);
		    ModifyGoColor(&rgb, pen.AltColor);
		}
		if (XmToggleButtonGadgetGetState(joinStyleToggle))
		    ModifyGoJoinStyle(pen.JoinStyle);
		if (XmToggleButtonGadgetGetState(lineStyleToggle))
		    ModifyGoLineStyle(pen.LineStyle);
		if (XmToggleButtonGadgetGetState(capStyleToggle))
		    ModifyGoCapStyle(pen.CapStyle);
		if (XmToggleButtonGadgetGetState(lineWidthToggle))
		    ModifyGoLineWidth(atof(pen.LineWidth));
	    }
	}
    }
    GFREE(name);

    // brush management;
    name = XmTextFieldGetString(brushTextEdit);
    if (strcmp(name, UNNAMED) != 0)
	// a named brush is used;
	ModifyGoBrush(name);
    else {
	if (WhichObject & BRUSH) {
	    // the user clicked on Go to choose a brush;
	    WhichColor = BRUSH_COLOR;
	    if (brush.Color[0] != '#') {
		XLookupColor(dpy, DefaultColormap(dpy, DefaultScreen(dpy)),
			     brush.Color, &exact_rgb, &real_rgb);
		rgb.red = real_rgb.red;
		rgb.green = real_rgb.green;
		rgb.blue = real_rgb.blue;
	    } else
		rgb = HexaStringToRgb(brush.Color);
	    ModifyGoColor(&rgb, brush.Color);
	    ModifyGoFillRule(brush.FillRule);
	} else {
	    if (MapEditAllAttrib) {
		// the user clicked on "edit unnamed attributes";
		if (XmToggleButtonGadgetGetState(brushColorToggle)) {
		    WhichColor = BRUSH_COLOR;
		    if (brush.Color[0] != '#') {
			XLookupColor(dpy, DefaultColormap(dpy,
							  DefaultScreen(dpy)),
				     brush.Color, &exact_rgb, &real_rgb);
			rgb.red = real_rgb.red;
			rgb.green = real_rgb.green;
			rgb.blue = real_rgb.blue;
		    } else
			rgb = HexaStringToRgb(brush.Color);
		    ModifyGoColor(&rgb, brush.Color);
		}
		if (XmToggleButtonGadgetGetState(fillRuleToggle))
		    ModifyGoFillRule(brush.FillRule);
	    }
	}
    }
    GFREE(name);

    // textbrush management;
    name = XmTextFieldGetString(textBrushTextEdit);
    if (strcmp(name, UNNAMED) != 0)
	// a named textbrush is used;
	ModifyGoText(name);
    else {
	if (WhichObject & TEXTBRUSH) {
	    // the user clicked on a Go to choose a textbrush;
	    WhichColor = TEXT_COLOR;
	    if (text.Color[0] != '#') {
		XLookupColor(dpy, DefaultColormap(dpy, DefaultScreen(dpy)),
			     text.Color, &exact_rgb, &real_rgb);
		rgb.red = real_rgb.red;
		rgb.green = real_rgb.green;
		rgb.blue = real_rgb.blue;
	    } else
		rgb = HexaStringToRgb(text.Color);
	    ModifyGoColor(&rgb, text.Color);
	    ModifyGoFont(text.Font);
	} else {
	    if (MapEditAllAttrib) {
		// the user clicked on "edit unnmaed attributes";
		if (XmToggleButtonGadgetGetState(textColorToggle)) {
		    WhichColor = TEXT_COLOR;
		    if (text.Color[0] != '#') {
			XLookupColor(dpy, DefaultColormap(dpy,
							  DefaultScreen(dpy)),
				     text.Color, &exact_rgb, &real_rgb);
			rgb.red = real_rgb.red;
			rgb.green = real_rgb.green;
			rgb.blue = real_rgb.blue;
		    } else
			rgb = HexaStringToRgb(text.Color);
		    ModifyGoColor(&rgb, text.Color);
		}
		if (XmToggleButtonGadgetGetState(fontToggle))
		    ModifyGoFont(text.Font);
	    }
	}
    }
    GFREE(name);

}



/**
FUNCTION
    EditChooseGo()
DESCRIPTION
    
**/
static void _edit_choosego_postcond(int, class Go*);
    
static void EditChooseGo(Widget, int obj, caddr_t)
{
    WhichAttributes = EDIT;
    ChooseGo(obj, _edit_choosego_postcond, obj);
}

static void _edit_choosego_postcond(int closure, class Go*)
{
    int obj = closure;
    switch(obj) {
      case PEN:
	if (WhichAction == MODIFY)
	    if (WhichObject == NONE)
		WhichObject = PEN;
	    else
		WhichObject |= PEN;
	XmTextFieldSetString(penTextEdit, UNNAMED);
	break;
      case BRUSH:
	if (WhichObject == NONE)
	    WhichObject = BRUSH;
	else
	    WhichObject |= BRUSH;
	XmTextFieldSetString(brushTextEdit, UNNAMED);
	break;
      case TEXTBRUSH:
	if (obj == TEXTBRUSH)
	    if (WhichObject == NONE)
		WhichObject = TEXTBRUSH;
	    else
		WhichObject |= TEXTBRUSH;
	XmTextFieldSetString(textBrushTextEdit, UNNAMED);
	break;
    }
}



/******************************************************************************
                           REINIT FUNCTION
******************************************************************************/



/**
FUNCTION
    ReinitSheet()
DESCRIPTION
    Reinits all the sheet.
**/
static void ReinitSheet()
{
    WhichAttributes = EDIT;

    pen = reinit_pen;
    brush = reinit_brush;
    text = reinit_text;
    
    if (WhichAction == MODIFY) {
	XmToggleButtonGadgetSetState(backgroundToggle, False, False);
	XmToggleButtonGadgetSetState(outlineToggle, False, False);
	XmToggleButtonGadgetSetState(roundedToggle, False, False);
    }
    XmTextFieldSetString(penTextEdit, all_attrib.PenText);
    XmTextFieldSetString(brushTextEdit, all_attrib.BrushText);
    XmTextFieldSetString(textBrushTextEdit, all_attrib.TextBrushText);
    XmToggleButtonGadgetSetState(backgroundWid, all_attrib.BackgroundIsSet,
				 False);
    ModifyGoBackground((int)all_attrib.BackgroundIsSet);
    XmToggleButtonGadgetSetState(outlineWid, all_attrib.OutlineIsSet, False);
    ModifyGoOutline((int)all_attrib.OutlineIsSet);
    XmToggleButtonGadgetSetState(roundedWid, all_attrib.RoundedIsSet, False);
    ModifyGoRounded((int)all_attrib.RoundedIsSet);

    ModifyGoEditPenSheet();
    ModifyGoEditBrushSheet();
    ModifyGoEditTextSheet();

}
