#ifdef REV_INFO
static char rcsid[] = "$Id: editmenu.cc,v 1.1.1.1 1992/03/18 09:05:08 uep Exp $";
#endif
/*****************************************************************************
MODULE:  Go Editor
FILE:    editmenu.cc
AUTHOR:  Aline Jacomet
DATE:    September 11th 1991

DESCRIPTION:
    

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include "main.h"

#include <Xm/Form.h>
#include <Xm/TextF.h>
#include <Xm/RowColumn.h>
#include <Xm/PushBG.h>
#include <Xm/LabelG.h>
#include <Xm/ToggleBG.h>


static Widget alignDialog = NULL;
static Widget spaceText = NULL;
static Widget rotateDialog = NULL;
static Widget mirrorDialog = NULL;
static Widget center = NULL;
static Widget spacingDialog = NULL;
static Widget spacingText = NULL;


/*
 * EXTERN FUNCTIONS
 */

// in file goEdition.cc;
extern void AlignGo(int, double);    
extern void RotateGo(int, int);
extern void MirrorGo(void*, int);
extern void SpacingGo(int, double);

// in file namedattrib.cc;
extern void UnmanageWidget(Widget, Widget, caddr_t);




                           /*************
			    * ALIGNMENT *
			    *************/

/***
 * Redo textfield translations for interval.
 ***/
static char *TextEventBindings1 =
"Shift<Key>osfQuickPaste:	do-quick-action()\n\
 Meta <Key>osfQuickPaste:	quick-cut-set()\n\
 Alt <Key>osfQuickPaste:	quick-cut-set()\n\
 <Key>osfQuickPaste:		quick-copy-set()\n\
 Meta <Key>osfPrimaryPaste:	cut-primary()\n\
 Alt <Key>osfPrimaryPaste:	cut-primary()\n\
 <Key>osfPrimaryPaste:		copy-primary()\n\
 Meta <Key>osfCut:		cut-primary()\n\
 Alt <Key>osfCut:		cut-primary()\n\
 <Key>osfCut:			cut-clipboard()\n\
 <Key>osfPaste:			paste-clipboard()\n\
 Meta <Key>osfCopy:		copy-primary()\n\
 Alt <Key>osfCopy:		copy-primary()\n\
 <Key>osfCopy:			copy-clipboard()\n\
 Shift <Key>osfBeginLine:	beginning-of-line(extend)\n\
 <Key>osfBeginLine:		beginning-of-line()\n\
 Shift <Key>osfEndLine:		end-of-line(extend)\n\
 <Key>osfEndLine:		end-of-line()\n\
 Ctrl <Key>osfPageUp:		page-left()\n\
 Ctrl <Key>osfPageDown:		page-right()\n\
 <Key>osfClear:			clear-selection()\n\
 ~Meta ~Alt <Key>osfBackSpace:	delete-previous-character()\n\
 Shift Meta<Key>osfDelete:	cut-primary()\n\
 Shift Alt<Key>osfDelete:	cut-primary()\n\
 Shift ~Ctrl ~Meta ~Alt <Key>osfDelete:		cut-clipboard()\n\
 ~Shift Ctrl ~Meta ~Alt <Key>osfDelete:		delete-to-end-of-line()\n\
 ~Shift ~Ctrl ~Meta ~Alt <Key>osfDelete:	delete-next-character()\n";


static char *TextEventBindings2 =
"Ctrl Meta<Key>osfInsert:	copy-primary()\n\
 Ctrl Alt<Key>osfInsert:	copy-primary()\n\
 Shift ~Ctrl ~Meta ~Alt <Key>osfInsert:		paste-clipboard()\n\
 ~Shift Ctrl ~Meta ~Alt <Key>osfInsert:		copy-clipboard()\n\
 ~Shift <Key>osfSelect:		set-anchor()\n\
 Shift ~Ctrl <Key>osfSelect:	key-select()\n\
 <Key>osfActivate:		activate()\n\
 <Key>osfAddMode:		toggle-add-mode()\n\
 <Key>osfHelp:			Help()\n\
 <Key>osfCancel:		process-cancel()\n\
 Shift Ctrl <Key>osfLeft:	backward-word(extend)\n\
 Ctrl <Key>osfLeft:		backward-word()\n\
 Shift <Key>osfLeft:		key-select(left)\n\
 <Key>osfLeft:			backward-character()\n\
 Shift Ctrl <Key>osfRight:	forward-word(extend)\n\
 Ctrl <Key>osfRight:		forward-word()\n\
 Shift <Key>osfRight:		key-select(right)\n\
 <Key>osfRight:			forward-character()\n\
 <Key>osfUp:			traverse-prev()\n\
 <Key>osfDown:			traverse-next()\n\
 Ctrl ~Meta ~Alt <Key>slash:	select-all()\n\
 Ctrl ~Meta ~Alt <Key>backslash:	deselect-all()\n\
 Shift ~Meta ~Alt <Key>Tab:	prev-tab-group()\n\
 ~Meta ~Alt <Key>Tab:		next-tab-group()\n\
 ~Shift ~Meta ~Alt <Key>Return:	activate()\n";

static char *TextEventBindings3 =
" Ctrl ~Shift ~Meta ~Alt <Key>space:	set-anchor()\n\
 Shift ~Ctrl ~Meta ~Alt <Key>space:	key-select()\n\
 ~Ctrl ~Meta ~Alt <Key>0:	self-insert()\n\
 ~Ctrl ~Meta ~Alt <Key>1:	self-insert()\n\
 ~Ctrl ~Meta ~Alt <Key>2:	self-insert()\n\
 ~Ctrl ~Meta ~Alt <Key>3:	self-insert()\n\
 ~Ctrl ~Meta ~Alt <Key>4:	self-insert()\n\
 ~Ctrl ~Meta ~Alt <Key>5:	self-insert()\n\
 ~Ctrl ~Meta ~Alt <Key>6:	self-insert()\n\
 ~Ctrl ~Meta ~Alt <Key>7:	self-insert()\n\
 ~Ctrl ~Meta ~Alt <Key>8:	self-insert()\n\
 ~Ctrl ~Meta ~Alt <Key>9:	self-insert()\n\
 ~Ctrl ~Meta ~Alt <Key>period:	self-insert()\n\
 ~Ctrl ~Meta ~Alt <Key>minus:	self-insert()\n\
 ~Ctrl  Shift ~Meta ~Alt<Btn1Down>:	extend-start()\n\
  Ctrl ~Shift ~Meta ~Alt<Btn1Down>:	move-destination()\n\
 ~Ctrl ~Shift ~Meta ~Alt<Btn1Down>:	grab-focus()\n\
 ~Ctrl ~Meta ~Alt<Btn1Motion>:		extend-adjust()\n\
 ~Ctrl ~Meta ~Alt<Btn1Up>:		extend-end()\n\
 <FocusIn>:				focusIn()\n\
 <FocusOut>:				focusOut()\n\
 <Enter>:				enter()\n\
 <Leave>:				leave()\n\
 <Unmap>:				unmap()";


static void Align(void*, int type)
{
    char* s = XmTextFieldGetString(spaceText);
    AlignGo(type, (double)atof(s));
}


static void createAlignDialog (Widget parent)
{
    Widget rc ;
    Arg args [15];
    Widget buttons[10];

    int n = 0;
    XtSetArg (args[n], XmNresizePolicy, XmRESIZE_GROW); n++;
    XtSetArg (args[n], XmNwidth, 121); n++;
    XtSetArg (args[n], XmNheight, 400); n++;
    alignDialog = XmCreateFormDialog(parent, "AlignDialog", args, n);

    n = 0;
    int i = 0;
    XtSetArg (args[i], XmNnumColumns, 2); i++;
    XtSetArg (args[i], XmNpacking, XmPACK_COLUMN); i++;
    XtSetArg (args[i], XmNleftAttachment, XmATTACH_POSITION); i++;
    XtSetArg (args[i], XmNleftPosition, 10); i++;
    XtSetArg (args[i], XmNtopAttachment, XmATTACH_POSITION); i++;
    XtSetArg (args[i], XmNtopPosition, 5); i++;
    rc = XmCreateRowColumn (alignDialog, "AlignRowColumn", args, i);

    XtSetArg (args[0], XmNlabelPixmap, alignTopIcon);
    XtSetArg (args[1], XmNlabelType, XmPIXMAP);
    buttons[n] = XmCreatePushButtonGadget (rc, "AlignTop", args, 2);
    XtAddCallback (buttons[n], XmNactivateCallback, 
		   (XtCallbackProc) Align, ALIGN_TOP);
    n++;

    XtSetArg (args[0], XmNlabelPixmap, alignLeftIcon);
    XtSetArg (args[1], XmNlabelType, XmPIXMAP);
    buttons[n] = XmCreatePushButtonGadget (rc, "AlignLeft", args, 2);
    XtAddCallback (buttons[n], XmNactivateCallback, 
		   (XtCallbackProc) Align, (XtPointer) ALIGN_LEFT);
    n++;

    XtSetArg (args[0], XmNlabelPixmap, alignHorizontalIcon);
    XtSetArg (args[1], XmNlabelType, XmPIXMAP);
    buttons[n] = XmCreatePushButtonGadget (rc, "AlignHorizontal", args, 2);
    XtAddCallback (buttons[n], XmNactivateCallback, 
		   (XtCallbackProc) Align, (XtPointer) ALIGN_HORIZONTAL);
    n++;
    
    XtSetArg (args[0], XmNlabelPixmap, alignTopBotIcon);
    XtSetArg (args[1], XmNlabelType, XmPIXMAP);
    buttons[n] = XmCreatePushButtonGadget (rc, "AlignTopBot", args, 2);
    XtAddCallback (buttons[n], XmNactivateCallback, 
		   (XtCallbackProc) Align, (XtPointer) ALIGN_TOP_BOTTOM);
    n++;
    
    XtSetArg (args[0], XmNlabelPixmap, alignLeftRightIcon);
    XtSetArg (args[1], XmNlabelType, XmPIXMAP);
    buttons[n] = XmCreatePushButtonGadget (rc, "AlignLeftRight", args, 2);
    XtAddCallback (buttons[n], XmNactivateCallback, 
		   (XtCallbackProc) Align, (XtPointer) ALIGN_LEFT_RIGHT);
    n++;
    
    XtSetArg (args[0], XmNlabelPixmap, alignBottomIcon);
    XtSetArg (args[1], XmNlabelType, XmPIXMAP);
    buttons[n] = XmCreatePushButtonGadget (rc, "AlignBottom", args, 2);
    XtAddCallback (buttons[n], XmNactivateCallback, 
		   (XtCallbackProc) Align, (XtPointer) ALIGN_BOTTOM);
    n++;

    XtSetArg (args[0], XmNlabelPixmap, alignRightIcon);
    XtSetArg (args[1], XmNlabelType, XmPIXMAP);
    buttons[n] = XmCreatePushButtonGadget (rc, "AlignRight", args, 2);
    XtAddCallback (buttons[n], XmNactivateCallback, 
		   (XtCallbackProc) Align, (XtPointer) ALIGN_RIGHT);
    n++;
    
    XtSetArg (args[0], XmNlabelPixmap, alignVerticalIcon);
    XtSetArg (args[1], XmNlabelType, XmPIXMAP);
    buttons[n] = XmCreatePushButtonGadget (rc, "AlignVertical", args, 2);
    XtAddCallback (buttons[n], XmNactivateCallback, 
		   (XtCallbackProc) Align, (XtPointer) ALIGN_VERTICAL);
    n++;
    
    XtSetArg (args[0], XmNlabelPixmap, alignBotTopIcon);
    XtSetArg (args[1], XmNlabelType, XmPIXMAP);
    buttons[n] = XmCreatePushButtonGadget (rc, "AlignBotTop", args, 2);
    XtAddCallback (buttons[n], XmNactivateCallback, 
		   (XtCallbackProc) Align, (XtPointer) ALIGN_BOTTOM_TOP);
    n++;
    
    XtSetArg (args[0], XmNlabelPixmap, alignRightLeftIcon);
    XtSetArg (args[1], XmNlabelType, XmPIXMAP);
    buttons[n] = XmCreatePushButtonGadget (rc, "AlignRightLeft", args, 2);
    XtAddCallback (buttons[n], XmNactivateCallback, 
		   (XtCallbackProc) Align, (XtPointer) ALIGN_RIGHT_LEFT);
    n++;
    
    XtManageChildren (buttons, n);
    XtManageChild (rc);

    n = 0;
    XtSetArg (args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
    XtSetArg (args[n], XmNleftPosition, 10); n++;
    XtSetArg (args[n], XmNbottomAttachment, XmATTACH_POSITION); n++;
    XtSetArg (args[n], XmNbottomPosition, 75); n++;
    Widget w = XmCreateLabelGadget(alignDialog, "SpaceLabel", args, n);
    XtManageChild(w);

    char* text_event_bindings =
	(char *) XtMalloc(strlen(TextEventBindings1) +
			  strlen(TextEventBindings2) +
			  strlen(TextEventBindings3) + 1);

    strcpy(text_event_bindings, TextEventBindings1);
    strcat(text_event_bindings, TextEventBindings2);
    strcat(text_event_bindings, TextEventBindings3);
    XtTranslations text_translations =
	XtParseTranslationTable(text_event_bindings);
    XtFree(text_event_bindings);
    n = 0;
    XtSetArg(args[n], XmNtranslations, text_translations); n++;
    XtSetArg (args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
    XtSetArg (args[n], XmNleftPosition, 10); n++;
    XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
    XtSetArg (args[n], XmNrightPosition, 90); n++;
    XtSetArg (args[n], XmNbottomAttachment, XmATTACH_POSITION); n++;
    XtSetArg (args[n], XmNbottomPosition, 83); n++;
    spaceText = XmCreateTextField(alignDialog, "Space", args, n);
    XtManageChild(spaceText);

    n = 0;
    XtSetArg (args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
    XtSetArg (args[n], XmNleftPosition, 10); n++;
    XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
    XtSetArg (args[n], XmNrightPosition, 90); n++;
    XtSetArg (args[n], XmNbottomAttachment, XmATTACH_POSITION); n++;
    XtSetArg (args[n], XmNbottomPosition, 95); n++;
    w = XmCreatePushButtonGadget (alignDialog, "Unmap", args, n);
    XtManageChild(w);
    XtAddCallback (w, XmNactivateCallback, 
		   (XtCallbackProc)UnmanageWidget, (XtPointer)alignDialog);
}

void DisplayAlignDialog (Widget, Widget parent, caddr_t)
{
    if ( !alignDialog ) 
        createAlignDialog (parent);
    XmTextFieldSetString(spaceText, "0");
    XtManageChild (alignDialog);
}



                           /************
			    * ROTATION *
			    ************/



static void Rotate(void*, int type)
{
    RotateGo(type, (int)XmToggleButtonGadgetGetState(center));
}

static void createRotateDialog (Widget parent)
{
    Widget rc ;
    Arg args [15];
    Widget buttons[7];

    int n = 0;
    XtSetArg (args[n], XmNresizePolicy, XmRESIZE_GROW); n++;
    XtSetArg (args[n], XmNwidth, 130); n++;
    XtSetArg (args[n], XmNheight, 280); n++;
    rotateDialog = XmCreateFormDialog(parent, "RotateDialog", args, n);

    n = 0;
    int i = 0;
    XtSetArg (args[i], XmNnumColumns, 2); i++;
    XtSetArg (args[i], XmNpacking, XmPACK_COLUMN); i++;
    XtSetArg (args[i], XmNleftAttachment, XmATTACH_POSITION); i++;
    XtSetArg (args[i], XmNleftPosition, 10); i++;
    XtSetArg (args[i], XmNtopAttachment, XmATTACH_POSITION); i++;
    XtSetArg (args[i], XmNtopPosition, 5); i++;
    rc = XmCreateRowColumn (rotateDialog, "RotateRowColumn", args, i);

    XtSetArg (args[0], XmNlabelPixmap, rotate45Icon);
    XtSetArg (args[1], XmNlabelType, XmPIXMAP);
    buttons[n] = XmCreatePushButtonGadget (rc, "Rotate45", args, 2);
    XtAddCallback (buttons[n], XmNactivateCallback, 
		   (XtCallbackProc) Rotate, ROTATE_45);
    n++;

    XtSetArg (args[0], XmNlabelPixmap, rotate135Icon);
    XtSetArg (args[1], XmNlabelType, XmPIXMAP);
    buttons[n] = XmCreatePushButtonGadget (rc, "Rotate135", args, 2);
    XtAddCallback (buttons[n], XmNactivateCallback, 
		   (XtCallbackProc) Rotate, (XtPointer) ROTATE_135);
    n++;

    XtSetArg (args[0], XmNlabelPixmap, rotate225Icon);
    XtSetArg (args[1], XmNlabelType, XmPIXMAP);
    buttons[n] = XmCreatePushButtonGadget (rc, "Rotate225", args, 2);
    XtAddCallback (buttons[n], XmNactivateCallback, 
		   (XtCallbackProc) Rotate, (XtPointer) ROTATE_225);
    n++;
    
    XtSetArg (args[0], XmNlabelPixmap, rotate315Icon);
    XtSetArg (args[1], XmNlabelType, XmPIXMAP);
    buttons[n] = XmCreatePushButtonGadget (rc, "Rotate315", args, 2);
    XtAddCallback (buttons[n], XmNactivateCallback, 
		   (XtCallbackProc) Rotate, (XtPointer) ROTATE_315);
    n++;
    
    XtSetArg (args[0], XmNlabelPixmap, rotate90Icon);
    XtSetArg (args[1], XmNlabelType, XmPIXMAP);
    buttons[n] = XmCreatePushButtonGadget (rc, "Rotate90", args, 2);
    XtAddCallback (buttons[n], XmNactivateCallback, 
		   (XtCallbackProc) Rotate, (XtPointer) ROTATE_90);
    n++;
    
    XtSetArg (args[0], XmNlabelPixmap, rotate180Icon);
    XtSetArg (args[1], XmNlabelType, XmPIXMAP);
    buttons[n] = XmCreatePushButtonGadget (rc, "Rotate180", args, 2);
    XtAddCallback (buttons[n], XmNactivateCallback, 
		   (XtCallbackProc) Rotate, (XtPointer) ROTATE_180);
    n++;

    XtSetArg (args[0], XmNlabelPixmap, rotate270Icon);
    XtSetArg (args[1], XmNlabelType, XmPIXMAP);
    buttons[n] = XmCreatePushButtonGadget (rc, "Rotate270", args, 2);
    XtAddCallback (buttons[n], XmNactivateCallback, 
		   (XtCallbackProc) Rotate, (XtPointer) ROTATE_270);
    n++;
    
    XtManageChildren (buttons, n);
    XtManageChild (rc);

    XmString str = XmStringCreate("Choose center", XmSTRING_DEFAULT_CHARSET);
    n = 0;
    XtSetArg (args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
    XtSetArg (args[n], XmNleftPosition, 10); n++;
    XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
    XtSetArg (args[n], XmNrightPosition, 90); n++;
    XtSetArg (args[n], XmNbottomAttachment, XmATTACH_POSITION); n++;
    XtSetArg (args[n], XmNbottomPosition, 84); n++;
    XtSetArg (args[n], XmNlabelString, str); n++;
    center = XmCreateToggleButtonGadget (rotateDialog, "Center", args, n);
    XtManageChild(center);
    XmStringFree(str);

    n = 0;
    XtSetArg (args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
    XtSetArg (args[n], XmNleftPosition, 10); n++;
    XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
    XtSetArg (args[n], XmNrightPosition, 90); n++;
    XtSetArg (args[n], XmNbottomAttachment, XmATTACH_POSITION); n++;
    XtSetArg (args[n], XmNbottomPosition, 95); n++;
    Widget w = XmCreatePushButtonGadget (rotateDialog, "Unmap", args, n);
    XtManageChild(w);
    XtAddCallback (w, XmNactivateCallback, 
		   (XtCallbackProc)UnmanageWidget, (XtPointer)rotateDialog);
}

void DisplayRotateDialog (Widget, Widget parent, caddr_t)
{
    if ( !rotateDialog ) 
        createRotateDialog (parent);
    XmToggleButtonGadgetSetState(center, False, False);
    XtManageChild (rotateDialog);
}



                           /*****************
			    * MIRROR IMAGES *
			    *****************/



static void createMirrorDialog (Widget parent)
{
    Widget rc ;
    Arg args [15];
    Widget buttons[3];

    int n = 0;
    XtSetArg (args[n], XmNresizePolicy, XmRESIZE_GROW); n++;
    XtSetArg (args[n], XmNwidth, 121); n++;
    XtSetArg (args[n], XmNheight, 150); n++;
    mirrorDialog = XmCreateFormDialog(parent, "MirrorDialog", args, n);

    n = 0;
    int i = 0;
    XtSetArg (args[i], XmNnumColumns, 2); i++;
    XtSetArg (args[i], XmNpacking, XmPACK_COLUMN); i++;
    XtSetArg (args[i], XmNleftAttachment, XmATTACH_POSITION); i++;
    XtSetArg (args[i], XmNleftPosition, 10); i++;
    XtSetArg (args[i], XmNtopAttachment, XmATTACH_POSITION); i++;
    XtSetArg (args[i], XmNtopPosition, 5); i++;
    rc = XmCreateRowColumn (mirrorDialog, "MirrorRowColumn", args, i);

    XtSetArg (args[0], XmNlabelPixmap, mirrorIIcon);
    XtSetArg (args[1], XmNlabelType, XmPIXMAP);
    buttons[n] = XmCreatePushButtonGadget (rc, "MirrorI", args, 2);
    XtAddCallback (buttons[n], XmNactivateCallback, 
		   (XtCallbackProc) MirrorGo, MIRROR_I);
    n++;

    XtSetArg (args[0], XmNlabelPixmap, mirrorIJIcon);
    XtSetArg (args[1], XmNlabelType, XmPIXMAP);
    buttons[n] = XmCreatePushButtonGadget (rc, "MirrorIJ", args, 2);
    XtAddCallback (buttons[n], XmNactivateCallback, 
		   (XtCallbackProc) MirrorGo, (XtPointer) MIRROR_IJ);
    n++;

    XtSetArg (args[0], XmNlabelPixmap, mirrorJIcon);
    XtSetArg (args[1], XmNlabelType, XmPIXMAP);
    buttons[n] = XmCreatePushButtonGadget (rc, "MirrorJ", args, 2);
    XtAddCallback (buttons[n], XmNactivateCallback, 
		   (XtCallbackProc) MirrorGo, (XtPointer) MIRROR_J);
    n++;

    XtManageChildren (buttons, n);
    XtManageChild (rc);

    n = 0;
    XtSetArg (args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
    XtSetArg (args[n], XmNleftPosition, 10); n++;
    XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
    XtSetArg (args[n], XmNrightPosition, 90); n++;
    XtSetArg (args[n], XmNbottomAttachment, XmATTACH_POSITION); n++;
    XtSetArg (args[n], XmNbottomPosition, 95); n++;
    Widget w = XmCreatePushButtonGadget (mirrorDialog, "Unmap", args, n);
    XtManageChild(w);
    XtAddCallback (w, XmNactivateCallback, 
		   (XtCallbackProc)UnmanageWidget, (XtPointer)mirrorDialog);
}

void DisplayMirrorDialog (Widget, Widget parent, caddr_t)
{
    if ( !mirrorDialog ) 
        createMirrorDialog (parent);
    XtManageChild (mirrorDialog);
}



                           /**************
			    * SPACING GO *
			    **************/



static void Spacing(void*, int type)
{
    char* s = XmTextFieldGetString(spacingText);
    SpacingGo(type, (double)atof(s));
}


static void createSpacingDialog (Widget parent)
{
    Widget rc ;
    Arg args [15];
    Widget buttons[2];

    int n = 0;
    XtSetArg (args[n], XmNresizePolicy, XmRESIZE_GROW); n++;
    XtSetArg (args[n], XmNwidth, 121); n++;
    XtSetArg (args[n], XmNheight, 180); n++;
    spacingDialog = XmCreateFormDialog(parent, "SpacingDialog", args, n);

    n = 0;
    int i = 0;
    XtSetArg (args[i], XmNnumColumns, 2); i++;
    XtSetArg (args[i], XmNpacking, XmPACK_COLUMN); i++;
    XtSetArg (args[i], XmNleftAttachment, XmATTACH_POSITION); i++;
    XtSetArg (args[i], XmNleftPosition, 10); i++;
    XtSetArg (args[i], XmNtopAttachment, XmATTACH_POSITION); i++;
    XtSetArg (args[i], XmNtopPosition, 5); i++;
    rc = XmCreateRowColumn (spacingDialog, "SpacingRowColumn", args, i);

    XtSetArg (args[0], XmNlabelPixmap, spaceXIcon);
    XtSetArg (args[1], XmNlabelType, XmPIXMAP);
    buttons[n] = XmCreatePushButtonGadget (rc, "SpacingX", args, 2);
    XtAddCallback (buttons[n], XmNactivateCallback, 
		   (XtCallbackProc) Spacing, SPACING_X);
    n++;

    XtSetArg (args[0], XmNlabelPixmap, spaceYIcon);
    XtSetArg (args[1], XmNlabelType, XmPIXMAP);
    buttons[n] = XmCreatePushButtonGadget (rc, "SpacingY", args, 2);
    XtAddCallback (buttons[n], XmNactivateCallback, 
		   (XtCallbackProc) Spacing, (XtPointer) SPACING_Y);
    n++;

    XtManageChildren (buttons, n);
    XtManageChild (rc);

    n = 0;
    XtSetArg (args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
    XtSetArg (args[n], XmNleftPosition, 10); n++;
    XtSetArg (args[n], XmNbottomAttachment, XmATTACH_POSITION); n++;
    XtSetArg (args[n], XmNbottomPosition, 50); n++;
    Widget w = XmCreateLabelGadget(spacingDialog, "SpaceLabel", args, n);
    XtManageChild(w);

    char* text_event_bindings =
	(char *) XtMalloc(strlen(TextEventBindings1) +
			  strlen(TextEventBindings2) +
			  strlen(TextEventBindings3) + 1);

    strcpy(text_event_bindings, TextEventBindings1);
    strcat(text_event_bindings, TextEventBindings2);
    strcat(text_event_bindings, TextEventBindings3);
    XtTranslations text_translations =
	XtParseTranslationTable(text_event_bindings);
    XtFree(text_event_bindings);
    n = 0;
    XtSetArg(args[n], XmNtranslations, text_translations); n++;
    XtSetArg (args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
    XtSetArg (args[n], XmNleftPosition, 10); n++;
    XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
    XtSetArg (args[n], XmNrightPosition, 90); n++;
    XtSetArg (args[n], XmNbottomAttachment, XmATTACH_POSITION); n++;
    XtSetArg (args[n], XmNbottomPosition, 70); n++;
    spacingText = XmCreateTextField(spacingDialog, "Space", args, n);
    XtManageChild(spacingText);

    n = 0;
    XtSetArg (args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
    XtSetArg (args[n], XmNleftPosition, 10); n++;
    XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
    XtSetArg (args[n], XmNrightPosition, 90); n++;
    XtSetArg (args[n], XmNbottomAttachment, XmATTACH_POSITION); n++;
    XtSetArg (args[n], XmNbottomPosition, 90); n++;
    w = XmCreatePushButtonGadget (spacingDialog, "Unmap", args, n);
    XtManageChild(w);
    XtAddCallback (w, XmNactivateCallback, 
		   (XtCallbackProc)UnmanageWidget, (XtPointer)spacingDialog);
}

void DisplaySpacingDialog (Widget, Widget parent, caddr_t)
{
    if ( !spacingDialog ) 
        createSpacingDialog (parent);
    XmTextFieldSetString(spacingText, "0");
    XtManageChild (spacingDialog);
}
