#ifdef REV_INFO
static char rcsid[] = "$Id: filemenu.cc,v 1.7 1992/12/10 16:23:22 jojic Exp $";
#endif
/*****************************************************************************
MODULE:  Go Editor
FILE:    filemenu.cc
AUTHOR:  Patricia Langle and Aline Jacomet
DATE:    May 14th 1991

DESCRIPTION:
    

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#include <sys/types.h>
#include <sys/stat.h>

#include "main.h"

#include <Xm/FileSB.h>
#include <Xm/SelectioB.h>
#include <Xm/ToggleBG.h>
#include <Xm/Form.h>
#include <Xm/LabelG.h>
#include <Xm/Text.h>
#include <Xm/RowColumn.h>
#include <Xm/PushBG.h>
#include <Xm/MessageB.h>

/*
 * EXTERN FUNCTIONS
 */

// in file goLoadSave.cc;
extern void LoadGo(void*, char*);
extern void SaveGo(char*, unsigned char, unsigned char);
extern void GoToPostscript(char*, unsigned char);
extern GoNew();

// in file message.cc;
#ifndef GoLICENSE
extern void NoLicense(void*, void*, caddr_t);
#endif
extern void ConfirmFileName(void*);
extern void ConfirmPSFileName(void*);
extern void NoFile(void*);




                                   /********
				    * LOAD *
				    ********/




static Widget loadBox = NULL;
static void cancelLoadBox (Widget);
static void okLoadBox (Widget, caddr_t, XmFileSelectionBoxCallbackStruct*);
static char saveDefFileName[256];


/**
FUNCTION
    InitLoadBox (parent)
DESCRIPTION
    Creates the file selection box and sets callbacks
**/
Widget InitLoadBox (Widget parent)
{
    XDefineCursor(XtDisplay((Widget)main_window),
		  XtWindow((Widget)main_window), clock_cursor);
    XFlush(XtDisplay((Widget)main_window));
    Widget l = XmCreateFileSelectionDialog (parent, "LoadObjectFile", NULL, 0);

    XtAddCallback (l, XmNcancelCallback,
		   (XtCallbackProc)cancelLoadBox, NULL);
    XtAddCallback (l, XmNokCallback,
		   (XtCallbackProc)okLoadBox, NULL);
    XtUnmanageChild (XmSelectionBoxGetChild (l, XmDIALOG_HELP_BUTTON));
    XUndefineCursor(XtDisplay((Widget)main_window),
		  XtWindow((Widget)main_window));
    XFlush(XtDisplay((Widget)main_window));
    return l;
}


/**
FUNCTION
    ActiveLoadBox (parent)
DESCRIPTION
    Creates loadBox if needed, map it and loads the chosen file
**/
void ActiveLoadBox (void*, void*, caddr_t)
{
    if ( !loadBox )
        loadBox = InitLoadBox ((Widget)main_window) ;
    XtManageChild (loadBox);
}


/**
FUNCTION
    cancelLoadBox()
DESCRIPTION
    Unmaps the load box widget.
**/
static void cancelLoadBox (Widget loadBox)
{
    XtUnmanageChild (loadBox);
}


/**
FUNCTION
    okLoadBox()
DESCRIPTION
    Loads the choosen file.
**/
static void okLoadBox (Widget loadBox, caddr_t,
		       XmFileSelectionBoxCallbackStruct* callbackStruct)
{
    char *filename;
    XtUnmanageChild (loadBox);
    XmStringGetLtoR (callbackStruct->value, XmSTRING_DEFAULT_CHARSET,
		     &filename);
    LoadGo (graphic, filename);
    XtFree (filename);

}



                                   /********
				    * SAVE *
				    ********/


static Widget	saveBox;
static Widget	saveFileNameWidget,
		allObjectWidget,
		selectedObjectWidget,
                asciiFileWidget,
                binaryFileWidget,
                headerWidget;
static void	cancelSaveBox();
static void	reinitSaveBox();
static void	verifySaveBox();


#ifndef GoLICENSE
static void no_binary_callback(Widget, caddr_t, XmToggleButtonCallbackStruct*);
#endif
char*           header = NULL;



/**
FUNCTION
    InitSaveBox (parent)
DESCRIPTION
    Creates the save dialog box and sets callbacks
**/
static Widget InitSaveBox (Widget parent)
{
    Widget ps, rb, rb1;
    Arg args[10];
    char *filename = NULL;

    XDefineCursor(XtDisplay((Widget)main_window),
		  XtWindow((Widget)main_window), clock_cursor);
    XFlush(XtDisplay((Widget)main_window));

    int n = 0;
    XtSetArg (args[n], XmNresizePolicy, XmRESIZE_GROW); n++;
    XtSetArg (args[n], XmNautoUnmanage, FALSE); n++;

    ps = XmCreateFormDialog (parent, "SaveObjectFile", args, n);

    n = 0;
    XtSetArg (args[n], XmNtopAttachment, XmATTACH_POSITION); n++;
    XtSetArg (args[n], XmNtopPosition, 5); n++;
    XtSetArg (args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
    XtSetArg (args[n], XmNleftPosition, 10); n++;
    XtManageChild (XmCreateLabelGadget (ps, "FileNameLabel", args, n));

    n = 0;
    XtSetArg (args[n], XmNtopAttachment, XmATTACH_POSITION); n++;
    XtSetArg (args[n], XmNtopPosition, 5); n++;
    XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
    XtSetArg (args[n], XmNrightPosition, 90); n++;
    XtSetArg (args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
    XtSetArg (args[n], XmNleftPosition, 35); n++;
    saveFileNameWidget = XmCreateText (ps, "FileNameText", args, n);
    XtManageChild (saveFileNameWidget);

    n = 0;
    XtSetArg (args[n], XmNtopAttachment, XmATTACH_POSITION); n++;
    XtSetArg (args[n], XmNtopPosition, 20); n++;
    XtSetArg (args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
    XtSetArg (args[n], XmNleftPosition, 10); n++;
    rb = XmCreateRadioBox (ps, "SelectionChoice", args, n);
    allObjectWidget = XmCreateToggleButtonGadget (rb, "AllObjects", NULL, 0);
    selectedObjectWidget = XmCreateToggleButtonGadget (rb, "SelectedObjects",
						       NULL, 0);
    n = 0;
    XtSetArg (args[n], XmNtopAttachment, XmATTACH_POSITION); n++;
    XtSetArg (args[n], XmNtopPosition, 40); n++;
    XtSetArg (args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
    XtSetArg (args[n], XmNleftPosition, 10); n++;
    rb1 = XmCreateRadioBox (ps, "FormatFile", args, n);
    binaryFileWidget = XmCreateToggleButtonGadget (rb1, "BinaryFile", NULL, 0);
    asciiFileWidget = XmCreateToggleButtonGadget (rb1, "AsciiFile", NULL, 0);
    XtManageChild (rb);
    XtManageChild (allObjectWidget);
    XtManageChild (selectedObjectWidget);
    XtManageChild (rb1);
    XtManageChild (asciiFileWidget);
    XtManageChild (binaryFileWidget);
#ifndef GoLICENSE
    XtAddCallback(binaryFileWidget, XmNvalueChangedCallback,
		  (XtCallbackProc)no_binary_callback, NULL);
#endif    

    n = 0;
    XtSetArg (args[n], XmNtopAttachment, XmATTACH_POSITION); n++;
    XtSetArg (args[n], XmNtopPosition, 68); n++;
    XtSetArg (args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
    XtSetArg (args[n], XmNleftPosition, 10); n++;
    Widget w = XmCreateLabelGadget(ps, "HeaderLabel", args, n);
    XtManageChild(w);

    n = 0;
    XtSetArg (args[n], XmNtopAttachment, XmATTACH_POSITION); n++;
    XtSetArg (args[n], XmNtopPosition, 65); n++;
    XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
    XtSetArg (args[n], XmNrightPosition, 90); n++;
    XtSetArg (args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (args[n], XmNleftOffset, 10); n++;
    XtSetArg (args[n], XmNleftWidget, w); n++;
    headerWidget = XmCreateText(ps, "HeaderText", args, n);
    XtManageChild(headerWidget);

    n = 0;
    XtSetArg(args[n], XmNbottomAttachment, XmATTACH_POSITION); n++;
    XtSetArg(args[n], XmNbottomPosition, 95); n++;
    XtSetArg(args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
    XtSetArg(args[n], XmNleftPosition, 10); n++;
    XtSetArg(args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
    XtSetArg(args[n], XmNrightPosition, 30); n++;
    Widget button = XmCreatePushButtonGadget((Widget) ps, "Apply", args, n);
    XtManageChild(button);
    XtAddCallback (button , XmNactivateCallback, (XtCallbackProc)verifySaveBox,
		   NULL);

    n = 0;
    XtSetArg(args[n], XmNbottomAttachment, XmATTACH_POSITION); n++;
    XtSetArg(args[n], XmNbottomPosition, 95); n++;
    XtSetArg(args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
    XtSetArg(args[n], XmNleftPosition, 40); n++;
    XtSetArg(args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
    XtSetArg(args[n], XmNrightPosition, 60); n++;
    button = XmCreatePushButtonGadget((Widget) ps, "Reinit", args, n);
    XtManageChild(button);
    XtAddCallback (button, XmNactivateCallback, (XtCallbackProc)reinitSaveBox,
		   NULL);

    n = 0;
    XtSetArg(args[n], XmNbottomAttachment, XmATTACH_POSITION); n++;
    XtSetArg(args[n], XmNbottomPosition, 95); n++;
    XtSetArg(args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
    XtSetArg(args[n], XmNleftPosition, 70); n++;
    XtSetArg(args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
    XtSetArg(args[n], XmNrightPosition, 90); n++;
    button = XmCreatePushButtonGadget((Widget) ps, "Cancel", args, n);
    XtManageChild(button);
    XtAddCallback (button, XmNactivateCallback, (XtCallbackProc)cancelSaveBox,
		   NULL);

    filename = XmTextGetString (saveFileNameWidget);
    strcpy (saveDefFileName, filename);
    XtFree (filename);

    reinitSaveBox();

    XUndefineCursor(XtDisplay((Widget)main_window),
		  XtWindow((Widget)main_window));
    XFlush(XtDisplay((Widget)main_window));
    
    return (ps);
}


/**
FUNCTION
    no_binary_callback()
DESCRIPTION
    
**/
#ifndef GoLICENSE
static void no_binary_callback(Widget tb, caddr_t,
			       XmToggleButtonCallbackStruct* call)
{
    if (call->set) {
	XmToggleButtonGadgetSetState(tb, False, False);
	XmToggleButtonGadgetSetState(asciiFileWidget, True, False);
	NoLicense(NULL, main_window, NULL);
    }
}
#endif


/**
FUNCTION
    ActiveSaveBox (saveBox)
DESCRIPTION
    Creates saveBox if needed, actives it and saves the Go tree
**/
void ActiveSaveBox (void*, void* parent, caddr_t)
{
    if ( !saveBox )
        saveBox = InitSaveBox ((Widget)parent);
    if (header)
	XmTextSetString(headerWidget, header);
    XtManageChild (saveBox);
}


/**
FUNCTION
    reinitSaveBox()
DESCRIPTION
    
**/
static void reinitSaveBox()
{
    Arg args[1];
    XmTextSetString (saveFileNameWidget, saveDefFileName);
    XtSetArg (args[0], XmNcursorPosition,
	strlen (saveDefFileName));
    XtSetValues (saveFileNameWidget, args, 1);

    XtSetArg (args[0], XmNset, True);
    XtSetValues (allObjectWidget, args, 1);

    XtSetArg (args[0], XmNset, False);
    XtSetValues (selectedObjectWidget, args, 1);

#ifdef GoLICENSE    
    XtSetArg (args[0], XmNset, True);
#else    
    XtSetArg (args[0], XmNset, False);
#endif
    XtSetValues (binaryFileWidget, args, 1);

#ifdef GoLICENSE    
    XtSetArg (args[0], XmNset, False);
#else
    XtSetArg (args[0], XmNset, True);
#endif
    XtSetValues (asciiFileWidget, args, 1);

    if (header)
	XmTextSetString(headerWidget, header);
}



/**
FUNCTION
    applySaveBox()
DESCRIPTION
    Unmaps the save box and loads Gos.
**/
void applySaveBox ()
{
    char *filename = XmTextGetString(saveFileNameWidget);
    Arg args[1];
    Boolean all;
    XtUnmanageChild (saveBox);
    strcpy (saveDefFileName, filename);
    XtFree (filename);
    
    XtSetArg (args[0], XmNset, &all);    
    XtGetValues (allObjectWidget, args, 1);
    
    XtSetArg (args[0], XmNset, NULL);
    XtGetValues (binaryFileWidget, args, 1);

    if (header)
	XtFree(header);
    header = XmTextGetString(headerWidget);
    
    SaveGo(saveDefFileName, (unsigned char)all, (unsigned char)args[0].value);
}

#ifndef S_ISREG
#define	S_ISREG(mode)	(((mode) & S_IFMT) == S_IFREG)
#endif

/**
FUNCTION
    verifySaveBox()
DESCRIPTION
    Verifies the file name.
**/
void verifySaveBox ()
{
    char *filename = XmTextGetString(saveFileNameWidget);

    /*
     At this point, test if the file exists and if so,
     ask for confirmation
     */

    struct stat file_status;
    if (stat(filename, &file_status) == 0) {
	// file exists;
	if (S_ISREG(file_status.st_mode))
	    ConfirmFileName(main_window);
	else NoFile(main_window);        // not a regular file;
    }
    else if (errno != ENOENT) 
	NoFile(main_window);             // file access error;
    else applySaveBox();                 // ok;

    XtFree (filename);
}

/**
FUNCTION
    cancelSaveBox()
DESCRIPTION
    Unmaps the save box.
**/
static void cancelSaveBox ()
{
    XtUnmanageChild (saveBox);
}




                                   /**************
				    * POSTSCRIPT *
				    **************/


static Widget	postscriptBox;
static Widget	postscriptFileNameWidget,
		p_allObjectWidget,
		p_selectedObjectWidget;
static void	cancelPostscriptBox();
static void	reinitPostscriptBox();
static void	verifyPostscriptBox();
       void	applyPostscriptBox();



/**
FUNCTION
    InitPostscriptBox (parent)
DESCRIPTION
    Creates the postscript dialog box and sets callbacks
**/
static Widget InitPostscriptBox (Widget parent)
{
    Widget ps, rb;
    Arg args[6];
    char *filename = NULL;

    XDefineCursor(XtDisplay((Widget)main_window),
		  XtWindow((Widget)main_window), clock_cursor);
    XFlush(XtDisplay((Widget)main_window));

    int n = 0;
    XtSetArg (args[n], XmNresizePolicy, XmRESIZE_GROW); n++;
    XtSetArg (args[n], XmNautoUnmanage, FALSE); n++;

    ps = XmCreateFormDialog (parent, "PostscriptObjectFile", args, n);

    n = 0;
    XtSetArg (args[n], XmNtopAttachment, XmATTACH_POSITION); n++;
    XtSetArg (args[n], XmNtopPosition, 12); n++;
    XtSetArg (args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
    XtSetArg (args[n], XmNleftPosition, 10); n++;
    XtManageChild (XmCreateLabelGadget (ps, "FileNameLabel", args, n));

    n = 0;
    XtSetArg (args[n], XmNtopAttachment, XmATTACH_POSITION); n++;
    XtSetArg (args[n], XmNtopPosition, 10); n++;
    XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
    XtSetArg (args[n], XmNrightPosition, 90); n++;
    XtSetArg (args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
    XtSetArg (args[n], XmNleftPosition, 35); n++;
    postscriptFileNameWidget = XmCreateText (ps, "FileNameText", args, n);
    XtManageChild (postscriptFileNameWidget);

    n = 0;
    XtSetArg (args[n], XmNtopAttachment, XmATTACH_POSITION); n++;
    XtSetArg (args[n], XmNtopPosition, 40); n++;
    XtSetArg (args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
    XtSetArg (args[n], XmNleftPosition, 10); n++;
    rb = XmCreateRadioBox (ps, "SelectionChoice", args, n);
    p_allObjectWidget = XmCreateToggleButtonGadget (rb, "AllObjects", NULL, 0);
    p_selectedObjectWidget = XmCreateToggleButtonGadget (rb, "SelectedObjects",
						       NULL, 0);
    XtManageChild (rb);
    XtManageChild (p_allObjectWidget);
    XtManageChild (p_selectedObjectWidget);

    n = 0;
    XtSetArg(args[n], XmNbottomAttachment, XmATTACH_POSITION); n++;
    XtSetArg(args[n], XmNbottomPosition, 90); n++;
    XtSetArg(args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
    XtSetArg(args[n], XmNleftPosition, 10); n++;
    XtSetArg(args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
    XtSetArg(args[n], XmNrightPosition, 30); n++;
    Widget button = XmCreatePushButtonGadget((Widget) ps, "Apply", args, n);
    XtManageChild(button);
    XtAddCallback (button , XmNactivateCallback,
		   (XtCallbackProc)verifyPostscriptBox, NULL);

    n = 0;
    XtSetArg(args[n], XmNbottomAttachment, XmATTACH_POSITION); n++;
    XtSetArg(args[n], XmNbottomPosition, 90); n++;
    XtSetArg(args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
    XtSetArg(args[n], XmNleftPosition, 40); n++;
    XtSetArg(args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
    XtSetArg(args[n], XmNrightPosition, 60); n++;
    button = XmCreatePushButtonGadget((Widget) ps, "Reinit", args, n);
    XtManageChild(button);
    XtAddCallback (button, XmNactivateCallback,
		   (XtCallbackProc)reinitPostscriptBox, NULL);

    n = 0;
    XtSetArg(args[n], XmNbottomAttachment, XmATTACH_POSITION); n++;
    XtSetArg(args[n], XmNbottomPosition, 90); n++;
    XtSetArg(args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
    XtSetArg(args[n], XmNleftPosition, 70); n++;
    XtSetArg(args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
    XtSetArg(args[n], XmNrightPosition, 90); n++;
    button = XmCreatePushButtonGadget((Widget) ps, "Cancel", args, n);
    XtManageChild(button);
    XtAddCallback (button, XmNactivateCallback,
		   (XtCallbackProc)cancelPostscriptBox, NULL);

    filename = XmTextGetString (postscriptFileNameWidget);
    strcpy (saveDefFileName, filename);
    XtFree (filename);

    reinitPostscriptBox();

    XUndefineCursor(XtDisplay((Widget)main_window),
		  XtWindow((Widget)main_window));
    XFlush(XtDisplay((Widget)main_window));
    
    return (ps);
}


/**
FUNCTION
    ActivePostscriptBox (postscriptBox)
DESCRIPTION
    Creates postscriptBox if needed, actives it and postscripts the Go tree
**/
void ActivePostscriptBox (void*, void* parent, caddr_t)
{
    if ( !postscriptBox )
        postscriptBox = InitPostscriptBox ((Widget)parent);
    XtManageChild (postscriptBox);
}


/**
FUNCTION
    reinitPostscriptBox()
DESCRIPTION
    
**/
static void reinitPostscriptBox()
{
    Arg args[1];
    XmTextSetString (postscriptFileNameWidget, saveDefFileName);
    XtSetArg (args[0], XmNcursorPosition,
	strlen (saveDefFileName));
    XtSetValues (postscriptFileNameWidget, args, 1);

    XtSetArg (args[0], XmNset, True);
    XtSetValues (p_allObjectWidget, args, 1);

    XtSetArg (args[0], XmNset, False);
    XtSetValues (p_selectedObjectWidget, args, 1);
}


/**
FUNCTION
    verifyPostscriptBox()
DESCRIPTION
    Verifies the file name.
**/
void verifyPostscriptBox ()
{
    char *filename = XmTextGetString(postscriptFileNameWidget);

    /*
     At this point, test if the file exists and if so,
     ask for confirmation
     */

    struct stat file_status;
    if (stat(filename, &file_status) == 0) {
	// file exists;
	if (S_ISREG(file_status.st_mode))
	    ConfirmPSFileName(main_window);
	else NoFile(main_window);        // not a regular file;
    }
    else if (errno != ENOENT) 
	NoFile(main_window);             // file access error;
    else applyPostscriptBox();           // ok;

    XtFree (filename);
}


/**
FUNCTION
    applyPostscriptBox()
DESCRIPTION
    Unmaps the postscript box and loads Gos.
**/
void applyPostscriptBox ()
{
    char *filename = XmTextGetString(postscriptFileNameWidget);
    Arg args[1];
    Boolean all;
    XtUnmanageChild (postscriptBox);
    strcpy (saveDefFileName, filename);
    XtFree (filename);
    
    XtSetArg (args[0], XmNset, &all);    
    XtGetValues (p_allObjectWidget, args, 1);
    
    GoToPostscript(saveDefFileName, (unsigned char)all);
}


/**
FUNCTION
    cancelPostscriptBox()
DESCRIPTION
    Unmaps the postscript box.
**/
static void cancelPostscriptBox ()
{
    XtUnmanageChild (postscriptBox);
}



                                 /********
				  * QUIT *
				  ********/


/*
      User Interface for QUIT
      =======================
      Functions are:
            Quit()
            QuitDialog()
	    ThatIsTheEndOfTheWorld()
*/



static void ThatIsTheEndOfTheWorld (Display* display, Window window, int) 
{
   XWindowAttributes windowAttributes;
   Window eowWindow;
   GC eowGCcopy, eowGCclear;
   XGCValues eowGCValues;
   int i, delta;


   XGetWindowAttributes (display, window, &windowAttributes);
   eowWindow = XCreateSimpleWindow (display, window,
				    windowAttributes.x, windowAttributes.y,
				    windowAttributes.width,
				    windowAttributes.height,
				    1,
				    BlackPixel (display, 0),
				    WhitePixel (display, 0));
  XSetWindowBackgroundPixmap (display, eowWindow, None);
  XMapWindow (display, eowWindow);
  XFlush(display);

  /* Selon le type on detruit le contenu */
  eowGCcopy = XCreateGC (display, eowWindow, 0, &eowGCValues);
  eowGCValues.function = GXclear;
  eowGCclear = XCreateGC (display, eowWindow, GCFunction, &eowGCValues);
  for (i=0, delta=16; i<windowAttributes.width; i+=delta) {
     XCopyArea (display, eowWindow, eowWindow, eowGCcopy,
		i, 0, 
		windowAttributes.width-i, windowAttributes.height,
		i+delta, 0);
     XCopyArea (display, eowWindow, eowWindow, eowGCclear,
		0, 0, i, windowAttributes.height, 0, 0);
     XFlush (display);
  }
  exit (0);
}


static void Quit (Widget, Widget parent)
{
    ThatIsTheEndOfTheWorld (XtDisplay(parent), XtWindow(parent), 0);
}


void QuitDialog(void*, void* parent, caddr_t)
{
    static Widget quit_dialog = NULL;

    if (!quit_dialog) {
	Arg args[3];
	int n = 0;
	XtSetArg(args[n], XmNautoUnmanage, True); n++;
	XtSetArg(args[n], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); n++;
	XtSetArg(args[n], XmNdefaultButtonType, XmDIALOG_OK_BUTTON); n++;
        quit_dialog = XmCreateQuestionDialog ((Widget)parent, "QuitDialog",
					      args, n);
        XtUnmanageChild (XmMessageBoxGetChild
                           (quit_dialog, XmDIALOG_HELP_BUTTON));
        XtAddCallback (quit_dialog, XmNokCallback,
		       (XtCallbackProc)Quit, (XtPointer)parent);
    }
    XtManageChild (quit_dialog);
}



                               /*******
				* NEW *
				*******/


void NewDialog(void*, void* parent, caddr_t)
{
    static Widget confirm_new = NULL;
    if (! confirm_new) {
	Arg args[3];
	int n = 0;
	XtSetArg(args[n], XmNautoUnmanage, True); n++;
	XtSetArg(args[n], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); n++;
	XtSetArg(args[n], XmNdefaultButtonType, XmDIALOG_OK_BUTTON); n++;
	confirm_new = 
	    XmCreateQuestionDialog((Widget) parent, "ConfirmNew", args, n);
	XtUnmanageChild(XmMessageBoxGetChild(confirm_new,
					     XmDIALOG_HELP_BUTTON));
	XtAddCallback(confirm_new, XmNokCallback, 
		      (XtCallbackProc)GoNew, NULL);
    }
    XtManageChild(confirm_new);
}
