#ifdef REV_INFO
static char rcsid[] = "$Id: fontsel.cc,v 1.1.1.1 1992/03/18 09:05:20 uep Exp $";
#endif
/*****************************************************************************
MODULE:  Go Editor
FILE:    fontsel.cc
AUTHOR:  Aline Jacomet
DATE:    May 24th 1991

DESCRIPTION:
    Font Management

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include <Go/Tools/Set.h>
#include <Go/Tools/Alloc.h>

#include <Xm/Text.h>
#include <Xm/ToggleBG.h>
#include <Xm/SelectioB.h>

#include "classes.h"
#include "main.h"


/***
 *  Global
 ***/
extern Widget text_font;
extern Widget edit_text_font;
extern Widget fontToggle;

extern int WhichAction;

extern Display* dpy;


/***
 *  Static Variables
 ***/
static Widget fontboxDialog;



/*
 * EXTERN FUNCTIONS
 */

// in file goAttrib.cc;
extern void ModifyGoFont(char*);

// in file message.cc;
extern void Error(void*);
extern void NotInList(void*, void*, caddr_t);



static void Apply(Widget, caddr_t, XmSelectionBoxCallbackStruct* cb)
{
    char* name;
    XmStringGetLtoR(cb->value, XmSTRING_DEFAULT_CHARSET, &name);
    if (WhichAttributes == NAMED)
	XmTextSetString(text_font, name);
    else {
	if (WhichAction != FAST_MODIFY) {
	    XmTextSetString(edit_text_font, name);
	    if (WhichAction == MODIFY)
		XmToggleButtonGadgetSetState(fontToggle, True, False);
	}
    }
    ModifyGoFont(name);
    GFREE(name);
}



static void CreateSelectionDialog()
{
    Arg args[2];

    int n = 0;
    XtSetArg(args[n], XmNmustMatch, True); n++;
    XtSetArg(args[n], XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL); n++;
    fontboxDialog = XmCreateSelectionDialog((Widget)main_window,
					       "FontBox", args, n);
    XtAddCallback(fontboxDialog, XmNokCallback,
		  (XtCallbackProc)Apply, NULL);
    XtAddCallback(fontboxDialog, XmNnoMatchCallback,
		  (XtCallbackProc)NotInList, (XtPointer)main_window);
    Widget wid = XmSelectionBoxGetChild(fontboxDialog,
					XmDIALOG_APPLY_BUTTON);
    XtUnmanageChild(wid);
    wid = XmSelectionBoxGetChild(fontboxDialog, XmDIALOG_HELP_BUTTON);
    XtUnmanageChild(wid);

    int nb;
    char** fontnames = XListFonts(dpy, "-*", 1000, &nb);
    if (fontnames) {
	GSet list;
	for (n = 0; n < nb; n++)
	    list.add((const void*)XmStringCreate(fontnames[n],
						 XmSTRING_DEFAULT_CHARSET));
	const void** tmp = list.allSet(nb);
	XtSetArg(args[0], XmNlistItems, (XmStringTable)tmp);
	XtSetArg(args[1], XmNlistItemCount, nb);
	XtSetValues(fontboxDialog, args, 2);
	XFreeFontNames(fontnames);
    } else {
	// error;
	Error(main_window);
    }
    
}



/**
FUNCTION
    MapFontList()
DESCRIPTION
    Creates if necessary, and maps the font selection box.
**/
void MapFontList(Widget, int wa, caddr_t)
{
    Boolean create = (fontboxDialog == NULL);
    WhichAttributes = wa;
    
    if (create) {
	XDefineCursor(XtDisplay((Widget)graphic), XtWindow((Widget)graphic),
		      clock_cursor);
	XDefineCursor(XtDisplay((Widget)main_window),
			XtWindow((Widget)main_window), clock_cursor);
	XFlush(XtDisplay((Widget)graphic));
	CreateSelectionDialog();
    }
    
    if (create) {
	XFlush(XtDisplay((Widget)graphic));
	XUndefineCursor(XtDisplay((Widget)graphic), XtWindow((Widget)graphic));
	XUndefineCursor(XtDisplay((Widget)main_window),
			XtWindow((Widget)main_window));
    }
    XtManageChild(fontboxDialog);
}


