#ifdef REV_INFO
static char rcsid[] = "$Id: goAttrib.cc,v 1.3 1992/12/10 16:24:25 jojic Exp $";
#endif
/*****************************************************************************
MODULE:  Go Editor
FILE:    goAttrib.cc
AUTHOR:  Aline Jacomet
DATE:    May 29th 1991

DESCRIPTION:
    

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include <assert.h>
#include <stdio.h>

#include "goMain.h"

#include <Go/XGoInterf.h>

#include <Go/Tools/Rgb.h>
#include <Go/Tools/Alloc.h>
#include <Go/Tools/Dash.h>
#include <Go/Tools/StringAssoc.h>
#include <Go/Tools/Color.h>
#include <Go/Tools/Font.h>
#include <Go/Tools/TextBrush.h>

#include <Go/Geometric.h>
#include <Go/Polyline.h>
#include <Go/Text.h>
#include <Go/ChoosePoint.h>
#include <Go/FullSensor.h>

#include "classes.h"


/*
 * VARIABLES
 */

extern GStringAssoc NamedPen;
extern GStringAssoc NamedBrush;
extern GStringAssoc NamedText;

extern PenAttributes reinit_pen;
extern BrushAttributes reinit_brush;
extern TextAttributes reinit_text;
extern PenAttributes pen;
extern BrushAttributes brush;
extern TextAttributes text;
extern PenAttributes named_pen;
extern BrushAttributes named_brush;
extern TextAttributes named_text;

extern GDash* goDash;

GAssoc ColorToName;

static GNumber height;   // text's height;



/*
 * FUNCTIONS
 */

// in textsheet.cc;
extern void ErrorFont();

// in message.cc;
extern void NotGo(void*);

static void ShColorDestroyed(void*, const GShColor*);


                          /*********
			   * TOOLS *
			   *********/


#define SELECTED_PART(text) \
((GoTextual*)text)->firstSelected() != -1 && \
    ((GoTextual*)text)->lastSelected() != -1


static char* RgbToHexaString(const GRgb* rgb)
{
    static char str[8];
    str[0] = '#';
    sprintf(&str[1], "%2x", rgb->red / 257);
    if (str[1] == ' ')
	str[1] = '0';
    sprintf(&str[3], "%2x", rgb->green / 257);
    if (str[3] == ' ')
	str[3] = '0';
    sprintf(&str[5], "%2x", rgb->blue / 257);
    if (str[5] == ' ')
	str[5] = '0';
    str[7] = '\0';
    return str;
}


GRgb HexaStringToRgb(const char* name)
{
    GRgb rgb;
    int value;
    char red[3];
    strncpy(red, name+1, 2);
    red[2] = '\0';
    sscanf(red, "%x", &value);
    rgb.red = (ushort)(((ushort)value) * 257);

    char green[3];
    strncpy(green, name+3, 2);
    green[2] = '\0';
    sscanf(green, "%x", &value);
    rgb.green = (ushort)(((ushort)value) * 257);

    char blue[3];
    strncpy(blue, name+5, 2);
    blue[2] = '\0';
    sscanf(blue, "%x", &value);
    rgb.blue = (ushort)(((ushort)value) * 257);

    return rgb;
}




                          /**********************
			   * UNNAMED ATTRIBUTES *
			   **********************/



/***
 *   Pen Modifications
 ***/


void show_outline(void* state, Go* go)
{
    if (go != named_goroot && go != edit_goroot && go != goroot &&
	go->getMetaclass()->isSubclass(GoGeometric::metaclass()))
	((GoGeometric*)go)->outline(*((GBool*)state)); 
}

void ModifyGoOutline(int set)
{
    GBool state = (GBool)set;
    switch (WhichAttributes) {
      case NAMED:
	named_goroot->forEveryGo(show_outline, &state);
	break;
      case EDIT:
	edit_goroot->forEveryGo(show_outline, &state);
	break;
      case MAIN:
	goroot->forEverySelectedGo(show_outline, &state);
	break;
    }
}



void show_rounded(void* state, Go* go)
{
    if (go != named_goroot && go != edit_goroot && go != goroot &&
	go->getMetaclass()->isSubclass(GoPolyline::metaclass()))
	((GoPolyline*)go)->round(*((GBool*)state)); 
}

void ModifyGoRounded(int set)
{
    GBool state = (GBool)set;
    switch (WhichAttributes) {
      case NAMED:
	named_goroot->forEveryGo(show_rounded, &state);
	break;
      case EDIT:
	edit_goroot->forEveryGo(show_rounded, &state);
	break;
      case MAIN:
	goroot->forEverySelectedGo(show_rounded, &state);
	break;
    }
}




void modify_line_style(void* l, Go* go)
{
    if (go != named_goroot && go != edit_goroot && go != goroot &&
	go->getMetaclass()->isSubclass(GoGeometric::metaclass())) {
	if (((GoGeometric*)go)->pen().isNamed()) {
	    GShPen* pen = ((GoGeometric*)go)->pen().shPen()->dup();
	    GRename(pen, NULL);
	    GSetLineStyle(pen, *((ushort*)l));
	    ((GoGeometric*)go)->pen().modify(pen);
	    pen->destroy();
	} else 
	    ((GoGeometric*)go)->pen().lineStyle(*((ushort*)l));
	((GoGeometric*)go)->pen().dash(goDash);	
    }
}

void ModifyGoLineStyle(int cs)
{
    ushort c = (ushort)cs;
    switch (WhichAttributes) {
      case NAMED:
	named_goroot->forEveryGo(modify_line_style, &c);
	break;
      case EDIT:
	edit_goroot->forEveryGo(modify_line_style, &c);
	break;
      case MAIN:
	goroot->forEverySelectedGo(modify_line_style, &c);
	break;
    }
}





void modify_line_width(void* l, Go* go)
{
    if (go != named_goroot && go != edit_goroot && go != goroot &&
	go->getMetaclass()->isSubclass(GoGeometric::metaclass())) {
	if (((GoGeometric*)go)->pen().isNamed()) {
	    GShPen* pen = ((GoGeometric*)go)->pen().shPen()->dup();
	    GRename(pen, NULL);
	    GSetLineWidth(pen, *((float*)l));
	    ((GoGeometric*)go)->pen().modify(pen);
	    pen->destroy();
	} else 
	    ((GoGeometric*)go)->pen().lineWidth(*((float*)l));
    }
}

void ModifyGoLineWidth(double l)
{
    GNumber lw = (GNumber)l;
    switch (WhichAttributes) {
      case NAMED:
	named_goroot->forEveryGo(modify_line_width, &lw);
	break;
      case EDIT:
	edit_goroot->forEveryGo(modify_line_width, &lw);
	break;
      case MAIN:
	goroot->forEverySelectedGo(modify_line_width, &lw);
	break;
    }
}




void modify_cap_style(void* l, Go* go)
{
    if (go != named_goroot && go != edit_goroot && go != goroot &&
	go->getMetaclass()->isSubclass(GoGeometric::metaclass())) {
	if (((GoGeometric*)go)->pen().isNamed()) {
	    GShPen* pen = ((GoGeometric*)go)->pen().shPen()->dup();
	    GRename(pen, NULL);
	    GSetCapStyle(pen, *((ushort*)l));
	    ((GoGeometric*)go)->pen().modify(pen);
	    pen->destroy();
	} else 
	    ((GoGeometric*)go)->pen().capStyle(*((ushort*)l));
    }
}

void ModifyGoCapStyle(int cs)
{
    ushort c = (ushort)cs;
    switch (WhichAttributes) {
      case NAMED:
	named_goroot->forEveryGo(modify_cap_style, &c);
	break;
      case EDIT:
	edit_goroot->forEveryGo(modify_cap_style, &c);
	break;
      case MAIN:
	goroot->forEverySelectedGo(modify_cap_style, &c);
	break;
    }	
}




void modify_join_style(void* l, Go* go)
{
    if (go != named_goroot && go != edit_goroot && go != goroot &&
	go->getMetaclass()->isSubclass(GoGeometric::metaclass())) {
	if (((GoGeometric*)go)->pen().isNamed()) {
	    GShPen* pen = ((GoGeometric*)go)->pen().shPen()->dup();
	    GRename(pen, NULL);
	    GSetJoinStyle(pen, *((ushort*)l));
	    ((GoGeometric*)go)->pen().modify(pen);
	    pen->destroy();
	} else
	    ((GoGeometric*)go)->pen().joinStyle(*((ushort*)l));
    }
}

void ModifyGoJoinStyle(int cs)
{
    ushort c = (ushort)cs;
    switch (WhichAttributes) {
      case NAMED:
	named_goroot->forEveryGo(modify_join_style, &c);
	break;
      case EDIT:
	edit_goroot->forEveryGo(modify_join_style, &c);
	break;
      case MAIN:
	goroot->forEverySelectedGo(modify_join_style, &c);
	break;
    }
}





void modify_pen_color(void* c, Go* go)
{
    if (go != named_goroot && go != edit_goroot && go != goroot &&
	go->getMetaclass()->isSubclass(GoGeometric::metaclass())) {
	if (((GoGeometric*)go)->pen().isNamed()) {
	    GShPen* pen = ((GoGeometric*)go)->pen().shPen()->dup();
	    GRename(pen, NULL);
	    GSetColor(pen, ((GColor*)c)->shColor());
	    ((GoGeometric*)go)->pen().modify(pen);
	    pen->destroy();
	} else
	    ((GoGeometric*)go)->pen().color((GColor*)c);
    }
}

void modify_pen_alt_color(void* c, Go* go)
{
    if (go != named_goroot && go != edit_goroot && go != goroot &&
	go->getMetaclass()->isSubclass(GoGeometric::metaclass())) {
	if (((GoGeometric*)go)->pen().isNamed()) {
	    GShPen* pen = ((GoGeometric*)go)->pen().shPen()->dup();
	    GRename(pen, NULL);
	    GSetAltColor(pen, ((GColor*)c)->shColor());
	    ((GoGeometric*)go)->pen().modify(pen);
	    pen->destroy();
	} else
	    ((GoGeometric*)go)->pen().altColor((GColor*)c);
    }
}




void modify_named_pensheet(void* p, Go* go)
{
    if (go != named_goroot && go != edit_goroot &&
	go->getMetaclass()->isSubclass(GoGeometric::metaclass()))
	((GoGeometric*)go)->pen().modify((GShPen*)p);
}

void modify_pensheet(void* v, Go* go)
{
    if (go != named_goroot && go != edit_goroot &&
	go->getMetaclass()->isSubclass(GoGeometric::metaclass())) {
	ulong mask = GCOLOR | GALT_COLOR | GLINE_WIDTH | GLINE_STYLE |
	    GCAP_STYLE | GJOIN_STYLE | GDASH;
	((GoGeometric*)go)->pen().modify(mask, (GPenValues*)v);
    }
}

void ModifyGoEditPenSheet()
{
    GPenValues values;
    values.line_width = atof(pen.LineWidth);
    values.line.style = pen.LineStyle;
    values.line.cap = pen.CapStyle;
    values.line.join = pen.JoinStyle;
    values.dash = goDash->shDash()->dup();
    values.color = GCreateShColor(&pen.Rgb_Color);
    int nb;
    if (! ColorToName.get(values.color, nb)) {
	values.color->destroyPostcond(ShColorDestroyed, NULL);
	ColorToName.set(values.color,
			(strlen(pen.Color) + 1) * sizeof(char),
			pen.Color);
    }
    values.alt_color = GCreateShColor(&pen.Rgb_AltColor);
    if (! ColorToName.get(values.alt_color, nb)) {
	values.alt_color->destroyPostcond(ShColorDestroyed, NULL);
	ColorToName.set(values.alt_color,
			(strlen(pen.AltColor) + 1) * sizeof(char),
			pen.AltColor);
    }
    edit_goroot->forEveryGo(modify_pensheet, &values);
    
    values.color->destroy();
    values.alt_color->destroy();
    values.dash->destroy();
}


void ModifyGoNamedPenSheet()
{
    GPenValues values;
    values.line_width = atof(named_pen.LineWidth);
    values.line.style = named_pen.LineStyle;
    values.line.cap = named_pen.CapStyle;
    values.line.join = named_pen.JoinStyle;
    values.dash = goDash->shDash()->dup();
    values.color = GCreateShColor(&named_pen.Rgb_Color);
    int nb;
    if (! ColorToName.get(values.color, nb)) {
	values.color->destroyPostcond(ShColorDestroyed, NULL);
	ColorToName.set(values.color,
			(strlen(named_pen.Color) + 1) * sizeof(char),
			named_pen.Color);
    }
    values.alt_color = GCreateShColor(&named_pen.Rgb_AltColor);
    if (! ColorToName.get(values.alt_color, nb)) {
	values.alt_color->destroyPostcond(ShColorDestroyed, NULL);
	ColorToName.set(values.alt_color,
			(strlen(named_pen.AltColor) + 1) * sizeof(char),
			named_pen.AltColor);
    }
    named_goroot->forEveryGo(modify_pensheet, &values);
    
    values.color->destroy();
    values.alt_color->destroy();
    values.dash->destroy();
}




/***
 *  Brush Modifications
 ***/


void show_background(void* state, Go* go)
{
    if (go != named_goroot && go != edit_goroot && go != goroot)
	if (go->getMetaclass()->isSubclass(GoGeometric::metaclass())) {
	    ((GoGeometric*)go)->background(*((GBool*)state));
	} else
	    if (go->getMetaclass()->isSubclass(GoTextual::metaclass()))
		if (go->getMetaclass()->isSubclass(GoText::metaclass()) &&
		    SELECTED_PART(go))
		    ((GoText*)go)->background(*((GBool*)state),
					      ((GoText*)go)->firstSelected(),
					      ((GoText*)go)->lastSelected());
		else
		    ((GoTextual*)go)->mapBackground(*((GBool*)state));
}

void ModifyGoBackground(int set)
{
    GBool state = (GBool)set;
    switch (WhichAttributes) {
      case NAMED:
	named_goroot->forEveryGo(show_background, &state);
	break;
      case EDIT:
	edit_goroot->forEveryGo(show_background, &state);
	break;
      case MAIN:
	goroot->forEverySelectedGo(show_background, &state);
	break;
    }
}






void modify_fill_rule(void* l, Go* go)
{
    if (go != named_goroot && go != edit_goroot && go != goroot &&
	go->getMetaclass()->isSubclass(GoGeometric::metaclass())) {
	if (((GoGeometric*)go)->brush().isNamed()) {
	    GShBrush* brush = ((GoGeometric*)go)->brush().shBrush()->dup();
	    GRename(brush, NULL);
	    GSetFillRule(brush, *((ushort*)l));
	    ((GoGeometric*)go)->brush().modify(brush);
	    brush->destroy();
	} else
	    ((GoGeometric*)go)->brush().fillRule(*((ushort*)l));
    } else
	if (go != named_goroot && go != edit_goroot && go != goroot &&
	    go->getMetaclass()->isSubclass(GoTextual::metaclass())) {
	    if (go->getMetaclass()->isSubclass(GoText::metaclass()) &&
		SELECTED_PART(go)) {
		GBrushValues values;
		ulong mask = GFILL_RULE;
		values.fill.rule = *((ushort*)l);
		((GoText*)go)->textBackgroundBrush(mask, &values,
						((GoText*)go)->firstSelected(),
						((GoText*)go)->lastSelected());
		((GoText*)go)->background(GTRUE,
					  ((GoText*)go)->firstSelected(),
					  ((GoText*)go)->lastSelected());
	    } else {
		if (((GoTextual*)go)->backgroundBrush().isNamed()) {
		    GShBrush* brush =
			((GoTextual*)go)->backgroundBrush().shBrush()->dup();
		    GRename(brush, NULL);
		    GSetFillRule(brush, *((ushort*)l));
		    ((GoTextual*)go)->backgroundBrush().modify(brush);
		    brush->destroy();
		} else
		    ((GoTextual*)go)->backgroundBrush().fillRule(*((ushort*)l));
	    }
	}
}

void ModifyGoFillRule(int cs)
{
    ushort c = (ushort)cs;
    switch (WhichAttributes) {
      case NAMED:
	named_goroot->forEveryGo(modify_fill_rule, &c);
	break;
      case EDIT:
	edit_goroot->forEveryGo(modify_fill_rule, &c);
	break;
      case MAIN:
	goroot->forEverySelectedGo(modify_fill_rule, &c);
	break;
    }
}







void modify_brush_color(void* c, Go* go)
{
    if (go != named_goroot && go != edit_goroot && go != goroot &&
	go->getMetaclass()->isSubclass(GoGeometric::metaclass())) {
	if (((GoGeometric*)go)->brush().isNamed()) {
	    GShBrush* brush = ((GoGeometric*)go)->brush().shBrush()->dup();
	    GRename(brush, NULL);
	    GSetColor(brush, ((GColor*)c)->shColor());
	    ((GoGeometric*)go)->brush().modify(brush);
	    brush->destroy();
	} else 
	    ((GoGeometric*)go)->brush().color((GColor*)c);
    } else 
	if (go != named_goroot && go != edit_goroot && go != goroot &&
	    go->getMetaclass()->isSubclass(GoTextual::metaclass())) {
	    if (go->getMetaclass()->isSubclass(GoText::metaclass()) &&
		SELECTED_PART(go)) {
		GBrushValues values;
		ulong mask = GCOLOR;
		values.color = ((GColor*)c)->shColor()->dup();
		((GoText*)go)->textBackgroundBrush(mask, &values,
						((GoText*)go)->firstSelected(),
						((GoText*)go)->lastSelected());
		((GoText*)go)->background(GTRUE,
					  ((GoText*)go)->firstSelected(),
					  ((GoText*)go)->lastSelected());
		values.color->destroy();
	    } else {
		if (((GoTextual*)go)->backgroundBrush().isNamed()) {
		    GShBrush* brush =((GoTextual*)go)->backgroundBrush().shBrush()->dup();
		    GRename(brush, NULL);
		    GSetColor(brush, ((GColor*)c)->shColor());
		    ((GoTextual*)go)->backgroundBrush().modify(brush);
		brush->destroy();
		} else
		    ((GoTextual*)go)->backgroundBrush().color((GColor*)c);
	    }
	}
}





void modify_brushsheet(void* v, Go* go)
{
    if (go != named_goroot && go != edit_goroot &&
	go->getMetaclass()->isSubclass(GoGeometric::metaclass())) {
	ulong mask = GCOLOR | GFILL_RULE;
	((GoGeometric*)go)->brush().modify(mask, (GBrushValues*)v);
    } else
	if (go != named_goroot && go != edit_goroot &&
	    go->getMetaclass()->isSubclass(GoTextual::metaclass())) {
	    ulong mask = GCOLOR | GFILL_RULE;
	    if (go->getMetaclass()->isSubclass(GoText::metaclass()) &&
		SELECTED_PART(go)) {
		((GoText*)go)->textBackgroundBrush(mask, (GBrushValues*)v,
						((GoText*)go)->firstSelected(),
						((GoText*)go)->lastSelected());
		((GoText*)go)->background(GTRUE,
					  ((GoText*)go)->firstSelected(),
					  ((GoText*)go)->lastSelected());
	    } else {
		((GoTextual*)go)->backgroundBrush().modify(mask,
							   (GBrushValues*)v);
	    }
	}
}

void ModifyGoEditBrushSheet()
{
    GBrushValues values;
    values.fill.rule = brush.FillRule;
    values.color = GCreateShColor(&brush.Rgb_Color);
    int nb;
    if (! ColorToName.get(values.color, nb)) {
	values.color->destroyPostcond(ShColorDestroyed, NULL);
	ColorToName.set(values.color,
			(strlen(brush.Color) + 1) * sizeof(char),
			brush.Color);
    }
    edit_goroot->forEveryGo(modify_brushsheet, &values);
    values.color->destroy();
}

void ModifyGoNamedBrushSheet()
{
    GBrushValues values;
    values.fill.rule = named_brush.FillRule;
    values.color = GCreateShColor(&named_brush.Rgb_Color);
    int nb;
    if (! ColorToName.get(values.color, nb)) {
	values.color->destroyPostcond(ShColorDestroyed, NULL);
	ColorToName.set(values.color,
			(strlen(named_brush.Color) + 1) * sizeof(char),
			named_brush.Color);
    }
    named_goroot->forEveryGo(modify_brushsheet, &values);
    values.color->destroy();
}



/***
 *  TextBrush Modifications
 ***/




void modify_textsheet(void* v, Go* go)
{
    if (go != named_goroot && go != edit_goroot &&
	go->getMetaclass()->isSubclass(GoTextual::metaclass())) {
	ulong mask = GCOLOR | GFONT;
	if (go->getMetaclass()->isSubclass(GoText::metaclass()) &&
	    SELECTED_PART(go))
	    ((GoText*)go)->textStringTextBrush(mask, (GTextBrushValues*)v,
					       ((GoText*)go)->firstSelected(),
					       ((GoText*)go)->lastSelected());
	else
	    ((GoTextual*)go)->stringTextBrush().modify(mask,
						       (GTextBrushValues*)v);
	((GoTextual*)go)->lineHeight(height, ((GoText*)go)->firstSelected(),
				     ((GoText*)go)->lastSelected());
    }
}

void ModifyGoEditTextSheet()
{
    GTextBrushValues values;
    GFont* f;
    f = edit_interf->xFontNameToFont(text.Font);
    values.font = f->shFont()->dup();
    values.color = GCreateShColor(&text.Rgb_Color);
    int nb;
    if (! ColorToName.get(values.color, nb)) {
	values.color->destroyPostcond(ShColorDestroyed, NULL);
	ColorToName.set(values.color,
			(strlen(text.Color) + 1) * sizeof(char),
			text.Color);
    }
    height = edit_interf->fontHeightInPixels(text.Font) * edit_interf->pixelHeight();
    edit_goroot->forEveryGo(modify_textsheet, &values);
    
    values.color->destroy();
    delete f;
    values.font->destroy();
}


void ModifyGoNamedTextSheet()
{
    GTextBrushValues values;
    GFont* f;
    f = named_interf->xFontNameToFont(named_text.Font);
    values.font = f->shFont()->dup();
    values.color = GCreateShColor(&named_text.Rgb_Color);
    int nb;
    if (! ColorToName.get(values.color, nb)) {
	values.color->destroyPostcond(ShColorDestroyed, NULL);
	ColorToName.set(values.color,
			(strlen(named_text.Color) + 1) * sizeof(char),
			named_text.Color);
    }
    height = named_interf->fontHeightInPixels(text.Font) * named_interf->pixelHeight();
    named_goroot->forEveryGo(modify_textsheet, &values);
    
    values.color->destroy();
    delete f;
    values.font->destroy();
}




void modify_font(void* f, Go* go)
{
    if (go != named_goroot && go != edit_goroot && go != goroot &&
	go->getMetaclass()->isSubclass(GoTextual::metaclass())) {
	if (go->getMetaclass()->isSubclass(GoText::metaclass()) &&
	    SELECTED_PART(go)) {
	    GTextBrushValues values;
	    ulong mask = GFONT;
	    values.font = ((GFont*)f)->shFont()->dup();
	    ((GoText*)go)->textStringTextBrush(mask, &values,
					       ((GoText*)go)->firstSelected(),
					       ((GoText*)go)->lastSelected());
	    values.font->destroy();
	} else {
	    if (((GoTextual*)go)->stringTextBrush().isNamed()) {
		GShTextBrush* brush =
		    ((GoTextual*)go)->stringTextBrush().shTextBrush()->dup();
		GRename(brush, NULL);
		GSetFont(brush, ((GFont*)f)->shFont());
		((GoTextual*)go)->stringTextBrush().modify(brush);
		brush->destroy();
	    } else
		((GoTextual*)go)->stringTextBrush().font((GFont*)f);
	}
	((GoTextual*)go)->lineHeight(height, ((GoTextual*)go)->firstSelected(),
				     ((GoTextual*)go)->lastSelected());
    }
}

void ModifyGoFont(char* name)
{
    GFont* f = NULL;
    switch (WhichAttributes) {
      case NAMED:
	f = named_interf->xFontNameToFont(name);
	break;
      case EDIT:
	f = edit_interf->xFontNameToFont(name);
	break;
      case MAIN:
	f = interf->xFontNameToFont(name);
	break;
    }
    if (f == NULL)
	ErrorFont();
    else {
	switch (WhichAttributes) {
	  case NAMED:
	    height = named_interf->fontHeightInPixels(name) *
		named_interf->pixelHeight();
	    named_goroot->forEveryGo(modify_font, f);
	    break;
	  case EDIT:
	    height = edit_interf->fontHeightInPixels(name) *
		edit_interf->pixelHeight();
	    edit_goroot->forEveryGo(modify_font, f);
	    break;
	  case MAIN:
	    height = interf->fontHeightInPixels(name) *
		interf->pixelHeight();
	    goroot->forEverySelectedGo(modify_font, f);
	    break;
	}
	delete f;
    }
}



static void modify_font_without_height(void* f, Go* go)
{
    if (go != named_goroot && go != edit_goroot && go != goroot &&
	go->getMetaclass()->isSubclass(GoTextual::metaclass())) {
	if (go->getMetaclass()->isSubclass(GoText::metaclass()) &&
	    SELECTED_PART(go)) {
	    GTextBrushValues values;
	    ulong mask = GFONT;
	    values.font = ((GFont*)f)->shFont()->dup();
	    ((GoText*)go)->textStringTextBrush(mask, &values,
					       ((GoText*)go)->firstSelected(),
					       ((GoText*)go)->lastSelected());
	    values.font->destroy();
	} else {
	    if (((GoTextual*)go)->stringTextBrush().isNamed()) {
		GShTextBrush* brush =((GoTextual*)go)->stringTextBrush().shTextBrush()->dup();
		GRename(brush, NULL);
		GSetFont(brush, ((GFont*)f)->shFont());
		((GoTextual*)go)->stringTextBrush().modify(brush);
	    brush->destroy();
	    } else
		((GoTextual*)go)->stringTextBrush().font((GFont*)f);
	}
    }
}

void ModifyGoFontWithoutHeight(char* name)
{
    GFont* f = NULL;
    f = interf->xFontNameToFont(name);
    if (f == NULL)
	ErrorFont();
    else
	goroot->forEverySelectedGo(modify_font_without_height, f);
    delete f;
}





void modify_text_color(void* c, Go* go)
{
    if (go != named_goroot && go != edit_goroot && go != goroot &&
	go->getMetaclass()->isSubclass(GoTextual::metaclass())) {
	if (go->getMetaclass()->isSubclass(GoText::metaclass()) &&
	    SELECTED_PART(go)) {
	    GTextBrushValues values;
	    ulong mask = GCOLOR;
	    values.color = ((GColor*)c)->shColor()->dup();
	    ((GoText*)go)->textStringTextBrush(mask, &values,
					       ((GoText*)go)->firstSelected(),
					       ((GoText*)go)->lastSelected());
	    values.color->destroy();
	} else {
	    if (((GoTextual*)go)->stringTextBrush().isNamed()) {
		GShTextBrush* brush =((GoTextual*)go)->stringTextBrush().shTextBrush()->dup();
		GRename(brush, NULL);
		GSetColor(brush, ((GColor*)c)->shColor());
		((GoTextual*)go)->stringTextBrush().modify(brush);
		brush->destroy();
	    } else
		((GoTextual*)go)->stringTextBrush().color((GColor*)c);
	}
    }
}


void ModifyGoColor(GRgb* c, char* name)
{
    GColor color(c);
    int nb;
    if (! ColorToName.get(color.shColor(), nb)) {
	color.shColor()->destroyPostcond(ShColorDestroyed, NULL);
	ColorToName.set(color.shColor(),
			(strlen(name) + 1) * sizeof(char), name);
    }
    switch(WhichColor) {
      case PEN_COLOR:
	if (WhichAttributes == NAMED) {
	    named_pen.Rgb_Color = *c;
	    named_goroot->forEveryGo(modify_pen_color, &color);
	} else {
	    pen.Rgb_Color = *c;
	    if (WhichAttributes == EDIT) {
		edit_goroot->forEveryGo(modify_pen_color, &color);
	    } else
		goroot->forEverySelectedGo(modify_pen_color, &color);
	}
	break;
      case PEN_ALT_COLOR:
	if (WhichAttributes == NAMED) {
	    named_pen.Rgb_AltColor = *c;
	    named_goroot->forEveryGo(modify_pen_alt_color, &color);
	} else {
	    pen.Rgb_AltColor = *c;
	    if (WhichAttributes == EDIT) {
		edit_goroot->forEveryGo(modify_pen_alt_color, &color);
	    } else
		goroot->forEverySelectedGo(modify_pen_alt_color, &color);
	}
	break;
      case BRUSH_COLOR:
	if (WhichAttributes == NAMED) {
	    named_brush.Rgb_Color = *c;
	    named_goroot->forEveryGo(modify_brush_color, &color);
	} else {
	    brush.Rgb_Color = *c;
	    if (WhichAttributes == EDIT) {
		edit_goroot->forEveryGo(modify_brush_color, &color);
	    } else
		goroot->forEverySelectedGo(modify_brush_color, &color);
	}
	break;
      case TEXT_COLOR:
	if (WhichAttributes == NAMED) {
	    named_text.Rgb_Color = *c;
	    named_goroot->forEveryGo(modify_text_color, &color);
	} else {
	    text.Rgb_Color = *c;
	    if (WhichAttributes == EDIT) {
		edit_goroot->forEveryGo(modify_text_color, &color);
	    } else
		goroot->forEverySelectedGo(modify_text_color, &color);
	}
	break;
    }
}





                              /*************
			       * NAMED PEN *
			       *************/



void CreateGoNamedPen()
{
    GPenValues values;
    ulong mask = GCOLOR | GALT_COLOR | GLINE_WIDTH | GLINE_STYLE |
	GCAP_STYLE | GJOIN_STYLE | GDASH;
    values.line_width = atof(named_pen.LineWidth);
    values.line.style = named_pen.LineStyle;
    values.line.cap = named_pen.CapStyle;
    values.line.join = named_pen.JoinStyle;
    values.dash = goDash->shDash()->dup();
    values.color = GCreateShColor(&named_pen.Rgb_Color);
    int nb;
    if (! ColorToName.get(values.color, nb)) {
	values.color->destroyPostcond(ShColorDestroyed, NULL);
	ColorToName.set(values.color,
			(strlen(named_pen.Color) + 1) * sizeof(char),
			named_pen.Color);
    }
    values.alt_color = GCreateShColor(&named_pen.Rgb_AltColor);
    if (! ColorToName.get(values.alt_color, nb)) {
	values.alt_color->destroyPostcond(ShColorDestroyed, NULL);
	ColorToName.set(values.alt_color,
			(strlen(named_pen.AltColor) + 1) * sizeof(char),
			named_pen.AltColor);
    }
    
    GShPen* p = GCreateShPen(mask, &values, named_pen.PenText);
    NamedPen.set(named_pen.PenText, p);
    
    values.color->destroy();
    values.alt_color->destroy();
    values.dash->destroy();
}


void ModifyGoNamedPen()
{
    GShPen* p = (GShPen*)NamedPen.get(named_pen.PenText);
    
    GPenValues values;
    ulong mask = GCOLOR | GALT_COLOR | GLINE_WIDTH | GLINE_STYLE |
	GCAP_STYLE | GJOIN_STYLE | GDASH;
    values.line_width = atof(named_pen.LineWidth);
    values.line.style = named_pen.LineStyle;
    values.line.cap = named_pen.CapStyle;
    values.line.join = named_pen.JoinStyle;
    values.dash = goDash->shDash()->dup();
    values.color = GCreateShColor(&named_pen.Rgb_Color);
    int nb;
    if (! ColorToName.get(values.color, nb)) {
	values.color->destroyPostcond(ShColorDestroyed, NULL);
	ColorToName.set(values.color,
			(strlen(named_pen.Color) + 1) * sizeof(char),
			named_pen.Color);
    }
    values.alt_color = GCreateShColor(&named_pen.Rgb_AltColor);
    if (! ColorToName.get(values.alt_color, nb)) {
	values.alt_color->destroyPostcond(ShColorDestroyed, NULL);
	ColorToName.set(values.alt_color,
			(strlen(named_pen.AltColor) + 1) * sizeof(char),
			named_pen.AltColor);
    }
    
    GModify(p, mask, &values);
    
    values.color->destroy();
    values.alt_color->destroy();
    values.dash->destroy();
}


void FromShPenToEditPen(char* name)
{
    reinit_pen = pen;

    GFREE(pen.PenText);
    pen.PenText = (char*)GMALLOC(strlen(name) + 1);
    strcpy(pen.PenText, name);
    const GShPen* p = (const GShPen*)NamedPen.get(pen.PenText);

    char buf[20];
    sprintf(buf, "%.2f", p->lineWidth());
    GFREE(pen.LineWidth);
    pen.LineWidth = (char*)GMALLOC(strlen(buf) + 1);
    strcpy(pen.LineWidth, buf);

    pen.LineStyle = p->lineStyle();
    pen.CapStyle = p->capStyle();
    pen.JoinStyle = p->joinStyle();
    const GShColor* color = p->color();
    const GShColor* altcolor = p->altColor();
    if (color->rgb())
	pen.Rgb_Color = *(color->rgb());
    else {                             // TO BE MODIFIED (it's can be pattern);
	int gray = color->gray();
	pen.Rgb_Color.red = gray;
	pen.Rgb_Color.green = gray;
	pen.Rgb_Color.blue = gray;
    }
    if (altcolor->rgb())
	pen.Rgb_AltColor = *(altcolor->rgb());
    else {                             // TO BE MODIFIED (it's can be pattern);
	int gray = altcolor->gray();
	pen.Rgb_AltColor.red = gray;
	pen.Rgb_AltColor.green = gray;
	pen.Rgb_AltColor.blue = gray;
    }
    int nb;
    const char* color_name = (const char*)ColorToName.get(color, nb);
    const char* altcolor_name = (const char*)ColorToName.get(altcolor, nb);

    if (color_name) {
	GFREE(pen.Color);
	pen.Color = (char*)GMALLOC(strlen(color_name) + 1);
	strcpy(pen.Color, color_name);
    } else {
	GFREE(pen.Color);
	char* str = RgbToHexaString(color->rgb());
	pen.Color = (char*)GMALLOC(strlen(str) + 1);
	strcpy(pen.Color, str);
    }
    if (altcolor_name) {
	GFREE(pen.AltColor);
	pen.AltColor = (char*)GMALLOC(strlen(altcolor_name) + 1);
	strcpy(pen.AltColor, altcolor_name);
    } else {
	GFREE(pen.AltColor);
	char* str = RgbToHexaString(altcolor->rgb());
	pen.AltColor = (char*)GMALLOC(strlen(str) + 1);
	strcpy(pen.AltColor, str);
    }
}

void FromShPenToNamedPen(char* name)
{
    GFREE(named_pen.PenText);
    named_pen.PenText = (char*)GMALLOC(strlen(name) + 1);
    strcpy(named_pen.PenText, name);
    const GShPen* p = (const GShPen*)NamedPen.get(named_pen.PenText);

    char buf[20];
    sprintf(buf, "%.2f", p->lineWidth());
    GFREE(named_pen.LineWidth);
    named_pen.LineWidth = (char*)GMALLOC(strlen(buf) + 1);
    strcpy(named_pen.LineWidth, buf);

    named_pen.LineStyle = p->lineStyle();
    named_pen.CapStyle = p->capStyle();
    named_pen.JoinStyle = p->joinStyle();
    const GShColor* color = p->color();
    const GShColor* altcolor = p->altColor();
    if (color->rgb())
	named_pen.Rgb_Color = *(color->rgb());
    else {                             // TO BE MODIFIED (it's can be pattern);
	int gray = color->gray();
	named_pen.Rgb_Color.red = gray;
	named_pen.Rgb_Color.green = gray;
	named_pen.Rgb_Color.blue = gray;
    }
    if (altcolor->rgb())
	named_pen.Rgb_AltColor = *(altcolor->rgb());
    else {                             // TO BE MODIFIED (it's can be pattern);
	int gray = altcolor->gray();
	named_pen.Rgb_AltColor.red = gray;
	named_pen.Rgb_AltColor.green = gray;
	named_pen.Rgb_AltColor.blue = gray;
    }
    int nb;
    const char* color_name = (const char*)ColorToName.get(color, nb);
    const char* altcolor_name = (const char*)ColorToName.get(altcolor, nb);

    if (color_name) {
	GFREE(named_pen.Color);
	named_pen.Color = (char*)GMALLOC(strlen(color_name) + 1);
	strcpy(named_pen.Color, color_name);
    } else {
	GFREE(named_pen.Color);
	char* str = RgbToHexaString(color->rgb());
	named_pen.Color = (char*)GMALLOC(strlen(str) + 1);
	strcpy(named_pen.Color, str);
    }
    if (altcolor_name) {
	GFREE(named_pen.AltColor);
	named_pen.AltColor = (char*)GMALLOC(strlen(altcolor_name) + 1);
	strcpy(named_pen.AltColor, altcolor_name);
    } else {
	GFREE(named_pen.AltColor);
	char* str = RgbToHexaString(altcolor->rgb());
	named_pen.AltColor = (char*)GMALLOC(strlen(str) + 1);
	strcpy(named_pen.AltColor, str);
    }
}


/***
 *  Pen Modifications
 ***/


void modify_pen(void* p, Go* go)
{
    if (go != goroot &&
	go->getMetaclass()->isSubclass(GoGeometric::metaclass()))
	((GoGeometric*)go)->pen().modify((GShPen*)p);
}

void ModifyGoPen(char* name)
{
    GShPen* p = (GShPen*)NamedPen.get(name);
    goroot->forEverySelectedGo(modify_pen, p);
}

    

                              /***************
			       * NAMED BRUSH *
			       ***************/



void CreateGoNamedBrush()
{
    GBrushValues values;
    ulong mask = GCOLOR | GFILL_RULE;
    values.fill.rule = named_brush.FillRule;
    values.color = GCreateShColor(&named_brush.Rgb_Color);
    int nb;
    if (! ColorToName.get(values.color, nb)) {
	values.color->destroyPostcond(ShColorDestroyed, NULL);
	ColorToName.set(values.color,
			(strlen(named_brush.Color) + 1) * sizeof(char),
			named_brush.Color);
    }
    
    GShBrush* p = GCreateShBrush(mask, &values, named_brush.BrushText);
    NamedBrush.set(named_brush.BrushText, p);
    
    values.color->destroy();
}


void ModifyGoNamedBrush()
{
    GShBrush* p = (GShBrush*)NamedBrush.get(named_brush.BrushText);
    
    GBrushValues values;
    ulong mask = GCOLOR | GFILL_RULE;
    values.fill.rule = named_brush.FillRule;
    values.color = GCreateShColor(&named_brush.Rgb_Color);
    int nb;
    if (! ColorToName.get(values.color, nb)) {
	values.color->destroyPostcond(ShColorDestroyed, NULL);
	ColorToName.set(values.color,
			(strlen(named_brush.Color) + 1) * sizeof(char),
			named_brush.Color);
    }
    
    GModify(p, mask, &values);
    
    values.color->destroy();
}


void FromShBrushToEditBrush(char* name)
{
    reinit_brush = brush;

    GFREE(brush.BrushText);
    brush.BrushText = (char*)GMALLOC(strlen(name) + 1);
    strcpy(brush.BrushText, name);
    const GShBrush* p = (const GShBrush*)NamedBrush.get(brush.BrushText);
    brush.FillRule = p->fillRule();
    const GShColor* color = p->color();
    if (color->rgb())
	brush.Rgb_Color = *(color->rgb());
    else {                             // TO BE MODIFIED (it's can be pattern);
	int gray = color->gray();
	brush.Rgb_Color.red = gray;
	brush.Rgb_Color.green = gray;
	brush.Rgb_Color.blue = gray;
    }
    int nb;
    const char* color_name = (const char*)ColorToName.get(color, nb);

    if (color_name) {
	GFREE(brush.Color);
	brush.Color = (char*)GMALLOC(strlen(color_name) + 1);
	strcpy(brush.Color, color_name);
    } else {
	GFREE(brush.Color);
	char* str = RgbToHexaString(color->rgb());
	brush.Color = (char*)GMALLOC(strlen(str) + 1);
	strcpy(brush.Color, str);
    }
}

    
void FromShBrushToNamedBrush(char* name)
{
    GFREE(named_brush.BrushText);
    named_brush.BrushText = (char*)GMALLOC(strlen(name) + 1);
    strcpy(named_brush.BrushText, name);
    const GShBrush* p = (const GShBrush*)NamedBrush.get(named_brush.BrushText);
    named_brush.FillRule = p->fillRule();
    const GShColor* color = p->color();
    if (color->rgb())
	named_brush.Rgb_Color = *(color->rgb());
    else {                             // TO BE MODIFIED (it's can be pattern);
	int gray = color->gray();
	named_brush.Rgb_Color.red = gray;
	named_brush.Rgb_Color.green = gray;
	named_brush.Rgb_Color.blue = gray;
    }
    int nb;
    const char* color_name = (const char*)ColorToName.get(color, nb);

    if (color_name) {
	GFREE(named_brush.Color);
	named_brush.Color = (char*)GMALLOC(strlen(color_name) + 1);
	strcpy(named_brush.Color, color_name);
    } else {
	GFREE(named_brush.Color);
	char* str = RgbToHexaString(color->rgb());
	named_brush.Color = (char*)GMALLOC(strlen(str) + 1);
	strcpy(named_brush.Color, str);
    }
}

    



/***
 *  Brush Modifications
 ***/




void modify_brush(void* b, Go* go)
{
    if (go != goroot &&
	go->getMetaclass()->isSubclass(GoGeometric::metaclass())) {
	((GoGeometric*)go)->brush().modify((GShBrush*)b);
    } else
	if (go != goroot &&
	    go->getMetaclass()->isSubclass(GoTextual::metaclass())) {
	    if (go->getMetaclass()->isSubclass(GoText::metaclass()) &&
		SELECTED_PART(go)) {
		((GoText*)go)->textBackgroundBrush((GShBrush*)b,
						((GoText*)go)->firstSelected(),
						((GoText*)go)->lastSelected());
		((GoText*)go)->background(GTRUE,
					  ((GoText*)go)->firstSelected(),
					  ((GoText*)go)->lastSelected());
	    } else {
		((GoTextual*)go)->backgroundBrush().modify((GShBrush*)b);
	    }
	}
}


void ModifyGoBrush(char* name)
{
    GShBrush* b = (GShBrush*)NamedBrush.get(name);
    goroot->forEverySelectedGo(modify_brush, b);
}


                              /*******************
			       * NAMED TEXTBRUSH *
			       *******************/



void CreateGoNamedText()
{
    TextStruct str;
    GTextBrushValues values;
    ulong mask = GCOLOR | GFONT;
    values.color = GCreateShColor(&named_text.Rgb_Color);
    int nb;
    if (! ColorToName.get(values.color, nb)) {
	values.color->destroyPostcond(ShColorDestroyed, NULL);
	ColorToName.set(values.color,
		      (strlen(named_text.Color) + 1) * sizeof(char),
			named_text.Color);
    }
    GFont* f = interf->xFontNameToFont(named_text.Font);
    values.font = f->shFont()->dup();
    str.height = interf->fontHeightInPixels(named_text.Font) * interf->pixelHeight();
    
    str.sh = GCreateShTextBrush(mask, &values, named_text.TextBrushText);
    NamedText.set(named_text.TextBrushText, sizeof(TextStruct), &str);
    
    values.color->destroy();
    values.font->destroy();
    delete f;
}

static void modify_line_height(void* h, Go* go)
{
    // PROBLEM;
    if (go->getMetaclass()->isSubclass(GoTextual::metaclass())) {
	const char* name = ((GoTextual*)go)->stringTextBrush().name();
	if (name)
	    if (strcmp(name, named_text.TextBrushText) == 0)
		((GoTextual*)go)->lineHeight(*((GNumber*)h));
    }
}

void ModifyGoNamedText()
{
    int nb;
    TextStruct* str = (TextStruct*)
	NamedText.get(named_text.TextBrushText, nb);
    
    GTextBrushValues values;
    ulong mask = GCOLOR | GFONT;
    values.color = GCreateShColor(&named_text.Rgb_Color);
    if (! ColorToName.get(values.color, nb)) {
	values.color->destroyPostcond(ShColorDestroyed, NULL);
	ColorToName.set(values.color,
			(strlen(named_text.Color) + 1) * sizeof(char),
			named_text.Color);
    }
    GFont* f = interf->xFontNameToFont(named_text.Font);
    values.font = f->shFont()->dup();
    ((TextStruct*)str)->height = interf->fontHeightInPixels(named_text.Font) * interf->pixelHeight();
    
    GModify(str->sh, mask, &values);
    goroot->forEveryGo(modify_line_height, &str->height);
    
    NamedText.set(named_text.TextBrushText, sizeof(TextStruct), str);
    
    values.color->destroy();
    values.font->destroy();
    delete f;
}


void FromShTextToEditText(char* name)
{
    reinit_text = text;

    int nb;
    GFREE(text.TextBrushText);
    text.TextBrushText = (char*)GMALLOC(strlen(name) + 1);
    strcpy(text.TextBrushText, name);
    const TextStruct* str = (const TextStruct*)
	NamedText.get(text.TextBrushText, nb);
    const GShColor* color = str->sh->color();
    if (color->rgb())
	text.Rgb_Color = *(color->rgb());
    else {                             // TO BE MODIFIED (it's can be pattern);
	int gray = color->gray();
	text.Rgb_Color.red = gray;
	text.Rgb_Color.green = gray;
	text.Rgb_Color.blue = gray;
    }
    const char* color_name = (const char*)ColorToName.get(color, nb);

    if (color_name) {
	GFREE(text.Color);
	text.Color = (char*)GMALLOC(strlen(color_name) + 1);
	strcpy(text.Color, color_name);
    } else {
	GFREE(text.Color);
	char* str = RgbToHexaString(color->rgb());
	text.Color = (char*)GMALLOC(strlen(str) + 1);
	strcpy(text.Color, str);
    }

    const GShFont* f = str->sh->font();
    const char* f_name = interf->fontName(GUnitCoordSys(), str->height, f);
    GFREE(text.Font);
    text.Font = (char*)GMALLOC(strlen(f_name) + 1);
    strcpy(text.Font, f_name);
}


void FromShTextToNamedText(char* name)
{
    int nb;
    GFREE(named_text.TextBrushText);
    named_text.TextBrushText = (char*)GMALLOC(strlen(name) + 1);
    strcpy(named_text.TextBrushText, name);
    const TextStruct* str = (const TextStruct*)
	NamedText.get(named_text.TextBrushText, nb);
    const GShColor* color = str->sh->color();
    if (color->rgb())
	named_text.Rgb_Color = *(color->rgb());
    else {                             // TO BE MODIFIED (it's can be pattern);
	int gray = color->gray();
	named_text.Rgb_Color.red = gray;
	named_text.Rgb_Color.green = gray;
	named_text.Rgb_Color.blue = gray;
    }
    const char* color_name = (const char*)ColorToName.get(color, nb);

    if (color_name) {
	GFREE(named_text.Color);
	named_text.Color = (char*)GMALLOC(strlen(color_name) + 1);
	strcpy(named_text.Color, color_name);
    } else {
	GFREE(named_text.Color);
	char* str = RgbToHexaString(color->rgb());
	named_text.Color = (char*)GMALLOC(strlen(str) + 1);
	strcpy(named_text.Color, str);
    }

    const GShFont* f = str->sh->font();
    const char* f_name = interf->fontName(GUnitCoordSys(), str->height, f);
    GFREE(named_text.Font);
    named_text.Font = (char*)GMALLOC(strlen(f_name) + 1);
    strcpy(named_text.Font, f_name);
}



/***
 *  TextBrush Modifications
 ***/


void modify_text(void* t, Go* go)
{
    if (go != goroot &&
	go->getMetaclass()->isSubclass(GoTextual::metaclass())) {
	if (go->getMetaclass()->isSubclass(GoText::metaclass()) &&
	    SELECTED_PART(go))
	    ((GoText*)go)->textStringTextBrush(((TextStruct*)t)->sh,
					       ((GoText*)go)->firstSelected(),
					       ((GoText*)go)->lastSelected());
	else
	    ((GoTextual*)go)->stringTextBrush().modify(((TextStruct*)t)->sh);
	((GoTextual*)go)->lineHeight(((TextStruct*)t)->height,
				     ((GoTextual*)go)->firstSelected(),
				     ((GoTextual*)go)->lastSelected());
    }
}

void ModifyGoText(char* name)
{
    int nb;
    TextStruct* t = (TextStruct*)NamedText.get(name, nb);
    goroot->forEverySelectedGo(modify_text, t);
}


                             /***************
			      * CHOOSE A GO *
			      ***************/



void FromPenValuesToEditPen(const GPenValues* val, const char* name)
{
    GFREE(pen.PenText);
    pen.PenText = (char*)GMALLOC(strlen(name) + 1);
    strcpy(pen.PenText, name);

    char buf[20];
    sprintf(buf, "%.2f", val->line_width);
    GFREE(pen.LineWidth);
    pen.LineWidth = (char*)GMALLOC(strlen(buf) + 1);
    strcpy(pen.LineWidth, buf);
    
    pen.LineStyle = val->line.style;
    pen.CapStyle = val->line.cap;
    pen.JoinStyle = val->line.join;
    if (val->color->rgb())
	pen.Rgb_Color = *(val->color->rgb());
    else {                             // TO BE MODIFIED (it's can be pattern);
	int gray = val->color->gray();
	pen.Rgb_Color.red = gray;
	pen.Rgb_Color.green = gray;
	pen.Rgb_Color.blue = gray;
    }
    if (val->alt_color->rgb())
	pen.Rgb_AltColor = *(val->alt_color->rgb());
    else {                             // TO BE MODIFIED (it's can be pattern);
	int gray = val->alt_color->gray();
	pen.Rgb_AltColor.red = gray;
	pen.Rgb_AltColor.green = gray;
	pen.Rgb_AltColor.blue = gray;
    }
    int nb;
    const char* color_name = (const char*)ColorToName.get(val->color, nb);
    const char* altcolor_name = (const char*)ColorToName.get(val->alt_color,
							     nb);

    if (color_name) {
	GFREE(pen.Color);
	pen.Color = (char*)GMALLOC(strlen(color_name) + 1);
	strcpy(pen.Color, color_name);
    } else {
	GFREE(pen.Color);
	char* str = RgbToHexaString(val->color->rgb());
	pen.Color = (char*)GMALLOC(strlen(str) + 1);
	strcpy(pen.Color, str);
    }
    if (altcolor_name) {
	GFREE(pen.AltColor);
	pen.AltColor = (char*)GMALLOC(strlen(altcolor_name) + 1);
	strcpy(pen.AltColor, altcolor_name);
    } else {
	GFREE(pen.AltColor);
	char* str = RgbToHexaString(val->alt_color->rgb());
	pen.AltColor = (char*)GMALLOC(strlen(str) + 1);
	strcpy(pen.AltColor, str);
    }
}

void FromPenValuesToNamedPen(const GPenValues* val)
{
    GFREE(named_pen.PenText);
    named_pen.PenText = (char*)GMALLOC(strlen(UNNAMED) + 1);
    strcpy(named_pen.PenText, UNNAMED);

    char buf[20];
    sprintf(buf, "%.2f", val->line_width);
    GFREE(named_pen.LineWidth);
    named_pen.LineWidth = (char*)GMALLOC(strlen(buf) + 1);
    strcpy(named_pen.LineWidth, buf);
    
    named_pen.LineStyle = val->line.style;
    named_pen.CapStyle = val->line.cap;
    named_pen.JoinStyle = val->line.join;
    if (val->color->rgb())
	named_pen.Rgb_Color = *(val->color->rgb());
    else {                             // TO BE MODIFIED (it's can be pattern);
	int gray = val->color->gray();
	named_pen.Rgb_Color.red = gray;
	named_pen.Rgb_Color.green = gray;
	named_pen.Rgb_Color.blue = gray;
    }
    if (val->alt_color->rgb())
	named_pen.Rgb_AltColor = *(val->alt_color->rgb());
    else {                             // TO BE MODIFIED (it's can be pattern);
	int gray = val->alt_color->gray();
	named_pen.Rgb_AltColor.red = gray;
	named_pen.Rgb_AltColor.green = gray;
	named_pen.Rgb_AltColor.blue = gray;
    }
    int nb;
    const char* color_name = (const char*)ColorToName.get(val->color, nb);
    const char* altcolor_name = (const char*)ColorToName.get(val->alt_color,
							     nb);

    if (color_name) {
	GFREE(named_pen.Color);
	named_pen.Color = (char*)GMALLOC(strlen(color_name) + 1);
	strcpy(named_pen.Color, color_name);
    } else {
	GFREE(named_pen.Color);
	char* str = RgbToHexaString(val->color->rgb());
	named_pen.Color = (char*)GMALLOC(strlen(str) + 1);
	strcpy(named_pen.Color, str);
    }
    if (altcolor_name) {
	GFREE(named_pen.AltColor);
	named_pen.AltColor = (char*)GMALLOC(strlen(altcolor_name) + 1);
	strcpy(named_pen.AltColor, altcolor_name);
    } else {
	GFREE(named_pen.AltColor);
	char* str = RgbToHexaString(val->alt_color->rgb());
	named_pen.AltColor = (char*)GMALLOC(strlen(str) + 1);
	strcpy(named_pen.AltColor, str);
    }
}


void FromBrushValuesToEditBrush(const GBrushValues* val, const char* name)
{
    GFREE(brush.BrushText);
    brush.BrushText = (char*)GMALLOC(strlen(name) + 1);
    strcpy(brush.BrushText, name);

    brush.FillRule = val->fill.rule;
    if (val->color->rgb())
	brush.Rgb_Color = *(val->color->rgb());
    else {                             // TO BE MODIFIED (it's can be pattern);
	int gray = val->color->gray();
	brush.Rgb_Color.red = gray;
	brush.Rgb_Color.green = gray;
	brush.Rgb_Color.blue = gray;
    }
    int nb;
    const char* color_name = (const char*)ColorToName.get(val->color, nb);

    if (color_name) {
	GFREE(brush.Color);
	brush.Color = (char*)GMALLOC(strlen(color_name) + 1);
	strcpy(brush.Color, color_name);
    } else {
	GFREE(brush.Color);
	char* str = RgbToHexaString(val->color->rgb());
	brush.Color = (char*)GMALLOC(strlen(str) + 1);
	strcpy(brush.Color, str);
    }
}


void FromBrushValuesToNamedBrush(const GBrushValues* val)
{
    GFREE(named_brush.BrushText);
    named_brush.BrushText = (char*)GMALLOC(strlen(UNNAMED) + 1);
    strcpy(named_brush.BrushText, UNNAMED);

    named_brush.FillRule = val->fill.rule;
    if (val->color->rgb())
	named_brush.Rgb_Color = *(val->color->rgb());
    else {                             // TO BE MODIFIED (it's can be pattern);
	int gray = val->color->gray();
	named_brush.Rgb_Color.red = gray;
	named_brush.Rgb_Color.green = gray;
	named_brush.Rgb_Color.blue = gray;
    }
    int nb;
    const char* color_name = (const char*)ColorToName.get(val->color, nb);

    if (color_name) {
	GFREE(named_brush.Color);
	named_brush.Color = (char*)GMALLOC(strlen(color_name) + 1);
	strcpy(named_brush.Color, color_name);
    } else {
	GFREE(named_brush.Color);
	char* str = RgbToHexaString(val->color->rgb());
	named_brush.Color = (char*)GMALLOC(strlen(str) + 1);
	strcpy(named_brush.Color, str);
    }
}


void FromTextBrushValuesToEditText(const GTextBrushValues* val, GNumber height,
				   GCoordSys& cs, const char* name)
{
    GFREE(text.TextBrushText);
    text.TextBrushText = (char*)GMALLOC(strlen(name) + 1);
    strcpy(text.TextBrushText, name);

    const char* f_name = interf->fontName(cs, height, val->font);
    GFREE(text.Font);
    text.Font = (char*)GMALLOC(strlen(f_name) + 1);
    strcpy(text.Font, f_name);
    if (val->color->rgb())
	text.Rgb_Color = *(val->color->rgb());
    else {                             // TO BE MODIFIED (it's can be pattern);
	int gray = val->color->gray();
	text.Rgb_Color.red = gray;
	text.Rgb_Color.green = gray;
	text.Rgb_Color.blue = gray;
    }
    int nb;
    const char* color_name = (const char*)ColorToName.get(val->color, nb);

    if (color_name) {
	GFREE(text.Color);
	text.Color = (char*)GMALLOC(strlen(color_name) + 1);
	strcpy(text.Color, color_name);
    } else {
	GFREE(text.Color);
	char* str = RgbToHexaString(val->color->rgb());
	text.Color = (char*)GMALLOC(strlen(str) + 1);
	strcpy(text.Color, str);
    }
}


void FromTextBrushValuesToNamedText(const GTextBrushValues* val, GNumber height,
				    GCoordSys& cs)
{
    GFREE(named_text.TextBrushText);
    named_text.TextBrushText = (char*)GMALLOC(strlen(UNNAMED) + 1);
    strcpy(named_text.TextBrushText, UNNAMED);

    const char* f_name = interf->fontName(cs, height, val->font);
    GFREE(named_text.Font);
    named_text.Font = (char*)GMALLOC(strlen(f_name) + 1);
    strcpy(named_text.Font, f_name);
    if (val->color->rgb())
	named_text.Rgb_Color = *(val->color->rgb());
    else {                             // TO BE MODIFIED (it's can be pattern);
	int gray = val->color->gray();
	named_text.Rgb_Color.red = gray;
	named_text.Rgb_Color.green = gray;
	named_text.Rgb_Color.blue = gray;
    }
    int nb;
    const char* color_name = (const char*)ColorToName.get(val->color, nb);

    if (color_name) {
	GFREE(named_text.Color);
	named_text.Color = (char*)GMALLOC(strlen(color_name) + 1);
	strcpy(named_text.Color, color_name);
    } else {
	GFREE(named_text.Color);
	char* str = RgbToHexaString(val->color->rgb());
	named_text.Color = (char*)GMALLOC(strlen(str) + 1);
	strcpy(named_text.Color, str);
    }
}


static void _choose_go(void*, GoSensor*);

struct _ChooseGoClosure
{
    int   object;
    void (*proc)(int,Go*);
    int closure;
    _ChooseGoClosure(int o, void (*p)(int, Go*), int cl)
    {
	object = o;
	proc = p;
	closure = cl;
    }
};

void ChooseGo(int object, void (*postcond)(int,Go*), int p_cl)
{
    _ChooseGoClosure* closure = new _ChooseGoClosure(object,
						     postcond, p_cl);
    
    // 1: first choose a point;
    GoFullSensor::create("choosePoint", goroot, NULL,
			 _choose_go, closure);
}

static void _choose_go(void* cl, GoSensor* sensor)
{
    if (sensor->interactor()->state() != GITOR_DONE)
    {
	delete (_ChooseGoClosure*)cl;
	return;
    }

    _ChooseGoClosure* closure = (_ChooseGoClosure*)cl;
    GPoint p = ((GChoosePoint*)sensor->interactor())->choosenPoint();
		
    Go* go = goroot->goAt(p, 0.5);
    if (go) {
	switch(closure->object) {
	  case PEN:
	    if (go->getMetaclass()->isSubclass(GoGeometric::metaclass())) {
		const GPenValues* val = ((GoGeometric*)go)->pen().values();
		if (WhichAttributes == EDIT) {
		    FromPenValuesToEditPen(val, UNNAMED);
		    ModifyGoEditPenSheet();
		} else {
		    FromPenValuesToNamedPen(val);
		    ModifyGoNamedPenSheet();
		}
	    }
	    break;
	  case BRUSH:
	    const GBrushValues* val = NULL;
	    if (go->getMetaclass()->isSubclass(GoGeometric::metaclass())) {
		val = ((GoGeometric*)go)->brush().values();
	    } else
		if (go->getMetaclass()->isSubclass(GoTextual::metaclass())) {
		    val = ((GoTextual*)go)->backgroundBrush().values();
		}
	    if (val) {
		if (WhichAttributes == EDIT) {
		    FromBrushValuesToEditBrush(val, UNNAMED);
		    ModifyGoEditBrushSheet();
		} else {
		    FromBrushValuesToNamedBrush(val);
		    ModifyGoNamedBrushSheet();
		}
	    }
	    break;
	  case TEXTBRUSH:
	    if (go->getMetaclass()->isSubclass(GoTextual::metaclass())) {
		// PROBLEM;
		const GTextBrushValues* val =
		    ((GoTextual*)go)->stringTextBrush().values();
		if (WhichAttributes == EDIT) {
		    FromTextBrushValuesToEditText(val,
						  ((GoTextual*)go)->lineHeight(),
						  go->coordsys(), UNNAMED);
		    ModifyGoEditTextSheet();
		} else {
		    FromTextBrushValuesToNamedText(val,
						   ((GoTextual*)go)->lineHeight(),
						   go->coordsys());
		    ModifyGoNamedTextSheet();
		}
	    }
	    break;
	}
    } else {
	NotGo(main_window);
    }

    // call the previous callback;
    if (closure->proc)
	(*closure->proc)(closure->closure, go);

    delete closure;
} 




void selected_objects(void* t, Go* go)
{
    if (go != goroot) {
	GPointerArray* tab = (GPointerArray*)t;
	tab->add(go);
    }
}


int HaveTheySamePen(int* outline_drawn, int* rounded)
{
    *outline_drawn = (int)GFALSE;
    *rounded = (int)GFALSE;
    GPointerArray* tab = new GPointerArray;
    goroot->forEverySelectedGo(selected_objects, tab);
    
    Go** gos = (Go**)tab->data();
    int nb = tab->nelems();
    if (nb == 0) {
	delete tab;
	return (int)GFALSE;
    }
    const GShPen* sh_pen = NULL;
    for (int i = 0; i < nb; i++) {
	if (gos[i]->getMetaclass()->isSubclass(GoGeometric::metaclass())) {
	    if (((GoGeometric*)gos[i])->outline())
		*outline_drawn = (int)GTRUE;
	    
	    if (! sh_pen)
		sh_pen = ((GoGeometric*)gos[i])->pen().shPen();
	    else
		if (((GoGeometric*)gos[i])->pen().shPen() != sh_pen) {
		    delete tab;
		    return (int)GFALSE;
		}
	}
	if (gos[i]->getMetaclass()->isSubclass(GoPolyline::metaclass()))
	    if (((GoPolyline*)gos[i])->round())
		*rounded = (int)GTRUE;
    }

    if (sh_pen) {
	const GPenValues* val = sh_pen->values();
	const char* name = sh_pen->name();
	if (name && name[0] != '\0')
	    FromPenValuesToEditPen(val, name);
	else
	    FromPenValuesToEditPen(val, UNNAMED);
	delete tab;
	return (int)GTRUE;
    } else {
	delete tab;
	return (int)GFALSE;
    }
}
    

int HaveTheySameBrush(int* background_drawn)
{
    *background_drawn = (int)GFALSE;
    GPointerArray* tab = new GPointerArray;
    goroot->forEverySelectedGo(selected_objects, tab);

    Go** gos = (Go**)tab->data();
    int nb = tab->nelems();
    if (nb == 0) {
	delete tab;
	return (int)GFALSE;
    }
    const GShBrush* sh_brush = NULL;
    for (int i = 0; i < nb; i++) {
	if (gos[i]->getMetaclass()->isSubclass(GoGeometric::metaclass())) {
	    if (((GoGeometric*)gos[i])->background())
		*background_drawn = (int)GTRUE;
	    if (! sh_brush)
		sh_brush = ((GoGeometric*)gos[i])->brush().shBrush();
	    else
		if (((GoGeometric*)gos[i])->brush().shBrush() != sh_brush) {
		    delete tab;
		    return (int) GFALSE;
		}
	}
	if (gos[i]->getMetaclass()->isSubclass(GoTextual::metaclass())) {
	    if (((GoTextual*)gos[i])->backgroundMapped())
		*background_drawn = (int)GTRUE;
	    if (! sh_brush)
		sh_brush = ((GoTextual*)gos[i])->backgroundBrush().shBrush();
	    else
		if (((GoTextual*)gos[i])->backgroundBrush().shBrush() != sh_brush) {
		    delete tab;
		    return (int)GFALSE;
		}
	}
    }

    if (sh_brush) {
	const GBrushValues* val = sh_brush->values();
	const char* name = sh_brush->name();
	if (name && name[0] != '\0')
	    FromBrushValuesToEditBrush(val, name);
	else
	    FromBrushValuesToEditBrush(val, UNNAMED);
	delete tab;
	return (int)GTRUE;
    } else {
	delete tab;
	return (int)GFALSE;
    }
}
    

int HaveTheySameText()
{
    GPointerArray* tab = new GPointerArray;
    goroot->forEverySelectedGo(selected_objects, tab);

    Go** gos = (Go**)tab->data();
    int nb = tab->nelems();
    if (nb == 0) {
	delete tab;
	return (int)GFALSE;
    }
    const GShTextBrush* sh_text = NULL;
    GNumber height = 0;
    GCoordSys cs;
    for (int i = 0; i < nb; i++) {
	if (gos[i]->getMetaclass()->isSubclass(GoTextual::metaclass())) {
	    if (! sh_text) {
		height = ((GoTextual*)gos[i])->lineHeight();
		sh_text = ((GoTextual*)gos[i])->stringTextBrush().shTextBrush();
		cs = gos[i]->coordsys();
	    } else
		if (((GoTextual*)gos[i])->stringTextBrush().shTextBrush()!=sh_text ||
		    ((GoTextual*)gos[i])->lineHeight() != height ||
		    gos[i]->coordsys().I != cs.I ||
		    gos[i]->coordsys().J != cs.J) {
		    delete tab;
		    return (int)GFALSE;
		}
	}
    }

    if (sh_text) {
	const GTextBrushValues* val = sh_text->values();
	const char* name = sh_text->name();
	if (name && name[0] != '\0')
	    FromTextBrushValuesToEditText(val, height, cs, name);
	else
	    FromTextBrushValuesToEditText(val, height, cs, UNNAMED);
	delete tab;
	return (int)GTRUE;
    } else {
	delete tab;
	return (int)GFALSE;
    }
}
    


                          /*********************
			   * Destroy a ShColor *
			   *********************/


static void ShColorDestroyed(void*, const GShColor* sh_color)
{
    ColorToName.rem(sh_color);
}
