#ifdef REV_INFO
static char rcsid[] = "$Id: goCreation.cc,v 1.9 1992/12/30 13:32:04 jojic Exp $";
#endif
/*****************************************************************************
MODULE:  Go Editor
FILE:    goCreation.cc
AUTHOR:  Aline Jacomet
DATE:    May 14th 1991

DESCRIPTION:
    

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include "goMain.h"

#include <Go/Tools/Alloc.h>
#include <Go/Tools/Colormap.h>
#include <Go/Tools/StringAssoc.h>
#include <Go/Tools/Undo.h>

#include <Go/XGoInterf.h>
#include <Go/Rectangle.h>
#include <Go/Circle.h>
#include <Go/RegPoly.h>

#include <Go/Bezier.h>
#include <Go/BSpline.h>
#include <Go/Cardinal.h>
#include <Go/Beta.h>
#include <Go/Tau.h>

#include <Go/Label.h>
#include <Go/Text.h>
#include <Go/SText.h>
#ifdef GoLICENSE
#include <Go/ScText.h>
#endif

#include <Go/Image.h>
#include <Go/GoInDisp.h>
#include <Go/BGoUIEd.h>

#include "classes.h"

/*
 * VARIABLES
 */

extern GStringAssoc NamedPen;
extern GStringAssoc NamedBrush;
extern GStringAssoc NamedText;

extern AllAttributes all_attrib;

extern void* curveBtn;

/*
 * FUNCTIONS
 */

// in xtools.cc;
extern GoInterf* CreateInterfGetImage();
extern GBool HandleXEventOf(void* interf, void* xevent);
extern void  RaiseWindow(void* interf);

// in file goUndo.cc;
extern void UndoCreate(void*);




                                     /*************
                                      * Callbacks *
                                      *************/




void godraw_common_after_create (GoGeometric* newgo)
{    
    newgo->outline (all_attrib.OutlineIsSet);
    newgo->background (all_attrib.BackgroundIsSet);
    if (newgo->getMetaclass()->isSubclass (GoPolyline::metaclass()))
	((GoPolyline*)newgo)->round (all_attrib.RoundedIsSet);
	
    if (strcmp(all_attrib.PenText, UNNAMED) != 0)
    {
	GShPen* p = (GShPen*)NamedPen.get(all_attrib.PenText);
	newgo->pen().modify(p);
    }
    else
    {
	ulong mask = GCOLOR | GALT_COLOR | GLINE_WIDTH | GLINE_STYLE |
	    GCAP_STYLE | GJOIN_STYLE;
	GPenValues values;
	values.line_width = atof(all_attrib.pen.LineWidth);
	values.line.style = all_attrib.pen.LineStyle;
	values.line.cap = all_attrib.pen.CapStyle;
	values.line.join = all_attrib.pen.JoinStyle;
	values.color = interf->colorFromName(all_attrib.pen.Color);
	values.alt_color = interf->colorFromName(all_attrib.pen.AltColor);
	newgo->pen().modify(mask, &values);
	values.color->destroy();
	values.alt_color->destroy();
    }
	
    if (strcmp(all_attrib.BrushText, UNNAMED) != 0)
    {
	GShBrush* b = (GShBrush*)NamedBrush.get(all_attrib.BrushText);
	newgo->brush().modify(b);
    }
    else
    {
	ulong mask = GCOLOR | GFILL_RULE;
	GBrushValues val;
	val.fill.rule = all_attrib.brush.FillRule;
	val.color = interf->colorFromName(all_attrib.brush.Color);
	newgo->brush().modify(mask, &val);
	val.color->destroy();
    }

    UndoReparentStruct* str = (UndoReparentStruct*)
	GMALLOC(sizeof(UndoReparentStruct));
    str->parent = newgo->parent();
    str->next = newgo->next();
    str->go = newgo;
    undoMgr.push(UndoCreate, str, (GUndoProc)free);
}


void godraw_after_create_polyline (void* w, Go* go)
{
    GoGeometric* newgo = (GoGeometric*) go;

    if (newgo != NULL)
    {
	godraw_common_after_create (newgo);
	BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
	bgoui->createPolyline (godraw_after_create_polyline, w);
    }
}


/**
FUNCTION
    CreatePolyline()
DESCRIPTION
    Polyline creation.
**/
void CreatePolyline(void* w, void*, caddr_t)
{
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
    bgoui->cancelGrabbers();
    
    bgoui->createPolyline (godraw_after_create_polyline, w);
}


void godraw_after_create_polygon (void* w, Go* go)
{
    GoGeometric* newgo = (GoGeometric*) go;

    if (newgo != NULL)
    {
	godraw_common_after_create (newgo);
	BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom(goroot);
	bgoui->createPolygon (godraw_after_create_polygon, w);
    }
}


/**
FUNCTION
    CreatePolygon()
DESCRIPTION
    Polygon creation.
**/
void CreatePolygon(void* w, void*, caddr_t)
{
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
    bgoui->cancelGrabbers();
        
    bgoui->createPolygon (godraw_after_create_polygon, w);
}


void godraw_after_create_hexagon (void* w, Go* go)
{
    GoGeometric* newgo = (GoGeometric*) go;

    if (newgo != NULL)
    {
	godraw_common_after_create (newgo);
	BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom(goroot);
	bgoui->createRegPolygon (8, godraw_after_create_hexagon, w);
    }
}


/**
FUNCTION
    CreateHexagon()
DESCRIPTION
    Hexagon creation.
**/
void CreateHexagon(void* w, void*, caddr_t)
{
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
    bgoui->cancelGrabbers();

    bgoui->createRegPolygon (8, godraw_after_create_hexagon, w);
}


void godraw_after_create_hand (void* w, Go* go)
{
    GoGeometric* newgo = (GoGeometric*) go;

    if (newgo != NULL)
    {
	godraw_common_after_create (newgo);
	BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
	bgoui->createHandPolyline (godraw_after_create_hand, w);
    }
}


/**
FUNCTION
    CreateHand()
DESCRIPTION

**/
void CreateHand(void* w, void*, caddr_t)
{
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
    bgoui->cancelGrabbers();
    
    bgoui->createHandPolyline (godraw_after_create_hand, w);
}    

/**
FUNCTION
    CreateImage()
DESCRIPTION
    Image creation.
**/

static GoRoot*   goroot_getimage;
static GoInterf* interf_getimage = NULL;
static void _popup_getimage()
{
    /*
     * because Bug in Xt library during XtRealizeWidget();
     * with TRANSPARENT WIDGET: We create a window
     * and handle it using the Filters functionnalities
     * of the go input dispatcher.
     * because in this case we do not have widget.
     * This bug is in motif 1.1 and 1.2.0 will be fixed in 1.2.1
     */
    if (interf_getimage == NULL)
    {
	/*
	 * Creation of an invisible window used in CreateImage()
	 */
	interf_getimage = CreateInterfGetImage();
	goroot_getimage = GoRoot::create();
	goroot_getimage->interf(interf_getimage);
	(void) new BGoUIEditor (goroot_getimage);
    }

    interf_getimage->map();

    // put window on top;
    RaiseWindow(interf_getimage);

    // append a filter in the XtInputDispatcher to get event;
    // bypass the widget dispatching event;

    GInputDispatcher()->filterProc(HandleXEventOf, interf_getimage, GADD);

}

static void _popdown_getimage()
{
    interf_getimage->unmap();
    GInputDispatcher()->filterProc(HandleXEventOf, interf_getimage, GREMOVE);
}

static void _create_rectangle_for_image_postcond(void*, Go*);
static void _create_image_postcond(void*, Go*);

void CreateImage(void*, void*, caddr_t)
{
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
    bgoui->cancelGrabbers();
    
    _popup_getimage();

    ((BGoUIEditor*) GoUIFrom (goroot_getimage))
	->createRectangle(_create_rectangle_for_image_postcond, NULL);
}

struct _ImageConfig {
    GShImageBrush* imagebrush;
    GShPixelArray* pxlarray;
    _ImageConfig(GShImageBrush* ib, GShPixelArray* pa)
    {
	imagebrush = ib;
	pxlarray = pa;
    }
};


void CreateImageWithImageBrushAndPixelArray(GImageBrushValues& ib_values,
					    GPixelArrayValues& pa_values)
{
    // CREATE THE PIXELARRAY;
    // AND the IMAGEBRUSH;
	
    GShPixelArray* pxlarray = GCreateShPixelArray(GPXL_VALUE | GPXL_GIVEN,
						&pa_values);
    if (pxlarray->refcnt() > 1) {
	GFREE(pa_values.data);
	pa_values.data = NULL;
    }
    
    ulong ib_mask = GPLANE_MASK|GFILL_STYLE;
    
    if (ib_values.colormap)
	ib_mask |= GCOLORMAP;
    
    GShImageBrush* imagebrush = GCreateShImageBrush(ib_mask, &ib_values);

    if (ib_values.colormap)
	ib_values.colormap->destroy();

    // ASK GOUI TO CREATE IMAGE;
    // IN FACT TO PLACE IT;
    
    ((BGoUIEditor*) GoUIFrom (goroot))
	->createImage (pxlarray->values()->width * interf->pixelWidth(),
		       pxlarray->values()->height * interf->pixelHeight(),
		       _create_image_postcond, new _ImageConfig(imagebrush,
								pxlarray));
}

static void _create_rectangle_for_image_postcond(void* /* null */,
						 Go* rect)
{
    if (!rect) {
	_popdown_getimage();
	return;
    }

    // LOCAL VARIABLES;
    
    GImageBrushValues ib_values;
    GPixelArrayValues pa_values;
    GNumber x;
    GNumber y;
    GNumber w;
    GNumber h;
    GCoordSys unit;
    GRegion clip;

    // A RECTANGLE IS CREATED;
    // READ ITS POS, WIDTH, HEIGHT;

    rect->addMyClip(clip);
    const GBox& box = clip.bigbox();
    x = box.xmin;
    y = box.ymin;
    w = box.width();
    h = box.height();
    rect->destroy();
    
    // AND GET THE CORRESPONDING IMAGE;
    // ONTO THE INTERFACE;

    interf_getimage->getImage(ib_values, pa_values, unit, GPoint(x,y),
			      w, h);

    // UNMAP TRANSPARENT WINDOW;
    _popdown_getimage();


    CreateImageWithImageBrushAndPixelArray(ib_values, pa_values);
}


static void _create_image_postcond(void* closure, Go* go)
{
    _ImageConfig* ic = (_ImageConfig*)closure;
    GoImage* image = (GoImage*)go;
    
    
    if (image)
    {
	// set the wanted pxlarray and imagebrush;
	// and handle the the UNDO;
	
	image->imageBrush().modify(ic->imagebrush);
	image->pixelArray().modify(ic->pxlarray);

	UndoReparentStruct* str = (UndoReparentStruct*)
	    GMALLOC(sizeof(UndoReparentStruct));
	str->parent = image->parent();
	str->next = image->next();
	str->go = image;
	undoMgr.push(UndoCreate, str, (GUndoProc)free);
    }

    // destroy created resources for the image;
    // because thau are duplicated by the GoImage;
    // we do not use it anymore;
    
    ic->imagebrush->destroy();
    ic->pxlarray->destroy();
    delete ic;
    return;
}


void godraw_after_create_circle (void* w, Go* go)
{
    GoGeometric* newgo = (GoGeometric*) go;

    if (newgo != NULL)
    {
	godraw_common_after_create (newgo);
	BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
	bgoui->createCircle (godraw_after_create_circle, w);
    }
}


/**
FUNCTION
    CreateCircle()
DESCRIPTION
    Circle creation.
**/
void CreateCircle(void* w, void*, caddr_t)
{
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
    bgoui->cancelGrabbers();
    
    bgoui->createCircle (godraw_after_create_circle, w);
}


void godraw_after_create_ellipse (void* w, Go* go)
{
    GoGeometric* newgo = (GoGeometric*) go;

    if (newgo != NULL)
    {
	godraw_common_after_create (newgo);
	BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
	bgoui->createEllipse (godraw_after_create_ellipse, w);
    }
}


/**
FUNCTION
    CreateEllipse()
DESCRIPTION
    Ellipse creation.
**/
void CreateEllipse(void* w, void*, caddr_t)
{
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
    bgoui->cancelGrabbers();

    bgoui->createEllipse (godraw_after_create_ellipse, w);
}


void godraw_after_create_circle_arc (void* w, Go* go)
{
    GoGeometric* newgo = (GoGeometric*) go;

    if (newgo != NULL)
    {
	godraw_common_after_create (newgo);
	BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
	bgoui->createCircleArc (godraw_after_create_circle_arc, w);
    }
}


/**
FUNCTION
    CreateCircleArc()
DESCRIPTION
     Circle Arc creation.
**/
void CreateCircleArc(void* w, void*, caddr_t)
{
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
    bgoui->cancelGrabbers();
    
    bgoui->createCircleArc (godraw_after_create_circle_arc, w);
}


void godraw_after_create_ellipse_arc (void* w, Go* go)
{
    GoGeometric* newgo = (GoGeometric*) go;

    if (newgo != NULL)
    {
	godraw_common_after_create (newgo);
	BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
	bgoui->createEllipseArc (godraw_after_create_ellipse_arc, w);
    }
}


/**
FUNCTION
    CreateEllipseArc()
DESCRIPTION
     Ellipse Arc creation.
**/
void CreateEllipseArc(void* w, void*, caddr_t)
{
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
    bgoui->cancelGrabbers();
    
    bgoui->createEllipseArc (godraw_after_create_ellipse_arc, w);
}


void godraw_after_create_rectangle (void* w, Go* go)
{
    GoGeometric* newgo = (GoGeometric*) go;

    if (newgo != NULL)
    {
	godraw_common_after_create (newgo);
	BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
	bgoui->createRectangle (godraw_after_create_rectangle, w);
    }
}


/**
FUNCTION
    CreateRectangle()
DESCRIPTION
    Rectangle creation.
**/
void CreateRectangle(void* w, void*, caddr_t)
{
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
    bgoui->cancelGrabbers();
    
    bgoui->createRectangle (godraw_after_create_rectangle, w);
}


void godraw_after_create_square (void* w, Go* go)
{
    GoGeometric* newgo = (GoGeometric*) go;

    if (newgo != NULL)
    {
	godraw_common_after_create (newgo);
	BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
	bgoui->createSquare (godraw_after_create_square, w);
    }
}


/**
FUNCTION
    CreateSquare()
DESCRIPTION
    Square creation.
**/
void CreateSquare(void* w, void*, caddr_t)
{
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
    bgoui->cancelGrabbers();
    
    bgoui->createSquare (godraw_after_create_square, w);
}


void godraw_after_create_bezier (void* w, Go* go)
{
    GoGeometric* newgo = (GoGeometric*) go;

    if (newgo != NULL)
    {
	godraw_common_after_create (newgo);
	BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
	bgoui->createPolypoint (godraw_after_create_bezier, w,
				GoBezier::metaclass()->classname());
    }
}


/**
FUNCTION
    CreateBezier()
DESCRIPTION
    Bezier creation.
**/
void CreateBezier(void*, void*, caddr_t)
{
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
    bgoui->cancelGrabbers();
    
    bgoui->createPolypoint (godraw_after_create_bezier, curveBtn,
			    GoBezier::metaclass()->classname());
}


void godraw_after_create_spline (void* w, Go* go)
{
    GoGeometric* newgo = (GoGeometric*) go;

    if (newgo != NULL)
    {
	godraw_common_after_create (newgo);
	BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
	bgoui->createPolypoint (godraw_after_create_spline, w,
			      GoBSpline::metaclass()->classname());
    }
}


/**
FUNCTION
    CreateBSpline()
DESCRIPTION
    BSpline creation.
**/
void CreateBSpline(void*, void*, caddr_t)
{
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
    bgoui->cancelGrabbers();
    
    bgoui->createPolypoint (godraw_after_create_spline, curveBtn,
			  GoBSpline::metaclass()->classname());
}


void godraw_after_create_cardinal (void* w, Go* go)
{
    GoGeometric* newgo = (GoGeometric*) go;

    if (newgo != NULL)
    {
	godraw_common_after_create (newgo);
	BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
	bgoui->createPolypoint (godraw_after_create_cardinal, w,
			      GoCardinal::metaclass()->classname());
    }
}

/**
FUNCTION
    CreateCardinal()
DESCRIPTION
    Cardinal creation.
**/
void CreateCardinal(void*, void*, caddr_t)
{
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
    bgoui->cancelGrabbers();
    
    bgoui->createPolypoint (godraw_after_create_cardinal, curveBtn,
			  GoCardinal::metaclass()->classname());
}


void godraw_after_create_beta_spline (void* w, Go* go)
{
    GoGeometric* newgo = (GoGeometric*) go;

    if (newgo != NULL)
    {
	godraw_common_after_create (newgo);
	BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
	bgoui->createPolypoint (godraw_after_create_beta_spline, w,
			      GoBetaSpline::metaclass()->classname());
    }
}


/**
FUNCTION
    CreateBetaSpline()
DESCRIPTION
    Beta Spline creation.
**/
void CreateBetaSpline(void*, void*, caddr_t)
{
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
    bgoui->cancelGrabbers();
    
    bgoui->createPolypoint (godraw_after_create_beta_spline, curveBtn,
			  GoBetaSpline::metaclass()->classname());
}


void godraw_after_create_tau_spline (void* w, Go* go)
{
    GoGeometric* newgo = (GoGeometric*) go;

    if (newgo != NULL)
    {
	godraw_common_after_create (newgo);
	BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
	bgoui->createPolypoint (godraw_after_create_tau_spline, w,
			  GoTauSpline::metaclass()->classname());
    }
}


/**
FUNCTION
    CreateTauSpline()
DESCRIPTION
    TauSpline creation.
**/
void CreateTauSpline(void*, void*, caddr_t)
{
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
    bgoui->cancelGrabbers();
    
    bgoui->createPolypoint (godraw_after_create_tau_spline, curveBtn,
			  GoTauSpline::metaclass()->classname());
}


void godraw_after_create_text (void*, Go* go)
{
    GoTextual* newgo = (GoTextual*) go;

    if (!newgo)
	return;
    newgo->mapBackground(all_attrib.BackgroundIsSet);
    newgo->underlinePosition(0.9);
    
    if (strcmp(all_attrib.BrushText, UNNAMED) != 0) {
	GShBrush* b = (GShBrush*)NamedBrush.get(all_attrib.BrushText);
	newgo->backgroundBrush().modify(b);
    } else {
	ulong mask = GCOLOR | GFILL_RULE;
	GBrushValues val;
	val.fill.rule = all_attrib.brush.FillRule;
	val.color = interf->colorFromName(all_attrib.brush.Color);
	newgo->backgroundBrush().modify(mask, &val);
	val.color->destroy();
    }
    
    if (strcmp(all_attrib.TextBrushText, UNNAMED) != 0) {
	int nb;
	const TextStruct* str = (const TextStruct*)
	    NamedText.get(all_attrib.TextBrushText, nb);
	newgo->stringTextBrush().modify(str->sh);
	newgo->lineHeight(str->height);
    } else {
	ulong mask = GCOLOR | GFONT;
	GTextBrushValues values;
	GFont* f = interf->xFontNameToFont(all_attrib.text.Font);
	values.font = f->shFont()->dup();
	values.color = interf->colorFromName(all_attrib.text.Color);
	newgo->stringTextBrush().modify(mask, &values);
	values.color->destroy();
	values.font->destroy();
	delete f;
	
	GNumber height = interf->fontHeightInPixels(all_attrib.text.Font) *
	    interf->pixelHeight();
	newgo->lineHeight(height);
    }
    UndoReparentStruct* str = (UndoReparentStruct*)
	    GMALLOC(sizeof(UndoReparentStruct));
    str->parent = newgo->parent();
    str->next = newgo->next();
    str->go = newgo;
    undoMgr.push(UndoCreate, str, (GUndoProc)free);
}


/**
FUNCTION
    CreateLabel()
DESCRIPTION
    Label creation.
**/
void CreateLabel(void*, void*, caddr_t)
{
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
    bgoui->cancelGrabbers();
    
    bgoui->createLabel(godraw_after_create_text, NULL);
}




/**
FUNCTION
    CreateSimpleText()
DESCRIPTION
    Simple Text creation.
**/
void CreateSimpleText(void*, void*, caddr_t)
{
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
    bgoui->cancelGrabbers();
    
    bgoui->createSimpleText(godraw_after_create_text, NULL);
}    


/**
FUNCTION
    CreateText()
DESCRIPTION
    Text creation.
**/
void CreateText(void*, void*, caddr_t)
{
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
    bgoui->cancelGrabbers();
    
    bgoui->createSimpleFormText(godraw_after_create_text, NULL);
}


#include <Go/Tools/Error.h>

/**
FUNCTION
    CreateMCText()
DESCRIPTION
    Text creation.
**/
#ifdef GoLICENSE
void CreateMCText(void*, void*, caddr_t)
{
    GWarning("CreateMCText not implemented");
}


/**
FUNCTION
    CreateScrolledText()
DESCRIPTION
    Scrolled Text creation.
**/
void CreateScrolledText(void*, void*, caddr_t)
{
    GWarning("CreateScrolledText not implemented");
}
#endif /* GoLICENCE */
