#ifdef REV_INFO
static char rcsid[] = "$Id: goEdition.cc,v 1.11 1992/12/23 08:57:07 jacomet Exp $";
#endif
/*****************************************************************************
MODULE:  Go Editor
FILE:    goEdition.cc
AUTHOR:  Aline Jacomet
DATE:    May 14th 1991

DESCRIPTION:
    

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include "goMain.h"

#include <Go/Tools/Assoc.h>
#include <Go/Tools/Undo.h>

#include <Go/TextDefs.h>
#include <Go/Rectangle.h>
#include <Go/SText.h>
#include <Go/RegPoly.h>
#include <Go/Composite.h>
#include <Go/BGoUIEd.h>
#include <Go/FullSensor.h>
#include <Go/Interactor.h>
#include <Go/ChoosePoint.h>
#include <Xm/PushB.h>

#ifdef GoLICENSE
#include <Go/Ext/XtCBInterf.h>
#endif

extern GAssoc key_assoc;

/*
 * FUNCTIONS
 */

// in goTools.cc;
extern int GetNumMainSelectedObjects();

// in goUndo.cc;
extern void UndoCoordSys(void*);
extern void UndoDestroy(void*);
extern void FreeObj(void*);
extern void UndoCreate(void*);
extern void UndoDestroy(void*);
extern void UndoSelectOne(void*);
extern void UndoReparent(void*);

// in file goSelection.cc;
extern void undo_unselect_all(void*, Go*);

// in message.cc;
extern void NoSelectedObject(void*);
extern void Error(void*);





GoComposite* group_all_selected_subtrees_and_prepare_undo_cs
	(UndoCSStruct*& undoCS)
{
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
    GoComposite* group = bgoui->groupAllSelectedSubtrees ();
    if (group == NULL)
	return NULL;

    Go* child = group->child();
    for (int num_children = 0; child != NULL; child = child->next())
	num_children++;

    undoCS = (UndoCSStruct*) GMALLOC ((1 + num_children) * sizeof *undoCS);
    UndoCSStruct* pUndo = undoCS;
    for (child = group->child(); child != NULL; child = child->next(), pUndo++)
    {
	pUndo->go = child;
	pUndo->cs = child->coordsys();
    }
    pUndo->go = NULL;

    return group;
}


struct CSActionInfo 
{
    const char* actionName;
    GoComposite* group;
    UndoCSStruct* undoCS;

    CSActionInfo (const char* n, GoComposite* g, UndoCSStruct* u) 
    {
	actionName = n;
	group = g;
	undoCS = u;
    }

    ~CSActionInfo ()
    {
	GFREE (undoCS);
	BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (group->root());
	bgoui->ungroup (group);
    }
};


void change_cs_selected_postcond (void* closure, GoSensor* sensor)
{
    CSActionInfo* csaction_info = (CSActionInfo*) closure;

    if (sensor->interactor()->state() == GITOR_DONE)
    {
	if (!strcmp(csaction_info->actionName, "setXAxis"))
	{
	    extern void do_cs_action (void*, GoSensor*);
	    csaction_info->actionName = "setYAxis";
	    GoFullSensor::create("choosePoint", sensor->root(), NULL,
				 do_cs_action, csaction_info);
	    return;
	}

	undoMgr.openMark ();

	for (UndoCSStruct* pUndo = (UndoCSStruct*) csaction_info->undoCS;
	     pUndo->go != NULL;
	     pUndo++)
	{
	    UndoCSStruct* undoStruct = new UndoCSStruct;
	    undoStruct->go = pUndo->go;
	    undoStruct->cs = pUndo->cs;
	    undoStruct->proc = NULL;    // unused here;
	    undoMgr.push (UndoCoordSys, undoStruct, (GUndoProc) free);
	}

	undoMgr.closeMark();
    }	

    for (Go* go = csaction_info->group->child(); go != NULL; go  = go->next())
	go->select();

    delete csaction_info;
}


void do_cs_action (void* closure, GoSensor* choosePointSensor)
{
    CSActionInfo* csaction_info = (CSActionInfo*) closure;

    if (choosePointSensor->interactor()->state() != GITOR_DONE)
	delete csaction_info;
    else
	GoFullSensor::create (csaction_info->actionName, csaction_info->group, NULL,
			      change_cs_selected_postcond, csaction_info);
}	  


void CancelGrabbingSensors (BGoUIEditor* bgoui)
{
    GoSensor* sor;
    while ((sor = bgoui->grabbingSensor()) != NULL)
	sor->destroy();
}



void change_cs_on_selected (const char* sensorName)
{
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
    CancelGrabbingSensors (bgoui);

    UndoCSStruct* undoCS;
    GoComposite* group =
	group_all_selected_subtrees_and_prepare_undo_cs (undoCS);

    if (group == NULL)
	return;
    
    CSActionInfo* csaction_info = new CSActionInfo (sensorName, group, undoCS);
    GoFullSensor::create ("choosePoint", group->root(), NULL,
			  do_cs_action, csaction_info);
}


void Move(void*, void*, caddr_t)
{
    change_cs_on_selected ("move");
}


void RescaleAll(void*, void*, caddr_t)
{
    change_cs_on_selected ("rescale");
}


typedef struct align_struct {
    Go*    go;
    double space;
} AlignStruct;

static GBool first_time;


static void align_left(void* r, Go* go)
{
    Go* ref_go = ((AlignStruct*)r)->go;
    if (ref_go != go) {
	static GBox ref_box;
	if (first_time) {
	    // first Go for alignment;
	    ref_box = ref_go->myClip().bigbox();
	    if (ref_box.empty())
		ref_box = ref_go->clip().bigbox();
	    first_time = GFALSE;
	}
	GBox box = go->myClip().bigbox();
	if (box.empty())
	    box = go->clip().bigbox();
	
	// translation vector;
	GVector v(ref_box.xmin - box.xmin + ((AlignStruct*)r)->space, 0);
	go->move(go->origin() + v);
    }
}

static void align_right(void* r, Go* go)
{
    Go* ref_go = ((AlignStruct*)r)->go;
    if (ref_go != go) {
	static GBox ref_box;
	if (first_time) {
	    // first Go for alignment;
	    ref_box = ref_go->myClip().bigbox();
	    if (ref_box.empty())
		ref_box = ref_go->clip().bigbox();
	    first_time = GFALSE;
	}
	GBox box = go->myClip().bigbox();
	if (box.empty())
	    box = go->clip().bigbox();
	
	// translation vector;
	GVector v(ref_box.xmax - box.xmax + ((AlignStruct*)r)->space, 0);
	go->move(go->origin() + v);
    }
}

static void align_top(void* r, Go* go)
{
    Go* ref_go = ((AlignStruct*)r)->go;
    if (ref_go != go) {
	static GBox ref_box;
	if (first_time) {
	    // first Go for alignment;
	    ref_box = ref_go->myClip().bigbox();
	    if (ref_box.empty())
		ref_box = ref_go->clip().bigbox();
	    first_time = GFALSE;
	}
	GBox box = go->myClip().bigbox();
	if (box.empty())
	    box = go->clip().bigbox();
	
	// translation vector;
	GVector v(0, ref_box.ymin - box.ymin + ((AlignStruct*)r)->space);
	go->move(go->origin() + v);
    }
}

static void align_bottom(void* r, Go* go)
{
    Go* ref_go = ((AlignStruct*)r)->go;
    if (ref_go != go) {
	static GBox ref_box;
	if (first_time) {
	    // first Go for alignment;
	    ref_box = ref_go->myClip().bigbox();
	    if (ref_box.empty())
		ref_box = ref_go->clip().bigbox();
	    first_time = GFALSE;
	}
	GBox box = go->myClip().bigbox();
	if (box.empty())
	    box = go->clip().bigbox();
	
	// translation vector;
	GVector v(0, ref_box.ymax - box.ymax + ((AlignStruct*)r)->space);
	go->move(go->origin() + v);
    }
}

static void align_vertical(void* r, Go* go)
{
    Go* ref_go = ((AlignStruct*)r)->go;
    if (ref_go != go) {
	static GBox ref_box;
	if (first_time) {
	    // first Go for alignment;
	    ref_box = ref_go->myClip().bigbox();
	    if (ref_box.empty())
		ref_box = ref_go->clip().bigbox();
	    first_time = GFALSE;
	}
	GBox box = go->myClip().bigbox();
	if (box.empty())
	    box = go->clip().bigbox();
	
	// translation vector;
	GVector v(((ref_box.xmin + ref_box.xmax) / 2) -
		 ((box.xmin + box.xmax) / 2) + ((AlignStruct*)r)->space,
		 0);
	go->move(go->origin() + v);
    }
}

static void align_horizontal(void* r, Go* go)
{
    Go* ref_go = ((AlignStruct*)r)->go;
    if (ref_go != go) {
	static GBox ref_box;
	if (first_time) {
	    // first Go for alignment;
	    ref_box = ref_go->myClip().bigbox();
	    if (ref_box.empty())
		ref_box = ref_go->clip().bigbox();
	    first_time = GFALSE;
	}
	GBox box = go->myClip().bigbox();
	if (box.empty())
	    box = go->clip().bigbox();
	
	// translation vector;
	GVector v(0,
		 ((ref_box.ymin + ref_box.ymax) / 2) -
		 ((box.ymin + box.ymax) / 2) + ((AlignStruct*)r)->space);
	go->move(go->origin() + v);
    }
}

static void align_bottom_top(void* r, Go* go)
{
    Go* ref_go = ((AlignStruct*)r)->go;
    if (ref_go != go) {
	static GBox ref_box;
	if (first_time) {
	    // first Go for alignment;
	    ref_box = ref_go->myClip().bigbox();
	    if (ref_box.empty())
		ref_box = ref_go->clip().bigbox();
	    first_time = GFALSE;
	}
	GBox box = go->myClip().bigbox();
	if (box.empty())
	    box = go->clip().bigbox();
	
	// translation vector;
	GVector v(0, ref_box.ymin - box.ymax + ((AlignStruct*)r)->space);
	go->move(go->origin() + v);
    }
}

static void align_top_bottom(void* r, Go* go)
{
    Go* ref_go = ((AlignStruct*)r)->go;
    if (ref_go != go) {
	static GBox ref_box;
	if (first_time) {
	    // first Go for alignment;
	    ref_box = ref_go->myClip().bigbox();
	    if (ref_box.empty())
		ref_box = ref_go->clip().bigbox();
	    first_time = GFALSE;
	}
	GBox box = go->myClip().bigbox();
	if (box.empty())
	    box = go->clip().bigbox();
	
	// translation vector;
	GVector v(0, ref_box.ymax - box.ymin + ((AlignStruct*)r)->space);
	go->move(go->origin() + v);
    }
}

static void align_right_left(void* r, Go* go)
{
    Go* ref_go = ((AlignStruct*)r)->go;
    if (ref_go != go) {
	static GBox ref_box;
	if (first_time) {
	    // first Go for alignment;
	    ref_box = ref_go->myClip().bigbox();
	    if (ref_box.empty())
		ref_box = ref_go->clip().bigbox();
	    first_time = GFALSE;
	}
	GBox box = go->myClip().bigbox();
	if (box.empty())
	    box = go->clip().bigbox();
	
	// translation vector;
	GVector v(ref_box.xmin - box.xmax + ((AlignStruct*)r)->space, 0);
	go->move(go->origin() + v);
    }
}

static void align_left_right(void* r, Go* go)
{
    Go* ref_go = ((AlignStruct*)r)->go;
    if (ref_go != go) {
	static GBox ref_box;
	if (first_time) {
	    // first Go for alignment;
	    ref_box = ref_go->myClip().bigbox();
	    if (ref_box.empty())
		ref_box = ref_go->clip().bigbox();
	    first_time = GFALSE;
	}
	GBox box = go->myClip().bigbox();
	if (box.empty())
	    box = go->clip().bigbox();
	
	// translation vector;
	GVector v(ref_box.xmax - box.xmin + ((AlignStruct*)r)->space, 0);
	go->move(go->origin() + v);
    }
}


void undo_cs_proc(void*, Go* go)
{
    UndoCSStruct* old_cs = (UndoCSStruct*) GMALLOC(sizeof(UndoCSStruct));
    old_cs->go = go;
    old_cs->cs = go->coordsys();
    undoMgr.push(UndoCoordSys, old_cs, (GUndoProc)free);
}


static void _align_go(void*, GoSensor*);

struct _AlignClosure {
    int type;
    double space;
    _AlignClosure(int t, double s)
    {
	type = t;
	space = s;
    }
};

void AlignGo(int type, double space)
{
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
    CancelGrabbingSensors (bgoui);
    
    GoFullSensor::create("choosePoint",goroot, NULL, _align_go,
			 new _AlignClosure(type, space));
}

static void _align_go(void* closure, GoSensor* sensor)
{
    _AlignClosure* align = (_AlignClosure*)closure;
    
    if (sensor->interactor()->state() != GITOR_DONE)
    {
	delete align;
	return;
    }

    GPoint p = ((GChoosePoint*)sensor->interactor())->choosenPoint();

    // retreive type and space;
    int type = align->type;
    double space = align->space;
    delete align;
    
    // continue;
    Go* ref_go = goroot->goAt(p, 0.5);
    if (!ref_go)
	return;

    AlignStruct str;
    str.go = ref_go;
    str.space = space;

    first_time = GTRUE;
    
    undoMgr.openMark();
    goroot->forEverySelectedGo(undo_cs_proc, NULL);
    undoMgr.closeMark();
    
    switch(type) {
      case ALIGN_LEFT:
	goroot->forEverySelectedGo(align_left, &str);
	break;
      case ALIGN_RIGHT:
	goroot->forEverySelectedGo(align_right, &str);
	break;
      case ALIGN_BOTTOM:
	goroot->forEverySelectedGo(align_bottom, &str);
	break;
      case ALIGN_TOP:
	goroot->forEverySelectedGo(align_top, &str);
	break;
      case ALIGN_VERTICAL:
	goroot->forEverySelectedGo(align_vertical, &str);
	break;
      case ALIGN_HORIZONTAL:
	goroot->forEverySelectedGo(align_horizontal, &str);
	break;
      case ALIGN_TOP_BOTTOM:
	goroot->forEverySelectedGo(align_top_bottom, &str);
	break;
      case ALIGN_BOTTOM_TOP:
	goroot->forEverySelectedGo(align_bottom_top, &str);
	break;
      case ALIGN_LEFT_RIGHT:
	goroot->forEverySelectedGo(align_left_right, &str);
	break;
      case ALIGN_RIGHT_LEFT:
	goroot->forEverySelectedGo(align_right_left, &str);
	break;
   }
}



void Rotate(void*, void*, caddr_t)
{
    change_cs_on_selected ("rotate");
}    


typedef struct rotate_struct {
    GNumber angle;
    GPoint center;
} RotateStruct;

static void rotate_it(void* str, Go* go)
{
    if (((RotateStruct*)str)->center.x == GBIGGEST_NUMBER &&
	((RotateStruct*)str)->center.y == GBIGGEST_NUMBER) {
	GBox box = go->clip().bigbox();
	((RotateStruct*)str)->center.x = (box.xmin + box.xmax) / 2;
	((RotateStruct*)str)->center.y = (box.ymin + box.ymax) / 2;
    }
    go->rotate(-((RotateStruct*)str)->angle, ((RotateStruct*)str)->center);
}


static void _effective_rotate(RotateStruct&, int);
    
static void _rotate_go(void* closure, GoSensor* sensor)
{
    if (sensor->interactor()->state() != GITOR_DONE)
	return;

    int type = (int)closure;
    RotateStruct str;
    str.center = ((GChoosePoint*)sensor->interactor())->choosenPoint();

    _effective_rotate(str, type);
}
    
void RotateGo(int type, int choose_center)
{
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
    CancelGrabbingSensors (bgoui);
    
    if (choose_center) {
	
        (void)GoFullSensor::create("choosePoint", goroot, NULL,
				   _rotate_go, (void*)type);
	
    } else {
	RotateStruct str;
	str.center.x = GBIGGEST_NUMBER;
	str.center.y = GBIGGEST_NUMBER;
	_effective_rotate(str, type);
    }
}
 
static void _effective_rotate(RotateStruct& str, int type)
{
    undoMgr.openMark();
    goroot->forEverySelectedGo(undo_cs_proc, NULL);
    undoMgr.closeMark();

    switch(type) {
      case ROTATE_45:
	str.angle = PI / 4.0;
	goroot->forEverySelectedGo(rotate_it, &str);
	break;
      case ROTATE_90:
	str.angle = PI / 2.0;
	goroot->forEverySelectedGo(rotate_it, &str);
	break;
      case ROTATE_135:
	str.angle = PI / 4.0 * 3.0;
	goroot->forEverySelectedGo(rotate_it, &str);
	break;
      case ROTATE_180:
	str.angle = PI;
	goroot->forEverySelectedGo(rotate_it, &str);
	break;
      case ROTATE_225:
	str.angle = PI / 4.0 * 5.0;
	goroot->forEverySelectedGo(rotate_it, &str);
	break;
      case ROTATE_270:
	str.angle = PI / 2.0 * 3.0;
	goroot->forEverySelectedGo(rotate_it, &str);
	break;
      case ROTATE_315:
	str.angle = PI / 4.0 * 7.0;
	goroot->forEverySelectedGo(rotate_it, &str);
	break;
    }
}


void structure_proc (void* closure, Go* go)
{
    GAssoc* assoc = (GAssoc*)closure;
    GroupStruct str;
    str.parent = go->parent();
    str.next = go->Next();
    assoc->set(go, sizeof(GroupStruct), &str);
}


void Deform(void*, void*, caddr_t)
{
    change_cs_on_selected ("setXAxis");
}


static void mirror_proc(void* type, Go* go)
{
    // bigbox center has to be identical after transformation;
    go->unselect();
    GCoordSys cs = go->coordsys();
    GBox old_box = go->clip().bigbox();
    switch(*((int*)type)) {
      case MIRROR_I:
	cs.I = -(cs.I);
	go->setCoordsys(cs);
	break;
      case MIRROR_J:
	cs.J = -(cs.J);
	go->setCoordsys(cs);
	break;
      case MIRROR_IJ:
	cs.I = -(cs.I);
	cs.J = -(cs.J);
	go->setCoordsys(cs);
	break;
    }
    GBox new_box = go->clip().bigbox();
    GVector v((old_box.xmax + old_box.xmin) / 2 -
	      ((new_box.xmin + new_box.xmax) / 2),
	      (old_box.ymin + old_box.ymax) / 2 -
	      ((new_box.ymin + new_box.ymax) / 2));
    go->move(go->origin() + v);
    go->select();
}


void MirrorGo(void*, int type)
{
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
    CancelGrabbingSensors (bgoui);
    
    undoMgr.openMark();
    goroot->forEverySelectedGo(undo_cs_proc, NULL);
    undoMgr.closeMark();

    goroot->forEverySelectedGo(mirror_proc, &type);
}



uint nbGoInTabGo;
static int type;

void fillTabGo(void* tab, Go* go)
{
    go->unselect();
    ((Go**)tab)[nbGoInTabGo] = go;
    nbGoInTabGo++;
}

static int SortGo(void* first, void* second)
{
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
    CancelGrabbingSensors (bgoui);
    
    switch(type) {
      case SPACING_X:
	if ((*((Go**)first))->clip().bigbox().xmin <
	    (*((Go**)second))->clip().bigbox().xmin) {
	    return -1;
	} else {
	    if ((*((Go**)first))->clip().bigbox().xmin ==
		(*((Go**)second))->clip().bigbox().xmin)
		return 0;
	    else {
		if ((*((Go**)first))->clip().bigbox().xmin >
		    (*((Go**)second))->clip().bigbox().xmin)
		    return 1;
	    }
	}
	break;
      case SPACING_Y:
	if ((*((Go**)first))->clip().bigbox().ymin <
	    (*((Go**)second))->clip().bigbox().ymin) {
	    return -1;
	} else {
	    if ((*((Go**)first))->clip().bigbox().ymin ==
		(*((Go**)second))->clip().bigbox().ymin)
		return 0;
	    else {
		if ((*((Go**)first))->clip().bigbox().ymin >
		    (*((Go**)second))->clip().bigbox().ymin)
		    return 1;
	    }
	}
	break;
    }
    return 0;
}

void SpacingGo(int t, double dist)
{
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
    CancelGrabbingSensors (bgoui);
    
    type = t;
    int nb_go = GetNumMainSelectedObjects();
    if (nb_go == 0)
	return;

    undoMgr.openMark();
    goroot->forEverySelectedGo(undo_cs_proc, NULL);
    undoMgr.closeMark();
    
    nbGoInTabGo = 0;
    Go** tab_go = (Go**)GMALLOC(nb_go * sizeof(Go*));
    goroot->forEverySelectedGo(fillTabGo, tab_go);
    qsort((char*)tab_go, nb_go, sizeof(Go*), SortGo);

    for (int i = 0 ; i < nb_go - 1; i++) {
	GBox ref = tab_go[i]->clip().bigbox();
	GVector v;
	switch(type) {
	  case SPACING_X:
	    v.x = ref.xmax - tab_go[i+1]->clip().bigbox().xmin + dist;
	    v.y = 0;
	    break;
	  case SPACING_Y:
	    v.x = 0;
	    v.y = ref.ymax - tab_go[i+1]->clip().bigbox().ymin + dist;
	    break;
	}
	tab_go[i+1]->move(tab_go[i+1]->origin() + v);
	tab_go[i]->select();
    }
    tab_go[nb_go - 1]->select();
    
    GFREE(tab_go);
}


#ifdef GoLICENSE
static void add_go(void* set, Go* go)
{
    ((GSet*)set)->add(go);
}
#endif


void Copy (void*, void*, caddr_t
#ifdef GoLICENSE
	                         data
#endif
	   )
{
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
    CancelGrabbingSensors (bgoui);
    
    int nb = GetNumMainSelectedObjects();
    if (nb == 0) {
	NoSelectedObject(main_window);
	return;
    }

#ifdef GoLICENSE    
    GSet set;
    goroot->forEverySelectedSubtree(add_go, &set);
    GTimeStamp t = ((XButtonEvent*)
		    ((XmPushButtonCallbackStruct*)data)->event)->time;
    if (nb == 1) {
	Go* go = (Go*)set.first();
	if (go->getMetaclass()->isSubclass(GoTextual::metaclass())) {
	    if (((GoTextual*)go)->firstSelected() != -1) {
		// A string is selected in this text;
		// Copy it;
		cb_interf->copy(interf, t, GISO_LATIN_TEXT,
				((GoTextual*)go)->selectedString(),
				strlen(((GoTextual*)go)->selectedString()) + 1,
				GTRUE, NULL, NULL);
		return;
	    }
	}
    }
    cb_interf->copy(interf, t, GGO, &set, 0, GTRUE, NULL, NULL);
#else
    cb_interf->copy(goroot);
#endif    
}


#ifdef GoLICENSE
static Boolean ask_go;
static GTimeStamp paste_time;
#endif

#ifdef GoLICENSE
void paste_function(void*, GoInterf*, GClipboardDataFormat type,
		    void* data, ulong, GClipboardError error)
{
    if (error == GCOMMUNICATION_ERROR || error == GFORMAT_ERROR) {
	Error(main_window);
	return;
    }
    if (error == GCONVERSION_ERROR) {
	if (ask_go == GTRUE) {
	    ask_go = GFALSE;
	    cb_interf->paste(interf, paste_time, GISO_LATIN_TEXT,
			     paste_function, NULL);
	    return;
	} else {
	    Error(main_window);
	    return;
	}
    }
    
    undoMgr.openMark();
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
    bgoui->unselectAll ();

    GSet gos;
    if (type == GGO) {
	// The stacking order is changed by the copy/paste;
	// Change it again;
	int nb;
	void** all_gos = ((GSet*)data)->allSet(nb);
	for (int i = nb - 1; i >= 0; i--)
	    gos.add(all_gos[i]);
	
    } else
	if (type == GISO_LATIN_TEXT) {
	    GoTextual* text = GoSimpleText::create();
	    text->setString((char*)data, strlen((char*)data));
	    gos.add(text);
	}

    for (Go* go = (Go*)gos.first(); go; go = (Go*)gos.next()) {
	UndoReparentStruct* str = (UndoReparentStruct*)
	    GMALLOC(sizeof(UndoReparentStruct));
	str->go = go;
	str->parent = NULL;
	str->next = NULL;
	go->reparent(bgoui->currentParent());
	undoMgr.push(UndoReparent, str, (GUndoProc)free);
	
	// Will be unused when sensors would be duplicated;
	// TO REMOVE;
	if (! go->selected()){
	    undoMgr.push(UndoSelectOne, go);
	    go->select();
	}
    }

    goroot->flush();
    Move(NULL, NULL, NULL);
    undoMgr.closeMark();

    delete data;
}
#endif

void Paste(void*, void*, caddr_t
#ifdef GoLICENSE
	   			 data
#endif
	   )
{
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
    CancelGrabbingSensors (bgoui);
    
#ifdef GoLICENSE    
    ask_go = GTRUE;
    paste_time = ((XButtonEvent*)
		  ((XmPushButtonCallbackStruct*)data)->event)->time;
    cb_interf->paste(interf, paste_time, GGO, paste_function, NULL);
#else
    cb_interf->paste(goroot);
#endif    
}

#ifdef GoLICENSE
static void cut_proc (void*, Go* go)
{
    UndoReparentStruct* str = (UndoReparentStruct*)
	GMALLOC(sizeof(UndoReparentStruct));
    str->parent = go->parent();
    str->next = go->next();
    str->go = go;
    undoMgr.push(UndoDestroy, str, FreeObj);
    go->reparent(NULL);
}
#endif


void Cut (void*, void*, caddr_t
#ifdef GoLICENSE
	 		       data
#endif
	 )
{
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
    CancelGrabbingSensors (bgoui);
    
#ifdef GoLICENSE    
    int nb = GetNumMainSelectedObjects();
    if (nb == 0) {
	NoSelectedObject(main_window);
	return;
    }
    
    Copy(NULL, NULL, data);
    undoMgr.openMark();
    goroot->forEverySelectedSubtree(cut_proc, NULL);
    undoMgr.closeMark();
#else
    cb_interf->cut(goroot);
#endif    
}


void Clear(void*, void* graph, caddr_t)
{
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
    CancelGrabbingSensors (bgoui);
    
    if (graph == help_graphic) {
	while (help_goroot->child())
	    help_goroot->child()->destroy();
    } else {
	undoMgr.openMark();
	while (goroot->child()) {
	    UndoReparentStruct* str = (UndoReparentStruct*)
		GMALLOC(sizeof(UndoReparentStruct));
	    str->parent = goroot->child()->parent();
	    str->next = goroot->child()->next();
	    str->go = goroot->child();
	    undoMgr.push(UndoDestroy, str, FreeObj);
	    goroot->child()->reparent(NULL);
	}
	undoMgr.closeMark();
    }
}
