#ifdef REV_INFO
static char rcsid[] = "$Id: goGoUI.cc,v 1.4 1992/12/15 13:50:10 jojic Exp $";
#endif
/*****************************************************************************
MODULE:  Go Editor
FILE:    goGoUI.cc
AUTHOR:  Aline Jacomet
DATE:    August 9th 1991

DESCRIPTION:
    

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include "goMain.h"

#include <Go/Tools/Alloc.h>
#include <Go/Tools/Undo.h>
    
#include <Go/Text.h>
#include <Go/Polypoint.h>
#include <Go/FullSensor.h>
#include <Go/Interactor.h>
#include <Go/BGoUIEd.h>


/*
 * EXTERN FUNCTIONS
 */
// in file goEdition.cc;
extern GBox childrenBox(Go*);
extern void fillTabGo(void*, Go*);
extern void undo_cs_proc(void*, Go*);

// in file goUndo.cc;
extern void UndoCoordSys(void*);
extern void UndoThisCoordSys(void*);
extern void UndoDestroy(void*);
extern void FreeObj(void*);
extern void UndoCurrentParent(void*);
extern void UndoMovePolyPoint(void*);

extern uint numSelectedObjects;
extern uint nbGoInTabGo;


void ChangeCS_after (void* closure, GoSensor* sensor)
{
    if (sensor->interactor()->state() != GITOR_DONE ||
	sensor->root() != goroot)
	GFREE (closure);
    else 
    {
	UndoCSStruct* old_cs = (UndoCSStruct*) closure;
	undoMgr.push (&UndoCoordSys, old_cs, (GUndoProc) free);
    }
}


void ChangeThisCS_after (void* closure, GoSensor* sensor)
{
    if (sensor->interactor()->state() != GITOR_DONE ||
	sensor->root() != goroot)
	GFREE (closure);
    else 
    {
	UndoCSStruct* old_cs = (UndoCSStruct*) closure;
	undoMgr.push (&UndoThisCoordSys, old_cs, (GUndoProc) free);
    }
}


static void ChangeGoCS
	(Go* go, const GEvent* event, const char* sorName, GBool thisgo_only = GFALSE)
{
    UndoCSStruct* old_cs = NULL;
    if (go->root() == goroot)
    {
	old_cs = (UndoCSStruct*)GMALLOC(sizeof(UndoCSStruct));
	old_cs->cs = go->coordsys();
	old_cs->go = go;
    }
	
    void (*postcond)(void*, GoSensor*);
    postcond = &ChangeCS_after;
    if (thisgo_only)
	postcond = &ChangeThisCS_after;
		    
    (void) GoFullSensor::create (sorName, go, event, postcond, old_cs);
}


void Move (GoRoot* root, const GEvent* event)
{
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (root);
    Go* go = root->goAt (event->position(), bgoui->maxDistance());
    if (go == NULL)
	return;
    
    ChangeGoCS (go, event, "move");
}


void Rotate (GoRoot* root, const GEvent* event)
{
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (root);
    Go* go = root->goAt (event->position(), bgoui->maxDistance ());
    if (go == NULL)
	return;

    ChangeGoCS (go, event, "rotateAroundCenter");
}    


void Rescale (GoRoot* root, const GEvent* event)
{
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (root);
    Go* go = root->goAt (event->position(), bgoui->maxDistance());
    if (go == NULL)
	return;

    ChangeGoCS (go, event, "rescale");
}


void Destroy(Go* go)
{
    if (go == NULL)
	return;

    if (go->root() != goroot)
	go->destroy();
    else {
	BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);

	undoMgr.openMark();
	if (go->selected()) {
	    go->unselect();
	}
	if (go == bgoui->currentParent()) {
	    undoMgr.push(UndoCurrentParent, go);
	    bgoui->currentParent (go->root());
	}
	UndoReparentStruct* str = (UndoReparentStruct*)
	    GMALLOC(sizeof(UndoReparentStruct));
	str->go = go;
	str->parent = go->parent();
	str->next = go->next();
	undoMgr.push(UndoDestroy, str, FreeObj);
	go->reparent(NULL);
	undoMgr.closeMark();
    }
}


void MoveThis(Go* go, const GEvent* event)
{
    ChangeGoCS (go, event, "moveThis", GTRUE);
}    


void RescaleThis(Go* go, const GEvent* event)
{
    ChangeGoCS (go, event, "rescaleThis", GTRUE);
}




