#ifdef REV_INFO
static char rcsid[] = "$Id: goLoadSave.cc,v 1.9 1992/12/21 16:48:51 jacomet Exp $";
#endif
/*****************************************************************************
MODULE:  Go Editor
FILE:    goLoadSave.cc
AUTHOR:  Aline Jacomet
DATE:    May 15th 1991

DESCRIPTION:
    

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include <signal.h>

#include <Go/XtGoInterf.h>
#include <Go/PSGoInterf.h>

#include "goMain.h"
#include "classes.h"

#include <Go/GoAscii.h>
#ifdef GoLICENSE
#include <Go/GoBinary.h>
#endif
#include <Go/Tools/FileMedium.h>
#include <Go/Tools/Alloc.h>
#include <Go/Tools/FilePath.h>
#include <Go/Tools/Atom.h>
#include <Go/Tools/StringAssoc.h>
#include <Go/Tools/Undo.h>

#include <Go/Geometric.h>
#include <Go/Textual.h>
#include <Go/BGoUIEd.h>




/*
 * VARIABLES
 */

GSet SaveShared;
GSet HasDestroyPostcond;

extern GAssoc key_assoc;
extern GStringAssoc NamedPen;
extern GStringAssoc NamedBrush;
extern GStringAssoc NamedText;

extern AllAttributes all_attrib;

extern char* lpr_value;

extern char* header;



/*
 * EXTERN FUNCTIONS
 */

// in file message.cc;
extern void NoFile(void*);
extern void Error(void*);
extern void CannotDestroy(void*, char*);
extern void ConfirmDestroyShared(void*, char*, void*);

// in file goEdition.cc;
extern void structure_proc(void*, Go*);
extern void Clear(void*, void*, caddr_t);

// in file viewmenu.cc;
extern int WhichOrientation();

// in file namedattrib.cc;
extern void ChangePenList();
extern void ChangeBrushList();
extern void ChangeTextBrushList();

// in goUndo.cc;
extern void UndoCreate(void*);

// in goStructure.cc;
void HandleComposite(void*, Go* go);

    
    
void SaveAssoc(void*, const Go*, GoStream*);
void sharedDestroyed(void*, const GSharedObject*);


/*
      Loading a Go File:
          Create a GoFileStream
	  Open it in read
	  Restore the Go (sub)tree under the current
		parent Go

*/

void InitializeNamedText(void*, Go* go)
{
    if (go->getMetaclass()->isSubclass(GoTextual::metaclass()))
	if (((GoTextual*)go)->stringTextBrush().shTextBrush() ->isNamed()) {
	    TextStruct str;
	    str.sh = ((GoTextual*)go)->stringTextBrush().shTextBrush()->dup();
	    str.height = ((GoTextual*)go)->lineHeight();
	    NamedText.set(((GoTextual*)go)->stringTextBrush().shTextBrush()->name(), sizeof(TextStruct), &str);
	    if (! HasDestroyPostcond.contains(str.sh)) {
		HasDestroyPostcond.add(str.sh);
		str.sh->destroyPostcond((void(*)(void*, const GShTextBrush*))sharedDestroyed, NULL);
	    }
	    str.sh->destroy();
	}
}


void LoadGo(void* graphic_wid, char* filename)
{
    GFileMedium* medium = new GFileMedium();
    if (! medium->open(filename, "r")) {
	GoStream* gostream = NULL;
#ifdef GoLICENSE	
	if (isGoBinary(medium))
	     gostream = new GoBinary(medium);
	else
#endif	    
	    gostream = new GoAscii(medium) ;
	gostream->openRead();
	if (header)
	    GFREE(header);
	ulong nb;
	gostream->readHeader(&nb, &header);
	Go* go;
	char* class_name;
	GPen* pen = NULL;
	GBrush* brush = NULL;
	undoMgr.openMark();
	do {
	    if (!gostream->restoreObj((void**)&go, &class_name)) {
		Error(main_window);
		break;
	    } else
	    if (go) {
		if (graphic_wid == graphic) {
		    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
		    go->reparent(bgoui->currentParent());

		    UndoReparentStruct* str = (UndoReparentStruct*)
			GMALLOC(sizeof(UndoReparentStruct));
		    str->parent = go->parent();
		    str->next = go->next();
		    str->go = go;
		    undoMgr.push(UndoCreate, str, (GUndoProc)free);

		    // set the applicationComposite flag for Composite;
		    go->ForEveryGo(HandleComposite, NULL);

		} else
		    go->reparent(help_goroot);
		
		if (graphic_wid != help_graphic) {
		    if (!pen || !brush)
			if (go->getMetaclass()->isSubclass
			    (GoGeometric::metaclass())) {
			pen = &((GoGeometric*)go)->pen();
			brush = &((GoGeometric*)go)->brush();
		    }
		    if (go->getMetaclass()->isSubclass(GoTextual::metaclass()))
			if (! brush)
			    brush = &((GoTextual*)go)->backgroundBrush();
		    
		    go->ForEveryGo(InitializeNamedText, NULL);
		}
	    }
	} while (go);
	
	gostream->close();
	delete gostream;
	medium->close();
	undoMgr.closeMark();
	/*
	 * Parse SharedObject lists to update NamedPen and NamedBrush
	 */
	if (pen)
	    for (GSharedObject* sh_obj = pen->shPen()->headOfList(); sh_obj;
		 sh_obj = sh_obj->next())
		if (sh_obj->isNamed()) {
		    NamedPen.set(sh_obj->name(), sh_obj);
		    if (! HasDestroyPostcond.contains(sh_obj)) {
			HasDestroyPostcond.add(sh_obj);
			sh_obj->destroyPostcond(sharedDestroyed, NULL);
		    }
		}
	if (brush)
	    for (sh_obj = brush->shBrush()->headOfList(); sh_obj;
		 sh_obj = sh_obj->next())
		if (sh_obj->isNamed()) {
		    NamedBrush.set(sh_obj->name(), sh_obj);
		    if (! HasDestroyPostcond.contains(sh_obj)) {
			HasDestroyPostcond.add(sh_obj);
			sh_obj->destroyPostcond(sharedDestroyed, NULL);
		    }
		}
    } else
	NoFile(main_window);
    delete medium;
}



void RestoreAssoc(void*, Go* go, GoStream* stream)
{
    char c;
    char* key = NULL;
    char* val = NULL;
    stream->readChar(&c);
    while(c == 'a') {
	stream->readString(0, &key);
	stream->readString(0, &val);
	GAtom key_atom = GCreateAtom(key);
	go->assoc()->set(key_atom, strlen(val) + 1, val);
	if (! key_assoc.get(go))
	    go->savePostcond(SaveAssoc, NULL);
	key_assoc.add(go, key_atom);
	stream->readChar(&c);
	GFREE(key);
	GFREE(val);
    }
}





/*
      Saving all or selected Go
          Create a GoFileStream
	  Open it in write
	  Save GoRoot or every selected Go in the stream
*/


static GBool error;


static void save_proc(void* stream, Go* go)
{
    if (!error)
	if (! ((GoStream*)stream)->saveObj(go, go->getMetaclass()->classname())) {
	    Error(main_window);
	    error = GTRUE;
	}
}
    
    
    
void SaveGo (char* filename, unsigned char all, unsigned char binary)
{
    GFileMedium* medium = new GFileMedium();
    if (medium->open(filename, "w")) {
	Error(main_window);
	delete medium;
	return;
    }

    GoStream* gostream = NULL;
    if (! binary)
	gostream = new GoAscii(medium);
#ifdef GoLICENSE    
    else
	gostream = new GoBinary(medium);
#endif    

    gostream->openWrite();

    if (header)
	gostream->writeHeader(strlen(header) + 1, header);

    error = GFALSE;

    if (all) {
	for (Go* ch = goroot->child(); ch; ch = ch->next())
	    if (! gostream->saveObj(ch, ch->getMetaclass()->classname())) {
		Error(main_window);
		break;
	    }
    } else {
	goroot->forEverySelectedSubtree(save_proc, gostream);
    }

    gostream->close();
    medium->close();
    delete gostream;
    delete medium;
}



void SaveAssoc(void*, const Go* go, GoStream* stream)
{
    int nb = 0;
    GAtom* key_atom = (GAtom*)key_assoc.get(go, nb);
    if (nb > 0) {
	int n;
	for (int i = 0; i < nb; i++) {
	    char* value = (char*)go->assoc()->get(key_atom[i], n);
	    stream->writeChar('a', "assoc");
	    stream->writeString(GAtomToString(key_atom[i]), "key");
	    stream->writeString(value, "value");
	}
    }
    stream->writeChar('e', "end of assoc");
}



/*
  Write a PostScript File.
*/


static void ps_proc(void* psroot, Go* go)
{
    go->unselect();
    go->reparent((Go*)psroot);
}


void GoToPostscript(char* filename, unsigned char all)
{
    GFileMedium* medium = new GFileMedium();
    if (medium->open(filename, "w")) {
	Error(main_window);
	delete medium;
	return;
    }

    GoInterf* psinterf = new PSGoInterf(medium,
					(GLayoutMode)WhichOrientation());
    psinterf->map();

    if (all) {
	goroot->interf(psinterf);
	goroot->flush();
	goroot->interf(interf);
    } else {
	GoRoot* psroot = GoRoot::create();
	psroot->interf(psinterf);
	psroot->map();
	GAssoc assoc;
	goroot->forEverySelectedSubtree(structure_proc, &assoc);
	
	goroot->forEverySelectedSubtree(ps_proc, psroot);
	psroot->flush();
	
	int nb;
	void** tab = assoc.getKeys(nb);
	for (int i = 0; i < nb; i++) {
	    int n;
	    GroupStruct* str = (GroupStruct*)assoc.get(tab[i], n);
	    ((Go*)tab[i])->reparent(str->parent, str->next);
	    ((Go*)tab[i])->select();
	}
	
	psroot->destroy();
    }
    delete psinterf;
    delete medium;
}




                             /*********
			      * PRINT *
			      *********/

void Print(void*, void*, caddr_t)
{
    if (lpr_value) {
	FILE* f = popen(lpr_value, "w");
	if (f) {
	    signal(SIGPIPE, SIG_IGN);
	    GFileMedium* medium = new GFileMedium(f);
	    GoInterf* psinterf = new PSGoInterf(medium,
					(GLayoutMode)WhichOrientation());
	    psinterf->map();
	    goroot->interf(psinterf);
	    goroot->flush();
	    goroot->interf(interf);
	    delete psinterf;
	    delete medium;
	    pclose(f);
	    signal(SIGPIPE, SIG_DFL);
	} else
	    Error(main_window);
    } else
	Error(main_window);
}



                             /**********************
			      * SAVE NAMED OBJECTS *
			      **********************/


void sharedDestroyed(void*, const GSharedObject* shared)
{
    if (SaveShared.contains(shared))
	SaveShared.remove(shared);
    const char* name = shared->name();
    
    if (strcmp(shared->getClassname(), GShPen::classname()) == 0) {
	NamedPen.rem(name);
	if (strcmp(all_attrib.PenText, name) == 0) {
	    GFREE(all_attrib.PenText);
	    all_attrib.PenText = (char*)GMALLOC(strlen(UNNAMED) + 1);
	    strcpy(all_attrib.PenText, UNNAMED);
	    GFREE(all_attrib.pen.PenText);
	    all_attrib.pen.PenText = (char*)GMALLOC(strlen(UNNAMED) + 1);
	strcpy(all_attrib.pen.PenText, UNNAMED);
	}
    } else
    if ( strcmp(((GSharedObject*)shared)->getClassname(),
		GShBrush::classname()) == 0) {
	NamedBrush.rem(name);
	if (strcmp(all_attrib.BrushText, name) == 0) {
	    GFREE(all_attrib.BrushText);
	    all_attrib.BrushText = (char*)GMALLOC(strlen(UNNAMED) + 1);
	    strcpy(all_attrib.BrushText, UNNAMED);
	    GFREE(all_attrib.brush.BrushText);
	    all_attrib.brush.BrushText = (char*)GMALLOC(strlen(UNNAMED) + 1);
	    strcpy(all_attrib.brush.BrushText, UNNAMED);
	}
    } else
    if ( strcmp(((GSharedObject*)shared)->getClassname(),
		GShTextBrush::classname()) == 0) {
	NamedText.rem(name);
	if (strcmp(all_attrib.TextBrushText, name) == 0) {
	    GFREE(all_attrib.TextBrushText);
	    all_attrib.TextBrushText = (char*)GMALLOC(strlen(UNNAMED) + 1);
	    strcpy(all_attrib.TextBrushText, UNNAMED);
	    GFREE(all_attrib.text.TextBrushText);
	    all_attrib.text.TextBrushText = (char*)GMALLOC(strlen(UNNAMED)+1);
	    strcpy(all_attrib.text.TextBrushText, UNNAMED);
	}
    }
}


int saveNumber(const GNumber* n, GoStream* stream)
{
    return (stream->writeNumber(*n, "Text Line Height"));
}


int restoreNumber(GNumber** n, GoStream* stream)
{
    if (*n = new GNumber) {
	GNumber num;
	if (!stream->readNumber(&num))
	    return GFALSE;
	**n = num;
	return GTRUE;
    }
    return GFALSE;
}


void* OpenWrite()
{
    GFileMedium* medium = new GFileMedium();
    char* all_name = GGetFilePath("app-files/godraw", NAMED_OBJECTS_FILE);
    if (all_name) {
	if (medium->open(all_name, "w")) {
	    delete medium;
	    return NULL;
	}
	GFREE(all_name);
    } else
	if (medium->open(NAMED_OBJECTS_FILE, "w")) {
	    delete medium;
	    return NULL;
	}
#ifdef GoLICENSE
    GoStream* gostream = new GoBinary(medium);
#else    
    GoStream* gostream = new GoAscii(medium);
#endif
    gostream->openWrite();
    return gostream;
}


void CloseAll(void* stream)
{
    ((GoStream*)stream)->close();
    GFileMedium* medium = (GFileMedium*)((GoStream*)stream)->medium();
    delete (GoStream*)stream;
    medium->close();
    delete medium;
}


int SaveGoObject(void* stream, char* name, int type)
{
    switch(type) {
      case PEN:
	GShPen* p = (GShPen*)NamedPen.get(name);
	if (! ((GoStream*)stream)->saveObj(p, p->getClassname()))
	    return 0;
	if (! SaveShared.contains(p)) {
	    SaveShared.add(p);
	    p->dup();
	}
	break;
      case BRUSH:
	GShBrush* b = (GShBrush*)NamedBrush.get(name);
	if (! ((GoStream*)stream)->saveObj(b, b->getClassname()))
	    return 0;
	if (! SaveShared.contains(b)) {
	    SaveShared.add(b);
	    b->dup();
	}
	break;
      case TEXTBRUSH:
	int nb;
	TextStruct* str = (TextStruct*)
	    NamedText.get(name, nb);
	if (! ((GoStream*)stream)->saveObj(str->sh, str->sh->getClassname()))
	    return 0;
	if (! ((GoStream*)stream)->saveObj(&str->height, "GNumber"))
	    return 0;
	if (! SaveShared.contains(str->sh)) {
	    SaveShared.add(str->sh);
	    str->sh->dup();
	}
	break;
      default:
	return 0;
    }
    return 1;
}


void DestroyIt(void*, void* shared)
{
    if (strcmp(((GSharedObject*)shared)->getClassname(),
	     GShPen::classname()) == 0) {
	((GSharedObject*)shared)->destroy();
	ChangePenList();
    } else
	if (strcmp(((GSharedObject*)shared)->getClassname(),
		 GShBrush::classname()) == 0) {
	    ((GSharedObject*)shared)->destroy();
	    ChangeBrushList();
	} else
	    if (strcmp(((GSharedObject*)shared)->getClassname(),
		     GShTextBrush::classname()) == 0) {
		((GSharedObject*)shared)->destroy();
		ChangeTextBrushList();
	    }
}



void DestroyGoObject(char* name, int type)
{
    switch(type) {
      case PEN:
	GShPen* p = (GShPen*)NamedPen.get(name);
	if (p->refcnt() == 1)
	    ConfirmDestroyShared(main_window, name, p);
	else
	    CannotDestroy(main_window, name);
	break;
      case BRUSH:
	GShBrush* b = (GShBrush*)NamedBrush.get(name);
	if (b->refcnt() == 1)
	    ConfirmDestroyShared(main_window, name, b);
	else
	    CannotDestroy(main_window, name);
	break;
      case TEXTBRUSH:
	int nb;
	TextStruct* str = (TextStruct*)
	    NamedText.get(name, nb);
	if (str->sh->refcnt() == 1)
	    ConfirmDestroyShared(main_window, name, str->sh);
	else
	    CannotDestroy(main_window, name);
	break;
    }
}





                                 /*******
				  * NEW *
				  *******/


void GoNew()
{
    Clear(NULL, graphic, NULL);
}
