/* $Id: goMain.h,v 1.6 1992/12/14 12:59:20 jojic Exp $ */
/*****************************************************************************
MODULE:  Go Editor
FILE:    goMain.h
AUTHOR:  Aline Jacomet
DATE:    May 14th 1991

DESCRIPTION:
    

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#ifndef _goMain_h
#define _goMain_h

#include <Go/Go.h>
#include <Go/mGo.h>
#include <Go/GoRoot.h>    
#include <Go/GoUI.h>
#include <Go/TextDefs.h>

#ifdef GoLICENSE
#include <Go/Ext/XtCBInterf.h>
#else
#include "Clipboard.h"
#endif

#include "common.h"


class XGoInterf;
class GXtClipboardInterf;
class GUndoManager;


                               /***********
				* DEFINES *
				***********/

#define GOUIFILE	"godraw.goui"




                               /******
				* GO *
				******/

extern GoRoot* goroot;
extern GoRoot* named_goroot;
extern GoRoot* edit_goroot;
extern GoRoot* help_goroot;

extern XGoInterf* interf;
extern XGoInterf* named_interf;
extern XGoInterf* edit_interf;
extern XGoInterf* help_interf;

#ifdef GoLICENSE
extern GXtClipboardInterf* cb_interf;
#else
extern GoDrawClipboard* cb_interf;
#endif

#include <Go/Tools/Undo.h>
extern GUndoManager undoMgr;




                              /*******************
			       * UNDO STRUCTURES *
			       *******************/

typedef struct _UndoStruct {
    GUndoProc proc;
    Go* go;
};

typedef struct _UndoCSStruct : public _UndoStruct {
    GCoordSys cs;
} UndoCSStruct;

typedef struct _UndoReparentStruct : public _UndoStruct {
    Go* parent;
    Go* next;
} UndoReparentStruct;

typedef struct _UndoBrotherStruct : public _UndoStruct {
    Go* next;
} UndoBrotherStruct;

typedef struct _UndoRadiiStruct : public _UndoStruct {
    GNumber radiusX;
    GNumber radiusY;
} UndoRadiiStruct;

typedef struct _UndoArcPointStruct : public _UndoStruct {
    GNumber start;
    GNumber amplitude;
} UndoArcPointStruct;

typedef struct _UndoTextSizeStruct : public _UndoStruct {
    GNumber size;
} UndoTextSizeStruct;

typedef struct _UndoPolyPointStruct : public _UndoStruct {
    int    index;
    GPoint point;
} UndoPolyPointStruct;

typedef struct _UndoRectStruct : public _UndoStruct {
    GNumber width;
    GNumber height;
} UndoRectStruct;

typedef struct _UndoRegPolyStruct : public _UndoStruct {
    GNumber      angle;
    GNumber      radius;
} UndoRegPolyStruct;



                              /**********
			       * OTHERS *
			       **********/


typedef struct _textStruct {
    class GShTextBrush* sh;
    GNumber       height;
} TextStruct;


typedef struct _GroupStruct {
    Go* parent;
    Go* next;
} GroupStruct;

#endif /* _goMain_h */
