#ifdef REV_INFO
static char rcsid[] = "$Id: goMiscel.cc,v 1.6 1992/12/10 16:24:37 jojic Exp $";
#endif
/*****************************************************************************
MODULE:  Go Editor
FILE:    goMiscel.cc
AUTHOR:  Aline Jacomet and Philippe Smadja.
DATE:    July 9th 1991

DESCRIPTION:
    

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include "goMain.h"
#include <stdio.h>

#include <Go/Ext/Gif.h>
#include <Go/Tools/Atom.h>
#include <Go/Tools/Alloc.h>
#include <Go/Tools/Undo.h>
#include <Go/Tools/Error.h>

#include <Go/Image.h>

#include <Go/Textual.h>
#include <Go/BGoUIEd.h>
#include <Go/XGoInterf.h>



/*
 * FUNCTIONS
 */

// in file goLoadSave.cc;
extern void SaveAssoc(void*, const Go*, GoStream*);

// in file goUndo.cc;
extern void UndoCreate(void*);

// in goMiscel.cc;
extern void ReallyOk(char*, char*);

// in file message.cc;
extern void ConfirmKeyName(void*);
extern void NoFile(void*);

// in goCreation.cc;
extern void* CreateSimpleText(void*, void*, caddr_t);
extern void* CreateText(void*, void*, caddr_t);
extern void  CreateImageWithImageBrushAndPixelArray(GImageBrushValues&,
						    GPixelArrayValues&);
#ifdef GoLICENSE
extern void* CreateScrolledText(void*, void*, caddr_t);
#endif




                          /********************
			   * Assoc management *
			   ********************/



GAssoc key_assoc;
Go* selected_go = NULL;

static char* key_stat;
static char* value_stat;

static void destroy_assoc(void*, const Go*);

// in file miscelmenu.cc;
extern void AddItem(char*);



void selected(void*, Go* go)
{
    selected_go = go;
}
    
void AssocRemove(char* key, char* value)
{
    goroot->forEverySelectedGo(selected, NULL);
    // using CreateAtom is the only way to know the atom associated to the
    // string !!!!! But ref_count is incremented ==> FreeAtom();
    GAtom key_atom = GCreateAtom(key);
    selected_go->assoc()->rem(key_atom, strlen(value) + 1, value);
    GFreeAtom(key_atom);
    key_assoc.rem(selected_go, key_atom);
    // This GFreeAtom() correspond to the GCreateAtom in AssocOk();
    GFreeAtom(key_atom);
}


void AssocReallyOk(void*, unsigned int replace, void*)
{
    char old_item[1000];
    GAtom key_atom = GCreateAtom(key_stat);
    if (replace) {
	int nb;
	char* value = (char*)selected_go->assoc()->get(key_atom, nb);
	strcpy(old_item, key_stat);
	strcat(old_item, " ----> ");
	strcat(old_item, value);
    }	
    selected_go->assoc()->set(key_atom, strlen(value_stat) + 1, value_stat);
    if (! key_assoc.get(selected_go)) {
	selected_go->savePostcond(SaveAssoc, NULL);
	selected_go->destroyPostcond(destroy_assoc, NULL, GADD);
    }
    if (! replace)
	key_assoc.add(selected_go, key_atom);
    char item[1000];
    strcpy(item, key_stat);
    strcat(item, " ----> ");
    strcat(item, value_stat);
    if (replace)
	ReallyOk(old_item, item);
    else
	ReallyOk(NULL, item);
    // Free strings returned by XmText(Field)GetString in Ok();
    GFREE(key_stat);
    GFREE(value_stat);
}


void AssocOk(char* key, char* value)
{
    // I have to define two static variables because AssocReallyOk is a;
    // callback;
    key_stat = key;
    value_stat = value;
    goroot->forEverySelectedGo(selected, NULL);
    GAtom key_atom = GCreateAtom(key);
    int nb;
    if (selected_go->assoc()->get(key_atom, nb)) {
	// this key is already used;
	ConfirmKeyName(main_window);
    } else
	AssocReallyOk(NULL, GFALSE, NULL);
    GFreeAtom(key_atom);
}


void InitializeList()
{
    goroot->forEverySelectedGo(selected, NULL);
    int nb_keys = 0;
    GAtom* keys = (GAtom*)key_assoc.get(selected_go, nb_keys);
    char str[1000];
    int n = 0;
    for (int i = 0; i < nb_keys; i++) {
	strcpy(str, GAtomToString(keys[i]));
	strcat(str, " ----> ");
	strcat(str, (char*)selected_go->assoc()->get(keys[i], n));
	AddItem(str);
    }
}


void dup_assoc(void*, Go* src, Go* cpy)
{
    int nb;
    Go** gos = (Go**)key_assoc.getKeys(nb);
    GBool cont = GFALSE;
    for (int i = 0; i < nb; i++)
	if (gos[i] == src) {
	    cont = GTRUE;
	    break;
	}
    GFREE(gos);
    
    if (cont) {
	int nb_keys, n;
	GAtom* keys = (GAtom*)key_assoc.get(src, nb_keys);
	for (i = 0; i < nb_keys; i++) {
	    char* value = (char*)src->assoc()->get(keys[i], n);
	    cpy->assoc()->set(keys[i], strlen(value) + 1, value);
	    if (! key_assoc.get(cpy)) {
		cpy->savePostcond(SaveAssoc, NULL);
		cpy->destroyPostcond(destroy_assoc, NULL);
	    }
	    key_assoc.add(cpy, keys[i]);
	}
    }
}


static void destroy_assoc(void*, const Go* go)
{
    int nb_keys;
    key_assoc.get(go, nb_keys);
    if (nb_keys > 0)
	key_assoc.rem(go);
}


                          /***********************
			   * Gif File -> GoImage *
			   ***********************/


void LoadGif(char* filename)
{
    GImageBrushValues ib_values;
    GPixelArrayValues pa_values;

    if (!GGifLoad(filename, ib_values, pa_values)) {
	NoFile(main_window);
	return;
    }
    
    CreateImageWithImageBrushAndPixelArray(ib_values, pa_values);
}


                            /***************************
			     * Ascii File -> GoTextual *
			     ***************************/

static void _init_text_string(void* closure, Go* go)
{
    if (!go)
	return;

    const char* string = (const char*)closure;
    ((GoTextual*)go)->setString(string, strlen(string));

    // handle undo;
    // It was a creation;
    UndoReparentStruct* str = new UndoReparentStruct;
    str->parent = go->parent();
    str->next = go->next();
    str->go = go;
    undoMgr.push(UndoCreate, str, (GUndoProc)free);
}

/**
FUNCTION
    InitializeText()
DESCRIPTION
    
**/
void InitializeText(char* text)
{
    BGoUIEditor* bgoui = (BGoUIEditor*)GoUIFrom(goroot);
    bgoui->createSimpleFormText(_init_text_string, (void*)text);
}


/**
FUNCTION
    InitializeSimpleText()
DESCRIPTION
    
**/
void InitializeSimpleText(char* text)
{
    BGoUIEditor* bgoui = (BGoUIEditor*)GoUIFrom(goroot);
    bgoui->createSimpleText(_init_text_string, (void*)text);
}


/**
FUNCTION
    InitializeScrolledText()
DESCRIPTION
    
**/
#ifdef GoLICENSE
void InitializeScrolledText(char* text)
{
    GWarning ("InitializeScrolledText not implemented");
}
#endif

