#ifdef REV_INFO
static char rcsid[] = "$Id: goPopup.cc,v 1.8 1992/12/29 14:40:24 smadja Exp $";
#endif
/*****************************************************************************
MODULE:  Go Editor
FILE:    goPopup.cc
AUTHOR:  Aline Jacomet
DATE:    August 7th 1991

DESCRIPTION:
    

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include "goMain.h"

#include <Go/Tools/Undo.h>
#include <Go/PointSensor.h>
#include <Go/Rectangle.h>
#include <Go/RegPoly.h>
#include <Go/Bezier.h>
#include <Go/BSpline.h>
#include <Go/Cardinal.h>
#include <Go/Tau.h>
#include <Go/Beta.h>
#include <Go/Circle.h>
#include <Go/Label.h>
#include <Go/Text.h>
#include <Go/SText.h>
#include <Go/SimpleForm.h>

#ifdef GoLICENSE
#include <Go/ScText.h>
#include <Go/MCForm.h>
#endif

#include <Go/Image.h>
#include <Go/BGoUIEd.h>
#include <Go/XGoInterf.h>
#include <Go/FullSensor.h>
#include <Go/Interactor.h>
#include <Go/ChoosePoint.h>

#ifdef GoLICENSE
#define SCROLLEDTEXT_OFFSET 5
#endif

#define SELECTED_PART(text) \
((GoTextual*)text)->firstSelected() != -1 && \
    ((GoTextual*)text)->lastSelected() != -1


/*
 * EXTERN FUNCTIONS
 */

// in file popupmenu.cc;
extern void ActivateToggle(void*);
extern void InitializeObjectName(void*, const char*);
extern void PolySensitiveItems();
extern void PolyUnsensitiveItems();
extern void BezierSensitiveItems();
extern void BezierUnsensitiveItems();
extern void BCardinalSensitiveItems();
extern void BCardinalUnsensitiveItems();
extern void BetaTauSensitiveItems();
extern void BetaTauUnsensitiveItems();
extern void ArcPie(int);
extern void ArcClosed(int);
extern void MapMenu(void*, int, int);

// in file goUndo.cc;
extern void UndoAddEdge(void*);
extern void UndoDeleteEdge(void*);
extern void UndoArcAngle(void*);
extern void UndoRadii(void*);
extern void UndoTextWidth(void*);
extern void UndoTextHeight(void*);
extern void UndoAddPolyPoint(void*);
extern void UndoDeletePolyPoint(void*);
extern void UndoRectResize(void*);
extern void UndoRegPolyResize(void*);
extern void UndoMovePolyPoint(void*);
extern void GoSelectPolypoint(Go*);
extern void GoUnselectPolypoint(Go*);


extern void* reg_menu;
extern void* poly_menu;
extern void* bezier_menu;
extern void* betatau_menu;
extern void* bcardinal_menu;
extern void* small_menu;
extern void* circle_menu;
extern void* arc_menu;
extern void* label_menu;
extern void* text_menu;

#ifdef GoLICENSE
extern void* scroll_menu;
extern void* mctext_menu;
#endif

extern void* image_menu;

extern void* label_presentation[4];
extern void* text_presentation[4];
extern void* text_justification[3];

#ifdef GoLICENSE
extern void* scroll_presentation[4];
extern void* mctext_presentation[4];
extern void* mctext_justification[4];
extern void* mctext_language[7];
#endif

extern char* colorname[15];


static void InitializeLabelMenu();
static void InitializeTextMenu();
#ifdef GoLICENSE
static void InitializeScrolledTextMenu();
#endif

Go* menu_go;

int selected;
int closed;
int pie;

static Go* round_go;
static Go* tension_go;
static Go* bias_go;
static Go* thickness_go;
static Go* position_go;
static Go* space_go;

#ifdef GoLICENSE
static Go* inter_go;
static GoText* column_go;
static GoScrolledText* scroll_go;
static GoScrolledText* size_go;
#endif

#define SELECTED_PART(text) \
((GoTextual*)text)->firstSelected() != -1 && \
    ((GoTextual*)text)->lastSelected() != -1



static void PointSelected(Go* go, int* selected)
{
    Go* son;
    *selected = 0;
    for (son = go->Child(); son; son = son->Next()) 
	if (son->getMetaclass()->isSubclass
	    (GoPointSensor::metaclass())) {
	    *selected = 1;
	    return;
	}
}


void* WhichMenu(int* selected, int* closed, int* pie)
{
    *selected = 0;
    *closed = 0;
    *pie = 0;
    if (menu_go->getMetaclass()->isSubclass(GoRectangle::metaclass())) {
	return small_menu;
    } else
    if (menu_go->getMetaclass()->isSubclass(GoRegPolygon::metaclass())){
	return reg_menu;
    } else
    if (menu_go->getMetaclass()->isSubclass (GoPolyline::metaclass())){
	PointSelected(menu_go, selected);
	return poly_menu;
    } else
    if (menu_go->getMetaclass()->isSubclass (GoBezier::metaclass())){
	PointSelected(menu_go, selected);
	return bezier_menu;
    } else
    if (menu_go->getMetaclass() == GoBetaSpline::metaclass() ||
	menu_go->getMetaclass() == GoTauSpline::metaclass()) {
	PointSelected(menu_go, selected);
	return betatau_menu;
    } else
    if (menu_go->getMetaclass() == GoBSpline::metaclass() ||
	menu_go->getMetaclass() == GoCardinal::metaclass()) {
	PointSelected(menu_go, selected);
	return bcardinal_menu;
    } else
    if (menu_go->getMetaclass() == GoCircle::metaclass() ||
	menu_go->getMetaclass() == GoEllipse::metaclass()){
	return circle_menu;
    } else
    if (menu_go->getMetaclass() == GoArc::metaclass()) {
	if (((GoArc*)menu_go)->closed())
	    *closed = 1;
	if (((GoArc*)menu_go)->pie())
	    *pie = 1;
	return arc_menu;
    } else
    if (menu_go->getMetaclass() == GoLabel::metaclass()) {
	return label_menu;
    } else
#ifdef GoLICENSE	
    if (menu_go->getMetaclass() == GoScrolledText::metaclass()) {
	return scroll_menu;
    } else
    if (menu_go->getMetaclass() == GoText::metaclass() &&
	strcmp(((GoText*)menu_go)->formatter()->getClassname(),
	     GMCFormatter::classname()) == 0) {
	return mctext_menu;
    } else
#endif	
    if (menu_go->getMetaclass() == GoText::metaclass() ||
	menu_go->getMetaclass() == GoSimpleText::metaclass()) {
	return text_menu;
    } else
    if (menu_go->getMetaclass() == GoImage::metaclass()) {
	return image_menu;
    } else
	return NULL;
}


void MapPopupMenu(GoRoot* goroot, const GPoint* p)
{
    menu_go = goroot->goAt(*p, 0.5);
    if (menu_go) {
	void* menu = WhichMenu(&selected, &closed, &pie);
	if (menu) {
	    if (menu == small_menu)
		InitializeObjectName(small_menu,
				     menu_go->getMetaclass()->classname());
	    else
	    if (menu == reg_menu)
		InitializeObjectName(reg_menu,
				     menu_go->getMetaclass()->classname());
	    else
	    if (menu == poly_menu) {
		if (selected)
		    PolySensitiveItems();
		else
		    PolyUnsensitiveItems();
		InitializeObjectName(poly_menu,
				     menu_go->getMetaclass()->classname());
	    } else
	    if (menu == circle_menu) {
		InitializeObjectName(circle_menu,
				     menu_go->getMetaclass()->classname());
	    } else
	    if (menu == bezier_menu) {
		if (selected)
		    BezierSensitiveItems();
		else
		    BezierUnsensitiveItems();
		InitializeObjectName(bezier_menu,
				     menu_go->getMetaclass()->classname());
	    } else
	    if (menu == bcardinal_menu) {
		if (selected)
		    BCardinalSensitiveItems();
		else
		    BCardinalUnsensitiveItems();
		InitializeObjectName(bcardinal_menu,
				     menu_go->getMetaclass()->classname());
	    } else
	    if (menu == betatau_menu) {
		if (selected)
		    BetaTauSensitiveItems();
		else
		    BetaTauUnsensitiveItems();
		InitializeObjectName(betatau_menu,
				     menu_go->getMetaclass()->classname());
	    } else
	    if (menu == arc_menu) {
		if (closed)
		    ArcClosed(0);
		else
		    ArcClosed(1);
		if (pie)
		    ArcPie(0);
		else
		    ArcPie(1);
		InitializeObjectName(arc_menu,
				     menu_go->getMetaclass()->classname());
	    } else
	    if (menu == label_menu) {
		InitializeLabelMenu();
		InitializeObjectName(label_menu,
				     menu_go->getMetaclass()->classname());
	    } else
#ifdef GoLICENSE
	    if (menu == scroll_menu) {
		InitializeScrolledTextMenu();
		InitializeObjectName(scroll_menu,
				     menu_go->getMetaclass()->classname());
	    } else		
	    if (menu == mctext_menu) {
		InitializeTextMenu();
		InitializeObjectName(mctext_menu, "Multi Column GoText");
	    } else
#endif		
	    if (menu == text_menu) {
		InitializeTextMenu();
		if (menu_go->getMetaclass() == GoSimpleText::metaclass())
		    InitializeObjectName(text_menu, "GoSimpleText");
		else
		    InitializeObjectName(text_menu, "Single Column GoText");
	    } else		
	    if (menu == image_menu) {
		InitializeObjectName(image_menu, "GoImage");
	    }
	    int x = (int)(p->x / ((GoInterf*)interf)->pixelWidth());
	    int y = (int)(p->y / ((GoInterf*)interf)->pixelHeight());
	    MapMenu(menu, x, y);
	}
    }
}
                           /*************
			    * GoPolygon *
			    *************/


void GoSelectPolypoint (Go* go)
{
    if (!go->getMetaclass()->isSubclass (GoPolypoint::metaclass()))
	return;

    Go* son;
    for (son = go->Child(); son; son = son->Next())
	if (son->getMetaclass()->isSubclass(GoPointSensor::metaclass()))
	    return;

    GoPointSensor::create("Points", go, NULL, NULL, NULL);
}


void GoUnselectPolypoint (Go* go)
{
    if (!go->getMetaclass()->isSubclass (GoPolypoint::metaclass()))
	return;

    Go* son;
    for (son = go->Child(); son; son = son->Next())
	if (son->getMetaclass()->isSubclass (GoPointSensor::metaclass()))
	{
	    son->destroy();
	    return;
	}
}


void GoSelectUnselectPoint(void*, int* selected)
{
    if (*selected == 1)
	GoUnselectPolypoint(menu_go);
    else
	GoSelectPolypoint(menu_go);
}


static void _choose_polypoint_postcond(void* closure, GoSensor* sensor)
{
    if (sensor->interactor()->state() != GITOR_DONE)
	return;

    GPoint point = ((GChoosePoint*)sensor->interactor())->choosenPoint();
    
    Go* go = goroot->GoAt(point, 0.5);
    GBool ok = 	go &&
	(go == menu_go ||
	 (go->getMetaclass()->isSubclass(GoPointSensor::metaclass()) &&
	  go->parent() == menu_go));

    if (!ok)
	return;

    // call the doit proc;
    (*(void(*)(GPoint&))closure)(point);
}



// ADD PREVIOUS POINT;


static void _do_addpreviouspoint(GPoint& p)
{
    GoPolypoint *polypt = (GoPolypoint*) menu_go;
    int index = polypt->posToIndex(p);
    const GPoint &cur  = polypt->points()[index];
    const GPoint &prev = index ?
	polypt->points()[index - 1] :
    polypt->points()[polypt->nbPoint() - 1];
    UndoPolyPointStruct* str = (UndoPolyPointStruct*)
	GMALLOC(sizeof(UndoPolyPointStruct));
    str->go = polypt;
    str->index = index;
    polypt->addPoint(index,
		     GPoint((cur.x + prev.x)/2, (cur.y + prev.y)/2),
		     GFALSE);
    undoMgr.push(UndoAddPolyPoint, str, (GUndoProc)free);
}

void GoAddPreviousPoint()
{
    // 1er choose a point;
    
  GoFullSensor::create("choosePoint", goroot, NULL,
		       _choose_polypoint_postcond,
		       (void*)&_do_addpreviouspoint);
}

// ADD NEXT POINT;

static void _do_addnextpoint(GPoint& p)
{
    GoPolypoint *polypt = (GoPolypoint*) menu_go;

    int nb = polypt->nbPoint();
    int index = polypt->posToIndex(p);
    const GPoint &cur  = polypt->points()[index];
    const GPoint &next = (index == nb - 1) ?
	polypt->points()[0] : polypt->points()[index + 1];

    UndoPolyPointStruct* str = (UndoPolyPointStruct*)
	GMALLOC(sizeof(UndoPolyPointStruct));
    str->go = polypt;
    str->index = index + 1;
    polypt->addPoint(index + 1,
		     GPoint((cur.x + next.x)/2, (cur.y + next.y)/2),
		     GFALSE);
    undoMgr.push(UndoAddPolyPoint, str, (GUndoProc)free);
    goroot->flush();
}

void GoAddNextPoint()
{
    // 1er choose a point;
    
  GoFullSensor::create("choosePoint", goroot, NULL,
		       _choose_polypoint_postcond,
		       (void*)&_do_addnextpoint);
}

// DELETE POINT;

static void _do_deletepoint(GPoint& p)
{
    UndoPolyPointStruct* str = (UndoPolyPointStruct*)
	GMALLOC(sizeof(UndoPolyPointStruct));

    int index = ((GoPolypoint*)menu_go)->posToIndex(p);
    str->go = menu_go;
    str->index = index;
    str->point = (((GoPolypoint*)menu_go)->points())[index];
    ((GoPolypoint*)menu_go)->delPoint(index);
    undoMgr.push(UndoDeletePolyPoint, str, (GUndoProc)free);
}

void GoDeletePoint()
{
    // 1er choose a point;
    
  GoFullSensor::create("choosePoint", goroot, NULL,
		       _choose_polypoint_postcond,
		       (void*)&_do_deletepoint);
}



/**
FUNCTION
    _edit_postcond
DESCRIPTION
    function called after an edition;
    The closure is the wanted undo struct.
    Note that the undo struct contains the undo procedure
**/

static void _edit_postcond(void* closure, GoSensor* sensor)
{
    _UndoStruct* undo = (_UndoStruct*)closure;

    if (sensor->interactor()->state() != GITOR_DONE)
    {
	delete undo;
	return;
    }

    // push undo of the action;
    undoMgr.push(undo->proc, undo, (GUndoProc)free);
}

#include <Go/EdPolypoint.h>

/**
FUNCTION
    MoveClosestPolyPoint(Go* go, const GEvent* event)
DESCRIPTION
    Function called by Translator. sould be placed in goGoUI.cc
    But it uses static function of goPopup.cc
    It is better to put it here.
    
**/
void MoveClosestPolyPoint(Go* go, const GEvent* event)
{
    // the program MovePolypoint is created by BGoUIEditor;

    UndoPolyPointStruct* str = new UndoPolyPointStruct;
    str->go = go;
    str->proc = UndoMovePolyPoint;
    
    GoSensor* sensor =  GoFullSensor::create("movePoint", go,  event,
					     _edit_postcond, str);

    if (sensor->interactor() == NULL)
    {
	// the go is not subclass of Polypoint;
	delete str;
	sensor->destroy();
	return;
    }

    // init the undo struct;
    // by reading the edit polypoint grabber;
    
    GEditPolypoint* grabber = (GEditPolypoint*)sensor->interactor();
    // the grabber has decided to move the popint at index;
    str->index = grabber->currentPointIndex();
    str->point = (((GoPolypoint*)go)->points())[str->index];
}
    
                           /************************
			    * GoRectangle, GoImage *
			    ************************/

void GoResizeRect()
{
    UndoRectStruct* str = (UndoRectStruct*)GMALLOC(sizeof(UndoRectStruct));
    str->go = menu_go;
    str->proc =UndoRectResize;

    if (str->go->getMetaclass()->isSubclass(GoRectangle::metaclass()))
    {
	str->width = ((GoRectangle*)str->go)->width();
	str->height = ((GoRectangle*)str->go)->height();

    }
    else if (str->go->getMetaclass()->isSubclass(GoImage::metaclass())) {
	str->width = ((GoImage*)str->go)->width();
	str->height = ((GoImage*)str->go)->height();
    }

  GoFullSensor::create("edit", str->go, NULL, _edit_postcond, str);
}




                           /****************
			    * GoRegPolygon *
			    ****************/



void GoAddEdge()
{
    ((GoRegPolygon*)menu_go)->
	nbPoint(((GoRegPolygon*)menu_go)->GoPolypoint::nbPoint() + 1);
    undoMgr.push(UndoAddEdge, menu_go);
}

void GoDeleteEdge()
{
    if (((GoRegPolygon*)menu_go)->
	nbPoint(((GoRegPolygon*)menu_go)->GoPolypoint::nbPoint() - 1))
	undoMgr.push(UndoDeleteEdge, menu_go);
}

void GoResizeRegPoly()
{
    UndoRegPolyStruct* str = (UndoRegPolyStruct*)
	GMALLOC(sizeof(UndoRegPolyStruct));
    str->go = (GoRegPolygon*)menu_go;
    str->proc =UndoRegPolyResize;
    
	
    /*GB :
     * so far, regPolygonResize does not modify the number of summits.
     * This could be changed in the future, thus it might be a good idea to
     * store go->GoPolypoint::nbPoint() at this point for undo purposes.
     */
    str->angle = ((GoRegPolygon*)str->go)->angle();
    str->radius = ((GoRegPolygon*)str->go)->radius();

  GoFullSensor::create("edit", str->go, NULL, _edit_postcond, str);
}


void GoChangeRoundOffset(float offset)
{
    ((GoPolyline*)round_go)->roundOffset(offset);
}


float GoInitializeRoundOffset()
{
    // round_go is used because menu_go can be changed between the call of
    // GoInitializeRoundOffset and the call of GoChangeRoundOffset;	
    round_go = menu_go;
    ((GoPolyline*)menu_go)->round(GTRUE);
    return ((GoPolyline*)menu_go)->roundOffset();
}




                           /*********************************
			    * GoArc, GoEllipse and GoCircle *
			    *********************************/


void GoChangeRadii()
{
    UndoRadiiStruct* str = (UndoRadiiStruct*)
	GMALLOC(sizeof(UndoRadiiStruct));
    str->go = menu_go;
    str->proc = UndoRadii;
    str->radiusX = ((GoArc*)menu_go)->radiusX();
    str->radiusY = ((GoArc*)menu_go)->radiusY();

    // the edit program will modify the radius like circle ;
    // or ellipse depending the metaclass;
  GoFullSensor::create("edit", str->go, NULL, _edit_postcond, str);
    
}


static void _change_angle(void* /* null */, GoSensor* sensor);

void GoChangeAngle()
{

    // 1 first choose a point;
    (void)GoFullSensor::create("choosePoint", goroot, NULL,
			       _change_angle, NULL);
}

static void _change_angle(void* /* null */, GoSensor* sensor)
{
    if (sensor->interactor()->state() != GITOR_DONE)
    {
	// user dis cancel the choosepoint;
	return;
    }
    
    UndoArcPointStruct* str = (UndoArcPointStruct*)
	GMALLOC(sizeof(UndoArcPointStruct));
    str->go = menu_go;
    str->proc = UndoArcAngle;
    str->start = ((GoArc*)menu_go)->start();
    str->amplitude = ((GoArc*)menu_go)->amplitude();

  GoFullSensor::create("movePoint", str->go, NULL, _edit_postcond, str);
}


void GoChangePie(void*, int* pie)
{
    if (*pie == 1)
	((GoArc*)menu_go)->pie(GFALSE);
    else
	((GoArc*)menu_go)->pie(GTRUE);
}


void GoChangeClosed(void*, int* closed)
{
    if (*closed == 1)
	((GoArc*)menu_go)->closed(GFALSE);
    else
	((GoArc*)menu_go)->closed(GTRUE);
}



                            /***********
			     * GoCubic *
			     ***********/



void GoChangeTensionOffset(float offset)
{
    ((GoCubic*)tension_go)->tension(offset);
}


float GoInitializeTensionOffset()
{
    // tension_go is used because menu_go can be changed between the call of
    // GoInitializeTensionOffset and the call of GoChangeTensionOffset;	
    tension_go = menu_go;
    return ((GoCubic*)menu_go)->tension();
}


void GoChangeBiasOffset(float offset)
{
    if (bias_go->getMetaclass() == GoBetaSpline::metaclass())
	((GoBetaSpline*)bias_go)->bias(offset);
    else
	((GoTauSpline*)bias_go)->bias(offset);
}


float GoInitializeBiasOffset()
{
    // bias_go is used because menu_go can be changed between the call of
    // GoInitializeBiasOffset and the call of GoChangeBiasOffset;	
    bias_go = menu_go;
    if (bias_go->getMetaclass() == GoBetaSpline::metaclass())
	return ((GoBetaSpline*)menu_go)->bias();
    else
	return ((GoTauSpline*)menu_go)->bias();
}




                            /*************
			     * GoTextual *
			     *************/



void GoUnderlined(void*, int yes)
{
    GPos first, last;
    first = ((GoTextual*)menu_go)->firstSelected();
    last = ((GoTextual*)menu_go)->lastSelected();
    if (first == -1){
	first = 0;
	last = ((GoTextual*)menu_go)->length();
    }
    ((GoTextual*)menu_go)->underline((GBool)yes, first, last);
}


void GoChangeUnderlineThickness(float thick)
{
    if (menu_go->getMetaclass()->isSubclass(GoText::metaclass()) &&
	SELECTED_PART(menu_go)) {
	GPenValues values;
	ulong mask = GLINE_WIDTH;
	values.line_width = thick;
	((GoText*)menu_go)->textUnderlinePen(mask, &values,
					  ((GoText*)menu_go)->firstSelected(),
					  ((GoText*)menu_go)->lastSelected());
    } else
	((GoTextual*)thickness_go)->underlinePen().lineWidth(thick);
}


float GoInitializeUnderlineThickness()
{
    // thickness_go is used because menu_go can be changed between the call of
    // GoInitializeUnderlineThickness and the call of
    // GoChangeUnderlineThickness;	
    thickness_go = menu_go;
    return ((GoTextual*)menu_go)->underlinePen().lineWidth();
}


void GoChangeUnderlinePosition(float pos)
{
    if (position_go->getMetaclass()->isSubclass(GoText::metaclass()) &&
	SELECTED_PART(position_go))
	((GoTextual*)position_go)->underlinePosition(pos,
				((GoTextual*)position_go)->firstSelected(),
				((GoTextual*)position_go)->lastSelected());
    else
	((GoTextual*)position_go)->underlinePosition(pos);
}


float GoInitializeUnderlinePosition()
{
    // position_go is used because menu_go can be changed between the call of
    // GoInitializeUnderlinePosition and the call of GoChangeUnderlinePosition;
    position_go = menu_go;
    return ((GoTextual*)menu_go)->underlinePosition();
}


void GoChangeSpaceLine(float spc)
{
    ((GoTextual*)space_go)->interline(spc);
}


float GoInitializeSpaceLine()
{
    // space_go is used because menu_go can be changed between the call of
    // GoInitializeSpaceLine and the call of GoChangeSpaceLine;
    space_go = menu_go;
    return ((GoTextual*)menu_go)->interline();
}

#ifdef GoLICENSE
void GoChangeInterColumn(float spc)
{
    ((GMCFormatterCtxt*)((GoText*)inter_go)->formatter()->ctxt())->interColumn(spc);
}

float GoInitializeInterColumn()
{
    // inter_go is used because menu_go can be changed between the call of
    // GoInitializeInterColumn and the call of GoChangeInterColumn;
    inter_go = menu_go;
    return ((GMCFormatterCtxt*)((GoText*)inter_go)->formatter()->ctxt())->interColumn();
}
#endif


void GoChangePresentation(void*, int pre)
{
    switch(pre) {
      case 0:
	((GoTextual*)menu_go)->presentation(GNO_PRESENTATION);
	break;
      case 1:
	((GoTextual*)menu_go)->presentation(GLOWER_PRESENTATION);
	break;
      case 2:
	((GoTextual*)menu_go)->presentation(GUPPER_PRESENTATION);
	break;
      case 3:
	((GoTextual*)menu_go)->presentation(GTITLE_PRESENTATION);
	break;
    }
}


void GoChangeJustification(void*, int jus)
{
    switch(jus) {
      case 0:
	((GoTextual*)menu_go)->justification(GLEFT_JUSTIFICATION);
	break;
      case 1:
	((GoTextual*)menu_go)->justification(GCENTER_JUSTIFICATION);
	break;
      case 2:
	((GoTextual*)menu_go)->justification(GRIGHT_JUSTIFICATION);
	break;
      case 3:
	((GoTextual*)menu_go)->justification(GLR_JUSTIFICATION);
	break;
    }
}

#ifdef GoLICENSE
void GoChangeLanguage(void*, int lang)
{
    switch(lang) {
      case 0:
	((GMCFormatterCtxt*)((GoText*)menu_go)->formatter()->ctxt())->language(GENGLISH);
	break;
      case 1:
	((GMCFormatterCtxt*)((GoText*)menu_go)->formatter()->ctxt())->language(GFRENCH);
	break;
      case 2:
	((GMCFormatterCtxt*)((GoText*)menu_go)->formatter()->ctxt())->language(GGERMAN);
	break;
      case 3:
	((GMCFormatterCtxt*)((GoText*)menu_go)->formatter()->ctxt())->language(GSPANISH);
	break;
      case 4:
	((GMCFormatterCtxt*)((GoText*)menu_go)->formatter()->ctxt())->language(GITALIAN);
	break;
      case 5:
	((GMCFormatterCtxt*)((GoText*)menu_go)->formatter()->ctxt())->language(GPORTUGUESE);
	break;
      case 6:
	((GMCFormatterCtxt*)((GoText*)menu_go)->formatter()->ctxt())->language(GNO_LANGUAGE);
	break;
    }
}
#endif


static void InitializeLabelMenu()
{
    switch(((GoTextual*)menu_go)->presentation()) {
      case GNO_PRESENTATION:
	ActivateToggle(label_presentation[0]);
	break;
      case GLOWER_PRESENTATION:
	ActivateToggle(label_presentation[1]);
	break;
      case GUPPER_PRESENTATION:
	ActivateToggle(label_presentation[2]);
	break;
      case GTITLE_PRESENTATION:
	ActivateToggle(label_presentation[3]);
	break;
    }
}


#ifdef GoLICENSE
static void InitializeScrolledTextMenu()
{
    switch(((GoTextual*)menu_go)->presentation()) {
      case GNO_PRESENTATION:
	ActivateToggle(scroll_presentation[0]);
	break;
      case GLOWER_PRESENTATION:
	ActivateToggle(scroll_presentation[1]);
	break;
      case GUPPER_PRESENTATION:
	ActivateToggle(scroll_presentation[2]);
	break;
      case GTITLE_PRESENTATION:
	ActivateToggle(scroll_presentation[3]);
	break;
    }
}
#endif


static void InitializeTextMenu()
{
#ifdef GoLICENSE    
    if (! (menu_go->getMetaclass()->isSubclass(GoSimpleText::metaclass())) &&
	strcmp(((GoText*)menu_go)->formatter()->getClassname(),
	     GMCFormatter::classname()) == 0) {
	switch(((GoTextual*)menu_go)->presentation()) {
	  case GNO_PRESENTATION:
	    ActivateToggle(mctext_presentation[0]);
	    break;
	  case GLOWER_PRESENTATION:
	    ActivateToggle(mctext_presentation[1]);
	    break;
	  case GUPPER_PRESENTATION:
	    ActivateToggle(mctext_presentation[2]);
	    break;
	  case GTITLE_PRESENTATION:
	    ActivateToggle(mctext_presentation[3]);
	    break;
	}
	switch(((GoTextual*)menu_go)->justification()) {
	  case GLEFT_JUSTIFICATION:
	    ActivateToggle(mctext_justification[0]);
	    break;
	  case GCENTER_JUSTIFICATION:
	    ActivateToggle(mctext_justification[1]);
	    break;
	  case GRIGHT_JUSTIFICATION:
	    ActivateToggle(mctext_justification[2]);
	    break;
	  case GLR_JUSTIFICATION:
	    ActivateToggle(mctext_justification[3]);
	    break;
	}
	switch(((GMCFormatterCtxt*)((GoText*)menu_go)->formatter()->ctxt())->language()) {
	  case GENGLISH:
	    ActivateToggle(mctext_language[0]);
	    break;
	  case GFRENCH:
	    ActivateToggle(mctext_language[1]);
	    break;
	  case GGERMAN:
	    ActivateToggle(mctext_language[2]);
	    break;
	  case GSPANISH:
	    ActivateToggle(mctext_language[3]);
	    break;
	  case GITALIAN:
	    ActivateToggle(mctext_language[4]);
	    break;
	  case GPORTUGUESE:
	    ActivateToggle(mctext_language[5]);
	    break;
	  case GNO_LANGUAGE:
	    ActivateToggle(mctext_language[6]);
	    break;
	}
    } else {
#endif	
	switch(((GoTextual*)menu_go)->presentation()) {
	  case GNO_PRESENTATION:
	    ActivateToggle(text_presentation[0]);
	    break;
	  case GLOWER_PRESENTATION:
	    ActivateToggle(text_presentation[1]);
	    break;
	  case GUPPER_PRESENTATION:
	    ActivateToggle(text_presentation[2]);
	    break;
	  case GTITLE_PRESENTATION:
	    ActivateToggle(text_presentation[3]);
	    break;
	}
	switch(((GoTextual*)menu_go)->justification()) {
	  case GLEFT_JUSTIFICATION:
	    ActivateToggle(text_justification[0]);
	    break;
	  case GCENTER_JUSTIFICATION:
	    ActivateToggle(text_justification[1]);
	    break;
	  case GRIGHT_JUSTIFICATION:
	    ActivateToggle(text_justification[2]);
	    break;
	  case GLR_JUSTIFICATION:
	    break;
	}
#ifdef GoLICENSE	
    }
#endif    
}


void GoChangeUnderlineColor(void*, int i)
{
    GShColor* c = interf->colorFromName(colorname[i]);
    if (menu_go->getMetaclass()->isSubclass(GoText::metaclass()) &&
	SELECTED_PART(menu_go)) {
	GPenValues values;
	ulong mask = GCOLOR;
	values.color = c;
	((GoText*)menu_go)->textUnderlinePen(mask, &values,
					  ((GoText*)menu_go)->firstSelected(),
					  ((GoText*)menu_go)->lastSelected());
    } else
	((GoTextual*)menu_go)->underlinePen().color(c);
    c->destroy();
}


void GoChangeCursorColor(void*, int i)
{
    GShColor* c = interf->colorFromName(colorname[i]);
    ((GoTextual*)menu_go)->cursorPen().color(c);
    c->destroy();
}


void GoChangeSelectionColor(void*, int i)
{
    GShColor* c = interf->colorFromName(colorname[i]);
    ((GoTextual*)menu_go)->selectionBrush().color(c);
    c->destroy();
}


#ifdef GoLICENSE
void GoInitializeScroll()
{
    // scroll_go is used because menu_go can be changed between the call of
    // GoInitializeScroll and the call of GoScrollUp, GoScrollDown,
    // GoScrollLeft or GoScrollRight;
    scroll_go = (GoScrolledText*)menu_go;
}


void GoScrollUp()
{
    scroll_go->scrollUp();
}


void GoScrollDown()
{
    scroll_go->scrollDown();
}


void GoScrollLeft()
{
    GNumber scrollX = scroll_go->hScroll() - SCROLLEDTEXT_OFFSET;
    if (scrollX >= 0)
	scroll_go->hScroll(scrollX);
    else
	scroll_go->hScroll(0);
}


void GoScrollRight()
{
    GNumber scrollX = scroll_go->hScroll() + SCROLLEDTEXT_OFFSET;
    if (scrollX <= scroll_go->textWidth())
	scroll_go->hScroll(scrollX);
}


void GoInitializeSize()
{
    // size_go is used because menu_go can be changed between the call of
    // GoInitializeSize and the call of GoWidth+, GoWidth-,
    // GoHeight+ or GoHeight-;
    size_go = (GoScrolledText*)menu_go;
}


void GoWidthPlus()
{
    GNumber width = ((GoTextual*)size_go)->width() + SCROLLEDTEXT_OFFSET;
    size_go->width(width);
}


void GoWidthMinus()
{
    GNumber width = ((GoTextual*)size_go)->width() - SCROLLEDTEXT_OFFSET;
    if (width > 0)
	size_go->width(width);
}


void GoHeightPlus()
{
    ushort nbLines = size_go->nbLinesToShow() + 1;
    size_go->nbLinesToShow(nbLines);
}


void GoHeightMinus()
{
    ushort nbLines = size_go->nbLinesToShow() - 1;
    if (nbLines > 0)
	size_go->nbLinesToShow(nbLines);
}


void GoInitializeNbColumn()
{
    // column_go is used because menu_go can be changed between the call of
    // GoInitializeNbColumn and the call of GoNbColumnPlus, GoNbColumnMinus,
    column_go = (GoText*)menu_go;
}


void GoNbColumnPlus()
{
    int column = ((GMCFormatterCtxt*)column_go->formatter()->ctxt())->numberOfColumns() + 1;
    ((GMCFormatterCtxt*)column_go->formatter()->ctxt())->numberOfColumns(column);
}


void GoNbColumnMinus()
{
    int column = ((GMCFormatterCtxt*)column_go->formatter()->ctxt())->numberOfColumns() - 1;
    if (column > 0)
	((GMCFormatterCtxt*)column_go->formatter()->ctxt())->numberOfColumns(column);
}


void GoResizeTextWidth()
{
/*    UndoTextSizeStruct* str = (UndoTextSizeStruct*)
	GMALLOC(sizeof(UndoTextSizeStruct));
    str->go = menu_go;
    str->size = ((GoText*)menu_go)->width();
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (menu_go->root());
    bgoui->resizeMCTextWidth((GoText*)menu_go, GTRUE);
    undoMgr.push(UndoTextWidth, str, (GUndoProc)free);
*/
    GWarning ("godraw::GoResizeTextWidth(): not implemented yet!");
}


void GoResizeTextHeight()
{
/*    UndoTextSizeStruct* str = (UndoTextSizeStruct*)
	GMALLOC(sizeof(UndoTextSizeStruct));
    str->go = menu_go;
    str->size = ((GMCFormatterCtxt*)((GoText*)menu_go)->formatter()->ctxt())->height();
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (menu_go->root());
    bgoui->resizeMCTextHeight((GoText*)menu_go, GTRUE);
    undoMgr.push(UndoTextHeight, str, (GUndoProc)free);
*/
    GWarning ("godraw::GoResizeTextHeight(): not implemented yet!");
}
#endif
