#ifdef REV_INFO
static char rcsid[] = "$Id: goSelection.cc,v 1.4 1992/12/10 16:24:40 jojic Exp $";
#endif
/*****************************************************************************
MODULE:  Go Editor
FILE:    goSelection.cc
AUTHOR:  Aline Jacomet
DATE:    May 14th 1991

DESCRIPTION:
    

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include "goMain.h"
#include <sys/types.h>

#include <Go/Rectangle.h>
#include <Go/BGoUIEd.h>
#include <Go/Textual.h>
#include <Go/TextSelPtn.h>
#include <Go/Sensor.h>

// in goEdition.cc:
extern void CancelGrabbingSensors (BGoUIEditor*);


/*
 * FUNCTIONS
 */

static void _select(void* closure, Go* rect)
{
    if (!rect)
	return;
    
    GBox box = rect->clip().bigbox();
    rect->destroy();

    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
    if (closure)
	bgoui->selectAllInBox (box);
    else
	bgoui->unselectAllInBox(box);
}


void RubberBandSelect (void*, void*, caddr_t)
{
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
    CancelGrabbingSensors (bgoui);
    
    bgoui->createRectangle (_select, (void*)GTRUE);
}

void RubberBandUnselect(void*, void*, caddr_t)
{
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
    CancelGrabbingSensors (bgoui);
    
    bgoui->createRectangle(_select, (void*)GFALSE);
}


void SelectLower (void*, void*, caddr_t)
{
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
    CancelGrabbingSensors (bgoui);
    
    bgoui->selectChildren ();
}


void SelectUpper (void*, void*, caddr_t)
{
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
    CancelGrabbingSensors (bgoui);
    
    bgoui->selectParent ();
}


void godraw_select_all_proc (void*, Go* go)
{
    if (go != goroot)
	go->select();
}

void SelectAll (void*, void*, caddr_t)
{
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
    CancelGrabbingSensors (bgoui);
    
    goroot->forEveryGo (godraw_select_all_proc, NULL);
}


/**
FUNCTION
    TextSelectPortion(GoTextual*, const GEvent*)
DESCRIPTION
    The selectPortion interactor handle multiple protion selection.
    In the current godraw: multiple portion are not handled
    So free the provius protion is any and reun the interactor.
    
**/

void TextSelectPortion(GoTextual* textual, const GEvent* event)
{
    if (textual == NULL ||
	!textual->getMetaclass()->isSubclass(GoTextual::metaclass()))
	return;

    static GBool done = GFALSE;
    if (!done)
    {
	(void)GTextSelectPortionPgm::create();
	done = GTRUE;
    }
    
    // limit the number of selected portion to 1;
    // no multiple portion;
    // deselect previous portion (if any);

    textual->selectPortion (-1, -1, GFALSE);

    // call the interactor to select a new one;
    (void)GoSensor::create("selectPortion", textual, event, NULL, NULL);
}
