#ifdef REV_INFO
static char rcsid[] = "$Id: goStructure.cc,v 1.4 1992/12/17 15:25:52 jojic Exp $";
#endif
/*****************************************************************************
MODULE:  Go Editor
FILE:    goStructure.cc
AUTHOR:  Aline Jacomet
DATE:    May 14th 1991

DESCRIPTION:
    

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include <stdio.h>

#include "goMain.h"

#include <Go/Tools/Assoc.h>
#include <Go/Tools/Undo.h>
#include <Go/Composite.h>
#include <Go/BGoUIEd.h>
#include <Go/XGoInterf.h>

extern GAssoc key_assoc;


/*
 * FUNCTIONS
 */

// in file message.cc;
extern void TooManySelectedObjects (void*);
extern void NoSelectedObject (void*);

// in file goUndo.cc;
extern void UndoReparent(void*);
extern void UndoCurrentParent(void*);
extern void UndoSelectOne(void*);
extern void UndoUnselectOne(void*);
extern void UndoRaise(void*);
extern void UndoLower(void*);
extern void UndoLeaf(void*);
extern void UndoUnleaf(void*);
extern void UndoGroup(void*);
extern void UndoUngroup(void*);
extern void UndoDestroy(void*);
extern void FreeObj(void*);

// in file goGoUI.cc;
extern void undo_unselect(void*, Go*);

//in file goEdition.cc:
extern void CancelGrabbingSensors (BGoUIEditor*);




                          /******************
			   * CURRENT PARENT *
			   ******************/


void CurParent(void*, void* main_window, caddr_t)
{
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
    CancelGrabbingSensors (bgoui);
    
    GQueue q;
    bgoui->enqueueAllSelectedGos (q);
    
    switch (q.length())
    {
      case 0:
	bgoui->currentParent (goroot);
	break;
      case 1:
	bgoui->currentParent ((Go*)q.getfirst());
	break;
      default:
	TooManySelectedObjects (main_window);
	break;
    }
}



                          /************
			   * REPARENT *
			   ************/


void Reparent(void*, void*, caddr_t)
{
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
    CancelGrabbingSensors (bgoui);
    
    GQueue q;
    bgoui->enqueueAllSelectedGos (q);

    if (q.length() == 0)
	NoSelectedObject (main_window);
    else
    {
	Go* parent = bgoui->currentParent();

	undoMgr.openMark();
	while (!q.empty())
	{
	    Go* go = (Go*) q.getfirst();
	    if (go != parent && !go->isAncestor (parent))
	    {
		UndoReparentStruct* str =
		    (UndoReparentStruct*) GMALLOC (sizeof (UndoReparentStruct));
		str->go = go;
		str->parent = go->parent();
		str->next = go->next();
		go->reparent (parent);
		undoMgr.push (UndoReparent, str, (GUndoProc) free);
	    }
	}
	undoMgr.closeMark();
    }
}


const char* godrawApplicationComposite = "godrawApplicationComposite";

void HandleComposite(void*, Go* go)
{
    if (go->getMetaclass() == GoComposite::metaclass())
    {
	// the address of the godrawApplicationComposite array gives;
	// an unique key;
	go->assoc()->set(godrawApplicationComposite, (void*)GTRUE);
    }
}


static const char* crossTypeKey = "crossTypeKey";

void godraw_restore_composite (void*, Go* composite, Go*)
{
    composite->addChildPostcond (godraw_restore_composite, NULL, GREMOVE);
    long ctyp = (long) composite->assoc()->get (crossTypeKey);
    composite->crossType ((GCrossType) ctyp);
}

void HandleEmptyComposite (Go* composite)
{
    if (composite->assoc()->get (godrawApplicationComposite) != NULL)
    {
	/* see the comment in Editor2.cc:handle_empty_composite() */
	long ctyp = (long) composite->crossType();
	composite->assoc()->set (crossTypeKey, (void*) ctyp);
	composite->addChildPostcond (godraw_restore_composite, NULL);
	composite->crossType (GCROSS_AS_NOTHING);
    }
}

                          /*********
			   * GROUP *
			   *********/


void MyGroup (void* closure)
{
    GQueue* q = (GQueue*) closure;

    GoComposite* comp = (GoComposite*) q->getlast();
    Go* next = (Go*) q->getlast();
    Go* parent = (Go*) q->getlast();
    Go* nextBrother = NULL;
    while (! q->empty())
    {
	Go* go = (Go*) q->getlast();

	go->reparent (comp, nextBrother);
	go->unselect();
	nextBrother = go;
    }
    comp->select();
    comp->reparent (parent, next);
    /* this comp is, again, visible: */
    comp->assoc()->add (godrawApplicationComposite, (void*) GTRUE);

    delete q;
    extern void MyUngroup (void*);
    undoMgr.push (MyUngroup, comp);
}


GBool RefuseDestroy (void*, const Go*)
{
    return GFALSE;
}


void FreeQueueAndDestroyComp (void* closure)
{
    GQueue* q = (GQueue*) closure;
    GoComposite* comp = (GoComposite*) q->getlast();
    comp->destroyPrecond (RefuseDestroy, NULL, GREMOVE);
    comp->destroy();
    delete q;
}


void BGouiUngroup (void* closure)
{
    GoComposite* comp = (GoComposite*) closure;
    
    comp->crossType (GCROSS_AS_NODE);

    GQueue* q = new GQueue;
    for (Go* child = comp->child(); child; child = child->next())
    {
	q->putlast (child);
	child->select ();
    }
    q->putlast (comp->parent());
    q->putlast (comp->next());
    q->putlast (comp);
    
    /* this comp will no more be visible: */
    comp->assoc()->rem (godrawApplicationComposite);

    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (comp->root());
    bgoui->ungroup (comp);

    undoMgr.push (MyGroup, q, FreeQueueAndDestroyComp);
}


void Group (void*, void*, caddr_t) 
{
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
    CancelGrabbingSensors (bgoui);
    
    GQueue q;
    bgoui->enqueueAllSelectedSubtrees (q);
    if (q.length() <= 1)
	NoSelectedObject (main_window);
    else
    {
	/* bgoui group: */
	
	GoComposite* comp = bgoui->groupAllSelectedSubtrees();
	bgoui->unselectAll();
	comp->assoc()->set (godrawApplicationComposite, (void*) GTRUE);
	comp->destroyPrecond (RefuseDestroy, NULL);
	comp->select();
	undoMgr.push (BGouiUngroup, comp);
    }
}    


                          /***********
			   * UNGROUP *
			   ***********/


void MyUngroup (void* closure)
{
    GoComposite* comp = (GoComposite*) closure;

    comp->crossType (GCROSS_AS_NODE);
    GQueue* q = new GQueue;
    for (Go* child = comp->child(); child; child = child->next())
	q->putlast (child);
    q->putlast (comp->parent());
    q->putlast (comp->next());
    q->putlast (comp);
    
    /* this comp is no more visible: */
    comp->assoc()->rem (godrawApplicationComposite);

    Go* next;
    for (child = comp->child(); child; child = next)
    {
	next = child->next();
	child->reparent (comp->parent(), comp->next());
	child->select();
    }

    comp->reparent (NULL);
    undoMgr.push (MyGroup, q, FreeQueueAndDestroyComp);
}


void Ungroup (void*, void*, caddr_t)
{
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
    CancelGrabbingSensors (bgoui);

    GQueue q;
    bgoui->enqueueAllSelectedGos (q);
    if (q.empty())
    {
	NoSelectedObject (main_window);
	return;
    }

    undoMgr.openMark();

    GBool somethingDone = GFALSE;
    while (!q.empty())
    {
	Go* comp = (Go*) q.getfirst();

	if (comp->getMetaclass() == GoComposite::metaclass())
	{
	    somethingDone = GTRUE;
	    MyUngroup (comp);
	}
    }

    undoMgr.closeMark();

    if (!somethingDone)
    {
	undoMgr.removeOneUndo ();
	NoSelectedObject (main_window);
    }    
}



                          /*********
			   * LEAF *
			   *********/

static void UnselectAllChildren(Go* go)
{
    for (Go* child = go->child(); child; child = child->next())
	UnselectAllChildren(child);
    go->unselect();
    undoMgr.push(UndoUnselectOne, go);
}
    

static void LeafProc (void*, Go* go)
{
    UnselectAllChildren(go);
    go->leaf(GTRUE);
    undoMgr.push(UndoLeaf, go);
    go->select();
    undoMgr.push(UndoSelectOne, go);
}


void Leaf(void*, void*, caddr_t)
{
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
    CancelGrabbingSensors (bgoui);

    undoMgr.openMark();
    goroot->forEverySelectedGo(LeafProc, NULL);
    undoMgr.closeMark();
}



                          /**********
			   * UNLEAF *
			   **********/


static void UnleafProc (void*, Go* go)
{
    go->leaf(GFALSE);
    undoMgr.push(UndoUnleaf, go);
}


void Unleaf(void*, void*, caddr_t)
{
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
    CancelGrabbingSensors (bgoui);

    undoMgr.openMark();
    goroot->forEverySelectedGo(UnleafProc, NULL);
    undoMgr.closeMark();
}



                          /*****************
			   * RAISE / LOWER *
			   *****************/


// CREATE A SPECIFIC INTERACTOR;
// we do not make a Program in a same way as;
// in the library because we do not to verify;
// application multiple creation etc...;

#include <Go/ChoosePoint.h>

/**
FUNCTION
    RaiseLower Interactor definition
DESCRIPTION
    The Raise Lower interactor is a syubclass of the GChhosePoint
    interactor. It gives the information that the user has clicked
    on BUTTON1 for raise and #define UNDEF 0
    BUTTON 3 or <esc> cancels the interactor.
**/

#define UNDEF 0
#define RAISE 1
#define LOWER 2
    

/**
FUNCTION
    RaiseLower Program definition
DESCRIPTION
   it is needed: the sensor finds the interactor via;
   the name of the program;
   the sensor will call the start of the founded program;
   to run the effective interactor;
**/

class _RaiseLowerChoosePointPgm : public GChoosePointPgm
{
  public:
    _RaiseLowerChoosePointPgm(const char* program_name);
    GInteractor* start(const GEvent*, Go*);
};


class _RaiseLowerChoosePoint : public GChoosePoint
{
    // we just have to reimplement isDoitEvent();
    // the cancel is the superclass one: BUTTON3 or <esc>;

  public:
    _RaiseLowerChoosePoint (_RaiseLowerChoosePointPgm*, const GEvent*, Go*);
    GBool isDoitEvent(const GEvent*) const;
    int user_want;
};

_RaiseLowerChoosePoint::
    _RaiseLowerChoosePoint(_RaiseLowerChoosePointPgm* pgm,
			   const GEvent* event, Go* go)
: GChoosePoint(pgm, event, go)
{
    user_want = UNDEF;
}

GBool _RaiseLowerChoosePoint::isDoitEvent(const GEvent* event) const
{
    if (event->type() == GBUTTON_PRESS_EVENT &&
	(event->button() == GBUTTON1 || event->button() == GBUTTON2))
    {
	if (event->button() == GBUTTON1)
	    ((_RaiseLowerChoosePoint*)this)->user_want = RAISE;
	else
	    ((_RaiseLowerChoosePoint*)this)->user_want = LOWER;
	return GTRUE;
    }
    
    return GFALSE;
}


_RaiseLowerChoosePointPgm::_RaiseLowerChoosePointPgm(const char* name)
    : GChoosePointPgm(name)
{
    // the name will be binded by the GChoosePointPgm;
    // so sensor will known this program;
}


GInteractor* _RaiseLowerChoosePointPgm::start(const GEvent* event,
					    Go* go)
{
    GInteractor* itor = new _RaiseLowerChoosePoint(this, event, go);
    Initialize(itor);
    return itor;
}


/**
FUNCTION
    Using the RaiseAndLower interactor.
DESCRIPTION
    RaiseAndLower function runs the RaiseLower interactor.
    When it terminates the 'raise_lower_proc' is called.
    If user selects RAISE: raise the go which is at mouse position;
    If user selects LOWER: lower the go which is at mouse position;
**/

#include <Go/Sensor.h>
#include <Go/FullSensor.h>


void raise_lower_proc(void* /*closure*/, GoSensor* sensor)
{
    _RaiseLowerChoosePoint* itor =
	(_RaiseLowerChoosePoint*)sensor->interactor();

    if (itor->state() != GITOR_DONE)
	return;

    GPoint position = itor->choosenPoint();
    Go* go = goroot->goAt(position, 0.5);
    if (!go)
	return;
    
    if (itor->user_want == RAISE)
    {
	go->raise();
	undoMgr.push(UndoRaise, go);
    }
    else
    {
	go->lower();
	undoMgr.push(UndoLower, go);
    }
}

void RaiseAndLower()
{
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
    CancelGrabbingSensors (bgoui);

    static GBool _makepgm = GTRUE;
    const char* godraw_raise_lower = "godraw_raise_lower";

    if (_makepgm)
    {
	_makepgm = GFALSE;
	char name[128];
	sprintf(name, "%s%s", "Go.", godraw_raise_lower);
	(void)new _RaiseLowerChoosePointPgm(name);
    }
    (void)GoFullSensor::create(godraw_raise_lower, goroot, NULL,
			       raise_lower_proc, NULL);
}

