#ifdef REV_INFO
static char rcsid[] = "$Id: goTools.cc,v 1.5 1992/12/10 16:24:43 jojic Exp $";
#endif
/*****************************************************************************
MODULE:  Go Editor
FILE:    goTools.cc
AUTHOR:  Aline Jacomet
DATE:    August 12th 1991

DESCRIPTION:
    

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include <stdio.h>

#include "goMain.h"

#include <Go/Tools/Atom.h>

#include <Go/Rectangle.h>
#include <Go/Circle.h>
#include <Go/RegPoly.h>
#include <Go/Bezier.h>
#include <Go/Beta.h>
#include <Go/Tau.h>
#include <Go/BSpline.h>
#include <Go/Cardinal.h>

#include <Go/Label.h>
#include <Go/Text.h>
#include <Go/SText.h>
#include <Go/SimpleForm.h>

#ifdef GoLICENSE
#include <Go/ScText.h>
#include <Go/MCForm.h>
#endif

#include <Go/BGoUIEd.h>

#include <Go/Image.h>



#define KEYSTRING        "function"
#define GONUMBER         "GoNumber"
#define SELECTEDGONUMBER "SelectedGoNumber"
#define SELECTEDGO       "SelectedGo"
#define SELECTEDGOLABEL  "SelectedGoLabel"
#define GOCURRENTPARENT  "CurrentParent"



/*
 * EXTERN FUNCTIONS
 */

// in file help.cc;
extern void MapHelpFile(void*, char*, void*);


extern GAssoc key_assoc;


uint numSelectedObjects;
static uint numObjects;




void GetNumSelectedProc (void*, Go*)
{
    numSelectedObjects++;
}

int GetNumMainSelectedObjects()
{
    numSelectedObjects = 0;
    goroot->forEverySelectedGo(GetNumSelectedProc, NULL);
    return numSelectedObjects;
}


static void GetNumProc (void*, Go* go)
{
    if (go != goroot)
	numObjects++;
}


int GetNumObjects()
{
    numObjects = 0;
    goroot->forEveryGo(GetNumProc, NULL);
    return numObjects;
}


static void GetGoNames (char* text, Go* go)
{
    GPoint orig;
    char x_position[12], y_position[12];
    const char* classNames = go->getMetaclass()->classname();
//    strcat (text, "\t\t");
    strcat (text, classNames);
    orig = go->origin();
    sprintf (x_position, "%.1f", orig.x);
    sprintf (y_position, "%.1f", orig.y);
    strcat (text, " (");
    strcat (text, x_position);
    strcat (text, ", ");
    strcat (text, y_position);
    strcat (text, ")\n");
}

void GetCurrentParentClassName(char* ctxt)
{
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
    GetGoNames(ctxt, bgoui->currentParent());
}

void GetClassNames(char* ctxt)
{
    goroot->forEverySelectedGo((void (*)(void *, Go *)) GetGoNames, ctxt);
}




void GoFlush()
{
    goroot->flush();
    if (named_goroot)
	named_goroot->flush();
    if (edit_goroot)
	edit_goroot->flush();
    if (help_goroot)
	help_goroot->flush();
}





                              /********
			       * HELP *
			       ********/



void CallHelpForGo(int x, int y)
{
    // WARNING: x and y are in pixels;
    GPoint p(x*pixel_size, y*pixel_size);

    Go* go = goroot->goAt(p, 0.5);
    if (go) {
	if (go->getMetaclass() == GoArc::metaclass())
	    MapHelpFile(NULL, ARC_FILE, NULL);
	else
	if (go->getMetaclass() == GoCircle::metaclass())
	    MapHelpFile(NULL, CIRCLE_FILE, NULL);
	else
	if (go->getMetaclass() == GoBezier::metaclass())
	    MapHelpFile(NULL, BEZIER_FILE, NULL);
	else
	if (go->getMetaclass() == GoBSpline::metaclass() ||
	    go->getMetaclass() == GoCardinal::metaclass())
	    MapHelpFile(NULL, BCARDINAL_FILE, NULL);
	else
	if (go->getMetaclass() == GoBetaSpline::metaclass() ||
	    go->getMetaclass() == GoTauSpline::metaclass())
	    MapHelpFile(NULL, BETATAU_FILE, NULL);
	else
	if (go->getMetaclass() == GoEllipse::metaclass())
	    MapHelpFile(NULL, ELLIPSE_FILE, NULL);
	else
	if (go->getMetaclass() == GoImage::metaclass())
	    MapHelpFile(NULL, IMAGE_FILE, NULL);
	else
	if (go->getMetaclass() == GoPolygon::metaclass())
	    MapHelpFile(NULL, POLYGON_FILE, NULL);
	else
	if (go->getMetaclass() == GoPolyline::metaclass())
	    MapHelpFile(NULL, POLYLINE_FILE, NULL);
	else
	if (go->getMetaclass() == GoRectangle::metaclass())
	    MapHelpFile(NULL, RECTANGLE_FILE, NULL);
	else
	if (go->getMetaclass() == GoRegPolygon::metaclass())
	    MapHelpFile(NULL, REGPOLYGON_FILE, NULL);
	else
	if (go->getMetaclass() == GoLabel::metaclass())
	    MapHelpFile(NULL, LABEL_FILE, NULL);
#ifdef GoLICENSE	
	else
	if (go->getMetaclass() == GoScrolledText::metaclass())
	    MapHelpFile(NULL, SCROLLTEXT_FILE, NULL);
	else
	if (go->getMetaclass() == GoText::metaclass()  &&
	     strcmp(((GoText*)go)->formatter()->getClassname(),
		  GMCFormatter::classname()) == 0)
	    MapHelpFile(NULL, MCTEXT_FILE, NULL);
#endif
	else
	if (go->getMetaclass() == GoSimpleText::metaclass() ||
	    (go->getMetaclass() == GoText::metaclass()  &&
	     strcmp(((GoText*)go)->formatter()->getClassname(),
		  GSimpleFormatter::classname()) == 0))
	    MapHelpFile(NULL, TEXT_FILE, NULL);
    }
}



void UpdateOverview()
{
    char string[1000];
    int nbSel = 0;
    Go* sel_go = NULL;
    Go* sel_number = NULL;
    
    GAtom key_atom = GCreateAtom(KEYSTRING);
    for (GoTextual* text = (GoTextual*)help_goroot->child(); text;
	 text = (GoTextual*)text->next()) {
	int n;
	char* value = (char*)text->assoc()->get(key_atom, n);
	if (n > 0 && value) {
	    if (strcmp(value, GONUMBER) == 0) {
		int num = GetNumObjects();
		sprintf(string, "%d", num);
		text->setString(string, strlen(string));
	    } else
	    if (strcmp(value, SELECTEDGONUMBER) == 0) {
		sel_number = text;
		nbSel = GetNumMainSelectedObjects();
		sprintf(string, "%d", nbSel);
		text->setString(string, strlen(string));
	    } else
	    if (strcmp(value, SELECTEDGOLABEL) == 0) {
		if (! nbSel)
		    text->unmap();
	    } else
	    if (strcmp(value, SELECTEDGO) == 0) {
		sel_go = text;
		if (nbSel) {
		    string[0] = '\0';
		    GetClassNames(string);
		    text->setString(string, strlen(string));
		} else
		    text->unmap();
	    } else
	    if (strcmp(value, GOCURRENTPARENT) == 0) {
		string[0] = '\0';
		GetCurrentParentClassName(string);
		text->setString(string, strlen(string) - 1); // without '\n';
		Go* text_label = text->previous();

		GBox align_clip;
		if (nbSel)
		    align_clip = sel_go->clip().bigbox();
		else
		    align_clip = sel_number->clip().bigbox();
		GBox label_clip = text_label->clip().bigbox();
		if (label_clip.ymin != align_clip.ymax + 2) {
		    text_label->move(text_label->origin() +
				     GVector(0, align_clip.ymax -
					    label_clip.ymin + 2));
		    text->move(text->origin() + GVector(0, align_clip.ymax -
						       label_clip.ymin + 2));
		}
	    }
	}
    }
}
