#ifdef REV_INFO
static char rcsid[] = "$Id: goUndo.cc,v 1.4 1992/12/10 16:13:17 jojic Exp $";
#endif
/*****************************************************************************
MODULE:  Go Editor
FILE:    goUndo.cc
AUTHOR:  Aline Jacomet
DATE:    January 21th 1992

DESCRIPTION:
    Undo management

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include "goMain.h"
#include <sys/types.h>
#include <Go/RegPoly.h>
#include <Go/Arc.h>
#ifdef GoLICENSE
#include <Go/MCForm.h>
#endif
#include <Go/Text.h>
#include <Go/Rectangle.h>
#include <Go/Image.h>
#include <Go/Composite.h>
#include <Go/Tools/Undo.h>
#include <Go/BGoUIEd.h>

/*
 * EXTERN FUNCTIONS
 */
// in file message.cc;
extern void UndoEmpty(void*);
extern void RedoEmpty(void*);

// in file goEdition.cc;
extern void cut_proc(void*, Go*);
extern void paste_proc(void*, Go*);

// in file goStructure.cc;
extern void Group(void*, void*, caddr_t);
extern void Ungroup(void*, void*, caddr_t);
extern void removeComposite(void*, Go*, Go*, Go*);


GUndoManager undoMgr;

void UndoUnselect(void*);
void UndoUnselectOne(void*);
void UndoUnleaf(void*);
void UndoLower(void*);
void UndoDestroy(void*);
void UndoDeletePolyPoint(void*);
void UndoDeleteEdge(void*);


typedef struct _UndoTextStruct {
    class GoTextual* go;
    char*      value;
    GPos       pos;
    GPos       old_size;
} UndoTextStruct;


void Undo(void*, void*, caddr_t)
{
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
    bgoui->cancelGrabbers();

    if (undoMgr.undoEmpty())
	UndoEmpty(main_window);
    else {
	undoMgr.undo();
    }
}
   

void Redo(void*, void*, caddr_t)
{
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
    bgoui->cancelGrabbers();

    if (undoMgr.undoUndoEmpty())
	RedoEmpty(main_window);
    else {
	undoMgr.undoUndo();
    }
}
   

void UndoCoordSys(void* str)
{
    UndoCSStruct* cs = (UndoCSStruct*)str;
    UndoCSStruct* old_cs = (UndoCSStruct*)GMALLOC(sizeof(UndoCSStruct));
    old_cs->go = cs->go;
    old_cs->cs = cs->go->coordsys();
    cs->go->setCoordsys(cs->cs);
    undoMgr.push(UndoCoordSys, old_cs, (GUndoProc)free);
    GFREE(str);
}


void UndoThisCoordSys(void* str)
{
    UndoCSStruct* cs = (UndoCSStruct*)str;
    UndoCSStruct* old_cs = (UndoCSStruct*)GMALLOC(sizeof(UndoCSStruct));
    old_cs->go = cs->go;
    old_cs->cs = cs->go->coordsys();
    cs->go->SetThisAbscs(cs->cs);
    undoMgr.push(UndoThisCoordSys, old_cs, (GUndoProc)free);
    GFREE(str);
}


void UndoReparent(void* str)
{
    UndoReparentStruct* par = (UndoReparentStruct*)str;
    UndoReparentStruct* old_par = (UndoReparentStruct*)
	GMALLOC(sizeof(UndoReparentStruct));
    old_par->go = par->go;
    old_par->parent = par->go->parent();
    old_par->next = par->go->next();
    par->go->reparent(par->parent, par->next);
    undoMgr.push(UndoReparent, old_par, (GUndoProc)free);
    GFREE(str);
}


void UndoCurrentParent(void* go)
{
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
    if ((Go*)go != bgoui->currentParent()) {
	undoMgr.push(UndoCurrentParent, bgoui->currentParent());
	bgoui->currentParent((Go*)go);
    }
}


void UndoBrother(void* str)
{
    UndoBrotherStruct* par = (UndoBrotherStruct*)str;
    UndoBrotherStruct* old_par = (UndoBrotherStruct*)
	GMALLOC(sizeof(UndoBrotherStruct));
    old_par->go = par->go;
    old_par->next = par->go->next();
    par->go->shift(par->next);
    undoMgr.push(UndoBrother, old_par, (GUndoProc)free);
    GFREE(str);
}


void UndoSelect(void* tab)
{
    for (int i = 0; ((Go**)tab)[i] != NULL; i++)
	((Go**)tab)[i]->unselect();

    Go** t = (Go**)GMALLOC((i + 1) * sizeof(Go*));
    for (i = 0; ((Go**)tab)[i] != NULL; i++)
	t[i] = ((Go**)tab)[i];
    t[i] = NULL;

    undoMgr.push(UndoUnselect, t, (GUndoProc)free);
    GFREE(tab);
}


void UndoUnselect(void* tab)
{
    for (int i = 0; ((Go**)tab)[i] != NULL; i++)
	((Go**)tab)[i]->select();

    Go** t = (Go**)GMALLOC((i + 1) * sizeof(Go*));
    for (i = 0; ((Go**)tab)[i] != NULL; i++)
	t[i] = ((Go**)tab)[i];
    t[i] = NULL;

    undoMgr.push(UndoSelect, t, (GUndoProc)free);
    GFREE(tab);
}


void UndoSelectOne(void* go)
{
    ((Go*)go)->unselect();
    undoMgr.push(UndoUnselectOne, go);
}


void UndoUnselectOne(void* go)
{
    ((Go*)go)->select();
    undoMgr.push(UndoSelectOne, go);
}


void UndoSelectAll(void* father)
{
    undoMgr.openMark();
    for (Go* child = ((Go*)father)->child(); child; child = child->next()) {
	child->unselect();
	undoMgr.push(UndoUnselectOne, child);
    }
    undoMgr.closeMark();
}


void UndoLeaf(void* go)
{
    undoMgr.push(UndoUnleaf, go);
    ((Go*)go)->leaf(GFALSE);
}


void UndoUnleaf(void* go)
{
    undoMgr.push(UndoLeaf, go);
    ((Go*)go)->leaf(GTRUE);
}


void UndoRaise(void* go)
{
    undoMgr.push(UndoLower, go);
    ((Go*)go)->lower();
}


void UndoLower(void* go)
{
    undoMgr.push(UndoRaise, go);
    ((Go*)go)->raise();
}


void FreeObj(void* str)
{
    ((UndoReparentStruct*)str)->go->destroy();
    GFREE(str);
}

void UndoCreate(void* s)
{
    UndoReparentStruct* str = (UndoReparentStruct*)
	GMALLOC(sizeof(UndoReparentStruct));
    str->parent = ((UndoReparentStruct*)s)->go->parent();
    str->next = ((UndoReparentStruct*)s)->go->next();
    str->go = ((UndoReparentStruct*)s)->go;
    undoMgr.push(UndoDestroy, str, FreeObj);
    str->go->reparent(NULL);
    GFREE(s);
}


void UndoDestroy(void* s)
{
    UndoReparentStruct* str = (UndoReparentStruct*)
	GMALLOC(sizeof(UndoReparentStruct));
    str->parent = NULL;
    str->next = NULL;
    str->go = ((UndoReparentStruct*)s)->go;
    undoMgr.push(UndoCreate, str);
    str->go->reparent(((UndoReparentStruct*)s)->parent,
		      ((UndoReparentStruct*)s)->next);
    GFREE(s);
}


void UndoAddEdge(void* go)
{
    if (((GoRegPolygon*)go)->
	nbPoint(((GoRegPolygon*)go)->GoPolypoint::nbPoint() - 1))
	undoMgr.push(UndoDeleteEdge, go);
}


void UndoDeleteEdge(void* go)
{
    if (((GoRegPolygon*)go)->
	nbPoint(((GoRegPolygon*)go)->GoPolypoint::nbPoint() + 1))
	undoMgr.push(UndoAddEdge, go);
}


void UndoRadii(void* s)
{
    UndoRadiiStruct* str = (UndoRadiiStruct*)GMALLOC(sizeof(UndoRadiiStruct));
    str->go = ((UndoRadiiStruct*) s)->go;
    str->radiusX = ((GoArc*)((UndoRadiiStruct*)s)->go)->radiusX();
    str->radiusY = ((GoArc*)((UndoRadiiStruct*)s)->go)->radiusY();
    ((GoArc*)str->go)->radiusX(((UndoRadiiStruct*)s)->radiusX);
    ((GoArc*)str->go)->radiusY(((UndoRadiiStruct*)s)->radiusY);
    undoMgr.push(UndoRadii, str, (GUndoProc)free);
}


void UndoArcAngle(void* s)
{
    UndoArcPointStruct* str = (UndoArcPointStruct*)
	GMALLOC(sizeof(UndoArcPointStruct));
    str->go = ((UndoArcPointStruct*) s)->go;
    str->start = ((GoArc*)((UndoArcPointStruct*)s)->go)->start();
    str->amplitude = ((GoArc*)((UndoArcPointStruct*)s)->go)->amplitude();
    ((GoArc*)str->go)->start(((UndoArcPointStruct*)s)->start);
    ((GoArc*)str->go)->amplitude(((UndoArcPointStruct*)s)->amplitude);
    undoMgr.push(UndoArcAngle, str, (GUndoProc)free);
}


/*
void UndoArcAmplitude(void* s)
{
    UndoArcPointStruct* str = (UndoArcPointStruct*)
	GMALLOC(sizeof(UndoArcPointStruct));
    str->go = ((UndoArcPointStruct*) s)->go;
    str->amplitude = ((GoArc*)((UndoArcPointStruct*)s)->go)->amplitude();
    ((GoArc*)str->go)->amplitude(((UndoArcPointStruct*)s)->amplitude);
    undoMgr.push(UndoArcAmplitude, str, (GUndoProc)free);
}
*/

#ifdef GoLICENSE
void UndoTextWidth(void* s)
{
    UndoTextSizeStruct* str = (UndoTextSizeStruct*)
	GMALLOC(sizeof(UndoTextSizeStruct));
    str->go = ((UndoTextSizeStruct*) s)->go;
    str->size = ((GoTextual*)((UndoTextSizeStruct*)s)->go)->width();
    ((GMCFormatterCtxt*)((GoText*)str->go)->formatter()->ctxt())->width(((UndoTextSizeStruct*)s)->size);
    undoMgr.push(UndoTextWidth, str, (GUndoProc)free);
}


void UndoTextHeight(void* s)
{
    UndoTextSizeStruct* str = (UndoTextSizeStruct*)
	GMALLOC(sizeof(UndoTextSizeStruct));
    str->go = ((UndoTextSizeStruct*) s)->go;
    str->size = ((GMCFormatterCtxt*)((GoText*)((UndoTextSizeStruct*)s)->go)->formatter()->ctxt())->height();
    ((GMCFormatterCtxt*)((GoText*)str->go)->formatter()->ctxt())->height(((UndoTextSizeStruct*)s)->size);
    undoMgr.push(UndoTextHeight, str, (GUndoProc)free);
}
#endif

void UndoAddPolyPoint(void* s)
{
    UndoPolyPointStruct* str = (UndoPolyPointStruct*)
	GMALLOC(sizeof(UndoPolyPointStruct));
    str->go = ((UndoPolyPointStruct*)s)->go;
    str->index = ((UndoPolyPointStruct*)s)->index;
    str->point = (((GoPolypoint*)str->go)->points())[str->index];
    ((GoPolypoint*)str->go)->delPoint(((UndoPolyPointStruct*)s)->index);
    undoMgr.push(UndoDeletePolyPoint, str, (GUndoProc)free);
}


void UndoDeletePolyPoint(void* s)
{
    UndoPolyPointStruct* str = (UndoPolyPointStruct*)
	GMALLOC(sizeof(UndoPolyPointStruct));
    str->go = ((UndoPolyPointStruct*)s)->go;
    ((GoPolypoint*)str->go)->addPoint(((UndoPolyPointStruct*)s)->index,
				      ((UndoPolyPointStruct*)s)->point,
				      GFALSE);
    str->index = ((GoPolypoint*)str->go)->posToIndex(((UndoPolyPointStruct*)s)->point);
    undoMgr.push(UndoAddPolyPoint, str, (GUndoProc)free);
}


void UndoMovePolyPoint(void* s)
{
    UndoPolyPointStruct* str = (UndoPolyPointStruct*)
	GMALLOC(sizeof(UndoPolyPointStruct));
    str->go = ((UndoPolyPointStruct*)s)->go;
    str->index = ((UndoPolyPointStruct*)s)->index;
    str->point = (((GoPolypoint*)str->go)->points())[str->index];
    ((GoPolypoint*)str->go)->movePoint(((UndoPolyPointStruct*)s)->index,
		       ((UndoPolyPointStruct*)s)->point, GFALSE);
    undoMgr.push(UndoMovePolyPoint, str, (GUndoProc)free);
}


void UndoRegPolyResize(void* closure)
{
    UndoRegPolyStruct* s = (UndoRegPolyStruct*) closure;

    UndoRegPolyStruct* str =
	(UndoRegPolyStruct*) GMALLOC (sizeof (UndoRegPolyStruct));
    str->go = s->go;
    GoRegPolygon* polygon = (GoRegPolygon*) str->go;

    str->angle = polygon->angle();
    str->radius = polygon->radius();
    polygon->angle (s->angle);
    polygon->radius (s->radius);
    undoMgr.push(UndoRegPolyResize, str, (GUndoProc)free);
}


void UndoRectResize(void* s)
{
    UndoRectStruct* str = (UndoRectStruct*)
	GMALLOC(sizeof(UndoRectStruct));
    str->go = ((UndoRectStruct*)s)->go;
    if (str->go->getMetaclass()->isSubclass(GoRectangle::metaclass())) {
	str->width = ((GoRectangle*)str->go)->width();
	str->height = ((GoRectangle*)str->go)->height();
	((GoRectangle*)str->go)->width(((UndoRectStruct*)s)->width);
	((GoRectangle*)str->go)->height(((UndoRectStruct*)s)->height);
    } else
	if (str->go->getMetaclass()->isSubclass(GoImage::metaclass())) {
	str->width = ((GoImage*)str->go)->width();
	str->height = ((GoImage*)str->go)->height();
	((GoImage*)str->go)->config(((UndoRectStruct*)s)->width,
				    ((UndoRectStruct*)s)->height);
    }
    undoMgr.push(UndoRectResize, str, (GUndoProc)free);
}



static void FreeText(void* str)
{
    GFREE(((UndoTextStruct*)str)->value);
    GFREE(str);
}

static void UndoSetString(void* s)
{
    ((UndoTextStruct*)s)->go->setString(((UndoTextStruct*)s)->value,
					strlen(((UndoTextStruct*)s)->value));
}

static void UndoInsertChar(void* s)
{
    ((UndoTextStruct*)s)->go->deleteChar(((UndoTextStruct*)s)->pos);
}

static void UndoDeleteChar(void* s)
{
    ((UndoTextStruct*)s)->go->insertChar(((UndoTextStruct*)s)->value[0],
					 ((UndoTextStruct*)s)->pos);
}

static void UndoInsertString(void* s)
{
    ((UndoTextStruct*)s)->go->deleteString(strlen(((UndoTextStruct*)s)->value),
					   ((UndoTextStruct*)s)->pos);
}

static void UndoDeleteString(void* s)
{
    ((UndoTextStruct*)s)->go->insertString(((UndoTextStruct*)s)->value,
					   strlen(((UndoTextStruct*)s)->value),
					   ((UndoTextStruct*)s)->pos);
}

static void UndoReplaceChar(void* s)
{
    ((UndoTextStruct*)s)->go->replaceChar(((UndoTextStruct*)s)->value[0],
					  ((UndoTextStruct*)s)->pos);
}

static void UndoReplaceString(void* s)
{
    ((UndoTextStruct*)s)->go->replaceString(((UndoTextStruct*)s)->pos,
					    ((UndoTextStruct*)s)->old_size,
					    ((UndoTextStruct*)s)->value,
					  strlen(((UndoTextStruct*)s)->value));
}




                      /************************************
		       * Text value modification postcond *
		       ************************************/

void UndoTextModification(void*, GoTextual* go, GModifyValueStruct* str)
{
    if (go->root() != goroot)
	return;

    UndoTextStruct* s = (UndoTextStruct*)GMALLOC(sizeof(UndoTextStruct));
    s->go = go;
    
    switch(str->type) {
      case GSET_STRING_NOTIFY:
	s->value = (char*)GMALLOC(str->stringValue.len + 1);
	strcpy(s->value, str->stringValue.str);
	undoMgr.push(UndoSetString, s, FreeText);
	break;
      case GINSERT_CHAR_NOTIFY:
	s->value = (char*)GMALLOC(1);
	s->value[0] = str->charValue.Char;
	s->pos = str->charValue.pos;
	undoMgr.push(UndoInsertChar, s, FreeText);
	break;
      case GDELETE_CHAR_NOTIFY:
	s->value = (char*)GMALLOC(1);
	s->value[0] = str->charValue.Char;
	s->pos = str->charValue.pos;
	undoMgr.push(UndoDeleteChar, s, FreeText);
	break;
      case GINSERT_STRING_NOTIFY:
	s->value = (char*)GMALLOC(str->stringValue.len + 1);
	strcpy(s->value, str->stringValue.str);
	s->pos = str->stringValue.pos;
	undoMgr.push(UndoInsertString, s, FreeText);
	break;
      case GDELETE_STRING_NOTIFY:
	s->value = (char*)GMALLOC(str->stringValue.len + 1);
	strcpy(s->value, str->stringValue.str);
	s->pos = str->stringValue.pos;
	undoMgr.push(UndoDeleteString, s, FreeText);
	break;
      case GREPLACE_CHAR_NOTIFY:
	s->value = (char*)GMALLOC(1);
	s->value[0] = str->charValue.Char;
	s->pos = str->charValue.pos;
	undoMgr.push(UndoReplaceChar, s, FreeText);
	break;
      case GREPLACE_STRING_NOTIFY:
	s->value = (char*)GMALLOC(str->stringValue.len + 1);
	strcpy(s->value, str->stringValue.str);
	s->pos = str->stringValue.pos;
	s->old_size = str->stringValue.oldStringSize;
	undoMgr.push(UndoReplaceString, s, FreeText);
	break;
      case GPRESENTATION_CHANGE_NOTIFY:
	break;
    }
}
