#ifdef REV_INFO
static char rcsid[] = "$Id: help.cc,v 1.3 1992/12/30 12:31:30 jacomet Exp $";
#endif
/*****************************************************************************
MODULE:  Go Editor
FILE:    help.cc
AUTHOR:  Aline Jacomet and Patricia Langle
DATE:    August 12th 1991

DESCRIPTION:
    

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include <sys/types.h>
#include <sys/stat.h>

#include <Go/Tools/FilePath.h>
#include <Go/Tools/Alloc.h>

#include "main.h"
#include <stdio.h>

#include <Xm/Form.h>
#include <Xm/PushBG.h>
#include <Xm/ScrolledW.h>
#include <Xm/MessageB.h>
#include <Xm/LabelG.h>
#include <Xm/List.h>

#include <Xb/Graphic.h>



/*
 * EXTERN FUNCTIONS
 */

// in file namedattrib.cc;
extern void UnmanageWidget(Widget, Widget, caddr_t);

// in file message.cc;
extern void NoHelp(void*);
extern void NoPrevPage(void*);
extern void NoNextPage(void*);

// in file goTools.cc;
extern int  GetNumMainSelectedObjects();
extern int  GetNumObjects();
extern void GetClassNames(char*);
extern void GetCurrentParentClassName(char*);

// in file goInit.cc;
extern void InitGoHelp(void*);

// in file goLoadSave.cc;
extern void LoadGo(void*, char*);
extern void Clear(void*, void*, caddr_t);

// in file goTools.cc;
extern void CallHelpForGo(int, int);
extern void UpdateOverview();


#define INDEX_LIST    "index.txt"
#define I_ASSOC       "i-assoc."
#define I_BRUSH       "i-brush."
#define I_GO          "i-go."
#define I_ARC         "i-arc."
#define I_BETASPLINE  "i-betaspline."
#define I_BEZIER      "i-bezier."
#define I_BUSLINK     "i-buslink."
#define I_BSPLINE     "i-bspline."
#define I_CARD_SPLINE "i-cardspline."
#define I_CIRCLE      "i-circle."
#define I_CUBIC       "i-cubic."
#define I_CURVE       "i-curve."
#define I_ELLIPSE     "i-ellipse."
#define I_GEOMETRIC   "i-geometric."
#define I_GRAPH       "i-graph."
#define I_GRAPHELT    "i-graphelt."
#define I_GRAPHNODE   "i-graphnode."
#define I_GRAPHSTR    "i-graphstr."
#define I_IMAGE       "i-image."
#define I_INTERFACE   "i-interface."
#define I_LABEL       "i-label."
#define I_LINK        "i-link."
#define I_NODE        "i-node."
#define I_POLYGON     "i-polygon."
#define I_POLYLINE    "i-polyline."
#define I_POLYPOINT   "i-polypoint."
#define I_RECTANGLE   "i-rect."
#define I_REG_POLY    "i-reg-poly."
#define I_ROOT        "i-root."
#define I_SCR_TXT     "i-scr-txt."
#define I_SIM_LINK    "i-sim-link."
#define I_SIM_TXT     "i-sim-txt."
#define I_STREAM      "i-stream."
#define I_TAUSPLINE   "i-tauspline."
#define I_TEXT        "i-text."
#define I_TEXTUAL     "i-textual."
#define I_TREE        "i-tree."
#define I_TREENODE    "i-treenode."
#define I_GOUI        "i-goui."
#define I_GRAPHIC     "i-graphic."
#define I_PEN         "i-pen."
#define I_PSINTERF    "i-psinterf."
#define I_SENSOR      "i-sensor."
#define I_SHARED      "i-shared."
#define I_TXTBRUSH    "i-txtbrush."
#define I_WININTERF   "i-wininterf."
#define I_XINTERF     "i-xinterf."
#define I_XMINTERF    "i-xminterf."



/*
        HELP MANAGEMENT
        ---------------
*/


void* help_graphic;

static Widget helpBox = NULL;
static Widget helpLabel = NULL;
static Widget helpList = NULL;
static Widget helpScroll = NULL;
static Widget unmap;
static Widget previous;
static Widget next;
static Widget done;

static char filename[100];

static int count;

static Boolean listMapped;

void MapHelpFile(void*, char*, void*);


static Boolean LoadFile(char* file_str)
{
    char count_txt[5];
    sprintf(count_txt, "%d", count);
    char* name = (char*)GMALLOC(strlen("help/") + strlen(file_str) +
				 strlen(count_txt) + 1);
    strcat(strcpy(name, "help/"), file_str);
    strcat(name, count_txt);
    char* all_name = GGetFilePath("app-files/godraw", name);
    GFREE(name);
    if (!all_name)
	return False;
    
    Clear(NULL, help_graphic, NULL);
    LoadGo(help_graphic, all_name);
    GFREE(all_name);
    if (strcmp(file_str, HELP_OVERVIEW_FILE) == 0)
	UpdateOverview();
    return True;
}


static void NextFile(Widget, caddr_t, caddr_t)
{
    count++;
    if (! LoadFile(filename)) {
	count--;
	NoNextPage(main_window);
    }
}


static void PreviousFile(Widget, caddr_t, caddr_t)
{
    count--;
    if (! LoadFile(filename)) {
	count++;
	NoPrevPage(main_window);
    }
}


static void ReadIndex(Widget, caddr_t, caddr_t)
{
    XmString* rem_table = NULL;
    Arg args[5];
    int n = 0;
    XtSetArg(args[n], XmNselectedItems, &rem_table); n++;
    XtGetValues(helpList, args, n);

    count = 0;
    char* str = NULL;
    XmStringGetLtoR(rem_table[0], XmSTRING_DEFAULT_CHARSET, &str);
    char* name = NULL;
    if (strcmp("Assoc", str) == 0)
	name = I_ASSOC;
    else
    if (strcmp("Brush", str) == 0)
	name = I_BRUSH;
    else
    if (strcmp("Go", str) == 0)
	name = I_GO;
    else
    if (strcmp("GoArc", str) == 0)
	name = I_ARC;
    else
    if (strcmp("GoBetaSpline", str) == 0)
	name = I_BETASPLINE;
    else
    if (strcmp("GoBezier", str) == 0)
	name = I_BEZIER;
    else
    if (strcmp("GoBusLink", str) == 0)
	name = I_BUSLINK;
    else
    if (strcmp("GoBSpline", str) == 0)
	name = I_BSPLINE;
    else
    if (strcmp("GoCardinal", str) == 0)
	name = I_CARD_SPLINE;
    else
    if (strcmp("GoCircle", str) == 0)
	name = I_CIRCLE;
    else
    if (strcmp("GoCubic", str) == 0)
	name = I_CUBIC;
    else
    if (strcmp("GoCurve", str) == 0)
	name = I_CURVE;
    else
    if (strcmp("GoEllipse", str) == 0)
	name = I_ELLIPSE;
    else
    if (strcmp("GoGeometric", str) == 0)
	name = I_GEOMETRIC;
    else
    if (strcmp("GoGraph", str) == 0)
	name = I_GRAPH;
    else
    if (strcmp("GoGraphElement", str) == 0)
	name = I_GRAPHELT;
    else
    if (strcmp("GoGraphNode", str) == 0)
	name = I_GRAPHNODE;
    else
    if (strcmp("GoGraphStruct", str) == 0)
	name = I_GRAPHSTR;
    else
    if (strcmp("GoImage", str) == 0)
	name = I_IMAGE;
    else
    if (strcmp("GoInterface", str) == 0)
	name = I_INTERFACE;
    else
    if (strcmp("GoLabel", str) == 0)
	name = I_LABEL;
    else
    if (strcmp("GoLink", str) == 0)
	name = I_LINK;
    else
    if (strcmp("GoNode", str) == 0)
	name = I_NODE;
    else
    if (strcmp("GoPolygon", str) == 0)
	name = I_POLYGON;
    else
    if (strcmp("GoPolyline", str) == 0)
	name = I_POLYLINE;
    else
    if (strcmp("GoPolypoint", str) == 0)
	name = I_POLYPOINT;
    else
    if (strcmp("GoRectangle", str) == 0)
	name = I_RECTANGLE;
    else
    if (strcmp("GoRegPolygon", str) == 0)
	name = I_REG_POLY;
    else
    if (strcmp("GoRoot", str) == 0)
	name = I_ROOT;
    else
    if (strcmp("GoScrolledText", str) == 0)
	name = I_SCR_TXT;
    else
    if (strcmp("GoSensor", str) == 0)
	name = I_SENSOR;
    else
    if (strcmp("GoSimpleLink", str) == 0)
	name = I_SIM_LINK;
    else
    if (strcmp("GoSimpleText", str) == 0)
	name = I_SIM_TXT;
    else
    if (strcmp("GoStream", str) == 0)
	name = I_STREAM;
    else
    if (strcmp("GoTauSpline", str) == 0)
	name = I_TAUSPLINE;
    else
    if (strcmp("GoText", str) == 0)
	name = I_TEXT;
    else
    if (strcmp("GoTextual", str) == 0)
	name = I_TEXTUAL;
    else
    if (strcmp("GoTree", str) == 0)
	name = I_TREE;
    else
    if (strcmp("GoTreeNode", str) == 0)
	name = I_TREENODE;
    else
    if (strcmp("GoUI", str) == 0)
	name = I_GOUI;
    else
    if (strcmp("Graphic Widget", str) == 0)
	name = I_GRAPHIC;
    else
    if (strcmp("Pen", str) == 0)
	name = I_PEN;
    else
    if (strcmp("Shared Objects", str) == 0)
	name = I_SHARED;
    else
    if (strcmp("TextBrush", str) == 0)
	name = I_TXTBRUSH;
    else
    if (strcmp("PSGoInterface", str) == 0)
	name = I_PSINTERF;
    else
    if (strcmp("WinGoInterface", str) == 0)
	name = I_WININTERF;
    else
    if (strcmp("XGoInterface", str) == 0)
	name = I_XINTERF;
    else
    if (strcmp("XmGoInterface", str) == 0)
	name = I_XMINTERF;
    else {
	printf("ERROR\n");
	return;
    }
    
    filename[0] = '\0';
    strcpy(filename, name);
    if (! LoadFile(name))
	NoHelp(main_window);
}


static Boolean InitializeList()
{
    char helpdir[256];

    strcat (strcpy(helpdir, "help/"), INDEX_LIST);
    char* all_name = GGetFilePath("app-files/godraw", helpdir);
    
    if (all_name) {
	struct stat file_status;
	stat(all_name, &file_status);
	if (file_status.st_size > 0) {
	    char* txt = (char*)GMALLOC((unsigned int)file_status.st_size);
	    FILE* file = fopen(all_name, "r");
	    if (file) {
		while (fgets(txt, (int)file_status.st_size, file)) {
		    if (txt[0] != '#') {
			// '#' is a commentary;
			txt[strlen(txt)-1] = '\0';  // without '\n';
			XmString str =
			    XmStringCreate(txt, XmSTRING_DEFAULT_CHARSET);
			XmListAddItem(helpList, str, 0);
			XmStringFree(str);
			txt[0] = '\0';
		    }
		}
	    } else {
		GFREE(txt);
		GFREE(all_name);
		return False;
	    }
	    GFREE(txt);
	} else {
	    GFREE(all_name);
	    return False;
	}
    } else
	return False;
    
    GFREE(all_name);
    return True;
}


static Widget CreateGoHelpBox(Widget parent)
{
    Arg args[20];

    int n = 0;
    XtSetArg(args[n], XmNunitType, Xm100TH_MILLIMETERS); n++;
    XtSetArg(args[n], XmNwidth, 22100); n++;
    XtSetArg(args[n], XmNheight, 17900); n++;
    XtSetArg(args[n], XmNautoUnmanage, False); n++;
    XtSetArg(args[n], XmNresizePolicy, XmRESIZE_NONE); n++;
    Widget form = XmCreateFormDialog(parent, "HelpDialog", args, n);

    /*
     * Label
     */
    n = 0;
    XtSetArg(args[n], XmNunitType, XmPIXELS); n++;
    XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNtopOffset, 10); n++;
    XtSetArg(args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
    XtSetArg(args[n], XmNleftPosition, 40); n++;
    helpLabel = XmCreateLabelGadget(form, "HelpLabel", args, n);
    XtManageChild(helpLabel);

    /*
     * PushButtons
     */
    n = 0;
    XtSetArg(args[n], XmNunitType, XmPIXELS); n++;
    XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNbottomOffset, 10); n++;
    XtSetArg(args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
    XtSetArg(args[n], XmNleftPosition, 70); n++;
    XtSetArg(args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
    XtSetArg(args[n], XmNrightPosition, 90); n++;
    unmap = XmCreatePushButtonGadget(form, "HelpUnmap", args, n);
    XtManageChild(unmap);
    XtAddCallback(unmap, XmNactivateCallback, (XtCallbackProc)UnmanageWidget,
		  (XtPointer)form);

    n = 3;
    XtSetArg(args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
    XtSetArg(args[n], XmNleftPosition, 10); n++;
    XtSetArg(args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
    XtSetArg(args[n], XmNrightPosition, 30); n++;
    previous = XmCreatePushButtonGadget(form, "HelpPrevious", args, n);
    XtManageChild(previous);
    XtAddCallback(previous, XmNactivateCallback, (XtCallbackProc)PreviousFile,
		  (XtPointer)form);

    n = 3;
    XtSetArg(args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
    XtSetArg(args[n], XmNleftPosition, 40); n++;
    XtSetArg(args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
    XtSetArg(args[n], XmNrightPosition, 60); n++;
    next = XmCreatePushButtonGadget(form, "HelpNext", args, n);
    XtManageChild(next);
    XtAddCallback(next, XmNactivateCallback, (XtCallbackProc)NextFile,
		  (XtPointer)form);

    /*
     * ScrolledWindow and Graphic
     */
    n = 0;
    XtSetArg(args[n], XmNunitType, XmPIXELS); n++;
    XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg(args[n], XmNtopOffset, 10); n++;
    XtSetArg(args[n], XmNtopWidget, helpLabel); n++;
    XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNleftOffset, 10); n++;
    XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNrightOffset, 10); n++;
    XtSetArg(args[n], XmNbottomAttachment, XmATTACH_WIDGET); n++;
    XtSetArg(args[n], XmNbottomWidget, unmap); n++;
    XtSetArg(args[n], XmNbottomOffset, 10); n++;
    XtSetArg(args[n], XmNscrollingPolicy, XmAUTOMATIC); n++;
    XtSetArg(args[n], XmNscrollBarDisplayPolicy, XmAS_NEEDED); n++;
    helpScroll = XmCreateScrolledWindow(form, "HelpScroll", args, n);
    XtManageChild(helpScroll);

    n = 0;
    XtSetArg(args[n], XmNunitType, Xm100TH_MILLIMETERS); n++;
    XtSetArg(args[n], XmNwidth, 21000); n++;
    XtSetArg(args[n], XmNheight, 14900); n++;
    XtSetArg(args[n], XmNresizePolicy, XmRESIZE_GROW); n++;
    XtSetArg(args[n], XbNautomaticResize, True); n++;
    XtSetArg(args[n], XbNscaleOnResize, False); n++;
    help_graphic = XbCreateGraphic(helpScroll, "HelpGraphic", args, n);
    XtManageChild((Widget)help_graphic);
    XmScrolledWindowSetAreas(helpScroll, NULL, NULL, (Widget)help_graphic);
    
    /*
     * List and its PushButton
     */
    n = 0;
    XtSetArg(args[n], XmNunitType, XmPIXELS); n++;
    XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg(args[n], XmNtopOffset, 10); n++;
    XtSetArg(args[n], XmNtopWidget, helpLabel); n++;
    XtSetArg(args[n], XmNbottomAttachment, XmATTACH_WIDGET); n++;
    XtSetArg(args[n], XmNbottomWidget, unmap); n++;
    XtSetArg(args[n], XmNbottomOffset, 10); n++;
    XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNleftOffset, 10); n++;
    XtSetArg(args[n], XmNrightAttachment, XmATTACH_WIDGET); n++;
    XtSetArg(args[n], XmNrightOffset, 10); n++;
    XtSetArg(args[n], XmNrightWidget, helpScroll); n++;
    XtSetArg(args[n], XmNlistSizePolicy, XmCONSTANT); n++;
    XtSetArg(args[n], XmNselectionPolicy, XmSINGLE_SELECT); n++;
    helpList = XmCreateScrolledList(form, "HelpList", args, n);
    listMapped = False;
    if (! InitializeList())
	printf("An error occured in List initialization\n");

    n = 0;
    XtSetArg(args[n], XmNunitType, XmPIXELS); n++;
    XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNbottomOffset, 10); n++;
    XtSetArg(args[n], XmNleftAttachment, XmATTACH_OPPOSITE_WIDGET); n++;
    XtSetArg(args[n], XmNleftOffset, 20); n++;
    XtSetArg(args[n], XmNleftWidget, helpList); n++;
    XtSetArg(args[n], XmNrightAttachment, XmATTACH_OPPOSITE_WIDGET); n++;
    XtSetArg(args[n], XmNrightOffset, 35); n++;
    XtSetArg(args[n], XmNrightWidget, helpList); n++;
    done = XmCreatePushButtonGadget(form, "HelpDone", args, n);
    XtAddCallback(done, XmNactivateCallback, (XtCallbackProc)ReadIndex,
		  NULL);

    
    XtRealizeWidget(form);

    /*
     * Initialization of Go
     */
    void* help_interf = (void*)XbGraphicGoInterf((XbGraphicWidget)
						 help_graphic);
    InitGoHelp(help_interf);

    return form;
}



static void ManageList(Boolean manage)
{
    Arg args[5];
    int n;
    listMapped = manage;
    if (! manage) {
	n = 0;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNleftOffset, 10); n++;
	XtSetValues(helpScroll, args, n);

	n = 0;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
	XtSetArg(args[n], XmNleftPosition, 70); n++;
	XtSetArg(args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
	XtSetArg(args[n], XmNrightPosition, 90); n++;
	XtSetValues(unmap, args, n);

	n = 0;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
	XtSetArg(args[n], XmNleftPosition, 40); n++;
	XtSetArg(args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
	XtSetArg(args[n], XmNrightPosition, 60); n++;
	XtSetValues(next, args, n);

	n = 0;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
	XtSetArg(args[n], XmNleftPosition, 10); n++;
	XtSetArg(args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
	XtSetArg(args[n], XmNrightPosition, 30); n++;
	XtSetValues(previous, args, n);

	XtUnmanageChild(helpList);
	XtUnmanageChild(done);
    } else {
	n = 0;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
	XtSetArg(args[n], XmNleftPosition, 15); n++;
	XtSetValues(helpScroll, args, n);

	n = 0;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
	XtSetArg(args[n], XmNleftPosition, 80); n++;
	XtSetArg(args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
	XtSetArg(args[n], XmNrightPosition, 95); n++;
	XtSetValues(unmap, args, n);

	n = 0;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
	XtSetArg(args[n], XmNleftPosition, 55); n++;
	XtSetArg(args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
	XtSetArg(args[n], XmNrightPosition, 70); n++;
	XtSetValues(next, args, n);

	n = 0;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
	XtSetArg(args[n], XmNleftPosition, 30); n++;
	XtSetArg(args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
	XtSetArg(args[n], XmNrightPosition, 45); n++;
	XtSetValues(previous, args, n);

	XtManageChild(helpList);
	XtManageChild(done);

	Clear(NULL, help_graphic, NULL);
    }
}


void MapHelpFile(void*, char* file_str, void*)
{
    count = 0;

    if (! helpBox) {
	XDefineCursor(XtDisplay((Widget)graphic), XtWindow((Widget)graphic),
		      clock_cursor);
	XDefineCursor(XtDisplay((Widget)main_window),
			XtWindow((Widget)main_window), clock_cursor);
	XFlush(XtDisplay((Widget)graphic));

	helpBox = CreateGoHelpBox((Widget)main_window);

	XFlush(XtDisplay((Widget)graphic));
	XUndefineCursor(XtDisplay((Widget)graphic), XtWindow((Widget)graphic));
	XUndefineCursor(XtDisplay((Widget)main_window),
			XtWindow((Widget)main_window));
    }

    XmString str = NULL;
    if (strcmp(file_str, HELP_INDEX_FILE) == 0) {
	str = XmStringCreate("Index Help", XmSTRING_DEFAULT_CHARSET);
	if (! listMapped)
	    ManageList(True);
	    Arg arg;
	    XtSetArg(arg, XmNlabelString, str);
	    XtSetValues(helpLabel, &arg, 1);
	    
	    strcpy(filename, file_str);
	    XtManageChild(helpBox);
    } else {
	if (LoadFile(file_str)) {
	    if (listMapped)
		ManageList(False);
	    if (strcmp(file_str, HELP_FILE) == 0) {
		str = XmStringCreate("Help on Help", XmSTRING_DEFAULT_CHARSET);
	    } else
	    if (strcmp(file_str, HELP_OVERVIEW_FILE) == 0) {
		str = XmStringCreate("Overview Help",
				     XmSTRING_DEFAULT_CHARSET);
	    } else
	    if (strcmp(file_str, HELP_KEYBOARD_FILE) == 0) {
		str = XmStringCreate("Keyboard Help",
				     XmSTRING_DEFAULT_CHARSET);
	    } else
	    if (strcmp(file_str, HELP_GODRAW_FILE) == 0) {
		str = XmStringCreate("GoDraw Information",
				     XmSTRING_DEFAULT_CHARSET);
	    } else
	    if (strcmp(file_str, HELP_GOINFO_FILE) == 0) {
		str = XmStringCreate("Graphic Object Information",
				     XmSTRING_DEFAULT_CHARSET);
	    } else
		str = XmStringCreate("Context-Sensitive Help",
				     XmSTRING_DEFAULT_CHARSET);
	    Arg arg;
	    XtSetArg(arg, XmNlabelString, str);
	    XtSetValues(helpLabel, &arg, 1);
	    
	    strcpy(filename, file_str);
	    XtManageChild(helpBox);
	} else
	    NoHelp(main_window);
    }
}




                          /**************************
			   * Context-Sensitive Help *
			   **************************/

XmGadget InputInGadget (CompositeWidget cw, int x, int y)
{
    register int i;
    register Widget widget;
    Widget* widgetList;
    int nb, n;
    Arg args[4];

    n = 0;
    XtSetArg(args[n], XmNchildren, &widgetList); n++;
    XtSetArg(args[n], XmNnumChildren, &nb);; n++;
    XtGetValues((Widget)cw, args, n);
    
    for (i = 0; i < nb; i++) {
	widget = widgetList[i];
	
	if (XmIsGadget (widget) && XtIsManaged (widget)) {
	    Dimension w, h;
	    Position w_x, w_y;
	    n = 0;
	    XtSetArg(args[n], XmNx, &w_x); n++;
	    XtSetArg(args[n], XmNy, &w_y); n++;
	    XtSetArg(args[n], XmNwidth, &w); n++;
	    XtSetArg(args[n], XmNheight, &h); n++;
	    XtGetValues(widget, args, n);
	    if (x >= w_x && y >= w_y && x < w_x + w && y < w_y + h)
		return ((XmGadget) widget);
	}
    }
    return (NULL);
}


static Widget TrackingLocate(Widget widget, Cursor cursor, Boolean confineTo,
			     XEvent* event)
{
    Window confine_to;
    Widget child;
    
    if (widget == NULL)
	return(NULL);
    Widget target = NULL;
    Window w = XtWindowOfObject(widget);
    if (confineTo)
        confine_to = w;
    else
        confine_to = None;

    Time lastTime = XtLastTimestampProcessed(XtDisplay(widget));
    XmUpdateDisplay(widget);
    XtGrabPointer(widget, True,
		  (unsigned int)(ButtonPressMask | ButtonReleaseMask), 
                  GrabModeAsync, GrabModeAsync, confine_to, cursor,
		  lastTime);

    Boolean BailOut = GFALSE;

    while (!BailOut) {
        XMaskEvent(XtDisplay(widget), (ButtonReleaseMask | ButtonPressMask),
		   event);
        if (event->type == ButtonRelease) {
            if (((XButtonEvent*)event)->button & Button1) {
/****************
 *
 * First, look if the window is equal to the grab window. If so, make
 * sure that the location is inside the window.
 *
 ****************/
                if ((!confineTo) && (((XButtonEvent*)event)->window == w)) {
		    Arg args[5];
		    Dimension w, h;
		    Position x, y;
		    int n = 0;
		    XtSetArg(args[n], XmNx, &x); n++;
		    XtSetArg(args[n], XmNy, &y); n++;
		    XtSetArg(args[n], XmNwidth, &w); n++;
		    XtSetArg(args[n], XmNheight, &h); n++;
		    XtGetValues(widget, args, n);
		    
                    if ((((XButtonEvent*)event)->x < x) ||
                        (((XButtonEvent*)event)->y < y) ||
                        (((XButtonEvent*)event)->x > (x + w)) ||
                        (((XButtonEvent*)event)->y > (y + h))) {
                            XtUngrabPointer(widget, lastTime);
                            return(NULL);
                        }
                }
/****************
 *
 * Now get the window, and see if it's a gadget
 *
 ****************/
                target = XtWindowToWidget(((XButtonEvent*)event)->display,
					  ((XButtonEvent*)event)->window);
                if (target) {
                    if ((XmIsManager(target)) && 
                        (child = (Widget )
			 InputInGadget((CompositeWidget)target,
				       ((XButtonEvent*)event)->x,
				       ((XButtonEvent*)event)->y)))
                        target = child;
                }
            }                    
            BailOut = GTRUE;
        }
    }
    XtUngrabPointer(widget, lastTime);
    return(target);
}


void QueryForHelp(Widget, caddr_t, XmAnyCallbackStruct* cbs)
{
    Widget help_widget;
    Cursor cursor = XCreateFontCursor(XtDisplay((Widget)main_window),
				      XC_question_arrow);

    XEvent event;
    if (help_widget = TrackingLocate((Widget)main_window, cursor, False,
				     &event)) {
	if (help_widget)
	    if (help_widget == (Widget)graphic) {
		XButtonEvent* ev = ((XButtonEvent*)&event);
		CallHelpForGo(ev->x, ev->y);
	    } else {
		cbs->reason = XmCR_HELP;
		XtCallCallbacks(help_widget, XmNhelpCallback, (XtPointer)&cbs);
	    }
    }
    XFreeCursor(XtDisplay((Widget)main_window), cursor);
}

