#ifdef REV_INFO
static char rcsid[] = "$Id: main.cc,v 1.5 1992/12/21 11:47:33 jojic Exp $";
#endif
/*****************************************************************************
MODULE:  Go editor
FILE:    main.c
AUTHOR:  Aline Jacomet
DATE:    May 14th 1991

DESCRIPTION:
    Creation of the main window.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include <Go/Tools/FilePath.h>
#include <Go/Tools/Alloc.h>
#include <Go/Tools/Undo.h>

#include "main.h"

#include <X11/Xresource.h>

#include <Xm/MainW.h>
#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <Xm/RowColumn.h>
#include <Xm/PushBG.h>
#include <Xm/ToggleBG.h>
#include <Xm/ScrolledW.h>
#include <X11/Intrinsic.h>
#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>
#include <Xb/Graphic.h>


static char* className = "XmGoDraw";
static char* applicationName = "godraw";

static Widget createBtn[13];

char* lpr_value = NULL;

void* main_window;
void* graphic;
Widget scroll;

extern Widget orientation_tg[2];
extern Widget form_tg[6];
//extern Widget menubarBtn[9];

extern GUndoManager undoMgr;



/*
 * EXTERN FUNCTIONS
 */

// in file cursors.cc;
extern void CreateCursors(Widget);

// in file menubar.cc;
extern Widget CreateMenuBar(Widget);

// in file pixmap.cc;
extern void CreatePixmaps(Widget);

// in file goCreation.cc;
extern void CreatePolyline(void*, void*, caddr_t);
extern void CreatePolygon(void*, void*, caddr_t);
extern void CreateHexagon(void*, void*, caddr_t);
extern void CreateImage(void*, void*, caddr_t);
extern void CreateHand(void*, void*, caddr_t);
extern void CreateCircleArc(void*, void*, caddr_t);
extern void CreateEllipseArc(void*, void*, caddr_t);
extern void CreateCircle(void*, void*, caddr_t);
extern void CreateEllipse(void*, void*, caddr_t);
extern void CreateRectangle(void*, void*, caddr_t);
extern void CreateSquare(void*, void*, caddr_t);

// in file curvemenu.cc;
extern Widget CreateCurveButton(Widget);

// in file textmenu.cc;
extern Widget CreateTextButton(Widget);

// in file popupmenu.cc;
extern void CreatePolypointMenus(Widget);

// in file goInit.cc;
extern void InitGo(void*);
extern void RunEventLoop();

// in file goAttrib.cc;
extern void GoFlush();

// in file help.cc;
extern void MapHelpFile(void*, char*, void*);


GBool noFinger = GFALSE;




                                     /********
                                      * Main *
                                      ********/


main(Cardinal argc, char**argv)
{
    Arg args[20];
    XtAppContext app;

    Widget toplevel = XtAppInitialize(&app, className, NULL, 0,
				      &argc, argv, NULL, NULL, 0); 

    for (unsigned int i = 1; i < argc; i++)
	if (!strcmp (argv[i], "nofinger"))
	    noFinger = GTRUE;

    /*
     * lpr and undo resources;
     */
    XrmDatabase applicationDB;
    char* lpr_str[100];
    XrmValue value;
    
    XrmInitialize();
    char* all_name = GGetFilePath(NULL, "XmGoDraw");
    if (all_name) 
	applicationDB = XrmGetFileDatabase(all_name);
    else {
	all_name = (char*)GMALLOC(strlen("/usr/lib/X11/app-defaults/") +
				  strlen(applicationName) + 1);
	strcat(strcpy(all_name, "/usr/lib/X11/app-defaults/"),
	       className);
	applicationDB = XrmGetFileDatabase(all_name);
    }
    char* appli_res = (char*)GMALLOC(strlen(applicationName) + 5);
    strcat(strcpy(appli_res, applicationName), ".lpr");
    char* class_res = (char*)GMALLOC(strlen(className) + 5);
    strcat(strcpy(class_res, className), ".lpr");
    if (XrmGetResource(applicationDB, appli_res, class_res, lpr_str, &value)) {
	lpr_value = (char*)GMALLOC((int)value.size + 1);
	strncpy(lpr_value, value.addr, (int)value.size);
    }
    GFREE(class_res);
    GFREE(appli_res);

    appli_res = (char*)GMALLOC(strlen(applicationName) + 11);
    strcat(strcpy(appli_res, applicationName), ".undoStack");
    class_res = (char*)GMALLOC(strlen(className) + 11);
    strcat(strcpy(class_res, className), ".undoStack");
    if (XrmGetResource(applicationDB, appli_res, class_res, lpr_str, &value)) {
	char* undo_value = (char*)GMALLOC((int)value.size + 1);
	strncpy(undo_value, value.addr, (int)value.size);
	undoMgr.maxCount(atoi(undo_value));
	GFREE(undo_value);
    }
    GFREE(class_res);
    GFREE(appli_res);

    GFREE(all_name);
    
    /*
     * Creation of cursors and pixmaps.
     */
    CreateCursors(toplevel);
    CreatePixmaps(toplevel);

    /*
     * Creation of MainWindow and the MenuBar
     */
    main_window = (void*) XmCreateMainWindow(toplevel, "mainwindow", NULL, 0);
    XtManageChild((Widget)main_window);

    Widget menu = CreateMenuBar((Widget)main_window);
    
    /*
     * Creation of the panel
     */
    Widget form = XmCreateForm((Widget)main_window, "form", NULL, 0);
    XtManageChild(form);
    
    int n = 0;
    XtSetArg(args[n], XmNworkWindow, form); n++;
    XtSetArg(args[n], XmNmenuBar, menu); n++;
    XtSetValues((Widget)main_window, args, n);

    n = 0;
    XtSetArg(args[n], XmNtraversalOn, False); n++;
    XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNleftOffset, 10); n++;
    XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNtopOffset, 10); n++;
    XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNbottomOffset, 10); n++;
    Widget frame = XmCreateFrame(form, "frame", args, n);
    XtManageChild(frame);

    XtSetArg(args[0], XmNtraversalOn, False);
    Widget rc = XmCreateRowColumn(frame, "rc", args, 1);
    XtManageChild(rc);

    n = 0;
    XtSetArg(args[0], XmNtraversalOn, False);
    XtSetArg(args[1], XmNlabelType, XmPIXMAP);
    
    XtSetArg(args[2], XmNlabelInsensitivePixmap, iPolylineIcon);
    XtSetArg (args[3], XmNlabelPixmap, polylineIcon);
    createBtn[n] = XmCreatePushButtonGadget (rc, "Polyline", args, 4);
    XtAddCallback (createBtn[n], XmNactivateCallback,
		   (XtCallbackProc)CreatePolyline, NULL);
    XtAddCallback (createBtn[n], XmNhelpCallback,
		   (XtCallbackProc)MapHelpFile, POLYLINE_CR_FILE);
    n++;

    XtSetArg(args[2], XmNlabelInsensitivePixmap, iPolygonIcon);
    XtSetArg (args[3], XmNlabelPixmap, polygonIcon);
    createBtn[n] = XmCreatePushButtonGadget (rc, "Polygon", args, 4);
    XtAddCallback (createBtn[n], XmNactivateCallback,
		   (XtCallbackProc)CreatePolygon, NULL);
    XtAddCallback (createBtn[n], XmNhelpCallback,
		   (XtCallbackProc)MapHelpFile, POLYGON_CR_FILE);
    n++;

    XtSetArg(args[2], XmNlabelInsensitivePixmap, iHexagonIcon);
    XtSetArg (args[3], XmNlabelPixmap, hexagonIcon);
    createBtn[n] = XmCreatePushButtonGadget (rc, "Hexagon", args, 4);
    XtAddCallback (createBtn[n], XmNactivateCallback,
		   (XtCallbackProc)CreateHexagon, NULL);
    XtAddCallback (createBtn[n], XmNhelpCallback,
		   (XtCallbackProc)MapHelpFile, REGPOLYGON_CR_FILE);
    n++;

    XtSetArg(args[2], XmNlabelInsensitivePixmap, iRectangleIcon);
    XtSetArg (args[3], XmNlabelPixmap, rectangleIcon);
    createBtn[n] = XmCreatePushButtonGadget (rc, "Rectangle", args, 4);
    XtAddCallback (createBtn[n], XmNactivateCallback,
		   (XtCallbackProc)CreateRectangle, NULL);
    XtAddCallback (createBtn[n], XmNhelpCallback,
		   (XtCallbackProc)MapHelpFile, RECTANGLE_CR_FILE);
    n++;

    XtSetArg(args[2], XmNlabelInsensitivePixmap, iSquareIcon);
    XtSetArg (args[3], XmNlabelPixmap, squareIcon);
    createBtn[n] = XmCreatePushButtonGadget (rc, "Square", args, 4);
    XtAddCallback (createBtn[n], XmNactivateCallback,
		   (XtCallbackProc)CreateSquare, NULL);
    XtAddCallback (createBtn[n], XmNhelpCallback,
		   (XtCallbackProc)MapHelpFile, SQUARE_CR_FILE);
    n++;

    XtSetArg(args[2], XmNlabelInsensitivePixmap, iEllipseArcIcon);
    XtSetArg (args[3], XmNlabelPixmap, ellipseArcIcon);
    createBtn[n] = XmCreatePushButtonGadget (rc, "EllipseArc", args, 4);
    XtAddCallback (createBtn[n], XmNactivateCallback,
		   (XtCallbackProc)CreateEllipseArc, NULL);
    XtAddCallback (createBtn[n], XmNhelpCallback,
		   (XtCallbackProc)MapHelpFile, ARC_ELLIPSE_CR_FILE);
    n++;

    XtSetArg(args[2], XmNlabelInsensitivePixmap, iCircleArcIcon);
    XtSetArg (args[3], XmNlabelPixmap, circleArcIcon);
    createBtn[n] = XmCreatePushButtonGadget (rc, "CircleArc", args, 4);
    XtAddCallback (createBtn[n], XmNactivateCallback,
		   (XtCallbackProc)CreateCircleArc, NULL);
    XtAddCallback (createBtn[n], XmNhelpCallback,
		   (XtCallbackProc)MapHelpFile, ARC_CIRCLE_CR_FILE);
    n++;

    XtSetArg(args[2], XmNlabelInsensitivePixmap, iEllipseIcon);
    XtSetArg (args[3], XmNlabelPixmap, ellipseIcon);
    createBtn[n] = XmCreatePushButtonGadget (rc, "Ellipse", args, 4);
    XtAddCallback (createBtn[n], XmNactivateCallback,
		   (XtCallbackProc)CreateEllipse, NULL);
    XtAddCallback (createBtn[n], XmNhelpCallback,
		   (XtCallbackProc)MapHelpFile, ELLIPSE_CR_FILE);
    n++;

    XtSetArg(args[2], XmNlabelInsensitivePixmap, iCircleIcon);
    XtSetArg (args[3], XmNlabelPixmap, circleIcon);
    createBtn[n] = XmCreatePushButtonGadget (rc, "Circle", args, 4);
    XtAddCallback (createBtn[n], XmNactivateCallback,
		   (XtCallbackProc)CreateCircle, NULL);
    XtAddCallback (createBtn[n], XmNhelpCallback,
		   (XtCallbackProc)MapHelpFile, CIRCLE_CR_FILE);
    n++;

    XtSetArg(args[2], XmNlabelInsensitivePixmap, iHandIcon);
    XtSetArg (args[3], XmNlabelPixmap, handIcon);
    createBtn[n] = XmCreatePushButtonGadget (rc, "Hand", args, 4);
    XtAddCallback (createBtn[n], XmNactivateCallback,
		   (XtCallbackProc)CreateHand, NULL);
    XtAddCallback (createBtn[n], XmNhelpCallback,
		   (XtCallbackProc)MapHelpFile, HAND_CR_FILE);
    n++;

    XtSetArg(args[2], XmNlabelInsensitivePixmap, iImageIcon);
    XtSetArg (args[3], XmNlabelPixmap, imageIcon);
    createBtn[n] = XmCreatePushButtonGadget (rc, "Image", args, 4);
    XtAddCallback (createBtn[n], XmNactivateCallback,
		   (XtCallbackProc)CreateImage, NULL);
    XtAddCallback (createBtn[n], XmNhelpCallback,
		   (XtCallbackProc)MapHelpFile, IMAGE_CR_FILE);
    n++;

    createBtn[n] = CreateTextButton(rc);
    n++;

    createBtn[n] = CreateCurveButton(rc);
    n++;

    XtManageChildren(createBtn, n);

    /*
     * Creation of the drawing window
     */
    n = 0;
    XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNrightOffset, 10); n++;
    XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNtopOffset, 10); n++;
    XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNbottomOffset, 10); n++;
    XtSetArg(args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
    XtSetArg(args[n], XmNleftOffset, 10); n++;
    XtSetArg(args[n], XmNleftWidget, frame); n++;
    XtSetArg(args[n], XmNscrollingPolicy, XmAUTOMATIC); n++;
    XtSetArg(args[n], XmNscrollBarDisplayPolicy, XmAS_NEEDED); n++;
    scroll = XmCreateScrolledWindow(form, "scroll", args, n);
    XtManageChild(scroll);

    n = 0;
    XtSetArg(args[n], XmNresizePolicy, XmRESIZE_GROW); n++;
    graphic = XbCreateGraphic(scroll, "graphic", args, n);
    XtManageChild((Widget)graphic);

    XmScrolledWindowSetAreas(scroll, NULL, NULL, (Widget)graphic);

    Widget hs, vs;
    n = 0;
    XtSetArg(args[n], XmNhorizontalScrollBar, &hs); n++;
    XtSetArg(args[n], XmNverticalScrollBar, &vs); n++;
    XtGetValues(scroll, args, n);
    n = 0;
    XtSetArg(args[n], XmNtraversalOn, False); n++;
    XtSetValues(hs, args, n);
    XtSetValues(vs, args, n);

    CreatePolypointMenus((Widget)graphic);
    
    /*
     * Realize all widget.
     */
    XtRealizeWidget(toplevel);

    /*
     * Initialization of Go
     * Widget must be realized before
     * getting the graphic widget interface.
     */

    void* xinterf = (void*)XbGraphicGoInterf((XbGraphicWidget) graphic);
    InitGo(xinterf);

    /*
     * Initialization of format and orientation menus.
     */
    XmToggleButtonGadgetSetState(orientation_tg[DEFAULT_ORIENTATION], True,
				 True);
    XmToggleButtonGadgetSetState(form_tg[DEFAULT_FORMAT], True,
				 True);

    
    RunEventLoop();
}



void SensitiveCreateBtn(void* /*w*/, int /*yes*/)
{
    /**  philippe ************************************
      
      go-1.2: on each interaction: the pointer is grabbed
      inside the graphic window. Cf GoInterf::grab().
      So we do not have to set insensitive the other
      creation push button
      Arg arg;
      XtSetArg(arg, XmNsensitive, yes);
      for (int i = 0; i < 13; i++)
      if (createBtn[i] != w)
      XtSetValues(createBtn[i], &arg, 1);
      //    for (i = 0; i < 8; i++)
      //	XtSetValues(menubarBtn[i], &arg, 1);
      
      *********************************************/
}
