#ifdef REV_INFO
static char rcsid[] = "$Id: menubar.cc,v 1.4 1992/12/10 16:24:46 jojic Exp $";
#endif
/*****************************************************************************
MODULE:  Go Editor
FILE:    menubar.cc
AUTHOR:  Aline Jacomet
DATE:    May 14th 1991

DESCRIPTION:
    creation of menubar of the main window.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include <Go/Tools/Rgb.h>
#include <Go/Tools/Alloc.h>

#include <Go/BGoUIEd.h>
#include "main.h"

#include <Xm/RowColumn.h>
#include <Xm/CascadeBG.h>
#include <Xm/PushBG.h>
#include <Xm/PushB.h>
#include <Xm/ToggleBG.h>
#include <Xm/SeparatoG.h>



#define MENUBAR      9
#define FILE_MENU    9
#define EDITION      14
#define STRUCT       7
#define SELECT       7
#define ATTRIB       8
#define NAMED_ATTRIB 3
#define VIEW         4
#define MISCEL       4
#define HELP         8
#define ORIENTATION  2
#define FORMAT       7
#define COLOR        15
#define FONT         13
#define LINEWIDTH    10



/*
 * EXTERN FUNCTIONS
 */

// in file filemenu.cc;
extern void ActiveLoadBox(void*, void*, caddr_t);    
extern void ActiveSaveBox(void*, void*, caddr_t);    
extern void ActivePostscriptBox(void*, void*, caddr_t);
extern void NewDialog(void*, void*, caddr_t);
extern void QuitDialog(void*, void*, caddr_t);

// in file goAttrib.cc;
extern void ModifyGoColor(GRgb*, char*);
extern void ModifyGoBackground(int);
extern void ModifyGoOutline(int);
extern void ModifyGoLineWidth(double);
extern void ModifyGoFontWithoutHeight(char*);

// in file goLoadSave.cc;
extern void Print(void*, void*, caddr_t);

// in file goEdition.cc;
extern void Move(void*, void*, caddr_t);    
extern void RescaleAll(void*, void*, caddr_t);    
extern void Align(void*, int);    
extern void Rotate(void*, void*, caddr_t);    
extern void Deform(void*, void*, caddr_t);    
extern void Copy(void*, void*, caddr_t);    
extern void Paste(void*, void*, caddr_t);    
extern void Cut(void*, void*, caddr_t);    
extern void CancelGrabbingSensors (class BGoUIEditor*);
    
// in file goStructure.cc;
extern void CurParent(void*, void*, caddr_t);    
extern void Reparent(void*, void*, caddr_t);    
extern void Group(void*, void*, caddr_t);    
extern void Ungroup(void*, void*, caddr_t);    
extern void Leaf(void*, void*, caddr_t);    
extern void Unleaf(void*, void*, caddr_t);    
extern void RaiseAndLower();

// in file goSelection.cc;
extern void SelectAll(void*, void*, caddr_t);    
extern void SelectUpper(void*, void*, caddr_t);    
extern void SelectLower(void*, void*, caddr_t);    
extern void RubberBandSelect(void*, void*, caddr_t);    
extern void RubberBandUnselect(void*, void*, caddr_t);    

// in goUndo.cc;
extern void Undo(void*, void*, caddr_t);
extern void Redo(void*, void*, caddr_t);

// in file editattrib.cc;
extern void CurrentDialog(void*, void*, caddr_t);    
extern void ModifyDialog(void*, void*, caddr_t);    

// in file miscelmenu.cc;
extern void DisplayAssocDialog(Widget, Widget, caddr_t);
extern void Image(void*, void*, caddr_t);    
extern void AsciiFile(void*, void*, caddr_t);    
extern void ZoomDisplay(Widget, Widget parent, caddr_t);

// in file help.cc;
extern void HelpObject(Widget, void*);
extern void HelpIndex(Widget, void*);
extern void HelpKeyboard(Widget, void*);
extern void HelpHelp(Widget, void*);
extern void HelpGoDraw(Widget, void*);
extern void HelpGoInfo(Widget, void*);
extern void MapHelpFile(void*, char*, void*);
extern void QueryForHelp(Widget, caddr_t, XmAnyCallbackStruct*);
extern void MapInfoMessage(Widget, Widget);

// in file editmenu.cc;
extern void DisplayAlignDialog(Widget, Widget, caddr_t);
extern void DisplaySpacingDialog(Widget, Widget, caddr_t);
extern void DisplayRotateDialog(Widget, Widget, caddr_t);
extern void DisplayMirrorDialog(Widget, Widget, caddr_t);

// in file namedattrib.cc;
extern void NamedDialog(Widget, Widget, caddr_t);
extern void SaveNamedObjects(Widget, Widget, caddr_t);
extern void DestroyNamedObjects(Widget, Widget, caddr_t);

// in file viewmenu.cc;
extern void ChangeFormat(Widget, int, XmToggleButtonCallbackStruct*);
extern void ChangeOrientation(Widget, int, XmToggleButtonCallbackStruct*);

// in file colorsel.cc;
extern void MapColorList(Widget, Widget, caddr_t);

// in file fontsel.cc;
extern void MapFontList(Widget, int, caddr_t);

// in file message.cc;
extern void NotImplemented(void*, void*, caddr_t);



void* current_entry;
void* modif_entry;
void* pen_modif_entry;
void* brush_modif_entry;
void* text_modif_entry;
void* line_modif_entry;
void* font_modif_entry;

Widget orientation_tg[ORIENTATION];
Widget form_tg[FORMAT];
Widget menubarBtn[MENUBAR];

char* colorname[COLOR];
XColor color[COLOR - 2];
int nb_color;

Display* dpy;
Colormap cmap;

static Widget color_tg[3][COLOR];
static Widget lw_tg[LINEWIDTH];
static Widget font_tg[FONT];

extern int WhichAction;


/*
 * Widget's names
 */
static char* cbname[] = {"File", "Edit", "Structure", "Selection", "Attributes", "Named Attributes", "View", "Miscellaneous", "Help"};
static char* filename[] = {"New", "Separator", "Load", "Save", "Separator", "Print", "To PostScript", "Separator", "Quit"};
static char* editname[] = {"Move", "Rescale", "Rotate", "Predefined Rotate", "Deform", "Mirror Images", "Align", "Spacing", "Copy", "Paste", "Cut", "Separator", "Undo", "Redo"};
static char* structname[] = {"Current Parent", "Reparent", "Group", "Ungroup", "Leaf", "Unleaf", "Raise/Lower"};
static char* selectname[] = {"Select Lower Level", "Select Upper Level", "Select Some Objects", "Select All", "Separator", "Unselect Some Objects", "Unselect All"};
static char* attribname[] = {"Current", "Modify", "Separator", "PenColor", "LineWidth", "BrushColor", "TextColor", "Font"};
static char* namedname[] = {"EditNamedAttrib", "SaveNamedAttrib", "DestroyNamedAttrib"};
static char* viewname[] = {"Zoom", "Separator", "Format", "Orientation"};
static char* miscelname[] = {"Assoc Management", "Separator", "Load An Ascii File", "Load A Gif File"};
static char* helpname[] = {"Context-Sensitive", "Separator", "Overview", "Index", "Keyboard", "Using Help", "GoDraw Information", "Go Information"};
static char* formname[] = {"A5", "A4", "A3", "American Letter", "B5", "B4", "Automatic Resize"};
static char* orientname[] = {"Portrait", "Landscape"};
static char* lwname[] = {"1", "2", "3", "4", "5", "6", "7", "8", "9", "10"};
static char* fontname[] = {"Courier Roman", "Courier Italic", "Courier Bold", "Courier Bold Italic", "Helvetica Roman", "Helvetica Italic", "Helvetica Bold", "Helvetica Bold Italic", "Times Roman", "Times Italic", "Times Bold", "Times Bold Italic", "Others"};

static char* font_xname[] = {"-adobe-courier-medium-r-*--*-*-*-*-*-*-iso8859-1", "-adobe-courier-medium-o-*--*-*-*-*-*-*-iso8859-1", "-adobe-courier-bold-r-*--*-*-*-*-*-*-iso8859-1", "-adobe-courier-bold-o-*--*-*-*-*-*-*-iso8859-1", "-adobe-helvetica-medium-r-*--*-*-*-*-*-*-iso8859-1", "-adobe-helvetica-medium-o-*--*-*-*-*-*-*-iso8859-1", "-adobe-helvetica-bold-r-*--*-*-*-*-*-*-iso8859-1", "-adobe-helvetica-bold-o-*--*-*-*-*-*-*-iso8859-1", "-adobe-times-medium-r-*--*-*-*-*-*-*-iso8859-1", "-adobe-times-medium-i-*--*-*-*-*-*-*-iso8859-1", "-adobe-times-bold-r-*--*-*-*-*-*-*-iso8859-1", "-adobe-times-bold-i-*--*-*-*-*-*-*-iso8859-1"};


/*
 * Callbacks
 */
static void(* filecall[FILE_MENU])(void*, void*, caddr_t);
static void(* editcall[EDITION])(void*, void*, caddr_t);
static void(* structcall[STRUCT])(void*, void*, caddr_t);
static void(* selectcall[SELECT])(void*, void*, caddr_t);
static void(* attribcall[ATTRIB])(void*, void*, caddr_t);
static void(* namedcall[NAMED_ATTRIB])(Widget, Widget, caddr_t);
static void(* viewcall[VIEW])(void*, void*, caddr_t);
static void(* miscelcall[MISCEL])(void*, void*, caddr_t);
static void(* helpcall[HELP])(void*, void*, caddr_t);

static void ChangeColor(Widget, int, caddr_t);
static void ChangeLineWidth(Widget, int, caddr_t);
static void ChangeFont(Widget, char*, caddr_t);


    
void menubar_unselect_all (void*, void*, caddr_t)
{
    extern void CancelGrabbingSensors (BGoUIEditor*);
    extern GoRoot* goroot;
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
    CancelGrabbingSensors (bgoui);
    bgoui->unselectAll();
}



Widget CreateMenuBar(Widget parent)
{
    filecall[0]   = NewDialog;
    filecall[1]   = NULL;
    filecall[2]   = ActiveLoadBox;
    filecall[3]   = ActiveSaveBox;
    filecall[4]   = NULL;
    filecall[5]   = Print;
    filecall[6]   = ActivePostscriptBox;
    filecall[7]   = NULL;
    filecall[8]   = QuitDialog;
    editcall[0]   = Move;
    editcall[1]   = RescaleAll;
    editcall[2]   = Rotate;
    editcall[3]   = (void (*)(void *, void *, caddr_t))DisplayRotateDialog;
    editcall[4]   = Deform;
    editcall[5]   = (void (*)(void *, void *, caddr_t))DisplayMirrorDialog;
    editcall[6]   = (void (*)(void *, void *, caddr_t))DisplayAlignDialog;
    editcall[7]   = (void (*)(void *, void *, caddr_t))DisplaySpacingDialog;
    editcall[8]   = Copy;
    editcall[9]   = Paste;
    editcall[10]  = Cut;
    editcall[11]  = NULL;
    editcall[12]  = Undo;
    editcall[13]  = Redo;
    structcall[0] = CurParent;
    structcall[1] = Reparent;
    structcall[2] = Group;
    structcall[3] = Ungroup;
    structcall[4] = Leaf;
    structcall[5] = Unleaf;
    structcall[6] = (void (*)(void *, void *, caddr_t)) RaiseAndLower;
    selectcall[0] = SelectLower;
    selectcall[1] = SelectUpper;
    selectcall[2] = RubberBandSelect;
    selectcall[3] = SelectAll;
    selectcall[4] = NULL;
    selectcall[5] = RubberBandUnselect;
    selectcall[6] = menubar_unselect_all;
    attribcall[0] = CurrentDialog;
    attribcall[1] = ModifyDialog;
    attribcall[2] = NULL;
    attribcall[3] = NULL;
    attribcall[4] = NULL;
    attribcall[5] = NULL;
    attribcall[6] = NULL;
    attribcall[7] = NULL;
    namedcall[0]  = NamedDialog;
    namedcall[1]  = SaveNamedObjects;
    namedcall[2]  = DestroyNamedObjects;
    viewcall[0]   = (void (*)(void *, void *, caddr_t))ZoomDisplay;
    miscelcall[0] = (void (*)(void *, void *, caddr_t)) DisplayAssocDialog;
    miscelcall[1] = NULL;
    miscelcall[2] = AsciiFile;
    miscelcall[3] = Image;
    helpcall[0]   = (void (*)(void *, void *, caddr_t))QueryForHelp;
    helpcall[1]   = NULL;
    helpcall[2]   = (void (*)(void *, void *, caddr_t))MapHelpFile;
    helpcall[3]   = (void (*)(void *, void *, caddr_t))MapHelpFile;
    helpcall[4]   = (void (*)(void *, void *, caddr_t))MapHelpFile;
    helpcall[5]   = (void (*)(void *, void *, caddr_t))MapHelpFile;
    helpcall[6]   = (void (*)(void *, void *, caddr_t))MapHelpFile;
    helpcall[7]   = (void (*)(void *, void *, caddr_t))MapHelpFile;

    Widget pmp[MENUBAR];
    Arg args[5];

    Widget menu = XmCreateMenuBar(parent, "menubar", NULL, 0);
    XtManageChild(menu);

    char* cbdata[MENUBAR];
    cbdata[0] = FILE_FILE;
    cbdata[1] = EDIT_FILE;
    cbdata[2] = STRUCTURE_FILE;
    cbdata[3] = SELECTION_FILE;
    cbdata[4] = ATTRIBUTES_FILE;
    cbdata[5] = NAM_ATTRIBUTES_FILE;
    cbdata[4] = VIEW_FILE;
    cbdata[7] = MISCEL_FILE;
    cbdata[8] = HELP_FILE;
    for (int i = 0; i < MENUBAR; i++) {
	pmp[i] = XmCreatePulldownMenu(menu, "pull", NULL, 0);
	XtSetArg(args[0], XmNsubMenuId, pmp[i]);
	menubarBtn[i] = XmCreateCascadeButtonGadget(menu, cbname[i], args, 1);
	XtAddCallback(menubarBtn[i], XmNhelpCallback,
		      (XtCallbackProc)MapHelpFile, (XtPointer)cbdata[i]);
    }
    XtManageChildren(menubarBtn, MENUBAR);
    XtManageChildren(pmp, MENUBAR);


    Widget button;
    
    /*
     * File menu
     */
    void* filedata[FILE_MENU];
    filedata[0] = filedata[2] = filedata[3] = filedata[6] = filedata[8] =
	main_window;
    filedata[1] = filedata[4] = filedata[5] = filedata[7] = NULL;

    button = XmCreatePushButtonGadget(pmp[0], filename[0], NULL, 0);
    XtAddCallback(button, XmNactivateCallback, (XtCallbackProc)filecall[0],
		  (XtPointer)filedata[0]);
    XtManageChild(button);
    button = XmCreateSeparatorGadget(pmp[0], filename[1], NULL, 0);
    XtManageChild(button);
    for (i = 2; i < 4; i++) {
	button = XmCreatePushButtonGadget(pmp[0], filename[i], NULL, 0);
	XtManageChild(button);
	XtAddCallback(button, XmNactivateCallback,
		      (XtCallbackProc)filecall[i], (XtPointer)filedata[i]);
    }
    button = XmCreateSeparatorGadget(pmp[0], filename[4], NULL, 0);
    XtManageChild(button);
    for (i = 5; i < 7; i++) {
	button = XmCreatePushButtonGadget(pmp[0], filename[i],
					  NULL, 0);
	XtAddCallback(button, XmNactivateCallback,
		      (XtCallbackProc)filecall[i],
		      (XtPointer)filedata[i]);
	XtManageChild(button);
    }
    button = XmCreateSeparatorGadget(pmp[0], filename[7], NULL, 0);
    XtManageChild(button);
    button = XmCreatePushButtonGadget(pmp[0], filename[FILE_MENU-1],
				      NULL, 0);
    XtAddCallback(button, XmNactivateCallback,
		  (XtCallbackProc)filecall[FILE_MENU-1],
		  (XtPointer)filedata[FILE_MENU-1]);
    XtManageChild(button);
    
    /*
     * Edit menu
     */
    void* editdata[EDITION];
    editdata[0] = editdata[1] = editdata[2] = editdata[4] =
	editdata[8] = editdata[10] = editdata[11] = editdata[12] =
	    editdata[13] = NULL;
    editdata[3] = editdata[5] = editdata[6] = editdata[7] = editdata[9] =
	main_window;
    for (i = 0; i < EDITION - 3; i++) {
	button = XmCreatePushButtonGadget(pmp[1], editname[i], NULL, 0);
	XtManageChild(button);
	XtAddCallback(button, XmNactivateCallback,
		      (XtCallbackProc)editcall[i],
		      (XtPointer)editdata[i]);
    }
    button = XmCreateSeparatorGadget(pmp[1], editname[EDITION-3], NULL, 0);
    XtManageChild(button);
    for (i = EDITION - 2; i < EDITION; i++) {
	button = XmCreatePushButtonGadget(pmp[1], editname[i], NULL, 0);
	XtAddCallback(button, XmNactivateCallback,
		      (XtCallbackProc)editcall[i],
		      (XtPointer)editdata[i]);
	XtManageChild(button);
    }

    /*
     * Structure menu
     */
    void* structdata[STRUCT];
    structdata[0] = main_window;
    structdata[1] = structdata[2] = structdata[3] = structdata[4] =
	structdata[5] = structdata[6] = NULL;
    for (i = 0; i < STRUCT; i++) {
	button = XmCreatePushButtonGadget(pmp[2], structname[i], NULL, 0);
	XtManageChild(button);
	XtAddCallback(button, XmNactivateCallback,
		      (XtCallbackProc)structcall[i],
		      (XtPointer)structdata[i]);
    }

    
    /*
     * Selection menu
     */
    void* selectdata[SELECT];
    selectdata[0] = selectdata[1] = selectdata[2] = selectdata[3]
	= selectdata[5] = selectdata[6] = main_window;
    selectdata[4] = NULL;
    for (i = 0; i < 4; i++) {
	button = XmCreatePushButtonGadget(pmp[3], selectname[i], NULL, 0);
	XtManageChild(button);
	XtAddCallback(button, XmNactivateCallback,
		      (XtCallbackProc)selectcall[i],
		      (XtPointer)selectdata[i]);
    }
    button = XmCreateSeparatorGadget(pmp[3], selectname[4], NULL, 0);
    XtManageChild(button);
    for (i = 5; i < SELECT; i++) {
	button = XmCreatePushButtonGadget(pmp[3], selectname[i], NULL, 0);
	XtManageChild(button);
	XtAddCallback(button, XmNactivateCallback,
		      (XtCallbackProc)selectcall[i], (XtPointer)selectdata[i]);
    }

    
    /*
     * Attributes menu
     */
    void* attribdata[ATTRIB];
    attribdata[0] = attribdata[1] = attribdata[3] = attribdata[4] =
	attribdata[5] = attribdata[6] = attribdata[7] = main_window;
    attribdata[2] = NULL;

    /*
     * Current and Modify Attributes
     */
    for (i = 0; i < 2; i++) {
	button = XmCreatePushButtonGadget(pmp[4], attribname[i], NULL, 0);
	XtManageChild(button);
	XtAddCallback(button, XmNactivateCallback,
		      (XtCallbackProc)attribcall[i],
		      (XtPointer)attribdata[i]);
	if (i == 0)
	    current_entry = button;
	else
	    modif_entry = button;
    }
    button = XmCreateSeparatorGadget(pmp[4], attribname[2], NULL, 0);
    XtManageChild(button);

    dpy = XtDisplay((Widget)main_window);
    int screen = DefaultScreen(dpy);
    cmap = DefaultColormap(dpy, screen);
    Visual* visual = DefaultVisual(dpy, screen);

    colorname[13] = (char*)GMALLOC(strlen("None") + 1);
    strcpy(colorname[13], "None");
    colorname[14] = (char*)GMALLOC(strlen("Others") + 1);
    strcpy(colorname[14], "Others");

    if ((visual->c_class == StaticGray || visual->c_class == GrayScale) &&
	visual->map_entries == 2) {
	color[0].red = 65535;
	color[0].green = 65535;
	color[0].blue = 65535;
	color[0].flags = DoRed | DoGreen | DoBlue;
	color[1].red = 0;
	color[1].green = 0;
	color[1].blue = 0;
	color[1].flags = DoRed | DoGreen | DoBlue;
	colorname[0] = (char*)GMALLOC(strlen("White") + 1);
	strcpy(colorname[0], "White");
	colorname[1] = (char*)GMALLOC(strlen("Black") + 1);
	strcpy(colorname[1], "Black");
	nb_color = 2;
    } else
	if ((visual->c_class == StaticGray || visual->c_class == GrayScale) &&
	    visual->map_entries == 4) {
	    color[0].red = 65535;
	    color[0].green = 65535;
	    color[0].blue = 65535;
	    color[0].flags = DoRed | DoGreen | DoBlue;
	    color[1].red = 54016;
	    color[1].green = 54016;
	    color[1].blue = 54016;
	    color[1].flags = DoRed | DoGreen | DoBlue;
	    color[2].red = 26880;
	    color[2].green = 26880;
	    color[2].blue = 26880;
	    color[2].flags = DoRed | DoGreen | DoBlue;
	    color[3].red = 0;
	    color[3].green = 0;
	    color[3].blue = 0;
	    color[3].flags = DoRed | DoGreen | DoBlue;
	    colorname[0] = (char*)GMALLOC(strlen("White") + 1);
	    strcpy(colorname[0], "White");
	    colorname[1] = (char*)GMALLOC(strlen("Light Gray") + 1);
	    strcpy(colorname[1], "Light Gray");
	    colorname[2] = (char*)GMALLOC(strlen("Dim Gray") + 1);
	    strcpy(colorname[2], "Dim Gray");
	    colorname[3] = (char*)GMALLOC(strlen("Black") + 1);
	    strcpy(colorname[3], "Black");
	    nb_color = 4;
	} else {
	    color[0].red = 65535;
	    color[0].green = 65535;
	    color[0].blue = 65535;
	    color[0].flags = DoRed | DoGreen | DoBlue;
	    color[1].red = 0;
	    color[1].green = 0;
	    color[1].blue = 0;
	    color[1].flags = DoRed | DoGreen | DoBlue;
	    color[2].red = 65535;
	    color[2].green = 0;
	    color[2].blue = 0;
	    color[2].flags = DoRed | DoGreen | DoBlue;
	    color[3].red = 65535;
	    color[3].green = 42240;
	    color[3].blue = 0;
	    color[3].flags = DoRed | DoGreen | DoBlue;
	    color[4].red = 42240;
	    color[4].green = 10752;
	    color[4].blue = 10752;
	    color[4].flags = DoRed | DoGreen | DoBlue;
	    color[5].red = 0;
	    color[5].green = 65535;
	    color[5].blue = 0;
	    color[5].flags = DoRed | DoGreen | DoBlue;
	    color[6].red = 0;
	    color[6].green = 0;
	    color[6].blue = 65535;
	    color[6].flags = DoRed | DoGreen | DoBlue;
	    color[7].red = 0;
	    color[7].green = 65535;
	    color[7].blue = 65535;
	    color[7].flags = DoRed | DoGreen | DoBlue;
	    color[8].red = 65535;
	    color[8].green = 0;
	    color[8].blue = 65535;
	    color[8].flags = DoRed | DoGreen | DoBlue;
	    color[9].red = 37888;
	    color[9].green = 0;
	    color[9].blue = 54016;
	    color[9].flags = DoRed | DoGreen | DoBlue;
	    color[10].red = 65535;
	    color[10].green = 65535;
	    color[10].blue = 0;
	    color[10].flags = DoRed | DoGreen | DoBlue;
	    color[11].red = 54016;
	    color[11].green = 54016;
	    color[11].blue = 54016;
	    color[11].flags = DoRed | DoGreen | DoBlue;
	    color[12].red = 26880;
	    color[12].green = 26880;
	    color[12].blue = 26880;
	    color[12].flags = DoRed | DoGreen | DoBlue;
	    colorname[0] = (char*)GMALLOC(strlen("White") + 1);
	    strcpy(colorname[0], "White");
	    colorname[1] = (char*)GMALLOC(strlen("Black") + 1);
	    strcpy(colorname[1], "Black");
	    colorname[2] = (char*)GMALLOC(strlen("Red") + 1);
	    strcpy(colorname[2], "Red");
	    colorname[3] = (char*)GMALLOC(strlen("Orange") + 1);
	    strcpy(colorname[3], "Orange");
	    colorname[4] = (char*)GMALLOC(strlen("Brown") + 1);
	    strcpy(colorname[4], "Brown");
	    colorname[5] = (char*)GMALLOC(strlen("Green") + 1);
	    strcpy(colorname[5], "Green");
	    colorname[6] = (char*)GMALLOC(strlen("Blue") + 1);
	    strcpy(colorname[6], "Blue");
	    colorname[7] = (char*)GMALLOC(strlen("Cyan") + 1);
	    strcpy(colorname[7], "Cyan");
	    colorname[8] = (char*)GMALLOC(strlen("Magenta") + 1);
	    strcpy(colorname[8], "Magenta");
	    colorname[9] = (char*)GMALLOC(strlen("Dark Violet") + 1);
	    strcpy(colorname[9], "Dark Violet");
	    colorname[10] = (char*)GMALLOC(strlen("Yellow") + 1);
	    strcpy(colorname[10], "Yellow");
	    colorname[11] = (char*)GMALLOC(strlen("Light Gray") + 1);
	    strcpy(colorname[11], "Light Gray");
	    colorname[12] = (char*)GMALLOC(strlen("Dim Gray") + 1);
	    strcpy(colorname[12], "Dim Gray");
	    nb_color = 13;
	}
    
    /*
     * GPen GColor
     */
    Widget pmp1 = XmCreatePulldownMenu(pmp[4], "pull1", NULL, 0);
    XtSetArg(args[0], XmNsubMenuId, pmp1);
    pen_modif_entry = XmCreateCascadeButtonGadget(pmp[4], attribname[3],
						  args, 1);
    XtManageChild((Widget)pen_modif_entry);
    for (i = 0; i < nb_color; i++) {
	/*
	 * GColor's Allocation
	 */
	if (!XAllocColor(dpy, cmap, &color[i]))
	    color_tg[0][i] = NULL;
	else {
	    XtSetArg(args[0], XmNbackground, color[i].pixel);
	    color_tg[0][i] = XmCreatePushButton(pmp1, colorname[i], args, 1);
	    XtManageChild(color_tg[0][i]);
	    XtAddCallback(color_tg[0][i], XmNactivateCallback,
			  (XtCallbackProc)ChangeColor, (XtPointer)i);
	}
    }
    color_tg[0][COLOR-2] = XmCreatePushButton(pmp1, colorname[COLOR-2],
					      NULL, 0);
    XtManageChild(color_tg[0][COLOR-2]);
    XtAddCallback(color_tg[0][COLOR-2], XmNactivateCallback,
		  (XtCallbackProc)ChangeColor, (XtPointer)(COLOR-2));
    color_tg[0][COLOR-1] = XmCreatePushButton(pmp1, colorname[COLOR-1],
					      NULL, 0);
    XtManageChild(color_tg[0][COLOR-1]);
    XtAddCallback(color_tg[0][COLOR-1], XmNactivateCallback,
		  (XtCallbackProc)ChangeColor, (XtPointer)(COLOR-1));

    /*
     * Line Width
     */
    pmp1 = XmCreatePulldownMenu(pmp[4], "pull1", NULL, 0);
    XtSetArg(args[0], XmNsubMenuId, pmp1);
    line_modif_entry = XmCreateCascadeButtonGadget(pmp[4], attribname[4],
						   args, 1);
    XtManageChild((Widget)line_modif_entry);
    XtSetArg(args[0], XmNlabelType, XmPIXMAP);
    for (i = 0; i < 10; i++) {
	XtSetArg (args[1], XmNlabelPixmap, lwIcon[i]);
	lw_tg[i] = XmCreatePushButtonGadget(pmp1, lwname[i], args, 2);
	XtManageChild(lw_tg[i]);
	XtAddCallback(lw_tg[i], XmNactivateCallback,
		      (XtCallbackProc)ChangeLineWidth, (XtPointer)(i+1));
    }

    
    /*
     * GBrush and GTextBrush GColor
     */
    for (int j = 1; j < 3; j++) {
	pmp1 = XmCreatePulldownMenu(pmp[4], "pull1", NULL, 0);
	XtSetArg(args[0], XmNsubMenuId, pmp1);
	if (j == 1) {
	    brush_modif_entry = XmCreateCascadeButtonGadget(pmp[4],
							    attribname[j+4],
							    args, 1);
	    XtManageChild((Widget)brush_modif_entry);
	} else {
	    text_modif_entry = XmCreateCascadeButtonGadget(pmp[4],
							   attribname[j+4],
							   args, 1);
	    XtManageChild((Widget)text_modif_entry);
	}
	for (i = 0; i < nb_color; i++) {
	    /*
	     * GColor's Allocation
	     */
	    if (!XAllocColor(dpy, cmap, &color[i]))
		color_tg[j][i] = NULL;
	    else {
		XtSetArg(args[0], XmNbackground, color[i].pixel);
		color_tg[j][i] = XmCreatePushButton(pmp1, colorname[i],
						    args, 1);
		XtManageChild(color_tg[j][i]);
		XtAddCallback(color_tg[j][i], XmNactivateCallback,
			      (XtCallbackProc)ChangeColor, (XtPointer)i);
	    }
	}
	if ( j == 1) {
	    color_tg[j][COLOR-2] = XmCreatePushButton(pmp1, colorname[COLOR-2],
						      NULL, 0);
	    XtManageChild(color_tg[j][COLOR-2]);
	    XtAddCallback(color_tg[j][COLOR-2], XmNactivateCallback,
			  (XtCallbackProc)ChangeColor, (XtPointer)(COLOR-2));
	}
	color_tg[j][COLOR-1] = XmCreatePushButton(pmp1, colorname[COLOR-1],
						  NULL, 0);
	XtManageChild(color_tg[j][COLOR-1]);
	XtAddCallback(color_tg[j][COLOR-1], XmNactivateCallback,
		      (XtCallbackProc)ChangeColor, (XtPointer)(COLOR-1));
    }

    /*
     * GFont
     */
    pmp1 = XmCreatePulldownMenu(pmp[4], "pull1", NULL, 0);
    XtSetArg(args[0], XmNsubMenuId, pmp1);
    font_modif_entry = XmCreateCascadeButtonGadget(pmp[4], attribname[7],
						   args, 1);
    XtManageChild((Widget)font_modif_entry);
    XmFontList font_list[FONT-1];
    for (i = 0; i < FONT-1; i++) {
	XFontStruct* font_str = XLoadQueryFont(dpy, font_xname[i]);
	if (font_str)
	    font_list[i] = XmFontListCreate(font_str,
					    XmSTRING_DEFAULT_CHARSET);
	else
	    font_list[i] = NULL;
    }
    for (i = 0; i < FONT-1; i++)
	if (font_list[i]) {
	    XtSetArg(args[0], XmNfontList, font_list[i]);
	    font_tg[i] = XmCreatePushButtonGadget(pmp1, fontname[i], args, 1);
	    XtManageChild(font_tg[i]);
	    XtAddCallback(font_tg[i], XmNactivateCallback,
			  (XtCallbackProc)ChangeFont,
			  (XtPointer)font_xname[i]);
	}
    font_tg[FONT-1] = XmCreatePushButtonGadget(pmp1, fontname[FONT-1],
					       NULL, 0);
    XtManageChild(font_tg[FONT-1]);
    XtAddCallback(font_tg[FONT-1], XmNactivateCallback,
		  (XtCallbackProc)ChangeFont, NULL);
    
    
    /*
     * Named Attributes Menu
     */
    void* nameddata[NAMED_ATTRIB];
    nameddata[0] = nameddata[1] = nameddata[2] = main_window;
    for (i = 0; i < NAMED_ATTRIB; i++) {
	button = XmCreatePushButtonGadget(pmp[5], namedname[i], NULL, 0);
	XtAddCallback(button, XmNactivateCallback,
		      (XtCallbackProc)namedcall[i],
		      (XtPointer)nameddata[i]);
	XtManageChild(button);
    }

    
    
    /*
     * View menu
     */
    void* viewdata[VIEW];
    viewdata[0] = main_window;
    int formdata[FORMAT];
    formdata[0] = A5;
    formdata[1] = A4;
    formdata[2] = A3;
    formdata[3] = LETTER;
    formdata[4] = B5;
    formdata[5] = B4;
    formdata[6] = AUTOMATIC_RESIZE;
    int orientdata[ORIENTATION];
    orientdata[0] = PORTRAIT;
    orientdata[1] = LANDSCAPE;

    button = XmCreatePushButtonGadget(pmp[6], viewname[0], NULL, 0);
    XtManageChild(button);
    XtAddCallback(button, XmNactivateCallback,
		  (XtCallbackProc)viewcall[0], (XtPointer)viewdata[0]);
    button = XmCreateSeparatorGadget(pmp[6], viewname[1], NULL, 0);
    XtManageChild(button);

    pmp1 = XmCreatePulldownMenu(pmp[6], "pull1", NULL, 0);
    XtSetArg(args[0], XmNsubMenuId, pmp1);
    Widget cb1 = XmCreateCascadeButtonGadget(pmp[6], viewname[2], args, 1);
    XtManageChild(cb1);
    
    for (i = 0; i < FORMAT; i++) {
	form_tg[i] = XmCreateToggleButtonGadget(pmp1, formname[i], NULL, 0);
	XtManageChild(form_tg[i]);
	XtAddCallback(form_tg[i], XmNvalueChangedCallback,
		      (XtCallbackProc)ChangeFormat, (XtPointer)formdata[i]);
    }

    Widget pmp2 = XmCreatePulldownMenu(pmp[6], "pull2", NULL, 0);
    XtSetArg(args[0], XmNsubMenuId, pmp2);
    Widget cb2 = XmCreateCascadeButtonGadget(pmp[6], viewname[3], args, 1);
    XtManageChild(cb2);
    
    for (i = 0; i < ORIENTATION; i++) {
	orientation_tg[i] = XmCreateToggleButtonGadget(pmp2, orientname[i],
						       NULL, 0);
	XtManageChild(orientation_tg[i]);
	XtAddCallback(orientation_tg[i], XmNvalueChangedCallback,
		      (XtCallbackProc)ChangeOrientation,
		      (XtPointer)orientdata[i]);
    }

    
    
    /*
     * Miscellaneous menu
     */
    void* misceldata[MISCEL];
    misceldata[1] = NULL;
    misceldata[0] = misceldata[2] = misceldata[3] = main_window;
    button = XmCreatePushButtonGadget(pmp[7], miscelname[0], NULL, 0);
    XtManageChild(button);
    XtAddCallback(button, XmNactivateCallback,
		  (XtCallbackProc)miscelcall[0], (XtPointer)misceldata[0]);
    button = XmCreateSeparatorGadget(pmp[7], miscelname[1], NULL, 0);
    XtManageChild(button);
    for (i = 2; i < MISCEL; i++) {
	button = XmCreatePushButtonGadget(pmp[7], miscelname[i], NULL, 0);
	XtManageChild(button);
	XtAddCallback(button, XmNactivateCallback,
		      (XtCallbackProc)miscelcall[i], (XtPointer)misceldata[i]);
    }
    
    /*
     * Help menu
     */
    void* helpdata[HELP];
    helpdata[0] = helpdata[1] = NULL;
    helpdata[2] = HELP_OVERVIEW_FILE;
    helpdata[3] = HELP_INDEX_FILE;
    helpdata[4] = HELP_KEYBOARD_FILE;
    helpdata[5] = HELP_FILE;
    helpdata[6] = HELP_GODRAW_FILE;
    helpdata[7] = HELP_GOINFO_FILE;
    button = XmCreatePushButtonGadget(pmp[8], helpname[0], NULL, 0);
    XtManageChild(button);
    XtAddCallback(button, XmNactivateCallback,
		  (XtCallbackProc)helpcall[0], (XtPointer)helpdata[0]);
    button = XmCreateSeparatorGadget(pmp[8], helpname[1], NULL, 0);
    XtManageChild(button);
    for (i = 2; i < HELP; i++) {
	button = XmCreatePushButtonGadget(pmp[8], helpname[i], NULL, 0);
	XtManageChild(button);
	XtAddCallback(button, XmNactivateCallback,
		      (XtCallbackProc)helpcall[i], (XtPointer)helpdata[i]);
    }
    
    int n = 0;
    XtSetArg(args[n], XmNmenuHelpWidget, menubarBtn[MENUBAR - 1]); n++;
    XtSetValues(menu, args, n);

    return menu;
}




                           /**********************
			    * COLOR MODIFICATION *
			    **********************/

extern class GoRoot* goroot;

static void ChangeColor(Widget w, int num, caddr_t)
{
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
    CancelGrabbingSensors (bgoui);

    WhichAttributes = MAIN;
    if (num < 13) {
	if (color_tg[0][num] == w) {
	    ModifyGoOutline(1);
	    WhichColor = PEN_COLOR;
	} else
	    if (color_tg[1][num] == w) {
		WhichColor = BRUSH_COLOR;
		ModifyGoBackground(1);
	    } else
		if (color_tg[2][num] == w)
		    WhichColor = TEXT_COLOR;
	GRgb rgb(color[num].red, color[num].green, color[num].blue);
	ModifyGoColor(&rgb, colorname[num]);
    } else
	if (num == 13) {
	    if (color_tg[0][num] == w) {
		// No outline;
		ModifyGoOutline(0);
	    } else
		if (color_tg[1][num] == w) {
		    // No background;
		    ModifyGoBackground(0);
		}
	} else {
	    WhichAction = FAST_MODIFY;
	    if (color_tg[0][num] == w) {
		// PEN_COLOR;
		ModifyGoOutline(1);
		WhichColor = PEN_COLOR;
		MapColorList(NULL, (Widget)main_window, NULL);
	    } else
		if (color_tg[1][num] == w) {
		    // BRUSH_COLOR;
		    ModifyGoBackground(1);
		    WhichColor = BRUSH_COLOR;
		    MapColorList(NULL, (Widget)main_window, NULL);
		} else {
		    WhichColor = TEXT_COLOR;
		    MapColorList(NULL, (Widget)main_window, NULL);
		}
	}
}




                           /***************************
			    * LINE WIDTH MODIFICATION *
			    ***************************/


static void ChangeLineWidth(Widget, int nb_pixel, caddr_t)
{
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
    CancelGrabbingSensors (bgoui);

    WhichAttributes = MAIN;
    ModifyGoLineWidth((double)(nb_pixel * pixel_size));
}



                           /*********************
			    * FONT MODIFICATION *
			    *********************/


static void ChangeFont(Widget, char* xname, caddr_t)
{
    BGoUIEditor* bgoui = (BGoUIEditor*) GoUIFrom (goroot);
    CancelGrabbingSensors (bgoui);

    if (xname)
	ModifyGoFontWithoutHeight(xname);
    else {
	WhichAction = FAST_MODIFY;
	MapFontList(NULL, MAIN, NULL);
    }
}

