#ifdef REV_INFO
static char rcsid[] = "$Id: message.cc,v 1.3 1992/12/10 16:24:49 jojic Exp $";
#endif
/*****************************************************************************
MODULE:  Go Editor
FILE:    message.cc
AUTHOR:  Patricia Langle and Aline Jacomet
DATE:    May 14th 1991

DESCRIPTION:
    message management

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include "main.h"
#include <stdio.h>

#include <Xm/MessageB.h>



/*
 * EXTERN FUNCTIONS
 */

// in file filemenu.cc;
extern void applySaveBox();
extern void applyPostscriptBox();

// in file goMiscel.cc;
extern void AssocReallyOk(void*, unsigned int, void*);

// in file pensheet.cc;
extern void VerifyPen(Widget, caddr_t, caddr_t);

// in file brushsheet.cc;
extern void VerifyBrush(Widget, caddr_t, caddr_t);

// in file textsheet.cc;
extern void VerifyTextBrush(Widget, caddr_t, caddr_t);


// in file goLoadSave.cc;
extern void DestroyIt(void*, void*);


static Widget not_implemented = NULL;
static Widget no_selected_object = NULL;
static Widget too_many_selected_objects = NULL;
static Widget not_named_object = NULL;
static Widget not_go = NULL;
static Widget object_name = NULL;
static Widget undo_empty = NULL;
static Widget redo_empty = NULL;
static Widget error = NULL;
static Widget no_file = NULL;
static Widget no_help = NULL;
static Widget no_prev_page = NULL;
static Widget no_next_page = NULL;
static Widget confirm_file_name = NULL;
static Widget confirm_PSfile_name = NULL;
static Widget confirm_key_name = NULL;
static Widget not_in_list = NULL;
#ifndef GoLICENSE
static Widget no_license = NULL;
#endif


void NotImplemented (void*, void* main_window, caddr_t)
{
    if (!not_implemented) {
	Arg args[3];
	int n = 0;
	XtSetArg(args[n], XmNautoUnmanage, True); n++;
	XtSetArg(args[n], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); n++;
	XtSetArg(args[n], XmNdefaultButtonType, XmDIALOG_CANCEL_BUTTON); n++;
	not_implemented = XmCreateInformationDialog((Widget)main_window,
						    "NotImplemented", args, n);
	XtUnmanageChild (XmMessageBoxGetChild (not_implemented, 
					       XmDIALOG_OK_BUTTON));
	XtUnmanageChild (XmMessageBoxGetChild (not_implemented, 
					       XmDIALOG_HELP_BUTTON));
    }
    XtManageChild (not_implemented);
}


#ifndef GoLICENSE
void NoLicense (void*, void* main_window, caddr_t)
{
    if (!no_license) {
	Arg args[3];
	int n = 0;
	XtSetArg(args[n], XmNautoUnmanage, True); n++;
	XtSetArg(args[n], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); n++;
	XtSetArg(args[n], XmNdefaultButtonType, XmDIALOG_CANCEL_BUTTON); n++;
	not_implemented = XmCreateInformationDialog((Widget)main_window,
						    "NoLicense", args, n);
	XtUnmanageChild (XmMessageBoxGetChild (not_implemented, 
					       XmDIALOG_OK_BUTTON));
	XtUnmanageChild (XmMessageBoxGetChild (not_implemented, 
					       XmDIALOG_HELP_BUTTON));
    }
    XtManageChild (not_implemented);
}
#endif

void NoSelectedObject (void* main_window)
{
    if (!no_selected_object) {
	Arg args[3];
	int n = 0;
	XtSetArg(args[n], XmNautoUnmanage, True); n++;
	XtSetArg(args[n], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); n++;
	XtSetArg(args[n], XmNdefaultButtonType, XmDIALOG_CANCEL_BUTTON); n++;
	no_selected_object = XmCreateWarningDialog((Widget)main_window, 
						   "NoSelectedObject",
						   args, n);
	XtUnmanageChild (XmMessageBoxGetChild (no_selected_object, 
					       XmDIALOG_OK_BUTTON));
	XtUnmanageChild (XmMessageBoxGetChild (no_selected_object, 
					       XmDIALOG_HELP_BUTTON));
    }
    XtManageChild (no_selected_object);
}


void Error (void* main_window)
{
    if (!error) {
	Arg args[3];
	int n = 0;
	XtSetArg(args[n], XmNautoUnmanage, True); n++;
	XtSetArg(args[n], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); n++;
	XtSetArg(args[n], XmNdefaultButtonType, XmDIALOG_CANCEL_BUTTON); n++;
	error = XmCreateErrorDialog((Widget)main_window, "Error", args, n);
	XtUnmanageChild (XmMessageBoxGetChild (error, 
					       XmDIALOG_OK_BUTTON));
	XtUnmanageChild (XmMessageBoxGetChild (error, 
					       XmDIALOG_HELP_BUTTON));
    }
    XtManageChild (error);
}



void NoFile (void* main_window)
{
    if (!no_file) {
	Arg args[3];
	int n = 0;
	XtSetArg(args[n], XmNautoUnmanage, True); n++;
	XtSetArg(args[n], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); n++;
	XtSetArg(args[n], XmNdefaultButtonType, XmDIALOG_CANCEL_BUTTON); n++;
	no_file = XmCreateErrorDialog ((Widget)main_window, "NoFile", args, n);
	XtUnmanageChild (XmMessageBoxGetChild (no_file, 
					       XmDIALOG_OK_BUTTON));
	XtUnmanageChild (XmMessageBoxGetChild (no_file, 
					       XmDIALOG_HELP_BUTTON));
    }
    XtManageChild (no_file);
}



void NoHelp (void* main_window)
{
    if (!no_help) {
	Arg args[3];
	int n = 0;
	XtSetArg(args[n], XmNautoUnmanage, True); n++;
	XtSetArg(args[n], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); n++;
	XtSetArg(args[n], XmNdefaultButtonType, XmDIALOG_CANCEL_BUTTON); n++;
	no_help = XmCreateInformationDialog ((Widget)main_window,
					     "NoHelp", args, n);
	XtUnmanageChild (XmMessageBoxGetChild (no_help, 
					       XmDIALOG_OK_BUTTON));
	XtUnmanageChild (XmMessageBoxGetChild (no_help, 
					       XmDIALOG_HELP_BUTTON));
    }
    XtManageChild (no_help);
}


void NoPrevPage(void* main_window)
{
    if (!no_prev_page) {
	Arg args[3];
	int n = 0;
	XtSetArg(args[n], XmNautoUnmanage, True); n++;
	XtSetArg(args[n], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); n++;
	XtSetArg(args[n], XmNdefaultButtonType, XmDIALOG_CANCEL_BUTTON); n++;
	no_prev_page = XmCreateInformationDialog ((Widget)main_window,
						  "NoPrevPage", args, n);
	XtUnmanageChild (XmMessageBoxGetChild (no_prev_page, 
					       XmDIALOG_OK_BUTTON));
	XtUnmanageChild (XmMessageBoxGetChild (no_prev_page, 
					       XmDIALOG_HELP_BUTTON));
    }
    XtManageChild (no_prev_page);
}


void NoNextPage(void* main_window)
{
    if (!no_next_page) {
	Arg args[3];
	int n = 0;
	XtSetArg(args[n], XmNautoUnmanage, True); n++;
	XtSetArg(args[n], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); n++;
	XtSetArg(args[n], XmNdefaultButtonType, XmDIALOG_CANCEL_BUTTON); n++;
	no_next_page = XmCreateInformationDialog ((Widget)main_window,
					     "NoNextPage", args, n);
	XtUnmanageChild (XmMessageBoxGetChild (no_next_page, 
					       XmDIALOG_OK_BUTTON));
	XtUnmanageChild (XmMessageBoxGetChild (no_next_page, 
					       XmDIALOG_HELP_BUTTON));
    }
    XtManageChild (no_next_page);
}


void TooManySelectedObjects (void* main_window)
{
    if (!too_many_selected_objects) {
	Arg args[3];
	int n = 0;
	XtSetArg(args[n], XmNautoUnmanage, True); n++;
	XtSetArg(args[n], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); n++;
	XtSetArg(args[n], XmNdefaultButtonType, XmDIALOG_CANCEL_BUTTON); n++;
	too_many_selected_objects =
	    XmCreateWarningDialog ((Widget)main_window,
				   "TooManySelectedObjects", args, n);
	XtUnmanageChild (XmMessageBoxGetChild (too_many_selected_objects, 
					       XmDIALOG_OK_BUTTON));
	XtUnmanageChild (XmMessageBoxGetChild (too_many_selected_objects, 
					       XmDIALOG_HELP_BUTTON));
    }
    XtManageChild (too_many_selected_objects);
}


void NotNamedObject (void*, void* parent, caddr_t)
{
    if (!not_named_object) {
	Arg args[3];
	int n = 0;
	XtSetArg(args[n], XmNautoUnmanage, True); n++;
	XtSetArg(args[n], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); n++;
	XtSetArg(args[n], XmNdefaultButtonType, XmDIALOG_CANCEL_BUTTON); n++;
	not_named_object =
	    XmCreateErrorDialog((Widget)parent, "NotNamedObject", args, n);
	XtUnmanageChild (XmMessageBoxGetChild (not_named_object, 
					       XmDIALOG_OK_BUTTON));
	XtUnmanageChild (XmMessageBoxGetChild (not_named_object, 
					       XmDIALOG_HELP_BUTTON));
    }
    XtManageChild (not_named_object);
}


void NotInList (void*, void* parent, caddr_t)
{
    if (!not_in_list) {
	Arg args[3];
	int n = 0;
	XtSetArg(args[n], XmNautoUnmanage, True); n++;
	XtSetArg(args[n], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); n++;
	XtSetArg(args[n], XmNdefaultButtonType, XmDIALOG_CANCEL_BUTTON); n++;
	not_in_list =
	    XmCreateErrorDialog ((Widget)parent, "NotInList", args, n);
	XtUnmanageChild (XmMessageBoxGetChild (not_in_list, 
					       XmDIALOG_OK_BUTTON));
	XtUnmanageChild (XmMessageBoxGetChild (not_in_list, 
					       XmDIALOG_HELP_BUTTON));
    }
    XtManageChild (not_in_list);
}


void NotGo (void* parent)
{
    if (!not_go) {
	Arg args[3];
	int n = 0;
	XtSetArg(args[n], XmNautoUnmanage, True); n++;
	XtSetArg(args[n], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); n++;
	XtSetArg(args[n], XmNdefaultButtonType, XmDIALOG_CANCEL_BUTTON); n++;
	not_go = XmCreateErrorDialog ((Widget)parent, "NotGo", args, n);
	XtUnmanageChild (XmMessageBoxGetChild (not_go, 
					       XmDIALOG_OK_BUTTON));
	XtUnmanageChild (XmMessageBoxGetChild (not_go, 
					       XmDIALOG_HELP_BUTTON));
    }
    XtManageChild (not_go);
}



void ObjectName (void* parent)
{
    if (!object_name) {
	Arg args[3];
	int n = 0;
	XtSetArg(args[n], XmNautoUnmanage, True); n++;
	XtSetArg(args[n], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); n++;
	XtSetArg(args[n], XmNdefaultButtonType, XmDIALOG_CANCEL_BUTTON); n++;
	object_name =
	    XmCreateWarningDialog ((Widget)parent, "ObjectName", args, n);
	XtUnmanageChild (XmMessageBoxGetChild (object_name, 
					       XmDIALOG_OK_BUTTON));
	XtUnmanageChild (XmMessageBoxGetChild (object_name, 
					       XmDIALOG_HELP_BUTTON));
    }
    XtManageChild (object_name);
}


void UndoEmpty(void* parent)
{
    if (!undo_empty) {
	Arg args[3];
	int n = 0;
	XtSetArg(args[n], XmNautoUnmanage, True); n++;
	XtSetArg(args[n], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); n++;
	XtSetArg(args[n], XmNdefaultButtonType, XmDIALOG_CANCEL_BUTTON); n++;
	undo_empty =
	    XmCreateInformationDialog ((Widget)parent, "UndoEmpty", args, n);
	XtUnmanageChild (XmMessageBoxGetChild (undo_empty, 
					       XmDIALOG_OK_BUTTON));
	XtUnmanageChild (XmMessageBoxGetChild (undo_empty, 
					       XmDIALOG_HELP_BUTTON));
    }
    XtManageChild (undo_empty);
}


void RedoEmpty(void* parent)
{
    if (!redo_empty) {
	Arg args[3];
	int n = 0;
	XtSetArg(args[n], XmNautoUnmanage, True); n++;
	XtSetArg(args[n], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); n++;
	XtSetArg(args[n], XmNdefaultButtonType, XmDIALOG_CANCEL_BUTTON); n++;
	redo_empty =
	    XmCreateInformationDialog ((Widget)parent, "RedoEmpty", args, n);
	XtUnmanageChild (XmMessageBoxGetChild (redo_empty, 
					       XmDIALOG_OK_BUTTON));
	XtUnmanageChild (XmMessageBoxGetChild (redo_empty, 
					       XmDIALOG_HELP_BUTTON));
    }
    XtManageChild (redo_empty);
}


static void Destroy(Widget, Widget message)
{
    XtDestroyWidget(message);
}

void CannotDestroy(void* parent, char* name)
{
    Arg args[5];
    char buf[100];
    sprintf(buf, "The named object %s is used by a Graphic Object.\n It cannot be destroyed.", name);
    XmString str = XmStringCreateLtoR(buf, XmSTRING_DEFAULT_CHARSET);
    int n = 0;
    XtSetArg(args[n], XmNautoUnmanage, True); n++;
    XtSetArg(args[n], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); n++;
    XtSetArg(args[n], XmNdefaultButtonType, XmDIALOG_CANCEL_BUTTON); n++;
    XtSetArg(args[n], XmNmessageString, str); n++;
    Widget cannot_destroy = XmCreateWarningDialog ((Widget)parent,
						   "CannotDestroy", args, n);
    XmStringFree(str);
    XtUnmanageChild(XmMessageBoxGetChild(cannot_destroy, XmDIALOG_OK_BUTTON));
    XtUnmanageChild(XmMessageBoxGetChild(cannot_destroy,
					 XmDIALOG_HELP_BUTTON));
    XtAddCallback(cannot_destroy, XmNcancelCallback, (XtCallbackProc)Destroy,
		  (XtPointer)cannot_destroy);
    XtManageChild(cannot_destroy);
}


void ConfirmDestroyShared(void* parent, char* name, void* shared)
{
    Arg args[5];
    char buf[100];
    sprintf(buf, "Do you really want to destroy the named object %s ?", name);
    XmString str = XmStringCreateLtoR(buf, XmSTRING_DEFAULT_CHARSET);
    int n = 0;
    XtSetArg(args[n], XmNautoUnmanage, True); n++;
    XtSetArg(args[n], XmNmessageString, str); n++;
    XtSetArg(args[n], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); n++;
    XtSetArg(args[n], XmNdefaultButtonType, XmDIALOG_OK_BUTTON); n++;
    Widget confirm_destroy_shared = 
	XmCreateQuestionDialog((Widget) parent, "ConfirmDestroyShared",
			       args, n);
    XtUnmanageChild(XmMessageBoxGetChild(confirm_destroy_shared,
					 XmDIALOG_HELP_BUTTON));
    XtAddCallback(confirm_destroy_shared, XmNokCallback, 
		  (XtCallbackProc)DestroyIt, (XtPointer)shared);
    XtAddCallback(confirm_destroy_shared, XmNokCallback, 
		  (XtCallbackProc)Destroy, (XtPointer)confirm_destroy_shared);
    XtAddCallback(confirm_destroy_shared, XmNcancelCallback,
		  (XtCallbackProc)Destroy, (XtPointer)confirm_destroy_shared);
    XtManageChild(confirm_destroy_shared);
}


void ConfirmFileName(void* parent)
{
    if (! confirm_file_name) {
	Arg args[3];
	int n = 0;
	XtSetArg(args[n], XmNautoUnmanage, True); n++;
	XtSetArg(args[n], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); n++;
	XtSetArg(args[n], XmNdefaultButtonType, XmDIALOG_OK_BUTTON); n++;
	confirm_file_name = 
	    XmCreateQuestionDialog((Widget) parent, "ConfirmFileName",
				   args, n);
	XtUnmanageChild(XmMessageBoxGetChild(confirm_file_name,
					     XmDIALOG_HELP_BUTTON));
	XtAddCallback(confirm_file_name, XmNokCallback, 
		      (XtCallbackProc)applySaveBox, NULL);
    }
    XtManageChild(confirm_file_name);
}


void ConfirmPSFileName(void* parent)
{
    if (! confirm_PSfile_name) {
	Arg args[3];
	int n = 0;
	XtSetArg(args[n], XmNautoUnmanage, True); n++;
	XtSetArg(args[n], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); n++;
	XtSetArg(args[n], XmNdefaultButtonType, XmDIALOG_OK_BUTTON); n++;
	confirm_PSfile_name = 
	    XmCreateQuestionDialog((Widget) parent, "ConfirmPSFileName",
				   args, n);
	XtUnmanageChild(XmMessageBoxGetChild(confirm_PSfile_name,
					     XmDIALOG_HELP_BUTTON));
	XtAddCallback(confirm_PSfile_name, XmNokCallback, 
		      (XtCallbackProc)applyPostscriptBox, NULL);
    }
    XtManageChild(confirm_PSfile_name);
}


void ConfirmKeyName(void* parent)
{
    if (! confirm_key_name) {
	Arg args[3];
	int n = 0;
	XtSetArg(args[n], XmNautoUnmanage, True); n++;
	XtSetArg(args[n], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); n++;
	XtSetArg(args[n], XmNdefaultButtonType, XmDIALOG_OK_BUTTON); n++;
	confirm_key_name = 
	    XmCreateQuestionDialog((Widget) parent, "ConfirmKeyName", args, n);
	XtUnmanageChild(XmMessageBoxGetChild(confirm_key_name,
					     XmDIALOG_HELP_BUTTON));
	XtAddCallback(confirm_key_name, XmNokCallback, 
		      (XtCallbackProc)AssocReallyOk, (XtPointer)TRUE);
    }
    XtManageChild(confirm_key_name);
}


void AcknowledgePenName(void* parent)
{
    static Widget acknowledge_dialog = NULL;

    if (!acknowledge_dialog) {
	Arg args[3];
	int n = 0;
	XtSetArg(args[n], XmNautoUnmanage, True); n++;
	XtSetArg(args[n], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); n++;
	XtSetArg(args[n], XmNdefaultButtonType, XmDIALOG_OK_BUTTON); n++;
        acknowledge_dialog = XmCreateQuestionDialog ((Widget)parent,
						     "AcknowledgePenName",
						     args, n);
        XtUnmanageChild (XmMessageBoxGetChild
			 (acknowledge_dialog, XmDIALOG_HELP_BUTTON));
        XtAddCallback (acknowledge_dialog, XmNokCallback,
		       (XtCallbackProc)VerifyPen, (XtPointer)parent);
    }
    XtManageChild (acknowledge_dialog);
}


void AcknowledgeBrushName(void* parent)
{
    static Widget acknowledge_dialog = NULL;

    if (!acknowledge_dialog) {
	Arg args[3];
	int n = 0;
	XtSetArg(args[n], XmNautoUnmanage, True); n++;
	XtSetArg(args[n], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); n++;
	XtSetArg(args[n], XmNdefaultButtonType, XmDIALOG_OK_BUTTON); n++;
        acknowledge_dialog = XmCreateQuestionDialog ((Widget)parent,
						     "AcknowledgeBrushName",
						     args, n);
        XtUnmanageChild (XmMessageBoxGetChild
			 (acknowledge_dialog, XmDIALOG_HELP_BUTTON));
        XtAddCallback (acknowledge_dialog, XmNokCallback,
		       (XtCallbackProc)VerifyBrush, (XtPointer)parent);
    }
    XtManageChild (acknowledge_dialog);
}


void AcknowledgeTextBrushName(void* parent)
{
    static Widget acknowledge_dialog = NULL;

    if (!acknowledge_dialog) {
	Arg args[3];
	int n = 0;
	XtSetArg(args[n], XmNautoUnmanage, True); n++;
	XtSetArg(args[n], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); n++;
	XtSetArg(args[n], XmNdefaultButtonType, XmDIALOG_OK_BUTTON); n++;
        acknowledge_dialog = XmCreateQuestionDialog ((Widget)parent,
						    "AcknowledgeTextBrushName",
						     args, n);
        XtUnmanageChild (XmMessageBoxGetChild
			 (acknowledge_dialog, XmDIALOG_HELP_BUTTON));
        XtAddCallback (acknowledge_dialog, XmNokCallback,
		       (XtCallbackProc)VerifyTextBrush, (XtPointer)parent);
    }
    XtManageChild (acknowledge_dialog);
}
