#ifdef REV_INFO
static char rcsid[] = "$Id: miscelmenu.cc,v 1.5 1992/12/10 16:24:51 jojic Exp $";
#endif
/*****************************************************************************
MODULE:  Go Editor
FILE:    miscelmenu.cc
AUTHOR:  Aline Jacomet
DATE:    May 14th 1991

DESCRIPTION:
    

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include <sys/types.h>
#include <stdio.h>
#include <sys/stat.h>

#include <Go/Tools/FilePath.h>
#include <Go/Tools/Alloc.h>

#include "main.h"

#include <Xm/TextF.h>
#include <Xm/Text.h>
#include <Xm/List.h>
#include <Xm/Form.h>
#include <Xm/PushBG.h>
#include <Xm/LabelG.h>
#include <Xm/SelectioB.h>
#include <Xm/ToggleBG.h>
#include <Xm/RowColumn.h>
#include <Xm/FileSB.h>




/*
 * EXTERN FUNCTIONS
 */

// in file goMiscel.cc;
extern void AssocOk(char*, char*);
extern void AssocRemove(char*, char*);
extern void LoadGif(char*); 
extern void InitializeSimpleText(char*);
extern void InitializeText(char*);
#ifdef GoLICENSE
extern void InitializeScrolledText(char*);
#endif

// in file namedattrib.cc;
extern void UnmanageWidget(Widget, Widget, caddr_t);

// in file goTools.cc;
extern int  GetNumMainSelectedObjects();

// in file message.cc;
extern void NoSelectedObject(void*);
extern void TooManySelectedObjects(void*);
extern void Error(void*);
#ifndef GoLICENSE
extern void NoLicense(void*, void*, caddr_t);
#endif




                           /********************
			    * ASSOC MANAGEMENT *
			    ********************/


static Widget assocWidget = NULL;
static Widget listWidget = NULL;
static Widget key = NULL;
static Widget value = NULL;


// in file goMiscel.cc;
extern void InitializeList();


static void Ok(Widget, caddr_t, caddr_t)
{
    char* key_str = XmTextFieldGetString(key);
    char* value_str = XmTextGetString(value);
    if (key_str && key_str[0] != '\0')
	AssocOk(key_str, value_str);
}

void ReallyOk(char* old_item, char* new_item)
{
    // Put the new association in the GList widget;
    if (old_item) {
	XmString old = XmStringCreateLtoR(old_item, XmSTRING_DEFAULT_CHARSET);
	XmString n = XmStringCreateLtoR(new_item, XmSTRING_DEFAULT_CHARSET);
	XmListReplaceItems(listWidget, &old, 1, &n);
	XmStringFree(old);
	XmStringFree(n);
    } else {
	XmString str = XmStringCreateLtoR(new_item, XmSTRING_DEFAULT_CHARSET);
	XmListAddItem(listWidget, str, 0);
	XmStringFree(str);
    }
    XmTextFieldSetString(key, "");
    XmTextSetString(value, "");
}


void AddItem(char* item)
{
    XmString str = XmStringCreateLtoR(item, XmSTRING_DEFAULT_CHARSET);
    XmListAddItem(listWidget, str, 0);
    XmStringFree(str);
}


static void Remove(Widget, caddr_t, caddr_t)
{
    int nb_to_rem;
    XmString* rem_table;
    Arg args[5];
    int n = 0;
    XtSetArg(args[n], XmNselectedItemCount, &nb_to_rem); n++;
    XtSetArg(args[n], XmNselectedItems, &rem_table); n++;
    XtGetValues(listWidget, args, n);
    for (n = 0; n < nb_to_rem; n++) {
	// Separate the key and the value;
	char* str;
	XmStringGetLtoR(rem_table[n], XmSTRING_DEFAULT_CHARSET, &str);
	char* key_str = NULL;
	for (int i = 0; str[i] != '\0'; i++) {
	    if (str[i] == ' ') {
		char sep[8];
		strncpy(sep, &str[i], 7);
		sep[7] = '\0';
		if (strcmp(sep, " ----> ") == 0) {
		    key_str = (char*)GMALLOC(&str[i] - str + 1);
		    strncpy(key_str, str, &str[i] - str);
		    key_str[&str[i] - str] = '\0';
		    break;
		}
	    }
	}

	if (key_str) {
	    char* value_str = (char*)GMALLOC(strlen(str) - strlen(key_str) - 6);
	    strcpy(value_str, str + strlen(key_str) + 7);
	    AssocRemove(key_str, value_str);
	    GFREE(key_str);
	    GFREE(value_str);
	    GFREE(str);
	}
    }
    XmListDeleteItems(listWidget, rem_table, nb_to_rem);
}


void CreateAssocDialog(Widget parent)
{
    Arg args[5];
    int n = 0;
    XtSetArg(args[n], XmNautoUnmanage, GFALSE); n++;
    XtSetArg(args[n], XmNresizable, GFALSE); n++;
    XtSetArg(args[n], XmNresizePolicy, XmRESIZE_NONE); n++;
    assocWidget = XmCreateFormDialog(parent, "AssocDialog", args, n);
    XtManageChild(XmCreateLabelGadget(assocWidget, "AssocLabel", NULL, 0));

    n = 0;
    XtSetArg(args[n], XmNlistSizePolicy, XmCONSTANT); n++;
    listWidget = XmCreateScrolledList(assocWidget, "AssocList", args, n);
    XtManageChild(listWidget);

    XtManageChild(XmCreateLabelGadget(assocWidget, "KeyLabel", NULL, 0));
    key = XmCreateTextField(assocWidget, "AssocKey", NULL, 0);
    XtManageChild(key);

    XtManageChild(XmCreateLabelGadget(assocWidget, "ValueLabel", NULL, 0));
    value = XmCreateText(assocWidget, "AssocValue", NULL, 0);
    XtManageChild(value);

    Widget add = XmCreatePushButtonGadget(assocWidget, "AssocAdd", NULL, 0);
    XtManageChild(add);
    XtAddCallback(add, XmNactivateCallback, (XtCallbackProc)Ok, NULL);

    Widget remove = XmCreatePushButtonGadget(assocWidget, "AssocRemove",
					     NULL, 0);
    XtManageChild(remove);
    XtAddCallback(remove, XmNactivateCallback, (XtCallbackProc)Remove,
		  NULL);

    Widget unmap = XmCreatePushButtonGadget(assocWidget, "AssocUnmap",
					    NULL, 0);
    XtManageChild(unmap);
    XtAddCallback(unmap, XmNactivateCallback, (XtCallbackProc)UnmanageWidget,
		  (XtPointer)assocWidget);
}

 
void DisplayAssocDialog(Widget, Widget parent, caddr_t)
{
    int num = GetNumMainSelectedObjects();
    if (num == 0) {
	NoSelectedObject(main_window);
	return;
    } else
	if (num > 1) {
	    TooManySelectedObjects(main_window);
	    return;
	}
    
    if (!assocWidget) {
	XDefineCursor(XtDisplay((Widget)graphic), XtWindow((Widget)graphic),
		      clock_cursor);
	XDefineCursor(XtDisplay((Widget)main_window),
		      XtWindow((Widget)main_window), clock_cursor);
	XFlush(XtDisplay((Widget)graphic));

	CreateAssocDialog(parent);
	
	XFlush(XtDisplay((Widget)graphic));
	XUndefineCursor(XtDisplay((Widget)graphic), XtWindow((Widget)graphic));
	XUndefineCursor(XtDisplay((Widget)main_window),
			XtWindow((Widget)main_window));
    }

    // Initialize items in the GList widget;
    XmListDeleteAllItems(listWidget);
    InitializeList();

    XtManageChild(assocWidget);
}




                           /******************
			    * GIF -> GoImage *
			    ******************/


static Widget gif_name = NULL;


static void cancelLoadGifFile (Widget w)
{
    XtUnmanageChild (w);
}


static void LoadImage(void*, caddr_t, XmFileSelectionBoxCallbackStruct* call_data)
{
    char *filename;
    XtUnmanageChild(gif_name);
    XmStringGetLtoR (call_data->value, XmSTRING_DEFAULT_CHARSET, &filename);
    LoadGif(filename);
    GFREE(filename);
}   


void Image(void*, void* parent, caddr_t)
{
    if (! gif_name) {
	XDefineCursor(XtDisplay((Widget)main_window),
		      XtWindow((Widget)main_window), clock_cursor);
	XFlush(XtDisplay((Widget)main_window));
	gif_name = XmCreateFileSelectionDialog((Widget)parent, "LoadGifFile",
						NULL, 0);
	XtAddCallback (gif_name, XmNokCallback,
		       (XtCallbackProc)LoadImage, NULL);
	XtAddCallback (gif_name, XmNcancelCallback,
		       (XtCallbackProc)cancelLoadGifFile, NULL);
	XtUnmanageChild (XmSelectionBoxGetChild (gif_name,
						 XmDIALOG_HELP_BUTTON));
	XUndefineCursor(XtDisplay((Widget)main_window),
			XtWindow((Widget)main_window));
	XFlush(XtDisplay((Widget)main_window));
    }
    XtManageChild(gif_name);
}




                          /***************************
			   * Ascii File -> GoTextual *
			   ***************************/


static Widget ascii_name = NULL;
static Widget textToggle, simTextToggle, scrolTextToggle, fileName;

#ifndef GoLICENSE
static void no_scroltext_callback(Widget, caddr_t,
				  XmToggleButtonCallbackStruct*);
#endif

static void LoadAsciiFile(Widget, Widget parent, caddr_t)
{
    int mod, u, nbToShift;
    char *pmin, *pmax;
    char* name = XmTextGetString(fileName);
    struct stat file_status;
    if (stat(name, &file_status)) {
	Error(main_window);
	return;
    }

    char* texte = (char*)GMALLOC((unsigned int)file_status.st_size * 2);
    FILE* file = fopen(name, "r");
    if (file && file_status.st_size != 0) {
	fread(texte, 1, (int)file_status.st_size, file);
	texte[(int)file_status.st_size] = '\0';
	int i = 0;
	char* ptr = texte;
	char* maxi = texte + strlen (texte);
	int nbTabs = 0;
	int nbBS = 0;
	while (ptr < maxi){
	    switch (*ptr){
	      case '\t':
		*ptr = ' ';
		mod = i%8 + 1;
		nbToShift = 8 - mod;
		if (nbToShift){
		    pmin = maxi;
		    pmax = maxi+nbToShift;
		    maxi = pmax;
		    u = pmin - ptr;
		    while (u--) *pmax-- = *pmin--;
		    while (nbToShift) *(ptr+nbToShift--) = ' ';
		}
		i++;
		nbTabs++;
		break;
		
	      case '\b':
		/* controle sequence */
		int cpt;
		if (*(ptr - 1) != '_') break;
		ptr--;
		for (cpt = 0; ptr + cpt < maxi; cpt++)
		    *(ptr+cpt) = *(ptr+cpt+2);
		maxi--;
		maxi--;
		nbBS++;
		break;
		
	      case '\n':
		i = 0;
		break;
		
	      default:
		i++;
		break;
		}
	    ptr++;
	}
	
	/* unknow /377 */
	*(maxi-1) = '\0';
	
	XtUnmanageChild(parent);
	if (XmToggleButtonGadgetGetState(textToggle))
	    InitializeText(texte);
	else
	    if (XmToggleButtonGadgetGetState(simTextToggle))
		InitializeSimpleText(texte);
#ifdef GoLICENSE	
	    else
		if (XmToggleButtonGadgetGetState(scrolTextToggle))
		    InitializeScrolledText(texte);
#endif	
    } else
	Error(main_window);
    GFREE(texte);
}


static void ReinitAsciiFile()
{
    XmTextSetString(fileName, "");
    XmToggleButtonGadgetSetState(textToggle, True, False);
    XmToggleButtonGadgetSetState(simTextToggle, False, False);
    XmToggleButtonGadgetSetState(scrolTextToggle, False, False);
}


#ifndef GoLICENSE
static void no_scroltext_callback(Widget tb, caddr_t,
				  XmToggleButtonCallbackStruct* call)
{
    if (call->set) {
	XmToggleButtonGadgetSetState(tb, False, False);
	XmToggleButtonGadgetSetState(textToggle, True, False);
	XmToggleButtonGadgetSetState(simTextToggle, True, False);
	NoLicense(NULL, main_window, NULL);
    }
}
#endif

static Widget CreateAsciiName(Widget parent)
{
    Arg arg;
    XtSetArg(arg, XmNautoUnmanage, False);
    Widget f = XmCreateFormDialog(parent, "LoadAsciiFile", &arg, 1);
    XtManageChild(XmCreateLabelGadget(f, "AsciiFileLabel", NULL, 0));
    fileName = XmCreateText(f, "AsciiFileName", NULL, 0);
    XtManageChild(fileName);

    Widget rb = XmCreateRadioBox(f, "RadioBox", NULL, 0);
    XtManageChild(rb);
    textToggle = XmCreateToggleButtonGadget(rb, "TextToggle", NULL, 0);
    XtManageChild(textToggle);
    simTextToggle = XmCreateToggleButtonGadget(rb, "SimTextToggle", NULL, 0);
    XtManageChild(simTextToggle);
    scrolTextToggle = XmCreateToggleButtonGadget(rb, "ScrolTextToggle",
						 NULL, 0);
    XtManageChild(scrolTextToggle);
#ifndef GoLICENSE
    XtAddCallback(scrolTextToggle, XmNvalueChangedCallback,
		  (XtCallbackProc)no_scroltext_callback, NULL);
#endif    
    XmToggleButtonGadgetSetState(textToggle, True, False);

    Widget w = XmCreatePushButtonGadget(f, "Apply", NULL, 0);
    XtManageChild(w);
    XtAddCallback(w, XmNactivateCallback, (XtCallbackProc)LoadAsciiFile,
		  (XtPointer)f);

    w = XmCreatePushButtonGadget(f, "Reinit", NULL, 0);
    XtManageChild(w);
    XtAddCallback(w, XmNactivateCallback, (XtCallbackProc)ReinitAsciiFile,
		  NULL);

    w = XmCreatePushButtonGadget(f, "Cancel", NULL, 0);
    XtManageChild(w);
    XtAddCallback(w, XmNactivateCallback, (XtCallbackProc)UnmanageWidget,
		  (XtPointer)f);

    return f;
}


void AsciiFile(void*, void* parent, caddr_t)
{
    if (! ascii_name) {
	XDefineCursor(XtDisplay((Widget)main_window),
		      XtWindow((Widget)main_window), clock_cursor);
	XFlush(XtDisplay((Widget)main_window));

	ascii_name = CreateAsciiName((Widget)parent);

	XUndefineCursor(XtDisplay((Widget)main_window),
			XtWindow((Widget)main_window));
	XFlush(XtDisplay((Widget)main_window));
    }
    XtManageChild(ascii_name);
}


