#ifdef REV_INFO
static char rcsid[] = "$Id: namedattrib.cc,v 1.1.1.1 1992/03/18 09:29:37 uep Exp $";
#endif
/*****************************************************************************
MODULE:  Go Editor
FILE:    namedattrib.cc
AUTHOR:  Aline Jacomet
DATE:    May 21th 1991

DESCRIPTION:
    

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include <Go/Tools/StringAssoc.h>
#include <Go/Tools/Alloc.h>

#include <Xm/Form.h>
#include <Xm/PushBG.h>
#include <Xm/List.h>
#include <Xm/Frame.h>
#include <Xm/LabelG.h>
#include <Xm/TextF.h>

#include <Xb/Graphic.h>

#include "classes.h"
#include "main.h"


/***
 * widget creation
 ***/
static Widget CreateNamedGCDialog(Widget);
static void   CreatePenNameWid(Widget);
static void   CreateBrushNameWid(Widget);
static void   CreateTextBrushNameWid(Widget);

/***
 * feedback on the local graphic widget namedGcDraw
 ***/


/***
 * reinit function
 ***/
static void ReinitText();


/***
 * miscellaneous
 ***/
static void PrepareGCDialog(Widget);
static void InitDialog();

void VerifFunction(Widget, caddr_t, caddr_t);


/***
 *  Static Variables
 ***/
static Widget namedGcDialog;
static Widget namedGcDraw;
static Widget namedGcFrame;


/* Attributes subwidgets */
static Widget penLabel;
static Widget brushLabel;
static Widget textBrushLabel;
static Widget penList;
static Widget brushList;
static Widget textBrushList;

static Widget apply;

static Widget savePenList;
static Widget saveBrushList;
static Widget saveTxtBrushList;

static Widget destroyPenList;
static Widget destroyBrushList;
static Widget destroyTxtBrushList;



/****
 * Global
 ****/

GStringAssoc NamedPen;
GStringAssoc NamedBrush;
GStringAssoc NamedText;

Widget penText;
Widget brushText;
Widget textBrushText;

extern Widget penTextEdit;
extern Widget brushTextEdit;
extern Widget textBrushTextEdit;

extern PenAttributes named_pen;
extern BrushAttributes named_brush;
extern TextAttributes named_text;
extern PenAttributes pen;
extern BrushAttributes brush;
extern TextAttributes text;
extern AllAttributes all_attrib;

extern int WhichObject;

extern Boolean penMapped;
extern Boolean brushMapped;
extern Boolean textBrushMapped;


/*
 * EXTERN FUNCTIONS
 */

// in file goAttrib.cc;
extern void ModifyGoBackground(int);
extern void ModifyGoNamedPenSheet();
extern void ModifyGoNamedBrushSheet();
extern void ModifyGoNamedTextSheet();
extern void FromShPenToNamedPen(char*);
extern void FromShBrushToNamedBrush(char*);
extern void FromShTextToNamedText(char*);
extern void FromShPenToEditPen(char*);
extern void FromShBrushToEditBrush(char*);
extern void FromShTextToEditText(char*);
extern void ModifyGoEditPenSheet();
extern void ModifyGoEditBrushSheet();
extern void ModifyGoEditTextSheet();
extern void ModifyGoColor(GRgb*, char*);
extern GRgb  HexaStringToRgb(const char*);

// in file goLoadSave.cc;
extern void* OpenWrite();
extern int   SaveGoObject(void* stream, char* name, int type);
extern void  CloseAll(void*);
extern int   DestroyGoObject(char*, int);

// in file editattrib.cc;
extern void SetUnsensitive(Widget, Widget);

// in file pensheet.cc;
extern void MapPenAttrib(void*, void*, caddr_t);

// in file brushsheet.cc;
extern void MapBrushAttrib(void*, void*, caddr_t);

// in file textsheet.cc;
extern void MapTextBrushAttrib(void*, void*, caddr_t);

// in file selectionbox.cc;
extern void SelectNamedPen(void*, int, caddr_t);    
extern void SelectNamedBrush(void*, int, caddr_t);    
extern void SelectNamedText(void*, int, caddr_t);    

// in file namedattrib.cc;
extern void ModifyColor(Widget, char*);

// in file message.cc;
extern void NotNamedObject(void*, void*, caddr_t);
extern void Error(void*);

// in file help.cc;
extern void MapHelpFile(void*, char*, void*);

// in file goInit.cc;
extern void InitNamedGo(void*);




                            /*************************
			     * EDIT NAMED ATTRIBUTES *
			     *************************/


/**
FUNCTION
   SensitiveButtons()
DESCRIPTION
    The button (Apply) of the NamedGCDialog is sensitive.
**/
void SensitiveButtons(Widget, Boolean* map, caddr_t)
{
    *map = False;
    if (! (penMapped || brushMapped || textBrushMapped)) {
	Arg arg;
	XtSetArg(arg, XmNsensitive, True);
	XtSetValues(apply, &arg, 1);
    }
}


/**
FUNCTION
   UnmanageWidget()
DESCRIPTION
    
**/
void UnmanageWidget(Widget, Widget w, caddr_t)
{
    XtUnmanageChild(w);
}




/**
FUNCTION
    NamedDialog()
DESCRIPTION
    Display the PropertySheet to named named pens, brushes and textBrushes
    (Validation toggles and background and outline toggles are not managed)
**/
void NamedDialog(Widget, Widget parent, caddr_t)
{
    Boolean create = (namedGcDialog == NULL);

    if (!create && XtIsManaged(namedGcDialog))
	return;
    
    if (create) {
	XDefineCursor(XtDisplay((Widget)main_window),
		      XtWindow((Widget)main_window), clock_cursor);
	XFlush(XtDisplay((Widget)graphic));
	namedGcDialog = CreateNamedGCDialog(parent);
    }

    WhichAttributes = NAMED;
    ReinitText();

    XtManageChild(namedGcDialog);

    if (create) {
	PrepareGCDialog(namedGcDraw);
	XFlush(XtDisplay((Widget)graphic));
	XUndefineCursor(XtDisplay((Widget)main_window),
			XtWindow((Widget)main_window));
    }

    ModifyGoBackground(True);
    InitDialog();
}


/**
FUNCTION
    CreateNamedGCDialog()
DESCRIPTION
    Create the PropertySheet used to manage the GCs.
    It is the same box to update the current GC, to modify the GC of the 
    selected objects or to edit named pens, brushes and textbrushes.
    In the second case, toggle buttons permit to valid each entry of the 
    PropertySheet.
**/
static Widget CreateNamedGCDialog(Widget main_window)
{
    Widget ps;
    int n = 0;
    Arg args[20];


    n = 0;
    XtSetArg(args[n], XmNresizePolicy, XmRESIZE_NONE); n++;
    XtSetArg(args[n], XmNautoUnmanage, False); n++;
    ps = XmCreateFormDialog(main_window, "NamedGCDialog", args, n);
    XtAddCallback(ps, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)NAMED_ATTRIB_FILE);


    /* Apply button */
    apply = XmCreatePushButtonGadget(ps, "Apply", NULL, 0);
    XtManageChild(apply);
    XtAddCallback(apply, XmNactivateCallback,
		  (XtCallbackProc) UnmanageWidget, (XtPointer)ps);
    XtAddCallback(apply, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)NAMED_ATTRIB_FILE);
    
    /* pen edition */
    n = 0;
    XtSetArg(args[n], XmNmarginWidth, 5); n++;
    XtSetArg(args[n], XmNmarginHeight, 5); n++;
    Widget w = XmCreatePushButtonGadget(ps, "PenEdit", args, n);
    XtManageChild(w);
    XtAddCallback(w, XmNactivateCallback, (XtCallbackProc)SetUnsensitive,
		  (XtPointer) apply);
    XtAddCallback(w, XmNactivateCallback, (XtCallbackProc)MapPenAttrib,
		  (XtPointer)main_window);
    XtAddCallback(w, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)NAMED_ATTRIB_FILE);

    /* pen text */
    CreatePenNameWid(ps);

    /* brush edition */
    n = 0;
    XtSetArg(args[n], XmNmarginWidth, 5); n++;
    XtSetArg(args[n], XmNmarginHeight, 5); n++;
    w = XmCreatePushButtonGadget(ps, "BrushEdit", args, n);
    XtManageChild(w);
    XtAddCallback(w, XmNactivateCallback, (XtCallbackProc)SetUnsensitive,
		  (XtPointer) apply);
    XtAddCallback(w, XmNactivateCallback, (XtCallbackProc)MapBrushAttrib,
		  (XtPointer)main_window);
    XtAddCallback(w, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)NAMED_ATTRIB_FILE);

    /* brush text */
    CreateBrushNameWid(ps);

    /* text brush edition */
    n = 0;
    XtSetArg(args[n], XmNmarginWidth, 5); n++;
    XtSetArg(args[n], XmNmarginHeight, 5); n++;
    w = XmCreatePushButtonGadget(ps, "TextBrushEdit", args, n);
    XtManageChild(w);
    XtAddCallback(w, XmNactivateCallback, (XtCallbackProc)SetUnsensitive,
		  (XtPointer) apply);
    XtAddCallback(w, XmNactivateCallback,
		  (XtCallbackProc)MapTextBrushAttrib, (XtPointer)main_window);
    XtAddCallback(w, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)NAMED_ATTRIB_FILE);

    /* textbrush text */
    CreateTextBrushNameWid(ps);

    /* frame and its child, namedGcDraw widget */
    namedGcFrame = XmCreateFrame(ps, "NamedGCFrame", NULL, 0);
    XtAddCallback(namedGcFrame, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)NAMED_ATTRIB_FILE);
    XtManageChild(namedGcFrame);
    namedGcDraw = XbCreateGraphic(namedGcFrame, "NamedGCDraw", NULL, 0);
    XtAddCallback(namedGcDraw, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)NAMED_ATTRIB_FILE);
    XtManageChild(namedGcDraw);

    return(ps);
}


    

/**
FUNCTION
    CreatePenNameWid()
DESCRIPTION
    Creation of text and label used to edit named pen attributes.
**/
static void CreatePenNameWid(Widget parent)
{
    penLabel = XmCreateLabelGadget(parent, "PenLabel", NULL, 0);
    XtAddCallback(penLabel, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)NAMED_ATTRIB_FILE);
    XtManageChild(penLabel);

    Arg args[2];
    int n = 0;
    XtSetArg(args[n], XmNmarginWidth, 5); n++;
    XtSetArg(args[n], XmNmarginHeight, 5); n++;
    penList = XmCreatePushButtonGadget(parent, "PenList", args, n);
    XtManageChild(penList);
    XtAddCallback(penList, XmNactivateCallback, (XtCallbackProc)SelectNamedPen,
		  (XtPointer) NAMED);
    XtAddCallback(penList, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)NAMED_ATTRIB_FILE);

    penText = XmCreateTextField(parent, "PenText", NULL, 0);
    XtManageChild(penText);
    XtAddCallback(penText, XmNactivateCallback,
		  (XtCallbackProc)VerifFunction, NULL);
    XtAddCallback(penText, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)NAMED_ATTRIB_FILE);
}


/**
FUNCTION
    CreateBrushNameWid()
DESCRIPTION
    Creation of text and label used to edit named brush attributes.
**/
static void CreateBrushNameWid(Widget parent)
{
    brushLabel = XmCreateLabelGadget(parent, "BrushLabel", NULL, 0);
    XtAddCallback(brushLabel, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)NAMED_ATTRIB_FILE);
    XtManageChild(brushLabel);

    Arg args[2];
    int n = 0;
    XtSetArg(args[n], XmNmarginWidth, 5); n++;
    XtSetArg(args[n], XmNmarginHeight, 5); n++;
    brushList = XmCreatePushButtonGadget(parent, "BrushList", args, n);
    XtManageChild(brushList);
    XtAddCallback(brushList, XmNactivateCallback,
		  (XtCallbackProc) SelectNamedBrush, (XtPointer) NAMED);
    XtAddCallback(brushList, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)NAMED_ATTRIB_FILE);
 
    brushText = XmCreateTextField(parent, "BrushText", NULL, 0);
    XtManageChild(brushText);
    XtAddCallback(brushText, XmNactivateCallback,
		  (XtCallbackProc)VerifFunction, NULL);
    XtAddCallback(brushText, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)NAMED_ATTRIB_FILE);
}



/**
FUNCTION
    CreateTextBrushNameWid()
DESCRIPTION
    Creation of text and label used to edit named textbrush attributes.
**/
static void CreateTextBrushNameWid(Widget parent)
{
    textBrushLabel = XmCreateLabelGadget(parent, "TextBrushLabel", NULL, 0);
    XtAddCallback(textBrushLabel, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)NAMED_ATTRIB_FILE);
    XtManageChild(textBrushLabel);

    Arg args[2];
    int n = 0;
    XtSetArg(args[n], XmNmarginWidth, 5); n++;
    XtSetArg(args[n], XmNmarginHeight, 5); n++;
    textBrushList = XmCreatePushButtonGadget(parent, "TextBrushList", args, n);
    XtManageChild(textBrushList);
    XtAddCallback(textBrushList, XmNactivateCallback,
		  (XtCallbackProc) SelectNamedText, (XtPointer) NAMED);
    XtAddCallback(textBrushList, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)NAMED_ATTRIB_FILE);

    textBrushText = XmCreateTextField(parent, "TextBrushText", NULL, 0);
    XtManageChild(textBrushText);
    XtAddCallback(textBrushText, XmNactivateCallback,
		  (XtCallbackProc)VerifFunction, NULL);
    XtAddCallback(textBrushText, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)NAMED_ATTRIB_FILE);
}




/*---------------------------------------------------------------------------*/


/**
FUNCTION
    InitDialog()
DESCRIPTION
    Displays the gos in the attributes graphic widget with the right attributes
    Initializes global variables.
**/
static void InitDialog()
{
    WhichAttributes = NAMED;
    named_pen = all_attrib.pen;
    named_brush = all_attrib.brush;
    named_text = all_attrib.text;
    
    if (strcmp(all_attrib.PenText, UNNAMED) != 0)
	FromShPenToNamedPen(all_attrib.PenText);
    ModifyGoNamedPenSheet();

    if (strcmp(all_attrib.BrushText, UNNAMED) != 0)
	FromShBrushToNamedBrush(all_attrib.BrushText);
    ModifyGoNamedBrushSheet();
    
    if (strcmp(all_attrib.TextBrushText, UNNAMED) != 0)
	FromShTextToNamedText(all_attrib.TextBrushText);
    ModifyGoNamedTextSheet();
    
}


/**
FUNCTION
    VerifFunction()
DESCRIPTION
    Verifies if the name is correct
**/
void VerifFunction(Widget w, caddr_t, caddr_t)
{
    if (w == penText || w == brushText || w == textBrushText)
	WhichAttributes = NAMED;
    else
	WhichAttributes = EDIT;

    int nb;
    char* object = XmTextFieldGetString(w);
    if (w == penText && NamedPen.get(object)) {
	FromShPenToNamedPen(object);
	ModifyGoNamedPenSheet();
	WhichObject = NONE;
    } else
	if (w == penTextEdit && NamedPen.get(object)) {
	    FromShPenToEditPen(object);
	    ModifyGoEditPenSheet();
	} else
	    if (w == brushText && NamedBrush.get(object)) {
		FromShBrushToNamedBrush(object);
		ModifyGoNamedBrushSheet();
		WhichObject = NONE;
	    } else
		if (w == brushTextEdit && NamedBrush.get(object)) {
		    FromShBrushToEditBrush(object);
		    ModifyGoEditBrushSheet();
		} else
		    if (w == textBrushText && NamedText.get(object, nb)) {
			FromShTextToNamedText(object);
			ModifyGoNamedTextSheet();
			WhichObject = NONE;
		    } else
		      if (w == textBrushTextEdit &&
			  NamedText.get(object, nb)) {
			  FromShTextToEditText(object);
			  ModifyGoEditTextSheet();
		      } else {
			NotNamedObject(NULL, (Widget)main_window, NULL);
			if (w == penText) {
			    XmTextFieldSetString(w, named_pen.PenText);
			} else
			  if (w == brushText) {
			      XmTextFieldSetString(w, named_brush.BrushText);
			  } else
			    if (w == textBrushText) {
				XmTextFieldSetString(w, named_text.TextBrushText);
			    } else
			      if (w == penTextEdit) {
				  XmTextFieldSetString(w, pen.PenText);
			      } else
				if (w == brushTextEdit) {
				    XmTextFieldSetString(w, brush.BrushText);
				} else
				  if (w == textBrushTextEdit) {
				      XmTextFieldSetString(w, text.TextBrushText);
				  }
		    }
    
    GFREE(object);
}



/**
FUNCTION
    PrepareGCDialog()
DESCRIPTION
    Creates and initializes gos used in the attributes graphic widget.
**/
static void PrepareGCDialog(Widget drawWid)
{
    void* interf = (void*)XbGraphicGoInterf((XbGraphicWidget)drawWid);
    InitNamedGo(interf);
}



/**
FUNCTION
    ModifyNamedColor()
DESCRIPTION
    Modifies colors of gos used in the attributes graphic widget.
**/
void ModifyNamedColor(Widget w, int wc, caddr_t)
{
    WhichAttributes = NAMED;
    WhichColor = wc;
    char* color = XmTextFieldGetString(w);
    ModifyColor(w, color);
    GFREE(color);
}



/**
FUNCTION
    ModifyColor()
DESCRIPTION
    Modifies colors of gos used in the attributes graphic widget.
**/
void ModifyColor(Widget w, char* color)
{
    if (color[0] != '#') {
	XColor exact_rgb, real_rgb;
	Display* dpy = XtDisplay(w);
	if (! XLookupColor(dpy, DefaultColormap(dpy, DefaultScreen(dpy)),
			   color, &exact_rgb, &real_rgb)) {
	    switch(WhichColor) {
	      case PEN_COLOR:
		if (WhichAttributes == NAMED) {
		    XmTextFieldSetString(w, named_pen.Color);
		    ModifyColor(w, named_pen.Color);
		    return;
		} else {
		    XmTextFieldSetString(w, pen.Color);
		    ModifyColor(w, pen.Color);
		    return;
		}
		break;
	      case PEN_ALT_COLOR:
		if (WhichAttributes == NAMED) {
		    XmTextFieldSetString(w, named_pen.AltColor);
		    ModifyColor(w, named_pen.AltColor);
		    return;
		} else {
		    XmTextFieldSetString(w, pen.AltColor);
		    ModifyColor(w, pen.AltColor);
		    return;
		}
		break;
	      case BRUSH_COLOR:
		if (WhichAttributes == NAMED) {
		    XmTextFieldSetString(w, named_brush.Color);
		    ModifyColor(w, named_brush.Color);
		    return;
		} else {
		    XmTextFieldSetString(w, brush.Color);
		    ModifyColor(w, brush.Color);
		    return;
		}
		break;
	      case TEXT_COLOR:
		if (WhichAttributes == NAMED) {
		    XmTextFieldSetString(w, named_text.Color);
		    ModifyColor(w, named_text.Color);
		    return;
		} else {
		    XmTextFieldSetString(w, text.Color);
		    ModifyColor(w, text.Color);
		    return;
		}
		break;
	    }
	}
	
	GRgb rgb(real_rgb.red, real_rgb.green, real_rgb.blue);
	ModifyGoColor(&rgb, color);
    } else {
	GRgb rgb = HexaStringToRgb(color);
	ModifyGoColor(&rgb, color);
    }
}



/**
FUNCTION
    ReinitText()
DESCRIPTION
    Reinits named pen name.
**/
static void ReinitText()
{
    XmTextFieldSetString(penText, all_attrib.PenText);
    XmTextFieldSetString(brushText, all_attrib.BrushText);
    XmTextFieldSetString(textBrushText, all_attrib.TextBrushText);
}



                            /*************************
			     * SAVE NAMED ATTRIBUTES *
			     *************************/



static void WriteFile(Widget, Widget dialog)
{
    void* stream = OpenWrite();
    if (! stream) {
	Error(main_window);
	return;
    }

    int nb_to_save;
    XmString* save_table;
    Arg args[5];
    int n = 0;
    XtSetArg(args[n], XmNselectedItemCount, &nb_to_save); n++;
    XtSetArg(args[n], XmNselectedItems, &save_table); n++;
    XtGetValues(savePenList, args, n);
    for (int i = 0; i < nb_to_save; i++) {
	char* str;
	XmStringGetLtoR(save_table[i], XmSTRING_DEFAULT_CHARSET, &str);
	if (!SaveGoObject(stream, str, PEN)) {
	    Error(main_window);
	    CloseAll(stream);
	    return;
	}
    }
    XmListDeselectAllItems(savePenList);
    
    n = 0;
    XtSetArg(args[n], XmNselectedItemCount, &nb_to_save); n++;
    XtSetArg(args[n], XmNselectedItems, &save_table); n++;
    XtGetValues(saveBrushList, args, n);
    for (i = 0; i < nb_to_save; i++) {
	char* str;
	XmStringGetLtoR(save_table[i], XmSTRING_DEFAULT_CHARSET, &str);
	if (!SaveGoObject(stream, str, BRUSH)) {
	    Error(main_window);
	    CloseAll(stream);
	    return;
	}
    }
    XmListDeselectAllItems(saveBrushList);

    n = 0;
    XtSetArg(args[n], XmNselectedItemCount, &nb_to_save); n++;
    XtSetArg(args[n], XmNselectedItems, &save_table); n++;
    XtGetValues(saveTxtBrushList, args, n);
    for (i = 0; i < nb_to_save; i++) {
	char* str;
	XmStringGetLtoR(save_table[i], XmSTRING_DEFAULT_CHARSET, &str);
	if (!SaveGoObject(stream, str, TEXTBRUSH)) {
	    Error(main_window);
	    CloseAll(stream);
	    return;
	}
    }
    XmListDeselectAllItems(saveTxtBrushList);

    CloseAll(stream);
    XtUnmanageChild(dialog);
}


static Widget CreatePenList(Widget parent)
{
    Arg args[5];
    Widget w = XmCreateLabelGadget(parent, "NamedPenLabel", NULL, 0);
    XtManageChild(w);
    int n = 0;
    XtSetArg(args[n], XmNlistSizePolicy, XmCONSTANT); n++;
    w = XmCreateScrolledList(parent, "NamedPenList", args, n);
    XtManageChild(w);

    return w;
}


static Widget CreateBrushList(Widget parent)
{
    Arg args[5];
    Widget w = XmCreateLabelGadget(parent, "NamedBrushLabel", NULL, 0);
    XtManageChild(w);
    int n = 0;
    XtSetArg(args[n], XmNlistSizePolicy, XmCONSTANT); n++;
    w = XmCreateScrolledList(parent, "NamedBrushList", args, n);
    XtManageChild(w);

    return w;
}


static Widget CreateTxtBrushList(Widget parent)
{
    Arg args[5];
    Widget w = XmCreateLabelGadget(parent, "NamedTxtBrushLabel", NULL, 0);
    XtManageChild(w);
    int n = 0;
    XtSetArg(args[n], XmNlistSizePolicy, XmCONSTANT); n++;
    w = XmCreateScrolledList(parent, "NamedTxtBrushList", args, n);
    XtManageChild(w);

    return w;
}


static Widget CreateSaveDialog(Widget parent)
{
    Arg args[5];
    int n = 0;
    XtSetArg(args[n], XmNautoUnmanage, False); n++;
    XtSetArg(args[n], XmNresizable, False); n++;
    XtSetArg(args[n], XmNresizePolicy, XmRESIZE_NONE); n++;
    Widget form = XmCreateFormDialog(parent, "SaveNamedDialog", args, n);
    savePenList = CreatePenList(form);
    saveBrushList = CreateBrushList(form);
    saveTxtBrushList = CreateTxtBrushList(form);
    Widget w = XmCreatePushButtonGadget(form, "WriteFile", NULL, 0);
    XtManageChild(w);
    XtAddCallback(w, XmNactivateCallback, (XtCallbackProc)WriteFile,
		  (XtPointer)form);
    w = XmCreatePushButtonGadget(form, "Unmap", NULL, 0);
    XtManageChild(w);
    XtAddCallback(w, XmNactivateCallback, (XtCallbackProc)UnmanageWidget,
		  (XtPointer)form);

    return form;
}


static void InitializePenList(Widget penList)
{
    Arg args[5];
    int nb;
    char** l = NamedPen.getKeys(nb);
    XmString* list_name = (XmString*) GMALLOC(nb * sizeof(XmString));
    for (int i = 0; i < nb; i++)
	list_name[i] = XmStringCreate(l[i], XmSTRING_DEFAULT_CHARSET);
    XtSetArg(args[0], XmNitems, (XmStringTable)list_name);
    XtSetArg(args[1], XmNitemCount, nb);
    XtSetValues(penList, args, 2);
    GFREE(l);
    GFREE(list_name);
}

    
static void InitializeBrushList(Widget brushList)
{
    Arg args[5];
    int nb;
    char** l = NamedBrush.getKeys(nb);
    XmString* list_name = (XmString*) GMALLOC(nb * sizeof(XmString));
    for (int i = 0; i < nb; i++)
	list_name[i] = XmStringCreate(l[i], XmSTRING_DEFAULT_CHARSET);
    XtSetArg(args[0], XmNitems, (XmStringTable)list_name);
    XtSetArg(args[1], XmNitemCount, nb);
    XtSetValues(brushList, args, 2);
    GFREE(l);
    GFREE(list_name);
}    


static void InitializeTextList(Widget txtList)
{
    Arg args[5];
    int nb;
    char** l = NamedText.getKeys(nb);
    XmString* list_name = (XmString*) GMALLOC(nb * sizeof(XmString));
    for (int i = 0; i < nb; i++)
	list_name[i] = XmStringCreate(l[i], XmSTRING_DEFAULT_CHARSET);
    XtSetArg(args[0], XmNitems, (XmStringTable)list_name);
    XtSetArg(args[1], XmNitemCount, nb);
    XtSetValues(txtList, args, 2);
    GFREE(l);
    GFREE(list_name);
}


void SaveNamedObjects(Widget, Widget parent, caddr_t)
{
    static Widget dialog = NULL;
    if (!dialog)
	dialog = CreateSaveDialog(parent);
    InitializePenList(savePenList);
    InitializeBrushList(saveBrushList);
    InitializeTextList(saveTxtBrushList);
    XtManageChild(dialog);
}



                            /****************************
			     * DESTROY NAMED ATTRIBUTES *
			     ****************************/



void ChangePenList()
{
    XmListDeleteAllItems(destroyPenList);
    InitializePenList(destroyPenList);
}


void ChangeBrushList()
{
    XmListDeleteAllItems(destroyBrushList);
    InitializeBrushList(destroyBrushList);
}


void ChangeTextBrushList()
{
    XmListDeleteAllItems(destroyTxtBrushList);
    InitializeTextList(destroyTxtBrushList);
}


static void DestroyNamed(Widget)
{
    int nb_to_destroy;
    XmString* destroy_table;
    Arg args[5];
    int n = 0;
    XtSetArg(args[n], XmNselectedItemCount, &nb_to_destroy); n++;
    XtSetArg(args[n], XmNselectedItems, &destroy_table); n++;
    XtGetValues(destroyPenList, args, n);
    for (int i = 0; i < nb_to_destroy; i++) {
	char* str;
	XmStringGetLtoR(destroy_table[i], XmSTRING_DEFAULT_CHARSET, &str);
	DestroyGoObject(str, PEN);
    }
    XmListDeselectAllItems(destroyPenList);

    n = 0;
    XtSetArg(args[n], XmNselectedItemCount, &nb_to_destroy); n++;
    XtSetArg(args[n], XmNselectedItems, &destroy_table); n++;
    XtGetValues(destroyBrushList, args, n);
    for (i = 0; i < nb_to_destroy; i++) {
	char* str;
	XmStringGetLtoR(destroy_table[i], XmSTRING_DEFAULT_CHARSET, &str);
	DestroyGoObject(str, BRUSH);
    }
    XmListDeselectAllItems(destroyBrushList);

    n = 0;
    XtSetArg(args[n], XmNselectedItemCount, &nb_to_destroy); n++;
    XtSetArg(args[n], XmNselectedItems, &destroy_table); n++;
    XtGetValues(destroyTxtBrushList, args, n);
    for (i = 0; i < nb_to_destroy; i++) {
	char* str;
	XmStringGetLtoR(destroy_table[i], XmSTRING_DEFAULT_CHARSET, &str);
	DestroyGoObject(str, TEXTBRUSH);
    }
    XmListDeselectAllItems(destroyTxtBrushList);
}


static Widget CreateDestroyDialog(Widget parent)
{
    Arg args[5];
    int n = 0;
    XtSetArg(args[n], XmNautoUnmanage, False); n++;
    XtSetArg(args[n], XmNresizable, False); n++;
    XtSetArg(args[n], XmNresizePolicy, XmRESIZE_NONE); n++;
    Widget form = XmCreateFormDialog(parent, "DestroyNamedDialog", args, n);
    destroyPenList = CreatePenList(form);
    destroyBrushList = CreateBrushList(form);
    destroyTxtBrushList = CreateTxtBrushList(form);
    Widget w = XmCreatePushButtonGadget(form, "DestroyNamed", NULL, 0);
    XtManageChild(w);
    XtAddCallback(w, XmNactivateCallback, (XtCallbackProc)DestroyNamed, NULL);
    w = XmCreatePushButtonGadget(form, "Unmap", NULL, 0);
    XtManageChild(w);
    XtAddCallback(w, XmNactivateCallback, (XtCallbackProc)UnmanageWidget,
		  (XtPointer)form);
    
    return form;
}


void DestroyNamedObjects(Widget, Widget parent, caddr_t)
{
    static Widget dialog = NULL;
    if (!dialog)
	dialog = CreateDestroyDialog(parent);
    InitializePenList(destroyPenList);
    InitializeBrushList(destroyBrushList);
    InitializeTextList(destroyTxtBrushList);
    XtManageChild(dialog);
}

