#ifdef REV_INFO
static char rcsid[] = "$Id: pensheet.cc,v 1.3 1992/12/10 16:24:53 jojic Exp $";
#endif
/*****************************************************************************
MODULE:  Go Editor
FILE:    pensheet.cc
AUTHOR:  Aline Jacomet
DATE:    May 22th 1991

DESCRIPTION:
    

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include <Go/Tools/StringAssoc.h>
#include <Go/Tools/Alloc.h>

#include <Xm/Form.h>
#include <Xm/PushBG.h>
#include <Xm/SeparatoG.h>
#include <Xm/LabelG.h>
#include <Xm/TextF.h>
#include <Xm/RowColumn.h>

#include "classes.h"
#include "main.h"




/***
 * widget function
 ***/
static Widget CreatePenDialog(Widget);

void MapPenAttrib(void*, void*, caddr_t);

static void ModifyLineStyle(Widget, int ls, caddr_t);
static void ModifyCapStyle(Widget, int ls, caddr_t);
static void ModifyJoinStyle(Widget, int ls, caddr_t);
static void ModifyPenSheet();
static void ReinitPenSheet(Widget, Boolean, caddr_t);
static void Verify(Widget, caddr_t, caddr_t);
static void NamedChooseGo(Widget, caddr_t, caddr_t);
void VerifyPen(Widget, caddr_t, caddr_t);


/***
 * Modify Go
 ***/
void ModifyLineWidth(Widget, int, caddr_t);



/***
 *  Static Variables
 ***/
static Widget penDialog;
static Widget name;
static Widget lineStyle_menu;
static Widget capStyle_menu;
static Widget joinStyle_menu;
static Widget lineWidth;
static Widget line_wid[3];
static Widget cap_wid[4];
static Widget join_wid[3];
static Widget color_label;
static Widget color_list;
static Widget altColor_label;
static Widget altColor_list;
static Widget lineWidth_label;
static Widget joinStyle_label;



/***
 *  Global output attributes
 ***/
PenAttributes named_pen;

Widget pen_color = NULL;
Widget pen_alt_color = NULL;

Boolean penMapped = False;

extern Widget penText;

extern GStringAssoc NamedPen;
extern AllAttributes all_attrib;



/*
 * EXTERN FUNCTIONS
 */

// in file namedattrib.cc;
extern void SensitiveButtons(Widget, Boolean*, caddr_t);
extern void UnmanageWidget(Widget, Widget, caddr_t);
extern void ModifyNamedColor(Widget, int, caddr_t);

// in file colorsel.cc;
extern void MapColorList(Widget, Widget, caddr_t);

// in file goAttrib.cc;
extern void ModifyGoLineWidth(double);
extern void ModifyGoCapStyle(int);
extern void ModifyGoJoinStyle(int);
extern void ModifyGoLineStyle(int);
extern void ModifyGoNamedPenSheet();
extern void CreateGoNamedPen();
extern void ModifyGoNamedPen();
extern int ChooseGo(int, void (*)(int, class Go*), int);

// in file message.cc;
extern void ObjectName(void*);
extern void AcknowledgePenName(void*);

// in file help.cc;
extern void MapHelpFile(void*, char*, void*);




/***
 * Redo textfield translations for line width.
 ***/
extern char *LineWidthEventBindings1;
extern char *LineWidthEventBindings2;
extern char *LineWidthEventBindings3;


/**
FUNCTION
    GetItem()
DESCRIPTION
    Returns the selected item in an option menu.
**/
int GetItem(Widget* tab, int max_item, Widget w)
{
    for (int i = 0; i < max_item; i++)
	if (tab[i] == w)
	    return i;
    return -1;
}



/**
FUNCTION
    MapPenAttrib()
DESCRIPTION
    Maps and creates, if necessary, the pen attributes editor.
**/
void MapPenAttrib(void*, void* parent, caddr_t)
{
    Boolean create = (penDialog == NULL);

    if (!create && XtIsManaged(penDialog))
	return;
    if (create) {
	XDefineCursor(XtDisplay((Widget)graphic), XtWindow((Widget)graphic),
		      clock_cursor);
	XDefineCursor(XtDisplay((Widget)main_window),
		      XtWindow((Widget)main_window), clock_cursor);
	XFlush(XtDisplay((Widget)graphic));
	penDialog = CreatePenDialog((Widget)parent);
    }

    ReinitPenSheet(NULL, TRUE, NULL);
    XtManageChild(penDialog);
    
    if (create) {
	XFlush(XtDisplay((Widget)graphic));
	XUndefineCursor(XtDisplay((Widget)graphic), XtWindow((Widget)graphic));
	XUndefineCursor(XtDisplay((Widget)main_window),
			XtWindow((Widget)main_window));
    }
    penMapped = True;
}


/**
FUNCTION
    CreatePenDialog()
DESCRIPTION
    Creates the pen attributes editor.
**/
static Widget CreatePenDialog(Widget parent)
{
    Widget w;
    Arg args[20];
    int n = 0;
    XtSetArg(args[n], XmNautoUnmanage, False); n++;
    XtSetArg(args[n], XmNresizePolicy, XmRESIZE_NONE); n++;
    Widget ps = XmCreateFormDialog(parent, "PenDialog", args, n);
    XtAddCallback(ps, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)PEN_ATTRIB_FILE);

    /* Apply, Reinit, Cancel buttons */
    w = XmCreatePushButtonGadget(ps, "Apply", NULL, 0);
    XtManageChild(w);
    XtAddCallback(w, XmNactivateCallback,
		  (XtCallbackProc)SensitiveButtons, (XtPointer)&penMapped);
    XtAddCallback(w, XmNactivateCallback, (XtCallbackProc)Verify,
		  (XtPointer)ps);
    XtAddCallback(w, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)PEN_ATTRIB_FILE);

    w = XmCreatePushButtonGadget(ps, "Reinit", NULL, 0);
    XtManageChild(w);
    XtAddCallback(w, XmNactivateCallback, (XtCallbackProc)ReinitPenSheet,
		  (XtPointer) TRUE);
    XtAddCallback(w, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)PEN_ATTRIB_FILE);
    
    w = XmCreatePushButtonGadget(ps, "Cancel", NULL, 0);
    XtManageChild(w);
    XtAddCallback(w, XmNactivateCallback,
		  (XtCallbackProc)SensitiveButtons, (XtPointer)&penMapped);
    XtAddCallback(w, XmNactivateCallback,
		  (XtCallbackProc)ModifyPenSheet, NULL);
    XtAddCallback(w, XmNactivateCallback,
		  (XtCallbackProc) UnmanageWidget, (XtPointer)ps);
    XtAddCallback(w, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)PEN_ATTRIB_FILE);

    /* click button */
    w = XmCreatePushButtonGadget(ps, "ClickButton", NULL, 0);
    XtManageChild(w);
    XtAddCallback(w, XmNactivateCallback, (XtCallbackProc)NamedChooseGo, NULL);
    XtAddCallback(w, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)PEN_ATTRIB_FILE);
    
    /* label and separators */
    w = XmCreateSeparatorGadget(ps, "Separator1", NULL, 0);
    XtAddCallback(w, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)PEN_ATTRIB_FILE);
    XtManageChild(w);
    w = XmCreateSeparatorGadget(ps, "Separator2", NULL, 0);
    XtAddCallback(w, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)PEN_ATTRIB_FILE);
    XtManageChild(w);
    w = XmCreateLabelGadget(ps, "PenLabel", NULL, 0);
    XtManageChild(w);
    XtAddCallback(w, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)PEN_ATTRIB_FILE);
    
    /* name */
    Widget nameLabel = XmCreateLabelGadget(ps, "NameLabel", NULL, 0);
    XtAddCallback(nameLabel, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)PEN_ATTRIB_FILE);
    XtManageChild(nameLabel);

    name = XmCreateTextField(ps, "Name", NULL, 0);
    XtAddCallback(name, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)PEN_ATTRIB_FILE);
    XtManageChild(name);

    /* color and alternate color */
    color_label = XmCreateLabelGadget(ps, "ColorLabel", NULL, 0);
    XtAddCallback(color_label, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)PEN_ATTRIB_FILE);
    XtManageChild(color_label);
    n = 0;
    XtSetArg(args[n], XmNmarginWidth, 5); n++;
    XtSetArg(args[n], XmNmarginHeight, 5); n++;
    color_list = XmCreatePushButtonGadget(ps, "ColorList", args, n);
    XtAddCallback(color_list, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)PEN_ATTRIB_FILE);
    XtManageChild(color_list);
    pen_color = XmCreateTextField(ps, "ColorName", NULL, 0);
    XtAddCallback(color_list, XmNactivateCallback,
		  (XtCallbackProc)MapColorList, (XtPointer)pen_color);
    XtAddCallback(pen_color, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)PEN_ATTRIB_FILE);
    XtAddCallback(pen_color, XmNactivateCallback,
		  (XtCallbackProc) ModifyNamedColor, PEN_COLOR);
    XtManageChild(pen_color);

    altColor_label = XmCreateLabelGadget(ps, "AltColorLabel", NULL, 0);
    XtAddCallback(altColor_label, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)PEN_ATTRIB_FILE);
    XtManageChild(altColor_label);
    n = 0;
    XtSetArg(args[n], XmNmarginWidth, 5); n++;
    XtSetArg(args[n], XmNmarginHeight, 5); n++;
    altColor_list = XmCreatePushButtonGadget(ps, "AltColorList", args, n);
    XtAddCallback(altColor_list, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)PEN_ATTRIB_FILE);
    XtManageChild(altColor_list);
    pen_alt_color = XmCreateTextField(ps, "AltColorName", NULL, 0);
    XtAddCallback(altColor_list, XmNactivateCallback,
		  (XtCallbackProc)MapColorList, (XtPointer)pen_alt_color);
    XtAddCallback(pen_alt_color, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)PEN_ATTRIB_FILE);
    XtAddCallback(pen_alt_color, XmNactivateCallback,
		  (XtCallbackProc) ModifyNamedColor,
		  (XtPointer) PEN_ALT_COLOR);
    XtManageChild(pen_alt_color);

    /* line width */
    char* line_width_event_bindings =
	(char *) GMALLOC(strlen(LineWidthEventBindings1) +
			  strlen(LineWidthEventBindings2) +
			  strlen(LineWidthEventBindings3) + 1);

    strcpy(line_width_event_bindings, LineWidthEventBindings1);
    strcat(line_width_event_bindings, LineWidthEventBindings2);
    strcat(line_width_event_bindings, LineWidthEventBindings3);
    XtTranslations line_width_translations =
	XtParseTranslationTable(line_width_event_bindings);
    GFREE(line_width_event_bindings);
    
    lineWidth_label = XmCreateLabelGadget(ps, "LineWidthLabel",
						 NULL, 0);
    XtAddCallback(lineWidth_label, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)PEN_ATTRIB_FILE);
    XtManageChild(lineWidth_label);

    n = 0;
    XtSetArg(args[n], XmNtranslations, line_width_translations); n++;
    lineWidth = XmCreateTextField(ps, "LineWidth", args, n);
    XtManageChild(lineWidth);
    XtAddCallback(lineWidth, XmNactivateCallback,
		  (XtCallbackProc)ModifyLineWidth, (XtPointer)NAMED);
    XtAddCallback(lineWidth, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)PEN_ATTRIB_FILE);

    /* line style */
    Widget lineStyle_label = XmCreateLabelGadget(ps, "LineStyleLabel", NULL,0);
    XtAddCallback(lineStyle_label, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)PEN_ATTRIB_FILE);
    XtManageChild(lineStyle_label);
    Widget lineStyle_pull = XmCreatePulldownMenu(ps, "LineStylePull", NULL, 0);
    n = 0;
    XtSetArg(args[n], XmNsubMenuId, lineStyle_pull); n++;
    XtSetArg(args[n], XmNmarginWidth, 0); n++;
    XtSetArg(args[n], XmNmarginHeight, 0); n++;
    lineStyle_menu = XmCreateOptionMenu(ps, "LineStyleMenu", args, n);
    XtManageChild(lineStyle_menu);
    n = 0;
    XtSetArg(args[n], XmNlabelPixmap, lineSolidIcon); n++;
    XtSetArg(args[n], XmNlabelType, XmPIXMAP); n++;
    line_wid[LINESOLID] = XmCreatePushButtonGadget(lineStyle_pull, "lineSolid",
					     args, n);
    XtAddCallback(line_wid[LINESOLID], XmNactivateCallback,
		  (XtCallbackProc)ModifyLineStyle, (XtPointer)LINESOLID);
    XtAddCallback(line_wid[LINESOLID], XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)PEN_ATTRIB_FILE);
    n = 0;
    XtSetArg(args[n], XmNlabelPixmap, lineOnOffDashIcon); n++;
    XtSetArg(args[n], XmNlabelType, XmPIXMAP); n++;
    line_wid[LINEONOFF] = XmCreatePushButtonGadget(lineStyle_pull,
						   "lineOnOffDash", args, n);
    XtAddCallback(line_wid[LINEONOFF], XmNactivateCallback,
		  (XtCallbackProc)ModifyLineStyle, (XtPointer)LINEONOFF);
    XtAddCallback(line_wid[LINEONOFF], XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)PEN_ATTRIB_FILE);
    n = 0;
    XtSetArg(args[n], XmNlabelPixmap, lineDoubleDashIcon); n++;
    XtSetArg(args[n], XmNlabelType, XmPIXMAP); n++;
    line_wid[LINEDOUBLE] = XmCreatePushButtonGadget(lineStyle_pull,
						    "lineDoubleDash", args, n);
    XtAddCallback(line_wid[LINEDOUBLE], XmNactivateCallback,
		  (XtCallbackProc)ModifyLineStyle, (XtPointer)LINEDOUBLE);
    XtAddCallback(line_wid[LINEDOUBLE], XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)PEN_ATTRIB_FILE);
    XtManageChildren(line_wid, 3);

    /* cap style */
    Widget capStyle_label = XmCreateLabelGadget(ps, "CapStyleLabel", NULL, 0);
    XtAddCallback(capStyle_label, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)PEN_ATTRIB_FILE);
    XtManageChild(capStyle_label);
    Widget capStyle_pull = XmCreatePulldownMenu(ps, "CapStylePull", NULL, 0);
    n = 0;
    XtSetArg(args[n], XmNsubMenuId, capStyle_pull); n++;
    XtSetArg(args[n], XmNmarginWidth, 0); n++;
    XtSetArg(args[n], XmNmarginHeight, 0); n++;
    capStyle_menu = XmCreateOptionMenu(ps, "CapStyleMenu", args, n);
    XtManageChild(capStyle_menu);
    cap_wid[0] = NULL;
    cap_wid[1] = XmCreatePushButtonGadget(capStyle_pull, "capButt", NULL, 0);
    XtAddCallback(cap_wid[1], XmNactivateCallback,
		  (XtCallbackProc)ModifyCapStyle, (XtPointer)CAPBUTT);
    XtAddCallback(cap_wid[1], XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)PEN_ATTRIB_FILE);
    XtManageChild(cap_wid[1]);
    cap_wid[2] = XmCreatePushButtonGadget(capStyle_pull, "capRound",
					  NULL, 0);
    XtAddCallback(cap_wid[2], XmNactivateCallback,
		  (XtCallbackProc)ModifyCapStyle, (XtPointer)CAPROUND);
    XtAddCallback(cap_wid[2], XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)PEN_ATTRIB_FILE);
    XtManageChild(cap_wid[2]);
    cap_wid[3] = XmCreatePushButtonGadget(capStyle_pull,
					  "capProjecting", NULL, 0);
    XtAddCallback(cap_wid[3], XmNactivateCallback,
		  (XtCallbackProc)ModifyCapStyle, (XtPointer)CAPPROJECT);
    XtAddCallback(cap_wid[3], XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)PEN_ATTRIB_FILE);
    XtManageChild(cap_wid[3]);
    
    /* join style */
    joinStyle_label = XmCreateLabelGadget(ps, "JoinStyleLabel", NULL, 0);
    XtAddCallback(joinStyle_label, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)PEN_ATTRIB_FILE);
    XtManageChild(joinStyle_label);
    Widget joinStyle_pull = XmCreatePulldownMenu(ps, "JoinStylePull", NULL, 0);
    n = 0;
    XtSetArg(args[n], XmNsubMenuId, joinStyle_pull); n++;
    XtSetArg(args[n], XmNmarginWidth, 0); n++;
    XtSetArg(args[n], XmNmarginHeight, 0); n++;
    joinStyle_menu = XmCreateOptionMenu(ps, "JoinStyleMenu", args, n);
    XtManageChild(joinStyle_menu);
    n = 0;
    XtSetArg(args[n], XmNlabelPixmap, joinMiterIcon); n++;
    XtSetArg(args[n], XmNlabelType, XmPIXMAP); n++;
    join_wid[JOINMITER] = XmCreatePushButtonGadget(joinStyle_pull,
						   "joinMitter", args, n);
    XtAddCallback(join_wid[JOINMITER], XmNactivateCallback,
		  (XtCallbackProc)ModifyJoinStyle, (XtPointer)JOINMITER);
    XtAddCallback(join_wid[JOINMITER], XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)PEN_ATTRIB_FILE);
    n = 0;
    XtSetArg(args[n], XmNlabelPixmap, joinRoundIcon); n++;
    XtSetArg(args[n], XmNlabelType, XmPIXMAP); n++;
    join_wid[JOINROUND] = XmCreatePushButtonGadget(joinStyle_pull, "joinRound",
						   args, n);
    XtAddCallback(join_wid[JOINROUND], XmNactivateCallback,
		  (XtCallbackProc)ModifyJoinStyle, (XtPointer)JOINROUND);
    XtAddCallback(join_wid[JOINROUND], XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)PEN_ATTRIB_FILE);
    n = 0;
    XtSetArg(args[n], XmNlabelPixmap, joinBevelIcon); n++;
    XtSetArg(args[n], XmNlabelType, XmPIXMAP); n++;
    join_wid[JOINBEVEL] = XmCreatePushButtonGadget(joinStyle_pull, "joinBevel",
						   args, n);
    XtAddCallback(join_wid[JOINBEVEL], XmNactivateCallback,
		  (XtCallbackProc)ModifyJoinStyle, (XtPointer)JOINBEVEL);
    XtAddCallback(join_wid[JOINBEVEL], XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)PEN_ATTRIB_FILE);
    XtManageChildren(join_wid, 3);
    
    /* attach widgets */
    n = 0;
    XtSetArg(args[n], XmNleftAttachment, XmATTACH_OPPOSITE_WIDGET); n++;
    XtSetArg(args[n], XmNleftWidget, nameLabel); n++;
    XtSetValues(lineStyle_label, args, n);
    XtSetValues(capStyle_label, args, n);
    XtSetValues(color_label, args, n);
    n = 0;
    XtSetArg(args[n], XmNleftAttachment, XmATTACH_OPPOSITE_WIDGET); n++;
    XtSetArg(args[n], XmNleftWidget, name); n++;
    XtSetValues(lineStyle_menu, args, n);
    XtSetValues(capStyle_menu, args, n);
    XtSetValues(color_list, args, n);
    n = 0;
    XtSetArg(args[n], XmNrightAttachment, XmATTACH_OPPOSITE_WIDGET); n++;
    XtSetArg(args[n], XmNrightWidget, pen_color); n++;
    XtSetValues(name, args, n);
    XtSetValues(lineStyle_menu, args, n);
    XtSetValues(capStyle_menu, args, n);
    n = 0;
    XtSetArg(args[n], XmNleftAttachment, XmATTACH_OPPOSITE_WIDGET); n++;
    XtSetArg(args[n], XmNleftWidget, altColor_label); n++;
    XtSetValues(lineWidth_label, args, n);
    XtSetValues(joinStyle_label, args, n);
    n = 0;
    XtSetArg(args[n], XmNleftAttachment, XmATTACH_OPPOSITE_WIDGET); n++;
    XtSetArg(args[n], XmNleftWidget, altColor_list); n++;
    XtSetValues(lineWidth, args, n);
    XtSetValues(joinStyle_menu, args, n);
    n = 0;
    XtSetArg(args[n], XmNrightAttachment, XmATTACH_OPPOSITE_WIDGET); n++;
    XtSetArg(args[n], XmNrightWidget, pen_alt_color); n++;
    XtSetValues(lineWidth, args, n);
    XtSetValues(joinStyle_menu, args, n);

    return ps;
}


/**
FUNCTION
    ModifyLineWidth()
DESCRIPTION
    Modifies line width in the attributes widget.
**/
void ModifyLineWidth(Widget w, int i, caddr_t)
{
    WhichAttributes = i;
    char* lw = XmTextFieldGetString(w);
    ModifyGoLineWidth((double)atof(lw));
}


/**
FUNCTION
    ModifyLineStyle()
DESCRIPTION
    
**/
static void ModifyLineStyle(Widget, int ls, caddr_t)
{
    WhichAttributes = NAMED;
    ModifyGoLineStyle(ls);
}



/**
FUNCTION
    ModifyCapStyle()
DESCRIPTION
    
**/
static void ModifyCapStyle(Widget, int ls, caddr_t)
{
    WhichAttributes = NAMED;
    ModifyGoCapStyle(ls);
}



/**
FUNCTION
    ModifyJoinStyle()
DESCRIPTION
    
**/
static void ModifyJoinStyle(Widget, int ls, caddr_t)
{
    WhichAttributes = NAMED;
    ModifyGoJoinStyle(ls);
}


/**
FUNCTION
    ModifyPenSheet()
DESCRIPTION
    
**/
static void ModifyPenSheet()
{
    WhichAttributes = NAMED;
    ModifyGoNamedPenSheet();
}


/*****************************************************************************/


/**
FUNCTION
    ReinitPenSheet()
DESCRIPTION
    
**/
static void ReinitPenSheet(Widget, Boolean reinit_name, caddr_t)
{
  if (reinit_name)
    XmTextFieldSetString(name, named_pen.PenText);
    XmTextFieldSetString(lineWidth, named_pen.LineWidth);
    XmTextFieldSetString(pen_color, named_pen.Color);
    XmTextFieldSetString(pen_alt_color, named_pen.AltColor);
    
    Arg arg;
    XtSetArg(arg, XmNmenuHistory, line_wid[named_pen.LineStyle]);
    XtSetValues(lineStyle_menu, &arg, 1);
    
    XtSetArg(arg, XmNmenuHistory, cap_wid[named_pen.CapStyle]);
    XtSetValues(capStyle_menu, &arg, 1);
    
    XtSetArg(arg, XmNmenuHistory, join_wid[named_pen.JoinStyle]);
    XtSetValues(joinStyle_menu, &arg, 1);

    WhichAttributes = NAMED;
    ModifyGoNamedPenSheet();
}


/**
FUNCTION
    ReinitPen()
DESCRIPTION
    
**/
void ReinitPen()
{
    named_pen = all_attrib.pen;
    if (penDialog)
	ReinitPenSheet(NULL, TRUE, NULL);
}


/**
FUNCTION
    Verify()
DESCRIPTION
    
**/
static void Verify(Widget, caddr_t parent, caddr_t)
{
    WhichAttributes = NAMED;
    char *name_str = XmTextFieldGetString(name);
    if (name_str[0] == '\0' ||
	strcmp(name_str, UNNAMED) == 0) {
	GFREE(name_str);
	ObjectName(penDialog);
	return;
    }
    if (NamedPen.get(name_str)) {
	AcknowledgePenName(parent);
	GFREE(name_str);
	return;
    }
    VerifyPen(NULL, parent, NULL);
    GFREE(name_str);
}

void VerifyPen(Widget, caddr_t parent, caddr_t)
{
    Arg arg;
    Widget w;

    char *name_str = XmTextFieldGetString(name);

    XtSetArg(arg, XmNmenuHistory, &w);
    XtGetValues(lineStyle_menu, &arg, 1);
    named_pen.LineStyle = GetItem(line_wid, 3, w);

    XtSetArg(arg, XmNmenuHistory, &w);
    XtGetValues(capStyle_menu, &arg, 1);
    named_pen.CapStyle = GetItem(cap_wid, 4, w);

    XtSetArg(arg, XmNmenuHistory, &w);
    XtGetValues(joinStyle_menu, &arg, 1);
    named_pen.JoinStyle = GetItem(join_wid, 3, w);

    GFREE(named_pen.LineWidth);
    named_pen.LineWidth = XmTextFieldGetString(lineWidth);

    GFREE(named_pen.Color);
    named_pen.Color = XmTextFieldGetString(pen_color);

    GFREE(named_pen.AltColor);
    named_pen.AltColor = XmTextFieldGetString(pen_alt_color);

    if (strcmp(name_str, UNNAMED) != 0) {
	GFREE(named_pen.PenText);
	named_pen.PenText = (char*)GMALLOC(strlen(name_str) + 1);
	strcpy(named_pen.PenText, name_str);
	if (NamedPen.get(name_str))
	    ModifyGoNamedPen();
	else
	    CreateGoNamedPen();
	XmTextFieldSetString(penText, named_pen.PenText);
    }

    if (strcmp(all_attrib.PenText, name_str) == 0) {
	/*
	 * This pen is used as current attributes ==> I have to
	 * modify all_attrib.pen;
	 */
	all_attrib.pen = named_pen;
    }
    GFREE(name_str);

    XtUnmanageChild((Widget)parent);
}


/**
FUNCTION
    NamedChooseGo()
DESCRIPTION
    
**/
void NamedChooseGo_cb (int, Go*)
{
    ReinitPenSheet(NULL, FALSE, NULL);
}

static void NamedChooseGo(Widget, caddr_t, caddr_t)
{
    WhichAttributes = NAMED;
    ChooseGo(PEN, NamedChooseGo_cb, 0);
}


