#ifdef REV_INFO
static char rcsid[] = "$Id: pixmap.cc,v 1.3 1992/09/24 07:20:01 jacomet Exp $";
#endif
/*****************************************************************************
MODULE:  Go editor
FILE:    pixmap.c
AUTHOR:  Patricia Langle and Aline Jacomet
DATE:    May 14th 1991

DESCRIPTION:
    Creation of the pixmaps used as icons.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#include "main.h"

#include "PIXMAPS/polyline"
#include "PIXMAPS/polygon"
#include "PIXMAPS/hexagon"
#include "PIXMAPS/text"
#include "PIXMAPS/circle"
#include "PIXMAPS/ellipse"
#include "PIXMAPS/ellipse_arc"
#include "PIXMAPS/circle_arc"
#include "PIXMAPS/image"
#include "PIXMAPS/hand"
#include "PIXMAPS/rectangle"
#include "PIXMAPS/square"
#include "PIXMAPS/curve"

#include "PIXMAPS/iPolyline"
#include "PIXMAPS/iPolygon"
#include "PIXMAPS/iHexagon"
#include "PIXMAPS/iText"
#include "PIXMAPS/iCircle"
#include "PIXMAPS/iEllipse"
#include "PIXMAPS/iEllipse_arc"
#include "PIXMAPS/iCircle_arc"
#include "PIXMAPS/iImage"
#include "PIXMAPS/iHand"
#include "PIXMAPS/iRectangle"
#include "PIXMAPS/iSquare"
#include "PIXMAPS/iCurve"

#include "PIXMAPS/lineSolid"
#include "PIXMAPS/lineOnOffDash"
#include "PIXMAPS/lineDoubleDash"
#include "PIXMAPS/evenOddRule"
#include "PIXMAPS/windingRule"

#include "PIXMAPS/joinBevel"
#include "PIXMAPS/joinMiter"
#include "PIXMAPS/joinRound"

#include "PIXMAPS/alignBottom"
#include "PIXMAPS/alignTop"
#include "PIXMAPS/alignLeft"
#include "PIXMAPS/alignRight"
#include "PIXMAPS/alignHorizon"
#include "PIXMAPS/alignVertical"
#include "PIXMAPS/alignTopBot"
#include "PIXMAPS/alignBotTop"
#include "PIXMAPS/alignRightLeft"
#include "PIXMAPS/alignLeftRight"

#include "PIXMAPS/spaceX"
#include "PIXMAPS/spaceY"

#include "PIXMAPS/mirrorI"
#include "PIXMAPS/mirrorJ"
#include "PIXMAPS/mirrorIJ"

#ifdef __GNUC__
/* copy of PIXMAPS/lw1: include "PIXMAPS/lw1" makes an error with gnu version 2  */
/* lw1,v 1.1.1.1 1992/03/18 09:01:16 uep Exp $ */
#define lw1_width 16
#define lw1_height 16
static char lw1_bits[] = {
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xfe, 0x7f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
#else
#include "PIXMAPS/lw1"
#endif

#include "PIXMAPS/lw2"
#include "PIXMAPS/lw3"
#include "PIXMAPS/lw4"
#include "PIXMAPS/lw5"
#include "PIXMAPS/lw6"
#include "PIXMAPS/lw7"
#include "PIXMAPS/lw8"
#include "PIXMAPS/lw9"
#include "PIXMAPS/lw10"

#include "PIXMAPS/rotate45"
#include "PIXMAPS/rotate90"
#include "PIXMAPS/rotate135"
#include "PIXMAPS/rotate180"
#include "PIXMAPS/rotate225"
#include "PIXMAPS/rotate270"
#include "PIXMAPS/rotate315"

#include <Go/Tools/Rgb.h>


Pixmap rectangleIcon, circleIcon, polylineIcon, imageIcon, handIcon,
       polygonIcon, hexagonIcon, textIcon, ellipseIcon, ellipseArcIcon,
       circleArcIcon, squareIcon, curveIcon;

Pixmap iRectangleIcon, iCircleIcon, iPolylineIcon, iImageIcon, iHandIcon,
       iPolygonIcon, iHexagonIcon, iTextIcon, iEllipseIcon, iEllipseArcIcon,
       iCircleArcIcon, iSquareIcon, iCurveIcon;

Pixmap lineSolidIcon, lineOnOffDashIcon, lineDoubleDashIcon,
       evenOddRuleIcon, windingRuleIcon;

Pixmap joinMiterIcon, joinBevelIcon, joinRoundIcon;

Pixmap alignBottomIcon, alignTopIcon, alignLeftIcon, alignRightIcon,
       alignHorizontalIcon, alignVerticalIcon, alignTopBotIcon,
       alignBotTopIcon, alignRightLeftIcon, alignLeftRightIcon;

Pixmap spaceXIcon, spaceYIcon;

Pixmap mirrorIIcon, mirrorJIcon, mirrorIJIcon;

Pixmap lwIcon[10];

Pixmap rotate45Icon, rotate90Icon, rotate135Icon, rotate180Icon, rotate225Icon,
       rotate270Icon, rotate315Icon;


extern GRgb HexaStringToRgb(const char*);


void
CreatePixmaps(Widget toplevel)
{
    char* fcolor = XGetDefault(XtDisplay(toplevel), "XmGoDraw",
			       "iconForeground");
    char* bcolor = XGetDefault (XtDisplay(toplevel),"XmGoDraw",
				"iconBackground");

    XColor usedColor, exactColor;

    if (fcolor[0] == '#') {
	GRgb f = HexaStringToRgb(fcolor);
	usedColor.red = f.red;
	usedColor.green = f.green;
	usedColor.blue = f.blue;
	XAllocColor(XtDisplay(toplevel), 
		    DefaultColormap(XtDisplay(toplevel), 
				    DefaultScreen(XtDisplay(toplevel))),
		    &usedColor); 
    } else 
	XAllocNamedColor(XtDisplay(toplevel), 
			 DefaultColormap(XtDisplay(toplevel), 
					 DefaultScreen(XtDisplay(toplevel))),
			 fcolor, &usedColor, &exactColor); 

    unsigned long fg = usedColor.pixel;
    
    if (bcolor[0] == '#') {
	GRgb b = HexaStringToRgb(bcolor);
	usedColor.red = b.red;
	usedColor.green = b.green;
	usedColor.blue = b.blue;
	XAllocColor(XtDisplay(toplevel), 
		    DefaultColormap(XtDisplay(toplevel), 
				    DefaultScreen(XtDisplay(toplevel))),
		    &usedColor); 
    } else 
	XAllocNamedColor(XtDisplay(toplevel), 
			 DefaultColormap(XtDisplay(toplevel), 
					 DefaultScreen(XtDisplay(toplevel))),
			 bcolor, &usedColor, &exactColor); 

    unsigned long bg = usedColor.pixel;


    polylineIcon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)),
			polyline_bits, polyline_width, polyline_height,
			fg, bg, DefaultDepth(XtDisplay(toplevel), 
				     DefaultScreen(XtDisplay(toplevel))));

    polygonIcon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), polygon_bits,
			polygon_width, polygon_height, fg, bg,
			DefaultDepth(XtDisplay(toplevel), 
				     DefaultScreen(XtDisplay(toplevel))));

    textIcon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), text_bits,
			text_width, text_height, fg, bg,
			DefaultDepth(XtDisplay(toplevel), 
				     DefaultScreen(XtDisplay(toplevel))));

    hexagonIcon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), hexagon_bits,
			hexagon_width, hexagon_height, fg, bg,
			DefaultDepth(XtDisplay(toplevel), 
				     DefaultScreen(XtDisplay(toplevel))));

    circleIcon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), circle_bits,
			circle_width, circle_height, fg, bg,
			DefaultDepth(XtDisplay(toplevel), 
				     DefaultScreen(XtDisplay(toplevel))));

    ellipseIcon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), ellipse_bits,
			ellipse_width, ellipse_height, fg, bg,
			DefaultDepth(XtDisplay(toplevel), 
				     DefaultScreen(XtDisplay(toplevel))));

    ellipseArcIcon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)),
			ellipse_arc_bits,
			ellipse_arc_width, ellipse_arc_height, fg, bg,
			DefaultDepth(XtDisplay(toplevel), 
				     DefaultScreen(XtDisplay(toplevel))));

    circleArcIcon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)),
			circle_arc_bits,
			circle_arc_width, circle_arc_height, fg, bg,
			DefaultDepth(XtDisplay(toplevel), 
				     DefaultScreen(XtDisplay(toplevel))));

    imageIcon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), image_bits,
			image_width, image_height, fg, bg, 
			DefaultDepth(XtDisplay(toplevel),
				     DefaultScreen(XtDisplay(toplevel))));

    handIcon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), hand_bits,
			hand_width, hand_height, fg, bg, 
			DefaultDepth(XtDisplay(toplevel),
				     DefaultScreen(XtDisplay(toplevel))));

    rectangleIcon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), rectangle_bits,
			rectangle_width, rectangle_height, fg, bg, 
			DefaultDepth(XtDisplay(toplevel),
				     DefaultScreen(XtDisplay(toplevel))));

    squareIcon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), square_bits,
			square_width, square_height, fg, bg, 
			DefaultDepth(XtDisplay(toplevel),
				     DefaultScreen(XtDisplay(toplevel))));

    curveIcon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), curve_bits,
			curve_width, curve_height, fg, bg, 
			DefaultDepth(XtDisplay(toplevel),
				     DefaultScreen(XtDisplay(toplevel))));

    iPolylineIcon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)),
			iPolyline_bits, iPolyline_width, iPolyline_height,
			fg, bg, DefaultDepth(XtDisplay(toplevel), 
				     DefaultScreen(XtDisplay(toplevel))));

    iPolygonIcon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), iPolygon_bits,
			iPolygon_width, iPolygon_height, fg, bg,
			DefaultDepth(XtDisplay(toplevel), 
				     DefaultScreen(XtDisplay(toplevel))));

    iTextIcon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), iText_bits,
			iText_width, iText_height, fg, bg,
			DefaultDepth(XtDisplay(toplevel), 
				     DefaultScreen(XtDisplay(toplevel))));

    iHexagonIcon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), iHexagon_bits,
			iHexagon_width, iHexagon_height, fg, bg,
			DefaultDepth(XtDisplay(toplevel), 
				     DefaultScreen(XtDisplay(toplevel))));

    iCircleIcon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), iCircle_bits,
			iCircle_width, iCircle_height, fg, bg,
			DefaultDepth(XtDisplay(toplevel), 
				     DefaultScreen(XtDisplay(toplevel))));

    iEllipseIcon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), iEllipse_bits,
			iEllipse_width, iEllipse_height, fg, bg,
			DefaultDepth(XtDisplay(toplevel), 
				     DefaultScreen(XtDisplay(toplevel))));

    iEllipseArcIcon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)),
			iEllipse_arc_bits,
			iEllipse_arc_width, iEllipse_arc_height, fg, bg,
			DefaultDepth(XtDisplay(toplevel), 
				     DefaultScreen(XtDisplay(toplevel))));

    iCircleArcIcon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)),
			iCircle_arc_bits,
			iCircle_arc_width, iCircle_arc_height, fg, bg,
			DefaultDepth(XtDisplay(toplevel), 
				     DefaultScreen(XtDisplay(toplevel))));

    iImageIcon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), iImage_bits,
			iImage_width, iImage_height, fg, bg, 
			DefaultDepth(XtDisplay(toplevel),
				     DefaultScreen(XtDisplay(toplevel))));

    iHandIcon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), iHand_bits,
			iHand_width, iHand_height, fg, bg, 
			DefaultDepth(XtDisplay(toplevel),
				     DefaultScreen(XtDisplay(toplevel))));

    iRectangleIcon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)),
			iRectangle_bits,
			iRectangle_width, iRectangle_height, fg, bg, 
			DefaultDepth(XtDisplay(toplevel),
				     DefaultScreen(XtDisplay(toplevel))));

    iSquareIcon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), iSquare_bits,
			iSquare_width, iSquare_height, fg, bg, 
			DefaultDepth(XtDisplay(toplevel),
				     DefaultScreen(XtDisplay(toplevel))));

    iCurveIcon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), iCurve_bits,
			iCurve_width, iCurve_height, fg, bg, 
			DefaultDepth(XtDisplay(toplevel),
				     DefaultScreen(XtDisplay(toplevel))));

    lineSolidIcon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), lineSolid_bits,
			lineSolid_width, lineSolid_height, fg, bg, 
			DefaultDepth(XtDisplay(toplevel),
				     DefaultScreen(XtDisplay(toplevel))));

    lineOnOffDashIcon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), 
			lineOnOffDash_bits,
			lineOnOffDash_width, lineOnOffDash_height, fg, bg, 
			DefaultDepth(XtDisplay(toplevel),
				     DefaultScreen(XtDisplay(toplevel))));

    lineDoubleDashIcon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), 
			lineDoubleDash_bits,
			lineDoubleDash_width, lineDoubleDash_height, fg, bg, 
			DefaultDepth(XtDisplay(toplevel),
				     DefaultScreen(XtDisplay(toplevel))));

    evenOddRuleIcon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), 
			evenOddRule_bits,
			evenOddRule_width, evenOddRule_height, fg, bg, 
			DefaultDepth(XtDisplay(toplevel),
				     DefaultScreen(XtDisplay(toplevel))));

    windingRuleIcon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), 
			windingRule_bits,
			windingRule_width, windingRule_height, fg, bg, 
			DefaultDepth(XtDisplay(toplevel),
				     DefaultScreen(XtDisplay(toplevel))));

    joinBevelIcon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), 
			joinBevel_bits,
			joinBevel_width, joinBevel_height, fg, bg, 
			DefaultDepth(XtDisplay(toplevel),
				     DefaultScreen(XtDisplay(toplevel))));
    joinMiterIcon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), 
			joinMiter_bits,
			joinMiter_width, joinMiter_height, fg, bg, 
			DefaultDepth(XtDisplay(toplevel),
				     DefaultScreen(XtDisplay(toplevel))));
    joinRoundIcon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), 
			joinRound_bits,
			joinRound_width, joinRound_height, fg, bg, 
			DefaultDepth(XtDisplay(toplevel),
				     DefaultScreen(XtDisplay(toplevel))));
    alignBottomIcon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), 
			alignBottom_bits,
			alignBottom_width, alignBottom_height, fg, bg, 
			DefaultDepth(XtDisplay(toplevel),
				     DefaultScreen(XtDisplay(toplevel))));
    alignTopIcon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), 
			alignTop_bits,
			alignTop_width, alignTop_height, fg, bg, 
			DefaultDepth(XtDisplay(toplevel),
				     DefaultScreen(XtDisplay(toplevel))));
    alignLeftIcon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), 
			alignLeft_bits,
			alignLeft_width, alignLeft_height, fg, bg, 
			DefaultDepth(XtDisplay(toplevel),
				     DefaultScreen(XtDisplay(toplevel))));
    alignRightIcon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), 
			alignRight_bits,
			alignRight_width, alignRight_height, fg, bg, 
			DefaultDepth(XtDisplay(toplevel),
				     DefaultScreen(XtDisplay(toplevel))));
    alignHorizontalIcon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), 
			alignHorizon_bits,
			alignHorizon_width, alignHorizon_height, fg, bg, 
			DefaultDepth(XtDisplay(toplevel),
				     DefaultScreen(XtDisplay(toplevel))));
    alignVerticalIcon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), 
			alignVertical_bits,
			alignVertical_width, alignVertical_height, fg, bg, 
			DefaultDepth(XtDisplay(toplevel),
				     DefaultScreen(XtDisplay(toplevel))));
    alignTopBotIcon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), 
			alignTopBot_bits,
			alignTopBot_width, alignTopBot_height, fg, bg, 
			DefaultDepth(XtDisplay(toplevel),
				     DefaultScreen(XtDisplay(toplevel))));
    alignBotTopIcon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), 
			alignBotTop_bits,
			alignBotTop_width, alignBotTop_height, fg, bg, 
			DefaultDepth(XtDisplay(toplevel),
				     DefaultScreen(XtDisplay(toplevel))));
    alignRightLeftIcon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), 
			alignRightLeft_bits,
			alignRightLeft_width, alignRightLeft_height, fg, bg, 
			DefaultDepth(XtDisplay(toplevel),
				     DefaultScreen(XtDisplay(toplevel))));
    alignLeftRightIcon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), 
			alignLeftRight_bits,
			alignLeftRight_width, alignLeftRight_height, fg, bg, 
			DefaultDepth(XtDisplay(toplevel),
				     DefaultScreen(XtDisplay(toplevel))));

    spaceXIcon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), 
			spaceX_bits,
			spaceX_width, spaceX_height, fg, bg, 
			DefaultDepth(XtDisplay(toplevel),
				     DefaultScreen(XtDisplay(toplevel))));
    spaceYIcon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), 
			spaceY_bits,
			spaceY_width, spaceY_height, fg, bg, 
			DefaultDepth(XtDisplay(toplevel),
				     DefaultScreen(XtDisplay(toplevel))));

    mirrorIIcon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), 
			mirrorI_bits,
			mirrorI_width, mirrorI_height, fg, bg, 
			DefaultDepth(XtDisplay(toplevel),
				     DefaultScreen(XtDisplay(toplevel))));
    mirrorJIcon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), 
			mirrorJ_bits,
			mirrorJ_width, mirrorJ_height, fg, bg, 
			DefaultDepth(XtDisplay(toplevel),
				     DefaultScreen(XtDisplay(toplevel))));
    mirrorIJIcon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), 
			mirrorIJ_bits,
			mirrorIJ_width, mirrorIJ_height, fg, bg, 
			DefaultDepth(XtDisplay(toplevel),
				     DefaultScreen(XtDisplay(toplevel))));

    lwIcon[0] = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), 
			lw1_bits, lw1_width, lw1_height, fg, bg, 
			DefaultDepth(XtDisplay(toplevel),
				     DefaultScreen(XtDisplay(toplevel))));
    lwIcon[1] = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), 
			lw2_bits, lw2_width, lw2_height, fg, bg, 
			DefaultDepth(XtDisplay(toplevel),
				     DefaultScreen(XtDisplay(toplevel))));
    lwIcon[2] = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), 
			lw3_bits, lw3_width, lw3_height, fg, bg, 
			DefaultDepth(XtDisplay(toplevel),
				     DefaultScreen(XtDisplay(toplevel))));
    lwIcon[3] = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), 
			lw4_bits, lw4_width, lw4_height, fg, bg, 
			DefaultDepth(XtDisplay(toplevel),
				     DefaultScreen(XtDisplay(toplevel))));
    lwIcon[4] = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), 
			lw5_bits, lw5_width, lw5_height, fg, bg, 
			DefaultDepth(XtDisplay(toplevel),
				     DefaultScreen(XtDisplay(toplevel))));
    lwIcon[5] = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), 
			lw6_bits, lw6_width, lw6_height, fg, bg, 
			DefaultDepth(XtDisplay(toplevel),
				     DefaultScreen(XtDisplay(toplevel))));
    lwIcon[6] = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), 
			lw7_bits, lw7_width, lw7_height, fg, bg, 
			DefaultDepth(XtDisplay(toplevel),
				     DefaultScreen(XtDisplay(toplevel))));
    lwIcon[7] = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), 
			lw8_bits, lw8_width, lw8_height, fg, bg, 
			DefaultDepth(XtDisplay(toplevel),
				     DefaultScreen(XtDisplay(toplevel))));
    lwIcon[8] = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), 
			lw9_bits, lw9_width, lw9_height, fg, bg, 
			DefaultDepth(XtDisplay(toplevel),
				     DefaultScreen(XtDisplay(toplevel))));
    lwIcon[9] = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), 
			lw10_bits, lw10_width, lw10_height, fg, bg, 
			DefaultDepth(XtDisplay(toplevel),
				     DefaultScreen(XtDisplay(toplevel))));

    rotate45Icon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), 
			rotate45_bits, rotate45_width, rotate45_height,
			fg, bg, DefaultDepth(XtDisplay(toplevel),
				DefaultScreen(XtDisplay(toplevel))));
    rotate90Icon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), 
			rotate90_bits, rotate90_width, rotate90_height,
			fg, bg, DefaultDepth(XtDisplay(toplevel),
				DefaultScreen(XtDisplay(toplevel))));
    rotate135Icon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), 
			rotate135_bits, rotate135_width, rotate135_height,
			fg, bg, DefaultDepth(XtDisplay(toplevel),
				DefaultScreen(XtDisplay(toplevel))));
    rotate180Icon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), 
			rotate180_bits, rotate180_width, rotate180_height,
			fg, bg, DefaultDepth(XtDisplay(toplevel),
				DefaultScreen(XtDisplay(toplevel))));
    rotate225Icon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), 
			rotate225_bits, rotate225_width, rotate225_height,
			fg, bg, DefaultDepth(XtDisplay(toplevel),
				DefaultScreen(XtDisplay(toplevel))));
    rotate270Icon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), 
			rotate270_bits, rotate270_width, rotate270_height,
			fg, bg, DefaultDepth(XtDisplay(toplevel),
				DefaultScreen(XtDisplay(toplevel))));
    rotate315Icon = XCreatePixmapFromBitmapData (XtDisplay(toplevel), 
			DefaultRootWindow(XtDisplay(toplevel)), 
			rotate315_bits, rotate315_width, rotate315_height,
			fg, bg, DefaultDepth(XtDisplay(toplevel),
				DefaultScreen(XtDisplay(toplevel))));
}
