#ifdef REV_INFO
static char rcsid[] = "$Id: popupmenu.cc,v 1.4 1992/12/10 16:10:10 jojic Exp $";
#endif
/*****************************************************************************
MODULE:  Go Editor
FILE:    popupmenu.cc
AUTHOR:  Aline Jacomet
DATE:    August 7th 1991

DESCRIPTION:
  Management of menus displayed when clicking on a Go.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include <Go/Go.h>
#include <Go/mGo.h>

#include "main.h"

#include <Xm/Form.h>
#include <Xm/RowColumn.h>
#include <Xm/CascadeBG.h>
#include <Xm/PushBG.h>
#include <Xm/PushB.h>
#include <Xm/ToggleBG.h>
#include <Xm/SeparatoG.h>
#include <Xm/ArrowBG.h>
#include <Xm/Scale.h>
#include <Xm/LabelG.h>



void* reg_menu;
void* poly_menu;
void* bezier_menu;
void* bcardinal_menu;
void* betatau_menu;
void* circle_menu;
void* arc_menu;
void* small_menu;
void* label_menu;
void* text_menu;
#ifdef GoLICENSE
void* scroll_menu;
void* mctext_menu;
#endif
void* image_menu;

Widget reg_label;
Widget poly_label;
Widget bezier_label;
Widget bcardinal_label;
Widget betatau_label;
Widget circle_label;
Widget arc_label;
Widget small_label;
Widget label_label;
Widget text_label;
#ifdef GoLICENSE
Widget scroll_label;
Widget mctext_label;
#endif
Widget image_label;

void* text_presentation[4];
void* label_presentation[4];
void* text_justification[3];
#ifdef GoLICENSE
void* scroll_presentation[4];
void* mctext_presentation[4];
void* mctext_justification[4];
void* mctext_language[7];
#endif

extern char* colorname[15];
extern XColor color[13];
extern int nb_color;

extern Display* dpy;
extern Colormap cmap;

extern Go* menu_go;

extern int selected;
extern int closed;
extern int pie;


static char* presentation_name[4] = {"No Presentation", "Lower Presentation", "Upper Presentation", "Title Presentation"};
static char* justification_name[3] = {"Left Justification", "Center Justification", "Right Justification"};
static char* name[3] = {"Underline Color", "Cursor Color", "Selection Color"};

#ifdef GoLICENSE
static char* mclanguage_name[7] = {"English", "French", "German", "Spanish", "Italian", "Portuguese", "No Language"};
static char* mcjustification_name[4] = {"Left Justification", "Center Justification", "Right Justification", "Left Right Justification"};
#endif

static void(* func[3])(void*, int);

static XmString select_label;
static XmString unselect_label;
static XmString closed_label;
static XmString opened_label;
static XmString pie_label;
static XmString chord_label;

static Widget poly_select_item;
static Widget poly_unselect_item[3];
static Widget bezier_select_item;
static Widget bezier_unselect_item[3];
static Widget bcardinal_select_item;
static Widget bcardinal_unselect_item[3];
static Widget betatau_select_item;
static Widget betatau_unselect_item[3];
static Widget arc_closed;
static Widget arc_pie;
static void ChangeRoundOffset(Widget, Widget parent);
static void ChangeTensionOffset(Widget, Widget parent);
static void ChangeBiasOffset(Widget, Widget parent);
static void ChangeSpaceLine(Widget, Widget parent);
static void ChangeUnderlineThickness(Widget, Widget parent);
static void ChangeUnderlinePosition(Widget, Widget parent);

#ifdef GoLICENSE
static void Scrolling(Widget, Widget parent);
static void ChangeSize(Widget, Widget parent);
static void ChangeInterColumn(Widget, Widget parent);
static void ChangeNbColumn(Widget, Widget parent);
#endif

static void UnactivateToggles(Widget, int, XmToggleButtonCallbackStruct*);

void InitializeObjectName(void*, const char*);

    

/*
 * EXTERN FUNCTIONS
 */

// in file goPopup.cc;
extern void GoSelectUnselectPoint(void*, int*);
extern void GoAddPreviousPoint();
extern void GoAddNextPoint();
extern void GoDeletePoint();
extern void GoAddEdge();
extern void GoDeleteEdge();
extern void GoChangeRadii();
extern void GoChangeAngle();
extern void GoChangePie(void*, int*);
extern void GoChangeClosed(void*, int*);
extern void GoChangeRoundOffset(float);
extern void GoChangeTensionOffset(float);
extern void GoChangeBiasOffset(float);
extern float GoInitializeRoundOffset();
extern float GoInitializeTensionOffset();
extern float GoInitializeBiasOffset();
extern void GoUnderlined(void*, int);
extern void GoChangeUnderlineThickness(float);
extern void GoChangeUnderlinePosition(float);
extern void GoChangePresentation(void*, int);
extern void GoChangeJustification(void*, int);
extern void GoChangeLanguage(void*, int);
extern void GoChangeSpaceLine(float);
extern float GoInitializeSpaceLine();
extern float GoInitializeUnderlineThickness();
extern float GoInitializeUnderlinePosition();
extern void GoChangeUnderlineColor(void*, int);
extern void GoChangeCursorColor(void*, int);
extern void GoChangeSelectionColor(void*, int);
extern void GoResizeRect();
extern void GoResizeRegPoly();

#ifdef GoLICENSE
extern void GoChangeInterColumn(float);
extern float GoInitializeInterColumn();
extern void GoInitializeScroll();
extern void GoScrollUp();
extern void GoScrollDown();
extern void GoScrollLeft();
extern void GoScrollRight();
extern void GoInitializeSize();
extern void GoInitializeNbColumn();
extern void GoWidthPlus();
extern void GoWidthMinus();
extern void GoHeightPlus();
extern void GoHeightMinus();
extern void GoNbColumnPlus();
extern void GoNbColumnMinus();
extern void GoResizeTextWidth();
extern void GoResizeTextHeight();
#endif

// in file namedattrib.cc;
extern void UnmanageWidget(Widget, Widget, caddr_t);



void CreatePolypointMenus(Widget parent)
{
    select_label = XmStringCreate("Select Points", XmSTRING_DEFAULT_CHARSET);
    unselect_label = XmStringCreate("Unselect Points",
				    XmSTRING_DEFAULT_CHARSET);
    closed_label = XmStringCreate("Close Arc", XmSTRING_DEFAULT_CHARSET);
    opened_label = XmStringCreate("Open Arc", XmSTRING_DEFAULT_CHARSET);
    pie_label = XmStringCreate("Pie Arc", XmSTRING_DEFAULT_CHARSET);
    chord_label = XmStringCreate("Chord Arc", XmSTRING_DEFAULT_CHARSET);
    
    /*
     * Polyline Menu
     */
    poly_menu = XmCreatePopupMenu(parent, "PolylineMenu", NULL, 0);

    poly_label = XmCreateLabelGadget((Widget)poly_menu, "ObjectName", NULL, 0);
    XtManageChild(poly_label);
    Widget w = XmCreateSeparatorGadget((Widget)poly_menu, "Separator",
				       NULL, 0);
    XtManageChild(w);
    poly_select_item = XmCreatePushButtonGadget((Widget) poly_menu,
						"Select/Unselect", NULL, 0);
    XtManageChild(poly_select_item);
    XtAddCallback(poly_select_item, XmNactivateCallback,
		  (XtCallbackProc)GoSelectUnselectPoint, (XtPointer)&selected);

    int i = 0;
    poly_unselect_item[i] = XmCreatePushButtonGadget((Widget) poly_menu,
						     "AddForward", NULL, 0);
    XtAddCallback(poly_unselect_item[i], XmNactivateCallback,
		  (XtCallbackProc)GoAddPreviousPoint, NULL);
    i++;
    poly_unselect_item[i] = XmCreatePushButtonGadget((Widget) poly_menu,
						     "AddBackward", NULL, 0);
    XtAddCallback(poly_unselect_item[i], XmNactivateCallback,
		  (XtCallbackProc)GoAddNextPoint, NULL);
    i++;
    poly_unselect_item[i] = XmCreatePushButtonGadget((Widget) poly_menu,
						     "Delete", NULL, 0);
    XtAddCallback(poly_unselect_item[i], XmNactivateCallback,
		  (XtCallbackProc)GoDeletePoint, NULL);
    i++;
    XtManageChildren(poly_unselect_item, i);

    w = XmCreatePushButtonGadget((Widget) poly_menu, "RoundOffset",
					NULL, 0);
    XtManageChild(w);
    XtAddCallback(w, XmNactivateCallback,
		  (XtCallbackProc)ChangeRoundOffset, (XtPointer)main_window);


    /*
     * Bezier Menu
     */
    bezier_menu = XmCreatePopupMenu(parent, "BezierMenu", NULL, 0);

    bezier_label = XmCreateLabelGadget((Widget)bezier_menu, "ObjectName",
				       NULL, 0);
    XtManageChild(bezier_label);
    w = XmCreateSeparatorGadget((Widget)bezier_menu, "Separator", NULL, 0);
    XtManageChild(w);

    bezier_select_item = XmCreatePushButtonGadget((Widget) bezier_menu,
						  "Select/Unselect", NULL, 0);
    XtManageChild(bezier_select_item);
    XtAddCallback(bezier_select_item, XmNactivateCallback,
		  (XtCallbackProc)GoSelectUnselectPoint, (XtPointer)&selected);

    i = 0;
    bezier_unselect_item[i] = XmCreatePushButtonGadget((Widget) bezier_menu,
						      "AddForward", NULL, 0);
    XtAddCallback(bezier_unselect_item[i], XmNactivateCallback,
		  (XtCallbackProc)GoAddPreviousPoint, NULL);
    i++;
    bezier_unselect_item[i] = XmCreatePushButtonGadget((Widget) bezier_menu,
						      "AddBackward", NULL, 0);
    XtAddCallback(bezier_unselect_item[i], XmNactivateCallback,
		  (XtCallbackProc)GoAddNextPoint, NULL);
    i++;
    bezier_unselect_item[i] = XmCreatePushButtonGadget((Widget) bezier_menu,
						      "Delete", NULL, 0);
    XtAddCallback(bezier_unselect_item[i], XmNactivateCallback,
		  (XtCallbackProc)GoDeletePoint, NULL);
    i++;
    XtManageChildren(bezier_unselect_item, i);



    /*
     * BCardinal Menu
     */
    bcardinal_menu = XmCreatePopupMenu(parent, "BCardinalMenu", NULL, 0);

    bcardinal_label = XmCreateLabelGadget((Widget)bcardinal_menu, "ObjectName",
					  NULL, 0);
    XtManageChild(bcardinal_label);
    w = XmCreateSeparatorGadget((Widget)bcardinal_menu, "Separator", NULL, 0);
    XtManageChild(w);

    bcardinal_select_item = XmCreatePushButtonGadget((Widget) bcardinal_menu,
						  "Select/Unselect", NULL, 0);
    XtManageChild(bcardinal_select_item);
    XtAddCallback(bcardinal_select_item, XmNactivateCallback,
		  (XtCallbackProc)GoSelectUnselectPoint, (XtPointer)&selected);

    i = 0;
    bcardinal_unselect_item[i] =
	XmCreatePushButtonGadget((Widget)bcardinal_menu, "AddForward",
				 NULL, 0);
    XtAddCallback(bcardinal_unselect_item[i], XmNactivateCallback,
		  (XtCallbackProc)GoAddPreviousPoint, NULL);
    i++;
    bcardinal_unselect_item[i] =
	XmCreatePushButtonGadget((Widget)bcardinal_menu, "AddBackward",
				 NULL, 0);
    XtAddCallback(bcardinal_unselect_item[i], XmNactivateCallback,
		  (XtCallbackProc)GoAddNextPoint, NULL);
    i++;
    bcardinal_unselect_item[i] =
	XmCreatePushButtonGadget((Widget)bcardinal_menu, "Delete", NULL, 0);
    XtAddCallback(bcardinal_unselect_item[i], XmNactivateCallback,
		  (XtCallbackProc)GoDeletePoint, NULL);
    i++;
    XtManageChildren(bcardinal_unselect_item, i);
    w = XmCreatePushButtonGadget((Widget)bcardinal_menu, "Tension", NULL, 0);
    XtAddCallback(w, XmNactivateCallback, (XtCallbackProc)ChangeTensionOffset,
		  (XtPointer)main_window);
    XtManageChild(w);



    /*
     * BetaTau Menu
     */
    betatau_menu = XmCreatePopupMenu(parent, "BetaTauMenu", NULL, 0);

    betatau_label = XmCreateLabelGadget((Widget)betatau_menu, "ObjectName",
					NULL, 0);
    XtManageChild(betatau_label);
    w = XmCreateSeparatorGadget((Widget)betatau_menu, "Separator", NULL, 0);
    XtManageChild(w);

    betatau_select_item = XmCreatePushButtonGadget((Widget) betatau_menu,
						  "Select/Unselect", NULL, 0);
    XtManageChild(betatau_select_item);
    XtAddCallback(betatau_select_item, XmNactivateCallback,
		  (XtCallbackProc)GoSelectUnselectPoint, (XtPointer)&selected);

    i = 0;
    betatau_unselect_item[i] =
	XmCreatePushButtonGadget((Widget)betatau_menu, "AddForward",
				 NULL, 0);
    XtAddCallback(betatau_unselect_item[i], XmNactivateCallback,
		  (XtCallbackProc)GoAddPreviousPoint, NULL);
    i++;
    betatau_unselect_item[i] =
	XmCreatePushButtonGadget((Widget)betatau_menu, "AddBackward",
				 NULL, 0);
    XtAddCallback(betatau_unselect_item[i], XmNactivateCallback,
		  (XtCallbackProc)GoAddNextPoint, NULL);
    i++;
    betatau_unselect_item[i] =
	XmCreatePushButtonGadget((Widget)betatau_menu, "Delete", NULL, 0);
    XtAddCallback(betatau_unselect_item[i], XmNactivateCallback,
		  (XtCallbackProc)GoDeletePoint, NULL);
    i++;
    XtManageChildren(betatau_unselect_item, i);
    w = XmCreatePushButtonGadget((Widget)betatau_menu, "Tension", NULL, 0);
    XtAddCallback(w, XmNactivateCallback, (XtCallbackProc)ChangeTensionOffset,
		  (XtPointer)main_window);
    XtManageChild(w);
    w = XmCreatePushButtonGadget((Widget)betatau_menu, "Bias", NULL, 0);
    XtAddCallback(w, XmNactivateCallback, (XtCallbackProc)ChangeBiasOffset,
		  (XtPointer)main_window);
    XtManageChild(w);



    /*
     * Regular Polygon Menu
     */
    Widget btn[3];
    reg_menu = XmCreatePopupMenu(parent, "RegPolygonMenu", NULL, 0);

    reg_label = XmCreateLabelGadget((Widget)reg_menu, "ObjectName", NULL, 0);
    XtManageChild(reg_label);
    w = XmCreateSeparatorGadget((Widget)reg_menu, "Separator", NULL, 0);
    XtManageChild(w);

    i = 0;
    btn[i] = XmCreatePushButtonGadget((Widget) reg_menu, "Add", NULL, 0);
    XtAddCallback(btn[i], XmNactivateCallback, (XtCallbackProc)GoAddEdge,
		  NULL);
    i++;
    btn[i] = XmCreatePushButtonGadget((Widget) reg_menu, "Delete", NULL, 0);
    XtAddCallback(btn[i], XmNactivateCallback, (XtCallbackProc)GoDeleteEdge,
		  NULL);
    i++;
    btn[i] = XmCreatePushButtonGadget((Widget) reg_menu, "Resize", NULL, 0);
    XtAddCallback(btn[i], XmNactivateCallback, (XtCallbackProc)GoResizeRegPoly,
		  NULL);
    i++;
    XtManageChildren(btn, i);

    w = XmCreatePushButtonGadget((Widget) reg_menu, "RoundOffset", NULL, 0);
    XtAddCallback(w, XmNactivateCallback,
		  (XtCallbackProc)ChangeRoundOffset, (XtPointer)main_window);
    XtManageChild(w);


    /*
     * Rectangle and Square Menu
     */
    small_menu = XmCreatePopupMenu(parent, "RectAndSquareMenu", NULL, 0);

    small_label = XmCreateLabelGadget((Widget)small_menu, "ObjectName",
				      NULL, 0);
    XtManageChild(small_label);
    w = XmCreateSeparatorGadget((Widget)small_menu, "Separator", NULL, 0);
    XtManageChild(w);

    w = XmCreatePushButtonGadget((Widget) small_menu, "Resize", NULL, 0);
    XtAddCallback(w, XmNactivateCallback, (XtCallbackProc)GoResizeRect, NULL);
    XtManageChild(w);

    w = XmCreatePushButtonGadget((Widget) small_menu, "RoundOffset", NULL, 0);
    XtAddCallback(w, XmNactivateCallback,
		  (XtCallbackProc)ChangeRoundOffset, (XtPointer)main_window);
    XtManageChild(w);


    /*
     * Circle Menu
     */
    circle_menu = XmCreatePopupMenu(parent, "CircleMenu", NULL, 0);

    circle_label = XmCreateLabelGadget((Widget)circle_menu, "ObjectName",
				       NULL, 0);
    XtManageChild(circle_label);
    w = XmCreateSeparatorGadget((Widget)circle_menu, "Separator", NULL, 0);
    XtManageChild(w);

    w = XmCreatePushButtonGadget((Widget) circle_menu, "Resize", NULL, 0);
    XtAddCallback(w, XmNactivateCallback,
		  (XtCallbackProc)GoChangeRadii, NULL);
    XtManageChild(w);



    /*
     * Arc Menu
     */
    Widget button[3];
    arc_menu = XmCreatePopupMenu(parent, "ArcMenu", NULL, 0);

    arc_label = XmCreateLabelGadget((Widget)arc_menu, "ObjectName", NULL, 0);
    XtManageChild(arc_label);
    w = XmCreateSeparatorGadget((Widget)arc_menu, "Separator", NULL, 0);
    XtManageChild(w);

    i = 0;
    button[i] = XmCreatePushButtonGadget((Widget)arc_menu, "Angle", NULL, 0);
    XtAddCallback(button[i], XmNactivateCallback,
		  (XtCallbackProc)GoChangeAngle, NULL);
    i++;
    button[i] = XmCreatePushButtonGadget((Widget)arc_menu, "Resize", NULL, 0);
    XtAddCallback(button[i], XmNactivateCallback,
		  (XtCallbackProc)GoChangeRadii, NULL);
    i++;
    XtManageChildren(button, i);
    arc_pie = XmCreatePushButtonGadget((Widget)arc_menu, "Pie/Chord", NULL, 0);
    XtAddCallback(arc_pie, XmNactivateCallback,
		  (XtCallbackProc)GoChangePie, (XtPointer)&pie);
    XtManageChild(arc_pie);
    arc_closed = XmCreatePushButtonGadget((Widget)arc_menu, "Open/Close",
					  NULL, 0);
    XtAddCallback(arc_closed, XmNactivateCallback,
		  (XtCallbackProc)GoChangeClosed, (XtPointer)&closed);
    XtManageChild(arc_closed);

    
    /*
     * Label Menu
     */
    Widget b[8];
    label_menu = XmCreatePopupMenu(parent, "LabelMenu", NULL, 0);

    label_label = XmCreateLabelGadget((Widget)label_menu, "ObjectName",
				      NULL, 0);
    XtManageChild(label_label);
    w = XmCreateSeparatorGadget((Widget)label_menu, "Separator", NULL, 0);
    XtManageChild(w);

    i = 0;
    b[i] = XmCreatePushButtonGadget((Widget)label_menu, "Underline",
					 NULL, 0);
    XtAddCallback(b[i], XmNactivateCallback,
		  (XtCallbackProc)GoUnderlined, (XtPointer)True);
    i++;
    b[i] = XmCreatePushButtonGadget((Widget)label_menu, "No Underline",
					 NULL, 0);
    XtAddCallback(b[i], XmNactivateCallback,
		  (XtCallbackProc)GoUnderlined, (XtPointer)False);
    i++;
    b[i] = XmCreatePushButtonGadget((Widget)label_menu,
					 "UnderlineThickness", NULL, 0);
    XtAddCallback(b[i], XmNactivateCallback,
		  (XtCallbackProc)ChangeUnderlineThickness,
		  (XtPointer)main_window);
    i++;
    b[i] = XmCreatePushButtonGadget((Widget)label_menu,
					 "UnderlinePosition", NULL, 0);
    XtAddCallback(b[i], XmNactivateCallback,
		  (XtCallbackProc)ChangeUnderlinePosition,
		  (XtPointer)main_window);
    i++;
    Widget pmp = XmCreatePulldownMenu((Widget)label_menu, "pull1", NULL, 0);
    Arg arg;
    XtSetArg(arg, XmNsubMenuId, pmp);
    b[i] = XmCreateCascadeButtonGadget((Widget)label_menu, "Presentation",
				       &arg, 1);
    for (int j = 0; j < 4; j++) {
	label_presentation[j] =
	    XmCreateToggleButtonGadget(pmp, presentation_name[j], NULL, 0);
	XtManageChild((Widget)label_presentation[j]);
	XtAddCallback((Widget)label_presentation[j], XmNvalueChangedCallback,
		      (XtCallbackProc)GoChangePresentation, (XtPointer)j);
	XtAddCallback((Widget)label_presentation[j], XmNvalueChangedCallback,
		      (XtCallbackProc)UnactivateToggles, (XtPointer)j);
    }
    i++;

    func[0] = GoChangeUnderlineColor;
    func[1] = GoChangeCursorColor;
    func[2] = GoChangeSelectionColor;
    for (int k = 0; k < 3; k++) {
	pmp = XmCreatePulldownMenu((Widget)label_menu, "pull1", NULL, 0);
	XtSetArg(arg, XmNsubMenuId, pmp);
	b[i] = XmCreateCascadeButtonGadget((Widget)label_menu, name[k],
					   &arg, 1);
	for (j = 0; j < nb_color; j++) {
	    /*
	     * Color's Allocation
	     */
	    if (XAllocColor(dpy, cmap, &color[j])) {
		XtSetArg(arg, XmNbackground, color[j].pixel);
		w = XmCreatePushButton(pmp, colorname[j], &arg, 1);
		XtManageChild(w);
		XtAddCallback(w, XmNactivateCallback,
			      (XtCallbackProc)func[k], (XtPointer)j);
	    }
	}
	i++;
    }
    
    XtManageChildren(b, i);

    
    /*
     * ScrolledText Menu
     */
#ifdef GoLICENSE    
    Widget bt[10];
    scroll_menu = XmCreatePopupMenu(parent, "ScrolledTextMenu", NULL, 0);

    scroll_label = XmCreateLabelGadget((Widget)scroll_menu, "ObjectName",
				       NULL, 0);
    XtManageChild(scroll_label);
    w = XmCreateSeparatorGadget((Widget)scroll_menu, "Separator", NULL, 0);
    XtManageChild(w);

    i = 0;
    bt[i] = XmCreatePushButtonGadget((Widget)scroll_menu, "Underline",
				     NULL, 0);
    XtAddCallback(bt[i], XmNactivateCallback,
		  (XtCallbackProc)GoUnderlined, (XtPointer)True);
    i++;
    bt[i] = XmCreatePushButtonGadget((Widget)scroll_menu, "No Underline",
					 NULL, 0);
    XtAddCallback(bt[i], XmNactivateCallback,
		  (XtCallbackProc)GoUnderlined, (XtPointer)False);
    i++;
    bt[i] = XmCreatePushButtonGadget((Widget)scroll_menu,
					 "UnderlineThickness", NULL, 0);
    XtAddCallback(bt[i], XmNactivateCallback,
		  (XtCallbackProc)ChangeUnderlineThickness,
		  (XtPointer)main_window);
    i++;
    bt[i] = XmCreatePushButtonGadget((Widget)scroll_menu,
					 "UnderlinePosition", NULL, 0);
    XtAddCallback(bt[i], XmNactivateCallback,
		  (XtCallbackProc)ChangeUnderlinePosition,
		  (XtPointer)main_window);
    i++;
    bt[i] = XmCreatePushButtonGadget((Widget)scroll_menu,
					 "Scrolling", NULL, 0);
    XtAddCallback(bt[i], XmNactivateCallback,
		  (XtCallbackProc)Scrolling, (XtPointer)main_window);
    i++;
    bt[i] = XmCreatePushButtonGadget((Widget)scroll_menu,
					 "Size", NULL, 0);
    XtAddCallback(bt[i], XmNactivateCallback,
		  (XtCallbackProc)ChangeSize, (XtPointer)main_window);
    i++;
    pmp = XmCreatePulldownMenu((Widget)scroll_menu, "pull1", NULL, 0);
    XtSetArg(arg, XmNsubMenuId, pmp);
    bt[i] = XmCreateCascadeButtonGadget((Widget)scroll_menu, "Presentation",
					&arg, 1);
    for (j = 0; j < 4; j++) {
	scroll_presentation[j] =
	    XmCreateToggleButtonGadget(pmp, presentation_name[j], NULL, 0);
	XtManageChild((Widget)scroll_presentation[j]);
	XtAddCallback((Widget)scroll_presentation[j], XmNvalueChangedCallback,
		      (XtCallbackProc)GoChangePresentation, (XtPointer)j);
	XtAddCallback((Widget)scroll_presentation[j], XmNvalueChangedCallback,
		      (XtCallbackProc)UnactivateToggles, (XtPointer)j);
    }
    i++;

    for (k = 0; k < 3; k++) {
	pmp = XmCreatePulldownMenu((Widget)scroll_menu, "pull1", NULL, 0);
	XtSetArg(arg, XmNsubMenuId, pmp);
	bt[i] = XmCreateCascadeButtonGadget((Widget)scroll_menu, name[k],
					    &arg, 1);
	for (j = 0; j < nb_color; j++) {
	    /*
	     * Color's Allocation
	     */
	    if (XAllocColor(dpy, cmap, &color[j])) {
		XtSetArg(arg, XmNbackground, color[j].pixel);
		w = XmCreatePushButton(pmp, colorname[j], &arg, 1);
		XtManageChild(w);
		XtAddCallback(w, XmNactivateCallback,
			      (XtCallbackProc)func[k], (XtPointer)j);
	    }
	}
	i++;
    }
    
    XtManageChildren(bt, i);
#endif    

    
    /*
     * Single Column Text and GoSimpleText Menu
     */
    Widget buttons[10];
    text_menu = XmCreatePopupMenu(parent, "TextMenu", NULL, 0);

    text_label = XmCreateLabelGadget((Widget)text_menu, "ObjectName", NULL, 0);
    XtManageChild(text_label);
    w = XmCreateSeparatorGadget((Widget)text_menu, "Separator", NULL, 0);
    XtManageChild(w);

    i = 0;
    buttons[i] = XmCreatePushButtonGadget((Widget)text_menu, "Underline",
					 NULL, 0);
    XtAddCallback(buttons[i], XmNactivateCallback,
		  (XtCallbackProc)GoUnderlined, (XtPointer)True);
    i++;
    buttons[i] = XmCreatePushButtonGadget((Widget)text_menu, "No Underline",
					 NULL, 0);
    XtAddCallback(buttons[i], XmNactivateCallback,
		  (XtCallbackProc)GoUnderlined, (XtPointer)False);
    i++;
    buttons[i] = XmCreatePushButtonGadget((Widget)text_menu,
					 "UnderlineThickness", NULL, 0);
    XtAddCallback(buttons[i], XmNactivateCallback,
		  (XtCallbackProc)ChangeUnderlineThickness,
		  (XtPointer)main_window);
    i++;
    buttons[i] = XmCreatePushButtonGadget((Widget)text_menu,
					 "UnderlinePosition", NULL, 0);
    XtAddCallback(buttons[i], XmNactivateCallback,
		  (XtCallbackProc)ChangeUnderlinePosition,
		  (XtPointer)main_window);
    i++;
    buttons[i] = XmCreatePushButtonGadget((Widget)text_menu, "Interline",
					  NULL, 0);
    XtAddCallback(buttons[i], XmNactivateCallback,
		  (XtCallbackProc)ChangeSpaceLine, (XtPointer)main_window);
    i++;
    pmp = XmCreatePulldownMenu((Widget)text_menu, "pull1", NULL, 0);
    XtSetArg(arg, XmNsubMenuId, pmp);
    buttons[i] = XmCreateCascadeButtonGadget((Widget)text_menu, "Presentation",
					     &arg, 1);
    for (j = 0; j < 4; j++) {
	text_presentation[j] =
	    XmCreateToggleButtonGadget(pmp, presentation_name[j], NULL, 0);
	XtManageChild((Widget)text_presentation[j]);
	XtAddCallback((Widget)text_presentation[j], XmNvalueChangedCallback,
		      (XtCallbackProc)GoChangePresentation, (XtPointer)j);
	XtAddCallback((Widget)text_presentation[j], XmNvalueChangedCallback,
		      (XtCallbackProc)UnactivateToggles, (XtPointer)j);
    }
    i++;
    pmp = XmCreatePulldownMenu((Widget)text_menu, "pull1", NULL, 0);
    XtSetArg(arg, XmNsubMenuId, pmp);
    buttons[i] = XmCreateCascadeButtonGadget((Widget)text_menu,
					     "Justification", &arg, 1);
    for (j = 0; j < 3; j++) {
	text_justification[j] =
	    XmCreateToggleButtonGadget(pmp, justification_name[j], NULL, 0);
	XtManageChild((Widget)text_justification[j]);
	XtAddCallback((Widget)text_justification[j], XmNvalueChangedCallback,
		      (XtCallbackProc)GoChangeJustification, (XtPointer)j);
	XtAddCallback((Widget)text_justification[j], XmNvalueChangedCallback,
		      (XtCallbackProc)UnactivateToggles, (XtPointer)j);
    }
    i++;
    for (k = 0; k < 3; k++) {
	pmp = XmCreatePulldownMenu((Widget)text_menu, "pull1", NULL, 0);
	XtSetArg(arg, XmNsubMenuId, pmp);
	buttons[i] = XmCreateCascadeButtonGadget((Widget)text_menu, name[k],
						 &arg, 1);
	for (j = 0; j < nb_color; j++) {
	    /*
	     * Color's Allocation
	     */
	    if (XAllocColor(dpy, cmap, &color[j])) {
		XtSetArg(arg, XmNbackground, color[j].pixel);
		w = XmCreatePushButton(pmp, colorname[j], &arg, 1);
		XtManageChild(w);
		XtAddCallback(w, XmNactivateCallback,
			      (XtCallbackProc)func[k], (XtPointer)j);
	    }
	}
	i++;
    }
    
    XtManageChildren(buttons, i);

    
    /*
     * MultiColumn Text Menu
     */
#ifdef GoLICENSE    
    Widget mcbuttons[15];
    mctext_menu = XmCreatePopupMenu(parent, "MCTextMenu", NULL, 0);

    mctext_label = XmCreateLabelGadget((Widget)mctext_menu, "ObjectName",
				       NULL, 0);
    XtManageChild(mctext_label);
    w = XmCreateSeparatorGadget((Widget)mctext_menu, "Separator", NULL, 0);
    XtManageChild(w);

    i = 0;
    mcbuttons[i] = XmCreatePushButtonGadget((Widget)mctext_menu, "Underline",
					 NULL, 0);
    XtAddCallback(mcbuttons[i], XmNactivateCallback,
		  (XtCallbackProc)GoUnderlined, (XtPointer)True);
    i++;
    mcbuttons[i] = XmCreatePushButtonGadget((Widget)mctext_menu, "No Underline",
					 NULL, 0);
    XtAddCallback(mcbuttons[i], XmNactivateCallback,
		  (XtCallbackProc)GoUnderlined, (XtPointer)False);
    i++;
    mcbuttons[i] = XmCreatePushButtonGadget((Widget)mctext_menu,
					 "UnderlineThickness", NULL, 0);
    XtAddCallback(mcbuttons[i], XmNactivateCallback,
		  (XtCallbackProc)ChangeUnderlineThickness,
		  (XtPointer)main_window);
    i++;
    mcbuttons[i] = XmCreatePushButtonGadget((Widget)mctext_menu,
					 "UnderlinePosition", NULL, 0);
    XtAddCallback(mcbuttons[i], XmNactivateCallback,
		  (XtCallbackProc)ChangeUnderlinePosition,
		  (XtPointer)main_window);
    i++;
    mcbuttons[i] = XmCreatePushButtonGadget((Widget)mctext_menu, "Interline",
					  NULL, 0);
    XtAddCallback(mcbuttons[i], XmNactivateCallback,
		  (XtCallbackProc)ChangeSpaceLine, (XtPointer)main_window);
    i++;
    mcbuttons[i] = XmCreatePushButtonGadget((Widget)mctext_menu, "Intercolumn",
					  NULL, 0);
    XtAddCallback(mcbuttons[i], XmNactivateCallback,
		  (XtCallbackProc)ChangeInterColumn, (XtPointer)main_window);
    i++;
    mcbuttons[i] = XmCreatePushButtonGadget((Widget)mctext_menu, "NbColumn",
					  NULL, 0);
    XtAddCallback(mcbuttons[i], XmNactivateCallback,
		  (XtCallbackProc)ChangeNbColumn, (XtPointer)main_window);
    i++;
    mcbuttons[i] = XmCreatePushButtonGadget((Widget)mctext_menu,
					    "Resize Width", NULL, 0);
    XtAddCallback(mcbuttons[i], XmNactivateCallback,
		  (XtCallbackProc)GoResizeTextWidth, NULL);
    i++;
    mcbuttons[i] = XmCreatePushButtonGadget((Widget)mctext_menu,
					    "Resize Height", NULL, 0);
    XtAddCallback(mcbuttons[i], XmNactivateCallback,
		  (XtCallbackProc)GoResizeTextHeight, NULL);
    i++;
    pmp = XmCreatePulldownMenu((Widget)mctext_menu, "pull1", NULL, 0);
    XtSetArg(arg, XmNsubMenuId, pmp);
    mcbuttons[i] = XmCreateCascadeButtonGadget((Widget)mctext_menu,
					     "Presentation", &arg, 1);
    for (j = 0; j < 4; j++) {
	mctext_presentation[j] =
	    XmCreateToggleButtonGadget(pmp, presentation_name[j], NULL, 0);
	XtManageChild((Widget)mctext_presentation[j]);
	XtAddCallback((Widget)mctext_presentation[j], XmNvalueChangedCallback,
		      (XtCallbackProc)GoChangePresentation, (XtPointer)j);
	XtAddCallback((Widget)mctext_presentation[j], XmNvalueChangedCallback,
		      (XtCallbackProc)UnactivateToggles, (XtPointer)j);
    }
    i++;
    pmp = XmCreatePulldownMenu((Widget)mctext_menu, "pull1", NULL, 0);
    XtSetArg(arg, XmNsubMenuId, pmp);
    mcbuttons[i] = XmCreateCascadeButtonGadget((Widget)mctext_menu,
					     "Justification", &arg, 1);
    for (j = 0; j < 4; j++) {
	mctext_justification[j] =
	    XmCreateToggleButtonGadget(pmp, mcjustification_name[j], NULL, 0);
	XtManageChild((Widget)mctext_justification[j]);
	XtAddCallback((Widget)mctext_justification[j], XmNvalueChangedCallback,
		      (XtCallbackProc)GoChangeJustification, (XtPointer)j);
	XtAddCallback((Widget)mctext_justification[j], XmNvalueChangedCallback,
		      (XtCallbackProc)UnactivateToggles, (XtPointer)j);
    }
    i++;
    pmp = XmCreatePulldownMenu((Widget)mctext_menu, "pull1", NULL, 0);
    XtSetArg(arg, XmNsubMenuId, pmp);
    mcbuttons[i] = XmCreateCascadeButtonGadget((Widget)mctext_menu,
					     "Language", &arg, 1);
    for (j = 0; j < 7; j++) {
	mctext_language[j] =
	    XmCreateToggleButtonGadget(pmp, mclanguage_name[j], NULL, 0);
	XtManageChild((Widget)mctext_language[j]);
	XtAddCallback((Widget)mctext_language[j], XmNvalueChangedCallback,
		      (XtCallbackProc)GoChangeLanguage, (XtPointer)j);
	XtAddCallback((Widget)mctext_language[j], XmNvalueChangedCallback,
		      (XtCallbackProc)UnactivateToggles, (XtPointer)j);
    }
    i++;
    for (k = 0; k < 3; k++) {
	pmp = XmCreatePulldownMenu((Widget)mctext_menu, "pull1", NULL, 0);
	XtSetArg(arg, XmNsubMenuId, pmp);
	mcbuttons[i] = XmCreateCascadeButtonGadget((Widget)mctext_menu,
						   name[k], &arg, 1);
	for (j = 0; j < nb_color; j++) {
	    /*
	     * Color's Allocation
	     */
	    if (XAllocColor(dpy, cmap, &color[j])) {
		XtSetArg(arg, XmNbackground, color[j].pixel);
		w = XmCreatePushButton(pmp, colorname[j], &arg, 1);
		XtManageChild(w);
		XtAddCallback(w, XmNactivateCallback,
			      (XtCallbackProc)func[k], (XtPointer)j);
	    }
	}
	i++;
    }
    
    XtManageChildren(mcbuttons, i);
#endif    

    /*
     * Image Menu
     */
    image_menu = XmCreatePopupMenu(parent, "ImageMenu", NULL, 0);

    image_label = XmCreateLabelGadget((Widget)image_menu, "ObjectName",
				      NULL, 0);
    XtManageChild(image_label);

    w = XmCreateSeparatorGadget((Widget)image_menu, "Separator", NULL, 0);
    XtManageChild(w);
    w = XmCreatePushButtonGadget((Widget)image_menu, "Resize", NULL, 0);
    XtAddCallback(w, XmNactivateCallback, (XtCallbackProc)GoResizeRect,
		  NULL);
    XtManageChild(w);
}


void PolySensitiveItems()
{
    Arg arg;
    XtSetArg(arg, XmNsensitive, True);
    for (int i = 0; i < 3; i++)
	XtSetValues(poly_unselect_item[i], &arg, 1);
    XtSetArg(arg, XmNlabelString, unselect_label);
    XtSetValues(poly_select_item, &arg, 1);
}
    

void PolyUnsensitiveItems()
{
    Arg arg;
    XtSetArg(arg, XmNsensitive, False);
    for (int i = 0; i < 3; i++)
	XtSetValues(poly_unselect_item[i], &arg, 1);
    XtSetArg(arg, XmNlabelString, select_label);
    XtSetValues(poly_select_item, &arg, 1);
}
    

void BezierSensitiveItems()
{
    Arg arg;
    XtSetArg(arg, XmNsensitive, True);
    for (int i = 0; i < 3; i++)
	XtSetValues(bezier_unselect_item[i], &arg, 1);
    XtSetArg(arg, XmNlabelString, unselect_label);
    XtSetValues(bezier_select_item, &arg, 1);
}
    

void BezierUnsensitiveItems()
{
    Arg arg;
    XtSetArg(arg, XmNsensitive, False);
    for (int i = 0; i < 3; i++)
	XtSetValues(bezier_unselect_item[i], &arg, 1);
    XtSetArg(arg, XmNlabelString, select_label);
    XtSetValues(bezier_select_item, &arg, 1);
}
    

void BCardinalSensitiveItems()
{
    Arg arg;
    XtSetArg(arg, XmNsensitive, True);
    for (int i = 0; i < 3; i++)
	XtSetValues(bcardinal_unselect_item[i], &arg, 1);
    XtSetArg(arg, XmNlabelString, unselect_label);
    XtSetValues(bcardinal_select_item, &arg, 1);
}
    

void BCardinalUnsensitiveItems()
{
    Arg arg;
    XtSetArg(arg, XmNsensitive, False);
    for (int i = 0; i < 3; i++)
	XtSetValues(bcardinal_unselect_item[i], &arg, 1);
    XtSetArg(arg, XmNlabelString, select_label);
    XtSetValues(bcardinal_select_item, &arg, 1);
}
    

void BetaTauSensitiveItems()
{
    Arg arg;
    XtSetArg(arg, XmNsensitive, True);
    for (int i = 0; i < 3; i++)
	XtSetValues(betatau_unselect_item[i], &arg, 1);
    XtSetArg(arg, XmNlabelString, unselect_label);
    XtSetValues(betatau_select_item, &arg, 1);
}
    

void BetaTauUnsensitiveItems()
{
    Arg arg;
    XtSetArg(arg, XmNsensitive, False);
    for (int i = 0; i < 3; i++)
	XtSetValues(betatau_unselect_item[i], &arg, 1);
    XtSetArg(arg, XmNlabelString, select_label);
    XtSetValues(betatau_select_item, &arg, 1);
}


void ArcPie(int pie)
{
    Arg arg;
    if (pie)
	XtSetArg(arg, XmNlabelString, pie_label);
    else
	XtSetArg(arg, XmNlabelString, chord_label);
    XtSetValues(arc_pie, &arg, 1);
}


void ArcClosed(int closed)
{
    Arg arg;
    if (closed)
	XtSetArg(arg, XmNlabelString, closed_label);
    else
	XtSetArg(arg, XmNlabelString, opened_label);
    XtSetValues(arc_closed, &arg, 1);
}


void MapMenu(void* menu, int x, int y)
{
    XButtonPressedEvent event;
    // x and are the position in the graphic widget;
    Position x_root, y_root;
    XtTranslateCoords((Widget)graphic, (Position)x, (Position)y,
		      &x_root, &y_root);
    event.x_root = x_root;
    event.y_root = y_root;
    event.x = x;
    event.y = y;
    XmMenuPosition((Widget) menu, &event);
    XtManageChild((Widget) menu);
}


static void ChangeRoundScale(Widget, caddr_t, XmScaleCallbackStruct* call)
{
    GoChangeRoundOffset((float)(call->value / 100.));
}

static void ChangeRoundOffset(Widget, Widget parent)
{
    static Widget round_scale = NULL;
    static Widget f = NULL;
    if (! f) {
	f = XmCreateFormDialog(parent, "RoundOffsetDialog", NULL, 0);
	Arg args[5];
	int n = 0;
	XtSetArg(args[n], XmNminimum, 0); n++;
	XtSetArg(args[n], XmNmaximum, 50); n++;
	XtSetArg(args[n], XmNshowValue, True); n++;
	XtSetArg(args[n], XmNdecimalPoints, 2); n++;
	round_scale = XmCreateScale(f, "RoundScale", args, n);
	XtManageChild(round_scale);
	XtAddCallback(round_scale, XmNdragCallback,
		      (XtCallbackProc) ChangeRoundScale, NULL);
	XtAddCallback(round_scale, XmNvalueChangedCallback,
		      (XtCallbackProc) ChangeRoundScale, NULL);
	Widget w = XmCreatePushButtonGadget(f, "Unmap", NULL, 0);
	XtManageChild(w);
    }
    int offset = (int)(GoInitializeRoundOffset() * 100);
    Arg arg;
    XtSetArg(arg, XmNvalue, offset);
    XtSetValues(round_scale, &arg, 1);
    XtManageChild(f);
}
	

static void ChangeTensionScale(Widget, caddr_t, XmScaleCallbackStruct* call)
{
    GoChangeTensionOffset((float)(call->value / 10.));
}

static void ChangeTensionOffset(Widget, Widget parent)
{
    static Widget tension_scale = NULL;
    static Widget f = NULL;
    if (! f) {
	f = XmCreateFormDialog(parent, "TensionOffsetDialog", NULL, 0);
	Arg args[5];
	int n = 0;
	XtSetArg(args[n], XmNminimum, 0); n++;
	XtSetArg(args[n], XmNmaximum, 100); n++;
	XtSetArg(args[n], XmNshowValue, True); n++;
	XtSetArg(args[n], XmNdecimalPoints, 1); n++;
	tension_scale = XmCreateScale(f, "TensionScale", args, n);
	XtManageChild(tension_scale);
	XtAddCallback(tension_scale, XmNdragCallback,
		      (XtCallbackProc) ChangeTensionScale, NULL);
	XtAddCallback(tension_scale, XmNvalueChangedCallback,
		      (XtCallbackProc) ChangeTensionScale, NULL);
	Widget w = XmCreatePushButtonGadget(f, "Unmap", NULL, 0);
	XtManageChild(w);
    }
    int offset = (int)(GoInitializeTensionOffset() * 10);
    Arg arg;
    XtSetArg(arg, XmNvalue, offset);
    XtSetValues(tension_scale, &arg, 1);
    XtManageChild(f);
}
	

static void ChangeBiasScale(Widget, caddr_t, XmScaleCallbackStruct* call)
{
    GoChangeBiasOffset((float)(call->value / 10.));
}

static void ChangeBiasOffset(Widget, Widget parent)
{
    static Widget bias_scale = NULL;
    static Widget f = NULL;
    if (! f) {
	f = XmCreateFormDialog(parent, "BiasOffsetDialog", NULL, 0);
	Arg args[5];
	int n = 0;
	XtSetArg(args[n], XmNminimum, 0); n++;
	XtSetArg(args[n], XmNmaximum, 100); n++;
	XtSetArg(args[n], XmNshowValue, True); n++;
	XtSetArg(args[n], XmNdecimalPoints, 1); n++;
	bias_scale = XmCreateScale(f, "BiasScale", args, n);
	XtManageChild(bias_scale);
	XtAddCallback(bias_scale, XmNdragCallback,
		      (XtCallbackProc) ChangeBiasScale, NULL);
	XtAddCallback(bias_scale, XmNvalueChangedCallback,
		      (XtCallbackProc) ChangeBiasScale, NULL);
	Widget w = XmCreatePushButtonGadget(f, "Unmap", NULL, 0);
	XtManageChild(w);
    }
    int offset = (int)(GoInitializeBiasOffset() * 10);
    Arg arg;
    XtSetArg(arg, XmNvalue, offset);
    XtSetValues(bias_scale, &arg, 1);
    XtManageChild(f);
}
	

static void ChangeThicknessScale(Widget, caddr_t, XmScaleCallbackStruct* call)
{
    GoChangeUnderlineThickness((float)(call->value / 10.));
}

static void ChangeUnderlineThickness(Widget, Widget parent)
{
    static Widget thickness_scale = NULL;
    static Widget f = NULL;
    if (! f) {
	f = XmCreateFormDialog(parent, "UnderlineThicknessDialog", NULL, 0);
	Arg args[5];
	int n = 0;
	XtSetArg(args[n], XmNminimum, 0); n++;
	XtSetArg(args[n], XmNmaximum, 50); n++;
	XtSetArg(args[n], XmNshowValue, True); n++;
	XtSetArg(args[n], XmNdecimalPoints, 1); n++;
	thickness_scale = XmCreateScale(f, "UnderlineThicknessScale", args, n);
	XtManageChild(thickness_scale);
	XtAddCallback(thickness_scale, XmNdragCallback,
		      (XtCallbackProc) ChangeThicknessScale, NULL);
	XtAddCallback(thickness_scale, XmNvalueChangedCallback,
		      (XtCallbackProc) ChangeThicknessScale, NULL);
	Widget w = XmCreatePushButtonGadget(f, "Unmap", NULL, 0);
	XtManageChild(w);
    }
    int offset = (int)(GoInitializeUnderlineThickness() * 10);
    Arg arg;
    XtSetArg(arg, XmNvalue, offset);
    XtSetValues(thickness_scale, &arg, 1);
    XtManageChild(f);
}
	

static void ChangePositionScale(Widget, caddr_t, XmScaleCallbackStruct* call)
{
    GoChangeUnderlinePosition((float)(call->value / 100.));
}

static void ChangeUnderlinePosition(Widget, Widget parent)
{
    static Widget position_scale = NULL;
    static Widget f = NULL;
    if (! f) {
	f = XmCreateFormDialog(parent, "UnderlinePositionDialog", NULL, 0);
	Arg args[5];
	int n = 0;
	XtSetArg(args[n], XmNminimum, 0); n++;
	XtSetArg(args[n], XmNmaximum, 99); n++;
	XtSetArg(args[n], XmNshowValue, True); n++;
	XtSetArg(args[n], XmNdecimalPoints, 2); n++;
	position_scale = XmCreateScale(f, "UnderlinePositionScale", args, n);
	XtManageChild(position_scale);
	XtAddCallback(position_scale, XmNdragCallback,
		      (XtCallbackProc) ChangePositionScale, NULL);
	XtAddCallback(position_scale, XmNvalueChangedCallback,
		      (XtCallbackProc) ChangePositionScale, NULL);
	Widget w = XmCreatePushButtonGadget(f, "Unmap", NULL, 0);
	XtManageChild(w);
    }
    int offset = (int)(GoInitializeUnderlinePosition() * 100);
    Arg arg;
    XtSetArg(arg, XmNvalue, offset);
    XtSetValues(position_scale, &arg, 1);
    XtManageChild(f);
}
	

static void ChangeSpaceScale(Widget, caddr_t, XmScaleCallbackStruct* call)
{
    GoChangeSpaceLine((float)(call->value / 10.));
}

static void ChangeSpaceLine(Widget, Widget parent)
{
    static Widget space_scale = NULL;
    static Widget f = NULL;
    if (! f) {
	f = XmCreateFormDialog(parent, "SpaceLineDialog", NULL, 0);
	Arg args[5];
	int n = 0;
	XtSetArg(args[n], XmNminimum, 0); n++;
	XtSetArg(args[n], XmNmaximum, 200); n++;
	XtSetArg(args[n], XmNshowValue, True); n++;
	XtSetArg(args[n], XmNdecimalPoints, 1); n++;
	space_scale = XmCreateScale(f, "SpaceLineScale", args, n);
	XtManageChild(space_scale);
	XtAddCallback(space_scale, XmNdragCallback,
		      (XtCallbackProc) ChangeSpaceScale, NULL);
	XtAddCallback(space_scale, XmNvalueChangedCallback,
		      (XtCallbackProc)ChangeSpaceScale, NULL);
	Widget w = XmCreatePushButtonGadget(f, "Unmap", NULL, 0);
	XtManageChild(w);
    }
    int offset = (int)(GoInitializeSpaceLine() * 10);
    Arg arg;
    XtSetArg(arg, XmNvalue, offset);
    XtSetValues(space_scale, &arg, 1);
    XtManageChild(f);
}
	

#ifdef GoLICENSE
static void ChangeInterColumnScale(Widget, caddr_t,
				   XmScaleCallbackStruct* call)
{
    GoChangeInterColumn((float)(call->value / 10.));
}

static void ChangeInterColumn(Widget, Widget parent)
{
    static Widget inter_scale = NULL;
    static Widget f = NULL;
    if (! f) {
	f = XmCreateFormDialog(parent, "InterColumnDialog", NULL, 0);
	Arg args[5];
	int n = 0;
	XtSetArg(args[n], XmNminimum, 0); n++;
	XtSetArg(args[n], XmNmaximum, 200); n++;
	XtSetArg(args[n], XmNshowValue, True); n++;
	XtSetArg(args[n], XmNdecimalPoints, 1); n++;
	inter_scale = XmCreateScale(f, "InterColumnScale", args, n);
	XtManageChild(inter_scale);
	XtAddCallback(inter_scale, XmNdragCallback,
		      (XtCallbackProc) ChangeInterColumnScale, NULL);
	XtAddCallback(inter_scale, XmNvalueChangedCallback,
		      (XtCallbackProc)ChangeInterColumnScale, NULL);
	Widget w = XmCreatePushButtonGadget(f, "Unmap", NULL, 0);
	XtManageChild(w);
    }
    int offset = (int)(GoInitializeInterColumn() * 10);
    Arg arg;
    XtSetArg(arg, XmNvalue, offset);
    XtSetValues(inter_scale, &arg, 1);
    XtManageChild(f);
}
#endif	


static void UnactivateToggles(Widget w, int ind,
			      XmToggleButtonCallbackStruct* data)
{
    if (data->set == False)
	XmToggleButtonGadgetSetState(w, True, False);
    
    if (w == (Widget)text_presentation[ind]) {
	for (int i = 0; i < 4; i++)
	    if (i != ind)
		XmToggleButtonGadgetSetState((Widget)text_presentation[i],
					     False, False);
    } else
    if (w == text_justification[ind]) {
	for (int i = 0; i < 3; i++)
	    if (i != ind)
		XmToggleButtonGadgetSetState((Widget)text_justification[i],
					     False, False);
    } else
#ifdef GoLICENSE	
    if (w == (Widget)mctext_presentation[ind]) {
	for (int i = 0; i < 4; i++)
	    if (i != ind)
		XmToggleButtonGadgetSetState((Widget)mctext_presentation[i],
					     False, False);
    } else
    if (w == mctext_justification[ind]) {
	for (int i = 0; i < 4; i++)
	    if (i != ind)
		XmToggleButtonGadgetSetState((Widget)mctext_justification[i],
					     False, False);
    } else
    if (w == mctext_language[ind]) {
	for (int i = 0; i < 7; i++)
	    if (i != ind)
		XmToggleButtonGadgetSetState((Widget)mctext_language[i],
					     False, False);
    } else
#endif	
    if (w == label_presentation[ind]) {
	for (int i = 0; i < 4; i++)
	    if (i != ind)
		XmToggleButtonGadgetSetState((Widget) label_presentation[i],
					     False, False);
    }
#ifdef GoLICENSE
    else
    if (w == scroll_presentation[ind]) {
	for (int i = 0; i < 4; i++)
	    if (i != ind)
		XmToggleButtonGadgetSetState((Widget) scroll_presentation[i],
					     False, False);
    }
#endif    
}


void ActivateToggle(void* w)
{
    XmToggleButtonGadgetSetState((Widget)w, True, True);
}

#ifdef GoLICENSE
static void Scrolling(Widget, Widget parent)
{
    static Widget f = NULL;
    if (! f) {
	f = XmCreateFormDialog(parent, "ScrollingDialog", NULL, 0);
	Arg args[5];
	int n = 0;
	XtSetArg(args[n], XmNarrowDirection, XmARROW_LEFT); n++;
	Widget w = XmCreateArrowButtonGadget(f, "LeftArrow", args, n);
	XtManageChild(w);
	XtAddCallback(w, XmNactivateCallback,
		      (XtCallbackProc)GoScrollLeft, NULL);
	n = 0;
	XtSetArg(args[n], XmNarrowDirection, XmARROW_UP); n++;
	w = XmCreateArrowButtonGadget(f, "UpArrow", args, n);
	XtManageChild(w);
	XtAddCallback(w, XmNactivateCallback,
		      (XtCallbackProc)GoScrollUp, NULL);
	n = 0;
	XtSetArg(args[n], XmNarrowDirection, XmARROW_RIGHT); n++;
	w = XmCreateArrowButtonGadget(f, "RightArrow", args, n);
	XtManageChild(w);
	XtAddCallback(w, XmNactivateCallback,
		      (XtCallbackProc)GoScrollRight, NULL);
	n = 0;
	XtSetArg(args[n], XmNarrowDirection, XmARROW_DOWN); n++;
	w = XmCreateArrowButtonGadget(f, "DownArrow", args, n);
	XtManageChild(w);
	XtAddCallback(w, XmNactivateCallback,
		      (XtCallbackProc)GoScrollDown, NULL);
	w = XmCreatePushButtonGadget(f, "Unmap", NULL, 0);
	XtManageChild(w);
    }
    GoInitializeScroll();
    XtManageChild(f);
}


static void ChangeSize(Widget, Widget parent)
{
    static Widget f = NULL;
    if (! f) {
	Arg arg;
	XtSetArg(arg, XmNautoUnmanage, False);
	f = XmCreateFormDialog(parent, "SizeDialog", &arg, 1);
	Widget w = XmCreateLabelGadget(f, "Width", NULL, 0);
	XtManageChild(w);
	w = XmCreatePushButtonGadget(f, "Width+", NULL, 0);
	XtAddCallback(w, XmNactivateCallback, (XtCallbackProc)GoWidthPlus,
		      NULL);
	XtManageChild(w);
	w = XmCreatePushButtonGadget(f, "Width-", NULL, 0);
	XtAddCallback(w, XmNactivateCallback, (XtCallbackProc)GoWidthMinus,
		      NULL);
	XtManageChild(w);
	w = XmCreateLabelGadget(f, "Height", NULL, 0);
	XtManageChild(w);
	w = XmCreatePushButtonGadget(f, "Height+", NULL, 0);
	XtAddCallback(w, XmNactivateCallback, (XtCallbackProc)GoHeightPlus,
		      NULL);
	XtManageChild(w);
	w = XmCreatePushButtonGadget(f, "Height-", NULL, 0);
	XtAddCallback(w, XmNactivateCallback, (XtCallbackProc)GoHeightMinus,
		      NULL);
	XtManageChild(w);
	w = XmCreatePushButtonGadget(f, "Unmap", NULL, 0);
	XtAddCallback(w, XmNactivateCallback, (XtCallbackProc)UnmanageWidget,
		      (XtPointer)f);
	XtManageChild(w);
    }
    GoInitializeSize();
    XtManageChild(f);
}


static void ChangeNbColumn(Widget, Widget parent)
{
    static Widget f = NULL;
    if (! f) {
	Arg arg;
	XtSetArg(arg, XmNautoUnmanage, False);
	f = XmCreateFormDialog(parent, "NbColumnDialog", &arg, 1);
	Widget w = XmCreateLabelGadget(f, "NbColumnLabel", NULL, 0);
	XtManageChild(w);
	w = XmCreatePushButtonGadget(f, "NbColumn+", NULL, 0);
	XtAddCallback(w, XmNactivateCallback, (XtCallbackProc)GoNbColumnPlus,
		      NULL);
	XtManageChild(w);
	w = XmCreatePushButtonGadget(f, "NbColumn-", NULL, 0);
	XtAddCallback(w, XmNactivateCallback, (XtCallbackProc)GoNbColumnMinus,
		      NULL);
	XtManageChild(w);
	w = XmCreatePushButtonGadget(f, "Unmap", NULL, 0);
	XtAddCallback(w, XmNactivateCallback, (XtCallbackProc)UnmanageWidget,
		      (XtPointer)f);
	XtManageChild(w);
    }
    GoInitializeNbColumn();
    XtManageChild(f);
}
#endif

void InitializeObjectName(void* menu, const char* name)
{
    XmString str = XmStringCreate((char*)name, XmSTRING_DEFAULT_CHARSET);
    Arg arg;
    XtSetArg(arg, XmNlabelString, str);
    Widget label = NULL;
    if (menu == reg_menu)
	label = reg_label;
    else
    if (menu == poly_menu)
	label = poly_label;
    else
    if (menu == bezier_menu)
	label = bezier_label;
    else
    if (menu == bcardinal_menu)
	label = bcardinal_label;
    else
    if (menu == betatau_menu)
	label = betatau_label;
    else
    if (menu == circle_menu)
	label = circle_label;
    else
    if (menu == arc_menu)
	label = arc_label;
    else
    if (menu == small_menu)
	label = small_label;
    else
    if (menu == label_menu)
	label = label_label;
    else
    if (menu == text_menu)
	label = text_label;
    else
#ifdef GoLICENSE
    if (menu == scroll_menu)
	label = scroll_label;
    else
    if (menu == mctext_menu)
	label = mctext_label;
    else
#endif	
    if (menu == image_menu)
	label = image_label;
    if (label) {
	XtSetValues(label, &arg, 1);
	XmStringFree(str);
    }
}
