#ifdef REV_INFO
static char rcsid[] = "$Id: selectbox.cc,v 1.1.1.1 1992/03/18 09:30:07 uep Exp $";
#endif
/*****************************************************************************
MODULE:  Go Editor
FILE:    selectionbox.cc
AUTHOR:  Aline Jacomet
DATE:    May 24th 1991

DESCRIPTION:
    SelectionBox Widget Management

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include <Go/Tools/StringAssoc.h>
#include <Go/Tools/Alloc.h>

#include <Xm/SelectioB.h>
#include <Xm/TextF.h>

#include "main.h"



extern Widget penText;
extern Widget brushText;
extern Widget textBrushText;
extern Widget penTextEdit;
extern Widget brushTextEdit;
extern Widget textBrushTextEdit;

extern GStringAssoc NamedPen;
extern GStringAssoc NamedBrush;
extern GStringAssoc NamedText;

static int WhichObject;


/*
 * EXTERN FUNCTIONS
 */

// in file goAttrib.cc;
extern void FromShPenToNamedPen(char*);
extern void FromShPenToEditPen(char*);
extern void FromShBrushToNamedBrush(char*);
extern void FromShBrushToEditBrush(char*);
extern void FromShTextToNamedText(char*);
extern void FromShTextToEditText(char*);
extern void ModifyGoNamedPenSheet();
extern void ModifyGoNamedBrushSheet();
extern void ModifyGoNamedTextSheet();
extern void ModifyGoEditPenSheet();
extern void ModifyGoEditBrushSheet();
extern void ModifyGoEditTextSheet();

// in file message.cc;
extern void NotInList(void*, void*, caddr_t);



/***
 *  Static Variables
 ***/
static Widget selectionDialog;


static void Apply(Widget, caddr_t, XmSelectionBoxCallbackStruct* cb)
{
    char* name;
    XmStringGetLtoR(cb->value, XmSTRING_DEFAULT_CHARSET, &name);
    switch (WhichObject) {
      case PEN:
	if (WhichAttributes == NAMED) {
	    XmTextFieldSetString(penText, name);
	    FromShPenToNamedPen(name);
	    ModifyGoNamedPenSheet();
	} else {
	    XmTextFieldSetString(penTextEdit, name);
	    FromShPenToEditPen(name);
	    ModifyGoEditPenSheet();
	}
	break;
      case BRUSH:
	if (WhichAttributes == NAMED)  {
	    XmTextFieldSetString(brushText, name);
	    FromShBrushToNamedBrush(name);
	    ModifyGoNamedBrushSheet();
	} else {
	    XmTextFieldSetString(brushTextEdit, name);
	    FromShBrushToEditBrush(name);
	    ModifyGoEditBrushSheet();
	}
	break;
      case TEXTBRUSH:
	if (WhichAttributes == NAMED) {
	    XmTextFieldSetString(textBrushText, name);
	    FromShTextToNamedText(name);
	    ModifyGoNamedTextSheet();
	} else {
	    XmTextFieldSetString(textBrushTextEdit, name);
	    FromShTextToEditText(name);
	    ModifyGoEditTextSheet();
	}
	break;
    }
    
    GFREE(name);
}


static void CreateSelectionDialog()
{
    Arg args[2];

    int n = 0;
    XtSetArg(args[0], XmNmustMatch, True); n++;
    XtSetArg(args[n], XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL); n++;
    selectionDialog = XmCreateSelectionDialog((Widget)main_window,
					      "SelectionBox", args, n);
    XtAddCallback(selectionDialog, XmNokCallback,
		  (XtCallbackProc)Apply, NULL);
    XtAddCallback(selectionDialog, XmNnoMatchCallback,
		  (XtCallbackProc)NotInList, (XtPointer)main_window);
    Widget wid = XmSelectionBoxGetChild(selectionDialog,
					XmDIALOG_APPLY_BUTTON);
    XtUnmanageChild(wid);
    wid = XmSelectionBoxGetChild(selectionDialog, XmDIALOG_HELP_BUTTON);
    XtUnmanageChild(wid);
}



void SelectNamedPen(void*, int wa, caddr_t)
{
    Arg args[2];
    Boolean create = (selectionDialog == NULL);

    WhichAttributes = wa;
    WhichObject = PEN;

    if (create)
	CreateSelectionDialog();

    int nb;
    char** l = NamedPen.getKeys(nb);
    XmString* list_name = (XmString*) GMALLOC(nb * sizeof(XmString));
    for (int i = 0; i < nb; i++)
	list_name[i] = XmStringCreate(l[i], XmSTRING_DEFAULT_CHARSET);
    XtSetArg(args[0], XmNlistItems, (XmStringTable)list_name);
    XtSetArg(args[1], XmNlistItemCount, nb);
    XtSetValues(selectionDialog, args, 2);
    GFREE(l);
    GFREE(list_name);
    
    XtManageChild(selectionDialog);
}


void SelectNamedBrush(void*, int wa, caddr_t)
{
    Arg args[2];
    Boolean create = (selectionDialog == NULL);

    WhichAttributes = wa;
    WhichObject = BRUSH;
    
    if (create)
	CreateSelectionDialog();

    int nb;
    char** l = NamedBrush.getKeys(nb);
    XmString* list_name = (XmString*) GMALLOC(nb * sizeof(XmString));
    for (int i = 0; i < nb; i++)
	list_name[i] = XmStringCreate(l[i], XmSTRING_DEFAULT_CHARSET);
    XtSetArg(args[0], XmNlistItems, (XmStringTable)list_name);
    XtSetArg(args[1], XmNlistItemCount, nb);
    XtSetValues(selectionDialog, args, 2);
    GFREE(l);
    GFREE(list_name);
    
    XtManageChild(selectionDialog);
}


void SelectNamedText(void*, int wa, caddr_t)
{
    Arg args[2];
    Boolean create = (selectionDialog == NULL);

    WhichAttributes = wa;
    WhichObject = TEXTBRUSH;
    
    if (create)
	CreateSelectionDialog();

    int nb;
    char** l = NamedText.getKeys(nb);
    XmString* list_name = (XmString*) GMALLOC(nb * sizeof(XmString));
    for (int i = 0; i < nb; i++)
	list_name[i] = XmStringCreate(l[i], XmSTRING_DEFAULT_CHARSET);
    XtSetArg(args[0], XmNlistItems, (XmStringTable)list_name);
    XtSetArg(args[1], XmNlistItemCount, nb);
    XtSetValues(selectionDialog, args, 2);
    GFREE(l);
    GFREE(list_name);
    
    XtManageChild(selectionDialog);
}
