#ifdef REV_INFO
static char rcsid[] = "$Id: textmenu.cc,v 1.3 1992/12/10 16:24:55 jojic Exp $";
#endif
/*****************************************************************************
MODULE:  Go Editor
FILE:    textmenu.cc
AUTHOR:  Patricia Langle and Aline Jacomet
DATE:    May 14th 1991

DESCRIPTION:
    menu for texts.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include "main.h"

#include <Xm/PushBG.h>
#include <Xm/RowColumn.h>



/*
 * EXTERN FUNCTIONS
 */

// in file goCreation.cc;
extern void CreateLabel(void*, void*, caddr_t);
extern void* CreateSimpleText(void*, void*, caddr_t);
extern void* CreateText(void*, void*, caddr_t);
#ifdef GoLICENSE
extern void* CreateMCText(void*, void*, caddr_t);
extern void* CreateScrolledText(void*, void*, caddr_t);
#endif

// in file help.cc;
extern void MapHelpFile(void*, char*, void*);

// in file message.cc;
#ifndef GoLICENSE
extern void NoLicense(void*, void*, caddr_t);
#endif



void InitTextEntries(Widget parent)
{
    Arg arg;
    Widget button;
    XtSetArg(arg, XmNtraversalOn, False);
    button = XmCreatePushButtonGadget(parent, "GoLabel", &arg, 1);
    XtAddCallback (button, XmNactivateCallback,
		   (XtCallbackProc)CreateLabel, NULL);
    XtManageChild(button);

    button = XmCreatePushButtonGadget(parent, "GoSimpleText", &arg, 1);
    XtAddCallback (button, XmNactivateCallback,
		   (XtCallbackProc)CreateSimpleText, NULL);
    XtManageChild(button);

    button = XmCreatePushButtonGadget(parent, "Single Column GoText", &arg, 1);
    XtAddCallback (button, XmNactivateCallback,
		   (XtCallbackProc)CreateText, NULL);
    XtManageChild(button);

    button = XmCreatePushButtonGadget(parent, "Multi Column GoText", &arg, 1);
#ifdef GoLICENSE    
    XtAddCallback (button, XmNactivateCallback, (XtCallbackProc)CreateMCText,
		   NULL);
#else    
    XtAddCallback (button, XmNactivateCallback, (XtCallbackProc)NoLicense,
		   (XtPointer)main_window);
#endif
    XtManageChild(button);

    button = XmCreatePushButtonGadget(parent, "GoScrolledText", &arg, 1);
#ifdef GoLICENSE    
    XtAddCallback (button, XmNactivateCallback,
		   (XtCallbackProc)CreateScrolledText, NULL);
#else    
    XtAddCallback (button, XmNactivateCallback, (XtCallbackProc)NoLicense,
		   (XtPointer)main_window);
#endif
    XtManageChild(button);
}

void PostTextMenu(Widget w, Widget text_popup, XmAnyCallbackStruct*)
{
    Arg args[10];
    Position root_x,root_y;
    XtTranslateCoords(w, 0, 0, &root_x, &root_y);
    int n = 0;
    XtSetArg(args[n], XmNx, root_x);  n++;
    XtSetArg(args[n], XmNy, root_y);  n++;
    XtSetValues(text_popup, args, n);
    XtManageChild (text_popup);
}


Widget CreateTextButton(Widget parent)
{
    Arg args[4];
    Widget button;
    XtSetArg(args[0], XmNtraversalOn, False);
    XtSetArg(args[1], XmNwhichButton, 1);
    Widget text_popup = XmCreatePopupMenu(parent, "TextPopup", args, 2);
    InitTextEntries(text_popup);
    int n = 0;
    XtSetArg(args[n], XmNlabelPixmap, textIcon); n++;
    XtSetArg(args[n], XmNtraversalOn, False); n++;
    XtSetArg(args[n], XmNlabelType, XmPIXMAP); n++;
    XtSetArg(args[n], XmNlabelInsensitivePixmap, iTextIcon); n++;
    button =  XmCreatePushButtonGadget(parent, "Texts", args, n);
    XtAddCallback (button, XmNarmCallback,
		   (XtCallbackProc)PostTextMenu, (XtPointer)text_popup);
    XtAddCallback (button, XmNhelpCallback,
		   (XtCallbackProc)MapHelpFile, TEXT_CR_FILE);
    return button;
}
