#ifdef REV_INFO
static char rcsid[] = "$Id: textsheet.cc,v 1.2 1992/12/10 16:24:57 jojic Exp $";
#endif
/*****************************************************************************
MODULE:  Go Editor
FILE:    textsheet.cc
AUTHOR:  Aline Jacomet
DATE:    May 22th 1991

DESCRIPTION:
    

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include <Go/Tools/StringAssoc.h>
#include <Go/Tools/Alloc.h>

#include <Xm/Form.h>
#include <Xm/PushBG.h>
#include <Xm/LabelG.h>
#include <Xm/SeparatoG.h>
#include <Xm/TextF.h>
#include <Xm/Text.h>

#include "classes.h"
#include "main.h"



/***
 * widget function
 ***/
static Widget CreateTextDialog(Widget);

void MapTextBrushAttrib(void*, void*, caddr_t);

static void ModifyTextSheet();
static void ReinitTextSheet(Widget, Boolean, caddr_t);
static void Verify(Widget, caddr_t, caddr_t);
static void NamedChooseGo(Widget, caddr_t, caddr_t);
void VerifyTextBrush(Widget, caddr_t, caddr_t);


/***
 *  Static Variables
 ***/
static Widget textDialog;
static Widget name;
static Widget color_label;
static Widget color_list;
static Widget font_label;
static Widget font_list;



/***
 *  Global output attributes
 ***/
TextAttributes named_text;

Widget text_color = NULL;
Widget text_font = NULL;

Boolean textBrushMapped = False;

extern Widget textBrushText;

extern GStringAssoc NamedText;
extern AllAttributes all_attrib;



/*
 * EXTERN FUNCTIONS
 */

// in file namedattrib.cc;
extern void SensitiveButtons(Widget, Boolean*, caddr_t);
extern void UnmanageWidget(Widget, Widget, caddr_t);
extern void ModifyNamedColor(Widget, int, caddr_t);

// in file colorsel.cc;
extern void MapColorList(Widget, Widget, caddr_t);

// in file fontsel.cc;
extern void MapFontList(Widget, int, caddr_t);

// in file goAttrib.cc;
extern void ModifyGoNamedTextSheet();
extern void CreateGoNamedText();
extern void ModifyGoNamedText();
extern int ChooseGo(int, void(*)(int, class Go*), int);

// in file message.cc;
extern void ObjectName(void*);
extern void AcknowledgeTextBrushName(void*);

// in file help.cc;
extern void MapHelpFile(void*, char*, void*);



/**
FUNCTION
    MapTextBrushAttrib()
DESCRIPTION
    Maps and creates, if necessary, the text attributes editor.
**/
void MapTextBrushAttrib(void*, void* parent, caddr_t)
{
    Boolean create = (textDialog == NULL);

    if (!create && XtIsManaged(textDialog))
	return;
    if (create) {
	XDefineCursor(XtDisplay((Widget)graphic), XtWindow((Widget)graphic),
		      clock_cursor);
	XDefineCursor(XtDisplay((Widget)main_window),
		      XtWindow((Widget)main_window), clock_cursor);
	XFlush(XtDisplay((Widget)graphic));
	textDialog = CreateTextDialog((Widget)parent);
    }

    ReinitTextSheet(NULL, TRUE, NULL);
    XtManageChild(textDialog);
    
    if (create) {
	XFlush(XtDisplay((Widget)graphic));
	XUndefineCursor(XtDisplay((Widget)graphic), XtWindow((Widget)graphic));
	XUndefineCursor(XtDisplay((Widget)main_window),
			XtWindow((Widget)main_window));
    }
    textBrushMapped = True;
}


/**
FUNCTION
    CreateTextDialog()
DESCRIPTION
    Creates the text attributes editor.
**/
static Widget CreateTextDialog(Widget parent)
{
    Widget w;
    Arg args[20];
    int n = 0;
    XtSetArg(args[n], XmNautoUnmanage, False); n++;
    XtSetArg(args[n], XmNresizePolicy, False); n++;
    Widget ps = XmCreateFormDialog(parent, "TextBrushDialog", args, n);
    XtAddCallback(ps, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)TEXT_ATTRIB_FILE);

    /* Apply, Reinit, Cancel buttons */
    w = XmCreatePushButtonGadget(ps, "Apply", NULL, 0);
    XtManageChild(w);
    XtAddCallback(w, XmNactivateCallback,
		  (XtCallbackProc)SensitiveButtons,
		  (XtPointer)&textBrushMapped);
    XtAddCallback(w, XmNactivateCallback, (XtCallbackProc)Verify,
		  (XtPointer)ps);
    XtAddCallback(w, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)TEXT_ATTRIB_FILE);

    w = XmCreatePushButtonGadget(ps, "Reinit", NULL, 0);
    XtManageChild(w);
    XtAddCallback(w, XmNactivateCallback, (XtCallbackProc)ReinitTextSheet,
		  (XtPointer) TRUE);
    XtAddCallback(w, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)TEXT_ATTRIB_FILE);
    
    w = XmCreatePushButtonGadget(ps, "Cancel", NULL, 0);
    XtManageChild(w);
    XtAddCallback(w, XmNactivateCallback, (XtCallbackProc) SensitiveButtons,
		  (XtPointer) &textBrushMapped);
    XtAddCallback(w, XmNactivateCallback,
		  (XtCallbackProc)ModifyTextSheet, NULL);
    XtAddCallback(w, XmNactivateCallback, (XtCallbackProc) UnmanageWidget,
		  (XtPointer)ps);
    XtAddCallback(w, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)TEXT_ATTRIB_FILE);

    /* click button */
    w = XmCreatePushButtonGadget(ps, "ClickButton", NULL, 0);
    XtManageChild(w);
    XtAddCallback(w, XmNactivateCallback, NamedChooseGo, NULL);
    XtAddCallback(w, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)TEXT_ATTRIB_FILE);

    /* label and separators */
    w = XmCreateSeparatorGadget(ps, "Separator1", NULL, 0);
    XtAddCallback(w, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)TEXT_ATTRIB_FILE);
    XtManageChild(w);
    w = XmCreateSeparatorGadget(ps, "Separator2", NULL, 0);
    XtAddCallback(w, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)TEXT_ATTRIB_FILE);
    XtManageChild(w);
    w = XmCreateLabelGadget(ps, "TextBrushLabel", NULL, 0);
    XtManageChild(w);
    XtAddCallback(w, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)TEXT_ATTRIB_FILE);
    
    
    /* name */
    Widget nameLabel = XmCreateLabelGadget(ps, "NameLabel", NULL, 0);
    XtManageChild(nameLabel);
    XtAddCallback(nameLabel, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)TEXT_ATTRIB_FILE);

    name = XmCreateTextField(ps, "Name", NULL, 0);
    XtAddCallback(name, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)TEXT_ATTRIB_FILE);
    XtManageChild(name);

    /* color */
    color_label = XmCreateLabelGadget(ps, "ColorLabel", NULL, 0);
    XtAddCallback(color_label, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)TEXT_ATTRIB_FILE);
    XtManageChild(color_label);
    n = 0;
    XtSetArg(args[n], XmNmarginWidth, 5); n++;
    XtSetArg(args[n], XmNmarginHeight, 5); n++;
    color_list = XmCreatePushButtonGadget(ps, "ColorList", args, n);
    XtAddCallback(color_list, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)TEXT_ATTRIB_FILE);
    XtManageChild(color_list);
    text_color = XmCreateTextField(ps, "ColorName", NULL, 0);
    XtManageChild(text_color);
    XtAddCallback(color_list, XmNactivateCallback,
		  (XtCallbackProc)MapColorList, (XtPointer)text_color);
    XtAddCallback(text_color, XmNactivateCallback,
		  (XtCallbackProc) ModifyNamedColor,
		  (XtPointer) TEXT_COLOR);
    XtAddCallback(text_color, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)TEXT_ATTRIB_FILE);

    /* font */
    font_label = XmCreateLabelGadget(ps, "FontLabel", NULL, 0);
    XtAddCallback(font_label, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)TEXT_ATTRIB_FILE);
    XtManageChild(font_label);
    n = 0;
    XtSetArg(args[n], XmNmarginWidth, 5); n++;
    XtSetArg(args[n], XmNmarginHeight, 5); n++;
    font_list = XmCreatePushButtonGadget(ps, "FontList", args, n);
    XtAddCallback(font_list, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)TEXT_ATTRIB_FILE);
    XtAddCallback(font_list, XmNactivateCallback,
		  (XtCallbackProc)MapFontList, (XtPointer)NAMED);
    XtManageChild(font_list);
    
    // Create XmText because of a XmTextField bug when string is too long;
    n = 0;
    XtSetArg(args[n], XmNeditable, False); n++;
    text_font = XmCreateText(ps, "FontName", args, n);
    XtAddCallback(text_font, XmNhelpCallback,
		  (XtCallbackProc)MapHelpFile, (XtPointer)TEXT_ATTRIB_FILE);
    XtManageChild(text_font);

    
    /* attach widgets */
    n = 0;
    XtSetArg(args[n], XmNleftAttachment, XmATTACH_OPPOSITE_WIDGET); n++;
    XtSetArg(args[n], XmNleftWidget, nameLabel); n++;
    XtSetValues(color_label, args, n);
    XtSetValues(font_label, args, n);
    n = 0;
    XtSetArg(args[n], XmNleftAttachment, XmATTACH_OPPOSITE_WIDGET); n++;
    XtSetArg(args[n], XmNleftWidget, name); n++;
    XtSetValues(color_list, args, n);
    XtSetValues(font_list, args, n);
    n = 0;
    XtSetArg(args[n], XmNrightAttachment, XmATTACH_OPPOSITE_WIDGET); n++;
    XtSetArg(args[n], XmNrightWidget, text_color); n++;
    XtSetValues(name, args, n);

    return ps;
}


/**
FUNCTION
    ModifTextSheet()
DESCRIPTION
    
**/
static void ModifyTextSheet()
{
    WhichAttributes = NAMED;
    ModifyGoNamedTextSheet();
}


/*****************************************************************************/


/**
FUNCTION
    ReinitTextSheet()
DESCRIPTION
    
**/
static void ReinitTextSheet(Widget, Boolean reinit_name, caddr_t)
{
    if (reinit_name)
        XmTextFieldSetString(name, named_text.TextBrushText);
    XmTextFieldSetString(text_color, named_text.Color);
    XmTextSetString(text_font, named_text.Font);

    WhichAttributes = NAMED;
    ModifyGoNamedTextSheet();
}


/**
FUNCTION
    ReinitText()
DESCRIPTION
    
**/
void ReinitText()
{
    named_text = all_attrib.text;
    if (textDialog)
	ReinitTextSheet(NULL, TRUE, NULL);
}


/**
FUNCTION
    Verify()
DESCRIPTION
    
**/
static void Verify(Widget, caddr_t parent, caddr_t)
{
    WhichAttributes = NAMED;
    char *name_str = XmTextFieldGetString(name);
    if (name_str[0] == '\0' ||
	strcmp(name_str, UNNAMED) == 0) {
	GFREE(name_str);
	ObjectName(textDialog);
	return;
    }
    int nb;
    if (NamedText.get(name_str, nb)) {
	AcknowledgeTextBrushName(parent);
	GFREE(name_str);
	return;
    }
    VerifyTextBrush(NULL, parent, NULL);
    GFREE (name_str);
} 


void VerifyTextBrush(Widget, caddr_t parent, caddr_t)
{
    char *name_str = XmTextFieldGetString(name);
    GFREE(named_text.Color);
    named_text.Color = XmTextFieldGetString(text_color);

    GFREE(named_text.Font);
    named_text.Font = XmTextGetString(text_font);

    if (strcmp(name_str, UNNAMED) != 0) {
	GFREE(named_text.TextBrushText);
	named_text.TextBrushText = (char*)GMALLOC(strlen(name_str) + 1);
	strcpy(named_text.TextBrushText, name_str);
	int nb;
	if (NamedText.get(name_str, nb))
	    ModifyGoNamedText();
	else
	    CreateGoNamedText();
	XmTextFieldSetString(textBrushText, named_text.TextBrushText);
    }
    
    if (strcmp(all_attrib.TextBrushText, name_str) == 0) {
	/*
	 * This textbrush is used as current attributes ==> I have to
	 * modify all_attrib.text;
	 */
	all_attrib.text = named_text;
    }
    GFREE(name_str);

    XtUnmanageChild((Widget)parent);
}




/**
FUNCTION
    ErrorFont()
DESCRIPTION
    
**/
void ErrorFont()
{
    XmTextSetString(text_font, named_text.Font);
}


/**
FUNCTION
    NamedChooseGo()
DESCRIPTION
    
**/
void textsheet_NamedChooseGo_cb (int, Go*)
{
    ReinitTextSheet(NULL, FALSE, NULL);
}

static void NamedChooseGo(Widget, caddr_t, caddr_t)
{
    WhichAttributes = NAMED;
    ChooseGo(TEXTBRUSH, textsheet_NamedChooseGo_cb, 0);
}
