#ifdef REV_INFO
static char rcsid[] = "$Id: viewmenu.cc,v 1.2 1992/07/09 08:04:02 jacomet Exp $";
#endif
/*****************************************************************************
MODULE:  Go Editor
FILE:    viewmenu.cc
AUTHOR:  Aline Jacomet
DATE:    September 24th 1991

DESCRIPTION:
    

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include "main.h"

#include <Xm/Scale.h>
#include <Xm/ScrolledW.h>
#include <Xm/ScrollBar.h>
#include <Xm/Form.h>
#include <Xm/PushBG.h>
#include <Xm/ToggleBG.h>

#include <Xb/Graphic.h>


/*
 * EXTERN FUNCTIONS
 */

// in file goView.cc;
extern void GoChangeZoom(float);


extern Widget orientation_tg[2];
extern Widget form_tg[7];



                         /********
			  * Zoom *
			  ********/


static Widget zoomWidget = NULL;
static Widget zoomScale = NULL;
static Widget clipWidget = NULL;

static float zoom_offset = 1.0;
static float ex_zoom_offset = 1.0;

static Widget hscroll, vscroll;

extern Widget scroll;


static void ChangeZoom(Widget, caddr_t, XmScaleCallbackStruct* call)
{
    ex_zoom_offset = zoom_offset;
    zoom_offset = (float)(call->value / 10.);

    if (ex_zoom_offset == zoom_offset)
	return;

    int value, slider, incr, page;
    if (zoom_offset > 1.0) {
	Arg args[5];
	Dimension w, h;
	Position x, y;
	int n = 0;
	XtSetArg(args[n], XmNwidth, &w); n++;
	XtSetArg(args[n], XmNheight, &h); n++;
	XtGetValues(clipWidget, args, n);
	n = 0;
	XtSetArg(args[n], XmNx, &x); n++;
	XtSetArg(args[n], XmNy, &y); n++;
	XtGetValues((Widget)graphic, args, n);

	int hmax, vmax, hmin, vmin, hsize, vsize;
	n = 0;
	XtSetArg(args[n], XmNsliderSize, &hsize); n++;
	XtSetArg(args[n], XmNmaximum, &hmax); n++;
	XtSetArg(args[n], XmNminimum, &hmin); n++;
	XtGetValues(hscroll, args, n);
	n = 0;
	XtSetArg(args[n], XmNsliderSize, &vsize); n++;
	XtSetArg(args[n], XmNmaximum, &vmax); n++;
	XtSetArg(args[n], XmNminimum, &vmin); n++;
	XtGetValues(vscroll, args, n);

	/*
	 * I add a pixel to x_value and y_value to avoid to have x(y)_value;
	 * less than the minimum;
	 */
	int x_value = (int)((zoom_offset / ex_zoom_offset) * (w / 2 - x) -
			    (w / 2) + 1);
	int y_value = (int)((zoom_offset / ex_zoom_offset) * (h / 2 - y) -
			    (h / 2) + 1);

	/*
	 * If the Graphic 's XbNautomaticResize resource is False, goroot;
         * becomes bigger than graphic widget so x(y)_value may be greater;
         * than the maximum minus the slider size;
	 */
	if (x_value > hmax - hsize)
	    x_value = hmax - hsize;
	if (y_value > vmax - vsize)
	    y_value = vmax - vsize;

	if (x_value < hmin)
	    x_value = hmin;
	if (y_value < vmin)
	    y_value = vmin;

	XmScrollBarGetValues(hscroll, &value, &slider, &incr, &page);
	XmScrollBarSetValues(hscroll, x_value, slider, incr, page, True);
	XmScrollBarGetValues(vscroll, &value, &slider, &incr, &page);
	XmScrollBarSetValues(vscroll, y_value, slider, incr, page, True);
    } else {
	XmScrollBarGetValues(hscroll, &value, &slider, &incr, &page);
	XmScrollBarSetValues(hscroll, 0, slider, incr, page, True);
	XmScrollBarGetValues(vscroll, &value, &slider, &incr, &page);
	XmScrollBarSetValues(vscroll, 0, slider, incr, page, True);
    }

    GoChangeZoom(zoom_offset);
}


static void CreateZoomDialog(Widget parent)
{
    Arg args[15];
    clipWidget = XtParent((Widget)graphic);
    
    zoomWidget = XmCreateFormDialog(parent, "ZoomDialog", NULL, 0);

    int n = 0;
    XtSetArg(args[n], XmNshowValue, True); n++;
    XtSetArg(args[n], XmNdecimalPoints, 1); n++;
    XtSetArg(args[n], XmNminimum, 1); n++;
    XtSetArg(args[n], XmNmaximum, 50); n++;
    XtSetArg(args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
    XtSetArg(args[n], XmNleftPosition, 10); n++;
    XtSetArg(args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
    XtSetArg(args[n], XmNrightPosition, 90); n++;
    XtSetArg(args[n], XmNtopAttachment, XmATTACH_POSITION); n++;
    XtSetArg(args[n], XmNtopPosition, 10); n++;
    XtSetArg(args[n], XmNbottomAttachment, XmATTACH_POSITION); n++;
    XtSetArg(args[n], XmNbottomPosition, 75); n++;
    zoomScale = XmCreateScale(zoomWidget, "ZoomScale", args, n);
    XtManageChild(zoomScale);
    XtAddCallback(zoomScale, XmNvalueChangedCallback,
		  (XtCallbackProc) ChangeZoom, NULL);

    n = 0;
    XtSetArg(args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
    XtSetArg(args[n], XmNleftPosition, 10); n++;
    XtSetArg(args[n], XmNbottomAttachment, XmATTACH_POSITION); n++;
    XtSetArg(args[n], XmNbottomPosition, 95); n++;
    Widget w = XmCreatePushButtonGadget(zoomWidget, "Unmap", args, n);
    XtManageChild(w);

    n = 0;
    XtSetArg(args[n], XmNverticalScrollBar, &vscroll); n++;
    XtSetArg(args[n], XmNhorizontalScrollBar, &hscroll); n++;
    XtGetValues(scroll, args, n);
}


void ZoomDisplay(Widget, Widget parent, caddr_t)
{
    if (!zoomWidget)
	CreateZoomDialog(parent);
    Arg arg;
    XtSetArg(arg, XmNvalue, zoom_offset*10);
    XtSetValues(zoomScale, &arg, 1);
    
    XtManageChild(zoomWidget);
}




                             /**************************
			      * FORMAT and ORIENTATION *
			      **************************/


void ChangeOrientation(Widget wid, int o, XmToggleButtonCallbackStruct* data)
{
    if (data->set == False) {
	/*
	 * user clicks on the already selected orientation ==> let it stay
	 * at True
	 */
	XmToggleButtonGadgetSetState(wid, True, False);
	return;
    }

    for (int i = 0; i < 2; i++) {
	if (orientation_tg[i] != wid)
	    XmToggleButtonGadgetSetState(orientation_tg[i], False, False);
    }
    
    Arg args[2];
    Dimension w, h, inter;
    int n = 0;
    XtSetArg(args[n], XmNwidth, &w); n++;
    XtSetArg(args[n], XmNheight, &h); n++;
    XtGetValues((Widget)graphic, args, n);
    Boolean do_it = False;
    switch(o) {
      case LANDSCAPE:
	if (h > w)
	    do_it = True;
	break;
      case PORTRAIT:
	if (h < w)
	    do_it = True;
	break;
    }
    if (do_it) {
	inter = w;
	w = h;
	h = inter;
	n = 0;
	XtSetArg(args[n], XmNwidth, w); n++;
	XtSetArg(args[n], XmNheight, h); n++;
	XtSetValues((Widget)graphic, args, n);
    }
}


void ChangeFormat(Widget wid, int f, XmToggleButtonCallbackStruct* data)
{
    if (data->set == False) {
	/*
	 * user clicks on the already selected format ==> let it stay
	 * at True
	 */
	XmToggleButtonGadgetSetState(wid, True, False);
	return;
    }

    for (int i = 0; i < 7; i++) {
	if (form_tg[i] != wid)
	    XmToggleButtonGadgetSetState(form_tg[i], False, False);
    }
    
    Boolean land = XmToggleButtonGadgetGetState(orientation_tg[LANDSCAPE]);
    Dimension w = 0;
    Dimension h = 0;
    Boolean automatic_resize = False;
    switch(f) {
      case A3:
	if (land) {
	    w = 42000;
	    h = 29700;
	} else {
	    w = 29700;
	    h = 42000;
	}
	break;
      case A4:
	if (land) {
	    w = 29700;
	    h = 21000;
	} else {
	    w = 21000;
	    h = 29700;
	}
	break;
      case A5:
	if (land) {
	    w = 21000;
	    h = 14900;
	} else {
	    w = 14900;
	    h = 21000;
	}
	break;
      case LETTER:
	if (land) {
	    w = 28000;
	    h = 21700;
	} else {
	    w = 21700;
	    h = 28000;
	}
	break;
      case B5:
	if (land) {
	    w = 25800;
	    h = 18300;
	} else {
	    w = 18300;
	    h = 25800;
	}
	break;
      case B4:
	if (land) {
	    w = 36600;
	    h = 25800;
	} else {
	    w = 25800;
	    h = 36600;
	}
	break;
      case AUTOMATIC_RESIZE:
	automatic_resize = True;
	break;
    }
    int n = 0;
    Arg args[4];
    if (! automatic_resize) {
	XtSetArg(args[n], XmNunitType, Xm100TH_MILLIMETERS); n++;
	XtSetArg(args[n], XmNwidth, w); n++;
	XtSetArg(args[n], XmNheight, h); n++;
    }
    XtSetArg(args[n], XbNautomaticResize, automatic_resize); n++;
    XtSetValues((Widget)graphic, args, n);
}



int WhichOrientation()
{
    Boolean land = XmToggleButtonGadgetGetState(orientation_tg[LANDSCAPE]);
    if (land)
	return LANDSCAPE;
    else
	return PORTRAIT;
}
