/* $Id: clock.h,v 1.3 1992/10/21 17:40:46 andre Exp $ */
/*****************************************************************************
MODULE:  Examples
FILE:    clock.h
AUTHOR:  Michel Andre
DATE:    07-01-92

DESCRIPTION: See clock.cc
    

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include	<time.h>
#include	<Go/Rectangle.h>
#include	<Go/Circle.h>
#include	<Go/Label.h>
#include	<Go/Tools/Color.h>


/* ABSTRACT CLASS CLOCK */
/************************/

class goClock : public GoStdDS
{
    public:
                        goClock();
	virtual void	update(){
	    time (&_time);
	    _timeBuf = localtime (&_time);
	}
	void            shift (int timeDifference);
	virtual void    reframe (GNumber){}

    protected:
	struct tm*	_timeBuf;
	long		_time;
	int             _timeDifference;
	GBool		_firstTime;
};



/* ANALOGIC CLOCK */
/******************/

class analogicGoClock : public goClock
{
    public:
			analogicGoClock (GNumber clockRadius);
	void		update();
	void		reframe (GNumber goClockRadius);
	GoCircle*	circle() const {return
			((analogicGoClock*)this)->_circle;}
	GoPolygon*	hourHand() const {return
			((analogicGoClock*)this)->_hourHand;}
	GoPolygon*	minuteHand() const {return
			((analogicGoClock*)this)->_minuteHand;}
	GoLabel*	label() const {return
			((analogicGoClock*)this)->_label;}
	GColor&		highLightColor() const {return
			((analogicGoClock*)this)->_highLightCol;}
	GColor		_highLightCol;
			
    private:
	GoCircle*       _circle;
	GoPolygon*	_hourHand;
	GoPolygon*	_minuteHand;
	GoRectangle*	_secondsRing[60];
	GoLabel*	_label;
	GNumber		_clockRadius;
	int		_lastSecond;
};



/* DIGITAL CLOCK */
/*****************/

class digit : public GoStdDS
{
    public:
			digit (unsigned char = 0);
	void		setValue (unsigned char = 0);
	unsigned char	getValue() const {return _value;}
	void		segmentsSize (GNumber = 5);

    private:
	GoRectangle*	_segments[7];
	unsigned char	_value;
	GNumber		_segmentSize;
	void		_segmentsUpdate();
};


class digitalGoClock : public goClock
{
    public:
			digitalGoClock (GNumber clockWidth);
	void   		update();
        void    	reframe (GNumber clockWidth);
	GoRectangle*	background() const {return
			((digitalGoClock*)this)->_background;}

    private:
	digit*		_hour1;
	digit*		_hour2;
	digit*		_minute1;
	digit*		_minute2;
	digit*		_second1;
	digit*		_second2;
	GoRectangle*	_background;
	GNumber		_clockWidth;
};
