#ifdef REV_INFO
static char rcsid[] = "$Id: tstClock.cc,v 1.6 1992/12/28 16:19:53 andre Exp $";
#endif
/*****************************************************************************
MODULE:  Examples
FILE:    tstClock.cc
AUTHOR:  Michel Andre
DATE:    07-01-92

DESCRIPTION:
    

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include <signal.h>
#include <stdio.h>
#include <Go/Tools/FilePath.h>
#include <Go/GoRoot.h>
#include <Go/XGoInterf.h>
#include <Go/XInDisp.h>
#include <Go/Event.h>
#include <assert.h>
#include "clock.h"

#ifdef GNUC_VERSION_1
#include <stream.h>
#else
#include <fstream.h>
#endif


#define		RESSOURCES_DIR	"app-files/clock"
#define		RESSOURCES_FILE	"clock.def"
#define         SECOND          1000 /* milliseconds */


GoRoot		*goroot;
GoInterf	*interf;
analogicGoClock *anaClock[24];
digitalGoClock  *digClock[24];
int             nbClock = 0;


/* defaults */
GNumber	clockRadius = 20.0;
char	clockColor[32] = "white";
char	winBackColor[32] = "black";
char	hourHandColor[32] = "black";
char	minuteHandColor[32] = "black";
char	fontName[128] = "-misc-fixed-medium-r-normal--8-60-100-100-c-50-iso8859-1";
char	textColor[32] = "black";

char*   siteName [16];
int     timeDif [16];


static void
syntaxError (char* fileName, int lineNumber)
{
    fprintf (stderr, "syntax error at line %d of file %s\n",
        lineNumber, fileName);
    exit (1);
}


static void
loadRessources (char* fileName)
{
#ifdef GNUC_VERSION_1
    istream file (fileName, "r");
#else
    ifstream file (fileName);
#endif

    if (!file.fail()){

        /* current line counter */
        int lineNumber = 0;

        /* read buffer */
        char line [128];

        /* parameter */
        char* param;

#ifdef __GNUC__
    while (!file.fail()) {
#else
    while (!file.eof()) {
#endif
	    file.getline (line, 128);

            lineNumber++;

            /* comment line  or empty line or CVS version line */
            if (*line != '#' && *line != 0 && *line != '!'){

	        /* no value */
                if ((param = strchr (line, ' ')) == NULL)
                    syntaxError (fileName, lineNumber);

                /* pass the blank character */
                ++param;

                /* substitute \n by \0 */
                for (int i = 0; i < 128; i++)
                    if (*(param + i) == '\n'){
                        *(param+i) = '\0';
                        break;
                    }

		/* window background color */
                if (!strncmp (line, "winBackColor", strlen ("winBackColor")))
                    strcpy (winBackColor, param);

	        /* clock ressources */
		else if (!strncmp (line, "clockRadius", strlen ("clockRadius")))
		    sscanf (param, "%f", &clockRadius);
                else if (!strncmp (line, "clockColor", strlen ("clockColor")))
                    strcpy (clockColor, param);

		/* window background color */
                else if (!strncmp (line, "winBackColor", strlen ("winBackColor")))
                    strcpy (winBackColor, param);

	        /* hour hand ressources */
                else if (!strncmp (line, "hourHandColor", strlen ("hourHandColor")))
                    strcpy (hourHandColor, param);

	        /* minute hand ressources */
                else if (!strncmp (line, "minuteHandColor", strlen ("minuteHandColor")))
                    strcpy (minuteHandColor, param);

	        /* text ressources */
                else if (!strncmp (line, "fontName", strlen ("fontName")))
                    strcpy (fontName, param);
                else if (!strncmp (line, "textColor", strlen ("textColor")))
                    strcpy (textColor, param);

		/* sites ressources */
		else if (!strncmp (line, "site", strlen ("site"))){
                    sscanf (param, "%d", &timeDif[nbClock]);
		    char* str = strchr (param, ' ');
		    if (!str) syntaxError (fileName, lineNumber);
		    siteName[nbClock] = (char*)malloc (strlen (str) + 1);
		    strcpy (siteName[nbClock], str);
		    nbClock++;
		}

	        /* syntax error */
                else
                    syntaxError (fileName, lineNumber);
	    }
	}

	file.close();
    }
}


static void
createClock (int index, int time_dif, char* site)
{
    GoLabel* siteLabel = NULL;
    
    /* clock creation */
    anaClock[index] = new analogicGoClock (clockRadius);
    digClock[index] = new digitalGoClock (clockRadius*1.5);
    anaClock[index]->reparent (goroot);
    digClock[index]->reparent (goroot);
    anaClock[index]->shift (time_dif);
    digClock[index]->shift (time_dif);

    if (index == 0)
	anaClock[index]->move (
	    GPoint (clockRadius*1.1, clockRadius*1.1 + 10));
    else
	anaClock[index]->move (
	    GPoint (2*clockRadius*1.1, 0), anaClock[index-1]);
    
    digClock[index]->move (GPoint (-digClock[index]->clip().bigbox().width()/2,
	clockRadius * 1.1), anaClock[index]);

    if (site){
	siteLabel = GoLabel::create();
	siteLabel->reparent (goroot);
	siteLabel->setString (site, strlen (site));
	siteLabel->lineHeight (8);
	GNumber siteLabelWidth = siteLabel->myClip().bigbox().width();
	siteLabel->move (GPoint (-siteLabelWidth/2, -(clockRadius*1.1 + 8)), anaClock[index]);
    }

    /* background color */
    GShColor *c = interf->colorFromName (clockColor);
    anaClock[index]->circle()->brush().color (c);
    digClock[index]->background()->brush().color (c);
    c->destroy();

    /* labels font and color */
    GTextBrushValues values;
    GFont* f = ((XGoInterf*)interf)->xFontNameToFont (fontName);
    values.font = f->shFont()->dup();
    values.color = interf->colorFromName (textColor);
    anaClock[index]->label()->stringTextBrush().modify
	(GCOLOR | GFONT, &values);
    if (siteLabel)
	siteLabel->stringTextBrush().modify (GCOLOR | GFONT, &values);
    values.font->destroy();
    delete f;
    
    /* second ring highlight color */
    anaClock[index]->highLightColor().modify (values.color);
    values.color->destroy();
    
    /* hour hand */
    c = interf->colorFromName (hourHandColor);
    anaClock[index]->hourHand()->brush().color (c);
    c->destroy();
    
    /* minute hand */
    c = interf->colorFromName (minuteHandColor);
    anaClock[index]->minuteHand()->brush().color (c);    
    c->destroy();
}


static void turnClock (void* null)
{
    static GBool firstTime = GTRUE;
    assert (null == NULL);

    for (int i = 0; i < nbClock; i++){
        anaClock[i]->update();
	digClock[i]->update();
    }

    if (firstTime)
	firstTime = GFALSE;
    else
	GInputDispatcher()->timerProc (turnClock, NULL, SECOND);
}


/**
FUNCTION
    interfIsResized (void*, GoInterf*)
DESCRIPTION
    This postcond is called a each interf->resize().
    The interf->resize() is called on each ConfigureNotify event.
    Here we want that goroot follows the interf sizes.
**/

static void interfIsResized (void* root, GoInterf* itrf)
{
    assert (goroot == (GoRoot*)root);
    assert (itrf == interf);
    goroot->rescale (interf->width(), interf->height());
}


/**
FUNCTION
    flushGoRoot()
DESCRIPTION
   This is an idle procedure called by the inputdispatcher
   when no event is left in the input queue.
**/

static GIdle flushGoRoot(void* root)
{
    assert (goroot == (GoRoot*)root);
    goroot->flush();
    return GIDLE_AFTER_NEXTEVENT;
}


main (int, char**)
{
    /* load the ressources */
    char* ressourcesFilePath = GGetFilePath
	(RESSOURCES_DIR, RESSOURCES_FILE);
    loadRessources (ressourcesFilePath);
    GFREE (ressourcesFilePath);

    /* interf and goroot creation */
    interf = new XGoInterf; 
    goroot = (GoRoot*)GoRoot::create (interf);
    interf->resizePostcond (interfIsResized, goroot);
    interf->resize (nbClock * clockRadius*1.1 * 2,
       clockRadius*3 + 4);

    /* background of the window */
    GShColor* c = interf->colorFromName (winBackColor);
    interf->background (c);
    c->destroy();

    /* clocks creation */
    for (int i = 0; i < nbClock; i++){
	createClock (i, timeDif[i], siteName[i]);
	free (siteName[i]);
    }

    /* alarm the clock */
    turnClock (NULL);
    
    interf->map();
    goroot->map();

    /* the following is specific to XInputDispatcher class */
    /* note that the constructor is also specific !!! */
    GInputDispatcher() = new XInputDispatcher(((XGoInterf*)interf)->display());
    ((XInputDispatcher*)GInputDispatcher())->addXGoInterf((XGoInterf*)interf);

    /* end of specific */
    GInputDispatcher()->idleProc(flushGoRoot, goroot);
    GInputDispatcher()->timerProc(turnClock, NULL, SECOND);

    /* main loop */
    GInputDispatcher()->runEventLoop();

    /* NOT REACHED */
    assert (GFALSE);
}



