#ifdef REV_INFO
static char rcsid[] = "$Id: dames.cc,v 1.6 1992/12/31 14:01:32 jojic Exp $";
#endif
/*****************************************************************************
MODULE:  Examples
FILE:    dames.cc
AUTHOR:  Jacques Davy
DATE:    February 91

DESCRIPTION:
    Jeu de Dames

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#include <stdio.h>
#include <Go/GoRoot.h>
#include <Go/GoUI.h>
#include <Go/Rectangle.h>
#include <Go/Circle.h>
#include <Go/Label.h>
#include <Go/XGoInterf.h>
#include <Go/XInDisp.h>
#include <Go/FullSensor.h>
#include <Go/MoveGrabber.h>
#include <Go/Translator.h>
#include <Go/Tools/ShColor.h>
#include <Go/Tools/Pen.h>
#include <Go/Tools/Brush.h>
#include <Go/Tools/Bind.h>

#define ROWS      10             /* row number */
#define COLUMNS   10             /* column number */
#define ELEM      10             /* number of row or colomn */
#define ELEM_SZ   20.0           /* size of each square */
#define BORDER_SZ 3.0            /* outline width */
#define PIECE_SZ  (ELEM_SZ-3.5)  /* piece size   */
#define BLACKS    20             /* black pieces */
#define WHITES    20             /* white pieces */

static char goNposition[] = "position";

static void CreateSharedResources();
static Go*  CreateBoard();
static Go*  CreateScorer(const char* brush_name);
static void IncrementScorer (Go* scorer, int inc_score);

/* interactive actions */
static void MovePiece(GoRoot*, const GEvent*);

/* Pre and post condition */
static GBool PreReparent (void*, const Go*, const Go* new_parent, const Go* (*next_brother));
static void PostReparent (void*, Go*, Go* old_parent, Go* old_next_brother);


Go *black_scorer, *white_scorer;


/*
 * MANAGE USER INPUT
 */

static GIdle flush(void* goroot)
{
    // this function is called when no event is available on the input queue;
    // and before  "handle_next_event";
    ((GoRoot*)goroot)->flush();
    return GIDLE_AFTER_NEXTEVENT;
}



main()
{
    // Create a GoInterf
    GoInterf* gointerf = new XGoInterf;

    // Create a GoRoot
    GoRoot* goroot = GoRoot::create();
    goroot->interf(gointerf);

    CreateSharedResources();

    // Create the board and pieces
    Go* board = CreateBoard();
    board->move(GPoint(BORDER_SZ, BORDER_SZ));
    board->reparent(goroot);

    // Create the scorers
    black_scorer = CreateScorer("black_brush");
    black_scorer->reparent(goroot);
    black_scorer->move(GPoint(ELEM*ELEM_SZ + 4*BORDER_SZ,
				  BORDER_SZ));
    white_scorer = CreateScorer("white_brush");
    white_scorer->reparent(goroot);
    white_scorer->move(GPoint(ELEM*ELEM_SZ + 4*BORDER_SZ,
				  (ELEM-1)*ELEM_SZ + 2*BORDER_SZ));
    

    // define input dispacher before creating GoUI;
    // here I use an XInputDispatcher;
    GInputDispatcher() =
	new XInputDispatcher(((XGoInterf*)gointerf)->display());

    // create the object to dispatch event to sensors;
    (void) new GoUI (goroot);

    // bind my functions;
    GBind("MovePiece", MovePiece, "Function");

    // declare  program() used by sensor to create interactor;
    // here I use only the "move" one;
    (void)GMoveGrabberPgm::create();

    // read the user preference;;
    GLoadTranslatorPgms("dames.goui");

    // run the toplevel translator on the full window.
    (void)GoFullSensor::create("toplevel", goroot, NULL, NULL, NULL);
    
    // Setting game rules
    (GoCircle::metaclass())->reparentPrecond (PreReparent, NULL);
    (GoCircle::metaclass())->reparentPostcond (PostReparent, NULL);

    // Map objects and handle user events
    GRegion clip;
    goroot->addClip(clip);
    gointerf->resize ((GNumber)(clip.bigbox().width() + ELEM_SZ),
		      (GNumber)(clip.bigbox().height() + BORDER_SZ));
    gointerf->map();
    goroot->map();

    // INIT and RUN THE GLOBAL INPUT DISPATCHER;
    // I choose the X one;

    GInputDispatcher()->idleProc(flush, goroot);

    // specific to XInputDispatcher;
    ((XInputDispatcher*)GInputDispatcher())->addXGoInterf((XGoInterf*)gointerf);
   
    // main loop; 
    (void)GInputDispatcher()->runEventLoop();

    // NOT REACHED HERE;
}

/* resources creation */
static void CreateSharedResources()
{
    GRgb rgb(255*256,255*256,207*256);
    GBrushColor("board_brush", GCreateShColor(&rgb));

    GRgb rgb2(56*256,31*256,18*256);
    GPenColor("border_pen", GCreateShColor(&rgb2));
    GPenLineWidth("border_pen", BORDER_SZ);

    GRgb rgb3(151*256,137*256,85*256);
    GBrushColor("square_brush", GCreateShColor(&rgb3));

    GRgb rgb4(0, 0, 0);
    GBrushColor("black_brush", GCreateShColor(&rgb4));

    GRgb rgb5(255*256, 255*256, 255*256);
    GBrushColor("white_brush", GCreateShColor(&rgb5));
}

/*
 * Board and pieces creation
 */
static Go* CreateBoard ()
{
    /*
     * board:
     *      GoRectangle,
     *      has an outline (BORDER_SZ and "border_brush")
     *      has a background ("board_brush")
     */
    GoRectangle* board = new GoRectangle (ELEM*ELEM_SZ + BORDER_SZ,
					  ELEM*ELEM_SZ + BORDER_SZ);
    board->pen().modify("border_pen");
    board->background(GTRUE);
    board->brush().modify("board_brush");
    board->map();

    for (int row = 0; row < ROWS; row++)
        for (int col = 0; col < COLUMNS/2; col++)
        {
            /*
             * black squares:
	     *       have no outline,
	     *       use the "square_brush"
	     *       are children of board
	     *       row/column (position) are stored as association
             */
            GoRectangle* black_pos = new GoRectangle (ELEM_SZ, ELEM_SZ);
            black_pos->outline(GFALSE);
	    black_pos->background(GTRUE);
	    black_pos->brush().modify("square_brush");
	    black_pos->reparent(board);
	    GNumber shift = ( row & 1 ? ELEM_SZ : 0.0 );
	    black_pos->move(GPoint(BORDER_SZ/2 + ELEM_SZ*2*col + shift,
				       BORDER_SZ/2 + ELEM_SZ*row), board);
	    black_pos->assoc()->set(goNposition,
				    (const void*)((long)((row+1<<16)+(2*col)+
						  (row&1?1:0)+1)));
	    black_pos->map();

	    /*
	     * pieces:
	     *       have no outline (except when they are dame: "border_brush")
	     *       use either the "black_brush" or the "white_brush"
	     *       each is child of a black position (black_pos)
	     *       a precondition at the class level checks the reparent
	     *       a postcondition at the class level executes the
	     *         reparent post processing.
	     */
            if (row < 4 || row > 5)
	    {
                GoCircle* piece = new GoCircle (PIECE_SZ/2);
		piece->outline(GFALSE);
		piece->pen().modify("border_pen");
	        piece->background(GTRUE);
	        if (row < 4)
	            piece->brush().modify("black_brush");
		else
		    piece->brush().modify("white_brush");
		piece->reparent(black_pos);
		piece->move(GPoint(black_pos->width()/2,
				       black_pos->height()/2),
				 black_pos);
		piece->map();
	    }
        }

    return board;
}

/* Create scorer */
static Go* CreateScorer (const char*)
{
    GoLabel* label = new GoLabel;
    label->setString ("0", 1);
    label->lineHeight (ELEM_SZ/2);
    label->map();
    return label;
}

/*
 * User Actions
 */

static void _duped_piece_is_moved(void* closure, GoSensor* sensor)
{
    if (sensor->interactor()->state() != GITOR_DONE)
    {
	// the user did cancel the move;
	sensor->parent()->destroy();
	return;
    }
    
    // destroy the duped_go (parent of the sensor);
    sensor->parent()->destroy();

    // find the go at the current position;
    Go* piece = (Go*)closure;
    GPoint position = piece->interf()->getPointer();
    Go* new_parent = piece->root()->goAt(position, 0);
    if (new_parent)
	piece->parentRelativeReparent(new_parent);

}

static void MovePiece(GoRoot* goroot, const GEvent* event)
{
    // read the mouse position;
    GPoint position;
    if (event->type() == GBUTTON_PRESS_EVENT ||
	event->type() == GBUTTON_RELEASE_EVENT)
	position = event->position();
    else
	position = goroot->interf()->getPointer();

    // Find concerned piece and do move;

    Go* go = goroot->goAt(position, 0);
    if (go && (go->getMetaclass() == GoCircle::metaclass()))
    {
	// It is a piece;
	// dup the go and move it;
	Go* duped_one = go->dupThis();
	duped_one->setCoordsys(go->coordsys());
	duped_one->reparent(goroot);
	goroot->flush();
	(void)GoFullSensor::create("move", duped_one, event,
				   _duped_piece_is_moved, go);
    }
}


/* 
 * Constraints
 * Pre reparent condition
 */
static Go* piece_to_be_destroyed = NULL;

static GBool PreReparent(void*, const Go* piece,
			 const Go* new_parent, const Go* (*))
{
    // Reparent due to a piece destroying
    if (!new_parent)
        return GTRUE;

    // check if the new parent is a black position
    long new_pos = (long)new_parent->assoc()->get(goNposition);

    if (!new_pos)
        return GTRUE;  // new_parent does not define the Nposition;

    if (piece->parent() == new_parent)
        return GFALSE;
    if (new_parent == piece)
	return GFALSE;

    // check if the new position is valid (has to be competed)
    long old_pos = (long)piece->parent()->assoc()->get(goNposition);
    int row_direction;
    if (strcmp(((GoCircle*)piece)->brush().name(), "black_brush") == 0)
        row_direction = 1;
    else
        row_direction = -1;
    int delta_row = (int)((new_pos>>16) - (old_pos>>16));
    int delta_col = (int)((new_pos&0xffff) - (old_pos&0xffff));

    if (delta_row * row_direction < 0 ||
	abs(delta_row) > 2 ||
	abs(delta_row) < 1 ||
	abs(delta_col) != abs(delta_row))
        return GFALSE;

    // a piece has to be destroyed ?
    if (abs(delta_row) == 2)
    {
        GNumber y = (((old_pos>>16) + (delta_row/2))*ELEM_SZ)-(ELEM_SZ/2);
	GNumber x = (((old_pos&0xffff) + (delta_col/2))*ELEM_SZ)-(ELEM_SZ/2);
        Go* go = piece->root()->goAt(GPoint(x,y), 0);
        if (!go || (go->getMetaclass() != GoCircle::metaclass()))
            return GFALSE;
        if (strcmp(((GoCircle*)go)->brush().name(),
		   ((GoCircle*)piece)->brush().name()) == 0)
            return GFALSE;
	piece_to_be_destroyed = go;
    }
    return GTRUE;
}

/*
 * Post Reparent condition
 */
static void PostReparent(void*, Go* go, Go*, Go*)
{
    if (!go->parent())
        return;
    if ((long)go->parent()->assoc()->get(goNposition) == 0)
	return;
    
    // reparent due to a piece destroying

    int inc_score = 0;

    // is there a piece to destroy?
    if (piece_to_be_destroyed)
    {
        inc_score++;
	piece_to_be_destroyed->destroy();
        piece_to_be_destroyed = NULL;
    }

    // does a piece become a "dame"?
    GoCircle *piece = (GoCircle*)go;
    if (!piece->outline())
    {
        long row = ((long)piece->parent()->assoc()->get(goNposition))>>16;
        if (((row == 1) &&
	     (strcmp(piece->brush().name(), "white_brush")==0)) ||
	    ((row == ROWS) &&
	     (strcmp(piece->brush().name(), "black_brush")==0)))
	{
            piece->outline(GTRUE);
	    inc_score += 10;
        }
    }

    // Update scorer
    if (inc_score > 0)
    {
        if (strcmp(piece->brush().name(), "black_brush") == 0)
            IncrementScorer (black_scorer, inc_score);
        else
            IncrementScorer (white_scorer, inc_score);
    }
}


/* Update Scorer */
static void IncrementScorer (Go* scorer, int inc_score)
{
    GoLabel *label = (GoLabel*)scorer;
    int new_score = atoi(label->getString()) + inc_score;

    char new_string[100];
    sprintf (new_string, "%d", new_score);
    label->setString (new_string, strlen(new_string));
}
