#ifdef REV_INFO
static char rcsid[] = "$Id: flags.cc,v 1.12 1992/12/31 14:02:15 jojic Exp $";
#endif
/*****************************************************************************
MODULE:  
FILE:    flags.cc
AUTHOR:  Olivier Jojic and Aline Jacomet
DATE:    December 1991

DESCRIPTION:
    International Text Viewer: a simple Go application ...

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#include <Go/Tools/FilePath.h>
#include <Go/Tools/Atom.h>
#include <Go/Tools/Error.h>

#include <Go/Ext/Converter.h>

#include <Go/Init.h>

#include <Go/XGoInterf.h>
#include <Go/XInDisp.h>
#include <Go/GoUI.h>
#include <Go/FullSensor.h>
#include <Go/Translator.h>
#include <Go/Tools/Bind.h>
#include <Go/GoRoot.h>
#include <Go/GoStream.h>

#include <Go/Text.h>

#define TRANSLATOR_FILE      "flags.goui"
#define GOSTREAM_FILE  "flags.str"
#define LANGUAGE_KEY   "language"
#define SELECTION_KEY  "selection"


GoRoot*    goroot;
GoInterf*  gointerf;
GoUI*	   goui;
Go* current_text = NULL;
Go* current_rect = NULL;

GAtom language_key;
GAtom selection_key;

void create_goroot_gointerf_and_indisp ();
void initialize_goui ();
void read_my_gostream ();
void restore_assoc (void*, Go*, GoStream*);
void event_loop ();
void map_another_text (GoRoot*, const GPoint*);
void test_and_map (void*, Go*);

/*
 * MAIN PROGRAM
 */
main (int, char**)
{
    create_goroot_gointerf_and_indisp();
    initialize_goui();
    read_my_gostream();
    event_loop();
}

/*
 * INITIALIZE GoRoot, GoInterf and GoUI
 */
void create_goroot_gointerf_and_indisp ()
{
    gointerf = new XGoInterf;
    gointerf->map();

    // Change the background color;
    GShColor* color = gointerf->colorFromName("gray93");
    gointerf->background(color);
    color->destroy();

    goroot = GoRoot::create();

    // GoRoot has to know GoInterf;
    goroot->interf((GoInterf*)gointerf);
    // INIT and RUN THE GLOBAL INPUT DISPATCHER;
    // I choose the X one;


    GInputDispatcher() =
	new XInputDispatcher(((XGoInterf*)gointerf)->display());

    // specific to XInputDispatcher;
    ((XInputDispatcher*)GInputDispatcher())->addXGoInterf((XGoInterf*)gointerf);
}


void initialize_goui ()
{
    goui = new GoUI (goroot);

    // Get the fullpath of the GoUI file;
    char* filename = GGetFilePath("app-files/flags", TRANSLATOR_FILE);
    if (! filename)
	GError("File %s does not exist.\n", TRANSLATOR_FILE);

    // Load the translators file;
    GLoadTranslatorPgms(filename);
    GFREE(filename);

    // Translators have to know the "map_another_text" function;
    GBind("MapAnotherText", &map_another_text);

    GoFullSensor* toplevel = GoFullSensor::create();
    toplevel->program("toplevel");
    toplevel->reparent(goroot);
}


/*
 * READ the GoStream and INITIALIZE the Go tree
 */
void read_my_gostream ()
{
    // Get the all path of the Gostream file;
    char* filename = GGetFilePath("app-files/flags", GOSTREAM_FILE);
    if (! filename)
	GError("File %s does not exist.\n", GOSTREAM_FILE);

    GoExtensionsInit();

    // To restore the saved associations;
    (void)Go::metaclass()->restorePostcond(restore_assoc, NULL);
    language_key = GCreateAtom(LANGUAGE_KEY);
    selection_key = GCreateAtom(SELECTION_KEY);

    // Read the GoStream file;
    GSet restored_go;
    GSet restored_shared;
    uint nb_restored_go = 0;
    uint nb_restored_shared = 0;
    GConverter converter;
    if (converter.gostreamFileToGo(filename, &restored_go, nb_restored_go,
				   &restored_shared, nb_restored_shared)
	== GOK) {
	if (nb_restored_go)
	    for (Go* go = (Go*)restored_go.first(); go;
		 go = (Go*)restored_go.next())
		go->reparent(goroot);
	else {
	    GFREE(filename);
	    GError("File %s is empty.\n", GOSTREAM_FILE);
	}
	GFREE(filename);
    } else {
	GFREE(filename);
	GError("An error occured during the %s file loading.\n",
	       GOSTREAM_FILE);
    }
}

void restore_assoc (void*, Go* go, GoStream* stream)
{
    const char notYetFinished = 'a';
    char c;
    char* key = NULL;
    char* val = NULL;

    for (;;) {
	/*
	 * Read the saved associations in GoStream file;
	 */
	stream->readChar(&c);
	if (c != notYetFinished)
	    break;
	stream->readString(0, &key);
	stream->readString(0, &val);

	if (strcmp(key, LANGUAGE_KEY) == 0)  {
	    if (go->getMetaclass()->isSubclass(GoTextual::metaclass())) {
		// This object is a text;
		go->assoc()->set(language_key, strlen(val) + 1, val);
		go->unmap();
	    } else {
		// This object is a flag;
		go->assoc()->set(language_key, strlen(val) + 1, val);
	    }
	} else
	    if (strcmp(key, SELECTION_KEY) == 0)  {
		// This object is a selection rectangle;
		go->assoc()->set(selection_key, strlen(val) + 1, val);
		go->unmap();
	    }
	GFREE(key);
	GFREE(val);
    }
}

/*
 * MANAGE USER INPUT
 */

static GIdle flush(void* closure)
{
    ((GoRoot*)closure)->flush();
    return GIDLE_AFTER_NEXTEVENT;

}

void event_loop ()
{
    // INIT and RUN THE GLOBAL INPUT DISPATCHER;
    // I choose the X one;

    GInputDispatcher() = new XInputDispatcher(((XGoInterf*)gointerf)->display());
    GInputDispatcher()->idleProc(flush, goroot);

    // specific to XInputDispatcher;
    ((XInputDispatcher*)GInputDispatcher())->addXGoInterf((XGoInterf*)gointerf);
   
    // main loop; 
    (void)GInputDispatcher()->runEventLoop();

    // NOT REACHED HERE;
}

void map_another_text (GoRoot* root, const GPoint* p)
{
    Go* go = root->goAt(*p, 0.5);
    if (go &&
	! go->getMetaclass()->isSubclass(GoTextual::metaclass())) {
	// The user clicked on a flag or on a selection rectangle;
	int n;
	char* lang_flag = (char*)go->assoc()->get(language_key, n);
	if (lang_flag == NULL)   // This is a selection rectangle;
	    return;

	// Unmap the current text and the current hightlight rectangle
	if (current_text != NULL)
	{
	    current_text->unmap();
	    current_text = NULL;
	}
	if (current_rect != NULL)
	{
	    current_rect->unmap();
	    current_rect = NULL;
	}

	root->forEveryGo (test_and_map, lang_flag);
    }
}

void test_and_map (void* closure, Go* go)
{
   char* flag_lang = (char*)closure;
    // Is the Go a text?;
    if (!current_text &&
	go->getMetaclass()->isSubclass(GoTextual::metaclass()))
    {
	int n;
	const char* text_lang =(const char*)go->assoc()->get(language_key, n);
	// Is the language the expected one?;
	if (strcmp(text_lang, flag_lang) == 0)
	{
	    current_text = go;
	    go->map();
	    // If the expected rectangle is already found, stop!
	    if (current_rect)
		goroot->stopRecursion (NULL);
	}
	return;
    }

    // Is the Go a highlight rectangle?;
    if (!current_rect)
    {
	int n;
	const char* rect_lang = (const char*)go->assoc()->get(selection_key, n);
	if (!rect_lang)
	    // The Go is not a "highlight rectangle";
	    return;
	
	if (strcmp(rect_lang, flag_lang) == 0)
	{
	    current_rect = go;
	    go->map();
	    // If the expected text is already found, stop!
	    if (current_text)
		goroot->stopRecursion (NULL);
	}
	return;
    }
}

