#ifdef REV_INFO
static char rcsid[] = "$Id: goHanoi.cc,v 1.7 1992/12/11 13:29:36 smadja Exp $";
#endif
/*
 * goHanoi.cc
 *
 * An example of go-animation.
 */
#include <stdio.h>
#include <ctype.h>
#include <Go/Rectangle.h>
#include <Go/GoRoot.h>
#include <Go/XGoInterf.h>
#include <Go/XInDisp.h>
#include <Go/Label.h>

static GBool use_default_interf = GFALSE;

static char goNdisk[] = "disk";

static int disk_count = 8;
static GNumber mm_step = 2.;

static const GNumber BgX = 5;
static const GNumber BgY = 5;
static const GNumber BgW = 130;
static const GNumber BgH = 90;
static const GNumber Border = 5;

static const GNumber StickH = (BgH - 2*Border) * 0.75;
static const GNumber StickW = 2;
static const GNumber StickY = BgY + BgH - Border - StickH;
inline GNumber StickCenterX (int stick) {
    return BgX + (BgW * (1+stick)) / 4.;
}
inline GNumber StickX (int stick) {
    return StickCenterX(stick) - StickW/2;
}

inline GNumber DiskH () {
    return min((GNumber)5., StickH / disk_count / 2);
}
inline GNumber DiskYDist () {
    return DiskH() / 3;
}
static GNumber DiskW (int disk) {
    GNumber minW = StickW + 2 * DiskH();		//min width for disk "0";
    GNumber maxW = (StickX(1) - StickX(0)) * (GNumber).8; //max width for "disk_count-1";
    return minW + disk * (maxW - minW) / (disk_count - 1);
}
static GNumber DiskX (int disk, int stick) {
    return StickCenterX(stick) - DiskW(disk)/2;
}
static GNumber DiskY (int position_on_stick) {
    GNumber maxY = StickY + StickH - DiskH(); //for position 0;
    return maxY - position_on_stick * (DiskH() + DiskYDist());
}


static Go** sticks[3];
GoRoot* root;


static int FreePositionOnStick (int stick)
{
    for (int i = 0; i < disk_count; i++)
	if (sticks[stick][i] == NULL)
	    return i;
    return disk_count;
}

static void Move (int src, int dst)
{
    if (!use_default_interf)
    {
	// handle user input if any;
	while (GInputDispatcher()->eventReady())
	    GInputDispatcher()->handleNextEvent();
    }
    int src_pos = FreePositionOnStick(src) - 1;
    int dst_pos = FreePositionOnStick(dst);
    Go* go = sticks[src][src_pos];
    GCoordSys cs(go->coordsys());

    //move disk up:
    GNumber dest_y = BgY + Border;
    do
    {
	cs.O.y -= mm_step;
	if (cs.O.y <= dest_y)
	    cs.O.y = dest_y;
	go->setCoordsys(cs);
	root->flush();
    } while (cs.O.y > dest_y);

    //move disk left or right:
    GNumber dest_x = DiskX((int)go->assoc()->get(goNdisk), dst);
    do
    {
	if (dest_x < cs.O.x)
	{
	    cs.O.x -= mm_step;
	    if (cs.O.x <= dest_x)
		cs.O.x = dest_x;
	}
	else
	{
	    cs.O.x += mm_step;
	    if (cs.O.x >= dest_x)
		cs.O.x = dest_x;
	}
	go->setCoordsys(cs);
	root->flush();
    } while (cs.O.x != dest_x);

    //move disk down:
    dest_y = DiskY(dst_pos);
    do
    {
	cs.O.y += mm_step;
	if (cs.O.y >= dest_y)
	    cs.O.y = dest_y;
	go->setCoordsys(cs);
	root->flush();
    } while (cs.O.y < dest_y);

    // set up data:
    sticks[src][src_pos] = NULL;
    sticks[dst][dst_pos] = go;
}

static void Run (int n, int src, int dst, int tmp)
{
    if (n <= 0)
	return;
    Run(n - 1, src, tmp, dst);
    Move(src, dst);
    Run(n - 1, tmp, dst, src);
}

static void InitializeGos ()
{
    GoInterf* interf = GoDefaultGoInterf();
    if (!use_default_interf)
	interf = new XGoInterf;
    
    interf->map();
    root = GoRoot::create(interf);
}

static void CreateBackgroundAndSticksAndDisks ()
{
    GShColor* blue = root->interf()->colorFromName("blue");
    GShColor* red = root->interf()->colorFromName("red");
    GShColor* yellow = root->interf()->colorFromName("yellow");
    GShColor* green = root->interf()->colorFromName("green");

    //creation of the background:
    GoRectangle* bg = GoRectangle::create();
    bg->reparent(root);
    bg->setCoordsys(GCoordSys(GPoint(BgX, BgY),
			     GVector(BgW, 0),
			     GVector(0, BgH)));
    bg->background(GTRUE);
    bg->brush().color(blue);

    //creation of "GoHanoi" label:
    GoLabel* label = GoLabel::create();
    label->setString("GoHanoi", strlen("GoHanoi"));
    label->stringTextBrush().color(green);
    GFont* font;
    if (use_default_interf)
	font = new GFont;
    else
	font = ((XGoInterf*)root->interf())->xFontNameToFont
	    ("-adobe-times-bold-i-normal--34-240-100-100-p-170-iso8859-1");
    label->stringTextBrush().font(font);
    label->reparent(bg);
    label->move(GPoint(BgX + Border, BgY + Border));
    label->lineHeight(10.);

    //creation of disks:
    for (int stick = 0; stick < 3; stick++)
    {
	sticks[stick] = (Go**)malloc(disk_count * sizeof(Go*));
	bzero(sticks[stick], disk_count * sizeof(Go*));
    }
    for (int disk = disk_count - 1; disk >= 0; --disk)
    {
	GoRectangle* godisk = GoRectangle::create();
	godisk->reparent(bg);
	godisk->setCoordsys
	    (GCoordSys(GPoint(DiskX(disk, 0), DiskY(disk_count - 1 - disk)),
		      GVector(DiskW(disk), 0),
		      GVector(0, DiskH())));
	godisk->background(GTRUE);
	godisk->brush().color(yellow);
	godisk->assoc()->set(goNdisk, (void*)disk);
	sticks[0][disk_count - 1 - disk] = godisk;
    }

    //creation of sticks:
    for (stick = 0; stick < 3; stick++)
    {
	GoRectangle* gostick = GoRectangle::create();
	gostick->reparent(bg);
	gostick->setCoordsys(GCoordSys(GPoint(StickX(stick), StickY),
				      GVector(StickW, 0),
				      GVector(0, StickH)));
	gostick->background(GTRUE);
	gostick->brush().color(red);
    }

    blue->destroy();
    red->destroy();
    yellow->destroy();
    green->destroy();
}

static void SyntaxError (char* progname)
{
    printf("** %s usage: %s [disk-count [mm-step [default-interf]]]\n",
	   progname, progname);
    exit(1);
}

static GBool IsInteger (char* s)
{
    for (; *s; s++)
	if (!isdigit(*s))
	    return GFALSE;
    return GTRUE;
}

static GBool IsFloat (char* s)
{
    for (; *s; s++)
	if (!isdigit(*s) && *s != '.')
	    return GFALSE;
    return GTRUE;
}

static void ParseInput (int argc, char** argv)
{
    disk_count = 8;
    mm_step = 5;

    if (argc > 4)
	SyntaxError(argv[0]);

    if (argc >= 2)
    {
	if (!IsInteger(argv[1]))
	    SyntaxError(argv[0]);
	disk_count = atoi(argv[1]);

	if (argc >= 3)
	{
	    if (!IsFloat(argv[2]))
		SyntaxError(argv[0]);
	    mm_step = atof(argv[2]);

	    if (argc == 4)
	    {
		if (strcmp(argv[3], "default-interf"))
		    SyntaxError(argv[0]);
		use_default_interf = GTRUE;
	    }
	}
    }
}


void WaitForButtonPress (char* msg)
{
    root->interf()->addInputMask(GBUTTON_PRESS_MASK);
    root->flush();

    printf("press a buttonm to %s\n", msg);

    if (use_default_interf)
    {
	(void)fgetc(stdin);
	printf("ok\n"), fflush(stdout);
    }

    else
    {
	// we return from it on BUTTON_PRESS;
	// do not handle the return value;
	(void)GInputDispatcher()->runEventLoop();
    }
}

void StopEventLoopOnButtonPress(void* null, GoInterf* interf,
				const GEvent* event)
{
    if (event->type() == GBUTTON_PRESS_EVENT)
	GInputDispatcher()->stopEventLoop(NULL);
}

void InitInput()
{
    if (use_default_interf)
	return;
    
    // FIRST INIT THE GLOBAL INPUT DISPATCHER;
    
    // we use an X one not the Xtoolkit one;
    GoInterf* interf = (GoInterf*)root->interf();
    
    // the display defines the input queue;
    GInputDispatcher() =
	new XInputDispatcher(((XGoInterf*)interf)->display());
    
    // ask the input display to handle xinterf;
    // this function is specific od XInputDispatcher class;
   ((XInputDispatcher*)GInputDispatcher())->
       addXGoInterf((XGoInterf*)interf);


    // SECOND INIT event postcond;
    root->interf()->eventPostcond(StopEventLoopOnButtonPress, NULL);

    // input mask;
    interf->addInputMask(GBUTTON_PRESS_MASK);
}
    
main (int argc, char** argv)
{
    ParseInput(argc, argv);
    InitializeGos();
    CreateBackgroundAndSticksAndDisks();
    InitInput();
    WaitForButtonPress("start");
    Run(disk_count, 0, 1, 2);
    WaitForButtonPress("stop");
}
