#ifdef REV_INFO
static char rcsid[] = "$Id: constr.cc,v 1.13 1992/12/31 14:10:48 jojic Exp $";
#endif /* REV_INFO */
/*****************************************************************************
MODULE:  Examples
FILE:    constr.cc
AUTHOR:  Philippe Smadja
DATE:    Jan 92

DESCRIPTION:
  This program is an example of constaints in GoUI.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  license from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#include <Go/Init.h>
#include <Go/XGoInterf.h>
#include <Go/XInDisp.h>
#include <Go/GoRoot.h>
#include <Go/Circle.h>
#include <Go/Rectangle.h>
#include <Go/Label.h>
#include <Go/BGoUIEd.h>
#include <Go/FullSensor.h>
#include <Go/EdRectangle.h>
#include <Go/Event.h>
#include <Go/Translator.h>
#include <Go/Tools/FilePath.h>
#include <Go/Tools/Bind.h>
#include <assert.h>
#include <stdio.h>

GoRoot* goroot;

/**
FUNCTION
    circleConstraint()
DESCRIPTION
    this constraint set the RELCS (as a square) for each circle;
    The constraint is called on each "fastDraw()" in GoUI.
    It returns always GTRUE to accept the draw.
**/

static GBool circleConstraint(void* closure, const Go* /* go */ ,
			     GCoordSys* new_relcs, const GCoordSys*,
			      const GCoordSys*)
{
    assert (closure == NULL);

    GNumber i_size = new_relcs->I.size();
    GNumber j_size = new_relcs->J.size();
    new_relcs->I.x = i_size;
    new_relcs->I.y = 0;

    new_relcs->J.x = 0;
    new_relcs->J.y = j_size;
    return GTRUE;
}



// show the size of a rectangle a rectangle edition;
// I have to make a grabber;
// An to re implement the draw method;
// A subclass a GEditRectangle::edit();

class MyGrabberPgm;

// THE INTERACTOR ITSELF;

class MyGrabber : public GEditRectangle
{
  public:
    virtual void draw();
    MyGrabber(MyGrabberPgm*, const GEvent*, GoRectangle*);
    ~MyGrabber();
    
  protected:
    void Initialize();
  private:
    GoLabel* label;
};


// THE PROGRAM TO ACCES IT;
// The constructor will bind name of the program to the program;
// so sensor will be able to retrieive the program and to call;
// the strat method;
// calling the start method will return an running Interactor.
    
class MyGrabberPgm : public GEditRectanglePgm
{
  public:
    MyGrabberPgm (const char* name) : GEditRectanglePgm(name) {}
    GInteractor* start(const GEvent*, Go* go);
};

GInteractor*  MyGrabberPgm::start(const GEvent* event, Go* go)
{
    GInteractor* itor = new MyGrabber(this, event, (GoRectangle*)go);
    Initialize(itor);
    return itor;
}
				      
MyGrabber::~MyGrabber()
{
    if (label)
	label->destroy();
}

MyGrabber::MyGrabber(MyGrabberPgm* pgm, const GEvent* event,
		     GoRectangle* rectangle)
    : GEditRectangle(pgm, event, rectangle)
{
    label = NULL;
}

void MyGrabber::Initialize()
{
    label = GoLabel::create();
    label->move(GPoint(10,10));
    label->reparent(go()->root());
    label->map();

    // superclass last;
    (void)GEditRectangle::Initialize();
}


void MyGrabber::draw()
{
  GEditRectangle::draw();

    // draw the text;
    char sizes[64];
    const GoRectangle* rect = (GoRectangle*)WorkGo();
    GCoordSys abscs = WorkGo()->coordsys();
    sprintf (sizes, "%dx%d", (int)(abscs.I.size() * rect->width()+ 0.5),
	     (int)(abscs.J.size() * rect->height() + 0.5));
    label->setString(sizes, strlen(sizes));
}	  
	

// END of MyGrabber definition;


static void Exit()
{
    GInputDispatcher()->stopEventLoop(0);
}

/*
 * MAIN
 */

main (int, char**)
{
    // init the Go lib

    GoToolsInit();

    // If you want that your GoUI will be able to create any;
    // king of gos use: GoInit();
	

    // make the  interface which will have the Gos

    GoInterf* interf = new XGoInterf;

    // INIT INPUT DISPATCHER;
    GInputDispatcher() = new XInputDispatcher(((XGoInterf*)interf)->display());

    // make the global goroot;
    
    goroot = GoRoot::create();         
    goroot->interf(interf);
    interf->map();
    goroot->map();

    // load the user environment
#define GOUI_FILE "constr.goui"

    char* filename = GGetFilePath("app-files/constr", GOUI_FILE);
    if (!filename)
	GError("File %s does not exits.\n", GOUI_FILE);

    GLoadTranslatorPgms(filename);
    GFREE(filename);

    // Create the GoFullSensor referenced in the translator file;
    GoAlwaysFirstChild(GoFullSensor::create("goui", goroot, NULL, NULL, NULL),
		       GADD);

    // Patches the default edit rectangle program;
    // by binding the program with the same name as the default one;
    
    (void)new MyGrabberPgm("GoRectangle.edit");
    
    // bind exit;
    GBind ("Exit", &Exit, "Function");

    // add my user constraint on all circles;
    GoCircle::metaclass()->relcsPrecond(circleConstraint, NULL, GADD);

    // the next line is specific of the InputDispatcher for X
    ((XInputDispatcher*)GInputDispatcher())->addXGoInterf((XGoInterf*)interf);
    // end of specific;
    
    // new GoUI;
    BGoUIEditor* ed = new BGoUIEditor (goroot);

    //MAIN LOOP FOR EVENTS.

    (void)GInputDispatcher()->runEventLoop();

    goroot->destroy();
    delete interf;
    delete ed;
    delete GInputDispatcher();
}




