#ifdef REV_INFO
static char rcsid[] = "$Id: ex1.cc,v 1.2 1992/12/29 08:42:41 jacomet Exp $";
#endif
/*****************************************************************************
MODULE:  Example
FILE:    ex1.cc
AUTHOR:  Philippe Smadja.
DATE:    

DESCRIPTION:
  Create two rectangle ands make a rotation around the gravity center.
  
  This program is an simple example of a Go program.
  It does not  use the GoUI and does not handle events.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include <unistd.h>

#include <Go/XGoInterf.h>
#include <Go/Rectangle.h>
#include <Go/GoRoot.h>
#include <assert.h>

main()
{
    /**
      Xspecific XGoInterf method
      **/
    GoInterf* interf = new XGoInterf();
    /**
      end of X specific method
      **/

    /** create a GoRoot **/
    GoRoot* goroot = GoRoot::create();
    
    /** attach the interf to the goroot **/
    goroot->interf(interf);

    /** map the interface: for X the window will be mapped **/
    interf->map();


    /** Precond are not called **/
    GoRectangle* r1 = GoRectangle::create(10,20);
    assert (r1);
    r1->reparent(goroot);
    r1->move(GPoint(50,50));    /** absolute one **/
    r1->map();

    /** Precond  are called **/
    GoRectangle* r2 = GoRectangle::create(20,10);
    if (r2) {
	/**
	  r2 is created. In a real application
	  Precondition may refuse a creation of a rectangle
	  so r2 may be NULL.
	  **/
	r2->reparent(r1);
	r2->move(GPoint(2,1), r1);  /** relative to r1 coordsys **/
	r2->map();
    }

    sleep(3);  /** wait the effective map  of the window **/                    
    
    /** map the goroot **/
    goroot->map();

    /**
      at this time you do not see anything on the window
      because the  request are not flushed
      **/

    GRegion clip;
    r1->addClip(clip);    /** the clip of the r1 and his children **/
    GPoint absolute_center;
    absolute_center.x = clip.bigbox().xmin + clip.bigbox().width() / 2;
    absolute_center.y = clip.bigbox().ymin + clip.bigbox().height() / 2;
    
    /**
      the absolute GCoordSys of the object
      Note that it is a copy
      **/
    GCoordSys cs = r1->coordsys();

    /**
      Put the center relative to cs
      **/
    GPoint relative_center = cs.from(absolute_center);

    /* for ever ... */
    int usedToAvoidWarning = 1;
    while (usedToAvoidWarning){
	cs.rotate(relative_center, PI/180) ; /** 1 degree **/
	r1->setCoordsys(cs);
	goroot->flush();      /* flush the requests. Here we have
				 only one request to flush
				 **/
	interf->flush();
    }
}
