/* $Id: BGoUIEd.h,v 1.5 1992/12/30 13:31:09 jojic Exp $ */
/*****************************************************************************
MODULE:  Go User Interface
FILE:    BGoUIEd.h
AUTHOR:  Go team
DATE:    January 90

DESCRIPTION:
    

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#ifndef _BGoUIEd_h
#define _BGoUIEd_h

#include <Go/GoUISelector.h>
#include <Go/GoRoot.h>
#include <Go/Event.h>

#define BGOUI_LINE_HEIGHT_AT_CREATION_TIME 7 /* mm for font + margin */

typedef void (*GoUICreateGoPostcond)(void* closure, Go*);
class GoMetaclass;

class BGoUIEditor : public GoUISelector
{
  public:
    BGoUIEditor (GoRoot*);
   ~BGoUIEditor();

    virtual void root (GoRoot*);

    /*
     * a tool to choose a box.
     */

    void chooseBox(void (*postcond)(void* closure, GBool done, GBox&),
		   void* closure);
    
    /*
     * current parent management.
     */

    Go* currentParent () const;
    void currentParent (Go*);


    /*
     * creation
     */


    // The other method are convenient methods;
    // except for RegPolygon and Image;
    // ********************************
	
    // createPolypoint is use to creation every;
    // subclass of GoPolypoint in particular all the Curve;
    // Note that the polypoint is not instanciable;
    // This is why subclass_name MUST be NON NULL.
	
    void createPolypoint(GoUICreateGoPostcond proc,
			 void* closure,
			 const char* subclass_name);

    void createPolyline(GoUICreateGoPostcond proc = NULL,
			void* closure = NULL,
			const char* subclass_name = NULL);
	
    void createHandPolyline(GoUICreateGoPostcond proc = NULL,
			    void* closure = NULL,
			    const char* subclass_name = NULL);

    void createPolygon(GoUICreateGoPostcond proc = NULL,
		       void* closure = NULL,
		       const char* subclass_name = NULL);

    void createRectangle(GoUICreateGoPostcond proc = NULL,
			 void* closure = NULL,
			 const char* subclass_name = NULL);

    void createSquare(GoUICreateGoPostcond proc = NULL,
		      void* closure = NULL,
		      const char* subclass_name = NULL);

    void createCircle(GoUICreateGoPostcond proc = NULL,
		      void* closure = NULL,
		      const char* subclass_name = NULL);

    void createEllipse(GoUICreateGoPostcond proc = NULL,
		       void* closure = NULL,
		       const char* subclass_name = NULL);

    void createCircleArc(GoUICreateGoPostcond proc = NULL,
			 void* closure = NULL,
			 const char* subclass_name = NULL);

    void createEllipseArc(GoUICreateGoPostcond proc = NULL,
			  void* closure = NULL,
			  const char* subclass_name = NULL);

    void createImage(GNumber width, GNumber height,
		     GoUICreateGoPostcond proc = NULL,
		     void* closure = NULL,
		     const char* subclass_name = NULL);

    void createRegPolygon(int nb_edge,
			  GoUICreateGoPostcond proc = NULL,
			  void* closure = NULL,
			  const char* subclass_name = NULL);

    void createLabel(GoUICreateGoPostcond proc = NULL,
		     void* closure = NULL,
		     const char* subclass_name = NULL);

    void createSimpleText(GoUICreateGoPostcond proc = NULL,
			  void* closure = NULL,
			  const char* subclass_name = NULL);

    void createSimpleFormText(GoUICreateGoPostcond proc = NULL,
			      void* closure = NULL,
			      const char* subclass_name = NULL);

    /*
     * action on selected Objects
     * and misc methods;
     */

    void actOnAllSelectedSubtrees(const char* program,
				  const GEvent* = NULL,
				  void (*)(void*,int state_result) = NULL,
				  void* closure = NULL);
    void actOnAllSelectedGos(const char* program,
			     const GEvent* = NULL,
			     void (*)(void*, int state_result) = NULL,
			     void* closure= NULL);
    
  private:

    void         _create_edit_programs();
    
    // general method to create a Go;
    // the function chooses a point, creates the go;
    // reparent it onto the current parent;
    // and run the edit program (if non NULL);

    // It is not public because the closure;
    // It interpreted inside the internall callback;
    
    void _create_go(GoUICreateGoPostcond proc,
		    void* closure,
		    const GoMetaclass* mclass,
		    const char* subclass_name,
		    const char* interactor_name = "edit");

    const char* root_sensor_name;
    Go* current_parent;
    int max_go_count;
    int fast_draw_depth;

    friend void _bgouied_root_destroy_proc(BGoUIEditor*, const GoRoot*);
    friend void _bgouied_go_removed_proc (void*, Go*, Go*);
};

// a convenient function;
extern  void GoTextualMakeSensitive(const char* program = "textEdition");

#endif /* _BGoUIEd_h */


