/* $Id: Beta.h,v 1.3 1992/12/11 08:34:22 jojic Exp $ */
/*****************************************************************************
MODULE:  Geometric
FILE:    Beta.h
AUTHOR:  Gilles Benati
DATE:    September 20th, 1991

DESCRIPTION:

	GoBetaSpline is the graphic object class for Beta Splines. Beta Spline
  is an approximating spline with tension and bias. For a discussion about
  tension see GoCubic.h header. Bias refers to the symmetry of the curve near a
  control point. A highly biased curve will be flatter on one side of a knot
  than on the other. The tension and bias of Beta Spline can be varied widely.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#ifndef _GoBetaSpline_h
#define _GoBetaSpline_h

#include <Go/Cubic.h>
#include <Go/mBeta.h>

class GoBetaSpline : public GoCubic {
    friend class GoBetaSplineMetaclass;
    
  public:

    GoBetaSpline();
    GoBetaSpline(int nb, const GPoint* points);

    // pre/post conditions;
    void biasPrecond(GBool func(void*, const GoBetaSpline*, GNumber*),
		     void* closure, GAddOrRemove = GADD);
    void biasPostcond(void func(void*, GoBetaSpline*),
		      void* closure, GAddOrRemove = GADD);

    GNumber bias() const { return _bias; }
    GBool   bias(GNumber);

    static GoBetaSplineMetaclass* metaclass();
    static GoBetaSpline* create(int nb, const GPoint *points = NULL);
    static GoBetaSpline* create() {
	return (GoBetaSpline*) Go::Create(GoBetaSpline::metaclass());
    }

    static int	 save(const GoBetaSpline*, GoStream*);
    static int	 restore(GoBetaSpline**, GoStream*);   // creates + restores;
    static int	 readParams(GoBetaSpline*, GoStream*); // restores only;

  protected:


    virtual void m_dup(const Go* src, Go*, Go*, GBool, GBool);
//  virtual GBool m_contains(const GPoint&, GNumber maxDist = 0) const;
//  virtual void m_add_clip(GRegion&) const;
//  virtual void m_opaque_clip(GRegion&) const;
//  virtual void m_draw(const GRegion&) const;
//  virtual void m_wired_draw(const GCoordSys&, const GPen&, GQuality) const;

    virtual void initCoeffMatrix ();

  private:

    GNumber _bias;

    // pre/post condition flags;
    GBool _has_bias_precond  :1;
    GBool _has_bias_postcond :1;

    // call pre/post conditions;
    GBool _call_bias_precond(GNumber*) const;
    void _call_bias_postcond();

    // call metaclass pre/post conditions;
    GBool _call_class_bias_precond(GNumber*) const;
    void _call_class_bias_postcond();

    virtual GoMetaclass* m_metaclass() const;
    virtual void         m_check() const;

    GBool _set_bias(GNumber);
};

#endif /* _GoBetaSpline_h */
