/* $Id: Bezier.h,v 1.3 1992/12/11 08:34:25 jojic Exp $ */
/*****************************************************************************
MODULE: Geometric
FILE:   Bezier.h
AUTHOR: Gilles Benati
DATE:   September 20th, 1991


DESCRIPTION :

        This is one of the basic curves. A bezier curve of n-th order is
    characterised by n+1 control points. In general the curve does not pass
    through all control points, but it passses through the two end points,
    and is tangent at the end points to the corresponding edge of the
    polygon of the control points. The curve will always lie within the
    convex hull of the control points. The curve does not provide localized
    control: moving a control point will change the shape of every part of
    the curve. GoBezier implements the n-th order Bezier and allows the user
    to interactively manipulate the control points and observe the variation
    in the curve shape. As the GoBezier inherits behaviour from GoPolypoint,
    see the GoPolypoint documentation for interactive manipulation of control
    points.
    

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#ifndef _GoBezier_h
#define _GoBezier_h

#include <Go/Curve.h>
#include <Go/mBezier.h>


class GoBezier : public GoCurve {
    friend class GoBezierMetaclass;

  public:

    static GoBezierMetaclass* metaclass();
    static GoBezier* create(int nb, const GPoint *points);
    static GoBezier* create() {
	return (GoBezier*) Go::Create(GoBezier::metaclass());
    }

    GoBezier() : GoCurve(0, NULL) { }
    GoBezier(int nb, const GPoint* points) : GoCurve(nb, points) { }

    static int	 save(const GoBezier*, GoStream*);
    static int	 restore(GoBezier**, GoStream*);   // creates + restores;
    static int	 readParams(GoBezier* bezier, GoStream* gostream) {
	return GoCurve::readParams(bezier, gostream); // restores only;
    }

  protected:


//  virtual void m_dup(const Go* src, Go*, Go*, GBool, GBool);
//  virtual GBool m_contains(const GPoint&, GNumber maxDist = 0) const;
//  virtual void m_add_clip(GRegion&) const;
//  virtual void m_opaque_clip(GRegion&) const;
    virtual void m_draw(const GRegion&) const;
//  virtual void m_wired_draw(const GCoordSys&, const GPen&, GQuality) const;

  private:
    
    virtual GoMetaclass* m_metaclass() const;
    virtual void         m_check() const;

    virtual GPoint* getCurvePoints(int& np, int nb_ctlr, const GPoint* ctrl) const;

    GNumber blend(int i, GNumber u, int nb_ctrl) const;
};



#endif /* _GoBezier_h */
