/* $Id: Cardinal.h,v 1.3 1992/12/11 08:34:27 jojic Exp $ */
/*****************************************************************************
MODULE:  Geometric
FILE:    Cardinal.h
AUTHOR:  Gilles Benati
DATE:    September 20th, 1991

DESCRIPTION:

	This is the graphic object for Cardinal Splines. Cardinal Spline is an
  interploating  spline (i.e.: the curve passes through the control points) with
  tension. For a discussion about tension see GoCubic.h header. The tension of
  a Cardinal spline varies betwwen 0 and 1. A Cardinal Spline with tension 1 is
  a Catmull Rom Spline.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#ifndef _GoCardinal_h
#define _GoCardinal_h

#include <Go/Cubic.h>
#include <Go/mCardinal.h>

class GoCardinal : public GoCubic {
    friend class GoCardinalMetaclass;

  public:

    GoCardinal(int nb = 0, const GPoint* points = NULL);

    static GoCardinalMetaclass* metaclass();
    static GoCardinal* create(int nb, const GPoint *points = NULL);
    static GoCardinal* create() {
	return (GoCardinal*) Go::Create(GoCardinal::metaclass());
    }

    static int	 save(const GoCardinal*, GoStream*);
    static int	 restore(GoCardinal**, GoStream*);   // creates + restores;
    static int	 readParams(GoCardinal* cardinal, GoStream* gostream) {
	return GoCubic::readParams(cardinal, gostream); // restores only;
    }

  protected:
    virtual void initCoeffMatrix ();

  private:

    virtual GoMetaclass* m_metaclass() const;
    virtual void         m_check() const;
};

#endif /* _GoCardinal_h */



