/* $Id: Circle.h,v 1.3 1992/12/11 08:34:29 jojic Exp $ */
/*****************************************************************************
MODULE:  Graphic Objects
FILE:    Circle.h
DATE:    Feb 1991

DESCRIPTION:
    This is one of the basic graphic objects.
    It simply draws the arc between angle1 and angle2 that is inside
    the OBJECT relative box ((0,0),(1,1)).

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
*****************************************************************************/
#ifndef _GoCircle_h
#define _GoCircle_h

#include <Go/Ellipse.h>
#include <Go/mCircle.h>

class GoCircle : public GoEllipse {
    friend class GoCircleMetaclass;
    
  public:
    
    static GoCircleMetaclass* metaclass();
    static GoCircle* create(GNumber radius);
    static GoCircle* create() {
	return (GoCircle*) Go::Create(GoCircle::metaclass());
    }

    GoCircle(GNumber radius = 1.) : GoEllipse(radius, radius) { }

    virtual GBool radiusX(GNumber radius);
    virtual GBool radiusY(GNumber radius);
    virtual GBool reset(GNumber radius, GNumber = 0., GNumber = 0., GNumber = 0.);

    GBool         radius(GNumber radius) { return radiusX(radius); }
    GNumber       radius()  const        { return GoArc::radiusX(); }
    
    static int	 save(const GoCircle*, GoStream*);
    static int	 restore(GoCircle**, GoStream*);   // creates + restores;
    static int	 readParams(GoCircle*, GoStream*); // restores only;

  protected:


    virtual GBool m_contains(const GPoint&, GNumber maxDist = 0) const;
    virtual void m_add_clip(GRegion&) const;
    virtual void m_add_opaque_clip(GRegion&) const;
    virtual void m_draw(const GRegion&) const;
    virtual void m_wired_draw(const GCoordSys&, const GPen&, GQuality) const;
//  virtual void m_dup(const Go* src, Go*, Go*,GBool, GBool);
    
  private:
    virtual GoMetaclass* m_metaclass() const;
    virtual void         m_check() const;
};

#endif /* _GoCircle_h */
