/* $Id: ClipSensor.h,v 1.1 1992/12/10 15:37:11 jojic Exp $ */
/*****************************************************************************
MODULE: Graphic Objects
FILE:   ClipSensor.h
AUTHOR: Philippe Smadja
DATE:   Dec 92

DESCRIPTION :

   The m_contains methos will return OK if the point is inside
   the clip ( bounding box) of its parent;

COPYRIGHT :    
  Copyright (c) 1992 Bull S.A.
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull S.A. may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull S.A..
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull S.A. under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull S.A..

  Bull S.A. assumes no responsibility for the use or inability to use
  any of its software. The software is provided "AS IS" without warranty
  of any kind, and Bull S.A. expressly disclaims all implied warranties,
  including but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull S.A. be liable for any special, incidental,
  indirect or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
*****************************************************************************/
#ifndef _ClipSensor_h
#define _ClipSensor_h

#include <Go/Sensor.h>

class GoClipSensorMetaclass;


/*
 *----------------------
 * The GoClipSensor class
 *----------------------
 */

class GoClipSensor : public GoSensor
{
  public:

    /* mandatory static methods: */

    static GoClipSensorMetaclass* metaclass();
    static GoClipSensor* create ();
    static GoClipSensor* create
	(const char* name, Go* parent, const GEvent* start_event = NULL,
	 void (*)(void* cl, GoSensor*) = NULL, void* closure = NULL);

    static int save (const GoClipSensor*, GoStream*);
    static int restore (GoClipSensor**, GoStream*);   // creates + restores;
    static int readParams (GoClipSensor*, GoStream*); // restores only;


  protected:

    /* specific protected methods */

    GoClipSensor ();

    /* mandatory protected virtual methods: */

    ~GoClipSensor ();
    virtual GBool m_contains (const GPoint&, GNumber = 0) const;
    virtual void m_dup (const Go*, Go*, Go*, GBool, GBool);

  private:

    /* specific private methods: */


    /* mandatory private virtual methods */

    virtual GoMetaclass* m_metaclass() const;

    /* friends: */

    friend class GoClipSensorMetaclass;
};



/*
 *-------------------------------
 * The GoClipSensorMetaclass class
 *-------------------------------
 */

class GoClipSensorMetaclass : public GoSensorMetaclass
{
  public:
    /* pre- and post- conditions: */

  protected:

             GoClipSensorMetaclass ();  //to get the GoClipSensorMetaclass use:
    virtual ~GoClipSensorMetaclass ();  //        GoClipSensor::metaclass()

  private:

    /* mandatory private virtual methods: */

    virtual const char*       m_classname () const;
    virtual GoMetaclass*      m_superclass () const;
    virtual const GActionRec* m_actions () const;
    virtual Go*               m_create () const;

    /* private data */

    /* friends */

    friend class GoClipSensor;
};



/*
 *----------------
 * inline methods
 *----------------
 */


inline GoClipSensor* GoClipSensor::create ()
{
    return (GoClipSensor*) Go::Create(GoClipSensor::metaclass());
}

#endif /* _ClipSensor_h */
