/* $Id: EdAsRect.h,v 1.2 1992/12/22 13:05:25 smadja Exp $ */
/*****************************************************************************
MODULE:  BGo  (User Interface)
FILE:    EdRect.h
AUTHOR:  Philippe Smadja
DATE:    July 92

DESCRIPTION:

  This class compute rectangle parameter  in interaction.
  It does not inherit behavior from any classes.
	 
COPYRIGHT:
  Copyright (c) 1990, 1991, 1992 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.

*****************************************************************************/
#ifndef _GEdAsRect_H
#define _GEdAsRect_H

#include <Go/Tools/Number.h>
#include <Go/Tools/Point.h>
#include <Go/Tools/CoordSys.h>


/*
 * values for rectangle manipulation
 */

#define G_NO_GRAVITY		(1<<0)
#define G_MOVE_SOUTH		(1<<1)
#define G_MOVE_EAST		(1<<2)
#define G_MOVE_NORTH		(1<<3)
#define G_MOVE_WEST		(1<<4)
#define G_OPPOSITE_GRAVITY	(1<<5)
#define G_AS_SQUARE		(1<<6)

/*
 * Note that the following "bitwise or" values are also valid:
 *   G_NO_GRAVITY | G_AS_SQUARE
 *
 *   G_MOVE_SOUTH | G_MOVE_EAST
 *   G_MOVE_SOUTH | G_MOVE_WEST
 *   G_MOVE_NORTH | G_MOVE_EAST
 *   G_MOVE_NORTH | G_MOVE_WEST
 */

extern int GRectangleGravityToCursorIdent(int gravity);

/*
 *-------------------------------------------------------
 * The GEditAsRectangle class
 *-------------------------------------------------------
 */

class GEditAsRectangle
{
  public:

    GEditAsRectangle (GNumber width, GNumber height,
		      int gravity);
    
    virtual ~GEditAsRectangle ();

    virtual void  compute (const GPoint& point);

    // read resulting values;
    
    const GPoint& topLeft() const ;
    GNumber width() const ; 
    GNumber height() const;
    int gravity() const ;
    
  protected:
    void          NoGravity(const GPoint&);
    void          OppositeGravity(const GPoint&);
    
  private:
    int       _mask;     // cf gravity method the GEditRectanglePgm class;
    GPoint    _top_left;
    GNumber   _width;
    GNumber   _height;
    GNumber   _initial_width;
    GNumber   _initial_height;
};

inline const GPoint& GEditAsRectangle::topLeft() const { return _top_left; } ;
inline GNumber GEditAsRectangle::width() const { return _width; } ;
inline GNumber GEditAsRectangle::height() const { return _height; } ;
inline int     GEditAsRectangle::gravity() const { return _mask; } ;
    
#endif /* _GEdAsRect_H */


