/* $Id: EdGeometric.h,v 1.1 1992/12/10 15:37:15 jojic Exp $ */
/*****************************************************************************
MODULE:  BGo  (User Interface)
FILE:    EdGeometric.h
AUTHOR:  Philippe Smadja
DATE:    July 92

DESCRIPTION:
 * Its purpose is to unmap / outline and background of the
 * geometric during the edition;

COPYRIGHT:
  Copyright (c) 1990, 1991, 1992 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software

  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.

*****************************************************************************/
#ifndef _GEdGeometric_H
#define _GEdGeometric_H

#include <Go/Grabber.h>
#include <Go/Tools/GCdef.h>
#include <Go/Tools/Pen.h>

/**
  *-------------------------------------------------------
  * The GEditGeometricPgm class 
  * NOT INSTANCIABLE:
  * It handles drawing quality and Pen of editing;
  * for the superclass.
  *-------------------------------------------------------
  */

class GEditGeometricPgm : public GGrabberPgm
{
  public:
    
    // my public methods;
    
    // read pgm;
    GQuality drawingQuality() const { return _drawing_quality; };
    GPen&    drawingPen() const { return ((GEditGeometricPgm*)this)->
				      _drawing_pen; };

    // reset pgm;
    void     drawingQuality(GQuality quality) { _drawing_quality = quality;};

  protected:
    GEditGeometricPgm (const char*);

  private:
    GQuality  _drawing_quality;
    GPen      _drawing_pen;
};



/*
 *-------------------------------------------------------
 * The GEditGeometric class:
 *-------------------------------------------------------
 */

class GoGeometric;

class GEditGeometric : public GGrabber
{
  public:
    static void*  interactionKey();

 protected:

    GEditGeometric (GEditGeometricPgm*, const GEvent*, GoGeometric*);
    virtual ~GEditGeometric ();
    virtual void  Initialize ();
    GoGeometric*  WorkGo() const;
    GQuality      DrawingQuality() const;
	
  private:
    GoGeometric* _hide_go;
    GoGeometric* _work_go;
    GQuality     _drawing_quality; // copied from program;
};

inline GoGeometric* GEditGeometric::WorkGo() const
{
    return _work_go;
}

inline GQuality GEditGeometric::DrawingQuality() const
{
    return _drawing_quality;
}


#endif /* _GEdGeometric_H */


