/* $Id: EdImage.h,v 1.1 1992/12/10 15:37:18 jojic Exp $ */
/*****************************************************************************
MODULE:  BGo  (User Interface)
FILE:    EdImage.h
AUTHOR:  Philippe Smadja
DATE:    July 92

DESCRIPTION:

  GEditImage is a subclass of GEditGrabber
  
  It uses the superclass to compute interaction. It uses a image to
  initialize value at the start time and reset image (doit method).
	 
COPYRIGHT:
  Copyright (c) 1990, 1991, 1992 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.

*****************************************************************************/
#ifndef _GEdImage_H
#define _GEdImage_H

#include <Go/EdAsRect.h>
#include <Go/Grabber.h>
#include <Go/Tools/Pen.h>

/*
  *-------------------------------------------------------
  * The GEditImagePgm class
  *-------------------------------------------------------
  */

class GEditImagePgm : public GGrabberPgm
{
  public:
    static GEditImagePgm* create (); // my name is "GoImage.edit"

    // superclasses virtual method;
    virtual GInteractor* start (const GEvent*, Go*);

    // read pgm;
    int gravity() const { return _gravity; } ;
    GPen&    pen() const { return ((GEditImagePgm*)this)->_pen; };

    // reset pgm;
    void gravity(int gr);

  protected:
    GEditImagePgm (const char*);

  private:
    int _gravity;
    GPen _pen;
};


/*
 *-------------------------------------------------------
 * The GEditImage class
 *-------------------------------------------------------
 */

class GoImage;
class GEvent;


class GEditImage : public GGrabber
{
  public:

    static void*   interactionKey();
    GEditImage (GEditImagePgm*, const GEvent*, GoImage*);
    virtual ~GEditImage ();

    virtual void  draw ();
    virtual void  doit ();
    virtual void  compute (const GEvent*);  // superclass first;
	
  protected:
    virtual void Initialize();
    const GPoint& TopLeft() const ;
    GNumber Width() const ; 
    GNumber Height() const;

  private:
    GEditAsRectangle* _computer;
    GoImage*  _work_image;
    GPen* _pen;    // it will be a duplication of the pgm one;
};


inline const GPoint& GEditImage::TopLeft() const
{
    return _computer->topLeft();
}

inline GNumber GEditImage::Width() const
{
    return _computer->width();
}

inline GNumber GEditImage::Height() const
{
    return _computer->height();
}

#endif /* _GEdImage_H */


