/* $Id: EdPolypoint.h,v 1.1 1992/12/10 15:37:25 jojic Exp $ */
/*****************************************************************************
MODULE:  BGo  (User Interface)
FILE:    EdPolypoint.h
AUTHOR:  Philippe Smadja
DATE:    July 92

DESCRIPTION:

  GEditRectangle is a subclass of GGrabber
  
  GRectabgleGrabber is used for the programming of user interactions ala
  ``elastic rectangle''.
  Such interactions follow the same scheme: The Grabber one.
	- drawings,
	- wait for a motion event,
	- clear the previous drawings,
	- re-computation of data values (using the new mouse position),

	This loop stops on some specific user events, which may generate a succes
	or a cancellation of the interaction.

	Every action from the list above, correspond to a virtual method of
	the GGrabber class. The virtual method ``handle(event)''
	executes previous actions.

  The GGrabber::handle method is fine for GGrabber so we have only to implement
  the following methods:
         draw();
	 compute();
	 doIt();
	 isDoItEvent();
	 isCancelEvent();
	 
COPYRIGHT:
  Copyright (c) 1990, 1991, 1992 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software

  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.

*****************************************************************************/
#ifndef _GEdPolypoint_H
#define _GEdPolypoint_H

#include <Go/EdGeometric.h>
#include <Go/Event.h>
#include <Go/Tools/GCdef.h>
#include <Go/Tools/Point.h>
#include <Go/Tools/CoordSys.h>
#include <Go/Tools/Array.h>
#include <Go/Tools/Pen.h>


/*
 * mode of editing for GEditPolypoint interactor;
 */

#define GEDIT_POLYPOINT 1
#define GMOVE_POINT 2




/**
  *-------------------------------------------------------
  * The GMovePolypointPgm class.
  *-------------------------------------------------------
  */

class GMovePolypointPgm : public GEditGeometricPgm
{
  public:
    static GMovePolypointPgm* create (); 

    // superclasses virtual method;
    virtual GInteractor* start (const GEvent*, Go*);

  protected:
    GMovePolypointPgm(const char*);
};


/**
  *-------------------------------------------------------
  * The GEditPolypointPgm class.
  *-------------------------------------------------------
  */

class GEditPolypointPgm : public GEditGeometricPgm
{
  public:
    static GEditPolypointPgm* create (); 

    // superclasses virtual method;
    virtual GInteractor* start (const GEvent*, Go*);

    // my public methods;
    
    // read pgm;
    GButton  addPointButton() const { return _add_point_button; };

    // reset pgm;
    void     addPointButton(GButton button) { _add_point_button = button;};

  protected:
    
    GEditPolypointPgm (const char*);
    virtual GEventMask   inputMask() const;

  private:
    GButton   _add_point_button;
};

class GoPolypoint;
class GEvent;

/*
 *-------------------------------------------------------
 * The GEditPolypoint class:
 * IT IMPLEMENTS BOTH EDITION AND MOVE of a single point.
 *-------------------------------------------------------
 */

class GEditPolypoint : public GEditGeometric
{
  public:

    GEditPolypoint (GEditGeometricPgm*, const GEvent*, GoPolypoint*,
		    int mode, GButton addpointbutton);
    virtual ~GEditPolypoint ();

    virtual void  draw();
    virtual void  compute (const GEvent*);  // superclass first;
    virtual void  doit ();
    virtual GBool isDoitEvent(const GEvent*) const;
    virtual GBool handle(const GEvent*);
    
    int      nbPoints() const;
    const GPoint* points() const;
    int     currentPointIndex() const;

  protected:

    virtual void  Initialize ();

    // read private data;
    
    int           NbPoints() const ;
    const GPoint* Points()  const ;
    int           Index()   const;
    GBool         NewPoint() const;
    GoPolypoint*  WorkGo() const;

  private:
    int       _mode;      // GEDIT_POLYPOINT, GMOVE_POINT.
    GButton   _add_point_button;
    int       _index;     // index for point  which follows the mouse;
    GBool     _newPoint;
};


inline int GEditPolypoint::Index() const
{
    return _index;
}

inline GBool GEditPolypoint::NewPoint() const
{
    return _newPoint;
}

inline GoPolypoint* GEditPolypoint::WorkGo() const
{
    return (GoPolypoint*) GEditGeometric::WorkGo();
}

inline int GEditPolypoint::currentPointIndex() const
{
	return Index();
}

inline const GPoint* GEditPolypoint::points() const
{
	return Points();
}

inline int GEditPolypoint::nbPoints() const
{
	return NbPoints();
}

#endif /* _GEdPolypoint_H */


