/* $Id: EdRectangle.h,v 1.1 1992/12/10 15:37:27 jojic Exp $ */
/*****************************************************************************
MODULE:  BGo  (User Interface)
FILE:    EdRectangle.h
AUTHOR:  Philippe Smadja
DATE:    July 92

DESCRIPTION:

  We define 2 programs;
  GEditRectanglePgm and GEditSquarePgm;
  
  GEditRectangle is a subclass of GEditGeometric
  It is used both by the 2 programs;
  
  It uses the superclass to compute interaction. It uses a rectangle to
  initialize value at the start time and reset rectangle (doit method).
	 
COPYRIGHT:
  Copyright (c) 1990, 1991, 1992 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.

*****************************************************************************/
#ifndef _GEdRect_H
#define _GEdRect_H

#include <Go/EdAsRect.h>
#include <Go/EdGeometric.h>

/*
  *-------------------------------------------------------
  * The GEditRectanglePgm class
  *-------------------------------------------------------
  */

class GEditRectanglePgm : public GEditGeometricPgm
{
  public:
    static GEditRectanglePgm* create (); // my name is "GoRectangle.edit"

    // superclasses virtual method;
    virtual GInteractor* start (const GEvent*, Go*);

    // read the gravity;
    int gravity() const { return _gravity; } ;

    // program the wanted gravity;
    void gravity(int gr) ;

  protected:
    GEditRectanglePgm (const char*);

  private:
    int _gravity;
};


/*
 *-------------------------------------------------------
 * GEditRectanglePgm class
 *-------------------------------------------------------
 */

class GEditSquarePgm : public GEditRectanglePgm 
{
  public:
    static GEditSquarePgm* create (); // my name is "GoRectangle.squareEdit"

    // superclasses virtual method;
    virtual GInteractor* start (const GEvent*, Go*);

  protected:
    GEditSquarePgm (const char*);
};


/*
 *-------------------------------------------------------
 * The GEditRectangle class
 *-------------------------------------------------------
 */

class GoRectangle;
class GEvent;


class GEditRectangle : public GEditGeometric
{
  public:

    GEditRectangle (GEditRectanglePgm*, const GEvent*, GoRectangle*);
    virtual ~GEditRectangle ();

    virtual void  draw ();
    virtual void  doit ();
    virtual void  compute (const GEvent*);  // superclass first;
	
  protected:
    GQuality DrawingQuality();
    const GPoint& TopLeft() const ;
    GNumber Width() const ; 
    GNumber Height() const;

  private:
    GEditAsRectangle* _computer;
};


inline const GPoint& GEditRectangle::TopLeft() const
{
    return _computer->topLeft();
}

inline GNumber GEditRectangle::Width() const
{
    return _computer->width();
}

inline GNumber GEditRectangle::Height() const
{
    return _computer->height();
}

#endif /* _GEdRect_H */


