/* $Id: Event.h,v 1.4 1992/12/11 08:34:36 jojic Exp $ */
/*****************************************************************************
MODULE:  Go User Interface
FILE:    Event.h
AUTHOR:  Olivier Jojic
DATE:    Jan 90

DESCRIPTION:

...


COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#ifndef _Event_h
#define _Event_h

#include <Go/Tools/Point.h>
#include <Go/Tools/Region.h>
#include <Go/EventDefs.h>

class GoInterf;
class GoSensor;

/*
 * GEvent class definition
 * ------------------------
 */
class GEvent
{
  public:
    virtual ~GEvent() {}
    
    inline GoInterf* origin () const { return (GoInterf*)_origin; }
    inline GEventType type() const { return _type; }
    inline GTimeStamp time() const { return _time; }
    
    virtual char character() const;
const char* string() const; //A RETIRER
    virtual const char* message() const;
    virtual GButton button() const;
    virtual GFunctionKey functionKey() const;
    virtual ulong modifiers() const;
    virtual GPoint position() const;
    virtual const GRegion& region() const;
    virtual const GoSensor* sensor() const;

  private:
    GEventType _type;
    const GoInterf* _origin ;
    GTimeStamp _time;           // time of event delivery: 0 if unknown;
    
  protected:
    inline GEvent(GEventType tp, const GoInterf* gointerf = NULL, GTimeStamp time = 0)
                  { _type = tp; _origin = gointerf; _time = time; }

};    

/*
 * (sub)GButtonEvent class definitions
 * ------------------------------------
 */
class GButtonEvent : public GEvent
{
  public:
    virtual GButton button() const;
    virtual ulong modifiers() const;
    virtual GPoint position() const;

  protected:
    GButton _button;
    ulong   _modifiers;
    GPoint  _pos;

    GButtonEvent(GEventType type, GButton button, ulong modmask,
		 const GPoint& pos, const GoInterf* origin = NULL, GTimeStamp time = 0);
};

/**/
class GButtonPressEvent : public GButtonEvent
{
  public:
    GButtonPressEvent(GButton button, ulong modmask, const GPoint& pos,
		      const GoInterf* origin = NULL, GTimeStamp time = 0);
};

class GButtonReleaseEvent : public GButtonEvent
{
  public:
    GButtonReleaseEvent(GButton button, ulong modmask, const GPoint& pos,
			 const GoInterf* origin = NULL, GTimeStamp time = 0);
};

class GDoubleClickEvent : public GButtonEvent
{
  public:
    GDoubleClickEvent(GButton button, ulong modmask, const GPoint& pos,
		      const GoInterf* origin = NULL, GTimeStamp time = 0);
};

class GTripleClickEvent : public GButtonEvent
{
  public:
    GTripleClickEvent(GButton button, ulong modmask, const GPoint& pos,
		      const GoInterf* origin = NULL, GTimeStamp time = 0);
};

/*
 * GMessageEvent definitions
 * ----------------------------------------------
 */
class GMessageEvent : public GEvent
{
  public:
    GMessageEvent(const char* message, ulong modmask, const GPoint& pos,
		 const GoInterf* origin =  NULL, GTimeStamp time = 0);
    ~GMessageEvent();

    virtual const char* message() const;
    virtual ulong modifiers() const;
    virtual GPoint position() const;

  private:
    char* _string;
    ulong _modifiers;
    GPoint _pos;
};

/*
 * GCharEventclass definitions
 * ---------------------------
 */
class GCharEvent : public GEvent
{
  public:
    GCharEvent(char c, ulong modmask, const GPoint& pos,
	       const GoInterf* origin = NULL, GTimeStamp time = 0);

    virtual char  character() const;
    virtual ulong modifiers() const;
    virtual GPoint position() const;

  private:
    char  _char;
    ulong _modifiers;
    GPoint _pos;
};
    
/*
 * GFunctionKey class definition
 * ------------------------------
 */
class GFunctionKeyEvent : public GEvent
{
  public:
    GFunctionKeyEvent (GFunctionKey funkey, ulong modmask, const GPoint& pos,
		       const GoInterf* gointerf = NULL, GTimeStamp time = 0);

    virtual GFunctionKey functionKey() const;
    virtual ulong modifiers() const;
    virtual GPoint position() const;

  private:
    GFunctionKey _functionKey;
    ulong        _modifiers;
    GPoint       _pos;
};

/*
 * GExposeEvent class definition
 * ------------------------------
 */
class GExposeEvent : public GEvent
{
  public:
    GExposeEvent (const GRegion& region, const GoInterf* origin = NULL, GTimeStamp time = 0);
    ~GExposeEvent ();
    virtual const GRegion& region() const;

  private:
    GRegion* _region;
};

/*
 * GMotionEvent class definition
 * ------------------------------
 */
class GMotionEvent : public GEvent
{
  public:
    GMotionEvent (const GPoint& p, const GoInterf* origin = NULL, GTimeStamp time = 0);
    virtual GPoint position() const;

  private:
    GPoint _pos;
};

/*
 * GEnterSensorEvent and GLeaveSensorEvent class definitions
 * -----------------------------------------------------------
 */
class GEnterSensorEvent : public GEvent
{
  public:
    GEnterSensorEvent (const GoSensor* sensor,
		       const GoInterf* gointerf = NULL, GTimeStamp time = 0);
    virtual const GoSensor* sensor() const;

  private:
    const GoSensor* _sensor;
};

class GLeaveSensorEvent : public GEvent
{
  public:
    GLeaveSensorEvent (const GoSensor* sensor,
		       const GoInterf* gointerf = NULL, GTimeStamp time = 0);
    virtual const GoSensor* sensor() const;

  private:
    const GoSensor* _sensor;
};


/*
 * GFocusInEvent and GFocusOutEvent class definitions
 * --------------------------------------------------
 */

class GFocusInEvent : public GEvent
{
  public:
    GFocusInEvent (const GoSensor* sensor,
		   const GoInterf* gointerf = NULL, GTimeStamp time = 0);
    virtual const GoSensor* sensor() const;

  private:
    const GoSensor* _sensor;
};

class GFocusOutEvent : public GEvent
{
  public:
    GFocusOutEvent (const GoSensor* sensor,
		    const GoInterf* gointerf = NULL, GTimeStamp time = 0);
    virtual const GoSensor* sensor() const;

  private:
    const GoSensor* _sensor;
};

#endif /* _Event_h */
