/* $Id: EventDefs.h,v 1.2 1992/12/11 08:34:38 jojic Exp $ */
/*****************************************************************************
MODULE:  Go User Interface
FILE:    EventDefs.h
AUTHOR:  Philippe Smadja
DATE:    Jul 92

DESCRIPTION:
   define for Event. This file is include both by <Go/Event.h> and <Go/cEvent.h>

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  license from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#ifndef _EventDefs_h
#define _EventDefs_h

/*
 * Time of an event: the origin and unit is GoInterf
 * implementation dependant
 */
typedef ulong GTimeStamp;

/*
 * event types
 * -----------
 */
enum GEventType
{
    GBUTTON_PRESS_EVENT,
    GBUTTON_RELEASE_EVENT,
    GDOUBLE_CLICK_EVENT,
    GTRIPLE_CLICK_EVENT,
    GCHAR_EVENT,
    GMESSAGE_EVENT,
    GFUNCTION_KEY_EVENT,
    GEXPOSE_EVENT,
    GMOTION_EVENT,
    GENTER_SENSOR_EVENT,
    GLEAVE_SENSOR_EVENT,
    GFOCUS_IN_EVENT,
    GFOCUS_OUT_EVENT,
    GMAX_EVENT /* must be the last */
};

/*
 * Event mask
 * ----------
 */
#define  GBUTTON_PRESS_MASK_INDEX	0
#define  GBUTTON_RELEASE_MASK_INDEX     1
#define  GDOUBLE_CLICK_MASK_INDEX       2
#define  GTRIPLE_CLICK_MASK_INDEX       3
#define  GCHAR_MASK_INDEX               4
#define  GMESSAGE_MASK_INDEX		5
#define  GFUNCTION_KEY_MASK_INDEX	6
#define  GEXPOSE_MASK_INDEX		7
#define  GMOTION_MASK_INDEX             8
#define  GENTER_SENSOR_MASK_INDEX       9
#define  GLEAVE_SENSOR_MASK_INDEX       10
#define  GFOCUS_IN_MASK_INDEX    	11
#define  GFOCUS_OUT_MASK_INDEX		12
#define  GCOMPRESS_MOTION_MASK_INDEX    13
/** the last one gives the number of mask **/
#define  GN_EVENT_MASK                14

typedef unsigned int GEventMask;
    
#define GBUTTON_PRESS_MASK 	(1 << GBUTTON_PRESS_MASK_INDEX)
#define GBUTTON_RELEASE_MASK    (1 << GBUTTON_RELEASE_MASK_INDEX)
#define GDOUBLE_CLICK_MASK      (1 << GDOUBLE_CLICK_MASK_INDEX)
#define GTRIPLE_CLICK_MASK      (1 << GTRIPLE_CLICK_MASK_INDEX)
#define GCHAR_MASK              (1 << GCHAR_MASK_INDEX)
#define GMESSAGE_MASK 	        (1 << GMESSAGE_MASK_INDEX)
#define GFUNCTION_KEY_MASK 	(1 << GFUNCTION_KEY_MASK_INDEX)
#define GEXPOSE_MASK 	        (1 << GEXPOSE_MASK_INDEX)
#define GMOTION_MASK 	        (1 << GMOTION_MASK_INDEX)
#define GENTER_SENSOR_MASK      (1 << GENTER_SENSOR_MASK_INDEX)
#define GLEAVE_SENSOR_MASK      (1 << GLEAVE_SENSOR_MASK_INDEX)
#define GFOCUS_IN_MASK		(1 << GFOCUS_IN_MASK_INDEX)
#define GFOCUS_OUT_MASK		(1 << GFOCUS_OUT_MASK_INDEX)
#define GCOMPRESS_MOTION_MASK   (1 << GCOMPRESS_MOTION_MASK_INDEX)
#define GALL_MASK               (~0)


/*
 * Function keys
 * -------------
 */
enum GFunctionKey
{
    GKEY_LEFT,
    GKEY_RIGHT,
    GKEY_UP,
    GKEY_DOWN,
    GKEY_INSERT,
    GKEY_DELETE,
    GKEY_BACKSPACE,
    GKEY_HOME,
    GKEY_END,
    GKEY_PAGE_UP,
    GKEY_PAGE_DOWN,
    GKEY_TAB,
    GKEY_BACKTAB,
    GKEY_HELP,
    GKEY_F1,
    GKEY_F2,
    GKEY_F3,
    GKEY_F4,
    GKEY_F5,
    GKEY_F6,
    GKEY_F7,
    GKEY_F8,
    GKEY_F9,
    GKEY_F10,
    GKEY_F11,
    GKEY_F12,
    GKEY_ESC
};
    
/*
 * Buttons
 * -------
 */
enum GButton
{
    GBUTTON1 = 1,
    GBUTTON2,
    GBUTTON3,
    GBUTTON4,
    GBUTTON5,
    GBUTTON6,
    GBUTTON7,
    GBUTTON8
};

/*
 * Modifiers
 * ---------
 */
enum GModifier
{
    GNONE      = 0,
    GSHIFT     = (1 << 0),
    GCONTROL   = (1 << 2),
    GALTERNATE = (1 << 3)
};

#endif /* _EventDefs_h */
