/* $Id: GoAscii.h,v 1.2 1992/10/28 16:38:52 mbikina Exp $ */
/*****************************************************************************
MODULE:  GoStream
FILE:    GoAscii.cc
AUTHOR:  Gilles Benati
DATE:    May 31st, 1991

DESCRIPTION:
    

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#ifndef _GoAscii_h
#define _GoAscii_h

#include <ctype.h>
#include <Go/GoStream.h>

class GoAscii : public GoStream {

  public:
    GoAscii(GMedium* medium = NULL)  : GoStream(medium) {}
	    
    virtual ~GoAscii();

    virtual int openRead(GMedium* medium = NULL);
    
    virtual int	writeBool(GBool, const char* = NULL);
    virtual int	readBool(GBool*);

    virtual int	writeShort(short, const char* = NULL);
    virtual int	readShort(short*);

    virtual int	writeInt(int, const char* = NULL);
    virtual int	readInt(int*);

    virtual int	writeLong(long, const char* = NULL);
    virtual int	readLong(long*);

    virtual int	writeChar(char, const char* = NULL);
    virtual int	readChar(char*);

    virtual int	writeNumber(GNumber, const char* = NULL);
    virtual int	readNumber(GNumber*);

    virtual int	writeDouble(double, const char* = NULL);
    virtual int	readDouble(double*);

    virtual int	writeVector(const GVector&, const char* = NULL);
    virtual int	readVector(GVector*);

    virtual int	writePoint(const GPoint&, const char* = NULL);
    virtual int	readPoint(GPoint*);

    virtual int	writeBox(const GBox&, const char* = NULL);
    virtual int	readBox(GBox*);

    virtual int	writeRgb(const GRgb&, const char* = NULL);
    virtual int	readRgb(GRgb*);

    virtual int	writeCoordSys(const GCoordSys&, const char* = NULL);
    virtual int	readCoordSys(GCoordSys*);

    virtual int	writeRegion(const GRegion&, const char* = NULL);
    virtual int	readRegion(GRegion*);

    virtual int	writeString(const char*, const char* = NULL);
    virtual int	readString(unsigned long, char**); // **read spec carefully !**;

    virtual int	writeBuffer(unsigned long, const char*, const char* = NULL);
    virtual int	readBuffer(unsigned long*, char**);// **read spec carefully !**;
    
    virtual int	writeHeader(unsigned long, const char*);
    virtual int	readHeader(unsigned long*, char**);
    
  protected:

    GPointerArray _head_obj;   // Restore only. Holds "top level obj" id s; 
    
    // preliminary version;
    int go_to_next_obj();
    int is_obj_desc_line();
    int is_obj_desc_begin(GBool*);
    int get_obj_id(_GId* id);

    int skip_blanks();

    int read_digits(char* str, int length);
    int read_double(char* str, int length);
    int	read_line(char* str, int length);
    int	read_ident(char* str, int length);

    
    int		writeTypeString(const char* type_name);
    int		writeComment(const char* comment);

    virtual int	readObjId(_GId*);
    virtual int	readObjBegin();
    virtual int	readObjEnd();
    virtual int readObj(_GId*);
    virtual int goToNextObj();
    virtual int goToObj(const _GObjRec&);
    
    virtual int	writeObj(_GId, const char* classname, const char* comment);
    virtual int	writeObjId(_GId);
    virtual int	writeObjBegin(GBool first_obj);
    virtual int	writeObjEnd();

  private:

    int write_scalar_param_line(const char* type_str, const char* format,
				long value, const char* comment);

/********* NOT IMPLEMENTED YET *******

  public:


    virtual int	writeFooter();
    virtual int	writeComment(const char*);
    virtual int	deleteFooter();

//	... ?;

****************/

};

#endif /* ndef _GoAscii_h */
