/* $Id: GoInDisp.h,v 1.4 1992/11/25 15:19:09 smadja Exp $ */
/*****************************************************************************
MODULE:  Tools
FILE:    InDisp.h
AUTHOR:  Philippe Smadja
DATE:    29 May 1990

DESCRIPTION:
    Input Dispatcher

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#ifndef __InDisp_h
#define __InDisp_h

#include <Go/machdef.h>
#include <Go/Tools/Array.h>
#include <Go/Tools/Callback.h>

#define GIDLE_AGAIN 					0
#define GIDLE_AFTER_NEXTEVENT 	1
#define GIDLE_REMOVE					2
typedef int GIdle;

typedef GIdle (*GInDispIdleProc)(void* closure);
typedef void (*GInDispTimerProc)(void* closure);
typedef GBool (*GInDispFilterProc)(void* closure, void* event);


class GoInputDispatcher {
  public:
    
    virtual       ~GoInputDispatcher();

    // method to be implemented in a subclass;
    
    virtual void  timerProc(GInDispTimerProc proc,   void* closure,
			    ulong interval, GAddOrRemove add = GADD) = 0;
    virtual void* runEventLoop() = 0;
    virtual GBool eventReady() = 0;
    virtual void  handleNextEvent() = 0;

    // implemented in InDisp but can be reimplemented;
    // these methods proposes a structure to add/rem filter and idle;
    // A subclass can read it: protected methods are provide;
    
    virtual void  idleProc(GInDispIdleProc proc,   void* closure,
			   GAddOrRemove add = GADD);

    virtual void  filterProc(GInDispFilterProc proc,   void* closure,
			     GAddOrRemove add = GADD);
    
    virtual void  stopEventLoop(void* value);

    inline  GBool stopped() { return _stop; } ;

  protected:
    
    GoInputDispatcher();

    virtual GBool CallFilters(void* event);
    virtual GBool CallIdles();
    virtual void  ResetIdles();


    // data;
    
    GBool  _stop:1,
           _reset_idle:1;
    void*  _value;
    GArray _filters;
    GArray _idles;
};

/*
 * declaration of the global input dispatcher of the application;
 * It will be initialized to NULL the .cc file.
 */
extern GoInputDispatcher*&  GInputDispatcher();

#endif /* _GoInDisp_h */


