/* $Id: GoSelDisp.h,v 1.1 1992/12/10 15:37:32 jojic Exp $ */
/*****************************************************************************
MODULE: Graphic Objects
FILE:   GoSelDisp.h
AUTHOR: jojic
DATE:   Nov 9 1992

DESCRIPTION :
  
COPYRIGHT :    
  Copyright (c) 1992 Bull S.A.
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull S.A. may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull S.A..
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull S.A. under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull S.A..

  Bull S.A. assumes no responsibility for the use or inability to use
  any of its software. The software is provided "AS IS" without warranty
  of any kind, and Bull S.A. expressly disclaims all implied warranties,
  including but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull S.A. be liable for any special, incidental,
  indirect or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
*****************************************************************************/
#ifndef _GoSelDisp_h
#define _GoSelDisp_h

#include <Go/GoBasicDS.h>
#include <Go/mGoBasicDS.h>
#include <Go/Tools/Brush.h>

class GoSelectionDisplayerMetaclass;


/*
 *----------------------
 * The GoSelectionDisplayer class
 *----------------------
 */

class GoSelectionDisplayer : public GoBasicDS
{
  public:

    /* specific public methods: */
	
    GBrush& inBrush () const {
	return ((GoSelectionDisplayer*)this)->in_brush;
    }
    GBrush& outBrush () const {
	return ((GoSelectionDisplayer*)this)->out_brush;
    }

    GBrush& altInBrush () const {
	return ((GoSelectionDisplayer*)this)->alt_in_brush;
    }
    GBrush& altOutBrush () const {
	return ((GoSelectionDisplayer*)this)->alt_out_brush;
    }

    void alternateDisplayFor (Go*, GAddOrRemove = GADD);
    

    /* mandatory static methods: */

    static GoSelectionDisplayerMetaclass* metaclass();
    static GoSelectionDisplayer* create ();

    static int save (const GoSelectionDisplayer*, GoStream*);
    static int restore (GoSelectionDisplayer**, GoStream*);   // creates + restores;
    static int readParams (GoSelectionDisplayer*, GoStream*); // restores only;


    /* pre- and post- conditions: */


  protected:

    /* specific protected methods */

    GoSelectionDisplayer ();

    /* mandatory protected virtual methods: */

    ~GoSelectionDisplayer ();
    virtual GAssoc* d_get_assoc () const;
    virtual GBool m_contains (const GPoint&, GNumber = 0) const;
    virtual void m_add_clip (GRegion&) const;
    virtual void m_add_opaque_clip (GRegion&) const;
    virtual void m_draw (const GRegion&) const;
    virtual void m_dup (const Go*, Go*, Go*, GBool, GBool);

  private:

    /* specific private methods: */

    void recompute_my_clip ();
    void compute_or_draw_display_boxes (const GBox&, GBox[4], GBool, GBool) const;
    
    /* mandatory private virtual methods */

    virtual GoMetaclass* m_metaclass() const;

    /* friends: */

    friend void seldisp_select_post (void*, Go*);
    friend void seldisp_root_post (void*, Go*, Go*);
    friend void seldisp_clip_post (void*, Go*);
    friend void seldisp_reparent_post (void*, Go*, Go*, Go*);
    friend void seldisp_add_to_my_clip (void*, Go*);
    friend class GoSelectionDisplayerMetaclass;

    /* private data: */

    GAssoc _assoc;
    GRegion my_clip;
    GBrush _brush;
    GStack disp_boxes_stack;
    GBool precise_clip;
    GBrush in_brush;
    GBrush out_brush;
    GBrush alt_in_brush;
    GBrush alt_out_brush;
    GSet alternate_set;
};




/*
 *-------------------------------
 * The GoSelectionDisplayerMetaclass class
 *-------------------------------
 */

class GoSelectionDisplayerMetaclass : public GoBasicDSMetaclass
{
  public:
    /* pre- and post- conditions: */

  protected:

             GoSelectionDisplayerMetaclass ();  //to get the GoSelectionDisplayerMetaclass use:
    virtual ~GoSelectionDisplayerMetaclass ();  //        GoSelectionDisplayer::metaclass()

  private:

    /* mandatory private virtual methods: */

    virtual const char*       m_classname () const;
    virtual GoMetaclass*      m_superclass () const;
    virtual const GActionRec* m_actions () const;
    virtual Go*               m_create () const;

    /* private data */

    /* friends */

    friend class GoSelectionDisplayer;
};



/*
 *----------------
 * inline methods
 *----------------
 */


inline GoSelectionDisplayer* GoSelectionDisplayer::create ()
{
    return (GoSelectionDisplayer*) Go::Create(GoSelectionDisplayer::metaclass());
}

#endif /* _GoSelDisp_h */
