/* $Id: GoUI.h,v 1.10 1992/12/31 13:08:15 jojic Exp $ */
/*****************************************************************************
MODULE:  Go Kernel (User Interface)
FILE:    GoUI.h
AUTHOR:  Oliver Jojic
DATE:    June 92

DESCRIPTION:
     The GoUI object dispatches GEvents to GoSensors.
     You can set one GoUI per GoRoot.
     GoUI gets events from the GoRoot's associated GoInterf,
  but you may yourself send it an event through the
  ``goui->dispatch(event)'' method.
     You may set filters which will be called before the event is dispatched.
  This allows, for instance, to emulate a three buttons mouse on systems
  where the mouse has only one or two buttons.
     GoUI manages a grabbing sensor stack (the top of the stack catches all
  events) and a focusing sensor (this sensor catches all char events).
     You may facilitate the picking of small graphic objects by setting
  the value of maxDistance() (the distance is in millimeters).

COPYRIGHT:
  Copyright (c) 1990, 1991, 1992 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.

*****************************************************************************/
#ifndef _GoUI_h
#define _GoUI_h

#include <Go/Event.h>
#include <Go/Tools/Callback.h>
#include <Go/Tools/Stack.h>


/* used classes */
class GoInterf;
class Go;
class GoSensor;
class GoRoot;



class GoUI
{
  public:
    GoUI (GoRoot*);
    virtual ~GoUI ();

    /* event dispatching */

    void addInputMask (GEventMask);
    void removeInputMask (GEventMask);

    virtual void dispatch (const GEvent*);

    void filter (GEvent* filter_proc(void*, GoUI*, const GEvent*),
		 void* closure, GAddOrRemove = GADD);

    GNumber maxDistance () const { return _max_distance; }
    void maxDistance (GNumber max) { _max_distance = max; }

    /* grab and focus */

    void grab (GoSensor*);
    void ungrab (GoSensor* = NULL);
    GoSensor* grabbingSensor () const;
    void cancelGrabbers ();

    void focus (GoSensor* = NULL);
    GoSensor* focusingSensor () const { return _focusing_sensor; }

    /* misc. */

    GoRoot* root () const { return _root; }
    virtual void root (GoRoot*);

    friend GoUI* GoUIFrom (const GoRoot* root);

    const GPoint& pointerPos () const { return _pointer_pos; }

  private:
    void incr_or_decr_event_mask_count (GEventMask, GAddOrRemove);
    void set_current_sensor (GoSensor*, GTimeStamp = 0);
    void recompute_current_sensor (GoSensor* = NULL);
    void _reset ();

    GoRoot* _root;
    GEventMask _event_mask;
    short _event_mask_count[GN_EVENT_MASK];
    GNumber _max_distance;
    GStack _grabbing_sensors;
    GoSensor* _focusing_sensor;
    GoSensor* _current_sensor;
    struct FilterList* _filter_list;
    GBool _enter_leave_already_managed;
    GBool _event_already_managed;
    const GEvent* _event;
    GPoint _pointer_pos;

    friend void dispatch_recur(void*, GoSensor*);
    friend void recompute_sensor_recur(void*, GoSensor*);
    friend void goui_roots_interf_changed (void*, Go*, GoInterf*);
    friend void goui_sensors_root_changed (void*, Go*, Go*);
};

#endif /* _GoUI_h */
