/* $Id: Grabber.h,v 1.1 1992/12/10 15:37:36 jojic Exp $ */
/*****************************************************************************
MODULE:  Go Kernel (User Interface)
FILE:    Grabber.h
AUTHOR:  Oliver Jojic
DATE:    July 92

DESCRIPTION:

COPYRIGHT:
  Copyright (c) 1990, 1991, 1992 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.

*****************************************************************************/
#ifndef _Grabber_h
#define _Grabber_h

#include <Go/ItorPgm.h>
#include <Go/Tools/Point.h>


/*
  *-------------------------------------------------------
  * Programmation of a Grabber
  *-------------------------------------------------------
  */

class GGrabberPgm : public GInteractorPgm
{
    /* instanciable subclasses must have a start method with following parameters:
       		start (const GEvent*, Go*)
		*/

  protected:
    GGrabberPgm (const char* name);
    virtual GEventMask inputMask () const;

    friend class GGrabber;
};




class GoSensor;
class GoUI;


/*
 *-------------------------------------------------------
 * Execution of a Grabber
 *-------------------------------------------------------
 */

class GGrabber : public GInteractor
{
  public:

    virtual ~GGrabber ();

    virtual void compute (const GEvent*); //superclass first

    virtual void draw (); //superclass first
    virtual void drawAll ();
    virtual void clear ();
    virtual void clearAll ();
    
    virtual GBool isDoitEvent (const GEvent*) const;
    virtual GBool isCancelEvent (const GEvent*) const;

    GoSensor* sensor () const;

    /* superclasses virtual methods */
    virtual GBool careAbout (const GEvent*) const;
    virtual GBool handle (const GEvent*);
    virtual void doit ();
    virtual void cancel ();

  protected:

    const GGrabberPgm* Pgm () const { return (GGrabberPgm*) pgm(); }

    GGrabber (GGrabberPgm*, const GEvent*, Go*);

    virtual void Initialize (); //superclass last

    /* protected data */
    GoSensor* _sensor;
    GBool draw_from_expose;
    GPoint first_pos;
    GPoint prev_pos;
    GPoint _pos;
    GVector extra_vector;
    GoUI* _goui;
};

#endif /* _Grabber_h */
