/* $Id: Graph.h,v 1.4 1992/12/02 12:06:48 jacomet Exp $ */
/*****************************************************************************
MODULE:  BULL Graphic Objects
FILE:    Graph.h
AUTHOR:  Aline Jacomet
DATE:    February 8th 1991

DESCRIPTION:
         Definition of the graph object class.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#ifndef _GoGraph_h
#define _GoGraph_h

#include <Go/Go.h>
#include <Go/GraphStr.h>
#include <Go/mGraph.h>
#include <Go/GraphNode.h>
#include <Go/Link.h>


/*
 * GoGraph definition
 */
class GoGraph : public GoGraphStruct {
    friend class GoGraphMetaclass;
    
  public:
    /*
     * Pre and Post Conditions ...;
     */
    void addNodePrecond(GBool (*)(void*, GoGraph*, GoGraphNode*, GBool*,
				  GoMetaclass*),
			void* closure, GAddOrRemove = GADD);
    void addNodePostcond(void (*)(void*, GoGraph*, GoGraphNode*, GBool*,
				  GoMetaclass*),
			 void* closure, GAddOrRemove = GADD);
    void removeNodePrecond(GBool (*)(void*, GoGraph*, GoGraphNode*),
			   void* closure, GAddOrRemove = GADD);
    void removeNodePostcond(void (*)(void*, GoGraph*, GoGraphNode*),
			    void* closure, GAddOrRemove = GADD);
    void addLinkPrecond(GBool (*)(void*, GoGraph*, GoGraphNode*, GoGraphNode*,
				  GoMetaclass*),
			void* closure, GAddOrRemove = GADD);
    void addLinkPostcond(void (*)(void*, GoGraph*, GoGraphNode*, GoGraphNode*,
				  GoMetaclass*),
			 void* closure, GAddOrRemove = GADD);
    void removeLinkPrecond(GBool (*)(void*, GoGraph*, GoLink*),
			   void* closure, GAddOrRemove = GADD);
    void removeLinkPostcond(void (*)(void*, GoGraph*, GoLink*),
			    void* closure, GAddOrRemove = GADD);
    
    /*
     * others ...;
     */
    static GoGraphMetaclass* metaclass();
    static GoGraph*          create();

    GoGraphNode* addNode(GoGraphNode*, GBool is_previous = GTRUE,
			 GoMetaclass* linkclass = NULL);
    GBool        removeNode(GoGraphNode*);
    GoLink*      addLink(GoGraphNode* node1, GoGraphNode* node2,
			 GoMetaclass* linkclass = NULL);
    GBool        removeLink(GoLink*);
    GBool        removeLink(GoGraphNode*, GoGraphNode*);
    void         rotateGraph(GNumber, GPoint& = *(GPoint*)NULL);

    /*
     * programmers, don't look at these functions;
     */
    inline void rmLink(GoLink* link) {
	removeLink(link);
    }

    /*
     * save, restore;
     */
    static int save(const GoGraph*, GoStream*);
    static int restore(GoGraph**, GoStream*);
    static int readParams(GoGraph*, GoStream*);

  protected:
    GoGraphCondFlags _flags;
				 
    GoGraph();
    virtual ~GoGraph();

  private:

    virtual GoMetaclass* m_metaclass() const;

    GBool _call_add_node_precond(GoGraphNode*, GBool*, GoMetaclass*) const;
    void  _call_add_node_postcond(GoGraphNode*, GBool*, GoMetaclass*);
    GBool _call_remove_node_precond(GoGraphNode*) const;
    void  _call_remove_node_postcond(GoGraphNode*);
    GBool _call_add_link_precond(GoGraphNode*, GoGraphNode*,GoMetaclass*)const;
    void  _call_add_link_postcond(GoGraphNode*, GoGraphNode*, GoMetaclass*);
    GBool _call_remove_link_precond(GoLink*) const;
    void  _call_remove_link_postcond(GoLink*);

    GBool _call_class_add_node_precond(GoGraphNode*, GBool*,
				       GoMetaclass*) const;
    void  _call_class_add_node_postcond(GoGraphNode*, GBool*, GoMetaclass*);
    GBool _call_class_remove_node_precond(GoGraphNode*) const;
    void  _call_class_remove_node_postcond(GoGraphNode*);
    GBool _call_class_add_link_precond(GoGraphNode*, GoGraphNode*,
				       GoMetaclass*) const;
    void  _call_class_add_link_postcond(GoGraphNode*, GoGraphNode*,
					GoMetaclass*);
    GBool _call_class_remove_link_precond(GoLink*) const;
    void  _call_class_remove_link_postcond(GoLink*);
};


INLINE GoGraph* GoGraph::create() {
    return (GoGraph*) Go::Create(GoGraph::metaclass());
}

#endif /* _GoGraph_h */

