/* $Id: GraphElt.h,v 1.6 1992/12/11 08:34:54 jojic Exp $ */
/*****************************************************************************
MODULE:  BULL Graphic Objects
FILE:    GraphElt.h
AUTHOR:  Aline Jacomet
DATE:    January 11th 1991

DESCRIPTION:
         Definition of the graph element object abstract class.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#ifndef _GoGraphElement_h
#define _GoGraphElement_h

#include <Go/Go.h>
#include <Go/mGraphElt.h>
#include <Go/GraphStr.h>
#include <Go/Tools/Assoc.h>


class GoGraphElement : public Go {
    friend class GoGraphElementMetaclass;
    friend class GoGraphStruct;
    friend class GoGraph;
    friend class GoGraphNode;
    friend class GoTree;
    friend class GoTreeNode;
    friend GBool accept_reparent(void*, const Go*, const Go*, const Go**);

  public:
    static GoGraphElementMetaclass* metaclass();
    
    /*
     * save, restore;
     */
    static int save(const GoGraphElement*, GoStream*);
    static int readParams(GoGraphElement*, GoStream*);

  protected:
    GAssoc _assoc;
    GPoint _origin;          // of the coordinate system;

    GoGraphElement();
    virtual ~GoGraphElement();
    
    virtual void m_dup(const Go*, Go*, Go*, GBool, GBool);

  private:
    Go*      _parent;
    Go*      _child;
    Go*      _next;
    GRegion  _clip;
    GBool    in_destroy;       // When I'm destroying a node or a link,
                               // in_destroy is GTRUE so I accept reparent
                               // on NULL.

    virtual GAssoc* d_get_assoc() const;
    virtual Go*     d_get_parent() const;
    virtual Go*     d_get_previous() const;
    virtual Go*     d_get_next() const;
    virtual Go*     d_get_first_child() const;
    virtual GBool   d_store_children_array() const;
    virtual void    d_set_previous(const Go*);

    virtual void d_add_child(const Go*, int);
    virtual void d_remove_child(const Go*, int);
    virtual void d_set_next(const Go*);
    virtual void d_set_tree(const Go*, const Go*, const Go*);

    virtual GBool            d_store_relcs() const;
    virtual GBool            d_store_abscs() const;
    virtual const GCoordSys& d_get_abscs(const GCoordSys*) const;
    virtual void             d_set_coordsys(const GCoordSys&, const GCoordSys&,
                                           const GCoordSys&);

    virtual GRegion* d_get_clip() const;

    virtual GoMetaclass* m_metaclass() const;
};

#endif /* _GoGraphElement_h */
