/* $Id: GraphStr.h,v 1.6 1992/12/11 08:34:57 jojic Exp $ */
/*****************************************************************************
MODULE:  BULL Graphic Objects
FILE:    GraphStr.h
AUTHOR:  Aline Jacomet
DATE:    January 11th 1991

DESCRIPTION:
         Definition of the graph structure object abstract class.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#ifndef _GoGraphStruct_h
#define _GoGraphStruct_h

#include <Go/Go.h>
#include <Go/mGraphStr.h>
#include <Go/Tools/Assoc.h>
#include <Go/Tools/Array.h>
#include <Go/GraphElt.h>
#include <Go/Tools/Brush.h>
#include <Go/GraphDef.h>


class GoNode;
class GoLink;

struct GApplicationRec {
    void (*proc)(void*, Go*, GoNode*, const GPoint&);
    void* closure;
};


class GoGraphStruct : public Go {
    friend class GoGraphStructMetaclass;
    friend class GoGraph;
    friend class GoTree;
    friend class GoGraphElement;
    friend class GoNodeMetaclass;
    friend class GoTreeNode;
    friend class GoGraphNode;
    friend class GoLink;

  public:
    /*
     * get ...;
     */
    const GoNode* firstNode() const {
	return first_node;
    }
    ushort numberNodes() const {
	return nb_node;
    }
    ushort numberLinks() const {
	return nb_link;
    }
    GBool background() const {
	return _background;
    }
    GBrush& brush() const {
	const GBrush* b = &_brush;
	return (GBrush&)*b;
    }
    ushort positionType() const {
	return position_type;
    }
    ushort anchoringType() const {
	return anchor_type;
    }
    GoMetaclass* defaultLinkType() const {
	return link_type;
    }

    /*
     * set ...;
     */
    GBool positionType(ushort);
    GBool anchoringType(ushort);
    void background(GBool);
    GBool defaultLinkType(GoMetaclass*);
    void applicationFunction(void (*)(void*, Go*, GoNode*, const GPoint&) ,
			     void* closure);

    /*
     * others ...;
     */
    static GoGraphStructMetaclass* metaclass();

    void makePosition(GoNode* node=NULL, const GPoint& = *(GPoint*)NULL);
    
    /*
     * programmers, don't look at these functions;
     */
    void setPositionType(ushort pos) {
	positionType(pos);
    }
    void setBackground(GBool back) {
	background(back);
    }
    void setAnchoringType(ushort pos) {
	anchoringType(pos);
    }
    
    /*
     * save, restore;
     */
    static int save(const GoGraphStruct*, GoStream*);
    static int readParams(GoGraphStruct*, GoStream*);

  protected:
    GAssoc   _assoc;
    GoNode*  first_node;
    ushort   nb_node;
    ushort   nb_link;
    GoMetaclass* link_type;
    ushort   anchor_type;
    ushort   position_type;

    virtual void m_dup(const Go*, Go*, Go*, GBool, GBool);

    void set_coordsys(Go*);
    friend void graphstr_set_coordsys(void*, Go*);

    GoGraphStruct();
    virtual ~GoGraphStruct();

  private:
    GCoordSys       _abscs;
    GCoordSys       _relcs;
    Go*             _parent;
    Go*             _next;
    Go*             _prev;
    GPointerArray   _children;
    GRegion         _clip;
    GBool           _background;
    GBrush          _brush;
    GBox            my_clip;
    GApplicationRec app_function;

    friend void graphstr_brush_changed(void*, GBrush*, ulong);

    virtual GAssoc* d_get_assoc() const;
    virtual Go*     d_get_parent() const;
    virtual Go*     d_get_previous() const;
    virtual Go*     d_get_next() const;
    virtual Go*     d_get_first_child() const;
    virtual GBool   d_store_children_array() const;
    virtual Go**    d_get_children(int& count) const;
    virtual void    d_set_previous(const Go*);

    virtual void   d_add_child(const Go*, int);
    virtual void   d_remove_child(const Go*, int);
    virtual void   d_set_next(const Go*);
    virtual void   d_set_tree(const Go*, const Go*, const Go*);

    virtual GBool            d_store_relcs() const;
    virtual const GCoordSys& d_get_relcs(const GCoordSys*) const;
    virtual GBool            d_store_abscs() const;
    virtual const GCoordSys& d_get_abscs(const GCoordSys*) const;
    virtual void             d_set_coordsys (const GCoordSys&,const GCoordSys&,
					     const GCoordSys&);

    virtual GRegion* d_get_clip() const;

    virtual void    m_add_clip(GRegion&) const;
    virtual void    m_add_opaque_clip(GRegion&) const;
    virtual void    m_draw(const GRegion&) const;
    virtual void    m_wired_draw(const GCoordSys&, const GPen&,
				 GQuality) const;
    
    virtual GoMetaclass* m_metaclass() const;
    virtual GBool   m_contains(const GPoint&, GNumber maxDist=0) const;
    
    virtual void left_right_placement(GoNode*, const GPoint&);
    virtual void up_bottom_placement(GoNode*, const GPoint&);
    virtual void right_left_placement(GoNode*, const GPoint&);
    virtual void bottom_up_placement(GoNode*, const GPoint&);
};


#endif /* _GoGraphStruct_h */
