/* $Id: Image.h,v 1.2 1992/12/11 08:34:58 jojic Exp $ */
/*****************************************************************************        Copyright (c) BULL S.A 1990

MODULE: Graphic Objects
FILE:   Image.h
AUTHOR: Philippe Smadja
DATE:   May 1991

DESCRIPTION :

COPYRIGHT :    
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
*****************************************************************************/
#ifndef _GoImage_h
#define _GoImage_h

#include <Go/GoStdDS.h>
#include <Go/mImage.h>
#include <Go/Tools/PixelArray.h>
#include <Go/Tools/ImageBrush.h>

class GoImage : public GoStdDS {
    friend class GoImageMetaclass;
    static void  _imagebrush_changed(void* goimage, GImageBrush*, ulong);
    static void  _pxlarray_changed  (void* goimage, GPixelArray*, ulong);
 
 public:
    GoImage(const GShImageBrush* = NULL, const GShPixelArray* = NULL);
    ~GoImage();
    
    static GoImageMetaclass* metaclass();
    static GoImage*          create();
    static GoImage*          create(const GShImageBrush*, const GShPixelArray*);

    GNumber width()  const;
    GNumber height() const;
    const  GPoint& origin() const;
    GImageBrush& imageBrush() const{
        return ((GoImage*)this)->_imagebrush;}

    GPixelArray& pixelArray() const{
        return ((GoImage*)this)->_pxlarray;}

    
    GBool config(GNumber w = -1, GNumber h = -1, const GPoint* origin = NULL);
 
    // pre/post conditions;
    
    void configPrecond(GBool func(void*, const GoImage*, GNumber*, GNumber*, GPoint*),
		       void* closure, GAddOrRemove = GADD);
    void configPostcond(void func(void*, GoImage*),
			void* closure, GAddOrRemove = GADD);

    // call precond in public;
    
    GBool callConfigPrecond(GNumber*, GNumber*, GPoint*) const;
    // save/restore management;
    
    static int save(const GoImage*, class GoStream*);
    static int restore(GoImage**, class GoStream*);
    static int readParams(GoImage*, class GoStream*);

  protected:

    virtual GBool m_contains(const GPoint&, GNumber maxDist = 0) const;
    virtual void m_add_clip(GRegion&) const;
    virtual void m_add_opaque_clip(GRegion&) const;
    virtual void m_draw(const GRegion&) const;
    virtual void m_wired_draw(const GCoordSys&, const GPen&, GQuality) const;
    virtual void m_dup(const Go* src, Go* parcopy, Go* prevcopy, GBool donext, GBool doChildren);
    
  private:

    // data.

    GBool         _has_config_precond   :1;
    GBool         _has_config_postcond  :1;
 
    GNumber       _width;
    GNumber       _height;
    GPoint        _origin;
    
    GPixelArray   _pxlarray;
    GImageBrush   _imagebrush;
    GShImageBrush* _old_shimagebrush;
    
    // pre/post conditions;

    GBool call_config_precond(GNumber*, GNumber*, GPoint*) const;
    void call_config_postcond();

    virtual GoMetaclass* m_metaclass() const;
 
};



inline GNumber GoImage::width()  const { return _width;}
inline GNumber GoImage::height() const { return _height;}
inline const GPoint& GoImage::origin() const { return _origin; }

inline GBool GoImage::callConfigPrecond(GNumber* w, GNumber* h,
					GPoint* origin) const
{
    return call_config_precond(w, h, origin);
}
#endif /* _GoImage_h */
