/* $Id: Line.h,v 1.2 1992/12/11 08:35:02 jojic Exp $ */
/*****************************************************************************
MODULE:  Graphic Objects
FILE:    Line.h
AUTHOR:  Gilles Benati
DATE:    March 5th, 1992

DESCRIPTION:
    <to be defined>

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#ifndef _GoLine_h
#define _GoLine_h


#include <Go/GoStdDS.h>
#include <Go/mLine.h>

class GoLine : public GoStdDS {
    friend class GoLineMetaclass;

  public:

    GoLine(const GPoint&, const GPoint&);
    ~GoLine();

    static GoLineMetaclass* metaclass();
    static GoLine* create(const GPoint&, const GPoint&);
    static GoLine* create() {
	return (GoLine*) Go::Create(GoLine::metaclass());
    }

    // some silly compilers complain if "return _pen" or "return _brush";
    GPen& pen() const { return ((GoLine*) this)->_pen; }

    const GPoint& first()  const { return _first; }
    const GPoint& second() const { return _second; }
    
    virtual GBool reset(const GPoint&, const GPoint&);
    virtual GBool first(const GPoint&);
    virtual GBool second(const GPoint&);

    // pre/post conditions;
    void firstPrecond(GBool func(void*, const GoLine*, GPoint*),
		      void* closure, GAddOrRemove = GADD);
    void firstPostcond(void func(void*, GoLine*),
		       void* closure, GAddOrRemove = GADD);
    void secondPrecond(GBool func(void*, const GoLine*, GPoint*),
		       void* closure, GAddOrRemove = GADD);
    void secondPostcond(void func(void*, GoLine*),
			void* closure, GAddOrRemove = GADD);
    void resetPrecond(GBool func(void*, const GoLine*, GPoint*, GPoint*),
		      void* closure, GAddOrRemove = GADD);
    void resetPostcond(void func(void*, GoLine*),
		       void* closure, GAddOrRemove = GADD);

    /*
     * call precondition in public;
     */
    GBool callFirstPrecond(GPoint*) const;
    GBool callSecondPrecond(GPoint*) const;
    GBool callResetprecond(GPoint*, GPoint*) const;

    GBool callClassFirstPrecond(GPoint*) const;
    GBool callClassSecondPrecond(GPoint*) const;
    GBool callClassResetPrecond(GPoint*, GPoint*) const;

  protected:

    static int	save(const GoLine*, GoStream*);
    static int	restore(GoLine**, GoStream*);   // creates + restores;
    static int	readParams(GoLine*, GoStream*); // restores only;

    virtual void  m_dup(const Go* src, Go*, Go*, GBool, GBool);
    virtual GBool m_contains(const GPoint&, GNumber maxDist = 0) const;
    virtual void  m_add_clip(GRegion&) const;
    virtual void  m_add_opaque_clip(GRegion&) const;
    virtual void  m_draw(const GRegion&) const;
    virtual void  m_wired_draw(const GCoordSys&, const GPen&, GQuality) const;

    GBool Reset(const GPoint&, const GPoint&);
    
    const GShPen* sh_pen() const { return _old_sh_pen; }
    void  init_sh_pen() {
	if (_old_sh_pen)
	    _old_sh_pen->destroy();
	_old_sh_pen = _pen.shPen()->dup();
    }


  private:

    GShPen* _old_sh_pen;
    GPen    _pen;		// used to draw the segment;

    GPoint  _first;		// segment extremities;
    GPoint  _second;

    // pre/post condition flags;
    GBool _has_first_precond   :1;
    GBool _has_first_postcond  :1;
    GBool _has_second_precond  :1;
    GBool _has_second_postcond :1;
    GBool _has_reset_precond   :1;
    GBool _has_reset_postcond  :1;

    // call pre/post conditions;
    GBool _call_first_precond(GPoint*) const;
    GBool _call_second_precond(GPoint*) const;
    GBool _call_reset_precond(GPoint*, GPoint*) const;
    void  _call_first_postcond();
    void  _call_second_postcond();
    void  _call_reset_postcond();

    // call metaclass pre/post conditions;
    GBool _call_class_first_precond(GPoint*) const;
    GBool _call_class_second_precond(GPoint*) const;
    GBool _call_class_reset_precond(GPoint*, GPoint*) const;
    void  _call_class_first_postcond();
    void  _call_class_second_postcond();
    void  _call_class_reset_postcond();

    virtual GoMetaclass* m_metaclass() const;
    virtual void         m_check() const;

    void _set_both(GPoint, GPoint);

    // private postcondition on _pen modifications;
    static void _pen_changed(GoLine*, const GPen*, ulong);
};

/*
 * inline methods;
 */

inline GBool GoLine::callFirstPrecond(GPoint* point) const
{
    return _call_first_precond( point);
}

inline GBool GoLine::callSecondPrecond(GPoint* point) const
{
    return _call_second_precond( point);
}

inline GBool GoLine::callResetprecond(GPoint* first, GPoint* second) const
{
    return _call_reset_precond(first, second);
}

inline GBool GoLine::callClassFirstPrecond(GPoint* point) const
{
    return _call_class_first_precond( point);
}

inline GBool GoLine::callClassSecondPrecond(GPoint* point) const
{
    return _call_class_second_precond( point);
}

inline GBool GoLine::callClassResetPrecond(GPoint* first, GPoint* second) const
{
    return _call_class_reset_precond(first, second);
}

#endif /* _GoLine_h */
