/* $Id: Link.h,v 1.5 1992/12/11 08:35:04 jojic Exp $ */
/*****************************************************************************
MODULE:  BULL Graphic Objects
FILE:    GoLink.h
AUTHOR:  Aline Jacomet
DATE:    January 11th 1991

DESCRIPTION:
         Definition of the link object class.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#ifndef _GoLink_h
#define _GoLink_h

#include <Go/Go.h>
#include <Go/GraphElt.h>
#include <Go/mLink.h>
#include <Go/Tools/Pen.h>




/*
 * GoLink definition
 */
class GoLink : public GoGraphElement {
    friend class GoGraphStruct;
    friend class GoTree;
    friend class GoGraph;
    friend class GoTreeNode;
    friend class GoTreeNodeMetaclass;
    friend class GoGraphNode;
    friend class GoGraphNodeMetaclass;
    friend class GoLinkMetaclass;

  public:
    /*
     * get ...;
     */
    virtual GPen& pen() const;
    inline GoNode* previousNode() const{
	return prev_node;
    }
    inline GoNode* nextNode() const{
	return next_node;
    }
    inline GPoint previousPoint() const{
	return prev_point;
    }
    inline GPoint nextPoint() const{
	return next_point;
    }

    /*
     * Pre and Post Conditions ...;
     */
    void attachLinkPrecond(GBool (*)(void*, GoLink*, GoGraphNode*,
				     GoGraphNode*),
			   void* closure, GAddOrRemove = GADD);
    void attachLinkPostcond(void (*)(void*, GoLink*, GoGraphNode*,
				     GoGraphNode*),
			    void* closure, GAddOrRemove = GADD);
    void anchoringPointModifyPrecond(void (*)(void*, GoLink*, GPoint*,GPoint*),
				     void* closure, GAddOrRemove = GADD);
    void anchoringPointModifyPostcond(void (*)(void*, GoLink*,GPoint*,GPoint*),
				      void* closure, GAddOrRemove = GADD);
    
    /*
     * others ...;
     */
    static GoLinkMetaclass* metaclass();
    static GoLink*          create();
    
    GBool attachLink(GoGraphNode*, GoGraphNode*);
    /*
     * save, restore;
     */
    static int save(const GoLink*, GoStream*);
    static int restore(GoLink**, GoStream*);
    static int readParams(GoLink*, GoStream*);
	
    
  protected:
    GPoint  prev_point;
    GPoint  next_point;
    GoNode* prev_node;
    GoNode* next_node;
    GBool   do_draw;     // extrem nodes contain objects ==> link may be drawn;
    GoLinkCondFlags _flags;

    virtual void m_dup(const Go*, Go*, Go*, GBool, GBool);
    GoLink();
    virtual ~GoLink();

    GBool _call_attach_link_precond(GoGraphNode*, GoGraphNode*) const;
    void  _call_attach_link_postcond(GoGraphNode*, GoGraphNode*);
    void  _call_point_modify_precond(GPoint*, GPoint*);
    void  _call_point_modify_postcond(GPoint*, GPoint*);
    GBool _call_class_attach_link_precond(GoGraphNode*, GoGraphNode*)
	const;
    void _call_class_attach_link_postcond(GoGraphNode*, GoGraphNode*);
    void _call_class_point_modify_precond(GPoint*, GPoint*);
    void _call_class_point_modify_postcond(GPoint*, GPoint*);

  private:

    virtual GoMetaclass* m_metaclass() const;
    virtual void _constraints();
};


INLINE GoLink* GoLink::create() {
    return (GoLink*) Go::Create(GoLink::metaclass());
}


#endif /* _GoLink_h */
