/* $Id: Node.h,v 1.5 1992/12/11 08:35:05 jojic Exp $ */
/*****************************************************************************
MODULE:  BULL Graphic Objects
FILE:    Node.h
AUTHOR:  Aline Jacomet
DATE:    January 11th 1991

DESCRIPTION:
         Definition of the node object abstract class.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#ifndef _GoNode_h
#define _GoNode_h

#include <Go/Go.h>
#include <Go/GraphElt.h>
#include <Go/mNode.h>
#include <Go/Link.h>


class GoNode : public GoGraphElement {
    friend void node_brush_modified(void* node, GBrush*, ulong);
    friend class GoSimpleLink;
    friend class GoNodeMetaclass;
    
  public:
    static GoNodeMetaclass* metaclass();

    /*
     * get ...;
     */
    inline GBrush& brush() const {
	if (!p_brush)
	    return *(GBrush*)NULL;
	else
	    return *p_brush;
    }
    inline GBool background() const {
	return _background;
    }

    /*
     * set ...;
     */
    void background(GBool);
    
    /*
     * programmers, don't look at these functions;
     */
    inline void setBackground(GBool back) {
	background(back);
    }

    /*
     * save, restore;
     */
    static int save(const GoNode*, GoStream*);
    static int readParams(GoNode*, GoStream*);

  protected:
    GoNode();
    virtual ~GoNode();
    virtual void m_dup(const Go*, Go*, Go*, GBool, GBool);

  private:
    GBrush* p_brush;
    GBool   _background;
    GNumber width_2;           // width / 2 of my_clip;
    GNumber height_2;          // height / 2 of my_clip;
    
    virtual GoMetaclass* m_metaclass() const;
    virtual void   m_draw(const GRegion&) const;
    virtual void   m_wired_draw(const GCoordSys&, const GPen&, GQuality) const;
    virtual void   m_add_clip(GRegion&) const;
    virtual void   m_add_opaque_clip(GRegion&) const;
    virtual GBool  m_contains(const GPoint&, GNumber) const;
};


#endif /* _GoNode_h */
