/* $Id: PSGoInterf.h,v 1.4 1992/12/21 09:21:19 smadja Exp $ */
/*****************************************************************************
MODULE:  GoInterface
FILE:    PSGoInterf.h
AUTHOR:  Philippe Smadja, Michel Marcuse 
DATE:    Jul 1991

DESCRIPTION:
  The PSGoInterf is the PostScript Interface of the Go library.
  Like others interface it is Go independant.

  No input methods are available (.i.e getPointer, getImage, setPointer).

  It implements the output virtual methods of the GoInterf superclass.

  It implements also the Query methods, specialy for the Font.

  Clip and Opaque clip for the PSGoInterf are the same as the superclass one
  specialy for the font which are deformable.
  So we do not have to implement virtual clips methods.

  virtual "Contains" methods are use in interactive mode, so we do not need
  to implement these methods.

  Specifics constructors and methods are implemented to handle different
  output medium.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#ifndef _PSGoInterf_h
#define _PSGoInterf_h

#include <Go/GoInterf.h>
#include <Go/Tools/Error.h>
#include <Go/Tools/Assoc.h>

class GMedium;

class GPen;
class GBrush;
class GTextBrush;
class GImageBrush;

class GShDash;
class GShColor;
class GShFont;

enum GLayoutMode
{
    GPORTRAIT,
    GLANDSCAPE
};


struct GCurrentPSContext
{
    float       lineWidth;
    float       miterLimit;
    ushort      capStyle;
    ushort      joinStyle;
    GShColor*   shcolor;
    GShDash*    shdash;
    GShFont*    shfont;
    GVector     shfont_I;
    GVector     shfont_J;
    
    GCurrentPSContext();
    ~GCurrentPSContext();
};


class PSGoInterf : public GoInterf
{
  public:

    /**
      specific constructors
      **/

    /* no parameter: no output possible
       wait for a setOutput methods to
       exec the output requests
       */
    PSGoInterf(GLayoutMode lm = GPORTRAIT);

    /* output onto a file
       we will create a GFileMedium
       for type cf fopen().
       **/
    PSGoInterf(const char* filename, const char* type, GLayoutMode = GPORTRAIT);

    PSGoInterf(GMedium* medium, GLayoutMode lm = GPORTRAIT);

    /**
      specific methods
      the method with filename returns GTRUE if the file if the open result
      is OK else it returns GFALSE
      **/
    GBool  setOutput(const char* filename, const char* openwrite_type); /**
									 for type
									 cf fopen()
									 **/
    void  setOutput(GMedium* medium);

    const GMedium* medium();                /* returns the used medium */
    
    /**
      The following implement  subset of the GoInterf virtual methods
      This subset is the minimum one at the moment.
      **/

    ~PSGoInterf();

    virtual GNumber pixelWidth () const {
	return 25.4/300;    // return for 300 dpi ??
    }
    virtual GNumber pixelHeight () const {
	return pixelWidth();
    }


    /**
      INPUT QUERY METHODS  unavailable on PSGoInterf
      **********************************************
      **/
    virtual GPoint getPointer(const GCoordSys* = NULL) {
	GError("PSGoInterf::%s no query method:", "getPointer");
	return GPoint(0,0);
    }

    virtual void  setPointer(const GPoint&, const GCoordSys* = NULL) {
	GError("PSGoInterf::%s no query method:", "setPointer");
    }

    virtual GBool captureEvent(GEventMask, GBool, GTimeStamp = GCURRENT_TIME)    {
	GError("PSGoInterf::%s no input method:", "captureEvent");
	return GFALSE;
    }
    virtual void unCaptureEvent(GTimeStamp = GCURRENT_TIME)    {
	GError("PSGoInterf::%s no input method:", "eventReady");
    }
    
    virtual ulong inputFd()    {
	GError("PSGoInterf::%s no input method:", "inputFd");
	return 0;
    }
    
    virtual void getImage(GImageBrushValues&, /* result */
			  GPixelArrayValues&, /* result */
			  const GCoordSys& ,
			  const GPoint&,	/* origin */
			  GNumber, /* width */
			  GNumber, /* height */
			  ushort, /* depth = 0 */
			  GBool,	/* compress_colormap = GTRUE */
			  ushort, /* x_resolution = 0 */
			  ushort, /* y_resolution = 0 */
			  GQuality /* quality = GmaxQuality */) {
	GError("PSGoInterf::%s no query method:", "getImage");
    }


    virtual GShColor* colorFromName(const char* /* color_name */) const {
	GError("PSGoInterf::%s no query method:", "colorFromName");
	return NULL;
    }

    /**
      OUTPUT METHODS
      **************
      **/
    virtual void clear (const GRegion&);

    virtual void polyline  (const GShPen*, const GShBrush*,
			    const GCoordSys&, int npt, const GPoint*,
			    GQuality = GMAX_QUALITY);

    virtual void image	   (const GShImageBrush*,
			    const GCoordSys&, const GPoint&, GNumber, GNumber,
			    const GShPixelArray*, GQuality quality = GMAX_QUALITY);

    virtual void polygon   (const GShPen*, const GShBrush*,
			    const GCoordSys&, int npt, const GPoint*,
			    GQuality quality = GMAX_QUALITY);

    virtual void string	   (const GShTextBrush*, const GShBrush*,
			    const GCoordSys&, const GPoint&, GNumber,
			    int, const char*,
			    GQuality quality = GMAX_QUALITY);

    virtual void ellipse   (const GShPen* pen, const GShBrush* brush,
			    const GCoordSys& cs, const GPoint& origin,
			    GNumber radiusX, GNumber radiusY,
			    GNumber angle1 = 0, GNumber angle2 = 2*PI,
			    GQuality quality = GMAX_QUALITY);

    virtual void ellipseChord   (const GShPen* pen, const GShBrush* brush,
				 const GCoordSys& cs, const GPoint& origin,
				 GNumber radiusX, GNumber radiusY,
				 GNumber angle1, GNumber angle2, GBool closed,
				 GQuality quality = GMAX_QUALITY);

    virtual void ellipsePie   (const GShPen* pen, const GShBrush* brush,
			       const GCoordSys& cs, const GPoint& origin,
			       GNumber radiusX, GNumber radiusY,
			       GNumber angle1, GNumber angle2, GBool closed,
			       GQuality quality = GMAX_QUALITY);

    /**
      FONT QUERY METHODS
      ******************
      **/

    virtual const  char* fontName      (const GCoordSys&, GNumber,
					const GShFont*) ;

    virtual GNumber stringLeftBearing  (const char*, int,
					GNumber, const GShFont*) ;
    virtual GNumber stringRightBearing (const char*,  int,
					GNumber,  const GShFont*) ;
    virtual GNumber stringAscent       (const char*, int,
					GNumber,  const GShFont*) ;
    virtual GNumber stringDescent      (const char*, int,
					GNumber,     const GShFont*) ;
    virtual GNumber stringWidth        (const char*, int,
					GNumber,     const GShFont*) ;

    virtual GNumber fontItalicAngle        (const GShFont*) ;

    virtual GNumber fontAscent             (GNumber,  const GShFont*) ;
    virtual GNumber fontDescent            (GNumber,  const GShFont*) ;
    virtual GNumber fontUnderlineThickness (GNumber,  const GShFont*) ;
    virtual GNumber fontUnderlinePosition  (GNumber,  const GShFont*);
    virtual GNumber fontStrikeoutAscent    (GNumber,  const GShFont*) ;
    virtual GNumber fontStrikeoutDescent   (GNumber,  const GShFont*) ;
    virtual GNumber fontMinSpace           (GNumber, const GShFont*) ;
    virtual GNumber fontMaxSpace           (GNumber, const GShFont*) ;
    virtual GNumber fontNormSpace          (GNumber, const GShFont*) ;
    virtual GNumber fontEndSpace           (GNumber, const GShFont*) ;

  private:
    // data;

    GLayoutMode _layoutMode;
    GCoordSys   _psMatrix;
    GMedium*    _medium;
    GBool       _myMedium : 1,
               _firstImageCall : 1;
    GCurrentPSContext _current;
    GAssoc      _assocFont;
    GAssoc      _assocPixelArray;
    GAssoc      _assocColormap;
    int        _currentFontNumber;            // to make unique PS ident;
    int        _currentPixelArrayNumber;
    int        _currentColormapNumber;
    
    void _output(const char* s, GBool eol = GTRUE);
    void _commonInit(GMedium*, GLayoutMode);
    void _display(const GShPen* pen, const GShBrush* brush);
    void _fill(const GShBrush* brush, GBool saverestore);
    void _stroke(const GShPen* pen);
    void _setColor(const GShColor*);
    void _setDash(const GNumber* l, int n, GNumber offset);
    void _regToPath( const GRegion& reg);
    void _makeBoxePath(const GBox& box);
    void _makePolyPath (const GCoordSys& cs, const GPoint* pts, int npt,
			GBool closed,
			GBool path_will_be_duplicated);
    void _outputPixelArrayData (const GShPixelArray*, GBool);
    struct FontAssoc* _getFontAssociation(const GShFont*);
    void  _freeFontAssociations();
    struct ColormapAssoc* _getColormapAssociation(const GShColormap*,
						  long refcnt);
    struct PixelArrayAssoc* _getPixelArrayAssociation(const GShPixelArray*,
						      long refcnt);
};

#endif
