/* $Id: PointSensor.h,v 1.1 1992/12/10 15:37:54 jojic Exp $ */
/*****************************************************************************
MODULE:  Go User Interaction
FILE:    PointSensor.h
AUTHOR:  Gilles Benati
DATE:    September 20th, 1991

DESCRIPTION:
    

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#ifndef _PointSensor_h
#define _PointSensor_h

#include <Go/StdSensor.h>
#include <Go/mPointSensor.h>

class GoPointSensor;

typedef void (*GSensorPostcond)(void* closure, GoSensor*);
typedef void (*GPointSensorPostcond)(void* closure, GoPointSensor*);

class GoPointSensor : public GoStdSensor
{
    friend class GoPointSensorMetaclass;

  public:

    static GoPointSensorMetaclass* metaclass();
    static GoPointSensor* create(const char* program, Go* parent,
				 const GEvent*,
				 GPointSensorPostcond, void* closure);
    static GoPointSensor* create() {
	return (GoPointSensor*) Go::Create(GoPointSensor::metaclass());
    }

    GoPointSensor();

    static int	 save(const GoPointSensor*, GoStream*);
    static int	 restore(GoPointSensor**, GoStream*);   // creates + restores;
    static int	 readParams(GoPointSensor* psen, GoStream* gostream) {
	return GoStdSensor::readParams(psen, gostream);   // restores only;
    }
    
  protected:

    /*
     * This is just meant to allow GoPointSensorMetaclass to..;
     * use these protected member functions;
     */
    void ProtectedRedrawBackAndFore() { RedrawBackAndFore(); }
    void ProtectedMyClipHasChanged()  { MyClipHasChanged(); }
    void ProtectedRedrawForeOnly()    { RedrawForeOnly(); }

    void ComputeBox();
    
  private:

    GBox _bounding_box;

    virtual GoMetaclass* m_metaclass() const;
    virtual GBool        m_contains(const GPoint&, GNumber) const;
    virtual void         m_add_clip(GRegion&) const;
//  virtual void         m_add_opaque_clip(GRegion&) const;
    virtual void         m_draw(const GRegion&) const;

    static void _box_changed(void*, Go* go);
    static void _parent_changed(void*, Go*, Go*, Go*);
};


#endif /* _PointSensor_h */
