/* $Id: Polygon.h,v 1.3 1992/12/11 08:35:07 jojic Exp $ */
/*****************************************************************************
MODULE: Geometric
FILE:   Polygon.h
AUTHOR: Gilles Benati
DATE:   September 20th, 1991

DESCRIPTION :

    This is one of the basic graphic objects. It simply draws the object
  relative polygon (a closed polyline).
    
    The origin of the coord system is not related to any of the point in the
  point array, i.e. adding or (re)moving points will not affect the coord sys).

COPYRIGHT :
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
*****************************************************************************/
#ifndef _GoPolygon_h
#define _GoPolygon_h

#include <Go/Polyline.h>
#include <Go/mPolygon.h>

class GoPolygon : public GoPolyline {
    friend class GoPolygonMetaclass;

  public:

    GoPolygon() : GoPolyline(0, NULL) { }
    GoPolygon(int nb, const GPoint* points) : GoPolyline(nb, points) { }

    static GoPolygonMetaclass* metaclass();
    static GoPolygon* create(int nb, const GPoint *points);
    static GoPolygon* create() {
	return (GoPolygon*) Go::Create(GoPolygon::metaclass());
    }

    static int	 save(const GoPolygon*, GoStream*);
    static int	 restore(GoPolygon**, GoStream*);   // creates + restores;
    static int	 readParams(GoPolygon* polg, GoStream* gostream) {
	return GoPolyline::readParams(polg, gostream); // restores only;
    }

  protected:


    virtual GBool m_contains(const GPoint&, GNumber maxDist = 0) const;
    virtual void m_add_clip(GRegion&) const;
//  virtual void m_add_opaque_clip(GRegion&) const;
    virtual void m_draw(const GRegion&) const;
    virtual void m_wired_draw(const GCoordSys&, const GPen&, GQuality) const;

//  GPoint getNewPosition(int, GBool&);
//  GPoint getNewPoint(int, GBool&);

  private:

    virtual GoMetaclass* m_metaclass() const;
    virtual void         m_check() const;

    virtual int get_round_points(GPoint** roundPoints) const;
};

#endif /* _GoPolygon_h */
