/* $Id: PolyptGrab.h,v 1.1 1992/12/10 15:37:56 jojic Exp $ */
/*****************************************************************************
MODULE:  Go Kernel (User Interface)
FILE:    PolyptGrab.h
AUTHOR:  Oliver Jojic
DATE:    July 92

DESCRIPTION:

  GPolypointGrabber is a grabber specialized for the edition of
  polypoint graphic objects.
  The name of the GPolypointGrabber is "GoPolypoint.edit".
  Therefore, to activate this interaction on a GoPolypoint, it is
  sufficient to add to this go a child sensor who's name is "edit".
  Afterwards, each call to ``sensor->handle(event)'' will execute
  step by step the ``polypoint edit'' interaction.
  If you use the GoUI object, the call to the sensor's handle method
  is automatically done.

  While editing the polypoint, preconditions on add, del and move points
  are called.

  The press with button 1 on a point will move that point until the button
  is released. The press with button 1 on a segment will add a new point
  on this segment (or will fix the first point, if there are not yet
  any point in the GoPolypoint).
  The press with button 2 on a point will delete that point.
  The press with button 3 anywhere will finish the interaction.
  The press on the ESCAPE key will cancel the edition.

  While you are moving a point, the cursor is invisible. Otherwise,
  the cursor is a crosshair.


COPYRIGHT:
  Copyright (c) 1990, 1991, 1992 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.

*****************************************************************************/
#ifndef _PolypointGrab_h
#define _PolypointGrab_h


#include <Go/SorGrabber.h>
#include <Go/Tools/Array.h>



/*
 *-------------------------------------------------------
 * The GPolypointGrabber class
 *-------------------------------------------------------
 */

class GPolypointGrabber : public GSorGrabber
{
  public:

    GPolypointGrabber ();

  protected:

    virtual GInteractor* dup () const;
    virtual GBool careAbout (const GEvent*) const;
    void DoDup (const GInteractor* origin);

    virtual void Initialize (const GEvent*, void*);
    virtual void Draw (const GRegion* = NULL);
    virtual GBool IsConfirmEvent (const GEvent*) const;
    virtual void Compute (const GPoint&);
    virtual void DoIt ();
    virtual void Cancel ();


    /* utility methods */

    int PositionToIndex (const GPoint&) const;
    int PositionToSegment (const GPoint&) const;
    virtual void DrawAll ();


    /* protected data */

    int _index; // index of the point being moved;
    GArray _points;
    GBool do_not_draw;
};

#endif /* _PolypointGrab_h */
