/* $Id: RegPoly.h,v 1.3 1992/12/11 08:35:13 jojic Exp $ */
/*****************************************************************************
        Copyright (c) BULL S.A 1990

MODULE: Geometric
FILE:   RegPoly.h
AUTHOR: Gilles Benati
DATE:   Feb 1991

DESCRIPTION :


COPYRIGHT :
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#ifndef _GoRegPoly_h
#define _GoRegPoly_h

#include <Go/Polygon.h>
#include <Go/mRegPoly.h>

class GoRegPolygon : public GoPolygon {
    friend class GoRegPolygonMetaclass;

  public:
    
    static GoRegPolygonMetaclass*  metaclass();
    static GoRegPolygon*           create() {
	return (GoRegPolygon*) Go::Create(GoRegPolygon::metaclass());
    }
    static GoRegPolygon*           create(int n, GNumber radius, GNumber angle);

    GoRegPolygon (int n = 0, GNumber radius = 1.0, GNumber firstAngle = 0);

    GNumber radius() const;
    GNumber angle() const {
	return(_nbPoint < 1 ? 0. : _points[0].angle());
    }

    GBool reset(int n, GNumber radius = 1.0, GNumber firstAngle = 0);
    virtual GBool nbPoint(int);
    virtual GBool angle(GNumber);
    virtual GBool radius(GNumber);
    
    /* inhibited functions of the base class */
    virtual GBool reset(int, const GPoint*);           // { return GFALSE; };
    virtual GBool addPoint(int, const GPoint&, GBool);  // { return GFALSE; };
    virtual GBool movePoint(int, const GPoint&, GBool); // { return GFALSE; };
    virtual GBool delPoint(int);                      // { return GFALSE; };

    // pre/post conditions;
    void radiusPrecond(GBool func(void*, const GoRegPolygon*, GNumber*),
		       void* closure, GAddOrRemove = GADD);
    void radiusPostcond(void func(void*, GoRegPolygon*),
			void* closure, GAddOrRemove = GADD);
    void anglePrecond(GBool func(void*, const GoRegPolygon*, GNumber*),
		      void* closure, GAddOrRemove = GADD);
    void anglePostcond(void func(void*, GoRegPolygon*),
		       void* closure, GAddOrRemove = GADD);
    void nbPointPrecond(GBool func(void*, const GoRegPolygon*, int*),
			void* closure, GAddOrRemove = GADD);
    void nbPointPostcond(void func(void*, GoRegPolygon*),
			 void* closure, GAddOrRemove = GADD);
    void resetPrecond(GBool func(void*, const GoRegPolygon*,
				int*, GNumber*, GNumber*),
		      void* closure, GAddOrRemove = GADD);
    void resetPostcond(void func(void*, GoRegPolygon*),
		       void* closure, GAddOrRemove = GADD);

    // call preconditions in public;
    GBool callRadiusPrecond(GNumber*) const;
    GBool callAnglePrecond(GNumber*) const;
    GBool callNbPointPrecond(int*) const;
    GBool callResetPrecond(int* , GNumber*, GNumber*) const;
    
    static int	 save(const GoRegPolygon*, GoStream*);
    static int	 restore(GoRegPolygon**, GoStream*);   // creates + restores;
    static int	 readParams(GoRegPolygon*, GoStream*); // restores only;

    
  protected:
    

//  virtual GBool m_contains(const GPoint&, GNumber maxDist = 0) const;
//  virtual void m_add_clip(GRegion&) const;
//  virtual void m_add_opaque_clip(GRegion&) const;
//  virtual void m_draw(const GRegion&) const;
//  virtual void m_wired_draw(const GCoordSys&, const GPen&, GQuality) const;
//  virtual void m_dup(const Go* src, Go*, Go*,GBool,GBool);

    GBool Reset(int n, GNumber, GNumber, GBool basic = GFALSE);
    
  private:

    // pre/post condition flags;
    GBool _has_radius_precond   :1;
    GBool _has_radius_postcond  :1;
    GBool _has_angle_precond    :1;
    GBool _has_angle_postcond   :1;
    GBool _has_nbPoint_precond  :1;
    GBool _has_nbPoint_postcond :1;
    GBool _has_reset_precond    :1;
    GBool _has_reset_postcond   :1;

    // call pre/post conditions;
    GBool _call_radius_precond(GNumber*) const;
    GBool _call_angle_precond(GNumber*) const;
    GBool _call_nbPoint_precond(int*) const;
    GBool _call_reset_precond(int* , GNumber*, GNumber*) const;
    void _call_radius_postcond();
    void _call_angle_postcond();
    void _call_nbPoint_postcond();
    void _call_reset_postcond();

    // call metaclass pre/post conditions;
    GBool _call_class_radius_precond(GNumber*) const;
    GBool _call_class_angle_precond(GNumber*) const;
    GBool _call_class_nbPoint_precond(int*) const;
    GBool _call_class_reset_precond(int* , GNumber*, GNumber*) const;
    void _call_class_radius_postcond();
    void _call_class_angle_postcond();
    void _call_class_nbPoint_postcond();
    void _call_class_reset_postcond();

    virtual GoMetaclass* m_metaclass() const;
    virtual void         m_check() const;

    GBool _radius(GNumber);
    GBool _angle(GNumber);
};


inline GBool GoRegPolygon::callRadiusPrecond(GNumber* radius) const
{
    return _call_radius_precond(radius);
}

inline GBool GoRegPolygon::callAnglePrecond(GNumber* angle) const
{
    return _call_angle_precond(angle);
}

inline GBool GoRegPolygon::callNbPointPrecond(int* nbpoint) const
{
    return _call_nbPoint_precond(nbpoint);
}

inline GBool GoRegPolygon::callResetPrecond(int* nb, GNumber* radius,
				     GNumber* angle) const
{
    return _call_reset_precond(nb, radius, angle);
}


#endif /* _GoRegPoly_h */
