/* $Id: SimpleGrPgm.h,v 1.1 1992/12/10 15:38:13 jojic Exp $ */
/*****************************************************************************
MODULE:  Go Kernel (User Interface)
FILE:    SimpleGrPgm.h
AUTHOR:  Philippe Smadja
DATE:    Nov 92

DESCRIPTION:

  The purpose of the SimpleGrabberPgm program is to simplify the write of a
  simple grabber.
  The simple grabber is defined by the fact that its Program
  has not specific data.
  So the writing of the corresponding program is not necessary
  EXCEPT the necessity to name it (cf Sensor).

  The purpose the GSimpleGrabberPgm is to create a instance of a program
  with a name given by application.

  Sensor calls the `start' of the program. The `start' will  call the
  `constructor' of the Grabber givent by application.
  After it calls Initialize of this grabber.

  Note that currently , we have not the generalized metaclass object,
  so you have to give a 'function' which will create  your wanted grabber.
  At last a interface with metaclass will be provided;
  
  
COPYRIGHT:
  Copyright (c) 1990, 1991, 1992 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.

*****************************************************************************/
#ifndef _SimpleGrPgm_h
#define _SimpleGrPgm_h

#include <Go/Grabber.h>

/*
 *-------------------------------------------------------
 * Programmation of a SimpleGrabberPgm
 *-------------------------------------------------------
 */

class GSimpleGrabberPgm;
class GEvent;
class Go;

typedef GGrabber* (*GGrabberNew)(GSimpleGrabberPgm*, const GEvent*, Go*);

class GSimpleGrabberPgm : public GGrabberPgm
{
  public:
    static GSimpleGrabberPgm* create(const char* name,
				  GGrabberNew  interactor_constructor);

    // superclasses virtual method;
    virtual GInteractor* start (const GEvent*, Go*);

    GGrabberNew constructor() const { return _constructor; }
    void  constructor(GGrabberNew c) { _constructor = c;}

  protected:
    GSimpleGrabberPgm (const char* name, GGrabberNew constructor);

  private:
    GGrabberNew _constructor;
};

/*
 *-------------------------------------------------------
 * Execution of this Grabber
 * MUST be provide by application via
 * the `interactor_constructor'
 *-------------------------------------------------------
 */


#endif /* _SimpleGrPgm_h */
