/* $Id: StdSensor.h,v 1.1 1992/12/10 15:38:15 jojic Exp $ */
/*****************************************************************************
MODULE: Graphic Objects
FILE:   StdSensor.h
AUTHOR: jojic
DATE:   Jul 2 1992

DESCRIPTION :
  
COPYRIGHT :    
  Copyright (c) 1992 Bull S.A.
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull S.A. may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull S.A..
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull S.A. under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull S.A..

  Bull S.A. assumes no responsibility for the use or inability to use
  any of its software. The software is provided "AS IS" without warranty
  of any kind, and Bull S.A. expressly disclaims all implied warranties,
  including but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull S.A. be liable for any special, incidental,
  indirect or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
*****************************************************************************/
#ifndef _StdSensor_h
#define _StdSensor_h

#include <Go/Sensor.h>

class GoStdSensorMetaclass;


/*
 *----------------------
 * The GoStdSensor class
 *----------------------
 */

class GoStdSensor : public GoSensor
{
  public:

    /* mandatory static methods: */

    static GoStdSensorMetaclass* metaclass();
    static GoStdSensor* create ();

    static int save (const GoStdSensor*, GoStream*);
    static int restore (GoStdSensor**, GoStream*);   // creates + restores;
    static int readParams (GoStdSensor*, GoStream*); // restores only;

  protected:

    /* specific protected methods */

    GoStdSensor ();

    /* mandatory protected virtual methods: */

    ~GoStdSensor ();
    virtual void m_dup (const Go*, Go*, Go*, GBool, GBool);

  private:

    /* mandatory private virtual methods */

    virtual GoMetaclass* m_metaclass() const;

    /* virtual data management */

    virtual Go*		d_get_first_child () const;

    virtual void	d_remove_child (const Go*, int child_index);
    virtual void 	d_add_child (const Go*, int child_index);

    virtual GBool	d_store_abscs () const;
    virtual GBool	d_store_relcs () const;
    virtual const GCoordSys& d_get_abscs (const GCoordSys* parabs=NULL) const;

    virtual void 	d_set_coordsys (const GCoordSys& relcs,
					const GCoordSys& abscs,
					const GCoordSys& parabs);

    /* friends: */

    friend class GoStdSensorMetaclass;

    /* private data: */

    Go* _child;
    GCoordSys _abscs;
};



/*
 *-------------------------------
 * The GoStdSensorMetaclass class
 *-------------------------------
 */

class GoStdSensorMetaclass : public GoSensorMetaclass
{
  public:
    /* pre- and post- conditions: */

  protected:

             GoStdSensorMetaclass ();  //to get the GoStdSensorMetaclass use:
    virtual ~GoStdSensorMetaclass ();  //        GoStdSensor::metaclass()

  private:

    /* mandatory private virtual methods: */

    virtual const char*       m_classname () const;
    virtual GoMetaclass*      m_superclass () const;
    virtual const GActionRec* m_actions () const;
    virtual Go*               m_create () const;

    /* friends */

    friend class GoStdSensor;
};



/*
 *----------------
 * inline methods
 *----------------
 */


inline GoStdSensor* GoStdSensor::create ()
{
    return (GoStdSensor*) Go::Create(GoStdSensor::metaclass());
}

#endif /* _StdSensor_h */
