/* $Id: Assoc.h,v 1.1.1.1 1992/03/18 09:54:39 uep Exp $ */
/*****************************************************************************
MODULE:  Tools
FILE:    Assoc.h
AUTHOR:  Olivier Jojic
DATE:    10 June 1989

DESCRIPTION:
    The "Assoc" class allows you to associate one or more values to some keys.
    "key" must be integer-like (i.e. short, int, pointer, ...). If you want
    to use names as key (i.e. "strcmp" will be used to check equality of keys)
    you must use the "StringAssoc" class.

    Retriving of values for a given key is done using a hash-coding.
    The size of the hash-table is settable by the programmer.
    By default, while the number of associations if small, the hash-table has
    only one entry. And when the number of associations increases, the hash-table
    behave automatically 256 entries.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#ifndef _Assoc_h
#define _Assoc_h

#include <Go/machdef.h>


class GAssoc
{
  public:
    GAssoc();
    ~GAssoc();

    void  setArraySize(int); //must be a power of 2 <= 256

    void  set (const void* key, const void* value);
    void  set (const void* key, int size, const void* addr);
    int   add (const void* key, const void* elem);
    int   add (const void* key, int size, const void* addr);
    void* get (const void* key) const;
    const void* get (const void* key, int& numElem) const;
    void  rem (const void* key);
    int   rem (const void* key, const void* elem);
    int   rem (const void* key, int size, const void* addr);

    void reset();
    void** getKeys(int& count) const;

  private:
    ushort              arraySize;
    class GLinearAssoc** array;

    int hash(const void* key) const;
    int keyToIndex(const void* key) const;
    void maybeSwitchToHash();

};


#endif /* _Assoc_h */
