/* $Id: Brush.h,v 1.1.1.1 1992/03/18 09:54:57 uep Exp $ */
/*****************************************************************************
MODULE:  GoGC
FILE:    Brush.h
AUTHOR:  Jacques Davy
DATE:    November 1990

DESCRIPTION:
    This file describes the Brush object.

    The Brush object provides a regular object interface of a shared object
    ShBrush (see ShBrush.h). It also allows to set some pre/post conditions.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#ifndef _Brush_h
#define _Brush_h

#include <Go/machdef.h>
#include <Go/Tools/ShBrush.h>
#include <Go/Tools/Assoc.h>
#include <Go/Tools/Callback.h>

/*
 * Condition flags
 */
struct GBrushCondFlags
{
    GBool _has_modify_precond : 1;
    GBool _has_modify_postcond : 1;
    GBool _has_destroy_postcond : 1;

    GBrushCondFlags() {
	_has_modify_precond  = GFALSE;
	_has_modify_postcond  = GFALSE;
	_has_destroy_postcond  = GFALSE;
    }
};

/*
 * GBrush class definition
 */
class GBrush
{
  public:
    GBrush ();
    GBrush (const GBrush* brush);
    GBrush (const GShBrush* sh_brush);
    GBrush (ulong mask, const GBrushValues* values, const char* name = NULL);
    GBrush (const char* name);
    ~GBrush ();

    // Getting values
    inline const GShBrush*     shBrush() const { return _sh_brush; }
    inline       ulong         mask() const { return _sh_brush->mask(); }
    inline const GBrushValues* values() const { return _sh_brush->values(); }

    inline const char* name() const { return _sh_brush->name(); }
    inline       GBool isNamed() const { return _sh_brush->isNamed(); }

    inline ulong function () const { return _sh_brush->function(); }
    inline ulong planeMask () const { return _sh_brush->planeMask(); }
    inline const class GShColor* color () const { return _sh_brush->color(); }
    inline const class GShColor* altColor () const {
	return _sh_brush->altColor();
    }

    inline ushort fillStyle () const { return _sh_brush->fillStyle(); }
    inline ushort fillRule () const { return _sh_brush->fillRule(); }
    inline const class GShPattern* tile () const { return _sh_brush->tile(); }
    inline const class GShPattern* stipple () const {
	return _sh_brush->stipple();
    }
    inline GNumber xtsOrigin () const { return _sh_brush->xtsOrigin(); }
    inline GNumber ytsOrigin () const { return _sh_brush->ytsOrigin(); }

    // Modifying methods
    void modify (ulong mask, const GBrushValues* values);
    void modify (const char* name);
    void modify (const GShBrush* sh_brush);

    void function (ulong function);
    void planeMask (ulong plane_mask);
    void color (const class GColor* color);
    void color (const class GShColor* sh_color);
    void color (const char* color_name);
    void altColor (const class GColor* alt_color);
    void altColor (const class GShColor* sh_alt_color);
    void altColor (const char* alt_color_name);

    void fillStyle (ushort fill_style);
    void fillRule (ushort fill_rule);
    void tile (const class GPattern* tile);
    void tile (const class GShPattern* sh_tile);
    void tile (const char* tile_name);
    void stipple (const class GPattern* stipple);
    void stipple (const class GShPattern* sh_stipple);
    void stipple (const char* stipple_name);
    void tsOrigin (GNumber x_origin, GNumber y_origin);

    // Pre and Post Conditions
    void modifyPrecond (GBool (*)(void*, const GBrush*, ulong&, GBrushValues*),
			const void* closure,
			GAddOrRemove = GADD) const;
    void modifyPostcond (void (*)(void*, GBrush*, ulong), const void* closure,
			 GAddOrRemove = GADD) const;
    void destroyPostcond (void (*)(void*, const GBrush*), const void* closure,
			  GAddOrRemove = GADD) const;
    // friends
    friend void Call_call_modify_postcond (GBrush* This, const GShBrush*,
					   ulong); 

  private:
    GShBrush* _sh_brush;

  protected:
    GBrushCondFlags _flags;
    GAssoc _assoc;

    int _addCallback (const char* type, GCBProc proc, const void* closure) const;
    int _remCallback (const char* type, GCBProc proc, const void* closure) const;

    GBool _call_modify_precond (ulong& modify_mask, GBrushValues* new_values);
    void _call_modify_postcond (ulong modify_mask);
    void _call_destroy_postcond ();
};

#endif  /* _Brush_h */
