/* $Id: Buffer.h,v 1.1.1.1 1992/03/18 09:55:02 uep Exp $ */
/******************************************************************************
MODULE: Tools
FILE:	Buffer.h
AUTHOR:	Aline Jacomet
DATE:   April 24th 1990

DESCRIPTION: classes CharBuf and Buffer definition
	     The class CharBuf defines a memory buffer.
	     The class Buffer defines a linked list of memory buffers. 
	     The class Buffer defines methods to write data in buffers, to
	     read these data from the buffers, to write these data in a file,
	     etc ...


COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
 
#ifndef _Buffer_h
#define _Buffer_h

#include <Go/machdef.h>
#include <Go/Tools/Bool.h>
#include <stdio.h>

// Basic class for linked list;

class GCharBuf {
    friend class GBuffer;
    friend unsigned long GCopy(class GBuffer*, class GBuffer*);

  public:
    
    GCharBuf(ulong s, ulong t) { 
	// s = size in characters;
	buffer = new char[s]; 
	next = 0; 
	first_offset = t + 1;
    }

    ~GCharBuf() { delete buffer; }
	
  private:

    ulong next;   /* Offset a ajouter au pointeur sur le debut de GCharBuf pour
		   avoir le GCharBuf suivant */
    ulong first_offset;
    char *buffer;
};


// Linked list class;

class GBuffer {
    friend class GCharBuf;

  public:
    GBuffer(unsigned int s = BUFSIZ) { 
	// The parameter of GBuffer::GBuffer is the size in bytes !!!!!!;
	size = s / sizeof(char);       // size in characters;
	total_written = 0;
	head = NULL; 
	cursor_write = NULL;
	cursor_read = NULL; 
	cursor_end = NULL; 
	ptr_write = NULL;
	ptr_read = NULL;
	end_write = NULL;
    }

    ~GBuffer(void);

    unsigned long write(const char *,unsigned long);
    // return offset where bytes have been written;

    friend unsigned long GCopy(GBuffer *, GBuffer *);
    // return offset where bytes have been written;

    unsigned long read(char *,unsigned long);
    // return offset where bytes have been read;
    
    char* readString();                         // return the string;
    
    unsigned long writeInFile(FILE *);
    // return number of bytes written in file;
    
    unsigned long readFile(FILE *);
    // return number of bytes read from file;

    int resetWrite(unsigned long offset=1);
    
    int seekWrite(unsigned long);	        // return 0 if OK, otherwise 1;
    
    void appendWrite(void);
    
    int seekRead(unsigned long);		// return 0 if OK, otherwise 1;
    
    void appendRead(void);
    
    unsigned long getNumberBytes(void) {
	return total_written;
    }

    unsigned long getCurrentReadOffset() const {
	return (cursor_read ?
		ptr_read - cursor_read->buffer + cursor_read->first_offset : 0);
    }

    int stepBackRead() {
	if (! cursor_read)
	    return 0;
	else if (ptr_read != cursor_read->buffer) {
	    ptr_read--;
	    return 1;
	}
	else return seekRead(cursor_read->first_offset -1);
    }

  private:
    GCharBuf *head; 	         // start of list;
    GCharBuf *cursor_write;       // GCharBuf which can be written;
    GCharBuf *cursor_read;        // GCharBuf which can be read;
    GCharBuf *cursor_end;         // last written GCharBuf;
    char *ptr_write;             // first byte which can be written;
    char *ptr_read;	         // first byte which can be read;
    char *end_write;	         // last byte in the "cursor_end";
                                 // buffer which has been written by the user;
    unsigned int size; 		 // size (in chars) of GCharBuf buffer;
    unsigned long total_written; // number of written bytes;
    GCharBuf * add(void);

};

#endif /* _Buffer_h */
