/* $Id: Cons.h,v 1.2 1992/09/25 11:54:29 smadja Exp $ */
/*****************************************************************************
MODULE:  Tools
FILE:    Cons.h
AUTHOR:  Olivier Jojic
DATE:    21 Jun 1989

DESCRIPTION:
    These files implement a "lisp like" list tool.

    A Cons contains a "car" (your data) and a "cdr" (the continuation
    of the list).
    A Cons which car is "a" and cdr is "l" is noted: <a l>
    The empty list is noted: GNIL
    So a list with two elements "a" and "b" is noted: <a <b nil>>
       or, with an abbreviated syntax:    <a b>

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#ifndef _Cons_h
#define _Cons_h

#define GNIL 0

class GCons
{
    void* _car;
    GCons* _cdr;

  public:
    //constructors
                  GCons(const void*, const GCons*);
    friend GCons* GConsList(void*, ...);
    friend void   GNewl(GCons*&, const void*);
    friend void   GNewr(GCons*&, const void*);
    friend void   GNconc(GCons*&, const GCons*);
    friend GCons* GAppend(GCons*, const GCons*);

    //destructors
                 ~GCons();
    friend void  GRemq(GCons*&, const void*);
    friend void* GNextl(GCons*&);

    //positionning inside the list
    friend void*   GCar(const GCons*);
    friend GCons*  GCdr(const GCons*);
    friend void*   GCadr(const GCons*);
    friend GCons*  GCddr(const GCons*);
    friend GCons*  GLast(const GCons*);
    friend GCons*& GMemq(GCons*&, const void*);
    
    //updating the list
    friend void  GRplaca(GCons* l, const void* p) {
	l->_car = (void*)p;
    }
    friend void  GRplacd(GCons* l, const GCons* d) {
	l->_cdr = (GCons*)d;
    }
    friend void  GRplac(GCons* l, const void* a, const GCons* d)
	{ l->_car = (void*)a; l->_cdr = (GCons*)d; }

    //various
    friend int GLength(const GCons* l);
};


/**
FUNCTION
    Returns the first elem of the list.
**/
inline void* GCar (const GCons* l) {
    return (l != GNIL) ? l->_car : GNIL;
}

/**
FUNCTION
    Returns the "cdr" of the list.
**/
inline GCons* GCdr (const GCons* l) {
    return (l != GNIL) ? l->_cdr : GNIL;
}


/**
FUNCTION
    Returns the second elem of the list.
    
DESCRIPTION
    The "cdr" of the list must be non nil.
**/
inline void* GCadr (const GCons* l) {
    return l->_cdr->_car;
}

/**
FUNCTION
    Returns the second "cdr" of the list.
    
DESCRIPTION
    The first cdr must be non nil.
**/
inline GCons* GCddr (const GCons* l) {
    return l->_cdr->_cdr;
}


#endif /* _Cons_h */
