/* $Id: Dash.h,v 1.2 1992/12/11 14:29:04 smadja Exp $ */
/*****************************************************************************
MODULE:  Tools
FILE:    Dash.h
AUTHOR:  Jacques Davy
DATE:    November 1990

DESCRIPTION:
    This file describes the Dash object.

    The Dash object gives a regular interface to the shared object
    ShDash (see ShDash.h) and it allows to set some pre/post conditions.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#ifndef _Dash_h
#define _Dash_h

#include <Go/Tools/ShDash.h>
#include <Go/Tools/Assoc.h>
#include <Go/Tools/Callback.h>

/*
 * Condition flags
 */
struct GDashCondFlags
{
    GBool _has_modify_precond : 1;
    GBool _has_modify_postcond : 1;
    GBool _has_destroy_postcond : 1;

    GDashCondFlags() {
	_has_modify_precond  = GFALSE;
	_has_modify_postcond  = GFALSE;
	_has_destroy_postcond  = GFALSE;
    }
};

/*
 * GDash class definition
 */
class GDash
{
  public:
    GDash ();
    GDash (const GDash* dash);
    GDash (const GShDash* sh_dash);
    GDash (ulong mask, const GDashValues* values, const char* name = NULL);
    GDash (const char* name);
    ~GDash ();

    // Getting values
    const GShDash* shDash() const { return _sh_dash; }
    const GDashValues* values() const { return _sh_dash->values(); }

    const char* name() const { return _sh_dash->name(); }
    GBool isNamed() const { return _sh_dash->isNamed(); }

    GNumber dash_offset() const { return _sh_dash->dash_offset(); }
    ushort dash_number() const { return _sh_dash->dash_number(); }
    const GNumber* dash_list () const { return _sh_dash->dash_list(); }

    // Modifying methods
    void modify (ulong mask, const GDashValues* values);
    void modify (const GShDash*);
    void modify (const char* name);

    // Pre and Post Conditions
    void modifyPrecond (GBool (*)(void*, const GDash*, ulong&, GDashValues*),
			const void* closure,
			GAddOrRemove = GADD) const;
    void modifyPostcond (void (*)(void*, GDash*, ulong),
			 const void* closure,
			 GAddOrRemove = GADD) const;
    void destroyPostcond (void (*)(void*, const GDash*),
			  const void* closure,
			  GAddOrRemove = GADD) const;
    // friends
    friend void Call_call_modify_postcond (GDash* This, const GShDash*,ulong); 

  private:
    GShDash* _sh_dash;

  protected:
    GDashCondFlags _flags;
    GAssoc _assoc;

    int _addCallback (const char* type, GCBProc proc,
		      const void* closure) const;
    int _remCallback (const char* type, GCBProc proc,
		      const void* closure) const;

    GBool _call_modify_precond (ulong& modify_mask, GDashValues*);
    void _call_modify_postcond (ulong modify_mask);
    void _call_destroy_postcond ();
};

#endif /* _Dash_h */
