/* $Id: ImageBrush.h,v 1.1.1.1 1992/03/18 09:56:06 uep Exp $ */
/*****************************************************************************
MODULE:  GoGC
FILE:    ImageBrush.h
AUTHOR:  Jacques Davy
DATE:    November 1990

DESCRIPTION:
    This file describes the ImageBrush object.

    The ImageBrush object provides a regular object interface of a shared object
    ShImageBrush (see ShImageBrush.h). It also allows to set some pre/post
    conditions.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#ifndef _ImageBrush_h
#define _ImageBrush_h

#include <Go/Tools/ShImageBrush.h>
#include <Go/Tools/Assoc.h>
#include <Go/Tools/Callback.h>

/*
 * Condition flags
 */
struct GImageBrushCondFlags
{
    GBool _has_modify_precond : 1;
    GBool _has_modify_postcond : 1;
    GBool _has_destroy_postcond : 1;

    GImageBrushCondFlags() {
	_has_modify_precond  = GFALSE;
	_has_modify_postcond  = GFALSE;
	_has_destroy_postcond  = GFALSE;
    }
};

/*
 * GImageBrush class definition
 */
class GImageBrush
{
  private:
    GShImageBrush* _sh_ibrush;


  protected:
    GImageBrushCondFlags _flags;
    GAssoc _assoc;

    int _addCallback (const char* type, GCBProc proc, const void* closure) const;
    int _remCallback (const char* type, GCBProc proc, const void* closure) const;

    GBool _call_modify_precond (ulong& modify_mask, GImageBrushValues* new_values);
    void _call_modify_postcond (ulong modify_mask);
    void _call_destroy_postcond ();


  public:
    GImageBrush ();
    GImageBrush (const GImageBrush* ibrush);
    GImageBrush (const GShImageBrush* sh_tbrush);
    GImageBrush (ulong mask, const GImageBrushValues* values, const char* name = NULL);
    GImageBrush (const char* name);
    ~GImageBrush ();

    // Getting values
    inline const GShImageBrush* shImageBrush() const { return _sh_ibrush; }
    inline const GImageBrushValues* values() const { return _sh_ibrush->values(); }
    inline ulong mask() const { return _sh_ibrush->mask(); }

    inline const char* name() const { return _sh_ibrush->name(); }
    inline GBool isNamed() const { return _sh_ibrush->isNamed(); }

    inline ulong function () const { return _sh_ibrush->function(); }
    inline ulong planeMask () const { return _sh_ibrush->planeMask(); }
    inline ulong fillStyle () const { return _sh_ibrush->fillStyle(); }
    inline const class GShColormap* colormap () const { return _sh_ibrush->colormap(); }
    inline const class GShPattern* stipple () const { return _sh_ibrush->stipple(); }
    
    // Modifying methods
    void modify (ulong mask, const GImageBrushValues* values);
    void modify (const char* name);
    void modify (const GShImageBrush* sh_ibrush);

    void function (ulong function);
    void planeMask (ulong plane_mask);
    void fillStyle (ushort fill_style);

    void colormap (const class GColormap* colormap);
    void colormap (const class GShColormap* sh_colormap);
    void colormap (const char* colormap_name);

    void stipple (const class GPattern* stipple);
    void stipple (const class GShPattern* sh_stipple);
    void stipple (const char* stipple_name);

    // Pre and Post Conditions
    void modifyPrecond (GBool (*)(void*, const GImageBrush*, ulong&, GImageBrushValues*),
			const void* closure,
			GAddOrRemove = GADD) const;
    void modifyPostcond (void (*)(void*, GImageBrush*, ulong), const void* closure,
			 GAddOrRemove = GADD) const;
    void destroyPostcond (void (*)(void*, const GImageBrush*), const void* closure,
			  GAddOrRemove = GADD) const;
    // friends
    friend void Call_call_modify_postcond (GImageBrush* This,
					   const GShImageBrush*, ulong); 
};

#endif  /* _ImageBrush_h */
