/* $Id: Matrix.h,v 1.1.1.1 1992/03/18 09:56:28 uep Exp $ */
/*****************************************************************************
MODULE:  Tools
FILE:    Matrix.h
AUTHOR:  Olivier Jojic
DATE:    jan 90

DESCRIPTION:

The Matrix class  with classic functions.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
*****************************************************************************/
#ifndef _Matrix_h
#define _Matrix_h

#include <Go/Tools/Vector.h>


class GMatrix
{
  public:

    GMatrix();
    GMatrix(const GVector&, const GVector&);
    GMatrix(GNumber angle);
    GMatrix(const GMatrix&);

    friend GMatrix GRotationMatrixFromVectors(const GVector&, const GVector&);

    const GVector& vx() const;
    const GVector& vy() const;

    GMatrix& operator *= (GNumber);
    GMatrix& operator /= (GNumber);
    GMatrix& rotate(GNumber angle);

    GMatrix operator ~ () const;
    friend GMatrix operator * (const GMatrix&, const GMatrix&);
    friend GVector operator * (const GMatrix&, const GVector&);
    friend GVector& operator *= (GVector&, const GMatrix&);
    GNumber deter() const;

  private:
    
    GVector _vx;
    GVector _vy;
    
};


/*
 * inline methods:
 */

inline const GVector& GMatrix::vx () const
{
    return _vx;
}

inline const GVector& GMatrix::vy () const
{
    return _vy;
}

inline GMatrix::GMatrix () : _vx(1, 0), _vy(0, 1)
{
}

inline GMatrix::GMatrix (const GVector& x, const GVector& y) : _vx(x), _vy(y)
{
}

inline GMatrix::GMatrix (GNumber angle)
    : _vx(cos(angle), sin(angle)), _vy(sin(angle), -cos(angle))
{
}

inline GMatrix::GMatrix (const GMatrix& m) : _vx(m._vx), _vy(m._vy)
{
}

inline GMatrix& GMatrix::operator *= (GNumber n)
{
    _vx *= n;
    _vy *= n;
    return *this;
}

inline GMatrix& GMatrix::operator /= (GNumber n)
{
    _vx /= n;
    _vy /= n;
    return *this;
}

inline GMatrix& GMatrix::rotate (GNumber angle)
{
    _vx.rotate(angle);
    _vy.rotate(angle);
    return *this;
}

inline GNumber GMatrix::deter () const
{
    return _vx.x * _vy.y - _vx.y * _vy.x;
}

#endif /* _Matrix_h */
