/* $Id: PixelArray.h,v 1.1.1.1 1992/03/18 09:56:51 uep Exp $ */
/*****************************************************************************
MODULE:  Tools
FILE:    PixelArray.h
AUTHOR:  Jacques Davy
DATE:    November 1990

DESCRIPTION:
    This file describes the PixelArray object.

    The PixelArray object gives a regular interface to the shared object
    ShPixelArray(see ShPxlArray.h) and it allows to set some pre/post conditions.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#ifndef _PixelArray_h
#define _PixelArray_h


#include <Go/Tools/ShPixelArray.h>
#include <Go/Tools/Assoc.h>
#include <Go/Tools/Callback.h>

/*
 * Condition flags
 */
struct GPxlArrayCondFlags
{
    GBool _has_modify_precond : 1;
    GBool _has_modify_postcond : 1;
    GBool _has_destroy_postcond : 1;

    GPxlArrayCondFlags() {
	_has_modify_precond  = GFALSE;
	_has_modify_postcond  = GFALSE;
	_has_destroy_postcond  = GFALSE;
    }
};

/*
 * GPixelArray class definition
 */
class GPixelArray
{
  public:
    GPixelArray ();
    GPixelArray (const GPixelArray* pixelarray);
    GPixelArray (const GShPixelArray* sh_pxl_array);
    GPixelArray (ulong mask, const GPixelArrayValues* values,
		 const char* name = NULL,
		 GDataSpace = GDATA_COPY);
    GPixelArray (const char* name);
    ~GPixelArray ();
    
    // Getting values;
    inline const GShPixelArray* shPixelArray() const { return _sh_pixelArray; }
    inline const GPixelArrayValues* values() const {
	return _sh_pixelArray->values();
    }
    inline const ulong mask() const { return _sh_pixelArray->mask(); }
    
    inline const char* name() const { return _sh_pixelArray->name(); }
    inline GBool isNamed() const { return _sh_pixelArray->isNamed(); }

    inline ulong bytesPerLine () const {
	return _sh_pixelArray->bytesPerLine();
    }
    inline ulong dataSize () const { return _sh_pixelArray->dataSize(); }

    // Modifying methods;
    void modify (ulong mask, const GPixelArrayValues* values);
    void modify (const GShPixelArray*);
    void modify (const char* name);
    
    // Getting subPixelArray's;
    inline GShPixelArray* subPixelArray (ushort x, ushort y, ushort width, ushort height) const
                                       { return _sh_pixelArray->subPixelArray (x, y, width, height); }
    inline GShPixelArray* subPixelArray (GNumber x, GNumber y, GNumber width, GNumber height) const
                                       { return _sh_pixelArray->subPixelArray (x, y, width, height); }
    inline void subPixelArrayValues (ushort x, ushort y, ushort width, ushort height, GPixelArrayValues& v) const
                                       { _sh_pixelArray->subPixelArrayValues (x, y, width, height, v); }

    // Pre and Post Conditions;
    void modifyPrecond (GBool (*)(void*, const GPixelArray*, ulong&, GPixelArrayValues*),
                        const void* closure,
			GAddOrRemove = GADD) const;
    void modifyPostcond (void (*)(void*, GPixelArray*, ulong),
                         const void* closure,
			 GAddOrRemove = GADD) const;
    void destroyPostcond (void (*)(void*, const GPixelArray*),
                          const void* closure,
			  GAddOrRemove = GADD) const;
    // friends;
    friend void Call_call_modify_postcond (GPixelArray* This,
                                           const GShPixelArray*, ulong); 

  private:
    GShPixelArray* _sh_pixelArray;

  protected:
    GPxlArrayCondFlags _flags;
    GAssoc _assoc;

    int _addCallback (const char* type, GCBProc proc, const void* closure) const;
    int _remCallback (const char* type, GCBProc proc, const void* closure) const;

    GBool _call_modify_precond (ulong& modify_mask, GPixelArrayValues*);
    void _call_modify_postcond (ulong modify_mask);
    void _call_destroy_postcond ();
};

#endif /* _PixelArray_h */
