/* $Id: ShImageBrush.h,v 1.6 1992/09/25 11:51:56 smadja Exp $ */
/*****************************************************************************
MODULE:  Tools/GC
FILE:    ShImageBrush.h
AUTHOR:  Jacques Davy
DATE:    October 1990

DESCRIPTION:
    This file describes the ShImageBrush shared object.
    The ShImageBrush object is used by the GoInterf objects when drawing images.
    The ImageBrushValues structure may be used to set ShImageBrush parameters.
    Unset parameters have default values (se below).

    ShImageBrush objects are shared objects (SharedObject subclass). See
    also ImageBrush objects.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  license from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#ifndef _ShImageBrush_h
#define _ShImageBrush_h

#include <Go/machdef.h>
#include <Go/Tools/SharedObj.h>
#include <Go/Tools/Number.h>

/* Mask definitions for GSharedObject operations */
#include <Go/Tools/GCdef.h>

/* Value Structure for GSharedObject operations (with defaults) */
struct GImageBrushValues {
    ulong function;             // logical operation (Gcopy)
    ulong plane_mask;           // plane mask        (GallPlanes)

    /* fill attributes */
    struct {
       unsigned style : 2;      // fill style (GfillSolid or GFillStipple)
    } fill;

    /* image attributes */

    /** CLASS OUTLINE*: TO BE DEFINED FOR GENERAL OUTLINE HANDLING **/
    class GShPattern* stipple;   // stipple 1 plane pattern for stippling (none)
    class GShColormap* colormap; // colormap object   (unamed white and black colormap)
};

#define GMODIFY_IMAGEBRUSHVALUE(image_brush, maskBit, field, newValue)\
    {\
        GImageBrushValues values;\
        values.field = newValue;\
        GModify(image_brush, maskBit, &values);\
    }

#define GUPDATE_IMAGEBRUSHVALUE(name, maskBit, field, newValue)\
    {\
        GImageBrushValues values;\
	values.field = newValue;\
        GModify (name, maskBit, &values);\
    }

/* GShImageBrush class definition */
class GShImageBrush : public GSharedObject
{
    friend class GImageBrush;

  public:

    friend GShImageBrush* GCreateShImageBrush ();
    friend GShImageBrush* GCreateShImageBrush (ulong mask,
					       const GImageBrushValues* values,
					       const char* name = NULL);
    friend GShImageBrush* GCreateShImageBrush (const char* name);

    static int save(const GShImageBrush*, GoStream*);
    static int restore(GShImageBrush**, GoStream*);
    static const char* classname();
    static int readParams(GShImageBrush*, GoStream*);

    virtual const char* getClassname() const;

    inline GShImageBrush* dup() const;
    inline GBool match (const GShImageBrush* image_brush) const;
    inline void updatePostcond (void (*)(void*, GShImageBrush*, ulong),
				const void* closure, GAddOrRemove = GADD) const;
    inline void destroyPostcond (void (*)(void*, const GShImageBrush*),
				 const void* closure, GAddOrRemove = GADD) const;

    // Getting values;
    inline const GImageBrushValues* values() const { return &_values; }

    inline ulong function () const { return _values.function; }
    inline ulong planeMask () const { return _values.plane_mask; }
    inline ushort fillStyle () const { return _values.fill.style; }
    inline const class GShColormap* colormap () const { return _values.colormap; }
    inline const class GShPattern*  stipple () const { return _values.stipple;}

    // Modifying methods;
    friend void GModify (GShImageBrush*& image_brush, ulong mask,
			 const GImageBrushValues* values);
    friend void GRename (GShImageBrush*& image_brush, const char* name);

    friend void GSetPlaneMask (GShImageBrush*& image_brush, ulong plane_mask);
    friend void GSetColormap (GShImageBrush*& image_brush,
			      const class GShColormap* colormap);
    friend void GSetStipple (GShImageBrush*& image_brush,
			     const class GShPattern* stipple);

    // updating named GShImageBrush;
    friend void GModify (const char* name, ulong mask,
			 const GImageBrushValues* values);

    friend void GImageBrushFunction (const char* name, ulong function);
    friend void GImageBrushPlaneMask (const char* name, ulong plane_mask);
    friend void GImageBrushFillStyle (const char* name, ulong fill_style);
    friend void GImageBrushColormap (const char* name,
				     const class GShColormap* colormap);
    friend void GImageBrushStipple(const char* name,
				   const class GShPattern* stipple);

    // friends for callbacks;
    friend void ColormapIsUpdated(GShImageBrush* This,
				  const GShColormap* colormap,
				  ulong mask);
    friend void StippleIsUpdated(GShImageBrush* This,
				 const GShPattern* stipple,
				 ulong mask);

  protected:

    inline GList*& _head () { return _headOfList; }
    inline int  _valueSize () { return (sizeof(GImageBrushValues)); }
    virtual GBool _match (ulong mask, const void* values) const;
    virtual void _makeNewValues (ulong mask, const void* values,
				 ulong& newMask, void* newValues) const;
    virtual ulong _update (ulong mask, const void* values);
    friend GShImageBrush* _newGShImageBrush (GFunctionList* funcList, ulong mask,
					     const void* values,
					     const char* name = NULL);

    GShImageBrush (GFunctionList* funcList, ulong mask,
		   const GImageBrushValues* values,
		   const char* name = NULL);
    virtual ~GShImageBrush ();


  private:

    GImageBrushValues _values;
    static GList* _headOfList;
    static GFunctionList _funcList;
};

/*
 * Inline Methods
 */
inline void GModify (GShImageBrush*& image_brush,
		     ulong mask, const GImageBrushValues* values)
    { image_brush->modify((GSharedObject*&)image_brush, mask, (const void*)values); }

inline void GRename (GShImageBrush*& image_brush, const char* name)
    { image_brush->rename ((GSharedObject*&)image_brush, name); }

inline void GSetPlaneMask (GShImageBrush*& image_brush, ulong plane_mask)
    { GMODIFY_IMAGEBRUSHVALUE(image_brush, GPLANE_MASK, plane_mask, plane_mask); }

inline void GSetColormap (GShImageBrush*& image_brush, const class GShColormap* colormap)
    { GMODIFY_IMAGEBRUSHVALUE(image_brush, GPLANE_MASK, colormap, (class GShColormap*)colormap); }

inline void GSetStipple (GShImageBrush*& image_brush, const class GShPattern* stipple)
    { GMODIFY_IMAGEBRUSHVALUE(image_brush, GPLANE_MASK, stipple, (class GShPattern*)stipple); }

/**/
inline void GImageBrushFunction (const char* name, ulong function)
    { GUPDATE_IMAGEBRUSHVALUE (name, GFUNCTION, function, function); }

inline void GImageBrushPlaneMask (const char* name, ulong plane_mask)
    { GUPDATE_IMAGEBRUSHVALUE (name, GPLANE_MASK, plane_mask, plane_mask); }

inline void GImageBrushFillStyle (const char* name, ulong fill_style)
    { GUPDATE_IMAGEBRUSHVALUE (name, GPLANE_MASK, fill.style, fill_style); }

inline void GImageBrushColormap (const char* name, const class GShColormap* colormap)
    { GUPDATE_IMAGEBRUSHVALUE (name, GPLANE_MASK, colormap, (class GShColormap*)colormap); }

inline void GImageBrushStipple (const char* name, const class GShPattern* stipple)
    { GUPDATE_IMAGEBRUSHVALUE (name, GPLANE_MASK, stipple, (class GShPattern*)stipple); }

/**/
inline GShImageBrush* GShImageBrush::dup() const
    { return (GShImageBrush*)((GSharedObject*)this)->dup(); }

/**/
inline GBool GShImageBrush::match (const GShImageBrush* image_brush) const
    { return ((GShImageBrush*)this)->_match (image_brush->mask(), image_brush->values()); }

/**/
inline void GShImageBrush::updatePostcond (void cb(void*, GShImageBrush*, ulong),
				 const void* closure, GAddOrRemove add) const
{
    GSharedObject::updatePostcond (
                  (void(*)(void*, GSharedObject*, ulong))cb, closure, add);
}

/**/
inline void GShImageBrush::destroyPostcond (void cb(void*, const GShImageBrush*),
				  const void* closure, GAddOrRemove add) const
{
    GSharedObject::destroyPostcond ((void(*)(void*, const GSharedObject*))cb,
				    closure, add);
}

#undef GMODIFY_IMAGEBRUSHVALUE
#undef GUPDATE_IMAGEBRUSHVALUE

#endif /* _ShImageBrush_h */
