/* $Id: Stack.h,v 1.3 1992/12/11 14:25:19 smadja Exp $ */
/*****************************************************************************
MODULE:  Tools
FILE:    Stack.h
AUTHOR:  Oliver Jojic
DATE:    October 1990

DESCRIPTION:
    

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#ifndef _Stack_h
#define _Stack_h

#include <Go/machdef.h>
#include <Go/Tools/Error.h>
#include <Go/Tools/Bool.h>


class GStack
{
  public:
    GStack();
    ~GStack();

    void push(void*);
    void* pop();

    GBool empty() const;
    void clear();
    void* top() const;
    void top(void*);
    void** pointer () const { return &array[index]; }
    void** data (int& count) const { count = index; return array; }

  private:
    int array_size;
    int index;
    void** array;
    
    void incr_array();

};


INLINE void GStack::push (void* obj)
{
    if (index == array_size)
	incr_array();
    array[index] = obj;
    index++;
}

INLINE GBool GStack::empty () const
{
    return index == 0;
}

#ifndef INLINE_NO_RETURN
INLINE
#endif
#if !defined(INLINE_NO_RETURN) || defined(__CC_OF_STACK__)
void* GStack::pop ()
{
    if (empty())
    {
	GWarning("Stack.h:pop(): stack is empty !");
	return NULL;
    }

    return array[--index];
}
#endif


#ifndef INLINE_NO_RETURN
INLINE
#endif
#if !defined(INLINE_NO_RETURN) || defined(__CC_OF_STACK__)
void* GStack::top () const
{
    if (index == 0)
    {
	GWarning("Stack.h:top(): stack is empty");
	return NULL;
    }

    return array[index - 1];
}
#endif

#ifndef INLINE_NO_RETURN
INLINE
#endif
#if !defined(INLINE_NO_RETURN) || defined(__CC_OF_STACK__)
void GStack::top (void* obj)
{
    if (index == 0)
    {
	GWarning("Stack.h:top(void*): stack is empty");
	return;
    }

    array[index - 1] = obj;
}
#endif

#endif /* _Stack_h */
