/* $Id: TextBrush.h,v 1.1.1.1 1992/03/18 09:58:40 uep Exp $ */
/*****************************************************************************
MODULE:  Tools/GC
FILE:    TextBrush.h
AUTHOR:  Jacques Davy
DATE:    November 1990

DESCRIPTION:
    This file describes the GTextBrush object.

    The GTextBrush object provides a regular object interface of a shared object
    GShTextBrush (see ShTextBrush.h). It also allows to set some pre/post
    conditions.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#ifndef _TextBrush_h
#define _TextBrush_h

#include <Go/Tools/ShTextBrush.h>
#include <Go/Tools/Assoc.h>
#include <Go/Tools/Callback.h>

/*
 * Condition flags
 */
struct GTextBrushCondFlags
{
    GBool _has_modify_precond : 1;
    GBool _has_modify_postcond : 1;
    GBool _has_destroy_postcond : 1;

    GTextBrushCondFlags() {
	_has_modify_precond  = GFALSE;
	_has_modify_postcond  = GFALSE;
	_has_destroy_postcond  = GFALSE;
    }
};

/*
 * GTextBrush class definition
 */
class GTextBrush
{
  public:
    GTextBrush ();
    GTextBrush (const GTextBrush* tbrush);
    GTextBrush (const GShTextBrush* sh_tbrush);
    GTextBrush (ulong mask, const GTextBrushValues* values, const char* name = NULL);
    GTextBrush (const char* name);
    ~GTextBrush ();

    // Getting values
    inline const GShTextBrush* shTextBrush() const { return _sh_tbrush; }
    inline const GTextBrushValues* values() const { return _sh_tbrush->values(); }
    inline ulong mask() const { return _sh_tbrush->mask(); }

    inline const char* name() const { return _sh_tbrush->name(); }
    inline GBool isNamed() const { return _sh_tbrush->isNamed(); }

    inline ulong function () const { return _sh_tbrush->function(); }
    inline ulong planeMask () const { return _sh_tbrush->planeMask(); }
    inline const class GShColor* color () const { return _sh_tbrush->color(); }
    inline const class GShColor* altColor () const { return _sh_tbrush->altColor(); }

    inline ushort fillStyle () const { return _sh_tbrush->fillStyle(); }
    inline const class GShPattern* tile () const { return _sh_tbrush->tile(); }
    inline const class GShPattern* stipple () const { return _sh_tbrush->stipple(); }
    inline GNumber xtsOrigin () const { return _sh_tbrush->xtsOrigin(); }
    inline GNumber ytsOrigin () const { return _sh_tbrush->ytsOrigin(); }

    inline const class GShFont* font () const {return _sh_tbrush->font(); }

    // Modifying methods
    void modify (ulong mask, const GTextBrushValues* values);
    void modify (const char* name);
    void modify (const GShTextBrush* sh_tbrush);

    void function (ulong function);
    void planeMask (ulong plane_mask);
    void color (const class GColor* color);
    void color (const class GShColor* sh_color);
    void color (const char* color_name);
    void altColor (const class GColor* alt_color);
    void altColor (const class GShColor* sh_alt_color);
    void altColor (const char* alt_color_name);

    void fillStyle (ushort fill_style);
    void tile (const class GPattern* tile);
    void tile (const class GShPattern* sh_tile);
    void tile (const char* tile_name);
    void stipple (const class GPattern* stipple);
    void stipple (const class GShPattern* sh_stipple);
    void stipple (const char* stipple_name);
    void tsOrigin (GNumber x_origin, GNumber y_origin);

    void font (const class GFont* font);
    void font (const class GShFont* sh_font);
    void font (const char* font_name);

    // Pre and Post Conditions
    void modifyPrecond (GBool (*)(void*, const GTextBrush*, ulong&, GTextBrushValues*),
			const void* closure,
			GAddOrRemove = GADD) const;
    void modifyPostcond (void (*)(void*, GTextBrush*, ulong), const void* closure,
			 GAddOrRemove = GADD) const;
    void destroyPostcond (void (*)(void*, const GTextBrush*), const void* closure,
			  GAddOrRemove = GADD) const;
    // friends
    friend void Call_call_modify_postcond (GTextBrush* This, const GShTextBrush*, ulong); 


  private:
    GShTextBrush* _sh_tbrush;


  protected:
    GTextBrushCondFlags _flags;
    GAssoc _assoc;

    int _addCallback (const char* type, GCBProc proc, const void* closure) const;
    int _remCallback (const char* type, GCBProc proc, const void* closure) const;

    GBool _call_modify_precond (ulong& modify_mask, GTextBrushValues* new_values);
    void _call_modify_postcond (ulong modify_mask);
    void _call_destroy_postcond ();
};

#endif  /* _TextBrush_h */
