/* $Id: Tree.h,v 1.5 1992/12/11 08:35:27 jojic Exp $ */
/*****************************************************************************
MODULE:  BULL Graphic Objects
FILE:    Tree.h
AUTHOR:  Aline Jacomet and Gabriel Jureidini
DATE:    January 11th 1991

DESCRIPTION:
         Definition of the tree object class.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#ifndef _GoTree_h
#define _GoTree_h

#include <Go/Go.h>
#include <Go/GraphStr.h>
#include <Go/mTree.h>
#include <Go/TreeNode.h>
#include <Go/Link.h>




/*
 * GoTree definition
 */
class GoTree : public GoGraphStruct {
    friend class GoTreeMetaclass;
    
  public:
    /*
     * set ...;
     */
    GBool distX(GNumber);
    GBool distY(GNumber);
    GBool alignmentType(ushort);

    /*
     * get ...;
     */
    inline GNumber distX() const {
	return dist_x;
    }
    inline GNumber distY() const {
	return dist_y;
    }
    inline ushort alignmentType() const {
	return alignment_type;
    }

    /*
     * Pre and Post Conditions ...;
     */
    void addNodePrecond(GBool (*)(void*, GoTree*, GoTreeNode*, GoMetaclass*,
				  GoTreeNode*),
			void* closure, GAddOrRemove = GADD);
    void addNodePostcond(void (*)(void*, GoTree*, GoTreeNode*, GoMetaclass*,
				  GoTreeNode*),
			 void* closure, GAddOrRemove = GADD);
    void removeNodePrecond(GBool (*)(void*, GoTree*, GoTreeNode*),
			   void* closure, GAddOrRemove = GADD);
    void removeNodePostcond(void (*)(void*, GoTree*, GoTreeNode*),
			    void* closure, GAddOrRemove = GADD);
    
    /*
     * others ...;
     */
    static GoTreeMetaclass* metaclass();
    static GoTree*	    create();
    
    GoTreeNode* addNode(GoTreeNode*, GoMetaclass* linkclass = NULL,
			GoTreeNode* = NULL);
    GBool removeNode(GoTreeNode*);

    /*
     * programmers, don't look at these functions.
     */
    inline void setDistX(GNumber d) {
	distX(d);
    }
    inline void setDistY(GNumber d) {
	distY(d);
    }
    inline void setAlignmentType(ushort u) {
	alignmentType(u);
    }

    /*
     * save, restore;
     */
    static int save(const GoTree*, GoStream*);
    static int restore(GoTree**, GoStream*);
    static int readParams(GoTree*, GoStream*);

  protected:
    GoTreeCondFlags _flags;

    virtual void m_dup(const Go*, Go*, Go*, GBool, GBool);
    GoTree();
    virtual ~GoTree();

  private:
    GNumber dist_x;  
    GNumber dist_y;
    ushort alignment_type;

    virtual GoMetaclass* m_metaclass() const;

    void remove(GoTreeNode*);
    void reparent_subtree(GoTreeNode*, GoTree*);
    
    void left_right_placement(GoNode*, const GPoint&);
    void find_fields_h(GoTreeNode*, GNumber*, GNumber*, GNumber*, GNumber*,
		       GNumber*, int*, int*, GNumber*, GBool*, GPoint*);
    void gmnhg(GoTreeNode*, GNumber*, const GPoint&, GPoint*);
    void modif_node1_h(GoTreeNode*, GNumber*, GNumber, GNumber, GNumber*, int*,
		       GNumber*, GPoint*);
    void right_left_placement(GoNode*, const GPoint&);
    void find_fields_h2(GoTreeNode*, GNumber*, GNumber*, GNumber*, GNumber*,
			GNumber*, int*, int*, GNumber*, GBool*, GPoint*);
    void gmnhg2(GoTreeNode*, GNumber*, const GPoint&, GPoint*);
    void modif_node2_h(GoTreeNode*, GNumber*, GNumber, GNumber, GNumber*, int*,
		       GNumber*, GPoint*);
    void up_bottom_placement(GoNode*, const GPoint&);
    void find_fields_v(GoTreeNode*, GNumber*, GNumber*, GNumber*, GNumber*,
		       GNumber*, int*, int*, GNumber*, GBool*, GPoint*);  
    void gmng(GoTreeNode*, GNumber*, const GPoint&, GPoint*);   
    void modif_node1_v(GoTreeNode*, GNumber*, GNumber, GNumber, GNumber*, int*,
		       GNumber*, GPoint*);
    void bottom_up_placement(GoNode*, const GPoint&);
    void find_fields_v2(GoTreeNode*, GNumber*, GNumber*, GNumber*, GNumber*,
			GNumber*, int*, int*, GNumber*, GBool*, GPoint*);  
    void gmng2(GoTreeNode*, GNumber*, const GPoint&, GPoint*);   
    void modif_node2_v(GoTreeNode*, GNumber*, GNumber, GNumber, GNumber*, int*,
		       GNumber*, GPoint*);
    
    GBool _call_add_node_precond(GoTreeNode*, GoMetaclass*, GoTreeNode*) const;
    void _call_add_node_postcond(GoTreeNode*, GoMetaclass*, GoTreeNode*);
    GBool _call_remove_node_precond(GoTreeNode*) const;
    void _call_remove_node_postcond(GoTreeNode*);

    GBool _call_class_add_node_precond( GoTreeNode*, GoMetaclass*,
				       GoTreeNode*) const;
    void  _call_class_add_node_postcond(GoTreeNode*, GoMetaclass*,
					GoTreeNode*);
    GBool _call_class_remove_node_precond(GoTreeNode*) const;
    void  _call_class_remove_node_postcond(GoTreeNode*);
};

INLINE GoTree* GoTree::create() {
    return (GoTree*) Go::Create(GoTree::metaclass());
}

#endif /* _GoTree_h */
