/* $Id: mArc.h,v 1.1.1.1 1992/03/18 10:05:34 uep Exp $ */
/*****************************************************************************
MODULE:  Graphic Objects
FILE:    mArc.h
DATE:    Feb 1991

DESCRIPTION:

  GoArc Metaclass Description
  
COPYRIGHT :
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
*****************************************************************************/
#ifndef _mGoArc_h
#define _mGoArc_h

#include <Go/mGeometric.h>

class GoArcMetaclass : public GoGeometricMetaclass {
    friend class GoArc;
    
  public:

    // pre/post conditions;
    void radiusXPrecond(GBool func(void*, const GoArc*, GNumber*),
			void* closure, GAddOrRemove = GADD);
    void radiusXPostcond(void func(void*, GoArc*),
			 void* closure, GAddOrRemove = GADD);
    void radiusYPrecond(GBool func(void*, const GoArc*, GNumber*),
			void* closure, GAddOrRemove = GADD);
    void radiusYPostcond(void func(void*, GoArc*),
			 void* closure, GAddOrRemove = GADD);
    void amplitudePrecond(GBool func(void*, const GoArc*, GNumber*),
			  void* closure, GAddOrRemove = GADD);
    void amplitudePostcond(void func(void*, GoArc*),
			   void* closure, GAddOrRemove = GADD);
    void startPrecond(GBool func(void*, const GoArc*, GNumber*),
		      void* closure, GAddOrRemove = GADD);
    void startPostcond(void func(void*, GoArc*),
		       void* closure, GAddOrRemove = GADD);
    void resetPrecond(GBool func(void*, const GoArc*,
				GNumber*, GNumber*, GNumber*, GNumber*),
		      void* closure, GAddOrRemove = GADD);
    void resetPostcond(void func(void*, GoArc*),
		       void* closure, GAddOrRemove = GADD);
    
    void piePrecond(GBool func(void*, const GoArc*, GBool*),
		    void* closure, GAddOrRemove = GADD);
    void piePostcond(void func(void*, GoArc*),
		     void* closure, GAddOrRemove = GADD);
    void closedPrecond(GBool func(void*, const GoArc*, GBool*),
		       void* closure, GAddOrRemove = GADD);
    void closedPostcond(void func(void*, GoArc*),
			void* closure, GAddOrRemove = GADD);

  protected:
    
    GoArcMetaclass();
    virtual ~GoArcMetaclass();

  private:

    // pre/post condition flags;
    GBool _has_radiusX_precond    :1;
    GBool _has_radiusX_postcond   :1;
    GBool _has_radiusY_precond    :1;
    GBool _has_radiusY_postcond   :1;
    GBool _has_amplitude_precond  :1;
    GBool _has_amplitude_postcond :1;
    GBool _has_start_precond      :1;
    GBool _has_start_postcond     :1;
    GBool _has_reset_precond      :1;
    GBool _has_reset_postcond     :1;

    GBool _has_pie_precond	 :1;
    GBool _has_pie_postcond	 :1;
    GBool _has_closed_precond	 :1;
    GBool _has_closed_postcond	 :1;

    virtual const char*       m_classname() const;
    virtual GoMetaclass*      m_superclass() const;
    virtual const GActionRec* m_actions() const;
    virtual Go*               m_create() const;
};

#endif /* _mGoArc_h */

