/* $Id: mPointSensor.h,v 1.1 1992/12/10 15:38:23 jojic Exp $ */
/*****************************************************************************
MODULE:  Go User Interaction
FILE:    mPointtSensor.h
AUTHOR:  Gilles Benati
DATE:    September 20th, 1991

DESCRIPTION:

  GoPointSensor Metaclass description
	
COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#ifndef _mPointSensor_h
#define _mPointSensor_h

#include <Go/StdSensor.h>
#include <Go/Tools/Pen.h>
#include <Go/Tools/Brush.h>

class GoPointSensorMetaclass : public GoStdSensorMetaclass {
    friend class GoPointSensor;

  public:
    
    // do not use the class constructor and destructor;
    GoPointSensorMetaclass();
    virtual ~GoPointSensorMetaclass();

    // some silly compilers complain if "return _pen" or "return _brush";
    GPen&   pen()    const { return ((GoPointSensorMetaclass*) this)->_pen; }
    GBrush& brush()  const { return ((GoPointSensorMetaclass*) this)->_brush; }
    GNumber radius() const { return _radius; }

    virtual GBool radius(GNumber);

    // pre/post conditions;
    void radiusPrecond(GBool f(void*, const Go* /*NULL*/,
			       const GoPointSensorMetaclass*, GNumber*),
		       void* closure, GAddOrRemove = GADD);
    void radiusPostcond(void f(void*, Go* /*NULL*/, GoPointSensorMetaclass*),
			void* closure, GAddOrRemove = GADD);

  protected:

    const GShPen* sh_pen() const { return _old_sh_pen; }
    void  init_sh_pen() {
	if (_old_sh_pen)
	    _old_sh_pen->destroy();
	_old_sh_pen = _pen.shPen()->dup();
    }

  private:

    GShPen* _old_sh_pen;
    GPen    _pen;
    GBrush  _brush;
    GNumber _radius;

    // pre/post conditions flags;
    GBool _has_radius_precond     :1;
    GBool _has_radius_postcond    :1;

    // private postconditions on _brush and _pen modifications;
    static void _pen_changed(GoPointSensorMetaclass*, const GPen*, ulong);
    static void _brush_changed(GoPointSensorMetaclass*, const GBrush*, ulong);
    
    // call pre/post conditions;
    GBool _call_radius_precond(GNumber*) const;
    void  _call_radius_postcond();

    virtual Go*          m_create() const;
    virtual const char*  m_classname() const;
    virtual GoMetaclass* m_superclass() const;
    virtual const GActionRec* m_actions () const;
    virtual void         m_check(const Go*) const;
};



#endif /* _mPointSensor_h */

