/* $Id: mPolypoint.h,v 1.1.1.1 1992/03/18 10:08:06 uep Exp $ */
/*****************************************************************************
MODULE: Graphic Objects
FILE:   mPolypoint.h
AUTHOR: Aboobacker T.A, Gilles Benati
DATE:   Feb 1991

DESCRIPTION :

       This is the base class for polyline and curve families. It manages an
  array of points and provides common functions to manipulate this array :
  reset the array, add, move or delete a point...
  The origin of the coord system is not related to any of the point in the
  point array, i.e. adding or (re)moving points will not affect the coord sys.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
*****************************************************************************/
#ifndef _mGoPolypoint_h
#define _mGoPolypoint_h

#include <Go/mGeometric.h>

class GoPolypointMetaclass : public GoGeometricMetaclass {
    friend class GoPolypoint;
    
  public:
    
    // pre/post conditions;
    void addPointPrecond(GBool func(void*, const GoPolypoint*, int*, GPoint*),
			 void* closure, GAddOrRemove = GADD);
    void addPointPostcond(void func(void*, GoPolypoint*, int),
			  void* closure, GAddOrRemove = GADD);
    void delPointPrecond(GBool func(void*, const GoPolypoint*, int*),
			 void* closure, GAddOrRemove = GADD);
    void delPointPostcond(void func(void*, GoPolypoint*, int),
			  void* closure, GAddOrRemove = GADD);
    void movePointPrecond(GBool func(void*, const GoPolypoint*, int*, GPoint*),
			  void* closure, GAddOrRemove = GADD);
    void movePointPostcond(void func(void*, GoPolypoint*, int),
			   void* closure, GAddOrRemove = GADD);
    void resetPointsPrecond(GBool
			    func(void*, const GoPolypoint*, int, const GPoint*),
			    void* closure, GAddOrRemove = GADD);
    void resetPointsPostcond(void func(void*, GoPolypoint*),
			     void* closure, GAddOrRemove = GADD);
    
  protected:

    GoPolypointMetaclass();
    virtual ~GoPolypointMetaclass();
    
  private:

    // pre/post condition flags;
    GBool _has_add_point_precond     :1;
    GBool _has_add_point_postcond    :1;
    GBool _has_del_point_precond     :1;
    GBool _has_del_point_postcond    :1;
    GBool _has_move_point_precond    :1;
    GBool _has_move_point_postcond   :1;
    GBool _has_reset_points_precond  :1;
    GBool _has_reset_points_postcond :1;
    
    virtual const char*      m_classname() const;
    virtual GoMetaclass*     m_superclass() const;
    virtual const GActionRec* m_actions () const;
//  virtual Go*              m_create() const;
};


#endif /* _mGoPolypoint_h */



