/* $Id: mTree.h,v 1.1.1.1 1992/03/18 10:09:27 uep Exp $ */
/*****************************************************************************
MODULE:  BULL Graphic Objects
FILE:    mTree.h
AUTHOR:  Aline Jacomet and Gabriel Jureidini
DATE:    January 11th 1991

DESCRIPTION:
         Definition of the tree object metaclass.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#ifndef _mGoTree_h
#define _mGoTree_h

#include <Go/mGraphStr.h>


/*
 * Condition Flags
 */
struct GoTreeCondFlags {
    GBool _has_add_node_precond : 1;
    GBool _has_add_node_postcond : 1;
    GBool _has_remove_node_precond : 1;
    GBool _has_remove_node_postcond : 1;
    GoTreeCondFlags() {
	_has_add_node_precond = GFALSE;
	_has_add_node_postcond = GFALSE;
	_has_remove_node_precond = GFALSE;
	_has_remove_node_postcond = GFALSE;
    }
};


class GoTreeMetaclass : public GoGraphStructMetaclass {
    friend class GoTree;
    
  public:
    /*
     * Pre and Post Conditions ...;
     */
    void addNodePrecond(GBool (*)(void*, const GoTree*, GoTreeNode*,
				  GoMetaclass*, GoTreeNode*),
			void* closure, GAddOrRemove = GADD);
    void addNodePostcond(void (*)(void*, GoTree*, GoTreeNode*, GoMetaclass*,
				  GoTreeNode*),
			 void* closure, GAddOrRemove = GADD);
    void removeNodePrecond(GBool (*)(void*, const GoTree*, GoTreeNode*),
			   void* closure, GAddOrRemove = GADD);
    void removeNodePostcond(void (*)(void*, GoTree*, GoTreeNode*),
			    void* closure, GAddOrRemove = GADD);
    
  protected:
    GoTreeCondFlags _flags;

    GoTreeMetaclass();
    virtual ~GoTreeMetaclass();

  private:
    virtual const char*      m_classname() const;
    virtual GoMetaclass*     m_superclass() const;
    virtual Go*              m_create() const;
    virtual const GActionRec* m_actions() const;
};


#endif /* _mGoTree_h */
