#ifdef REV_INFO
static char rcsid[] = "$Id: BGoUICreation.cc,v 1.8 1992/12/31 10:16:32 smadja Exp $";
#endif
/*****************************************************************************
MODULE:  Go Extensions (User Interface)
FILE:    GoUIEditor.cc
AUTHOR:  
DATE:    June 92

DESCRIPTION:
  See the GoUIEditor.h header.

COPYRIGHT:
  Copyright (c) 1990, 1991, 1992 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.

*****************************************************************************/

#include <Go/BGoUIEd.h>
#include <Go/Interactor.h>
#include <Go/Sensor.h>
#include <Go/GoRoot.h>
#include <Go/GoInterf.h>
#include <Go/FullSensor.h>
#include <Go/ChoosePoint.h>
#include <Go/Event.h>
#include <Go/Form.h>
#include <Go/Tools/Callback.h>
#include <Go/Tools/GeomUtil.h>
#include <assert.h>


// the following struct will be a "closure". It is
// used for creation to identify the wanted creation;
// It is internal to GoUI.

struct _Closure 
{
    const char* classname;
    const char* interactor_name;
    GCallbackRec cbrec;
    Go*   wanted_parent;
    
    _Closure(const char* class_name, const char* itor_name,
	     Go* parent,
	     GoUICreateGoPostcond proc,
	     void* cl) : cbrec( (GCBProc)proc, cl)
    {
	classname = class_name;
	interactor_name = itor_name;
	wanted_parent  = parent;
    }
    virtual ~_Closure();
};

_Closure::~_Closure()
{
}

struct _RegPolygonClosure : _Closure
{
    int nb_edges;
    _RegPolygonClosure(const char* class_name, const char* itor_name,
		       Go* parent,
		       GoUICreateGoPostcond proc,
		       void* cl, int nb) :
			   _Closure(class_name, itor_name,
				    parent, proc, cl)
	{
	    nb_edges = nb;
	}
};

struct _ImageClosure : _Closure
{
    GNumber width;
    GNumber height;
    _ImageClosure(const char* class_name, const char* itor_name,
		       Go* parent,
		       GoUICreateGoPostcond proc,
		       void* cl,
		       GNumber w, GNumber h) :
			   _Closure(class_name, itor_name,
				    parent, proc, cl)
	{
	    width = w;
	    height = h;
	}
};


struct _TextClosure : _Closure
{
    GFormatter* formatter;
    _TextClosure(const char* class_name, const char* itor_name,
		 Go* parent,
		 GoUICreateGoPostcond proc,
		 void* cl,
		 GFormatter* form) :
		     _Closure(class_name, itor_name,
			      parent, proc, cl)
	{
	    formatter = form;
	}
    virtual ~_TextClosure();
};

_TextClosure::~_TextClosure()
{
    if (formatter)
	delete formatter;
}

#include <Go/mGo.h>
#include <Go/Polyline.h>
#include <Go/Rectangle.h>
#include <Go/Circle.h>
#include <Go/Ellipse.h>
#include <Go/RegPoly.h>
#include <Go/Image.h>
#include <Go/Textual.h>
#include <Go/Label.h>
#include <Go/SText.h>
#include <Go/Text.h>

// STATIC FUNCTION;
// The postcond are extern because of PC:DLL.

/**
FUNCTION
    _edit_sensor_postcond;
DESCRIPTION
    This function is called at the end of the edition of a new
    created Go (cf the static _bgoui_choosepoint_sensor_postcond() function).
    If user has cancelled the edition we destroy the Go and calls the
    application with a NULL go.
    Otherwise wee map the go (it is reparented to the current parent)
    and calls application with the created go.
**/
    
void _bgoui_edit_sensor_postcond(void* closure, GoSensor* sensor)
{
    GCallbackRec* cbrec = (GCallbackRec*)closure;

    GInteractor* itor = sensor->interactor();
    Go* go = sensor->parent();
    
    if (itor == NULL || itor->state() != GITOR_DONE)
    {
	go->destroy();
	go = NULL;
    }
    else
    {
	// map the go;
	go->map();
    }

    if (cbrec->proc)
    {
	// and calls the appli postcond;
	GCallCallbacks(cbrec, 1, go);
    }
    
    delete cbrec;
}


/**
FUNCTION
    _choosepoint_sensor_poscond;
DESCRIPTION
    this private function is called when a user has choosen
    a point on a screen (cf the static createGo() function).
    The wanted creation is determinated  by closure.
    We create tghe wanted go, reparent it to the current parent
    and if edition is necessary (assoc->interactor_name != NULL)
    we run the editing;
**/
void _bgoui_choosepoint_sensor_postcond(void* cl,
					GoSensor* sensor)
{
    // read the wanted creation;
    _Closure* closure = (_Closure*)cl;

    // read the result;
    if (sensor->interactor()->state() == GITOR_CANCELLED)
    {
	if (closure->cbrec.proc)
	{
	    // call the application with a NULL go;
	    GCallCallbacks(&closure->cbrec, 1, NULL);
	}
    }
    else
    {
	// read the position to do the go placement;
    	GPoint position =((GChoosePoint*)sensor->interactor())->choosenPoint();
    
	Go* go = Go::create(closure->classname);
	go->unmap();

	// handle specific classes;
	GoMetaclass* mclass = go->getMetaclass();
	
	// REGPOLYGON;
	if (mclass->isSubclass(GoRegPolygon::metaclass()))
	{
	    // get the number of edges;

	    int nb = ((_RegPolygonClosure*)closure)->nb_edges;
	    
	    ((GoRegPolygon*)go)->reset(/* edges */   nb,
				       /* radius */  1,
				       /* angle */   0);
	}
	// TEXT;
	if (mclass->isSubclass(GoText::metaclass()))
	{
	    _TextClosure* tcl = (_TextClosure*)closure;
	   ((GoText*)go)->formatter(tcl->formatter);
	    tcl->formatter = NULL; // it is given to Text;
	}

	// TEXTUAL
	if (mclass->isSubclass(GoTextual::metaclass()))
	{
	    ((GoTextual*)go)->lineHeight(BGOUI_LINE_HEIGHT_AT_CREATION_TIME);
	    go->select();
	}

	// IMAGE;
	if (go->getMetaclass()->isSubclass(GoImage::metaclass()))
	{
	    _ImageClosure* iws = (_ImageClosure*)closure;
	    ((GoImage*)go)->config(iws->width, iws->height);
	}
    
	go->move(position);
	go->reparent(closure->wanted_parent);

	
	if (closure->interactor_name != NULL)
	{
	    // edit the go;
	    (void)GoFullSensor::
		create(closure->interactor_name, go, NULL,
		       _bgoui_edit_sensor_postcond,
		       new GCallbackRec(closure->cbrec.proc,
					closure->cbrec.closure));
	}
	else
	{
	    // in any case map the Go.
	    go->map();
		
	    if (closure->cbrec.proc)
	    {
		// call the application with the go;
		GCallCallbacks(&closure->cbrec, 1, go);
	    }
	}
    }
    
    // free the previous closure;
    delete closure;;
}

    
/**
FUNCTION
    _verify_metaclass()
DESCRIPTION
    returns the wanted metaclass and the classname;
**/

const GoMetaclass* _verify_metaclass(const GoMetaclass* superclass,
				     const char* subclass_name,
				     const char*& result)
{
    if (subclass_name == NULL)
    {
	result = superclass->classname();
	return superclass;
    }
    else
    {
	const GoMetaclass* metaclass = Go::metaclass(subclass_name);
	result = subclass_name;
	if (! metaclass->isSubclass(superclass))
	{
	    GWarning("BGoUIEditor _verify_metaclass(): \"%s\" is not a subclass of \"%s\"",
		     result, superclass->classname());
	    result= NULL;
	    metaclass = NULL;
	}
	return metaclass;
    }
}
	
/**
FUNCTION
    _create_go()
DESCRIPTION
    1er step: place the Go.
    The following function starts the choosePoint interactor.
    with a Full sensor.
**/

void BGoUIEditor::_create_go(GoUICreateGoPostcond proc,
			     void* closure,
			     const GoMetaclass* mclass,
			     const char* subclass_name,
			     const char* interactor_name /* ="edit" */)
{
    const char* classname;
    const GoMetaclass* metaclass
	= _verify_metaclass(mclass, subclass_name, classname);

    if (metaclass == NULL)
	return;

    _Closure* _closure =  new _Closure(classname, interactor_name,
				       currentParent(), proc, closure);

    (void)GoFullSensor::create("choosePoint",
			       currentParent(), NULL,
			       _bgoui_choosepoint_sensor_postcond,
			       _closure);
}

/**
FUNCTION
    creation... (proc, closure)
DESCRIPTION
    If the user does not cancel the creation.
    The created Go will reparented to the current parent
    and  the application procedure will be called.
**/

void BGoUIEditor::createPolyline(GoUICreateGoPostcond proc, void* closure, const char* subclass_name)
{
    _create_go(proc, closure, GoPolyline::metaclass(), subclass_name);
}

void BGoUIEditor::createHandPolyline(GoUICreateGoPostcond proc, void* closure, const char* subclass_name)
{
    _create_go(proc, closure, GoPolyline::metaclass(), subclass_name,
	       "handEdit");
}


void BGoUIEditor::createPolygon(GoUICreateGoPostcond proc,
				void* closure, const char* subclass_name)
{
    _create_go(proc, closure, GoPolygon::metaclass(), subclass_name);
}

void BGoUIEditor::createRectangle(GoUICreateGoPostcond proc,
				  void* closure, const char* subclass_name)
{
    _create_go(proc, closure, GoRectangle::metaclass(), subclass_name);
}

void BGoUIEditor::createSquare(GoUICreateGoPostcond proc,
			       void* closure, const char* subclass_name)
{
    // we do not have yet a GoSquare;
    // we create a rectangle with width = height;
    // at the initialization time;
    _create_go(proc, closure, GoRectangle::metaclass(), subclass_name,
	       "squareEdit");
}

void BGoUIEditor::createCircle(GoUICreateGoPostcond proc,
			       void* closure, const char* subclass_name)
{
    _create_go(proc, closure, GoCircle::metaclass(), subclass_name);
}

void BGoUIEditor::createEllipse(GoUICreateGoPostcond proc,
				void* closure, const char* subclass_name)
{
    _create_go(proc, closure, GoEllipse::metaclass(), subclass_name);
}

void BGoUIEditor::createLabel(GoUICreateGoPostcond proc,
			      void* closure, const char* subclass_name)
{
    _create_go(proc, closure, GoLabel::metaclass(), subclass_name, NULL);
}


void BGoUIEditor::createSimpleText(GoUICreateGoPostcond proc,
				   void* closure, const char* subclass_name)
{
    _create_go(proc, closure, GoSimpleText::metaclass(), subclass_name, NULL);
}



/**
FUNCTION
    create arc....
DESCRIPTION
    do serial interactor.
    create circle/ellipse; choose point to show the first point;
    and drag the amplitude;
**/

void _bgouied_arc_end(void* closure, GoSensor* sensor)
{
    Go* arc = sensor->parent();
    GCallbackRec* _closure = (GCallbackRec*)closure;
    if (sensor->interactor()->state() != GITOR_DONE)
    {
	// delete the created arc;
	arc->destroy();
	arc = NULL;
    }

    // the arc was unmapped during the interaction;
    // remap it;
    arc->map();
    
    // call the application;
    if (_closure->proc)
	(*_closure->proc)(_closure->closure, arc);
    delete _closure;
    return;
}
    
    
    
void _bgouied_arc_continue(void* closure, GoSensor* sensor)
{
    GCallbackRec* _closure = (GCallbackRec*)closure;
    if (sensor->interactor()->state() != GITOR_DONE)
    {
	// delete the created arc;
	sensor->parent()->destroy();
	// call the application;
	if (_closure->proc)
	    (*_closure->proc)(_closure->closure, NULL);
	delete _closure;
	return;
    }

    GoArc* arc = (GoArc*)sensor->parent();
    
    const GPoint& point =
	((GChoosePoint*)sensor->interactor())->choosenPoint();
    GPoint relative = arc->coordsys().from(point);

    GNumber angle = GEllipseGetAngle(arc->radiusX(), arc->radiusY(),
				     relative);
    arc->pie(GTRUE);
    arc->closed(GTRUE);
    arc->start(angle);
    
    // choose the end of the arc;
    sensor->postcond(_bgouied_arc_end, closure);
    sensor->program("editEnd");
}

void _bgouied_after_arc_creation(void* closure, Go* go)
{
    GCallbackRec* _closure = (GCallbackRec*)closure;
    if (!go)
    {
	// call the application;
	if (_closure->proc)
	    (*_closure->proc)(_closure->closure, NULL);
	delete _closure;
	return;
    }

    // choose point to select first angle;
  GoFullSensor::create("choosePoint", go, NULL, _bgouied_arc_continue,
		       _closure);
}
	

void BGoUIEditor::createCircleArc(GoUICreateGoPostcond proc,
				  void* closure, const char* subclass_name)
{
    GCallbackRec* cbrec = new GCallbackRec((GCBProc)proc, closure);
    
    _create_go(_bgouied_after_arc_creation, cbrec, GoArc::metaclass(),
	       subclass_name,  "editRadii");
}


void BGoUIEditor::createEllipseArc(GoUICreateGoPostcond proc,
				   void* closure, const char* subclass_name)
{
    GCallbackRec* cbrec = new GCallbackRec((GCBProc)proc, closure);

    // "edit" the default editing program;
    // it is equivalent to "editRadxy";
    _create_go(_bgouied_after_arc_creation, cbrec, GoArc::metaclass(),
	       subclass_name,  "edit");
}



/**
FUNCTION
    createSimpleFormText()
DESCRIPTION
    create a go text with a simple form formatter;    
**/
#include <Go/SimpleForm.h>
void BGoUIEditor::createSimpleFormText(GoUICreateGoPostcond proc,
				       void* closure, const char* subclass_name)
{
    const char* classname;
    const GoMetaclass* metaclass
	= _verify_metaclass(GoText::metaclass(),
			    subclass_name, classname);

    if (metaclass == NULL)
	return;

    _TextClosure* _closure =
	new _TextClosure(classname, NULL  /* no interaction */,
			 currentParent(), proc,
			 closure,
			 new GSimpleFormatter);
    
    (void)GoFullSensor::create("choosePoint",
			       currentParent(), NULL,
			       _bgoui_choosepoint_sensor_postcond,
			       _closure);
}

/**
FUNCTION
    createPolypoint()
DESCRIPTION
    subclass name cannot be NULL;
    The methos is used to create subclass of Polypoint;
    In particular GoCurve and subclass.
**/

void BGoUIEditor::createPolypoint(GoUICreateGoPostcond proc,
				  void* closure, const char* subclass_name)
{
    if (subclass_name == NULL)
    {
	GWarning("BGoUIEditor::createPolypoint classname must be non NULL");
	if (proc)
	{
	    // call the application;
	    (*proc)(closure, NULL);
	}
	return;
    }
	
    _create_go(proc, closure, GoPolypoint::metaclass(), subclass_name);
}

/**
FUNCTION
    createRegPolygon
DESCRIPTION
    equivalent to _create_go.
    use the sensor association to add the number of edges
    for the wanted Reg Polygon.
**/
void BGoUIEditor::createRegPolygon(int nb, GoUICreateGoPostcond proc,
				   void* closure, const char* subclass_name)
{
    const char* classname;
    const GoMetaclass* metaclass
	= _verify_metaclass(GoRegPolygon::metaclass(),
			    subclass_name, classname);

    if (metaclass == NULL)
	return;

    _RegPolygonClosure* _closure =
	new _RegPolygonClosure(classname, "edit",
			       currentParent(), proc, closure,
			       nb);

    (void)GoFullSensor::create("choosePoint",
			       currentParent(), NULL,
			       _bgoui_choosepoint_sensor_postcond,
			       _closure);
}


void BGoUIEditor::createImage(GNumber width, GNumber height,
			      GoUICreateGoPostcond proc,
			      void* closure, const char* subclass_name)
{
    const char* classname;
    const GoMetaclass* metaclass
	= _verify_metaclass(GoImage::metaclass(),
			    subclass_name, classname);

    if (metaclass == NULL)
	return;


    // find the wanted edition;
    const char* edit;
    
    if (width == 0 || height == 0)
    {
	// ask the user to place and edit the image;
	edit = "edit";
    }
    else
    {
	// width and height are non 0.;
	// ask the use to place the image;
	// do not edit it;
	edit = NULL;
    }

    _ImageClosure* _closure =
	new _ImageClosure(classname, edit,
			  currentParent(), proc, closure,
			  width, height);
    
    (void)GoFullSensor::create("choosePoint",
			       currentParent(), NULL,
			       _bgoui_choosepoint_sensor_postcond,
			       _closure);
}


/**
FUNCTION
    BGoUIEditor::_create_edit_programs()
DESCRIPTION
    Creates the edit programs used by goui.
    edit programs are used at creation time
    or by application to edit an object.
**/

#include <Go/EdRectangle.h>
#include <Go/EdPolypoint.h>
#include <Go/EdRegPolygon.h>
#include <Go/EdPolyline.h>
#include <Go/EdPolygon.h>
#include <Go/EdArc.h>
#include <Go/EdCircle.h>
#include <Go/EdEllipse.h>
#include <Go/EdHandPLine.h>
#include <Go/EdImage.h>

void BGoUIEditor::_create_edit_programs()
{
    // EDIT;
    (void)GEditRectanglePgm::create();
    (void)GEditSquarePgm::create();
    (void)GEditRegPolygonPgm::create();
    (void)GEditPolylinePgm::create();
    (void)GEditPolygonPgm::create();
    (void)GEditPolypointPgm::create();
    (void)GEditEllipsePgm::create();
    (void)GEditCirclePgm::create();
    (void)GEditArcBeginPgm::create();
    (void)GEditArcEndPgm::create();
    (void)GEditArcRadiiPgm::create();
    (void)GEditArcRadxyPgm::create();
    (void)GEditHandPolylinePgm::create();
    (void)GEditImagePgm::create();
    
    // MOVE;
    (void)GMoveArcPointPgm::create();
    (void)GMovePolypointPgm::create();
    (void)GMovePolylinePgm::create();
    (void)GMovePolygonPgm::create();

    // CHOOSEPOINT;
    (void)GChoosePointPgm::create();
}









