#ifdef REV_INFO
static char rcsid[] = "$Id: BGoUIEd.cc,v 1.10 1992/12/31 13:09:13 jojic Exp $";
#endif
/*****************************************************************************
MODULE:  Go Kernel (User Interface)
FILE:    BGoUIEd.cc
AUTHOR:  Go team
DATE:    January 91

DESCRIPTION:
    See also GoUI.h.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include <assert.h>
#include <Go/Tools/Bool.h>
#include <Go/BGoUIEd.h>
#include <Go/ChoosePoint.h>
#include <Go/GoSelDisp.h>
#include <Go/FullSensor.h>
#include <Go/GoInterf.h>
#include <Go/Translator.h>
#include <Go/Tools/Error.h>
#include <Go/Tools/Alloc.h>
#include <Go/Tools/Callback.h>
#include <Go/Tools/Bind.h>

// CS Grabber;
#include <Go/MoveThisGr.h>
#include <Go/RescThRectGr.h>
#include <Go/RescThisGr.h>
#include <Go/RotThisGr.h>
#include <Go/XAxisGr.h>
#include <Go/YAxisGr.h>
#include <Go/RotCenGr.h>

#include <ctype.h>


//  static functions for this file.cc ***********************;
//  note that callback are extern ( cf PC.dll).
    
void _bgouied_go_removed_proc (void* closure, Go* /*go*/, Go* old_root)
{
    BGoUIEditor* bgoui = (BGoUIEditor*) closure;
    assert(bgoui->GoUI::root() == old_root /* PS && bgoui->current_parent == go */);

    bgoui->currentParent (NULL);
}


void _bgouied_set_current_parent (BGoUIEditor* bgoui, Go* go)
{
    bgoui->currentParent(go);
}

void _bgouied_create_rectangle (BGoUIEditor* bgoui, const char* sub)
{
    bgoui->createRectangle(NULL, NULL, sub);
}

void _bgouied_create_polyline (BGoUIEditor* bgoui, const char* sub)
{
    bgoui->createPolyline(NULL, NULL, sub);
}

void _bgouied_create_polypoint (BGoUIEditor* bgoui, const char* sub)
{
    bgoui->createPolypoint(NULL, NULL, sub);
}

void _bgouied_create_hand_polyline
    (BGoUIEditor* bgoui, const char* sub)
{
    bgoui->createHandPolyline(NULL, NULL, sub);
}

void _bgouied_create_polygon
    (BGoUIEditor* bgoui, const char* sub)
{
    bgoui->createPolygon(NULL, NULL, sub);
}

void _bgouied_create_label (BGoUIEditor* bgoui, const char* sub)
{
    bgoui->createLabel(NULL, NULL, sub);
}

void _bgouied_create_simple_text (BGoUIEditor* bgoui, const char* sub)
{
    bgoui->createSimpleText(NULL, NULL, sub);
}

void _bgouied_create_simpleform_text
    (BGoUIEditor* bgoui, const char* sub)
{
    bgoui->createSimpleFormText(NULL, NULL, sub);
}

void _bgouied_create_ellipse (BGoUIEditor* bgoui, const char* sub)
{
    bgoui->createEllipse(NULL, NULL, sub);
}

void _bgouied_create_circle (BGoUIEditor* bgoui, const char* sub)
{
    bgoui->createCircle(NULL, NULL, sub);
}

void _bgouied_create_square (BGoUIEditor* bgoui, const char* sub)
{
    bgoui->createSquare(NULL, NULL, sub);
}

void _bgouied_create_reg_polygon
    (BGoUIEditor* bgoui, int nb, const char* sub)
{
    bgoui->createRegPolygon(nb, NULL, NULL, sub);
}

void _bgouied_create_ellipse_arc
    (BGoUIEditor* bgoui, const char* sub)
{
    bgoui->createEllipseArc(NULL, NULL, sub);
}

void _bgouied_create_circle_arc
    (BGoUIEditor* bgoui, const char* sub)
{
    bgoui->createCircleArc(NULL, NULL, sub);
}

void _bgouied_choose_box(BGoUIEditor* bgoui)
{
    bgoui->chooseBox(NULL, NULL);
}

void _bgouied_act_on_all_selected_gos(BGoUIEditor* bgoui,
				      const char* program,
				      const GEvent* event)
{
    bgoui->actOnAllSelectedGos(program, event);
}

void _bgouied_act_on_all_selected_subtrees(BGoUIEditor* bgoui,
					   const char* program,
					   const GEvent* event)
{
    bgoui->actOnAllSelectedSubtrees(program, event);
}




// end of static ********************************************;

BGoUIEditor::BGoUIEditor (GoRoot* goroot) : GoUISelector (goroot)
{
    max_go_count = 16;
    fast_draw_depth = 16;

    static GBool not_first_call = GFALSE;
    if  (!not_first_call)
    {
	not_first_call = GTRUE;

	const int numSetDrawCount = 5;
	GoCSGrabberPgm* pgms[numSetDrawCount];
	pgms[0] = GMoveGrabberPgm::create();
	pgms[1] = GRescaleGrabberPgm::create();
	pgms[2] = GRescaleRectangleGrabberPgm::create ();
	pgms[3] = GRotateGrabberPgm::create ();
	pgms[4] = GRotateAroundCenterGrabberPgm::create ();
	    
	for (int i = 0; i < numSetDrawCount; i++)
	{
	    pgms[i]->maxDrawCount (max_go_count);
	    pgms[i]->maxDrawDepth (fast_draw_depth);
	}

	(void) GMoveThisGrabberPgm::create();
	(void) GRotateThisGrabberPgm::create();
	(void) GRescaleThisGrabberPgm::create();
	(void) GRescaleThisRectangleGrabberPgm::create();
	(void) GChoosePointPgm::create();
	(void) GXAxisGrabberPgm::create();
	(void) GYAxisGrabberPgm::create();


	// EDIT;
	_create_edit_programs();


	// CREATION BINDING;
	
	GBind("BGoUIEditor::setCurrentParent",
	      (void*) &_bgouied_set_current_parent);
	GBind("BGoUIEditor::createRectangle",
	      (void*) &_bgouied_create_rectangle);
	GBind("BGoUIEditor::createPolyline",
	      (void*) &_bgouied_create_polyline);
	GBind("BGoUIEditor::createPolypoint",
	      (void*) &_bgouied_create_polypoint);
	GBind("BGoUIEditor::createHandPolyline",
	      (void*) &_bgouied_create_hand_polyline);
	GBind("BGoUIEditor::createPolygon",
	      (void*) &_bgouied_create_polygon);
	GBind("BGoUIEditor::createLabel",
	      (void*) &_bgouied_create_label);
	GBind("BGoUIEditor::createSimpleText",
	      (void*) &_bgouied_create_simple_text);
	GBind("BGoUIEditor::createSimpleFormText",
	      (void*) &_bgouied_create_simpleform_text);
	GBind("BGoUIEditor::createEllipse",
	      (void*) &_bgouied_create_ellipse);
	GBind("BGoUIEditor::createCircle",
	      (void*) &_bgouied_create_circle);
	GBind("BGoUIEditor::createSquare",
	      (void*) &_bgouied_create_square);
	GBind("BGoUIEditor::createRegPolygon",
	      (void*) &_bgouied_create_reg_polygon);
	GBind("BGoUIEditor::createEllipseArc",
	      (void*) &_bgouied_create_ellipse_arc);
	GBind("BGoUIEditor::createCircleArc",
	      (void*) &_bgouied_create_circle_arc);

	/*
	 * misc methods;
	 */

	GBind("BGoUIEditor::chooseBox",
	      (void*) &_bgouied_choose_box);
	GBind("BGoUIEditor::actOnAllSelectedGos",
	      (void*) &_bgouied_act_on_all_selected_gos);
	GBind("BGoUIEditor::actOnAllSelectedSubtrees",
	      (void*) &_bgouied_act_on_all_selected_subtrees);
    }	

    current_parent = GoUI::root();
}
    
    
BGoUIEditor::~BGoUIEditor ()
{
    if (current_parent != GoUI::root())
	currentParent (NULL);
}
	
	
/*
 * current parent management
 ***************************
 */
	

void BGoUIEditor::currentParent (Go* go)
{
    GoRoot* goroot = GoUI::root();

    if (go == NULL)
	go = goroot;

    if (go == currentParent())
	return;

    GoSelectionDisplayer* seldisp = NULL;
    for (Go* child = goroot->Child(); child != NULL; child = child->Next())
	if (child->getMetaclass()->isSubclass (GoSelectionDisplayer::metaclass()))
	{
	    seldisp = (GoSelectionDisplayer*) child;
	    break;
	}
    
    if (seldisp == NULL)
	GWarning("BGoUIEd.cc: currentParent(): no selection displayer!");

    if (current_parent != goroot)
    {
	if (seldisp != NULL)
	    seldisp->alternateDisplayFor (current_parent, GREMOVE);
	current_parent->rootPostcond (_bgouied_go_removed_proc, this, GREMOVE);
    }

    current_parent = go;
    
    if (current_parent != goroot)
    {
	if (seldisp != NULL)
	    seldisp->alternateDisplayFor (current_parent, GADD);
	current_parent->rootPostcond (_bgouied_go_removed_proc, this);
    }
}



/*
 * a tool to choose a box.
 *************************
 */

struct ChooseBoxInfo 
{
    void (*callback)(void*, GBool, GBox&);
    void* closure;
    GBox  box;

    ChooseBoxInfo (void (*f)(void*, GBool, GBox&), void* c)
    {
	callback = f;
	closure = c;
    }
};


void choose_box_after_rescale (void* closure, GoSensor* rescaleSensor)
{
    ChooseBoxInfo* cbinfo = (ChooseBoxInfo*) closure;
    
    if (rescaleSensor->interactor()->state() != GITOR_DONE)
    {
	rescaleSensor->parent()->destroy();
	if (cbinfo->callback)
	    (*cbinfo->callback) (cbinfo->closure, GFALSE, cbinfo->box);
    }	
    else
    {
	GRegion clip;
	rescaleSensor->parent()->addMyClip(clip);
	cbinfo->box = clip.bigbox();
	rescaleSensor->parent()->destroy();

	if (cbinfo->callback)
	    (*cbinfo->callback) (cbinfo->closure, GTRUE, cbinfo->box);
    }

    delete cbinfo;
}


void choose_box_after_move (void* closure, GoSensor* moveSensor)
{
    if (moveSensor->interactor()->state() != GITOR_DONE)
    {
	ChooseBoxInfo* cbinfo = (ChooseBoxInfo*) closure;
	moveSensor->parent()->destroy();
	if (cbinfo->callback)
	    (*cbinfo->callback) (cbinfo->closure, GFALSE, cbinfo->box);

	delete cbinfo;
    }
    else
    {
	// use another program on the same sensor;
	moveSensor->postcond(choose_box_after_rescale, closure);
	moveSensor->program("rescale");
    }

}


Go* BGoUIEditor::currentParent () const 
{
    return (current_parent == NULL) ? GoUI::root() : current_parent;
}


#include <Go/Rectangle.h>

void BGoUIEditor::chooseBox
    (void (*callback)(void*, GBool, GBox&), void* closure)
{
    GoRectangle* rect = GoRectangle::create();
    rect->unmap();
    rect->reset (20, 10);
    rect->move (GoUI::root()->interf()->getPointer());
    rect->reparent (GoUI::root());
    
    GoFullSensor::create ("move", rect, NULL, choose_box_after_move,
			  new ChooseBoxInfo(callback, closure));
}


void BGoUIEditor::root (GoRoot* newRoot)
{
    GoUISelector::root (newRoot);
    currentParent (GoUI::root());
}
