#ifdef REV_INFO
static char rcsid[] = "$Id: BGoUIGeom.cc,v 1.2 1992/12/16 16:08:52 jojic Exp $";
#endif
/*****************************************************************************
MODULE:  GoUI (User Interface for GoGeometric Objects)
FILE:    BGoUIGeom.cc
AUTHOR:  Gilles Benati, Philippe Smadja
DATE:    March 91

DESCRIPTION:

  The file constains the the global function GoSelectPolypoint,
                                             GoUnselectPolypoint.
  It creates a GoPointSensor with a NULL interactor name.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/


#include <Go/BGoUIEd.h>
#include <Go/ChoosePoint.h>
#include <Go/FullSensor.h>
#include <Go/Composite.h>
#include <Go/Tools/Atom.h>
#include <assert.h>

/**
FUNCTION
    actOnAllSelectedGos()
    actOnAllSelectedSubtrees(
DESCRIPTION
    group all selected go or subtrees
    choose a point:
    if ok
       exec the program;
    ungroup;
    call application with the state of the last interactor.
**/

struct _Closure {
    GAtom atom;
    void (*procedure)(void*, int);
    void* closure;
    _Closure (const char* program, void (*pr)(void*, int), void* cl)
    {
	atom = GCreateAtom(program);
	procedure = pr;
	closure =  cl;
    }
    ~_Closure()
    {
	GFreeAtom(atom);
    }
    inline const char* program() const { return GAtomToString(atom); } 
};

void _bgouigeom_after_action(void* closure, GoSensor* sensor)
{
    GoUISelector* goui = (GoUISelector*) GoUIFrom (sensor->root());
    GBool state = sensor->interactor()->state();
    goui->ungroup ((GoComposite*) sensor->parent());

    _Closure* _closure = (_Closure*) closure;
    if (_closure->procedure)
	(*_closure->procedure) (_closure->closure, state);

    delete _closure;
}

void _bgouigeom_after_choosepoint(void* closure, GoSensor* sensor)
{
    if (sensor->interactor()->state() != GITOR_DONE)
    {
	_bgouigeom_after_action(closure, sensor);
	return;
    }

    const char* program = ((_Closure*)closure)->program();

    // update the sensor to continue;
    // In this case the sensor will not be destroyed;

    sensor->program(program);
    sensor->postcond(_bgouigeom_after_action, closure);
}

void BGoUIEditor::actOnAllSelectedGos(const char* program, const GEvent* event,
				      void (*proc)(void*, int), void* closure)
{
    if (!program)
    {
	if (proc)
	    (*proc)(closure, GITOR_CANCELLED);
 	return;
    }
    
    GoComposite* comp  = groupAllSelectedGos();
    if (!comp)
    {
	if (proc)
	    (*proc)(closure, GITOR_CANCELLED);
	return;
    }

    (void)GoFullSensor::create("choosePoint", comp, event,
			       _bgouigeom_after_choosepoint,
			       new _Closure(program, proc, closure));
}


void BGoUIEditor::actOnAllSelectedSubtrees(const char* program,
					   const GEvent* event,
					   void (*proc)(void*, int),
					   void* closure)
{
    if (!program)
    {
	if (proc)
	    (*proc)(closure, GITOR_CANCELLED);
 	return;
    }
    
    GoComposite* comp  = groupAllSelectedSubtrees();
    if (!comp)
    {
	if (proc)
	    (*proc)(closure, GITOR_CANCELLED);
	return;
    }

    (void)GoFullSensor::create("choosePoint", comp, event,
			       _bgouigeom_after_choosepoint,
			       new _Closure(program, proc, closure));
}

