#ifdef REV_INFO
static char rcsid[] = "$Id: BGoUIText.cc,v 1.3 1992/12/30 13:29:39 jojic Exp $";
#endif
/*****************************************************************************
MODULE:  Go Kernel (User Interface for GoTextual Objects)
FILE:    BGoUIText.cc
AUTHOR:  Michel Andre
DATE:    March 91

DESCRIPTION:
    These functions may be used in the gouifiles as is.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include <assert.h>
#include <Go/BGoUIEd.h>
#include <Go/Sensor.h>
#include <Go/Textual.h>
#include <Go/Tools/Atom.h>

/**
FUNCTION
    GoTextualMakeSensitive(const char* program)
DESCRIPTION
    Convenience function used to sensitive/unsensitive
    the GoTextual objects. Precond on save to avoid to save
    the sensors.
    The run the interator program when a Textual is selected.
    It destroy it when it is unselected;
    Note that program must non NULL.
**/

// note that callback are extern because of PC.DLL;

GBool _bgoui_not_save_sensor (void*, const Go*)
{
    return GFALSE;
}

void _bgoui_make_sensitive (void* closure, Go* go)
{
    const char* program = GAtomToString(GAtom(closure));
    
    GoTextual* text = (GoTextual*)go;
    if (go->selected())
    {
        GoSensor* sensor = GoSensor::create (program, text, NULL, NULL, NULL);
        sensor->savePrecond (_bgoui_not_save_sensor, NULL);

	// focus / enter/leave may be handled  by the program itselft;
    }
    else
    {
        if (text->firstSelected() != -1)
	    text->selectPortion(-1, -1, GFALSE);
        Go* nxt;
        for (Go* ch=(Go*)text->Child(); ch; ch=nxt){
            nxt = (Go*)ch->Next();
            if (ch->isSensor() &&
                !strcmp(((GoSensor*)ch)->program(), program))
	    {
		GoUI* goui = GoUIFrom (text->root());
		if (goui &&
		    goui->focusingSensor() == (GoSensor*) ch)
		    goui->focus (NULL);
		    
                ch->destroy();
                break;
            }
        }
    }
}

void GoTextualMakeSensitive(const char* program_name)
{
    if (program_name == NULL)
	return;
    
    GAtom program = GCreateAtom(program_name);
    (GoTextual::metaclass())->selectPostcond(
	_bgoui_make_sensitive, (void*)program, GADD);
}
