#ifdef REV_INFO
static char rcsid[] = "$Id: GoSelDisp.cc,v 1.3 1992/12/30 13:29:19 jojic Exp $";
#endif
/*****************************************************************************
MODULE:  Graphic Objects
FILE:    GoSelDisp.cc
AUTHOR:  jojic
DATE:    Nov 9 1992

DESCRIPTION:
  See also the GoSelDisp.h header.
  

COPYRIGHT:
  Copyright (c) 1992 Bull S.A.
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull S.A. may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull S.A..
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull S.A. under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull S.A..

  Bull S.A. assumes no responsibility for the use or inability to use
  any of its software. The software is provided "AS IS" without warranty
  of any kind, and Bull S.A. expressly disclaims all implied warranties,
  including but not limited to the implied warranties of merchantability
  and fitness for a particular purpose.

  In no event shall Bull S.A. be liable for any special, incidental,
  indirect or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#include <Go/GoSelDisp.h>
#include <Go/GoRoot.h>
#include <Go/GoInterf.h>
#include <assert.h>



/*
 *-------------------------------------------------------
 * pre- and post- conditions.
 *-------------------------------------------------------
 */

GBool seldisp_root_add_child_pre
    (void* closure, const Go*, const Go*, const Go* (*pNext))
{
    GoSelectionDisplayer* seldisp = (GoSelectionDisplayer*) closure;
    if (*pNext == NULL)
	*pNext = seldisp;
    return GTRUE;
}


/*
 *-------------------------------------------------------
 * GoSelectionDisplayer public methods
 *-------------------------------------------------------
 */



/*
 *-------------------------------------------------------
 * GoSelectionDisplayer private methods
 *-------------------------------------------------------
 */



GBool seldisp_save_pre (void*, const Go*)
{
    return GFALSE;
}

    
void seldisp_reparent_post (void*, Go* go, Go* oldp, Go*)
{
    GoSelectionDisplayer* seldisp = (GoSelectionDisplayer*) go;
    
    if (oldp != NULL)
	oldp->addChildPrecond (seldisp_root_add_child_pre, seldisp, GREMOVE);
    if (seldisp->root())
	seldisp->root()->
	    addChildPrecond (seldisp_root_add_child_pre, seldisp);

    seldisp->my_clip.setEmpty ();
    seldisp->recompute_my_clip ();
    seldisp->MyClipHasChanged ();
}


void GoSelectionDisplayer::alternateDisplayFor (Go* go, GAddOrRemove add)
{
    if (add == GADD)
	alternate_set.add (go);
    else
	alternate_set.remove (go);

    if (go->root() != root())
	return;
    
    MyClipHasChanged ();
    if (!go->selected())
	go->clipPostcond (seldisp_clip_post, this, add);
}


void seldisp_select_post (void* closure, Go* go)
{
    GoSelectionDisplayer* seldisp = (GoSelectionDisplayer*) closure;

    if (go->root() != seldisp->root())
	return;

    seldisp->MyClipHasChanged();
    if (!seldisp->alternate_set.contains (go))
	go->clipPostcond (seldisp_clip_post, seldisp,
			  go->selected() ? GADD : GREMOVE);
}

	
void seldisp_root_post (void* closure, Go* go, Go* oldroot)
{
    GoSelectionDisplayer* seldisp = (GoSelectionDisplayer*) closure;

    if (seldisp->root() == NULL ||
	!go->selected() && !seldisp->alternate_set.contains (go))
	return;

    if (oldroot == seldisp->root())
    {
	seldisp->MyClipHasChanged();
	go->clipPostcond (seldisp_clip_post, seldisp, GREMOVE);
    }
    else if (go->root() == seldisp->root())
    {
	seldisp->MyClipHasChanged();
	go->clipPostcond (seldisp_clip_post, seldisp);
    }
}


void seldisp_clip_post (void* closure, Go* go)
{
    GoSelectionDisplayer* seldisp = (GoSelectionDisplayer*) closure;
    if (go != seldisp)
	seldisp->MyClipHasChanged ();
    else
	seldisp->recompute_my_clip ();
}


static const int RegularFlag = 1;
static const int AlternateFlag = 2;


void seldisp_add_to_my_clip (void* closure, Go* go)
{
    GoSelectionDisplayer* seldisp = (GoSelectionDisplayer*) closure;

    GRegion goClip;
    if (go->crossType() == GCROSS_AS_LEAF)
	go->addClip (goClip);
    else
    {
	go->addMyClip (goClip);
	if (goClip.empty())
	    go->addClip (goClip);
    }

    if (goClip.empty())
	return;

    GBox box (goClip.bigbox());

    assert (sizeof (GNumber) <= sizeof (void*));
    seldisp->disp_boxes_stack.push (*(void**) &box.xmin);
    seldisp->disp_boxes_stack.push (*(void**) &box.ymin);
    seldisp->disp_boxes_stack.push (*(void**) &box.xmax);
    seldisp->disp_boxes_stack.push (*(void**) &box.ymax);

    seldisp->disp_boxes_stack.push ((void*)
				    ((go->selected() ? RegularFlag : 0) |
				     (seldisp->alternate_set.contains (go) ? AlternateFlag : 0)));

    GBox displayBoxes[4];
    seldisp->compute_or_draw_display_boxes (box, displayBoxes,
					    go->selected(), seldisp->alternate_set.contains (go));
    if (!seldisp->precise_clip)
	GAdd (seldisp->my_clip,
	      GBox(displayBoxes[0].xmin, displayBoxes[0].ymin,
		   displayBoxes[3].xmax, displayBoxes[3].ymax),
	      seldisp->my_clip);
    else
	for (int i = 0; i < 4; i++)
	    GAdd (seldisp->my_clip, displayBoxes[i], seldisp->my_clip);

    if (seldisp->my_clip.numboxes() > 256)
    {
	seldisp->precise_clip = GFALSE;
	seldisp->my_clip = seldisp->my_clip.bigbox();
    }
}


void GoSelectionDisplayer::compute_or_draw_display_boxes
    (const GBox& box, GBox displayBoxes[4], GBool regDisplay, GBool altDisplay) const
{
    GoInterf* goint = interf();
    GNumber pixw = goint->pixelWidth();
    GNumber pixh = goint->pixelHeight();
    const int pixWidth = 1;
    const int pixLength = 4;
    const int pixOutExtra = 1;

    GNumber xOutWidth = (pixWidth + 2 * pixOutExtra) * pixw;
    GNumber xOutLength = (pixLength + 2 * pixOutExtra) * pixw;

    GNumber yOutWidth = (pixWidth + 2 * pixOutExtra) * pixh;
    GNumber yOutLength = (pixLength + 2 * pixOutExtra) * pixh;

    GNumber xOutLeft = box.xmin;
    GNumber yOutUp = box.ymin;

    GNumber xOutRight = box.xmax;
    GNumber yOutDown = box.ymax;

    if (displayBoxes == NULL)
    {
	GNumber xInWidth = pixWidth * pixw;
	GNumber xInLength = pixLength * pixw;

	GNumber yInWidth = pixWidth * pixh;
	GNumber yInLength = pixLength * pixh;
	
	GNumber xInLeft = xOutLeft + pixOutExtra * pixw;
	GNumber yInUp = yOutUp + pixOutExtra * pixh;

	GNumber xInRight = xOutRight - pixOutExtra * pixw;
	GNumber yInDown = yOutDown - pixOutExtra * pixh;
	
	GCoordSys unitCS;

#ifdef __GNUC__
# define _ADDR(brush) &brush
#else
  /* ATT 2.1 gives unjustified warning: "address of const struct" */
  /* It is unjustified because the GoInterf::rectangle() takes a const */
  /* reference for the wanted brush */
# define _ADDR(brush)  &(((GoSelectionDisplayer*)this)->brush)
#endif
	
	if (altDisplay)
	{
	    goint->rectangle (NULL, _ADDR(alt_out_brush), unitCS,
			      GPoint (xOutLeft, yOutUp), xOutRight - xOutLeft, yOutWidth);
	    goint->rectangle (NULL, _ADDR(alt_out_brush), unitCS,
			      GPoint (xOutLeft, yOutUp), xOutWidth, yOutDown - yOutUp);
	    goint->rectangle (NULL, _ADDR(alt_out_brush), unitCS,
			      GPoint (xOutRight - xOutWidth, yOutUp), xOutWidth, yOutDown - yOutUp);
	    goint->rectangle (NULL, _ADDR(alt_out_brush), unitCS,
			      GPoint (xOutLeft, yOutDown - yOutWidth), xOutRight - xOutLeft, yOutWidth);

	    goint->rectangle (NULL, _ADDR(alt_in_brush), unitCS,
			      GPoint (xInLeft, yInUp), xInRight - xInLeft, yInWidth);
	    goint->rectangle (NULL, _ADDR(alt_in_brush), unitCS,
			      GPoint (xInLeft, yInUp), xInWidth, yInDown - yInUp);
	    goint->rectangle (NULL, _ADDR(alt_in_brush), unitCS,
			      GPoint (xInRight - xInWidth, yInUp), xInWidth, yInDown - yInUp);
	    goint->rectangle (NULL, _ADDR(alt_in_brush), unitCS,
			      GPoint (xInLeft, yInDown - yInWidth), xInRight - xInLeft, yInWidth);
	}

	if (regDisplay)
	{
	    GPoint outUpLeft (xOutLeft, yOutUp);
	    goint->rectangle (NULL, _ADDR(out_brush), unitCS,
			      outUpLeft, xOutLength, yOutWidth);
	    goint->rectangle (NULL, _ADDR(out_brush), unitCS,
			      outUpLeft, xOutWidth, yOutLength);

	    goint->rectangle (NULL, _ADDR(out_brush), unitCS,
			      GPoint(xOutRight - xOutLength, yOutUp),
			      xOutLength, yOutWidth);
	    goint->rectangle (NULL, _ADDR(out_brush), unitCS,
			      GPoint(xOutRight - xOutWidth, yOutUp),
			      xOutWidth, yOutLength);

	    goint->rectangle (NULL, _ADDR(out_brush), unitCS,
			      GPoint(xOutLeft, yOutDown - yOutWidth),
			      xOutLength, yOutWidth);
	    goint->rectangle (NULL, _ADDR(out_brush), unitCS,
			      GPoint(xOutLeft, yOutDown - yOutLength),
			      xOutWidth, yOutLength);

	    goint->rectangle (NULL, _ADDR(out_brush), unitCS,
			      GPoint(xOutRight-xOutLength, yOutDown-yOutWidth),
			      xOutLength, yOutWidth);
	    goint->rectangle (NULL, _ADDR(out_brush), unitCS,
			      GPoint(xOutRight-xOutWidth, yOutDown-yOutLength),
			      xOutWidth, yOutLength);

	    GPoint inUpLeft (xInLeft, yInUp);
	    goint->rectangle (NULL, _ADDR(in_brush), unitCS,
			      inUpLeft, xInLength, yInWidth);
	    goint->rectangle (NULL, _ADDR(in_brush), unitCS,
			      inUpLeft, xInWidth, yInLength);

	    goint->rectangle (NULL, _ADDR(in_brush), unitCS,
			      GPoint(xInRight - xInLength, yInUp),
			      xInLength, yInWidth);
	    goint->rectangle (NULL, _ADDR(in_brush), unitCS,
			      GPoint(xInRight - xInWidth, yInUp),
			      xInWidth, yInLength);

	    goint->rectangle (NULL, _ADDR(in_brush), unitCS,
			      GPoint(xInLeft, yInDown - yInWidth),
			      xInLength, yInWidth);
	    goint->rectangle (NULL, _ADDR(in_brush), unitCS,
			      GPoint(xInLeft, yInDown - yInLength),
			      xInWidth, yInLength);

	    goint->rectangle (NULL, _ADDR(in_brush), unitCS,
			      GPoint(xInRight-xInLength, yInDown-yInWidth),
			      xInLength, yInWidth);
	    goint->rectangle (NULL, _ADDR(in_brush), unitCS,
			      GPoint(xInRight-xInWidth, yInDown-yInLength),
			      xInWidth, yInLength);
	}
    }
    else
    {
	if (altDisplay)
	{
	    displayBoxes[0].xmin = xOutLeft;
	    displayBoxes[0].xmax = xOutRight;
	    displayBoxes[0].ymin = yOutUp;
	    displayBoxes[0].ymax = yOutUp + yOutWidth;
	    
	    displayBoxes[1].xmin = xOutLeft;
	    displayBoxes[1].xmax = xOutLeft + xOutWidth;
	    displayBoxes[1].ymin = yOutUp + yOutWidth;
	    displayBoxes[1].ymax = yOutDown - yOutWidth;

	    displayBoxes[2].xmin = xOutRight - xOutWidth;
	    displayBoxes[2].xmax = xOutRight;
	    displayBoxes[2].ymin = displayBoxes[1].ymin;
	    displayBoxes[2].ymax = displayBoxes[1].ymax;

	    displayBoxes[3].xmin = displayBoxes[0].xmin;
	    displayBoxes[3].xmax = displayBoxes[0].xmax;
	    displayBoxes[3].ymin = yOutDown - yOutWidth;
	    displayBoxes[3].ymax = yOutDown;
	}
	else
	{
	    displayBoxes[0].xmin = xOutLeft;
	    displayBoxes[0].xmax = xOutLeft + xOutLength;
	    displayBoxes[0].ymin = yOutUp;
	    displayBoxes[0].ymax = yOutUp + yOutLength;

	    displayBoxes[1].xmin = xOutRight - xOutLength;
	    displayBoxes[1].xmax = xOutRight;
	    displayBoxes[1].ymin = displayBoxes[0].ymin;
	    displayBoxes[1].ymax = displayBoxes[0].ymax;
    
	    displayBoxes[2].xmin = displayBoxes[0].xmin;
	    displayBoxes[2].xmax = displayBoxes[0].xmax;
	    displayBoxes[2].ymin = yOutDown - yOutLength;
	    displayBoxes[2].ymax = yOutDown;
    
	    displayBoxes[3].xmin = displayBoxes[1].xmin;
	    displayBoxes[3].xmax = displayBoxes[1].xmax;
	    displayBoxes[3].ymin = displayBoxes[2].ymin;
	    displayBoxes[3].ymax = displayBoxes[2].ymax;
	}
    }
}


void GoSelectionDisplayer::recompute_my_clip ()
{
    GRegion old_region (my_clip);
    my_clip.setEmpty ();
    disp_boxes_stack.clear ();
    precise_clip = GTRUE;
    if (root() != NULL)
	root()->ForEverySelectedGo (seldisp_add_to_my_clip, this);
    for (Go* altGo = (Go*) alternate_set.first();
	 altGo != NULL;
	 altGo = (Go*) alternate_set.next())
	if (!altGo->selected())
	    seldisp_add_to_my_clip (this, altGo);

    if (!precise_clip)
	RedrawBackAndFore(my_clip);
    else
    {
	GSub (old_region, my_clip, old_region);
	RedrawBackAndFore (old_region);
	GSub (my_clip, old_region, old_region);
	RedrawForeOnly (old_region);
    }
}



/*
 *-------------------------------------------------------
 * GoSelectionDisplayerMetaclass methods
 *-------------------------------------------------------
 */

/*
 This global variable is used to store the unique instance
 of the GoSelectionDisplayerMetaclass
 */
static GoSelectionDisplayerMetaclass* SelectionDisplayer_metaclass = NULL;


/*
 * The GoSelectionDisplayerMetaclass constructor.
 */
GoSelectionDisplayerMetaclass::GoSelectionDisplayerMetaclass ()
{
    /* initialization of metaclass private data: */
}


/*
 * The GoSelectionDisplayerMetaclass destructor.
 */
GoSelectionDisplayerMetaclass::~GoSelectionDisplayerMetaclass ()
{
    /* If we are deleting the unique GoSelectionDisplayerMetaclass instance
       (and not some subclass), we must remove "this" from the
       set of subclasses in the superclass, and set NULL to the
       global pointer "SelectionDisplayer_metaclass".
       Quid about GoSelectionDisplayer instances ? should we delete them ?
       */
    if (this == SelectionDisplayer_metaclass)
    {
        superclass()->RemoveSubclass(this);
	SelectionDisplayer_metaclass = NULL;
    }
}


/*
 * The virtual method "m_classname" returns the name (string) of
 * this class, i.e. "GoSelectionDisplayer".
 */
const char* GoSelectionDisplayerMetaclass::m_classname () const
{
    return "GoSelectionDisplayer";
}


/*
 * The virtual method "m_superclass" returns the unique instance of
 * the superclass of this GoSelectionDisplayerMetaclass class.
 */
GoMetaclass* GoSelectionDisplayerMetaclass::m_superclass () const
{
    return GoBasicDS::metaclass();
}


/*
 * The action table of GoSelectionDisplayer objects.
 * Functions inside this table will be accessible through
 * their name (i.e. a string). This is usefull to implement some
 * interpreted languages dealing with graphic objects.
 */
static GActionRec SelectionDisplayer_actions[] = {
    {NULL, NULL} //this must be the last line;
};


/*
 * This method returns the action table of GoSelectionDisplayer objects.
 */
const GActionRec* GoSelectionDisplayerMetaclass::m_actions () const
{
    return SelectionDisplayer_actions;
}


/*
 * This method creates and returns a new GoSelectionDisplayer instance.
 * This allows the creation of a new instance of an object, having only
 * the name of the object class.
 */
Go* GoSelectionDisplayerMetaclass::m_create () const
{
    return new GoSelectionDisplayer;
}



/*
 *-------------------------------------------------------
 * GoSelectionDisplayer methods (derived from Go)
 *-------------------------------------------------------
 */


/*
 * This static method returns the unique GoSelectionDisplayerMetaclass instance.
 * If this instance does not already exist, it is created. In such a case
 * the static global variable "SelectionDisplayer_metaclass" is set up and this
 * metaclass instance is added to the set of subclass instances of the
 * superclass.
 */
GoSelectionDisplayerMetaclass* GoSelectionDisplayer::metaclass ()
{
    if (SelectionDisplayer_metaclass == NULL)
    {
	SelectionDisplayer_metaclass = new GoSelectionDisplayerMetaclass;
        SelectionDisplayer_metaclass->superclass()
	    ->AddSubclass(SelectionDisplayer_metaclass);

	SelectionDisplayer_metaclass->
	    reparentPostcond (seldisp_reparent_post, NULL);
	SelectionDisplayer_metaclass->savePrecond (seldisp_save_pre, NULL);
    }

    return SelectionDisplayer_metaclass;
}


/*
 * This is the GoSelectionDisplayer constructor.
 */
GoSelectionDisplayer::GoSelectionDisplayer ()
{
    GRgb rgb_black(0,0,0);
    GShColor* blackColor = GCreateShColor (&rgb_black);
    GRgb rgb_white(GMAX_RGB_VALUE, GMAX_RGB_VALUE, GMAX_RGB_VALUE);
    GShColor* whiteColor = GCreateShColor (&rgb_white);

    out_brush.color (blackColor);
    in_brush.color (whiteColor);
    alt_out_brush.color (blackColor);
    alt_in_brush.color (whiteColor);

    blackColor->destroy();
    whiteColor->destroy();
    
    crossType (GCROSS_AS_NOTHING);
    clipPostcond (seldisp_clip_post, this);
    Go::metaclass()->selectPostcond (seldisp_select_post, this);
    Go::metaclass()->rootPostcond (seldisp_root_post, this);

    GoAlwaysLastChild (this);
}


/*
 * This is the GoSelectionDisplayer destructor.
 */
GoSelectionDisplayer::~GoSelectionDisplayer ()
{
    Go::metaclass()->selectPostcond (seldisp_select_post, this, GREMOVE);
}


/*
 * This virtual method adds the clip of this object, i.e. a GRegion
 * bigger than or equal to the drawing of this object.
 */
void GoSelectionDisplayer::m_add_clip (GRegion& region) const
{
    GAdd (region, my_clip, region);
}


/*
 * This virtual method adds the opaque clip of this object, i.e. a GRegion
 * smaller than or equal to the drawing of this object.
 */
void GoSelectionDisplayer::m_add_opaque_clip (GRegion&) const
{
}


/*
 * This virtual method returns GTRUE if "p" is inside the drawing of this
 * object. With an accepted distance "maxd".
 */
GBool GoSelectionDisplayer::m_contains (const GPoint&, GNumber) const
{
    return GFALSE;
}


/*
 * This virtual method draws at least the part of the object which is
 * inside "region".
 * It is often more expensive to take care about this region than to
 * draw the full object.
 */
void GoSelectionDisplayer::m_draw (const GRegion&) const
{
    int count;
    GNumber* pNum = (GNumber*) disp_boxes_stack.data (count);
    pNum += count;

    for (; count > 0; count -= 5)
    {
	pNum -= 1;
	int flags = * (int*) pNum;

	GNumber ymax = *--pNum;
	GNumber xmax = *--pNum;
	GNumber ymin = *--pNum;
	GNumber xmin = *--pNum;
	GBox box(xmin, ymin, xmax, ymax);
	compute_or_draw_display_boxes (box, NULL, flags & RegularFlag, flags & AlternateFlag);
    }
}


/*
 * This private virtual method must copy its own class private data
 * from "src" to "this".
 * This method is used by the "dup(go)" method. The dup method returns a
 * copy of the "go" subtree.
 *
 * If dup was called on "this", then this next brother must not be dupped,
 * "donext" is then FALSE.
 * If dup was called on some ancestor of "this", then this next brother
 * must also be dupped. The flag "donext" is TRUE. In that case, IF THIS
 * CLASS stores a "_next" field, it must call the recursive Dup on the
 * next brother.
 */
void GoSelectionDisplayer::m_dup (const Go* src, Go* parcopy, Go* prevcopy, GBool donext, GBool doChildren)
{
    GoBasicDS::m_dup(src, parcopy, prevcopy, donext, doChildren);
}
	

/*
 * This private virtual method returns the metaclass of "this".
 */
GoMetaclass* GoSelectionDisplayer::m_metaclass () const
{
    return GoSelectionDisplayer::metaclass();
}




/*
 * This static method is responsible to save "this" go on "stream".
 * It must write its private data and then call the save of its superclass.
 */
int GoSelectionDisplayer::save (const GoSelectionDisplayer*, GoStream*)
{
    GError ("GoSelectionDisplayer::save(): should never be called !");
    //never reached:
    return 0;
}


/*
 * This static method is responsible to restore "this" go from a stream.
 * For that purpose, it reads its private data and then calls the
 * readParams of its superclass.
 */
int GoSelectionDisplayer::readParams (GoSelectionDisplayer*, GoStream*)
{
    GError ("GoSelectionDisplayer::readParams(): should never be called !");
    //never reached:
    return 0;
}


GAssoc* GoSelectionDisplayer::d_get_assoc () const
{
    return (GAssoc*) &((GoSelectionDisplayer*)this)->_assoc;
}

