#ifdef REV_INFO
static char rcsid[] = "$Id: GoUISelector.cc,v 1.7 1992/12/31 13:09:23 jojic Exp $";
#endif
/*****************************************************************************
MODULE:  GoUI
FILE:    GoUISelector.cc
AUTHOR:  Olivier Jojic
DATE:    Nov 92

DESCRIPTION:
    

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  license from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include <Go/GoUISelector.h>
#include <Go/Composite.h>
#include <Go/GoRoot.h>
#include <Go/GoInDisp.h>
#include <Go/GoInterf.h>
#include <Go/Tools/Dash.h>
#include <Go/Sensor.h>
#include <Go/FullSensor.h>
#include <Go/Interactor.h>
#include <Go/MoveGrabber.h>
#include <Go/ChoosePoint.h>
#include <Go/GoSelDisp.h>
#include <Go/Tools/Bind.h>


void goui_select (GoUISelector* goui, const GPoint* p)
{
    goui->select (*p);
}


void goui_select_unselect (GoUISelector* goui, const GPoint* p)
{
    goui->selectUnselect (*p);
}


void goui_unselect_all (GoUISelector* goui)
{
    goui->unselectAll ();
}


void goui_select_parent (GoUISelector* goui)
{
    goui->selectParent ();
}


void goui_select_previous (GoUISelector* goui)
{
    goui->selectPrevious ();
}


void goui_select_next (GoUISelector* goui)
{
    goui->selectNext ();
}


void goui_select_children (GoUISelector* goui)
{
    goui->selectChildren ();
}


void goui_select_first_child (GoUISelector* goui)
{
    goui->selectFirstChild ();
}


void goui_group_all_selected_gos (GoUISelector* goui)
{
    goui->groupAllSelectedGos ();
}


void goui_group_all_selected_subtrees (GoUISelector* goui)
{
    goui->groupAllSelectedSubtrees ();
}


void goui_ungroup (GoUISelector* goui, Go* go)
{
    if (go->getMetaclass() == GoComposite::metaclass())
	goui->ungroup ((GoComposite*) go);
}


void goui_ungroup_selections (GoUISelector* goui)
{
    goui->ungroup ();
}


GoUISelector::GoUISelector (GoRoot* goroot) : GoUI (goroot)
{
    static GBool notFirstCall = GFALSE;

    if (!notFirstCall)
    {
	const char* typeFunction = "Function";

	GBind ("GoUISelector::select", &goui_select, typeFunction);
	GBind ("GoUISelector::selectUnselect", &goui_select_unselect,
	       typeFunction);
	GBind ("GoUISelector::unselectAll", &goui_unselect_all, typeFunction);
	GBind ("GoUISelector::selectParent", &goui_select_parent, typeFunction);
	GBind ("GoUISelector::selectPrevious", &goui_select_previous,
	       typeFunction);
	GBind ("GoUISelector::selectNext", &goui_select_next,
	       typeFunction);
	GBind ("GoUISelector::selectChildren", &goui_select_children,
	       typeFunction);
	GBind ("GoUISelector::selectFirstChild", &goui_select_first_child,
	       typeFunction);
	GBind ("GoUISelector::group", &goui_group_all_selected_gos,
	       typeFunction);
	GBind ("GoUISelector::groupGos", &goui_group_all_selected_gos,
	       typeFunction);
	GBind ("GoUISelector::groupSubtrees", &goui_group_all_selected_subtrees,
	       typeFunction);
	GBind ("GoUISelector::ungroup", &goui_ungroup, typeFunction);
	GBind ("GoUISelector::ungroupSelections", &goui_ungroup_selections,
	       typeFunction);

	notFirstCall = GTRUE;
    }

    selection_displayer = GoSelectionDisplayer::create();

    (void)GChoosePointPgm::create();
    (void)GMoveGrabberPgm::create();
    
    if (GoUI::root() != NULL)
	selection_displayer->reparent (GoUI::root());
}


GoUISelector::~GoUISelector ()
{
}


void GoUISelector::select (const GPoint& absPoint)
{
    GoRoot* goroot = GoUI::root();
    if (goroot != NULL)
    {
	Go* go = goroot->goAt (absPoint, maxDistance());
	unselectAll ();
	if (go != NULL && go != goroot)
	{
	    if (!go->selected ())
		go->select ();
	}
    }
}


void GoUISelector::selectUnselect (const GPoint& absPoint)
{
    GoRoot* goroot = GoUI::root();
    if (goroot != NULL)
    {
	Go* go = goroot->goAt (absPoint, maxDistance());
	if (go != NULL && go != goroot)
	{
	    if (go->selected ())
		go->unselect ();
	    else
		go->select ();
	}
    }
}

	
void goui_select_all_in_box (void* closure, Go* go)
{
    if (go == go->root())
	return;

    const GBox* box = (const GBox*) closure;

    GRegion goClip;
    go->addMyClip (goClip);
    if (goClip.empty ())
	go->addClip (goClip);

    GPoint firstCorner (goClip.bigbox().xmin, goClip.bigbox().ymin);
    GPoint secondCorner (goClip.bigbox().xmax, goClip.bigbox().ymax);
    
    if (box->contains (firstCorner) && box->contains (secondCorner))
	go->select ();
}    


void goui_unselect_all_in_box (void* closure, Go* go)
{
    const GBox* box = (const GBox*) closure;

    GRegion goClip;
    go->addMyClip (goClip);
    if (goClip.empty ())
	go->addClip (goClip);

    GPoint firstCorner (goClip.bigbox().xmin, goClip.bigbox().ymin);
    GPoint secondCorner (goClip.bigbox().xmax, goClip.bigbox().ymax);
    
    if (box->contains (firstCorner) && box->contains (secondCorner))
	go->unselect ();
}    


void GoUISelector::selectAllInBox (const GBox& box)
{
    GoRoot* goroot = GoUI::root();
    if (goroot != NULL)
	goroot->forEveryGo (goui_select_all_in_box, (GBox*)&box);
}


void GoUISelector::unselectAllInBox (const GBox& box)
{
    GoRoot* goroot = GoUI::root();
    if (goroot != NULL)
	goroot->forEverySelectedGo (goui_unselect_all_in_box, (GBox*)&box);
}


void goui_unselect (void*, Go* go)
{
    go->unselect ();
}


void GoUISelector::unselectAll ()
{
    GoRoot* goroot = GoUI::root();
    if (goroot != NULL)
	goroot->forEverySelectedGo (goui_unselect, NULL);
}

    
void goui_select_parent (void*, Go* go)
{
    Go* parent = go->parent();
    if (parent != parent->root())
    {
	go->unselect();
	parent->select();
    }
}


static void common_select_in_tree (GoRoot* root, void p(void*, Go*))
{
    if (root != NULL)
	root->forEverySelectedGo (p, NULL);
}


void GoUISelector::selectParent ()
{
    common_select_in_tree (GoUI::root(), goui_select_parent);
}


void goui_select_previous (void*, Go* go)
{
    Go* previous = go->previous();
    if (previous != NULL)
    {
	go->unselect();
	previous->select();
    }
}


void GoUISelector::selectPrevious ()
{
    common_select_in_tree (GoUI::root(), goui_select_previous);
}


void goui_select_next (void*, Go* go)
{
    Go* next = go->next();
    if (next != NULL)
    {
	go->unselect();
	next->select();
    }
}


void GoUISelector::selectNext ()
{
    common_select_in_tree (GoUI::root(), goui_select_next);
}


void goui_select_first_child (void*, Go* go)
{
    Go* firstChild = go->child();
    if (firstChild != NULL)
    {
	go->unselect();
	firstChild->select();
    }
}


void GoUISelector::selectFirstChild ()
{
    common_select_in_tree (GoUI::root(), goui_select_first_child);
}


void goui_select_children (void*, Go* go)
{
    Go* child = go->child();
    if (child != NULL)
    {
	go->unselect();
	for (; child != NULL; child = child->next())
	    child->select();
    }
}


void nothing_is_selected_cb (void* closure, Go* go)
{
    GBool* pNothing = (GBool*) closure;
    *pNothing = GFALSE;
    go->root()->stopRecursion (NULL);
}


GBool GoUISelector::nothingIsSelected () const
{
    GBool nothing = GTRUE;
    GoUI::root()->forEverySelectedGo (nothing_is_selected_cb, &nothing);
    return nothing;
}


void GoUISelector::selectChildren ()
{
    if (!nothingIsSelected())
	common_select_in_tree (GoUI::root(), goui_select_children);
    else
	for (Go* go = GoUI::root()->child(); go != NULL; go = go->next())
	    go->select();
}


void goui_enqueue (void* closure, Go* go)
{
    GQueue* q = (GQueue*) closure;
    q->putfirst (go);
}


void GoUISelector::enqueueAllSelectedGos (GQueue& q)
{
    GoRoot* goroot = GoUI::root();
    if (goroot != NULL)
	goroot->forEverySelectedGo (goui_enqueue, &q);
}


void GoUISelector::enqueueAllSelectedSubtrees (GQueue& q)
{
    GoRoot* goroot = GoUI::root();
    if (goroot != NULL)
	goroot->forEverySelectedSubtree (goui_enqueue, &q);
}


/* used by (un)groupAllSelectedGos */
struct StructureRec 
{
    Go* go;
    Go* parent;
    Go* next;

    StructureRec(Go* g) { go = g; parent = g->parent(); next = g->next(); }
};

static char* gouiGroupKey = "gouiGroupKey";


static GoComposite* common_group_all_selected (const GQueue& q)
{
    GoComposite* group = GoComposite::create ();
    GoRoot* root = NULL;

    int qRef;
    q.initRef (qRef);
    void* value;
    while (q.getRef (qRef, value))
    {
	Go* go = (Go*) value;

	if (root == NULL)
	{
	    root = go->root ();
	    group->reparent (root);
	}
	
	StructureRec strRec (go);
	group->assoc()->add (gouiGroupKey, sizeof strRec, &strRec);
	go->reparent (group);
    }

    return group;
}

GoComposite* GoUISelector::group (const GQueue& q)
{
    if (q.empty())
	return NULL;

    return common_group_all_selected (q);
}


GoComposite* GoUISelector::groupAllSelectedGos ()
{
    GQueue q;
    enqueueAllSelectedGos(q);
    if (q.empty())
	return NULL;

    return common_group_all_selected (q);
}


GoComposite* GoUISelector::groupAllSelectedSubtrees ()
{
    GQueue q;
    enqueueAllSelectedSubtrees (q);
    if (q.empty())
	return NULL;

    return common_group_all_selected (q);
}


void GoUISelector::ungroup ()
{
    GQueue q;
    enqueueAllSelectedGos (q);

    while (!q.empty())
    {
	Go* go = (Go*)q.getfirst();
	if (go->getMetaclass() == GoComposite::metaclass())
	    ungroup ((GoComposite*) go);
    }
}


void GoUISelector::ungroup (GoComposite* group)
{
    int count;
    StructureRec* pStr =
	(StructureRec*) group->assoc()->get (gouiGroupKey, count);

    for (int i = count - 1; i >= 0; --i)
	pStr[i].go->reparent (pStr[i].parent, pStr[i].next);
	
    group->destroy();
}


void GoUISelector::root (GoRoot* goroot)
{
    GoUI::root (goroot);
    selection_displayer->reparent (goroot);
}

	
