#ifdef REV_INFO
static char rcsid[] = "$Id: CSGrabber.cc,v 1.4 1992/12/24 10:04:12 jojic Exp $";
#endif
 /*****************************************************************************
MODULE:  Go Kernel (User Interface)
FILE:    CSGrabber.cc
AUTHOR:  Oliver Jojic
DATE:    July 92

DESCRIPTION:

COPYRIGHT:
  Copyright (c) 1990, 1991, 1992 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.

*****************************************************************************/

#include <Go/CSGrabber.h>
#include <Go/Tools/Bind.h>
#include <Go/Tools/Pen.h>
#include <Go/Tools/Cursor.h>
#include <Go/Go.h>
#include <Go/mGo.h>


void GCollectSomeGosFromThisTree (GQueue& q, Go* go, int max_depth, int max_count)
{
    if (go == NULL || max_depth < 0 || max_count < 0)
	return;

    int depth = 1;
    int count = 1;
    q.putfirst (go);

    GQueue qA, qB;
    GQueue* currentDepthQ = &qA;
    GQueue* nextDepthQ = &qB;
    currentDepthQ->putfirst(go);

    while (depth < max_depth && count < max_count)
    {
	int current_count;
	Go** pGo = (Go**)currentDepthQ->arrayFromFirst(current_count);
	if (current_count == 0)
	    break;
	for (int i = 0;
	     i < current_count && count < max_count;
	     ++i, ++pGo)
	{
	    for (Go* child = (*pGo)->Child(); child; child = child->Next())
	    {
		nextDepthQ->putlast(child);
		q.putlast(child);
		if (++count >= max_count)
		    break;
	    }
	}

	/* it is now time to add the next level */
	depth += 1;
	GQueue* tempQ = currentDepthQ;
	currentDepthQ = nextDepthQ;
	nextDepthQ = tempQ;
	nextDepthQ->clear();
    }
}


GoCSGrabberPgm::GoCSGrabberPgm (const char* name)
	: GGrabberPgm (name)
{
    call_cs_preconds = GTRUE;
    max_count = 32;
    max_depth = 16;

    // init the default pen to invert mode;
    _drawing_quality = GMAX_QUALITY;
    _drawing_pen.function(GINVERT);
    
    // the user does not determinate the direction;
    // so indicate any direction;
    cursorIdent(GCURSOR_ARROW_4D);
}    


void GoCSGrabberPgm::collectGosToBeDrawn (GQueue& q, Go* go)  const
{
    GCollectSomeGosFromThisTree(q, go, max_depth, max_count);
}


void remove_go_from_q (void* closure, const Go* destroyedGo)
{
    GQueue* q = (GQueue*) closure;

    int qRef;
    q->initRef (qRef);
    void* tmp;
    while (q->getRef (qRef, tmp))
    {
	Go* go = (Go*) tmp;
	if (go == destroyedGo)
	{
	    q->prevRef (qRef);
	    q->remRef (qRef);
	}
    }
}

	
void GoCSGrabber::doit ()
{
    GGrabber::doit();

    Go::metaclass()->destroyPostcond (remove_go_from_q, &drawn_gos, GREMOVE);

    if (change_subtree_cs)
	_go->setCoordsys (_cs);
    else
	_go->SetThisAbscs (_cs);
}


void GoCSGrabber::cancel ()
{
    GGrabber::cancel();
    Go::metaclass()->destroyPostcond (remove_go_from_q, &drawn_gos, GREMOVE);
}


void GoCSGrabber::draw ()
{
    if (!draw_from_expose && Pgm()->callCSPreconds())
	_go->callCSPrecond (_cs);

    clear ();
}

void GoCSGrabber::clear ()
{
    _go->wiredDraw (_cs, _drawing_pen, _drawing_quality);

    if (change_subtree_cs)
    {
	GCoordSys mulcs (_cs * ~_go->coordsys());
	int count;
	Go** pGo = (Go**) drawn_gos.arrayFromFirst (count);
	for (int i = 1; i < count; i++)
	    pGo[i]->wiredDraw (mulcs * pGo[i]->coordsys(),
			       _drawing_pen, _drawing_quality);
    }
}


void GoCSGrabber::Initialize ()
{
    if (change_subtree_cs)
	Pgm()->collectGosToBeDrawn (drawn_gos, _go);
    Go::metaclass()->destroyPostcond (remove_go_from_q, &drawn_gos);

    GGrabber::Initialize ();
}


GoCSGrabber::GoCSGrabber (GoCSGrabberPgm* pgm, const GEvent* event, Go* go)
    : GGrabber (pgm, event, go)
{
    change_subtree_cs = GTRUE;
    // copy the pgm Pen.
    _drawing_pen.modify(pgm->drawingPen().shPen());

    // copy the pgm quality;
    _drawing_quality = pgm->drawingQuality();
    _cs = _go->coordsys();
}
