#ifdef REV_INFO
static char rcsid[] = "$Id: ChoosePoint.cc,v 1.2 1992/12/22 10:48:30 jojic Exp $";
#endif
/*****************************************************************************
MODULE:  Go Kernel (User Interface)
FILE:    ChoosePoint.cc
AUTHOR:  Philippe Smadja.
DATE:    June 92

DESCRIPTION:
  See also ChoosePoint.h header.

  The edition of a polypoint consists to ADD points inside the polypoint.

  
COPYRIGHT:
  Copyright (c) 1990, 1991, 1992 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.

*****************************************************************************/

#include <Go/ChoosePoint.h>
#include <Go/GoInterf.h>
#include <Go/Tools/Bind.h>
#include <Go/Tools/Cursor.h>
#include <Go/mGo.h>
#include <stdio.h>
#include <assert.h>

/*
 * PROGRAMS
 **********
 */

// CHOOSE A POINT;

    
GChoosePointPgm::GChoosePointPgm (const char* name)
    : GGrabberPgm (name)
{
    // patch the default cursor;
    cursorIdent(GCURSOR_CROSSHAIR);
}

GChoosePointPgm* GChoosePointPgm::create ()
{
    void* value;
    const char* type;
    char  ChoosePointName[128];

    sprintf (ChoosePointName, "%s.choosePoint",
	   Go::metaclass()->classname());
    
    if (GBinded (ChoosePointName, value, type))
    {
	if (type != GInteractorPgm::type())
	{
	    GWarning ("GChoosePointPgm::create: the name \"%s\" is already used as a \"%s\" type.",
		      ChoosePointName, (type == NULL) ? "(NULL)" : type);
	    return NULL;
	}

	return (GChoosePointPgm*) value;
    }

    return new GChoosePointPgm (ChoosePointName);
}


GInteractor* GChoosePointPgm::start (const GEvent* event, Go* go)
{
    GInteractor* itor = new GChoosePoint (this, event, go);
    Initialize (itor);
    return itor;
}

/*
 * INTERACTOR
 ************
 */

GChoosePoint::GChoosePoint (GChoosePointPgm* pgm, const GEvent* event,
			    Go* go)
    : GGrabber(pgm, event, go)
{
    // invalid point;
    _point.x = -999;
    _point.y = -999;
}


/**
FUNCTION
    ::handle(const GEvent*)
DESCRIPTION
     look for cancel or doit event;
**/

GBool GChoosePoint::handle(const GEvent* event)
{
    if (state() != GITOR_RUNNING)
    {
	GWarning ("GGrabber::handle(): my state is not GITOR_RUNNING");
	return GFALSE;
    }

    if (isDoitEvent (event))
    {
	doit();
	return GTRUE;
    }

    if (isCancelEvent (event))
    {
	cancel ();
	return GTRUE;
    }
    return GFALSE;
}
    

GChoosePoint::~GChoosePoint ()
{
    // nothing to do;
}

void GChoosePoint::doit()
{
    // superclass first;
  GGrabber::doit();

    // read the point position;
    _point = go()->interf()->getPointer();
}


GEventMask GChoosePointPgm::inputMask () const
{
    return GGrabberPgm::inputMask() & ~GBUTTON_RELEASE_MASK;
}


GBool GChoosePoint::isDoitEvent (const GEvent* event) const
{
    return
	(event->type() == GBUTTON_RELEASE_EVENT)
	    ? GFALSE
	    : GGrabber::isDoitEvent (event);
}
