#ifdef REV_INFO
static char rcsid[] = "$Id: EdArc.cc,v 1.6 1992/12/31 10:15:27 smadja Exp $";
#endif
/*****************************************************************************
MODULE:  Go Kernel (User Interface)
FILE:    EdArc.cc
AUTHOR:  Philippe Smadja/Gilles Benati
DATE:    June 92

DESCRIPTION:
  See the EdArc.h header.
  We implement 3 programmes:
  1: to edit the first angle of the arc: name: "editBegin"
  2: to edit the last angle of the arc: name:  "editEnd"
  3: to edit the radii of the arc       name:  "editRadii"
  4: to edit the radxy of the arc       name:  "edit"
  5: to let the interactor to compute BEGIN or END and to edit it:
                                        name:  "movePoint"

COPYRIGHT:
  Copyright (c) 1990, 1991, 1992 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.

*****************************************************************************/

#include <Go/EdArc.h>
#include <Go/Arc.h>
#include <Go/Ellipse.h>
#include <Go/Circle.h>
#include <Go/Tools/Bind.h>
#include <Go/GoInterf.h>
#include <Go/GoRoot.h>
#include <Go/Tools/Error.h>
#include <Go/Tools/GeomUtil.h>
#include <stdio.h>

/*
 * PROGRAMS
 **********
 */

// BEGIN PROGRAM:
    

GEditArcBeginPgm* GEditArcBeginPgm::create ()
{
    void* value;
    const char* type;
    char  editArcName[128];

    sprintf (editArcName, "%s.editBegin",
	     GoArc::metaclass()->classname());

    if (GBinded (editArcName, value, type))
    {
	if (type != GInteractorPgm::type())
	{
	    GWarning ("GEditArcBeginPgm::create: the name \"%s\" is already used as a \"%s\" type.",
		      editArcName, (type == NULL) ? "(NULL)" : type);
	    return NULL;
	}

	return (GEditArcBeginPgm*) value;
    }

    return new GEditArcBeginPgm (editArcName);
}


GInteractor* GEditArcBeginPgm::start (const GEvent* event, Go* go)
{
    if (go->getMetaclass()->isSubclass(GoArc::metaclass()))
    {
	GInteractor* itor = new GEditArc (this, event,
					  (GoArc*)go, G_DO_BEGIN);
	Initialize (itor);
	return itor;
    }
    return NULL;
}


// END PROGRAM:

GEditArcEndPgm* GEditArcEndPgm::create ()
{
    void* value;
    const char* type;
    char  editArcName[128];

    sprintf (editArcName, "%s.editEnd",
	     GoArc::metaclass()->classname());

    if (GBinded (editArcName, value, type))
    {
	if (type != GInteractorPgm::type())
	{
	    GWarning ("GEditArcEndPgm::create: the name \"%s\" is already used as a \"%s\" type.",
		      editArcName, (type == NULL) ? "(NULL)" : type);
	    return NULL;
	}

	return (GEditArcEndPgm*) value;
    }

    return new GEditArcEndPgm (editArcName);
}


GInteractor* GEditArcEndPgm::start (const GEvent* event, Go* go)
{
    if (go->getMetaclass()->isSubclass(GoArc::metaclass()))
    {
	GInteractor* itor = new GEditArc (this, event,
					  (GoArc*)go, G_DO_END);
	Initialize (itor);
	return itor;
    }
    return NULL;
}



// RADII PROGRAM::
    
GEditArcRadiiPgm* GEditArcRadiiPgm::create ()
{
    void* value;
    const char* type;
    char  editArcName[128];

    sprintf (editArcName, "%s.editRadii",
	     GoArc::metaclass()->classname());

    if (GBinded (editArcName, value, type))
    {
	if (type != GInteractorPgm::type())
	{
	    GWarning ("GEditArcRadiiPgm::create: the name \"%s\" is already used as a \"%s\" type.",
		      editArcName, (type == NULL) ? "(NULL)" : type);
	    return NULL;
	}

	return (GEditArcRadiiPgm*) value;
    }

    return new GEditArcRadiiPgm (editArcName);
}


GInteractor* GEditArcRadiiPgm::start (const GEvent* event, Go* go)
{
    if (go->getMetaclass()->isSubclass(GoArc::metaclass()))
    {
	GInteractor* itor = new GEditArc (this, event,
					  (GoArc*)go, G_DO_RADII);
	Initialize (itor);
	return itor;
    }
    return NULL;
}


// RADXY PROGRAM:
// default editing program: name GoArc.edit.
    
GEditArcRadxyPgm* GEditArcRadxyPgm::create ()
{
    void* value;
    const char* type;
    char  editArcName[128];

    sprintf (editArcName, "%s.edit",
	     GoArc::metaclass()->classname());

    if (GBinded (editArcName, value, type))
    {
	if (type != GInteractorPgm::type())
	{
	    GWarning ("GEditArcRadxyPgm::create: the name \"%s\" is already used as a \"%s\" type.",
		      editArcName, (type == NULL) ? "(NULL)" : type);
	    return NULL;
	}

	return (GEditArcRadxyPgm*) value;
    }

    return new GEditArcRadxyPgm (editArcName);
}


GInteractor* GEditArcRadxyPgm::start (const GEvent* event, Go* go)
{
    if (go->getMetaclass()->isSubclass(GoArc::metaclass()))
    {
	GInteractor* itor = new GEditArc (this, event,
					  (GoArc*)go, G_DO_RADX | G_DO_RADY);
	Initialize (itor);
	return itor;
    }
    return NULL;
}


// MOVEPOINT PROGRAM.
    
GMoveArcPointPgm* GMoveArcPointPgm::create ()
{
    void* value;
    const char* type;
    char  moveArcPoint[128];

    sprintf (moveArcPoint, "%s.movePoint",
	     GoArc::metaclass()->classname());

    if (GBinded (moveArcPoint, value, type))
    {
	if (type != GInteractorPgm::type())
	{
	    GWarning ("GMoveArcPointPgm::create: the name \"%s\" is already used as a \"%s\" type.",
		      moveArcPoint, (type == NULL) ? "(NULL)" : type);
	    return NULL;
	}

	return (GMoveArcPointPgm*) value;
    }

    return new GMoveArcPointPgm (moveArcPoint);
}


/**
FUNCTION
    GMoveArcPointPgm::start()
DESCRIPTION
    computes the closest point of the arc to determinate
    If the user wants to move BEGIN or END.
**/

GInteractor* GMoveArcPointPgm::start (const GEvent* event, Go* go)
{
    if (go->getMetaclass()->isSubclass(GoArc::metaclass()))
    {
	GPoint position;
	if (event == NULL || (event->type() != GBUTTON_PRESS_EVENT &&
			      event->type() != GBUTTON_RELEASE_EVENT &&
			      event->type() != GCHAR_EVENT &&
			      event->type() != GMOTION_EVENT))
	    position = go->interf()->getPointer();
	else
	    position = event->position();

	// now position is available;

	GPoint pos = go->coordsys().from(position);
	GoArc* arc = (GoArc*)go;

	GNumber begin = arc->start();
	GNumber end =  begin + arc->amplitude();
	if (end > 2*PI)
	    end -= 2*PI;
	GNumber angle = GEllipseGetAngle(arc->radiusX(), arc->radiusY(),
					 pos);
	
	int begin_or_end;
	if (fabs(angle-begin) < fabs(angle-end))
	    begin_or_end = G_DO_BEGIN;
	else
	    begin_or_end = G_DO_END;
	
	GInteractor* itor = new GEditArc (this, event, arc,
					  begin_or_end);
	Initialize (itor);
	return itor;
    }
    return NULL;
}


/*
 * INTERACTOR
 ************
 */

// RADII, BEGIN, END;

GEditArc::GEditArc (GEditGeometricPgm* pgm, const GEvent* event,
		    GoArc* arc, int mask)
    : GEditGeometric(pgm, event, arc)
{
    _mask = mask;
}


void GEditArc::compute(const GEvent* event)
{
  GEditGeometric::compute(event);

    // _pos is now initialized;
    MyCompute();
}


GEditArc::~GEditArc ()
{
    if (_goWasMapped)
	go()->map();
}


void GEditArc::doit()
{
  GEditGeometric::doit();

    // reset the arc;
    // using the working_go values;
    
    GoArc* arc =  (GoArc*)go();
    GoArc* wgo = (GoArc*)WorkGo();
    
    arc->reset (wgo->radiusX(), wgo->radiusY(),
		wgo->amplitude(), wgo->start());
}

void GEditArc::draw()
{
    WorkGo()->draw();
}


void GEditArc::Initialize()
{
    //  put the closed flag is needed;
    int current_mask = mask();
    mask (current_mask);

    // unmap the go();
    _goWasMapped = go()->mapped();
    if (_goWasMapped)
    {
	go()->unmap();
	go()->root()->flush();
    }
    
    // update private values;
    // using the moude position;
    MyCompute();
    
    // call superclass;
  GEditGeometric::Initialize();
}

/***
 *
 * arc computation
 * definition of protected methods.
 *
 ***/

void  GEditArc::MyCompute()
{
    /*
     * _mask determines the behavior according to the bits that are set
     *	o  G_DO_RADX : modify radiusX (compatible with G_DO_RADY)
     *	o  G_DO_RADY : modify radiusY (compatible with G_DO_RADX)
     *	o  G_DO_RADII: modify and keep equal radiusX and radiusY
     *	o  G_DO_BEGIN: modify angle1 (keep angle1 < angle2)
     *	o  G_DO_END  : modify angle2 (keep angle1 < angle2)
     */
    GPoint rel_pos = go()->coordsys().from(_pos);
    GNumber amplitude = 0.;
    GoArc* w_arc = (GoArc*)WorkGo();
    
    if (_mask & G_DO_RADII)
	_radiusX = _radiusY = rel_pos.size();
    else {

	if (!(_mask & G_DO_RADX))
	    _radiusX = w_arc->radiusX();
	else _radiusX = fabs(rel_pos.x);

	if (!(_mask & G_DO_RADY))
	    _radiusY = w_arc->radiusY();
	else _radiusY = fabs(rel_pos.y);
    }
    if (_mask & G_DO_BEGIN) {
	_angle2 = w_arc->start() + w_arc->amplitude();
	_angle1 = GEllipseGetAngle(_radiusX, _radiusY, rel_pos);
	// angle1 in [0, 2PI];

	// adjust angle1 and angle2 so that angle1 < angle2;
	if (_angle1 >= _angle2 || _angle2 - _angle1 > 2*PI) {
	    _angle2 = GModulo(_angle2);
	    if (_angle2 == 0)
		_angle2 = 2*PI;
	    else if (_angle1 >= _angle2)
		_angle1 -= 2*PI;
	}
	amplitude = _angle2 - _angle1;
    }
    else if (_mask & G_DO_END) {
	_angle1 = w_arc->start();
	_angle2 = GEllipseGetAngle(_radiusX, _radiusY, rel_pos);
	// angle2 in [0, 2PI];
	
	// adjust angle1 and angle2 so that angle1 < angle2;
	if (_angle1 >= _angle2 || _angle2 - _angle1 > 2*PI) {
	    _angle1 = GModulo(_angle1);
	    if (_angle2 == 0)
		_angle2 = 2*PI;
	    else if (_angle1 >= _angle2)
		_angle1 -= 2*PI;
	}
	amplitude = _angle2 - _angle1;
    } else {
	_angle1 = w_arc->start();
	amplitude = w_arc->amplitude();
    }


    if (_radiusX == 0 || _radiusY == 0 ||
	amplitude == 0)
	return;
	
    GBool ok = ((GoArc*)go())->callResetPrecond(&_radiusX, &_radiusY,
						&amplitude, &_angle1);
    if (ok)
    {
	// reset working go before draw;
	w_arc->reset(_radiusX, _radiusY, amplitude, _angle1);
    }
}


void GEditArc::mask(int mask)
{
    _mask = mask;
    if (_mask & (G_DO_BEGIN|G_DO_END))
    {
	// Put the closed attributes to TRUE to draw the axes;
	(void)((GoArc*)WorkGo())->closed(GTRUE);
    }
}
