#ifdef REV_INFO
static char rcsid[] = "$Id: EdGeometric.cc,v 1.2 1992/12/10 14:23:55 smadja Exp";
#endif
/*****************************************************************************
MODULE:  Go Kernel (User Interface)
FILE:    EdGeometric.cc
AUTHOR:  Philippe Smadja.
DATE:    June 92

DESCRIPTION:
  See also EdGeometric.h header.

  The poupose of this class is to unmap outline and background of a
  Geometric during its edition.

  
COPYRIGHT:
  Copyright (c) 1990, 1991, 1992 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.

*****************************************************************************/

#include <Go/EdGeometric.h>
#include <Go/Tools/Bind.h>
#include <Go/Geometric.h>
#include <Go/GoRoot.h>
#include <Go/GoInterf.h>
#include <assert.h>

/*
 * NO PROGRAM
 ************
 */

GEditGeometricPgm::GEditGeometricPgm(const char* name)
    : GGrabberPgm(name)
{
    _drawing_quality = GMIN_QUALITY;
    _drawing_pen.function(GINVERT);
}

/*
 * INTERACTOR
 ************
 */

void* GEditGeometric::interactionKey()
{
    // returns unique ident;
    
   return (void*)"GEditGeometric::interactionKey";
}


GEditGeometric::GEditGeometric (GEditGeometricPgm* pgm, const GEvent* event,
				GoGeometric* go)
    : GGrabber(pgm, event, go)
{
    _drawing_quality = pgm->drawingQuality();

#ifndef GEDITGEOMETRIC__HIDE__
    _hide_go = NULL;
#else
    const GColor* background = go->interf()->background();
    if (!background)
	_hide_go = NULL;
    else
    {
	// dup the go to hide it;
	_hide_go = (GoGeometric*)go->dupThis();
	if (_hide_go)
	{
	    // If go stores a relative coordinate system, this one is copied;
	    // into duped go. But duped go has not yet any parent;
	    // Therefore the absolute position, which interest us;
	    // in this context, is lost;
	    // So:
	    _hide_go->setCoordsys (go->coordsys());
	    _hide_go->unmap();

	    // put _hide_go at the same place;
	    // and modify attributes to clear;
	    // with the color of the background;
	    // If any otherwise put a white outline;
	
	    _hide_go->pen().color(background);
	    _hide_go->pen().function(GCOPY);
	    _hide_go->pen().lineStyle(GFILL_SOLID);

	    _hide_go->outline(GTRUE);
	    _hide_go->background(GFALSE);
	}
    }
#endif
    
    // duplicate the Go;
    _work_go = (GoGeometric*)go->dupThis();
    if (_work_go)
    {
	// cf dupThis() comments above;
	_work_go->setCoordsys (go->coordsys());

	// add info for application control;
	// It can control interaction via metaclass pre/post conditions;

	_work_go->assoc()->set(interactionKey(), (void*)go);

	// reset _work_go to the wanted drawing;
	_work_go->unmap();
	_work_go->outline(GTRUE);
	_work_go->background(GFALSE);
	
	GPen& pen = pgm->drawingPen();
	_work_go->pen().modify(pen.shPen());
    }
}


GEditGeometric::~GEditGeometric()
{
    if (_hide_go)
      _hide_go->destroy();
    
    if (_work_go)
	_work_go->destroy();
}

void GEditGeometric::Initialize()
{
    if (_hide_go)
    {
	//reparent _hide_go on the top of the go tree;
	_hide_go->reparent(go()->root());
	_hide_go->map();
	
	// flush request BEFORE drawall;
	go()->root()->flush();
    }

    if (_work_go)
    {
	_work_go->reparent(go()->root());
	// do not map it;
    }
    
  GGrabber::Initialize();
}


