#ifdef REV_INFO
static char rcsid[] = "$Id: EdHandPoly.cc,v 1.2 1992/12/10 14:23:55 smadja Exp";
#endif
/*****************************************************************************
MODULE:  Go Kernel (User Interface)
FILE:    EdHandPoly.cc
AUTHOR:  Philippe Smadja.
DATE:    June 92

DESCRIPTION:
  See also EdHandPLine.h header.

  The edition of a polyline consists of moving a point of the polyline
  which is selected by the psotion of the mouse at start time.

  
COPYRIGHT:
  Copyright (c) 1990, 1991, 1992 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.

*****************************************************************************/

#include <Go/EdHandPLine.h>
#include <Go/Tools/Bind.h>
#include <Go/Polyline.h>
#include <Go/GoInterf.h>
#include <stdio.h>
#include <assert.h>

/*
 * PROGRAMS
 **********
 */

// EDIT HANDPOLYLINE;

GEditHandPolylinePgm::GEditHandPolylinePgm (const char* name)
    : GEditGeometricPgm (name)
{
}

GEditHandPolylinePgm* GEditHandPolylinePgm::create ()
{
    void* value;
    const char* type;
    char  editHandPolylineName[128];

    sprintf (editHandPolylineName, "%s.handEdit",
	     GoPolyline::metaclass()->classname());
    
    if (GBinded (editHandPolylineName, value, type))
    {
	if (type != GInteractorPgm::type())
	{
	    GWarning ("GEditHandPolylinePgm::create: the name \"%s\" is already used as a \"%s\" type.",
		      editHandPolylineName, (type == NULL) ? "(NULL)" : type);
	    return NULL;
	}

	return (GEditHandPolylinePgm*) value;
    }

    return new GEditHandPolylinePgm (editHandPolylineName);
}


GInteractor* GEditHandPolylinePgm::start (const GEvent* event, Go* go)
{
    // only for Polyline.
    if (go->getMetaclass() == GoPolyline::metaclass())
    {
	GInteractor* itor = new GEditHandPolyline (this, event,
						   (GoPolyline*)go);
	Initialize (itor);
	return itor;
    }
    return NULL;
}


/**
  * INTERACTOR
  ************
  **/

GEditHandPolyline::GEditHandPolyline (GEditGeometricPgm* pgm,
					const GEvent* event,
					GoPolyline* go)
    : _pts(sizeof(GPoint), 128), GEditGeometric(pgm, event, go)
{
    _newPoint = GFALSE;
    _index = 0;
}


void GEditHandPolyline::Initialize()
{
    // 1: copy the point array;
    
    GoPolyline* polyline = (GoPolyline*)go();
    _pts.add(polyline->points(), polyline->nbPoint());


    // compute the current index;
    if (_pts.nelems() == 0)
    {
	GPoint zero(0,0);
	_pts.add(&zero);
	_index = 0;
    }
    else
	_index = polyline->posToIndex(_pos);
    
    // 2: get the ABSOLUTE points;
    GPoint* points = (GPoint*)_pts.data();
    GCoordSys cs = polyline->coordsys();
    
    for (int i = 0; i < _pts.nelems(); i++, points++)
    {
	GPoint tmp = *points;
    	*points = cs.to(tmp);
    }

    // call superclass;
  GEditGeometric::Initialize();
}


GEditHandPolyline::~GEditHandPolyline ()
{
    // nothing to do; for me;
}


void GEditHandPolyline::compute (const GEvent* event)
{
  GEditGeometric::compute(event);

    // add _pos to the points array;
    // Note that we try to minimize the number of points;
    // For instance, if 3 points are on the same line we take;
    // only 2 points;

    
    if(_pts.nelems())
    {
	// do not get the current point if;
	// it is too near the previous one;
	GoInterf* intf = go()->interf();
	
	const GPoint*  _prev = &((GPoint*)_pts.data())[_index];
	if (fabs(_prev->x - _pos.x) <= 2*intf->pixelWidth() &&
	    fabs(_prev->y - _pos.y) <= 2*intf->pixelHeight())
	{
	    _newPoint = GFALSE;
	    return;
	}
    }

    // we will add a new point;
    // or patch the current one;
    // In any case:
    _newPoint = GTRUE;
    
    GPoint* points = (GPoint*)_pts.data();
	    
    if (_pts.nelems() >= 2)
    {
	// compute the angle between the point-1 and point;
	// and the angle between pos and point;
	// If there are equal replace current point by pos;
	// else add pos to the list;

	GPoint* currentPoint = &points[_index];
	long a0 = (long)
	    ((points[_index-1] - *currentPoint).angle() * 180 / PI + 0.5);
	long a1 = (long)((*currentPoint - _pos).angle() * 180 / PI + 0.5);

	if (max(a0,a1) - min(a0,a1) < 10)
	{
	    // clear from _index to _index+1;
	    // is done yet ( before ::compute())
	    
	    // clear from _index-1 to _index (the current one);
	    _index--;
	    clear();
	    
	    // reset index and patch the current point;
	    _index++;
	    *currentPoint = _pos;
	}
	else
	{
	    // add current point;
	    _pts.add(&_pos, 1, ++_index);
	}
    }
    else
    {
	// add current point;
	_pts.add(&_pos, 1, ++_index); 
    }
}

void GEditHandPolyline::doit()
{
  GEditGeometric::doit();

    // put the point array retative to coordsys;
    
    GCoordSys& cs = go()->coordsys();
    GPoint* points = (GPoint*)_pts.data();
    
    for (int i = 0 ; i < _pts.nelems(); i++, points++)
    {
	GPoint tmp = *points;
	*points = cs.from(tmp);
    }

    // reset the polyline with the point array;
    
    ((GoPolyline*)go())->reset(_pts.nelems(), (const GPoint*)_pts.data());
}


/**
FUNCTION
   GEditHandPolyline::drawAll
DESCRIPTION
    call the virtual wired draw of the working polypoint.
**/

void GEditHandPolyline::drawAll()
{
    go()->interf()->polyline(&WorkGo()->pen(), NULL, GUnitCoordSys(),
			     _pts.nelems(),
			     (const GPoint*)_pts.data());
}

void GEditHandPolyline::draw()
{
    if (!_newPoint)
	return;
    
    const GPoint* points = (const GPoint*)_pts.data() + _index - 1;
    int nb = 3;

    if (_index == 0)
	points++, nb--;

    if (_index == _pts.nelems() -1)
	nb--;

    go()->interf()->polyline(&WorkGo()->pen(), NULL,
			     GUnitCoordSys() /* the unit one */,
			     nb, points);
}


void GEditHandPolyline::clear()
{
    if (!_newPoint)
	return;

    // _newPoint will be reseted by compute;
    // clear from _index to _index+1;
    if (_index == _pts.nelems()-1)
	return;

    go()->interf()->polyline(&WorkGo()->pen(), NULL,
			     GUnitCoordSys(),
			     2, ((const GPoint*)_pts.data()) + _index);
}

void GEditHandPolyline::clearAll()
{
    drawAll();
}
