#ifdef REV_INFO
static char rcsid[] = "$Id: EdImage.cc,v 1.2 1992/12/10 14:23:55 smadja Exp";
#endif
/*****************************************************************************
MODULE:  Go Kernel (User Interface)
FILE:    EdImage.cc
AUTHOR:  Philippe Smadja/Gilles Benati
DATE:    June 92

DESCRIPTION:
  See the EdImage.h header.

COPYRIGHT:
  Copyright (c) 1990, 1991, 1992 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.

*****************************************************************************/

#include <Go/EdImage.h>
#include <Go/Image.h>
#include <Go/GoRoot.h>
#include <Go/Tools/Bind.h>
#include <stdio.h>

/*
 * PROGRAMME
 ***********
 */

GEditImagePgm::GEditImagePgm (const char* name)
    : GGrabberPgm (name)
{
    gravity (G_MOVE_SOUTH | G_MOVE_EAST);
    _pen.function(GINVERT);
}

void GEditImagePgm::gravity(int gravity)
{
    _gravity = gravity;
    int cid = GRectangleGravityToCursorIdent(gravity);
    // set the cursor ident;
    cursorIdent(cid);
}

GEditImagePgm* GEditImagePgm::create ()
{
    void* value;
    const char* type;
    char  editImageName[128];

    sprintf (editImageName, "%s.edit",
	     GoImage::metaclass()->classname());
    
    if (GBinded (editImageName, value, type))
    {
	if (type != GInteractorPgm::type())
	{
	    GWarning ("GEditImagePgm::create: the name \"%s\" is already used as a \"%s\" type.",
		      editImageName, (type == NULL) ? "(NULL)" : type);
	    return NULL;
	}

	return (GEditImagePgm*) value;
    }

    return new GEditImagePgm (editImageName);
}


GInteractor* GEditImagePgm::start (const GEvent* event, Go* go)
{
    if (go->getMetaclass()->isSubclass(GoImage::metaclass()))
    {
	GInteractor* itor = new GEditImage (this, event,
						(GoImage*)go);
	Initialize (itor);
	return itor;
    }
    return NULL;
}


/*
 * INTERACTOR
 ************
 */

void* GEditImage::interactionKey()
{
    // returns unique ident;
    
   return (void*)"GEditImage::interactionKey";
}

GEditImage::GEditImage (GEditImagePgm* pgm, const GEvent* event,
			GoImage* image)
    : GGrabber(pgm, event, image)
{
    _computer = new GEditAsRectangle(image->width(), image->height(),
				     pgm->gravity());
    // duplicate the Go;
    _work_image = (GoImage*)image->dupThis();
    if (_work_image)
    {
	// add info for application control;
	// It can control interaction via metaclass pre/post conditions;
	_work_image->assoc()->set(interactionKey(), (void*)image);

	// reset _work_image to the wanted drawing;
	_work_image->unmap();
    }
    _pen = new GPen(&pgm->pen());
}


void GEditImage::Initialize()
{
    if (_work_image)
    {
	_work_image->reparent(go()->root());
	// do not map it;
    }
  GGrabber::Initialize();
}

GEditImage::~GEditImage ()
{
    delete _computer;
    if (_work_image)
	_work_image->destroy();
    delete _pen;
}


void  GEditImage::compute(const GEvent* event)
{
    if (_work_image == NULL)
	return;
  GGrabber::compute(event);

    // _pos is now initialized;
    GPoint relative = go()->coordsys().from(_pos);
    _computer->compute(relative);

    // call precondition of the go();
    GNumber width = Width();
    GNumber height = Height();
    GPoint  origin = TopLeft();
    GBool ok = ((GoImage*)go())->callConfigPrecond(&width, &height, &origin);
    if (ok)
    {
	// reset the working go;
	_work_image->config(width, height, &origin);
    }
}

void GEditImage::doit()
{
    if (_work_image == NULL)
	return;
    
  GGrabber::doit();

    // reset the image;
    // using the working_go values;

    GoImage* image =  (GoImage*)go();

    image->config (_work_image->width(), _work_image->height(),
		   &_work_image->origin());
}


/**
FUNCTION
    ::draw
DESCRIPTION
    use the write draw of the image.
**/
void  GEditImage::draw()
{
    if (_work_image)
	_work_image->wiredDraw(_work_image->coordsys(), *_pen);
}
