#ifdef REV_INFO
static char rcsid[] = "$Id: EdLine.cc,v 1.2 1992/12/10 14:23:55 smadja Exp";
#endif
/*****************************************************************************
MODULE:  Go Kernel (User Interface)
FILE:    EdLine.cc
AUTHOR:  Philippe Smadja.
DATE:    Dec 92

DESCRIPTION:
  See the EdLine.h header.

COPYRIGHT:
  Copyright (c) 1990, 1991, 1992 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.

*****************************************************************************/

#include <Go/EdLine.h>
#include <Go/Line.h>
#include <Go/GoRoot.h>
#include <Go/Tools/Bind.h>
#include <Go/Tools/Cursor.h>
#include <stdio.h>

/*
 * PROGRAMS
 **********
 */

// LINE;
//**********;
    
  
GEditLinePgm::GEditLinePgm (const char* name)
               : GGrabberPgm (name)
{
    _pen.function(GINVERT);
    cursorIdent(GCURSOR_CROSSHAIR);
}

GEditLinePgm* GEditLinePgm::create ()
{
    void* value;
    const char* type;
    char  editLineName[128];

    sprintf (editLineName, "%s.edit",
	     GoLine::metaclass()->classname());
    
    if (GBinded (editLineName, value, type))
    {
	if (type != GInteractorPgm::type())
	{
	    GWarning ("GEditLinePgm::create: the name \"%s\" is already used as a \"%s\" type.",
		      editLineName, (type == NULL) ? "(NULL)" : type);
	    return NULL;
	}

	return (GEditLinePgm*) value;
    }

    return new GEditLinePgm (editLineName);
}


GInteractor* GEditLinePgm::start (const GEvent* event, Go* go)
{
    if (go->getMetaclass()->isSubclass(GoLine::metaclass()))
    {
	GInteractor* itor = new GEditLine (this, event,
					   (GoLine*)go);
	Initialize (itor);
	return itor;
    }
    return NULL;
}

/*
 * INTERACTOR
 ************
 */

void* GEditLine::interactionKey()
{
    // returns unique ident;
    
   return (void*)"GEditLine::interactionKey";
}

GEditLine::GEditLine (GEditLinePgm* pgm, const GEvent* event,
		      GoLine* line)
    : GGrabber(pgm, event, line)
{
    _work_line = (GoLine*)line->dupThis();
    if (_work_line)
    {
	// add info for application control;
	// It can control interaction via metaclass pre/post conditions;
	_work_line->assoc()->set(interactionKey(), (void*)line);

	// reset _work_line to the wanted drawing;
	_work_line->unmap();
    }
    _pen = new GPen(&pgm->pen());
}


GEditLine::~GEditLine ()
{
    if (_work_line)
	_work_line->destroy();
    delete _pen;
}

void GEditLine::Initialize()
{
    if (_work_line == NULL)
	return;

    // choose the point to move;
    GPoint first = ((GoLine*)go())->first();
    GPoint second = ((GoLine*)go())->first();
    GPoint current = go()->coordsys().from(_pos);

    // compute if we want to move first or second;
    
    _first =
	GSquareDistance(first, current) < GSquareDistance(second, current);

    if (_work_line)
    {
	_work_line->reparent(go()->root());
	// do not map it;
    }

    // call superclass last;
    GGrabber::Initialize();
}


void GEditLine::compute(const GEvent* event)
{
    if (_work_line == NULL)
	return;
    
    GGrabber::compute(event);

    // _pos is now initialized;
    GPoint relative = go()->coordsys().from(_pos);

    // call precondition of the go();
    // if ok: reset the working go;

    if (_first)
    {
	GBool ok = ((GoLine*)go())->callFirstPrecond(&relative);
	if (ok)
	    _work_line->first(relative);
    }
    else
    {
	GBool ok = ((GoLine*)go())->callSecondPrecond(&relative);
	if (ok)
	    _work_line->second(relative);
    }
}

void GEditLine::doit()
{
    if (_work_line == NULL)
	return;
    
    GGrabber::doit();

    // reset the line;
    // using the working_go values;

    GoLine* line =  (GoLine*)go();
    if (_first)
	line->first(_work_line->first());
    else
	line->second(_work_line->second());
}

void GEditLine::draw()
{
    if (_work_line)
	_work_line->wiredDraw(_work_line->coordsys(), *_pen);
}





