#ifdef REV_INFO
static char rcsid[] = "$Id: EdPolygon.cc,v 1.2 1992/12/10 14:23:55 smadja Exp";
#endif
/*****************************************************************************
MODULE:  Go Kernel (User Interface)
FILE:    EdPolygon.cc
AUTHOR:  Philippe Smadja.
DATE:    June 92

DESCRIPTION:
  See also EdPolygon.h header.

  The edition of a polygon consists of moving a point of the polygon
  which is selected by the psotion of the mouse at start time.

  
COPYRIGHT:
  Copyright (c) 1990, 1991, 1992 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.

*****************************************************************************/

#include <Go/EdPolygon.h>
#include <Go/Tools/Bind.h>
#include <Go/Polygon.h>
#include <Go/GoInterf.h>
#include <stdio.h>
#include <assert.h>

/*
 * PROGRAMS
 **********
 */


// EDIT POLYGON;

GEditPolygonPgm::GEditPolygonPgm (const char* name)
    : GEditPolylinePgm (name)
{
}

GEditPolygonPgm* GEditPolygonPgm::create ()
{
    void* value;
    const char* type;
    char  editPolygonName[128];

    sprintf (editPolygonName, "%s.edit",
	     GoPolygon::metaclass()->classname());
    
    if (GBinded (editPolygonName, value, type))
    {
	if (type != GInteractorPgm::type())
	{
	    GWarning ("GEditPolygonPgm::create: the name \"%s\" is already used as a \"%s\" type.",
		      editPolygonName, (type == NULL) ? "(NULL)" : type);
	    return NULL;
	}

	return (GEditPolygonPgm*) value;
    }

    return new GEditPolygonPgm (editPolygonName);
}


GInteractor* GEditPolygonPgm::start (const GEvent* event, Go* go)
{
    // THE edit polygon interactor is specific for GoPolygon;
    // may be for subclass (with specific ::draw() method);
    // it is not available so we do not take subclass of Polygon;

    if (go->getMetaclass() == GoPolygon::metaclass())
    {
	GInteractor* itor = new GEditPolygon (this, event,
					      (GoPolygon*)go,
					      GEDIT_POLYPOINT,
					      addPointButton());
	Initialize (itor);
	return itor;
    }

    if (go->getMetaclass()->isSubclass(GoPolypoint::metaclass()))
    {
	GInteractor* itor = new GEditPolypoint (this, event,
						(GoPolypoint*)go,
						GEDIT_POLYPOINT,
						addPointButton());
	Initialize (itor);
	return itor;
    }
    return NULL;
}






// MOVE A POINT OF A POLYGON;

GMovePolygonPgm::GMovePolygonPgm (const char* name)
    : GMovePolylinePgm (name)
{
}

GMovePolygonPgm* GMovePolygonPgm::create ()
{
    void* value;
    const char* type;
    char  movePolygonName[128];

    sprintf (movePolygonName, "%s.movePoint",
	     GoPolygon::metaclass()->classname());
    
    if (GBinded (movePolygonName, value, type))
    {
	if (type != GInteractorPgm::type())
	{
	    GWarning ("GMovePolygonPgm::create: the name \"%s\" is already used as a \"%s\" type.",
		      movePolygonName, (type == NULL) ? "(NULL)" : type);
	    return NULL;
	}

	return (GMovePolygonPgm*) value;
    }

    return new GMovePolygonPgm (movePolygonName);
}


GInteractor* GMovePolygonPgm::start (const GEvent* event, Go* go)
{
    // THE move polygon interactor is specific for GoPolygon;
    // may be for subclass (with specific ::draw() method);
    // it is not available so we do not take subclass of Polygon.

    if (go->getMetaclass() == GoPolygon::metaclass())
    {
	GInteractor* itor = new GEditPolygon (this, event,
					      (GoPolygon*)go,
					      GMOVE_POINT,
					      (GButton)0);
	Initialize (itor);
	return itor;
    }
    return NULL;
}


/*
 * INTERACTOR
 ************
 */

// It handles both Move and Edit program;

GEditPolygon::GEditPolygon (GEditGeometricPgm* pgm, const GEvent* event,
			    GoPolygon* go, int mode,
			    GButton addpoint_button)
    : GEditPolyline(pgm, event, go, mode, addpoint_button)
{
}

GEditPolygon::~GEditPolygon ()
{
    // nothing to do for me;
}


/**
FUNCTION
	GEditPolygon::draw/drawAll
DESCRIPTION
    draw: only the last line of the polygon and line to point 0.
    drawAll: all the polygon.
**/


void GEditPolygon::DrawLastLine()
{
    // terminate to draw the line from last point to first;
    int nb = NbPoints();
    if (nb <= 2)
	return;

    const GPoint* last = Points() + nb -1;
    const GPoint* first = Points();

    WorkGo()->interf()->line(&WorkGo()->pen(), go()->coordsys(), *first, *last,
			     DrawingQuality());
}

void GEditPolygon::draw()
{
  GEditPolyline::draw();
    DrawLastLine();
}

void GEditPolygon::drawAll()
{
    ((GoPolygon*)WorkGo())->draw();
}

/**
FUNCTION
    GEditPolygon::clear/clearAll()
DESCRIPTION
    the draw is in invert so clear si draw and clearAll is drawAll.
**/
void  GEditPolygon::clear()
{
  GEditPolyline::clear();
    if (NewPoint())
	DrawLastLine();
}

void  GEditPolygon::clearAll()
{
    drawAll();
}


