#ifdef REV_INFO
static char rcsid[] = "$Id: EdPolyline.cc,v 1.2 1992/12/10 14:23:55 smadja Exp";
#endif
/*****************************************************************************
MODULE:  Go Kernel (User Interface)
FILE:    EdPolyline.cc
AUTHOR:  Philippe Smadja.
DATE:    June 92

DESCRIPTION:
  See also EdPolyline.h header.

  The edition of a polyline consists of moving a point of the polyline
  which is selected by the psotion of the mouse at start time.

  
COPYRIGHT:
  Copyright (c) 1990, 1991, 1992 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.

*****************************************************************************/

#include <Go/EdPolyline.h>
#include <Go/Tools/Bind.h>
#include <Go/Polyline.h>
#include <Go/GoInterf.h>
#include <stdio.h>
#include <assert.h>

/*
 * PROGRAMS
 **********
 */


// EDIT POLYLINE;

GEditPolylinePgm::GEditPolylinePgm (const char* name)
    : GEditPolypointPgm (name)
{
}

GEditPolylinePgm* GEditPolylinePgm::create ()
{
    void* value;
    const char* type;
    char  editPolylineName[128];

    sprintf (editPolylineName, "%s.edit",
	     GoPolyline::metaclass()->classname());
    
    if (GBinded (editPolylineName, value, type))
    {
	if (type != GInteractorPgm::type())
	{
	    GWarning ("GEditPolylinePgm::create: the name \"%s\" is already used as a \"%s\" type.",
		      editPolylineName, (type == NULL) ? "(NULL)" : type);
	    return NULL;
	}

	return (GEditPolylinePgm*) value;
    }

    return new GEditPolylinePgm (editPolylineName);
}


GInteractor* GEditPolylinePgm::start (const GEvent* event, Go* go)
{
    // THE edit polyline interactor is specific for GoPolyline;
    // may be for subclass (with specific ::draw() method);
    // it is not available so we do not take subclass of Polyline;
			    
    if (go->getMetaclass() == GoPolyline::metaclass())
    {
	GInteractor* itor = new GEditPolyline (this, event,
					       (GoPolyline*)go,
					       GEDIT_POLYPOINT,
					       addPointButton());
	Initialize (itor);
	return itor;
    }
    
    if (go->getMetaclass()->isSubclass(GoPolypoint::metaclass()))
    {
	GInteractor* itor = new GEditPolypoint (this, event,
						(GoPolypoint*)go,
						GEDIT_POLYPOINT,
						addPointButton());
	Initialize (itor);
	return itor;
    }
    return NULL;
}



// MOVE A POINT OF A POLYPOINT;

GMovePolylinePgm::GMovePolylinePgm (const char* name)
    : GMovePolypointPgm (name)
{
}

GMovePolylinePgm* GMovePolylinePgm::create ()
{
    void* value;
    const char* type;
    char  movePolylineName[128];

    sprintf (movePolylineName, "%s.movePoint",
	     GoPolyline::metaclass()->classname());
    
    if (GBinded (movePolylineName, value, type))
    {
	if (type != GInteractorPgm::type())
	{
	    GWarning ("GMovePolylinePgm::create: the name \"%s\" is already used as a \"%s\" type.",
		      movePolylineName, (type == NULL) ? "(NULL)" : type);
	    return NULL;
	}

	return (GMovePolylinePgm*) value;
    }

    return new GMovePolylinePgm (movePolylineName);
}


GInteractor* GMovePolylinePgm::start (const GEvent* event, Go* go)
{
    // THE edit polyline interactor is specific for GoPolyline;
    // may be for subclass (with specific ::draw() method);
    // it is not available so we do not take subclass of Polyline.
			    
    if (go->getMetaclass() == GoPolyline::metaclass())
    {
	GInteractor* itor = new GEditPolyline (this, event,
					       (GoPolyline*)go,
					       GMOVE_POINT,
					       (GButton)0);
	Initialize (itor);
	return itor;
    }
    return NULL;
}


/*
 * INTERACTOR
 ************
 */

// It handles both MOVE and EDIT program (mode parameter);

GEditPolyline::GEditPolyline (GEditGeometricPgm* pgm, const GEvent* event,
			      GoPolyline* go, int mode,
			      GButton addpoint_button)
    : GEditPolypoint(pgm, event, go, mode, addpoint_button)
{
}

GEditPolyline::~GEditPolyline ()
{
    // nothing to do for me;
}


/**
FUNCTION
	GEditPolyline::draw/drawAll
DESCRIPTION
    draw: only the last line of the polyline.
    drawAll: all the polyline.
**/

void GEditPolyline::draw()
{
    const GPoint* points = Points() + Index() -1;
    int nb = 3;

    if (Index() == 0)
	points++, nb--;
    
    if (Index() == NbPoints()-1)
	nb--;

    go()->interf()->polyline(&WorkGo()->pen(), NULL, 
			     WorkGo()->coordsys(),
			     nb, points, DrawingQuality());
}


void GEditPolyline::drawAll()
{
    ((GoPolyline*)WorkGo())->GoPolyline::draw();
}


/**
FUNCTION
    GEditPolyline::clear/clearAll()
DESCRIPTION
    the draw is in invert so clear si draw and clearAll is drawAll.
**/
void  GEditPolyline::clear()
{
    if (NewPoint())
    {
	// clear from Index() to Index()+1;
        if (Index() == NbPoints()-1)
	    return;

	go()->interf()->polyline(&WorkGo()->pen(), NULL,
				 WorkGo()->coordsys(),
				 2, Points() + Index(),
				 DrawingQuality());
	
    }
    else
	draw();
}

void  GEditPolyline::clearAll()
{
    drawAll();
}
