#ifdef REV_INFO
static char rcsid[] = "$Id: EdPolypoint.cc,v 1.2 1992/12/10 14:23:55 smadja Exp";
#endif
/*****************************************************************************
MODULE:  Go Kernel (User Interface)
FILE:    EdPolypoint.cc
AUTHOR:  Philippe Smadja.
DATE:    June 92

DESCRIPTION:
  See also EdPolypoint.h header.

  The edition of a polypoint consists to ADD points inside the polypoint.

  
COPYRIGHT:
  Copyright (c) 1990, 1991, 1992 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.

*****************************************************************************/

#include <Go/EdPolypoint.h>
#include <Go/Tools/Bind.h>
#include <Go/Polypoint.h>
#include <Go/Tools/Cursor.h>
#include <stdio.h>
#include <assert.h>

/*
 * PROGRAMS
 **********
 */


// MOVE A POINT OF A POLYPOINT;

    
GMovePolypointPgm::GMovePolypointPgm (const char* name)
    : GEditGeometricPgm (name)
{
}


GMovePolypointPgm* GMovePolypointPgm::create ()
{
    void* value;
    const char* type;
    char  movePolypointName[128];

    sprintf (movePolypointName, "%s.movePoint",
	   GoPolypoint::metaclass()->classname());
    
    if (GBinded (movePolypointName, value, type))
    {
	if (type != GInteractorPgm::type())
	{
	    GWarning ("GMovePolypointPgm::create: the name \"%s\" is already used as a \"%s\" type.",
		      movePolypointName, (type == NULL) ? "(NULL)" : type);
	    return NULL;
	}

	return (GMovePolypointPgm*) value;
    }

    return new GMovePolypointPgm (movePolypointName);
}


GInteractor* GMovePolypointPgm::start (const GEvent* event, Go* go)
{
    if (go->getMetaclass()->isSubclass(GoPolypoint::metaclass()))
    {
	GInteractor* itor = new GEditPolypoint (this, event,
						(GoPolypoint*)go,
						GMOVE_POINT,
						(GButton)0);
	Initialize (itor);
	return itor;
    }
    return NULL;
}



// EDIT_POLYPOINT PROGRAM;
    
GEditPolypointPgm::GEditPolypointPgm (const char* name)
    : GEditGeometricPgm (name)
{
    _add_point_button = GBUTTON1;
    cursorIdent(GCURSOR_CROSSHAIR);
}

GEventMask GEditPolypointPgm::inputMask() const
{
    return GEditGeometricPgm::inputMask();
}

GEditPolypointPgm* GEditPolypointPgm::create ()
{
    void* value;
    const char* type;
    char  editPolypointName[128];

    sprintf (editPolypointName, "%s.edit",
	   GoPolypoint::metaclass()->classname());
    
    if (GBinded (editPolypointName, value, type))
    {
	if (type != GInteractorPgm::type())
	{
	    GWarning ("GEditPolypointPgm::create: the name \"%s\" is already used as a \"%s\" type.",
		      editPolypointName, (type == NULL) ? "(NULL)" : type);
	    return NULL;
	}

	return (GEditPolypointPgm*) value;
    }

    return new GEditPolypointPgm (editPolypointName);
}


GInteractor* GEditPolypointPgm::start (const GEvent* event, Go* go)
{
    if (go->getMetaclass()->isSubclass(GoPolypoint::metaclass()))
    {
	GInteractor* itor = new GEditPolypoint (this, event,
						(GoPolypoint*)go,
						GEDIT_POLYPOINT,
						addPointButton());
	Initialize (itor);
	return itor;
    }
    return NULL;
}


/*
 * INTERACTOR
 ************
 */

GEditPolypoint::GEditPolypoint (GEditGeometricPgm* pgm, const GEvent* event,
				GoPolypoint* go, int mode,
				GButton add_point_button)
    : GEditGeometric(pgm, event, go)
{
    _mode = mode;

    // Note that add_point_button does nor have sense;
    // it 'move' is true;
    _add_point_button = add_point_button;

}


void GEditPolypoint::Initialize()
{
    if (WorkGo())
    {
	if (_mode == GEDIT_POLYPOINT && WorkGo()->nbPoint() == 0)
	{
	    // call go preconditions;
	    _index = 0;
	    GPoint zero(0,0);
	    GBool ok = ((GoPolypoint*)go())->
		callAddPointPrecond(&_index, &zero);
	    if (ok)
		WorkGo()->addPoint(_index, zero, GFALSE);
	}
	else
	{
	    // compute the closest point;
	    // It defines the start of the updating;
	    _index =  WorkGo()->posToIndex(_pos);
	}
    
	_newPoint = GFALSE;

	if (_mode == GEDIT_POLYPOINT)
	{
	    // add( insert) an another point at the same place;

	    // call go preconditions;
	    GPoint current = *(WorkGo()->points() + _index);
	    GBool ok = ((GoPolypoint*)go())->
		callAddPointPrecond(&_index, &current);

	    if (ok)
	    {
		WorkGo()->addPoint(_index, current, GFALSE);
		_index++;
	    }
	}
    }

  GEditGeometric::Initialize();
}


GEditPolypoint::~GEditPolypoint ()
{
}


void GEditPolypoint::compute (const GEvent* event)
{
  GEditGeometric::compute(event);

    if (WorkGo())
    {
	// now _pos is initialized;
	// do my computation;
    
	GPoint relative;

	if (_newPoint)
	{
	    // previous relative pos;
	    relative = WorkGo()->coordsys().from(prev_pos);
	    
	    _newPoint = GFALSE;
	    // call go preconditions;
	    GBool ok = ((GoPolypoint*)go())->
		callAddPointPrecond(&_index, &relative);
	    if (ok)
	    {
		// on work_go now;
		WorkGo()->addPoint(_index, relative, GFALSE);
		_index++;
	    }
	}
	else
	{
	    // current relative point;
	    relative = WorkGo()->coordsys().from(_pos);

	    // call go preconditions;
	    GBool ok = ((GoPolypoint*)go())->
		callMovePointPrecond(&_index, &relative);
	    if (ok)
	    {
		// update the work go;
		WorkGo()->movePoint(_index, relative, GFALSE);
	    }
	}
    }
}

void GEditPolypoint::doit()
{
  GEditGeometric::doit();

    if (WorkGo())
    {
	// reset the polypoint;
	// the last point was not accepted by the user;

	if (!_newPoint && _mode != GMOVE_POINT)
	    WorkGo()->delPoint(_index);

	((GoPolypoint*)go())->reset(WorkGo()->nbPoint(),
				    WorkGo()->points());
    }
}

/***
 *
 * polypoint computation
 * definition of protected methods.
 *
 ***/

/**
FUNCTION
    Points()/NbPoints()
DESCRIPTION
    return the points array to draw.
    Warning: They must be used at the same time ||
**/
const GPoint* GEditPolypoint::Points() const
{
    if (WorkGo())
	return (const GPoint*)WorkGo()->points();
    else
	return NULL;
}

int GEditPolypoint::NbPoints() const
{
    if (WorkGo())
	return WorkGo()->nbPoint();
    else
	return 0;
}
    
GBool GEditPolypoint::isDoitEvent(const GEvent* event) const
{
    return  GEditGeometric::isDoitEvent(event);
}

/**
FUNCTION
    EdPolypoint::handle()
DESCRIPTION
    
    Incremental rubber band drawing of a polyline (or polygon) :
        o  On Move we do not add or remove point.
    	o  button 'add_point_button':
	     adds the pointer (mouse) position to the array of points
	o  control (add_point_button) removes last point
**/
GBool GEditPolypoint::handle(const GEvent* event)
{
    // handle my events;
    if (_mode == GEDIT_POLYPOINT &&
	(event->type() == GBUTTON_RELEASE_EVENT ||
	 event->type() == GBUTTON_PRESS_EVENT) &&
	event->button() == _add_point_button)
    {
	if (event->type() == GBUTTON_PRESS_EVENT)
	{
	    if (event->modifiers() == 0)
	    {
		// add an another last point at the same place;
		// nothing to draw;
		// by copying  the current point;
		// This point does not have to be drawn;

		_newPoint = GTRUE;
	    }
	    else if (event->modifiers() & (long)GCONTROL &&
		     NbPoints() > 2)
	    {
		assert (_index > 0);
		if (_newPoint)
		    _newPoint = GFALSE;
		else
		{
		    // clear all drawing;
		    clearAll();

		    // remove the last point;
		    WorkGo()->delPoint(_index);
		    if (--_index == 0)
			_index = 1;

		    // redo all drawing;;
		    drawAll();
		}
	    }
	}

	// for both press and release of add_point_button;
	// my events are handled;
	return GTRUE;
    }

    // not my event ==>;
    // call the superclass  handler;
    
    return GEditGeometric::handle(event);

}


/**
FUNCTION
    ::draw
DESCRIPTION
    call the virtual wired draw of the working polypoint.
**/

void GEditPolypoint::draw()
{
    if (WorkGo())
	WorkGo()->wiredDraw(WorkGo()->coordsys(), WorkGo()->pen(),
			    DrawingQuality());
}






