#ifdef REV_INFO
static char rcsid[] = "$Id: EdRectangle.cc,v 1.3 1992/12/23 12:10:39 smadja Exp $";
#endif
/*****************************************************************************
MODULE:  Go Kernel (User Interface)
FILE:    EdRectangle.cc
AUTHOR:  Philippe Smadja/Gilles Benati
DATE:    June 92

DESCRIPTION:
  See the EdRectangle.h header.

COPYRIGHT:
  Copyright (c) 1990, 1991, 1992 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.

*****************************************************************************/

#include <Go/EdRectangle.h>
#include <Go/Rectangle.h>
#include <Go/Tools/Bind.h>
#include <Go/Tools/Cursor.h>
#include <stdio.h>

/*
 * PROGRAMS
 **********
 */

// RECTANGLE;
//**********;
    
  
GEditRectanglePgm::GEditRectanglePgm (const char* name)
    : GEditGeometricPgm (name)
{
    gravity (G_OPPOSITE_GRAVITY);
}

void  GEditRectanglePgm::gravity(int gravity)
{
    _gravity = gravity;
    int cid = GRectangleGravityToCursorIdent(gravity);
    // set the cursor ident;
    cursorIdent(cid);
}
    
GEditRectanglePgm* GEditRectanglePgm::create ()
{
    void* value;
    const char* type;
    char  editRectangleName[128];

    sprintf (editRectangleName, "%s.edit",
	     GoRectangle::metaclass()->classname());
    
    if (GBinded (editRectangleName, value, type))
    {
	if (type != GInteractorPgm::type())
	{
	    GWarning ("GEditRectanglePgm::create: the name \"%s\" is already used as a \"%s\" type.",
		      editRectangleName, (type == NULL) ? "(NULL)" : type);
	    return NULL;
	}

	return (GEditRectanglePgm*) value;
    }

    return new GEditRectanglePgm (editRectangleName);
}


GInteractor* GEditRectanglePgm::start (const GEvent* event, Go* go)
{
    if (go->getMetaclass()->isSubclass(GoRectangle::metaclass()))
    {
	GInteractor* itor = new GEditRectangle (this, event,
						(GoRectangle*)go);
	Initialize (itor);
	return itor;
    }
    return NULL;
}

// SQUARE PGM;
//************;

#define GoSquare GoRectangle


GEditSquarePgm::GEditSquarePgm (const char* name)
    : GEditRectanglePgm (name)
{
    gravity(G_NO_GRAVITY | G_AS_SQUARE);
}


GEditSquarePgm* GEditSquarePgm::create ()
{
    void* value;
    const char* type;
    char  editSquareName[128];

    sprintf (editSquareName, "%s.squareEdit",
	     GoSquare::metaclass()->classname());
    
    if (GBinded (editSquareName, value, type))
    {
	if (type != GInteractorPgm::type())
	{
	    GWarning ("GEditSquarePgm::create: the name \"%s\" is already used as a \"%s\" type.",
		      editSquareName, (type == NULL) ? "(NULL)" : type);
	    return NULL;
	}

	return (GEditSquarePgm*) value;
    }

    return new GEditSquarePgm (editSquareName);
}


GInteractor* GEditSquarePgm::start (const GEvent* event, Go* go)
{
    if (go->getMetaclass()->isSubclass(GoSquare::metaclass()))
    {
	GInteractor* itor = new GEditRectangle (this, event,
						(GoSquare*)go);
	Initialize (itor);
	return itor;
    }
    return NULL;
}


/*
 * INTERACTOR
 ************
 */

GEditRectangle::GEditRectangle (GEditRectanglePgm* pgm, const GEvent* event,
				GoRectangle* rectangle)
    : GEditGeometric(pgm, event, rectangle)
{
    _computer = new GEditAsRectangle(rectangle->width(), rectangle->height(),
				     pgm->gravity());
}


GEditRectangle::~GEditRectangle ()
{
    delete _computer;
}



void GEditRectangle::compute(const GEvent* event)
{
  GEditGeometric::compute(event);

    // _pos is now initialized;
    // compute the relative  mouse position;
    
    GPoint relative = go()->coordsys().from(_pos);
    _computer->compute(relative);

    // call precondition of the go();
    GNumber width = Width();
    GNumber height = Height();
    GBool ok = ((GoRectangle*)go())->callResetPrecond(&width, &height);
    if (ok)
    {
	// reset the working go;
	((GoRectangle*)WorkGo())->reset(width, height);
    }

    // some gravity may modify the origin;
    // The GoRectangle does not have the origin: so do moveThis();

    const GPoint& top_left = _computer->topLeft();

    GCoordSys abscs = WorkGo()->coordsys();
    abscs.O = go()->coordsys().to(top_left);

    // call preconditions;
    ok = go()->callCSPrecond(abscs);
    if (ok)
    {
	// move the go if needed;
	    WorkGo()->SetThisAbscs(abscs);
    }
}

void GEditRectangle::doit()
{
  GEditGeometric::doit();

    // reset the rectangle;
    // using the working_go values;

    GoRectangle* rectangle =  (GoRectangle*)go();
    GoRectangle* wgo = (GoRectangle*)WorkGo();
    GBool ismapped = rectangle->mapped();
    if (ismapped)
	rectangle->unmap();
    // update the rectangle to be edited;
    rectangle->reset (wgo->width(), wgo->height());
    rectangle->SetThisAbscs(WorkGo()->coordsys());

    if (ismapped)
	rectangle->map();
}

void GEditRectangle::draw()
{
    WorkGo()->draw();
}





