#ifdef REV_INFO
static char rcsid[] = "$Id: EdRegPolygon.cc,v 1.3 1992/12/23 14:16:10 smadja Exp $";
#endif
/*****************************************************************************
MODULE:  Go Kernel (User Interface)
FILE:    EdRegPolygon.cc
AUTHOR:  Philippe Smadja/Gilles Benati
DATE:    June 92

DESCRIPTION:
  See the EdRegPoly.h header.

COPYRIGHT:
  Copyright (c) 1990, 1991, 1992 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.

*****************************************************************************/

#include <Go/EdRegPolygon.h>
#include <Go/RegPoly.h>
#include <Go/Tools/Bind.h>
#include <Go/Tools/Cursor.h>
#include <Go/EdAsRect.h>  /* to compute cursor */
#include <stdio.h>

/*
 * PROGRAMME
 ***********
 */
GEditRegPolygonPgm::GEditRegPolygonPgm (const char* name)
    : GEditGeometricPgm (name)
{
    int cid = GRectangleGravityToCursorIdent(G_NO_GRAVITY);
    cursorIdent(cid);
}

GEditRegPolygonPgm* GEditRegPolygonPgm::create ()
{
    void* value;
    const char* type;
    char  editRegPolygonName[128];

    sprintf (editRegPolygonName, "%s.edit",
	     GoRegPolygon::metaclass()->classname());
    
    if (GBinded (editRegPolygonName, value, type))
    {
	if (type != GInteractorPgm::type())
	{
	    GWarning ("GEditRegPolygonPgm::create: the name \"%s\" is already used as a \"%s\" type.",
		      editRegPolygonName, (type == NULL) ? "(NULL)" : type);
	    return NULL;
	}

	return (GEditRegPolygonPgm*) value;
    }

    return new GEditRegPolygonPgm (editRegPolygonName);
}


GInteractor* GEditRegPolygonPgm::start (const GEvent* event, Go* go)
{
    if (go->getMetaclass()->isSubclass(GoRegPolygon::metaclass()))
    {
	GInteractor* itor = new GEditRegPolygon (this, event,
						(GoRegPolygon*)go);
	Initialize (itor);
	return itor;
    }
    return NULL;
}


/*
 * INTERACTOR
 ************
 */

GEditRegPolygon::GEditRegPolygon (GEditRegPolygonPgm* pgm, const GEvent* event,
				  GoRegPolygon* regpolygon)
    : GEditGeometric(pgm, event, regpolygon)
{
    // needs a cast because some compiler are not enought cleaver;
    _nb = ((GoPolypoint*)regpolygon)->nbPoint();
}

void GEditRegPolygon::Initialize()
{
    // cs and deform_cs have the same origin;
    // but deform_cs.I will follow the pointer (mouse) position;
    _deform_cs.O = go()->coordsys().O;

    MyCompute();
    
    // now call the superclass;
  GEditGeometric::Initialize();
}

GEditRegPolygon::~GEditRegPolygon ()
{
    // nothing to do;
}


void GEditRegPolygon::doit()
{
  GGrabber::doit();

    // reset the regpolygon;
    // note that the _cs.I and _cs.J are not modified;
    // It is not a rescale:

    GoRegPolygon* regpolygon =  (GoRegPolygon*)go();

    // reset  radius and angle;
    GVector first_rad = go()->coordsys().from(_deform_cs.I);
    regpolygon->reset(_nb, first_rad.size(), first_rad.angle());
}

void GEditRegPolygon::draw()
{
    ((GoRegPolygon*)WorkGo())->draw();
}



void GEditRegPolygon::compute (const GEvent* event)
{
  GGrabber::compute(event);

    // prev_pos and _pos are initialized;
    MyCompute();
}

void GEditRegPolygon::MyCompute()
{
    // compute deform_cs instead of reseting summits;
    // deform_cs.O does not move (same as cs.O);
    // but deform_cs.I follows the pointer (mouse) position;
    // and deform_cs.J rotates and rescales accordingly;

    const GCoordSys& cs = go()->coordsys();
    _deform_cs.I.x = _pos.x - cs.O.x;
    _deform_cs.I.y = _pos.y - cs.O.y;
    _deform_cs.J   = cs.from(_pos);

    GNumber tmp   = _deform_cs.J.x;
    _deform_cs.J.x = -_deform_cs.J.y;
    _deform_cs.J.y = tmp;
    _deform_cs.J   = cs.to(_deform_cs.J);

    // compute radius and angle;
    
    GVector first_rad = cs.from(_deform_cs.I);
    GNumber radius = first_rad.size();
    if (radius < 1)
	radius = 1;
    GNumber angle = first_rad.angle();

    // call go preconditions;
    int nb = _nb;
    GBool ok = ((GoRegPolygon*)go())->callResetPrecond(&nb, &radius, &angle);
    if (ok)
    {
	if (nb != _nb)
	    GWarning("GEditRegPolygon::compute() application wants  %s",
		     "a different number of edges: the request is ignored");
	
	// reset the working go before new drawing;
	((GoRegPolygon*)WorkGo())->reset(_nb, radius, angle);
    }
}



