#ifdef REV_INFO
static char rcsid[] = "$Id: RescaleGr.cc,v 1.3 1992/12/14 08:37:50 jojic Exp $";
#endif
/*****************************************************************************
MODULE:  Go Kernel (User Interface)
FILE:    RescaleGrabber.cc
AUTHOR:  Oliver Jojic
DATE:    June 92

DESCRIPTION:
  See also the RescaleGrabber.h header.

COPYRIGHT:
  Copyright (c) 1990, 1991, 1992 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.

*****************************************************************************/

#include <Go/RescaleGr.h>
#include <Go/Go.h>
#include <Go/Tools/Bind.h>


const char* rescaleGrabberName = "Go.rescale";


GRescaleGrabberPgm* GRescaleGrabberPgm::create ()
{
    void* value;
    const char* type;

    if (GBinded (rescaleGrabberName, value, type))
    {
	if (strcmp (type, GInteractorPgm::type()))
	{
	    GWarning ("GRescaleGrabberPgm::create: the name \"%s\" is already used as a \"%s\" type.",
		      rescaleGrabberName, (type == NULL) ? "(NULL)" : type);
	    return NULL;
	}

	return (GRescaleGrabberPgm*) value;
    }

    return new GRescaleGrabberPgm (rescaleGrabberName);
}


GInteractor* GRescaleGrabberPgm::start (const GEvent* event, Go* go)
{
    GInteractor* itor = new GRescaleGrabber (this, event, go);
    Initialize (itor);
    return itor;
}


GRescaleGrabberPgm::GRescaleGrabberPgm (const char* name)
	: GoCSGrabberPgm (name)
{
}


void GRescaleGrabber::compute (const GEvent* event)
{
    GoCSGrabber::compute (event);

    GPoint relp(clip_cs.from(_pos));
    compute_direction (relp);

    if (around_center)
    {
	if (relp.x < 0)
	    relp.x = - relp.x;
	if (relp.y < 0)
	    relp.y = - relp.y;
    }
    
    GCoordSys newcs (clip_cs);
    if (_direction & GDIR_RIGHT)
	newcs.I *= relp.x;
    else if (_direction & GDIR_LEFT)
    {
	newcs.O += clip_cs.I * relp.x;
	newcs.I = (1 - relp.x) * clip_cs.I;
    }

    if (_direction & GDIR_DOWN)
	newcs.J *= relp.y;
    else if (_direction & GDIR_UP)
    {
	newcs.O += clip_cs.J * relp.y;
	newcs.J = clip_cs.J * (1 - relp.y);
    }

    _cs = newcs * clip_cs_inv * _go->coordsys();
}


GRescaleGrabber::GRescaleGrabber (GRescaleGrabberPgm* pgm, const GEvent* event, Go* go)
	: GoCSGrabber (pgm, event, go)
{
    around_center = GTRUE;

    /* If we rescale directly with "go->coordsys()" we may got into such problems:
    		suppose a GoRectangle with a unary coordinate system but with a width and an height
		of 100. If we use only the coordinate system of the Go, each time the mouse will be moved
		by one mm, the rectangle width and height will be extended by 100 mm !
	To avoid this bad behavior we do not work directly on "go->coordsys()" but on "clip_cs",
    	which is approximatively the coordinate system of the Go where I and J vectors are
	 extended until the output clip of the Go.
	     */

    GRegion clip; go->addMyClip (clip);
    if (clip.empty())
	go->addClip (clip);

    /* clip_box is available for subclasses who would recompute a differant clip_cs */
    clip_box = clip.bigbox();

    int six = (_cs.I.x >= 0) ? 1 : -1;
    int siy = (_cs.I.y >= 0) ? 1 : -1;
    int sjx = (_cs.J.x >= 0) ? 1 : -1;
    int sjy = (_cs.J.y >= 0) ? 1 : -1;

    GNumber deter = six*_cs.I.x * sjy*_cs.J.y - siy*_cs.I.y * sjx*_cs.J.x;
    if (deter == 0)
	deter = .001; //BUG!!!
    GNumber lambda =
	(clip_box.width() * sjy*_cs.J.y - clip_box.height() * sjx*_cs.J.x)
	    / deter;
    GNumber mu =
	(six*_cs.I.x * clip_box.height() - siy*_cs.I.y * clip_box.width())
	    / deter;

    clip_cs.O = GPoint (clip_box.xmin, clip_box.ymin);
    clip_cs.I = lambda * _cs.I;
    clip_cs.J = mu * _cs.J;

    if (six < 0) clip_cs.O.x -= clip_cs.I.x;
    if (sjx < 0) clip_cs.O.x -= clip_cs.J.x;
    if (siy < 0) clip_cs.O.y -= clip_cs.I.y;
    if (sjy < 0) clip_cs.O.y -= clip_cs.J.y;

    clip_cs.I *= 0.5;
    clip_cs.J *= 0.5;
    clip_cs.O += clip_cs.I + clip_cs.J;
    
    clip_cs_inv = ~clip_cs;

    _direction = 0;
    compute_direction (clip_cs.from(_pos));
}


void GRescaleGrabber::compute_direction (GPoint relp)
{
    if (around_center)
    {
	if (relp.x <= 0)
	    relp.x = (relp.x == 0) ? .5 : - relp.x;
	if (relp.y <= 0)
	    relp.y = (relp.y == 0) ? .5 : - relp.y;
    }
    
    if (relp.x <= 0)
    {
	_direction &= ~GDIR_RIGHT;
	_direction |= GDIR_LEFT;
    }
    else if (relp.x >= 1)
    {
	_direction &= ~GDIR_LEFT;
	_direction |= GDIR_RIGHT;
    }

    if (relp.y <= 0)
    {
	_direction &= ~GDIR_DOWN;
	_direction |= GDIR_UP;
    }
    else if (relp.y >= 1)
    {
	_direction &= ~GDIR_UP;
	_direction |= GDIR_DOWN;
    }
}
