#ifdef REV_INFO
static char rcsid[] = "$Id: SimpleGrPgm.cc,v 1.1 1992/12/10 14:42:10 jojic Exp $";
#endif
/*****************************************************************************
MODULE:  Go Kernel (User Interface)
FILE:    SGrabberPgm.cc
AUTHOR:  Philippe Smadja.
DATE:    Nov 92

DESCRIPTION:

  See SGrabberPgm.h

COPYRIGHT:
  Copyright (c) 1990, 1991, 1992 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.

*****************************************************************************/

#include <Go/SimpleGrPgm.h>
#include <Go/Tools/Bind.h>
#include <Go/Tools/Cursor.h>
#include <Go/Tools/Error.h>

/*
 * NO INTERACTOR
 ***************
 */


// It must provided by application;
// via the constructor;


/*
 * PROGRAM
 *********
 */

GSimpleGrabberPgm::GSimpleGrabberPgm (const char* name,
				       GGrabberNew constructor)
: GGrabberPgm (name)
{
    // patch the default cursor;
    cursorIdent(GCURSOR_CROSSHAIR);

    _constructor = constructor;
}

GSimpleGrabberPgm* GSimpleGrabberPgm::create (const char* pgm_name,
					      GGrabberNew constructor)
{
    if (pgm_name == NULL || constructor == NULL)
	return NULL;
    
    const char* type;
    void* value;

    if (GBinded (pgm_name, value, type))
    {
	if (type != GInteractorPgm::type())
	{
	    GWarning ("GSimpleGrabberPgm::create: the name \"%s\" is already used as a \"%s\" type.",
		      pgm_name, (type == NULL) ? "(NULL)" : type);
	    return NULL;
	}

	// reset the used constructor;
	((GSimpleGrabberPgm*)value)->constructor(constructor);

	// return the program;
	return (GSimpleGrabberPgm*)value;
    } 
    
    return new GSimpleGrabberPgm (pgm_name, constructor);
}


GInteractor* GSimpleGrabberPgm::start (const GEvent* event, Go* go)
{
    if (_constructor)
    {
	GInteractor* itor = (*_constructor)(this, event, go);
	Initialize (itor);
	return itor;
    }
    return NULL;
}


