#ifdef REV_INFO
static char rcsid[] = "$Id: TextSelPtn.cc,v 1.2 1992/12/10 14:23:55 smadja Exp";
#endif
/*****************************************************************************
MODULE:  Go Kernel (User Interface)
FILE:    TextSelPtn.cc
AUTHOR:  Philippe Smadja/ Michel Andre
DATE:    Nov 92

DESCRIPTION:
  The TextSelPortion is an Interactor which handle the selection
  of a portion of text. It is highlight the selected area.
  and call 'selectPortion()' of the GoTextual with the area.
  The highlight is done the the Textual itself.
  
  See also the TextSelPtn.h header.

COPYRIGHT:
  Copyright (c) 1990, 1991, 1992 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.

*****************************************************************************/

#include <Go/TextSelPtn.h>
#include <Go/Textual.h>
#include <Go/Tools/Bind.h>
#include <Go/Tools/Cursor.h>
#include <Go/GoRoot.h>
#include <stdio.h>

/*
 * PROGRAM
 *********
 */

GTextSelectPortionPgm::GTextSelectPortionPgm (const char* name)
    : GGrabberPgm (name)
{
    // this is like a resize with gravity G_MOVE_EAST;
    // set the cursor;

    cursorIdent(GCURSOR_RESIZE_EAST);
}

GTextSelectPortionPgm* GTextSelectPortionPgm::create ()
{
    void* value;
    const char* type;
    char  TextSelectPortionName[128];

    sprintf (TextSelectPortionName, "%s.selectPortion",
	     GoTextual::metaclass()->classname());
    
    if (GBinded (TextSelectPortionName, value, type))
    {
	if (type != GInteractorPgm::type())
	{
	    GWarning ("GTextSelectPortionPgm::create: the name \"%s\" is already used as a \"%s\" type.",
		      TextSelectPortionName, (type == NULL) ? "(NULL)" : type);
	    return NULL;
	}

	return (GTextSelectPortionPgm*) value;
    }

    return new GTextSelectPortionPgm (TextSelectPortionName);
}


GInteractor* GTextSelectPortionPgm::start (const GEvent* event, Go* go)
{
    if (go->getMetaclass()->isSubclass(GoTextual::metaclass()))
    {
	GInteractor* itor = new GTextSelectPortion (this, event,
						    (GoTextual*)go);
	Initialize (itor);
	return itor;
    }
    return NULL;
}

GTextSelectPortion::GTextSelectPortion (GTextSelectPortionPgm* pgm,
				  const GEvent* event, GoTextual* text)
    : GGrabber(pgm, event, text)
{
    _text_seg = new GTextSeg;
    _text_seg->first(-1);
    _text_seg->last(-1);

    text->selectionList().insertSeg(_text_seg);
}



void GTextSelectPortion::Initialize()
{

    // _pos is initialized;
    // compute first and last position;
    
    GCoordSys cs = go()->coordsys();
    _first_pos = _last_pos =
	((GoTextual*)go())->xyToPos(cs.from(_pos));

    // and init the text segement;
    _text_seg->first(_first_pos);
    _text_seg->last(_last_pos);

  GGrabber::Initialize();  // superclass last;
}

GTextSelectPortion::~GTextSelectPortion ()
{
    if (_text_seg)
	delete _text_seg;
}


void GTextSelectPortion::doit()
{
    // call superclass first;
  GGrabber::doit();

    _delete_tmp_portion();

    if (_first_pos != _last_pos)
    {
	// select the effective portion;
	((GoTextual*)go())->selectPortion(_first_pos, _last_pos, GTRUE);
    }
}

void GTextSelectPortion::cancel()
{
    // call superclass first;
  GGrabber::cancel();

    _delete_tmp_portion();

    if (_first_pos != _last_pos)
    {
	// unselect the effective portion;
	((GoTextual*)go())->selectPortion(_first_pos, _last_pos, GFALSE);
    }
}


void  GTextSelectPortion::compute(const GEvent* event)
{
  GGrabber::compute(event);

    GPos newpos;
    newpos = ((GoTextual*)go())->xyToPos(go()->coordsys().from(_pos));

    if (newpos == _last_pos)
	return;

    // reset the _text_seg
    if (newpos > _first_pos)
    {
	_text_seg->first(_first_pos);
	_text_seg->last(newpos);
    }
    else
    {
	_text_seg->first(newpos);
	_text_seg->last(_first_pos);
    }
}
/**
FUNCTION
    ::draw
DESCRIPTION
    use the selectPortion method of the Textual.
**/
void  GTextSelectPortion::draw()
{
    if (_last_pos != _text_seg->last())
    {
	GoTextual* text = (GoTextual*)go();
	GRegion region = text->boundariesToRegion
	    (_last_pos, _text_seg->last());

	if (text->backgroundMapped())
	    text->root()->redrawForeOnly (text, region);
	else
	    text->root()->redrawBackAndFore (region);

	// reset _first/_last_pos;
	_first_pos = _text_seg->first();
	_last_pos = _text_seg->last();
    }
}

void GTextSelectPortion::clear()
{
    // No clear;
}

void GTextSelectPortion::clearAll()
{
    // NO clearAll;
}



// PRIVATE;

void GTextSelectPortion::_delete_tmp_portion()
{
    GoTextual* text = (GoTextual*)go();

    // delete the last portion;
    _text_seg->first(-1);
    _text_seg->last(-1);

    text->selectionList().deletePortion(-2,0);

    delete _text_seg;
    _text_seg = NULL;
}


