#ifdef REV_INFO
static char rcsid[] = "$Id: Bezier.cc,v 1.2 1992/12/10 15:50:02 smadja Exp $";
#endif
/*****************************************************************************
MODULE:  Geometric Objects
FILE:    Bezier.cc
AUTHOR:  T.A Aboobacker
DATE:    Feb 1991

DESCRIPTION:
  See Go/Tools/Bezier.h header

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
*****************************************************************************/
//#include <Go/Tools/GeomUtil.h>
#include <Go/GoInterf.h>
#include <Go/GoStream.h>
#include <Go/Bezier.h>


static GNumber binomCoeff(int n, int i) {
    static int BCTable [] = {
	1, 
	1,   1, 
	1,   2,   1,
	1,   3,   3,   1,
	1,   4,   6,   4,   1,
	1,   5,  10,  10,   5,   1,
	1,   6,  15,  20,  15,   6,   1,
	1,   7,  21,  35,  35,  21,   7,   1,
	1,   8,  28,  56,  70,  56,  28,   8,   1,
	1,   9,  36,  84, 126, 126,  84,  36,   9,   1 };

    if (i > n)
	return 0;
    else if (n < 10)
	return(BCTable[n * (n + 1) / 2 + i]);
    else {
	GNumber bc = 1.0;
	int j;
	for (j = n - i + 1; j <= n; j++)
	    bc = bc * (GNumber) j;
	for (j = 1; j <= i; j++)
	    bc =  bc / (GNumber) j;
	return bc;
    }
}


/*
 *------------
 * CLASS PART
 *------------
 */

static GoBezierMetaclass* bezier_metaclass = NULL;

/**
FUNCTION
    GoBezierMetaclass* GoBezier::metaclass()
DESCRIPTION
    
**/
GoBezierMetaclass* GoBezier::metaclass() {
    if (bezier_metaclass == NULL) {
	bezier_metaclass = new GoBezierMetaclass;
	bezier_metaclass->superclass()->AddSubclass(bezier_metaclass);

        GoStream::bind(GoBezier::metaclass()->classname(),
		       (int (*) (const void*, GoStream*)) GoBezier::save,
		       (int (*) (void**, GoStream*)) GoBezier::restore);
    }
    return bezier_metaclass;
}


/**
FUNCTION
    ~GoBezierMetaclass()
DESCRIPTION
    
**/
GoBezierMetaclass::~GoBezierMetaclass() {
    if (this == bezier_metaclass) {
        superclass()->RemoveSubclass(this);
	bezier_metaclass = NULL;
    }
}


/**
FUNCTION
    const char* GoBezierMetaclass::m_classname() const
DESCRIPTION
    virtual private
**/
const char* GoBezierMetaclass::m_classname() const {
    return "GoBezier";
}


/**
FUNCTION
    GoMetaclass* GoBezierMetaclass::m_superclass() const
DESCRIPTION
    virtual private
**/
GoMetaclass* GoBezierMetaclass::m_superclass() const {
    return GoCurve::metaclass();
}


/**
FUNCTION
    Go* GoBezierMetaclass::m_create();
DESCRIPTION
    virtual private
**/
Go* GoBezierMetaclass::m_create() const {
    return (Go*) (new GoBezier);
}


/*
 *---------------------
 * PROTECTED FUNCTIONS
 *---------------------
 */


/**
FUNCTION
    int GoBezier::save(const GoBezier* bezier, GoStream* gostream);
DESCRIPTION
    
**/
int GoBezier::save(const GoBezier* bezier, GoStream* gostream) {
    return GoCurve::save(bezier, gostream);
}


/**
FUNCTION
    int GoBezier::restore(GoBezier** bezier, GoStream* gostream); 
DESCRIPTION
    
**/
int GoBezier::restore(GoBezier** bezier, GoStream* gostream) {
    // Might be optimized;;
    if (*bezier = GoBezier::create())
	return GoPolypoint::readParams(*bezier, gostream);
    return 0;
}

/*
 *-------------------
 * PRIVATE FUNCTIONS
 *-------------------
 */

/**
FUNCTION
    GoMetaclass* GoBezier::m_metaclass() const
DESCRIPTION
    virtual private
**/
GoMetaclass* GoBezier::m_metaclass() const {
    return GoBezier::metaclass();
}


/**
FUNCTION
    void GoBezier::m_check() const
DESCRIPTION
    virtual private
**/
void GoBezier::m_check() const {}


/**
FUNCTION
    GoBezier::m_draw(const GRegion&) const
DESCRIPTION
    virtual private    
**/
void GoBezier::m_draw(const GRegion&) const {
    const GShPen   *shpen   = outline()    ? sh_pen()          : NULL;
    const GShBrush *shbrush = background() ? brush().shBrush() : NULL;

    if (nbPoint() < 2 || ! (shbrush || shpen))
	return;
    
    interf()->bezier(shpen, shbrush, Abscs(),nbPoint(),points(), GMAX_QUALITY);
}


/**
FUNCTION
    GPoint* GoBezier::getCurvePoints(int& np) const;
DESCRIPTION
    
**/
GPoint* GoBezier::getCurvePoints(int& np, int nb_ctrl,
				 const GPoint* ctrl) const
{
    int k = (np > 0) ? np - 1 : 40;
    GPoint *drawPoints = new GPoint[k + 1];
    GNumber uinc = ((GNumber) 1.0) / k;
    GNumber u = 0;
    int i, j;
    for (i = 0 ; i <= k; ++i, u += uinc) {
	drawPoints[i].x = 0;
	drawPoints[i].y = 0;
	for (j = 0; j < nb_ctrl; j++)
	    drawPoints[i] += (ctrl[j] * blend(j, u, nb_ctrl));
    }
    np = k + 1;
    return drawPoints;
}


/**
FUNCTION
    GNumber GoBezier::blend(int i, GNumber u) const
DESCRIPTION
    
**/
GNumber GoBezier::blend(int i, GNumber u, int nb_ctrl) const
{
/*
 *   REMEMBER erratic results on operations with double numbers
 *   on certain machines (Olivetti!!)
 */
    int j, n = nb_ctrl - 1;
    double b = binomCoeff(n, i);
    for (j = 0; j < i; j++)
	b = b * u;

    for (j = 0; j < (n - i); j++)
	b = b * (1 - u);

    if (b <= (GNumber) 1e-30)
	return .0;
    if (b >= (GNumber) 1e+30)
	return 1.0;
    return (GNumber) b;
}


/*
 *------------------
 * PUBLIC FUNCTIONS
 *------------------
 */

/**
FUNCTION
    GoBezier* GoBezier::create(int nb, const GPoint *points);
DESCRIPTION
    The other legitimate GoBezier constructor : calls pre/postcond.
**/
GoBezier* GoBezier::create(int nb, const GPoint *points) {
    GoBezier *bezier = (GoBezier*) Go::Create(GoBezier::metaclass());
    if (bezier)
	bezier->reset(nb, points);
    return bezier;
}

