#ifdef REV_INFO
static char rcsid[] = "$Id: Cardinal.cc,v 1.1.1.1 1992/03/18 10:14:37 uep Exp $";
#endif
/*****************************************************************************
MODULE:  Geometric
FILE:    Cardinal.cc
AUTHOR:  Gilles Benati
DATE:    September 20th, 1991

DESCRIPTION:

  See the Go/Cardinal.h header.
    

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#include <assert.h>
#include <Go/GoStream.h>
#include <Go/Cardinal.h>


/*
 *------------
 * CLASS PART
 *------------
 */

static GoCardinalMetaclass* cardinal_metaclass = NULL;

/**
FUNCTION
    GoCardinalMetaclass* GoCardinal::metaclass()
DESCRIPTION
    
**/
GoCardinalMetaclass* GoCardinal::metaclass() {
    if (cardinal_metaclass == NULL) {
	cardinal_metaclass = new GoCardinalMetaclass;
	cardinal_metaclass->superclass()->AddSubclass(cardinal_metaclass);

        GoStream::bind(GoCardinal::metaclass()->classname(),
		       (int (*) (const void*, GoStream*)) GoCardinal::save,
		       (int (*) (void**, GoStream*)) GoCardinal::restore);
    }
    return cardinal_metaclass;
}


/**
FUNCTION
    ~GoCardinalMetaclass()
DESCRIPTION
    
**/
GoCardinalMetaclass::~GoCardinalMetaclass() {
    if (this == cardinal_metaclass) {
        superclass()->RemoveSubclass(this);
	cardinal_metaclass = NULL;
    }
}


/**
FUNCTION
    const char* GoCardinalMetaclass::m_classname() const
DESCRIPTION
    virtual private
**/
const char* GoCardinalMetaclass::m_classname() const {
    return "GoCardinal";
}


/**
FUNCTION
    GoMetaclass* GoCardinalMetaclass::m_superclass() const
DESCRIPTION
    virtual private
**/
GoMetaclass* GoCardinalMetaclass::m_superclass() const {
    return GoCubic::metaclass();
}


/**
FUNCTION
    Go* GoCardinalMetaclass::m_create();
DESCRIPTION
    virtual private
**/
Go* GoCardinalMetaclass::m_create() const {
    return (Go*) (new GoCardinal);
}


/*
 *---------------------
 * PROTECTED FUNCTIONS
 *---------------------
 */

/**
FUNCTION
    int GoCardinal::save(const GoCardinal* cardinal, GoStream* gostream);
DESCRIPTION
    **static member**
**/
int GoCardinal::save(const GoCardinal* cardinal, GoStream* gostream) {
    return GoCubic::save(cardinal, gostream);
}


/**
FUNCTION
    int GoCardinal::restore(GoCardinal** cardinal, GoStream* gostream); 
DESCRIPTION
    **static member**
**/
int GoCardinal::restore(GoCardinal** cardinal, GoStream* gostream) {
    // Might be optimized;;
    if (*cardinal = GoCardinal::create())
	return GoCubic::readParams(*cardinal, gostream);
    return 0;
}


/**
FUNCTION
    void GoCardinal::initCoeffMatrix();
DESCRIPTION
    virtual protected
**/
void GoCardinal::initCoeffMatrix() {
    _coeffMatrix[0][0] = -_tension;
    _coeffMatrix[0][1] = 2 - _tension;
    _coeffMatrix[0][2] = _tension - 2; ;
    _coeffMatrix[0][3] = _tension;
    _coeffMatrix[1][0] = 2 * _tension;
    _coeffMatrix[1][1] = _tension - 3;
    _coeffMatrix[1][2] = 3 - 2 * _tension;
    _coeffMatrix[1][3] = -_tension;
    _coeffMatrix[2][0] = -_tension;
    _coeffMatrix[2][1] = 0;
    _coeffMatrix[2][2] = _tension;
    _coeffMatrix[2][3] = 0;
    _coeffMatrix[3][0] = 0;
    _coeffMatrix[3][1] = 1;
    _coeffMatrix[3][2] = 0;
    _coeffMatrix[3][3] = 0;
    _inited = GTRUE;
}


/*
 *-------------------
 * PRIVATE FUNCTIONS
 *-------------------
 */

/**
FUNCTION
    GoMetaclass* GoCardinal::m_metaclass() const
DESCRIPTION
    virtual private
**/
GoMetaclass* GoCardinal::m_metaclass() const {
    return GoCardinal::metaclass();
}


/**
FUNCTION
    void GoCardinal::m_check() const
DESCRIPTION
    virtual private
**/
void GoCardinal::m_check() const {}


/*
 *------------------
 * PUBLIC FUNCTIONS
 *------------------
 */

/**
FUNCTION
    GoCardinal(int nb = 0, const GPoint* pts = NULL)
DESCRIPTION
    GoCardinal constructor. It creates a cardinal spline of tension 1.
**/
GoCardinal::GoCardinal(int nb, const GPoint* pts) : GoCubic(nb, pts) {
    _tension = 1;
}


/**
FUNCTION
    GoCardinal* GoCardinal::create(int nb, const GPoint *pts = NULL);
DESCRIPTION
    The other legitimate GoCardinal constructor : calls pre/postcond.
**/
GoCardinal* GoCardinal::create(int nb, const GPoint *pts) {
    GoCardinal *cardinal = (GoCardinal*) Go::Create(GoCardinal::metaclass());
    if (cardinal)
	cardinal->reset(nb, pts);
    return cardinal;
}
