#ifdef REV_INFO
static char rcsid[] = "$Id: Circle.cc,v 1.2 1992/12/10 15:52:20 smadja Exp $";
#endif
/*****************************************************************************
MODULE:  Graphic Objects
FILE:    Circle.cc
AUTHOR:  Gilles Benati
DATE:    Feb 1991

DESCRIPTION:
    See the Go/Tools/Circle.h header.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
*****************************************************************************/

#include <Go/GoInterf.h>
#include <Go/GoStream.h>
#include <Go/Circle.h>


/*
 *------------
 * CLASS PART
 *------------
 */

static GoCircleMetaclass* circle_metaclass = NULL;

/**
FUNCTION
    GoCircleMetaclass* GoCircle::metaclass()
DESCRIPTION
    
**/
GoCircleMetaclass* GoCircle::metaclass() {
    if (circle_metaclass == NULL) {
	circle_metaclass = new GoCircleMetaclass;
	circle_metaclass->superclass()->AddSubclass(circle_metaclass);

        GoStream::bind(GoCircle::metaclass()->classname(),
		       (int (*) (const void*, GoStream*)) GoCircle::save,
		       (int (*) (void**, GoStream*)) GoCircle::restore);
    }
    return circle_metaclass;
}


/**
FUNCTION
    ~GoCircleMetaclass()
DESCRIPTION
    
**/
GoCircleMetaclass::~GoCircleMetaclass() {
    if (this == circle_metaclass) {
        superclass()->RemoveSubclass(this);
	circle_metaclass = NULL;
    }
}


/**
FUNCTION
    const char* GoCircleMetaclass::m_classname() const
DESCRIPTION
    virtual private
**/
const char* GoCircleMetaclass::m_classname() const {
    return "GoCircle";
}


/**
FUNCTION
    GoMetaclass* GoCircleMetaclass::m_superclass() const
DESCRIPTION
    virtual private
**/
GoMetaclass* GoCircleMetaclass::m_superclass() const {
    return GoEllipse::metaclass();
}


/**
FUNCTION
    Go* GoCircleMetaclass::m_create() const;
DESCRIPTION
    virtual private
**/
Go* GoCircleMetaclass::m_create() const {
    return (Go*) (new GoCircle);
}


/*
 *----------------
 * PRIVATE MEMBERS
 *----------------
 */


/**
FUNCTION
    GoMetaclass* GoCircle::m_metaclass() const
DESCRIPTION
    virtual private
**/
GoMetaclass* GoCircle::m_metaclass() const {
    return GoCircle::metaclass();
}


/**
FUNCTION
    GoCircle::m_check() const
DESCRIPTION
    virtual private
**/
void GoCircle::m_check() const {
}


/*
 *------------------
 * PROTECTED MEMBERS
 *------------------
 */

/**
FUNCTION
    int GoCircle::save(const GoCircle* circle, GoStream* gostream);
DESCRIPTION
    **static member**, protected
**/
int GoCircle::save(const GoCircle* circle, GoStream* gostream) {
    return (gostream->writeNumber(circle->GoArc::radiusX(), "radius") &&
	    GoGeometric::save(circle, gostream));
}

/**
FUNCTION
    int GoCircle::restore(GoCircle** circle, GoStream* gostream);
DESCRIPTION
    **static member**, protected
**/
int GoCircle::restore(GoCircle** circle, GoStream* gostream) {
    GNumber radius;
    if (! gostream->readNumber(&radius))
	return 0;
    if (*circle = GoCircle::create(radius))
	return GoGeometric::readParams(*circle, gostream);
    return 0;
}

/**
FUNCTION
    int GoCircle::readParams(GoCircle* circle, GoStream* gostream);
DESCRIPTION
    **static member**, protected
**/
int GoCircle::readParams(GoCircle* circle, GoStream* gostream) {
    GNumber radius;
    if (! gostream->readNumber(&radius))
	return 0;
    if (radius < 0)
	radius = 1.;
    circle->_reset(radius, radius, 2*PI, 0.);
    return GoGeometric::readParams(circle, gostream);
}


/**
FUNCTION
    void GoCircle::m_draw(const GRegion&) const
DESCRIPTION
    virtual protected
**/
void GoCircle::m_draw(const GRegion&) const {
    const GShPen   *shpen   = outline()    ? sh_pen()          : NULL;
    const GShBrush *shbrush = background() ? brush().shBrush() : NULL;
    
    if (! (shbrush || shpen))
	return;

    interf()->circle(shpen, shbrush, Abscs(),
		     GPoint(0, 0), GoArc::radiusX(), 0, 2*PI, GMAX_QUALITY);
}


/**
FUNCTION
    void GoCircle::m_wired_draw(const GCoordSys& cs, const GPen& pem, Quality qulaity) const
DESCRIPTION
    virtual protected
**/
void GoCircle::m_wired_draw(const GCoordSys& cs, const GPen& pen,
			    GQuality quality) const
{
    interf()->circle(&pen, NULL, cs, GPoint(0, 0), GoArc::radiusX(),
		     0, 2*PI, quality);
}


/**
FUNCTION
    GoCircle::m_add_clip(GRegion& region) const
DESCRIPTION
    virtual protected
**/
void GoCircle::m_add_clip(GRegion& region) const {
    if (! (outline() || background()))
	return;

    interf()->circleClips(outline()    ? sh_pen()          : NULL,
			  background() ? brush().shBrush() : NULL,
			  Abscs(), GPoint(0, 0),
			  GoArc::radiusX(), &region, NULL, 0, 2*PI);
}


/**
FUNCTION
    GoCircle::m_add_opaque_clip(GRegion& region) const
DESCRIPTION
    virtual protected
**/
void GoCircle::m_add_opaque_clip(GRegion& region) const {
    if (! (outline() || background()))
	return;

    interf()->circleClips(outline()    ? sh_pen()          : NULL,
			  background() ? brush().shBrush() : NULL,
			  Abscs(), GPoint(0, 0),
			  GoArc::radiusX(), NULL, &region, 0, 2*PI);
}

/**
FUNCTION
    GBool GoCircle::m_contains(const GPoint& absPoint, GNumber maxDist) const
DESCRIPTION
    virtual protected
**/
GBool GoCircle::m_contains(const GPoint& absPoint, GNumber maxDist) const {
    const GShPen   *shpen   = outline()    ? sh_pen()          : NULL;
    const GShBrush *shbrush = background() ? brush().shBrush() : NULL;
    
    if (! (shbrush || shpen))
	return GFALSE;

    return interf()->circleContains(shpen, shbrush, Abscs(), GPoint(0, 0),
				  GoArc::radiusX(), absPoint, maxDist, 0, 2*PI);
}

/*
 *---------------
 * PUBLIC MEMBERS
 *---------------
 */

/**
FUNCTION
    GoCircle* GoCircle::create(GNumber radius)
DESCRIPTION
    Another legitimate GoCircle constructor (calls pre/postconditions)
**/
GoCircle* GoCircle::create(GNumber radius) {
    GoCircle *circle = (GoCircle*) Go::Create(GoCircle::metaclass());
    if (circle) {
	if (radius < 0)
	    radius = 1.;
	circle->_reset(radius, radius, 2*PI, 0.);
    }
    return circle;
}

/**
FUNCTION
    GBool GoCircle::radiusX(GNumber radius)
DESCRIPTION
    virtual public
**/
GBool GoCircle::radiusX(GNumber radius) {
    return Reset(radius, radius, 2*PI, 0);
}


/**
FUNCTION
    GBool GoCircle::radiusY(GNumber radius)
DESCRIPTION
    virtual public
**/
GBool GoCircle::radiusY(GNumber radius) {
    return Reset(radius, radius, 2*PI, 0);
}


/**
FUNCTION
    GBool GoCircle::reset(GNumber radius, GNumber = 0., GNumber = 0., GNumber = 0.)
DESCRIPTION
    virtual public
**/
GBool GoCircle::reset(GNumber radius, GNumber, GNumber, GNumber) {
    return Reset(radius, radius, 2*PI, 0);
}



