#ifdef REV_INFO
static char rcsid[] = "$Id: Ellipse.cc,v 1.2 1992/12/10 15:56:56 smadja Exp $";
#endif
/*****************************************************************************
MODULE:  Graphic Objects
FILE:    Ellipse.cc
AUTHOR:  Gilles Benati
DATE:    Feb 1991

DESCRIPTION:
    See the Go/Tools/Ellipse.h header.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
*****************************************************************************/

#include <Go/GoInterf.h>
#include <Go/GoStream.h>
#include <Go/Ellipse.h>


/*
 *------------
 * CLASS PART
 *------------
 */

static GoEllipseMetaclass* ellipse_metaclass = NULL;

/**
FUNCTION
    GoEllipseMetaclass* GoEllipse::metaclass()
DESCRIPTION
    
**/
GoEllipseMetaclass* GoEllipse::metaclass() {
    if (ellipse_metaclass == NULL) {
	ellipse_metaclass = new GoEllipseMetaclass;
	ellipse_metaclass->superclass()->AddSubclass(ellipse_metaclass);

        GoStream::bind(GoEllipse::metaclass()->classname(),
		       (int (*) (const void*, GoStream*)) GoEllipse::save,
		       (int (*) (void**, GoStream*)) GoEllipse::restore);
    }
    return ellipse_metaclass;
}


/**
FUNCTION
    ~GoEllipseMetaclass()
DESCRIPTION
    
**/
GoEllipseMetaclass::~GoEllipseMetaclass() {
    if (this == ellipse_metaclass) {
        superclass()->RemoveSubclass(this);
	ellipse_metaclass = NULL;
    }
}


/**
FUNCTION
    const char* GoEllipseMetaclass::m_classname() const
DESCRIPTION
    virtual private
**/
const char* GoEllipseMetaclass::m_classname() const {
    return "GoEllipse";
}


/**
FUNCTION
    GoMetaclass* GoEllipseMetaclass::m_superclass() const
DESCRIPTION
    virtual private
**/
GoMetaclass* GoEllipseMetaclass::m_superclass() const {
    return GoArc::metaclass();
}


/**
FUNCTION
    Go* GoEllipseMetaclass::m_create();
DESCRIPTION
    virtual private
**/
Go* GoEllipseMetaclass::m_create() const {
    return (Go*) (new GoEllipse);
}

/*
 *----------------
 * PRIVATE MEMBERS
 *----------------
 */

/**
FUNCTION
    GoMetaclass* GoEllipse::m_metaclass() const;
DESCRIPTION
    virtual private
**/
GoMetaclass* GoEllipse::m_metaclass() const {
    return GoEllipse::metaclass();
}


/**
FUNCTION
    GoEllipse::m_check() const;
DESCRIPTION
    virtual private
**/
void GoEllipse::m_check() const {
}


/*
 *------------------
 * PROTECTED MEMBERS
 *------------------
 */


/**
FUNCTION
    int GoEllipse::save(const GoEllipse* ell, GoStream* gostream);
DESCRIPTION
    **static member**, protected
**/
int GoEllipse::save(const GoEllipse* ell, GoStream* gostream) {
    return (gostream->writeNumber(ell->radiusX(), "radiusX") &&
	    gostream->writeNumber(ell->radiusY(), "radiusY") &&
	    GoGeometric::save(ell, gostream));
}

/**
FUNCTION
    int GoEllipse::restore(GoEllipse** ell, GoStream* gostream);
DESCRIPTION
    **static member**, protected
**/
int GoEllipse::restore(GoEllipse** ell, GoStream* gostream) {
    GNumber radx, rady;
    if (! (gostream->readNumber(&radx) && gostream->readNumber(&rady)))
	return 0;
    if (*ell = GoEllipse::create(radx, rady))
	return GoGeometric::readParams(*ell, gostream);
    return 0;
}

/**
FUNCTION
    int GoEllipse::readParams(GoEllipse* ell, GoStream* gostream);
DESCRIPTION
    **static member**, protected
**/
int GoEllipse::readParams(GoEllipse* ell, GoStream* gostream) {
    GNumber radx, rady;
    if (! (gostream->readNumber(&radx) && gostream->readNumber(&rady)))
	return 0;
    if (radx < 0.)
	radx = 1.;
    if (rady < 0.)
	rady = 1.;
    ell->_reset(radx, rady, 2*PI, 0.);
    return GoGeometric::readParams(ell, gostream);
}

/**
FUNCTION
    void GoEllipse::m_draw(const GRegion&) const
DESCRIPTION
    virtual protected
**/
void GoEllipse::m_draw(const GRegion&) const {
    const GShPen   *shpen   = outline()    ? sh_pen()          : NULL;
    const GShBrush *shbrush = background() ? brush().shBrush() : NULL;
    
    if (! (shbrush || shpen))
	return;
    
    interf()->ellipse(shpen, shbrush, Abscs(), GPoint(0, 0), radiusX(), radiusY());
}

/**
FUNCTION
    void GoEllipse::m_wired_draw(const GCoordSys& cs, const GPen&, Quality) const
DESCRIPTION
    virtual protected
**/
void GoEllipse::m_wired_draw(const GCoordSys& cs, const GPen& pen,
			     GQuality quality) const
{
    interf()->ellipse(&pen, NULL, cs, GPoint(0., 0.),
		      radiusX(), radiusY(),
		      0., 2*PI, quality);
}

/**
FUNCTION
    GoEllipse::m_add_clip(GRegion& region) const
DESCRIPTION
    virtual protected
**/
void GoEllipse::m_add_clip(GRegion& region) const {
    if (! (outline() || background()))
	return;

    interf()->ellipseClips(outline()    ? sh_pen()          : NULL,
			   background() ? brush().shBrush() : NULL,
			   Abscs(), GPoint(0, 0),
			   radiusX(), radiusY(), &region, NULL, 0, 2*PI);
}


/**
FUNCTION
    GoEllipse::m_add_opaque_clip(GRegion& region) const
DESCRIPTION
    virtual protected
**/
void GoEllipse::m_add_opaque_clip(GRegion& region) const {
    if (! (outline() || background()))
	return;

    interf()->ellipseClips(outline()    ? sh_pen()          : NULL,
			   background() ? brush().shBrush() : NULL,
			   Abscs(), GPoint(0, 0),
			   radiusX(), radiusY(), NULL, &region, 0, 2*PI);
}

/**
FUNCTION
    GBool GoEllipse::m_contains(const GPoint& absPoint, GNumber maxDist) const
DESCRIPTION
    virtual protected
**/
GBool GoEllipse::m_contains(const GPoint& absPoint, GNumber maxDist) const {
    const GShPen   *shpen   = outline()    ? sh_pen()          : NULL;
    const GShBrush *shbrush = background() ? brush().shBrush() : NULL;
    
    if (! (shbrush || shpen))
	return GFALSE;

    return interf()->ellipseContains(shpen, shbrush, Abscs(), GPoint(0, 0),
				     radiusX(), radiusY(), absPoint, maxDist,
				     0, 2*PI);
}

/*
 *---------------
 * PUBLIC MEMBERS
 *---------------
 */


/**
FUNCTION
    GoEllipse* GoEllipse::create(GNumber radiusX, GNumber radiusY = 1.)
DESCRIPTION
    Another legitimate GoEllipse constructor (calls pre/postconditions)
**/
GoEllipse* GoEllipse::create(GNumber radiusX, GNumber radiusY) {
    GoEllipse *ellipse = (GoEllipse*) Go::Create(GoEllipse::metaclass());

    if (ellipse)
	ellipse->_reset((radiusX < 0.) ? 1. : radiusX,
			(radiusY < 0.) ? 1. : radiusY, 2*PI, 0.);
    return ellipse;
}


/**
FUNCTION
    GBool GoEllipse::reset(GNumber radX, GNumber radY, GNumber = 0., GNumber = 0.)
DESCRIPTION
    virtual public 
**/
GBool GoEllipse::reset(GNumber radX, GNumber radY, GNumber, GNumber) {
    return Reset(radX, radY, 2*PI, 0.);
}


/**
FUNCTIONS
    GBool GoEllipse::amplitude(GNumber)
    GBool GoEllipse::start(GNumber)
    GBool GoEllipse::pie(GBool)
    GBool GoEllipse::closed(GBool)
DESCRIPTION
    virtual private : just disable superclass methods;
**/
GBool GoEllipse::amplitude(GNumber)	{ return GFALSE; }
GBool GoEllipse::start(GNumber)		{ return GFALSE; }
GBool GoEllipse::pie(GBool)		{ return GFALSE; }
GBool GoEllipse::closed(GBool)		{ return GFALSE; }


