#ifdef REV_INFO
static char rcsid[] = "$Id: Polygon.cc,v 1.2 1992/12/10 16:01:13 smadja Exp $";
#endif
/*****************************************************************************
MODULE: Geometric
FILE:   Polygon.cc
AUTHOR: Gilles Benati
DATE:   September 20th, 1991

DESCRIPTION :
    See the Go/Tools/Polygon.h header file
    
COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
*****************************************************************************/

#include <assert.h>
#include <Go/GoInterf.h>
#include <Go/GoStream.h>
#include <Go/Polygon.h>



/*
 *------------
 * CLASS PART
 *------------
 */

static GoPolygonMetaclass* polygon_metaclass = NULL;

/**
FUNCTION
    GoPolygonMetaclass* GoPolygon::metaclass()
DESCRIPTION
    **static member**
**/
GoPolygonMetaclass* GoPolygon::metaclass() {
    if (polygon_metaclass == NULL) {
	polygon_metaclass = new GoPolygonMetaclass;
	polygon_metaclass->superclass()->AddSubclass(polygon_metaclass);

        GoStream::bind(GoPolygon::metaclass()->classname(),
		       (int (*) (const void*, GoStream*)) GoPolygon::save,
		       (int (*) (void**, GoStream*)) GoPolygon::restore);
    }
    return polygon_metaclass;
}


/**
FUNCTION
    ~GoPolygonMetaclass()
DESCRIPTION
    
**/
GoPolygonMetaclass::~GoPolygonMetaclass() {
    if (this == polygon_metaclass) {
        superclass()->RemoveSubclass(this);
	polygon_metaclass = NULL;
    }
}


/**
FUNCTION
    const char* GoPolygonMetaclass::m_classname() const
DESCRIPTION
    virtual private
**/
const char* GoPolygonMetaclass::m_classname() const {
    return "GoPolygon";
}


/**
FUNCTION
    GoMetaclass* GoPolygonMetaclass::m_superclass() const
DESCRIPTION
    virtual private
**/
GoMetaclass* GoPolygonMetaclass::m_superclass() const {
    return GoPolyline::metaclass();
}

/**
FUNCTION
    Go* GoPolygonMetaclass::m_create();
DESCRIPTION
    virtual private
**/
Go* GoPolygonMetaclass::m_create() const {
    return (Go*) (new GoPolygon);
}



/*
 *-------------------
 * PRIVATE FUNCTIONS
 *-------------------
 */

/**
FUNCTION
    int GoPolygon::get_round_points(GPoint**) const
DESCRIPTION
    virtual private
**/
int GoPolygon::get_round_points(GPoint** roundPoints) const {
    GPoint center, *newPoints = new GPoint[22 * nbPoint()];
    GVector v1, v2;
    GNumber da = ((GNumber) PI) / 40, a = da;
    int i, j, k = 0;
    
    DeltaVect(points()[nbPoint() - 1], points()[0], points()[1], v1, v2);
    newPoints[k++] = points()[0] - v1;

    center = points()[0] - v1 - v2;
    for (j = 0; j < 19; ++j, a += da)
	newPoints[k++] = center + v1 * sin(a) + v2 * cos(a);

    newPoints[k++] = points()[0] - v2;

    for (i = 1; i < nbPoint() - 1; i++) {
	DeltaVect(points()[i - 1], points()[i], points()[i + 1], v1, v2);
	newPoints[k++] = points()[i] - v1;

	center = points()[i] - v1 - v2;
	for (j = 0, a = da; j < 19; ++j, a += da)
	    newPoints[k++] = center + v1 * sin(a) + v2 * cos(a);

	newPoints[k++] = points()[i] - v2;
    }

    DeltaVect(points()[i - 1], points()[i], points()[0], v1, v2);
    newPoints[k++] = points()[i] - v1;

    center = points()[i] - v1 - v2;
    for (j = 0, a = da; j < 19; ++j, a += da)
	newPoints[k++] = center + v1 * sin(a) + v2 * cos(a);

    newPoints[k++] = points()[i] - v2;
    
    *roundPoints = newPoints;
    return k;
}


/**
FUNCTION
    GoMetaclass* GoPolygon::m_metaclass() const
DESCRIPTION
    virtual private
**/
GoMetaclass* GoPolygon::m_metaclass() const {
    return GoPolygon::metaclass();
}


/**
FUNCTION
    void GoPolygon::m_check() const
DESCRIPTION
    virtual private
**/
void GoPolygon::m_check() const {
}



/*
 *-------------------
 * PROTECTED FUNCTIONS
 *-------------------
 */

/**
FUNCTION
    int GoPolygon::save(const GoPolygon* polg, GoStream* gostream);
DESCRIPTION
    **static member**
**/
int GoPolygon::save(const GoPolygon* polg, GoStream* gostream) {
    return GoPolyline::save(polg, gostream);
}


/**
FUNCTION
    int GoPolygon::restore(GoPolygon** polg, GoStream* gostream)
DESCRIPTION
    
**/
int GoPolygon::restore(GoPolygon** polg, GoStream* gostream) {
    // MIGHT BE OPTIMIZE !!!!!!!;
    if (*polg = GoPolygon::create())
	return GoPolyline::readParams(*polg, gostream);
    return 0;
}


/**
FUNCTION
    void GoPolygon::m_draw(const GRegion&) const
DESCRIPTION
    virtual protected
**/
void GoPolygon::m_draw(const GRegion&) const {
    const GShPen   *shpen   = outline()    ? sh_pen()          : NULL;
    const GShBrush *shbrush = background() ? brush().shBrush() : NULL;
	
    if (nbPoint() < 2 || ! (shbrush || shpen))
	return;

    GPoint *pts;
    int nb;

    if (round() && nbPoint() > 2)
	nb = get_round_points(&pts);
    else {
	nb = nbPoint();
	pts = (GPoint*) points();
    }
    
    if (nb == 2)
	interf()->polyline(shpen, NULL, Abscs(), nb, pts, GMAX_QUALITY);
    else
	interf()->polygon(shpen, shbrush, Abscs(), nb, pts, GMAX_QUALITY);

    if (pts != points())
	delete(pts);
}

/**
FUNCTION
    void GoPolygon::m_wired_draw(const GCoordSys& cs, const GPen&, GQuality) const
DESCRIPTION
    virtual protected
**/
void GoPolygon::m_wired_draw(const GCoordSys& cs, const GPen& pen,
			     GQuality quality) const
{
    if (nbPoint() <= 2)
	return;
    interf()->polygon(&pen, NULL, cs, nbPoint(), points(), quality);
}


/**
FUNCTION
    GBool GoPolygon::m_contains(const GPoint& absPoint, GNumber maxDist = 0) const
DESCRIPTION
    virtual protected
**/
GBool GoPolygon::m_contains(const GPoint& absPoint, GNumber maxDist) const {
    const GShPen   *shpen   = outline()    ? sh_pen()           : NULL;
    const GShBrush *shbrush = background() ? brush().shBrush()  : NULL;

    if (nbPoint() <= 0 || ! (shbrush || shpen))
	return GFALSE;
	
    GPoint *pts;
    int nb;

    if (round() && nbPoint() > 2)
	nb = get_round_points(&pts);
    else {
	nb = nbPoint();
	pts = (GPoint*) points();
    }
    GBool answer = interf()->polygonContains(shpen, shbrush, Abscs(), nb, pts,
					     absPoint, maxDist);
    if (pts != points())
	delete(pts);

    return answer;
}


/**
FUNCTION
    GoPolygon::m_add_clip(GRegion& region) const
DESCRIPTION
    virtual protected
**/
void GoPolygon::m_add_clip(GRegion& region) const {
    if (nbPoint() <= 0 || ! (outline() || background()))
	return;
    
    const GShPen   *shpen   = outline()    ? sh_pen()          : NULL;
    const GShBrush *shbrush = background() ? brush().shBrush() : NULL;
    
    if (nbPoint() >= 3 && round()) {
	GPoint *pts;
	int nb = get_round_points(&pts);

	if (shpen &&
	    (shpen->lineWidth() != 0) && (shpen->joinStyle() == GJOIN_MITER)) {
	    GShPen* tmp_shpen = shpen->dup();
	    GSetJoinStyle(tmp_shpen, GJOIN_ROUND);
	    interf()->polygonClips(tmp_shpen, shbrush, Abscs(), nb, pts,
				   &region, NULL);
	    tmp_shpen->destroy();
	}
	else interf()->polygonClips(shpen, shbrush, Abscs(), nb, pts,
				    &region, NULL);
	delete pts;
    }
    else interf()->polygonClips(shpen, shbrush, Abscs(), nbPoint(), points(),
				&region, NULL);
}


/*
 *------------------
 * PUBLIC FUNCTIONS
 *------------------
 */

/**
FUNCTION
    GoPolygon* GoPolygon::create(int nb, const GPoint *points = NULL);
DESCRIPTION
    The other legitimate GoPolygon constructor : calls pre/postcond.
    **static member**
**/
GoPolygon* GoPolygon::create(int nb, const GPoint *points) {
    GoPolygon *polygon = (GoPolygon*) Go::Create(GoPolygon::metaclass());
    if (polygon)
	polygon->reset(nb, points);
    return polygon;
}

