#ifdef REV_INFO
static char rcsid[] = "$Id: BusLink.cc,v 1.5 1992/12/10 16:08:11 smadja Exp $";
#endif
/*****************************************************************************
MODULE:  BULL Graphic Objects
FILE:    GoBusLink.cc
AUTHOR:  Aline Jacomet
DATE:    January 23th 1991

DESCRIPTION:
         Definition of the bus link object class.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include <assert.h>
#include <Go/BusLink.h>
#include <Go/mBusLink.h>
#include <Go/GoStream.h>
#include <Go/GoInterf.h>


#define GO_GEOM_PRECISION ((GNumber) .25)

    
/*
 *------------
 * CLASS PART
 *------------
 */


static GoBusLinkMetaclass* busLink_metaclass = NULL;

 
/**
FUNCTION
    GoBusLink::metaclass()
DESCRIPTION
    
**/
GoBusLinkMetaclass* GoBusLink::metaclass() {
    if (busLink_metaclass == NULL) {
	busLink_metaclass = new GoBusLinkMetaclass;
	busLink_metaclass->superclass()->AddSubclass(busLink_metaclass);
      GoStream::bind(GoBusLink::metaclass()->classname(),
		     (int (*) (const void*, GoStream*)) GoBusLink::save,
		     (int (*) (void**, GoStream*)) GoBusLink::restore);
    }
    return busLink_metaclass;
}


/**
FUNCTION
    ~GoBusLinkMetaclass()
DESCRIPTION
    
**/
GoBusLinkMetaclass::~GoBusLinkMetaclass() {
    if (this == busLink_metaclass) {
        superclass()->RemoveSubclass(this);
	busLink_metaclass = NULL;
    }
}


/**
FUNCTION
    GoBusLinkMetaclass::GoBusLinkMetaclass()
DESCRIPTION
    
**/
 GoBusLinkMetaclass::GoBusLinkMetaclass() {
}

/**
FUNCTION
    const char* GoBusLinkMetaclass::m_classname() const
DESCRIPTION

**/
const char* GoBusLinkMetaclass::m_classname() const
{
    return "GoBusLink";
}


/**
FUNCTION
    GoMetaclass* GoBusLinkMetaclass::m_superclass() const
DESCRIPTION

**/
GoMetaclass* GoBusLinkMetaclass::m_superclass() const
{
    return GoLink::metaclass();
}


/**
FUNCTION
    Go* GoBusLinkMetaclass::m_create();
DESCRIPTION

**/
Go* GoBusLinkMetaclass::m_create() const
{
    return (Go*)(new GoBusLink);
}


/**
FUNCTION
    GoBusLink::m_dup()
DESCRIPTION

**/
void GoBusLink::m_dup(const Go* src, Go* par, Go* prv, GBool b, GBool dochildren)
{
    GoLink::m_dup(src, par, prv, b, dochildren);

    GoBusLink* l_src = (GoBusLink*) src;

    _orientation = l_src->_orientation;
    pen().modify(l_src->pen().shPen());
}



/**
FUNCTION
    m_metaclass()
DESCRIPTION
    Returns the metaclass.
**/
GoMetaclass* GoBusLink::m_metaclass () const
{
    return GoBusLink::metaclass();
}


static GActionRec empty_action_array[] = { { NULL, NULL } };

/**
FUNCTION
    m_actions()
DESCRIPTION
    
**/
const GActionRec* GoBusLinkMetaclass::m_actions() const
{
    return empty_action_array;
}


/*
 *--------------
 * PRIVATE DATA
 *--------------
 */



/**
FUNCTION
    void GoBusLink::m_draw() const
DESCRIPTION
    
**/
void GoBusLink::m_draw(const GRegion&) const {
    if (do_draw) {
	GPoint point[4];
	calculate_points(point);
	const GPen* p = &_pen;
	interf()->polyline((GPen*)p, NULL,  GUnitCoordSys(), 4, point);
    }
}


/**
FUNCTION
    m_add_clip()
DESCRIPTION
    virtual private
**/
void GoBusLink::m_add_clip(GRegion& region) const {
    if (do_draw) {
	GPoint point[4];
	calculate_points(point);
	interf()->polylineClips(sh_pen(), NULL, GUnitCoordSys(), 4, point,
				&region, NULL, GMAX_QUALITY);
    }
}


/**
FUNCTION
    m_contains()
DESCRIPTION
    
**/
GBool GoBusLink::m_contains(const GPoint& absPoint, GNumber maxDist) const
{
    if (do_draw) {
	if (maxDist == 0)
	    maxDist = GO_GEOM_PRECISION;
	GPoint point[4];
	calculate_points(point);
	const GPen* p = &_pen;
	return interf()->polylineContains((GPen*)p, NULL, GUnitCoordSys(), 4,
					  point, absPoint, maxDist,
					  GMAX_QUALITY);
    } else
	return GFALSE;
}



/**
FUNCTION
    modify_pen() {
DESCRIPTION
    This method is a pen::modifyPostcond.
    It redraws the link with the new pen.
**/
void buslink_modify_pen (void* closure, GPen* pen, ulong n)
{
    GoBusLink* go = (GoBusLink*)closure;
    go->modify_pen(pen, n);
}

void GoBusLink::modify_pen(GPen* pen, ulong) {
    assert(pen == &_pen);
    GBool is_mapped = mapped();
    if (is_mapped)
	RedrawBackAndFore();
    init_sh_pen();
    MyClipHasChanged();
    if (is_mapped)
	RedrawForeOnly();
}


/**
FUNCTION
    calculate_points()
DESCRIPTION
    Calculates the two points between next_point and prev_point.
    It is assumed that point is declared as GPoint point[4];
**/
void GoBusLink::calculate_points(GPoint* point) const
{
    short pos = ((GoGraphStruct*)parent())->positionType();
    point[0] = prev_point;
    point[3] = next_point;
    if (pos == GAPPLICATION_DEFINE) {
	if (_orientation == GHORIZONTAL || _orientation == GUNDEFINED)
	    pos = GLEFT_TO_RIGHT;
	else
	    pos = GUP_TO_BOTTOM;
    }
    if (pos == GLEFT_TO_RIGHT || pos == GRIGHT_TO_LEFT) {
	point[1].x = (prev_point.x + next_point.x) /2;
	point[1].y = prev_point.y;
	point[2].x = point[1].x;
	point[2].y = next_point.y;
    } else
	if (pos == GUP_TO_BOTTOM || pos == GBOTTOM_TO_UP) {
	    point[1].x = prev_point.x;
	    point[1].y = (prev_point.y + next_point.y) / 2;
	    point[2].x = next_point.x;
	    point[2].y = point[1].y;
	}
}



/*
 *--------------
 * PUBLIC DATA
 *--------------
 */


/**
FUNCTION
    GoBusLink()
DESCRIPTION
    constructor
**/
 GoBusLink::GoBusLink()
{
    _orientation = GUNDEFINED;
    _old_sh_pen = NULL;
    init_sh_pen();
    _pen.modifyPostcond(buslink_modify_pen, this);
}

    
/**
FUNCTION
    ~GoBusLink()
DESCRIPTION
    destructor
**/
 GoBusLink::~GoBusLink()
{
    if (_old_sh_pen)
	_old_sh_pen->destroy();
}


/**
FUNCTION
    orientation()
DESCRIPTION
    
**/
GBool GoBusLink::orientation(short o)
{
    if (o < GHORIZONTAL || o > GUNDEFINED)
	return GFALSE;
    else {
	_orientation = o;
	return GTRUE;
    }
}


/**
FUNCTION
    sh_pen()
DESCRIPTION
    Returns the shared pen.
**/
const GShPen* GoBusLink::sh_pen() const
{
    return _old_sh_pen;
}



/**
FUNCTION
    int GoBusLink::save();
DESCRIPTION
    ** static member, protected **
**/
int GoBusLink::save(const GoBusLink* link, GoStream* gostream) {
    return(gostream->saveObj(link->pen().shPen(), "GShPen", "pen") &&
	   gostream->writeShort(link->_orientation, "orientation") &&
	 GoLink::save(link, gostream));
}


/**
FUNCTION
    int GoBusLink::readParams();
DESCRIPTION
    
**/
int GoBusLink::readParams(GoBusLink* link, GoStream* gostream) {
    GShPen* p = NULL;
    const char* class_name;
    if (!gostream->restoreObj((void**) &p, &class_name))
	return GFALSE;
    if (strcmp(class_name, "GShPen") != 0)
	GWarning("GoBusLink::readParams: can't restore pen.");
    else
	link->pen().modify(p);
    
    short s; 
    if (!gostream->readShort(&s))   
	return GFALSE;
    link->_orientation = s;
    
    return GoLink::readParams(link, gostream);
}


/**
FUNCTION
    int GoBusLink::restore();
DESCRIPTION
    
**/
int GoBusLink::restore(GoBusLink** link, GoStream* gostream) {
    if (*link = GoBusLink::create())
	return GoBusLink::readParams(*link, gostream);
    return GFALSE;
}

