#ifdef REV_INFO
static char rcsid[] = "$Id: Graph.cc,v 1.5 1992/12/10 16:10:38 smadja Exp $";
#endif
/*****************************************************************************
MODULE:  BULL Graphic Objects
FILE:    Graph.cc
AUTHOR:  Aline Jacomet
DATE:    February 8th 1991

DESCRIPTION:
         Definition of the graph object class.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include <Go/Graph.h>
#include <Go/mGraph.h>
#include <Go/GoStream.h>
#include <Go/Tools/Error.h>



extern GAssoc save_precond;


/*
 * GCallback Types
 */
static char goGraphNaddNodePrecond[] = "addNodePrecond";    
static char goGraphNaddNodePostcond[] = "addNodePostcond";    
static char goGraphNremoveNodePrecond[] = "removeNodePrecond";    
static char goGraphNremoveNodePostcond[] = "removeNodePostcond";    
static char goGraphNaddLinkPrecond[] = "addLinkPrecond";    
static char goGraphNaddLinkPostcond[] = "addLinkPostcond";    
static char goGraphNremoveLinkPrecond[] = "removeLinkPrecond";    
static char goGraphNremoveLinkPostcond[] = "removeLinkPostcond";    



/*
 *------------
 * CLASS PART
 *------------
 */



static GoGraphMetaclass* graph_metaclass = NULL;

/**
FUNCTION
    GoGraph::metaclass()
DESCRIPTION
    
**/
GoGraphMetaclass* GoGraph::metaclass() {
    if (graph_metaclass == NULL) {
	graph_metaclass = new GoGraphMetaclass;
	graph_metaclass->superclass()->AddSubclass(graph_metaclass);
      GoStream::bind(GoGraph::metaclass()->classname(),
		     (int (*) (const void*, GoStream*)) GoGraph::save,
		     (int (*) (void**, GoStream*)) GoGraph::restore);
    }
    return graph_metaclass;
}


/**
FUNCTION
    ~GoGraphMetaclass()
DESCRIPTION
    
**/
GoGraphMetaclass::~GoGraphMetaclass() {
    if (this == graph_metaclass) {
        superclass()->RemoveSubclass(this);
	graph_metaclass = NULL;
    }
}


/**
FUNCTION
    GoGraphMetaclass::GoGraphMetaclass()
DESCRIPTION
    
**/
 GoGraphMetaclass::GoGraphMetaclass() {
}


/**
FUNCTION
    const char* GoGraphMetaclass::m_classname() const
DESCRIPTION
    Returns the classname.
**/
const char* GoGraphMetaclass::m_classname() const
{
    return "GoGraph";
}


/**
FUNCTION
    GoMetaclass* GoGraphMetaclass::m_superclass() const
DESCRIPTION
    Returns the superclass id.
**/
GoMetaclass* GoGraphMetaclass::m_superclass() const
{
    return GoGraphStruct::metaclass();
}


/**
FUNCTION
    Go* GoGraphMetaclass::m_create();
DESCRIPTION
    Creates a GoGraph instance.
**/
Go* GoGraphMetaclass::m_create() const {
    return (Go*)(new GoGraph);
}



/**
FUNCTION
    m_metaclass()
DESCRIPTION
    Returns the metaclass.
**/
GoMetaclass* GoGraph::m_metaclass () const
{
    return GoGraph::metaclass();
}


static void rotate_graph (GoGraph* g, GNumber a, GPoint* p)
{
    g->rotateGraph(a, *p);
}

static GActionRec goGraph_action_array[] =
{ 
  { "rotateGraph(GNumber, GPoint&=*(GPoint*)NULL)",
	(GActionProc) rotate_graph },
  { NULL, NULL }
};


/**
FUNCTION
    m_actions()
DESCRIPTION
    Actions for user interaction.
**/
const GActionRec* GoGraphMetaclass::m_actions() const
{
    return goGraph_action_array;
}




/**
FUNCTION
   _call_class_add_node_precond() 
DESCRIPTION
    Calls preconditions on addNode.
**/
inline GBool GoGraph::_call_class_add_node_precond (GoGraphNode* node,
						    GBool* is_previous,
						    GoMetaclass* linkclass)
    const
{
    // call conditions on all super-Metaclasses;
    for (GoMetaclass* mc = getMetaclass(); mc; mc = mc->superclass()) {
	if (((GoGraphMetaclass*) mc)->_flags._has_add_node_precond &&
	    ! mc->CallPreconditions(goGraphNaddNodePrecond, this, node,
				    is_previous, linkclass))
	    return GFALSE;
	// stop when reaching GoGraph Metaclass instance;
	if (mc == GoGraph::metaclass())
	    return GTRUE;
    }
    return GTRUE;
}


/**
FUNCTION
    _call_class_add_node_postcond()
DESCRIPTION
    Calls postconditions on addNode.
**/
inline void GoGraph::_call_class_add_node_postcond (GoGraphNode* node,
						    GBool* is_previous,
						    GoMetaclass* linkclass)
{
    // call conditions on all super-Metaclasses;
    for (GoMetaclass* mc = getMetaclass(); mc; mc = mc->superclass()) {
	if (((GoGraphMetaclass*) mc)->_flags._has_add_node_postcond)
	    CallPostconditions(goGraphNaddNodePostcond, this, node,
			       is_previous, linkclass);
	// stop when reaching GoGraph Metaclass instance;
	if (mc == GoGraph::metaclass())
	    return;
    }
}


/**
FUNCTION
   _call_class_remove_node_precond() 
DESCRIPTION
    Calls preconditions on removeNode.
**/
inline GBool GoGraph::_call_class_remove_node_precond (GoGraphNode* node)
    const
{
    // call conditions on all super-Metaclasses;
    for (GoMetaclass* mc = getMetaclass(); mc; mc = mc->superclass()) {
	if (((GoGraphMetaclass*) mc)->_flags._has_remove_node_precond &&
	    ! mc->CallPreconditions(goGraphNremoveNodePrecond, this, node))
	    return GFALSE;
	// stop when reaching GoGraph Metaclass instance;
	if (mc == GoGraph::metaclass())
	    return GTRUE;
    }
    return GTRUE;
}


/**
FUNCTION
    _call_class_remove_node_postcond()
DESCRIPTION
    Calls postconditions on removeNode.
**/
inline void GoGraph::_call_class_remove_node_postcond (GoGraphNode* node)
{
    // call conditions on all super-Metaclasses;
    for (GoMetaclass* mc = getMetaclass(); mc; mc = mc->superclass()) {
	if (((GoGraphMetaclass*) mc)->_flags._has_remove_node_postcond)
	    CallPostconditions(goGraphNremoveNodePostcond, this, node);
	// stop when reaching GoGraph Metaclass instance;
	if (mc == GoGraph::metaclass())
	    return;
    }
}

/**
FUNCTION
   _call_class_add_link_precond() 
DESCRIPTION
    Calls preconditions on addLink.
**/
inline GBool GoGraph::_call_class_add_link_precond (GoGraphNode* prev,
						    GoGraphNode* next,
						    GoMetaclass* linkclass)
    const
{
    // call conditions on all super-Metaclasses;
    for (GoMetaclass* mc = getMetaclass(); mc; mc = mc->superclass()) {
	if (((GoGraphMetaclass*) mc)->_flags._has_add_link_precond &&
	    ! mc->CallPreconditions(goGraphNaddLinkPrecond, this, prev, next,
				    linkclass))
	    return GFALSE;
	// stop when reaching GoGraph Metaclass instance;
	if (mc == GoGraph::metaclass())
	    return GTRUE;
    }
    return GTRUE;
}


/**
FUNCTION
    _call_class_add_link_postcond()
DESCRIPTION
    Calls postconditions on addLink.
**/
inline void GoGraph::_call_class_add_link_postcond (GoGraphNode* prev,
						    GoGraphNode* next, 
						    GoMetaclass* linkclass)
{
    // call conditions on all super-Metaclasses;
    for (GoMetaclass* mc = getMetaclass(); mc; mc = mc->superclass()) {
	if (((GoGraphMetaclass*) mc)->_flags._has_add_link_postcond)
	    CallPostconditions(goGraphNaddLinkPostcond, this, prev, next,
			       linkclass);
	// stop when reaching GoGraph Metaclass instance;
	if (mc == GoGraph::metaclass())
	    return;
    }
}


/**
FUNCTION
   _call_class_remove_link_precond() 
DESCRIPTION
    Calls preconditions on removeLink.
**/
inline GBool GoGraph::_call_class_remove_link_precond(GoLink* link) const
{
    // call conditions on all super-Metaclasses;
    for (GoMetaclass* mc = getMetaclass(); mc; mc = mc->superclass()) {
	if (((GoGraphMetaclass*) mc)->_flags._has_remove_link_precond &&
	    ! mc->CallPreconditions(goGraphNremoveLinkPrecond, this, link))
	    return GFALSE;
	// stop when reaching GoGraph Metaclass instance;
	if (mc == GoGraph::metaclass())
	    return GTRUE;
    }
    return GTRUE;
}


/**
FUNCTION
    _call_class_remove_link_postcond()
DESCRIPTION
    Calls postconditions on removeLink.
**/
inline void GoGraph::_call_class_remove_link_postcond (GoLink* link)
{
    // call conditions on all super-Metaclasses;
    for (GoMetaclass* mc = getMetaclass(); mc; mc = mc->superclass()) {
	if (((GoGraphMetaclass*) mc)->_flags._has_remove_link_postcond)
	    CallPostconditions(goGraphNremoveLinkPostcond, this, link);
	// stop when reaching GoGraph Metaclass instance;
	if (mc == GoGraph::metaclass())
	    return;
    }
}


/**
FUNCTION
    addNodePrecond
DESCRIPTION
    Adds or removes precondition on addNode().   
**/
void GoGraphMetaclass::addNodePrecond (GBool cbproc (void*, GoGraph*,
						     GoGraphNode*, GBool*,
						     GoMetaclass*),
				       void* closure, GAddOrRemove add)
{
    _flags._has_add_node_precond =
	AddOrRemoveCondition(goGraphNaddNodePrecond, cbproc, closure, add);
}


/**
FUNCTION
    addNodePostcond
DESCRIPTION
    Adds or removes postcondition on addNode(). 
**/
void GoGraphMetaclass::addNodePostcond (void cbproc (void*, GoGraph*,
						     GoGraphNode*, GBool*,
						     GoMetaclass*),
					void* closure, GAddOrRemove add)
{
    _flags._has_add_node_postcond =
	AddOrRemoveCondition(goGraphNaddNodePostcond, cbproc, closure, add);
}


/**
FUNCTION
    removeNodePrecond
DESCRIPTION
    Adds or removes precondition on removeNode(). 
**/
void GoGraphMetaclass::removeNodePrecond (GBool cbproc (void*, GoGraph*,
						       GoGraphNode*),
					  void* closure, GAddOrRemove add)
{
    _flags._has_remove_node_precond =
	AddOrRemoveCondition(goGraphNremoveNodePrecond, cbproc, closure, add);
}


/**
FUNCTION
    removeNodePostcond
DESCRIPTION
    Adds or removes postcondition on removeNode(). 
**/
void GoGraphMetaclass::removeNodePostcond(void cbproc (void*, GoGraph*,
						       GoGraphNode*),
					  void* closure, GAddOrRemove add)
{
    _flags._has_remove_node_postcond =
	AddOrRemoveCondition(goGraphNremoveNodePostcond, cbproc, closure, add);
}


/**
FUNCTION
    addLinkPrecond
DESCRIPTION
    Adds or removes precondition on addLink(). 
**/
void GoGraphMetaclass::addLinkPrecond (GBool cbproc (void*, GoGraph*,
						    GoGraphNode*, GoGraphNode*,
						    GoMetaclass*),
				       void* closure, GAddOrRemove add)
{
    _flags._has_add_link_precond =
	AddOrRemoveCondition(goGraphNaddLinkPrecond, cbproc, closure, add);
}


/**
FUNCTION
    addLinkPostcond
DESCRIPTION
    Adds or removes postcondition on addLink(). 
**/
void GoGraphMetaclass::addLinkPostcond (void cbproc (void*, GoGraph*,
						     GoGraphNode*,
						     GoGraphNode*,
						     GoMetaclass*),
					void* closure, GAddOrRemove add)
{
    _flags._has_add_link_postcond =
	AddOrRemoveCondition(goGraphNaddLinkPostcond, cbproc, closure, add);
}


/**
FUNCTION
    removeLinkPrecond
DESCRIPTION
    Adds or removes precondition on removeLink().  
**/
void GoGraphMetaclass::removeLinkPrecond (GBool cbproc (void*, GoGraph*,
						       GoLink*),
					  void* closure, GAddOrRemove add)
{
    _flags._has_remove_link_precond =
	AddOrRemoveCondition(goGraphNremoveLinkPrecond, cbproc, closure, add);
}


/**
FUNCTION
    removeLinkPostcond
DESCRIPTION
   Adds or removes postcondition on removeLink().  
**/
void GoGraphMetaclass::removeLinkPostcond (void cbproc (void*, GoGraph*,
							GoLink*),
					   void* closure, GAddOrRemove add)
{
    _flags._has_remove_link_postcond =
	AddOrRemoveCondition(goGraphNremoveLinkPostcond, cbproc, closure, add);
}


/*
 *--------------
 * PRIVATE DATA
 *--------------
 */



/**
FUNCTION
   _call_add_node_precond() 
DESCRIPTION
    
**/
INLINE GBool GoGraph::_call_add_node_precond (GoGraphNode* node,
					      GBool* is_previous,
					      GoMetaclass* linkclass)
    const
{
    return (_flags._has_add_node_precond ?
	    CallPreconditions(goGraphNaddNodePrecond, node, is_previous,
			      linkclass) : GTRUE);
}


/**
FUNCTION
    _call_add_node_postcond()
DESCRIPTION
    
**/
INLINE void GoGraph::_call_add_node_postcond (GoGraphNode* node,
					      GBool* is_previous,
					      GoMetaclass* linkclass)
{
    if (_flags._has_add_node_postcond)
	CallPostconditions(goGraphNaddNodePostcond, node, is_previous,
			   linkclass);
}


/**
FUNCTION
   _call_remove_node_precond() 
DESCRIPTION
    
**/
INLINE GBool GoGraph::_call_remove_node_precond (GoGraphNode* node) const
{
    return (_flags._has_remove_node_precond ?
	    CallPreconditions(goGraphNremoveNodePrecond, node) : GTRUE);
}


/**
FUNCTION
    _call_remove_node_postcond()
DESCRIPTION
    
**/
INLINE void GoGraph::_call_remove_node_postcond (GoGraphNode* node)
{
    if (_flags._has_remove_node_postcond)
	CallPostconditions(goGraphNremoveNodePostcond, node);
}

/**
FUNCTION
   _call_add_link_precond() 
DESCRIPTION
    
**/
INLINE GBool GoGraph::_call_add_link_precond (GoGraphNode* prev,
					      GoGraphNode* next,
					      GoMetaclass* linkclass) const
{
    return (_flags._has_add_link_precond ?
	    CallPreconditions(goGraphNaddLinkPrecond, prev, next, linkclass)
	    : GTRUE);
}


/**
FUNCTION
    _call_add_link_postcond()
DESCRIPTION
    
**/
INLINE void GoGraph::_call_add_link_postcond (GoGraphNode* prev,
					      GoGraphNode* next, 
					      GoMetaclass* linkclass)
{
    if (_flags._has_add_link_postcond)
	CallPostconditions(goGraphNaddLinkPostcond, prev, next, linkclass);
}


/**
FUNCTION
   _call_remove_link_precond() 
DESCRIPTION
    
**/
INLINE GBool GoGraph::_call_remove_link_precond (GoLink* link) const
{
    return (_flags._has_remove_link_precond ?
	    CallPreconditions(goGraphNremoveLinkPrecond, link) : GTRUE);
}


/**
FUNCTION
    _call_remove_link_postcond()
DESCRIPTION
    
**/
INLINE void GoGraph::_call_remove_link_postcond (GoLink* link)
{
    if (_flags._has_remove_link_postcond)
	CallPostconditions(goGraphNremoveLinkPostcond, link);
}



/*
 *--------------
 * PUBLIC DATA
 *--------------
 */


/**
FUNCTION
    GoGraph()
DESCRIPTION
    constructor
**/
 GoGraph::GoGraph()
{
    position_type = GAPPLICATION_DEFINE;
}


/**
FUNCTION
    ~GoGraph()
DESCRIPTION
    destructor
**/
 GoGraph::~GoGraph()
{
}


    
/**
FUNCTION
    addNode()
DESCRIPTION
    Adds a node to the Graph.
**/
GoGraphNode* GoGraph::addNode(GoGraphNode* other, GBool is_previous,
			      GoMetaclass* type)
{
    if (_call_add_node_precond(other, &is_previous, type) &&
	_call_class_add_node_precond(other, &is_previous, type)) {
	if (!other || (other && other->parent() == this)) {
	    if (!type)
		type = link_type;
	    else
		if (!type->isSubclass(GoLink::metaclass())) {
		    GWarning("GoGraph::addNode: the specified type of link is not a GoLink or a subclass of GoLink.");
		    return NULL;
		}
	    
	    /*
	     * Create the new node.
	     */
	    GoGraphNode *node = GoGraphNode::create();
	    if (!first_node) {
		first_node = node;
		first_node->clipPostcond(graphstr_set_coordsys, this, GADD);
	    }
	    if (other) {
		/*
		 * Create the link.
		 */
		GoLink* link = (GoLink*)Go::Create(type);
		if (!link) {
		    node->in_destroy = GTRUE;
		    node->destroy();
		    return NULL;
		}
		if (is_previous) {
		    link->prev_node = other;
		    link->next_node = node;
		    /*
		     * GSet references.
		     */
		    other->next_link.add(link);
		    node->prev_link.add(link);
		} else {
		    link->prev_node = node;
		    link->next_node = other;
		    /*
		     * GSet references.
		     */
		    other->prev_link.add(link);
		    node->next_link.add(link);
		}
		link->reparent(this, Child());
		link->map();
	    }
	    node->reparent(this);
	    node->map();

	    _call_add_node_postcond(other, &is_previous, type);
	    _call_class_add_node_postcond(other, &is_previous, type);
	    return node;
	} else
	    return NULL;
    } else
	return NULL;
}


/**
FUNCTION
    removeNode()
DESCRIPTION
    Removes a node and its connected links.
**/
GBool GoGraph::removeNode(GoGraphNode* node)
{
    if (_call_remove_node_precond(node) &&
	_call_class_remove_node_precond(node)) {
	if (node->parent() == this) {
	    GoLink** prev = node->previousLink();
	    GoLink** next = node->nextLink();
	    int max_prev = node->nbPreviousLink();
	    int max_next = node->nbNextLink();
	    GoGraphNode* n;
	    /*
	     * Removes connected links.
	     */
	    for (int i = 0; i < max_prev; i++) {
		n = (GoGraphNode*)prev[i]->prev_node;
		n->next_link.rem(prev[i]);
		prev[i]->in_destroy = GTRUE;
		prev[i]->destroy();
	    }
	    for (i = 0; i < max_next; i++) {
		n = (GoGraphNode*)next[i]->next_node;
		n->prev_link.rem(next[i]);
		next[i]->in_destroy = GTRUE;
		next[i]->destroy();
	    }
	    /*
	     * Removes the node.
	     */
	    if (first_node == node) {
		first_node->clipPostcond(graphstr_set_coordsys, this, GREMOVE);
		int ind;
		    Go* c = Child();
		for(ind = 0;
		    c &&
		    !c->getMetaclass()->isSubclass(GoGraphNode::metaclass());
		    ind++)
		    c = Child(ind);
		first_node = (GoNode*)c;
		first_node->clipPostcond(graphstr_set_coordsys, this, GADD);
		set_coordsys(c);
	    }
	    _call_remove_node_postcond(node);
	    _call_class_remove_node_postcond(node);

	    node->in_destroy = GTRUE;
	    node->destroy();

	    delete prev;
	    delete next;

	    return GTRUE;
	} else
	    return GFALSE;
    } else
	return GFALSE;
}


/**
FUNCTION
    addLink()
DESCRIPTION
    Adds a link to the Graph.
**/
GoLink* GoGraph::addLink(GoGraphNode* prev, GoGraphNode* next, GoMetaclass* type)
{
    if (_call_add_link_precond(prev, next, type) &&
	_call_class_add_link_precond(prev, next, type)) {
	/*
	 * Test of parameters.
	 */
	if (!prev || !next) {
	    GWarning("GoGraph::addLink: one of the two specified nodes is NULL.");
	    return NULL;
	}
	if (prev->parent() != this || next->parent() != this) {
	    GWarning("GoGraph::addLink: the two specified nodes don't belong to the specified graph.");
	    return NULL;
	}
	
	if (!type)
	    type = link_type;
	else
	    if (!type->isSubclass(GoLink::metaclass())) {
		GWarning("GoGraph::addLink: the specified type of link is not a GoLink or a subclass of GoLink.");
		return NULL;
	    }
	
	/*
	 * Create the link.
	 */
	GoLink* link = (GoLink*)Go::Create(type);
	if (link) {
	    link->prev_node = prev;
	    link->next_node = next;
	    /*
	     * GSet references.
	     */
	    prev->next_link.add(link);
	    next->prev_link.add(link);
	    link->reparent(this, Child());
	    link->map();
	    /*
	     * set constraints on link if necessary.
	     */
	    link->_constraints();

	    _call_add_link_postcond(prev, next, type);
	    _call_class_add_link_postcond(prev, next, type);
	}
	return link;
    } else
	return NULL;
}


/**
FUNCTION
    removeLink()
DESCRIPTION
    Removes a link.
**/
GBool GoGraph::removeLink(GoLink* link)
{
    if (_call_remove_link_precond(link) &&
	_call_class_remove_link_precond(link)) {
	if (!link || link->parent() != this) {
	    GWarning("GoGraph::removeLink: the specified link does not exist in this graph.");
	    return GFALSE;
	}

	/*
	 * Removes references.
	 */
	if (link->prev_node)
	    ((GoGraphNode*)(link->prev_node))->next_link.rem(link);
	if (link->next_node)
	    ((GoGraphNode*)(link->next_node))->prev_link.rem(link);
	
	_call_remove_link_postcond(link);
	_call_class_remove_link_postcond(link);

	/*
	 * Deletes the link
	 */
	link->in_destroy = GTRUE;
	link->destroy();
	return GTRUE;
    } else
	return GFALSE;
}


/**
FUNCTION
    removeLink()
DESCRIPTION
    Removes a link.
**/
GBool GoGraph::removeLink(GoGraphNode* node1, GoGraphNode* node2)
{
    return removeLink(node1->link(node2));
}


/**
FUNCTION
    rotateGraph()
DESCRIPTION
    Rotates the graph but not nodes.
**/
void GoGraph::rotateGraph(GNumber angle, GPoint& center)
{
    GPoint ce;
    GRegion tmp_reg;

    if (! &center) {
	addClip(tmp_reg);
	GBox box = tmp_reg.bigbox();
	ce.x = (box.xmin + box.xmax) / 2;
	ce.y = (box.ymin + box.ymax) / 2;
    } else
	ce = center;

    GCoordSys cs = Abscs();
    cs.rotate(cs.from(ce), angle);
    setCoordsys(cs);
    for (Go* c = child(); c; c = c->next())
	if (c->getMetaclass()->isSubclass(GoGraphNode::metaclass()))
	    for (Go* ch = c->child(); ch; ch = ch->next()) {
		cs = ch->Abscs();
		tmp_reg = GEmptyRegion();
		ch->addClip(tmp_reg);
		GBox b = tmp_reg.bigbox();
		GPoint m((b.xmin + b.xmax) / 2, (b.ymin + b.ymax) / 2);
		cs.rotate(cs.from(m), -angle);
		ch->setCoordsys(cs);
	    }
}


/**
FUNCTION
    addNodePrecond
DESCRIPTION
    
**/
void GoGraph::addNodePrecond (GBool cbproc (void*, GoGraph*, GoGraphNode*,
					   GBool*, GoMetaclass*),
			      void* closure, GAddOrRemove add)
{
    _flags._has_add_node_precond =
	AddOrRemoveCondition(goGraphNaddNodePrecond, cbproc, closure, add);
}


/**
FUNCTION
    addNodePostcond
DESCRIPTION
    
**/
void GoGraph::addNodePostcond (void cbproc (void*, GoGraph*, GoGraphNode*,
					    GBool*, GoMetaclass*),
			       void* closure, GAddOrRemove add)
{
    _flags._has_add_node_postcond =
	AddOrRemoveCondition(goGraphNaddNodePostcond, cbproc, closure, add);
}


/**
FUNCTION
    removeNodePrecond
DESCRIPTION
    
**/
void GoGraph::removeNodePrecond (GBool cbproc (void*, GoGraph*, GoGraphNode*),
				 void* closure, GAddOrRemove add)
{
    _flags._has_remove_node_precond =
	AddOrRemoveCondition(goGraphNremoveNodePrecond, cbproc, closure, add);
}


/**
FUNCTION
    removeNodePostcond
DESCRIPTION
    
**/
void GoGraph::removeNodePostcond (void cbproc (void*, GoGraph*, GoGraphNode*),
				  void* closure, GAddOrRemove add)
{
    _flags._has_remove_node_postcond =
	AddOrRemoveCondition(goGraphNremoveNodePostcond, cbproc, closure, add);
}


/**
FUNCTION
    addLinkPrecond
DESCRIPTION
    
**/
void GoGraph::addLinkPrecond (GBool cbproc (void*, GoGraph*, GoGraphNode*,
					   GoGraphNode*, GoMetaclass*),
			      void* closure, GAddOrRemove add)
{
    _flags._has_add_link_precond =
	AddOrRemoveCondition(goGraphNaddLinkPrecond, cbproc, closure, add);
}


/**
FUNCTION
    addLinkPostcond
DESCRIPTION
    
**/
void GoGraph::addLinkPostcond (void cbproc (void*, GoGraph*, GoGraphNode*,
					    GoGraphNode*, GoMetaclass*),
			       void* closure, GAddOrRemove add)
{
    _flags._has_add_link_postcond =
	AddOrRemoveCondition(goGraphNaddLinkPostcond, cbproc, closure, add);
}


/**
FUNCTION
    removeLinkPrecond
DESCRIPTION
    
**/
void GoGraph::removeLinkPrecond (GBool cbproc (void*, GoGraph*, GoLink*),
				 void* closure, GAddOrRemove add)
{
    _flags._has_remove_link_precond =
	AddOrRemoveCondition(goGraphNremoveLinkPrecond, cbproc, closure, add);
}


/**
FUNCTION
    removeLinkPostcond
DESCRIPTION
    
**/
void GoGraph::removeLinkPostcond (void cbproc (void*, GoGraph*, GoLink*),
				  void* closure, GAddOrRemove add)
{
    _flags._has_remove_link_postcond =
	AddOrRemoveCondition(goGraphNremoveLinkPostcond, cbproc, closure, add);
}




/**
FUNCTION
    int GoGraph::save();
DESCRIPTION
    ** static member, protected **
**/
int GoGraph::save(const GoGraph* graph, GoStream* gostream) {
    save_precond.reset();
    
    GBool reinitialize_first_node = GFALSE;
    Go** tab = (Go**)graph->_children.data();
    int maxi = graph->_children.nelems();

    GPointerArray temp;
    for (int i = 0; i < maxi; i++)
	temp.add(tab[i]);

    for (i = 0; i < maxi; i++)
	if (!tab[i]->callSavePrecond()) {
	    if (tab[i]->getMetaclass()->isSubclass(GoNode::metaclass())) {
		GoLink** l = ((GoGraphNode*)tab[i])->allLink();
		for (int j = 0; l[j]; j++) {
		    save_precond.set(l[j], l[j]);
		    temp.rem(l[j]);
		}
		if (tab[i] == (Go*)graph->firstNode())
		    reinitialize_first_node = GTRUE;
	    } else
		if (tab[i]->getMetaclass()->isSubclass(GoLink::metaclass()))
		    save_precond.set(tab[i], tab[i]);
	    temp.rem(tab[i]);
	}

    Go** ch = (Go**)temp.data();
    maxi = temp.nelems();
    for (i = 0; i < maxi; i++)
	if (!gostream->saveObj(ch[i], ch[i]->getMetaclass()->classname(),
			       "child"))
	    return GFALSE;
    if (!gostream->saveObj(NULL, NULL, "end of children"))
	return GFALSE;

    GoGraphNode* f_n = NULL;
    if (reinitialize_first_node) {
	for (i = 0; i < maxi; i++)
	    if (ch[i]->getMetaclass()->isSubclass(GoGraphNode::metaclass())) {
		f_n = (GoGraphNode*)ch[i];
		break;
	    }
    } else
	f_n = (GoGraphNode*)graph->firstNode();

    if (f_n) {
	if (!gostream->saveObj(f_n, f_n->getMetaclass()->classname(),
			       "first node"))
	    return GFALSE;
    } else
	if (!gostream->saveObj(NULL, NULL, "first node"))
	    return GFALSE;
    
    return(GoGraphStruct::save(graph, gostream));
}


/**
FUNCTION
    int GoGraph::readParams();
DESCRIPTION
    
**/
int GoGraph::readParams(GoGraph* graph, GoStream* gostream) {
    const char* class_name;
    for (;;) {
	void* next_child;
	if (!gostream->restoreObj(&next_child, &class_name))
	    return GFALSE;
	if (next_child)
	    ((Go*)next_child)->DoReparent(graph);
	else
	    break;
    }

    GoNode* first;
    if (!gostream->restoreObj((void**)&first, &class_name))
	return GFALSE;
    if (first)
	if (strcmp(class_name, GoGraphNode::metaclass()->classname()) != 0) {
	    GWarning("GoGraph::readParams: can't restore first node.");
	    first = NULL;
	}
    graph->first_node = first;
    
    return GoGraphStruct::readParams(graph, gostream);
}


/**
FUNCTION
    int GoGraph::restore();
DESCRIPTION
    
**/
int GoGraph::restore(GoGraph** graph, GoStream* gostream) {
    if (*graph = GoGraph::create())
	return GoGraph::readParams(*graph, gostream);
    return GFALSE;
}

