#ifdef REV_INFO
static char rcsid[] = "$Id: GraphNode.cc,v 1.6 1992/12/10 16:12:57 smadja Exp $";
#endif
/*****************************************************************************
MODULE:  BULL Graphic Objects
FILE:    GraphNode.cc
AUTHOR:  Aline Jacomet
DATE:    February 8th 1991

DESCRIPTION:
         Definition of the graph node object class. 

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include <Go/GraphNode.h>
#include <Go/mGraphNode.h>
#include <Go/Graph.h>
#include <Go/GoStream.h>


extern GAssoc save_precond;


/*
 * GCallback Types
 */
static char goGraphNodeNattachNodePrecond[] = "attachNodePrecond";    
static char goGraphNodeNattachNodePostcond[] = "attachNodePostcond";    


/*
 *------------
 * CLASS PART
 *------------
 */


static GoGraphNodeMetaclass* graphNode_metaclass = NULL;

/**
FUNCTION
    GoGraphNode::metaclass()
DESCRIPTION
    
**/
GoGraphNodeMetaclass* GoGraphNode::metaclass() {
    if (graphNode_metaclass == NULL) {
	graphNode_metaclass = new GoGraphNodeMetaclass;
	graphNode_metaclass->superclass()->AddSubclass(graphNode_metaclass);
      GoStream::bind(GoGraphNode::metaclass()->classname(),
		     (int (*) (const void*, GoStream*)) GoGraphNode::save,
		     (int (*) (void**, GoStream*)) GoGraphNode::restore);
    }
    return graphNode_metaclass;
}


/**
FUNCTION
    ~GoGraphNodeMetaclass()
DESCRIPTION
    
**/
GoGraphNodeMetaclass::~GoGraphNodeMetaclass() {
    if (this == graphNode_metaclass) {
        superclass()->RemoveSubclass(this);
	graphNode_metaclass = NULL;
    }
}


/**
FUNCTION
    GoGraphNodeMetaclass::GoGraphNodeMetaclass()
DESCRIPTION
    
**/
 GoGraphNodeMetaclass::GoGraphNodeMetaclass() {
}


/**
FUNCTION
    const char* GoGraphNodeMetaclass::m_classname() const
DESCRIPTION

**/
const char* GoGraphNodeMetaclass::m_classname() const
{
    return "GoGraphNode";
}


/**
FUNCTION
    GoMetaclass* GoGraphNodeMetaclass::m_superclass() const
DESCRIPTION

**/
GoMetaclass* GoGraphNodeMetaclass::m_superclass() const
{
    return GoNode::metaclass();
}


/**
FUNCTION
    Go* GoGraphNodeMetaclass::m_create();
DESCRIPTION

**/
Go* GoGraphNodeMetaclass::m_create() const
{
    return (Go*)(new GoGraphNode);
}


/**
FUNCTION
    GoGraphNode::m_dup()
DESCRIPTION

**/
void GoGraphNode::m_dup
    (const Go* src, Go* par, Go* prv, GBool b, GBool dochildren)
{
    GoNode::m_dup(src, par, prv, b, dochildren);
}


/**
FUNCTION
    m_metaclass()
DESCRIPTION
    Returns the metaclass.
**/
GoMetaclass* GoGraphNode::m_metaclass () const
{
    return GoGraphNode::metaclass();
}




static GActionRec goGraphNode_action_array[] =
{ 
  { NULL, NULL }
};


/**
FUNCTION
    m_actions()
DESCRIPTION
    
**/
const GActionRec* GoGraphNodeMetaclass::m_actions() const
{
    return goGraphNode_action_array;
}


/**
FUNCTION
   _call_class_attach_node_precond() 
DESCRIPTION
    
**/
GBool GoGraphNode::_call_class_attach_node_precond (GoGraphNode* node,
						    GBool* prev,
						    GoMetaclass* linkclass,
						    GoGraph* graph) const
{
    // call conditions on all super-Metaclasses;
    for (GoMetaclass* mc = getMetaclass(); mc; mc = mc->superclass()) {
	if (((GoGraphNodeMetaclass*) mc)->_flags._has_attach_node_precond &&
	    ! mc->CallPreconditions(goGraphNodeNattachNodePrecond, this, node,
				    prev, linkclass, graph))
	    return GFALSE;
	// stop when reaching GoGraphNode Metaclass instance;
	if (mc == GoGraphNode::metaclass())
	    return GTRUE;
    }
    return GTRUE;
}


/**
FUNCTION
   _call_class_attach_node_postcond() 
DESCRIPTION
    
**/
void GoGraphNode::_call_class_attach_node_postcond (GoGraphNode* node,
						    GBool* prev,
						    GoMetaclass* linkclass,
						    GoGraph* graph)
{
    // call conditions on all super-Metaclasses;
    for (GoMetaclass* mc = getMetaclass(); mc; mc = mc->superclass()) {
	if (((GoGraphNodeMetaclass*) mc)->_flags._has_attach_node_postcond)
	    CallPostconditions(goGraphNodeNattachNodePostcond, this, node,
			       prev, linkclass, graph);
	// stop when reaching GoGraphNode Metaclass instance;
	if (mc == GoGraphNode::metaclass())
	    return;
    }
}



/**
FUNCTION
    attachNodePrecond
DESCRIPTION
    
**/
void GoGraphNodeMetaclass::attachNodePrecond (GBool cbproc (void*, GoGraphNode*,
							   GoGraphNode*, GBool*,
							   GoMetaclass*,
							   GoGraph*),
					      void* closure, GAddOrRemove add)
{
    _flags._has_attach_node_precond =
	AddOrRemoveCondition(goGraphNodeNattachNodePrecond, cbproc, closure,
			     add);
}


/**
FUNCTION
    attachNodePostcond
DESCRIPTION
    
**/
void GoGraphNodeMetaclass::attachNodePostcond (void cbproc (void*,
							    GoGraphNode*,
							    GoGraphNode*,
							    GBool*,
							    GoMetaclass*,
							    GoGraph*),
					       void* closure, GAddOrRemove add)
{
    _flags._has_attach_node_postcond =
	AddOrRemoveCondition(goGraphNodeNattachNodePostcond, cbproc, closure,
			     add);
}


/*
 *--------------
 * PRIVATE DATA
 *--------------
 */



/**
FUNCTION
    modif_clip()
DESCRIPTION
    My clip is modified => links have to be informed.
**/
void GoGraphNodeMetaclass::modif_clip(Go* node)
{
    GoNodeMetaclass::modif_clip(node);
    
    GoLink** tab = (GoLink**) ((GoGraphNode*)node)->prev_link.data();
    int maxi = ((GoGraphNode*)node)->prev_link.nelems();
    for (int i = 0; i < maxi; i++)
	tab[i]->_constraints();
    tab = (GoLink**) ((GoGraphNode*)node)->next_link.data();
    maxi = ((GoGraphNode*)node)->next_link.nelems();
    for (i = 0; i < maxi; i++)
	tab[i]->_constraints();
}
    


/*
 *----------------
 * PROTECTED DATA
 *----------------
 */


/**
FUNCTION
   _call_attach_node_precond() 
DESCRIPTION
    
**/
GBool GoGraphNode::_call_attach_node_precond (GoGraphNode* node,
					     GBool* prev,
					     GoMetaclass* linkclass,
					     GoGraph* graph) const
{
    return (_flags._has_attach_node_precond ?
	    CallPreconditions(goGraphNodeNattachNodePrecond, node, prev,
			      linkclass, graph) : GTRUE);
}


/**
FUNCTION
   _call_attach_node_postcond() 
DESCRIPTION
    
**/
void GoGraphNode::_call_attach_node_postcond (GoGraphNode* node,
					      GBool* prev,
					      GoMetaclass* linkclass,
					      GoGraph* graph)
{
    if (_flags._has_attach_node_postcond)
	CallPostconditions(goGraphNodeNattachNodePostcond, node, prev,
			   linkclass, graph);
}




/*
 *--------------
 * PUBLIC DATA
 *--------------
 */


/**
FUNCTION
    GoGraphNode()
DESCRIPTION
    constructor
**/
GoGraphNode::GoGraphNode() : prev_link(64), next_link(64)
{
}


/**
FUNCTION
    ~GoGraphNode()
DESCRIPTION
    destructor
**/
GoGraphNode::~GoGraphNode()
{
}


/**
FUNCTION
    nextLink()
DESCRIPTION
    Returns an array filled with next links.
**/
GoLink** GoGraphNode::nextLink() const
{
    int nb = next_link.nelems();
    if (!nb)
	return NULL;
    else {
	GoLink** tab = new GoLink* [nb+1];
	bcopy(next_link.data(), tab, sizeof(GoLink*) * nb);
	tab[nb] = NULL;
	return tab;
    }
}


/**
FUNCTION
    previousLink()
DESCRIPTION
    Returns an array filled with previous links.
**/
GoLink** GoGraphNode::previousLink() const
{
    int nb = prev_link.nelems();
    if (!nb)
	return NULL;
    else {
	GoLink** tab = new GoLink* [nb+1];
	bcopy(prev_link.data(), tab, sizeof(GoLink*) * nb);
	tab[nb] = NULL;
	return tab;
    }
}


/**
FUNCTION
    allLink()
DESCRIPTION
    Returns an array filled with all links.
**/
GoLink** GoGraphNode::allLink() const
{
    int nb_prev = prev_link.nelems();
    int nb_next = next_link.nelems();
    if ((nb_prev + nb_next) == 0)
	return NULL;
    else {
	GoLink** tab = new GoLink* [nb_prev + nb_next + 1];
	bcopy(prev_link.data(), tab, sizeof(GoLink*) * nb_prev);
	bcopy(next_link.data(), tab+nb_prev, sizeof(GoLink*) * nb_next);
	tab[nb_prev + nb_next] = NULL;
	return tab;
    }
}


/**
FUNCTION
    link()
DESCRIPTION
    Returns the link connected to "this" and "node".
**/
GoLink* GoGraphNode::link(const GoGraphNode* node) const
{
    GoLink** tab;
    GoLink* ret = NULL;
    
    if (prev_link.nelems()) {
	tab = (GoLink**)prev_link.data();
	for(int i = 0; i < prev_link.nelems(); i++)
	    if (tab[i]->prev_node == (GoNode*) node)
		ret = tab[i];
    }
    if (!ret)
	if (next_link.nelems()) {
	    tab = (GoLink**)next_link.data();
	    for(int i = 0; i < next_link.nelems(); i++)
		if (tab[i]->next_node == (GoNode*)node)
		    ret = tab[i];
	}
    return ret;
}


/**
FUNCTION
    nextNode()
DESCRIPTION
    Returns an array filled with next nodes.
**/
GoGraphNode** GoGraphNode::nextNode() const
{
    int nb = next_link.nelems();
    if (!nb)
	return NULL;
    else {
	GoGraphNode** tab = new GoGraphNode* [nb+1];
	GoLink** link = (GoLink**)next_link.data();
	for (int i = 0; i < nb; i++)
	    tab[i] = (GoGraphNode*)link[i]->next_node;
	tab[nb] = NULL;
	return tab;
    }
}


/**
FUNCTION
    previousNode()
DESCRIPTION
    Returns an array filled with previous nodes.
**/
GoGraphNode** GoGraphNode::previousNode() const
{
    int nb = prev_link.nelems();
    if (!nb)
	return NULL;
    else {
	GoGraphNode** tab = new GoGraphNode* [nb+1];
	GoLink** link = (GoLink**)prev_link.data();
	for (int i = 0; i < nb; i++)
	    tab[i] = (GoGraphNode*)link[i]->prev_node;
	tab[nb] = NULL;
	return tab;
    }
}


/**
FUNCTION
    allNode()
DESCRIPTION
    Returns an array filled with all nodes.
**/
GoGraphNode** GoGraphNode::allNode() const
{
    int nb_prev = prev_link.nelems();
    int nb_next = next_link.nelems();
    if ((nb_prev + nb_next) == 0)
	return NULL;
    else {
	int j;
	GoGraphNode** tab = new GoGraphNode* [nb_prev + nb_next + 1];
	GoLink** l_prev = (GoLink**)prev_link.data();
	GoLink** l_next = (GoLink**)next_link.data();
	for (int i = 0; i < nb_prev; i++)
	    tab[i] = (GoGraphNode*)l_prev[i]->prev_node;
	for (i = 0, j = nb_prev; i < nb_next; i++, j++)
	    tab[j] = (GoGraphNode*)l_next[i]->next_node;
	tab[nb_prev + nb_next] = NULL;
	return tab;
    }
}


/**
FUNCTION
    attachNode()
DESCRIPTION
    Attachs a node to a new graph, new extrem with a new link.
**/
GBool GoGraphNode::attachNode(GoGraphNode* extrem, GBool is_previous,
			     GoMetaclass* linkclass, GoGraph* graph)
{
    if (_call_attach_node_precond(extrem, &is_previous, linkclass, graph) &&
	_call_class_attach_node_precond(extrem, &is_previous, linkclass,
					graph)) {
	/*
	 * Test parameters
	 */

	/*
	 * linkclass
	 */
	if (!linkclass) {
	    if (graph)
		linkclass = graph->defaultLinkType();
	    else {
		if (extrem->parent())
		    linkclass =((GoGraph*)extrem->parent())->defaultLinkType();
		else {
		    GWarning("GoGraphNode::attachNode: the specified node belongs to no graph.");
		    return GFALSE;
		}
	    }
	} else
	    if (!linkclass->isSubclass(GoLink::metaclass())) {
		GWarning("GoGraphNode::attachNode: the specified type of link is not a GoLink or a subclass of GoLink.");
		return GFALSE;
	    }

	/*
	 * extrem and graph
	 */
	if (extrem) {
	    if (!extrem->getMetaclass()->isSubclass(GoGraphNode::metaclass())) {
		GWarning("GoGraphNode::attachNode: specified node is not an instance of GoGraphNode class.");
		return GFALSE;
	    }
	    if (! extrem->parent()->getMetaclass()->isSubclass(GoGraph::metaclass())) {
		GWarning("GoGraphNode::attachNode: the specified node does not belong to a GoGraph object.");
		return GFALSE;
	    }
	} else {
	    if (graph) {
		if (! graph->getMetaclass()->isSubclass(GoGraph::metaclass())) {
		    GWarning("GoGraphNode::attachNode: the specified GoGraph instance is not a graph object.");
		    return GFALSE;
		}
	    } else {
		GWarning("GoGraphNode::attachNode: the specified GoGraph instance is NULL.");
		return GFALSE;
	    }
	}
	if (graph && extrem) 
	    if (extrem->parent() != graph) {
		GWarning("GoGraphNode::attachNode: the specified 'extrem' node does not belong to the specified graph.");
		return GFALSE;
	    }

	GoGraph* new_graph;
	if (graph)
	    new_graph = graph;
	else
	    new_graph = (GoGraph*)extrem->parent();
	GoGraph* ex = (GoGraph*)parent();

	if (ex != new_graph) 
	    /*
	     * Reparent "this" on the new graph.
	     */
	    reparent(new_graph);
	
	GoLink** prev = previousLink();
	GoLink** next = nextLink();
	if ((nbPreviousLink() + nbNextLink()) != 0) {
	    /*
	     * Separate node and its graph
	     */
	    int max_prev = nbPreviousLink();
	    int max_next = nbNextLink();
	    GoGraphNode* n;
	    for (int i = 0; i < max_prev; i++) {
		n = (GoGraphNode*)prev[i]->prev_node;
		n->next_link.rem(prev[i]);
		prev[i]->in_destroy = GTRUE;
		prev[i]->destroy();
	    }
	    for (i = 0; i < max_next; i++) {
		n = (GoGraphNode*)next[i]->next_node;
		n->prev_link.rem(next[i]);
		next[i]->in_destroy = GTRUE;
		next[i]->destroy();
	    }
	    if (nbPreviousLink())
		prev_link.rem(0, prev_link.nelems());
	    if (nbNextLink())
		next_link.rem(0, next_link.nelems());
	    delete next;
	    delete prev;
	}

	if (ex)
	    if (ex != new_graph && ex->firstNode() == this) {
		ex->first_node->clipPostcond(graphstr_set_coordsys,
					     ex, GREMOVE);
		int ind;
		Go* c = Child();
		for(ind = 0;
		    c && !c->getMetaclass()->isSubclass(GoGraphNode::metaclass());
		    ind++)
		    c = ex->Child(ind);
		ex->first_node = (GoNode*)c;
		ex->first_node->clipPostcond(graphstr_set_coordsys,
					     ex, GADD);
		ex->set_coordsys(c);
	    }
	
	if (!extrem) {
	    if (! graph->first_node) {
		graph->first_node = this;
		this->clipPostcond(graphstr_set_coordsys, graph, GADD);
	    }
	} else {
	    /*
	     * Create the new link
	     */
	    GoLink* link = (GoLink*)Go::Create(linkclass);
	    if (!link)
		return GFALSE;
	    if (is_previous) {
		link->prev_node = extrem;
		link->next_node = this;
		/*
		 * GSet references.
		 */
		extrem->next_link.add(link);
		prev_link.add(link);
	    } else {
		link->prev_node = this;
		link->next_node = extrem;
		/*
		 * GSet references.
		 */
		extrem->prev_link.add(link);
		next_link.add(link);
	    }
	    link->reparent(new_graph, new_graph->Child());
	    link->map();
	    if (ex != NULL)
		link->_constraints();
	}
	
	if (!mappable())
	    map();

	_call_attach_node_postcond(extrem, &is_previous, linkclass, graph);
	_call_class_attach_node_postcond(extrem, &is_previous, linkclass,
					 graph);
	return GTRUE;
    } else
	return GFALSE;
}



/**
FUNCTION
    attachNodePrecond
DESCRIPTION
    
**/
void GoGraphNode::attachNodePrecond (GBool cbproc (void*, GoGraphNode*,
						  GoGraphNode*, GBool*,
						  GoMetaclass*, GoGraph*),
				     void* closure, GAddOrRemove add)
{
    _flags._has_attach_node_precond =
	AddOrRemoveCondition(goGraphNodeNattachNodePrecond, cbproc, closure,
			     add);
}


/**
FUNCTION
    attachNodePostcond
DESCRIPTION
    
**/
void GoGraphNode::attachNodePostcond (void cbproc (void*, GoGraphNode*,
						   GoGraphNode*, GBool*,
						   GoMetaclass*, GoGraph*),
				      void* closure, GAddOrRemove add)
{
    _flags._has_attach_node_postcond =
	AddOrRemoveCondition(goGraphNodeNattachNodePostcond, cbproc, closure,
			     add);
}



/**
FUNCTION
    int GoGraphNode::save();
DESCRIPTION
    ** static member, protected **
**/
int GoGraphNode::save(const GoGraphNode* n, GoStream* gostream)
{
    GoLink** tab = (GoLink**) n->prev_link.data();
    int maxi = n->prev_link.nelems();
    for (int i = 0; i < maxi; i++)
	if (!save_precond.get(tab[i]))
	    if (!gostream->saveObj(tab[i], tab[i]->getMetaclass()->classname(),
				   "previous link"))
		return GFALSE;
    if (!gostream->saveObj(NULL, NULL, "end of previous links"))
	return GFALSE;
	   
    tab = (GoLink**) n->next_link.data();
    maxi = n->next_link.nelems();
    for (i = 0; i < maxi; i++)
	if (!save_precond.get(tab[i]))
	    if (!gostream->saveObj(tab[i], tab[i]->getMetaclass()->classname(),
				   "next link"))
		return GFALSE;
    if (!gostream->saveObj(NULL, NULL, "end of next links"))
	return GFALSE;
	   
    return (GoNode::save(n, gostream));
}


/**
FUNCTION
    int GoGraphNode::readParams();
DESCRIPTION
    
**/
int GoGraphNode::readParams(GoGraphNode* n, GoStream* gostream)
{
    GoLink* link;
    const char* class_name;
    for (;;) {
	if (!gostream->restoreObj((void**)&link, &class_name))
	    return GFALSE;
	if (link)
	    n->prev_link.add(link);
	else
	    break;
    }

    for (;;) {
	if (!gostream->restoreObj((void**)&link, &class_name))
	    return GFALSE;
	if (link)
	    n->next_link.add(link);
	else
	    break;
    }

    return GoNode::readParams(n, gostream);
}


/**
FUNCTION
    int GoGraphNode::restore();
DESCRIPTION
    
**/
int GoGraphNode::restore(GoGraphNode** n, GoStream* gostream) {
    if (*n = GoGraphNode::create())
	return GoGraphNode::readParams(*n, gostream);
    return GFALSE;
}

