#ifdef REV_INFO
static char rcsid[] = "$Id: Link.cc,v 1.5 1992/12/10 16:17:17 smadja Exp $";
#endif
/*****************************************************************************
MODULE:  BULL Graphic Objects
FILE:    Link.cc
AUTHOR:  Aline Jacomet
DATE:    January 11th 1991

DESCRIPTION:
         Definition of the link object class.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include <Go/Link.h>
#include <Go/mLink.h>
#include <Go/GraphNode.h>
#include <Go/Tools/Error.h>
#include <Go/Graph.h>
#include <Go/GoStream.h>


/*
 * GCallback Types
 */
static char goLinkNattachLinkPrecond[] = "attachLinkPrecond";    
static char goLinkNattachLinkPostcond[] = "attachLinkPostcond";    
static char goLinkNanchoringPointModifyPrecond[] =
    "anchoringPointModifyPrecond";    
static char goLinkNanchoringPointModifyPostcond[] =
    "anchoringPointModifyPostcond";    


/*
 *------------
 * CLASS PART
 *------------
 */


static GoLinkMetaclass* link_metaclass = NULL;

/**
FUNCTION
    GoLink::metaclass()
DESCRIPTION
    
**/
GoLinkMetaclass* GoLink::metaclass() {
    if (link_metaclass == NULL) {
	link_metaclass = new GoLinkMetaclass;
	link_metaclass->superclass()->AddSubclass(link_metaclass);
      GoStream::bind(GoLink::metaclass()->classname(),
		     (int (*) (const void*, GoStream*)) GoLink::save,
		     (int (*) (void**, GoStream*)) GoLink::restore);
    }
    return link_metaclass;
}


/**
FUNCTION
    ~GoLinkMetaclass()
DESCRIPTION
    class destructor
**/
GoLinkMetaclass::~GoLinkMetaclass() {
    if (this == link_metaclass) {
        superclass()->RemoveSubclass(this);
	link_metaclass = NULL;
    }
}


/**
FUNCTION
    GoLinkMetaclass::GoLinkMetaclass()
DESCRIPTION
    class constructor
**/
 GoLinkMetaclass::GoLinkMetaclass() {
}


/**
FUNCTION
    const char* GoLinkMetaclass::m_classname() const
DESCRIPTION

**/
const char* GoLinkMetaclass::m_classname() const
{
    return "GoLink";
}


/**
FUNCTION
    GoMetaclass* GoLinkMetaclass::m_superclass() const
DESCRIPTION

**/
GoMetaclass* GoLinkMetaclass::m_superclass() const
{
    return GoGraphElement::metaclass();
}


/**
FUNCTION
    Go* GoLinkMetaclass::m_create();
DESCRIPTION

**/
Go* GoLinkMetaclass::m_create() const
{
    return (Go*)(new GoLink);
}


/**
FUNCTION
    GoLink::m_dup()
DESCRIPTION

**/
void GoLink::m_dup(const Go* src, Go* par, Go* prv, GBool b, GBool dochildren)
{
    GoGraphElement::m_dup(src, par, prv, b, dochildren);

    const GoLink* l_src = (const GoLink*) src;

    prev_point = l_src->prev_point;
    next_point = l_src->next_point;
    do_draw = l_src->do_draw;
}



/**
FUNCTION
    m_metaclass()
DESCRIPTION
    Returns the metaclass.
**/
GoMetaclass* GoLink::m_metaclass () const
{
    return GoLink::metaclass();
}


static GActionRec empty_action_array[] = { { NULL, NULL } };

/**
FUNCTION
    m_actions()
DESCRIPTION
    
**/
const GActionRec* GoLinkMetaclass::m_actions() const
{
    return empty_action_array;
}


/**
FUNCTION
    attachLinkPrecond
DESCRIPTION
    
**/
void GoLinkMetaclass::attachLinkPrecond (GBool cbproc (void*, GoLink*,
						      GoGraphNode*,
						      GoGraphNode*),
					 void* closure, GAddOrRemove add)
{
    _flags._has_attach_link_precond =
	AddOrRemoveCondition(goLinkNattachLinkPrecond, cbproc, closure, add);
}


/**
FUNCTION
    attachLinkPostcond
DESCRIPTION
    
**/
void GoLinkMetaclass::attachLinkPostcond (void cbproc (void*, GoLink*,
						       GoGraphNode*,
						       GoGraphNode*),
					  void* closure, GAddOrRemove add)
{
    _flags._has_attach_link_postcond =
	AddOrRemoveCondition(goLinkNattachLinkPostcond, cbproc, closure, add);
}


/**
FUNCTION
    anchoringPointModifyPrecond
DESCRIPTION
    
**/
void GoLinkMetaclass::anchoringPointModifyPrecond (void cbproc(void*, GoLink*,
							       GPoint*,
							       GPoint*),
						   void* closure,
						   GAddOrRemove add)
{
    _flags._has_point_modify_precond =
	AddOrRemoveCondition(goLinkNanchoringPointModifyPrecond, cbproc,
			     closure, add);
}


/**
FUNCTION
    anchoringPointModifyPostcond
DESCRIPTION
    
**/
void GoLinkMetaclass::anchoringPointModifyPostcond (void cbproc(void*, GoLink*,
								GPoint*,
								GPoint*),
						    void* closure,
						    GAddOrRemove add)
{
    _flags._has_point_modify_postcond =
	AddOrRemoveCondition(goLinkNanchoringPointModifyPostcond, cbproc,
			     closure, add);
}


/*
 *--------------
 * PRIVATE DATA
 *--------------
 */


/**
FUNCTION
   _call_class_attach_link_precond() 
DESCRIPTION
    
**/
GBool GoLink::_call_class_attach_link_precond (GoGraphNode* prev,
					       GoGraphNode* next) const
{
    // call conditions on all super-Metaclasses;
    for (GoMetaclass* mc = getMetaclass(); mc; mc = mc->superclass()) {
	if (((GoLinkMetaclass*) mc)->_flags._has_attach_link_precond &&
	    ! mc->CallPreconditions(goLinkNattachLinkPrecond, this, prev,
				    next))
	    return GFALSE;
	// stop when reaching GoLink Metaclass instance;
	if (mc == GoLink::metaclass())
	    return GTRUE;
    }
    return GTRUE;
}


/**
FUNCTION
   _call_class_attach_link_postcond() 
DESCRIPTION
    
**/
void GoLink::_call_class_attach_link_postcond (GoGraphNode* prev,
					       GoGraphNode* next)
{
    // call conditions on all super-Metaclasses;
    for (GoMetaclass* mc = getMetaclass(); mc; mc = mc->superclass()) {
	if (((GoLinkMetaclass*) mc)->_flags._has_attach_link_postcond)
	    CallPostconditions(goLinkNattachLinkPostcond, this, prev, next);
	// stop when reaching GoLink Metaclass instance;
	if (mc == GoLink::metaclass())
	    return;
    }
}


/**
FUNCTION
   _call_class_point_modify_precond() 
DESCRIPTION
    
**/
void GoLink::_call_class_point_modify_precond (GPoint* prev, GPoint* next)
{
    // call conditions on all super-Metaclasses;
    for (GoMetaclass* mc = getMetaclass(); mc; mc = mc->superclass()) {
	if (((GoLinkMetaclass*) mc)->_flags._has_point_modify_precond)
	    mc->CallPreconditions(goLinkNanchoringPointModifyPrecond, this,
				  prev, next);
	// stop when reaching GoLink Metaclass instance;
	if (mc == GoLink::metaclass())
	    return;
    }
}

/**
FUNCTION
   _call_class_point_modify_postcond() 
DESCRIPTION
    
**/
void GoLink::_call_class_point_modify_postcond (GPoint* prev, GPoint* next)
{
    // call conditions on all super-Metaclasses;
    for (GoMetaclass* mc = getMetaclass(); mc; mc = mc->superclass()) {
	if (((GoLinkMetaclass*) mc)->_flags._has_point_modify_postcond)
	    CallPostconditions(goLinkNanchoringPointModifyPostcond, this,
			       prev, next);
	// stop when reaching GoLink Metaclass instance;
	if (mc == GoLink::metaclass())
	    return;
    }
}




/**
FUNCTION
    _constraints()
DESCRIPTION
    Calculates the new connection points and sets the origin of the coordinate
    system at the center of the link's clip.
**/
void GoLink::_constraints()
{
    GBool is_mapped = mapped();
    GRegion tmp_reg;
    ((Go*)prev_node)->addClip(tmp_reg);
    GBox prev_clip = tmp_reg.bigbox();
    tmp_reg = GEmptyRegion();
    ((Go*)next_node)->addClip(tmp_reg);
    GBox next_clip = tmp_reg.bigbox();
    if (!prev_clip.empty() && !next_clip.empty()) {
	GPoint p, n;
	ushort anchor_type = GCENTER;
	if (parent())
	    anchor_type = ((GoGraphStruct*)parent())->anchoringType();
	if (anchor_type == GCENTER) {
	    p.x = (prev_clip.xmin + prev_clip.xmax) / 2;
	    p.y = (prev_clip.ymin + prev_clip.ymax) / 2;
	    n.x = (next_clip.xmin + next_clip.xmax) / 2;
	    n.y = (next_clip.ymin + next_clip.ymax) / 2;
	} else
	    if (anchor_type == GANGLE) {
		if (prev_clip.xmin > next_clip.xmin &&
		    prev_clip.ymin > next_clip.ymin) {
		    p.x = prev_clip.xmin;
		    p.y = prev_clip.ymin;
		    n.x = next_clip.xmax;
		    n.y = next_clip.ymax;
		} else
		if (prev_clip.xmin > next_clip.xmin &&
		    prev_clip.ymin < next_clip.ymin) {
		    p.x = prev_clip.xmin;
		    p.y = prev_clip.ymax;
		    n.x = next_clip.xmax;
		    n.y = next_clip.ymin;
		} else
		if (prev_clip.xmin < next_clip.xmin &&
		    prev_clip.ymin < next_clip.ymin) {
		    p.x = prev_clip.xmax;
		    p.y = prev_clip.ymax;
		    n.x = next_clip.xmin;
		    n.y = next_clip.ymin;
		} else
		if (prev_clip.ymin == next_clip.ymin &&
		    prev_clip.xmin < next_clip.xmin) {
		    p.x = prev_clip.xmax;
		    p.y = prev_clip.ymin;
		    n.x = next_clip.xmin;
		    n.y = next_clip.ymin;
		} else
		if (prev_clip.ymin == next_clip.ymin &&
		    prev_clip.xmin > next_clip.xmin) {
		    p.x = prev_clip.xmin;
		    p.y = prev_clip.ymin;
		    n.x = next_clip.xmax;
		    n.y = next_clip.ymin;
		} else
		if (prev_clip.xmin == next_clip.xmin &&
		    prev_clip.ymin < next_clip.ymin) {
		    p.x = prev_clip.xmin;
		    p.y = prev_clip.ymax;
		    n.x = next_clip.xmin;
		    n.y = next_clip.ymin;
		} else
		if (prev_clip.xmin == next_clip.xmin &&
		    prev_clip.ymin > next_clip.ymin) {
		    p.x = prev_clip.xmin;
		    p.y = prev_clip.ymin;
		    n.x = next_clip.xmin;
		    n.y = next_clip.ymax;
		} else {
		    p.x = prev_clip.xmax;
		    p.y = prev_clip.ymin;
		    n.x = next_clip.xmin;
		    n.y = next_clip.ymax;
		}
	    } else
		if (anchor_type == GMIDDLE_WIDTH) {
		    if (prev_clip.ymin > next_clip.ymin) {
			p.x = (prev_clip.xmin + prev_clip.xmax) / 2;
			p.y = prev_clip.ymin;
			n.x = (next_clip.xmin + next_clip.xmax) / 2;
			n.y = next_clip.ymax;
		    } else {
			p.x = (prev_clip.xmin + prev_clip.xmax) / 2;
			p.y = prev_clip.ymax;
			n.x = (next_clip.xmin + next_clip.xmax) / 2;
			n.y = next_clip.ymin;
		    }
		} else
		    if (anchor_type == GMIDDLE_HEIGHT) {
			if (prev_clip.xmin > next_clip.xmin) {
			    p.x = prev_clip.xmin;
			    p.y = (prev_clip.ymin + prev_clip.ymax)/2;
			    n.x = next_clip.xmax;
			    n.y = (next_clip.ymin + next_clip.ymax)/2;
			} else {
			    p.x = prev_clip.xmax;
			    p.y = (prev_clip.ymin + prev_clip.ymax)/2;
			    n.x = next_clip.xmin;
			    n.y = (next_clip.ymin + next_clip.ymax)/2;
			}
		    }
	
	if (p != prev_point || n != next_point) {
	    _call_point_modify_precond(&p, &n);
	    _call_class_point_modify_precond(&p, &n);
	    if (is_mapped)
		RedrawBackAndFore();
	    prev_point = p;
	    next_point = n;
	    
	    /*
	     * Coordinate system origin must be at the center of the link;
	     */
	    GCoordSys cs = Abscs();
	    cs.O.x = (prev_point.x + next_point.x) /2;
	    cs.O.y = (prev_point.y + next_point.y) /2;
	    setCoordsys(cs);
	    
	    do_draw = GTRUE;
	    MyClipHasChanged();
	    if (is_mapped)
		RedrawForeOnly();
	    _call_point_modify_postcond(&prev_point, &next_point);
	    _call_class_point_modify_postcond(&prev_point, &next_point);
	}
    } else {
	if (is_mapped)
	    RedrawBackAndFore();
	do_draw = GFALSE;
	MyClipHasChanged();
    }
}



/*
 *----------------
 * PROTECTED DATA
 *----------------
 */


/**
FUNCTION
   _call_attach_link_precond() 
DESCRIPTION
    
**/
GBool GoLink::_call_attach_link_precond (GoGraphNode* prev,
					GoGraphNode* next) const
{
    return (_flags._has_attach_link_precond ?
	    CallPreconditions(goLinkNattachLinkPrecond, prev, next) : GTRUE);
}


/**
FUNCTION
   _call_attach_link_postcond() 
DESCRIPTION
    
**/
void GoLink::_call_attach_link_postcond (GoGraphNode* prev, GoGraphNode* next)
{
    if (_flags._has_attach_link_postcond)
	CallPostconditions(goLinkNattachLinkPostcond, prev, next);
}



/**
FUNCTION
   _call_point_modify_precond() 
DESCRIPTION
    
**/
void GoLink::_call_point_modify_precond (GPoint* prev, GPoint* next)
{
    if (_flags._has_point_modify_precond)
	CallPreconditions(goLinkNanchoringPointModifyPrecond, prev, next);
}

/**
FUNCTION
   _call_point_modify_postcond() 
DESCRIPTION
    
**/
void GoLink::_call_point_modify_postcond (GPoint* prev, GPoint* next)
{
    if (_flags._has_point_modify_postcond)
	CallPostconditions(goLinkNanchoringPointModifyPostcond, prev, next);
}



/*
 *--------------
 * PUBLIC DATA
 *--------------
 */


/**
FUNCTION
    GoLink()
DESCRIPTION
    constructor
**/
 GoLink::GoLink() : prev_point(0,0), next_point(0,0)
{
    prev_node = NULL;
    next_node = NULL;
    do_draw = GFALSE;
}

    
/**
FUNCTION
    ~GoLink()
DESCRIPTION
    destructor
**/
 GoLink::~GoLink()
{
    prev_node = NULL;
    next_node = NULL;
}

    
/**
FUNCTION
    attachLink()
DESCRIPTION
    Attachs "this" to new extrem nodes.
**/
GBool GoLink::attachLink(GoGraphNode* prev, GoGraphNode* next)
{
   if (_call_attach_link_precond(prev, next) &&
       _call_class_attach_link_precond(prev, next)) {
       if (!prev || !next) {
	   GWarning("GoLink::attachLink: one of the specified nodes is NULL.");
	   return GFALSE;
       }
       if (!prev->getMetaclass()->isSubclass(GoGraphNode::metaclass()) || 
	   !next->getMetaclass()->isSubclass(GoGraphNode::metaclass())) {
	   GWarning("GoLink::attachLink: specified nodes are not instances of GoGraphNode class.");
		return GFALSE;
       }
	   
	   if (prev->parent() != next->parent()) {
	   GWarning("GoLink::attachLink: the two specified nodes do not belong to the same graph object.");
	   return GFALSE;
       }
       
       if (! prev->parent()->getMetaclass()->isSubclass(GoGraph::metaclass())) {
	   GWarning("GoLink::attachLink: the two specified nodes do not belong to a GoGraph object.");
	   return GFALSE;
       }

       GoGraph* new_graph = (GoGraph*)prev->parent();
       
        if (parent() != new_graph) {
	   /*
	    * Reparent "this" on the new_graph.
	    */
	   reparent(new_graph, new_graph->Child());
       }

       if (prev_node && next_node) {
	   /*
	    * Separate link of his graph
	    */
	   if (prev_node != prev) {
	       ((GoGraphNode*)(prev_node))->next_link.rem(this);
	       prev_node = NULL;
	   }
	   if (next_node != next) {
	       ((GoGraphNode*)(next_node))->prev_link.rem(this);
	       next_node = NULL;
	   }
       }

       /*
	* Attach link
	*/
       GoLink* l = this;
       if (!prev_node) {
	   prev_node = (GoNode*)prev;
	   prev->next_link.add(l);
       }
       if (!next_node) {
	   next_node = (GoNode*)next;
	   next->prev_link.add(l);
       }

       if (!mappable())
	   map();

       /*
	* GSet constraints, because link has extrem nodes now;
	*/
       _constraints();

       _call_attach_link_postcond(prev, next);
       _call_class_attach_link_postcond(prev, next);
       return GTRUE;
   } else
       return GFALSE;
   
}


/**
FUNCTION
    pen()
DESCRIPTION
    virtual function.
    Returns the pen used to draw the link.
**/
GPen& GoLink::pen() const
{
    return *(GPen*)NULL;
}



/**
FUNCTION
    attachLinkPrecond
DESCRIPTION
    
**/
void GoLink::attachLinkPrecond (GBool cbproc (void*, GoLink*, GoGraphNode*,
					     GoGraphNode*),
				void* closure, GAddOrRemove add)
{
    _flags._has_attach_link_precond =
	AddOrRemoveCondition(goLinkNattachLinkPrecond, cbproc, closure, add);
}


/**
FUNCTION
    attachLinkPostcond
DESCRIPTION
    
**/
void GoLink::attachLinkPostcond (void cbproc (void*, GoLink*, GoGraphNode*,
					      GoGraphNode*),
				 void* closure, GAddOrRemove add)
{
    _flags._has_attach_link_postcond =
	AddOrRemoveCondition(goLinkNattachLinkPostcond, cbproc, closure, add);
}


/**
FUNCTION
    anchoringPointModifyPrecond
DESCRIPTION
    
**/
void GoLink::anchoringPointModifyPrecond (void cbproc (void*, GoLink*, GPoint*,
							GPoint*),
					   void* closure, GAddOrRemove add)
{
    _flags._has_point_modify_precond =
	AddOrRemoveCondition(goLinkNanchoringPointModifyPrecond, cbproc,
			     closure, add);
}


/**
FUNCTION
    anchoringPointModifyPostcond
DESCRIPTION
    
**/
void GoLink::anchoringPointModifyPostcond (void cbproc (void*, GoLink*, GPoint*,
							GPoint*),
					   void* closure, GAddOrRemove add)
{
    _flags._has_point_modify_postcond =
	AddOrRemoveCondition(goLinkNanchoringPointModifyPostcond, cbproc,
			     closure, add);
}




/**
FUNCTION
    int GoLink::save();
DESCRIPTION
    ** static member, protected **
**/
int GoLink::save(const GoLink* link, GoStream* gostream) {
    if(! (gostream->writePoint(link->previousPoint(), "previous point") &&
	  gostream->writePoint(link->nextPoint(), "next point") &&
	  gostream->saveObj(link->previousNode(),
			    link->previousNode()->getMetaclass()->classname(),
			    "previous node") &&
	  gostream->saveObj(link->nextNode(),
			    link->nextNode()->getMetaclass()->classname(),
			    "next node") &&
	  gostream->writeBool(link->do_draw, "do draw")))
	return GFALSE;

    return (GoGraphElement::save(link, gostream));
}


/**
FUNCTION
    int GoLink::readParams();
DESCRIPTION
    
**/
int GoLink::readParams(GoLink* link, GoStream* gostream) {
    GPoint p;
    if (!gostream->readPoint(&p))
	return GFALSE;
    link->prev_point = p;
    if (!gostream->readPoint(&p))
	return GFALSE;
    link->next_point = p;

    GoNode* n;
    const char* class_name;
    if (!gostream->restoreObj((void**)&n, &class_name))
	return GFALSE;
    link->prev_node = n;
    if (!gostream->restoreObj((void**)&n, &class_name))
	return GFALSE;
    link->next_node = n;

    GBool b;
    if (!gostream->readBool(&b))
	return GFALSE;
    link->do_draw = b;

    return GoGraphElement::readParams(link, gostream);
}


/**
FUNCTION
    int GoLink::restore();
DESCRIPTION
    
**/
int GoLink::restore(GoLink** link, GoStream* gostream) {
    if (*link = GoLink::create())
	return GoLink::readParams(*link, gostream);
    return GFALSE;
}

