#ifdef REV_INFO
static char rcsid[] = "$Id: SimLink.cc,v 1.5 1992/12/10 16:23:13 smadja Exp $";
#endif
/*****************************************************************************
MODULE:  BULL Graphic Objects
FILE:    SimLink.cc
AUTHOR:  Aline Jacomet
DATE:    January 23th 1991

DESCRIPTION:
         Definition of the simple link object class.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include <assert.h>
#include <Go/SimLink.h>
#include <Go/mSimLink.h>
#include <Go/GoStream.h>
#include <Go/GoInterf.h>


#define GO_GEOM_PRECISION ((GNumber) .25)
#ifndef M_PI_2
#define M_PI_2 (PI/2.0)
#endif
    
/*
 *------------
 * CLASS PART
 *------------
 */


static GoSimpleLinkMetaclass* simpleLink_metaclass = NULL;

/**
FUNCTION
    GoSimpleLink::metaclass()
DESCRIPTION
    
**/
GoSimpleLinkMetaclass* GoSimpleLink::metaclass() {
    if (simpleLink_metaclass == NULL) {
	simpleLink_metaclass = new GoSimpleLinkMetaclass;
	simpleLink_metaclass->superclass()->AddSubclass(simpleLink_metaclass);
      GoStream::bind(GoSimpleLink::metaclass()->classname(),
		     (int (*) (const void*, GoStream*)) GoSimpleLink::save,
		     (int (*) (void**, GoStream*)) GoSimpleLink::restore);
    }
    return simpleLink_metaclass;
}


/**
FUNCTION
    ~GoSimpleLinkMetaclass()
DESCRIPTION
    
**/
GoSimpleLinkMetaclass::~GoSimpleLinkMetaclass() {
    if (this == simpleLink_metaclass) {
        superclass()->RemoveSubclass(this);
	simpleLink_metaclass = NULL;
    }
}


/**
FUNCTION
    GoSimpleLinkMetaclass::GoSimpleLinkMetaclass()
DESCRIPTION
    
**/
 GoSimpleLinkMetaclass::GoSimpleLinkMetaclass() {
}


/**
FUNCTION
    const char* GoSimpleLinkMetaclass::m_classname() const
DESCRIPTION

**/
const char* GoSimpleLinkMetaclass::m_classname() const
{
    return "GoSimpleLink";
}


/**
FUNCTION
    GoMetaclass* GoSimpleLinkMetaclass::m_superclass() const
DESCRIPTION

**/
GoMetaclass* GoSimpleLinkMetaclass::m_superclass() const
{
    return GoLink::metaclass();
}


/**
FUNCTION
    Go* GoSimpleLinkMetaclass::m_create();
DESCRIPTION

**/
Go* GoSimpleLinkMetaclass::m_create() const
{
    return (Go*)(new GoSimpleLink);
}


/**
FUNCTION
    GoSimpleLink::m_dup()
DESCRIPTION

**/
void GoSimpleLink::m_dup
    (const Go* src, Go* par, Go* prv, GBool b, GBool dochildren)
{
    GoLink::m_dup(src, par, prv, b, dochildren);

    GoSimpleLink* l_src = (GoSimpleLink*) src;

    pen().modify(l_src->pen().shPen());
}



/**
FUNCTION
    m_metaclass()
DESCRIPTION
    Returns the metaclass.
**/
GoMetaclass* GoSimpleLink::m_metaclass () const
{
    return GoSimpleLink::metaclass();
}



static GActionRec empty_action_array[] = { { NULL, NULL } };

/**
FUNCTION
    m_actions()
DESCRIPTION
    
**/
const GActionRec* GoSimpleLinkMetaclass::m_actions() const
{
    return empty_action_array;
}


/*
 *--------------
 * PRIVATE DATA
 *--------------
 */


/**
FUNCTION
    void GoSimpleLink::m_draw() const
DESCRIPTION
    
**/
void GoSimpleLink::m_draw(const GRegion& ) const {
    if (do_draw) {
	const GPen* p = &_pen;
	interf()->line((GPen*)p, GUnitCoordSys(), prev_point, next_point);
    }
}


/**
FUNCTION
    m_add_clip()
DESCRIPTION
    virtual private
**/
void GoSimpleLink::m_add_clip(GRegion& region) const {
    if (do_draw) {
	interf()->lineClips(sh_pen(), GUnitCoordSys(), prev_point, next_point,
			    &region, NULL, GMAX_QUALITY);
    }
}


/**
FUNCTION
    m_contains()
DESCRIPTION
    
**/
GBool GoSimpleLink::m_contains(const GPoint& absPoint, GNumber maxDist) const
{
    if (do_draw) {
	if (maxDist == 0)
	    maxDist = GO_GEOM_PRECISION;
	const GPen* p = &_pen;
	return interf()->lineContains((GPen*)p, GUnitCoordSys(), prev_point,
				      next_point, absPoint, maxDist,
				      GMAX_QUALITY);
    } else
	return GFALSE;
}



void simlink_modify_pen (void* closure, GPen* pen, ulong n)
{
    GoSimpleLink* l = (GoSimpleLink*) closure;
    l->modify_pen(pen, n);
}


/**
FUNCTION
    modify_pen() {
DESCRIPTION
    This method is a pen::modifyPostcond.
    It redraws the link with the new pen.
**/
void GoSimpleLink::modify_pen(GPen* pen, ulong ) {
    assert(pen == &_pen);
    GBool is_mapped = mapped();
    if (is_mapped)
	RedrawBackAndFore();
    init_sh_pen();
    MyClipHasChanged();
    if (is_mapped)
	RedrawForeOnly();
}



/*
 *--------------
 * PUBLIC DATA
 *--------------
 */


/**
FUNCTION
    GoSimpleLink()
DESCRIPTION
    constructor
**/
 GoSimpleLink::GoSimpleLink()
{
    _old_sh_pen = NULL;
    init_sh_pen();
    _pen.modifyPostcond(simlink_modify_pen, this);
}

    
/**
FUNCTION
    ~GoSimpleLink()
DESCRIPTION
    destructor
**/
 GoSimpleLink::~GoSimpleLink()
{
    if (_old_sh_pen)
	_old_sh_pen->destroy();
}


/**
FUNCTION
    sh_pen()
DESCRIPTION
    Returns the shared pen.
**/
const GShPen* GoSimpleLink::sh_pen() const
{
    return _old_sh_pen;
}



/**
FUNCTION
    int GoSimpleLink::save();
DESCRIPTION
    ** static member, protected **
**/
int GoSimpleLink::save(const GoSimpleLink* link, GoStream* gostream) {
    return(gostream->saveObj(link->pen().shPen(), "GShPen", "pen") &&
	 GoLink::save(link, gostream));
}


/**
FUNCTION
    int GoSimpleLink::readParams();
DESCRIPTION
    
**/
int GoSimpleLink::readParams(GoSimpleLink* link, GoStream* gostream) {
    GShPen* p = NULL;
    const char* class_name;
    if (!gostream->restoreObj((void**) &p, &class_name))
	return GFALSE;
    if (strcmp(class_name, "GShPen") != 0)
	GWarning("GoSimpleLink::readParams: can't restore pen.");
    else
	link->pen().modify(p);
    
    return GoLink::readParams(link, gostream);
}


/**
FUNCTION
    int GoSimpleLink::restore();
DESCRIPTION
    
**/
int GoSimpleLink::restore(GoSimpleLink** link, GoStream* gostream) {
    if (*link = GoSimpleLink::create())
	return GoSimpleLink::readParams(*link, gostream);
    return GFALSE;
}

