#ifdef REV_INFO
static char rcsid[] = "$Id: Tree.cc,v 1.4 1992/12/10 16:24:13 smadja Exp $";
#endif
/*****************************************************************************
MODULE:  BULL Graphic Objects
FILE:    Tree.cc
AUTHOR:  Aline Jacomet
DATE:    January 11th 1991

DESCRIPTION:
         Definition of the tree object class.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include <Go/Tree.h>
#include <Go/mTree.h>
#include <Go/GoStream.h>
#include <Go/Tools/Assoc.h>
#include <Go/Tools/Error.h>


extern GAssoc save_precond;

/*
 * GCallback Types
 */
static char goTreeNaddNodePrecond[] = "addNodePrecond";    
static char goTreeNaddNodePostcond[] = "addNodePostcond";    
static char goTreeNremoveNodePrecond[] = "removeNodePrecond";    
static char goTreeNremoveNodePostcond[] = "removeNodePostcond";    



/*
 *------------
 * CLASS PART
 *------------
 */



static GoTreeMetaclass* tree_metaclass = NULL;

/**
FUNCTION
    GoTree::metaclass()
DESCRIPTION
    
**/
GoTreeMetaclass* GoTree::metaclass() {
    if (tree_metaclass == NULL) {
	tree_metaclass = new GoTreeMetaclass;
	tree_metaclass->superclass()->AddSubclass(tree_metaclass);
      GoStream::bind(GoTree::metaclass()->classname(),
		     (int (*) (const void*, GoStream*)) GoTree::save,
		     (int (*) (void**, GoStream*)) GoTree::restore);
    }
    return tree_metaclass;
}


/**
FUNCTION
    ~GoTreeMetaclass()
DESCRIPTION
    
**/
GoTreeMetaclass::~GoTreeMetaclass() {
    if (this == tree_metaclass) {
        superclass()->RemoveSubclass(this);
	tree_metaclass = NULL;
    }
}


/**
FUNCTION
    GoTreeMetaclass::GoTreeMetaclass()
DESCRIPTION
    
**/
 GoTreeMetaclass::GoTreeMetaclass() {
}


/**
FUNCTION
    const char* GoTreeMetaclass::m_classname() const
DESCRIPTION
    Returns the class name.
**/
const char* GoTreeMetaclass::m_classname() const
{
    return "GoTree";
}


/**
FUNCTION
    GoMetaclass* GoTreeMetaclass::m_superclass() const
DESCRIPTION
    Returns the superclass.
**/
GoMetaclass* GoTreeMetaclass::m_superclass() const
{
    return GoGraphStruct::metaclass();
}


/**
FUNCTION
    Go* GoTreeMetaclass::m_create();
DESCRIPTION
    Creates a GoTree instance.
**/
Go* GoTreeMetaclass::m_create() const {
    return (Go*)(new GoTree);
}


/**
FUNCTION
    GoTree::m_dup()
DESCRIPTION
    Duplicates a GoTree.         
**/
void GoTree::m_dup
    (const Go* src, Go* parcopy, Go* prevcopy, GBool donext, GBool dochildren)
{
    GoGraphStruct::m_dup(src, parcopy, prevcopy, donext, dochildren);

    const GoTree* t_src = (const GoTree*) src;

    dist_x = t_src->dist_x;
    dist_y = t_src->dist_y;
}



/**
FUNCTION
    m_metaclass()
DESCRIPTION
    Returns the metaclass.
**/
GoMetaclass* GoTree::m_metaclass () const
{
    return GoTree::metaclass();
}


static void tree_set_dist_x (GoTree* t, GNumber d) { t->distX(d); }
static void tree_set_dist_y (GoTree* t, GNumber d) { t->distY(d); }
static void tree_set_alignment_type (GoTree* t, ushort u) {
    t->alignmentType(u);
}

static GActionRec goTree_action_array[] =
{ 
  { "setDistX(GNumber)", (GActionProc) tree_set_dist_x },
  { "setDistY(GNumber)", (GActionProc) tree_set_dist_y },
  { "setAlignmentType(ushort)",	(GActionProc) tree_set_alignment_type },
  { NULL, NULL }
};


/**
FUNCTION
    m_actions()
DESCRIPTION
    Actions for user interaction.
**/
const GActionRec* GoTreeMetaclass::m_actions() const
{
    return goTree_action_array;
}


/**
FUNCTION
   _call_class_add_node_precond() 
DESCRIPTION
    Calls preconditions on addNode.
**/
inline GBool GoTree::_call_class_add_node_precond (GoTreeNode* node,
						   GoMetaclass* linkclass,
						   GoTreeNode* brother) const
{
    // call conditions on all super-Metaclasses;
    for (GoMetaclass* mc = getMetaclass(); mc; mc = mc->superclass()) {
	if (((GoTreeMetaclass*) mc)->_flags._has_add_node_precond &&
	    ! mc->CallPreconditions(goTreeNaddNodePrecond, this, node,
				    linkclass, brother))
	    return GFALSE;
	// stop when reaching GoTree Metaclass instance;
	if (mc == GoTree::metaclass())
	    return GTRUE;
    }
    return GTRUE;
}


/**
FUNCTION
    _call_class_add_node_postcond()
DESCRIPTION
    Calls postconditions on addNode.
**/
inline void GoTree::_call_class_add_node_postcond (GoTreeNode* node,
						   GoMetaclass* linkclass,
						   GoTreeNode* brother)
{
    // call conditions on all super-Metaclasses;
    for (GoMetaclass* mc = getMetaclass(); mc; mc = mc->superclass()) {
	if (((GoTreeMetaclass*) mc)->_flags._has_add_node_postcond)
	    CallPostconditions(goTreeNaddNodePostcond, this, node, linkclass,
			       brother);
	// stop when reaching GoTree Metaclass instance;
	if (mc == GoTree::metaclass())
	    return;
    }
}


/**
FUNCTION
   _call_class_remove_node_precond() 
DESCRIPTION
    Calls preconditions on removeNode.
**/
inline GBool GoTree::_call_class_remove_node_precond (GoTreeNode* node) const
{
    // call conditions on all super-Metaclasses;
    for (GoMetaclass* mc = getMetaclass(); mc; mc = mc->superclass()) {
	if (((GoTreeMetaclass*) mc)->_flags._has_remove_node_precond &&
	    ! mc->CallPreconditions(goTreeNremoveNodePrecond, this, node))
	    return GFALSE;
	// stop when reaching GoTree Metaclass instance;
	if (mc == GoTree::metaclass())
	    return GTRUE;
    }
    return GTRUE;
}


/**
FUNCTION
    _call_class_remove_node_postcond()
DESCRIPTION
    Calls postconditions on removeNode.
**/
inline void GoTree::_call_class_remove_node_postcond (GoTreeNode* node)
{
    // call conditions on all super-Metaclasses;
    for (GoMetaclass* mc = getMetaclass(); mc; mc = mc->superclass()) {
	if (((GoTreeMetaclass*) mc)->_flags._has_remove_node_postcond)
	    CallPostconditions(goTreeNremoveNodePostcond, this, node);
	// stop when reaching GoTree Metaclass instance;
	if (mc == GoTree::metaclass())
	    return;
    }
}


/**
FUNCTION
    addNodePrecond
DESCRIPTION
    Adds or removes preconditions on addNode().
**/
void GoTreeMetaclass::addNodePrecond (GBool cbproc (void*, const GoTree*,
						   GoTreeNode*, GoMetaclass*,
						   GoTreeNode*),
				      void* closure, GAddOrRemove add)
{
    _flags._has_add_node_precond =
	AddOrRemoveCondition(goTreeNaddNodePrecond, cbproc, closure, add);
}


/**
FUNCTION
    addNodePostcond
DESCRIPTION
    Adds or removes postconditions on addNode().
**/
void GoTreeMetaclass::addNodePostcond (void cbproc (void*, GoTree*, GoTreeNode*,
						    GoMetaclass*, GoTreeNode*),
				       void* closure, GAddOrRemove add)
{
    _flags._has_add_node_postcond =
	AddOrRemoveCondition(goTreeNaddNodePostcond, cbproc, closure, add);
}


/**
FUNCTION
    removeNodePrecond
DESCRIPTION
    Adds or removes preconditions on removeNode().
**/
void GoTreeMetaclass::removeNodePrecond (GBool cbproc (void*, const GoTree*,
						      GoTreeNode*),
					 void* closure, GAddOrRemove add)
{
    _flags._has_remove_node_precond =
	AddOrRemoveCondition(goTreeNremoveNodePrecond, cbproc, closure, add);
}


/**
FUNCTION
    removeNodePostcond
DESCRIPTION
    Adds or removes postconditions on removeNode().
**/
void GoTreeMetaclass::removeNodePostcond (void cbproc (void*, GoTree*,
						       GoTreeNode*),
					  void* closure, GAddOrRemove add)
{
    _flags._has_remove_node_postcond =
	AddOrRemoveCondition(goTreeNremoveNodePostcond, cbproc, closure, add);
}




/*
 *--------------
 * PRIVATE DATA
 *--------------
 */



/**
FUNCTION
    remove()
DESCRIPTION
    Deletes node and all its children.
**/
void GoTree::remove(GoTreeNode* node)
{
    while (node->child_node) 
	remove(node->child_node);
    if (node->parent_link) {
	node->parent_link->in_destroy = GTRUE;
	node->parent_link->destroy();
    }
    node->in_destroy = GTRUE;
    node->destroy();
}



/**
FUNCTION
    reparent_subtree()
DESCRIPTION
    Reparents node and all its children on NULL.
**/
void GoTree::reparent_subtree(GoTreeNode* node, GoTree* tree)
{
    for(GoTreeNode* ch = node->childNode(); ch; ch = ch->nextNode())
	reparent_subtree(ch, tree);
    
    if (node->parent_link) {
	node->parent_link->in_destroy = GTRUE;
	node->parent_link->reparent(tree);
	node->parent_link->in_destroy = GFALSE;
    }
    node->in_destroy = GTRUE;
    node->reparent(tree);
    node->in_destroy = GFALSE;
}


/**
FUNCTION
   _call_add_node_precond() 
DESCRIPTION
    Calls preconditions on addNode.
**/
inline GBool GoTree::_call_add_node_precond (GoTreeNode* node,
					    GoMetaclass* linkclass,
					    GoTreeNode* brother) const
{
    return (_flags._has_add_node_precond ?
	    CallPreconditions(goTreeNaddNodePrecond, node, linkclass,
			      brother) : GTRUE);
}


/**
FUNCTION
    _call_add_node_postcond()
DESCRIPTION
    Calls postconditions on addNode.
**/
inline void GoTree::_call_add_node_postcond (GoTreeNode* node,
					     GoMetaclass* linkclass,
					     GoTreeNode* brother)
{
    if (_flags._has_add_node_postcond)
	CallPostconditions(goTreeNaddNodePostcond, node, linkclass, brother);
}


/**
FUNCTION
   _call_remove_node_precond() 
DESCRIPTION
    Calls preconditions on removeNode.
**/
inline GBool GoTree::_call_remove_node_precond (GoTreeNode* node) const
{
    return (_flags._has_remove_node_precond ?
	    CallPreconditions(goTreeNremoveNodePrecond, node) :
	    GTRUE);
}


/**
FUNCTION
    _call_remove_node_postcond()
DESCRIPTION
    Calls postconditions on removeNode.
**/
inline void GoTree::_call_remove_node_postcond (GoTreeNode* node)
{
    if (_flags._has_remove_node_postcond)
	CallPostconditions(goTreeNremoveNodePostcond, node);
}


/*
 *--------------
 * PUBLIC DATA
 *--------------
 */


/**
FUNCTION
    GoTree()
DESCRIPTION
    constructor
**/
 GoTree::GoTree()
{
    position_type = GUP_TO_BOTTOM;
    alignment_type = GON_BORDER;
    dist_x = 10;
    dist_y = 10;
}


/**
FUNCTION
    ~GoTree()
DESCRIPTION
    destructor
**/
 GoTree::~GoTree()
{
}


/**
FUNCTION
    distX()
DESCRIPTION
    
**/
GBool GoTree::distX(GNumber d)
{
    if (d >= 0) {
	dist_x = d;
	return GTRUE;
    } else
	return GFALSE;
}


/**
FUNCTION
    distY()
DESCRIPTION
    
**/
GBool GoTree::distY(GNumber d)
{
    if (d >= 0) {
	dist_y = d;
	return GTRUE;
    } else
	return GFALSE;
}

    
/**
FUNCTION
    alignmentType()
DESCRIPTION
    
**/
GBool GoTree::alignmentType(ushort t)
{
    if (t == GON_CENTER || t == GON_BORDER) {
	alignment_type = t;
	return GTRUE;
    } else
	return GFALSE;
}

    
/**
FUNCTION
    addNode()
DESCRIPTION
    Adds a node at the tree.
**/
GoTreeNode* GoTree::addNode(GoTreeNode* parent, GoMetaclass* type,
			    GoTreeNode* brother){
    if (_call_add_node_precond(parent, type, brother) &&
	_call_class_add_node_precond(parent, type, brother)) {

	/*
	 * Test parameters
	 */

	/*
	 * type
	 */
	if (!type)
	    type = link_type;
	else
	    if (!type->isSubclass(GoLink::metaclass())) {
		GWarning("GoTree::addNode: the specified type of link is not a GoLink or a subclass of GoLink.");
		return NULL;
	    }
	
	if (!type->isSubclass(GoLink::metaclass())) {
	    GWarning("GoTree::addNode: the specified GoMetaclass is not a GoLinkClass or subclass of GoLinkClass.");
	    return NULL;
	}
	/*
	 * brother
	 */
	if (brother) {
	    if (!brother->getMetaclass()->isSubclass(GoTreeNode::metaclass())) {
		GWarning("GoTree::addNode: the specified brother is not an instance of GoTreeNode class.");
		return NULL;
	    }
	    if (brother->parentNode() != parent) {
		GWarning("GoTree::addNode: the specified brother's parent node is not the specified parent node.");
		return NULL;
	    }
	}
	

	/*
	 * Create the new node
	 */
	GoTreeNode* node;
	if (! parent) {
	    if (first_node) {
		GWarning("GoTree::addNode: the tree object already has a root node.");
		return NULL;
	    } else {
		node = (GoTreeNode*)GoTreeNode::create();
		first_node = node;
		node->clipPostcond(graphstr_set_coordsys, this);
	    }
	} else {
	    node = (GoTreeNode*)GoTreeNode::create();
	    node->parent_node = parent;
	    if (brother) {
		if (brother == parent->child_node) {
		    parent->child_node = node;
		}
		node->prev_node = brother->prev_node;
		node->next_node = brother;
		brother->prev_node = node;
		if (node->prev_node)
		    node->prev_node->next_node = node;
	    } else
		if (parent->child_node) {
		    /*
		     * It is not the first child of "parent"
		     */
		    GoTreeNode* child = parent->child_node;
		    while (child->next_node)
			child = child->next_node;
		    child->next_node = node;
		    node->prev_node = child;
		} else {
		    /*
		     * It is the first child of "parent"
		     */
		    parent->child_node = node;
		}
	}
	node->reparent(this);
	if (!parent || (parent && parent->isOpened()))
	    node->map();
	
	if (parent) {
	    /*
	     * Create the new link 
	     */
	    GoLink* link = (GoLink*)Go::Create(type);
	    if (!link) {
		node->in_destroy = GTRUE;
		node->destroy();
		return NULL;
	    }
	    link->reparent(this, Child());
	    link->prev_node = (GoNode*)parent;
	    link->next_node = (GoNode*)node;
	    node->parent_link = (GoLink*)link;
	    if (parent->isOpened())
		link->map();
	}
	_call_add_node_postcond(parent, type, brother);
	_call_class_add_node_postcond(parent, type, brother);
	return node;
    } else
	return NULL;
    
}


/**
FUNCTION
    removeNode()
DESCRIPTION
    Removes a node and its children.
**/
GBool GoTree::removeNode(GoTreeNode* node)
{
    if (node && node->parent() == this) {
	if (_call_remove_node_precond(node) &&
	    _call_class_remove_node_precond(node)) {
	    if (firstNode() == node) {
		node->clipPostcond(graphstr_set_coordsys, this, GREMOVE);
		first_node = NULL;
	    }
	    remove(node);
	    
	    _call_remove_node_postcond(node);
	    _call_class_remove_node_postcond(node);
	    return GTRUE;
	} else
	    return GFALSE;
    } else
	return GFALSE;
}



/**
FUNCTION
    addNodePrecond
DESCRIPTION
    Adds or removes preconditions on addNode().
**/
void GoTree::addNodePrecond (GBool cbproc (void*, GoTree*, GoTreeNode*,
					  GoMetaclass*, GoTreeNode*),
			     void* closure, GAddOrRemove add)
{
    _flags._has_add_node_precond =
	AddOrRemoveCondition(goTreeNaddNodePrecond, cbproc, closure, add);
}


/**
FUNCTION
    addNodePostcond
DESCRIPTION
    Adds or removes postconditions on addNode().
**/
void GoTree::addNodePostcond (void cbproc (void*, GoTree*, GoTreeNode*,
					   GoMetaclass*, GoTreeNode*),
			      void* closure, GAddOrRemove add)
{
    _flags._has_add_node_postcond =
	AddOrRemoveCondition(goTreeNaddNodePostcond, cbproc, closure, add);
}


/**
FUNCTION
    removeNodePrecond
DESCRIPTION
   Adds or removes preconditions on removeNode().  
**/
void GoTree::removeNodePrecond (GBool cbproc (void*, GoTree*, GoTreeNode*),
				void* closure, GAddOrRemove add)
{
    _flags._has_remove_node_precond =
	AddOrRemoveCondition(goTreeNremoveNodePrecond, cbproc, closure, add);
}


/**
FUNCTION
    removeNodePostcond
DESCRIPTION
   Adds or removes postconditions on removeNode(). 
**/
void GoTree::removeNodePostcond (void cbproc (void*, GoTree*, GoTreeNode*),
				 void* closure, GAddOrRemove add)
{
    _flags._has_remove_node_postcond =
	AddOrRemoveCondition(goTreeNremoveNodePostcond, cbproc, closure, add);
}




/**
FUNCTION
    int save_tree()
DESCRIPTION
    
**/
static int save_tree(GoTreeNode* n, GoStream* gostream)
{
    for(GoTreeNode* ch = n->childNode(); ch; ch = ch->nextNode()) {
	if (ch->parentLink()->callSavePrecond() &&
	    ch->callSavePrecond())
	    save_tree(ch, gostream);
	else
	    save_precond.set(ch, ch);
    }
    
    if (n->parentLink())
	if (!gostream->saveObj(n->parentLink(),
			       n->parentLink()->getMetaclass()->classname(),
			       "child"))
	    return GFALSE;
    if (!gostream->saveObj(n, n->getMetaclass()->classname(), "child"))
	return GFALSE;
    return GTRUE;
}


/**
FUNCTION
    int GoTree::save();
DESCRIPTION
    ** static member, protected **
**/
int GoTree::save(const GoTree* tree, GoStream* gostream)
{
    save_precond.reset();
    
    if (tree->firstNode()->callSavePrecond())
	if (! save_tree((GoTreeNode*)tree->firstNode(), gostream))
	return GFALSE;
    
    for (Go* ch = (Go*)tree->Child(); ch; ch = ch->Next())
	if (! (ch->getMetaclass()->isSubclass(GoNode::metaclass()) ||
	       ch->getMetaclass()->isSubclass(GoLink::metaclass())))
	    if (ch->callSavePrecond())
		if (!gostream->saveObj(ch, ch->getMetaclass()->classname(),
				       "child"))
		    return GFALSE;
    
    if (!gostream->saveObj(NULL, NULL, "end of children"))
	return GFALSE;
    
    if (tree->firstNode()->callSavePrecond()) {
	if (!gostream->saveObj(tree->firstNode(),
			       tree->firstNode()->getMetaclass()->classname(),
			       "first node"))
	    return GFALSE;
    } else
	if (!gostream->saveObj(NULL, NULL, "first node"))
	    return GFALSE;
    
    return(gostream->writeNumber(tree->distX(), "dist x") &&
	   gostream->writeNumber(tree->distY(), "dist y") &&
	 GoGraphStruct::save(tree, gostream));
}


/**
FUNCTION
    int GoTree::readParams();
DESCRIPTION
    
**/
int GoTree::readParams(GoTree* tree, GoStream* gostream) {
    const char* class_name;
    for (;;) {
	Go* next_child;
	if (!gostream->restoreObj((void**)&next_child, &class_name))
	    return GFALSE;
	if (next_child)
	    next_child->DoReparent(tree);
	else
	    break;
    }

    GoNode* first;
    if (!gostream->restoreObj((void**)&first, &class_name))
	return GFALSE;
    if (first)
	if (strcmp(class_name, GoTreeNode::metaclass()->classname()) != 0) {
	    GWarning("GoTree::readParams: can't restore first node.");
	    first = NULL;
	}
    tree->first_node = first;
    
    GNumber n;
    if (!gostream->readNumber(&n))
	return GFALSE;
    tree->dist_x = n;
    if (!gostream->readNumber(&n))
	return GFALSE;
    tree->dist_y = n;
    return GoGraphStruct::readParams(tree, gostream);
}


/**
FUNCTION
    int GoTree::restore();
DESCRIPTION
    
**/
int GoTree::restore(GoTree** tree, GoStream* gostream) {
    if (*tree = GoTree::create())
	return GoTree::readParams(*tree, gostream);
    return GFALSE;
}
