#ifdef REV_INFO
static char rcsid[] = "$Id: TreePlct.cc,v 1.1.1.1 1992/03/18 10:23:17 uep Exp $";
#endif
/*****************************************************************************
MODULE:  BULL Graphic Objects
FILE:    Tree.cc
AUTHOR:  Aline Jacomet and Gabriel Jureidini
DATE:    January 11th 1991

DESCRIPTION:
         Definition of the tree object class.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include <Go/Tree.h>
#include <Go/mTree.h>
#include <Go/GoInterf.h>


#define max(a,b)  (((a) > (b)) ? (a) : (b))

#define MAX_LEVEL 60


static int num_node;            // for placement algorythms;



/**
FUNCTION
    left_right_placement()
DESCRIPTION
    Placement from left to right.
**/
void GoTree::left_right_placement(GoNode* node, const GPoint& point)
{
    GNumber y_final = 0.;
    GNumber y_first = 0.;
    GNumber place, modifier[MAX_LEVEL], next_pos[MAX_LEVEL];
    GNumber width_level[MAX_LEVEL];
    int act_width = 1;
    int j = -1;
    GBool son_not_marked = GTRUE;

    GPoint * center = new GPoint[nb_node+1];  
    for (short i = 0; i < MAX_LEVEL; ++i) {
	next_pos[i] = dist_y;
	modifier[i] = 0.;
	width_level[i] = 0.;
    }
    num_node = 0;
    find_fields_h ((GoTreeNode*)node, width_level, modifier, next_pos,
		   &y_final, &place, &act_width, &j, &y_first,
		   &son_not_marked, center);
    num_node = 0;
    gmnhg((GoTreeNode*)node, width_level, point, center);
    delete center;
}


/**
FUNCTION
    find_fields_h()
**/
void GoTree::find_fields_h(GoTreeNode* node, GNumber* width_level,
			   GNumber* modifier, GNumber* next_pos, GNumber* y_final,
			   GNumber* place, int *act_width, int* jp,
			   GNumber* y_first, GBool* son_not_marked,
			   GPoint* center) 
{
    ushort nb_n = ++num_node;
    if (! node->_marked) {
	node->_marked = GTRUE;
	*son_not_marked = GFALSE;
	++*jp;
	GRegion tmp_reg;
	node->addClip(tmp_reg);
	GBox clip = tmp_reg.bigbox();
	if (width_level[*act_width] == 0.0) 
	    width_level[*act_width] = clip.width();
	else
	    width_level[*act_width] = max (width_level[*act_width],
					   clip.width());
	if (node->childNode() && node->_open) {
	    GNumber y_first;
	    int j = -1;
	    GBool son_not_marked = GTRUE;
	    ++*act_width;
	    for (GoTreeNode* n = node->childNode(); n; n = n->nextNode())
		find_fields_h (n , width_level, modifier, next_pos, y_final,
			       place, act_width, &j, &y_first,
			       &son_not_marked, center);
	    --*act_width;
	    if (son_not_marked)
		*place = (clip.height() / 2) + next_pos[*act_width];
	    else {
		*place = (y_first +*y_final) / 2;
		modifier[*act_width] = max(modifier[*act_width],
					   (clip.height() / 2) +
					   next_pos[*act_width] -
					   *place);
		*place = *place + modifier[*act_width];
	    }
	}
	if (!node->_open || !node->childNode()) 
	    *place = (clip.height() / 2) + next_pos[*act_width];
	center[nb_n].x = 0.0;   
	center[nb_n].y = *place;

	node->_modif = modifier[*act_width];
	if (! *jp)
	    *y_first = *place;
	*y_final = *place;
	next_pos[*act_width] = *place + dist_y + (clip.height() / 2);
    }
}


/**
FUNCTION
    gmnhg()
**/
void GoTree::gmnhg(GoTreeNode* node, GNumber* vector_width, const GPoint& point,
		  GPoint* center)    
{
    GNumber modifier_sum = 0.;
    GNumber total_width = dist_x;
    GNumber delta_x = 0.;
    GNumber delta_y = 0.;
    int act_width = 1;
    if (&point) {
	if (alignment_type == GON_CENTER)
	    delta_x = point.x - (dist_x + (vector_width[act_width] / 2));
	else {
	    GRegion tmp_reg;
	    node->addClip(tmp_reg);
	    GBox clip = tmp_reg.bigbox();
	    delta_x = point.x - dist_x - (clip.width() / 2);
	}
	delta_y = point.y - center[1].y;
    }
    modif_node1_h (node, &modifier_sum, delta_x, delta_y, &total_width,
		   &act_width, vector_width, center);
}



/**
FUNCTION
    modif_node1_h()
**/
void GoTree::modif_node1_h(GoTreeNode* node, GNumber* modifier_sum,
			   GNumber delta_x, GNumber delta_y,
			   GNumber* total_width, int* act_width,
			   GNumber* vector_width,
			   GPoint* center) 
{
    ushort nb_n = ++num_node;
    if (node->_marked) {
	node->_marked = GFALSE;
	if (alignment_type == GON_CENTER)
	    node->move(GPoint(*total_width + (vector_width[*act_width] / 2) +
			     delta_x,
			     center[nb_n].y + *modifier_sum + delta_y));
	else {
	    GRegion tmp_reg;
	    node->addClip(tmp_reg);
	    GBox clip = tmp_reg.bigbox();
	    node->move(GPoint(*total_width + delta_x + (clip.width() / 2),
			     center[nb_n].y + *modifier_sum + delta_y));
	}
	if (node->childNode() && node->_open) {
	    *total_width += dist_x + vector_width[*act_width];
	    ++*act_width;
	    if (node->_modif)
		*modifier_sum += node->_modif;
	    for (GoTreeNode* n = node->childNode(); n; n = n->nextNode()) 
		modif_node1_h (n, modifier_sum, delta_x, delta_y, total_width,
			       act_width, vector_width, center);
	    --*act_width;
	    *total_width -= (dist_x + vector_width[*act_width]);
	    if (node->_modif) {
		*modifier_sum -= node->_modif;
		node->_modif = 0.;
	    }
	}
    }
}



/**
FUNCTION
    right_left_placement()
DESCRIPTION
    Placement from right to left.
**/
void GoTree::right_left_placement(GoNode* node, const GPoint& point)
{
    GNumber y_final = 0.;
    GNumber y_first = 0.;
    GNumber place, modifier[MAX_LEVEL], next_pos[MAX_LEVEL];
    GNumber width_level[MAX_LEVEL];
    int act_width = 1;
    int j = -1;
    GBool son_not_marked = GTRUE;

    GPoint * center = new GPoint[nb_node+1];  
    for (short i = 0; i < MAX_LEVEL; ++i) {
	next_pos[i] = dist_y;
	modifier[i] = 0.;
	width_level[i] = 0.;
    }
    num_node = 0;
    find_fields_h2 ((GoTreeNode*)node, width_level, modifier, next_pos,
		   &y_final, &place, &act_width, &j, &y_first,
		   &son_not_marked, center);
    num_node = 0;
    gmnhg2((GoTreeNode*)node, width_level, point, center);
    delete center;
}


/**
FUNCTION
    find_fields_h2()
**/
void GoTree::find_fields_h2(GoTreeNode* node, GNumber* width_level,
			   GNumber* modifier, GNumber* next_pos, GNumber* y_final,
			   GNumber* place, int *act_width, int* jp,
			   GNumber* y_first, GBool* son_not_marked,
			   GPoint* center) 
{
    ushort nb_n = ++num_node;
    GNumber win_height = interf()->height();
    if (! node->_marked) {
	node->_marked = GTRUE;
	*son_not_marked = GFALSE;
	++*jp;
	GRegion tmp_reg;
	node->addClip(tmp_reg);
	GBox clip = tmp_reg.bigbox();
	if (width_level[*act_width] == 0.0) 
	    width_level[*act_width] = clip.width();
	else
	    width_level[*act_width] = max (width_level[*act_width],
					   clip.width());
	if (node->childNode() && node->_open) {
	    GNumber y_first;
	    int j = -1;
	    GBool son_not_marked = GTRUE;
	    ++*act_width;
	    for (GoTreeNode* n = node->childNode(); n; n = n->nextNode())
		find_fields_h2 (n , width_level, modifier, next_pos, y_final,
			       place, act_width, &j, &y_first,
			       &son_not_marked, center);
	    --*act_width;
	    if (son_not_marked)
		*place = win_height - ((clip.height() / 2) +
				      next_pos[*act_width]);
	    else {
		*place = (y_first +*y_final) / 2;
		modifier[*act_width] = max(modifier[*act_width],
					   *place - (win_height -
						     (clip.height() / 2) -
						     next_pos[*act_width]));
		*place -= modifier[*act_width];
	    }
	}
	if (!node->_open || !node->childNode()) 
	    *place = win_height - ((clip.height() / 2) + next_pos[*act_width]);
	center[nb_n].x = 0.0;   
	center[nb_n].y = *place;

	node->_modif = modifier[*act_width];
	if (! *jp)
	    *y_first = *place;
	*y_final = *place;
	next_pos[*act_width] = win_height - (*place - dist_y -
					     (clip.height() / 2));
    }
}


/**
FUNCTION
    gmnhg2()
**/
void GoTree::gmnhg2(GoTreeNode* node, GNumber* vector_width, const GPoint& point,
		    GPoint* center)    
{
    GNumber modifier_sum = 0.;
    GNumber total_width = dist_x;
    GNumber delta_x = 0.;
    GNumber delta_y = 0.;
    int act_width = 1;
    if (&point) {
	if (alignment_type == GON_CENTER)
	    delta_x = point.x - (dist_x + (vector_width[act_width] / 2));
	else {
	    GRegion tmp_reg;
	    node->addClip(tmp_reg);
	    GBox clip = tmp_reg.bigbox();
	    delta_x = point.x - dist_x - (clip.width() / 2);
	}
	delta_y = point.y - center[1].y;
    }
    modif_node2_h (node, &modifier_sum, delta_x, delta_y, &total_width,
		   &act_width, vector_width, center);
}



/**
FUNCTION
    modif_node2_h()
**/
void GoTree::modif_node2_h(GoTreeNode* node, GNumber* modifier_sum,
			   GNumber delta_x, GNumber delta_y,
			   GNumber* total_width, int* act_width,
			   GNumber* vector_width,
			   GPoint* center) 
{
    ushort nb_n = ++num_node;
    GNumber win_width = interf()->width();
    if (node->_marked) {
	node->_marked = GFALSE;
	if (alignment_type == GON_CENTER)
	    node->move(GPoint(win_width - (*total_width +
					  (vector_width[*act_width] / 2) +
					  delta_x),
			     center[nb_n].y - *modifier_sum - delta_y));
	else {
	    GRegion tmp_reg;
	    node->addClip(tmp_reg);
	    GBox clip = tmp_reg.bigbox();
	    node->move(GPoint(win_width - (*total_width + delta_x +
					  (clip.width() / 2)),
			     center[nb_n].y - *modifier_sum - delta_y));
	}
	if (node->childNode() && node->_open) {
	    *total_width += dist_x + vector_width[*act_width];
	    ++*act_width;
	    if (node->_modif)
		*modifier_sum += node->_modif;
	    for (GoTreeNode* n = node->childNode(); n; n = n->nextNode()) 
		modif_node2_h (n, modifier_sum, delta_x, delta_y, total_width,
			       act_width, vector_width, center);
	    --*act_width;
	    *total_width -= (dist_x + vector_width[*act_width]);
	    if (node->_modif) {
		*modifier_sum -= node->_modif;
		node->_modif = 0.;
	    }
	}
    }
}


/**
FUNCTION
    up_bottom_placement()
DESCRIPTION
    Placement from up to bottom.
**/
void GoTree::up_bottom_placement(GoNode* node, const GPoint& point)
{
    GNumber x_final = 0.;
    GNumber x_first = 0.;
    GNumber place, modifier[MAX_LEVEL], next_pos[MAX_LEVEL];
    GNumber height_level[MAX_LEVEL];
    GPoint * center = new GPoint[nb_node+1];           
    int act_height = 1;
    int j = -1;
    GBool son_not_marked = GTRUE;

    for (int i = 0; i < MAX_LEVEL; i++) {
	next_pos[i] = dist_x;
	modifier[i] = 0.;
	height_level[i] = 0.;
    }
    num_node = 0;
    find_fields_v ((GoTreeNode*)node, height_level, modifier, next_pos,
		   &x_final, &place, &act_height, &j, &x_first,
		   &son_not_marked, center);
    num_node = 0;
    gmng((GoTreeNode*)node, height_level, point, center);
    delete center;
}


/**
FUNCTION
    find_fields_v()
**/
void GoTree::find_fields_v(GoTreeNode* node, GNumber* height_level,
			   GNumber* modifier, GNumber* next_pos, GNumber* x_final,
			   GNumber* place, int *act_height, int* jp,
			   GNumber* x_first, GBool* son_not_marked,
			   GPoint* center) 
{
    ushort nb_n = ++num_node;
    if (!node->_marked) {
	node->_marked = GTRUE;
	*son_not_marked = GFALSE;
	(*jp)++;
	GRegion tmp_reg;
	node->addClip(tmp_reg);
	GBox clip = tmp_reg.bigbox();
	if (height_level[*act_height] == 0.0)
	    height_level[*act_height] = clip.height();
	else
	    height_level[*act_height] = max (height_level[*act_height],
					     clip.height());
	if (node->childNode() && node->_open) {
	    GNumber x_first;
	    int j = -1;
	    GBool son_not_marked = GTRUE;
	    (*act_height)++;
	    for (GoTreeNode* n = node->childNode(); n; n = n->nextNode())
		find_fields_v(n, height_level, modifier, next_pos, x_final,
			      place, act_height, &j, &x_first,
			      &son_not_marked, center);
	    (*act_height)--;
	    if (son_not_marked)
		*place = (clip.width() / 2) + next_pos[*act_height];
	    else {
		*place = (x_first + *x_final) / 2;
		modifier[*act_height] = max (modifier[*act_height],
					    (clip.width() / 2) +
					    next_pos[*act_height] - *place);
		*place += modifier[*act_height];
	    }
	}
	if (!node->_open || !node->childNode()) 
	    *place = (clip.width() / 2) + next_pos[*act_height];
	center[nb_n].x = *place;  
	center[nb_n].y = 0.;
	node->_modif = modifier[*act_height];
	if (!*jp)
	    *x_first = *place;
	*x_final = *place;
	next_pos[*act_height] = *place + dist_x + (clip.width() / 2);
    }
}

	


/**
FUNCTION
    gmng()
**/
void GoTree::gmng(GoTreeNode* node, GNumber* vector_height, const GPoint& point,
		  GPoint* center)   
{
    GNumber modifier_sum = 0.;
    GNumber total_height = dist_y;
    GNumber delta_x = 0.;
    GNumber delta_y = 0.;
    int act_height = 1;
    if (&point) {
	if (alignment_type == GON_CENTER)
	    delta_y = point.y - (dist_y + (vector_height[act_height] / 2));
	else {
	    GRegion tmp_reg;
	    node->addClip(tmp_reg);
	    GBox clip = tmp_reg.bigbox();
	    delta_y = point.y - dist_y - (clip.height() / 2);
	}
	delta_x = point.x - center[1].x; 
    }
    modif_node1_v(node, &modifier_sum, delta_x, delta_y, &total_height,
		  &act_height, vector_height, center);
}


/**
FUNCTION
    modif_node1_v()
**/
void GoTree::modif_node1_v(GoTreeNode* node, GNumber* modifier_sum,
			   GNumber delta_x, GNumber delta_y,
			   GNumber* total_height, int* act_height,
			   GNumber* vector_height,
			   GPoint* center)
{
    ushort nb_n = ++num_node;
    if (node->_marked) {
	node->_marked = GFALSE;
	if (alignment_type == GON_CENTER)
	    node->move(GPoint(center[nb_n].x + *modifier_sum + delta_x,
			     *total_height + (vector_height[*act_height] / 2)
			     + delta_y));
	else {
	    GRegion tmp_reg;
	    node->addClip(tmp_reg);
	    GBox clip = tmp_reg.bigbox();
	    node->move(GPoint(center[nb_n].x + *modifier_sum + delta_x,
			     *total_height + delta_y + (clip.height() / 2)));
	}
	if (node->childNode() && node->_open) {
	    *total_height = *total_height + dist_y +
		vector_height[*act_height];
	    (*act_height)++;
	    if (node->_modif)
		*modifier_sum += node->_modif;
	    for (GoTreeNode* n = node->childNode(); n; n = n->nextNode())
		modif_node1_v(n, modifier_sum, delta_x, delta_y, total_height,
			      act_height, vector_height, center);
	    (*act_height)--;
	    *total_height = *total_height -
		(dist_y + vector_height[*act_height]);
	    if (node->_modif) {
		*modifier_sum = *modifier_sum - node->_modif;
		node->_modif = 0.;
	    }
	}
    }
}



/**
FUNCTION
    bottom_up_placement()
DESCRIPTION
    Placement from bottom to up.
**/
void GoTree::bottom_up_placement(GoNode* node, const GPoint& point)
{
    GNumber x_final = interf()->width();
    GNumber x_first = x_final;
    GNumber place, modifier[MAX_LEVEL], next_pos[MAX_LEVEL];
    GNumber height_level[MAX_LEVEL];
    GPoint * center = new GPoint[nb_node+1];           
    int act_height = 1;
    int j = -1;
    GBool son_not_marked = GTRUE;

    for (int i = 0; i < MAX_LEVEL; i++) {
	next_pos[i] = dist_x;
	modifier[i] = 0.;
	height_level[i] = 0.;
    }
    num_node = 0;
    find_fields_v2 ((GoTreeNode*)node, height_level, modifier, next_pos,
		   &x_final, &place, &act_height, &j, &x_first,
		   &son_not_marked, center);
    num_node = 0;
    gmng2((GoTreeNode*)node, height_level, point, center);
    delete center;
}


/**
FUNCTION
    find_fields_v2()
**/
void GoTree::find_fields_v2(GoTreeNode* node, GNumber* height_level,
			    GNumber* modifier, GNumber* next_pos,
			    GNumber* x_final, GNumber* place, int *act_height,
			    int* jp, GNumber* x_first, GBool* son_not_marked,
			    GPoint* center) 
{
    ushort nb_n = ++num_node;
    GNumber win_width = interf()->width();
    if (!node->_marked) {
	node->_marked = GTRUE;
	*son_not_marked = GFALSE;
	(*jp)++;
	GRegion tmp_reg;
	node->addClip(tmp_reg);
	GBox clip = tmp_reg.bigbox();
	if (height_level[*act_height] == 0.0)
	    height_level[*act_height] = clip.height();
	else
	    height_level[*act_height] = max (height_level[*act_height],
					     clip.height());
	if (node->childNode() && node->_open) {
	    GNumber x_first;
	    int j = -1;
	    GBool son_not_marked = GTRUE;
	    (*act_height)++;
	    for (GoTreeNode* n = node->childNode(); n; n = n->nextNode())
		find_fields_v2(n, height_level, modifier, next_pos, x_final,
			      place, act_height, &j, &x_first,
			      &son_not_marked, center);
	    (*act_height)--;
	    if (son_not_marked)
		*place = win_width - ((clip.width() / 2) +
				      next_pos[*act_height]);  // OK;
	    else {
		*place = (x_first + *x_final) / 2;
		modifier[*act_height] = max (modifier[*act_height],
					     *place - (win_width -
						       (clip.width() / 2) -
						       next_pos[*act_height]));
		*place -= modifier[*act_height];  //OK;
	    }
	}
	if (!node->_open || !node->childNode()) 
	    *place = win_width - ((clip.width() / 2) + next_pos[*act_height]);
	center[nb_n].x = *place;  
	center[nb_n].y = 0.;
	node->_modif = modifier[*act_height];
	if (!*jp)
	    *x_first = *place;
	*x_final = *place;
	next_pos[*act_height] = win_width - (*place - dist_x -
					     (clip.width() / 2)); //OK;
    }
}

	


/**
FUNCTION
    gmng2()
**/
void GoTree::gmng2(GoTreeNode* node, GNumber* vector_height, const GPoint& point,
		   GPoint* center)   
{
    GNumber modifier_sum = 0.;
    GNumber total_height = dist_y;
    GNumber delta_x = 0.;
    GNumber delta_y = 0.;
    int act_height = 1;
    if (&point) {
	if (alignment_type == GON_CENTER)
	    delta_y = point.y - (dist_y + (vector_height[act_height] / 2));
	else {
	    GRegion tmp_reg;
	    node->addClip(tmp_reg);
	    GBox clip = tmp_reg.bigbox();
	    delta_y = point.y - dist_y - (clip.height() / 2);
	}
	delta_x = point.x - center[1].x; 
    }
    modif_node2_v(node, &modifier_sum, delta_x, delta_y, &total_height,
		  &act_height, vector_height, center);
}


/**
FUNCTION
    modif_node2_v()
**/
void GoTree::modif_node2_v(GoTreeNode* node, GNumber* modifier_sum,
			   GNumber delta_x, GNumber delta_y,
			   GNumber* total_height, int* act_height,
			   GNumber* vector_height,
			   GPoint* center)
{
    GNumber win_height = interf()->height();
    ushort nb_n = ++num_node;
    if (node->_marked) {
	node->_marked = GFALSE;
	if (alignment_type == GON_CENTER)
	    node->move(GPoint(center[nb_n].x - *modifier_sum - delta_x,
			     win_height - (*total_height +
					   (vector_height[*act_height] / 2)
					   + delta_y)));
	else {
	    GRegion tmp_reg;
	    node->addClip(tmp_reg);
	    GBox clip = tmp_reg.bigbox();
	    node->move(GPoint(center[nb_n].x - *modifier_sum - delta_x,
			     win_height - (*total_height + delta_y +
					   (clip.height() / 2))));
	}
	if (node->childNode() && node->_open) {
	    *total_height = *total_height + dist_y +
		vector_height[*act_height];
	    (*act_height)++;
	    if (node->_modif)
		*modifier_sum += node->_modif;
	    for (GoTreeNode* n = node->childNode(); n; n = n->nextNode())
		modif_node2_v(n, modifier_sum, delta_x, delta_y, total_height,
			      act_height, vector_height, center);
	    (*act_height)--;
	    *total_height = *total_height -
		(dist_y + vector_height[*act_height]);
	    if (node->_modif) {
		*modifier_sum = *modifier_sum - node->_modif;
		node->_modif = 0.;
	    }
	}
    }
}


