#ifdef REV_INFO
static char rcsid[] = "$Id: TreeNode.cc,v 1.5 1992/12/10 16:24:54 smadja Exp $";
#endif
/*****************************************************************************
MODULE:  BULL Graphic Objects
FILE:    TreeNode.cc
AUTHOR:  Aline Jacomet
DATE:    January 11th 1991

DESCRIPTION:
         Definition of the tree node object class.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include <Go/TreeNode.h>
#include <Go/mTreeNode.h>
#include <Go/Tree.h>
#include <Go/GoStream.h>
#include <Go/Tools/Assoc.h>



extern GAssoc save_precond;


/*
 * GCallback Types
 */
static char goTreeNodeNattachNodePrecond[] = "attachNodePrecond";    
static char goTreeNodeNattachNodePostcond[] = "attachNodePostcond";    
static char goTreeNodeNshiftNodePrecond[] = "shiftNodePrecond";    
static char goTreeNodeNshiftNodePostcond[] = "shiftNodePostcond";    


/*
 *------------
 * CLASS PART
 *------------
 */



static GoTreeNodeMetaclass* treeNode_metaclass = NULL;

/**
FUNCTION
    GoTreeNode::metaclass()
DESCRIPTION
    
**/
GoTreeNodeMetaclass* GoTreeNode::metaclass() {
    if (treeNode_metaclass == NULL) {
	treeNode_metaclass = new GoTreeNodeMetaclass;
	treeNode_metaclass->superclass()->AddSubclass(treeNode_metaclass);
      GoStream::bind(GoTreeNode::metaclass()->classname(),
		     (int (*) (const void*, GoStream*)) GoTreeNode::save,
		     (int (*) (void**, GoStream*)) GoTreeNode::restore);
    }
    return treeNode_metaclass;
}


/**
FUNCTION
    ~GoTreeNodeMetaclass()
DESCRIPTION
    
**/
GoTreeNodeMetaclass::~GoTreeNodeMetaclass() {
    if (this == treeNode_metaclass) {
        superclass()->RemoveSubclass(this);
	treeNode_metaclass = NULL;
    }
}


/**
FUNCTION
    GoTreeNodeMetaclass::GoTreeNodeMetaclass()
DESCRIPTION
    
**/
 GoTreeNodeMetaclass::GoTreeNodeMetaclass() {
}


/**
FUNCTION
    const char* GoTreeNodeMetaclass::m_classname() const
DESCRIPTION

**/
const char* GoTreeNodeMetaclass::m_classname() const
{
    return "GoTreeNode";
}


/**
FUNCTION
    GoMetaclass* GoTreeNodeMetaclass::m_superclass() const
DESCRIPTION

**/
GoMetaclass* GoTreeNodeMetaclass::m_superclass() const
{
    return GoNode::metaclass();
}


/**
FUNCTION
    Go* GoTreeNodeMetaclass::m_create();
DESCRIPTION

**/
Go* GoTreeNodeMetaclass::m_create() const
{
    return (Go*)(new GoTreeNode);
}


/**
FUNCTION
    GoTreeNode::m_dup()
DESCRIPTION

**/
void GoTreeNode::m_dup
    (const Go* src, Go* par, Go* prv, GBool b, GBool dochildren)
{
    GoNode::m_dup(src, par, prv, b, dochildren);

    const GoTreeNode* n_src = (const GoTreeNode*) src;

    _open = n_src->_open;
    first_close = n_src->first_close;
    _marked = n_src->_marked;
    _modif = n_src->_modif;
}



/**
FUNCTION
    m_metaclass()
DESCRIPTION
    Returns the metaclass.
**/
GoMetaclass* GoTreeNode::m_metaclass () const
{
    return GoTreeNode::metaclass();
}

static void gotreenode_openNode(Go* go)
{
	((GoTreeNode*)go)->openNode();
}
static void gotreenode_closeNode(Go* go)
{
	((GoTreeNode*)go)->closeNode();
}
static GBool gotreenode_shiftNode(Go* go, GoTreeNode* p =NULL)
{
	return ((GoTreeNode*)go)->shiftNode(p);
}

static GActionRec goTreeNode_action_array[] =
{ 
  { "openNode()",
	(GActionProc) gotreenode_openNode },
  { "closeNode()",
	(GActionProc) gotreenode_closeNode },
  { "shiftNode(GoTreeNode*=NULL)",
	(GActionProc) gotreenode_shiftNode },
  { NULL, NULL }
};


/**
FUNCTION
    m_actions()
DESCRIPTION
    
**/
const GActionRec* GoTreeNodeMetaclass::m_actions() const
{
    return goTreeNode_action_array;
}




/**
FUNCTION
   _call_class_attach_node_precond() 
DESCRIPTION
    
**/
GBool GoTreeNode::_call_class_attach_node_precond (GoTreeNode* node,
						   GoMetaclass* linkclass,
						   GoTreeNode* next,
						   GoTree* tree) const
{
    // call conditions on all super-Metaclasses;
    for (GoMetaclass* mc = getMetaclass(); mc; mc = mc->superclass()) {
	if (((GoTreeNodeMetaclass*) mc)->_flags._has_attach_node_precond &&
	    ! mc->CallPreconditions(goTreeNodeNattachNodePrecond, this, node,
				    linkclass, next, tree))
	    return GFALSE;
	// stop when reaching GoTreeNode Metaclass instance;
	if (mc == GoTreeNode::metaclass())
	    return GTRUE;
    }
    return GTRUE;
}


/**
FUNCTION
   _call_class_attach_node_postcond() 
DESCRIPTION
    
**/
void GoTreeNode::_call_class_attach_node_postcond (GoTreeNode* node,
						   GoMetaclass* linkclass,
						   GoTreeNode* next,
						   GoTree* tree)
{
    // call conditions on all super-Metaclasses;
    for (GoMetaclass* mc = getMetaclass(); mc; mc = mc->superclass()) {
	if (((GoTreeNodeMetaclass*) mc)->_flags._has_attach_node_postcond)
	    CallPostconditions(goTreeNodeNattachNodePostcond, this, node,
			       linkclass, next, tree);
	// stop when reaching GoTreeNode Metaclass instance;
	if (mc == GoTreeNode::metaclass())
	    return;
    }
}


/**
FUNCTION
    _call_class_shift_node_precond()
DESCRIPTION
    
**/
GBool GoTreeNode::_call_class_shift_node_precond (GoTreeNode* node) const
{
    // call conditions on all super-Metaclasses;
    for (GoMetaclass* mc = getMetaclass(); mc; mc = mc->superclass()) {
	if (((GoTreeNodeMetaclass*) mc)->_flags._has_shift_node_precond &&
	    ! mc->CallPreconditions(goTreeNodeNshiftNodePrecond, this, node))
	    return GFALSE;
	// stop when reaching GoTreeNode Metaclass instance;
	if (mc == GoTreeNode::metaclass())
	    return GTRUE;
    }
    return GTRUE;
}


/**
FUNCTION
    _call_class_shift_node_postcond()
DESCRIPTION
    
**/
void GoTreeNode::_call_class_shift_node_postcond (GoTreeNode* node)
{
    // call conditions on all super-Metaclasses;
    for (GoMetaclass* mc = getMetaclass(); mc; mc = mc->superclass()) {
	if (((GoTreeNodeMetaclass*) mc)->_flags._has_shift_node_postcond)
	    CallPostconditions(goTreeNodeNshiftNodePostcond, this, node);
	// stop when reaching GoTreeNode Metaclass instance;
	if (mc == GoTreeNode::metaclass())
	    return;
    }
}



/**
FUNCTION
    attachNodePrecond
DESCRIPTION
    
**/
void GoTreeNodeMetaclass::attachNodePrecond (GBool cbproc (void*, GoTreeNode*,
							  GoTreeNode*,
							  GoMetaclass*,
							  GoTreeNode*,
							  GoTree*),
					     void* closure, GAddOrRemove add)
{
    _flags._has_attach_node_precond =
	AddOrRemoveCondition(goTreeNodeNattachNodePrecond, cbproc, closure,
			     add);
}


/**
FUNCTION
    attachNodePostcond
DESCRIPTION
    
**/
void GoTreeNodeMetaclass::attachNodePostcond (void cbproc (void*, GoTreeNode*,
							   GoTreeNode*,
							   GoMetaclass*,
							   GoTreeNode*,
							   GoTree*),
					      void* closure, GAddOrRemove add)
{
    _flags._has_attach_node_postcond =
	AddOrRemoveCondition(goTreeNodeNattachNodePostcond, cbproc, closure,
			     add);
}


/**
FUNCTION
    shiftNodePrecond
DESCRIPTION
    
**/
void GoTreeNodeMetaclass::shiftNodePrecond (GBool cbproc (void*, GoTreeNode*,
							 GoTreeNode*),
					    void* closure, GAddOrRemove add)
{
    _flags._has_shift_node_precond =
	AddOrRemoveCondition(goTreeNodeNshiftNodePrecond, cbproc, closure,
			     add);
}


/**
FUNCTION
    shiftNodePostcond
DESCRIPTION
    
**/
void GoTreeNodeMetaclass::shiftNodePostcond (void cbproc (void*, GoTreeNode*,
							  GoTreeNode*),
					     void* closure, GAddOrRemove add)
{
    _flags._has_shift_node_postcond =
	AddOrRemoveCondition(goTreeNodeNshiftNodePostcond, cbproc, closure,
			     add);
}


/*
 *--------------
 * PRIVATE DATA
 *--------------
 */



/**
FUNCTION
    reparent_on_tree()
DESCRIPTION
    Reparent on "tree" "this" and its subtree.
**/
void GoTreeNode::reparent_on_tree(GoTree* tree)
{
    for(GoTreeNode* n = child_node; n; n = n->next_node)
	n->reparent_on_tree(tree);

    if (parent_link)
	parent_link->reparent((Go*)tree, tree->Child());
    reparent((Go*)tree);
}


/**
FUNCTION
    open_tree()
DESCRIPTION
    Opens and maps "this" and its subtree.
**/
void GoTreeNode::open_tree()
{
    for(GoTreeNode* n = child_node; n; n = n->next_node)
	n->open_tree();

    if (parent_link)
	parent_link->map();
    _open = GTRUE;
    map();
}

/**
FUNCTION
    close_tree()
DESCRIPTION
    Closes and unmaps "this", its subtree, its next brothers and their
    subtree.
**/
void GoTreeNode::close_tree()
{
    while((child_node &&
	   (child_node->isOpened() || child_node->first_close)) ||
	  (next_node && (next_node->isOpened() || next_node->first_close))) {
	if (child_node && (child_node->isOpened() || child_node->first_close))
	    child_node->close_tree();
	else
	    next_node->close_tree();
    }
    if (parent_link)
	parent_link->unmap();
    _open = GFALSE;
    first_close = GFALSE;
    unmap();
}


/**
FUNCTION
    is_parent_correct()
DESCRIPTION
    Verifies if new_parent is not in the subtree of "this".
**/
GBool GoTreeNode::is_parent_correct(GoTreeNode* new_par)
{
    for(GoTreeNode* n = child_node; n; n = n->next_node)
	if (! n->is_parent_correct(new_par))
	    return GFALSE;

    if (this == new_par)
	return GFALSE;
    else
	return GTRUE;
}



/*
 *----------------
 * PROTECTED DATA
 *----------------
 */


/**
FUNCTION
   _call_attach_node_precond() 
DESCRIPTION
    
**/
GBool GoTreeNode::_call_attach_node_precond (GoTreeNode* node,
					    GoMetaclass* linkclass,
					    GoTreeNode* next,
					    GoTree* tree) const
{
    return (_flags._has_attach_node_precond ?
	    CallPreconditions(goTreeNodeNattachNodePrecond, node, linkclass,
			      next, tree) : GTRUE);
}


/**
FUNCTION
   _call_attach_node_postcond() 
DESCRIPTION
    
**/
void GoTreeNode::_call_attach_node_postcond (GoTreeNode* node,
					     GoMetaclass* linkclass,
					     GoTreeNode* next,
					     GoTree* tree)
{
    if (_flags._has_attach_node_postcond)
	CallPostconditions(goTreeNodeNattachNodePostcond, node, linkclass,
			   next, tree);
}


/**
FUNCTION
    _call_shift_node_precond()
DESCRIPTION
    
**/
GBool GoTreeNode::_call_shift_node_precond (GoTreeNode* node) const
{
    return (_flags._has_shift_node_precond ?
	    CallPreconditions(goTreeNodeNshiftNodePrecond, node) : GTRUE);
}


/**
FUNCTION
    _call_shift_node_postcond()
DESCRIPTION
    
**/
void GoTreeNode::_call_shift_node_postcond (GoTreeNode* node)
{
    if (_flags._has_shift_node_postcond)
	CallPostconditions(goTreeNodeNshiftNodePostcond, node);
}


/**
FUNCTION
    modif_clip()
DESCRIPTION
    Calls the "_constraints" function for all links connected to "node".
**/
void GoTreeNodeMetaclass::modif_clip(Go* node)
{
    GoNodeMetaclass::modif_clip(node);

    if (((GoTreeNode*)(node))->parent_link) {
	((GoTreeNode*)(node))->parent_link->_constraints();
    }
    for (GoTreeNode* n = ((GoTreeNode*)(node))->childNode(); n;
	 n = n->nextNode())
	if (n->parent_link)
	    n->parent_link->_constraints();
}
    


/*
 *--------------
 * PUBLIC DATA
 *--------------
 */


/**
FUNCTION
    GoTreeNode()
DESCRIPTION
    constructor
**/
GoTreeNode::GoTreeNode()
{
    parent_node = NULL;
    child_node  = NULL;
    next_node   = NULL;
    prev_node   = NULL;
    parent_link = NULL;
    _open       = GTRUE;
    first_close = GFALSE;
    _marked     = GFALSE;
    _modif      = 0.;
}


/**
FUNCTION
    ~GoTreeNode()
DESCRIPTION
    destructor
**/
GoTreeNode::~GoTreeNode()
{
    if (parent_node && parent_node->child_node == this) {
	parent_node->child_node = next_node;
	if (next_node)
	    next_node->prev_node = NULL;
    } else {
	if (prev_node)
	    prev_node->next_node = next_node;
	if (next_node)
	    next_node->prev_node = prev_node;
    }
    parent_node = NULL;
    child_node = NULL;
    next_node = NULL;
    prev_node = NULL;
    parent_link = NULL;
}


/**
FUNCTION
    shiftNode()
DESCRIPTION
    Changes the order of "this" and its brothers.
**/
GBool GoTreeNode::shiftNode(GoTreeNode* next)
{
    GBool cont = GTRUE;

    if (_call_shift_node_precond(next) &&
	_call_class_shift_node_precond(next)){
	if (next == next_node)
	    return GTRUE;
	if (!parent_node)
	    return GFALSE;
	
	GoTreeNode* n = parent_node->child_node;
	if (next) {
	    for(; n && cont; n = n->next_node)
		if (n == next)
		    cont = GFALSE;
	    if (cont)
		return GFALSE;
	} else
	    while (n->next_node)
		n = n->next_node;

	if (parent_node->child_node == this)
	    parent_node->child_node = next_node;
	if (prev_node)
	    prev_node->next_node = next_node;
	if (next_node)
	    next_node->prev_node = prev_node;
	if (next) {
	    if (next->prev_node)
		next->prev_node->next_node = this;
	    prev_node = next->prev_node;
	next->prev_node = this;
	    next_node = next;
	} else {
	    n->next_node = this;
	    prev_node = n;
	    next_node = NULL;
	}
	if (!prev_node)
	    parent_node->child_node = this;

	_call_shift_node_postcond(next);
	_call_class_shift_node_postcond(next);
	return GTRUE;
    } else
	return GFALSE;
}


/**
FUNCTION
    attachNode()
DESCRIPTION
    Attaches "this" on a new tree, new parent with new link.
**/
GBool GoTreeNode::attachNode(GoTreeNode* par, GoMetaclass* linkclass,
			    GoTreeNode* next, GoTree* tree)
{
    if (_call_attach_node_precond(par, linkclass, next, tree) &&
	_call_class_attach_node_precond(par, linkclass, next, tree)) {

	GoTree* ex_tree = (GoTree*) parent();
	
	/*
	 * Test of the paramaters
	 */
	
	/*
	 * linkclass
	 */
	if (!linkclass) {
	    if (tree)
		linkclass = tree->defaultLinkType();
	    else {
		if (par && par->parent())
		    linkclass =((GoTree*)par->parent())->defaultLinkType();
		else {
		    GWarning("GoTreeNode::attachNode: the specified node belongs on no tree and tree parameter is NULL.");
		    return GFALSE;
		}
	    }
	} else
	    if (!linkclass->isSubclass(GoLink::metaclass())) {
		GWarning("GoTreeNode::attachNode: the specified type of link is not a GoLink or a subclass of GoLink.");
		return GFALSE;
	    }

	/*
	 * par and tree.
	 */
	if (par) {
	    if (!par->getMetaclass()->isSubclass(GoTreeNode::metaclass())) {
		GWarning("GoTreeNode::attachNode: specified node is not an instance of GoTreeNode class.");
		return GFALSE;
	    }
	    if (! par->parent()->getMetaclass()->isSubclass(GoTree::metaclass())) {
		GWarning("GoTreeNode::attachNode: the specified node does not belong to a GoTree object.");
		return GFALSE;
	    }
	    if (par->parent() == ex_tree)
		if (! is_parent_correct(par)) {
		    GWarning("GoTreeNode::attachNode: you can't attach this node on a node of its subtree.");
		    return GFALSE;
		}
	} else {
	    if (tree) {
		if (! tree->getMetaclass()->isSubclass(GoTree::metaclass())) {
		    GWarning("GoTreeNode::attachNode: the specified GoTree instance is not a tree object.");
		    return GFALSE;
		}
		if (tree->first_node) {
		    GWarning("GoTreeNode::attachNode: the tree object already has a root node.");
		    return GFALSE;
		}
	    } else {
		GWarning("GoTreeNode::attachNode: the specified GoTree instance is NULL.");
		return GFALSE;
	    }
	}
	
	GoTree *new_tree;
	if (tree)
	    new_tree = tree;
	else
	    new_tree = (GoTree*)par->parent();
	
	if (parent_node) {
	    /*
	     * Separate node of his parent_node
	     */
	    if (parent_node->child_node == this)
		parent_node->child_node = next_node;
	    if (next_node)
		next_node->prev_node = prev_node;
	    if (prev_node)
		prev_node->next_node = next_node;
	    prev_node = NULL;
	    next_node = NULL;
	    parent_node = NULL;
	    if (parent_link)
		if (!par || linkclass != parent_link->getMetaclass()) {
		    parent_link->in_destroy = GTRUE;
		    parent_link->destroy();
		    parent_link = NULL;
		}
	}

	if ( ex_tree && ex_tree != new_tree)
	    if (this == ex_tree->firstNode()) {
		this->clipPostcond(graphstr_set_coordsys,  ex_tree, GREMOVE);
		ex_tree->first_node = NULL;
	    }

	/*
	 * Attach on new tree or new parent
	 */
	if (!par) {
	    tree->first_node = this;
	    this->clipPostcond(graphstr_set_coordsys, tree);
	} else {
	    parent_node = (GoTreeNode*)par;
	    if (((GoTreeNode*)par)->child_node) {
		/*
		 * It is not the first child of "par"
		 */
		GoTreeNode* child = ((GoTreeNode*)par)->child_node;
		while (child->next_node)
		    child = child->next_node;
		child->next_node = this;
		prev_node = child;
	    } else {
		/*
		 * It is the first child of "par"
		 */
		((GoTreeNode*)par)->child_node = this;
	    }

	    if (!parent_link) {
		/*
		 * Create the new link
		 */
		GoLink* link = (GoLink*)Go::Create(linkclass);
		if (!link)
		    return GFALSE;
		parent_link = (GoLink*)link;
		parent_link->prev_node = (GoNode*)par;
		parent_link->next_node = (GoNode*)this;
		parent_link->map();
	    } else {
		parent_link->prev_node = (GoNode*)par;
		if (parent_link->mappable())
		    parent_link->_constraints();
		else
		    parent_link->map();
	    }
	}

	/*
	 * Reparent on new_tree if necessary
	 */
	if (new_tree != ex_tree)
	    reparent_on_tree(new_tree);

	/*
	 * close if necessary
	 */
	if ( par && !par->isOpened())
	    close_tree();
	else
	    if (!mappable())
		map();

	if (next)
	    shiftNode(next);
	
	_call_attach_node_postcond(par, linkclass, next, tree);
	_call_class_attach_node_postcond(par, linkclass, next, tree);
	return GTRUE;
    } else
	return GFALSE;

}


/**
FUNCTION
    closeNode()
DESCRIPTION
    Closes "this" and its subtree. Unmaps the subtree.
**/
void GoTreeNode::closeNode()
{
    if (child_node)
	child_node->close_tree();
    _open = GFALSE;
    first_close = GTRUE;
    
    GoGraphStruct* str = (GoGraphStruct*)parent();
    if (str->background() && str->mapped()) {
	// Calculate new clip;
	GBox box(GEmptyBox());
	GRegion r;
	for (Go* ch = str->child(); ch; ch = ch->next())
	    if (ch->mappable() &&
		ch->getMetaclass()->isSubclass(GoNode::metaclass())) {
		r = GEmptyRegion();
		ch->addClip(r);
		box += r.bigbox();
	    }
	if (box != str->my_clip) {
	    str->RedrawBackAndFore(str->my_clip);
	    str->my_clip = box;
	    str->MyClipHasChanged();
	    str->RedrawForeOnly();
	}
    }
}


/**
FUNCTION
    openNode()
DESCRIPTION
    Opens "this" and its subtree. Maps the subtree.
**/
void GoTreeNode::openNode()
{
    first_close = GFALSE;
    open_tree();
    
    GoGraphStruct* str = (GoGraphStruct*)parent();
    if (str->background() && str->mapped()) {
	// Calculate new clip;
	GBox box(GEmptyBox());
	GRegion r;
	for (Go* ch = str->child(); ch; ch = ch->next())
	    if (ch->mappable() &&
		ch->getMetaclass()->isSubclass(GoNode::metaclass())) {
		r = GEmptyRegion();
		ch->addClip(r);
		box += r.bigbox();
	    }
	if (box != str->my_clip) {
	    str->RedrawBackAndFore(str->my_clip);
	    str->my_clip = box;
	    str->MyClipHasChanged();
	    str->RedrawForeOnly();
	}
    }
}


/**
FUNCTION
    attachNodePrecond
DESCRIPTION
    
**/
void GoTreeNode::attachNodePrecond (GBool cbproc (void*, GoTreeNode*,
						 GoTreeNode*,
						 GoMetaclass*, GoTreeNode*,
						 GoTree*),
				    void* closure, GAddOrRemove add)
{
    _flags._has_attach_node_precond =
	AddOrRemoveCondition(goTreeNodeNattachNodePrecond, cbproc, closure,
			     add);
}


/**
FUNCTION
    attachNodePostcond
DESCRIPTION
    
**/
void GoTreeNode::attachNodePostcond (void cbproc (void*, GoTreeNode*,
						  GoTreeNode*, GoMetaclass*,
						  GoTreeNode*, GoTree*),
				     void* closure, GAddOrRemove add)
{
    _flags._has_attach_node_postcond =
	AddOrRemoveCondition(goTreeNodeNattachNodePostcond, cbproc, closure,
			     add);
}


/**
FUNCTION
    shiftNodePrecond
DESCRIPTION
    
**/
void GoTreeNode::shiftNodePrecond (GBool cbproc (void*, GoTreeNode*,
						GoTreeNode*),
				   void* closure, GAddOrRemove add)
{
    _flags._has_shift_node_precond =
	AddOrRemoveCondition(goTreeNodeNshiftNodePrecond, cbproc, closure,
			     add);
}


/**
FUNCTION
    shiftNodePostcond
DESCRIPTION
    
**/
void GoTreeNode::shiftNodePostcond (void cbproc (void*, GoTreeNode*,
						 GoTreeNode*),
				    void* closure, GAddOrRemove add)
{
    _flags._has_shift_node_postcond =
	AddOrRemoveCondition(goTreeNodeNshiftNodePostcond, cbproc, closure,
			     add);
}




/**
FUNCTION
    int GoTreeNode::save();
DESCRIPTION
    ** static member, protected **
**/
int GoTreeNode::save(const GoTreeNode* n, GoStream* gostream) {
    if (n->parentNode() && !save_precond.get(n->parentNode())) {
	if (!gostream->saveObj(n->parentNode(),
			       n->parentNode()->getMetaclass()->classname(),
			       "parent node"))
	    return GFALSE;
    } else
	if (!gostream->saveObj(NULL, NULL, "parent node"))
	    return GFALSE;
    
    GoTreeNode* go = n->childNode();
    if (go)
	// looking for the new child node, if necessary;
	while(go && save_precond.get(go))
	    go = go->nextNode();
    
    if (go) {
	if (!gostream->saveObj(go, go->getMetaclass()->classname(),
			       "child node"))
	    return GFALSE;
    } else
	if (!gostream->saveObj(NULL, NULL, "child node"))
	    return GFALSE;
    

    go = n->nextNode();
    if (go)
	// looking for the new next node, if necessary;
	while(go && save_precond.get(go))
	    go = go->nextNode();
    
    if (go) {
	if (!gostream->saveObj(go, go->getMetaclass()->classname(),
			       "next node"))
	    return GFALSE;
    } else
	if (!gostream->saveObj(NULL, NULL, "next node"))
	    return GFALSE;

    
    go = n->previousNode();
    if (go)
	// looking for the new previous node, if necessary;
	while(go && save_precond.get(go))
	    go = go->previousNode();
    
    if (go) {
	if (!gostream->saveObj(go, go->getMetaclass()->classname(),
			       "previous node"))
	    return GFALSE;
    } else
	if (!gostream->saveObj(NULL, NULL, "previous node"))
	    return GFALSE;
    

    
    if (n->parentLink()) {
	if (!gostream->saveObj(n->parentLink(),
			       n->parentLink()->getMetaclass()->classname(),
			       "parent link"))
	    return GFALSE;
    } else
	if (!gostream->saveObj(NULL, NULL, "parent link"))
	    return GFALSE;

    if (!(gostream->writeBool(n->isOpened(), "is opened") &&
	  gostream->writeBool(n->first_close, "first close")))
	return GFALSE;
    
    return (GoNode::save(n, gostream));
}


/**
FUNCTION
    int GoTreeNode::readParams();
DESCRIPTION
    
**/
int GoTreeNode::readParams(GoTreeNode* n, GoStream* gostream) {
    Go* go;
    const char* class_name;

    if (!gostream->restoreObj((void**)&go, &class_name))
	return GFALSE;
    if (go) {
	if (strcmp(class_name, GoTreeNode::metaclass()->classname()) != 0) {
	    GWarning("GoTreeNode::readParams: can't restore parent node.");
	    // ****** ;
	    go = NULL;
	}
	n->parent_node = (GoTreeNode*)go;
    }
    
    if (!gostream->restoreObj((void**)&go, &class_name))
	return GFALSE;
    if (go) {
	if (strcmp(class_name, GoTreeNode::metaclass()->classname()) != 0) {
	    GWarning("GoTreeNode::readParams: can't restore child node.");
	    // ****** ;
	    go = NULL;
	}
	n->child_node = (GoTreeNode*)go;
    }
    
    if (!gostream->restoreObj((void**)&go, &class_name))
	return GFALSE;
    if (go) {
	if (strcmp(class_name, GoTreeNode::metaclass()->classname()) != 0) {
	    GWarning("GoTreeNode::readParams: can't restore next node.");
	    // ****** ;
	    go = NULL;
	}
	n->next_node = (GoTreeNode*)go;
    }
    
    if (!gostream->restoreObj((void**)&go, &class_name))
	return GFALSE;
    if (go) {
	if (strcmp(class_name, GoTreeNode::metaclass()->classname()) != 0) {
	    GWarning("GoTreeNode::readParams: can't restore previous node.");
	    // ****** ;
	    go = NULL;
	}
	n->prev_node = (GoTreeNode*)go;
    }
    
    if (!gostream->restoreObj((void**)&go, &class_name))
	return GFALSE;
    if (go)
	n->parent_link = (GoLink*)go;

    
    GBool b;
    if (!gostream->readBool(&b))
	return GFALSE;
    n->_open = b;
    if (!gostream->readBool(&b))
	return GFALSE;
    n->first_close = b;
    
    return GoNode::readParams(n, gostream);
}


/**
FUNCTION
    int GoTreeNode::restore();
DESCRIPTION
    
**/
int GoTreeNode::restore(GoTreeNode** n, GoStream* gostream) {
    if (*n = GoTreeNode::create())
	return GoTreeNode::readParams(*n, gostream);
    return GFALSE;
}











