#ifdef REV_INFO
static char rcsid[] = "$Id: Image.cc,v 1.5 1992/12/10 16:27:55 smadja Exp $";
#endif
/*****************************************************************************
MODULE:  Graphic Objects
FILE:    Image.cc
AUTHOR:  Philippe Smadja
DATE:    May 1991

DESCRIPTION:
    See the Image.h header.
    
COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
*****************************************************************************/

#include <assert.h>
#include <Go/Tools/Error.h>
#include <Go/GoStream.h>
#include <Go/Image.h>
#include <Go/GoInterf.h>

/*
 *--------
 * ACTIONS
 *--------
 */

/*
 *-------------------
 * PRE/POSTCONDITIONS
 *-------------------
 */

static char goimageNconfigPrecond[] = "";
static char goimageNconfigPostcond[] = "";

/*
 *------------
 * CLASS PART
 *------------
 */


static GoImageMetaclass* image_metaclass = NULL;

/**
FUNCTION
    GoImageMetaclass* GoImage::metaclass()
DESCRIPTION
    
**/
GoImageMetaclass* GoImage::metaclass() {
    if (image_metaclass == NULL) {
	image_metaclass = new GoImageMetaclass;
	image_metaclass->superclass()->AddSubclass(image_metaclass);

        GoStream::bind(GoImage::metaclass()->classname(),
		       (int (*) (const void*, GoStream*)) GoImage::save,
		       (int (*) (void**, GoStream*)) GoImage::restore);
    }
    return image_metaclass;
}


/**
FUNCTION
    GoImageMetaclass::~GoImageMetaclass()
DESCRIPTION
    
**/
GoImageMetaclass::~GoImageMetaclass() {
    if (this == image_metaclass) {
        superclass()->RemoveSubclass(this);
	image_metaclass = NULL;
    }
}


/**
FUNCTION
    GoImageMetaclass
DESCRIPTION
    GoImageMetaclass
**/
GoImageMetaclass::GoImageMetaclass()
{
    _has_config_precond = _has_config_postcond = GFALSE;
}

/**
FUNCTION
    Go* GoImageMetaclass::m_create() const
DESCRIPTION
    virtual private
**/
Go* GoImageMetaclass::m_create() const
{
    return (Go*) new GoImage;
}


/**
FUNCTION
    const char* GoImageMetaclass::m_classname() const
DESCRIPTION
    virtual private
**/
const char* GoImageMetaclass::m_classname() const
{
    return "GoImage";
}


/**
FUNCTION
    GoMetaclass* GoImageMetaclass::m_superclass() const
DESCRIPTION
    virtual private
**/
GoMetaclass* GoImageMetaclass::m_superclass() const
{
    return GoStdDS::metaclass();
}


/**
FUNCTION
    const GActionRec* GoImageMetaclass::m_actions () const
DESCRIPTION
    
**/
const GActionRec* GoImageMetaclass::m_actions () const
{
    /** unjustified warning with ATT 2.0 **/
    return NULL;
}


/*
 *-------------------
 * PRIVATE FUNCTIONS
 *-------------------
 */

/**
FUNCTION
    GoMetaclass* GoImage::m_metaclass() const
DESCRIPTION
    virtual private
**/
GoMetaclass* GoImage::m_metaclass() const {
    return GoImage::metaclass();
}


/**
FUNCTION
    void GoImage::m_dup(const Go* src, Go* dst, Go*, Go*,GBool,GBool)
DESCRIPTION
    virtual private
**/
void GoImage::m_dup
    (const Go* src, Go* parcopy, Go* prevcopy, GBool donext, GBool dochildren)
{
    GoStdDS::m_dup(src, parcopy, prevcopy, donext, dochildren);

    const GoImage *i_src = (const GoImage *) src;
      
    _width = i_src->width();
    _height = i_src->height();
    _origin = i_src->origin();
    _imagebrush.modify(i_src->imageBrush().shImageBrush());
    _pxlarray.modify(i_src->pixelArray().shPixelArray());
}


/**
FUNCTION
    GoImage::_imagebrush_changed(GoImage* goimage,
                                 const GImageBrush* imagebrush, ulong mask)
DESCRIPTION

**/

void GoImage::_imagebrush_changed(void* This,
				  /* const: compiler not enough cleaver */ GImageBrush*,
				  ulong mask)
{
    GoImage* goimg = (GoImage*)This;
    GBool was_mapped   = goimg->mapped();
    GBool draw_under   = GFALSE;

    if (was_mapped) {
	if (mask & GSTIPPLE || mask && GFILL_STYLE) {
	    goimg->RedrawBackAndFore();
	    draw_under = GTRUE;
	}
    }
 
    goimg->_old_shimagebrush->destroy();
    goimg->_old_shimagebrush = goimg->imageBrush().shImageBrush()->dup();

    
    if (was_mapped && !draw_under)
	goimg->RedrawForeOnly();
}


/**
FUNCTION
    GoImage_pxlarray_changed(GoImage* goimage,
                              const GPixelArray* pxlarray, ulong mask)
DESCRIPTION
    The pxlarray modification does not change the sizes of the Go.
    So we just have to tell the kernel that we have to redraw my clip.
**/

void  GoImage::_pxlarray_changed(void* This,
				 /* const compiler not enough cleaver */ GPixelArray*,
				 ulong)
 
{
    GoImage* goimg = (GoImage*)This;
    if (goimg->mapped())
	if (goimg->imageBrush().fillStyle() == GFILL_STIPPLED &&
	    goimg->imageBrush().stipple() != NULL)
	    goimg->RedrawBackAndFore();
	else
	    goimg->RedrawForeOnly();
}



/*
 *---------------------
 * PROTECTED FUNCTIONS
 *---------------------
 */

/**
FUNCTION
    GBool GoImage::m_contains(const GPoint& absPoint, GNumber maxDist) const
DESCRIPTION
    virtual protected
**/
GBool GoImage::m_contains(const GPoint& p, GNumber maxDist) const
{
    /** unjustified warning with ATT 2.0 for _imagebrush **/

    return interf()->imageContains(&((GoImage*)this)->_imagebrush, coordsys(),
				   _origin, _width, _height, p, maxDist);
}


/**
FUNCTION
    GoImage::m_add_clip(GRegion&) const
DESCRIPTION
    virtual protected
**/
void GoImage::m_add_clip(GRegion& clip) const
{
    /** unjustified warning with ATT 2.0 for _imagebrush **/

    interf()->imageClips(&((GoImage*)this)->_imagebrush, coordsys(), _origin,
			 _width, _height, &clip, NULL);
}


/**
FUNCTION
    GoImage::m_add_opaque_clip(GRegion&) const
DESCRIPTION
    virtual protected
**/
void GoImage::m_add_opaque_clip(GRegion& opaque_clip) const
{
    /** unjustified warning with ATT 2.0 for _imagebrush **/

    interf()->imageClips(&((GoImage*)this)->_imagebrush, coordsys(), _origin,
			 _width, _height, NULL, &opaque_clip);
}


/**
FUNCTION
    void GoImage::m_draw(const GRegion&) const
DESCRIPTION
    virtual protected
**/
void GoImage::m_draw(const GRegion& /*unused*/) const
{
    /** unjustified warning with ATT 2.0 for _imahebrush and _pxlarray **/
    
    if (_pxlarray.mask() & GPXL_VALUE  &&
	_pxlarray.values()->width &&
	_pxlarray.values()->height)
	interf()->image(&((GoImage*)this)->_imagebrush, coordsys(), _origin,
			_width, _height, &((GoImage*)this)->_pxlarray,
			GMAX_QUALITY);
}


/**
FUNCTION
    void GoImage::m_wired_draw(const GCoordSys&, const GPen& pen, GQuality quality) const
DESCRIPTION
    virtual protected
    Draw the image in wired mode. Just a draw the  outline.
**/
void GoImage::m_wired_draw(const GCoordSys& cs, const GPen& pen,
			   GQuality quality) const
{
    interf()->rectangle(&pen, NULL, cs, _origin,
			_width, _height, quality);
}


/**
  CALL pre and Post on Object and metaclass
  **/
GBool GoImage::call_config_precond(GNumber* w, GNumber* h, GPoint* origin) const
{
    return (_has_config_precond ?
	    CallPreconditions(goimageNconfigPrecond, w, h, origin) : GTRUE);
}

INLINE void GoImage::call_config_postcond()
{
    if (_has_config_postcond)
	CallPostconditions(goimageNconfigPostcond);
}


GBool GoImageMetaclass::call_config_precond(const GoImage* img,
						  GNumber* w, GNumber* h, GPoint* origin) const
{
    return (_has_config_precond ?
	    CallPreconditions(goimageNconfigPrecond, img, w, h, origin) : GTRUE);
}

INLINE void GoImageMetaclass::call_config_postcond(GoImage* img)
{
    if (_has_config_postcond)
	CallPostconditions(goimageNconfigPostcond, img);
}

/**
FUNCTION
    GoImage::config(GNumber w, GNumber h, const GPoint* origin)
DESCRIPTION
   If precond allow it, set _origin, _width, _height  by origin,w,h
   (possiblily modified by a precond)
   and call reset postconds.
**/
GBool GoImage::config(GNumber w, GNumber h, const GPoint* origin)
{
    GPoint new_O;
    if (origin) new_O = *origin;  else new_O = _origin;

    // we need local Numbers because parameters may be double;
    GNumber new_w;
    GNumber new_h;
    if (w == -1) new_w = _width;  else new_w = w;
    if (h == -1) new_h = _height; else new_h = h;
    
    if ((new_O != _origin || w != _width || h != _height) &&
	call_config_precond(&new_w, &new_h, &new_O) &&
        GoImage::metaclass()->call_config_precond(this, &new_w, &new_h, &new_O) &&
	(new_O != _origin || new_w != _width || new_h != _height)) {

	GBool was_mapped = mapped();
	GNumber old_w = _width;
	GNumber old_h = _height;
	GPoint  old_o = _origin;
  
	/**
	  inform the kernel before mofif
	  **/
	if (was_mapped && (old_w > new_w || old_h > new_h || new_O != old_o))
	    RedrawBackAndFore();

	/**
	  make the modif
	  **/
	_width = new_w;
	_height = new_h;
	_origin = new_O;
 	
    
	/**
	  inform the kernel after modif
	  **/
	MyClipHasChanged();
 
	if (was_mapped && old_o == new_O &&
	    (old_w < _width || old_w < _height))
	    RedrawForeOnly();

	// preverse this because thsi is not const in postcond;
	DifferDestroy();
	
	call_config_postcond();
        GoImage::metaclass()->call_config_postcond(this);

	// handle differed destroy request;
	HandleDestroy();

	// the action is done;
	return GTRUE;
    }

    // no modif;
    return GFALSE;
}


/**
  GADD/GREMOVE Precond and post condition On Object and metaclass
  **/

void GoImage::configPrecond(GBool func(void*, const GoImage*,
				      GNumber* w, GNumber* h, GPoint* origin),
			    void* closure, GAddOrRemove add)
{
    _has_config_precond =
	AddOrRemoveCondition(goimageNconfigPrecond, func, closure, add);
}

void GoImage::configPostcond(void func(void*, GoImage*),
			     void* closure, GAddOrRemove add)
{
    _has_config_postcond =
	AddOrRemoveCondition(goimageNconfigPostcond, func, closure, add);
}

void GoImageMetaclass::configPrecond(GBool func(void*, const GoImage*,
					       GNumber* w, GNumber* h, GPoint* origin),
				     void* closure, GAddOrRemove add)
{
    _has_config_precond =
	AddOrRemoveCondition(goimageNconfigPrecond, func, closure, add);
}

void GoImageMetaclass::configPostcond(void func(void*, GoImage*),
				      void* closure, GAddOrRemove add)
{
    _has_config_postcond =
	AddOrRemoveCondition(goimageNconfigPostcond, func, closure, add);
}




/**
FUNCTION
    GoImage::GoImage(const GShPixelArray* = NULL, const GShImageBrush* = NULL)
DESCRIPTION
    
**/

GoImage::GoImage(const GShImageBrush* imagebrush, const GShPixelArray* pxlarray)
{
    _has_config_precond = _has_config_postcond = GFALSE;
 
    _old_shimagebrush = _imagebrush.shImageBrush()->dup();

    _width = 1;   
    _height = 1;
    _origin.x = _origin.y = 0;

    _imagebrush.modifyPostcond(&GoImage::_imagebrush_changed, this, GADD);
    _pxlarray.modifyPostcond(&GoImage::_pxlarray_changed, this, GADD);

    if (imagebrush)
	_imagebrush.modify(imagebrush);
    if (pxlarray)
	_pxlarray.modify(pxlarray);
} 


/**
FUNCTION
    ~GoImage
DESCRIPTION
    free all the GoImage resources.
**/
GoImage::~GoImage()
{
    /**
      _imagebrush and _pxlarray are struct: -->automatic delete
      **/
    _old_shimagebrush->destroy();
}

/**
FUNCTION
    GoImage::create(const GShImageBrush* = NULL, const GShPixelArray* = NULL)
DESCRIPTION
    GoImage constructor with metaclass pre/post condition called.
   
**/
GoImage*  GoImage::create()
{
    return GoImage::create(NULL,NULL);
}

GoImage*  GoImage::create(const GShImageBrush* imagebrush, const GShPixelArray* pxlarray)
{
    GoImage* goimg = (GoImage*) Go::Create(GoImage::metaclass());
    if (goimg) {
	if (pxlarray)
	    goimg->pixelArray().modify(pxlarray);
	if (imagebrush)
	    goimg->imageBrush().modify(imagebrush);
    }
    return goimg;
}




/**
FUNCTION
    int GoImage::save(const GoImage* , GoStream*)
DESCRIPTION
    
**/
int GoImage::save(const GoImage* img, GoStream* gostream)
{
    const GShImageBrush* ib = img->imageBrush().shImageBrush();
    const GShPixelArray* pa = img->pixelArray().shPixelArray();
    
    return (gostream->writeNumber(img->width(), "width") &&
	    gostream->writeNumber(img->height(), "height") &&
	    gostream->writePoint (img->origin(), "origin") &&
	    gostream->saveObj    (ib, ib->classname()) &&
	    gostream->saveObj    (pa, pa->classname()) &&
	    GoStdDS::save(img, gostream));
}


static
int readGoImageParams(GoStream* gostream, GNumber* w, GNumber* h, GPoint* origin,
		      GShImageBrush** ib, GShPixelArray** pa)
{
    const char* class_name;
    
    if (! gostream->readNumber(w))
	return 0;
    if (! gostream->readNumber(h))
	return 0;
    if (! gostream->readPoint(origin))
	return 0;
    
    class_name = NULL;
    if (! gostream->restoreObj((void**)ib, &class_name))
	return 0;
    if (class_name == NULL || strcmp (class_name, GShImageBrush::classname()) != 0)
	return 0;
    
    class_name = NULL;
    if (! gostream->restoreObj((void**)pa, &class_name))
	return 0;
    if (class_name == NULL || strcmp (class_name, GShPixelArray::classname()) != 0)
	return 0;

    return 1;
}

/**
FUNCTION
    int GoImage::restore(GoImage** image, GoStream* gostream)
DESCRIPTION
    
**/
int GoImage::restore(GoImage** image, GoStream* gostream)
{
    GNumber w, h;
    GPoint origin;
    GShImageBrush* ib;
    GShPixelArray* pa;

    if (readGoImageParams(gostream, &w, &h, &origin, &ib, &pa) == 0)
	return 0;
    
    if ((*image = GoImage::create(ib, pa)) != NULL) {
	(*image)->_width = w;
	(*image)->_height = h;
	(*image)->_origin  = origin;

	return GoStdDS::readParams (*image, gostream);
    }
    return 0;
}


/**
FUNCTION
    int readParams(GoImage* image, GoStream* gostream)
DESCRIPTION
    uses by subclasses (if any) at restore time.
**/
int GoImage::readParams(GoImage* image, GoStream* gostream)
{
    GNumber w, h;
    GPoint origin;
    GShImageBrush* ib;
    GShPixelArray* pa;

    if (readGoImageParams(gostream, &w, &h, &origin, &ib, &pa) == 0)
	return 0;

    image->imageBrush().modify(ib);
    image->pixelArray().modify(pa);

    image->config(w, h, &origin);

    return GoStdDS::readParams(image, gostream);
}

