#ifdef REV_INFO
static char rcsid[] = "$Id: Composite.cc,v 1.2 1992/12/10 16:28:42 smadja Exp $";
#endif
/*****************************************************************************
MODULE:  Basic Graphic Objects
FILE:    Composite.cc
AUTHOR:  Oliver Jojic
DATE:    May 91

DESCRIPTION:
  See the Composite.h header.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include <Go/mComposite.h>
#include <Go/Composite.h>
#include <Go/GoStream.h>


static GoCompositeMetaclass* comp_metaclass = NULL;


GoCompositeMetaclass::~GoCompositeMetaclass ()
{
    if (this == comp_metaclass)
    {
	superclass()->RemoveSubclass(this);
	comp_metaclass = NULL;
    }
}


const char* GoCompositeMetaclass::m_classname () const
{
    return "GoComposite";
}


GoMetaclass* GoCompositeMetaclass::m_superclass () const
{
    return GoAbsDS::metaclass();
}


static GActionRec no_actions[] = { {NULL, NULL} };

const GActionRec* GoCompositeMetaclass::m_actions () const
{
    return no_actions;
}


Go* GoCompositeMetaclass::m_create () const
{
    return new GoComposite;
}


GoCompositeMetaclass* GoComposite::metaclass ()
{
    if (comp_metaclass == NULL)
    {
	comp_metaclass = new GoCompositeMetaclass;
	comp_metaclass->superclass()->AddSubclass(comp_metaclass);
      GoStream::bind(GoComposite::metaclass()->classname(),
		     (int (*) (const void*, GoStream*)) GoComposite::save,
		     (int (*) (void**, GoStream*)) GoComposite::restore);
    }

    return comp_metaclass;
}


void GoComposite::m_dup
    (const Go* src, Go* parcopy, Go* prevcopy, GBool donxt, GBool doch)
{
    GoAbsDS::m_dup(src, parcopy, prevcopy, donxt, doch);
}


GoMetaclass* GoComposite::m_metaclass () const
{
    return GoComposite::metaclass();
}


GRegion* GoComposite::d_get_clip () const
{
    const GRegion* r = &_clip;
    return (GRegion*)r;
}



/**
FUNCTION
    int GoComposite::save();
DESCRIPTION
    ** static member, protected **
**/
int GoComposite::save(const GoComposite* composite, GoStream* gostream) {
    return(GoAbsDS::save(composite, gostream));
}


/**
FUNCTION
    int GoComposite::readParams();
DESCRIPTION
    
**/
int GoComposite::readParams(GoComposite* composite, GoStream* gostream) {
    return GoAbsDS::readParams(composite, gostream);
}


/**
FUNCTION
    int GoComposite::restore();
DESCRIPTION
    
**/
int GoComposite::restore(GoComposite** composite, GoStream* gostream) {
    if (*composite = GoComposite::create())
	return GoComposite::readParams(*composite, gostream);
    return GFALSE;
}

GBool GoComposite::m_contains (const GPoint& abspt, GNumber maxd) const
{
    /*
     * We cannot call "this->subtreeContains()" because
     * the crossType of this GoComposite may be "CrossAsLeaf".
     */

    for (Go* ch = Child(); ch; ch = ch->Next())
	if (ch->crossType() != GCROSS_AS_NOTHING &&
	    ch->subtreeContains(abspt, maxd))
	    return GTRUE;

    return GFALSE;
}

