#ifdef REV_INFO
static char rcsid[] = "$Id: Form.cc,v 1.10 1992/11/16 15:56:27 andre Exp $";
#endif
/*****************************************************************************
MODULE:  GoText
FILE:	 Form.cc
AUTHOR:  Michel Andre
DATE:    August 91
 

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
*****************************************************************************/

#include	<Go/GoInterf.h>
#include	<Go/Form.h>
#include	<Go/Text.h>


#define FLOAT_EQUAL(x,y)	(fabs ((x) - (y)) < 0.0001)


/**
FUNCTION
    GFormatterCtxt::GFormatterCtxt();
DESCRIPTION
    The formatter context constructor.
**/

GFormatterCtxt::GFormatterCtxt()
{
    _formatter = NULL;
}


/**
FUNCTION
    classname();
DESCRIPTION
    The formatter classname method.
**/

const char*
GFormatter::classname(){
    return "GFormatter";
}


/**
FUNCTION
    getClassname();
DESCRIPTION
    The formatter getClassname method.
**/

const char*
GFormatter::getClassname() const{
    return classname();
}


/**
FUNCTION
    GFormatter();
DESCRIPTION
    The formatter constructor.
**/

GFormatter::GFormatter()
{
    _text = NULL;
    _enableReframe = GTRUE;
}


/**
FUNCTION
    ~GFormatter();
DESCRIPTION
    The formatter destructor.
**/

GFormatter::~GFormatter()
{
    delete _ctxt;
    if (_text) _text->_formatter = NULL;
}


/**
FUNCTION
    draw (const GRegion&) const;
DESCRIPTION
    Method used to draw the text background.
**/

void
GFormatter::draw (const GRegion&) const
{
    _text->interf()->rectangle (NULL, &(_text->backgroundBrush()),
	_text->Abscs(), GPoint (-_text->_leftMargin,
	    -_text->_topMargin), _text->_width +
		_text->_leftMargin + _text->_rightMargin,
		    _text->_height + _text->_topMargin +
			_text->_bottomMargin);
}


/**
FUNCTION
    clip (GRegion&) const;
DESCRIPTION
    The GSimpleFormatter clip method.
**/

void
GFormatter::clip (GRegion& reg) const
{
    GAdd (reg, _text->Abscs().to (GBox (
	-_text->_leftMargin,
	-_text->_topMargin,
	_text->_width + _text->_rightMargin,
	_text->_height + _text->_bottomMargin)), reg
    );
}


/**
FUNCTION
    setString();
DESCRIPTION
    The formatter setString method.
	- deletion of the lines
	- initialisation of the _firstLine and
	    _lastLine variables.
	
**/

void
GFormatter::setString()
{
GoTextLine* line = NULL;

    /* destroy of the lines */
    for (GoTextLine* prev = _text->_firstLine; prev;
	prev = line)
    {
	line = prev->NextLine();
	prev->destroy();
    }

    _text->_firstLine = NULL;
    _text->_lastLine = NULL;
}


/**
FUNCTION
    justification (GJustification);
DESCRIPTION
**/

void
GFormatter::justification (GJustification oldJ)
{
    if (oldJ != _text->_justification) organize();
}


void
GFormatter::organize (Go* line1, Go* line2)
{
    if (line1 == NULL)
	line1 = _text->_firstLine;

    if (line2 == NULL)
	line2 = _text->_lastLine;

    /* for each line between line1 and line2 */
    for (GoTextLine* line = GLINE (line1); line;
	line = line->NextLine())
    {
	GPoint lineOrig (lineOrigin (line));
	if (line->Relcs().O.y != lineOrig.y)
	    line2 = _text->_lastLine;

	GNumber alinea = indentation (line);
	GNumber lineWidth = textWidth (line);
	line->organize (lineWidth);

	/* place horizontally the line */
	switch (_text->_justification){
	    case GLEFT_JUSTIFICATION:
		lineOrig.x += alinea;
		break;

	    case GCENTER_JUSTIFICATION:
		lineOrig.x += (lineWidth - line->_width) / 2;
		break;

	    case GRIGHT_JUSTIFICATION:
		lineOrig.x += lineWidth - line->_width;
		break;

            case GLR_JUSTIFICATION:
                break;
        }

        if (!FLOAT_EQUAL (lineOrig.x, line->Relcs().O.x) ||
           (!FLOAT_EQUAL (lineOrig.y, line->Relcs().O.y)))
            line->move (lineOrig, _text);

        if (line == line2) break;
    }

    /* update text width and height */
    _text->_width = 0;
    _text->_width = _text->_Width();
    _text->_height = 0;
    for (line = _text->_firstLine; line;
	line = line->NextLine())
    {
	_text->_width = max (_text->_width,
	    line->Relcs().O.x + line->_width);
	_text->_height = max (_text->_height,
	    line->Relcs().O.y + line->_height);
    }
}


GNumber
GFormatter::textWidth (GoTextLine* line) const
{
    return GTEXT (line->parent())->_width;
}


GPoint
GFormatter::lineOrigin (GoTextLine* line) const
{
    return line->Relcs().O;
}


/**
FUNCTION
    refreshLine (GoTextLine*) const;
DESCRIPTION
    Method used to refresh the given line region.
**/

void
GFormatter::refreshLine (GoTextLine* line) const
{
    GPoint lineOrig = lineOrigin (line);
    GBox box = GBox (lineOrig, GPoint (
	lineOrig.x + textWidth (line) + _text->cursorPen().lineWidth(),
	    lineOrig.y + line->_height));

    GRegion reg = _text->Abscs().to (box);

    if (_text->backgroundMapped())
	_text->RedrawForeOnly (reg);
    else
	_text->RedrawBackAndFore (reg);
}


/**
FUNCTION
    xyToLine (const GPoint&) const;
DESCRIPTION
    Returns the line pointed by pt.
**/

GoTextLine*
GFormatter::xyToLine (const GPoint& pt) const
{
    if (pt.y < 0 || pt.y > _text->_height)
	return NULL;

    GoTextLine* prevLine = _text->_firstLine;
    for (GoTextLine* line = prevLine;
	line && line->Relcs().O.y < pt.y;
	    line = line->NextLine())
		prevLine = line;

    return prevLine;
}


/**
FUNCTION
    GPos xyToPos (GPoint&);
DESCRIPTION
    Returns the position of the pointed char.
**/

GPos
GFormatter::xyToPos (GPoint& pt) const
{
    if (pt.y < 0) return 0;
    else if (pt.y > _text->_height)
	return _text->length();

    GoTextLine* line = xyToLine (pt);
    if (!line) return _text->length();

    GNumber lineXOrig = line->Relcs().O.x;
    GNumber segXOrig = lineXOrig;
    char* str = _text->_string;

    /* search of the segment */
    for (GoTextSegment* seg = GSEG (line->Child());
	seg->Next(); seg = GSEG (seg->Next()))
	     if (pt.x <= (lineXOrig +
                   seg->Next()->Relcs().O.x)) break;

    segXOrig = seg->Relcs().O.x;

    /* search inside the segment */
    char *pp = str + seg->_firstPos;
    GPos nb = 1;
    GPos len = seg->_lastPos - seg->_firstPos;

    /* compute the width of of seg before pt.x */
    GNumber widthPortion = pt.x - lineXOrig - segXOrig;

    GoInterf* interf = _text->interf();
    while (interf->stringWidth (pp, nb, seg->lineHeight(),
	seg->stringShTextBrush()->font()) <= widthPortion
	    && nb <= len) nb++;

    return seg->_firstPos + --nb;
}


/**
FUNCTION
    posToXy (GPos pos);
DESCRIPTION
    Returns the coordinates of the given position.
**/

GPoint
GFormatter::posToXy (GPos pos) const
{
    if (pos == -1) pos = _text->_currentPos;
    GoTextSegment* seg = _text->posToSeg (pos);

    /* a cause bug compilateur retour Relcs() */
    GCoordSys csLine = GLINE (seg->parent())->Relcs();
    GCoordSys csSegment = seg->Relcs();

    GPoint pt (csLine.O.x + csSegment.O.x,
	csLine.O.y + csSegment.O.y);

    pt.x += _text->interf()->stringWidth (
	_text->_string + seg->_firstPos, pos - seg->_firstPos,
	    seg->lineHeight(), seg->stringShTextBrush()->font());

    return pt;
}

