#ifdef REV_INFO
static char rcsid[] = "$Id: SFEdit.cc,v 1.7 1992/12/23 15:49:14 andre Exp $";
#endif
/*****************************************************************************
MODULE:  GoText
FILE:	 Edit.cc
AUTHOR:  Michel Andre
DATE:    August 91
 

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
*****************************************************************************/

#include <Go/GoInterf.h>
#include <Go/Text.h>
#include <Go/SimpleForm.h>


#define SIZE_W  10
#define _update_W(cs,w) if ((cs).I.size()*(w) < SIZE_W) (w) = SIZE_W / (cs).I.size()


void
GSimpleFormatter::setString()
{
GoTextSegment* seg;

    GBool was_mapped;
    if (was_mapped = _text->mapped()) _text->TempUnmap();

    Go* rightBrother = _text->_lastLine->Next();

    /* destroy of the existing lines */
    GFormatter::setString();

    /* NULL string */
    if (!*(_text->_string)){
	GoTextLine* newLine = GoTextLine::create();
	newLine->parentRelativeReparent (_text, rightBrother);

	_text->_firstLine = newLine;
	_text->_lastLine = newLine;

	seg = GSEG (newLine->Child());
	seg->_firstPos = seg->_lastPos = 0;
    }

    /* string not empty */
    else{
        int l_index = 0;
        /* creation of the new lines */
        for (int nb_lines = _text->GoTextual::nbLines();
	    l_index < nb_lines; l_index++)
	{
	    GoTextLine* newLine = GoTextLine::create();
	    newLine->parentRelativeReparent (_text, rightBrother);

	    if (!_text->_firstLine)
		_text->_firstLine = newLine;

	    _text->_lastLine = newLine;

	    seg = GSEG (newLine->Child());
	    seg->_firstPos = _text->GoTextual::begLinePos (l_index);
	    seg->_lastPos = _text->GoTextual::endLinePos (l_index);
        }
    }

    /* update the segments width */
    if (_text->interf()){
	for (seg = GSEG (_text->_firstLine->Child());
	    seg; seg = seg->myNext())
	{
	    seg->updateWidth();
	    seg->_height = _text->_lineHeightMax;
	}

	organize();
    }

    if (was_mapped) _text->TempMap();
}


void
GSimpleFormatter::insertChar (char Char, GPos pos)
{
Go* firstSegToReparent;

    /* search of the line containing Char */
    GoTextLine* line = _text->_firstLine;
    for (int i = _text->GoTextual::lineNumber (pos);
	i--; line=line->NextLine());

    refreshLine (line);
    GCoordSys cs = _text->Abscs();
    GoTextSegment* seg = _text->posToSeg (pos);

    if (Char == '\n'){

        refreshLine (line);
	/* creation of a new GoTextLine */
	GoTextLine* newLine = GoTextLine::create();
	newLine->parentRelativeReparent (_text, line->Next());
	if (line == _text->_lastLine)
	    _text->_lastLine = newLine;

	newLine->_height = seg->_height;
	GSEG (newLine->Child())->destroy();

	GoTextSegment* newSeg = GSEG (seg->dup());
	newSeg->parentRelativeReparent (newLine);
	newSeg->_firstPos = pos;
	newSeg->_lastPos = pos;

	/* the following character is different of '\n' */
	if (*(_text->_string + pos + 1) != '\n'){

	    /* the entire segment is moved */
	    if (seg->_firstPos == pos){

		/* pos is the beginning of the line */
		if (seg == line->Child()){
		    newSeg->parentRelativeReparent (line, seg);
		    line->_height = seg->_height;
		    newSeg->updateWidth();
		}
		else
		    newSeg->destroy();

		firstSegToReparent=seg;
	    }

	    /* the second part of the segment is moved */
	    else{
		/* reframe of the cutted parts */
		newSeg->_firstPos = pos;
		newSeg->_lastPos = seg->_lastPos;
		seg->_lastPos = pos;
		seg->updateWidth();
		firstSegToReparent=seg->Next();
	    }

	    /* reparenting of the segments placed at the right of pos */
	    Go* next=NULL;
	    for (Go* go=firstSegToReparent; go; go=next){
		next = go->Next();
		go->parentRelativeReparent (newLine, NULL);
	    }
	}

	/* shift of the next segments */
	_text->shiftPos ((GoTextSegment*)newLine->Child(), 1);

	GSEG (newLine->Child())->updateWidth();
	organize();
    }

    /* Char != '\n' */
    else{
	GNumber oldSegWidth = seg->_width;
	seg->_lastPos++;
	seg->updateWidth();

        /* point used for the move */
        GPoint shiftPt (seg->_width - oldSegWidth, 0);

	/* update the width of line */
	Go* par = seg->parent();
	GoTextLine *line = GLINE (par);
        line->_width = 0;
        for (Go* gs = line->Child(); gs; gs=gs->Next())
            line->_width += GSEG (gs)->_width;

	/* shift to the right of its following segments */
	for (seg = seg->myNext(); seg; seg=seg->myNext()){
	    ++seg->_firstPos;
	    ++seg->_lastPos;

	    /* move of the following segments of seg */
	    if (seg->parent() == par)
		seg->move (shiftPt, seg);
	}

	/* the text width changed */
	if (line->_width > _text->_width){
	    _text->MyClipHasChanged();
	    if (_text->_justification == GLEFT_JUSTIFICATION){

		/* update _text->_width */
		GNumber oldWidth = _text->_width;
		_text->_width = line->_width;

                /* redraw the clip difference */
                _text->RedrawForeOnly (cs.to (GBox (oldWidth,
		    -_text->_topMargin,
		    _text->_width + _text->_rightMargin,
		    _text->_height + _text->_bottomMargin)));
	    }

	    else{
		_text->_width = line->_width;
		_text->RedrawBackAndFore();
		organize();
		return;
	    }
	}

	switch (_text->_justification){
	    case GLEFT_JUSTIFICATION:
	    case GLR_JUSTIFICATION:
		return;

	    case GCENTER_JUSTIFICATION:
		shiftPt.x /= -2;
		break;

	    case GRIGHT_JUSTIFICATION:
		shiftPt.x *= -1;
		break;
	}

	line->move (shiftPt, line);
    }
}


void
GSimpleFormatter::deleteString (const char*, GPos delSize, GPos pos)
{
    /* the string is empty */
    if (!_text->length()){
	setString();
	return;
    }

    GPos pos1 = pos;
    GPos pos2 = pos + delSize;
    _text->createPortion (pos1, pos2);
    GoTextSegment* s1 = _text->posToSeg (pos1);
    GoTextSegment* s2 = _text->posToSeg (pos2);
    Go* l1 = s1->parent();
    Go* l2 = s2->parent();

    /* s1 is the first segment of its line */
    GBool firstCharOfTheLine = !s1->Previous();

    GoTextSegment* s = s1;
    while (s != s2){
	GoTextSegment* sn = s->myNext();
	Go* l = s->parent();
	s->destroy();
	if (!(l->Child())){

	    /* l1 will be used for reparenting later */
	    if (l == l1)
		l1 = GLINE (l1)->PreviousLine();
	    l->destroy();
	}
	s = sn;
    }

    if (s2->_lastPos == pos2)
	s2->_firstPos = s2->_lastPos;

    if (l1 != l2 && l1 != NULL && !firstCharOfTheLine){
        /* reparenting of the childs of l2 on l1 */
        Go* gn;
        for (Go* go = l2->Child(); go; go=gn){
            gn = go->Next();
            go->parentRelativeReparent (l1);
        }
        l2->destroy();
    }

    /* update _firstLine */
    Go* child = _text->Child();
    while (child->getMetaclass() != GoTextLine::metaclass())
	child = child->Next();
    _text->_firstLine = GLINE (child);
    
    /* update _lastLine */
    while (child->Next() && child->Next()->getMetaclass() ==
	GoTextLine::metaclass()) child = child->Next();
    _text->_lastLine = GLINE (child);

    /* shift by step -delSize of the following segments */
    _text->shiftPos (s2, -delSize);

    s1 = _text->posToSeg (pos1);
    assembly (s1, s1);

    organize();
}


void
GSimpleFormatter::deleteChar (char Char, GPos pos)
{
GoTextSegment *seg;

    GCoordSys cs = _text->Abscs();

    /* search of the line containing Char */
    GoTextLine* line = _text->_firstLine;
    for (int i = _text->GoTextual::lineNumber (pos);
	i--; line=line->NextLine());

    refreshLine (line);

    if (Char == '\n'){

	GoTextLine* lineToDel = line->NextLine();
    
	/* shift by step -1 of the following segments */
	_text->shiftPos (GSEG (lineToDel->Child()), -1);

	/* reparenting of the segments of the next line */
	seg = GSEG (lineToDel->Child());
	if (seg->_lastPos != seg->_firstPos){
	    Go* next=NULL;
	    for (Go* go=seg; go; go=next){
		next = go->Next();
		go->parentRelativeReparent (line, NULL);
	    }

	    GoTextSegment* prev = GSEG (seg->Previous());
	    /* destroy of the first segment of line if it is a null one */
	    if (prev->_firstPos == prev->_lastPos)
	    	prev->destroy();
	    else
		/* the new neighbours segments have the same properties */
		if (seg->equiv (prev)){
		    prev->_lastPos = seg->_lastPos;
		    prev->updateWidth();
		    seg->destroy();
		}
	}

	/* the destroyed line may be the last */
	if (lineToDel == _text->_lastLine)
	    _text->_lastLine = line;

	/* destroy of the unused line */
	lineToDel->destroy();

	organize();
    }

    /* Char != '\n' */
    else{
	seg = _text->posToSeg (pos);

        /* refresh the current line */
        /* because of italicism */
        GRegion reg (cs.to (GBox (0, line->Relcs().O.y, _text->_width, 
	    line->Relcs().O.y + line->_height)));

        if (_text->backgroundMapped())
	    _text->RedrawForeOnly (reg);
        else
	    _text->RedrawBackAndFore (reg);

	/* redraw the segment and its cutted part before reframe */
	if (_text->_justification == GCENTER_JUSTIFICATION && !seg->Next())
	    seg->RedrawBackAndFore();

	GNumber oldSegWidth = seg->_width;
	seg->_lastPos--;
	seg->updateWidth();

	line = GLINE (seg->parent());

	/* move of the following segments */
	GPoint shiftPt (seg->_width - oldSegWidth, 0);
	for (Go* go=seg->Next(); go; go=go->Next())
	    go->move (shiftPt, go);

	/* shift to the left of its following segments */
	_text->shiftPos (seg->myNext(), -1);

	/* destroy of the segment if _lastPos == _firstPos */
	if (seg->_firstPos == seg->_lastPos && (seg->Next() || seg->Previous())){
	    seg->destroy();
	    GNumber segmentMaxHeight = 0;
	    for (go=line->Child(); go; go=go->Next())
		segmentMaxHeight = max(segmentMaxHeight, ((GoTextSegment*)go)->_height);
	    if (segmentMaxHeight < line->_height){
		organize();
		return;
	    }
	}

	/* line may be the longest line of the text */
	if (line->_width == _text->_width){

	    /* search of the longest line */
	    GNumber lineMaxWidth = 0;
	    line->_width += shiftPt.x;
	    for (go=_text->Child(); go; go=go->Next())
	        lineMaxWidth = max(lineMaxWidth, GLINE (go)->_width);

	    if (_text->_width > lineMaxWidth){
	        if (_text->_justification == GLEFT_JUSTIFICATION){
		    GNumber oldWidth = _text->_width;
		    _text->_width = lineMaxWidth;
                    _text->_width = _text->_Width();
		    _text->MyClipHasChanged();

		    /* the new right rectangle must be drawn */
		    _text->RedrawBackAndFore (cs.to (GBox (
			_text->_width, -_text->_topMargin,
			oldWidth + _text->_rightMargin,
			_text->_height + _text->_bottomMargin)));
	        }

	        else{
		    organize();
		    return;
	        }
	    }
	}

	switch (_text->_justification){
	    case GCENTER_JUSTIFICATION:
		shiftPt.x /= -2;
		break;

	    case GRIGHT_JUSTIFICATION:
		shiftPt.x *= -1;
		break;

	    default:
		shiftPt.x = 0;
		break;
	}

	line->move (shiftPt, line);
    }
}


/**
FUNCTION
    insertString (const char*, GPos, GPos)
DESCRIPTION
    This function is used to insert the string at the given position.
**/

void
GSimpleFormatter::insertString (const char* string, GPos stringLength, GPos pos)
{
    /* assume the string to be filled */
    if (stringLength == 0) return;

    GoTextSegment* currentSeg = _text->posToSeg (pos);
    GStyle* attOrig = currentSeg->_att;
    GBool brokenLine = GFALSE;

    for (GPos index = 0;  index < stringLength; index++)
	if (*(string + index) == '\n'){
	    brokenLine = GTRUE;
	    break;
	}

    GoTextSegment* firstSegToReparent = NULL;

    if (brokenLine){

	/* seg is to be cutted */
	if (pos < currentSeg->_lastPos){
	    firstSegToReparent = GSEG (currentSeg->dup());
	    firstSegToReparent->parentRelativeReparent (currentSeg->parent(),
		currentSeg->Next());
	    firstSegToReparent->_firstPos = pos;
	    firstSegToReparent->_lastPos = currentSeg->_lastPos;
	    currentSeg->_lastPos = pos;
	}
	else
	    firstSegToReparent = GSEG (currentSeg->Next());
    }

    for (index = 0; index < stringLength; index++){
	if (*(string + index) == '\n'){
	    currentSeg->_lastPos = pos + index;
	    currentSeg->updateWidth();
	    GoTextLine* newLine = GLINE (GoTextLine::create());
	    newLine->parentRelativeReparent (_text, currentSeg->parent()->Next());
	    currentSeg = GSEG (newLine->Child());

	    /* update currentSeg fields */
	    if (attOrig != NULL)
		currentSeg->_att = new GStyle (attOrig);
	    currentSeg->_firstPos = pos + index + 1;

	    /* update currentSeg->_height */
	    currentSeg->_height = _text->interf()->fontStrikeoutAscent
		(currentSeg->lineHeight(), currentSeg->stringShTextBrush()->font()) +
		    _text->interf()->fontStrikeoutDescent (currentSeg->lineHeight(),
			currentSeg->stringShTextBrush()->font());
	}
    }

    /* update _lastLine if necessary */
    if (brokenLine){

	currentSeg->_lastPos = pos + stringLength;

	Go* child = _text->_firstLine;
	while (child->Next() && child->Next()->getMetaclass()
	    == GoTextLine::metaclass()) child = child->Next();
	_text->_lastLine = GLINE (child);
    }

    else
	currentSeg->_lastPos += stringLength;

    /* reparenting of the segments */
    if (firstSegToReparent)
	for (Go* s = firstSegToReparent; s; s = s->Next())
	    s->parentRelativeReparent (currentSeg->parent(), NULL);

    currentSeg->updateWidth();

    /* shift the following segments */
    _text->shiftPos (currentSeg->myNext(), stringLength);

    reframe();
}


void
GSimpleFormatter::replaceChar (char, GPos pos)
{
    GCoordSys cs = _text->Abscs();

    /* search of the line containing Char */
    int i = _text->GoTextual::lineNumber (pos);
    for (GoTextLine* line = _text->_firstLine; i--; line = line->NextLine());

    refreshLine (line);

    /* segment containing pos */
    GoTextSegment* seg = _text->posToSeg (pos);
    GNumber oldSegWidth = seg->_width;
    seg->updateWidth();

    /* refresh the current line */
    /* because of italicism */
    GRegion reg (cs.to (GBox (0, line->Relcs().O.y, _text->_width, 
	line->Relcs().O.y + line->_height)));

    if (_text->backgroundMapped())
	_text->RedrawForeOnly (reg);
    else
	_text->RedrawBackAndFore (reg);

    /* move the following segments of seg */
    GPoint shiftPt (seg->_width - oldSegWidth, 0);
    for (Go* go=seg->Next(); go; go=go->Next())
	go->move (shiftPt, go);

    /* update the line width */
    line->_width += shiftPt.x;

    /* the text width increase */
    if (line->_width > _text->_width){
	if (_text->_justification == GLEFT_JUSTIFICATION){

	    /* update _text->_width */
	    GNumber oldWidth = _text->_width;
	    _text->MyClipHasChanged();
	    _text->_width = line->_width;

            /* redraw the clip difference */
            _text->RedrawForeOnly (cs.to (GBox (oldWidth,
		-_text->_topMargin,
		_text->_width + _text->_rightMargin,
		_text->_height + _text->_bottomMargin)));
	}
	else{
	    organize();
	    return;
	}
    }

    /* line may be the longest line of the text */
    if (line->_width - shiftPt.x == _text->_width && shiftPt.x < 0){

	/* search of the longest line */
	GNumber lineMaxWidth = 0;
	for (GoTextLine* line1=_text->_firstLine; line1; line1=line1->NextLine())
	    lineMaxWidth = max(lineMaxWidth, line1->_width);

	if (_text->_width > lineMaxWidth){
	    if (_text->_justification == GLEFT_JUSTIFICATION){
		GNumber oldWidth = _text->_width;
		_text->_width = lineMaxWidth;
                _text->_width = _text->_Width();
		_text->MyClipHasChanged();

		/* the new right rectangle must be drawn */
		_text->RedrawBackAndFore (cs.to (GBox (
		    _text->_width, -_text->_topMargin,
		    oldWidth + _text->_rightMargin,
		    _text->_height + _text->_bottomMargin)));
	    }
	    else{
		organize();
		return;
	    }
	}
    }

    switch (_text->_justification){
	case GCENTER_JUSTIFICATION:
	    shiftPt.x /= -2;
	    break;

	case GRIGHT_JUSTIFICATION:
	    shiftPt.x *= -1;
	    break;

	default:
	    shiftPt.x = 0;
	    break;
    }

    line->move (shiftPt, line);
}


void
GSimpleFormatter::replaceString (GPos pos, GPos oldStringSize, char*, GPos newStringSize)
{
    GPos breakPosition = pos + oldStringSize;

    /* cut if necessary the limits of the old sub-string */
    _text->createPortion (pos, breakPosition);

    GoTextSegment* seg = _text->posToSeg (pos);
    Go* parent = seg->parent();

    /* creation of a GoTextSegment with the new sub-string */
    GoTextSegment* newSeg = (GoTextSegment*)seg->dup();
    newSeg->reparent (parent, seg);
    newSeg->_firstPos = pos;
    newSeg->_lastPos = pos + newStringSize;
    newSeg->updateWidth();

    /* deletion of the intermediate segments */
    for (GoTextSegment* segNext = NULL; seg && seg->_lastPos <= breakPosition; seg = segNext){
	segNext = seg->myNext();
	seg->destroy();
    }

    /* shift of the following segments */
    GPos step = newStringSize - oldStringSize;
    _text->shiftPos (newSeg->myNext(), step);

    GoTextSegment* segFirst = newSeg;
    GoTextSegment* segLast = _text->posToSeg (pos + newStringSize);
    assembly (segFirst, segLast);

    /* reframe */
    reframe();
}
