#ifdef REV_INFO
static char rcsid[] = "$Id: SimpleForm.cc,v 1.7 1992/11/19 08:44:26 andre Exp $";
#endif
/*****************************************************************************
MODULE:  GoText
FILE:	 Simpleform.cc
AUTHOR:  Michel Andre
DATE:    August 91
 

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
*****************************************************************************/

#include <Go/GoStream.h>
#include <Go/Text.h>
#include <Go/SimpleForm.h>

extern GRegion	oldClip;


/**
FUNCTION
    GSimpleFormatter();
DESCRIPTION
    The SimpleFormatter constructor.
**/

GSimpleFormatter::GSimpleFormatter()
{
    _ctxt = new GSimpleFormatterCtxt;
    _ctxt->_formatter = this;
}


/**
FUNCTION
    init();
DESCRIPTION
    The SimpleFormatter init method for stream binding.
**/

static GBool _gostream_binding = GFALSE;

void
GSimpleFormatter::init()
{
    if (!_gostream_binding){
	GoStream::bind(GSimpleFormatter::classname(),
	    (int (*)(const void*, GoStream*))
		GSimpleFormatter::save,
	    (int (*)(void**, GoStream*))
		GSimpleFormatter::restore);
        _gostream_binding = GTRUE;
    }
}


/**
FUNCTION
    classname();
DESCRIPTION
    The SimpleFormatter classname method.
**/

const char*
GSimpleFormatter::classname(){
    return "GSimpleFormatter";
}


/**
FUNCTION
    getClassname();
DESCRIPTION
    The SimpleFormatter getClassname method.
**/

const char*
GSimpleFormatter::getClassname() const{
    return classname();
}


/**
FUNCTION
    dup();
DESCRIPTION
    The SimpleFormatter dup method.
**/

GFormatter*
GSimpleFormatter::dup()
{
    return (new GSimpleFormatter());
}


/**
FUNCTION
    assembly();
DESCRIPTION
    The SimpleFormatter assembly method.
**/

void
GSimpleFormatter::assembly (GoTextSegment* s1, GoTextSegment* s2)
{
    GoTextLine* line1 = GLINE (s2->parent());

    for (GoTextLine* line = GLINE (s1->parent());;
	line=line->NextLine())
    {
	GoTextSegment* prevSeg = GSEG (line->Child());
	for (GoTextSegment* seg = GSEG (prevSeg->Next());
	    seg; seg= GSEG (seg->Next()))
	{
	    if (prevSeg->equiv (seg)){
		prevSeg->_lastPos = seg->_lastPos;
		prevSeg->updateWidth();
		seg->destroy();
		seg = prevSeg;
	    }
	    else
		prevSeg = seg;
	}

	if (line == line1) break;
    }
}


/**
FUNCTION
    restore();
DESCRIPTION
    The SimpleFormatter restore method.
**/

int
GSimpleFormatter::restore (GSimpleFormatter** form, GoStream* gostream)
{
    *form = new GSimpleFormatter();
    return GFormatter::readParams (*form, gostream);
}


GPoint
GSimpleFormatter::lineOrigin (GoTextLine* line) const
{
    GNumber height = 0.0;
    for (GoTextLine* l = _text->_firstLine; l != line;
	l = l->NextLine())
	    height += l->_height + _text->_interline;

    return GPoint (0.0, height);
}


void
GSimpleFormatter::organize (Go* line1, Go* line2)
{
    if (line1 == NULL) line1 = _text->_firstLine;
    if (line2 == NULL) line2 = _text->_lastLine;

    /* organize the given lines */
    for (GoTextLine* line = GLINE (line1); line;
        line = line->NextLine())
    {
        line->organize (0);
        if (line == line2) break;
    }

    /* update text->_width and text->_height */
    _text->_width = _text->_height = 0;
    for (line = _text->_firstLine; line;
        line = line->NextLine())
    {
        _text->_width = max(_text->_width, line->_width);
        _text->_height += line->_height;
    }

    /* organize the text */
    GFormatter::organize (line1, line2);

    /* redraw the text clip differences */
    GRegion deltaClip;
    GRegion myClip = _text->myClip();

    /* the clip may have increased */
    GSub (myClip, oldClip, deltaClip);
    if (!deltaClip.empty()){
	_text->MyClipHasChanged();
	_text->RedrawForeOnly (deltaClip);
    }

    /* the clip may have decreased */
    GSub (oldClip, myClip, deltaClip);
    if (!deltaClip.empty()){
	_text->MyClipHasChanged();
	_text->RedrawBackAndFore (deltaClip);
    }
}

