#ifdef REV_INFO
static char rcsid[] = "$Id: LabelEdit.cc,v 1.3 1992/10/20 14:01:39 andre Exp $";
#endif
/*****************************************************************************
MODULE:  	GoLabel
FILE:    	Edit.cc
AUTHOR:		Andre Michel
DATE:    	March 1991

DESCRIPTION:
  GoLabel management (See GoTextual.h and GoLabel.h)

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#include <Go/GoInterf.h>
#include <Go/Label.h>


/**
FUNCTION
    _insertChar (char, GPos pos);
DESCRIPTION
    This function allows to insert a char Char at the
    position pos. The current position is set to pos + 1;
**/

void
GoLabel::_insertChar (char, GPos pos)
{
    GCoordSys cs = Abscs();

    /* beginning of the modified zone */
    GNumber currentPosX = interf()->stringWidth
	(_string, pos, _lineHeight, _stringTextBrush.font());

    /* compute of the new width */
    _width = _Width();
    MyClipHasChanged();

    /* redraw of the modified zone */
    GBox clipBox = cs.from (myClip().bigbox());
    clipBox.xmin = currentPosX - cursorPen().lineWidth();

    /* region to redraw */
    GRegion reg = cs.to (clipBox);

    if (backgroundMapped())
	RedrawForeOnly (reg);
    else
	RedrawBackAndFore (reg);
}


/**
FUNCTION
    _insertString (const char *str, GPos len, GPos pos);
DESCRIPTION
    Inserts the given string at the position pos.
    Same work than to insert a character.
**/

void
GoLabel::_insertString (const char*, GPos, GPos pos)
{
    _insertChar (0, pos);
}


/**
FUNCTION
    _setString (const char*, GPos);
DESCRIPTION
    Sets the given string.
**/

void
GoLabel::_setString (const char*, GPos)
{
    GCoordSys cs = Abscs();

    /* get clip before setString */
    GBox oldClipBox = cs.from (myClip().bigbox());

    /* compute of the new width */
    _width = _Width();
    MyClipHasChanged();

    /* get clip after setString */
    GBox newClipBox = cs.from (myClip().bigbox());

    /* RedrawBackAndFore of the two clips right difference */
    if (oldClipBox.xmax > newClipBox.xmax)
	RedrawBackAndFore (cs.to (GBox (newClipBox.xmax, newClipBox.ymin,
	    oldClipBox.xmax, oldClipBox.ymax)));

    /* redraw of the modified zone */
    GRegion reg = cs.to (newClipBox);

    if (backgroundMapped())
	RedrawForeOnly (reg);
    else
	RedrawBackAndFore (reg);
}



/**
FUNCTION
    _deleteChar (char, GPos);
DESCRIPTION
    This function allows to delete the character at the
    position pos.
**/

void
GoLabel::_deleteChar (char, GPos pos)
{
    GCoordSys cs = Abscs();

    /* get clip before deleteChar */
    GBox oldClipBox = cs.from (myClip().bigbox());

    /* beginning of the modified zone */
    GNumber currentPosX = interf()->stringWidth
	(_string, pos, _lineHeight, _stringTextBrush.font());

    /* compute of the new width */
    _width = _Width();
    MyClipHasChanged();

    /* get clip after deleteChar */
    GBox newClipBox = cs.from (myClip().bigbox());

    /* RedrawBackAndFore of the two clips difference */
    RedrawBackAndFore (cs.to (GBox (newClipBox.xmax, newClipBox.ymin,
	oldClipBox.xmax, oldClipBox.ymax)));

    /* redraw of the modified zone */
    newClipBox.xmin = currentPosX;
    GRegion reg = cs.to (newClipBox);

    if (backgroundMapped())
	RedrawForeOnly (reg);
    else
	RedrawBackAndFore (reg);
}


/**
FUNCTION
    _deleteString (const char*, GPos size, GPos pos);
DESCRIPTION
    This function allows to delete a string from the text.
    Same work than to delete a character.
**/

void
GoLabel::_deleteString (const char*, GPos, GPos pos)
{
    _deleteChar (0, pos);
}


/**
FUNCTION
    _replaceChar (char, GPos);
DESCRIPTION
    This function is used to replace a character inside the string.
**/

void
GoLabel::_replaceChar (char, GPos pos)
{
    GCoordSys cs = Abscs();

    /* get clip before replaceChar */
    GBox oldClipBox = cs.from (myClip().bigbox());

    /* compute of the new width */
    _width = _Width();
    MyClipHasChanged();

    /* get clip after replaceChar */
    GBox newClipBox = cs.from (myClip().bigbox());

    GNumber currentPosX = interf()->stringWidth (_string, pos,
	_lineHeight, _stringTextBrush.font());

    /* RedrawBackAndFore of the two clips right difference */
    if (oldClipBox.xmax > newClipBox.xmax)
	RedrawBackAndFore (cs.to (GBox (newClipBox.xmax, newClipBox.ymin,
	    oldClipBox.xmax, oldClipBox.ymax)));

    /* redraw of the modified zone */
    newClipBox.xmin = currentPosX;
    GRegion reg = cs.to (newClipBox);

    if (backgroundMapped())
	RedrawForeOnly (reg);
    else
	RedrawBackAndFore (reg);
}


/**
FUNCTION
    _replaceString (GPos, GPos, char*, GPos);
DESCRIPTION
    This function is used to replace a sub-string inside the string.
    Same work than to replace a character.
**/

void
GoLabel::_replaceString (GPos pos, GPos, char*, GPos)
{
    _replaceChar (0, pos);
}
