#ifdef REV_INFO
static char rcsid[] = "$Id: LabelUtil.cc,v 1.4 1992/10/26 11:28:05 andre Exp $";
#endif
/*****************************************************************************
MODULE:  	GoLabel
FILE:    	Tools.cc
AUTHOR:		Andre Michel
DATE:    	March 1991

DESCRIPTION:
  GoLabel management (See GoTextual.h and GoLabel.h)

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#include <Go/GoInterf.h>
#include <Go/Label.h>
#include <assert.h>


#define SIZE_W	10
#define _update_W(cs,w) if ((cs).I.size()*(w) < SIZE_W) (w) = SIZE_W / (cs).I.size()


/**
FUNCTION
    mapCursor (GBool set);
DESCRIPTION
    Maps / unmaps the cursor.
**/

void
GoLabel::mapCursor (GBool set)
{
    GCoordSys cs = Abscs();
    GRegion reg;

    /* where to place the cursor */
    GNumber xCursor = interf()->stringWidth (_string, _currentPos,
	_lineHeight, _stringTextBrush.font());

    /* region containing the cursor */
    interf()->lineClips (&_cursorPen, cs, GPoint (xCursor, 0),
        GPoint (xCursor, _height), &reg, NULL, GMAX_QUALITY);

    if (set){
	/* draw the cursor */
	_mask |= GCURSOR;
	RedrawForeOnly (reg);
    }

    else{
	/* undraw the cursor */
        _mask &= ~GCURSOR;
	if (backgroundMapped())
	    RedrawForeOnly (reg);
	else
	    RedrawBackAndFore (reg);
    }
}


/**
FUNCTION
    mapUnderline (GBool set);
DESCRIPTION
    Maps / unmaps the underlining.
**/

void
GoLabel::mapUnderline (GBool set)
{
    GCoordSys cs = Abscs();

    /* region containing the underline */
    GRegion reg;
    GNumber yLine = _height * _underlinePosition;
    interf()->lineClips (&_underlinePen, cs, GPoint (0, yLine),
	GPoint (_width, yLine), &reg, NULL, GMAX_QUALITY);

    if (set){
	/* draw the line */
	_mask |= GUNDERLINE;
	RedrawForeOnly (reg);
    }

    else{
	/* undraw the line */
	_mask &= ~GUNDERLINE;
	if (backgroundMapped())
	    RedrawForeOnly (reg);
	else
	    RedrawBackAndFore (reg);
    }
}


/**
FUNCTION
    xToPos (int, GNumber) const;
DESCRIPTION
    This function returns the position of char placed
    on the given line and x origin.
**/

GPos
GoLabel::xToPos (int, GNumber x) const
{
    /* balayage de la chaine */
    GPos size = length();
    for (GPos pos = 1; pos <= size; pos++)
	if (interf()->stringWidth (_string, pos,
	     _lineHeight, _stringTextBrush.font()) > x) break;

    return (--pos);
}


/**
FUNCTION
    GPos xyToPos (GPoint&);
DESCRIPTION
    Returns the position of the pointed char.
**/

GPos
GoLabel::xyToPos (GPoint& pt) const
{
    if (pt.y < 0) return (0);
    if (pt.y > _height) return length();
    return (xToPos (0, pt.x));
}


/**
FUNCTION
    posToXy (GPos pos);
DESCRIPTION
    Returns the coordinates of the given position.
**/

GPoint
GoLabel::posToXy (GPos pos) const
{
    if (pos == -1) pos = _currentPos;
    assert (pos >= 0 && pos <= length());

    GPoint pt (0, 0);
    pt.x = interf()->stringWidth (_string,
	pos, _lineHeight, _stringTextBrush.font());
    return (pt);
}


/**
FUNCTION
    boundariesToRegion (GPos pos1, GPos pos2) const;
DESCRIPTION
    Returns the GRegion defined by the 2 given positions.
**/

GRegion
GoLabel::boundariesToRegion (GPos pos1, GPos pos2) const
{
    assert (pos1 >=0 && pos1 <= length());
    assert (pos2 >=0 && pos2 <= length());

    if (pos1 == pos2)
	return GEmptyRegion();

    else{
	GCoordSys cs = Abscs();
        GNumber w1 = interf()->stringWidth (_string, pos1,
	    _lineHeight, _stringTextBrush.font());
        GNumber w2 = interf()->stringWidth (_string, pos2,
	    _lineHeight, _stringTextBrush.font());
	return (cs.to (GBox (GPoint (w1, 0), GPoint (w2, _height))));
    }
}


/**
FUNCTION
    _Width() const;
DESCRIPTION
    Returns the width of the GoLabel. 
    Used after a modification, because _width is not updated.
**/

GNumber
GoLabel::_Width() const
{
    GCoordSys cs = Abscs();

    GNumber w = interf()->stringWidth (_string, length(),
	_lineHeight, _stringTextBrush.font());

    if (selected())
	_update_W (cs, w);
		
    return w;
}


/**
FUNCTION
    resize();
DESCRIPTION
    Function called after a resize.
**/

void
GoLabel::resize()
{
    GBool was_mapped = mapped();
    if (was_mapped) TempUnmap();

    _lineHeightMax = interf()->fontStrikeoutAscent (_lineHeight,
	_stringTextBrush.font()) + interf()->fontStrikeoutDescent
	    (_lineHeight, _stringTextBrush.font());

    _width = _Width();
    _height = _lineHeightMax;

    MyClipHasChanged();
    if (was_mapped) TempMap();
}
