#ifdef REV_INFO
static char rcsid[] = "$Id: SText.cc,v 1.9 1992/12/10 16:34:16 smadja Exp $";
#endif
/*****************************************************************************
MODULE:  	GoSimpleText
FILE:    	SText.cc
AUTHOR:		Andre Michel
DATE:    	March 1991

DESCRIPTION:
  Habituals functions used by all predifined Gos.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#include <stdio.h>
#include <Go/GoInterf.h>
#include <Go/GoStream.h>
#include <Go/GoRoot.h>
#include <Go/SText.h>


static GoSimpleTextMetaclass* stext_metaclass = NULL;


/**
FUNCTION
    _noJustificationPrecond (void*, const GoTextual*, GModifyAttributeStruct*)
DESCRIPTION
    This function is used to avoid GLR_Justification.
**/

static
GBool _JustificationPrecond (void*, const GoTextual*, GModifyAttributeStruct* str)
{
GBool ret = GTRUE;

    if (str->type == GJUSTIFICATION_CHANGE_NOTIFY &&
	str->justification == GLR_JUSTIFICATION)
	    ret = GFALSE;

    return ret;
}


/**
FUNCTION
    _modifyUnderlinePenPostcond (void*, GPen*, ulong)
DESCRIPTION
    This function is called when a underline Pen modification occurs.
**/

static
void _modifyUnderlinePenPostcond (void* closure, GPen* pen, ulong mask)
{
    GoSimpleText* text = (GoSimpleText*)closure;

    /* only if goroot() != NULL */
    GoRoot* root = text->root();

    if (root != NULL && text->underlineMapped()){
        if (mask & GLINE_WIDTH || pen->isNamed())
            root->redrawBackAndFore (text->myClip());
        else
            text->mapUnderline (GTRUE);
    }
}


/**
FUNCTION
    _modifyStringTextBrush (TextBrush*, ulong);
DESCRIPTION
    This function is called when an oc modification occurs.
**/

static
void _modifyStringTextBrush (void* closure, GTextBrush* tbr, ulong mask)
{
    GoSimpleText* text = (GoSimpleText*)closure;

    /* only if goroot() != NULL */
    GoRoot* root = text->root();
    if (root != NULL){
	if (mask & GFONT || tbr->isNamed())
	    text->resize();
	else{
	    if (text->backgroundMapped())
		root->redrawForeOnly (text, text->myClip());
	    else
		root->redrawBackAndFore (text->myClip());
	}
    }
}


/**
FUNCTION
    _interlineJustificationPostcond (void*, GoSimpleText*, GModifyAttributeStruct*)
DESCRIPTION
    This function is used to refresh after a interline or justification modification.
**/

static
void _interlineJustificationPostcond (void*, GoTextual* go, GModifyAttributeStruct* str)
{
    GoSimpleText* text = (GoSimpleText*)go;
    switch (str->type){
	case GINTERLINE_CHANGE_NOTIFY:
	    if (text->interline() != str->interline)
		text->resize();
	    break;

	case GJUSTIFICATION_CHANGE_NOTIFY:
	    {
		GoRoot* root = text->root();
	        if (root != NULL && text->justification() !=
		    str->justification)
	        {
		    if (text->backgroundMapped())
		        root->redrawForeOnly (text, text->myClip());
		    else
		        root->redrawBackAndFore (text->myClip());
	        }
	    }	
	    break;

	default:
	    break;
    }
}


GoSimpleTextMetaclass* GoSimpleText::metaclass()
{
    if (stext_metaclass == NULL)
    {
	stext_metaclass = new GoSimpleTextMetaclass;
	stext_metaclass->superclass()->AddSubclass(stext_metaclass);
	GoStream::bind(stext_metaclass->classname(),
	    (int (*) (const void*, GoStream*)) GoSimpleText::save,
	    (int (*) (void**, GoStream*)) GoSimpleText::restore);

	/* precond on justification */
        stext_metaclass->modifyAttributePrecond (
	    _JustificationPrecond, NULL, GADD);

	/* postcond on interline and justification */
	stext_metaclass->modifyAttributePostcond (
	    _interlineJustificationPostcond, NULL, GADD);
    }

    return stext_metaclass;
}


GoSimpleTextMetaclass::~GoSimpleTextMetaclass()
{
    if (this == stext_metaclass)
    {
	superclass()->RemoveSubclass(this);
	stext_metaclass = NULL;
    }
}


GoMetaclass* GoSimpleTextMetaclass::m_superclass() const
{
    return GoTextual::metaclass();
}


GoMetaclass* GoSimpleText::m_metaclass() const
{
    return GoSimpleText::metaclass();
}


/**
FUNCTION
    Go* GoSimpleTextMetaclass::m_create();
DESCRIPTION
**/
Go*
GoSimpleTextMetaclass::m_create() const
{
    return (new GoSimpleText);
}


const char*
GoSimpleTextMetaclass::m_classname() const
{
    return "GoSimpleText";
}

/**
FUNCTION
    GoSimpleText();
DESCRIPTION
    The GoSimpleText constructor.
**/

GoSimpleText::GoSimpleText()
{
    _nbLines = 1;

    /* postcond on stringTextBrush modification */
    _stringTextBrush.modifyPostcond(
	_modifyStringTextBrush, this, GADD);

    /* precond on underlinePen modification */
    _underlinePen.modifyPostcond(
	_modifyUnderlinePenPostcond, this, GADD);
}


/**
FUNCTION
    m_dup (const Go*, Go*, Go*, Go*, GBool)
DESCRIPTION
    The GoSimpleText function of duplication.
**/

void
GoSimpleText::m_dup
    (const Go* orig, Go* parcopy, Go* nextcopy, GBool donext, GBool dochildren)
{
    GoTextual::m_dup(orig, parcopy, nextcopy, donext, dochildren);
    const GoSimpleText* text_orig = (const GoSimpleText*)orig;

    /* _nbLines variable */
    _nbLines = text_orig->_nbLines;

    /* underline segments */
    for (GTextSeg* s= text_orig->_underlineList.first();
	s; s=s->next())
    {
	/* creation of a new underlining segment */
	GTextSeg* new_s = new GTextSeg;
	new_s->first (s->first());
	new_s->last (s->last());
	_underlineList.insertSeg (new_s);
    }
}


/**
FUNCTION
    m_draw (const GRegion& Gr) const;
DESCRIPTION
    The GoSimpleText drawing function.
**/

void
GoSimpleText::m_draw (const GRegion& Gr) const
{
GPoint pt;
GRegion reg;

    GIntersect (myClip(), Gr, reg);
    if (reg.empty()) return;
    GCoordSys cs = Abscs();
    GBox bbox = cs.from(reg.bigbox());
    GoInterf* interf = this->interf();

    /* background */
    if (backgroundMapped())
	interf->rectangle (NULL, &backgroundBrush(), cs,
	    GPoint (-_leftMargin, -_topMargin),
		_width + _leftMargin + _rightMargin,
		    _height + _topMargin + _bottomMargin);

    /* selection list */
    for (GTextSeg* ts = selectionList().first();
	ts; ts=ts->next())
    {
	GPoint pt_deb = posToXy (ts->first());
	GPoint pt_fin = posToXy (ts->last());
	int nbLines = lineNumber (ts->last())
	    - lineNumber (ts->first());

	/* first rectangle */
	GNumber xx1 = pt_deb.x;
	GNumber xx2 = nbLines?_width:pt_fin.x;
	GNumber yy1 = pt_deb.y;
	GNumber yy2 = pt_deb.y+_lineHeightMax+_interline;
	yy2 = min(yy2, _height);
	interf->rectangle (NULL, &selectionBrush(), cs,
	    GPoint (xx1, yy1), xx2-xx1, yy2-yy1);

	/* second rectangle */
	if (nbLines >= 2){
	    xx1 = 0; xx2 = _width;
	    yy1 = yy2;
	    yy2 += (_lineHeightMax+_interline)*(nbLines-1);
	    yy2 = min(yy2, _height);
	    interf->rectangle (NULL, &selectionBrush(), cs,
		GPoint (xx1, yy1), xx2-xx1, yy2-yy1);
	}

	/* third rectangle */
	if (nbLines >= 1){
	    xx1 = 0; xx2 = pt_fin.x;
	    yy1 = yy2; yy2 += _lineHeightMax+_interline;
	    yy2 = min(yy2, _height);
	    interf->rectangle (NULL, &selectionBrush(), cs,
		GPoint (xx1, yy1), xx2-xx1, yy2-yy1);
    	}
    }

    /* cursor */
    if (cursorMapped()){
	pt = posToXy();
	interf->line (&cursorPen(), cs, pt,
	    GPoint (pt.x, pt.y + _lineHeightMax -
		interf->pixelHeight() / cs.J.size()));
    }

    /* string */
    if (stringMapped()){
        /* bbox left upper corner */
        GPoint pt_aux (bbox.xmin, bbox.ymin);
        char *deb = _string + xyToPos (pt_aux);
        char *max1 = _string + length();
        for(;;){
            char *fin = strchr (deb, '\n');
            if (!fin) fin = max1;
            pt = posToXy (deb-_string);
            if (pt.y >= bbox.ymax)
                break;
            else
                if (pt.y + _lineHeightMax > bbox.ymin)
                    interf->string (&stringTextBrush(),
			NULL, cs, pt, _lineHeight, fin-deb, deb);
            if (fin == max1) break;
            deb = ++fin;
        }
    }

    /* underlining */
    if ((underlineMapped()) && _underlineList.first()){
	GNumber h = _lineHeightMax * _underlinePosition;
        for (GTextSeg* s = _underlineList.first();
            s; s=s->next())
        {
            /* a segment may be on several lines */
            int nbLines = lineNumber (s->last())
                - lineNumber (s->first());

            GPos pos1 = s->first();
            while (nbLines-- >= 0){
                GPoint pt_deb = posToXy (pos1);
                GPoint pt_fin = posToXy (nbLines<0 ? s->last() :
                    endLinePos (lineNumber (pos1)));
                pt_deb.y += h; pt_fin.y += h;
                interf->line (&underlinePen(), cs, pt_deb, pt_fin);
                pos1 = endLinePos (lineNumber (pos1))+1;
            }
        }
    }
}


/**
FUNCTION
    contains (const GPoint &pt, GNumber maxDistance);
DESCRIPTION
    The GoSimpleText contains function.
**/

GBool
GoSimpleText::m_contains (const GPoint &abs_pt, GNumber) const
{
GPos pos;
GPoint begOfLine, endOfLine;
GBool ret;

    if (ret = GoTextual::m_contains (abs_pt)){
	if (!selected() && !backgroundMapped()){
	    GPoint& pt = Abscs().from (abs_pt);
	    pos = xyToPos (pt);

	    /* X search */
	    begOfLine = posToXy (begLinePos (lineNumber (pos)));
	    endOfLine = posToXy (endLinePos (lineNumber (pos)));
    
	    if (begOfLine.x > pt.x || pt.x > endOfLine.x)
		ret = GFALSE;

	    /* y search */
	    else
		if (pt.y < 0 || pt.y - begOfLine.y > _lineHeightMax)
		    ret = GFALSE;
	}
    }
    return ret;
}


/**
FUNCTION
    _lineH (GNumber, GPos, GPos);
DESCRIPTION
    The private method of changing the lineHeight.
**/

void
GoSimpleText::_lineH (GNumber newLineHeight, GPos, GPos)
{
    GBool was_mapped = mapped();
    if (was_mapped) TempUnmap();

    /* update of height and width */
    _lineHeight = newLineHeight;
    _width = _Width();
    _lineHeightMax = interf()->fontStrikeoutAscent (newLineHeight,
	_stringTextBrush.font()) + interf()->fontStrikeoutDescent
	    (newLineHeight, _stringTextBrush.font());
    _height = _Height();

    MyClipHasChanged();
    if (was_mapped) TempMap();
}


/**
FUNCTION
    _underlinePos (GNumber, GPos, GPos);
DESCRIPTION
    Sets the underlinePosition.
**/

void
GoSimpleText::_underlinePos (GNumber newUnderlinePosition, GPos, GPos)
{
    if (underlineMapped()){
	/* undraw the old lines */
	mapUnderline (GFALSE);

	_underlinePosition = newUnderlinePosition;

	/* draw the new lines */
	mapUnderline (GTRUE);
    }
    else
	_underlinePosition = newUnderlinePosition;
}


/**
FUNCTION
    _underline (GBool, GPos, GPos);
DESCRIPTION
    Underline or not the given segment.
**/

void
GoSimpleText::_underline (GBool set, GPos pos_min, GPos pos_max)
{
    if (pos_min != pos_max){
	if (pos_min == -1){
	    pos_min = 0;
	    pos_max = length();
	}

	if (set){
	    /* creation of a new underlining segment */
	    GTextSeg *new_s = new GTextSeg;
	    new_s->first (pos_min);
	    new_s->last (pos_max);
	    _underlineList.insertSeg (new_s);
	}
	else
	    _underlineList.deletePortion (pos_min, pos_max);
	
	/* redraw of the corresponding region */
	if (mapped() && underlineMapped()){

	    // segment drawn on nb lines
	    int nbLines = lineNumber (pos_max) -
		lineNumber (pos_min);
	    GNumber h = _lineHeightMax * _underlinePosition;
	    GPos pos1 = pos_min;
	    GRegion reg;
	    GCoordSys cs = Abscs();
	    while (nbLines-- >= 0){
		GPoint pt_deb = posToXy (pos1);
		GPoint pt_fin = posToXy (nbLines < 0 ? pos_max :
		    endLinePos (lineNumber (pos1)));
                interf()->lineClips (&underlinePen(), cs,
		    GPoint (pt_deb.x, pt_deb.y + h),
                        GPoint (pt_fin.x, pt_fin.y + h),
			    &reg, NULL, GMAX_QUALITY);
		pos1 = endLinePos (lineNumber (pos1))+1;
	    }

	    if (backgroundMapped())
		RedrawForeOnly (reg);
	    else
		RedrawBackAndFore (reg);
	}
    }
}


/**
FUNCTION
    int GoSimpleText::save (const GoSimpleText*, GoStream*);
DESCRIPTION
**/

int
GoSimpleText::save (const GoSimpleText* text, GoStream* gostream)
{
    /* save of the underline segments */
    GPos size = text->_underlineList.size();
    if (!gostream->writeInt (size,
	"underlineList size")) return (0);
    GTextSeg *s1 = (text->_underlineList).first();
    GPos i = 0;
    char chaine[16];
    while (s1){
	GPos first = s1->first();
	sprintf (chaine, "Seg_%d_first", i);
	if (!gostream->writeInt (first, chaine)) return (0);
	GPos last = s1->last();
	sprintf (chaine, "Seg_%d_last", i);
	if (!gostream->writeInt (last, chaine)) return (0);
	s1 = s1->next();
	i++;
    }

    return (GoTextual::save (text, gostream));
}


/**
FUNCTION
    int GoSimpleText::restore (GoSimpleText**, GoStream*);
DESCRIPTION
**/

int
GoSimpleText::restore (GoSimpleText** text, GoStream* gostream){
int ret = 0;

    if (*text = GoSimpleText::create()){

	/* underline segments */
	int size;
	if (!gostream->readInt (&size)) return (0);

	while (size--){
	    GTextSeg* new_s = new GTextSeg;
	    int aux;
	    if (!gostream->readInt (&aux)) return (0);
	    new_s->first (aux);
	    if (!gostream->readInt (&aux)) return (0);
	    new_s->last (aux);
	    (*text)->_underlineList.insertSeg (new_s);
	}

        if (ret = GoTextual::readParams (*text, gostream))
	    (*text)->_nbLines = (*text)->GoTextual::nbLines();
    }

    return ret;
}


/**
FUNCTION
    int GoSimpleText::readParams (GoSimpleText*, GoStream*);
DESCRIPTION
**/

int
GoSimpleText::readParams (GoSimpleText* text, GoStream* gostream)
{
GBool ret;

    /* underline segments */
    int size;
    if (!gostream->readInt (&size)) return (0);

    while (size--){
        GTextSeg* new_s = new GTextSeg;
	int aux;
        if (!gostream->readInt (&aux)) return (0);
	new_s->first (aux);
        if (!gostream->readInt (&aux)) return (0);
	new_s->last (aux);
	text->_underlineList.insertSeg (new_s);
    }

    if (ret = GoTextual::readParams (text, gostream))
	text->_nbLines = text->GoTextual::nbLines();

    return ret;
}
