#ifdef REV_INFO
static char rcsid[] = "$Id: STextEdit.cc,v 1.8 1992/11/19 08:42:07 andre Exp $";
#endif
/*****************************************************************************
MODULE:  	GoSimpleText
FILE:    	STextEdit.cc
AUTHOR:		Andre Michel
DATE:    	March 1991

DESCRIPTION:
  Habituals functions used by all predifined Gos.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#include <Go/GoInterf.h>
#include <Go/SText.h>


/**
FUNCTION
    _setString (const char*, GPos)
DESCRIPTION 
    This functions allows the user to set the value of the
    string itself. The new size has to be provided.
**/

void
GoSimpleText::_setString (const char*, GPos)
{
    _nbLines = GoTextual::nbLines();
    _underlineList.reinit();

    GBool was_mapped = mapped();
    if (was_mapped) TempUnmap();
    _width = _Width();
    _height = _Height();
    MyClipHasChanged();
    if (was_mapped) TempMap();
}


/**
FUNCTION
    _replaceChar (char, GPos);
DESCRIPTION
    This function is used to replace a character inside the string.
**/

void
GoSimpleText::_replaceChar (char, GPos pos)
{
    GNumber oldWidth = _width;
    _width = _Width();
    GCoordSys cs = Abscs();

    /* redraw of the (part of the) modified line */
    GPoint pt = posToXy (pos);
    GRegion reg = cs.to (GBox (0, pt.y,
	_width, pt.y + _lineHeightMax));

    if (_width > oldWidth){
        MyClipHasChanged();

        /* the entire text must be redrawn */
        if (_justification != GLEFT_JUSTIFICATION)
            GAdd (reg, cs.to (GBox (0, 0, _width, _height)), reg);

        /* redraw the clip difference */
        RedrawForeOnly (cs.to (GBox (oldWidth, -_topMargin,
            _width + _rightMargin, _height + _bottomMargin)));
    }

    else if (_width < oldWidth){
        MyClipHasChanged();

        /* the entire text must be redrawn */
        if (_justification != GLEFT_JUSTIFICATION)
            GAdd (reg, cs.to (GBox (0, 0, _width, _height)), reg);

        /* redraw the right part */
        RedrawBackAndFore (cs.to (GBox
            (_width, -_topMargin,
            oldWidth + _rightMargin,
            _height + _bottomMargin)));
    }

    if (backgroundMapped())
	RedrawForeOnly (reg);
    else
	RedrawBackAndFore (reg);
}


/**
FUNCTION
    _replaceString (GPos, GPos, char*, GPos);
DESCRIPTION
    This function is used to replace a sub-string inside the string.
    Same work than replaceChar.
**/

void
GoSimpleText::_replaceString (GPos pos , GPos, char*, GPos)
{
    _replaceChar (0, pos);
}


/**
FUNCTION
    _deleteChar (char, GPos pos);
DESCRIPTION
    This function allows to delete the character at the position pos.
**/

void
GoSimpleText::_deleteChar (char Char, GPos pos)
{
GRegion reg;

    if (Char == '\n') _nbLines--;

    /* shift of step -1 for the underlining segments */
    _underlineList.shift (pos, -1);
    GCoordSys cs = Abscs();
    int line = lineNumber (pos);
    GPos begOfLine = begLinePos (line);
    GPos endOfLine = endLinePos (line);
    GNumber oldWidth = _width;
    GPoint pt = posToXy (pos);

    GNumber currentLineWidth = interf()->stringWidth (_string+begOfLine,
	endOfLine-begOfLine, _lineHeight, _stringTextBrush.font());

    /* the number of lines remains the same */
    if (Char != '\n'){

        /* width of the delete character */
        GNumber charWidth = interf()->stringWidth (&Char, 1, _lineHeight,
            _stringTextBrush.font());

#define _RELATIVE_EPS _height/1000.0

	/* the width has decreased */
        if (currentLineWidth + charWidth + _RELATIVE_EPS >= _width){
	    _width = _Width();
	    MyClipHasChanged();

	    /* the entire text must be redrawn */
	    if (_justification != GLEFT_JUSTIFICATION)
		GAdd (reg, cs.to (GBox (0, 0, _width, _height)), reg);

	    /* redraw the right part */
	    RedrawBackAndFore (cs.to (GBox
		(_width, -_topMargin,
		oldWidth + _rightMargin,
		_height + _bottomMargin)));
	}

	/* redraw the line part */
	GAdd (reg, cs.to (GBox (0, pt.y, _width, pt.y + _lineHeightMax)), reg);
    }

    /* Char == '\n' */
    else{
	/* update _height */
	GNumber oldHeight = _height;
	_height = _Height();

	/* refresh the height minoration */
	RedrawBackAndFore (cs.to (GBox (-_leftMargin, _height,
	    _width + _rightMargin, oldHeight + _bottomMargin)));

	/* moved up region */
	GAdd (reg, cs.to (GBox (0, pt.y, _width, _height)), reg);
	MyClipHasChanged();

	/* the width has increased */
	if (currentLineWidth > _width){
	    _width = currentLineWidth;
	    if (_justification != GLEFT_JUSTIFICATION)
		GAdd (reg, cs.to (GBox (0, 0, oldWidth, pt.y)), reg);

	    /* the new right rectangle must be drawn */
	    RedrawForeOnly (cs.to (GBox
		(oldWidth, -_topMargin,
		_width + _rightMargin,
		_height + _bottomMargin)));
	}
    }

    if (backgroundMapped())
	RedrawForeOnly (reg);
    else
	RedrawBackAndFore (reg);
}


/**
FUNCTION
    _deleteString (const char*, GPos size, GPos pos);
DESCRIPTION
    This function allows to delete a string from the text.
**/

void
GoSimpleText::_deleteString (const char*, GPos delSize, GPos pos)
{
    /* used instead of oldHeight because of GoScrolledText */
    int old_nbl = _nbLines;
    _nbLines = GoTextual::nbLines();

    /* mise a jour des segments d'underline */
    _underlineList.shift (pos, -delSize);

    GNumber oldWidth = _width;

    if (_nbLines != old_nbl){
	GBool was_mapped = mapped();
	if (was_mapped) TempUnmap();
	_height = _Height();
	_width = _Width();
	MyClipHasChanged();
	if (was_mapped) TempMap();
    }

    else{
	_width = _Width();
	GPoint pt = posToXy (pos);
	GCoordSys cs = Abscs();

	/* line containing pos */
	GRegion reg = cs.to (GBox (GPoint (0, pt.y), GPoint (_width,
	    pt.y + _lineHeightMax + _interline)));

	if (_width < oldWidth){
	    if (_justification != GLEFT_JUSTIFICATION)
	    	reg = cs.to (GBox (0, 0, _width, _height));

	    RedrawBackAndFore (cs.to (GBox
		(_width, -_topMargin,
		oldWidth + _rightMargin,
		_height + _bottomMargin)));
	    MyClipHasChanged();
	}

	if (backgroundMapped())
	    RedrawForeOnly (reg);
	else
	    RedrawBackAndFore (reg);
    }
}


/**
FUNCTION
    _insertChar (char Char, GPos pos);
DESCRIPTION
    This function allows to insert a char Char at the pos pos.
    The current pos is set to pos + 1;
**/

void
GoSimpleText::_insertChar (char Char, GPos pos)
{
    if (Char == '\n') _nbLines++;

    /* update of underLine segments */
    _underlineList.shift (pos, 1);

    GRegion reg;
    GCoordSys cs = Abscs();
    GPoint pt = posToXy (pos);

    /* the number of lines remains the same */
    if (Char != '\n'){
	int line = lineNumber (pos);
	GPos begOfLine = begLinePos (line);
	GPos endOfLine = endLinePos (line);

	GNumber currentLineWidth = interf()->stringWidth
	    (_string + begOfLine, endOfLine - begOfLine,
		_lineHeight, _stringTextBrush.font());

	/* the width has increased */
	if (currentLineWidth > _width){

	    /* the entire text must be redrawn */
	    if (_justification != GLEFT_JUSTIFICATION)
		reg += cs.to (GBox (0, 0, currentLineWidth, _height));

	    /* redraw the clip difference */
	    RedrawForeOnly (cs.to (GBox (_width, -_topMargin,
		currentLineWidth + _rightMargin, _height + _bottomMargin))); 

	    _width = currentLineWidth;
	    MyClipHasChanged();
	}

	GNumber x = _justification == GLEFT_JUSTIFICATION ? pt.x : 0;
	reg += cs.to (GBox (x - cursorPen().lineWidth(), pt.y, _width,
	    pt.y + _lineHeightMax));
    }

    /* a new line is inserted */
    else{
	MyClipHasChanged();
	_height = _Height();

	char nextChar = *(_string + pos + 1);

	/* the width may have decreased, because a line is cutted */
	if (nextChar != '\0' && nextChar != '\n'){
	    GNumber oldWidth = _width;
	    _width = _Width();

	    if (oldWidth > _width){
                /* the entire text must be redrawn */
                if (_justification != GLEFT_JUSTIFICATION)
                    GAdd (reg, cs.to (GBox (0, 0, _width, pt.y)), reg);

                /* redraw the right part */
                RedrawBackAndFore (cs.to (GBox (_width, -_topMargin,
                    oldWidth + _rightMargin, _height + _bottomMargin)));
	    }
	}

	/* lower part */
	GAdd (reg, cs.to (GBox (-_leftMargin, pt.y,
	    _width + _rightMargin, _height + _bottomMargin)), reg);
    }

    if (backgroundMapped())
	RedrawForeOnly (reg);
    else
	RedrawBackAndFore (reg);
}


/**
FUNCTION
    _insertString (char *str, GPos len, GPos pos);
DESCRIPTION
    This function allows to insert the string str at the given pos.
**/

void
GoSimpleText::_insertString (const char*, GPos len, GPos pos)
{
    /* instead of oldHeight because of GoScrolledText */
    int old_nbl = _nbLines;
    _nbLines = GoTextual::nbLines();

    /* mise a jour des segments d'underLine */
    _underlineList.shift (pos, len);

    GNumber oldWidth = _width;
    GCoordSys cs = Abscs();

    /* the number of lines remains the same */
    if (_nbLines == old_nbl){
	GPoint pt = posToXy (pos);
	GRegion reg;
	int line = lineNumber (pos);
	GPos begOfLine = begLinePos (line);
	GPos endOfLine = endLinePos (line);

	GNumber currentLineWidth = interf()->stringWidth
	    (_string + begOfLine, endOfLine - begOfLine,
		_lineHeight, _stringTextBrush.font());

	/* the width has increased */
	if (currentLineWidth > _width){
	    _width = currentLineWidth;
	    MyClipHasChanged();

	    /* the entire text must be redrawn */
	    if (_justification != GLEFT_JUSTIFICATION)
		reg = cs.to (GBox (0, 0, oldWidth, _height));

	    /* redraw the clip difference */
	    RedrawForeOnly (cs.to (GBox (oldWidth, -_topMargin,
		_width + _rightMargin, _height + _bottomMargin))); 
	}

	GAdd (reg, cs.to (GBox (0, pt.y, _width,
	    pt.y + _lineHeightMax)), reg);

	if (backgroundMapped())
	    RedrawForeOnly (reg);
	else
	    RedrawBackAndFore (reg);
    }

    /* the clip is changed */
    else{
	TempUnmap();
	_height = _Height();
	_width = _Width();
	MyClipHasChanged();
	TempMap();
    }
}
