#ifdef REV_INFO
static char rcsid[] = "$Id: STextUtil.cc,v 1.5 1992/10/26 11:28:11 andre Exp $";
#endif
/*****************************************************************************
MODULE:  	GoSimpleText
FILE:    	STextUtil.cc
AUTHOR:		Andre Michel
DATE:    	March 1991

DESCRIPTION:
  Habituals functions used by all predifined Gos.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#include <Go/GoInterf.h>
#include <Go/SText.h>
#include <assert.h>


#define SIZE_W  10
#define _update_W(cs,w) if ((cs).I.size()*(w) < SIZE_W) (w) = SIZE_W / (cs).I.size()


/**
FUNCTION
    _Width()
DESCRIPTION
    This function returns the width of the GoSimpleText, ie the width of
    the largest line.
**/

GNumber
GoSimpleText::_Width() const
{
    char* ptr = _string;
    GNumber w = 0;
    GCoordSys cs = Abscs();
    for (;;){
	char* pt1 = strchr (ptr, '\n');
	GPos nb = pt1? pt1-ptr: strlen (ptr);
	w = max(w, interf()->stringWidth (ptr, nb,
	    _lineHeight, _stringTextBrush.font()));
	if (pt1)
	    ptr += ++nb;
	else
	    break;
    }
    if (selected())
	_update_W(cs, w);
    return w;
}


/**
FUNCTION
    xToPos (int, GNumber);
DESCRIPTION
    This function returns the position of char placed
    on the given line and x origin.
**/

GPos
GoSimpleText::xToPos (int line, GNumber x) const
{
    /* search of the first char position */
    char *pp = _string;
    while (line) {if (*pp == '\n') line--; pp++;}
    GPos deb = pp-_string;
    GPos fin = endLinePos (lineNumber (deb));
    GNumber u = interf()->stringWidth (pp, fin-deb,
	_lineHeight, _stringTextBrush.font());

    /* shift due to the justification */
    GNumber X = x - (_width-u)/2*_justification;
    GPos nb = 1;
    while (interf()->stringWidth (pp, nb,
	_lineHeight, _stringTextBrush.font())<= X
	     && deb+nb <= fin) nb++;
    return (deb + --nb);
}


/**
FUNCTION
    GPos xyToPos (Pt& pt) const;
DESCRIPTION
    This function returns the pos of the char pointed by pt.
    Pt is exprimed within the relative GCoordSys.
**/

GPos
GoSimpleText::xyToPos (GPoint& pt) const
{
    if (pt.y < 0) return (0);
    if (pt.y > _height) return length();

    /* recherche de la ligne */
    int line = (int)(pt.y/(_interline+_lineHeightMax));
    line = (int)(min((long)line,(long)_nbLines - 1));
    return xToPos (line, pt.x);
}


/**
FUNCTION
    posToXy (GPos pos) const;
DESCRIPTION
    This function returns the coordinates of the GPos pos.
**/

GPoint
GoSimpleText::posToXy (GPos pos) const
{
    if (pos == -1) pos = _currentPos;
    assert (pos >= 0 && pos <= length());

    int currentLine = lineNumber (pos);
    GPos deb = begLinePos (currentLine);
    GPoint pt (0, 0);
    if (pos > deb) pt.x += interf()->stringWidth (_string+deb,
	pos-deb, _lineHeight, _stringTextBrush.font());

    if (_justification != GLEFT_JUSTIFICATION){
	GPos end = endLinePos (lineNumber (pos));
	GNumber u = interf()->stringWidth (_string+deb,
	    end-deb, _lineHeight, _stringTextBrush.font());
	pt.x += (_width - u)/2*_justification;
    }

    pt.y = currentLine*(_interline+_lineHeightMax);
    return (pt);
}


/**
FUNCTION
    boundariesToRegion (GPos first, GPos last) const;
DESCRIPTION
    Returns the GRegion defined by the 2 given positions.
**/

GRegion
GoSimpleText::boundariesToRegion (GPos pos1, GPos pos2) const
{
GPos aux;

    assert (pos1 >= 0 && pos1 <= length());
    assert (pos2 >= 0 && pos2 <= length());

    if (pos1 == pos2)
	return GEmptyRegion();

    if (pos1 > pos2){
	aux = pos1;
	pos1 = pos2;
	pos2 = aux;
    }

    GPoint pt_deb = posToXy (pos1);
    GPoint pt_fin = posToXy (pos2);

    GRegion reg;
    GCoordSys cs = Abscs();
    int nbLines = lineNumber (pos2)-lineNumber (pos1);

    // premier rectangle
    GNumber xx1 = pt_deb.x;
    GNumber xx2 = nbLines?_width:pt_fin.x;
    GNumber yy1 = pt_deb.y;
    GNumber yy2 = pt_deb.y+_lineHeightMax+_interline;
    yy2 = min(yy2, _height);
    GAdd (reg, cs.to (GBox (GPoint (xx1, yy1),
	GPoint (xx2, yy2))), reg);

    // deuxieme rectangle
    if (nbLines >= 2){
        xx1 = 0;
        xx2 = _width;
        yy1 = yy2;
        yy2 += (_lineHeightMax+_interline)*(nbLines-1);
        yy2 = min(yy2, _height);
	GAdd (reg, cs.to (GBox (GPoint (xx1, yy1),
	    GPoint (xx2, yy2))), reg);
    }

    // troisieme rectangle
    if (nbLines >= 1){
        xx1 = 0;
        xx2 = pt_fin.x;
        yy1 = yy2;
        yy2 += _lineHeightMax+_interline;
        yy2 = min(yy2, _height);
	GAdd (reg, cs.to (GBox (GPoint (xx1, yy1),
	    GPoint (xx2, yy2))), reg);
    }

    return reg;
}


void
GoSimpleText::mapCursor (GBool set)
{
    if (set != cursorMapped()){
        GPoint cursorOrigin = posToXy();
	GCoordSys cs = Abscs();

        /* region containing the cursor */
        GRegion reg;
        interf()->lineClips (&_cursorPen, cs,
	    cursorOrigin, GPoint (cursorOrigin.x,
		cursorOrigin.y + _lineHeightMax),
		    &reg, NULL, GMAX_QUALITY);

	if (set){
            _mask |= GCURSOR;
            RedrawForeOnly (reg);
        }
        else{
            /* undraw the cursor */
            _mask &= ~GCURSOR;
	    if (backgroundMapped())
		RedrawForeOnly (reg);
	    else
		RedrawBackAndFore (reg);
        }
    }
}


void
GoSimpleText::mapUnderline (GBool set)
{
    if (!_underlineList.size()) return;

    GCoordSys cs = Abscs();
    GNumber h = _underlinePosition * _lineHeightMax;
    GRegion reg;

    for (GTextSeg *s = _underlineList.first(); s; s=s->next()){
        int nbLines = lineNumber (s->last()) -
	    lineNumber (s->first());

        GPoint pt_deb, pt_fin;
        GPos pos1 = s->first();
        while (nbLines-- >= 0){
            pt_deb = posToXy (pos1);
            pt_fin = (nbLines < 0) ? posToXy (s->last()) :
                posToXy (endLinePos (lineNumber (pos1)));

	    pt_deb.y += h;
	    pt_fin.y += h;

            interf()->lineClips (&_underlinePen, cs, pt_deb,
                pt_fin, &reg, NULL, GMAX_QUALITY);

            pos1 = endLinePos (lineNumber (pos1))+1;
        }
    }

    if (set){
        _mask |= GUNDERLINE;
        RedrawForeOnly (reg);
    }

    else{
        /* undraw the cursor */
        _mask &= ~GUNDERLINE;
	if (backgroundMapped())
	    RedrawForeOnly (reg);
	else
	    RedrawBackAndFore (reg);
    }
}


void
GoSimpleText::resize()
{
    GBool was_mapped = mapped();
    if (was_mapped) TempUnmap();
    
    _lineHeightMax = interf()->fontStrikeoutAscent (_lineHeight,
	_stringTextBrush.font()) + interf()->fontStrikeoutDescent
	    (_lineHeight, _stringTextBrush.font());

    _width = _Width();
    _height = _Height();

    MyClipHasChanged();
    if (was_mapped) TempMap();
}
