#ifdef REV_INFO
static char rcsid[] = "$Id: Style.cc,v 1.1 1992/12/23 13:50:42 andre Exp $";
#endif
/*****************************************************************************
MODULE:  Text
FILE:    Highlight.cc
AUTHOR:  Michel Andre
DATE:    1/7/92

DESCRIPTION: The highlight class contains a mask and tools (TextBrush, ...)
             used by a GoTextSegment.
    

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include <Go/Text.h>
#define VERSION_1_2             (1 << 15)


GStyle::GStyle (GStyle* att)
{
    _mask = att->_mask;

    /* copy of the variables values */
    _mapUnderlineSeg = att->_mapUnderlineSeg;
    _mapBackgroundSeg = att->_mapBackgroundSeg;
    _underlinePosition = att->_underlinePosition;
    _lineHeight = att->_lineHeight;
    _extraBaseline = att->_extraBaseline;

    /* underlineShPen */
    _underlineShPen = (att->_mask & GUNDERLINE_SHPEN) ?
	att->underlineShPen()->dup() : NULL;

    /* stringShTextBrush */
    _stringShTextBrush = (att->_mask & GSTRING_SHTEXTBRUSH) ?
	att->stringShTextBrush()->dup() : NULL;

    /* backgroundShBrush */
    _backgroundShBrush = (att->_mask & GBACKGROUND_SHBRUSH) ?
	att->backgroundShBrush()->dup() : NULL;
}


GStyle::GStyle()
{
    _mask = VERSION_1_2;
    _backgroundShBrush = NULL;
    _stringShTextBrush = NULL;
    _underlineShPen = NULL;
}


GStyle::~GStyle()
{
    if (_mask & GUNDERLINE_SHPEN)
	_underlineShPen->destroy();
    if (_mask & GSTRING_SHTEXTBRUSH)
	_stringShTextBrush->destroy();
    if (_mask & GBACKGROUND_SHBRUSH)
	_backgroundShBrush->destroy();
}


GBool
GStyle::equiv (const GStyle* textatt) const
{
    if (textatt == NULL || _mask != textatt->_mask)
        return GFALSE;
 
    if (_mask & GBACKGROUND_SHBRUSH &&
	_backgroundShBrush != textatt->_backgroundShBrush)
	    return GFALSE;

    if (_mask & GSTRING_SHTEXTBRUSH &&
	_stringShTextBrush != textatt->_stringShTextBrush)
	    return GFALSE;

    if (_mask & GUNDERLINE_SHPEN &&
	_underlineShPen != textatt->_underlineShPen)
	    return GFALSE;

    if (_mask & GUNDERLINE_POSITION &&
	_underlinePosition != textatt->_underlinePosition)
	    return GFALSE;

    if (_mask & GLINE_HEIGHT &&
	_lineHeight != textatt->_lineHeight)	
	    return GFALSE;

    if (_mask & GEXTRA_BASELINE &&
	_extraBaseline != textatt->_extraBaseline)	
	    return GFALSE;

    if (_mask & GMAP_UNDERLINE_SEG &&
	_mapUnderlineSeg != textatt->_mapUnderlineSeg)
	    return GFALSE;

    if (_mask & GMAP_BACKGROUND_SEG &&
	_mapBackgroundSeg != textatt->_mapBackgroundSeg)
	    return GFALSE;

    return GTRUE;
}


GBool
GStyle::equiv (const GoText* text) const
{
    if (_mask & GBACKGROUND_SHBRUSH &&
	_backgroundShBrush != text->backgroundBrush().shBrush())
	    return GFALSE;

    if (_mask & GSTRING_SHTEXTBRUSH &&
	_stringShTextBrush != text->stringTextBrush().shTextBrush())
	    return GFALSE;

    if (_mask & GUNDERLINE_SHPEN &&
	_underlineShPen != text->underlinePen().shPen())
	    return GFALSE;

    if (_mask & GUNDERLINE_POSITION &&
	_underlinePosition != text->underlinePosition())
	    return GFALSE;

    if (_mask & GLINE_HEIGHT &&
	_lineHeight != text->lineHeight())	
	    return GFALSE;

    if (_mask & GEXTRA_BASELINE &&
	_lineHeight != text->lineHeight())	
	    return GFALSE;

    if (_mask & GMAP_UNDERLINE_SEG &&
	_mapUnderlineSeg != text->mapUnderlineSeg())
	    return GFALSE;

    if (_mask & GMAP_BACKGROUND_SEG &&
	_mapBackgroundSeg != text->mapBackgroundSeg())
	    return GFALSE;

    return GTRUE;
}


void
GStyle::modify (const GoText* text)
{
    if (_mask & GBACKGROUND_SHBRUSH &&
	_backgroundShBrush == text->backgroundBrush().shBrush())
    {
	_backgroundShBrush->destroy();
	_mask &= ~GBACKGROUND_SHBRUSH;
	_backgroundShBrush = NULL;
    }

    if (_mask & GSTRING_SHTEXTBRUSH &&
	_stringShTextBrush == text->stringTextBrush().shTextBrush())
    {
	_stringShTextBrush->destroy();
	_mask &= ~GSTRING_SHTEXTBRUSH;
	_stringShTextBrush = NULL;
    }

    if (_mask & GUNDERLINE_SHPEN &&
	_underlineShPen == text->underlinePen().shPen())
    {
	_underlineShPen->destroy();
	_mask &= ~GUNDERLINE_SHPEN;
	_underlineShPen = NULL;
    }

    if (_mask & GUNDERLINE_POSITION &&
	_underlinePosition == text->underlinePosition())
	    _mask &= ~GUNDERLINE_POSITION;

    if (_mask & GLINE_HEIGHT &&
	_lineHeight == text->lineHeight())
	    _mask &= ~GLINE_HEIGHT;

    if (_mask & GEXTRA_BASELINE &&
	_extraBaseline == 0)
	    _mask &= ~GEXTRA_BASELINE;

    if (_mask & GMAP_UNDERLINE_SEG &&
	_mapUnderlineSeg == text->mapUnderlineSeg())
	    _mask &= ~GMAP_UNDERLINE_SEG;

    if (_mask & GMAP_BACKGROUND_SEG &&
	_mapBackgroundSeg == text->mapBackgroundSeg())
	    _mask &= ~GMAP_BACKGROUND_SEG;
}


void
GStyle::lineHeight (GNumber newLineHeight)
{
    if (newLineHeight == -1)
	_mask &= ~GLINE_HEIGHT;
    else{
	_mask |= GLINE_HEIGHT;
	_lineHeight = newLineHeight;
    }
}


void
GStyle::extraBaseline (GNumber newBaseline)
{
    if (newBaseline == -1)
	_mask &= ~GEXTRA_BASELINE;
    else{
	_mask |= GEXTRA_BASELINE;
	_extraBaseline = newBaseline;
    }
}


void
GStyle::underlinePosition (GNumber newPosition)
{
    if (newPosition == -1)
	_mask &= ~GUNDERLINE_POSITION;
    else{
	_mask |= GUNDERLINE_POSITION;
	_underlinePosition = newPosition;
    }
}


void
GStyle::mapUnderlineSeg (GBool map)
{
    if (map){
	_mask |= GMAP_UNDERLINE_SEG;
	_mapUnderlineSeg = map;
    }
    else
	_mask &= ~GMAP_UNDERLINE_SEG;
}


void
GStyle::mapBackgroundSeg (GBool map)
{
    if (map){
	_mask |= GMAP_BACKGROUND_SEG;
	_mapBackgroundSeg = map;
    }
    else
	_mask &= ~GMAP_BACKGROUND_SEG;
}


void
GStyle::backgroundShBrush (const GShBrush* sbr)
{
    if (_mask & GBACKGROUND_SHBRUSH){
	if (!sbr){
	    _backgroundShBrush->destroy();
	    _mask &= ~GBACKGROUND_SHBRUSH;
	}
	else{
	    if (!sbr->isNamed()){
		GModify (_backgroundShBrush, sbr->mask(),
		    sbr->values());
		GRename (_backgroundShBrush, NULL);
	    }
	}
    }

    else if (sbr){
	_backgroundShBrush = sbr->dup();
	_mask |= GBACKGROUND_SHBRUSH;
    }
}


void
GStyle::backgroundShBrush (const GBrush* br)
{
    if (!br)
	backgroundShBrush ((const GShBrush*)NULL);
    else
	backgroundShBrush (br->shBrush());
}


void
GStyle::backgroundShBrush (ulong mask, const GBrushValues* values)
{
    if (!mask) backgroundShBrush ((const GShBrush*)NULL);

    else if (_mask & GBACKGROUND_SHBRUSH){
	GModify (_backgroundShBrush, mask, values);
	GRename (_backgroundShBrush, NULL);
    }

    else{
	_backgroundShBrush = GCreateShBrush (mask, values);
	_mask |= GBACKGROUND_SHBRUSH;
    }
}


void
GStyle::stringShTextBrush (const GShTextBrush* sbr)
{
    if (_mask & GSTRING_SHTEXTBRUSH){
	if (!sbr){
	    _stringShTextBrush->destroy();
	    _mask &= ~GSTRING_SHTEXTBRUSH;
	}
	else{
	    if (!sbr->isNamed()){
		GModify (_stringShTextBrush, sbr->mask(),
		    sbr->values());
		GRename (_stringShTextBrush, NULL);
	    }
	}
    }

    else if (sbr){
	_mask |= GSTRING_SHTEXTBRUSH;
	_stringShTextBrush = sbr->dup();
    }
}


void
GStyle::stringShTextBrush (const GTextBrush* br)
{
    if (!br)
	stringShTextBrush ((const GShTextBrush*)NULL);
    else
	stringShTextBrush (br->shTextBrush());
}


void
GStyle::stringShTextBrush (ulong mask, const GTextBrushValues* values)
{
    /* the given mask may be null */
    if (!mask) stringShTextBrush ((const GShTextBrush*)NULL);

    else if (_mask & GSTRING_SHTEXTBRUSH){
	GModify (_stringShTextBrush, mask, values);
	GRename (_stringShTextBrush, NULL);
    }

    else{
	_stringShTextBrush = GCreateShTextBrush (mask, values);
	_mask |= GSTRING_SHTEXTBRUSH;
    }
}


void
GStyle::underlineShPen (const GShPen* spen)
{
    if (_mask & GUNDERLINE_SHPEN){
	if (!spen){
	    _underlineShPen->destroy();
	    _mask &= ~GUNDERLINE_SHPEN;
	}
	else{
	    if (!spen->isNamed()){
		GModify (_underlineShPen, spen->mask(),
		    spen->values());
		GRename (_underlineShPen, NULL);
	    }
	}
    }
    else if (spen){
	_underlineShPen = spen->dup();
	_mask |= GUNDERLINE_SHPEN;
    }
}


void
GStyle::underlineShPen (const GPen* pen)
{
    if (!pen)
	underlineShPen ((const GShPen*)NULL);
    else
	underlineShPen (pen->shPen());
}


void
GStyle::underlineShPen (ulong mask, const GPenValues* values)
{
    /* the given mask may be null */
    if (!mask) underlineShPen ((const GShPen*)NULL);

    else if (_mask & GUNDERLINE_SHPEN){
	GModify (_underlineShPen, mask, values);
	GRename (_underlineShPen, NULL);
    }

    else{
	_underlineShPen = GCreateShPen (mask, values);
	_mask |= GUNDERLINE_SHPEN;
    }
}
