#ifdef REV_INFO
static char rcsid[] = "$Id: Text.cc,v 1.15 1992/12/28 09:12:26 andre Exp $";
#endif
/*****************************************************************************
MODULE:  GoText
FILE:	 Text.cc
AUTHOR:  Michel Andre
DATE:    August 91
 

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
*****************************************************************************/

#include	<Go/GoStream.h>
#include	<Go/Text.h>
#include	<Go/Form.h>

#ifdef GoLICENSE
#include	"TextCont.h"
#endif /* GoLICENSE */

static GoTextMetaclass* gotext_metaclass = NULL;


GBool _modifyAttributePrecond (void*, const GoTextual* go, GModifyAttributeStruct* str)
{
    GoText* text = (GoText*)go;
    GFormatter* f = text->formatter();
    GBool ret = GTRUE;

    if (f && !f->isFrameFormatter() &&
         str->type == GJUSTIFICATION_CHANGE_NOTIFY &&
         str->justification == GLR_JUSTIFICATION)
    {
        ret = GFALSE;
    }
    return ret;
}


/**
FUNCTION
    _modifyAttributePostcond (GModifyAttributeStruct);
DESCRIPTION
    Text modify attribute postcond.
**/

void
_modifyAttributePostcond (void*, GoTextual* go, GModifyAttributeStruct* str)
{
    GoText* text = (GoText*)go;
    GFormatter* f = text->formatter();
 
    switch (str->type){
	case GJUSTIFICATION_CHANGE_NOTIFY:
	    f->justification (str->justification);
	    break;

	case GINTERLINE_CHANGE_NOTIFY:
	    f->reframe();
	    break;

	default:
	    break;
    }
}


/**
FUNCTION
    _modifyUnderlinePen (void*, GPen*, ulong);
DESCRIPTION
    This method is called when _underlinePen is modified.
**/

void _modifyUnderlinePen (void* closure, GPen*, ulong)
{
    GoText* text = GTEXT (closure);

    /* test of reparenting to avoid dup call */
    if (text->root()){
	for (GoTextSegment* seg = GSEG (text->_firstLine->Child());
	    seg; seg=seg->myNext())
	{
	    /* segments sharing the the same GPen than text */
	    if (!seg->_att ||
		!(seg->_att->mask() & GUNDERLINE_SHPEN))
	    {
		if (seg->mapUnderlineSeg()){
		    if (text->backgroundMapped())
		        text->RedrawForeOnly();
		    else
		        text->RedrawBackAndFore();
		}
	    }

	    /* not majority segments */
	    else{
		seg->_att->modify (text);
		if (!seg->_att->mask()){
		    delete seg->_att;
		    seg->_att = NULL;
		}
	    }
	}
    }
}


/**
FUNCTION
    _modifyStringTextBrush (void*, GTextBrush*, ulong);
DESCRIPTION
    This method is called when _stringTextBrush is modified.
**/

void _modifyStringTextBrush (void* closure, GTextBrush* tbr, ulong mask)
{
    GoText* text = GTEXT (closure);

    /* test of reparenting to avoid dup call */
    if (text->root()){
	GRegion region = GEmptyRegion();
	for (GoTextSegment* seg = GSEG (
	    text->_firstLine->Child()); seg; seg=seg->myNext())
	{
	    /* segments sharing the the same GTextBrush than text */
	    if (!seg->_att ||
		!(seg->_att->mask() & GSTRING_SHTEXTBRUSH))
	    {
		if (mask & GFONT || tbr->isNamed())
		    seg->updateWidth();
		else
		    seg->m_add_clip (region);
	    }

	    /* not majority segments */
	    else{
		seg->_att->modify (text);
		if (!seg->_att->mask()){
		    delete seg->_att;
		    seg->_att = NULL;
		}
	    }
	}

	if (mask & GFONT || tbr->isNamed()){
	    GFormatter* f = text->formatter();
	    if (f){
		f->reframe();
		text->MyClipHasChanged();
	    }
	}

	else{
	    if (text->backgroundMapped())
		text->RedrawForeOnly (region);
	    else
		text->RedrawBackAndFore (region);
	}
    }
}


/**
FUNCTION
    metaclass();
DESCRIPTION
    The GoText metaclass method.
**/

GoTextMetaclass* GoText::metaclass()
{
    if (gotext_metaclass == NULL){
	gotext_metaclass = new GoTextMetaclass;
	gotext_metaclass->superclass()->AddSubclass (
	    gotext_metaclass);
	GoStream::bind (gotext_metaclass->classname(),
	    (int (*) (const void*, GoStream*)) GoText::save,
	    (int (*) (void**, GoStream*)) GoText::restore);
	gotext_metaclass->modifyAttributePrecond (
	    _modifyAttributePrecond, NULL, GADD);
	gotext_metaclass->modifyAttributePostcond (
	    _modifyAttributePostcond, NULL, GADD);
    }

    return gotext_metaclass;
}


/**
FUNCTION
    ~GoTextMetaclass();
DESCRIPTION
    The GoTextMetaclass destructor.
**/

GoTextMetaclass::~GoTextMetaclass()
{
    if (this == gotext_metaclass){
	superclass()->RemoveSubclass(this);
	gotext_metaclass = NULL;
    }
}


/**
FUNCTION
    m_classname();
DESCRIPTION
    The GoTextMetaclass m_classname method.
**/

const char*
GoTextMetaclass::m_classname() const
{
    return "GoText";
}


/**
FUNCTION
    m_create();
DESCRIPTION
    The GoTextMetaclass m_create method.
**/

Go*
GoTextMetaclass::m_create() const
{
    return (Go*)new GoText;
}


/**
FUNCTION
    m_superclass();
DESCRIPTION
    The GoTextMetaclass m_superclass method.
**/

GoMetaclass*
GoTextMetaclass::m_superclass() const
{
    return GoTextual::metaclass();
}


/**
FUNCTION
    m_dup (const Go* orig, Go* parcopy, Go* prevcopy, GBool donext);
DESCRIPTION
    The GoText m_dup method.
**/

void
GoText::m_dup
    (const Go* orig, Go* parcopy, Go* prevcopy, GBool donext, GBool dochildren)
{
    GoTextual::m_dup (orig, parcopy, prevcopy, donext, dochildren);
    GoText* textOrig = GTEXT (orig);

    _mapUnderlineSeg = textOrig->_mapUnderlineSeg;
    _mapBackgroundSeg = textOrig->_mapBackgroundSeg;

    if (textOrig->formatter()){
	_formatter = textOrig->formatter()->dup();
	_formatter->_text = this;
    }

    if (!dochildren)
    {
	//we always have to dup GoTextLines:
	_firstLine = (GoTextLine*)
	    textOrig->_firstLine->Dup (this, NULL, GTRUE, GTRUE);
    }
    else
    {
	Go* child = Child();
	while (child->getMetaclass() != GoTextLine::metaclass())
	    child = child->Next();
	_firstLine = (GoTextLine*) child;
    }

    _lastLine = _firstLine;
    while (_lastLine->Next() && _lastLine->Next()->getMetaclass()
        == GoTextLine::metaclass())
	    _lastLine = (GoTextLine*) _lastLine->Next();
}


/**
FUNCTION
    GoText();
DESCRIPTION
    The GoText constructor.
**/

GoText::GoText()
{
    GoTextLine* line = GoTextLine::create();
    line->parentRelativeReparent (this);

    _firstLine = line;
    _lastLine = line;
    _formatter = NULL;
    _mapUnderlineSeg = GFALSE;
    _mapBackgroundSeg = GFALSE;

    /* postconditions on gc changements */
    _stringTextBrush.modifyPostcond (_modifyStringTextBrush, this, GADD);
    _underlinePen.modifyPostcond (_modifyUnderlinePen, this, GADD);
}


/**
FUNCTION
    m_contains (const GPoint &pt, GNumber maxDistance);
DESCRIPTION
    The GoTextual contains method.
**/

GBool
GoText::m_contains (const GPoint &abs_pt, GNumber) const
{
GBool ret = GFALSE;

    if (ret = GoTextual::m_contains (abs_pt)){
	if (!selected() && !backgroundMapped()){

	    /* translate to relative point */
	    GPoint pt = Abscs().from (abs_pt);

	    /* search of the line */
	    for (GoTextLine* line = _firstLine;
		line->NextLine(); line=line->NextLine())
	    {
		if (line->Relcs().O.y + line->_height > pt.y &&
		    line->Relcs().O.x + line->_width > pt.x)
		    break;
	    }

	    if (!line)
		return GFALSE;

	    /* search of the segment */
	    GNumber lineXOrigin = line->Relcs().O.x;
	    for (GoTextSegment* seg=(GoTextSegment*)line->Child();
		seg->Next(); seg=(GoTextSegment*)seg->Next())
	    {
		if (pt.x < lineXOrigin +
		    seg->Relcs().O.x + seg->_width) break;
	    }

	    GRegion reg;
	    seg->m_add_clip (reg);
	    ret = reg.contains (abs_pt);
	}
    }

    return ret;
}


/**
FUNCTION
    metaclass();
DESCRIPTION
    The GoText m_metaclass method.
**/

GoMetaclass*
GoText::m_metaclass() const
{
    return GoText::metaclass();
}


/**
FUNCTION
    m_draw (const GRegion&) const;
DESCRIPTION
    The GoText m_draw method.
**/

void
GoText::m_draw (const GRegion& reg) const
{
    /* background drawing */
    if (backgroundMapped())
	_formatter->draw (reg);
}


/**
FUNCTION
    m_add_clip (GRegion&);
DESCRIPTION
    The GoText m_add_clip method.
**/

void
GoText::m_add_clip (GRegion& reg) const
{
    if (formatter())
	formatter()->clip (reg);
}


/**
FUNCTION
    m_add_opaque_clip (GRegion&);
DESCRIPTION
    The GoText m_add_opaque_clip method.
**/

void
GoText::m_add_opaque_clip (GRegion& reg) const
{
    if (backgroundMapped() && formatter())
	formatter()->clip (reg);
}


/**
FUNCTION
    int GoText::save (const GoText*, GoStream*);
DESCRIPTION
    The GoText restore method.
**/

int
GoText::save (const GoText* text, GoStream* gostream)
{
    GBool b = text->_formatter != NULL;
    if (!(gostream->writeBool (b, "formatter"))) return 0;

    if (text->_formatter)
	if (!(gostream->saveObj(text->_formatter,
	    text->_formatter->getClassname(), "formatter")))
		return 0;

    return (GoTextual::save (text, gostream));
}


/**
FUNCTION
    int GoText::restore (GoText**, GoStream*);
DESCRIPTION
    The GoText restore method.
**/

int
GoText::restore (GoText** text, GoStream* gostream)
{
    if (*text = GoText::create()){
        (*text)->_firstLine->destroy();

	GBool b;
        if (!(gostream->readBool ((GBool*)&b))) return 0;
        if (b){
	    GFormatter* tf = NULL;
	    const char* class_name = NULL;
	    if (!(gostream->restoreObj ((void**) &tf, &class_name)))
		return 0;
	    else{
		(*text)->_formatter = tf;
		tf->_text = *text;
	    }
	}

        if (!GoTextual::readParams (*text, gostream))
	    return 0;

	/* to be updated with the resize method */
	(*text)->_firstLine = NULL;
	(*text)->_lastLine = NULL;
	return 1;
    }

    return 0;
}


/**
FUNCTION
    int GoText::readParams (GoText*, GoStream*);
DESCRIPTION
    The GoText restore method.
**/

int
GoText::readParams (GoText* text, GoStream* gostream){
GBool b;
const char* class_name = NULL;

    if (!(gostream->readBool ((GBool*)&b))) return 0;
    if (b && !(gostream->restoreObj (
	(void**) &text->_formatter, &class_name)))
	    return 0;

    if (!GoTextual::readParams (text, gostream))
	return 0;

    /* to be updated with the resize method */
    text->_firstLine = NULL;
    text->_lastLine = NULL;
    return 1;
}


/**
FUNCTION
    formatter()
DESCRIPTION
    This method is used to set/replace the text formatter.
**/

GBool
GoText::formatter (GFormatter* f)
{
    GFormatter* oldFormatter = _formatter;

    if (f){
	if (oldFormatter && oldFormatter->isFrameFormatter()
	    != f->isFrameFormatter()) return GFALSE;
	_formatter = f;
	f->_text = this;
	f->reframe();
    }

    _formatter = f;
    if (oldFormatter)
	oldFormatter->_text = NULL;

    MyClipHasChanged();
    return GTRUE;
}


/**
FUNCTION
    minWidth()
DESCRIPTION
    This method returns the largest word width.
**/

GNumber
GoText::minWidth()
{
GNumber width = 0;
GNumber maxWidth = 0;

    for (GoTextSegment* seg = GSEG (_firstLine->Child());
	seg; seg=seg->myNext())
    {
	width += seg->_width;
	/* the word may be hyphened */
	if (!seg->_hyphened){
	    maxWidth = max (maxWidth, width);
	    width = 0;
	}
    }

    return maxWidth;
}


/**
FUNCTION
    addGo (Go*, GPos)
DESCRIPTION
    This method is used to link a Go inside the text
    at the given position.
**/

GBool
#ifdef GoLICENSE
GoText::addGo (Go* addedGo, GPos pos)
{
    if (pos < 0 || pos > length() || addedGo == NULL)
	return GFALSE;

    createPortion (pos, pos);

    GoTextSegment *rightBrother = posToSeg (pos);
    if (pos != rightBrother->_firstPos){
	createPortion (rightBrother->_firstPos, pos);
	rightBrother = posToSeg (pos);
    }

    GoContainer *contGo = GoContainer::create();
    contGo->reparent (rightBrother->parent(), rightBrother);
    contGo->addGo (addedGo);
    contGo->_firstPos = pos;
    contGo->_lastPos = pos;

    /* reframe the text */
    if (_formatter)
	_formatter->reframe();

#else /* GoLICENSE */
GoText::addGo (Go*, GPos)
{

#endif /* GoLICENSE */
    
    return GTRUE;
}


/**
FUNCTION
    selectUnselectSubGo (const GPoint&)
DESCRIPTION
    This method is used to select/unselect
    a GoContainer componant of the text.
**/

void
#ifdef GoLICENSE
GoText::selectUnselectSubGo (const GPoint& absp)
{
    for (GoTextSegment* seg = GSEG (_firstLine->Child());
	seg; seg = seg->myNext())
    {
	if (GoContainer::metaclass() == seg->getMetaclass()){
	    Go* subGo = seg->Child();
	    if (subGo->contains (absp)){
		if (subGo->selected())
		    subGo->unselect();
		else
		    subGo->select();
		return;
	    }
	}
    }
#else /* GoLICENSE */

GoText::selectUnselectSubGo (const GPoint&)
{
#endif /* GoLICENSE */
}


/**
FUNCTION
    subGo (const GPoint&)
DESCRIPTION
    This method is used to access
    a sub Go componant of the text.
**/

Go*

#ifdef GoLICENSE
GoText::subGo (const GPoint& absp)
{
    for (GoTextSegment* seg = GSEG (_firstLine->Child());
	seg; seg = seg->myNext())
    {
	if (GoContainer::metaclass() == seg->getMetaclass()){
	    Go* subGo = seg->Child();
	    if (subGo->contains (absp))
		return subGo;
	}
    }
    
#else /* GoLICENSE */

GoText::subGo (const GPoint&)
{
    
#endif /* GoLICENSE */

    return NULL;
}


/**
FUNCTION
    extract (GPos, GPos)
DESCRIPTION
    Returns a GoText of which the string is
    contained between the 2 given positions.
**/

GoText*
GoText::extract (GPos pos1, GPos pos2)
{
    GPos len = length();

    /* this must be linked with a formatter */
    if (pos1 < 0 || pos2 < 0 || pos1 > len || pos2 > len ||
	!_formatter) return NULL;

    GoText* text = GTEXT (dup());

    /* dupThis is not available yet */
    Go* next;
    for (Go* go = text->Child(); go; go=next){
        next = go->Next();
        if (go->getMetaclass() != GoTextLine::metaclass())
            go->destroy();
    }

    text->_currentPos = 0;
    text->_selectionList.reinit();
    text->resize();

    /* right part to delete */
    if (pos2 < len){
        *(text->_string + pos2) = '\0';
        text->_formatter->deleteString (NULL, len - pos2, pos2);
    }

    /* left part to delete */
    if (pos1){
        strcpy (text->_string, text->_string + pos1);
        text->_formatter->deleteString (NULL, pos1, 0);
    }
    return text;
}


/**
FUNCTION
    merge (GoText, GPos)
DESCRIPTION
    Inserts a GoText.
    The formatters must belong to the same type.
**/

GBool
GoText::merge (GoText* text, GPos pos)
{
    if (pos < 0 || pos > length() || !_formatter ||
	!text->_formatter ||
	    (_formatter->isFrameFormatter()
	        != text->_formatter->isFrameFormatter()))
	return GFALSE;

    /* copy of the substring */
    /* update of _string */
    /* add one char for \0 */

    int thisSize = length();
    int textSize = text->length();
    _allocString (thisSize + textSize + 1);

    /* update of the string value */
    int i = thisSize - pos + 1;
    char* p = _string+thisSize+textSize;
    while (i--) *p = *(p-- - textSize);
    strncpy (_string + pos, text->_string, textSize);
    _updatePresentation (_present);

    /* update the segments of text */
    text->shiftPos (GSEG (text->_firstLine->Child()), pos);

    /* copy of the segments */
    createPortion (pos, pos);
    GoTextSegment* s = posToSeg (pos);
    GoTextSegment* next;
    GoTextSegment* sn = GSEG (text->_firstLine->Child());
    for (;sn; sn = next)
    {
	next = sn->myNext();

	/* don't copy empty segments */
	if (sn->_firstPos != sn->_lastPos)
	    sn->reparent (s->parent(), s);
    }

    /* update the segments of this */
    shiftPos (s, textSize);

    /* update the characteristics of the inserted segments */
    /* and reframe */
    resize();

    /* destroy text */
    text->destroy();

    return GTRUE;
}
