/* $Id: TextCont.h,v 1.6 1992/11/12 15:34:00 andre Exp $ */
/*****************************************************************************
MODULE:  Text
FILE:    TextCont.h
AUTHOR:  Michel Andre
DATE:    1/7/92

DESCRIPTION: The GoContainer class is used to include heterogenous
         Go inside a GoTextLine.
    

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#ifndef		_TextCont_h
#define		_TextCont_h

#include	<Go/TextSg.h>
#include        <Go/mTextSg.h>


class GoContainer;

class GoContainerMetaclass : public GoTextSegmentMetaclass
{
  protected:
    virtual ~GoContainerMetaclass();

  private:
    virtual const char*  m_classname() const;
    virtual GoMetaclass* m_superclass() const;
    virtual Go*          m_create() const;

    friend class GoContainer;
};


class GoContainer : public GoTextSegment
{
  public:
    static GoMetaclass*	metaclass();
    static GoContainer*	create();

    void		addGo (Go*);
                        GoContainer() {_child = NULL;}

    /* stream methods */
    static int          save (const GoContainer*, GoStream*);
    static int          restore (GoContainer**, GoStream*);

  protected:
    GoMetaclass* 	m_metaclass() const;
    void	 	m_add_clip (GRegion&) const;
    void		m_draw (const GRegion&) const{}
    GBool		contains (GoContainer&) const;
    void		updateWidth(){}
    void		updateHeight(){}

  private:
    virtual Go*         d_get_first_child() const {return _child;}
    virtual void        d_add_child (const Go* go, int) {_child = (Go*)go;}
    virtual void        d_remove_child (const Go*, int);
    Go*			_child;

    friend void         childClipPostcond (void*, Go*);

    friend class	GoContainerMetaclass;
    friend class	GoTextSegment;
    friend class	GFormatter;
    friend class	GSimpleFormatter;
    friend class	GMCFormatter;
    friend class	GMCFormatterCtxt;
    friend class	GoTextLine;
    friend class	GoText;
};

INLINE GoContainer* GoContainer::create(){
    return (GoContainer*) Go::Create(GoContainer::metaclass());
}

#endif			/* _TextCont_h */
