#ifdef REV_INFO
static char rcsid[] = "$Id: TextEdit.cc,v 1.16 1992/12/30 09:06:56 andre Exp $";
#endif
/*****************************************************************************
MODULE:  	GoText
FILE:    	TextEdit.cc
AUTHOR:		Andre Michel
DATE:    	March 1991

DESCRIPTION:
  Habituals functions used by all predifined Gos.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#include <Go/Tools/Error.h>
#include <Go/GoInterf.h>
#include <Go/Text.h>
#include <Go/Form.h>

#ifdef GoLICENSE
#include "TextCont.h"
#endif /* GoLICENSE */

extern GRegion oldClip;


/**
FUNCTION
    _setString (const char*, GPos)
DESCRIPTION 
    This functions allows the user to set the value of the
    string itself. The new size has to be provided.
**/

void
GoText::_setString (const char*, GPos)
{
    if (_formatter) _formatter->setString();
}


/**
FUNCTION
    _replaceChar (char, GPos);
DESCRIPTION
    This function is used to replace a character inside the string.
**/

void
GoText::_replaceChar (char oldChar, GPos pos)
{
    if (_formatter) _formatter->replaceChar (oldChar, pos);
}


/**
FUNCTION
    _replaceString (GPos, GPos, char*, GPos);
DESCRIPTION
    This function is used to replace a sub-string inside the string.
**/

void
GoText::_replaceString (GPos pos, GPos oldStringSize, char* newString, GPos newStringSize)
{
    if (_formatter)
	_formatter->replaceString (pos, oldStringSize,
            newString, newStringSize);
}


/**
FUNCTION
    _deleteChar (char, pos);
DESCRIPTION
    This function allows to delete the character at the position pos.
**/

void
GoText::_deleteChar (char Char, GPos pos)
{
    if (_formatter) _formatter->deleteChar (Char, pos);
}


/**
FUNCTION
    _deleteString (const char*, GPos, GPos);
DESCRIPTION
    This function allows to delete a string from the text.
**/

void
GoText::_deleteString (const char* str, GPos delSize, GPos pos)
{
    if (_formatter) _formatter->deleteString (str, delSize, pos);
}


/**
FUNCTION
    _insertChar (char Char, GPos pos);
DESCRIPTION
    This function allows to insert a char Char at the pos pos.
    The current pos is set to pos + 1;
**/

void
GoText::_insertChar (char Char, GPos pos)
{
    if (_formatter) _formatter->insertChar (Char, pos);
}


/**
FUNCTION
    _insertString (char *str, GPos len, GPos pos);
DESCRIPTION
    This function allows to insert the string str at the given pos.
**/

void
GoText::_insertString (const char* str, GPos len, GPos pos)
{
    if (_formatter) _formatter->insertString (str, len, pos);
}


/**
FUNCTION
    _present (GPresentation);
DESCRIPTION
    This function allows to insert the string str at the given pos.
**/

void
GoText::_presentation (GPresentation)
{
    GBool was_mapped = mapped();
    if (was_mapped) TempUnmap();
    for (GoTextSegment* s = GSEG (_firstLine->Child());	s; s=s->myNext())
        s->updateWidth();
    if (_formatter){
	_formatter->reframe();
	MyClipHasChanged();
    }
    if (was_mapped) TempMap();
}


/**
FUNCTION
    _lineH (GNumber, GPos, GPos);
DESCRIPTION
    Function used to modify the line height between 2 positions.
**/

void
GoText::_lineH (GNumber newLineHeight, GPos p1, GPos p2)
{
GoTextSegment* segFirst;
GoTextSegment* segLast;

    /* text creation time */
    if (!length()){
        _firstLine->_height = newLineHeight;
        _lineHeight = newLineHeight;
        GSEG(_firstLine->Child())->updateWidth();
        GSEG(_firstLine->Child())->updateHeight();
        if (_formatter) _formatter->reframe();
        MyClipHasChanged();
        return;
    }

    if (p1 != -1){
	createPortion (p1, p2);
        segFirst = posToSeg (p1);
        segLast = posToSeg (p2);

        /* update the lineHeight of every segment */
        for (GoTextSegment* seg = segFirst; seg; seg = seg->myNext()){
#ifdef GoLICENSE
	    if (seg->getMetaclass() != GoContainer::metaclass()){
#endif /* GoLICENSE */
	        if (!seg->_att && newLineHeight != -1)
	            seg->_att = new GStyle();
		if (seg->_att){
		    seg->_att->lineHeight (newLineHeight);
		    seg->updateHighlight();
		    seg->updateWidth();
		    seg->updateHeight();
		}
#ifdef GoLICENSE
	    }
#endif /* GoLICENSE */

	    /* end the loop */
	    if (seg->_lastPos == p2) break;
        }
    }

    /* change of the majority lineHeight */
    else{
	segFirst = GSEG (_firstLine->Child());
	segLast = lastSeg();
	_lineHeight = newLineHeight;

	/* update _lineHeightMax */
	_lineHeightMax = interf()->fontStrikeoutAscent
            (_lineHeight, _stringTextBrush.font()) +
                interf()->fontStrikeoutDescent
                    (_lineHeight, _stringTextBrush.font());

	for (GoTextSegment* seg = segFirst; seg; seg = seg->myNext()){
#ifdef GoLICENSE
	    if (seg->getMetaclass() != GoContainer::metaclass()){
#endif /* GoLICENSE */
		if (!seg->_att || !(seg->_att->mask() & GLINE_HEIGHT)){
		    seg->updateWidth();
		    seg->updateHeight();
		}
		else
		    seg->updateHighlight();
#ifdef GoLICENSE
	    }
#endif /* GoLICENSE */
	}
    }
    if (_formatter){
	_formatter->assembly (segFirst, segLast);
	_formatter->reframe();
    }
}


/**
FUNCTION
    _underlinePos (GNumber, GPos, GPos);
DESCRIPTION
    Function used to modify the Yunderline position of a portion.
**/

void
GoText::_underlinePos (GNumber newUnderPos, GPos p1, GPos p2)
{
GoTextSegment* segFirst;
GoTextSegment* segLast;

    if (p1 != -1){
        createPortion (p1, p2);
        segFirst = posToSeg (p1);
        segLast = posToSeg (p2);

        /* update the Y underlinePostion of every segment */
        for (GoTextSegment* seg = segFirst; seg; seg = seg->myNext()){
#ifdef GoLICENSE
	    if (seg->getMetaclass() != GoContainer::metaclass()){
#endif /* GoLICENSE */
	        if (!seg->_att && newUnderPos != -1)
	            seg->_att = new GStyle();
		if (seg->_att){
	            seg->_att->underlinePosition (newUnderPos);
		    seg->updateHighlight();
		    seg->RedrawBackAndFore();
		}
#ifdef GoLICENSE
   	    }
#endif /* GoLICENSE */

	    /* end the loop */
	    if (seg->_lastPos == p2) break;
        }
    }

    /* change of the majority underlinePostion */
    else{
        segFirst = GSEG (_firstLine->Child());
        segLast = lastSeg();
        _underlinePosition = newUnderPos;
        for (GoTextSegment* seg = segFirst; seg; seg = seg->myNext()){
#ifdef GoLICENSE
	    if (seg->getMetaclass() != GoContainer::metaclass()){
#endif /* GoLICENSE */
		seg->updateHighlight();
		seg->RedrawBackAndFore();
	    }
#ifdef GoLICENSE
        }
#endif /* GoLICENSE */
    }
    if (_formatter) _formatter->assembly (segFirst, segLast);
}


/**
FUNCTION
    background (GBool set, GPos p1, GPos p2)
DESCRIPTION
    Used to map/unmap a portion background.
**/

void
GoText::background (GBool set, GPos p1, GPos p2)
{
GoTextSegment* segFirst;
GoTextSegment* segLast;

    if (p1 != -1){
	createPortion (p1, p2);
	segFirst = posToSeg (p1);
	segLast = posToSeg (p2);

        /* update the underline state of every segment */
        for (GoTextSegment* seg = segFirst; seg; seg = seg->myNext()){
#ifdef GoLICENSE
            if (seg->getMetaclass() != GoContainer::metaclass()){
#endif /* GoLICENSE */
	        if (!seg->_att && set) seg->_att = new GStyle();
		if (seg->_att){
		    seg->_att->mapBackgroundSeg (set);
		    seg->updateHighlight();
		    seg->RedrawBackAndFore();
		}
#ifdef GoLICENSE
	    }
#endif /* GoLICENSE */

	    /* end the loop */
	    if (seg->_lastPos == p2) break;
        }
    }

    /* change of the majority mapBackgroundSeg */
    else{
        segFirst = GSEG (_firstLine->Child());
        segLast = lastSeg();
        _mapBackgroundSeg = set;
        for (GoTextSegment* seg = segFirst; seg; seg = seg->myNext()){
#ifdef GoLICENSE
            if (seg->getMetaclass() != GoContainer::metaclass()){
#endif /* GoLICENSE */
		seg->updateHighlight();
		seg->RedrawBackAndFore();
#ifdef GoLICENSE
	    }
#endif /* GoLICENSE */
        }
    }
    if (_formatter) _formatter->assembly (segFirst, segLast);
}


/**
FUNCTION
    _underline (GBool, GPos, GPos)
DESCRIPTION
    Used to underline a portion.
**/

void
GoText::_underline (GBool set, GPos p1, GPos p2)
{
GoTextSegment* segFirst;
GoTextSegment* segLast;

    if (p1 != -1){
	createPortion (p1, p2);
	segFirst = posToSeg (p1);
	segLast = posToSeg (p2);
	for (GoTextSegment* seg = segFirst; seg; seg = seg->myNext()){
#ifdef GoLICENSE
	    if (seg->getMetaclass() != GoContainer::metaclass()){
#endif /* GoLICENSE */

	        /* existing structure */
	        if (!seg->_att && set) seg->_att = new GStyle();
		if (seg->_att){
		    seg->_att->mapUnderlineSeg (set);
		    seg->updateHighlight();
		    seg->RedrawBackAndFore();
		}
#ifdef GoLICENSE
	    }
#endif /* GoLICENSE */
	    /* end the loop */
	    if (seg->_lastPos == p2) break;
        }
    }
    else{
        segFirst = GSEG (_firstLine->Child());
        segLast = lastSeg();
        _mapUnderlineSeg = set;
        for (GoTextSegment* seg = segFirst; seg; seg = seg->myNext()){
#ifdef GoLICENSE
            if (seg->getMetaclass() != GoContainer::metaclass()){
#endif /* GoLICENSE */
		seg->updateHighlight();
		seg->RedrawBackAndFore();
#ifdef GoLICENSE
	    }
#endif /* GoLICENSE */
        }
    }
    if (_formatter) _formatter->assembly (segFirst, segLast);
}


/**
FUNCTION
    textStringTextBrush (const GTextBrush*, GPos, GPos)
DESCRIPTION
    Used to change the string textBrush of a portion.
**/

void
GoText::textStringTextBrush (const GTextBrush* newTextBrush, GPos p1, GPos p2)
{
    if (newTextBrush)
	textStringTextBrush (newTextBrush->shTextBrush(), p1, p2);
    else
	textStringTextBrush (0, NULL, p1, p2);
}


/**
FUNCTION
    textStringTextBrush (const GShTextBrush*, GPos, GPos)
DESCRIPTION
    Used to change the string textBrush of a portion.
    If the new GShTextBrush is not named, unname the resulting segments.
**/

void
GoText::textStringTextBrush (const GShTextBrush* newShTextBrush, GPos p1, GPos p2)
{
    if (p1 == -1){
	p1 = 0;
	p2 = length();
    }

    if (newShTextBrush && !newShTextBrush->isNamed())
	textStringTextBrush (newShTextBrush->mask(),
	    newShTextBrush->values(), p1, p2);

    else{
	oldClip = myClip();
        if (p1 == p2) return;
        createPortion (p1, p2);
        GoTextSegment* segFirst = posToSeg (p1);

        /* update the string text brush of every segment */
        for (GoTextSegment* seg = segFirst; seg; seg = seg->myNext()){
#ifdef GoLICENSE
            if (seg->getMetaclass() != GoContainer::metaclass()){
#endif /* GoLICENSE */
	        if (!seg->_att && newShTextBrush)
	            seg->_att = new GStyle();
		if (seg->_att){
		    seg->_att->stringShTextBrush (newShTextBrush);
	            seg->updateHighlight();
	            seg->updateWidth();
		}
#ifdef GoLICENSE
	    }
#endif /* GoLICENSE */
	    /* end the loop */
	    if (seg->_lastPos == p2) break;
        }
        if (_formatter){
	    _formatter->assembly (segFirst, posToSeg (p2));
	    _formatter->reframe();
	    MyClipHasChanged();
        }

	/* redraw the text clip differences */
	GRegion deltaClip;
	GRegion myclip = myClip();

	/* the clip may have increased */
	GSub (myclip, oldClip, deltaClip);
	if (!deltaClip.empty()){
	    MyClipHasChanged();
	    RedrawForeOnly (deltaClip);
	}

	/* the clip may have decreased */
	GSub (oldClip, myclip, deltaClip);
	if (!deltaClip.empty()){
	    MyClipHasChanged();
	    RedrawBackAndFore (deltaClip);
	}

	if (backgroundMapped())
	    RedrawForeOnly (myclip);
	else
	    RedrawBackAndFore (myclip);
    }
}


/**
FUNCTION
    textStringTextBrush (ulong, const GTextBrushValues*, GPos, GPos)
DESCRIPTION
    Used to change the string textBrush of a portion.
**/

void
GoText::textStringTextBrush (ulong mask, const GTextBrushValues* values,
    GPos p1, GPos p2)
{
    if (p1 == -1){
	p1 = 0;
	p2 = length();
    }

    oldClip = myClip();
    if (p1 == p2) return;
    createPortion (p1, p2);
    GoTextSegment* segFirst = posToSeg (p1);

    /* update the string text brush of every segment */
    for (GoTextSegment* seg = segFirst; seg; seg = seg->myNext()){
#ifdef GoLICENSE
	if (seg->getMetaclass() != GoContainer::metaclass()){
#endif /* GoLICENSE */
	    if (!seg->_att && mask && values){
	        seg->_att = new GStyle();
                if (!(seg->_att->mask() & GSTRING_SHTEXTBRUSH))
                    seg->_att->stringShTextBrush (
                        _stringTextBrush.shTextBrush());
	    }
	    if (seg->_att){
		seg->_att->stringShTextBrush (mask, values);
		seg->updateHighlight();
		seg->updateWidth();
                seg->RedrawForeOnly();
	    }
#ifdef GoLICENSE
	}
#endif /* GoLICENSE */

	/* end the loop */
	if (seg->_lastPos == p2) break;
    }
    if (_formatter){
	_formatter->assembly (segFirst, posToSeg (p2));
	if (mask & GFONT){
	    _formatter->reframe();
	    MyClipHasChanged();

           /* redraw the text clip differences */
           GRegion deltaClip;
           GRegion myclip = myClip();

           /* the clip may have increased */
           GSub (myclip, oldClip, deltaClip);
           if (!deltaClip.empty()){
	       MyClipHasChanged();
	       RedrawForeOnly (deltaClip);
           }

           /* the clip may have decreased */
           GSub (oldClip, myclip, deltaClip);
           if (!deltaClip.empty()){
	       MyClipHasChanged();
	       RedrawBackAndFore (deltaClip);
           }

           if (backgroundMapped())
	       RedrawForeOnly (myclip);
           else
	       RedrawBackAndFore (myclip);
	}
    }
}


/**
FUNCTION
    textBackgroundBrush (const GBrush*, GPos, GPos)
DESCRIPTION
    Used to change the background brush of a portion.
**/

void
GoText::textBackgroundBrush (const GBrush* newBrush, GPos p1, GPos p2)
{
    if (newBrush)
	textBackgroundBrush (newBrush->shBrush(), p1, p2);
    else
	textBackgroundBrush (0, NULL, p1, p2);
}


/**
FUNCTION
    textBackgroundBrush (const GShBrush*, GPos, GPos)
DESCRIPTION
    Used to change the background brush of a portion.
    If the new GShBrush is not named, unname the resulting segments.
**/

void
GoText::textBackgroundBrush (const GShBrush* newShBrush, GPos p1, GPos p2)
{
    if (p1 == -1){
	p1 = 0;
	p2 = length();
    }
    if (newShBrush && !newShBrush->isNamed())
	textBackgroundBrush (newShBrush->mask(),
	    newShBrush->values(), p1, p2);
    else{
        if (p1 == p2) return;
        createPortion (p1, p2);
        GoTextSegment* segFirst = posToSeg (p1);

        /* update the background brush of every segment */
        for (GoTextSegment* seg = segFirst; seg; seg = seg->myNext()){
#ifdef GoLICENSE
            if (seg->getMetaclass() != GoContainer::metaclass()){
#endif /* GoLICENSE */
	        if (!seg->_att && newShBrush) seg->_att = new GStyle();
		if (seg->_att){
		    seg->_att->backgroundShBrush (newShBrush);
		    seg->updateHighlight();
		    seg->RedrawBackAndFore();
		}
#ifdef GoLICENSE
	    }
#endif /* GoLICENSE */
	    /* end the loop */
	    if (seg->_lastPos == p2) break;
        }
        if (_formatter) _formatter->assembly (segFirst, posToSeg (p2));
    }
}


/**
FUNCTION
    textBackgroundBrush (ulong, const GBrushValues*, GPos, GPos)
DESCRIPTION
    Used to change the background brush of a portion.
**/

void
GoText::textBackgroundBrush (ulong mask, const GBrushValues* values,
    GPos p1, GPos p2)
{
    if (p1 == -1){
	p1 = 0;
	p2 = length();
    }

    if (p1 == p2) return;
    createPortion (p1, p2);
    GoTextSegment* segFirst = posToSeg (p1);

    /* update the background brush of every segment */
    for (GoTextSegment* seg = segFirst; seg; seg = seg->myNext()){

#ifdef GoLICENSE
        if (seg->getMetaclass() != GoContainer::metaclass()){
#endif /* GoLICENSE */
	    if (!seg->_att && mask && values) seg->_att = new GStyle();
	    if (seg->_att){
		seg->_att->backgroundShBrush (mask, values);
		seg->updateHighlight();
		seg->RedrawBackAndFore();
	    }
#ifdef GoLICENSE
	}
#endif /* GoLICENSE */
	/* end the loop */
	if (seg->_lastPos == p2) break;
    }
    if (_formatter) _formatter->assembly (segFirst, posToSeg (p2));
}


/**
FUNCTION
    textUnderlinePen (const GPen*, GPos, GPos)
DESCRIPTION
    Used to change the underline pen of a portion.
**/

void
GoText::textUnderlinePen (const GPen* newPen, GPos p1, GPos p2)
{
    if (newPen)
	textUnderlinePen (newPen->shPen(), p1, p2);
    else
	textUnderlinePen (0, NULL, p1, p2);
}


/**
FUNCTION
    textUnderlinePen (const GShPen*, GPos, GPos)
DESCRIPTION
    Used to change the underline pen of a portion.
    If the new GShPen is not named, unname the resulting segments.
**/

void
GoText::textUnderlinePen (const GShPen* newShPen, GPos p1, GPos p2)
{
    if (p1 == -1){
	p1 = 0;
	p2 = length();
    }
    if (newShPen && !newShPen->isNamed())
	textUnderlinePen (newShPen->mask(), newShPen->values(), p1, p2);
    else{
        if (p1 == p2) return;
        createPortion (p1, p2);
        GoTextSegment* segFirst = posToSeg (p1);

        /* update the underline pen of every segment */
        for (GoTextSegment* seg = segFirst; seg; seg = seg->myNext()){
#ifdef GoLICENSE
            if (seg->getMetaclass() != GoContainer::metaclass()){
#endif /* GoLICENSE */
	        if (!seg->_att && newShPen)
	            seg->_att = new GStyle();
		if (seg->_att){
		    seg->_att->underlineShPen (newShPen);
		    seg->updateHighlight();
		    seg->RedrawBackAndFore();
		}
#ifdef GoLICENSE
	    }
#endif /* GoLICENSE */
	    /* end the loop */
	    if (seg->_lastPos == p2) break;
        }
        if (_formatter) _formatter->assembly (segFirst, posToSeg (p2));
    }
}


/**
FUNCTION
    textUnderlinePen (ulong, const GShPenValues*, GPos, GPos)
DESCRIPTION
    Used to change the underline pen of a portion.
**/

void
GoText::textUnderlinePen (ulong mask, const GPenValues* values,
    GPos p1, GPos p2)
{
    if (p1 == -1){
	p1 = 0;
	p2 = length();
    }

    if (p1 == p2) return;
    createPortion (p1, p2);
    GoTextSegment* segFirst = posToSeg (p1);

    /* update the background brush of every segment */
    for (GoTextSegment* seg = segFirst; seg; seg = seg->myNext()){
#ifdef GoLICENSE
        if (seg->getMetaclass() != GoContainer::metaclass()){
#endif /* GoLICENSE */
	    if (!seg->_att && mask && values)
	        seg->_att = new GStyle();
	    if (seg->_att){
		seg->_att->underlineShPen (mask, values);
		seg->updateHighlight();
		seg->RedrawBackAndFore();
	    }
#ifdef GoLICENSE
	}
#endif /* GoLICENSE */
	/* end the loop */
	if (seg->_lastPos == p2) break;
    }
    if (_formatter) _formatter->assembly (segFirst, posToSeg (p2));
}


/**
FUNCTION
    extraBaseline (GNumber, GPos, GPos)
DESCRIPTION
    Used to change the underline pen of a portion.
**/

GBool
#ifdef GoLICENSE
GoText::extraBaseline (GNumber exp, GPos p1, GPos p2)
{
GBool ret = GTRUE;

    if (p1 == p2) return GFALSE;
    GModifyAttributeStruct *str = new GModifyAttributeStruct;
    str->type = GEXTRA_BASELINE_CHANGE_NOTIFY;
    str->baseline = exp;
    str->firstPosition = p1;
    str->lastPosition = p2;

    /* extra baseline preconditions call */
    if (ret = _call_modify_attribute_precond (str) &&
	((GoTextMetaclass*)getMetaclass())->
	    _call_modify_attribute_precond (this, str) &&
	        GoText::metaclass()->_call_modify_attribute_precond (
                    this, str))
    {
        createPortion (p1, p2);
        GoTextSegment* segFirst = posToSeg (p1);
	oldClip = myClip();

        /* update the background brush of every segment */
        for (GoTextSegment* seg = segFirst; seg; seg = seg->myNext()){
	    if (!seg->_att && exp != -1) seg->_att = new GStyle();
	    if (seg->_att){
		seg->_att->extraBaseline (exp);
		seg->updateHighlight();
	    }
	    /* end the loop */
	    if (seg->_lastPos == p2) break;
        }
        if (_formatter){
	    GoTextLine* firstLine = GLINE (segFirst->parent());
	    _formatter->assembly (segFirst, posToSeg (p2));
	    _formatter->organize (firstLine, _lastLine);
        }

	/* redraw the text clip differences */
	GRegion deltaClip;
	GRegion myclip = myClip();

	/* the clip may have increased */
	GSub (myclip, oldClip, deltaClip);
	if (!deltaClip.empty()){
	    MyClipHasChanged();
	    RedrawForeOnly (deltaClip);
	}

	/* the clip may have decreased */
	GSub (oldClip, myclip, deltaClip);
	if (!deltaClip.empty()){
	    MyClipHasChanged();
	    RedrawBackAndFore (deltaClip);
	}
	if (backgroundMapped())
	    RedrawForeOnly (myclip);
	else
	    RedrawBackAndFore (myclip);

        /* extra baseline postconditions call */
        _call_modify_attribute_postcond (str);
        ((GoTextMetaclass*)getMetaclass())->
	    _call_modify_attribute_postcond (this, str);
        GoText::metaclass()->_call_modify_attribute_postcond (this, str);
    }
    delete str;

#else /* GoLICENSE */

GoText::extraBaseline (GNumber, GPos, GPos)
{
GBool ret = GTRUE;

#endif /* GoLICENSE */

    return ret;
}


/**
FUNCTION
    highlight (const GStyle, GPos, GPos)
DESCRIPTION
    Used to highlight the given portion.
**/

void
#ifdef GoLICENSE
GoText::highlight (const GStyle& hl, GPos p1, GPos p2)
{
    if (p1 == -1){
	p1 = 0;
	p2 = length();
    }

    
    ulong mask = hl.mask();
    GBool reframe_was_enabled = formatter()->_enableReframe;
    if (reframe_was_enabled) formatter()->disableReframe();

    if (mask & GMAP_UNDERLINE_SEG)
	underline (hl.mapUnderlineSeg(), p1, p2);
    else
	underline (GFALSE, p1, p2);

    if (mask & GMAP_BACKGROUND_SEG)
	background (hl.mapBackgroundSeg(), p1, p2);
    else
	background (GFALSE, p1, p2);

    if (mask & GBACKGROUND_SHBRUSH)
	textBackgroundBrush (hl.backgroundShBrush(), p1, p2);
    else
	textBackgroundBrush ((const GShBrush*)NULL, p1, p2);

    if (mask & GSTRING_SHTEXTBRUSH)
	textStringTextBrush (hl.stringShTextBrush(), p1, p2);
    else
	textStringTextBrush ((const GShTextBrush*)NULL, p1, p2);

    if (mask & GUNDERLINE_SHPEN)
	textUnderlinePen (hl.underlineShPen(), p1, p2);
    else
	textUnderlinePen ((const GShPen*)NULL, p1, p2);

    if (mask & GLINE_HEIGHT)
	lineHeight (hl.lineHeight(), p1, p2);
    else
	lineHeight (-1, p1, p2);

    if (mask & GUNDERLINE_POSITION)
	underlinePosition (hl.underlinePosition(), p1, p2);
    else
	underlinePosition (-1, p1, p2);

    if (mask & GEXTRA_BASELINE)
	extraBaseline (hl.extraBaseline(), p1, p2);
    else
	extraBaseline (-1, p1, p2);

    if (reframe_was_enabled){
        formatter()->enableReframe();
        formatter()->reframe();
    }

#else /* GoLICENSE */
    
GoText::highlight (const GStyle&, GPos, GPos)
{
#endif /* GoLICENSE */    
}




