#ifdef REV_INFO
static char rcsid[] = "$Id: TextLn.cc,v 1.13 1992/12/23 16:43:54 andre Exp $";
#endif
/*****************************************************************************
MODULE:  GoText
FILE:	 TextLn.cc
AUTHOR:  Michel Andre
DATE:    August 91
 

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
*****************************************************************************/

#include <Go/GoInterf.h>
#include <Go/GoStream.h>
#include <Go/Text.h>
#include <Go/mTextLn.h>
#include <Go/Form.h>

#ifdef GoLICENSE
#include "TextCont.h"
#endif  /* GoLICENSE */

#define VERSION_1_2		(1 << 15)
#define VERSION_1_2_BIS		(1 << 14)
extern long streamMask;

static GoTextLineMetaclass* gotextline_metaclass = NULL;


/**
FUNCTION
    metaclass()
DESCRIPTION
    The GoTextLine metaclass function.
**/

GoMetaclass* GoTextLine::metaclass()
{
    if (gotextline_metaclass == NULL){
	gotextline_metaclass = new GoTextLineMetaclass;
	gotextline_metaclass->superclass()->
	    AddSubclass (gotextline_metaclass);
	GoStream::bind (gotextline_metaclass->classname(),
	    (int (*) (const void*, GoStream*)) GoTextLine::save,
	    (int (*) (void**, GoStream*)) GoTextLine::restore);
    }
    return gotextline_metaclass;
}


/**
FUNCTION
    m_superclass()
DESCRIPTION
    The GoTextLineMetaclass destructor.
**/

GoTextLineMetaclass::~GoTextLineMetaclass()
{
    if (this == gotextline_metaclass){
	superclass()->RemoveSubclass (this);
	gotextline_metaclass = NULL;
    }
}


static GActionRec no_actions[] = {{NULL, NULL}};
const GActionRec* GoTextLineMetaclass::m_actions() const
{
    return no_actions;
}


/**
FUNCTION
    m_metaclass()
DESCRIPTION
    The GoTextLine m_metaclass function.
**/

GoMetaclass* GoTextLine::m_metaclass() const
{
    return GoTextLine::metaclass();
}


/**
FUNCTION
    m_superclass()
DESCRIPTION
    The GoTextLineMetaclass m_superclass function.
**/

GoMetaclass* GoTextLineMetaclass::m_superclass() const
{
    return Go::metaclass();
}


/**
FUNCTION
    m_create()
DESCRIPTION
    The GoTextLine m_create function.
**/

Go*
GoTextLineMetaclass::m_create() const
{
    return (Go*)(new GoTextLine);
}


/**
FUNCTION
    m_classname()
DESCRIPTION
    The GoTextLine m_classname function.
**/

const char*
GoTextLineMetaclass::m_classname() const
{
    return "GoTextLine";
}


/**
FUNCTION
    m_dup()
DESCRIPTION
    The GoTextLine m_dup function.
**/

void
GoTextLine::m_dup
    (const Go* orig, Go* parcpy, Go* prvcpy, GBool donxt, GBool dochildren)
{
    Go::m_dup (orig, parcpy, prvcpy, donxt, dochildren);

    const GoTextLine* line_orig = (const GoTextLine*)orig;
    _parent = parcpy;
    _origin = line_orig->_origin;
    _width = line_orig->_width;
    _height = line_orig->_height;

    for (Go* orig_ch = line_orig->_child; orig_ch; orig_ch = orig_ch->Next())
	if (_child = orig_ch->Dup(this, NULL, GTRUE, GTRUE))
	    break;

    if (donxt)
	for (Go* orig_nxt = line_orig->_next; orig_nxt; orig_nxt = orig_nxt->Next())
	    if (_next = orig_nxt->Dup (parcpy, this, GTRUE, GTRUE))
		break;
}


Go* GoTextLine::d_get_previous() const
{
    return (_parent)
	? _parent->Child(_parent->childIndex(this) - 1)
	    : NULL;
}


void GoTextLine::d_remove_child (const Go* go, int)
{
    if (go == _child)
	_child = go->Next();
}

void GoTextLine::d_add_child (const Go* go, int index)
{
    if (index == 0)
	_child = (Go*)go;
}

void GoTextLine::d_set_tree (const Go* parent, const Go*, const Go* next)
{
    _parent = (Go*)parent;
    _next = (Go*)next;
}

static GCoordSys static_relcs;
const GCoordSys& GoTextLine::d_get_relcs (const GCoordSys*) const
{
    static_relcs.O = _origin;
    return static_relcs;
}


GRegion* GoTextLine::d_get_clip () const
{
    const GRegion* region = &_clip;
    return (GRegion*)region;
}


/**
FUNCTION
    GoTextLine();
DESCRIPTION
    The GoTextLine constructor.
**/

GoTextLine::GoTextLine()
{
    _parent = NULL;
    _next = NULL;
    _child = NULL;
    _width = 0;
    _height = 0;
    GoTextSegment* seg = GoTextSegment::create();
    seg->parentRelativeReparent (this);
    crossType (GCROSS_AS_NOTHING);
}


int
GoTextLine::save (const GoTextLine* go, GoStream* stream)
{
    extern int save_children (const Go*, GoStream*);
    return
        save_children (go->_child, stream) &&
        Go::save (go, stream);
}


/**
FUNCTION
    int GoTextLine::restore (GoTextLine**, GoStream*);
DESCRIPTION
    The GoTextLine restore function.
    The GoTextLine save function is inherited from the
    Go class.
**/

int
GoTextLine::restore (GoTextLine** line, GoStream* gostream)
{
    extern int restore_children (Go*, GoStream*);
    
    if (*line = GoTextLine::create()){
	GSEG ((*line)->Child())->destroy();
	if (!(streamMask & VERSION_1_2_BIS)){
	    GCoordSys cs;
            if (!gostream->readCoordSys (&cs)) return 0;
	}
	return restore_children (*line, gostream) &&
	    Go::readParams (*line, gostream);
    }
    return 0;
}


/**
FUNCTION
    GoTextLine* GoTextLine::PreviousLine();
DESCRIPTION
    Returns the previous line.
**/

GoTextLine*
GoTextLine::PreviousLine()
{
    return (this == GTEXT (parent())->_firstLine) ?
	NULL : GLINE (Previous());
}


/**
FUNCTION
    GoTextLine* GoTextLine::NextLine();
DESCRIPTION
    Returns the following line.
**/

GoTextLine*
GoTextLine::NextLine()
{
    return (this == GTEXT (parent())->_lastLine) ?
	NULL : GLINE (Next());
}


/**
FUNCTION
    organize (GNumber width)
DESCRIPTION
    organizes the segments inside the line depending on the
    text justification, indentation and width.
**/

#define FLOAT_EQUAL(x,y)	(fabs ((x) - (y)) < 0.0001)

void
GoTextLine::organize (GNumber width)
{
    GoInterf* interf = this->interf();
    GPoint segmentOrigin (0, 0);
    GJustification justification = GTEXT (parent())
	->_justification;

    /* initialize the geometry of the line */
    _height = 0;
    _width = 0;
    GNumber segmentsSumWidth = 0;
    GNumber glueSumWidth = 0;
    GNumber lineBaseline = 0;

    for (GoTextSegment* seg = GSEG (Child());
        seg; seg = GSEG (seg->Next()))
    {
        GNumber segBaseline;

        /* container */
#ifdef GoLICENSE
        if (seg->getMetaclass() == GoContainer::metaclass())
	    segBaseline = seg->_height / 2;

        /* text segment */
        else
#endif /* GoLICENSE */
            segBaseline = interf->fontStrikeoutAscent (
                seg->lineHeight(),
                    seg->stringShTextBrush()->font());

        segBaseline += seg->extraBaseline();
        lineBaseline = max (segBaseline, lineBaseline);

        /* update line geometry */
        _height = max (_height,
            seg->_height + (GNumber)fabs (seg->extraBaseline()));
        segmentsSumWidth += seg->_width;

        /* add the glue width */
        if (justification == GLR_JUSTIFICATION && seg->Next())
            glueSumWidth += seg->normalSpace();
    }

    /* compute the ratio */
    GNumber ratio = 0;
    if (justification == GLR_JUSTIFICATION){
	if (GSEG (Child())->indented())
	    segmentOrigin.x =
		GTEXT (parent())->formatter()->indentation (this);

	/* glue if it is not the last line of a paragraph */
	GoTextLine* nextLine = NextLine();
	if ((nextLine && !GSEG (nextLine->Child())->indented()
	    && glueSumWidth != 0) || glueSumWidth + segmentsSumWidth > width)
		ratio = (segmentsSumWidth + glueSumWidth +
		    segmentOrigin.x - width) / glueSumWidth;
	_width = width;
    } 

    else
        _width = segmentsSumWidth;

    for (seg = GSEG (Child()); seg; seg = GSEG (seg->Next())){

#ifdef GoLICENSE
	/* container */
	if (seg->getMetaclass() == GoContainer::metaclass())
	    segmentOrigin.y = (_height - seg->_height) / 2;

	/* text segment */
	else
#endif /* GoLICENSE */
	    segmentOrigin.y = (seg->_height != _height) ?
		lineBaseline - interf->fontStrikeoutAscent
		    (seg->lineHeight(),
			seg->stringShTextBrush()->font()) :
		0;

	/* extra baseline */
	segmentOrigin.y -= seg->extraBaseline();

	if (!FLOAT_EQUAL (segmentOrigin.x, seg->Relcs().O.x) ||
	    !FLOAT_EQUAL (segmentOrigin.y, seg->Relcs().O.y))
		 seg->move (segmentOrigin, this);

	segmentOrigin.x += seg->_width;

	/* add the glue width if necessary */
	if (justification == GLR_JUSTIFICATION)
	    segmentOrigin.x += seg->normalSpace() * (1 - ratio);
    }
}






