#ifdef REV_INFO
static char rcsid[] = "$Id: TextSg.cc,v 1.17 1992/12/23 16:43:56 andre Exp $";
#endif
/*****************************************************************************
MODULE:  GoText
FILE:	 TextSg.cc
AUTHOR:  Michel Andre
DATE:    August 91
 

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
*****************************************************************************/

#include <ctype.h>
#include <Go/GoInterf.h>
#include <Go/GoStream.h>
#include <Go/Text.h>
#include <Go/mTextSg.h>

#ifdef GoLICENSE
#include "TextCont.h"
#endif /* GoLICENSE */


static GoTextSegmentMetaclass* gotextsegment_metaclass = NULL;

extern long streamMask;

#define VERSION_1_2		(1 << 15)
#define VERSION_1_2_BIS		(1 << 14)

GoMetaclass* GoTextSegment::metaclass()
{
    if (gotextsegment_metaclass == NULL)
    {
	gotextsegment_metaclass = new GoTextSegmentMetaclass;
	gotextsegment_metaclass->superclass()->
	    AddSubclass(gotextsegment_metaclass);
	GoStream::bind(gotextsegment_metaclass->classname(),
	    (int (*) (const void*, GoStream*)) GoTextSegment::save,
	    (int (*) (void**, GoStream*)) GoTextSegment::restore);
    }

    return gotextsegment_metaclass;
}

GoTextSegmentMetaclass::~GoTextSegmentMetaclass()
{
    if (this == gotextsegment_metaclass)
    {
	superclass()->RemoveSubclass(this);
	gotextsegment_metaclass = NULL;
    }
}

GoMetaclass* GoTextSegmentMetaclass::m_superclass() const
{
    return Go::metaclass();
}

GoMetaclass* GoTextSegment::m_metaclass() const
{
    return GoTextSegment::metaclass();
}

static GActionRec no_actions[] = {{NULL, NULL}};
const GActionRec* GoTextSegmentMetaclass::m_actions() const
{
    return no_actions;
}

Go*
GoTextSegmentMetaclass::m_create() const
{
    return new GoTextSegment;
}


const char*
GoTextSegmentMetaclass::m_classname() const
{
    return "GoTextSegment";
}


/**
FUNCTION
    m_dup()
DESCRIPTION
    The GoTextSegment m_dup function.
**/

void
GoTextSegment::m_dup
    (const Go* orig, Go* parcpy, Go* prvcpy, GBool donxt, GBool dochildren)
{
    Go::m_dup(orig, parcpy, prvcpy, donxt, dochildren);

    const GoTextSegment* seg_orig = (const GoTextSegment*)orig;
    _parent = parcpy;
    _origin = seg_orig->_origin;
    _firstPos = seg_orig->_firstPos;
    _lastPos = seg_orig->_lastPos;
    _width = seg_orig->_width;
    _height = seg_orig->_height;
    _hyphened = seg_orig->_hyphened;
    _att = (seg_orig->_att != NULL) ?
	new GStyle (seg_orig->_att) : NULL;

    if (donxt)
	for (Go* orig_nxt = seg_orig->_next; orig_nxt;
	    orig_nxt = orig_nxt->Next())
	        if (_next = orig_nxt->Dup(parcpy, this, GTRUE, GTRUE))
		    break;
}


Go* GoTextSegment::d_get_previous() const
{
    return (_parent)
	? _parent->Child (_parent->childIndex (this) - 1)
	    : NULL;
}


void GoTextSegment::d_remove_child (const Go*, int)
{
    GError ("GoTextSegment::d_remove_child()");
}

void GoTextSegment::d_add_child (const Go*, int)
{
    GError ("GoTextSegment::d_add_child()");
}

void GoTextSegment::d_set_tree (const Go* parent, const Go*, const Go* next)
{
    _parent = (Go*)parent;
    _next = (Go*)next;
}


static GCoordSys static_relcs;
const GCoordSys& GoTextSegment::d_get_relcs (const GCoordSys*) const
{
    static_relcs.O = _origin;
    return static_relcs;
}


/**
FUNCTION
    contains (const GPoint &pt, GNumber maxDistance);
DESCRIPTION
    The GoText contains function.
**/

GBool
GoTextSegment::m_contains (const GPoint &abs_pt, GNumber) const
{
    return myClip().contains (abs_pt);
}


/**
FUNCTION
    GoTextSegment();
DESCRIPTION
    The segment constructor.
**/

GoTextSegment::GoTextSegment()
{
    _parent = NULL;
    _next = NULL;
    _att = NULL;
    _firstPos = 0;
    _lastPos = 0;
    _width = 0;
    _height = 0;
    _hyphened = GFALSE;
    crossType (GCROSS_AS_NOTHING);
}


/**
FUNCTION
    ~GoTextSegment();
DESCRIPTION
    The segment destructor.
**/

GoTextSegment::~GoTextSegment()
{
    if (_att) delete _att;
}


void
GoTextSegment::m_draw (const GRegion&) const
{
    GCoordSys cs = Abscs();
    GoText* text = GTEXT (parent()->parent());

    const GShTextBrush* shtextbrush = stringShTextBrush();

    /* glue is not included in width */
    GNumber widthWithGlue =
	text->_justification == GLR_JUSTIFICATION && Next() ?
             Next()->Relcs().O.x - Relcs().O.x : _width;

    /* background */
    if (_firstPos != _lastPos && mapBackgroundSeg())
	interf()->rectangle (NULL, backgroundShBrush(), cs,
	    GPoint (0, 0), widthWithGlue, _height);

    /* selected part */
    char* str = text->_string + _firstPos;
    for (GTextSeg* s = text->selectionList().first();
	s; s=s->next())
    {
	if (s->first() >= _lastPos) break;
	if (s->last() > _firstPos){
	    GNumber x1 = (s->first() > _firstPos) ?
		interf()->stringWidth (str, s->first() - _firstPos,
		    lineHeight(), shtextbrush->font()) :
            0;

	    GNumber x2 = (s->last() < _lastPos) ?
		interf()->stringWidth (str, s->last() - _firstPos,
		    lineHeight(), shtextbrush->font()) :
            widthWithGlue;

	    interf()->rectangle (NULL, &(text->_selectionBrush), cs,
		GPoint (x1, 0), x2-x1, _height);
	}
    }

    /* cursor */
    GPos currentPos = text->_currentPos;
    if (text->cursorMapped() && currentPos >= _firstPos
	&& currentPos <= _lastPos)
    {
        GoTextSegment* myNext = this->myNext();
	
        /* the cursor may be drawn by another segment */
	if (currentPos != _lastPos || !myNext ||
               myNext->_firstPos != _lastPos || myNext->indented())
	{
	    GNumber xCursor = interf()->stringWidth
	        (text->_string+_firstPos, currentPos -_firstPos,
		    lineHeight(), shtextbrush->font());

	    interf()->line (&text->_cursorPen, cs, GPoint (xCursor, 0),
	        GPoint (xCursor, _height - interf()->pixelHeight() /
		    cs.J.size()));
        }
    }

    /* string */
    if (_firstPos != _lastPos && text->stringMapped()){
	int nbCharToDraw = _lastPos - _firstPos;

	/* the "hyphen character" */
	if (_hyphened){
	    char string [128];
	    strncpy (string, text->_string + _firstPos, nbCharToDraw);
	    string [nbCharToDraw++] = '-';
	    interf()->string (shtextbrush, NULL, cs, GPoint(0, 0),
		lineHeight(), nbCharToDraw, string);
	}
	else
	    interf()->string (shtextbrush, NULL, cs, GPoint(0, 0),
		lineHeight(), nbCharToDraw, text->_string + _firstPos);
    }

    /* underlining */
    if (_firstPos != _lastPos && mapUnderlineSeg()
        && (text->underlineMapped()))
    {
        GNumber h = _height * underlinePosition();
        interf()->line (underlineShPen(), cs, GPoint (0, h),
	    GPoint (widthWithGlue, h));
    }
}


/**
FUNCTION
    void GoTextSegment::m_add_clip (GRegion&) const
DESCRIPTION
    The GoTextSegment m_add_clip function.
**/

void
GoTextSegment::m_add_clip (GRegion& reg) const
{
    GoText* text = GTEXT (parent()->parent());
    GNumber width = _width;
    if (text->justification() == GLR_JUSTIFICATION && Next())
        width = Next()->Relcs().O.x - Relcs().O.x;

    GCoordSys cs = Abscs();
    GAdd (reg, cs.to (GBox (0, 0, width, _height)), reg);

    /* add the cursor region */
    GPos currentPos = text->_currentPos;
    if (text->cursorMapped() && currentPos >= _firstPos &&
        currentPos <= _lastPos)
    {
        const GTextBrush textBr = GTextBrush (stringShTextBrush());
	GNumber xCursor = interf()->stringWidth (text->_string+_firstPos,
	    currentPos -_firstPos, lineHeight(), (&textBr)->font());
        interf()->lineClips (&(text->_cursorPen), cs, GPoint (xCursor, 0),
            GPoint (xCursor, _height), &reg, NULL, GMAX_QUALITY);
    }

    /* add the underlining region */
    if (text->underlineMapped() && mapUnderlineSeg()){
	GNumber yLine = _height * underlinePosition();
	GPen pen = GPen (underlineShPen());
	interf()->lineClips ((const GPen*)&pen, cs, GPoint (0, yLine),
	    GPoint (width, yLine), &reg, NULL, GMAX_QUALITY);
    }
}


void
GoTextSegment::updateWidth()
{
    GoText* text = GTEXT (parent()->parent());
    char* str = text->_string;
    GNumber width = interf()->stringWidth (str + _firstPos,
	_lastPos - _firstPos, lineHeight(),
	    stringShTextBrush()->font());
    if (width != _width){
	_width = width;
	MyClipHasChanged();
    }
}


void
GoTextSegment::updateHeight()
{
    GNumber height = interf()->fontStrikeoutAscent
	(lineHeight(), stringShTextBrush()->font()) +
	    interf()->fontStrikeoutDescent (lineHeight(),
		stringShTextBrush()->font());

    if (height != _height){
	_height = height;
	MyClipHasChanged();
    }
}


GoTextSegment*
GoTextSegment::myNext() const
{
    Go* go = Next();
    if (!go){
	go = GLINE (parent())->NextLine();
	if (go) go=go->Child();
    }
    return GSEG (go);
}


GoTextSegment*
GoTextSegment::myPrev()
{
    Go* go = Previous();
    if (!go){
	go = GLINE (parent())->PreviousLine();
	if (go)
	    for (go=go->Child(); go->Next(); go=go->Next());
    }
    return GSEG (go);
}


/**
FUNCTION
    int GoTextSegment::save (const GoTextSegment*, GoStream*);
DESCRIPTION
    The GoTextSegment save method.
**/

int
GoTextSegment::save (const GoTextSegment* seg, GoStream* gostream)
{
    if (!gostream->writeInt (seg->_firstPos,
        "firstPos")) return 0;
    if (!gostream->writeInt (seg->_lastPos,
        "lastPos")) return 0;
    if (!gostream->writeBool (seg->_hyphened,
	"hyphened")) return 0;

    /* mask */
    ulong mask = seg->_att ? seg->_att->mask() : 0;
    if (!gostream->writeLong (mask, "mask")) return 0;

    if (mask){
        if (mask & GBACKGROUND_SHBRUSH &&
	    !gostream->saveObj (seg->_att->backgroundShBrush(),
	        GShBrush::classname(), "backgroundBrush"))
		    return 0;

        if (mask & GSTRING_SHTEXTBRUSH &&
	    !gostream->saveObj (seg->_att->stringShTextBrush(),
	        GShTextBrush::classname(), "stringTextBrush"))
		    return 0;

        if (mask & GUNDERLINE_SHPEN &&
	    !gostream->saveObj (seg->_att->underlineShPen(),
	        GShPen::classname(), "underlinePen"))
		    return 0;

        if (mask & GMAP_UNDERLINE_SEG &&
	    !gostream->writeBool (seg->_att->mapUnderlineSeg(),
	        "mapUnderlineSeg")) return 0;

        if (mask & GMAP_BACKGROUND_SEG &&
	    !gostream->writeBool (seg->_att->mapBackgroundSeg(),
	        "mapBackgroundSeg")) return 0;

        if (mask & GLINE_HEIGHT &&
	    !gostream->writeNumber (seg->_att->lineHeight(),
	        "lineHeight")) return 0;

        if (mask & GUNDERLINE_POSITION &&
	    !gostream->writeNumber (seg->_att->underlinePosition(),
	        "underlinePosition")) return 0;

        if (mask & GEXTRA_BASELINE &&
	    !gostream->writeNumber (seg->_att->extraBaseline(),
	        "extraBaseline")) return 0;
    }
    
    return Go::save (seg, gostream);
}


/**
FUNCTION
    int GoTextSegment::restore (GoTextSegment**, GoStream*);
DESCRIPTION
    The GoTextSegment restore method.
**/

int
GoTextSegment::restore (GoTextSegment** seg, GoStream* gostream)
{
    extern int restore_children (Go*, GoStream*);

    if (*seg = GoTextSegment::create()){
	const char* class_name = NULL;

        if (!gostream->readInt (&((*seg)->_firstPos)))
	    return 0;
        if (!gostream->readInt (&((*seg)->_lastPos)))
	    return 0;

	GBool b;

        /* indented (unused now) */
        if (!(streamMask & VERSION_1_2_BIS) &&
	    !gostream->readBool (&b)) return 0;

        if (!gostream->readBool (&b)) return 0;
        (*seg)->_hyphened = b;

	/* old version (linked bit) */
        if (!(streamMask & VERSION_1_2_BIS) &&
	    !gostream->readBool (&b)) return 0;

	/* segment attributes */
	long mask;
        if (!gostream->readLong (&mask)) return 0;

	if (mask != 0){
	    GNumber n;
	    (*seg)->_att = new GStyle;

	    if (mask & GBACKGROUND_SHBRUSH){
	        GShBrush* shbrush;
	        if (!gostream->restoreObj ((void**) &shbrush,
		    &class_name)) return 0;
	        (*seg)->_att->backgroundShBrush (shbrush);
	    }

	    if (mask & GSTRING_SHTEXTBRUSH){
	        GShTextBrush* shtextbrush;
	        if (!gostream->restoreObj ((void**) &shtextbrush,
		    &class_name)) return 0;
	        (*seg)->_att->stringShTextBrush (shtextbrush);
	    }

	    if (mask & GUNDERLINE_SHPEN){
	        GShPen* shpen;
	        if (!gostream->restoreObj ((void**) &shpen,
		    &class_name)) return 0;
	        (*seg)->_att->underlineShPen (shpen);
	    }

	    if (mask & GMAP_UNDERLINE_SEG){
	        if (!gostream->readBool (&b)) return 0;
	        (*seg)->_att->mapUnderlineSeg (b);
	    }

	    if (mask & GMAP_BACKGROUND_SEG){
	        if (!gostream->readBool (&b)) return 0;
	        (*seg)->_att->mapBackgroundSeg (b);
	    }

	    if (mask & GLINE_HEIGHT){
	        if (!gostream->readNumber (&n)) return 0;
	        (*seg)->_att->lineHeight (n);
	    }

	    if (mask & GUNDERLINE_POSITION){
	        if (!gostream->readNumber (&n)) return 0;
	        (*seg)->_att->underlinePosition (n);
	    }

	    if (mask & GEXTRA_BASELINE){
	        if (!gostream->readNumber (&n)) return 0;
	        (*seg)->_att->extraBaseline (n);
	    }
	}

	/* GoStdDS save method */
	if (!(streamMask & VERSION_1_2_BIS)){
	    GCoordSys cs;
	    if (!gostream->readCoordSys (&cs)) return 0;
	    if (!restore_children (*seg, gostream)) return 0;
	}

	return Go::readParams (*seg, gostream);
    }
    return 0;
}


/**
FUNCTION
    int GoTextSegment::readParams (GoTextSegment*, GoStream*);
DESCRIPTION
**/

int
GoTextSegment::readParams (GoTextSegment* seg, GoStream* gostream){
const char* class_name = NULL;
GBool b;

    if (!gostream->readInt (&seg->_firstPos)) return 0;
    if (!gostream->readInt (&seg->_lastPos)) return 0;
    if (!gostream->readBool (&b)) return 0;
    seg->_hyphened = b;

    /* segment attributes */
    long mask;
    if (!gostream->readLong (&mask)) return 0;

    if (mask != 0){
	GNumber n;
	seg->_att = new GStyle;

        if (mask & GBACKGROUND_SHBRUSH){
	    GShBrush* shbrush;
	    if (!gostream->restoreObj ((void**) &shbrush, &class_name))
	        return 0;
	    seg->_att->backgroundShBrush (shbrush);
        }

        if (mask & GSTRING_SHTEXTBRUSH){
	    GShTextBrush* shtextbrush;
	    if (!gostream->restoreObj ((void**) &shtextbrush, &class_name))
	        return 0;
	    seg->_att->stringShTextBrush (shtextbrush);
        }
       
        if (mask & GUNDERLINE_SHPEN){
	    GShPen* shpen;
	    if (!gostream->restoreObj ((void**) &shpen, &class_name))
	        return 0;
	    seg->_att->underlineShPen (shpen);
        }

        if (mask & GMAP_UNDERLINE_SEG){
	    if (!gostream->readBool (&b)) return 0;
	    seg->_att->mapUnderlineSeg (b);
        }

        if (mask & GMAP_BACKGROUND_SEG){
	    if (!gostream->readBool (&b)) return 0;
	    seg->_att->mapBackgroundSeg (b);
        }

        if (mask & GLINE_HEIGHT){
	    if (!gostream->readNumber (&n)) return 0;
	    seg->_att->lineHeight (n);
	}

        if (mask & GUNDERLINE_POSITION){
	    if (!gostream->readNumber (&n)) return 0;
	    seg->_att->underlinePosition (n);
	}

	if (mask & GEXTRA_BASELINE){
	    if (!gostream->readNumber (&n)) return 0;
	    seg->_att->extraBaseline (n);
	}
    }
    return Go::readParams (seg, gostream);
}


void
GSeg_split (GoTextSegment* seg, GPos pos)
{
    /* creation of a segment preceedind seg */
    GoTextSegment* seg1 = GSEG (seg->dup());
    seg1->parentRelativeReparent (seg->parent(), seg);
    seg1->_firstPos = seg->_firstPos;
    seg1->_lastPos = pos;
    seg1->updateWidth();

    /* update and reframe of segment */
    seg->_firstPos = pos;
    seg->_lastPos++;
    seg->updateWidth();
    seg->move (GPoint (seg1->_width, 0), seg1);
}


GBool
GSeg_append (GoTextSegment* seg)
{
GBool ret = GFALSE;
char firstCharOfNext;
GoTextSegment* next = seg->myNext();

    /* append if seg and next share the sames attributes */
    /* or if next is a spaced word */

    if (!next) return ret;

    GoText* txt = GTEXT (next->parent()->parent());

    /* called when deletion of the last character */
    if (next->_firstPos >= txt->length())
	firstCharOfNext = ' ';

    /* in the others cases */
    else
	firstCharOfNext = (*txt)[next->_firstPos];

    /* destroy of the next segment */
    if (seg->equiv (next) || isspace (firstCharOfNext)){
	seg->_lastPos = next->_lastPos - 1;
	next->destroy();
	seg->updateWidth();
	ret = GTRUE;
    }

    return ret;
}


const GShBrush*
GoTextSegment::backgroundShBrush() const
{
    return (_att && (_att->mask() & GBACKGROUND_SHBRUSH)) ?
	_att->backgroundShBrush() :
	GTEXT (parent()->parent())->backgroundBrush().shBrush();
}


const GShTextBrush*
GoTextSegment::stringShTextBrush() const
{
    return (_att && (_att->mask() & GSTRING_SHTEXTBRUSH)) ?
	_att->stringShTextBrush() :
	GTEXT (parent()->parent())->stringTextBrush().shTextBrush();
}


const GShPen*
GoTextSegment::underlineShPen() const
{
    return (_att && (_att->mask() & GUNDERLINE_SHPEN)) ?
	_att->underlineShPen() :
	GTEXT (parent()->parent())->underlinePen().shPen();
}


GNumber
GoTextSegment::lineHeight() const
{
    return (_att && (_att->mask() & GLINE_HEIGHT)) ?
	_att->lineHeight() :
	GTEXT (parent()->parent())->_lineHeight;
}


GNumber
GoTextSegment::underlinePosition() const
{
    return (_att && (_att->mask() & GUNDERLINE_POSITION)) ?
	_att->underlinePosition() :
	GTEXT (parent()->parent())->_underlinePosition;
}


GNumber
GoTextSegment::extraBaseline() const
{
    return (_att && (_att->mask() & GEXTRA_BASELINE)) ?
	_att->extraBaseline() :
	0;
}


GBool
GoTextSegment::mapUnderlineSeg() const
{
    return (_att && (_att->mask() & GMAP_UNDERLINE_SEG)) ?
	_att->mapUnderlineSeg() :
	GTEXT (parent()->parent())->_mapUnderlineSeg;
}


GBool
GoTextSegment::mapBackgroundSeg() const
{
    return (_att && (_att->mask() & GMAP_BACKGROUND_SEG)) ?
	_att->mapBackgroundSeg() :
	GTEXT (parent()->parent())->_mapBackgroundSeg;
}


GBool
GoTextSegment::equiv (GoTextSegment* seg) const
{
#ifdef GoLICENSE
    if (seg->getMetaclass() == GoContainer::metaclass() ||
        this->getMetaclass() == GoContainer::metaclass())
	    return GFALSE;
#endif /* GoLICENSE */

    return (_att == NULL) ?
	(seg->_att == NULL) :
	_att->equiv (seg->_att);
}


GNumber
GoTextSegment::normalSpace()
{
    GNumber w = 0;
    GoText* text = GTEXT (parent()->parent());
    char* str = text->_string;
    int n = 0; int lastpos = _lastPos;
    while (lastpos > _firstPos && isspace (*(str + lastpos - 1))){
	lastpos--; n++;
    }
    if (n)
        w = n * interf()->stringWidth (" ", 1,
	    lineHeight(), stringShTextBrush()->font());
    return w;
}


void
GoTextSegment::updateHighlight()
{
    if (_att){
	_att->modify (GTEXT (parent()->parent()));
	if (!_att->mask()){
	    delete _att;
	    _att = NULL;
	}
    }
}


GBool GoTextSegment::indented (const GoText* text) const
{
    if (!text)
	text = (GoText*)parent()->parent();

    return (_firstPos == 0 ||
	*(text->_string + _firstPos - 1) == '\n');
}




