#ifdef REV_INFO
static char rcsid[] = "$Id: TextUtil.cc,v 1.14 1992/12/23 16:43:59 andre Exp $";
#endif
/*****************************************************************************
MODULE:  GoText
FILE:	 TextUtil.cc
AUTHOR:  Michel Andre
DATE:    August 91
 

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
*****************************************************************************/

#include	<assert.h>
#include	<Go/GoInterf.h>
#include	<Go/Text.h>
#include	<Go/mText.h>
#include	<Go/Form.h>

#ifdef GoLICENSE
#include	"TextCont.h"
#endif /* GoLICENSE */

#define SIZE_W  10
#define _update_W(cs,w) if ((cs).I.size()*(w) < SIZE_W) (w) = SIZE_W / (cs).I.size()


/**
FUNCTION
    mapCursor (GBool set);
DESCRIPTION
    Maps / unmaps the cursor.
**/

void
GoText::mapCursor (GBool set)
{
    GoTextSegment* currentSeg = posToSeg (_currentPos);
    GPoint cursorOrigin = posToXy (_currentPos);

    /* region containing the cursor */
    GRegion regCursor;
    interf()->lineClips (&_cursorPen, Abscs(), cursorOrigin,
        GPoint (cursorOrigin.x, cursorOrigin.y +
	    currentSeg->_height), &regCursor, NULL, GMAX_QUALITY);

    if (set){
        _mask |= GCURSOR;
        RedrawForeOnly (regCursor);
    }
    else{
        _mask &= ~GCURSOR;
        RedrawBackAndFore (regCursor);
    }

    currentSeg->MyClipHasChanged();
}


/**
FUNCTION
    posToSeg (GPos);
DESCRIPTION
    Returns the segment of which the given position is inside.
**/

GoTextSegment*
GoText::posToSeg (GPos pos) const
{
    /* search of the line */
    GoTextLine* prevLine = _firstLine;
    for (GoTextLine* line = prevLine->NextLine();
	line; line=line->NextLine())
    {
	if (GSEG (line->Child())->_firstPos > pos)
	    break;
	prevLine = line;
    }

    /* search of the segment */
    GoTextSegment  *next;
    for (GoTextSegment* seg = GSEG (prevLine->Child());
	seg->_lastPos <= pos; seg=next){
	next = GSEG (seg->Next());
	if (!next || next->indented()) break;
    }
    return seg;
}


/**
FUNCTION
    boundariesToRegion (GPos first, GPos last) const;
DESCRIPTION
    Returns the GRegion defined by the 2 given positions.
**/

GRegion
GoText::boundariesToRegion (GPos first, GPos last) const
{
    if (first == last)
	return GEmptyRegion();

    if (first > last){
	GPos aux = first;
	first = last;
	last = aux;
    }

    GoTextSegment* seg = posToSeg (first);
    GPos pos = (GPos)min ((long)last, (long)seg->_lastPos);
    GRegion reg;

    do{
        GNumber w1 = interf()->stringWidth (_string + seg->_firstPos,
	    first - seg->_firstPos, seg->lineHeight(),
		seg->stringShTextBrush()->font());
        GNumber w2;
        if (pos == seg->_lastPos)
            w2 = seg->myClip().bigbox().width();
        else
            w2 = interf()->stringWidth (_string + seg->_firstPos,
	        pos - seg->_firstPos, seg->lineHeight(),
		    seg->stringShTextBrush()->font());

        GAdd (reg, seg->Abscs().to
	    (GBox (w1, 0, w2, seg->_height)), reg);

	if (seg=seg->myNext()){
	    if ((first = seg->_firstPos) >= last)
		break;
	    else
		pos = (GPos)min ((long)last, (long)seg->_lastPos);
	}
    }

    while (seg);
    return reg;
}


/**
FUNCTION
    xToPos (int, GNumber);
DESCRIPTION
    This function returns the position of char placed
    on the given line and x origin.
**/

GPos
GoText::xToPos (int lineNumber, GNumber x) const
{
    /* search of the given GoTextLine */
    for (GoTextLine* line = _firstLine; lineNumber--;
	line = line->NextLine());

    /* search of the segment */
    GNumber segXOrig = 0;
    GNumber lineXOrig = _formatter->indentation (line);
    for (GoTextSegment* seg = GSEG (line->Child());
        seg->Next(); seg = GSEG (seg->Next()))
    {
        segXOrig = seg->Relcs().O.x;
        if (x < (lineXOrig + segXOrig + seg->_width)) break;
    }

    segXOrig = seg->Relcs().O.x;

    /* search inside the segment */
    GPos nb = 1;
    char *pp = _string + seg->_firstPos;
    GPos segLength = seg->_lastPos - seg->_firstPos;

    /* compute the width of of seg before x */
    GNumber widthPortion = x - lineXOrig - segXOrig;
    while (interf()->stringWidth (pp, nb, seg->lineHeight(),
        seg->stringShTextBrush()->font()) <= widthPortion
            && nb <= segLength) nb++;

    return seg->_firstPos + --nb;
}


/**
FUNCTION
    GPos xyToPos (GPoint&);
DESCRIPTION
    Returns the position of the pointed char.
**/

GPos
GoText::xyToPos (GPoint& pt) const
{
    return _formatter ? _formatter->xyToPos (pt)
	              : 0;
}


/**
FUNCTION
    posToXy (GPos pos);
DESCRIPTION
    Returns the coordinates of the given position.
**/

GPoint
GoText::posToXy (GPos pos) const
{
    return _formatter ? _formatter->posToXy (pos)
	              : GPoint (0, 0);
}


/**
FUNCTION
    lastSeg();
DESCRIPTION
    Returns the last segment of the text.
**/

GoTextSegment*
GoText::lastSeg() const
{
    /* search of the last segment */
    for (GoTextSegment* seg = GSEG (_lastLine->Child());
	seg->Next(); seg = GSEG (seg->Next()));
    return seg;
}


/**
FUNCTION
    lineNumber (GPos);
DESCRIPTION
    Returns the line number of the given position.
**/

int
GoText::lineNumber (GPos pos) const
{
    if (pos == -1)
	pos = _currentPos;

    int lineNumber = 0;
    for (GoTextLine* line = _firstLine->NextLine();
	line; line=line->NextLine())
    {
	GPos p = GSEG (line->Child())->_firstPos;
	if (p > pos) break;
	lineNumber++;
    }

    return lineNumber;
}


/**
FUNCTION
    begLinePos (int);
DESCRIPTION
    Returns the position of the beginning of the given line.
**/

GPos
GoText::begLinePos (int n) const
{
    if (n == -1)
	n = lineNumber();

    /* search of the current GoTextLine */
    for (GoTextLine* line = _firstLine; n--;
	line = line->NextLine());

    return GSEG (line->Child())->_firstPos;
}


/**
FUNCTION
    endLinePos (int);
DESCRIPTION
    Returns the position of the end of the given line.
**/

GPos
GoText::endLinePos (int n) const
{
    if (n == -1)
	n = lineNumber();

    /* search of the current GoTextLine */
    for (GoTextLine* line = _firstLine; n--;
	line = line->NextLine());

    /* search of the last segment */
    for (GoTextSegment* seg = GSEG (line->Child());
	seg->Next(); seg = GSEG (seg->Next()));

    return seg->_lastPos;
}


/**
FUNCTION
    shiftPos (GoTextSegment*, GPos);
DESCRIPTION
    Shifts the segments positions values, beginning with the given segment.
**/

void
GoText::shiftPos (GoTextSegment* seg, GPos step)
{
    while (seg){
	seg->_firstPos += step;
	seg->_lastPos += step;
	seg=seg->myNext();
    }
}


/**
FUNCTION
    _Width() const;
DESCRIPTION
    Returns the real width of the text.
**/

GNumber
GoText::_Width() const
{
    GCoordSys cs = Abscs();
    GNumber w = _width;

    if (selected())
	_update_W (cs, w);

    return w;
}


/**
FUNCTION
    linesHeightSum();
DESCRIPTION
    Returns the sum of the height of the lines.
**/

GNumber
GoText::linesHeightSum()
{
GNumber h = 0;

    for (GoTextLine* line = _firstLine; line;
	line=line->NextLine())
             h += line->_height;
    return h;
}


/**
FUNCTION
    nbLines();
DESCRIPTION
    Returns the number of lines.
**/

GPos
GoText::nbLines() const
{
GPos nbl = 0;

    /* count the number of GoTextLine* childrens */
    for (GoTextLine* line = _firstLine; line;
	line=line->NextLine())
	    nbl++;
    return nbl;
}


/**
FUNCTION
    createPortion (GPos, GPos);
DESCRIPTION
    Creates a portion inside the text.
**/

void
GoText::createPortion (GPos pos1, GPos pos2)
{
GoTextSegment *newSeg;
GPos lastPosOfTheText = lastSeg()->_lastPos;

    /* can't compare with the strlen (_string) fonction */
    /* when used after a deleteString */
    assert (pos1 == -1 || pos1 >= 0 && pos1 <= lastPosOfTheText);
    assert (pos2 == -1 || pos2 >= 0 && pos2 <= lastPosOfTheText);

    if (pos2 < pos1){
        GPos aux = pos1;
        pos1 = pos2;
        pos2 = aux;
    }

    /* search of the first segment */
    GoTextSegment* seg = posToSeg (pos1);

    /* creation of a new segment */
    if (pos1 > seg->_firstPos){

        newSeg = GSEG (seg->dup());
        newSeg->parentRelativeReparent (seg->parent(), seg->Next());
        newSeg->_firstPos = pos1;
        newSeg->updateWidth();

        /* new right boundary for seg */
        seg->_lastPos = pos1;
        seg->updateWidth();
        newSeg->move (GPoint (seg->_width, 0), seg);
    }

    if (pos1 == pos2) return;

    seg = posToSeg (pos2);

    /* creation of a new segment */
    if (pos2 != seg->_firstPos && pos2 < seg->_lastPos){

        newSeg = GSEG (seg->dup());
        newSeg->parentRelativeReparent (seg->parent(), seg->Next());
        newSeg->_firstPos = pos2;
        newSeg->updateWidth();

        /* new right boundary for seg */
        seg->_lastPos = pos2;
        seg->updateWidth();
        newSeg->move (GPoint (seg->_width, 0), seg);
    }
}


/**
FUNCTION
    resize();
DESCRIPTION
    Function called after a resize.
    Called also after an interfPostcond to recompute the text and its segments.
**/

void
GoText::resize()
{
    /* update _firstLine and _lastLine after a dup */
    if (!_firstLine){
        Go* child = Child();
        while (child->getMetaclass() != GoTextLine::metaclass())
	    child = child->Next();
        _firstLine = GLINE (child);

        /* update _lastLine */
        while (child->Next() && child->Next()->getMetaclass()
	    == GoTextLine::metaclass()) child = child->Next();
        _lastLine = GLINE (child);
    }

    /* update _lineHeightMax */
    _lineHeightMax = interf()->fontStrikeoutAscent
	(_lineHeight, _stringTextBrush.font()) +
	    interf()->fontStrikeoutDescent
		(_lineHeight, _stringTextBrush.font());

    for (GoTextSegment* seg = GSEG (_firstLine->Child());
	seg; seg=seg->myNext())
    {
        seg->updateHeight();
	seg->updateWidth();
    }

#ifdef GoLICENSE
    /* link the sub Gos */
    for (Go* go = _firstLine->Child(); go;
	go = GSEG (go)->myNext())
    {
	if (go->getMetaclass() == GoContainer::metaclass()
	    && ((GoContainer*)go)->Child() == NULL)
		((GoContainer*)go)->addGo (go->Child());
    }
#endif /* GoLICENSE */

    GFormatter* f;
    if ((f = formatter()))
	f->reframe();
}


static void gotext_select_unselect_subgo(Go* go, const GPoint& absp)
{
    ((GoText*)go)->selectUnselectSubGo(absp);
}
	

static GActionRec go_text_action_array[] =
{
    {"selectUnselectSubGo",
	(GActionProc)gotext_select_unselect_subgo},
    {NULL, NULL}
};


/**
FUNCTION
    m_actions()
DESCRIPTION
    The GoTextClass m_actions function.
**/

const GActionRec*
GoTextMetaclass::m_actions() const
{
    return go_text_action_array;
}


/**
FUNCTION
    _emptySegment (GPos) const
DESCRIPTION
    Returns TRUE if the segment containing pos is empty.
**/

GBool
GoText::_emptySegment (GPos pos) const
{
    char charPos = *(_string + pos);
    char charPrevPos = pos > 0 ? *(_string + pos - 1) : 0;

    return
	pos == 0 && (charPos == '\n' || charPos == 0) ||
	charPrevPos == '\n' && (charPos == '\n' || charPos == 0);
}
