#ifdef REV_INFO
static char rcsid[] = "$Id: TextualCond.cc,v 1.2 1992/08/06 09:41:05 uep Exp $";
#endif
/*****************************************************************************
MODULE:  GoTextual
FILE:	 TextCond.cc
AUTHOR:  Michel Andre
DATE:    August 91
 

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
*****************************************************************************/

#include <Go/Textual.h>
#include <Go/mTextual.h>

/***************
* Callback Types
****************/

static char textualNcurrentPosPrecond[] = "1";
static char textualNcurrentPosPostcond[] = "1";
static char textualNmodifyValuePrecond[] = "1";
static char textualNmodifyValuePostcond[] = "1";
static char textualNmodifyAttributePrecond[] = "1";
static char textualNmodifyAttributePostcond[] = "1";
static char textualNselectionPrecond[] = "1";
static char textualNselectionPostcond[] = "1";


/**
FUNCTION
   _call_current_pos_precond()
DESCRIPTION
    The GoTextual currentPos precondition call function.
**/

GBool
GoTextual::_call_current_pos_precond (GPos new_pos)
{
    return (_flags._has_current_pos_precond ?
	CallPreconditions (textualNcurrentPosPrecond, (void*)new_pos) : GTRUE);
}


GBool
GoTextualMetaclass::_call_current_pos_precond (const GoTextual* go, GPos new_pos)
{
    return (_flags._has_current_pos_precond ?
	CallPreconditions(textualNcurrentPosPrecond, go, (void*)new_pos) : GTRUE);
}


/**
FUNCTION
   _call_current_pos_postcond()
DESCRIPTION
    The GoTextual currentPos postcondition call function.
**/

void
GoTextual::_call_current_pos_postcond (GPos old_pos)
{
    if (_flags._has_current_pos_postcond)
	CallPostconditions(textualNcurrentPosPostcond, (void*)old_pos);
}


void
GoTextualMetaclass::_call_current_pos_postcond (GoTextual* go, GPos old_pos)
{
    if (_flags._has_current_pos_postcond)
	CallPostconditions (textualNcurrentPosPostcond, go, (void*)old_pos);
}


/**
FUNCTION
   _call_modify_value_precond()
DESCRIPTION
    The GoTextual modifying value precondition call function.
**/

GBool
GoTextual::_call_modify_value_precond (GModifyValueStruct* str)
{
    return (_flags._has_modify_value_precond ?
	CallPreconditions(textualNmodifyValuePrecond, str) : GTRUE);
}


GBool
GoTextualMetaclass::_call_modify_value_precond (const GoTextual* go, GModifyValueStruct* str)
{
    return (_flags._has_modify_value_precond ?
	CallPreconditions (textualNmodifyValuePrecond, go, str) : GTRUE);
}


/**
FUNCTION
   _call_modify_value_postcond()
DESCRIPTION
    The GoTextual modifying value postcondition call function.
**/

void
GoTextual::_call_modify_value_postcond (GModifyValueStruct* str)
{
    if (_flags._has_modify_value_postcond)
	CallPostconditions (textualNmodifyValuePostcond, str);
}


void
GoTextualMetaclass::_call_modify_value_postcond (GoTextual* go, GModifyValueStruct* str)
{
    if (_flags._has_modify_value_postcond)
	CallPostconditions (textualNmodifyValuePostcond, go, str);
}


/**
FUNCTION
   _call_modify_attribute_precond()
DESCRIPTION
    The GoTextual modifying attribute precondition call function.
**/

GBool
GoTextual::_call_modify_attribute_precond (GModifyAttributeStruct* str)
{
    return (_flags._has_modify_attribute_precond ?
	CallPreconditions (textualNmodifyAttributePrecond, str) : GTRUE);
}


GBool
GoTextualMetaclass::_call_modify_attribute_precond (const GoTextual* go, GModifyAttributeStruct* str)
{
    return (_flags._has_modify_attribute_precond ?
	CallPreconditions (textualNmodifyAttributePrecond, go, str) : GTRUE);
}


/**
FUNCTION
   _call_modify_attribute_postcond()
DESCRIPTION
    The GoTextual modifying value postcondition call function.
**/

void
GoTextual::_call_modify_attribute_postcond (GModifyAttributeStruct* str)
{
    if (_flags._has_modify_attribute_postcond)
        CallPostconditions (textualNmodifyAttributePostcond, str);
}


void
GoTextualMetaclass::_call_modify_attribute_postcond (GoTextual* go, GModifyAttributeStruct* str)
{
    if (_flags._has_modify_attribute_postcond)
        CallPostconditions (textualNmodifyAttributePostcond, go, str);
}


/**
FUNCTION
   _call_selection_precond()
DESCRIPTION
    The GoTextual selection precondition call function.
**/

GBool
GoTextual::_call_selection_precond (GPos pos1, GPos pos2)
{
    return (_flags._has_selection_precond ?
	CallPreconditions (textualNselectionPrecond, (void*)pos1, (void*)pos2) : GTRUE);
}


GBool
GoTextualMetaclass::_call_selection_precond (const GoTextual* go, GPos pos1, GPos pos2)
{
    return (_flags._has_selection_precond ?
	CallPreconditions (textualNselectionPrecond, go, (void*)pos1, (void*)pos2) : GTRUE);
}


/**
FUNCTION
   _call_selection_postcond()
DESCRIPTION
    The GoTextual selection postcondition call function.
**/

void
GoTextual::_call_selection_postcond (GPos pos1, GPos pos2)
{
    if (_flags._has_selection_postcond)
	CallPostconditions (textualNselectionPostcond, (void*)pos1, (void*)pos2);
}


void
GoTextualMetaclass::_call_selection_postcond (GoTextual* go, GPos pos1, GPos pos2)
{
    if (_flags._has_selection_postcond)
	CallPostconditions (textualNselectionPostcond, go, (void*)pos1, (void*)pos2);
}


/*********************************************************
	modify value pre/postcond
**********************************************************/

void
GoTextual::modifyValuePrecond (GBool func (void*, const GoTextual*, GModifyValueStruct*),
	void* closure, GAddOrRemove add)
{
    _flags._has_modify_value_precond = 
	AddOrRemoveCondition (textualNmodifyValuePrecond, func, closure, add);
}


void
GoTextual::modifyValuePostcond (void func (void*, GoTextual*, GModifyValueStruct*),
	void* closure, GAddOrRemove add)
{
    _flags._has_modify_value_postcond = 
	AddOrRemoveCondition (textualNmodifyValuePostcond, func, closure, add);
}


void
GoTextualMetaclass::modifyValuePrecond (GBool func (void*, const GoTextual*, GModifyValueStruct*),
	void* closure, GAddOrRemove add)
{
    _flags._has_modify_value_precond =
	AddOrRemoveCondition (textualNmodifyValuePrecond, func, closure, add);
}


void
GoTextualMetaclass::modifyValuePostcond (void func (void*, GoTextual*, GModifyValueStruct*),
	void* closure, GAddOrRemove add)
{
    _flags._has_modify_value_postcond =
	AddOrRemoveCondition (textualNmodifyValuePostcond, func, closure, add);
}


/*********************************************************
	currentPos pre/postcond
**********************************************************/

void
GoTextual::currentPosPrecond (GBool func (void*, const GoTextual*, GPos),
	void* closure, GAddOrRemove add)
{
    _flags._has_current_pos_precond = 
	AddOrRemoveCondition (textualNcurrentPosPrecond, func, closure, add);
}


void
GoTextual::currentPosPostcond (void func (void*, GoTextual*, GPos),
	void* closure, GAddOrRemove add)
{
    _flags._has_current_pos_postcond = 
	AddOrRemoveCondition (textualNcurrentPosPostcond, func, closure, add);
}


void
GoTextualMetaclass::currentPosPrecond (GBool func (void*, const GoTextual*, GPos),
	void* closure, GAddOrRemove add)
{
    _flags._has_current_pos_precond = 
	AddOrRemoveCondition (textualNcurrentPosPrecond, func, closure, add);
}


void
GoTextualMetaclass::currentPosPostcond (void func (void*, GoTextual*, GPos),
	void* closure, GAddOrRemove add)
{
    _flags._has_current_pos_postcond = 
	AddOrRemoveCondition (textualNcurrentPosPostcond, func, closure, add);
}


/*********************************************************
	modify attribute pre/postcond
**********************************************************/

void
GoTextual::modifyAttributePrecond (GBool func (void*, const GoTextual*, GModifyAttributeStruct*),
    void* closure, GAddOrRemove add)
{
    _flags._has_modify_attribute_precond =
	AddOrRemoveCondition (textualNmodifyAttributePrecond, func, closure, add);
}


void
GoTextual::modifyAttributePostcond (void func (void*, GoTextual*, GModifyAttributeStruct*),
    void* closure, GAddOrRemove add)
{
    _flags._has_modify_attribute_postcond =
	AddOrRemoveCondition (textualNmodifyAttributePostcond, func, closure, add);
}


void
GoTextualMetaclass::modifyAttributePrecond (GBool func (void*, const GoTextual*, GModifyAttributeStruct*),
    void* closure, GAddOrRemove add)
{
    _flags._has_modify_attribute_precond =
	AddOrRemoveCondition (textualNmodifyAttributePrecond, func, closure, add);
}


void
GoTextualMetaclass::modifyAttributePostcond (void func (void*, GoTextual*, GModifyAttributeStruct*),
    void* closure, GAddOrRemove add)
{
    _flags._has_modify_attribute_postcond =
	AddOrRemoveCondition (textualNmodifyAttributePostcond, func, closure, add);
}


/*********************************************************
	selection pre/postcond
**********************************************************/

void
GoTextual::selectionPrecond (GBool func (void*, const GoTextual*, GPos, GPos),
    void* closure, GAddOrRemove add)
{
    _flags._has_selection_precond =
	AddOrRemoveCondition (textualNselectionPrecond, func, closure, add);
}


void
GoTextual::selectionPostcond (void func (void*, GoTextual*, GPos, GPos),
    void* closure, GAddOrRemove add)
{
    _flags._has_selection_postcond =
	AddOrRemoveCondition (textualNselectionPostcond, func, closure, add);
}


void
GoTextualMetaclass::selectionPrecond (GBool func (void*, const GoTextual*, GPos, GPos),
    void* closure, GAddOrRemove add)
{
    _flags._has_selection_precond =
	AddOrRemoveCondition (textualNselectionPrecond, func, closure, add);
}


void
GoTextualMetaclass::selectionPostcond (void func (void*, GoTextual*, GPos, GPos),
    void* closure, GAddOrRemove add)
{
    _flags._has_selection_postcond =
	AddOrRemoveCondition (textualNselectionPostcond, func, closure, add);
}
