#ifdef REV_INFO
static char rcsid[] = "$Id: TextualEdit.cc,v 1.4 1992/11/26 14:10:05 andre Exp $";
#endif
/*****************************************************************************
MODULE:      GoTextual
FILE:        Edit.cc
AUTHOR:      Andre Michel
DATE:        March 1991

DESCRIPTION:
  Text management for all the predefined text Gos. (See Textual.h)

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#include <assert.h>
#include <ctype.h>
#include <Go/Tools/Alloc.h>
#include <Go/Textual.h>
#include <Go/mTextual.h>


GRegion oldClip;


/**
FUNCTION
    insertChar (char, GPos)
DESCRIPTION
    This function is used to insert Char at the position pos.
**/

GBool
GoTextual::insertChar (char insertedChar, GPos pos)
{
GBool ret;

    if (pos == -1) pos = _currentPos;

    GPos size = length();
    assert (pos >= 0 && pos <= size);
    if (insertedChar == '\r') insertedChar = '\n';

    /* initilisation of the GModifyValueStruct */
    GModifyValueStruct* str = new GModifyValueStruct;
    str->type = GINSERT_CHAR_NOTIFY;
    str->charValue.pos = pos;
    str->charValue.Char = insertedChar;

    char* adpos = _string+pos;

    /* call to insertChar preconditions */
    if (ret = (_call_modify_value_precond (str) &&
	((GoTextualMetaclass*)getMetaclass())->
            _call_modify_value_precond (this, str)) &&
                 GoTextual::metaclass()->
                     _call_modify_value_precond (this, str))
    {
	oldClip = myClip();

	/* unselect the text */
	selectPortion (-1, -1, GFALSE);

	/* update of the work positions */
	if (_currentPos >= pos) _currentPos++;

	/* update of the string value */
	switch (_present){
	  case GLOWER_PRESENTATION:
	    if (isalpha (insertedChar) && isupper (insertedChar))
                insertedChar = tolower (insertedChar);
		break;

	  case GUPPER_PRESENTATION:
	    if (isalpha (insertedChar) && islower (insertedChar))
                insertedChar = toupper (insertedChar);
		break;

	  case GTITLE_PRESENTATION:
	    if (!isalpha (insertedChar)){
		if (isalpha (*adpos) && islower (*adpos))
		    *adpos = toupper (*adpos);
	    }
            else{
                if (isalpha (*adpos)){
                    if (islower (*adpos)){
                        if (isupper (insertedChar))
                            insertedChar = tolower (insertedChar);
                    }
                    else{
                        *adpos = tolower (*adpos);
                        if (islower (insertedChar))
                            insertedChar = toupper (insertedChar);
                    }
                }
                else
                    if (isalpha (*(adpos-1))){
                        if (isupper (insertedChar))
                            insertedChar = tolower (insertedChar);
                    }
                    else
                        if (islower (insertedChar))
                            insertedChar = toupper (insertedChar);
            }
            break;

	  case GNO_PRESENTATION:
	    break;
	}

	/* update of _string */
	/* size add 1 char for \0 and one for the new char */
	_allocString (size + 2);

	/* adpos may have changed */
	adpos = _string + pos;

	/* update of the string value */
	bcopy (adpos, adpos + 1, size - pos + 1);
	*adpos = insertedChar;

	/* call the effective sub-classe function */
	_insertChar (insertedChar, pos);

	/* call to insertChar postconditions */
	_call_modify_value_postcond (str);
	((GoTextualMetaclass*)getMetaclass())->
            _call_modify_value_postcond (this, str);
	GoTextual::metaclass()->_call_modify_value_postcond (this, str);
    }

    delete str;
    return ret;
}


/**
FUNCTION
    insertString (const char *s, GPos len, GPos pos)
DESCRIPTION
    This function is used to insert str at the position pos.
**/

GBool
GoTextual::insertString (const char *insertedString, GPos len, GPos pos)
{
GBool ret;

    GPos size = length();
    if (pos == -1) pos = _currentPos;
    assert (pos >= 0 && pos <= size);

    /* initilisation of the GModifyValueStruct */
    GModifyValueStruct* str = new GModifyValueStruct;
    str->type = GINSERT_STRING_NOTIFY;
    str->stringValue.str = (char*) GMALLOC (len + 1);
    strncpy ((char*)str->stringValue.str, insertedString, len);
    *((char*)str->stringValue.str+len) = '\0';
    str->stringValue.len = len;
    str->stringValue.pos = pos;

    /* call to insertString preconditions */
    if (ret = (_call_modify_value_precond (str) &&
	((GoTextualMetaclass*)getMetaclass())->
            _call_modify_value_precond (this, str)) &&
	        GoTextual::metaclass()->
                    _call_modify_value_precond (this, str))
    {
	oldClip = myClip();

	/* unselect the text */
	selectPortion (-1, -1, GFALSE);

	/* the cursor may drawn outside the textual frame */
	GBool cursorWasMapped = cursorMapped();
	if (cursorMapped()) mapCursor (GFALSE);

	/* update of the work positions */
	if (_currentPos >= pos) _currentPos += len;

	/* update of _string */
	/* add one char for \0 */
	_allocString (size+len+1);

	/* update of the string value */
	int i = size-pos+1;
	char* p = _string+size+len;
	while (i--) *p = *(p-- - len);
	strncpy (_string+pos, insertedString, len);
	_updatePresentation (_present);

	/* call the effective sub-classe function */
	_insertString (insertedString, len, pos);

	/* restore the cursor mapping state */
	if (cursorWasMapped)
	    mapCursor (GTRUE);

	/* call to insertString postconditions */
	_call_modify_value_postcond (str);
	((GoTextualMetaclass*)getMetaclass())->
            _call_modify_value_postcond (this, str);
	GoTextual::metaclass()->_call_modify_value_postcond (this, str);
    }

    GFREE (str->stringValue.str);
    delete str;
    return ret;
}


/**
FUNCTION
    deleteChar (GPos pos);
DESCRIPTION
    This function allows to delete the character at the position pos.
**/

GBool
GoTextual::deleteChar (GPos pos)
{
GBool ret;

    if (pos == -1) pos = _currentPos;
    GPos size = length();
    assert (pos >= 0 && pos <= size);

    /* no character to delete */
    if (pos == size) return (GFALSE);

    char* adpos = _string + pos;
    char deletedChar = *adpos;

    /* initilisation of the GModifyValueStruct */
    GModifyValueStruct* str = new GModifyValueStruct;
    str->type = GDELETE_CHAR_NOTIFY;
    str->charValue.pos = pos;
    str->charValue.Char = deletedChar;
        
    /* call to deleteChar preconditions */
    if (ret = (_call_modify_value_precond (str) &&
	((GoTextualMetaclass*)getMetaclass())->
            _call_modify_value_precond (this, str)) &&
	        GoTextual::metaclass()->
                    _call_modify_value_precond (this, str))
    {
	oldClip = myClip();

	/* unselect the text */
	selectPortion (-1, -1, GFALSE);

	/* the cursor may drawn outside the textual frame */
	GBool cursorWasMapped = cursorMapped();
	if (cursorMapped()) mapCursor (GFALSE);

	/* update of the work positions */
	if (_currentPos > pos) _currentPos--;

	/* update of the string value */
	strcpy (adpos, adpos+1);
	if (_present ==  GTITLE_PRESENTATION){
	    if (isalpha (deletedChar)){
		if (isupper (deletedChar) && isalpha (*adpos))
		    *adpos = toupper (*adpos);
	    }
	    else
	     if (isalpha (*adpos) && isalpha (*(adpos-1)))
		*adpos = tolower (*adpos);
	}

	/* call the effective sub-classe function */
	_deleteChar (deletedChar, pos);

	/* restore the cursor mapping state */
	if (cursorWasMapped)
	    mapCursor (GTRUE);

	/* call to deleteChar postconditions */
	_call_modify_value_postcond (str);
	((GoTextualMetaclass*)getMetaclass())->
            _call_modify_value_postcond (this, str);
	GoTextual::metaclass()->_call_modify_value_postcond (this, str);
    }

    delete str;
    return ret;
}


/**
FUNCTION
    deleteString (GPos, GPos);
DESCRIPTION
    This function allows to delete a substring inside _string.
**/

GBool
GoTextual::deleteString (GPos delSize, GPos pos)
{
GBool ret;

    if (pos == -1) pos = _currentPos;
    assert (pos >= 0 && pos+delSize <= length() && delSize >= 0);
    
    /* initilisation of the GModifyValueStruct */
    GModifyValueStruct* str = new GModifyValueStruct;
    str->type = GDELETE_STRING_NOTIFY;
    str->stringValue.pos = pos;
    str->stringValue.str = (char*)GMALLOC (delSize+1);
    strncpy ((char*)str->stringValue.str, _string+pos, delSize);
    *((char*)str->stringValue.str+delSize) = '\0';
    str->stringValue.len = delSize;
    
    /* call to deleteString preconditions */
    if (ret = (_call_modify_value_precond (str) &&
	((GoTextualMetaclass*)getMetaclass())->
            _call_modify_value_precond (this, str)) &&
	        GoTextual::metaclass()->
                    _call_modify_value_precond (this, str))
    {
	oldClip = myClip();

	/* unselect the text */
	selectPortion (-1, -1, GFALSE);

	/* the cursor may drawn outside the textual frame */
	GBool cursorWasMapped = cursorMapped();
	if (cursorMapped()) mapCursor (GFALSE);

	/* update of the work positions */
	if (_currentPos > pos)
	    _currentPos -= (GPos)min((long)delSize, (long)_currentPos-pos);
    
	strcpy (_string+pos, _string+pos+delSize);
	_updatePresentation (_present);

	/* call the effective sub-classe function */
	_deleteString ((const char*)str->stringValue.str, delSize, pos);

	/* restore the cursor mapping state */
	if (cursorWasMapped)
	    mapCursor (GTRUE);

	/* call to deleteString postconditions */
	_call_modify_value_postcond (str);
	((GoTextualMetaclass*)getMetaclass())->
            _call_modify_value_postcond (this, str);
	GoTextual::metaclass()->_call_modify_value_postcond (this, str);
    }

    GFREE (str->stringValue.str);
    delete str;
    return ret;
}


/**
FUNCTION
    setString (const char* newString, GPos newSize)
DESCRIPTION
    This function is used to set a new string value.
**/

GBool
GoTextual::setString (const char* newString, GPos newSize)
{
GBool ret;

    /* initilisation of the GModifyValueStruct */
    GModifyValueStruct* str = new GModifyValueStruct;
    str = new GModifyValueStruct;
    str->type = GSET_STRING_NOTIFY;
    str->stringValue.str = newString;
    str->stringValue.len = newSize;

    /* save of the values */
    GPos oldSize = length();
    char* oldValue = (char*) GMALLOC (oldSize + 1);
    strcpy (oldValue, _string);

    /* call to GModifyValue preconditions */
    if (ret = (_call_modify_value_precond (str) &&
	((GoTextualMetaclass*)getMetaclass())->
            _call_modify_value_precond (this, str)) &&
	        GoTextual::metaclass()->
                    _call_modify_value_precond (this, str))
    {
	oldClip = myClip();

	/* unselect the text */
	selectPortion (-1, -1, GFALSE);

	/* the cursor could be outside the GoTextual rectangle */
	GBool cursorWasMapped = cursorMapped();
	if (cursorWasMapped)
	    mapCursor (GFALSE);

	/* update of the work positions */
	_currentPos = 0;

	/* update of _string */
	/* add one char for \0 */
	_allocString (newSize+1);

	/* blank the right part of _string */
	char* ptr = _string;
	for (int i = newSize; i < _size; i++)
	    *(ptr + i) = '\0';

	/* update of the string value */
	strncpy (_string, newString, newSize);
	_updatePresentation (_present);

	str->stringValue.str = oldValue;
	str->stringValue.len = oldSize;

	/* call the effective sub-classe function */
	_setString (newString, newSize);

	/* restore the cursor mapping state */
	if (cursorWasMapped)
	    mapCursor (GTRUE);

	/* call to modifyValue postconditions */
	_call_modify_value_postcond (str);
	((GoTextualMetaclass*)getMetaclass())->
            _call_modify_value_postcond (this, str);
	GoTextual::metaclass()->_call_modify_value_postcond (this, str);
    }

    GFREE (oldValue);
    delete str;
    return ret;
}


/**
FUNCTION
    replaceChar (char, GPos);
DESCRIPTION
    This function is used to replace a character inside the string.
**/

GBool
GoTextual::replaceChar (char newChar, GPos pos)
{
GBool ret;

    if (pos == -1) pos = _currentPos;
    GPos size = length();
    assert (pos >= 0 && pos <= size);

    /* no character to replace */
    if (pos == size) return (GFALSE);

    /* save of the value */
    char oldChar = *(_string+pos);

    /* initilisation of the GModifyValueStruct */
    GModifyValueStruct* str = new GModifyValueStruct;
    str->type = GREPLACE_CHAR_NOTIFY;
    str->charValue.Char = newChar;
    str->charValue.pos = pos;

    /* call to modifyValue preconditions */
    if (ret = (_call_modify_value_precond (str) &&
	((GoTextualMetaclass*)getMetaclass())->
            _call_modify_value_precond (this, str)) &&
	        GoTextual::metaclass()->
                    _call_modify_value_precond (this, str))
    {
	oldClip = myClip();

	/* unselect the text */
	selectPortion (-1, -1, GFALSE);

	/* the cursor may drawn outside the textual frame */
	GBool cursorWasMapped = cursorMapped();
	if (cursorMapped()) mapCursor (GFALSE);

	/* update of the string value */
	*(_string+pos) = newChar;

	str->charValue.Char = oldChar;

	/* call the effective sub-classe function */
	_replaceChar (oldChar, pos);

	/* restore the cursor mapping state */
	if (cursorWasMapped)
	    mapCursor (GTRUE);

	/* call to modifyValue postconditions */
	_call_modify_value_postcond (str);
	((GoTextualMetaclass*)getMetaclass())->
            _call_modify_value_postcond (this, str);
	GoTextual::metaclass()->_call_modify_value_postcond (this, str);
    }

    delete str;
    return ret;
}


/**
FUNCTION
    replaceString (GPos, GPos, char*, GPos);
DESCRIPTION
    This function is used to replace a sub-string inside the string.
**/

GBool
GoTextual::replaceString (GPos pos, GPos oldStringSize, char* newString, GPos newStringSize)
{
GBool ret;

    assert (pos >= 0 && pos + oldStringSize <= length() && newStringSize >= 0);
    GPos shiftStep = newStringSize - oldStringSize;

    /* initilisation of the GModifyValueStruct */
    GModifyValueStruct* str = new GModifyValueStruct;
    str->type = GREPLACE_STRING_NOTIFY;
    str->stringValue.pos = pos;
    str->stringValue.oldStringSize = oldStringSize;
    str->stringValue.str = newString;
    str->stringValue.len = newStringSize;

    /* save of the old sub-string */
    char* oldString = (char*) GMALLOC (oldStringSize + 1);
    strncpy (oldString, _string + pos, oldStringSize);
    *(oldString + oldStringSize) = '\0';
    
    /* call to deleteString preconditions */
    if (ret = (_call_modify_value_precond (str) &&
	((GoTextualMetaclass*)getMetaclass())->
            _call_modify_value_precond (this, str)) &&
	        GoTextual::metaclass()->
                    _call_modify_value_precond (this, str))
    {
	oldClip = myClip();

	/* unselect the text */
	selectPortion (-1, -1, GFALSE);

	/* the cursor may drawn outside the textual frame */
	GBool cursorWasMapped = cursorMapped();
	if (cursorMapped()) mapCursor (GFALSE);

	/* update string */
	if (shiftStep > 0)
	    /* add one char for \0 */
	    _allocString (_size + shiftStep + 1);

	/* update of the string value */
	if (shiftStep < 0)

	    /* shift to the left */
	    strcpy (_string + pos + newStringSize,
                _string + pos + oldStringSize);
	else
	    if (shiftStep > 0){

		/* shift to the right */
		GPos nbCharToShift = length() - pos + oldStringSize;
		char* p = _string + length() + shiftStep;
		while (nbCharToShift--){
		    *p = *(p - shiftStep);
		    p--;
		}
	    }

	/* effective copy of the new sub-string */
	strncpy (_string+pos, newString, newStringSize);

	/* fill struct with the old values */
	str->stringValue.pos = pos;
	str->stringValue.oldStringSize = newStringSize;
	str->stringValue.str = oldString;
	str->stringValue.len = oldStringSize;

	/* call the effective sub-classe function */
	_replaceString (pos, oldStringSize, newString, newStringSize);

	/* restore the cursor mapping state */
	if (cursorWasMapped)
	    mapCursor (GTRUE);

	/* call to modifyValue postconditions */
	_call_modify_value_postcond (str);
	((GoTextualMetaclass*)getMetaclass())->
            _call_modify_value_postcond (this, str);
	GoTextual::metaclass()->_call_modify_value_postcond (this, str);
    }

    GFREE (oldString);
    delete str;
    return ret;
}


/**
FUNCTION
    presentation (GPresentation);
DESCRIPTION
    This function is used to set the presentation.
**/

GBool
GoTextual::presentation (GPresentation newPresentation)
{
GBool ret;

    GPresentation oldPresentation = _present;

    GModifyValueStruct* str = new GModifyValueStruct;
    str->type = GPRESENTATION_CHANGE_NOTIFY;
    str->presentation = newPresentation;

    /* call presentation preconditions */
    if (ret = (_call_modify_value_precond (str) &&
	((GoTextualMetaclass*)getMetaclass())->
            _call_modify_value_precond (this, str)) &&
	        GoTextual::metaclass()->
                    _call_modify_value_precond (this, str))
    {
	oldClip = myClip();

	/* unselect the text */
	selectPortion (-1, -1, GFALSE);

	/* the cursor may drawn outside the textual frame */
	GBool cursorWasMapped = cursorMapped();
	if (cursorMapped()) mapCursor (GFALSE);

        _updatePresentation (newPresentation);
        _present = newPresentation;
	_presentation (newPresentation);

        str->presentation = oldPresentation;

	/* restore the cursor mapping state */
	if (cursorWasMapped)
	    mapCursor (GTRUE);

        /* call presentation postconditions */
	_call_modify_value_postcond (str);
	((GoTextualMetaclass*)getMetaclass())->
            _call_modify_value_postcond (this, str);
	GoTextual::metaclass()->_call_modify_value_postcond (this, str);
    }

    delete str;
    return ret;
}


/**
FUNCTION
    _presentation (GPresentation);
DESCRIPTION
    This function is used to change the presentation.
**/

void
GoTextual::_presentation (GPresentation)
{
    GBool was_mapped = mapped();
    if (was_mapped) TempUnmap();
    _width = _Width();
    MyClipHasChanged();
    if (was_mapped) TempMap();
}
