#ifdef REV_INFO
static char rcsid[] = "$Id: TextualUtil.cc,v 1.11 1992/12/10 17:11:53 smadja Exp $";
#endif
/*****************************************************************************
MODULE:  GoTextual
FILE:	 TextualUtil.cc
AUTHOR:  Michel Andre
DATE:    August 91
 

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
*****************************************************************************/

#include <ctype.h>
#include <assert.h>
#include <Go/Tools/Alloc.h>
#include <Go/Textual.h>
#include <Go/mTextual.h>
#include <Go/Text.h>
#include <Go/SimpleForm.h>


#define isSeparator(Char) strchr(" .,;:!?\n",Char)
#define GRANUL_SIZE			32



void _textualutil_insertChar (GoTextual* text, char Char)
{
    if (Char == '\r')
	Char = '\n';

    text->insertChar (Char);
}



void _textualutil_deleteChar (GoTextual* text)
{
    /* delete a substring if there is only one segment selected */
    if (text->selectionList().size() == 1){
	GTextSeg* ts = text->selectionList().first();
	text->deleteString (ts->last() - ts->first(),
	    ts->first());
    }

    else
	if (text->getCurrentPos() < text->length())
	    text->deleteChar();
}



void _textualutil_deleteBeforeChar (GoTextual* text)
{
    GPos pos = text->getCurrentPos();
    if (pos > 0)
	if (text->currentPos (--pos))
	    text->deleteChar();
}



void _textualutil_cursor (GoTextual* text, GPoint& pt)
{
    text->currentPos (text->xyToPos (text->Abscs().from (pt)));
}



void _textualutil_mapBackground (GoTextual* text, GBool b)
{
    if (GoText::metaclass() == text->getMetaclass() &&
	text->selectionList().size())
    {
	for (GTextSeg* ts = text->selectionList().first();
	    ts; ts=ts->next())
	    GTEXT (text)->background (b, ts->first(),
		ts->last());
    }
    else
	text->mapBackground (b);
}



void _textualutil_presentation (GoTextual* text)
{
    text->presentation((GPresentation)((text->presentation()+1)%4));
}



void _textualutil_updateCharCase (GoTextual* text)
{
    char currentChar = (*text)[text->getCurrentPos()];
    if (isalpha (currentChar)){
        if (islower (currentChar))
            currentChar = toupper (currentChar);
        else
            currentChar = tolower (currentChar);
        text->replaceChar (currentChar);
    }

    text->cursorRight();
}



void _textualutil_underline (GoTextual* text, GBool set)
{
    /* underline all the text string */
    if (!text->selectionList().size())
	text->underline (set, 0, text->length());

    else
	for (GTextSeg* ts = text->selectionList().first();
	    ts; ts=ts->next())
		text->underline (set, ts->first(), ts->last());
}



void _textualutil_justification (GoTextual* text)
{
int times = 3;

    if (GoText::metaclass() == text->getMetaclass() &&
	strcmp (GTEXT (text)->formatter()->getClassname(),
	    GSimpleFormatter::classname()))
		times = 4;

    text->justification ((GJustification)
	((text->justification() + 1) % times));
}



void _textualutil_interline (GoTextual* text)
{
    int step = (int)text->GoTextual::lineHeightMax()/2;
    text->interline(((int)text->interline()+step)%(4*step));
}

void _textualutil_cursorUp(GoTextual* textual) {
    textual->cursorUp();
}
void _textualutil_cursorDown(GoTextual* textual) {
    textual->cursorDown();
}

void _textualutil_cursorLeft(GoTextual* textual) {
    textual->cursorLeft();
}
void _textualutil_cursorRight(GoTextual* textual) {
    textual->cursorRight();
}
void _textualutil_cursorBegText(GoTextual* textual) {
    textual->cursorBegText();
}
void _textualutil_cursorEndText(GoTextual* textual) {
    textual->cursorEndText();
}
void _textualutil_cursorBegLine(GoTextual* textual) {
    textual->cursorBegLine();
}
void _textualutil_cursorEndLine(GoTextual* textual) {
    textual->cursorEndLine();
}
void _textualutil_mapString(GoTextual* textual, GBool set) {
    textual->mapString(set);
}
void _textualutil_mapUnderline(GoTextual* textual, GBool set) {
    textual->mapUnderline(set);
}
void _textualutil_mapCursor(GoTextual* textual, GBool set) {
    textual->mapCursor(set);
}


static GActionRec go_textual_action_array[] =
{ 
    {"insertChar", (GActionProc)_textualutil_insertChar},
    {"deleteChar", (GActionProc)_textualutil_deleteChar},
    {"deleteBeforeChar", (GActionProc)_textualutil_deleteBeforeChar},
    {"cursorLeft", (GActionProc)_textualutil_cursorLeft},
    {"cursorRight", (GActionProc)_textualutil_cursorRight},
    {"cursorUp", (GActionProc)_textualutil_cursorUp},
    {"cursorDown", (GActionProc)_textualutil_cursorDown},
    {"cursorBegLine", (GActionProc)_textualutil_cursorBegLine},
    {"cursorEndLine", (GActionProc)_textualutil_cursorEndLine},
    {"cursorBegText", (GActionProc)_textualutil_cursorBegText},
    {"cursorEndText", (GActionProc)_textualutil_cursorEndText},
    {"cursor", (GActionProc)_textualutil_cursor},
    {"mapBackground", (GActionProc)_textualutil_mapBackground},
    {"mapString", (GActionProc)_textualutil_mapString},
    {"mapUnderline", (GActionProc)_textualutil_mapUnderline},
    {"mapCursor", (GActionProc)_textualutil_mapCursor},
    {"presentation", (GActionProc)_textualutil_presentation},
    {"updateCharCase", (GActionProc)_textualutil_updateCharCase},
    {"underline", (GActionProc)_textualutil_underline},
    {"justification", (GActionProc)_textualutil_justification},
    {"interline", (GActionProc)_textualutil_interline},
    {NULL, NULL}
};


/**
FUNCTION
    m_actions()
DESCRIPTION
    The GoTextualClass m_actions function.
**/

const GActionRec*
GoTextualMetaclass::m_actions() const
{
    return go_textual_action_array;
}


/**
FUNCTION
    _updatePresentation (GPresentation);
DESCRIPTION
    This function allows to change the value after a presentation change.
**/

void
GoTextual::_updatePresentation (GPresentation p){

    if (p != _present ){
	char* str = _string;
	switch (p){
            case GLOWER_PRESENTATION:
            while (*str){
                if (isalpha (*str) && isupper (*str))
		    *str = tolower (*str);
                str++;
            }
            break;

            case GUPPER_PRESENTATION:
            while (*str){
                if (isalpha (*str) && islower (*str))
		    *str = toupper (*str);
                str++;
            }
            break;

            case GTITLE_PRESENTATION:
            for (;;){
                while (*str && !isalpha (*str)) str++;
                if (!*str) break;
                if (islower (*str))
		    *str = toupper (*str);
                str++;
                while (*str && isalpha (*str)){
                    if (isupper (*str))
			*str = tolower (*str);
                    str++;
                }
            }
            break;

            case GNO_PRESENTATION:
            break;
        }
    }
}


/**
FUNCTION
    _updateMask (ulong mask, GBool set)
DESCRIPTION
    Function called to update the drawing mask;
**/

void
GoTextual::_updateMask (ulong mask, GBool set)
{
GRegion reg;

    if (set)
	_mask |= mask;
    else
	_mask &= ~mask;
    m_add_clip (reg);

    if (backgroundMapped())
	RedrawForeOnly (reg);
    else
	RedrawBackAndFore (reg);
}


/**
FUNCTION
    containChar (char c, GPos pos, GTextDirection dir,
        GBool caseMatching) const
DESCRIPTION
    Returns the position of the first occurence of c in the string 
    beginning the search at pos with the direction dir, with
    account of letter cases or not.
**/

GPos
GoTextual::containChar (char c, GPos pos, GTextDirection dir,
    GBool caseMatching) const
{
    GPos len = length();
    assert (pos >= 0 && pos <= len);
    GPos step = (dir == GTEXT_FORWARD) ? 1 : -1;
    char c2 = c;
    if (!caseMatching && isalpha (c))
         c2 = islower (c) ? toupper (c) : tolower (c);
    for (; pos >= 0 && pos <= len; pos += step)
        if (*(_string + pos) == c || *(_string + pos) == c2)
            return pos;
    return (-1);
}


/**
FUNCTION
    containString (char* str, GPos size, GPos pos,
        GTextDirection dir, GBool caseMatching) const
DESCRIPTION
    Returns the position of the first occurence of str in the string 
    beginning the search at pos with the direction dir.
**/

GPos
GoTextual::containString (char* str, GPos size, GPos pos,
    GTextDirection dir, GBool caseMatching) const
{
    GPos len = length();
    assert (pos >= 0 && pos <= len);
    GPos step = (dir == GTEXT_FORWARD) ? 1 : -1;
    for (; (pos >= 0 && dir == GTEXT_BACKWARD) ||
	   (pos + size <= len && dir == GTEXT_FORWARD); pos += step)
    {
        GBool ok = GTRUE;
        for (int i = 0; i < size && ok; i++){
            char c = *(str + i);
            char c2 = c;
            if (!caseMatching && isalpha (c))
                c2 = islower (c) ? toupper (c) : tolower (c);
            ok = *(_string + pos + i) == c ||
                 *(_string + pos + i) == c2;
        }
        if (ok && i == size) return pos;
    }
    return (-1);
}


/**
FUNCTION
    nbLines() const;
DESCRIPTION
    This function returns the number of lines of a text.
**/

GPos
GoTextual::nbLines() const
{
register GPos nb = 1;

    for (register char* ptr = _string; *ptr; ptr++)
        if (*ptr == '\n') nb++;
    return (nb);
}


/**
FUNCTION
    lineNumber (GPos pos) const;
DESCRIPTION
    This function returns the number of the line of the position pos.
    The first line is the line 0.
**/

int
GoTextual::lineNumber (GPos pos) const
{
    if (pos == -1) pos = _currentPos;
    assert (pos >= 0 && pos <= length());
    register char* ptr = _string;
    char* ppos = _string+pos;
    for (int nb = 0; ptr < ppos; ptr++)
        if (*ptr == '\n') nb++;
    return (nb);
}


/**
  UNCTION
    begLinePos (int line) const;
DESCRIPTION
    This function returns the GPos of the beginning of the given line.
**/

GPos
GoTextual::begLinePos (int line) const
{
    if (line == -1)
	line = lineNumber();

    register char* ptr = _string;
    while (*ptr){
	if (line == 0) break;
        if (*ptr == '\n') line--;
	ptr++;
    }

    return ((line == 0) ? ptr - _string : -1);
}

/**
FUNCTION
    endLinePos (int line) const;
DESCRIPTION
    This function returns the GPos of the end of the given line.
**/

GPos
GoTextual::endLinePos (int line) const
{
    if (line == -1)
	line = lineNumber();

    register char* ptr = _string;
    while (*ptr){
	if (line == 0) break;
        if (*ptr == '\n') line--;
	ptr++;
    }

    if (line == 0) for (; *ptr && (*ptr != '\n'); ptr++);
    return ((line == 0) ? ptr - _string : -1);
}


/**
FUNCTION
    selectedString();
DESCRIPTION
    This function returns a string containing the selected portion
    of the text. The application has to free this string.
**/

char*
GoTextual::selectedString()
{
    if (selectionList().size() != 1)
        return NULL;
    else{
	GTextSeg* ts = selectionList().first();
        char* ptr = (char*)GMALLOC (ts->last() - ts->first() + 1);
        strncpy (ptr, _string + ts->first(),
	    ts->last() - ts->first());
        *(ptr + ts->last() - ts->first()) = '\0';
        return ptr;
    }
}


/**
FUNCTION
    selectPortion (GPos first, GPos last);
DESCRIPTION
    This function selects a portion of text.
**/

GBool
GoTextual::selectPortion (GPos first, GPos last, GBool val)
{
GBool ret;

    assert (first >= -1 && first <= length());
    assert (last >= -1 && last <= length());

    if (last < first){
	GPos aux = first;
        first = last;
        last = aux;
    }

    /* precondition call */
    if (ret = (_call_selection_precond (first, last) &&
	((GoTextualMetaclass*)getMetaclass())->
	    _call_selection_precond (this, first, last) &&
		GoTextual::metaclass()->
		    _call_selection_precond (this, first, last)))
    {
	GRegion reg;
        /* just clear the old selection */
	if (first == -1){
	    for (GTextSeg* ts = _selectionList.first(); ts;
	        ts = ts->next())
		    GAdd (reg, boundariesToRegion
			(ts->first(), ts->last()), reg);
	    _selectionList.reinit();
	    if (backgroundMapped())
		RedrawForeOnly (reg);
	    else
		RedrawBackAndFore (reg);
	}
    
        /* add a new selection portion */
        else if (val){
            reg = boundariesToRegion (first, last);
	    GTextSeg* new_seg = new GTextSeg;
	    new_seg->first (first);
	    new_seg->last (last);
	    _selectionList.insertSeg (new_seg);
	    RedrawForeOnly (reg);
	}

        /* remove a selection portion */
	else{
	    reg = boundariesToRegion (first, last);
	    _selectionList.deletePortion (first, last);
	    if (backgroundMapped())
		RedrawForeOnly (reg);
	    else
		RedrawBackAndFore (reg);
	}

	/* postconditions call */
	_call_selection_postcond (first, last);
	((GoTextualMetaclass*)getMetaclass())->
	    _call_selection_postcond (this, first, last);
	GoTextual::metaclass()->
	    _call_selection_postcond (this, first, last);
    }

    return ret;
}


char
GoTextual::operator[](GPos pos) const
{
    assert (pos >= 0 && pos <= length());
    return _string[pos];
}


void
GoTextual::wordBoundaries (GPos pos, GPos &wordFirstPos, GPos &wordLastPos){

    if (isSeparator (*(_string+pos))){
        wordFirstPos = -1;
        wordLastPos = -1;
    }
    else{
        /* left boundary search */
        wordFirstPos = pos;
        while (!isSeparator (*(_string+wordFirstPos-1)) &&
	    wordFirstPos > 0)
		wordFirstPos--;

        /* right boundary search */
        wordLastPos = pos;
        while (!isSeparator (*(_string+wordLastPos)) &&
	    wordLastPos < length())
		wordLastPos++;
    }
}


void
GoTextual::lineBoundaries (GPos pos, GPos &lineFirstPos, GPos &lineLastPos){
    lineFirstPos = begLinePos (lineNumber (pos));
    lineLastPos = endLinePos (lineNumber (pos));
}


void
GoTextual::_allocString (GPos size)
{
    GPos len = (_size == 0) ? 0 : length();

    if (_size == 0){
	_size = (size/GRANUL_SIZE+1) * GRANUL_SIZE;
	_string = (char*)GMALLOC (_size);
    }

    else if (size >= _size){
	_size = (size/GRANUL_SIZE+1) * GRANUL_SIZE;
	GREALLOC (_string, _size);
    }

    else
	return;


    /* fill end of string with NULL char */
    while (len < _size){
	*(_string + len) = '\0';
	len++;
    }
}


/**
FUNCTION
    boundariesToRegion (GPos first, GPos last) const;
DESCRIPTION
    Returns the GRegion defined by the 2 given positions.
**/

GRegion
GoTextual::boundariesToRegion (GPos, GPos) const
{
    return GEmptyRegion();
}


/**
FUNCTION
    firstSelected();
DESCRIPTION
    This function returns the first position of the first segment
    of the selection list.
**/

GPos
GoTextual::firstSelected()
{
    GTextSeg* ts = selectionList().first();
    return ts ? ts->first() : -1;
}


/**
FUNCTION
    lastSelected();
DESCRIPTION
    This function returns the last position of the last segment
    of the selection list.
**/

GPos
GoTextual::lastSelected()
{
    if (!selectionList().size()) return -1;
    for (GTextSeg* ts = selectionList().first();
	ts->next(); ts=ts->next());
    return ts->last();
}
