#ifdef REV_INFO
static char rcsid[] = "$Id: Converter.cc,v 1.10 1992/12/30 14:55:51 jacomet Exp $";
#endif /* REV_INFO */
/*****************************************************************************
MODULE:  Data Conversion
FILE:    Converter.cc
AUTHOR:  Aline Jacomet
DATE:    March 1992

DESCRIPTION:
    See Reference Manual.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#ifdef _AIX
#include <stdio.h>
#include <errno.h>
#include <unistd.h>
#else
#include <osfcn.h>
#endif
#include <sys/types.h>
#include <sys/stat.h>

#include <Go/Tools/FileMedium.h>
#include <Go/Tools/SharedObj.h>
#include <Go/Ext/Converter.h>
#include <Go/Go.h>
#include <Go/mGo.h>
#include <Go/GoAscii.h>
#ifdef GoLICENSE
#include <Go/GoBinary.h>
#endif




                         /******************
			  * PUBLIC METHODS *
			  ******************/



/**********************************
 * Constructor/Destructor methods *
 **********************************/

GConverter::GConverter()
{
}

GConverter::~GConverter()
{
}


/**********************
 * Conversion methods *
 **********************/

ushort GConverter::gostreamFileToGo(const char* filename,
				     GSet* gos, uint& nb_gos,
				     GSet* shared, uint& nb_shared,
				     char** header) const
{
    ushort r = verify_filename(filename);
    if (r == GIS_NOT_REG_FILE || r == GFILE_ACCESS_ERROR || r == GNO_FILE)
	return r;
    
    ushort ret = GOK;
    nb_gos = nb_shared = 0;
    GFileMedium* medium = new GFileMedium();
    if (! medium->open(filename, "r")) {
	GoStream* gostream = NULL;
#ifdef GoLICENSE
	if (isGoBinary(medium))
	    gostream = new GoBinary(medium);
	else
#endif	    
	    gostream = new GoAscii(medium);
	gostream->openRead();
	if (header) {
	    ulong nb = 0;
	    gostream->readHeader(&nb, header);
	}
	void* obj;
	char* class_name;
	do {
	    if (!gostream->restoreObj(&obj, &class_name)) {
		for (Go* o = (Go*)gos->first(); o; o = (Go*)gos->next())
		    o->destroy();
		for (GSharedObject* s = (GSharedObject*)shared->first(); s;
		     s = (GSharedObject*)shared->next())
		    s->destroy();
		ret = GLOAD_STREAM_ERROR;
		break;
	    } else {
		if (obj){
		    if (class_name[0] == 'G' && class_name[1] == 'o') {
			gos->add(obj);
			nb_gos++;
		    } else 
			if (class_name[0] == 'G' && class_name[1] == 'S') {
			    shared->add(obj);
			    nb_shared++;
			} else {
			    for (Go* o = (Go*)gos->first(); o;
				 o = (Go*)gos->next())
				o->destroy();
			    for (GSharedObject* s = (GSharedObject*)
				 shared->first(); s;
				 s = (GSharedObject*)shared->next())
				s->destroy();
			    ret = GLOAD_STREAM_ERROR;
			    break;
			}
		}
	    }
	} while (obj);
	
	gostream->close();
	delete gostream;
	medium->close();
    } else
	ret = GFILE_ACCESS_ERROR;
    delete medium;
    return ret;
}


ushort GConverter::goToGostreamFile(const GSet* go, const char* filename,
				     GStreamType type, char* header) const
{
    ushort r = verify_filename(filename);
    if (r == GIS_NOT_REG_FILE || r == GFILE_ACCESS_ERROR)
	return r;
    
    GFileMedium* medium = new GFileMedium();
    if (medium->open(filename, "w")) {
	delete medium;
	return GFILE_ACCESS_ERROR;
    }
    GoStream* gostream = NULL;
    if (type == GBINARY_STREAM) {
#ifdef GoLICENSE	
	gostream = new GoBinary(medium);
#else
	GWarning("GConverter::gotoGostreamFile : The GoBinary is available with a license. I'm using a GoAscii\n");
	gostream = new GoAscii(medium);
#endif	
    } else
	gostream = new GoAscii(medium);
    gostream->openWrite();
    
    if (header)
	gostream->writeHeader(strlen(header) + 1, header);
    ushort ret = GOK;
    for (Go* obj = (Go*)go->first(); obj; obj = (Go*)go->next())
	if (! gostream->saveObj(obj, obj->getMetaclass()->classname())) {
	    ret = GSAVE_STREAM_ERROR;
	    break;
	}

    gostream->close();
    medium->close();
    delete gostream;
    delete medium;

    return ret;
}


ushort GConverter::sharedToGostreamFile(const GSet* sh, const char* filename,
					 GStreamType type, char* header) const
{
    ushort r = verify_filename(filename);
    if (r == GIS_NOT_REG_FILE || r == GFILE_ACCESS_ERROR)
	return r;
    
    GFileMedium* medium = new GFileMedium();
    if (medium->open(filename, "w")) {
	delete medium;
	return GFILE_ACCESS_ERROR;
    }
    GoStream* gostream = NULL;
    if (type == GASCII_STREAM)
	gostream = new GoAscii(medium);
    else {
#ifdef GoLICENSE	
	gostream = new GoBinary(medium);
#else
	GWarning("GConverter::sharedtoGostreamFile : The GoBinary is available with a license. I'm using a GoAscii\n");
	gostream = new GoAscii(medium);
#endif	
    }
    gostream->openWrite();

    if (header)
	gostream->writeHeader(strlen(header) + 1, header);
    ushort ret = GOK;
    for (GSharedObject* obj = (GSharedObject*)sh->first(); obj;
	 obj = (GSharedObject*)sh->next())
	if (! gostream->saveObj(obj, obj->getClassname())) {
	    ret = GSAVE_STREAM_ERROR;
	    break;
	}

    gostream->close();
    medium->close();
    delete gostream;
    delete medium;

    return ret;
}


char* GConverter::goToGostream(const GSet* go, ulong& size,
				GStreamType type, char* header) const
{
    char* filename = tempnam(NULL, NULL);
    if (! filename)
	return NULL;
    if (goToGostreamFile(go, filename, type, header) != GOK)
	return NULL;
    char* ret = fileToStream(filename, size);
    unlink(filename);
    GFREE(filename);
    return ret;
}


char* GConverter::sharedToGostream(const GSet* sh, ulong& size,
				    GStreamType type, char* header) const
{
    char* filename = tempnam(NULL, NULL);
    if (! filename)
	return NULL;
    if (sharedToGostreamFile(sh, filename, type, header) != GOK)
	return NULL;
    char* ret = fileToStream(filename, size);
    unlink(filename);
    GFREE(filename);
    return ret;
}


ushort GConverter::gostreamToGo(const char* stream, ulong size,
				 GSet* gos, uint& nb_gos,
				GSet* shared, uint& nb_shared,
				char** header) const
{
    char* filename = tempnam(NULL, NULL);
    if (!filename)
	return GERROR;
    ushort r = streamToFile(stream, size, filename);
    if (r != GOK)
	return r;
    ushort ret = gostreamFileToGo(filename, gos, nb_gos, shared, nb_shared,
				  header);
    unlink(filename);
    GFREE(filename);
    return ret;
}


ushort GConverter::streamToFile(const char* stream, const ulong size,
				 const char* filename) const
{
    ushort r = verify_filename(filename);
    if (r == GIS_NOT_REG_FILE || r == GFILE_ACCESS_ERROR)
	return r;
    
    FILE* file = fopen(filename, "w");
    if (file) {
	if (fwrite(stream, 1, (int)size, file) > 0) {
	    fclose(file);
	    return GOK;
	} else {
	    fclose(file);
	    return GERROR;
	}
    } else
	return GFILE_ACCESS_ERROR;
}


char* GConverter::fileToStream(const char* filename, ulong& size) const
{
    struct stat file_status;
    stat((char*)filename, &file_status);
    char* stream = NULL;
    if (file_status.st_size > 0) {
	stream = (char*)GMALLOC((unsigned int)file_status.st_size);
	size = file_status.st_size;
	FILE* file = fopen(filename, "r");
	if (file) {
	    if (fread(stream, 1, (int)file_status.st_size, file) <= 0)
		GFREE(stream);
	} else
	    GFREE(stream);
    }
    return stream;
}


/**
FUNCTION
    verify_filename()
DESCRIPTION
    
**/
ushort GConverter::verify_filename(const char* filename) const
{
    struct stat file_status;
    if (stat(filename, &file_status) == 0) {
	// file exists;
	if (S_ISREG(file_status.st_mode)){
	    // file is regular;
	    return GFILE_EXIST;
	} else {
	    // file is not regular;
	    return GIS_NOT_REG_FILE;
	}
    } else {
	// file cannot be reached;
	if (errno != ENOENT) {
	    // file exists;
	    return GFILE_ACCESS_ERROR;
	} else {
	    // file does not exist;
	    return GNO_FILE;
	}
    }
}
