#ifdef REV_INFO
static char rcsid[] = "$Id: XInDisp.cc,v 1.7 1992/12/10 14:01:35 jojic Exp $";
#endif
/*****************************************************************************
MODULE:  Tools
FILE:    XInDisp.cc
AUTHOR:  Philippe Smadja
DATE:    29 May 1990

DESCRIPTION:
    See Go/XInDisp.h

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#include <Go/XInDisp.h>
#include <Go/XGoInterf.h>
#include <Go/Tools/Alloc.h>
#include <errno.h>

#ifdef SYSV

   // get the clock ticks;
#  ifdef _POSIX_SOURCE
#        include <unistd.h>
         extern "C" {
	     long sysconf(int);
	 }
#  else
#        include <sys/param.h>   /* to get HZ <--> CLK_TCK */
#  endif
    
   // get the struct tms;

#  include <sys/types.h>
#  include <sys/times.h>

   extern "C" {
#ifdef _AIX
       int times(struct tms*);
#else
       long times(struct tms*);
#endif
   }

   // Get timeval struct;
   // I do not how to get timeval on each SYSV 
   // with each c++ compiler;
   // So I define the struct timeval which is every time
   // the same one except for hpux.


#  if (!defined(__TIMEVAL__) && !defined(_STRUCT_TIMEVAL))
#      ifdef hpux
             struct timeval {
		 unsigned long tv_sec;
		 long          tv_usec;
	     };
#            define _tv_sec_type      ulong
#      else
             struct timeval {
		 long tv_sec;
		 long tv_usec;
	     };
#            define _tv_sec_type      long
#      endif
#  endif

   extern "C" {
#  ifdef hpux
         int select(size_t, int*, int*, int*, const  timeval*);
#        define _select_mask_type int
#  else
         int select(ulong, ulong*,  ulong*, ulong*, const timeval* timer);
#        define _select_mask_type ulong
#  endif
   }


#else   /* NOT SYSV */
#  ifdef __GNUC__
#      include <sys/time.h>
#      include <sys/socket.h>
#  else
#      include <sys/time.h>
#  endif
#  define _select_mask_type ulong
#  define _tv_sec_type      long
#endif

struct timeval NO_BLOCK ;

/**
FUNCTION
    add/rem input
DESCRIPTION
    to be defined ????
**/


/**
FUNCTION
    add/rem timer
DESCRIPTION
    add/rem a struct Timer in/out the timer list;
**/
struct Timer {
    GInDispTimerProc procedure;
    void*            closure;
    ulong            interval;
    long             elapse;
    Timer(GInDispTimerProc p, void* c, ulong i, long e)
    {
	procedure = p;
	closure =   c;
	interval =  i;                  // in ms;
	elapse   =  e; 
    };
};

/**
FUNCTION
    static GetCurrentTime()
DESCRIPTION
    returns the current time in base  CLK_TCK
**/
static long GetCurrentTime(long
#ifndef SYSV
			   clk_tck
#endif
			   )
{
#ifdef SYSV
    struct tms t;
    return times(&t);
#else
    struct timeval t;
    (void)gettimeofday(&t, NULL);
    return  (long) (t.tv_sec * clk_tck + t.tv_usec * clk_tck / 1000000);
#endif
}
    
void XInputDispatcher::timerProc(GInDispTimerProc procedure ,
				 void* closure, ulong interval,
				 GAddOrRemove add)
{
    if (add == GADD)
    {
	Timer timer (procedure, closure, interval,
		     (long)(GetCurrentTime(_clk_tck) + MilliSecondToClock(interval)));
	_timers.add(&timer);
    }
    else
    {
	// find the timer and remove it;
	Timer* timer = (Timer*)_timers.data();
	for (int i = 0 ; i < _timers.nelems(); i++, timer++)
	    if (timer->procedure == procedure &&
		timer->closure == closure &&
		timer->interval == interval)
	    {
		_timers.rem(i);
		break;
	    }
    }
}

/**
FUNCTION
    runEventLoop()
DESCRIPTION
    it is like an main loop. But we can return from it;
**/
void* XInputDispatcher::runEventLoop()
{
    _stop = GFALSE;

    while (! _stop)
    {
	// the core of the while is exactly the code of handleNextEvent();
	XEvent xevent;
	NextEvent(&xevent); // read nextevent and handle timer, idle ;
	if (CallFilters(&xevent))
	    DispatchXEvent(&xevent);

	ResetIdles();
    }
    
    _stop = GFALSE;
    return _value;
}

static _select_mask_type mask[32] =
{
    1<<0, 1<<1, 1<<2, 1<<3, 1<<4, 1<<5, 1<<6, 1<<7, 1<<8, 1<<9,
    1<<10,  1<<11, 1<<12, 1<<13, 1<<14, 1<<15, 1<<16, 1<<17, 1<<18, 1<<19,
    1<<20,  1<<21, 1<<22, 1<<23, 1<<24, 1<<25, 1<<26, 1<<27, 1<<28, 1<<29,
    1<<30,  1<<31
 };

/**
FUNCTION
    eventReady()
DESCRIPTION
    return True is an event or a timer is pending
**/
GBool XInputDispatcher::eventReady()
{
    // test the input queue;
    if (XEventsQueued(_dpy, QueuedAlready))
	return GTRUE;

    // test the timers;
    if (_timers.nelems())
    {
	long current_time = GetCurrentTime(_clk_tck);
	Timer* timer = (Timer*)_timers.data();
	for (int i = 0 ; i < _timers.nelems(); i++, timer++)
	    if (current_time >= timer->elapse)
		return GTRUE;
    }

    // test if no event is read to be read in the input queue;
    _select_mask_type _readmask = mask[_dpy->fd];
    _select_mask_type readmask;
    while (readmask = _readmask, select(32,
#ifdef SYSV
					&readmask,
/* #elif defined (__GNUC__) not supported by each preprocessor */
#else
#if defined (__GNUC__)
					(void*)&readmask,
#else
					(fd_set*) &readmask,
#endif
#endif
					NULL, NULL,
					&NO_BLOCK) <  0)
    {
	if (errno == EINTR)
	    continue;
	GError ("XInputDispatcher::NextEvent() select: errno = %d", errno);
    }

    return (GBool)(readmask != 0);
}

/**
FUNCTION
    handleNextEvent()
DESCRIPTION
    read and dispatch the next event / block is no event is pending
**/

void XInputDispatcher::handleNextEvent()
{
    XEvent xevent;
    NextEvent(&xevent); // read nextevent and handle timer, idle ;
    if (CallFilters(&xevent))
	DispatchXEvent(&xevent);

    ResetIdles();
}


/******* PROTECTED METHODS ************************/


/**
FUNCTION
    protected  NextEvent()
DESCRIPTION
    read next event (wait if no event),
    Timer and Idle procedure are handled here
**/
void XInputDispatcher::NextEvent(XEvent* xevent)
{
    _select_mask_type _readmask = mask[_dpy->fd];
    _select_mask_type readmask;
    GBool again = GTRUE;
    

    while (GTRUE)
    {
	// timer elapses ???;
	long next_timer;
	GBool a_timer_was_called = HandleTimers(next_timer);

	// if event is already queues read it;
	// and returns;
	
	if (XEventsQueued(_dpy, QueuedAlready))
	{
	    // handle it;
	    XNextEvent(_dpy, xevent);
	    return;
	}
	
	readmask = 0;
	
	if (again || a_timer_was_called)
	{
	    // test idle time;
	    while (readmask = _readmask, select(32,
#ifdef SYSV
					&readmask,
#else
#if defined (__GNUC__)
					(void*)&readmask,
#else
					(fd_set*) &readmask,
#endif
#endif
					 NULL, NULL,
					 &NO_BLOCK) <  0)
	    {
		if (errno == EINTR)
		    continue;
		GError ("XInputDispatcher::NextEvent() select: errno = %d", errno);
	    }

	    if (readmask == 0) 
		again = CallIdles();
	}

	if (readmask == 0)
	{
	    if (XEventsQueued(_dpy, QueuedAlready))
	    {
		// if event is already queued during idle/time handling
		// read it and returns;
		XNextEvent(_dpy, xevent);
		return;
	    }
	
	    struct timeval* timer = NULL ; // --> block if no timer;
	    if (next_timer != MAXLONG)
	    {
		struct timeval timeval;
		timeval.tv_sec = (_tv_sec_type)(next_timer / _clk_tck);
		timeval.tv_usec = (next_timer % _clk_tck) * 1000000 / _clk_tck;
		timer = &timeval;
	    }
	    
	    while (readmask = _readmask, select(32,
#ifdef SYSV
					&readmask,
#else
#if defined (__GNUC__)
					(void*)&readmask,
#else
					(fd_set*) &readmask,
#endif
#endif
					NULL, NULL,
					timer) <  0)
	    {
		if (errno == EINTR)
		    continue;
		GError ("XInputDispatcher::NextEvent() select: errno = %d", errno);
	    }
	}

	if (readmask)
	{
	    // something is ready to be read; read it;	
	    XNextEvent(_dpy, xevent);
	    return;
	}
	// else a timer is pending: continue the loop;
    }
}



/**
FUNCTION
    protected HandleTimers()
DESCRIPTION
    Verity id timers elapses. If yes: call it;
    next_timer will be set to time when the next timer will elapses;
    If next_timer is MAXLONG no timer will elapse.
    It return true is a timer is called;
**/
GBool XInputDispatcher::HandleTimers(long& next_timer)
{
    next_timer = MAXLONG;
    GBool a_timer_has_been_called = GFALSE;
    if (_timers.nelems())
    {
	// take the current time, DO NOT RESET it;
	// after a timer execution;
	int i;
	long current_time = GetCurrentTime(_clk_tck);
	
	// dup the timer list in local GArray;
	
	GArray dup_timers(_timers.data(), _timers.nelems(), sizeof(Timer));
	
	Timer* timer = (Timer*)dup_timers.data();
	for (i = 0 ; i < dup_timers.nelems(); i++, timer++)
	    if (current_time >= timer->elapse)
	    {
		// remove it from the NON DUPPLICATED LIST;
		timerProc(timer->procedure, timer->closure, timer->interval, GREMOVE);
		// call it;
		(*timer->procedure)(timer->closure);
		// a call was done;
		a_timer_has_been_called = GTRUE;
	    } 

	// find the next timer;
	timer = (Timer*)_timers.data();
	for (i = 0 ; i < _timers.nelems(); i++, timer++)
	    next_timer = min(next_timer, timer->elapse - current_time);
    }
    
    return  a_timer_has_been_called;
}

/**
FUNCTION
    protected DispatchXEvent(XEvent*)
DESCRIPTION
   retreive the X interface associated to the window of the event
   and ask the interface to handle it;
**/
void XInputDispatcher::DispatchXEvent(XEvent* xevent)
{
    XGoInterf* interf = XWindowToXGoInterf(xevent->xany.window);
    if (interf)
	interf->handleXEvent(xevent);
}

/**
FUNCTION
    protected XWindowToXGoInterface(Window window)
DESCRIPTION
   retreive the X interface associated to the window of the event
**/
XGoInterf* XInputDispatcher::XWindowToXGoInterf(Window window)
{
    XGoInterf** xinterf = (XGoInterf**)_interfaces.data();
    for (int i = 0 ; i < _interfaces.nelems(); i++, xinterf++)
	if ((*xinterf)->drawable() == window)
	    return *xinterf;
    GWarning ("XInputDispatcher receives a event from an unknown interface");
    return NULL;
}



/**************** PUBLIC XINPUTDISPATCHER SPECIFIC   ************/


/**
FUNCTION
    add/rem Interface()
DESCRIPTION
    add an XGoInterf which will handle input
    It is a specific method of XInputDispatcher
**/
static void ThisInterfIsDestroyed(void* indisp, GoInterf* interf)
{
    ((XInputDispatcher*)indisp)->remXGoInterf( (XGoInterf*)interf);
}

void  XInputDispatcher::addXGoInterf(XGoInterf* xinterf)
{
    _interfaces.add(xinterf);
    xinterf->destroyPostcond(ThisInterfIsDestroyed, this, GADD);
}

void  XInputDispatcher::remXGoInterf(XGoInterf* xinterf)
{
    _interfaces.rem(xinterf);
    xinterf->destroyPostcond(ThisInterfIsDestroyed, this, GREMOVE);
}


/**
FUNCTION
    constructor/destructor
DESCRIPTION
    
**/

XInputDispatcher::XInputDispatcher (Display* dpy) : _timers(sizeof(Timer), 4)
{
    _dpy = dpy;
#ifdef SYSV
#      ifdef _POSIX_SOURCE
            _clk_tck= sysconf(_SC_CLK_TCK);
#      else
#           ifdef HZ         // from sys/param.h
                  _clk_tck = HZ;
#           else
                  _clk_tck = 100; // you have to see the current times manual.
#           endif
#     endif
#else
    // with gettimeofday we have the time in us;
    // --> translate it in ms;
    _clk_tck=1000;
#endif
}

XInputDispatcher::~XInputDispatcher()
{
}



