#ifdef REV_INFO
static char rcsid[] = "$Id: DefGoInt.cc,v 1.4 1992/09/25 15:23:18 smadja Exp $";
#endif
/*****************************************************************************
MODULE:  GoInterface
FILE:    DefGoInt.cc
AUTHOR:  Philippe Smadja
DATE:    April 1991

DESCRIPTION:
    This file implement the default GoInterf.
    No output done and error on input requests.
    returns 0 on font query requests.
    the colorFomName method implements the fucntion with
    standart colorname; cf <standart__Color> in the file.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include <Go/GoInterf.h>
#include <Go/Tools/Error.h>

class DefaultGoInterf : public GoInterf
{
  public:

    DefaultGoInterf();
    ~DefaultGoInterf();
    
    virtual void clear (const GRegion&) {}

    virtual void polyline  (const GShPen*, const GShBrush*,
			    const GCoordSys&, int npt, const GPoint*,
			    GQuality);

    virtual void image	   (const GShImageBrush*,
			    const GCoordSys&, const GPoint&, GNumber, GNumber,
			    const class GShPixelArray*, GQuality);

    virtual void polygon   (const GShPen*, const GShBrush*,
			    const GCoordSys&, int npt, const GPoint*, GQuality);

    virtual void string	   (const GShTextBrush*, const GShBrush*,
			    const GCoordSys&, const GPoint&, GNumber,
			    int, const char*,
			    GQuality);

    virtual GPoint getPointer(const GCoordSys* = NULL) ;
    virtual void setPointer(const GPoint&, const GCoordSys* = NULL) ;
    
    
    virtual void   getImage (GImageBrushValues& imagebrush_result,
			     GPixelArrayValues& pixelarray_result,
			     const GCoordSys& cs, const GPoint& origin,
			     GNumber width, GNumber height, ushort depth = 0,
			     GBool compress_colormap = GTRUE,
			     ushort x_resolution = 0, ushort y_resolution = 0,
			     GQuality quality = GMAX_QUALITY);


    virtual GShColor* colorFromName(const char* color_name) const;

    virtual GBool captureEvent(GEventMask, GBool, GTimeStamp);
    virtual void unCaptureEvent(GTimeStamp) ;
    virtual ulong inputFd();
    
    virtual GNumber pixelWidth () const; 
    virtual GNumber pixelHeight () const;

    virtual const char*  fontName      (const GCoordSys&, GNumber,
					const GShFont*);

    virtual GNumber stringLeftBearing  (const char*,
					int, GNumber, const GShFont*);
    virtual GNumber stringRightBearing (const char*,
					int, GNumber,
					const GShFont*);
    virtual GNumber stringAscent       (const char*,
					int, GNumber,
					const GShFont*);
    virtual GNumber stringDescent      (const char*,
				       int, GNumber,
				       const GShFont*);
    virtual GNumber stringWidth        (const char*,
					int, GNumber,
					const GShFont*);
    virtual GNumber fontAscent         (GNumber,
					const GShFont*);
    virtual GNumber fontDescent        (GNumber,
					const GShFont*);
    virtual GNumber fontUnderlineThickness (GNumber,
					    const GShFont*);
    virtual GNumber fontUnderlinePosition  (GNumber,
					    const GShFont*);
    virtual GNumber fontItalicAngle        (const GShFont*);

    virtual GNumber fontStrikeoutAscent    (GNumber,
					    const GShFont*);
    virtual GNumber fontStrikeoutDescent   (GNumber,
					    const GShFont*) ;
    virtual GNumber fontMinSpace         (GNumber,
					  const GShFont*);
    virtual GNumber fontMaxSpace         (GNumber,
					  const GShFont*);
    virtual GNumber fontNormSpace        (GNumber,
					  const GShFont*);
    virtual GNumber fontEndSpace         (GNumber,
					  const GShFont*);
};


DefaultGoInterf::DefaultGoInterf()
{
}

DefaultGoInterf::~DefaultGoInterf()
{
}

void DefaultGoInterf::polyline	(const GShPen*, const GShBrush*,
				 const GCoordSys&, int,
				 const GPoint*, GQuality)
{
}

void DefaultGoInterf::image	 (const GShImageBrush*,
				  const GCoordSys&, const GPoint&, GNumber, GNumber,
				  const class GShPixelArray*, GQuality)
{
}

void DefaultGoInterf::polygon   (const GShPen*, const GShBrush*,
				 const GCoordSys&, int, const GPoint*, GQuality)
{
}

void DefaultGoInterf::string	 (const GShTextBrush*, const GShBrush*,
				  const GCoordSys&, const GPoint&, GNumber,
				  int, const char*,
				  GQuality)
{
}


GPoint  DefaultGoInterf::getPointer(const GCoordSys*)
{
    GError("DefaultGoInterf::%s no query method:", "getPointer");
    return GPoint(0,0);
}

void DefaultGoInterf::setPointer(const GPoint&, const GCoordSys*)
{
}

void DefaultGoInterf::getImage (GImageBrushValues&, /* result */
				GPixelArrayValues&, /* result */
				const GCoordSys& ,
				const GPoint&,      /* origin */
				GNumber,            /* width */
				GNumber,            /* height */
				ushort,            /* depth = 0 */
				GBool,              /* compress_colormap = GTRUE */
				ushort,            /* x_resolution = 0 */
				ushort,            /* y_resolution = 0 */
				GQuality            /* quality = GmaxQuality */)
{
    GError("DefaultGoInterf::%s no query method:", "getImage");
}

/**
FUNCTION
    colorFromName()
DESCRIPTION
    make a GShColor with a set of standart color_name. If the colorname
    is not in the table, the function returns a black.
    Upper, lower case and blank are handled.
**/
    
#include <ctype.h>
#include <Go/Tools/ShColor.h>

struct __color {
    char* name;
    ushort red;
    ushort green;
    ushort blue;
    GShColor* shcolor;
};

static  __color standart__color[] = {
{ "black", 0, 0, 0, NULL },
{ "white", GMAX_RGB_VALUE, GMAX_RGB_VALUE, GMAX_RGB_VALUE, NULL},
{ "gray",  GMAX_RGB_VALUE/2, GMAX_RGB_VALUE/2, GMAX_RGB_VALUE/2, NULL},
{ "lightgray", GMAX_RGB_VALUE*3/4,GMAX_RGB_VALUE*3/4,GMAX_RGB_VALUE*3/4, NULL},
{ "dimgray", GMAX_RGB_VALUE/4,GMAX_RGB_VALUE/4,GMAX_RGB_VALUE/4, NULL},
{ "red" ,  GMAX_RGB_VALUE, 0, 0, NULL},
{ "green", 0, GMAX_RGB_VALUE, 0, NULL},
{ "blue" , 0, 0, GMAX_RGB_VALUE, NULL},
{ "magenta" , GMAX_RGB_VALUE, 0, GMAX_RGB_VALUE, NULL},
{ "yellow", GMAX_RGB_VALUE, GMAX_RGB_VALUE, 0, NULL},
{ "brown" , GMAX_RGB_VALUE*165/256, GMAX_RGB_VALUE*42/256, GMAX_RGB_VALUE*42/256, NULL},
};
    
GShColor*  DefaultGoInterf::colorFromName(const char* color_name) const
{

    int sz = strlen(color_name);
    if (sz < 63) {
	char cname[64];
	int i,j;
	/**
	  make Name
	  **/
	for (i = 0, j = 0; i < sz; color_name++, i++)
	    if (*color_name != ' ') {
		if (isupper(*color_name))
		    cname[j] = tolower(*color_name);
		else cname[j] = *color_name;
		j++;
	    }
	cname[j] = '\0';
	__color* color = &standart__color[0];
	int i_max = (int)(sizeof(standart__color)/ sizeof(__color));
	for (i = 0; i < i_max; i++, color++)
	    if (strcmp(color->name, cname) == 0) {
		if (color->shcolor == NULL) {
		    GRgb rgb (color->red, color->green, color->blue);
		    color->shcolor = GCreateShColor(&rgb);
		}
		return color->shcolor->dup();
	    }
    }
    
    GWarning ("DefaultGoInterf %s not in standart colorname table, black is assumed",
	     color_name-sz);
    return DefaultGoInterf::colorFromName("black");
}

GoInterf* GoDefaultGoInterf()
{
    static DefaultGoInterf* p = NULL;
    if (p == NULL)
	p = new DefaultGoInterf();

    return p;
}


GNumber DefaultGoInterf::pixelWidth () const {
	return 0.325;
} 
 
GNumber DefaultGoInterf::pixelHeight () const {
	return 0.325;
}
 
const char*  DefaultGoInterf::fontName      (const GCoordSys&, GNumber,
					const GShFont*)
{
	return "DefaultGoInterf::no FontName";
}


GNumber DefaultGoInterf:: stringLeftBearing  (const char*,
				int, GNumber, const GShFont*) {
	return 0;
}

GNumber DefaultGoInterf:: stringRightBearing (const char*,
				int, GNumber,
				const GShFont*) {
	return 0;
}
GNumber DefaultGoInterf:: stringAscent       (const char*,
				int, GNumber,
				const GShFont*) {
	return 0;
}
GNumber DefaultGoInterf:: stringDescent      (const char*,
			       int, GNumber,
			       const GShFont*) {
	return 0;
}
GNumber DefaultGoInterf:: stringWidth        (const char*,
				int, GNumber,
				const GShFont*) {
	return 0;
}


GNumber DefaultGoInterf:: fontAscent         (GNumber,
				const GShFont*) {
	return 0;
}

GNumber DefaultGoInterf:: fontDescent        (GNumber,
				const GShFont*) {
	return 0;
}

GNumber DefaultGoInterf:: fontUnderlineThickness (GNumber,
				    const GShFont*) {
	return 0;
}

GNumber DefaultGoInterf:: fontUnderlinePosition  (GNumber,
				    const GShFont*){
	return 0;
}

GNumber DefaultGoInterf:: fontItalicAngle        (const GShFont*) {
	return 0;
}

GNumber DefaultGoInterf:: fontStrikeoutAscent    (GNumber,
				    const GShFont*) {
	return 0;
}

GNumber DefaultGoInterf:: fontStrikeoutDescent   (GNumber,
				    const GShFont*) {
	return 0;
}

GNumber DefaultGoInterf:: fontMinSpace         (GNumber,
				  const GShFont*) {
	return 0;
}

GNumber DefaultGoInterf:: fontMaxSpace         (GNumber,
				  const GShFont*) {
	return 0;
}

GNumber DefaultGoInterf:: fontNormSpace        (GNumber,
				  const GShFont*) {
	return 0;
}

GNumber DefaultGoInterf:: fontEndSpace         (GNumber,
				  const GShFont*) {
	return 0;
}

GBool DefaultGoInterf::captureEvent(GEventMask, GBool, GTimeStamp)    {
	GError("DefaultGoInterf::%s no input method:", "captureEvent");
	return GFALSE;
}
void DefaultGoInterf::unCaptureEvent(GTimeStamp)    {
	GError("DefaultGoInterf::%s no input method:", "unCaptureEvent");
}

ulong DefaultGoInterf::inputFd()    {
	GError("DefaultGoInterf::%s no input method:", "inputFd");
	return 0;
}

