#ifdef REV_INFO
static char rcsid[] = "$Id: PSFont.cc,v 1.3 1992/12/10 14:03:59 jojic Exp $";
#endif
/*****************************************************************************
MODULE:  GoInterface
FILE:    PSGoInterf.cc
AUTHOR:  Philippe Smadja, Michel Marcuse. 
DATE:    Jul 1991

DESCRIPTION:
  see PSGoInterf.h

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/


#include <Go/PSGoInterf.h>
#include <assert.h>
#include "PSPrivate.h"

// regular attributes;

#include <Go/Tools/TextBrush.h>
#include <Go/Tools/Brush.h>

// shared attributes used;

#include <Go/Tools/ShFont.h>
#include <Go/Tools/ShColor.h>

// others tools;
#include <Go/Tools/Medium.h>
#include <Go/Tools/Error.h>
#include <Go/Tools/StringAssoc.h>
#include <Go/Tools/Alloc.h>
#include <Go/Tools/FilePath.h>

// for metrics
#include <stdio.h>
#include <ctype.h>

extern "C"
{
#include "parseAFM.h"
}

struct FontAssoc
{
    int       fontNumber;
    int       base_unit;
    GAssoc*   pairAssoc;
    FontInfo* fontInfo;
    GBool     defined :1,
              newEncoding :1,
              valid :1;
    
};

#define MAX_METRIC_NAME_SZ 128

void MakePSFamily(const char* family, char* s)
{
    if (family)
    {
	if (strcmp(family, "helvetica") == 0 ||
	    strcmp(family, "times")     == 0 ||
	    strcmp(family, "courier")   == 0 ||
	    strcmp(family, "lucida")    == 0)
	{
	    strcpy(s, family);
	    s[0] = toupper(s[0]);
	}
	else if (strcmp(family, "new century schoolbook") == 0)
	    strcpy(s, "NewCenturySchlbk");
	else if (strncmp(family, "lucida", 6) == 0)
	{
	    strcpy(s, family);
	    s[0] = toupper(s[0]);
	    s[6] = toupper(s[6]);
	}
	else
	{
	    GWarning ("Sorry %s MakePSFamily does not know the family: %s %s",
		     "PSGoInterf::", family,
		     "It assumes your name is the PostScript name");
	    strcpy(s, family);
	    s[0] = toupper(s[0]);
	}
    } else
	strcpy(s, "Courier");
}

static GBool MakePSWeight(const char* weight, char* s)
{
    if (weight && strcmp (weight, "medium") != 0)
    {
	sprintf(s, "-%s", weight);
	s[1] = toupper(s[1]);
	return GTRUE;
    }
    return GFALSE;
}

static void MakePSSlant(const char* slant, char* s, GBool w_set)
{
    if (slant) 
	if (*slant == 'o' || *slant == 'O')
	{
	    if (!w_set) 
		*s++ = '-';
	    sprintf(s, "Oblique");
	}
	else if (*slant == 'i' || *slant == 'I')
	{
	    if (!w_set) 
		*s++ = '-';
	    sprintf(s, "Italic");
	}
}

class translateObj
{
  public:
    virtual int toPsEncoding(const char* name) = NULL;
  private:
    // some compilo do not accept empty classes;
    char unused;
};

class iso8859_1 : public translateObj
{
  public:
    virtual int toPsEncoding(const char* name);
};


// the table begins by the 1er which differs;
// code is 161 dec;
    
const char* isoName_1[] =
{
"exclamdown", "cent", "sterling", "currency", "yen", "brokenbar", "section", "dieresis", 
"copyright", "ordfeminine", "guillemotleft", "logicalnot", "hyphen", "registered", 
"macron", "degree", "plusminus", "twosuperior", "threesuperior", "acute", "mu", 
"paragraph", "periodcentered", "cedilla", "onesuperior", "ordmasculine", 
"guillemotright", "onequarter", "onehalf", "threequarters", "questiondown",
"Agrave", "Aacute", "Acircumflex", "Atilde", "Adieresis", "Aring", "AE", 
"Ccedilla", "Egrave", "Eacute", "Ecircumflex", "Edieresis", 
"Igrave", "Iacute", "Icircumflex", "Idieresis", "Eth", "Ntilde", 
"Ograve", "Oacute", "Ocircumflex", "Otilde", "Odieresis", "multiply", "Oslash", 
"Ugrave", "Uacute", "Ucircumflex", "Udieresis", "Yacute", "Thorn",
"germandbls", "agrave", "aacute", "acircumflex", "atilde", "adieresis", "aring", "ae", 
"ccedilla", "egrave", "eacute", "ecircumflex", "edieresis", 
"igrave", "iacute", "icircumflex", "idieresis", "eth", "ntilde", 
"ograve", "oacute", "ocircumflex", "otilde", "odieresis", "divide", "oslash", 
"ugrave", "uacute", "ucircumflex", "udieresis", "yacute", "thorn", "ydieresis",
NULL,
};

int iso8859_1::toPsEncoding(const char* name)
{
    char** isoname = (char**)isoName_1;
    for (int i = 161; *isoname; i++,isoname++)
	if (strcmp (name, *isoname) == 0) 
	    return i;
    
    // the name is not in the current encoding sheme: here "iso8859_1".
    return -1;
}

#define ISO8859_1 0


translateObj* translateTable[] =
{
    NULL,
};


static GBool MakeCharMetricsEncoding(const char* charset, const char* charcode,
				    CharMetricInfo* cmi, int nmetrics)
{
    if (strcmp (charset, "iso8859") == 0)
    {
	if (strcmp(charcode, "1") == 0)
	{
	    if (translateTable[ISO8859_1] == NULL)
		translateTable[ISO8859_1] = new iso8859_1();
	    for (int i = 0; i < nmetrics; i++, cmi++)
		if (cmi->code > 161 || cmi->code < 0)
		    cmi->code = translateTable[ISO8859_1]->toPsEncoding(cmi->name);
	
	    return GTRUE;
	}
	GWarning("Sorry : MakePSFont() in PSGoInterf.cc: unknown %s: %s", "charcode", charcode);
	return GFALSE;
    }

    if (strcmp (charset, "adobe") == 0)
	return GFALSE;
    
    GWarning("Sorry : MakePSFont() in PSGoInterf.cc: unknown %s: %s", "charset", charset);
    return GFALSE;
}

/***
  FONT --------
  *************
  **/

#define GET_PAIR_IDENT(c1,c2) (void*)((int) ((uchar)(c1))*256 + (int)((uchar)(c2)))

/**
FUNCTION
    OpenMetric
DESCRIPTION
    first open the data base (if needed).
    the data base make the association between a Name (eg Courier-Bold)
    and the associated metric file;
    data base line syntax:      "metricname: filename"
**/

struct MetricAssoc
{
    char* filename;
    FILE* file;
};


static GStringAssoc* assocMetric = (GStringAssoc*)0;

static void CloseMetric(const char* metric)
{
    int count;
    MetricAssoc* ma = (MetricAssoc*)assocMetric->get(metric, count);
    assert (ma && count == 1);

    fclose(ma->file);
    ma->file = (FILE*)0;

    // ma->filename will be reopen if needed ;
}
	
static FILE* OpenMetric(const char* metric)
{
    if (assocMetric == (GStringAssoc*)-1)
	return NULL;

    if (assocMetric == (GStringAssoc*)0)
    {
	// open data base;
	char* filename = GGetFilePath(FONT_PATH, METRIC_DATA_BASE);
	FILE* fp;
	if (filename == NULL ||
	    (fp = fopen(filename, "r")) == NULL)
	{
	    // it seens like an error: so print warning is any case;
	    
	    int save_trace_level = GTraceLevel();
	    GTraceLevel() = 1;
	    GWarning ("PSGoInterf cannot open %s", METRIC_DATA_BASE);
	    GTraceLevel() = save_trace_level;

	    assocMetric = (GStringAssoc*)-1;
	    return NULL;
	}

	// load the data base;
	char line[128];
	while (fgets(line, sizeof(line), fp))
	{
	    char* s = line;
	    int  lg = strlen(line);
	    if (lg <= 2)
		continue;
	    if (line[lg-1] == '\n')
		line[--lg] = '\0';
	    for (int i = 0; i < lg; i++, s++)
	    {
		if (*s == '#')  // it is a comment
		    break;
		if (*s == ':' && i < lg - 2)
		{
		    s++; while(isspace(*s)) s++;
		    
		    MetricAssoc _ma;
		    _ma.filename = GGetFilePath(FONT_PATH, s);
		    _ma.file = (FILE*)0;
		    if (filename == NULL)
		    {
			GWarning("cannnot acces metric %s", s);
			break;
		    }
		    line[i] = '\0';

		    // make the association;
		    if (assocMetric == NULL)
			assocMetric = new GStringAssoc;
		    assocMetric->set(line, sizeof(_ma), &_ma);
		    break;
		}
	    }
	}

	// close the metric database;
	fclose(fp);
    }

    if (assocMetric == NULL)
	return NULL;

    // data base is loaded yet;
    int count;
    MetricAssoc* ma = (MetricAssoc*)assocMetric->get(metric, count);

    if (ma == NULL)
    {
	GWarning("no such metric \"%s\" in data base", metric);
	return NULL;
    }

    if (ma->file == (FILE*)-1) // an error was detected yet
	return NULL;

    if (ma->file == (FILE*)0)
	ma->file = fopen(ma->filename, "r");

    if (ma->file == NULL)
    {
	GWarning("cannot open metric file \"%s\"", ma->filename);
	ma->file = (FILE*)-1;
	return NULL;
    }
    return ma->file;
}
    
static void MakePSFont(const GShFont* shfont, FontAssoc* pf)
{
    ulong mask = shfont->mask();
    char  metricName[MAX_METRIC_NAME_SZ];

    /**
      not defined in the output
      **/
    pf->defined = GFALSE;
    pf->valid   = GFALSE;
    pf->newEncoding = GFALSE;
    pf->fontInfo = NULL;
    pf->base_unit = 0;
    
    /**
      make name
      **/
    MakePSFamily (mask & GFONT_FAMILY ? shfont->family() : NULL, metricName);
    GBool  w_set = MakePSWeight (mask & GFONT_WEIGHT ? shfont->weight() : NULL,
				&metricName[strlen(metricName)] );
    MakePSSlant  (mask & GFONT_SLANT ?  shfont->slant() : NULL,
		  &metricName[strlen(metricName)], w_set);


    /**
      load the data base to translate a ps font name to a file name;
      **/
    FILE* fp = OpenMetric(metricName);
    if (fp == NULL)
    {
	GWarning("MakePSFont() in PSGoInterf.cc: cannot load %s %s",
		metricName, "courier is assumned");

	sprintf(metricName, "Courier");
	fp = OpenMetric(metricName);
	if (fp == NULL)
	{
	    GWarning("MakePSFont() in PSGoInterf.cc: cannot load %s %s",
		    metricName, "");
	    return;
	}
    }

    /**
      load the metrics
      **/
    FontInfo* fontinfo;
    switch (parseFile(fp, &fontinfo, P_P | P_G | P_M | P_W)) {
      default: // error;
	CloseMetric(metricName);
	break;
      case 0: {
	  pf->valid = GTRUE;
	  CloseMetric(metricName);
	  /**
	    encoding
	    **/
	  if (shfont->charset() &&
	      strcmp(shfont->charset(), fontinfo->gfi->encodingScheme) != 0)
	  {
	      /**
		the encoding is different
		reencode the char metrics given by the parser
		to match the wanted encoding.
		the function returns GTRUE is the encoding is ok else GFALSE;
		**/
	      pf->newEncoding =  MakeCharMetricsEncoding(shfont->charset(),
							 shfont->charcode(),
							 fontinfo->cmi,
							 fontinfo->numOfChars);
	  }
	

	  // set the base unit: generaly it is 1000;
	  // not that lly is generaly negative;

	  BBox* bbox = &fontinfo->gfi->fontBBox;
	  pf->base_unit = bbox->ury - bbox->lly;
	  
	  /**
	    NOW make hash table to acces pairs
	    **/
	  GStringAssoc assocNameCode;
	  pf->fontInfo = fontinfo;
	  pf->pairAssoc = new GAssoc();
	  PairKernData* pkd = fontinfo->pkd;

	  for (int i = 0; i < fontinfo->numOfPairs; i++, pkd++)

	      // assoc x-amount (if non null) with the pair_ident
		  
		  if (pkd->xamt != 0) {
		      /**
			get code of name1
			**/
		      int code1 = (int)assocNameCode.get (pkd->name1);
		      if (code1 == 0)
		      {
			  for (int j = 0 ; j < fontinfo->numOfChars; j++)
			      if (strcmp (fontinfo->cmi[j].name, pkd->name1) == 0)
			      {
				  code1 = fontinfo->cmi[j].code;
				  break;
			      }
			  assocNameCode.set(pkd->name1, (void*)code1);
		      }

		      int code2 = (int)assocNameCode.get (pkd->name2);
		      if (code2 == 0)
		      {
			  for (int j = 0 ; j < fontinfo->numOfChars; j++)
			      if (strcmp (fontinfo->cmi[j].name, pkd->name2) == 0)
			      {
				  code2 = fontinfo->cmi[j].code;
				  break;
			      }
			  assocNameCode.set(pkd->name2, (void*)code2);
		      }

		      pf->pairAssoc->add(GET_PAIR_IDENT(code1,code2),  (void*)pkd->xamt);
		  }
	
	  // free the unused library storage;

	  GlobalFontInfo* gfi = pf->fontInfo->gfi;
	  GFREE(gfi->afmVersion);
	
	  // we conserve gfi->fontName;
	  GFREE(gfi->fullName);
	  GFREE(gfi->familyName);
	  GFREE(gfi->weight);
	  GFREE(gfi->version);
	  GFREE(gfi->notice);
	  GFREE(gfi->encodingScheme);
	
	  // CMI;
	  CharMetricInfo *temp = pf->fontInfo->cmi;
	  Ligature *node = temp->ligs;

	  // alloc for CWI;
	  pf->fontInfo->cwi = (int*)calloc(256, sizeof(int));

	  for (i = 0; i < pf->fontInfo->numOfChars; i++, temp++)
	  {
	      if (temp->code > 0)
	      {
		  // save the char width into cwi;
		  pf->fontInfo->cwi[temp->code] = temp->wx;
	      }

	      // free ligature list;
	      for (node = temp->ligs; node != NULL; node = node->next)
	      {
		  GFREE(node->succ); node->succ = NULL;
		  GFREE(node->lig); node->lig = NULL;
	      }
                
	      GFREE(temp->name); temp->name = NULL;
	  }
            
	  // PKD;
	  pkd = fontinfo->pkd;
	  for (i = 0 ; i < pf->fontInfo->numOfPairs; i++, pkd++)
	  {
	      GFREE (pkd->name1);
	      GFREE (pkd->name2);
	  }

	  // CWI will be freed  at destruction time;
      }
    }
}




/**
FUNCTION
    _getFontAssociation(shfont)
DESCRIPTION
    returns the FontAssoc struct associated to the font.
    If no association is set it makes  and returns it.
    so the result is always non NULL.
**/

FontAssoc* PSGoInterf::_getFontAssociation(const GShFont* shfont)
{
    int npf;
    FontAssoc* pf = (FontAssoc*)_assocFont.get((void*)shfont, npf);
    FontAssoc _pf;
    
    if (pf == NULL)
    {
	// make the PS font ident;
	_pf.fontNumber = _currentFontNumber++;

	// make the PS font name, FontInfo and pairs association.
	MakePSFont(shfont, &_pf);

	// assoc the ps to the shfont;
	_assocFont.set((void*)shfont, sizeof(_pf), &_pf);

	//  get the association;
	//  we cannot return the addr os _pf because it is local;
	pf = (FontAssoc*)_assocFont.get((void*)shfont, npf);

    }
    return pf;
}

void PSGoInterf::_freeFontAssociations()
{
    int nkeys;
    void** keys = _assocFont.getKeys(nkeys);
    void** _keys = keys;
    for (int i = 0; i < nkeys; i++, keys++)
    {
	int npf;
	FontAssoc* pf = (FontAssoc*)_assocFont.get(*keys, npf);
	assert (pf && npf == 1);
	
	if (pf->valid)
	{
	    // delete fontInfo;
	    GFREE (pf->fontInfo->gfi->fontName);
	    GFREE (pf->fontInfo->gfi);
	    GFREE (pf->fontInfo->cwi);
	    GFREE (pf->fontInfo);
	}
	
	// free the association;
	_assocFont.rem(*keys);
    }
    
    // free the keys table of the font association;
    GFREE(_keys);


    // metric open by OpenMetric() are conserved;
}



/**
FUNCTION
    ::string()
DESCRIPTION
    If "brush" is non NULL, it begins drawing the background defined with the "brush"
    It continues drawing the string with the specified (font/font_height).
**/

#define TypoToMm(base_unit,typo,font_height) ((font_height)*((float)(typo)) / (base_unit))

#define PUTCHAR(c) if ((c) < 32 || (c) >= 128) {\
	    sprintf (s, "\\%o", (int)(c));\
	    _medium->mputs(s);\
	} \
	else switch((c)) {\
	  case '\\':\
	  case '(':\
	  case ')':\
	    _medium->mputc( '\\');\
	  default:\
	    _medium->mputc((c));\
	    break;\
	}

#define DIX_MILLE 10000

inline GVector& TRUNCATE(GVector& v)
{    v.x = (GNumber)((int)(v.x*DIX_MILLE))/DIX_MILLE;
    v.y = (GNumber)((int)(v.y*DIX_MILLE))/DIX_MILLE;
    return v;
}


void PSGoInterf::string	 (const GShTextBrush* sh_tbrush, const GShBrush* shbrush,
			  const GCoordSys& cs, const GPoint& origin, GNumber height,
			  int nchar, const char* string,
			  GQuality quality)
{
    if (nchar == 0)
	return;
    
    char s[LINE_SZ];

    if (shbrush)
    {
	GNumber width = stringWidth(string, nchar, height, sh_tbrush->font());
	rectangle(NULL, shbrush, cs, origin,  width,  height,
		  quality);
    }
    
    // color;
    _setColor(sh_tbrush->color());
	
    // make font name;
    const GShFont* shfont = sh_tbrush->font();


    FontAssoc* pf = _getFontAssociation(shfont);
    if (!pf->valid)
	return;
    
    if (!pf->defined)
    {
	// define it;
	
	pf->defined = GTRUE;

	// ask ps to make the ps font with the current encoding;
	if (pf->newEncoding)
	    sprintf (s, "/f%d /%s /%s-%s-%s %s-%s %s def",

		     // /f%d gives the result font ident;
		     pf->fontNumber,
		     
		     // basename;
		     pf->fontInfo->gfi->fontName,
		     
		     // newfontname;
		     pf->fontInfo->gfi->fontName, shfont->charset(), shfont->charcode(),

		     // encoding vector;
		     shfont->charset(), shfont->charcode(),

		     // command;
		     ENCODEDEFINEFONT);
	else
	    sprintf (s, "/f%d /%s %s def", pf->fontNumber,
		     pf->fontInfo->gfi->fontName,
		     FINDFONT);
	_output(s);
    }
    
    GCoordSys _cs = _psMatrix*cs;
    GNumber   _factor = height * 1000 / pf->base_unit;

    GVector I;  I = TRUNCATE(_cs.I * _factor);
    GVector J;  J = TRUNCATE(_cs.J * _factor);

    if (_current.shfont != shfont || _current.shfont_I != I || _current.shfont_J != J)
    {
	if (_current.shfont)
	    _current.shfont->destroy();
	_current.shfont = shfont->dup();
	_current.shfont_I = I;
	_current.shfont_J = J;
	
        // make the font matrix with a J miror;
	sprintf (s, "f%d [%.4f %.4f %.4f %.4f 0 0] %s", pf->fontNumber,
		 I.x, I.y, -J.x, -J.y, MAKESETFONT);
	_output(s);
    };
    

    // origin;
    GPoint absp(_cs.to(origin + GPoint(0, TypoToMm(pf->base_unit, pf->fontInfo->gfi->fontBBox.ury,
						 height))));
    sprintf (s, "%.2f %.2f %s", absp.x, absp.y, MOVETO);
    _output(s);

    // string: code it;

    uchar* src= (uchar*)string;

    // open a show command;
    _medium->mputc('(');

    for (int i = 0 ; i < nchar-1; i++, src++)
    {
	uchar c1 = *src;
	uchar c2 = *(src+1);
	PUTCHAR(c1);
	int kern = (int)pf->pairAssoc->get(GET_PAIR_IDENT(c1,c2));
	if (kern)
	{
	    // close the current show command;
	    _medium->mputs(") ");
	    _output(SHOW);
		
	    // displace to handle kerning and open a new show command;
	    GVector delta = _cs.to(GVector(TypoToMm(pf->base_unit, kern,height),0));
	    sprintf (s, "%.2f %.2f %s\n(", delta.x, delta.y, RMOVETO);
	    _medium->mputs(s);
	}
    }
    
    // handle the last char and close the show command;
    
    PUTCHAR(*src);
    _medium->mputs(") ");
    _output(SHOW);
    _output(NEWPATH);
}


/**
  FONT QUERY METHODS
  ******************
  **/

/**
FUNCTION
    ::fontName()
DESCRIPTION
    returns the effectif font_name used to write with the specified (font/font_height)
**/
const  char* PSGoInterf::fontName      (const GCoordSys& /* cs */,
					GNumber          /* font_height */,
					const GShFont* shfont)
{
    FontAssoc* pf = _getFontAssociation(shfont);
    if (!pf->valid)
	return NULL;
    return pf->fontInfo->gfi->fontName;
}

/**
FUNCTION
    GFont metric
DESCRIPTION
    the following functions returns metrics of the font.
    Note that all the GNumber (input parameter or result) are relatives
    to the coordinate system "cs".
**/

GNumber PSGoInterf::stringWidth        (const char* string,
					int str_len, GNumber font_height,
					const GShFont* shfont)
{
    FontAssoc* pf = _getFontAssociation(shfont);
    if (!pf->valid)
	return 0;
    
    int width = 0;
    int sz = str_len / 2;
    uchar* s = (uchar*)string;
    uchar c1,c2;
    
    // handle even pairs + chars width;
    
    for (int i = 0 ; i < sz; i++)
    {
	c1 = *s++;
	c2 = *s++;
	width += pf->fontInfo->cwi[c1];
	width += pf->fontInfo->cwi[c2];
	width += (int)pf->pairAssoc->get(GET_PAIR_IDENT(c1,c2));
    }
    
    // handle odd pairs;
    
    s = (uchar*)(string + 1);
    sz = (str_len-1)/2;
    for (i = 0 ; i < sz; i++)
    {
	c1 = *s++;
	c2 = *s++;
	width += (int)pf->pairAssoc->get(GET_PAIR_IDENT(c1,c2));
    }

    // add the last char width is needed;
    
    if (str_len & 1)
	width += pf->fontInfo->cwi[string[str_len-1]];

    return TypoToMm(pf->base_unit, width, font_height);
}    

GNumber PSGoInterf::fontAscent (GNumber font_height,
				const GShFont* shfont)
{
    FontAssoc* pf = _getFontAssociation(shfont);
    if (!pf->valid)
	return 0;
    return  TypoToMm(pf->base_unit, pf->fontInfo->gfi->ascender, font_height);
}

GNumber PSGoInterf::fontDescent   (GNumber font_height,
				   const GShFont* shfont)
{
    FontAssoc* pf = _getFontAssociation(shfont);
    if (!pf->valid)
	return 0;
    return  -TypoToMm(pf->base_unit, pf->fontInfo->gfi->descender, font_height);
}


GNumber PSGoInterf::fontUnderlineThickness (GNumber font_height,
					    const GShFont* shfont)
{
    FontAssoc* pf = _getFontAssociation(shfont);
    if (!pf->valid)
	return 0;
    return TypoToMm(pf->base_unit, pf->fontInfo->gfi->underlineThickness, font_height);
}
    
GNumber PSGoInterf::fontUnderlinePosition  (GNumber font_height,
					    const GShFont* shfont)
{
    FontAssoc* pf = _getFontAssociation(shfont);
    if (!pf->valid)
	return 0;
    return TypoToMm(pf->base_unit, pf->fontInfo->gfi->underlinePosition, font_height);
}

GNumber PSGoInterf::fontItalicAngle        (const GShFont* shfont)
{
    FontAssoc* pf = _getFontAssociation(shfont);
    if (!pf->valid)
	return 0;
    return pf->fontInfo->gfi->italicAngle;
}

GNumber PSGoInterf::fontStrikeoutAscent    (GNumber font_height,
					    const GShFont* shfont)
{
    FontAssoc* pf = _getFontAssociation(shfont);
    if (!pf->valid)
	return 0;
    return  TypoToMm(pf->base_unit, pf->fontInfo->gfi->fontBBox.ury, font_height);
}

GNumber PSGoInterf::fontStrikeoutDescent   (GNumber font_height,
					    const GShFont* shfont)
{
    FontAssoc* pf = _getFontAssociation(shfont);
    if (!pf->valid)
	return 0;
    return  -TypoToMm(pf->base_unit, pf->fontInfo->gfi->fontBBox.lly, font_height);
}

GNumber PSGoInterf::fontMinSpace         (GNumber          /* font_height*/,
					  const GShFont*   /* shfont*/)
{
    GWarning("%sfont%s%s() %s", "PSGoInterf::", "Min", "Space", "not implemented");
    return 0;
}

GNumber PSGoInterf::fontMaxSpace         (GNumber          /* font_height*/,
					  const GShFont*   /* shfont*/)
{
    GWarning("%sfont%s%s() %s", "PSGoInterf::", "Max", "Space", "not implemented");
    return 0;
}

GNumber PSGoInterf::fontNormSpace         (GNumber           /* font_height*/,
					   const GShFont*    /* shfont*/)
{
    GWarning("%sfont%s%s() %s", "PSGoInterf::", "Norm", "Space", "not implemented");
    return 0;
}

GNumber PSGoInterf::fontEndSpace         (GNumber          /* font_height*/,
					  const GShFont*   /* shfont*/)
{
    GWarning("%sfont%s%s() %s", "PSGoInterf::", "End", "Space", "not implemented");
    return 0;
}

GNumber PSGoInterf::stringLeftBearing  (const char*     /* string */,
					int             /* str_len */,
					GNumber          /* font_height */,
					const GShFont*   /* shfont */)
{
    GWarning("%sstring%s() %s", "PSGoInterf::", "LeftBearing", "not implemented");
    return 0;
}

GNumber PSGoInterf::stringRightBearing  (const char*     /* string */,
					 int             /* str_len */,
					 GNumber          /* font_height */,
					 const GShFont*   /* shfont */)
{
    GWarning("%sstring%s() %s", "PSGoInterf::", "RightBearing", "not implemented");
    return 0;
}

GNumber PSGoInterf::stringAscent  (const char*     /* string */,
				   int             /* str_len */,
				   GNumber          /* font_height */,
				   const GShFont*   /* shfont */)
{
    GWarning("%sstring%s() %s", "PSGoInterf::", "Ascent", "not implemented");
    return 0;
}

GNumber PSGoInterf::stringDescent (const char*     /* string */,
				   int             /* str_len */,
				   GNumber          /* font_height */,
				   const GShFont*   /* shfont */)
{
    GWarning("%sstring%s() %s", "PSGoInterf::", "Descent", "not implemented");
    return 0;
}


