#ifdef REV_INFO
static char rcsid[] = "$Id: PSGoInterf.cc,v 1.5 1992/12/10 14:04:01 jojic Exp $";
#endif
/*****************************************************************************
MODULE:  GoInterface
FILE:    PSGoInterf.cc
AUTHOR:  Philippe Smadja, Michel Marcuse. 
DATE:    Jul 1991

DESCRIPTION:
  see PSGoInterf.h

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#include <Go/PSGoInterf.h>
#include <assert.h>
#include "PSPrivate.h"

// reguler atributes used;

#include <Go/Tools/Brush.h>
#include <Go/Tools/Pen.h>
#include <Go/Tools/ImageBrush.h>
#include <Go/Tools/PixelArray.h>
#include <Go/Tools/Pattern.h>

// sharedobjects used;

#include <Go/Tools/ShPen.h>
#include <Go/Tools/ShBrush.h>
#include <Go/Tools/ShDash.h>
#include <Go/Tools/ShFont.h>
#include <Go/Tools/ShColor.h>
#include <Go/Tools/ShColormap.h>
#include <Go/Tools/ShPixelArray.h>

// used tools;

#include <Go/Tools/Error.h>
#include <Go/Tools/Alloc.h>
#include <Go/Tools/Medium.h>
#include <Go/Tools/FileMedium.h>
#include <Go/Tools/FilePath.h>

// for stdout;
#include <stdio.h>

#define MAX_PS_IMAGE_WIDTH 3300
#define MAX_PS_PATH        1500
#define MAX_PS_STRING_SZ   65535
#define MAX_PS_MITER_LIMIT 114.0   /** = 1 / sin(0.5 degree)
				    **/
#define MAX_PS_DASHES      11
#define MARGIN  5                 /** A4 format CCITT x/y margin in mm.
				    **/

/**************************************************************************

  1: STATIC FILE DEFINITIONS
  -------------------------
  
***************************************************************************/

INLINE int CapStyleToPSCapStyle(int capstyle)
{
    return  capstyle-1;
}

#define JoinStyleToPSJoinStyle(js) (int)((js))


INLINE float MiterLimitToPSMiterLimit(GNumber miterlimit)
{
    if (miterlimit == GNO_MITER_LIMIT)
	return MAX_PS_MITER_LIMIT;
    else return (float)miterlimit;
}



/**
FUNCTION
    _getPixelArrayAssociation(shpixelArray)
DESCRIPTION
    returns the PixelArrayAssoc struct associated to the pixelArray.
    If no association is set it makes  and returns it.
    so the result is always non NULL.
**/

struct PixelArrayAssoc
{
    int  pixelArrayNumber;
    int  refcnt;
    GBool defined;
};

PixelArrayAssoc* PSGoInterf::_getPixelArrayAssociation(const GShPixelArray* shpixelarray,
						       long refcnt)
{
    int npf;
    PixelArrayAssoc* pf = (PixelArrayAssoc*)_assocPixelArray.get((void*)shpixelarray, npf);
    PixelArrayAssoc _pf;
    
    if (pf == NULL)
    {
	// make the PS pixelArray ident;

	_pf.pixelArrayNumber = _currentPixelArrayNumber++;
	_pf.refcnt = (int)refcnt;
	_pf.defined = GFALSE;

	// assoc the ps to the shpixelarray
	_assocPixelArray.set((void*)shpixelarray, sizeof(_pf), &_pf);

	//  get the association;
	//  we cannot return the addr of _pf because it is local;
	pf = (PixelArrayAssoc*)_assocPixelArray.get((void*)shpixelarray, npf);
    }
    return pf;
}

/**
FUNCTION
    _getColormapAssociation(shcolormap)
DESCRIPTION
    returns the ColormapAssoc struct associated to the colormap.
    If no association is set it makes  and returns it.
    so the result is always non NULL.
**/

struct ColormapAssoc
{
    int  colormapNumber;
    int  refcnt;
    GBool defined;
};

ColormapAssoc* PSGoInterf::_getColormapAssociation(const GShColormap* shcolormap,
						   long refcnt)
{
    int npf;
    ColormapAssoc* pf = (ColormapAssoc*)_assocColormap.get((void*)shcolormap, npf);
    ColormapAssoc _pf;
    
    if (pf == NULL)
    {
	// make the PS colormap ident;
	_pf.colormapNumber = _currentColormapNumber++;
	_pf.refcnt = (int)refcnt;
	_pf.defined = GFALSE;
	
	// assoc the ps to the shcolormap
	_assocColormap.set((void*)shcolormap, sizeof(_pf), &_pf);

	//  get the association;
	//  we cannot return the addr of _pf because it is local;
	pf = (ColormapAssoc*)_assocColormap.get((void*)shcolormap, npf);
    }
    return pf;
}

/**
FUNCTION
    _outputPixelArrayData()
DESCRIPTION
    write onto the medium the data in hexadecimal notation.
**/

void PSGoInterf::_outputPixelArrayData(const GShPixelArray* shpixelarray,
				       GBool invert_data)
{
    const GPixelArrayValues* values = shpixelarray->values();
    int bpl = (values->width * values->bits_per_pixel + 7) / 8;
    int translation = (int)shpixelarray->bytesPerLine() - bpl;
    if (translation < 0 ) 
	GError("%simage() %s", "PSGoInterf::", "bad pixel array");

    if (values->bits_per_pixel == 1 && values->depth == 1)
    {
	// HANDLE BIT_ORDER AT GENERATION TIME (BLACK AND WHITE);

	static uchar* _rollTable = NULL;
		
	if (values->format.bit == GLSB_FIRST && _rollTable == NULL)
	{
	    // init _rollTable;
	    _rollTable = (uchar*)GMALLOC(256);
	    for (int i = 0; i < 256;)
	    {
		uchar c = 0;
		for (int j = 0 ; j < 8; j++)
		    if (i & (0x01 << j))
			c |= (0x80 >> j);
		_rollTable[i++] = c;
	    }
	}

	uchar* data = values->data + values->xoffset * values->bits_per_pixel / 8;
	int count = 0;
	char s[16];
	for (int i = 0 ; i < values->height; i++, data += translation)
	    for (int j = 0 ; j <  bpl; j++, data++)
	    {
		if (count++ % 30 == 0)        // all the 60 chars write a '\n'
		    _medium->mputc ('\n');

		uchar c;
		if (values->format.bit == GLSB_FIRST)
		    c = _rollTable[*data];
		else
		    c = *data;
		if (invert_data)
		    c = ~c;

		sprintf (s, "%0.2x", c);
		_medium->mputs(s);
	    }
	_medium->mputc('\n');

    }
    else
    {
	// BYTE WILL BE HANDLED BY THE INTERPRETER (COLOR IMAGE);

	// invert_data is not available;
	// bits_per_pixel % 8 != 0 not handled here;
	assert (values->bits_per_pixel % 8 == 0);

	uchar* data = values->data + values->xoffset * values->bits_per_pixel / 8;
	int count = 0;
	char s[16];
	for (int i = 0 ; i < values->height; i++, data += translation)
	    for (int j = 0 ; j <  bpl; j++, data++)
	    {
		if (count++ % 30 == 0)        // all the 60 chars write a '\n'
		    _medium->mputc ('\n');
		sprintf (s, "%0.2x", *data);
		_medium->mputs(s);
	    }
	_medium->mputc('\n');
    }
}
	
/**
  CURRENT CONTEXT CONSTRUCTOR/DESTRUCTOR
  **************************************
  **/
GCurrentPSContext::GCurrentPSContext ()
{
    GRgb rgb;
    rgb.red = rgb.green = rgb.blue = 0;
    
    lineWidth  = 1;
    capStyle   = GCAP_BUTT;
    joinStyle  = GJOIN_MITER;
    miterLimit = 10;
    shcolor    = GCreateShColor(&rgb);
    shdash     = NULL;
    shfont     = NULL;
    shfont_I.x   = 0;
    shfont_I.y   = 0;
    shfont_J.x   = 0;
    shfont_J.y   = 0;
}

GCurrentPSContext::~GCurrentPSContext ()
{
    if (shcolor)
	shcolor->destroy();
    if (shdash)
	shdash->destroy();
    if (shfont)
	shfont->destroy();
}


/***************************************************************************

  2: PRIVATE PSGOINTERF CLASS METHODS
  ----------------------------------
  
***************************************************************************/

/**
FUNCTION
    _commonInit
DESCRIPTION
    read the prologue file and  put it onto the medium
**/

void PSGoInterf::_commonInit(GMedium* medium, GLayoutMode lm)
{
    _medium = medium;
    _firstImageCall = GTRUE;
    _currentFontNumber= _currentColormapNumber = _currentPixelArrayNumber = 0;
    
    GFileMedium prologue;
    
    char* filename = GGetFilePath(PS_PATH, PROLOGUE_FILE);

    if (filename == NULL || prologue.open(filename, "r") != 0)
    {
	// it seems like an error;
	int save_trace_level = GTraceLevel();
	GTraceLevel() = 1;
	GWarning ("PSGoInterf cannot open %s", PROLOGUE_FILE);
	GTraceLevel() = save_trace_level;
    }
    else
    {
	GFREE(filename);

	// put it onto the medium

	    char buffer[BUFSIZ];
	long n;
	while ( (n=prologue.read(buffer, BUFSIZ)) > 0)
	    if (_medium->write(buffer, n) != n)
	    {
		GWarning ("PSGoInterf cannot write prologue onto the medium");
		return;
	    }
	_output(INIT);
    }


    if ((_layoutMode = lm) == GPORTRAIT) 
    {
	_width  = 210;
	_height = 297;
	_psMatrix = GCoordSys(GPoint(-MARGIN,_height-MARGIN), GVector(1,0), GVector(0,-1));
    }
    else
    {
	// LANDSCAPE;
	_width =  297;
	_height = 210;
	_psMatrix = GCoordSys(GPoint(-MARGIN,-MARGIN), GVector(0,1), GVector(1,0));
    }
}

    
/**
FUNCTION
    _output
DESCRIPTION
    write the string paramter onto the medium.
    It adds a '\n'.
**/
void PSGoInterf::_output(const char* s, GBool eol /* = GTRUE */)
{
    _medium->mputs(s);
    if (eol)
	_medium->mputc('\n');
}
	
/**
FUNCTION
    ::setOutput( filename, type)
DESCRIPTION
    try to open the file with the "openwrite_type".
    The returns GTRUE if OK else GFALSE.
**/

GBool PSGoInterf::setOutput(const char* /* filename */,
			   const char* /* openwrite_type */)
{
    return GFALSE;
}

/**
FUNCTION
    ::setOutput( medium)
DESCRIPTION
    the GMedium is considered as available on write mode.
**/
void PSGoInterf::setOutput(GMedium* medium)
{
    if (_myMedium)
	delete _medium;
    _medium = medium;
    _myMedium = GFALSE;
}



/**
FUNCTION
    _display (const GShPen* pen, const GShBrush* brush)
DESCRIPTION
    display the current path.
    The function begins to fill the path if brush is available (!= NULL)
    and continues drawing the outline if "pen" is non NULL.
**/

void PSGoInterf::_display(const GShPen* pen, const GShBrush* brush)
{
    if (brush)
	_fill(brush, pen != NULL);
    if (pen)
	_stroke(pen);
}


/**
FUNCTION
    _fill(const GShBrush*, GBool saverestore context)
DESCRIPTION
    GPattern to be implemented.
**/
void PSGoInterf::_fill(const GShBrush* shbrush, GBool saverestore)
{
    _setColor(shbrush->color()); // before save;
    
    if (saverestore)
	_output(GSAVE);
    
    if (shbrush->fillRule() != GEVEN_ODD_RULE)
	_output(FILL) ;
    else 
	_output(EOFILL) ;

    if (saverestore)
	_output(GRESTORE);
}

/**
FUNCTION
    _stroke the path with a shpen.
DESCRIPTION
    
**/
void PSGoInterf::_stroke(const GShPen* shpen)
{
    char s[LINE_SZ];

    /* capstyle */
    if (_current.capStyle != shpen->capStyle()) {
	sprintf (s, "%d %s", CapStyleToPSCapStyle(shpen->capStyle()), SETLINECAP);
	_output (s);
	_current.capStyle = shpen->capStyle();
    }

    /* joinstyle*/
    if (_current.joinStyle != shpen->joinStyle()) {
	sprintf (s, "%d %s", JoinStyleToPSJoinStyle(shpen->joinStyle()), SETLINEJOIN);
	_output (s);
	_current.joinStyle = shpen->joinStyle();
    }

    /* miterlimit */
    float  miterlimit;
    if (shpen->mask() & GMITER_LIMIT)
	miterlimit = shpen->miterLimit();
    else
	miterlimit = GNO_MITER_LIMIT;
    
     if (_current.miterLimit != miterlimit) {
	sprintf (s, "%.2f %s",  MiterLimitToPSMiterLimit(miterlimit),  SETMITERLIMIT);
	_output (s);
	_current.miterLimit = miterlimit;
    }

    if (_current.lineWidth != shpen->lineWidth()) {
	sprintf (s, "%.2f %s", shpen->lineWidth(),  SETLINEWIDTH );
	_output (s);
	_current.lineWidth = shpen->lineWidth() ;
    }

    if (shpen->lineStyle() == GLINE_SOLID || (shpen->mask() & GDASH) == 0)
    {
	// no dash;
	if (_current.shdash)
	{
	    _current.shdash->destroy();
	    _current.shdash = NULL;
	    _setDash(NULL, 0, 0);
	}
	_setColor(shpen->color());
	_output(STROKE);
	return;
    }
    /**
      dash handling
      **/
    if (_current.shdash) 
	_current.shdash->destroy();

    _current.shdash = shpen->dash()->dup();
    const GShDash* shdash = _current.shdash;

    if (shpen->lineStyle() == GLINE_DOUBLE_DASH)
    {
#ifdef NO_ALLOCA
	GNumber* dashlist = (GNumber*)GMALLOC(sizeof(GNumber)*_current.shdash->dash_number());
#else
	GNumber* dashlist = (GNumber*)alloca(sizeof(GNumber)*_current.shdash->dash_number());
#endif
	dashlist[0] = shdash->dash_list()[shdash->dash_number() - 1];

	for (int i = 1 ; i < shdash->dash_number(); i++)
	    dashlist[i] = shdash->dash_list()[i-1];

	_setDash(dashlist, 
		 shdash->dash_number(),
		 shdash->dash_offset() + dashlist[0]);
	_setColor(shpen->altColor());
	_output(GSAVE);
	_output(STROKE);
	_output(GRESTORE);
#ifdef NO_ALLOCA
	GFREE(dashlist);
#endif
    }

    _setDash(_current.shdash->dash_list(),
	     _current.shdash->dash_number(),
	     _current.shdash->dash_offset());
    
    _setColor(shpen->color());
    _output(STROKE);
}

/**
FUNCTION
    _setColor (const GShColor*)
DESCRIPTION
    does not handle the halftone.
**/
void PSGoInterf::_setColor(const GShColor* shcolor)
{
    if (_current.shcolor != shcolor)
    {
	char s[LINE_SZ];
	_current.shcolor->destroy();
	_current.shcolor = shcolor->dup();
	if (shcolor->mask() & GCOLOR_RGB)
	{
	    sprintf (s, "%f %f %f %s", (float)(shcolor->rgb()->red) / GMAX_RGB_VALUE,
		     (float)(shcolor->rgb()->green) / GMAX_RGB_VALUE,
		     (float)(shcolor->rgb()->blue) / GMAX_RGB_VALUE,
		     SETRGBCOLOR);
	}
	else if (shcolor->mask() & GCOLOR_GRAY) 
	    sprintf (s, "%f %s", (float)(shcolor->gray()) / 256, SETGRAY);
	else
	{
	    sprintf (s, "%d %s", 0, SETGRAY);
	    GWarning("%s%s", "PSGoInterf::", "_setColor() bad color black is assumed");
	}
	_output(s);
    }
}

/**
FUNCTION
    _setDash ....
DESCRIPTION
    
**/
void PSGoInterf::_setDash(const GNumber* l, int n, GNumber offset)
{
    if (n > MAX_PS_DASHES)
	GWarning("%s%s %d %s", "PSGoInterf::", "_setDash() dash list is truncated to",
		MAX_PS_DASHES, " elements");
    char s[LINE_SZ];
    strcpy(s, "[");
    for (int i = 0 ; i < n && i < MAX_PS_DASHES; i++) 
	sprintf (&s[strlen(s)], " %.2f", l[i]);
    strcat(s, "]");
    sprintf(&s[strlen(s)], " %.2f %s", offset, SETDASH);
    _output(s);
}
		 
/**
FUNCTION
    _regToPath( const GRegion& reg)
DESCRIPTION
    make a PS path from GRegion.
**/
void PSGoInterf::_regToPath( const GRegion& reg)
{
    for (int i = 0 ; i < reg.numboxes(); i++)
	_makeBoxePath( reg.box(i));
}

void PSGoInterf::_makeBoxePath(const GBox& box)
{
    char s[LINE_SZ];
    sprintf(s, "%.2f %.2f %.2f %.2f %s", box.xmin, box.ymin,
	    box.width(), box.height(), RECTANGLE);
    _output(s);
}

/**
FUNCTION
    _makePolyPath
DESCRIPTION
    
**/
void PSGoInterf::_makePolyPath(const GCoordSys& cs, const GPoint* pts, int npt, GBool closed,
			       GBool path_will_be_duplicated)
{
    char s[LINE_SZ];
    GCoordSys _cs = _psMatrix*cs;

    GPoint absp(_cs.to(pts[0]));

    sprintf(s, "%.2f %.2f %s", absp.x, absp.y,  MOVETO);
    _output(s);

    int max_path = MAX_PS_PATH;
    if (path_will_be_duplicated)
	max_path /= 2;

    if (npt >= max_path)
    {
	npt = max_path - (int)closed;
	GWarning ("%s_makePolyPath(): number of segment truncated to %d",
		 "PSGoInterf::", npt);
    }

    for (int i = 1; i < npt; i++)
    {
	absp = _cs.to(pts[i]);
	sprintf(s, "%.2f %.2f %s", absp.x, absp.y,  LINETO);
	_output(s);
    }
    if (closed)
	_output(CLOSEPATH);
}


/*****************************************************************************

  3: VIRTUAL PUBLIC METHODS.
  -------------------------
  
********************************************************************************/

PSGoInterf::PSGoInterf(GLayoutMode lm)
{
    // the PS constructor with no parameter uses stdout;

    GMedium* medium = new GFileMedium(stdout);
    _commonInit(medium, lm);
    _myMedium = GTRUE;
}

PSGoInterf::PSGoInterf(const char* filename, const char* openwrite_type, GLayoutMode lm)
{
    GMedium* medium = new GFileMedium(filename, openwrite_type);
    _commonInit(medium, lm);
    _myMedium = GTRUE;
}

PSGoInterf::PSGoInterf(GMedium* medium, GLayoutMode lm)
{
    _commonInit(medium, lm);
    _myMedium = GFALSE;
}

/**
FUNCTION
    destructor
DESCRIPTION
    If the medium is a GFileMedium it flushes it.
    delete the PS Resources if needed.
**/
PSGoInterf::~PSGoInterf()
{
    _output(SHOWPAGE);
    _output(END_OF_PROGRAM);
    if (_myMedium) 
	delete _medium;
    
    // free the resources used to handle fonts;
    _freeFontAssociations();
}

/**
FUNCTION
    ::image()
DESCRIPTION
    It draws the Image describe if the GPixelArray with GShImageBrush parameter.
    Not that is the "stipple" field is set, it is resize/deforma" to match
    the image geometries.
**/

#define PSRGB(v) ((v)*256/(((ulong)GMAX_RGB_VALUE)+1))

void PSGoInterf::image	 (const GShImageBrush* ib,
			  const GCoordSys& cs, const GPoint& origin,
			  GNumber width, GNumber height,
			  const GShPixelArray* shpixelarray, GQuality /* quality */)
{
    if (ib->colormap() == NULL)
    {
	GWarning("%simage() %s", "PSGoInterf::", "with no colormap does nothing");
	return;
    }
    
    if (ib->mask() & GSTIPPLE &&
	ib->fillStyle() == GFILL_STIPPLED)
    {
	GWarning("%simage() %s", "PSGoInterf::",
		 "Sorry stippled image %s yet", "not implemented");
	return ;
    }
    

    ColormapAssoc* pc = NULL;
    GBool invert_pixelarray_data = GFALSE;
    
    char s[LINE_SZ];

    if (shpixelarray->values()->width > MAX_PS_IMAGE_WIDTH)
	GWarning("%simage %s: %d",  "PSGoInterf::",
		 "image() some interpreters do not accept image width more than",
		 MAX_PS_IMAGE_WIDTH);

    // IS IT BLACK AND WHITE IMAGE;
    
    if (ib->colormap()->nColor() == 2 && shpixelarray->values()->bits_per_pixel == 1 &&
	shpixelarray->values()->depth == 1)
    {
	GRgb black(0,0,0);
	GRgb white(GMAX_RGB_VALUE,GMAX_RGB_VALUE,GMAX_RGB_VALUE);
	const GShColor** colors = ib->colormap()->colors();

	if (*colors[0]->rgb() == white && *colors[1]->rgb() == black)
	    invert_pixelarray_data = GTRUE;
	else if (*colors[0]->rgb() != black || *colors[1]->rgb() != white)
	{
	    GWarning("%simage() %s", "PSGoInterf::",
		     "Depth 1 image with not black and white colormap: bw is assumed");
	}

	// continue by handling the pixelarray;
	
    }
    else if (shpixelarray->values()->bits_per_pixel % 8 != 0)
    {
	GWarning("%simage() %s", "PSGoInterf::",
		 "Sorry bit_per_pixel != 1 and bit_per_pixel % 8 != 0 not implemented yet");
	return;
    }
    else
    {
	// USE COLORMAP;

	if (_firstImageCall)
	{
	    _firstImageCall = GFALSE;
	    _medium->mputc('\n');

	    // put inside the medium the image function definition;
	    // The function is the file PS_image.pst;

	    GFileMedium tmp;
	    char* filename = GGetFilePath(PS_PATH, IMAGE_FILE);
	
	    if (filename == NULL || tmp.open(filename, "r") != 0)
	    {	
		// it seems like an error;
		int save_trace_level = GTraceLevel();
		GTraceLevel() = 1;
		GWarning ("PSGoInterf cannot open  file: %s", IMAGE_FILE);
		GTraceLevel() = save_trace_level;
	    }
	    else
	    {
		GFREE(filename);
		// put it onto the medium;

		char buffer[BUFSIZ];
		long n;
		while ( (n=tmp.read(buffer, BUFSIZ)) > 0)
		    if (_medium->write(buffer, n) != n)
		    {
			GWarning ("PSGoInterf cannot write onto the medium");
			return;
		    }
	    }
	}

	// note that we use getColormapAssociation with the ib->refcnt();
	// because the display of an object is normaly done once;
    
	pc = _getColormapAssociation(ib->colormap(), ib->refcnt());
	if (!pc->defined)
	{
	    // defined it;
	    pc->defined = GTRUE;

	    // write the colormap define command onto the medium;
	    // ie:     /c1 <ff0000 00ff00> def;

	    sprintf (s, "/c%d <", pc->colormapNumber);
	    _medium->mputs(s);
	
	    const GShColor** shcolor = ib->colormap()->colors();

	    for (ulong i = 0; i < ib->colormap()->nColor(); i++, shcolor++)
	    {
		if (i % 10 == 0)      // to make a nice PS program
		    _medium->mputc('\n');
	    
		if (*shcolor == NULL)
		{
		    // print black rgb;
		    _medium->mputs("000000 ");
		}
		else
		{
		    const GRgb* rgb = (*shcolor)->rgb();
		    if (rgb == NULL)
		    {
			uchar v = (*shcolor)->gray();
			// value 0..255 like in PS;
			sprintf (s, "%0.2x%0.2x%0.2x ", v, v, v);
		    } else  sprintf (s, "%0.2x%0.2x%0.2x ",
				     PSRGB(rgb->red), PSRGB(rgb->green), PSRGB(rgb->blue));
		    _medium->mputs(s);
		}
	    }
	    // end of define command;
	    _output("> def");
	}

    }

    // HANDLE THE  PIXELARRAY;
    // ALMOST COMMON  FOR COLOR AND BLACK AND WHITE;
    
    PixelArrayAssoc* pa =  _getPixelArrayAssociation(shpixelarray, shpixelarray->refcnt());
    const GPixelArrayValues* values = shpixelarray->values();
	

    if (!pa->defined &&
	shpixelarray->refcnt() > 1  &&
	shpixelarray->dataSize() < MAX_PS_STRING_SZ)
    {
	// DEFINE PIXELARRAY IN A PS OBJECT: A STRING;
	pa->defined = GTRUE;

	// write the pixelarray define command onto the medium;
	// for color:     /p1 [ <....>  width height bits_per_pixel] def;
	// for bw:        /p1 <....> def;
	
	if (pc)  // define the pixelarray dictionnary
	    sprintf (s, "/p%d [ <", pa->pixelArrayNumber);
	else
	    sprintf(s, "/p%d <", pa->pixelArrayNumber);
	
	_medium->mputs(s);

	// write the data;
	_outputPixelArrayData(shpixelarray, invert_pixelarray_data);

	// write the end  of the command;
	if (pc)  // pixelarray definition (dictionnary)
	    sprintf (s, "> %d %d %d] def",
		     values->width, values->height, values->bits_per_pixel);
	else    // only data source definition
	    sprintf(s, "> def");
	
	_output(s);
    }

    // Now the ident of colormap and pixelarray are known;
    // compute the output matrix and write the show image command;

    GCoordSys _cs = _psMatrix*cs;
    
    GVector I = _cs.I * width;
    GVector J = _cs.J * height;
    GPoint  O = _cs.to(origin);

    if (pc == NULL)
    {
	// BLACK AND WHITE;
    
	// matrix transformation;
	sprintf(s, "%s [%.2f %.2f %.2f %.2f %.2f %.2f] %s",
		GSAVE, I.x, I.y, J.x, J.y, O.x, O.y, CONCAT);
	_output(s);

	// image unit transformation;
	sprintf(s, "%d %d 1 [%d 0 0 %d 0 0]",
		values->width, values->height,
		values->width, values->height);
	_output(s);
	
	// data source;
	if (pa->defined)
	{
	    sprintf (s, "p%d ", pa->pixelArrayNumber);
	    _output(s);
	}
	else
	{
	    _output("{<");
	    _outputPixelArrayData(shpixelarray, invert_pixelarray_data);
	    _output(">}");
	}
	// image request and restore;
	_output(IMAGE);
	_output(GRESTORE);
    }
    else
    {
	// IMAGE WITH COLORMAP;
    
	if (pa->defined)
	{
	    // we use the effective colormap;
	    sprintf (s, "p%d ", pa->pixelArrayNumber);
	    sprintf (&s[strlen(s)], "c%d [%.2f %.2f %.2f %.2f %.2f %.2f] %s",
		     pc->colormapNumber, I.x, I.y, J.x, J.y, O.x, O.y,
		     CMAPIMAGE);

	    // write command;
	    _output(s);
	}
	else
	{
	    sprintf (s, "%d %d %d ",
		     values->width, values->height, values->bits_per_pixel);
	    sprintf (&s[strlen(s)], "c%d [%.2f %.2f %.2f %.2f %.2f %.2f] %s",
		     pc->colormapNumber, I.x, I.y, J.x, J.y, O.x, O.y,
		     CMAP_CURRENTFILE_IMAGE);

	    // write command;
	    _output(s);
	
	    // write the data;
	    _outputPixelArrayData(shpixelarray, invert_pixelarray_data);
	
	}
    }

    // free the PS ident according the used refcnt;

    if (--pa->refcnt == 0 && pa->defined)
    {
	// in PS;
	sprintf(s, "/p%d %s", pa->pixelArrayNumber, UNDEFINE);
	_output(s);
	
	// in C++;
	_assocPixelArray.rem((void*)shpixelarray);
    }

    if (pc && --pc->refcnt == 0)
    {
	// in PS;
	sprintf(s, "/c%d %s", pc->colormapNumber, UNDEFINE);
	_output(s);
	
	// in C++;
	_assocColormap.rem((void*)ib->colormap());
    }
}

/**
FUNCTION
    ::clear()
DESCRIPTION
    fills the parameter region with the background color
**/

void PSGoInterf::clear          (const GRegion& reg)
{
    if (_background)
    {
	_regToPath(reg);
	_setColor(_background->shColor());
	_output(FILL);
    }
}

/**
FUNCTION
    ::polyline()
DESCRIPTION
    As describe in the GoInterf specifications
    The function begins to fill the polyline if brush is available (!= NULL)
    and continues drawing the outline if "pen" is non NULL.
**/
void PSGoInterf::polyline	(const GShPen* pen, const GShBrush* brush,
				 const GCoordSys& cs, int np,
				 const GPoint* pts, GQuality)
{
    _makePolyPath(cs, pts, np, GFALSE,
		  pen && brush || pen && pen->lineStyle() == GLINE_DOUBLE_DASH);
    _display(pen, brush);
}

/**
FUNCTION
    ::polygon()
DESCRIPTION
    As describe in the GoInterf specifications
    The function begins to fill the polygon if brush is available (!= NULL)
    and continues drawing the outline if "pen" is non NULL.
**/
void PSGoInterf::polygon   (const GShPen* pen, const GShBrush* brush,
			    const GCoordSys& cs, int np, const GPoint* pts,
			    GQuality /* quality */)
{
    _makePolyPath(cs, pts, np, GTRUE,
		  pen && brush || pen && pen->lineStyle() == GLINE_DOUBLE_DASH);
    _display(pen, brush);
}


/**
FUNCTION
    ellipse (non closed, chord, pie)
DESCRIPTION
    asks PS to make arc path and _display it
**/
#define RadianToPSArcAngle(r) ((int)((r)*180/PI))

void PSGoInterf::ellipse (const GShPen* pen, const GShBrush* brush,
			  const GCoordSys& cs, const GPoint& origin,
			  GNumber radiusX, GNumber radiusY,
			  GNumber angle1, GNumber angle2, GQuality /* quality */)
{
    GCoordSys _cs = _psMatrix*cs;
    _cs.O = _cs.to(origin);
    _cs.I *= radiusX;
    _cs.J *= radiusY;
    char s[LINE_SZ];
    sprintf (s, "%d %d [%.2f %.2f %.2f %.2f %.2f %.2f] %s",
	     RadianToPSArcAngle(angle1),  RadianToPSArcAngle(angle2),
	     _cs.I.x, _cs.I.y, _cs.J.x, _cs.J.y, _cs.O.x, _cs.O.y,
	     ELLIPSE);
    _output(s);

    _display(pen, brush);
}

void PSGoInterf::ellipseChord (const GShPen* pen, const GShBrush* brush,
			       const GCoordSys& cs, const GPoint& origin,
			       GNumber radiusX, GNumber radiusY,
			       GNumber angle1, GNumber angle2, GBool closed,
			       GQuality quality)
{
    if (!closed && pen != NULL && angle2 - angle1 < 2*PI)
    {
	// could be optimized ;
      GoInterf::ellipseChord(pen, brush, cs, origin, radiusX, radiusY,
			     angle1, angle2, closed, quality);
	return;
    }

    // closed || pen == NULL || delta_angle > = 2*PI;
    // the behavior is like closed;
    
    GCoordSys _cs = _psMatrix*cs;
    _cs.O = _cs.to(origin);
    _cs.I *= radiusX;
    _cs.J *= radiusY;
    char s[LINE_SZ];
    sprintf (s, "%d %d [%.2f %.2f %.2f %.2f %.2f %.2f] %s",
	     RadianToPSArcAngle(angle1),  RadianToPSArcAngle(angle2),
	     _cs.I.x, _cs.I.y, _cs.J.x, _cs.J.y, _cs.O.x, _cs.O.y,
	     ELLIPSE_CHORD);
    _output(s);

    _display(pen, brush);
}

void PSGoInterf::ellipsePie (const GShPen* pen, const GShBrush* brush,
			     const GCoordSys& cs, const GPoint& origin,
			     GNumber radiusX, GNumber radiusY,
			     GNumber angle1, GNumber angle2, GBool closed,
			     GQuality quality)
{
    if (!closed && pen != NULL && angle2 - angle1 < 2*PI)
    {
	// could be optimized ;
      GoInterf::ellipsePie(pen, brush, cs, origin, radiusX, radiusY,
			   angle1, angle2, closed, quality);
	return;
    }
 
    // closed || pen == NULL || delta_angle > = 2*PI;
    // the behavior is like closed;

    GCoordSys _cs = _psMatrix*cs;
    _cs.O = _cs.to(origin);
    _cs.I *= radiusX;
    _cs.J *= radiusY;
    char s[LINE_SZ];
    sprintf (s, "%d %d [%.2f %.2f %.2f %.2f %.2f %.2f] %s",
	     RadianToPSArcAngle(angle1),  RadianToPSArcAngle(angle2),
	     _cs.I.x, _cs.I.y, _cs.J.x, _cs.J.y, _cs.O.x, _cs.O.y,
	     ELLIPSE_PIE);
    _output(s);
    _display(pen, brush);
}


