
#ifdef REV_INFO
static char rcsid[] = "$Id: XFont.cc,v 1.11 1992/09/25 15:45:15 smadja Exp $";
#endif
/*****************************************************************************
MODULE:  XINTERFACE
FILE:    XFont.cc
AUTHOR:  Philippe Smadja
DATE:    Nov 90

DESCRIPTION:
         This file contains:
	 1: the global functions for association.
	 2: the XGoInterf methods relatives to font.
	 3: the ShXFont methods  of the ShXFont object

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/


#include <Go/XGoInterf.h>
#include "XPrivate.h"
#include "XFontP.h"
#include <assert.h>

#include <Go/Tools/LengthUnit.h>
#include <Go/Tools/Cons.h>
#include <Go/Tools/Alloc.h>
#include <Go/Tools/Array.h>
#include <Go/Tools/Error.h>
#include <Go/Tools/Atom.h>
#include <Go/Tools/StringAssoc.h>


#include <X11/Xatom.h>
#include <ctype.h>
#include <string.h>
#include <stdio.h>

// cf ex: in release 5 update (David Flanagan);
#define MAX_FONTNAME_LENGHT 500

/**
  Font ASSOC TOOLS
  ****************
  **/
struct _dFont {
    Display*   dpy;
    GAssoc* assoc;
    int    nref;
};

static GArray& DFonts() {
    static GArray *d_fonts = NULL;

    if (! d_fonts)
	d_fonts = new GArray(sizeof(struct _dFont));

    return *d_fonts;
}


void   freeAssocFont(GAssoc* assoc)
{
    struct _dFont* dfont = (struct _dFont*)DFonts().data();
    int nbDfonts = DFonts().nelems();
    
    for (int i = 0 ; i  < nbDfonts; i++, dfont++)
	if (dfont->assoc == assoc)
	{
	    if( --dfont->nref == 0)
	    {
		delete assoc;
		DFonts().rem(i);
	    }
	    return ;
	}
}


GAssoc* getAssocFont(Display* dpy)
{
    struct _dFont* dfont = (struct _dFont*)DFonts().data();
    int nbDfonts = DFonts().nelems();
    
    for (int i = 0 ; i  < nbDfonts; i++, dfont++)
	if (dfont->dpy ==  dpy)
	{
	    dfont->nref++;
	    return dfont->assoc;
	}

    struct _dFont local;
    local.dpy = dpy;
    local.nref = 1;
    local.assoc = new GAssoc();

    DFonts().add(&local);
    
    return local.assoc;
}

/**
  FONTSTRUCT ASSOC TOOLS FOR THIS FILE.
  *************************************
  **/
struct _dFontStruct {
    Display*   dpy;
    GStringAssoc* assoc;
    int    nref;
};

// the assoc will make association between an atom;
// the full_fontname : the effective one and the following struct;

struct FontStructAssoc {
    int  effective_height;
    XFontStruct* font_struct;
    int  nref;
};
    
static GArray& DFontStructs() {
    static GArray* d_fontstructs = NULL;

    if (! d_fontstructs)
	d_fontstructs = new GArray(sizeof(struct _dFontStruct));

    return *d_fontstructs;
}


static void   FreeAssocFontStruct(GStringAssoc* assoc)
{
    struct _dFontStruct* dfontStruct = (struct _dFontStruct*)DFontStructs().data();
    int nbDfontStructs = DFontStructs().nelems();
    
    for (int i = 0 ; i  < nbDfontStructs; i++, dfontStruct++)
	if (dfontStruct->assoc == assoc)
	{
	    if( --dfontStruct->nref == 0)
	    {
		delete assoc;
		DFontStructs().rem(i);
	    }
	    return ;
	}
}


static GStringAssoc* GetAssocFontStruct(Display* dpy)
{
    struct _dFontStruct* dfontStruct = (struct _dFontStruct*)DFontStructs().data();
    int nbDfontStructs = DFontStructs().nelems();
    
    for (int i = 0 ; i  < nbDfontStructs; i++, dfontStruct++)
	if (dfontStruct->dpy ==  dpy)
	{
	    dfontStruct->nref++;
	    return dfontStruct->assoc;
	}

    struct _dFontStruct local;
    local.dpy = dpy;
    local.nref = 1;
    local.assoc = new GStringAssoc();

    DFontStructs().add(&local);
    
    return local.assoc;
}

/**
  STATIC FUNCTIONS
  **/

#define PIXEL_SIZE 7
#define POINT_SIZE 8
#define AVERAGE_WIDTH 12

static GBool IsScalableFont(const char* name)
{
    // this function is a copy of code written;
    // in R5-update - programmer supplement for R5 -;
    // David Flanagan - O.Reilly and Associates, Inc -;

    int i, field;

    // if (name == NULL || name[0] != '-') return GFALSE;

    for (i = field = 0; name[i] != '\0'; i++) {
	if (name[i] == '-') {
	    field++;
	    if (field == POINT_SIZE || field == PIXEL_SIZE ||
		field == AVERAGE_WIDTH)
		if(name[i+1] != '0' || name[i+2] != '-')
		    return GFALSE;
	}
    }

    if (field != 14) return GFALSE;
    else return GTRUE;
}

    
// returns the  wanted height (PIXEL or POINT).

static int HeightIndex(const char* fontname, int num_of_minus)
{
    int minus;
    int index;
    for (index = 0, minus = 0; index < MAX_FONTNAME_LENGHT; index++)
    {
	if (*fontname++ == '-')
	    if (++minus == num_of_minus)
	    {
		if (! isdigit(*fontname))
		    goto out;
		return index+1;
	    }
    }
  out:
    return  -1;
}


/**
  HeightIndex returns the index of the  height in pixel;
  of the fontname computed by lib --> assert if error;
  **/
static int Lib_PixelSizeIndex(const char* fontname)
{
    int ret = HeightIndex(fontname, PIXEL_SIZE);
    if (ret == -1)
	GError("HeightIndex() internal error");
    return ret;
}

static int X_PixelSizeIndex(const char* fontname)
{
    int ret = HeightIndex(fontname, PIXEL_SIZE);
    if (ret == -1)
	GWarning("bad X font naming (Server bug)");
    return ret;
}    

static int X_HeightInPixels(GStringAssoc* assocFontStruct, Display* dpy, const char* font_name)
{
    int one;
    FontStructAssoc* pf = (FontStructAssoc*)assocFontStruct->get(font_name, one);

    if (!pf)
    {
	FontStructAssoc _pf;

	// load the font and get its height;
	
	_pf.font_struct = XLoadQueryFont(dpy, font_name);
	_pf.effective_height =
	    _pf.font_struct->max_bounds.ascent +  _pf.font_struct->max_bounds.descent;

	// not referenced: so the association will be removed;
	// by  ShXFont::_update() or delete().
	_pf.nref = 0;

	// unload the font: it was a tempory used for height info.
	XFreeFont(dpy, _pf.font_struct);
	_pf.font_struct = NULL;
	
	// make association between name and height;
	assocFontStruct->set(font_name, sizeof(_pf), &_pf);
	pf = &_pf;
    }

    return pf->effective_height;
}

    
struct FontNameStruct
{
     const char* registry;
     const char* family;
     const char* weight;
     const char* slant;
     const char* width;
     const char* add_style;
           char* pixel_size;
           char* point_size;
     const char* resolution_x;
     const char* resolution_y;
     const char* spacing;
     const char* average_width;
     const char* charset_registry;
     const char* charset_encoding;
};

static void setXFontName (const FontNameStruct* f_struct,
			  char* font_name) // output
{
    char* p = font_name;
    p = strcat (strcpy (p, "-"), f_struct->registry);
    p = strcat (strcat (p, "-"), f_struct->family);
    p = strcat (strcat (p, "-"), f_struct->weight);
    p = strcat (strcat (p, "-"), f_struct->slant);
    p = strcat (strcat (p, "-"), f_struct->width);
    p = strcat (strcat (p, "-"), f_struct->add_style);
    p = strcat (strcat (p, "-"), f_struct->pixel_size);
    p = strcat (strcat (p, "-"), f_struct->point_size);
    p = strcat (strcat (p, "-"), f_struct->resolution_x);
    p = strcat (strcat (p, "-"), f_struct->resolution_y);
    p = strcat (strcat (p, "-"), f_struct->spacing);
    p = strcat (strcat (p, "-"), f_struct->average_width);
    p = strcat (strcat (p, "-"), f_struct->charset_registry);
    p = strcat (strcat (p, "-"), f_struct->charset_encoding);
    return;
}

#define DEFAULT_FONT  "-misc-fixed-medium-r-semicondensed-*"

static GAtom EffectiveDefaultFont(Display* dpy)
{

    int count;
    char** flist = XListFonts(dpy, DEFAULT_FONT,
			      (int)strlen(DEFAULT_FONT), &count);
    if (flist == NULL)
	GError("XListFonts:: for %s returns NULL", DEFAULT_FONT);

    GAtom atom = GCreateAtom(flist[0]);
    XFreeFontNames(flist);
    return atom;
}

static void ShFontToXFontName (const GShFont* sh_font,
			       GNumber width, GNumber height,
			       GNumber pixelWidth, GNumber pixelHeight,
			       char* font_name)
{
    FontNameStruct f_struct;
    ulong mask = sh_font->mask();
    char pixel_size[8];
    char point_size[8];
    char resolution_x[8];
    char resolution_y[8];

    f_struct.registry         = mask & GFONT_REGISTRY ? sh_font->registry() : "";
    f_struct.family           = mask & GFONT_FAMILY ? sh_font->family() : "";
    f_struct.weight           = mask & GFONT_WEIGHT ? sh_font->weight() : "";
    f_struct.slant            = mask & GFONT_SLANT ? sh_font->slant() : "";
    f_struct.width            = mask & GFONT_WIDTH ? sh_font->width() : "";
    f_struct.add_style        = mask & GFONT_ADD_STYLE ? sh_font->addStyle() : "";
    f_struct.spacing          = mask & GFONT_SPACING ? sh_font->spacing() : "";
    f_struct.charset_registry = mask & GFONT_CHARSET ? sh_font->charset() : "";
    f_struct.charset_encoding = mask & GFONT_CHARCODE ? sh_font->charcode() : "";
    f_struct.pixel_size       = pixel_size;
    f_struct.point_size       = point_size;
    f_struct.resolution_x     = resolution_x;
    f_struct.resolution_y     = resolution_y;
    f_struct.average_width    = "*";

    sprintf (pixel_size, "%d",  (int)(height/pixelHeight + 0.5));

    GLengthUnit font_unit(GINCH, 1.0/72.27);

    float  typo;
    typo = font_unit.from(GMETER, 1.0/1000, height)*10 + 0.5;
    sprintf (point_size, "%d", (int)typo);

    float width_div_height;
    
    if (height == FOR_METRIC)
    {
	assert (width == FOR_METRIC);
	width_div_height = 1;
    } else
	width_div_height = width / height;
    
    sprintf (resolution_x, "%d", (int)(25.4/pixelWidth  * width_div_height + 0.5));
    sprintf (resolution_y, "%d", (int)(25.4/pixelHeight + 0.5));

    setXFontName(&f_struct, font_name);
}


/**
  XGOINTERF FUNCTIONS
  *******************
  **/

static void ShFontIsDestroyed(void* closure, const GShFont* shfont)
{
    GAssoc* assoc = (GAssoc*)closure;
    int npf;
    FontAssoc* pf = (FontAssoc*)assoc->get(shfont, npf);
    assert (pf);
    for (; npf; npf--, pf++) 
    {
	pf->shxfont->destroy();
	pf->xinterfs->destroy();
    }
    
    /**
      remove the association
      and the callbacks
      **/
    assoc->rem(shfont);
}

static void ShFontIsUpdated(void* assoc,
			    GShFont* shfont /* should be declare const */,
			    ulong /* mask not used */)
{
    ShFontIsDestroyed(assoc, shfont);
    /**
      remove the callbacks
      **/
    shfont->updatePostcond(ShFontIsUpdated,
			   assoc, GREMOVE);
    shfont->destroyPostcond(ShFontIsDestroyed,
			    assoc, GREMOVE);
}

/**
  Query methods for font
  **********************
  **/

/**
FUNCTION
    XGoInterf::fontHeightInPixels
DESCRIPTION
    returns the theoric height in pixel of the font;
    for the current interface.
    the name MUST have the X fontname format;
    It may returns -1 on error.
    This method is a specific method of XGoInterf.
**/

int XGoInterf::fontHeightInPixels(const char* name)
{
    GStringAssoc* assoc =  GetAssocFontStruct(_dpy);
    char fontname[MAX_FONTNAME_LENGHT];
    int len = (int)strlen(name);
    if (len >= (int)sizeof(fontname))
    {
	GWarning("XGoInterf::fontHeightInPixels() bad name");
	return 0;
    }

    // make the name in lower case;

    for (int i = 0 ; i < len; i++)
	if (isupper(fontname[i] = name[i]))
		fontname[i] = tolower(fontname[i]);
    fontname[i] = '\0';
	
    int height =  X_HeightInPixels(assoc, _dpy, fontname);
    FreeAssocFontStruct(assoc);
    return height;
}

/**
FUNCTION
    XGoInterf::xFontNameToFont()
DESCRIPTION
    return the GFont which correspond to the X fontname
    This method is a specific method of XGoInterf.
**/

/* a tool */
int NextMinus(const char* s)
{
 char c;
 for (int i=0; ;i++)
     if (c= *s++, c == '\0' || c == '-')
	 return i;
}

GFont* XGoInterf::xFontNameToFont(const char* name)
{
    GShFont* shfont = xFontNameToShFont(name);
    GFont* font = new GFont(shfont);
    shfont->destroy();
    return font;
}

GShFont* XGoInterf::xFontNameToShFont(const char* name)
{
    if (name == NULL || *name++ != '-' || strlen(name) > MAX_FONTNAME_LENGHT)
	return NULL;

    GFontValues values;
    ulong mask = 0;
    int   lg;
    char  localName[MAX_FONTNAME_LENGHT];
    char* localIndex = localName;
    char* tmp;

#define _make(field, maskBit)\
    if (lg = NextMinus(name)) {\
	tmp = localIndex;\
	localIndex += lg+1;\
	values.field = tmp;\
	strncpy(values.field, name, lg);\
	values.field[lg] = '\0';\
	mask |= maskBit;\
	name += lg;\
    } \
    if (*name++ == '\0')\
	goto _return

#define _nextField()\
	name += NextMinus(name);\
        if (*name++ == '\0')\
	    goto _return
        
    _make(registry, GFONT_REGISTRY);
    _make(family, GFONT_FAMILY);
    _make(weight, GFONT_WEIGHT);
    _make(slant, GFONT_SLANT);
    _make(width, GFONT_WIDTH);
    _make(add_style, GFONT_ADD_STYLE) ;

    _nextField() ; /* pass pixel_size */
    _nextField() ; /* pass point_size */
    _nextField() ; /* pass resolution_x */
    _nextField() ; /* pass resolution_y */

    _make(spacing, GFONT_SPACING) ;
    _nextField() ; /* pass average_width */
    _make(charset_registry, GFONT_CHARSET) ;
    _make(charset_encoding, GFONT_CHARCODE);

 _return:
    return GCreateShFont(mask, &values);

#undef _make
#undef _nextField

}

/**
FUNCTION
    ::fontName()
DESCRIPTION
   returns the effective font name used to draw.
**/
const char*  XGoInterf::fontName(const GCoordSys& cs, GNumber height,
				 const GShFont* font) 
{
    const ShXFont* shxfont = _fontToShXFont(font, cs, height);
    return shxfont->fontName();
}


#define PROPERTY_MACRO(v) ((GNumber)v) / shxfont->height() * height

// FONT metrics methods;
// value are returned relavite to cs;
// the result is the theoric one;


GNumber XGoInterf::stringLeftBearing (const char* string, int size,
				      GNumber height, const GShFont* font)
{
    if (size == 0)
	return 0;

    const ShXFont* shxfont = _fontToShXFont(font, GUnitCoordSys(), FOR_METRIC);
    int unused;
    XCharStruct overall;
    XTextExtents((XFontStruct*)shxfont->fontStruct(), string, size,
		 &unused, &unused, &unused,
		 &overall);
    return PROPERTY_MACRO(overall.lbearing);
}

GNumber XGoInterf::stringRightBearing (const char* string, int size,
				      GNumber height, const GShFont* font)
{
    if (size == 0)
	return 0;

    const ShXFont* shxfont = _fontToShXFont(font, GUnitCoordSys(), FOR_METRIC);
    int unused;
    XCharStruct overall;
    XTextExtents((XFontStruct*)shxfont->fontStruct(), string, size,
		 &unused, &unused, &unused,
		 &overall);
    return PROPERTY_MACRO(overall.rbearing);
}

GNumber XGoInterf::stringAscent (const char* string, int size,
				GNumber height, const GShFont* font) 
{
    if (size == 0)
	return 0;

    const ShXFont* shxfont = _fontToShXFont(font, GUnitCoordSys(), FOR_METRIC);
    int unused;
    XCharStruct overall;
    XTextExtents((XFontStruct*)shxfont->fontStruct(), string, size,
		 &unused, &unused, &unused,
		 &overall);
    return  PROPERTY_MACRO(overall.ascent);
}

GNumber XGoInterf::stringDescent (const char* string, int size,
				  GNumber height, const GShFont* font) 
{
    if (size == 0)
	return 0;

    const ShXFont* shxfont = _fontToShXFont(font, GUnitCoordSys(), FOR_METRIC);
    int unused;
    XCharStruct overall;
    XTextExtents((XFontStruct*)shxfont->fontStruct(), string, size,
		 &unused, &unused, &unused,
		 &overall);
    return PROPERTY_MACRO(overall.descent);
}

GNumber XGoInterf::stringWidth (const char* string, int size,
				GNumber height, const GShFont* font) 
{
    if (size == 0)
	return 0;

    const ShXFont* shxfont = _fontToShXFont(font, GUnitCoordSys(), FOR_METRIC);
    int width_in_pixels = XTextWidth((XFontStruct*)shxfont->fontStruct(), string, size);

    return PROPERTY_MACRO(width_in_pixels);
}

GNumber XGoInterf::fontAscent (GNumber height,
			       const GShFont* font) 
{
    const ShXFont* shxfont = _fontToShXFont(font, GUnitCoordSys(), FOR_METRIC);
    ulong value = shxfont->fontStruct()->ascent;
    return PROPERTY_MACRO(value);
}


GNumber XGoInterf::fontDescent (GNumber height,
				const GShFont* font) 
{
    const ShXFont* shxfont = _fontToShXFont(font, GUnitCoordSys(), FOR_METRIC);
    ulong value = shxfont->fontStruct()->descent;
    return PROPERTY_MACRO(value);
}

GNumber XGoInterf::fontUnderlineThickness (GNumber height,
					   const GShFont* font) 
{
    const ShXFont* shxfont = _fontToShXFont(font, GUnitCoordSys(), FOR_METRIC);
    ulong thick;
    if (!XGetFontProperty ((XFontStruct*)shxfont->fontStruct(),
			   (Atom)XA_UNDERLINE_THICKNESS , &thick))
        thick = 1;
    return  PROPERTY_MACRO(thick);
}

GNumber XGoInterf::fontUnderlinePosition (GNumber height,
					  const GShFont* font) 
{
    const ShXFont* shxfont = _fontToShXFont(font, GUnitCoordSys(), FOR_METRIC);
    ulong pos;
    if (!XGetFontProperty ((XFontStruct*)shxfont->fontStruct(),
			   (Atom)XA_UNDERLINE_POSITION , &pos))
        pos = shxfont->fontStruct()->max_bounds.descent/2;

    return PROPERTY_MACRO(pos);
}

GNumber XGoInterf::fontItalicAngle (const GShFont* font) 
{
    const ShXFont* shxfont = _fontToShXFont(font, GUnitCoordSys(), FOR_METRIC);
    ulong angle;
    if (!XGetFontProperty ((XFontStruct*)shxfont->fontStruct(),
			   (Atom)XA_ITALIC_ANGLE , &angle))
	angle = 0;
    return angle;
}

GNumber XGoInterf::fontStrikeoutAscent (GNumber height,
					const GShFont* font) 
{
    const ShXFont* shxfont = _fontToShXFont(font, GUnitCoordSys(), FOR_METRIC);
    ulong value = shxfont->fontStruct()->max_bounds.ascent;
/****
    if (!XGetFontProperty ((XFontStruct*)shxfont->fontStruct(),
			   (Atom)XA_STRIKEOUT_ASCENT , &value))
	value = 1;
  ***/
    return PROPERTY_MACRO(value);
}

GNumber XGoInterf::fontStrikeoutDescent (GNumber height,
					 const GShFont* font) 
{
    const ShXFont* shxfont = _fontToShXFont(font, GUnitCoordSys(), FOR_METRIC);
    ulong value = shxfont->fontStruct()->max_bounds.descent;
/**
    if (!XGetFontProperty ((XFontStruct*)shxfont->fontStruct(),
			   (Atom)XA_STRIKEOUT_DESCENT , &value))
	value = 1;
  **/
    return PROPERTY_MACRO(value);
}

GNumber XGoInterf::fontMinSpace (GNumber height, const GShFont* font) 
{
    const ShXFont* shxfont = _fontToShXFont(font, GUnitCoordSys(), FOR_METRIC);
    ulong value;
    if (!XGetFontProperty ((XFontStruct*)shxfont->fontStruct(),
			   (Atom)XA_MIN_SPACE , &value))
	value = 1;
    return PROPERTY_MACRO(value);
}

GNumber XGoInterf::fontMaxSpace (GNumber height, const GShFont* font) 
{
    const ShXFont* shxfont = _fontToShXFont(font, GUnitCoordSys(), FOR_METRIC);
    ulong value;
    if (!XGetFontProperty ((XFontStruct*)shxfont->fontStruct(),
			   (Atom)XA_MAX_SPACE , &value))
	value = 1;
    return PROPERTY_MACRO(value);
}

GNumber XGoInterf::fontNormSpace (GNumber height, const GShFont* font) 
{
    const ShXFont* shxfont = _fontToShXFont(font, GUnitCoordSys(), FOR_METRIC);
    ulong value;
    if (!XGetFontProperty ((XFontStruct*)shxfont->fontStruct(),
			   (Atom)XA_NORM_SPACE , &value))
	value = 1;
    return PROPERTY_MACRO(value);
}

GNumber XGoInterf::fontEndSpace (GNumber height, const GShFont* font) 
{
    const ShXFont* shxfont = _fontToShXFont(font, GUnitCoordSys(), FOR_METRIC);
    ulong value;
    if (!XGetFontProperty ((XFontStruct*)shxfont->fontStruct(),
			   (Atom)XA_END_SPACE , &value))
	value = 1;
    return PROPERTY_MACRO(value);
}

static void MoveToBegin(void* src, void* begin, int size)
{
#ifdef NO_ALLOCA
    char* tmp = (char*)GMALLOC(size);
#else
    char* tmp = (char*)alloca(size);
#endif

    bcopy(src, tmp, size);
    bcopy(begin, ((char*)begin) + size,  (uint)src - (uint)begin);
    bcopy(tmp, begin, size);

#ifdef NO_ALLOCA
    GFREE(tmp);
#endif
}

GNumber ShXFont::heightForNaming(GNumber height) const
{
    // height is the max height of the font;
    // returns the height wanted for a fontname (ig point_size);
    return _pixelHeight * (int)(height*_for_naming/_pixelHeight);
}
    
const ShXFont* XGoInterf::_fontToShXFont(const GShFont* shfont, const GCoordSys& cs, GNumber height)
{
    GNumber width = FOR_METRIC;

    if (height != FOR_METRIC)
    {
	// cs does not matter to get metric;
	const ShXFont* metric = _fontToShXFont(shfont, cs, FOR_METRIC);

	height = metric->heightForNaming(cs.J.size() * height);
	width = cs.I.size() * height / cs.J.size();
    }

    FontAssoc*  pf;
    FontAssoc   _pf;
    int npf;
    int NPF;
    pf = (FontAssoc*)_assocFont->get(shfont, npf);
    NPF = npf;
    
    /**
      width/height is the effective width/height wanted in mm
      **/
    
    FontAssoc* PF = pf;
    for (; npf; npf--, pf++)
	if (pf->height == height && pf->width == width &&
	    pf->pixelHeight == _pixelHeight &&
	    pf->pixelWidth == pf->pixelWidth)
	{
	    /**
	      put the interface in used list if not yet
	      **/
	    _addXInterfIfNeeded(pf->xinterfs);
	    /**
	      put the current *pf to the beginning of the list
	      **/
	    if (pf != PF)
		MoveToBegin(pf, PF, sizeof(FontAssoc));
	    return PF->shxfont;
	}


    // we converve only shfont->refcnt() for an X one:

    if (NPF > shfont->refcnt())  // it is not >= because in release 4: we need a fontstruct;
                                 // to get metric;
    {
	// remove the last pf in use;
	pf = &PF[NPF-1];
	pf->shxfont->destroy();
	pf->xinterfs->destroy();
    }
    else
	pf = &_pf;

    char font_name[MAX_FONTNAME_LENGHT];

    
    ShFontToXFontName(shfont, width, height, _pixelWidth, _pixelHeight, font_name);

    ShXFontValues shxfontvalues;
    shxfontvalues.dpy = _dpy;
    shxfontvalues.pixelWidth = _pixelWidth;
    shxfontvalues.pixelHeight = _pixelHeight;
    shxfontvalues.font_name = GCreateAtom(font_name);

    pf->shxfont = GCreateShXFont(&shxfontvalues);
    pf->height = height;
    pf->pixelHeight = _pixelHeight;
    pf->width = width;
    pf->pixelWidth = _pixelWidth;
    
    void* t = this;
    pf->xinterfs = GCreateShArray(&t);
    
    /**
      make  association if not done yet;
      **/
    if (pf == &_pf)
	_assocFont->add(shfont, sizeof _pf, pf);

    if (NPF == 0) 
    {
	assert (pf == &_pf);
	
	// add postconds for the first association;

	shfont->updatePostcond( ShFontIsUpdated, _assocFont, GADD);
	shfont->destroyPostcond( ShFontIsDestroyed, _assocFont, GADD);
    }

    /**
      return the ShXFont
      **/
    return pf->shxfont;
}

Font  XGoInterf::_fontToXFont(const GShFont* shfont, const GCoordSys& cs, GNumber height)
{
    return _fontToShXFont(shfont, cs, height)->fontStruct()->fid;
}


void XGoInterf::_freeXFonts()
{
    int nkeys;
    void** keys = _assocFont->getKeys(nkeys);
    void** _keys = keys;
    for (int i = 0; i < nkeys; i++, keys++) 
    {
	int npf;
	FontAssoc* pf = (FontAssoc*)_assocFont->get(*keys, npf);
	assert (pf);
	int _npf = 0;
	int NPF = npf;
#ifdef NO_ALLOCA
	FontAssoc* _pf = (FontAssoc*)GMALLOC(npf * sizeof( FontAssoc));
#else
	FontAssoc* _pf = (FontAssoc*)alloca(npf * sizeof( FontAssoc));
#endif
	for (; npf; npf--, pf++) 
	{
	    /**
	      remove the interface from the used list
	      if the list becomes NULL remove all this associations
	      **/
	    _remXInterfIfInside(pf->xinterfs);
	    if  (pf->xinterfs == NULL) 
	    {
		/**
		  remove the callbacks
		  **/
		((GShFont*)*keys)->updatePostcond(ShFontIsUpdated,
						 _assocFont, GREMOVE);
		((GShFont*)*keys)->destroyPostcond(ShFontIsDestroyed,
						  _assocFont, GREMOVE);
		/**
		  destroy the associated xfonts
		  **/
		pf->shxfont->destroy();
		/**
		  save the structure for destroy association step.
		  **/
		_pf[_npf++] = *pf;
	    }
	}
	if (_npf == NPF )
	    _assocFont->rem(*keys);
	else
	    for (NPF=0; NPF < _npf; NPF++, _pf++)
		_assocFont->rem(*keys, sizeof(FontAssoc), _pf);
#ifdef NO_ALLOCA
	GFREE(_pf);
#endif
    }
    
    /**
      free the keys list
      **/
    GFREE(_keys);
}


/**
  PRIVATE SHXFONT METHODS
  ***********************
  **/

int ComparPixelSize(const void* p1, const void* p2)
{
    char** name1 = (char**)p1;
    char** name2 = (char**)p2;
    int h1 = atoi(*name1 + X_PixelSizeIndex(*name1));
    int h2 = atoi(*name2 + X_PixelSizeIndex(*name2));
    if (h1 > h2)
	return 1;
    return -1;
}

XFontStruct* ShXFont::_makeBestFont() 
{
    int one;
    FontStructAssoc* pf = NULL;
    XFontStruct* font_struct;

    /**
      assert valid font-name
      **/
#define font_name ((char*)_font_name)
    assert (font_name[0] == '-' ) ; 

    /**
      try to get an equivalent
      **/
    char pattern[MAX_FONTNAME_LENGHT] ;
    int hindex = Lib_PixelSizeIndex(font_name);			     
    /**
      pattern is in lower case, change undef fields to '*'
      **/
    int pattern_index = 0;
    for (int j = 0; j < hindex; j++, pattern_index++)
 	if (font_name[j] == '-' && font_name[j+1] == '-')
	{
	    pattern[pattern_index++] = '-';
	    pattern[pattern_index] = '*';
	}
	else
	    pattern[pattern_index] = font_name[j];
    
    pattern[pattern_index++] = '*' ;
    assert (pattern_index < (int)sizeof pattern);
    pattern[pattern_index] = '\0' ;
    int count ;
    
    /**
      find the font list matching the pattern;
      If match is false get the default font.
      **/
    
    int wanted_height = atoi(font_name+hindex); // it is the size in pixels;
      
    char** flist = XListFonts(_dpy, pattern, sizeof pattern,  &count);
    if (flist == NULL)
    {
	/**
	  get default
	  **/
	GWarning("%s no match for: \"%s\" \"%s\" is used",
		 "XGoInterf::_makeBestFont()",
		 pattern, DEFAULT_FONT);
	_full_fontname = EffectiveDefaultFont(_dpy);
    }
    else
    {

	// the effective size in pixel of the font depends:
	// of the font and screen resolutions;
	// the result may differ by 1 pixel because of difference between:
	// the font.bdf and the effective font.snf;
	// It does not matter: we reset the effective _height with:
	// the effective font_struct at loading time;
			    
	if (wanted_height == FOR_METRIC)
	{
	    int D = 0;
	    int N  = 0;
	    int n;
	    GBool first = GTRUE;
	    
	    // in x release <= 5 take the higher font available;
	    // use the PIXEL_SIZE criterium;
	    // IN RELEASE 5: May be we have to use the Font Service Protocol;
	    // To get Metrics of the font.
		

		for (n = 0 ; n < count ; n++ )
		{
		    int index = X_PixelSizeIndex(flist[n]);
		    if (index == -1)
			continue;
		    int d =  atoi(flist[n]+index) - 1000;
		    if (first ||  (d <= 0 && d > D))
		    {
			first = GFALSE ;
			D = d;
			N = n ;
			if (D == 0)
			    break;
		    }
		}
	    _full_fontname = GCreateAtom(flist[N]);
	}
	else
	{
	    // we want a specific height;
	    // X_HeightInPixels returns the EFFECTIVE HEIGHT of the font;


#ifdef NO_ALLOCA
	    char** sorted_flist = (char**)GMALLOC(count* sizeof(char*));
#else
	    char** sorted_flist = (char**)alloca(count*sizeof(char*));
#endif
	    bcopy(flist, sorted_flist, count*sizeof(char*));
	    qsort((char*)sorted_flist, count, sizeof(char*), ComparPixelSize);

	    if (IsScalableFont(sorted_flist[0]))
	    {
		// may be true only in release >= 5;
		// font_name must be well-formed to be loaded;
		// but remove the pixel_size value which may be inconsistent ;
		// for an effective font (point_size, average_width) and resolution;

		int i;
		// pass the pixel_size;
		for (i = hindex;  font_name[i] != '-'; i++);
		
		// copy the end of the name;
		strcpy(pattern + pattern_index, font_name + i);

		// make the corresponding atom;
		_full_fontname = GCreateAtom(pattern);
	    }
	    else
	    {
		int D = 0;
		int N  = 0;
		int n;
		GBool first = GTRUE;
		int max_height = 0;
		int max_height_index = 0;
		int min_height = 1000;
		int min_height_index = 0;
		
		FontStructAssoc* usedPF = NULL;
		for (n = 0 ; n < count ; n++ )
		{
		    pf = (FontStructAssoc*)_assocFontStruct->get(sorted_flist[n],one);
		    if (pf && pf->effective_height == wanted_height)
		    {
			N = n;
			usedPF = pf;
			break;
		    }
		    int index = X_PixelSizeIndex(sorted_flist[n]);
		    int height = atoi(sorted_flist[n]+index);
		    if (height > max_height)
			max_height = height, max_height_index = n;
		    if (height < wanted_height)
			min_height = height, min_height_index = n;
		}

		pf = usedPF;

		if (!pf)
		{
		    if (max_height < wanted_height)
			N = max_height_index;
		    else
		    {
			for (n = (int)max((long)min_height_index-2,0L); n < count; n++)
			{
			    // X_HeightInpIxels will load the font if needed;
			    int d =  X_HeightInPixels(_assocFontStruct,
						      _dpy, sorted_flist[n]) - wanted_height;

			    if (first ||  (d <= 0 && d > D))
			    {
				first = GFALSE;
				D = d;
				N = n ;
				if (D == 0)
				    break;
			    }
			    if (d >= 0)
				break;
			}
		    }
		}

		_full_fontname = GCreateAtom(sorted_flist[N]);
	    }
#ifdef NO_ALLOCA
	    GFREE(sorted_flist);
#endif
	}
	
	XFreeFontNames(flist) ;
    }

    /**
      load the font with the best height
      verify first if it is not loaded yet;
      the "again" label may be accesses to handle error;
      **/
	
  again:
	
    if (pf == NULL)
	pf = (FontStructAssoc*)_assocFontStruct->get(GAtomToString(_full_fontname), one);

    if (pf)
    {
	assert (one == 1);
	if (pf->font_struct == NULL)
	{
	    pf->font_struct = XLoadQueryFont(_dpy, GAtomToString(_full_fontname));
	    if (pf->effective_height != pf->font_struct->max_bounds.ascent +
		pf->font_struct->max_bounds.descent)
		GError("XGoInterf::makeBestFont() no:1");
	}
	font_struct = pf->font_struct;
	pf->nref++;
    }
    else
    {
	FontStructAssoc _pf;
	font_struct = XLoadQueryFont(_dpy, GAtomToString(_full_fontname));
	
	if (font_struct == NULL)
	{
	    GAtom default_fontname = EffectiveDefaultFont(_dpy);
	    if (_full_fontname == default_fontname)
	    {
		GError ("%s %s \"%s\" %s \"%s\"", "XGoInterf::_makeBestFont()",
			"cannot open", DEFAULT_FONT, "", "");
		// NOT REACHED;
	    }
		
	    GWarning ("%s %s \"%s\" %s \"%s\"", "XGoInterf::_makeBestFont()",
		      "cannot open", GAtomToString(_full_fontname), " try with",
		      DEFAULT_FONT);

	    GFreeAtom(_full_fontname);
	    _full_fontname = default_fontname;
	    goto again;

	}

	// update the font_struct association;

	_pf.effective_height = 
	    font_struct->max_bounds.ascent + font_struct->max_bounds.descent;
	_pf.font_struct = font_struct;
	_pf.nref = 1;
	_assocFontStruct->set(GAtomToString(_full_fontname),
			      sizeof (_pf), &_pf);
    }

    // take _height from the font_struct;
    // It is usefull only for the METRIC info;

    if (wanted_height == FOR_METRIC)
    {
	_height = (GNumber)(font_struct->max_bounds.ascent +
			    font_struct->max_bounds.descent);
    
	_relative_ascent = ((GNumber)font_struct->max_bounds.ascent) / _height;
	_for_naming = ((GNumber)(font_struct->ascent+font_struct->descent)) / _height;
    }
    else
    {
	_height = 0;            // on error we will have a division by 0 --> trap;
	_relative_ascent = 0;
	_for_naming = 0;
    }

    assert (font_struct) ;
    return font_struct;
    
#undef font_name
}

/*
 * static variable of the shxfont class
 */
GList*        ShXFont::_headOfList;
GFunctionList ShXFont::_funcList;

/**
  PROTECTED SHXFONT METHODS
  **/
ShXFont* _newShXFont (GFunctionList* funcList, ulong mask,
		      const void* values, const char* name)
{
    return new ShXFont(funcList, mask, (const ShXFontValues*)values, name);
}

/**/
 ShXFont::ShXFont (GFunctionList* funcList, ulong mask, const ShXFontValues* values, const char* name) : GSharedObject(funcList, mask, name)
{
    assert (mask == SHXFONTMASK);
    assert (name == NULL);

    _dpy = values->dpy;
    _pixelWidth = values->pixelWidth;
    _pixelHeight = values->pixelHeight;
    _font_name = values->font_name;
    _full_fontname = NULL;   // it will be initialized bt _makeBestFont();
    
    _assocFontStruct = GetAssocFontStruct(_dpy);
    
    /**
      CREATE FONT
      **/
    _font_struct = _makeBestFont();
}

/**/
GList*& ShXFont::_head ()
{
    return _headOfList;
}

/**/
int ShXFont::_valueSize ()
{
    return (sizeof(ShXFontValues));
}

/**/
GBool ShXFont::_match (ulong mask, const void* __values) const
{
    const ShXFontValues* values = (const ShXFontValues*)__values;
    assert ( mask == SHXFONTMASK);

    return
	_font_name == values->font_name &&
	_dpy == values->dpy &&
	_pixelWidth == values->pixelWidth &&
	_pixelHeight == values->pixelHeight;
}

/**/
void ShXFont::_makeNewValues (ulong mask, const void* values,
			      ulong& newMask, void* newValues) const
{
    *(ShXFontValues*)newValues = *(const ShXFontValues*)values;
    newMask = mask;
}

/**/
ulong ShXFont::_update (ulong mask, const void* __values)
{
    const ShXFontValues* values = (const ShXFontValues*)__values;
    int one;
    FontStructAssoc* pf = (FontStructAssoc*)_assocFontStruct->get(GAtomToString(_full_fontname), one);
    assert (pf  && one == 1);
    if (--pf->nref == 0)
    {
	// free the xfont struct;
	assert (pf->font_struct == _font_struct);
	XFreeFont(_dpy, _font_struct);

	// and remove the association;
	_assocFontStruct->rem(GAtomToString(_full_fontname));
    }

    GFreeAtom (_font_name);
    if (_full_fontname)
	GFreeAtom(_full_fontname);
    
    _dpy = values->dpy;
    _pixelWidth = values->pixelWidth;
    _pixelHeight = values->pixelHeight;
    _font_name = values->font_name;
    _full_fontname = NULL;    // it will be initialized bt _makeBestFont();

    /**
      CREATE FONT
      **/
    _font_struct = _makeBestFont();
    return mask;
}

/**/
 ShXFont::~ShXFont ()
{
    int one;
    FontStructAssoc* pf = (FontStructAssoc*)_assocFontStruct->get(GAtomToString(_full_fontname), one);
    assert (pf  && one == 1);
    if (--pf->nref == 0)
    {
	// free the xfont struct;
	assert (pf->font_struct == _font_struct);
	XFreeFont(_dpy, _font_struct);

	// and remove the association;
	_assocFontStruct->rem(GAtomToString(_full_fontname));
    }

    GFreeAtom (_font_name);
    if (_full_fontname)
	GFreeAtom(_full_fontname);
    
    FreeAssocFontStruct(_assocFontStruct);
}

/**
  PUBLIC SHXFONT METHODS
  **********************
  **/

/**/
ShXFont* GCreateShXFont (const ShXFontValues* values)
{
    return GCREATE_SHAREDOBJECT(ShXFont, SHXFONTMASK, values, NULL);
}


