/* $Id: XFontP.h,v 1.3 1992/09/25 15:26:05 smadja Exp $ */
/*****************************************************************************
MODULE:  XGoInterface
FILE:    ShXGC.h
AUTHOR:  Philippe Smadja
DATE:    Nov 1990

DESCRIPTION:
         ShXFont are shared object. The matches criterium are
	 1: the display.
	 2: the font_name. This font_name must be computed with the
	    wanted shfont and the wanted height.

COPYRIGHT :
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
*****************************************************************************/
#ifndef _XFontP_h
#define _XFontP_h

#include <Go/Tools/Font.h>
#include <Go/Tools/SharedObj.h>
#include <Go/Tools/ShArray.h>
#include <Go/Tools/Assoc.h>
#include <Go/Tools/StringAssoc.h>
#include <Go/Tools/Atom.h>

extern GAssoc* getAssocFont(Display*);
extern void    freeAssocFont(GAssoc*);

#define SHXFONTMASK (ulong)-1
#define FOR_METRIC  0         /* use it for shFontToXFont() height parameter to get METRIC. */
                           
    
struct ShXFontValues
{
    Display*   dpy;
    GNumber    pixelWidth;
    GNumber    pixelHeight;
    GAtom      font_name;
} ;

class ShXFont : public GSharedObject {
  private:
    static GList* _headOfList;
    static GFunctionList _funcList;

    Display*     _dpy;
    GNumber      _pixelWidth;      // in mm;
    GNumber      _pixelHeight;     // in mm;
    GAtom        _font_name;
    GAtom        _full_fontname;
    GNumber      _height;          // in pixel it is a GNumber for performance;
    GNumber      _relative_ascent; // maxascent / _height;
    GNumber      _for_naming; 
	
    GStringAssoc*      _assocFontStruct;
    XFontStruct* _font_struct;     // It is copied from _assocFontStruct for performance;
    XFontStruct* _makeBestFont();

  protected:
    
    GList*& _head();
    int    _valueSize ();
    virtual GBool   _match (ulong mask, const void* values) const;
    virtual void    _makeNewValues (ulong mask, const void* values,
				    ulong& newMask, void* newValues) const;
    virtual ulong   _update (ulong mask, const void* values);

    friend ShXFont* _newShXFont (GFunctionList* funcList,
				 ulong mask,
				 const void* values,
				 const char* name = NULL);

    ShXFont (GFunctionList* funcList, ulong mask, const ShXFontValues* values, const char* name = NULL);
    ~ShXFont ();

  public:
    friend  ShXFont* GCreateShXFont (const ShXFontValues* values);
    const inline XFontStruct* fontStruct() const { return _font_struct;};
    const inline char*        fontName() const   { return (char*)_full_fontname;};
    inline GNumber            height() const { return _height; } ;
    inline GNumber            relativeAscent() const {return _relative_ascent;};
    GNumber                   heightForNaming(GNumber h) const;
};

struct FontAssoc {
    ShXFont*    shxfont;
    GNumber      width;
    GNumber      height;
    GNumber      pixelWidth;
    GNumber      pixelHeight;
    GShArray*    xinterfs;
};

#endif  /* _XFont_h */




