/* $Id: XGCP.h,v 1.2 1992/09/25 15:26:09 smadja Exp $ */
/*****************************************************************************
MODULE:  XGoInterface
FILE:    XgcP.h
AUTHOR:  Philippe Smadja
DATE:    Nov 1990

DESCRIPTION:
  Declaration of a SharedObj to handle the sharing of the XGC between
                 several ShPen, ShBrush, ShImageBrush
  
COPYRIGHT :
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#ifndef _XGCP_h
#define _XGCP_h

#include <Go/Tools/ShPen.h>
#include <Go/Tools/ShBrush.h>
#include <Go/Tools/ShImageBrush.h>
#include <Go/Tools/ShTextBrush.h>
#include <Go/Tools/Assoc.h>


extern GAssoc* getAssocGC(Display*, uchar screen, uchar depth,
			 GC* copyGC, GC* clearGC, GC* setGC);
extern void   freeAssocGC(GAssoc*);

extern GAssoc* getAssocColorPattern(Display*, Colormap);
extern void   freeAssocColorPattern(GAssoc*);

#define SHXGCMASK (ulong)-1

struct ShXgcValues {
    Display*    dpy;           /** only display_name is used for match
				 criterium
				 **/
    uchar       depth;
    uchar       screen;
    Colormap    colormap;
    Visual*     visual;

    const ShXRegion*  shxregion;
    int         xoffset;
    int         yoffset;
    ulong       dash_ident;
    char*       dash_list;
    int         n_dashes;
    int         dash_offset;
    ulong       xgcmask;
    XGCValues*  xgcvalues;

    /**
      not in match criterium
      usefull for X functions
      **/
    Drawable      drawable;
} ;

class ShXgc : public GSharedObject 
{
  private:
    static GList* _headOfList;
    static GFunctionList _funcList;

    Display*    _dpy;
    uchar       _depth;
    uchar       _screen;
    Colormap    _colormap;    
    Visual*     _visual;
    
    ShXRegion*  _shxregion;
    int         _xoffset;
    int         _yoffset;

    ulong       _dash_ident;
    char*       _dash_list;
    int         _n_dashes;
    int         _dash_offset;
    
    ulong       _xgcmask;
    XGCValues   _xgcvalues;

    GC          _gc;
    
  protected:
    
    GList*& _head();
    int    _valueSize ();
    virtual GBool  _match (ulong mask, const void* values) const;
    virtual void   _makeNewValues (ulong mask, const void* values,
				   ulong& newMask, void* newValues) const;
    virtual ulong  _update (ulong mask, const void* values);

    ShXgc (GFunctionList* funcList, ulong mask, const ShXgcValues* values, const char* name = NULL);
    ~ShXgc ();
    friend ShXgc* _newShXgc (GFunctionList* funcList,
			     ulong mask,
			     const void* values,
			     const char* name = NULL);

  public:

    friend  ShXgc* GCreateShXgc (const ShXgcValues* values);
    friend  void GModify (ShXgc*& sh, const ShXgcValues* values);

    ShXgc* dup() const;
    const   ulong  xgcmask() { return _xgcmask; };
    const   XGCValues* xgcvalues() { return &_xgcvalues; };
    const   GC gc() { return _gc; }
    const   ShXRegion* shXRegion() { return _shxregion;}
    int     xoffset() { return _xoffset;}
    int     yoffset() { return _yoffset;}
};

#endif /* _XGC_h */




