#ifdef REV_INFO
static char rcsid[] = "$Id: XRegion.cc,v 1.3 1992/09/25 15:27:35 smadja Exp $";
#endif
/*****************************************************************************
MODULE:  X INTERFACE
FILE:    XRegion.cc
AUTHOR:  Philippe Smadja
DATE:    Nov 90

DESCRIPTION:
    Region management

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#include <assert.h>
#include "XRegionP.h"

/*
 * static variable of the shxregion class
 */
GList*        ShXRegion::_headOfList;
GFunctionList ShXRegion::_funcList;

/**
  PROTECTED SHXREGION METHODS
  **/
ShXRegion* _newShXRegion (GFunctionList* funcList, ulong mask,
			  const void* values, const char* name)
{
    return new ShXRegion(funcList, mask, (const ShXRegionValues*)values, name);
}

/**/
ShXRegion::ShXRegion (GFunctionList* funcList, ulong mask,
		      const ShXRegionValues* values, const char* name):
	GSharedObject(funcList, mask, name)
{
    assert (mask == SHXREGIONMASK);
    assert (name == NULL);

    _cs =  *values->cs;

    /**
      CREATE REGION
      **/
    _makeRegion();
}

/**/
GList*& ShXRegion::_head ()
{
    return _headOfList;
}

/**/
int ShXRegion::_valueSize ()
{
    return (sizeof(ShXRegionValues));
}

/**/
GBool ShXRegion::_match (ulong mask, const void* values) const
{
    
    assert ( mask == SHXREGIONMASK);
    return _cs == *((const ShXRegionValues*)values)->cs;
}

/**/
void ShXRegion::_makeNewValues (ulong mask, const void* values,
				 ulong& newMask, void* newValues) const
{
    *(ShXRegionValues*)newValues = *(const ShXRegionValues*)values;
    newMask = mask;
}

/**/
ulong ShXRegion::_update (ulong mask, const void* __values)
{
   const ShXRegionValues* values = (const ShXRegionValues*)__values;
    if (_xregion)
	XDestroyRegion(_xregion);
    
    _cs  = *values->cs;

    /**
      CREATE REGION
      **/
    _makeRegion();
    return mask;
}

/**/
ShXRegion::~ShXRegion ()
{
    if (_xregion)
	XDestroyRegion(_xregion);
}

/**
FUNCTION
    _makeRegion
DESCRIPTION
    The function compute the width and the height of the region and
    makes a Polygon region. of depth 1
    accorging the coordsys if the cs is not orthogonal.
**/
void ShXRegion::_makeRegion()
{
    /**
      compute the polygon to draw
      **/
    GRegion  region(_cs.to(GRegion(GBox(GPoint(0,0),GPoint(0,1),GPoint(1,0),GPoint(1,1)))));
    const GBox* box = &region.bigbox();

    /**
      compute width and height
      **/
    _width = (int)box->width();
    _height = (int)box->height();

    
    if (_width == 0 || _height == 0 || _cs.isOrthogonal())
	_xregion = NULL ;

    else
    {
	/**
	  polygon region.
	  **/
	XPoint xpts[5];
	for (int i=0, k=0; i < 2; i++) 
	    for (int j=0; j < 2; j++)
	    {
		GPoint absp = _cs.to(GPoint(i,j));
		xpts[k].x = (int)(absp.x - box->xmin);
		xpts[k++].y = (int)(absp.y - box->ymin);
	    }
	XPoint tmp = xpts[2];
	xpts[2]= xpts[3];
	xpts[3] = tmp;
	xpts[4] = xpts[0];
	_xregion = XPolygonRegion(xpts, 5, EvenOddRule);
    }
}    
  
/**
  PUBLIC SHXREGION METHODS
  **********************
  **/

/**/
ShXRegion* GCreateShXRegion (const GCoordSys& cs)
{
    ShXRegionValues values;
    values.cs = &cs;
    
    return GCREATE_SHAREDOBJECT(ShXRegion, SHXREGIONMASK, &values, NULL);
}




