#ifdef REV_INFO
static char rcsid[] = "$Id: XString.cc,v 1.3 1992/06/03 13:35:25 smadja Exp $";
#endif
/*****************************************************************************
MODULE:  X INTERFACE
FILE:    XString.cc
AUTHOR:  Philippe Smadja
DATE:    August 1990

DESCRIPTION:
  Implementation of string().
  in  the X environment.
  
COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#include <Go/XGoInterf.h>
#include "XFontP.h"
#include "XPrivate.h"

// for GRectangleClip():
#include <Go/Tools/GeomUtil.h>


void XGoInterf::string (const GShTextBrush* shtextbrush,
			const GShBrush* shbrush,
			const GCoordSys& cs,
			const GPoint& position, GNumber height, int nchar,
			const char* string, GQuality quality)
{
    GCoordSys _fontCS = cs;
    if (DEGREE(cs.I) != 0 || DEGREE(cs.J) != 90)
    {
	// GWarning ("deformed string -> min quality");
	quality = GMIN_QUALITY;
    }
    else  if (_dpy->release <= 4)
    {
	// in release <=4;
	// to compute the font used in the GC:
	// take the smallest right one;
	
	_fontCS.I.x = min(cs.I.x, cs.J.y);
	_fontCS.J.y = _fontCS.I.x;
    }
    

    const ShXFont* metric;
    const GShFont* shfont = shtextbrush->font();

    // the following ...font_struct will be casted in (XFontStruct*) for;
    // X requests because X prototypes do not use the "const" functionnality;

    const XFontStruct* metric_font_struct;
    const XFontStruct* effective_font_struct; 

    GC gc;
    
{   // tempory block because tmp_shfont is used here only;
	
    // I need on reference more : 1 for the used _fontCS and one for the;
    // cs because the ShFontToXFont() method keeps only ;
    // shfont->refcnt() XFont for an shfont;
    // shfont will be destroy: refcnt--  at end of processing;

    GShFont* tmp_shfont = shfont->dup();
    
    const ShXFont* shxfont;
    gc = _shTextBrushToGC(shtextbrush, _fontCS,
			  height, &shxfont, quality);
    effective_font_struct = shxfont->fontStruct();
	
    // get the metrics;
    metric = _fontToShXFont(shfont, GUnitCoordSys(), FOR_METRIC);
    metric_font_struct = metric->fontStruct();
	
    // remove the temporary shfont:
    // cf shfont->dup() comments above:
    
    tmp_shfont->destroy();
}


    // NOW we can  go:
    GNumber font_ascent  = metric->relativeAscent() * height;

    // compute position in pixels;
    
    GPoint pos = position;
    pos.y += font_ascent;
    GPoint absp(cs.to(pos));

    XPoint xp;
    
#ifdef __GNUC__
    // The following declaration is needed because;
    // of bug in g++1.39b on dpx2-300;
    // ** removing it can cause g++ to exit with fatal signal 6 ** !!!!!!!!
	XPoint *xpp = &xp;
    xpp->x = xp.x;     // no op;
#endif
    
 
    xp.x = mm_to_pixx(absp.x);
    xp.y = mm_to_pixy(absp.y);

    if (quality != GMIN_QUALITY)
    {
	// verify that mm_to_pixel conversion is OK for the wanted height;

	GPoint tmp(cs.to(position));
	int  y_position_in_pixels = mm_to_pixy(tmp.y);
	int  theoric_max_ascent_in_pixels = xp.y - y_position_in_pixels;

	if (theoric_max_ascent_in_pixels <
	    effective_font_struct->max_bounds.ascent)
	{
	    // it may occur because of difference between the theorical;
	    // metrics and the effective one;
	    theoric_max_ascent_in_pixels =
		effective_font_struct->max_bounds.ascent;
	}
	
	// compute the height of the bounding box;

	tmp = cs.to(GPoint(0, position.y + height));
	int  theoric_height_in_pixels =
	    mm_to_pixy(tmp.y) - y_position_in_pixels;

	if (theoric_max_ascent_in_pixels +
	    effective_font_struct->max_bounds.descent >
	    theoric_height_in_pixels)
	{
	    // it may occur because of mm_to_pixels conversion;
	    xp.y--;

	    // we do not decrement theoric_max_ascent_in_pixels;
	    // because we have 2 mm_to_pixels conversions;
	    // theoric_max_ascent_in_pixels--;
	}

	// test again: Now if it is TRUE: it means that the effective font:
	// the smaller one;
	// if greater than the wanted one: so draw in GMIN_QUALITY;
	// Note that we test with the logical font ascent and not;
	// the max_ascent;

	if (theoric_max_ascent_in_pixels <  effective_font_struct->ascent)
	    quality = GMIN_QUALITY;
    }

    if (shbrush || quality == GMIN_QUALITY)
    {
	// not that stringWidth() will use metric for get the info;

	GNumber string_width = stringWidth(string, nchar, height, shfont);

	if (shbrush)
	{
	    // draw the background;
	    rectangle(NULL, shbrush, cs, position + GPoint(0, 0),
		      string_width,  height, quality);

	    // do not return here;
	    // because we may pass here with quality != G_MIN_QUALITY;
	}

	if (quality == GMIN_QUALITY)
	{
	    // draw the baseline to show the text:
	    // do not worry about left and right bearing;
	
	    static GShPen* shpen = NULL;
	    if (shpen == NULL)
		shpen = GCreateShPen(); /* default one: black and
					   linewidth = 1 pixel */

	    GSetColor(shpen, shtextbrush->color());
	    GSetFunction(shpen, shtextbrush->function());

	    // compute p1 and p2;

	    GPoint p1 = position;
	    p1.y += font_ascent;
	
	    GPoint p2 = p1;
	    p2.x += string_width - _pixelHeight/cs.J.size(); // no last point;
	
	    // draw the line;
	    line (shpen, cs, p1, p2, quality);

	    return;
	}
    }


    // try to make the request with 1 XDrawString();
    // note that cs is orthogonal so delta_y will be always 0;
    // unless quality has been reseted to GMIN_QUALITY;

    GNumber div = metric->height() / height;
#define PROPERTY_MACRO(delta_x) (((GNumber)(delta_x))/div)


    // draw char by char: try to cumulate the XDrawString;
    // NOTE that the algorithm is correct because;
    // THE XTextExtend function DOES NOT HANDLE THE TRACKING !!!!!
    // See Xlib reference manual ;

    int width_in_pixels ;
    const char* s = string;
    const char* string_to_draw = string;
    int         nchar_to_draw = 1;
    int         cumulated_width_in_pixels = 0;

    for (int i = 0 ; i < nchar; i++, s++)
    {
	// compute the size of *s;
	// 1er with the effective font struct;
	// 2eme with the metric one to get the theorical deplacement;
	    
	width_in_pixels = XTextWidth((XFontStruct*)effective_font_struct,
				     s, 1);

	cumulated_width_in_pixels += width_in_pixels;
	    
	if (effective_font_struct != metric_font_struct)
	    width_in_pixels = XTextWidth((XFontStruct*)metric_font_struct,
					 s, 1);
	    
	// compute the next position;
	// y does not change;
	    
	
	pos.x += PROPERTY_MACRO(width_in_pixels);
	absp = cs.to(pos);
	int next_pos = mm_to_pixx(absp.x);

	if (xp.x + cumulated_width_in_pixels == next_pos && i < nchar-1)
	    nchar_to_draw++;
	else
	{
	    // draw the string;
	    
	    XDrawString(_dpy, _drawable, gc, xp.x, xp.y,
			string_to_draw, nchar_to_draw);

	    // reset variable;
	    string_to_draw = s+1;
	    nchar_to_draw = 1;
	    cumulated_width_in_pixels = 0;
		
	    // set the next position;
	    xp.x = next_pos;
	}
    }
}


#undef PROPERTY_MACRO








