#ifdef REV_INFO
static char rcsid[] = "$Id: Graphic.c,v 1.4 1992/12/10 14:04:58 jojic Exp $";
#endif
/*****************************************************************************
MODULE: Xm Go Interface
FILE:   Graphic.c
AUTHOR: Patricia Langle
DATE:   February 1991

DESCRIPTION: 
    A Graphic Widget automatically creates a XmGoInterface instance
    when it is realized.

    Each event, received by the Graphic Widget, is transmitted to the 
    XmGoInterf (see GoInterf_handle()).

    The XmNeditable resource is not managed.

    The XbNscaleOnResize resource, if TRUE, allows to rescale the
    GoRoot when the GraphicWidget is resized.

    The XbNautomaticResize resource, if TRUE, allows to resize
    the GraphicWidget when the GoRoot size changes (automaticResize).

    In any case, the widget size is controlled by the resize policy
    (XmNresizePolicy).

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  license from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#include "GraphicP.h"

#ifdef _NO_PROTO
extern int abs();

static Boolean SetValues();
static void Initialize();
static void Resize();
static void Destroy();
static void Realize();
static void HandleEvent();
static void HandleExposeEvent();
static void HandleMotionEvent();
static void GoRootSetInterf();
static void GoRootDestroyed();
static void GoRootResized();

#else  /* ifdef _NO_PROTO */

extern int abs(int);

static void Initialize  (XbGraphicWidget request, XbGraphicWidget new);
static void Resize      (XbGraphicWidget gw);
static void Destroy     (XbGraphicWidget gw);
static void Realize     (XbGraphicWidget gw, Mask *value_mask,
			 XSetWindowAttributes *attributes);
static void HandleEvent (XbGraphicWidget gw, caddr_t closure,
			 XmDrawingAreaCallbackStruct *call_data);
static void HandleExposeEvent(XbGraphicWidget gw, XExposeEvent* event,
			      Region region);
static void HandleMotionEvent(XbGraphicWidget gw, caddr_t closure,
			      XEvent *event);
static void GoRootSetInterf (void* gw, GoRoot* goroot, GoInterf* old_interf);
static void GoRootDestroyed (void* gw, GoRoot* goroot);
static void GoRootResized   (void* gw, GoRoot* goroot);
static Boolean SetValues    (XbGraphicWidget current, XbGraphicWidget request,
			     XbGraphicWidget new);
#endif /* ifdef _NO_PROTO else */

#define GWMessage1 "Not implemented yet!"

/****
 * Graphic Widget Resources
 ****/

static XtResource resources[] = 
{
    {
	XmNeditable, XmCEditable, XmRBoolean, sizeof(Boolean),
	XtOffset(XbGraphicWidget, graphic.editable),
	XmRImmediate, (caddr_t)TRUE
    },
    {
	XbNscaleOnResize, XbCGraphic, XmRBoolean, sizeof(Boolean),
	XtOffset(XbGraphicWidget, graphic.scale_on_resize),
	XmRImmediate, (caddr_t)FALSE
    },
    {
	XbNautomaticResize, XbCGraphic, XmRBoolean, sizeof(Boolean),
	XtOffset(XbGraphicWidget, graphic.auto_resize),
	XmRImmediate, (caddr_t)FALSE
    },
};

/****
 * Full class record constant
 ****/

XbGraphicClassRec xbGraphicClassRec = {
  {
/* core_class fields */
    /* superclass         */    (WidgetClass) &xmDrawingAreaClassRec,
    /* class_name         */    "XbGraphic",
    /* widget_size        */    sizeof(XbGraphicRec),
    /* class_initialize   */    NULL /*ClassInitialize*/,
    /* class_partinit     */    NULL,
    /* class_inited       */	FALSE,
    /* initialize         */    Initialize,
    /* Init hook	  */    NULL,
    /* realize            */    Realize,
    /* actions		  */	NULL,
    /* num_actions	  */	0,
    /* resources          */    resources,
    /* num_resources      */    XtNumber(resources),
    /* xrm_class          */    NULLQUARK,
    /* compress_motion	  */	TRUE,
    /* compress_exposure  */	TRUE /*XtExposeCompressMultiple ? */,
    /* compress_enterleave*/	TRUE,
    /* visible_interest   */    FALSE,
    /* destroy            */    Destroy,
    /* resize             */    Resize,
    /* expose             */    HandleExposeEvent,
    /* set_values         */    (XtSetValuesFunc )SetValues,
    /* set values hook    */    NULL,
    /* set values almost  */    XtInheritSetValuesAlmost,
    /* get values hook    */    NULL,
    /* accept_focus       */    NULL,
    /* Version            */    XtVersion,
    /* PRIVATE cb list    */    NULL,
    /* tm_table		  */    XtInheritTranslations,
    /* query_geometry     */    NULL,
    /* display_accelerator*/    NULL,
    /* extension          */    NULL,
  },
/* composite_class fields */
  {
    /* geometry_manager   */    (XtGeometryHandler)XtInheritGeometryManager,
    /* change_managed     */    (XtWidgetProc)XtInheritChangeManaged,
    /* insert_child	  */	XtInheritInsertChild,
    /* delete_child	  */	XtInheritDeleteChild,
    /* Extension          */    NULL,
  },
/* Constraint class Init */
  {
    NULL,
    0,
    0,
    NULL,
    NULL,
    NULL,
    NULL
  },
/* Manager Class */
   {		
      XtInheritTranslations,    		/* translations           */    
      NULL,					/* get resources      	  */
      0,					/* num get_resources 	  */
      NULL,					/* get_cont_resources     */
      0,					/* num_get_cont_resources */
      XmInheritParentProcess,                   /* parent_process         */
      NULL,					/* extension           */    
   },
/* Drawing Area - none */     
 {
     /* mumble */               0
 },
/* Graphic Widget class - none */     
 {
     /* mumble */               0
 }	
};

WidgetClass xbGraphicWidgetClass = (WidgetClass)&xbGraphicClassRec;

#ifdef _NO_PROTO
static void Initialize(request, new)
    XbGraphicWidget request, new;
#else /* _NO_PROTO */
static void Initialize(XbGraphicWidget request, XbGraphicWidget new)
#endif /* _NO_PROTO */
{
    new->graphic.gointerf = NULL;
    new->graphic.goroot = NULL;

    if (!new->graphic.editable)
	_XmWarning(new, GWMessage1);

    XtAddCallback ((Widget)new, XmNinputCallback, HandleEvent, NULL);
    XtAddEventHandler ((Widget)new, PointerMotionMask, FALSE, HandleMotionEvent, NULL);
}

#ifdef _NO_PROTO
static void InitGoInterf(gw)
    XbGraphicWidget gw;
#else
static void InitGoInterf(XbGraphicWidget gw)
#endif
{
    gw->graphic.gointerf = XmGoInterf_new((Widget)gw);
    GoRootClass_interfPostcond(GoRootSetInterf, gw, GADD);

    /* Save sizes for the scale on resize management */
    gw->graphic.old_width = gw->core.width;
    gw->graphic.old_height = gw->core.height;
}


#ifdef _NO_PROTO
static void Realize(gw, value_mask, attributes)
    XbGraphicWidget gw;
    Mask *value_mask;
    XSetWindowAttributes *attributes;
#else /* _NO_PROTO */
static void Realize(XbGraphicWidget gw, Mask *value_mask,
		    XSetWindowAttributes *attributes)
#endif /* _NO_PROTO */
{
    XmDrawingAreaWidgetClass superclass;
    superclass = (XmDrawingAreaWidgetClass) xmDrawingAreaWidgetClass;
    (*superclass->core_class.realize)((Widget)gw, value_mask, attributes);	

    InitGoInterf(gw);
}

#ifdef _NO_PROTO
static void GoRootSetInterf(w, goroot, old_interf)
    void* w;
    GoRoot* goroot;
    GoInterf* old_interf;
#else /* _NO_PROTO */
static void GoRootSetInterf(void* w, GoRoot* goroot, GoInterf* old_interf)
#endif /* _NO_PROTO */
{
    XbGraphicWidget gw = (XbGraphicWidget)w;
    if (gw->graphic.goroot &&
	(old_interf == gw->graphic.gointerf ||
	 GoRoot_interf(goroot) == gw->graphic.gointerf))
    {
        GoRoot_destroyPostcond(gw->graphic.goroot, GoRootDestroyed, (void*)gw, GREMOVE);
        if (gw->graphic.auto_resize)
            GoRoot_resizePostcond(gw->graphic.goroot, GoRootResized, (void*)gw, GREMOVE);
        gw->graphic.goroot = NULL;
    }

    if (GoRoot_interf(goroot) == gw->graphic.gointerf)
    {
        gw->graphic.goroot = goroot;
        GoRoot_destroyPostcond(gw->graphic.goroot, GoRootDestroyed, (void*)gw, GADD);
        if (gw->graphic.auto_resize)
	{
            GoRootResized((void*)gw, goroot);
            GoRoot_resizePostcond(gw->graphic.goroot, GoRootResized, (void*)gw, GADD);
	}
    }
}

#ifdef _NO_PROTO
static void Resize(gw)
    XbGraphicWidget gw;
#else /* _NO_PROTO */
static void Resize(XbGraphicWidget gw)
#endif /* _NO_PROTO */
{
    XmDrawingAreaWidgetClass superclass;

    superclass = (XmDrawingAreaWidgetClass) xmDrawingAreaWidgetClass;
        (*superclass->core_class.resize)((Widget)gw);	

    if (!XtIsRealized(gw))
	return;
    
    /* resize the gointerf */
    XmGoInterf_resize(gw->graphic.gointerf, gw->core.width, gw->core.height);

#define MIN_RESCALE 10   /* Should be a resource */
    /* scale the GoRoot if needed */
    if (gw->graphic.scale_on_resize && gw->graphic.old_width && gw->graphic.old_height)
    {
        if ((abs(gw->graphic.old_width - gw->core.width) > MIN_RESCALE) ||
	    (abs(gw->graphic.old_height - gw->core.height) > MIN_RESCALE))
	{
	    int w_per_cent = (gw->core.width * 100) / gw->graphic.old_width;
	    int h_per_cent = (gw->core.height * 100) / gw->graphic.old_height;
	    GoRoot_rescale(gw->graphic.goroot, w_per_cent, h_per_cent);
	}
    }
#undef MIN_RESCALE

    /* Save sizes for the scale on resize management */
    gw->graphic.old_width = gw->core.width;
    gw->graphic.old_height = gw->core.height;
}

#ifdef _NO_PROTO
static void GoRootResized(w, goroot)
    void* w;
    GoRoot* goroot;
#else /* _NO_PROTO */
static void GoRootResized(void* w, GoRoot* goroot)
#endif /* _NO_PROTO */
#define MIN_WIDTH  10   /* Should be resources */
#define MIN_HEIGHT 10
#define max(a, b) ((a)>(b))?(a):(b)
{
    XbGraphicWidget gw = (XbGraphicWidget)w;
    int new_width  = max(MIN_WIDTH, GoRoot_width(gw->graphic.goroot));
    int new_height = max(MIN_HEIGHT, GoRoot_height(gw->graphic.goroot));

    if (gw->drawing_area.resize_policy != XmRESIZE_NONE &&
	(new_width != gw->core.width || new_height != gw->core.height))
    {
	XtGeometryResult result;
	Dimension width_return, height_return;
	/* store and disable scaling_on resize to avoid looping for ever... */
        Boolean scaling = gw->graphic.scale_on_resize;
        gw->graphic.scale_on_resize = FALSE;

	if (gw->drawing_area.resize_policy == XmRESIZE_GROW) {
	    /* the widget should not shrink */
	    new_width  = max(new_width, gw->core.width);
	    new_height = max(new_height, gw->core.height);
	}
	
	result = XtMakeResizeRequest((Widget)gw, new_width, new_height, &width_return,
				     &height_return);
	switch (result) {
	  case XtGeometryYes:
	    Resize(gw);
	    break;
	  case XtGeometryAlmost:
	    /* check if the compromise is compatible with resize_policy */
	    if (gw->drawing_area.resize_policy == XmRESIZE_GROW) {
		width_return  = max(width_return, gw->core.width);
		height_return = max(height_return, gw->core.height);
	    }
	    /* try to resize once more */
	    result = XtMakeResizeRequest((Widget)gw, width_return, height_return,
					 NULL, NULL);
	    /* if the new request is not accepted, give up ! */
	    if (result == XtGeometryYes)
		Resize(gw);
	    break;
	  case XtGeometryNo:
	  case XtGeometryDone:
	    break;
	}
        gw->graphic.scale_on_resize = scaling;
    }
}
#undef MIN_WIDTH
#undef MIN_HEIGHT
#undef max

#ifdef _NO_PROTO
static Boolean SetValues(current, request, new)
    XbGraphicWidget current, request, new;
#else /* _NO_PROTO */
static Boolean SetValues(XbGraphicWidget current, XbGraphicWidget request,
			 XbGraphicWidget new)
#endif /* _NO_PROTO */
{
    Boolean ret = FALSE;

    if (new->graphic.auto_resize != current->graphic.auto_resize)
    {
        if (new->graphic.auto_resize)
            GoRoot_resizePostcond(new->graphic.goroot, GoRootResized, (void*)new, GADD);
        else
	    GoRoot_resizePostcond(new->graphic.goroot, GoRootResized, (void*)new, GREMOVE);
    }

    if (new->core.background_pixel != new->core.background_pixel)
        XmGoInterf_setBackground(new->graphic.gointerf, new->core.background_pixel);

    if (!new->graphic.editable) {
	_XmWarning(new, GWMessage1);
	new->graphic.editable = TRUE;
    }
    return (ret);
}


#ifdef _NO_PROTO
static void Destroy(gw)
    XbGraphicWidget gw;
#else /* _NO_PROTO */
static void Destroy(XbGraphicWidget gw)
#endif /* _NO_PROTO */
{
    GoRoot_destroyPostcond(gw->graphic.goroot, GoRootDestroyed, (void*)gw, GREMOVE);
    if (gw->graphic.auto_resize)
        GoRoot_resizePostcond(gw->graphic.goroot, GoRootResized, (void*)gw, GREMOVE);

    GoRootClass_interfPostcond(GoRootSetInterf, gw, GREMOVE);
    XmGoInterf_delete(gw->graphic.gointerf);
}

#ifdef _NO_PROTO
static void GoRootDestroyed(gw, goroot)
    void* gw;
    GoRoot* goroot;
#else /* _NO_PROTO */
static void GoRootDestroyed(void* gw, GoRoot* goroot)
#endif /* _NO_PROTO */
{
    ((XbGraphicWidget)gw)->graphic.goroot = NULL;
}


#ifdef _NO_PROTO
static void HandleMotionEvent(gw, closure, event)
    XbGraphicWidget gw;
    caddr_t closure;
    XEvent *event;
#else /* _NO_PROTO */
static void HandleMotionEvent(XbGraphicWidget gw,
			      caddr_t closure, XEvent *event)
#endif /* _NO_PROTO */
{
/*jojic    XmGoInterf_handle(gw->graphic.gointerf, event);*/
}

#ifdef _NO_PROTO
static void HandleExposeEvent(gw, event, region)
    XbGraphicWidget gw;
    XExposeEvent *event;
    Region region;
#else /* _NO_PROTO */
static void HandleExposeEvent(XbGraphicWidget gw,
			      XExposeEvent *event,
			      Region region)
#endif /* _NO_PROTO */
{
    XmDrawingAreaWidgetClass superclass;

    superclass = (XmDrawingAreaWidgetClass) xmDrawingAreaWidgetClass;
        (*superclass->core_class.expose)((Widget)gw, (XEvent*)event, region);	

/*jojic    XmGoInterf_handle(gw->graphic.gointerf, (XEvent*)event);*/
}

#ifdef _NO_PROTO
static void HandleEvent(gw, closure, call_data)
    XbGraphicWidget gw;
    caddr_t closure;
    XmDrawingAreaCallbackStruct *call_data;
#else /* _NO_PROTO */
static void HandleEvent(XbGraphicWidget gw, caddr_t closure,
			XmDrawingAreaCallbackStruct *call_data)
#endif /* _NO_PROTO */
{
/*jojic    XmGoInterf_handle(gw->graphic.gointerf, call_data->event);*/
}



/****
 * PUBLIC FUNCTIONS
 ****/


#ifdef _NO_PROTO
Widget XbCreateGraphic(parent, name, args, argCount)
    Widget   parent;
    String   name;
    ArgList  args;
    Cardinal argCount;
#else /* _NO_PROTO */
Widget XbCreateGraphic(Widget parent, String name, ArgList args, Cardinal argCount)
#endif /* _NO_PROTO */
{
    return(XtCreateWidget(name, xbGraphicWidgetClass, parent, args, argCount));
}

#ifdef _NO_PROTO
GoRoot* XbGraphicGoRoot(gw)
    XbGraphicWidget gw;
#else /* _NO_PROTO */
GoRoot* XbGraphicGoRoot(XbGraphicWidget gw)
#endif /* _NO_PROTO */
{
    return gw->graphic.goroot;
}


#ifdef _NO_PROTO
XmGoInterf* XbGraphicGoInterf(gw)
    XbGraphicWidget gw;
#else /* _NO_PROTO */
XmGoInterf* XbGraphicGoInterf(XbGraphicWidget gw)
#endif /* _NO_PROTO */
{
    if (!XtIsRealized(gw))
	return NULL;
    return gw->graphic.gointerf;
}
