#ifdef REV_INFO
static char rcsid[] = "$Id: XmGoInterf.cc,v 1.5 1992/12/10 14:05:00 jojic Exp $";
#endif
/*****************************************************************************
MODULE:  Xm INTERFACE
FILE:    XmGoInterf.cc
AUTHOR:  Philippe Smadja
DATE:    Feb 91

DESCRIPTION:
    The Xm Window System interface for Go.

    It is used with a Widget (eg the Graphic). It is the responsability
    of the widget to call after XtDispatchEvent()
    the XmGoInterf::handleXEvent()
    
    The other utility of this subclassing is to manage a XmToolkit error:
    some character (especially '\n') are received only with thr KeyRelease modifierr
    and not the KeyPress modifier.
    
COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#include <Go/XmGoInterf.h>


#define XK_MISCELLANY
#include <X11/keysymdef.h>

/**
FUNCTION
    add/rem InputMask
DESCRIPTION
    we have particular code because the standart XGoInterf does not
    select the KeyRelease event.
    It is a bug for Motif because it occurs that for some characters the KeyPress event
    may be keep  by particular widgets.
    So we have to handle KeyPress and KeyRelease.
**/

void xmgointerf_handle (void*, void* closure, XEvent* event)
{
    XmGoInterf* xminterf = (XmGoInterf*) closure;
    xminterf->handleXEvent(event);
}

void XmGoInterf::addInputMask (GEventMask mask)
{
    if (_xinputMask != 0)
	XtRemoveEventHandler
	    (widget(), (ulong)(_xinputMask & ~PointerMotionHintMask),
	     False, (XtEventHandler)xmgointerf_handle, (XtPointer)this);

    GoInterf::addInputMask(mask);
    _xinputMask =  GEventMaskToXEventMask(inputMask());
    
    if (_xinputMask & KeyPressMask)
	_xinputMask |= KeyReleaseMask;
      
    if (_xinputMask != 0)
	XtAddEventHandler
	    (widget(), (ulong)(_xinputMask & ~PointerMotionHintMask),
	     False, (XtEventHandler)xmgointerf_handle, (XtPointer)this);
}

void XmGoInterf::removeInputMask (GEventMask mask)
{
    if (_xinputMask != 0)
	XtRemoveEventHandler
	    (widget(), (ulong)(_xinputMask & ~PointerMotionHintMask),
	     False, (XtEventHandler)xmgointerf_handle, (XtPointer)this);

    GoInterf::removeInputMask(mask);
    _xinputMask =  GEventMaskToXEventMask(inputMask());
      if (_xinputMask & KeyPressMask)
	  _xinputMask |= KeyReleaseMask;
    if (_xinputMask != 0)
	XtAddEventHandler
	    (widget(), (ulong)(_xinputMask & ~PointerMotionHintMask),
	     False, (XtEventHandler)xmgointerf_handle, (XtPointer)this);
}

/**
FUNCTION
    handleXEvent
DESCRIPTION
    The function handles the KeyPress/KeyRelease if any
    and calls the standart GoInterf handler.
**/

void  XmGoInterf::handleXEvent(const XEvent* _xevent)
{
    if (_xinputMask == 0)
	addInputMask(0);

    XEvent tmp = *_xevent;
    XEvent* xevent = &tmp;
    
    switch (xevent->type) {
      case KeyPress:
      case KeyRelease:
	char c;
	KeySym keysym;
	XComposeStatus compose;
	const XKeyEvent* xkey = &xevent->xkey;
	int count = XLookupString((XKeyEvent*)xkey, &c, 1, &keysym, &compose);

	if (count && keysym == XK_Return)
	    if (xevent->type == KeyPress)
		break;
	    else
		xevent->type = KeyPress;

	if (xevent->type == KeyRelease)
	    break;
	/** no break **/
      default:
        XGoInterf::handleXEvent(xevent);
	break;
    }
}

