#ifdef REV_INFO
static char rcsid[] = "$Id: cGoGraphic.cc,v 1.2 1992/07/21 12:58:34 smadja Exp $";
#endif
/*****************************************************************************
MODULE:  Xm Go Interface
FILE:    cGoGraphic.cc
AUTHOR:  Jacques Davy
DATE:    February 1991

DESCRIPTION:
    Very partial C Binding Interface for using Go (Version 1.0) from
    the Graphic Widget

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  license from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#include <Go/machdef.h>
#include <Go/XmGoInterf.h>
#include <Go/GoRoot.h>
#include <Go/mGoRoot.h>

extern "C"
{
/**
FUNCTION
    XmGoInterf Interface
DESCRIPTION
    
**/
/* Creating a new XmGoInterf */
XmGoInterf* XmGoInterf_new (Widget w)
{
    return new XmGoInterf (w);
}

/* Deleting a XmGoInterf instance */
void XmGoInterf_delete (XmGoInterf* gointerf)
{
    if (gointerf)
        delete gointerf;
}

/* Mapping the XmGoInterf */
void XmGoInterf_map (XmGoInterf* gointerf)
{
    if (gointerf)
        gointerf->map();
}

void XmGoInterf_unmap (XmGoInterf* gointerf)
{
    if (gointerf)
        gointerf->unmap();
}

/* Resize the XmGoInterf drawable */
void XmGoInterf_resize (XmGoInterf* gointerf,
		       int new_width_in_pixels, int new_height_in_pixels)
{
    if (gointerf)
        gointerf->resize(new_width_in_pixels, new_height_in_pixels);
}

/* GSet background color */
void XmGoInterf_setBackground (XmGoInterf* gointerf, ulong pixel)
{
    if (gointerf)
        gointerf->background(pixel);
}

/* Send a XEvent to the XmGoInterf */
void XmGoInterf_handle (XmGoInterf* gointerf, const XEvent* event)
{
    if (gointerf)
        gointerf->handleXEvent(event);
}

/**
FUNCTION
    GoRoot Interface
DESCRIPTION
    
**/
GoInterf* GoRoot_interf(GoRoot* goroot)
{
    if (goroot)
        return goroot->interf();
    else
	return NULL;
}

void GoRoot_rescale(GoRoot* goroot, int w_per_cent, int h_per_cent)
{
    if (goroot)
    {
	GCoordSys tcs = GCoordSys (GPoint(0,0),
				 GVector(w_per_cent/100.0, 0),
				 GVector(0, h_per_cent/100.0));
	goroot->setCoordsys(tcs.to(goroot->Abscs()));
    }
}

int GoRoot_width(GoRoot* goroot)
{
    if (goroot)
    {
	GRegion clip;
	goroot->addClip(clip);
        return goroot->interf()->toPixelWidth(clip.bigbox().xmax) + 1;
    }
    else
        return 0;
}

int GoRoot_height(GoRoot* goroot)
{
    if (goroot)
    {
	GRegion clip;
	goroot->addClip(clip);
        return goroot->interf()->toPixelHeight(clip.bigbox().ymax) + 1;
    }
    else
        return 0;
}

void GoRootClass_interfPostcond(void func(void*, Go*, GoInterf* old_interf),
				void* closure, GAddOrRemove add)
{
    ((GoRootMetaclass*)GoRoot::metaclass())->interfPostcond(func, closure, add);
}

void GoRoot_resizePostcond(GoRoot* goroot, void func(void*, Go*), void* closure, GAddOrRemove add)
{
    if (goroot)
        goroot->clipPostcond(func, closure, add);
}

void GoRoot_destroyPostcond(GoRoot* goroot, void func(void*, const Go*), void* closure, GAddOrRemove add)
{
    if (goroot)
        goroot->destroyPostcond(func, closure, add);
}

}  /* extern "C" */
