#ifdef REV_INFO
static char rcsid[] = "$Id: GoBasicDS.cc,v 1.3 1992/12/10 13:58:57 jojic Exp $";
#endif
/*****************************************************************************
MODULE:  Go Kernel
FILE:    GoBasicDS.cc
AUTHOR:  Oliver Jojic
DATE:    April 91

DESCRIPTION:
  See the GoBasicDS.h header.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include <Go/GoBasicDS.h>
#include <Go/mGoBasicDS.h>


static GoBasicDSMetaclass* basicds_metaclass = NULL;


static GBool no_child (void*, const Go*, const Go*, const Go*(*))
{
    return GFALSE;
}


GoBasicDSMetaclass::GoBasicDSMetaclass ()
{
    addChildPrecond(no_child, NULL);
}

GoBasicDSMetaclass::~GoBasicDSMetaclass ()
{
    if (this == basicds_metaclass)
    {
        Go::metaclass()->RemoveSubclass(this);
	basicds_metaclass = NULL;
    }
}

const char* GoBasicDSMetaclass::m_classname () const
{
    return "GoBasicDS";
}

GoMetaclass* GoBasicDSMetaclass::m_superclass () const
{
    return Go::metaclass();
}

static GActionRec no_actions[] = { {NULL, NULL} };

const GActionRec* GoBasicDSMetaclass::m_actions () const
{
    return no_actions;
}

Go* GoBasicDSMetaclass::m_create () const
{
    GError("GoBasicDSMetaclass::m_create()");
    return NULL;
}


GoMetaclass* GoBasicDS::metaclass ()
{
    if (basicds_metaclass == NULL)
    {
	basicds_metaclass = new GoBasicDSMetaclass;
        Go::metaclass()->AddSubclass(basicds_metaclass);
    }

    return basicds_metaclass;
}


GoBasicDS::~GoBasicDS ()
{
}


void GoBasicDS::m_dup
    (const Go* src, Go* parcopy, Go* prevcopy, GBool donext, GBool dochildren)
{
    Go::m_dup(src, parcopy, prevcopy, donext, dochildren);

    _parent = parcopy;
    if (donext)
	for (Go* next = ((GoBasicDS*)src)->_next; next; next = next->Next())
	    if (_next = next->Dup(parcopy, this, GTRUE, dochildren))
		break;
}


GoMetaclass* GoBasicDS::m_metaclass () const
{
    return GoBasicDS::metaclass();
}


GAssoc* GoBasicDS::d_get_assoc () const
{
    return NULL;
}

Go* GoBasicDS::d_get_parent () const
{
    return _parent;
}

Go* GoBasicDS::d_get_previous () const
{
    return (_parent)
	? _parent->Child(_parent->childIndex(this) - 1)
	    : NULL;
}


Go* GoBasicDS::d_get_next () const
{
    return _next;
}


Go* GoBasicDS::d_get_first_child () const
{
    return NULL;
}


GBool GoBasicDS::d_store_children_array () const
{
    return GFALSE;
}


void GoBasicDS::d_set_previous (const Go*)
{
}


void GoBasicDS::d_set_next (const Go* go)
{
    _next = (Go*)go;
}


void GoBasicDS::d_remove_child (const Go*, int)
{
    GError("GoBasicDS::d_remove_child()");
}


void GoBasicDS::d_add_child (const Go*, int)
{
    GError("GoBasicDS::d_add_child()");
}


void GoBasicDS::d_set_tree
    (const Go* parent, const Go* /*previous*/, const Go* next)
{
    _parent = (Go*)parent;
    _next = (Go*)next;
}

GBool GoBasicDS::d_store_relcs () const
{
    return GTRUE;
}

GBool GoBasicDS::d_store_abscs () const
{
    return GFALSE;
}

const GCoordSys& GoBasicDS::d_get_relcs (const GCoordSys*) const
{
    return GUnitCoordSys();
}

void GoBasicDS::d_set_coordsys
    (const GCoordSys& relcs, const GCoordSys&, const GCoordSys&)
{
    if (relcs != GUnitCoordSys())
	GWarning("GoBasicDS::d_set_coordsys() not a unit coordsys !");
}


int GoBasicDS::save (const GoBasicDS* go, GoStream* stream)
{
    return Go::save(go, stream);
}

int GoBasicDS::readParams (GoBasicDS* go, GoStream* stream)
{
    return Go::readParams(go, stream);
}
