#ifdef REV_INFO
static char rcsid[] = "$Id: GoCond.cc,v 1.1.1.1 1992/03/18 10:35:02 uep Exp $";
#endif
/*****************************************************************************
MODULE:  Go Kernel
FILE:    GoCond.cc
AUTHOR:  Oliver Jojic
DATE:    April 91

DESCRIPTION:
  This file contains some methods dealing with pre and post conditions.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include <Go/Go.h>


GBool Go::AddOrRemoveCondition
    (const char* type, void* cb, void* closure, GAddOrRemove add)
{
    if (assoc() == NULL)
    {
	GWarning("Try to add a condition to a go without assoc");
	return GFALSE;
    }

    GCallbackRec rec((GCBProc)cb, closure);
    return
	0 < ((add == GADD)
	     ? assoc()->add(type, sizeof rec, &rec)
	     : assoc()->rem(type, sizeof rec, &rec));
}

GBool Go::CallPreconditions (const char* type) const
{
    int count;
    GCallbackRec* pcb = (GCallbackRec*)assoc()->get(type, count);

    for (int i = 0; i < count; i++, pcb++)
	if (!((GBool (*)(void*, const Go*))pcb->proc)
	    (pcb->closure, this))
	    return GFALSE;

    return GTRUE;
}

GBool Go::CallPreconditions
    (const char* type, void* a) const
{
    int count;
    GCallbackRec* pcb = (GCallbackRec*)assoc()->get(type, count);

    for (int i = 0; i < count; i++, pcb++)
	if (!((GBool (*)(void*, const Go*, void*))pcb->proc)
	    (pcb->closure, this, a))
	    return GFALSE;

    return GTRUE;
}

GBool Go::CallPreconditions
    (const char* type, void* a, void* b) const
{
    int count;
    GCallbackRec* pcb = (GCallbackRec*)assoc()->get(type, count);

    for (int i = 0; i < count; i++, pcb++)
	if (!((GBool (*)(void*, const Go*, void*, void*))pcb->proc)
	    (pcb->closure, this, a, b))
	    return GFALSE;

    return GTRUE;
}

GBool Go::CallPreconditions
    (const char* type, void* a, void* b, void* c) const
{
    int count;
    GCallbackRec* pcb = (GCallbackRec*)assoc()->get(type, count);

    for (int i = 0; i < count; i++, pcb++)
	if (!((GBool (*)(void*, const Go*, void*, void*, void*))pcb->proc)
	    (pcb->closure, this, a, b, c))
	    return GFALSE;

    return GTRUE;
}

GBool Go::CallPreconditions
    (const char* type, void* a, void* b, void* c, void* d) const
{
    int count;
    GCallbackRec* pcb = (GCallbackRec*)assoc()->get(type, count);

    for (int i = 0; i < count; i++, pcb++)
	if (!((GBool (*)(void*, const Go*, void*, void*, void*, void*))pcb->proc)
	    (pcb->closure, this, a, b, c, d))
	    return GFALSE;

    return GTRUE;
}

GBool Go::CallPreconditions
    (const char* type, void* a, void* b, void* c, void* d, void* e) const
{
    int count;
    GCallbackRec* pcb = (GCallbackRec*)assoc()->get(type, count);

    for (int i = 0; i < count; i++, pcb++)
	if (!((GBool (*)(void*, const Go*, void*, void*, void*,
			 void*, void*))pcb->proc)
	    (pcb->closure, this, a, b, c, d, e))
	    return GFALSE;

    return GTRUE;
}

void Go::CallPostconditions (const char* type)
{
    int count;
    GCallbackRec* pcb = (GCallbackRec*)assoc()->get(type, count);

    GCallCallbacks(pcb, count, this);
}

void Go::CallPostconditions (const char* type,  void* a)
{
    int count;
    GCallbackRec* pcb = (GCallbackRec*)assoc()->get(type, count);

    GCallCallbacks(pcb, count, this, a);
}

void Go::CallPostconditions
    (const char* type,  void* a, void* b)
{
    int count;
    GCallbackRec* pcb = (GCallbackRec*)assoc()->get(type, count);

    GCallCallbacks(pcb, count, this, a, b);
}

void Go::CallPostconditions
    (const char* type,  void* a, void* b, void* c)
{
    int count;
    GCallbackRec* pcb = (GCallbackRec*)assoc()->get(type, count);

    GCallCallbacks(pcb, count, this, a, b, c);
}

void Go::CallPostconditions
    (const char* type,  void* a, void* b, void* c, void* d)
{
    int count;
    GCallbackRec* pcb = (GCallbackRec*)assoc()->get(type, count);

    GCallCallbacks(pcb, count, this, a, b, c, d);
}

void Go::CallPostconditions
    (const char* type,  void* a, void* b, void* c, void* d, void* e)
{
    int count;
    GCallbackRec* pcb = (GCallbackRec*)assoc()->get(type, count);

    GCallCallbacks(pcb, count, this, a, b, c, d, e);
}
