#ifdef REV_INFO
static char rcsid[] = "$Id: GoNewDel.cc,v 1.6 1992/12/10 13:58:59 jojic Exp $";
#endif
/*****************************************************************************
MODULE:  Go Kernel
FILE:    GoNewDel.cc
AUTHOR:  Oliver Jojic
DATE:    April 91

DESCRIPTION:
  See the Go.h header.
  This file contains methods that create or destroy graphic objects.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#include <Go/GoRoot.h>
#include <Go/mGo.h>
#include <Go/Tools/Error.h>
#include <Go/GoStream.h>
#include <assert.h>


/*
 * declaration of static members:
 */
int Go::differ_destroy;

/*
 * pre and postcondition types:
 */
const char goNcreatePrecond[] = "createPrecond";
const char goNcreatePostcond[] = "createPostcond";
const char goNdupPrecond[] = "dupPrecond";
const char goNdupPostcond[] = "dupPostcond";
const char goNdestroyPrecond[] = "destroyPrecond";
const char goNdestroyPostcond[] = "destroyPostcond";
const char goNsavePrecond[] = "savePrecond";
const char goNsavePostcond[] = "savePostcond";
const char goNrestorePostcond[] = "restorePostcond";


void GoMetaclass::createPrecond
    (GBool proc(void*, const GoMetaclass*),
     void* closure, GAddOrRemove add)
{
    has_create_precond =
	AddOrRemoveCondition(goNcreatePrecond, proc, closure, add);
}

void GoMetaclass::createPostcond
    (void cb(void*, Go*),
     void* closure, GAddOrRemove add)
{
    has_create_postcond =
        AddOrRemoveCondition(goNcreatePostcond, cb, closure, add);
}

void Go::dupPrecond
    (GBool cb(void*, const Go*), void* closure, GAddOrRemove add)
{
    has_dup_precond =
	AddOrRemoveCondition(goNdupPrecond, cb, closure, add);
}

void Go::dupPostcond
    (void cb(void*, Go*, Go*), void* closure, GAddOrRemove add)
{
    has_dup_postcond =
	AddOrRemoveCondition(goNdupPostcond, cb, closure, add);
}

void GoMetaclass::dupPrecond
    (GBool cb(void*, const Go*),
     void* closure, GAddOrRemove add)
{
    has_dup_precond =
	AddOrRemoveCondition(goNdupPrecond, cb, closure, add);
}

void GoMetaclass::dupPostcond
    (void cb(void*, Go*, Go*),
     void* closure, GAddOrRemove add)
{
    has_dup_postcond =
	AddOrRemoveCondition(goNdupPostcond, cb, closure, add);
}

void Go::destroyPrecond
    (GBool cb(void*, const Go*), void* closure, GAddOrRemove add)
{
    has_destroy_precond =
	AddOrRemoveCondition(goNdestroyPrecond, cb, closure, add);
}

void Go::destroyPostcond
    (void cb(void*, const Go*), void* closure, GAddOrRemove add)
{
    has_destroy_postcond =
	AddOrRemoveCondition(goNdestroyPostcond, cb, closure, add);
}

void GoMetaclass::destroyPrecond
    (GBool cb(void*, const Go*), void* closure, GAddOrRemove add)
{
    has_destroy_precond =
	AddOrRemoveCondition(goNdestroyPrecond, cb, closure, add);
}

void GoMetaclass::destroyPostcond
    (void cb(void*, const Go*), void* closure, GAddOrRemove add)
{
    has_destroy_postcond =
	AddOrRemoveCondition(goNdestroyPostcond, cb, closure, add);
}

void Go::savePrecond
    (GBool cb(void*, const Go*), void* closure, GAddOrRemove add)
{
    has_save_precond =
	AddOrRemoveCondition(goNsavePrecond, cb, closure, add);
}

void GoMetaclass::savePrecond
    (GBool cb(void*, const Go*), void* closure, GAddOrRemove add)
{
    has_save_precond =
	AddOrRemoveCondition(goNsavePrecond, cb, closure, add);
}

void Go::savePostcond
    (void cb(void*, const Go*, GoStream*), void* closure, GAddOrRemove add)
{
    has_save_postcond =
	AddOrRemoveCondition(goNsavePostcond, cb, closure, add);
}

void GoMetaclass::savePostcond
    (void cb(void*, const Go*, GoStream*), void* closure, GAddOrRemove add)
{
    has_save_postcond =
	AddOrRemoveCondition(goNsavePostcond, cb, closure, add);
}

void GoMetaclass::restorePostcond
    (void cb(void*, Go*, GoStream*), void* closure, GAddOrRemove add)
{
    has_restore_postcond =
	AddOrRemoveCondition(goNrestorePostcond, cb, closure, add);
}

Go::Go ()
{
    cross_type = GCROSS_AS_NODE;
    _mappable = GTRUE;
    _selected = GFALSE;
    clip_not_ready();
    my_clip_was_updated = GFALSE;
    common_ancestor_mark = GFALSE;

    has_dup_precond = has_dup_postcond = GFALSE;
    has_destroy_precond = has_destroy_postcond = GFALSE;
    has_save_precond = has_save_postcond = GFALSE;
    has_shift_precond = has_shift_postcond = GFALSE;
    has_reparent_precond = has_reparent_postcond = GFALSE;
    has_remove_child_precond = has_remove_child_postcond = GFALSE;
    has_add_child_precond = has_add_child_postcond = GFALSE;
    has_root_precond = has_root_postcond = GFALSE;
    has_interf_precond = has_interf_postcond = GFALSE;
    has_cross_type_precond = has_cross_type_postcond =GFALSE;
    has_relcs_precond = has_relcs_postcond = GFALSE;
    has_abscs_precond = has_abscs_postcond = GFALSE;
    has_map_precond = has_map_postcond = GFALSE;
    has_select_precond = has_select_postcond = GFALSE;
    has_clip_postcond = has_my_clip_postcond = GFALSE;

    draw_this = GFALSE;
    draw_subtree = GFALSE;
    is_to_be_destroyed = GFALSE;
}


Go::~Go ()
{
    assert (differ_destroy == 0);
}



GBool Go::call_destroy_precond () const
{
    return (has_destroy_precond)
	? CallPreconditions(goNdestroyPrecond)
	    : GTRUE;
}

GBool Go::call_class_destroy_precond () const
{
    for (GoMetaclass* mc = getMetaclass(); mc; mc = mc->superclass())
	if (mc->has_destroy_precond &&
	    !mc->CallPreconditions(goNdestroyPrecond, this))
	    return GFALSE;

    return GTRUE;
}

void Go::call_destroy_postcond ()
{
    if (has_destroy_postcond)
	CallPostconditions(goNdestroyPostcond, this);
}

GBool Go::callSavePrecond () const
{
    if (has_save_precond &&
	!CallPreconditions(goNsavePrecond))
	return GFALSE;

    for (GoMetaclass* mc = getMetaclass(); mc; mc = mc->superclass())
	if (mc->has_save_precond &&
	    !mc->CallPreconditions(goNsavePrecond, this))
	    return GFALSE;

    return GTRUE;
}

void Go::call_save_postcond (GoStream* stream) const
{
    if (has_save_postcond)
	((Go*)this)->CallPostconditions(goNsavePostcond, stream);
}

void Go::call_class_destroy_postcond ()
{
    for (GoMetaclass* mc = getMetaclass(); mc; mc = mc->superclass())
	if (mc->has_destroy_postcond)
	    mc->CallPostconditions(goNdestroyPostcond, this);
}



static GStack& GoToBeDestroyed() {
    static GStack *go_to_be_destroyed = NULL;

    if (! go_to_be_destroyed)
	go_to_be_destroyed = new GStack();

    return *go_to_be_destroyed;
}

void Go::DifferDestroy()
{
    assert (differ_destroy >= 0);
    differ_destroy++;
}



// static function; no this parameter;

void Go::HandleDestroy()
{
    assert (differ_destroy > 0 );
    if (--differ_destroy == 0)
	while (!GoToBeDestroyed().empty())
	{
	    Go* go = (Go*)GoToBeDestroyed().pop();
	    
	    assert (go->is_to_be_destroyed);
	    assert (go->parent() == NULL);

	    // destroy go and children: postcond are called yet;
	    go->effective_destroy_recur();
	}
}

GBool Go::WillBeDestroyed() const
{
    if (is_to_be_destroyed)
	return GTRUE;

    int count;
    Go** go = (Go**) GoToBeDestroyed().data(count);
    for (int i = 0 ; i < count; i++, go++)
	if ((*go)->isAncestor(this))
	    return GTRUE;

    return GFALSE;
}



GBool Go::destroy ()
{
    // prevent multi destroy requests;
    if (WillBeDestroyed())
    {
	// postcond are called yet;
	return GTRUE;
    }
    
    // preserve lib;
    DifferDestroy();
	
    // call destroy conditions;

    GBool destroy_is_accepted = virtual_destroy(GTRUE);
    if (destroy_is_accepted)
    {
	// this has to be destroyed by HandleDestroy;

	GoToBeDestroyed().push(this);
	is_to_be_destroyed = GTRUE;

	// destroy children;
	virtual_destroy_recur();
    }

	
    // handle differed destroy request(s);
    // included this if destroy_is_accepted;

    HandleDestroy();

    // return destroyed or not;
    return destroy_is_accepted;
}


void Go::virtual_destroy_recur()
{
    assert (differ_destroy > 0);

    Go* child = Child();
    Go* prev = NULL;
    for (int i = 0; child;)
    {
	(void)child->virtual_destroy(GFALSE);
	if (child->parent() == this)
	{
	    child->virtual_destroy_recur();
	    prev = child;
	    child = child->Next(i++);
	}
	else
	{
	    // child is reparent to another parent;
	    if (prev == NULL)
		child = this->Child();
	    else
		child =  prev->Next(i);
	}
    }
}


inline GBool Go::callDestroyPrecond ()
{
    return
	call_destroy_precond ()
	    && call_class_destroy_precond ();
}


GBool Go::virtual_destroy(GBool father)
{
    // virtual destroy is to  call conditions;

    // PRECOND;
    
    if (! callDestroyPrecond ())
    {
	if (!father)
	{
	    // application refuses destroy of a child;
	    // propose to application the NULL new parent;
	    
	    GWarning("application refuses destroy of a child --> reparent to NULL");

	    Go* old_par = parent();

	    reparent(NULL);

	    if (old_par == parent())
	    {
		GError("Go::destroy() application refuses destroy AND reparent to another parent");
		// not reached;
	    }
	}
	
	// in all case:
	return  GFALSE;
    }
    
    if (father)
    {
	// propose to application the NULL new parent;
	reparent(NULL);

	if (parent())
	{
	    GWarning("Go::destroy() applications accepts destroy but refuses reparent to NULL");
	    GWarning(" --> cannot destroy this: %x", this);
	    return GFALSE;
	}
    }

    // POSTCOND;
    
    call_destroy_postcond();
    call_class_destroy_postcond();
    return GTRUE;
}


void Go::effective_destroy_recur()
{
    assert (differ_destroy == 0);
    Go* child;
    while (child=Child())
    {
	child->force_reparent(NULL);
	child->effective_destroy_recur();
    }

    getMetaclass()->_instances.remove(this);
    delete this;
}



Go* Go::create (const char* name)
{
    GoMetaclass* mc = Go::metaclass(name);
    if (mc == NULL)
    {
	GWarning("Go::create(\"%s\"): there is no such go class", name);
	return NULL;
    }

    return Go::Create(mc);
}


/*
 * This method creates a new Go of class "mc".
 * This method checks before that no create precondition will refuse
 * the creation.
 * After a successfull creation, this method will call create postconditions.
 */
Go* Go::Create (GoMetaclass* mc)
{
    if (!Go::call_class_create_precond(mc))
	return NULL;

    Go* go = mc->m_create();
    mc->_instances.add(go);

    // preverse lib;
    DifferDestroy();

    go->call_class_create_postcond();

    if (go->is_to_be_destroyed)
	go = NULL;
    
    HandleDestroy();
    
    return go;
}


/*
 * This method calls create preconditions
 * on every superclass of "mclass".
 * If one of these preconditions do not agree
 * with the creation of a new go (i.e. returns FALSE),
 * this method will return FALSE.
 */
GBool Go::call_class_create_precond (const GoMetaclass* mclass)
{
    for (const GoMetaclass* mc = mclass; mc; mc = mc->superclass())
	if (mc->has_create_precond)
	{
	    int count;
	    struct CBRec {
		GBool (*cb)(void*, const GoMetaclass*);
		void* closure;
	    } *pcb = (CBRec*)mc->_assoc.get(goNcreatePrecond, count);

	    for (int i = 0; i < count; i++, pcb++)
		if (!(*pcb->cb)(pcb->closure, mclass))
		    return GFALSE;
	}

    return GTRUE;
}


/*
 * This method calls create postconditions
 * on every superclass of "go".
 */
void Go::call_class_create_postcond ()
{
    for (GoMetaclass* mc = getMetaclass(); mc; mc = mc->superclass())
	if (mc->has_create_postcond)
	    mc->CallPostconditions(goNcreatePostcond, this);
}


inline GBool Go::call_dup_precond () const
{
    return (has_dup_precond)
	? CallPreconditions(goNdupPrecond)
	    : GTRUE;
}

inline void Go::call_dup_postcond (Go* copy)
{
    if (has_dup_postcond)
	CallPostconditions(goNdupPostcond, copy);
}

GBool Go::call_class_dup_precond () const
{
    for (GoMetaclass* mc = getMetaclass(); mc; mc = mc->superclass())
	if (mc->has_dup_precond &&
	    !mc->CallPreconditions(goNdupPrecond, this))
	    return GFALSE;

    return GTRUE;
}

void Go::call_class_dup_postcond (Go* copy)
{
    for (GoMetaclass* mc = getMetaclass(); mc; mc = mc->superclass())
	if (mc->has_dup_postcond)
	    mc->CallPostconditions(goNdupPostcond, this, copy);
}


inline GBool Go::callDupPrecond () const
{
    return
	call_dup_precond()
	    && call_class_dup_precond();
}


Go* Go::Dup
    (Go* parentCopy, Go* prevBrotherCopy,
     GBool dupNextBrother, GBool dupChildren)
{
    if (!call_dup_precond() ||
	!call_class_dup_precond())
	return NULL;

    Go* copy = Go::Create(getMetaclass());
    if (copy == NULL)
	return NULL;

    copy->m_dup(this, parentCopy, prevBrotherCopy, dupNextBrother, dupChildren);
    

    DifferDestroy();

    call_dup_postcond(copy);
    call_class_dup_postcond(copy);

    if (copy->is_to_be_destroyed)
	copy = NULL;

    HandleDestroy();

    return copy;
}


void Go::check () const
{
    if (Go::metaclass("Go") == NULL)
	GWarning("Go::check(): no Go class in class info set");
}


Go** Go::d_get_children (int& count) const
{
    GWarning("Go::d_get_children_array() called");
    count = 0;
    return NULL;
}

const GCoordSys& Go::d_get_relcs (const GCoordSys*) const
{
    GError("GoNewDel.cc: d_get_relcs was called");
    return GUnitCoordSys();
}

const GCoordSys& Go::d_get_abscs (const GCoordSys*) const
{
    GError("GoNewDel.cc: d_get_abscs was called");
    return GUnitCoordSys();
}


void Go::m_dup (const Go* orig, Go*, Go*, GBool, GBool)
{
    cross_type = orig->cross_type;
    _mappable = orig->_mappable;
    _selected = orig->_selected;
}


int Go::save (const Go* go, GoStream* stream)
{
    if (stream->writeChar(go->cross_type, "cross-type") &&
	stream->writeBool(go->_mappable, "mappable"))
    {
	go->call_save_postcond(stream);
	return GTRUE;
    }
    else
	return GFALSE;
}

void Go::call_class_restore_postcond (GoStream* stream)
{
    for (GoMetaclass* mc = getMetaclass(); mc; mc = mc->superclass())
	if (mc->has_restore_postcond)
	    mc->CallPostconditions(goNrestorePostcond, this, stream);
}

int Go::readParams (Go* go, GoStream* stream)
{
    char crtyp;
    GBool map;

    if (!(stream->readChar(&crtyp) && stream->readBool(&map)))
	return GFALSE;

    go->cross_type = crtyp;
    go->_mappable = map;
 
    go->call_class_restore_postcond(stream);
    
    return GTRUE;
}
