#ifdef REV_INFO
static char rcsid[] = "$Id: GoOutput.cc,v 1.8 1992/12/18 09:42:34 jojic Exp $";
#endif
/*****************************************************************************
MODULE:  Go Kernel
FILE:    GoOutput.cc
AUTHOR:  Oliver Jojic
DATE:    April 91

DESCRIPTION:
  This file contains methods dealing with go outputs.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include <Go/Go.h>
#include <Go/mGo.h>
#include <Go/GoRoot.h>
#include <assert.h>


/*
 * pre and post condition types:
 */
char goNmapPrecond[] = "mapPrecond";
char goNmapPostcond[] = "mapPostcond";
char goNselectPrecond[] = "selectPrecond";
char goNselectPostcond[] = "selectPostcond";


void Go::mapPrecond
    (GBool proc(void*, const Go*), void* closure, GAddOrRemove add)
{
    has_map_precond =
	AddOrRemoveCondition(goNmapPrecond, proc, closure, add);
}

void Go::mapPostcond
    (void cb(void*, Go*), void* closure, GAddOrRemove add)
{
    has_map_postcond =
	AddOrRemoveCondition(goNmapPostcond, cb, closure, add);
}

void Go::selectPrecond
    (GBool cb(void*, const Go*), void* closure, GAddOrRemove add)
{
    has_select_precond =
	AddOrRemoveCondition(goNselectPrecond, cb, closure, add);
}

void Go::selectPostcond
    (void cb(void*, Go*), void* closure, GAddOrRemove add)
{
    has_select_postcond =
	AddOrRemoveCondition(goNselectPostcond, cb, closure, add);
}

void GoMetaclass::mapPrecond
    (GBool cb(void*, const Go*),
     void* closure, GAddOrRemove add)
{
    has_map_precond =
	AddOrRemoveCondition(goNmapPrecond, cb, closure, add);
}

void GoMetaclass::mapPostcond
    (void cb(void*, Go*),
     void* closure, GAddOrRemove add)
{
    has_map_postcond =
	AddOrRemoveCondition(goNmapPostcond, cb, closure, add);
}

void GoMetaclass::selectPrecond
    (GBool cb(void*, const Go*), void* closure, GAddOrRemove add)
{
    has_select_precond =
	AddOrRemoveCondition(goNselectPrecond, cb, closure, add);
}

void GoMetaclass::selectPostcond
    (void cb(void*, Go*), void* closure, GAddOrRemove add)
{
    has_select_postcond =
	AddOrRemoveCondition(goNselectPostcond, cb, closure, add);
}


GBool Go::mapped () const
{
    for (const Go* go = this; go; go = go->parent())
	if (!go->_mappable)
	    return GFALSE;

    return GTRUE;
}


GBool Go::contains (const GPoint& p, GNumber maxdist) const
{
    if (crossType() != GCROSS_AS_LEAF)
	return m_contains(p, maxdist);
    else
	return subtreeContains (p, maxdist);
}



void _gooutput_subtree_contains_proc (void*, Go* go)
{
    go->stopRecursion((void*)GTRUE);
}

GBool Go::subtreeContains (const GPoint& p, GNumber maxdist) const
{
    if (((Go*)this)->ForEveryGoContainingPoint
	(_gooutput_subtree_contains_proc, NULL, p, maxdist))
	return GTRUE ;
    else
	return GFALSE;

}


void _gooutput_add_opclip_proc(void* closure, Go* go)
{
    GRegion* opclip = (GRegion*)closure;
    go->AddMyOpaqueClip(*opclip);
}

GRegion Go::opaqueClip () const
{
    GRegion opaque;
    ((Go*)this)->ForEveryMappedGo(_gooutput_add_opclip_proc, &opaque);
    return opaque;
}


void Go::addMyClip (GRegion& r) const
{
    if (crossType() != GCROSS_AS_LEAF)
	AddMyClip (r);
    else
	addClip (r);
}


void Go::AddMyClip (GRegion& r) const
{
    if (Child() == NULL)
    {
	GRegion* stored_clip = d_get_clip();
	if (stored_clip)
	{
	    if (!clip_is_ready)
	    {
		stored_clip->setEmpty();
		m_add_clip(*stored_clip);
		((Go*)this)->clip_is_ready = GTRUE;
	    }

	    r += *stored_clip;
	    return;
	}
    }
    
    m_add_clip(r);
}


void Go::addClip (GRegion& region) const
{
    GRegion* stored_clip = d_get_clip();

    if (clip_is_ready && stored_clip)
    {
	region += *stored_clip;
	return;
    }

    if (stored_clip == NULL)
    {
	AddMyClip(region);
	Go* child = Child();
	for (int i = 0; child; child = child->Next(i++))
	    if (child->mappable())
		child->addClip(region);
    }
    else
    {
	stored_clip->setEmpty();
	AddMyClip(*stored_clip);
	Go* child = Child();
	for (int i = 0; child; child = child->Next(i++))
	    if (child->mappable())
	    {
		child->addClip(*stored_clip);
#define MAX_BOXES_COUNT 512
		if (stored_clip->numboxes() > MAX_BOXES_COUNT)
		{
		    *stored_clip = stored_clip->bigbox();
		    child = child->Next(i++);
		    for (; child; child = child->Next(i++))
			if (child->mappable())
			{
			    child->addClip(*stored_clip);
			    *stored_clip = stored_clip->bigbox();
			}

		    break;
		}
	    }
	
	region += *stored_clip;
    }

    ((Go*)this)->clip_is_ready = GTRUE;
}


INLINE GBool Go::call_map_precond () const
{
    return (has_map_precond)
	? CallPreconditions(goNmapPrecond)
	    : GTRUE;
}

GBool Go::call_class_map_precond () const
{
    for (GoMetaclass* mc = getMetaclass(); mc; mc = mc->superclass())
	if (mc->has_map_precond &&
	    !mc->CallPreconditions(goNmapPrecond, this))
	    return GFALSE;

    return GTRUE;
}


INLINE void Go::call_map_postcond ()
{
    if (has_map_postcond)
	CallPostconditions(goNmapPostcond);
}

void Go::call_class_map_postcond ()
{
    for (GoMetaclass* mc = getMetaclass(); mc; mc = mc->superclass())
	if (mc->has_map_postcond)
	    mc->CallPostconditions(goNmapPostcond, this);
}


GBool Go::callMapPrecond (GBool mapOrUnmap) const
{
    return
	mapOrUnmap == mappable()
	    || call_map_precond()
		&& call_class_map_precond();
}


GBool Go::map ()
{
    if (_mappable)
	return GTRUE;

    if (_mappable ||
	!call_map_precond() ||
	!call_class_map_precond())
	return GFALSE;

    TempMap();
    
    // preserve lib ;
    DifferDestroy();
    
    call_map_postcond();
    call_class_map_postcond();

    HandleDestroy();
    return GTRUE;
}


GBool Go::unmap ()
{
    if (!_mappable ||
	!call_map_precond() ||
	!call_class_map_precond())
	return GFALSE;

    TempUnmap();

    // preverve lib;
    DifferDestroy();
    
    call_map_postcond();
    call_class_map_postcond();

    HandleDestroy();
    return GTRUE;
}

void Go::TempUnmap ()
{
    assert(_mappable);
    
    _mappable = GFALSE;

    Go* par = parent();
    if (par == NULL)
    {
	// this is the root of the go tree:
	if (this->is_root())
	    ((GoRoot*)this)->redraw_from_unmap(this);
	return;
    }
	  
    /* Do not check for an empty clip (do_not_have_drawings)
       if the object is not reparented to a GoRoot.
       Why? Because the clip of an object would be called in
       a Go constructor, which is usefull only if the object
       is already under a GoRoot. */
    Go* goroot = Root();
    if (goroot->is_root() && do_not_have_drawings())
	return;

    Go* nextpar;
    do // clips of parents are out of date now
    {
	if (par->clip_was_updated && !par->clip_is_ready)
	    // clips of this parent are already out of date:
	    break;
	
	par->clip_not_ready();

	// if this par is not mapped, its clips may not;
	// alter clips of its parent:
	if (!par->_mappable)
	    return; // also we have nothing to draw;
    }
    while ((nextpar = par->parent()) && (par = nextpar));

    // here, par is never NULL:
    if (par->mapped() && goroot->is_root())
	((GoRoot*) goroot)->redraw_from_unmap (this);
}

void Go::TempMap ()
{
    if (_mappable)
	return;

    _mappable = GTRUE;

    Go* par = parent();
    if (par == NULL)
    {
	// this is the root of the go tree:
	if (this->is_root())
	    ((GoRoot*)this)->redraw_from_map(this);
	return;
    }

    /* See the TempUnmap() comment */
    Go* goroot = Root();
    if (goroot->is_root() && do_not_have_drawings())
	return;

    Go* nextpar;
    do // clips of parents are out of date now
    {
	if (par->clip_was_updated && !par->clip_is_ready)
	    // clips of this parent are already out of date:
	    break;

	par->clip_not_ready();

	// if this par is not mapped, its clips may not;
	// alter clips of its parent:
	if (!par->_mappable)
	    return; // also we have nothing to draw;
    }
    while ((nextpar = par->parent()) && (par = nextpar));

    // here, par is never NULL:
    if (par->mapped() && goroot->is_root())
	((GoRoot*) goroot)->redraw_from_map (this);
}


INLINE GBool Go::call_select_precond () const
{
    return (has_select_precond)
	? CallPreconditions(goNselectPrecond)
	    : GTRUE;
}


INLINE void Go::call_select_postcond ()
{
    if (has_select_postcond)
	CallPostconditions(goNselectPostcond);
}

GBool Go::call_class_select_precond () const
{
    for (GoMetaclass* mc = getMetaclass(); mc; mc = mc->superclass())
	if (mc->has_select_precond &&
	    !mc->CallPreconditions(goNselectPrecond, this))
	    return GFALSE;

    return GTRUE;
}

void Go::call_class_select_postcond ()
{
    for (GoMetaclass* mc = getMetaclass(); mc; mc = mc->superclass())
	if (mc->has_select_postcond)
	    mc->CallPostconditions(goNselectPostcond, this);
}


GBool Go::callSelectPrecond (GBool selectOrUnselect) const
{
    return
	selectOrUnselect == _selected
	    || call_select_precond()
		&& call_class_select_precond();
}


GBool Go::select ()
{
    if (_selected ||
	!call_select_precond() ||
	!call_class_select_precond())
	return GFALSE;

    _selected = GTRUE;

    // preverse lib;
    DifferDestroy();
    
    call_select_postcond();
    call_class_select_postcond();

    HandleDestroy();
    return GTRUE;
}


GBool Go::unselect ()
{
    if (!_selected ||
	!call_select_precond() ||
	!call_class_select_precond())
	return GFALSE;

    _selected = GFALSE;

    // preverse lib;
    DifferDestroy();
    
    call_select_postcond();
    call_class_select_postcond();

    HandleDestroy();
    return GTRUE;
}



void Go::MyClipHasChanged ()
{
    my_clip_was_updated = GTRUE;

    for (Go* par = this; par; par = par->parent())
    {
	if (par->clip_was_updated && !par->clip_is_ready)
	    break;
	par->clip_not_ready();

	if (!par->_mappable)
	    break;
    }
}


void Go::RedrawForeOnly (const GRegion& region)
{
    GoRoot* goroot = root();
    if (goroot)
	goroot->redrawForeOnly(this,  region);
}


GRegion* Go::d_get_clip () const
{
    return NULL;
}


GBool Go::m_contains (const GPoint&, GNumber) const
{
    return GFALSE;
}


void Go::m_add_clip (GRegion&) const
{
}


void Go::m_add_opaque_clip (GRegion&) const
{
}


void Go::m_draw (const GRegion&) const
{
}


void Go::m_wired_draw (const GCoordSys&, const GPen&, GQuality) const
{
}


void Go::draw () const
{
    m_draw (GBiggestRegion());
}


GRegion Go::myClip () const
{
    if (crossType() != GCROSS_AS_LEAF)
	return MyClip();
    else
	return clip();
}


GRegion Go::MyClip () const
{
    GRegion myclip;
    addMyClip (myclip);
    return myclip;
}


GRegion Go::myOpaqueClip () const
{
    return MyOpaqueClip();
}


GRegion Go::MyOpaqueClip () const
{
    GRegion myopaque;
    AddMyOpaqueClip(myopaque);
    return myopaque;
}


void Go::addMyOpaqueClip (GRegion& r) const
{
    AddMyOpaqueClip (r);
}


GBool Go::do_not_have_drawings () const
{
    /* static to avoid calling region constructor recursively */
    STATIC_OBJECT (GRegion, region);

    m_add_clip (region);
    if (!region.empty())
    {
	region.setEmpty();
	return GFALSE;
    }
    
    for (Go* child = Child(); child; child = child->Next())
	if (!child->do_not_have_drawings())
	    return GFALSE;

    return GTRUE;
}
