#ifdef REV_INFO
static char rcsid[] = "$Id: GoStdDS.cc,v 1.2 1992/12/10 13:59:08 jojic Exp $";
#endif
/*****************************************************************************
MODULE:  Go Kernel
FILE:    GoStdDS.cc
AUTHOR:  Oliver Jojic
DATE:    April 91

DESCRIPTION:
  See the GoStdDS.h header.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include <Go/GoStdDS.h>
#include <Go/mGoStdDS.h>
#include <Go/GoTreeDS.h>
#include <Go/GoStream.h>


static GoStdDSMetaclass* relds_metaclass = NULL;

GoStdDSMetaclass::GoStdDSMetaclass ()
{
}

GoStdDSMetaclass::~GoStdDSMetaclass ()
{
    if (this == relds_metaclass)
    {
        Go::metaclass()->RemoveSubclass(this);
	relds_metaclass = NULL;
    }
}

const char* GoStdDSMetaclass::m_classname () const
{
    return "GoStdDS";
}

GoMetaclass* GoStdDSMetaclass::m_superclass () const
{
    return Go::metaclass();
}

static GActionRec no_actions[] = { {NULL, NULL} };

const GActionRec* GoStdDSMetaclass::m_actions () const
{
    return no_actions;
}

Go* GoStdDSMetaclass::m_create () const
{
    GError("GoStdDSMetaclass::m_create");
    return NULL;
}

GoMetaclass* GoStdDS::metaclass ()
{
    if (relds_metaclass == NULL)
    {
	relds_metaclass = new GoStdDSMetaclass;
        relds_metaclass->superclass()->AddSubclass(relds_metaclass);
    }

    return relds_metaclass;
}

GoStdDS::GoStdDS ()
{
    _parent = NULL;
    _next = NULL;
    _child = NULL;
}

GoStdDS::~GoStdDS ()
{
}

void GoStdDS::m_dup
    (const Go* src, Go* parcopy, Go* prevcopy, GBool donext, GBool dochildren)
{
    Go::m_dup(src, parcopy, prevcopy, donext, dochildren);

    const GoStdDS* orig = (GoStdDS*)src;
    _parent = parcopy;
    _relcs = orig->_relcs;

    if (dochildren)
	for (Go* orig_ch = orig->_child; orig_ch; orig_ch = orig_ch->Next())
	    if (_child = orig_ch->Dup(this, NULL, GTRUE, GTRUE))
		break;

    if (donext)
	for (Go* orig_nxt = orig->_next; orig_nxt; orig_nxt = orig_nxt->Next())
	    if (_next = orig_nxt->Dup(parcopy, this, GTRUE, dochildren))
		break;
}
	
GoMetaclass* GoStdDS::m_metaclass () const
{
    return GoStdDS::metaclass();
}

GAssoc* GoStdDS::d_get_assoc () const
{
    const GAssoc* a = &_assoc;
    return (GAssoc*)a;
}

Go* GoStdDS::d_get_parent () const
{
    return _parent;
}

Go* GoStdDS::d_get_previous () const
{
    return (_parent)
	? _parent->Child(_parent->childIndex(this) - 1)
	    : NULL;
}

Go* GoStdDS::d_get_next () const
{
    return _next;
}

Go* GoStdDS::d_get_first_child () const
{
    return _child;
}

GBool GoStdDS::d_store_children_array () const
{
    return GFALSE;
}

void GoStdDS::d_set_previous (const Go*)
{
}

void GoStdDS::d_set_next (const Go* go)
{
    _next = (Go*)go;
}

void GoStdDS::d_remove_child (const Go* go, int)
{
    if (go == _child)
	_child = go->Next();
}

void GoStdDS::d_add_child (const Go* go, int index)
{
    if (index == 0)
	_child = (Go*)go;
}

void GoStdDS::d_set_tree (const Go* parent, const Go*, const Go* next)
{
    _parent = (Go*)parent;
    _next = (Go*)next;
}

GBool GoStdDS::d_store_abscs () const
{
    return GFALSE;
}

GBool GoStdDS::d_store_relcs () const
{
    return GTRUE;
}

const GCoordSys& GoStdDS::d_get_relcs (const GCoordSys*) const
{
    return _relcs;
}

void GoStdDS::d_set_coordsys
    (const GCoordSys& relcs, const GCoordSys&, const GCoordSys&)
{
    _relcs = relcs;
}

GRegion* GoStdDS::d_get_clip () const
{
    const GRegion* region = &_clip;
    return (GRegion*)region;
}


int GoStdDS::save (const GoStdDS* go, GoStream* stream)
{
    extern int save_children(const Go*, GoStream*);

    return
	stream->writeCoordSys(((GoStdDS*)go)->_relcs, "relcs") &&
	save_children(go->_child, stream) &&
	Go::save(go, stream);
}

int GoStdDS::readParams (GoStdDS* go, GoStream* stream)
{
    extern int restore_children(Go*, GoStream*);

    return
	stream->readCoordSys(&go->_relcs) &&
	restore_children(go, stream) &&
	Go::readParams(go, stream);
}

