#ifdef REV_INFO
static char rcsid[] = "$Id: GoTreeDS.cc,v 1.4 1992/12/10 13:59:12 jojic Exp $";
#endif
/*****************************************************************************
MODULE:  Go Kernel
FILE:    GoTreeDS.cc
AUTHOR:  Oliver Jojic
DATE:    April 91

DESCRIPTION:
  See the GoTreeDS.h header.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include <Go/GoTreeDS.h>
#include <Go/mGoTreeDS.h>
#include <Go/GoStream.h>


static GoTreeDSMetaclass* treeds_metaclass = NULL;

GoTreeDSMetaclass::~GoTreeDSMetaclass ()
{
    if (this == treeds_metaclass)
    {
        Go::metaclass()->RemoveSubclass(this);
	treeds_metaclass = NULL;
    }
}

const char* GoTreeDSMetaclass::m_classname () const
{
    return "GoTreeDS";
}

GoMetaclass* GoTreeDSMetaclass::m_superclass () const
{
    return Go::metaclass();
}

static GActionRec no_actions[] = { {NULL, NULL} };

const GActionRec* GoTreeDSMetaclass::m_actions () const
{
    return no_actions;
}

Go* GoTreeDSMetaclass::m_create () const
{
    GError("GoTreeDSMetaclass::m_create");
    return NULL;
}

GoMetaclass* GoTreeDS::metaclass ()
{
    if (treeds_metaclass == NULL)
    {
	treeds_metaclass = new GoTreeDSMetaclass;
        treeds_metaclass->superclass()->AddSubclass(treeds_metaclass);
    }

    return treeds_metaclass;
}

GoTreeDS::~GoTreeDS ()
{
}

void GoTreeDS::m_dup
    (const Go* src, Go* parcopy, Go* prevcopy, GBool donext, GBool dochildren)
{
    Go::m_dup(src, parcopy, prevcopy, donext, dochildren);

    const GoTreeDS* orig = (GoTreeDS*)src;
    _parent = parcopy;

    if (dochildren)
	for (Go* orig_ch = orig->_child; orig_ch; orig_ch = orig_ch->Next())
	    if (_child = orig_ch->Dup(this, NULL, GTRUE, GTRUE))
		break;

    if (donext)
	for (Go* orig_nxt = orig->_next; orig_nxt; orig_nxt = orig_nxt->Next())
	    if (_next = orig_nxt->Dup(parcopy, this, GTRUE, dochildren))
		break;
}
	
GoMetaclass* GoTreeDS::m_metaclass () const
{
    return GoTreeDS::metaclass();
}

GAssoc* GoTreeDS::d_get_assoc () const
{
    return NULL;
}

Go* GoTreeDS::d_get_parent () const
{
    return _parent;
}

Go* GoTreeDS::d_get_previous () const
{
    return (_parent)
	? _parent->Child(_parent->childIndex(this) - 1)
	    : NULL;
}

Go* GoTreeDS::d_get_next () const
{
    return _next;
}

Go* GoTreeDS::d_get_first_child () const
{
    return _child;
}

GBool GoTreeDS::d_store_children_array () const
{
    return GFALSE;
}

void GoTreeDS::d_set_previous (const Go*)
{
}

void GoTreeDS::d_set_next (const Go* go)
{
    _next = (Go*)go;
}

void GoTreeDS::d_remove_child (const Go* go, int)
{
    if (go == _child)
	_child = go->Next();
}

void GoTreeDS::d_add_child (const Go* go, int index)
{
    if (index == 0)
	_child = (Go*)go;
}

void GoTreeDS::d_set_tree (const Go* parent, const Go*, const Go* next)
{
    _parent = (Go*)parent;
    _next = (Go*)next;
}

GBool GoTreeDS::d_store_abscs () const
{
    return GFALSE;
}

GBool GoTreeDS::d_store_relcs () const
{
    return GTRUE;
}

const GCoordSys& GoTreeDS::d_get_relcs (const GCoordSys*) const
{
    return GUnitCoordSys();
}

void GoTreeDS::d_set_coordsys
    (const GCoordSys& relcs, const GCoordSys&, const GCoordSys&)
{
    if (relcs != GUnitCoordSys())
	GError("GoTreeDS::d_set_coordsys()");
}

GBool relcs_precond
    (void*, const Go*, GCoordSys* newrel, const GCoordSys*, const GCoordSys*)
{
    *newrel = GUnitCoordSys();
    return GTRUE;
}

GoTreeDSMetaclass::GoTreeDSMetaclass ()
{
    relcsPrecond(relcs_precond, NULL);
}


int save_children (const Go* ch, GoStream* stream)
{
    for (; ch; ch = ch->Next())
	if (ch->callSavePrecond())
	{
	    if (!stream->saveObj(ch, ch->getMetaclass()->classname(), "child"))
		return GFALSE;
	}
    if (!stream->saveObj(NULL, NULL, "end of children"))
	return GFALSE;

    return GTRUE;
}

int GoTreeDS::save (const GoTreeDS* go, GoStream* stream)
{
    return
	save_children(go->_child, stream) &&
	Go::save(go, stream);
}


int restore_children (Go* go, GoStream* stream)
{
    for (;;)
    {
	void* next_child;
	const char* class_name;
	if (!stream->restoreObj(&next_child, &class_name))
	    return GFALSE;
	if (next_child == NULL)
	    break;
	((Go*)next_child)->DoReparent(go);
    }

    return GTRUE;
}

int GoTreeDS::readParams (GoTreeDS* go, GoStream* stream)
{
    return
	restore_children(go, stream) &&
	Go::readParams(go, stream);
}
