#ifdef REV_INFO
static char rcsid[] = "$Id: ToolsInit.cc,v 1.2 1992/12/10 14:05:56 jojic Exp $";
#endif
/*****************************************************************************
MODULE:  Go
FILE:    ToolsInit.cc
AUTHOR:  Gilles Benati
DATE:    July 10th, 1991

DESCRIPTION:

    This file contains a set of convenience functions performing various
    initializations.
    One of the goals here is to "force the link" with some modules of the
    library at compilation time.
    These functions can easily be rewritten by the application programmer if
    he feels like it, or more likely if he does not wish to use certain parts
    of the library.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#include <Go/GoStream.h>

#include <Go/Tools/ShBrush.h>
#include <Go/Tools/ShColor.h>
#include <Go/Tools/ShColormap.h>
#include <Go/Tools/ShDash.h>
#include <Go/Tools/ShFont.h>
#include <Go/Tools/ShImageBrush.h>
#include <Go/Tools/ShPattern.h>
#include <Go/Tools/ShPen.h>
#include <Go/Tools/ShPixelArray.h>
#include <Go/Tools/ShTextBrush.h>

#include <Go/Init.h>

void GoToolsInit() {

    static GBool done = GFALSE;
    if (done)
	return;
    done = GTRUE;
    
  GoStream::bind(GShBrush::classname(),
		 (int (*)(const void*, GoStream*))GShBrush::save,
		 (int (*)(void**, GoStream*))GShBrush::restore);
  GoStream::bind(GShColor::classname(),
		 (int (*)(const void*, GoStream*))GShColor::save,
		 (int (*)(void**, GoStream*))GShColor::restore);
  GoStream::bind(GShColormap::classname(),
		 (int (*)(const void*, GoStream*))GShColormap::save,
		 (int (*)(void**, GoStream*))GShColormap::restore);
  GoStream::bind(GShDash::classname(),
		 (int (*)(const void*, GoStream*))GShDash::save,
		 (int (*)(void**, GoStream*))GShDash::restore);
  GoStream::bind(GShFont::classname(),
		 (int (*)(const void*, GoStream*))GShFont::save,
		 (int (*)(void**, GoStream*))GShFont::restore);
  GoStream::bind(GShImageBrush::classname(),
		 (int (*)(const void*, GoStream*))GShImageBrush::save,
		 (int (*)(void**, GoStream*))GShImageBrush::restore);
  GoStream::bind(GShPattern::classname(),
		 (int (*)(const void*, GoStream*))GShPattern::save,
		 (int (*)(void**, GoStream*))GShPattern::restore);
  GoStream::bind(GShPen::classname(),
		 (int (*)(const void*, GoStream*))GShPen::save,
		 (int (*)(void**, GoStream*))GShPen::restore);
  GoStream::bind(GShPixelArray::classname(),
		 (int (*)(const void*, GoStream*))GShPixelArray::save,
		 (int (*)(void**, GoStream*))GShPixelArray::restore);
  GoStream::bind(GShTextBrush::classname(),
		 (int (*)(const void*, GoStream*))GShTextBrush::save,
		 (int (*)(void**, GoStream*))GShTextBrush::restore);
}
