#ifdef REV_INFO
static char rcsid[] = "$Id: GoAscii.cc,v 1.9 1992/12/30 15:29:06 mbikina Exp $";
#endif
/*****************************************************************************
MODULE:  GoStream
FILE:    GoAscii.cc
AUTHOR:  Gilles Benati, Philippe Smadja for read/write buffer.
DATE:    May 31st, 1991

DESCRIPTION:
    

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#include <stdio.h>
#include <assert.h>
#include <Go/Tools/Error.h>
#include <Go/Tools/Queue.h>
#include <Go/Tools/Alloc.h>
#include <Go/Tools/Medium.h>
#include <Go/GoAscii.h>


#define LINE_START		"\t"
#define ITEM_SEPARATOR		"\t: "

#define COMMENT_START		"\t# "
#define COMMENT_START_LEN	3

#define  HEADER_START           "%%"
#define  HEADER_END             "%%"	
#define  HEADER_START_LEN        2
#define  HEADER_MAX_LEN         512

#define NULL_OBJECT_STRING	"Nil"
#define OBJECT_PREFIX		"Obj"

#define NO_STRING		"?"
#define NO_STRING_LEN		1

#define CHAR_STRING		"char"
#define INT_STRING		"int"
#define LONG_STRING		"long"
#define SHORT_STRING		"short"
#define DOUBLE_STRING		"double"
#define STRING_STRING		"String"
#define BUFFER_STRING		"Buffer"
#define GBOOL_STRING		"GBool"
#define GNUMBER_STRING		"GNumber"
#define GVECTOR_STRING		"GVector"
#define GPOINT_STRING		"GPoint"
#define GBOX_STRING		"GBox"
#define GRGB_STRING		"GRgb"
#define GCOORDSYS_STRING	"GCoordSys"

// it is assume that GFALSE_STRING is longer than GTRUE_STRING;
// i.e. GFALSE_STRING_LEN >= GTRUE_STRING_LEN;
#define GTRUE_STRING		"GTRUE"
#define GTRUE_STRING_LEN	5
#define GFALSE_STRING		"GFALSE"
#define GFALSE_STRING_LEN	6

#define LMAXSTRING		62

#define GOTOC(c) (_medium->readuntil(c) != GEOM)

static const char* _char_code [256] = {
    "^\\@",  "^A",    "^B",    "^C",    "^D",    "^E",    "^F",    "^G",
    "^H",    "^I",    "^J",    "^K",    "^L",    "^M",    "^N",    "^O",
    "^P",    "^Q",    "^R",    "^S",    "^T",    "^U",    "^V",    "^W",
    "^X",    "^Y",    "^Z",    "^[",    "^\\",   "^]",    "^\\^",  "^_",

    " ",     "!",     "\\\"",  "#",     "$",     "%",     "&",     "'",
    "(",     ")",     "*",     "+",     ",",     "-",     ".",     "/",
    "0",     "1",     "2",     "3",     "4",     "5",     "6",     "7",
    "8",     "9",     ":",     ";",     "<",     "=",     ">",     "?",
    "\\@",   "A",     "B",     "C",     "D",     "E",     "F",     "G",
    "H",     "I",     "J",     "K",     "L",     "M",     "N",     "O",
    "P",     "Q",     "R",     "S",     "T",     "U",     "V",     "W",
    "X",     "Y",     "Z",     "[",     "\\\\",  "]",     "\\^",   "_",
    "`",     "a",     "b",     "c",     "d",     "e",     "f",     "g",
    "h",     "i",     "j",     "k",     "l",     "m",     "n",     "o",
    "p",     "q",     "r",     "s",     "t",     "u",     "v",     "w",
    "x",     "y",     "z",     "{",     "|",     "}",     "~",     "^^",

    "@^\\@", "@^A",   "@^B",   "@^C",   "@^D",   "@^E",   "@^F",   "@^G",
    "@^H",   "@^I",   "@^J",   "@^K",   "@^L",   "@^M",   "@^N",   "@^O",
    "@^P",   "@^Q",   "@^R",   "@^S",   "@^T",   "@^U",   "@^V",   "@^W",
    "@^X",   "@^Y",   "@^Z",   "@^[",   "@^\\",  "@^]",   "@^\\^", "@^_",

    "@ ",    "@!",    "@\\\"", "@#",    "@$",    "@%",    "@&",    "@'",
    "@(",    "@)",    "@*",    "@+",    "@,",    "@-",    "@.",    "@/",
    "@0",    "@1",    "@2",    "@3",    "@4",    "@5",    "@6",    "@7",
    "@8",    "@9",    "@:",    "@;",    "@<",    "@=",    "@>",    "@?",
    "@\\@",  "@A",    "@B",    "@C",    "@D",    "@E",    "@F",    "@G",
    "@H",    "@I",    "@J",    "@K",    "@L",    "@M",    "@N",    "@O",
    "@P",    "@Q",    "@R",    "@S",    "@T",    "@U",    "@V",    "@W",
    "@X",    "@Y",    "@Z",    "@[",    "@\\\\", "@]",    "@\\^",  "@_",
    "@`",    "@a",    "@b",    "@c",    "@d",    "@e",    "@f",    "@g",
    "@h",    "@i",    "@j",    "@k",    "@l",    "@m",    "@n",    "@o",
    "@p",    "@q",    "@r",    "@s",    "@t",    "@u",    "@v",    "@w",
    "@x",    "@y",    "@z",    "@{",    "@|",    "@}",    "@~",    "@^^", 
};

int GoAscii::openRead(GMedium* medium /* = NULL */) {
    if (! GoStream::openRead(medium)) {
	GWarning("GoAscii : open failed");
	return 0;
    }
    _medium->mseek(0);
    int nb_obj = 0;
    while (go_to_next_obj())
	nb_obj++;
    if (!nb_obj)
	GWarning("GoAscii : no object to be restored");
    _medium->mseek(0);
    return nb_obj;
}


int GoAscii::writeTypeString(const char* type_name) {
    char str[62];
    strcpy(str, LINE_START);
    strcat(str, type_name ? type_name : NO_STRING);
    strcat(str, ITEM_SEPARATOR);
    return _medium->write(str, strlen(str));
}



int GoAscii::writeComment(const char* comment) {
    _medium->mputs(COMMENT_START);
    if (! comment) {
	_medium->mputs(NO_STRING);
	_medium->mputc('\n');
	return  COMMENT_START_LEN + NO_STRING_LEN + 1;
    }
    else {
	_medium->mputs(comment);
	_medium->mputc('\n');
	return COMMENT_START_LEN + strlen(comment) + 1;
    }
}


GoAscii::~GoAscii() { }

int GoAscii::read_line(char* str, int size) {
    str = _medium->mgets(str, size);
    return  str != NULL;
}

int GoAscii::read_ident(char* str, int max_len) {
    register int i, c;
    
    for (i = 0; i < max_len && (c = _medium->mgetc()) != GEOM; i++, str++) {
	*str = c;
	if (! (isalnum(*str) || *str == '_')) {
	    _medium->mputback(*str);
	    return i;
	}
    }
    *++str = '\0';
    return (i == max_len) ? max_len : 0;
}

int GoAscii::read_digits(char* str, int max_len) {
    register int i, c;
    for (i = 0; i < max_len && (c = _medium->mgetc()) != GEOM; i++, str++) {
	*str = c;
	if (! isdigit(*str)) {
	    _medium->mputback(*str);
	    *str = '\0';
	    return i;
	}
    }
    *++str = '\0';
    return (i == max_len) ? max_len : 0;
}

int GoAscii::read_double(char* str, int max_len) {
    register int i, c;
    for (i = 0; i < max_len && (c = _medium->mgetc()) != GEOM; i++, str++) {
	*str = c;
	if (! (isdigit(*str) || *str == '+' || *str == '-' || *str == '.' ||
	       *str == 'e' || *str == 'E')) {
	    _medium->mputback(*str);
	    *str = '\0';
	    return i;
	}
    }
    *++str = '\0';
    return (i == max_len) ? max_len : 0;
}

int GoAscii::writeInt(int value, const char* comment) {
    // ***Check member function readInt if modifying this function;
    return write_scalar_param_line(INT_STRING, "%d", (long) value, comment);
}

int GoAscii::readInt(int* value) {
    // ***Check member function writeInt if modifying this function;
    long lval;
    int ret   = readLong(&lval);
    if (ret)
	*value = (int) lval;
    return ret;
}

int GoAscii::writeShort(short value, const char* comment) {
    // ***Check member function readShort if modifying this function;
    return write_scalar_param_line(SHORT_STRING, "%hd", (long) value, comment);
}


int GoAscii::readShort(short* value) {
    // ***Check member function writeShort if modifying this function;
    long lval;
    int ret = readLong(&lval);
    if (ret)
	*value = (short) lval;
    return ret;
}


int GoAscii::writeBool(GBool value, const char* comment) {
    // ***Check member function readBool if modifying this function;
    writeTypeString(GBOOL_STRING);
    if (value) {
	_medium->write(GTRUE_STRING, GTRUE_STRING_LEN);
	writeComment(comment);
	return GTRUE_STRING_LEN;
    }
    _medium->write(GFALSE_STRING, GFALSE_STRING_LEN);
    writeComment(comment);
    return GFALSE_STRING_LEN;
}

int GoAscii::readBool(GBool* value) {
    // ***Check member function writeBool if modifying this function;
    // assuming GFALSE_STRING_LEN >= GTRUE_STRING_LEN;
    char str[GFALSE_STRING_LEN];
    long pos = _medium->mtell();
    if (GOTOC(':') && skip_blanks() &&
	_medium->read(str, GFALSE_STRING_LEN)) {

	if (strncmp(str, GFALSE_STRING, GFALSE_STRING_LEN) == 0) {
	    *value = GFALSE;
	    return GOTOC('\n');
	}

	if (strncmp(str, GTRUE_STRING, GTRUE_STRING_LEN) == 0) {
	    *value = GTRUE;
	    _medium->mputback(str[4]);
	    return GOTOC('\n');
	}
    }
    _medium->mseek(pos);
    return 0;
}


int GoAscii::writeLong(long value, const char* comment) {
    // ***Check member function readLong if modifying this function;
    return write_scalar_param_line(LONG_STRING, "%ld", value, comment);
}

#define LINE_SIZE 256
int GoAscii::readLong(long* value) {
    // ***Check member function writeLong if modifying this function;
    char str[LINE_SIZE];
    long pos = _medium->mtell();
    
    if (GOTOC(':') && skip_blanks() && read_line(str, LINE_SIZE -1)) {
	char* c;
	long l = strtol(str, &c, 0);
	if (c != str && (isspace(*c) || *c == '#')) {
	    *value = l;
	    return 1;
	}
    }
    _medium->mseek(pos);
    return 0;
}
#undef LINE_SIZE

int GoAscii::readObj(_GId* id) {
    long pos = _medium->mtell();
    if (GOTOC(':') && skip_blanks()) {
	if (get_obj_id(id)) {
	    if (GOTOC('\n'))
		return 1;
	}
    }
    _medium->mseek(pos);
    return 0;
}


#define CHAR_SIZE 5
int GoAscii::writeChar(char value, const char* comment) {
    // ***Check member function readChar if modifying this function;
    writeTypeString(CHAR_STRING);
    char str[CHAR_SIZE];
    sprintf(str, "0x%x", value);     // exactly CHAR_SIZE chars (including \0);
    int ret = strlen(str);
    _medium->write(str, strlen(str));
    writeComment(comment);
    return ret;
}

int GoAscii::readChar(char* value) {
    // ***Check member function writeChar if modifying this function;
    char str[CHAR_SIZE];
    long pos = _medium->mtell();
    
    if (GOTOC(':') && skip_blanks()) {
	if (_medium->read(str, CHAR_SIZE -1) ==  CHAR_SIZE-1) {
	    str[CHAR_SIZE -1] = '\0';
	    char* c;
	    long i = strtol(str, &c, 0);
	    if (*c == '\0' || isspace(*c)) {
		*value = (char) i;
		if (GOTOC('\n'))
		    return 1;
	    }
	}
    }
    _medium->mseek(pos);
    return 0;
}
#undef CHAR_SIZE


#define DOUBLE_SIZE 32
int GoAscii::writeNumber(GNumber value, const char* comment) {
    // ***Check member function readNumber if modifying this function;
    writeTypeString(GNUMBER_STRING);
    char str[DOUBLE_SIZE];
    sprintf(str, "%g", value);
    int ret = strlen(str);
    _medium->write(str, strlen(str));
    writeComment(comment);
    return ret;
}

int GoAscii::readNumber(GNumber* value) {
    // ***Check member function writeNumber if modifying this function;
    double d;
    int ret = readDouble(&d);
    if (ret)
	*value = (GNumber) d;
    return ret;
}

int GoAscii::writeDouble(double value, const char* comment) {
    // ***Check member function readDouble if modifying this function;
    writeTypeString(DOUBLE_STRING);
    char str[40];
    sprintf(str, "%g", value);
    int ret = strlen(str);
    _medium->write(str, ret);
    writeComment(comment);
    return ret;
}

int GoAscii::readDouble(double* value) {
    // ***Check member function writeDouble if modifying this function;
    char str[DOUBLE_SIZE];
    long pos = _medium->mtell();
    
    if (GOTOC(':') && skip_blanks()) {
	int ret = read_double(str, DOUBLE_SIZE-1);
	if (ret) {
	    char* c;
	    double d = strtod(str, &c);
	    if (*c == '\0') {
		*value = d;
		if (GOTOC('\n'))
		    return 1;
	    }
	}
    }
    _medium->mseek(pos);
    return 0;
}


int GoAscii::writeVector(const GVector& value, const char* comment) {
    // ***Check member function readVector if modifying this function;
    writeTypeString(GVECTOR_STRING);
    char str[128];
    sprintf(str, "(%g, %g)", value.x, value.y);
    int ret = strlen(str);
    _medium->write(str, ret);
    writeComment(comment);
    return ret;
}

int GoAscii::readVector(GVector* value) {
    // ***Check member function writeVector if modifying this function;
    char str[DOUBLE_SIZE];
    long pos = _medium->mtell();
    
    if (GOTOC(':') && GOTOC('(') &&
	skip_blanks() && read_double(str, DOUBLE_SIZE-1)) {
	char* c;
	double y, x = strtod(str, &c);
	if (*c == '\0' && GOTOC(',') && skip_blanks() &&
	    read_double(str, DOUBLE_SIZE-1) && GOTOC(')')) {
	    y = strtod(str, &c);
	    if (*c == '\0') {
		value->x = x;
		value->y = y;
		if (GOTOC('\n'))
		    return  1;
	    }
	}
    }
    _medium->mseek(pos);
    return 0;
}

int GoAscii::writePoint(const GPoint& value, const char* comment) {
    // ***Check member function readPoint if modifying this function;
    writeTypeString(GPOINT_STRING);
    char str[128];
    sprintf(str, "(%g, %g)", value.x, value.y);
    int ret = strlen(str);
    _medium->write(str, ret);
    writeComment(comment);
    return ret;
}

int GoAscii::readPoint(GPoint* value) {
    // ***Check member function writePoint if modifying this function;
    return readVector(value);
}


int GoAscii::writeBox(const GBox& value, const char* comment) {
    // ***Check member function readBox if modifying this function;
    writeTypeString(GBOX_STRING);
    char str[256];
    sprintf(str, "[%g, %g, %g, %g]",
	    value.xmin, value.ymin, value.xmax, value.ymax);	
    int ret = strlen(str);
    _medium->write(str, ret);
    writeComment(comment);
    return ret;
}


int GoAscii::readBox(GBox* value) {
    // ***Check member function writeBox if modifying this function;
    char str[DOUBLE_SIZE];
    long pos = _medium->mtell();
    
    if (GOTOC(':') && GOTOC('[') &&
	skip_blanks() && read_double(str, DOUBLE_SIZE-1)) {
	char* c;
	double ymax, xmax, ymin, xmin = strtod(str, &c);
	if (*c == '\0' && GOTOC(',') &&
	    skip_blanks() && read_double(str, DOUBLE_SIZE-1)) {
	    ymin = strtod(str, &c);
	    if (*c == '\0' && GOTOC(',') &&
		skip_blanks() && read_double(str, DOUBLE_SIZE-1)) {
		xmax = strtod(str, &c);
		if (*c == '\0' && GOTOC(',') && skip_blanks() &&
		    read_double(str, DOUBLE_SIZE-1)&& GOTOC(']')) {
		    ymax = strtod(str, &c);
		    if (*c == '\0') {
			value->xmin = xmin;
			value->ymin = ymin;
			value->xmax = xmax;
			value->ymax = ymax;
			if (GOTOC('\n'))
			    return  1;
		    }
		}
	    }
	}
    }
    _medium->mseek(pos);
    return 0; 
}

#define RGB_SIZE 60
int GoAscii::writeRgb(const GRgb& value, const char* comment) {
    // ***Check member function readRgb if modifying this function;
    writeTypeString(GRGB_STRING);
    char str[RGB_SIZE];
    sprintf(str, "[%hu, %hu, %hu]", value.red, value.green, value.blue);
    int ret = strlen(str);
    _medium->write(str, ret);
    writeComment(comment);
    return ret;
}


int GoAscii::readRgb(GRgb* value) {
    // ***Check member function writeRgb if modifying this function;
    char str[RGB_SIZE];
    long pos = _medium->mtell();
    
    if (GOTOC(':') && skip_blanks()  &&
	(str[0] = _medium->mgetc()) == '[' && skip_blanks() &&
	read_digits(str, RGB_SIZE-1)) {
	char* c;
	ushort blue, green, red = (ushort) strtod(str, &c);
	if (*c == '\0' && GOTOC(',') &&
	    skip_blanks() && read_digits(str, RGB_SIZE-1)) {
	    green = (ushort) strtod(str, &c);
	    if (*c == '\0' && GOTOC(',') && skip_blanks() &&
		read_digits(str, RGB_SIZE-1) && GOTOC(']')) {
		blue = (ushort) strtod(str, &c);
		if (*c == '\0') {
		    value->red   = red;
		    value->green = green;
		    value->blue  = blue;
		    if (GOTOC('\n'))
			return  1;
		}
	    }
	}
    }
    _medium->mseek(pos);
    return 0; 
}

int GoAscii::writeCoordSys(const GCoordSys& value, const char* comment) {
    // ***Check member function readCoordSys if modifying this function;
    writeTypeString(GCOORDSYS_STRING);
    char str[256];
    sprintf(str, "O(%g, %g) I(%g, %g) J(%g, %g)",
	    value.O.x, value.O.y, value.I.x, value.I.y, value.J.x, value.J.y);
    int ret = strlen(str);
    _medium->write(str, ret);
    writeComment(comment);
    return ret;
}


int GoAscii::readCoordSys(GCoordSys* value) {
    // ***Check member function writeCoordSys if modifying this function;
    char str[DOUBLE_SIZE];
    long pos = _medium->mtell();
    
    if (GOTOC(':') && GOTOC('(') &&
	skip_blanks() && read_double(str, DOUBLE_SIZE-1)) {
	char* c;
	GPoint O;
	GVector I, J;
	O.x = (GNumber) strtod(str, &c);
	if (*c == '\0' && GOTOC(',') && skip_blanks() &&
	    read_double(str, DOUBLE_SIZE-1) && GOTOC(')')) {
	    O.y = strtod(str, &c);
	    if (*c == '\0' && GOTOC('(') &&
		skip_blanks() && read_double(str, DOUBLE_SIZE-1)) {
		I.x = strtod(str, &c);
		if (*c == '\0' && GOTOC(',') && skip_blanks() &&
		    read_double(str, DOUBLE_SIZE-1) && GOTOC(')')) {
		    I.y = strtod(str, &c);
		    if (*c == '\0' && GOTOC('(') &&
			skip_blanks() && read_double(str, DOUBLE_SIZE-1)) {
			J.x = strtod(str, &c);
			if (*c == '\0' && GOTOC(',') && skip_blanks() &&
			    read_double(str, DOUBLE_SIZE-1) && GOTOC(')')) {
			    J.y = strtod(str, &c);
			    if (*c == '\0') {
				value->O = O;
				value->I = I;
				value->J = J;
				if (GOTOC('\n'))
				    return  1;
			    }
			}
		    }
		}
	    }
	}
    }
    _medium->mseek(pos);
    return 0; 
}


int GoAscii::writeRegion(const GRegion&, const char*)	{ return 0; }
int GoAscii::readRegion(GRegion*)			{ return 0; }

#undef DOUBLE_SIZE



/**
FUNCTION
    int GoAscii::writeString(const char* str, const char* comment) 
DESCRIPTION
    writes str in _medium according to the format described thereafter:
       - the string is cut in pieces of LMAXSTRING-1 characters,
       - each piece is enclosed by double quotes "....",
       - \ " and new-line are replaced by \\ \" and \n (backslash n).
    
**/
int GoAscii::writeString(const char* str, const char* comment) {
    char store[LMAXSTRING];
    register int l = 0;
    register char *c1 = (char*) str;
    char *c0;

    writeTypeString(STRING_STRING);

    if (!c1 || !*c1)
	_medium->write("\"\"", 2);
    else {
	while (*c1) {
	    _medium->write("\t\"", 2);
	    for  (c0 = store; (l < LMAXSTRING-4) && *c1; c1++) {
		const char* code = _char_code[(unsigned char) *c1];
		int len = strlen(code);
		strcpy(c0, code);
		l  += len;
		c0 += len;
	    }
	    *c0 = '\0';
	    _medium->write(store, l);
	    l = 0;
	    if (*c1) 
		_medium->write("\"\n", 2);
	    else _medium->mputc('"');  // end of string: no new line needed;
	}
    }
    writeComment(comment);
    return 1;
}

INLINE char modifier(char c, int alt, int ctrl) {
    if (ctrl)
	c &= ((char) ~(1<<6));
    if (alt)
	c |= ((char) (1<<7));
    return c;
}

static int decode_string(char* dst, char* src, char** last, long size) {

    int alt = 0, ctrl = 0;
    char *c, *cur = dst;
    if (*src++ != '"') {
	*last = src;
	*dst = '\0';
    }
    else for (c = src; ((int) (cur - dst)) < size; c++) {
	switch (*c) {
	  case '\0':
	    GWarning("syntax error: EOL before \"");
	    *cur = '\0';
	    return 0;
	  case '"':
	    *cur = '\0';
	    *last = c + 1;
	    return (int) (cur - dst + 1);
	  case '@':
	    if (alt) {
		GWarning("syntax error: @@ encountered");
		alt = ctrl = 0;
	    }
	    else alt = 1;
	    break;
	  case '^':
	    if (ctrl) {
		//  code for del;
		*cur++ = modifier('\177', alt, 0);
		alt = ctrl = 0;
	    }
	    else ctrl = 1;
	    break;
	  case '\\':
	    c++; // do not break and do default;
	  default:
	    *cur++ = modifier(*c, alt, ctrl);
	    alt = ctrl = 0;
	    break;
	}
    }
    return 0;
}

int GoAscii::readString(unsigned long length, char** value) {
    // **read spec carefully !**;

#ifdef NO_ALLOCA
#define DELETE_IF_(allocate, queue) \
    if (allocate) { while (!queue.empty()) delete queue.getlast(); }
#else
#define DELETE_IF_(allocate, queue) 
#endif /* NO_ALLOCA */

    long pos = _medium->mtell();
    
    if (GOTOC(':')) {
	GBool allocate = (*value == NULL || length == 0);
	int len = 0;
	char  encoded_line[(LMAXSTRING)<<1]; // longer than decoded line;
	char* decoded_line = *value;
	GQueue queue;
	GBool go_on = GTRUE;

	if (allocate)
	    length = 0;
	
	while (go_on) {
	    if (allocate) {
#ifdef NO_ALLOCA
		decoded_line = new char[LMAXSTRING];
		if (! decoded_line) {
		    DELETE_IF_(allocate, queue);
		    return 0;
		}
#else
		decoded_line = (char*) alloca(LMAXSTRING);
#endif /* NO_ALLOCA */
		length += LMAXSTRING;
		queue.putlast(decoded_line);
	    }

	    if (! read_line(encoded_line, (LMAXSTRING)<<1)) {
		DELETE_IF_(allocate, queue);
		return 0;
	    }
	    char *start, *last;
	    for (start = encoded_line; isspace(*start); start++);
	    int size = decode_string(decoded_line, start, &last, length - len);
	    if (! size) {
		DELETE_IF_(allocate, queue);
		return 0;
	    }
	    for ( ; isspace(*last); last++);
	    switch (*last) {
	      case '\0':
		break;
	      case '#':
		// comment mark; we read the last string portion; stop looping;
		go_on = GFALSE;
		break;
	      default:
		DELETE_IF_(allocate, queue);
		return 0;
	    }
	    len += size;
	    if (! allocate)
		decoded_line += size - 1;
	}
	if (allocate) {
	    *value = new char[len+1];
	    if (! *value) {
		GWarning("readString: alloc failed!");
		DELETE_IF_(allocate, queue);
		return 0;
	    }
	    **value = '\0';
	    while(!queue.empty()) {
		decoded_line = (char*) queue.getfirst();
		strcat(*value, decoded_line);
#ifdef NO_ALLOCA
		delete decoded_line;
#endif /* NO_ALLOCA */
	    }
	}
	return len;
    }
    
    _medium->mseek(pos);
    return 0; 
#undef DELETE_IF_
}

/**
FUNCTION
    writeBuffer(ulong size, const char* buf, const char* comment)
DESCRIPTION
    format:
    size of the buffer,
    translation table of the 64 most referenced bytes in the buffer.
           These bytes are coded with 1 ascii character.
    value of the most referenced hight or low value of the remainding bytes,
           each byte which has the same hight/low value are coded on 1 ascci character
    The other bytes are coded with 2 charecters.
    size  of the coded buffer (included the '\n' which are not significant)
**/
inline uchar indexOfTheMaxReference(long* bytes, long* hight, long* low)
{
    long m = 0;
    int index = 0;
    long* save = bytes;
    for (int i = 0; i < 256; i++, bytes++)
	if (*bytes > m) {
	    index = i;
	    m = *bytes;
	}
    hight[ (index & 0xF0) >> 4] -= m;
    low  [ index & 0x0F] -= m;
    save[index] = -1;
    return (uchar)index;
}

#define CODE_0      0x40
#define CODE_1      0x20
#define DOUBLE_CHAR 0x60
#define SINGLE_CHAR 0x70
#define LINE_SIZE 80

static uchar CODE[2]  = {
    CODE_0, CODE_1,
};
    
int  GoAscii::writeBuffer(ulong size, const char* buf, const char* comment)
{
    char string[LINE_SIZE];

    /**
      must be call in the effective writeBuffer
      **/
    writeTypeString (BUFFER_STRING);
    
    /**
      write the size in ascii
      **/
    char tab[LINE_SIZE];
    sprintf (tab , "size:%d\n", size);
    _medium->mputs(tab);

    /**
      find the number of reference of each byte
      **/
    long bytes[256];
    long hight[16];
    long low[16];
    memset (bytes, 0, sizeof bytes);
    memset (hight, 0, sizeof hight);
    memset (low, 0, sizeof low);
    const uchar* cbuf = (const uchar*)buf;
    ulong i = 0;
    for (; i < size; i++) {
	char b = *cbuf++;
	bytes [b]++;
	hight [(b & 0xF0) >> 4]++;
	low   [b & 0x0F]++;
    }
    char codeToWrite[256];
    uchar translationTable[64];
    memset (codeToWrite, 0, sizeof codeToWrite);
    
    /**
      make  translation tables for the 64 most use values
      **/
    uchar index;
    for (int code = 0 ; code < 2; code++)
	for (int v = 0; v < 32; v++) {
	    index = indexOfTheMaxReference(bytes, hight, low);
	    codeToWrite[index] = CODE[code] + v;
	    translationTable[code * 32 + v] = index;
	}

    /**
      write the translation table
      **/
    for (i=0; i < 64;) {
	string[0] = '\0';
	for (int j = 0; j < 8; j++) {
	    char tab[16];
	    sprintf (tab, " %lu", translationTable[i++]);
	    strcat(string, tab);
	}
	assert (strlen(string) < LINE_SIZE);
	_medium->mputs (string);
	_medium->mputc ('\n');
    }

    /**
      write hight/low translator
      **/
    long maxH = -1;
    uchar indexH = 0;
    for (i=0; i < 16; i++)
	if (hight[i] > maxH) {
	    indexH = (uchar)i;
	    maxH = hight[i];
	}
    long maxL = -1;
    uchar indexL = 0;
    for (i=0; i < 16; i++)
	if (low[i] > maxL) {
	    indexL = (uchar)i;
	    maxL = low[i];
	}
    long maxRef = max(maxL, maxH);
    if (maxRef == maxL) 
	sprintf(string, "low: %d\n", indexL);
    else
	sprintf(string, "hight: %d\n", indexH);

    _medium->mputs(string);

    /**
      let 8 characters to write the effective coded size
      **/
    long codedSizeOffset = _medium->mtell();
    _medium->mseek(8, GMSEEK_FROM_CURRENT);

    /**
      write the translated buffer.
      **/
    ulong nbbytes = 0;
    if (maxRef == maxH)
	for (cbuf = (const uchar*)buf, i = 0; i < size; i++, cbuf++) {
	    char b = codeToWrite[*cbuf];
	    if (b == 0) {
		/**
		  translate in two ascii characters unless the maxRef byte
		  **/
		uchar Hight = *cbuf >> 4;
		uchar Low = *cbuf & 0x0F;
		if ( Hight == indexH && Low != 0x0F)
		    b = SINGLE_CHAR | Low;
		else {
		    b = DOUBLE_CHAR | Low;
		    if (nbbytes % LINE_SIZE == 0) {
			nbbytes++;
			_medium->mputc( '\n');
		    }
		    nbbytes++;
		    _medium->mputc(b);
		    b = DOUBLE_CHAR | Hight;
		}

	    }
	
	    if (nbbytes % LINE_SIZE == 0) {
		nbbytes++;
		_medium->mputc( '\n');
	    }
	    nbbytes++;
	    _medium->mputc(b);
	}

    else
	for (cbuf = (const uchar*)buf, i = 0; i < size; i++, cbuf++) {
	    char b = codeToWrite[*cbuf];
	    if (b == 0) {
		/**
		  translate in two ascii characters unless the maxRef byte
		  **/
		uchar Hight = *cbuf >> 4;
		uchar Low   = *cbuf & 0x0F;
		if ( Low == indexL && Hight != 0x0F)
		    b = SINGLE_CHAR | Hight;
		else {
		    b = DOUBLE_CHAR | Low;
		    if (nbbytes % LINE_SIZE == 0) {
			nbbytes++;
			_medium->mputc('\n');
		    }
		    nbbytes++;
		    _medium->mputc(b);

		    b = DOUBLE_CHAR | Hight;
		}

	    }
	
	    if (nbbytes % LINE_SIZE == 0) {
		nbbytes++;
		_medium->mputc( '\n');
	    }
	    nbbytes++;
	    _medium->mputc(b);
	}

    _medium->mputc('\n');	

    long currentPos = _medium->mtell();
    /**
      write the effective codesize on 8 characters
      the '\n' has been  written by the 1st "% LINE_SIZE" operation
      during the effective write of the  coded  characters
      **/
    _medium->mseek (codedSizeOffset);
    sprintf (string, "%8lu", nbbytes);
    _medium->mputs(string);

    _medium->mseek(currentPos);

    /**
      write the comment
      **/
    writeComment(comment);
    
    return 1;
}


/**
FUNCTION
    GoAscii::readBuffer(ulong* size, char** result)
DESCRIPTION
    If *size and *result are non NULL, they will be used to put the buffer.
    If the pointed size is too small readBuffer() will return  on error.
    if *size is 0 or *result is NULL  readBuffer() allocs the coded buffer size
    which is higher than the decoded one. It reads the medium inside the
    allocated area and translated the buffer inside the same area. After it
    reallocs the result to the effective size.
**/
#if defined( __MSC7__)
void go_sscanf(char * str, int* i0, int* i1, int* i2, int* i3,
		int* i4, int* i5, int* i6, int* i7)
{
char * begin = str;
char * end;
char tmp[16];
int v[8];
      
	for(int i = 0; i <8; i++ ) {		
		while(*begin < '0' && *begin >'9') 
			begin++;
		end = begin;
		while(*end >='0' && *end <= '9')
			end++;
		strset(tmp, '\0');
		strncpy(tmp, begin, end - begin);
		v[i] = atoi(tmp);
		begin = end;
	}
	*i0 = v[0]; *i1 = v[1]; *i2 = v[2]; *i3 = v[3];
	*i4 = v[4]; *i5 = v[5]; *i6 = v[6]; *i7 = v[7];
}
#endif

typedef int (*MGETC)(void*);

int GoAscii::readBuffer(ulong* size, char** result)
{
    int translator = 255;
    GBool low_translator = GFALSE;
#if defined( __MSC7__) || defined (__BORLANDC__)
    uchar huge * buf = NULL;
#else
    uchar *buf = NULL;
#endif
    ulong i = 0;
    GBool allocatedByReadBuffer = GFALSE; /* for error management */
    long codedSize  = 0;
    
    /**
      error management
      **/
    long previous = _medium->mtell();

    /**
      skip the BUFFER_STRING string
      **/

    GBool ok = (GOTOC(':') && skip_blanks());
    if (!ok)
	goto error;

    /**
      read the size of the buffer.
      **/
    char str[LINE_SIZE];

    if (GOTOC(':') && read_line(str, LINE_SIZE-1)) {
	char* c;
	ulong l = (ulong)strtol(str, &c, 0);
	if (c == str)
	    goto error;
	else if (*result && *size) {
	    if ( *size < l ) {
		GWarning ("GoAscii::readBuffer(), insufficient size given");
		goto error;
	    }
	}
	else  { *size = l;
		*result = NULL;
		allocatedByReadBuffer = GTRUE;
	    }
    }

    if (*size == 0) {
	/**
	  error --> restore the previous read offset
	  **/
	goto error;
    }
    
    /**
      read the translation table is it an in table because of sscanf().
      **/
    int translationTable[64];

    for (i = 0 ; i < 8; i++) 
	if (read_line(str, LINE_SIZE-1)) {
	    int index = (int)(i * 8);
#ifdef __MSC7__
	    go_sscanf(str,  &translationTable[index],   &translationTable[index+1],
	        &translationTable[index+2], &translationTable[index+3],
	        &translationTable[index+4], &translationTable[index+5],
	        &translationTable[index+6], &translationTable[index+7]);
#else
	    sscanf(str, "%d %d %d %d %d %d %d %d",
	        &translationTable[index],   &translationTable[index+1],
	        &translationTable[index+2], &translationTable[index+3],
	        &translationTable[index+4], &translationTable[index+5],
	        &translationTable[index+6], &translationTable[index+7]);
#endif
	} else
	    goto error;

    /**
      read the hight/low translator
      **/


    if (!read_line(str, LINE_SIZE-1))
	goto error;

    if (strncmp(str, "low:", 4) == 0) {
	int str_len = (int) strlen(str);
	int i;
	for (i = 4; i < str_len; i++)
	    if (!isspace(str[i])) 
		break;
	translator = atoi(&str[i]);
	low_translator = GTRUE;
    }
    else {
	int str_len = (int) strlen(str);
	int i;
	for (i = strlen("hight:"); i < str_len; i++)
	    if (!isspace(str[i]))
		break;
	translator = atoi(&str[i]);
	low_translator = GFALSE;
    }

    /**
      valid hexa translator is [0..15]
      **/
    if (translator < 0 || translator > 15)
	goto error;

    /**
      read the size of the coded buffer
      which is highter than the decode one (ascii buffer)
      **/
    
    if (!read_line(str, LINE_SIZE-1))
	goto error;

    int str_length;
    int is;
    str_length = (int) strlen(str);
    for (is = 0; is < str_length; is++)
	if (!isspace(str[is]))
	    break;
    codedSize = atol(&str[is]);
    if (codedSize <= 0 || ((ulong)codedSize) < *size)
	goto error;

    char c1, c2 ;

    if (allocatedByReadBuffer && *result == NULL)  {
	*result = (char*)GMALLOC(codedSize); /** src/dst  same buffer **/
	_medium->read(*result, codedSize);
	buf = (uchar*)(*result);
	uchar* srcBuf = buf;

#define GETC(c)  if ( (c=*srcBuf++) == '\n') c = *srcBuf++;

	if (low_translator)
	    for (i= 0; i < *size; i++) {
		GETC(c1);
		uchar Code = c1 & DOUBLE_CHAR;

		switch (Code) {

		  case CODE_0:
		    *buf++ = translationTable[c1 & 0x1F];
		    break;
		    
		  case CODE_1:
		    *buf++ = translationTable[32 + (c1 & 0x1F)];
		    break;

		  case DOUBLE_CHAR:
		    if ((c1 & SINGLE_CHAR) == SINGLE_CHAR) 
			*buf++ = ((c1 & 0x0F) << 4) | ((uchar)translator);
		    else {
			GETC(c2);
			if ((c2 & DOUBLE_CHAR) != DOUBLE_CHAR)
			    goto error;
			*buf++ = ((c2 & 0x0F) << 4) | (c1 & 0x0F);
		    }
		    break;
		
		  default:
		    goto error;
		}
	    }
	else  {
	    translator <<= 4;
	    for (i=0; i < *size; i++) {
		GETC(c1);
		uchar Code = c1 & DOUBLE_CHAR;

		switch (Code) {

		  case CODE_0:
		    *buf++ = translationTable[c1 & 0x1F];
		    break;

		  case CODE_1:
		    *buf++ = translationTable[32 + (c1 & 0x1F)];
		    break;

		  case DOUBLE_CHAR:
		    if ((c1 & SINGLE_CHAR) == SINGLE_CHAR) 
			*buf++ = ((uchar)translator) | (c1 & 0x0F);
		    else {
			GETC(c2);
			if ((c2 & DOUBLE_CHAR) != DOUBLE_CHAR)
			    goto error;
			*buf++ = ((c2 & 0x0F) << 4) | (c1 & 0x0F);
		    }
		    break;
		
		  default:
		    goto error;
		}
	    }
	}
	
#undef GETC
	/**
	  the buffer is read.
	  Realloc to the effective size
	  **/
	GREALLOC(*result, *size);
	return 1;

    } else {
	buf = (uchar*)(*result);
	/**
	  It is faster to have a pointer to a method
	  than to execute the virtual indirection for the medium class.
	  but we cannot do it the ATT 2.0
	  **/
#ifdef __GNUC__
	MGETC mgetc = (MGETC)_medium->mgetc;
#       define GETC(c) if ((c= (*mgetc)(_medium)) == '\n') c = (*mgetc)(_medium);
#else
#       define GETC(c) if ((c = _medium->mgetc()) == '\n') c = _medium->mgetc();
#endif
		
	if (low_translator) 
	    for (i= 0; i < *size; i++) {
		GETC(c1);
		uchar Code = c1 & DOUBLE_CHAR;
		
		switch (Code) {

		  case CODE_0:
		    *buf++ = translationTable[c1 & 0x1F];
		    break;

		  case CODE_1:
		    *buf++ = translationTable[32 + (c1 & 0x1F)];
		    break;

		  case DOUBLE_CHAR:
		    if ((c1 & SINGLE_CHAR) == SINGLE_CHAR) 
			*buf++ = ((c1 & 0x0F) << 4) | ((uchar)translator);
		    else {
			GETC(c2);
			if ((c2 & DOUBLE_CHAR) != DOUBLE_CHAR)
			    goto error;
			*buf++ = ((c2 & 0x0F) << 4) | (c1 & 0x0F);
		    }
		    break;
		
		  default:
		    goto error;
		}
	    }
	else  {
	    translator <<= 4;
	    for (i=0; i < *size; i++) {
		GETC(c1);
		uchar Code = c1 & DOUBLE_CHAR;

		switch (Code) {

		  case CODE_0:
		    *buf++ = translationTable[c1 & 0x1F];
		    break;

		  case CODE_1:
		    *buf++ = translationTable[32 + (c1 & 0x1F)];
		    break;

		  case DOUBLE_CHAR:
		    if ((c1 & SINGLE_CHAR) == SINGLE_CHAR) 
			*buf++ = ((uchar)translator) | (c1 & 0x0F);
		    else {
			GETC(c2);
			if ((c2 & DOUBLE_CHAR) != DOUBLE_CHAR)
			    goto error;
			*buf++ = ((c2 & 0x0F) << 4) | (c1 & 0x0F);
		    }
		    break;
		
		  default:
		    goto error;
		}
	    }
	}
#undef GETC
	/**
	  the buffer is read.
	  **/
	return 1;
    }
    
  error:
    if (allocatedByReadBuffer)
	GFREE(*result);
    _medium->mseek(previous);
    return 0;
}



int GoAscii::readObjBegin() {
    GBool dummy;
    return is_obj_desc_begin(&dummy);
}

int GoAscii::readObjEnd() {
    int c;
    long pos = _medium->mtell();
    if (skip_blanks() && (c = _medium->mgetc()) ==  '}')
	return 1;
    _medium->mseek(pos);
    return 0;
}

int GoAscii::readObjId(_GId* id) {
    long pos = _medium->mtell();
    int ret  = get_obj_id(id);
    if (! ret)
	_medium->mseek(pos);
    return ret;
}



int GoAscii::writeObjId(_GId id) {
    char str[40];
    if (id)
	sprintf(str, "%s%lu", OBJECT_PREFIX, id);
    else strcpy(str, NULL_OBJECT_STRING);
    int ret = strlen(str);
    _medium->write(str, ret);
    return ret;
}

int GoAscii::writeObj(_GId id, const char* name, const char* comment) {
    return(writeTypeString(name) && writeObjId(id) && writeComment(comment));
}


int GoAscii::writeObjBegin(GBool first_obj) {
    // *** when modifying this function, make sure that ...;
    // *** the function GoAscii::is_obj_desc_begin match;
    if (first_obj)
	return _medium->write(" Head {\n", 8);
    return _medium->write(" {\n", 3);
}


int GoAscii::writeObjEnd() {
    return (int) _medium->write("}\n\n", 3);
}


int GoAscii::readHeader (unsigned long *size, char** buffer)
{
    char  auxBuffer[HEADER_MAX_LEN+3],str[10], precC='\0';
    long  pos = _medium->mtell()    ;
    int   i   = 0 ,ret = 0, c ;
    GBool finSequence = GFALSE      ;

   _medium-> mseek(0);
    skip_blanks();
    _medium->mgets(str,HEADER_START_LEN+1);
    if ( (strcmp(str,HEADER_START)==0))
    {
	skip_blanks();
	while (((c = _medium->mgetc()) != GEOM) && (finSequence == GFALSE)  && (i < HEADER_MAX_LEN))
	{
	    auxBuffer[i] = c ; i ++ ;
	    if (c == '%')
	    {
		if  (precC == '%')
		    finSequence = GTRUE ;
		else  precC =  '%' ;
	    }
	}
	if (c != GEOM)
	{
	    auxBuffer[i-2] = '\0' ;
	    *buffer = new char[i] ;
	    strcpy(*buffer,auxBuffer);
	    *size = i - 3;
	    ret = 1 ;
	}
    }
    _medium->mseek(pos);
    return ret ;
}



int GoAscii::writeHeader(unsigned long size, const char* header)
{
    if ((header)  && (!(_mode & GFLAG_USER_HEADER)) && (size < HEADER_MAX_LEN))
    {
	_medium->mseek(0);
	_medium->mputs(HEADER_START);
	_medium->mputc('\n');
	_medium->mputs(header);
	_medium->mputc('\n');
	_medium->mputs(HEADER_END);
	_medium->mputc('\n');
	_medium->mputc('\n');
	return HEADER_START_LEN + strlen(header) + 1;
    }
    return 0 ;
}



/************* NOT IMPLEMENTED YET *******

int GoAscii::writeFooter(){}
int GoAscii::deleteFooter(){}

*******/


int GoAscii::skip_blanks() {
    int c;
    while (isspace((c=_medium->mgetc())));
    _medium->mputback((char)c);
    return  c != GEOM;
}


int GoAscii::write_scalar_param_line(const char* type_str, const char* format,
				     long value, const char* comment){
    writeTypeString(type_str);
    char str[40];
    sprintf(str, format, value);
    (void)_medium->mputs(str);
    writeComment(comment);
    return  strlen(str);
}


#define MAX_ID_LENGTH 40
int GoAscii::get_obj_id(_GId* id) {
    // Trying to read an object _GId, i.e. a string of the form
    // "Obj"<num> or "Nil". Note that "Obj0" and "Nil" are equivalent;
    // Reading starts at the current position in _medium.
    // If the characters read match Obj<num>;
    // 		- puts the int value <num> in id;
    //		- sets _medium reading pointer on the char following <num>;
    //		- returns 1;
    // Else;
    //		- return 0;
    //		- the medium pointer put back on the line where it was;
    //		  when we entered the function, i.e. if some '\n' have..;
    //		  been read, step back where we were bwfore reading);
							     
    char str[MAX_ID_LENGTH + 1];

    if (_medium->read(str, 3) == 3) {
	str[3] = '\0';
	if (! strcmp(str, OBJECT_PREFIX)) {
	    char* c;
	    int j;
	    for (j = 0, c = str;
		 j < MAX_ID_LENGTH && ((*c = _medium->mgetc()), isdigit(*c));
		 j++, c++) ;
	    if (j) {
		// we have read one char too many;
		_medium->mputback(*c);
		str[j++] = '\0';
		*id = atol(str);
		return 1;
	    }
	}
	else if (! strcmp(str, NULL_OBJECT_STRING)) {
	    *id = 0;
	    return 1;
	}
	else if (str[0] == '\n' || str[1] == '\n' || str[2] == '\n') {
	    // stay on the line we were when we entered the function;
	    // i.e. if one of the 3 chars we read is '\n', step back 3 times;
	    _medium->mseek(-3, GMSEEK_FROM_CURRENT);
	}
    }
    return 0;
}
#undef MAX_ID_LENGTH


int GoAscii::is_obj_desc_begin(GBool* first_obj) {
    // Trying to recognize a "start object description" string;
    // If the string read matches "Head {",
    //		first_obj is set to GTRUE, return 1;
    // If the string read matches "{",
    //		first_obj is set to GFALSE, return 1;
    // Else first_obj is set to GFALSE, return 1;
    // Returns 1 if successfull, else 0;
    // Sets first_obj to true if the obj description is 
    // Does not restore _medium reading pointer;
    int c;
    if (skip_blanks() && (c = _medium->mgetc()) != GEOM) {
	*first_obj = GFALSE;
	if (c == 'H') {
	    char str[3];
	    if (! (_medium->read(str, 3) == 3 &&
		   str[0] == 'e' && str[1] == 'a' && str[2] == 'd' &&
		   skip_blanks() && (c = _medium->mgetc()) != GEOM))
		return 0;
	    *first_obj = GTRUE;
	}
	return (c == '{');
    }
    return 0;
}


int GoAscii::is_obj_desc_line() {
    // Tries to recognize an object descritpion line;
    // (i.e. reads an object Id and the beginning of;
    //       an object description);
    // and records the object Id along with the object;
    // position in the medium in the appropriate GoStream assoc;
    // Returns 0 if the line is not an object description line;
    // Does not restore the medium pointer position in any case;

    _GId id = 0;
    
    if (skip_blanks()) {
	long obj_position = _medium->mtell();
	GBool first_obj = GFALSE;
	if (get_obj_id(&id) && is_obj_desc_begin(&first_obj)) {
	    int dummy;
	    if (_id_obj_assoc.get((void*) id, dummy))
		GWarning("Object %lu multiple definition", id);
	    _GObjRec obj_rec;
	    obj_rec.pos = obj_position;
	    if (first_obj)
		_head_obj.add((void*) obj_position);
	    _id_obj_assoc.set((void*) id, sizeof(_GObjRec), &obj_rec);
	    return 1;
	}
    }
    return 0;
}


int GoAscii::go_to_next_obj() {
    while (! is_obj_desc_line()) {
	if (_medium->readuntil('\n') == GEOM) {
	    return 0;
	}
    }
    return 1;
}

int GoAscii::goToNextObj() {
    if (_head_obj.nelems()) {
	_medium->mseek((long) _head_obj.data(0));
	_head_obj.rem(0);
	return 1;
    }
    //    GWarning("GoAscii: no more head object to restore");
    return 0;
}

int GoAscii::goToObj(const _GObjRec& obj_rec) {
    return _medium->mseek(obj_rec.pos) == 0;
}





