#ifdef REV_INFO
static char rcsid[] = "$Id: GoStream.cc,v 1.5 1992/10/28 16:37:08 mbikina Exp $";
#endif
/*****************************************************************************
MODULE:  GoStream
FILE:    GoStream.cc
AUTHOR:  Gilles Benati
DATE:    May 30th, 1991

DESCRIPTION:
                < see the GoStream.h header file >

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#include <Go/Tools/Error.h>
#include <Go/Tools/Alloc.h>
#include <Go/Tools/StringAssoc.h>
#include <Go/Tools/Medium.h>
#include <Go/GoStream.h>


// Holds save/restore functions by class names;
// Should really be a protected static member of GoStream;
// but C++ AT&T does not like it, so ... <sigh>;
static GStringAssoc* _class_func_assoc = NULL;



int GoStream::restoreObj(void** obj, const char (** class_name)) {
    if (! _class_func_assoc) {
	GWarning("GoStream : no save/restore function has been bound yet");
	return 0;
    }
    
    int        nb_obj  = 0;
    _GObjRec*  obj_rec = NULL;
    _GId       id      = (_GId) NULL;
    unsigned   long position = 0;
    *obj        = NULL;
    *class_name = NULL;

    if (_level) {
	// The request comes from an object restore method;
	// we have to ...;
	//	- read the object id as any object parameter;
	//	- get the corresponding object in _id_obj_assoc;
	//	- if the object has not been restored yet, do it now;
	//	- put the object in obj and return;
	if (! readObj(&id))
	    return 0;

	// if reading the NULL object (i.e. id == 0), return 1 with obj = NULL;
	if (!id)
	    return 1;
	    
	obj_rec = (_GObjRec*) _id_obj_assoc.get((void*) id, nb_obj);
	if (! nb_obj) {
	    GWarning("GoStream: obj %lu undefined", id);
	    return 0;
	}
	if (obj_rec->obj) {
	    *obj = obj_rec->obj;
	    *class_name = obj_rec->cname;
	    return 1;
	}
	// Now we must find the object description matching id and restore it;
	// we must be able to go back to our current position in _medium;
	// Remember to do _medium->mseek(position) before returning;
	position = _medium->mtell();

	if (! goToObj(*obj_rec)) {
	    GWarning("GoStream: obj %lu out of bound", id);
	    _medium->mseek(position);
	    return 0;
	}

	_GId id_check;
	if (! (readObjId(&id_check) && readObjBegin())) {
	    _medium->mseek(position);
	    return 0;
	}

	if (id != id_check)
	    GWarning("GoStream: id %lu does not match obj description %lu",
		    id, id_check); // go on anyway ! ;
    }
    else {
	// This is a top level request;
	// we have to ...;
	//	- go to the first or following object and restore it;
	//	- put the object in obj and return;
	
	if (! goToNextObj()) {
	    // no more objects to restore;
	    return 1;    // this is not an error, so do not return 0;
	}

	if (! (readObjId(&id) && readObjBegin()))
	    return 0;

	// if reading the NULL object (i.e. id == 0), return 1 with obj = NULL;
	if (! id)
	    return readObjEnd();

	obj_rec = (_GObjRec*) _id_obj_assoc.get((void*) id, nb_obj);
	if (! nb_obj) {
	    GWarning("GoStream: obj %lu undefined", id);
	    return 0;
	}
    }

    // Restore the object : here, we must be ready to read the class name;
    // If position != 0, remember to restore the _medium reading pointer;
    // by doing _medium->mseek(position) before returning;
    
    char str[256];
    char *classname = str;
    int return_val = 0;
        
    if (readClassName(256, &classname)) {
	// increase level (= number of restore recursion);
	// do not return before decreasing it ;
	_level++ ;

	int nb_rec ;
	const funcRec* func_rec = (const funcRec*)
	    _class_func_assoc->get(classname, nb_rec);
	if (! nb_rec) {
	    GWarning("GoStream: %s object class not registered", classname);
	    // we should try to do some error recovering here!!.....;
	}
	else if (nb_rec > 1) {
	    GWarning("GoStream: %s object class has several restore functions",
		    classname);
	    // we should try to do some error recovering here!!.....;
	}
	else if (! func_rec->restoreFunc) {
	    GWarning("GoStream: %s object class has no restore function",
		    classname);
	    // we should try to do some error recovering here!!.....;
	}
	else {
	    obj_rec->cname = func_rec->cname;
	    if ((*func_rec->restoreFunc)(&(obj_rec->obj), this)) {
		*obj        = obj_rec->obj;
		*class_name = obj_rec->cname;
		return_val  = 1;
	    }
	    else {
		obj_rec->obj   = NULL;
		obj_rec->cname = NULL;
	    }
	}
	_level--;
    }
    
    if (position)
	_medium->mseek(position);
    return return_val;
}



int GoStream::readClassName(unsigned long ul,char **className)
{
    int ret = readString(ul,className) ;
    return ret ;
}



int GoStream::saveNullObject()
{
    int ret = 0 ;
    ret = ( writeObjId(0) && writeObjBegin(GTRUE) && writeObjEnd()) ;
    return ret ;
}



/**
FUNCTION
    int GoStream::saveObj(const void* obj,
                          const char* class_name, const char* comment = NULL);

DESCRIPTION
    
**/
int GoStream::saveObj(const void* obj,
		      const char* class_name, const char* comment) {

    if (!(_mode & GFLAG_USER_HEADER))
	_mode = (_mode | GFLAG_USER_HEADER) ;
    
    if (! _class_func_assoc) {
	GWarning("GoStream : no save/restore function has been bound yet");
	return 0;
    }

    // first, check if obj is NULL, in which case, save it without...;
    // adding it in _obj_id_assoc;
    if (! obj) {
	if (_level)
	    return  writeObj(0, class_name, comment);
	else return saveNullObject(); 
    }
    else {
	int   nb_assoc;
	const IdRec *id_rec = (IdRec*) _obj_id_assoc.get(obj, nb_assoc);
	IdRec tmp;
	
	// if nb_assoc == 0, this is the first reference to obj;
	// it needs an id and an entry in _obj_id_assoc;

	if (! nb_assoc) {
	    int nb;
	    const funcRec* funcs = (const funcRec*)
		_class_func_assoc->get(class_name, nb);
	    if (! (nb && funcs->saveFunc)) {
		GWarning("GoStream : no registered save function for class %s",
			class_name);
		return 0;
	    }
	    tmp.funcs = *funcs;
	    tmp.id    = newId(obj);
	    id_rec    = &tmp;
	    _obj_id_assoc.set(obj, sizeof(IdRec), id_rec); // new assoc entry;
	    
	    if (_level)
		_to_be_done.add(obj);  // to be saved later (when _level == 0);
	}

	if (_level)
	    return writeObj(id_rec->id, id_rec->funcs.cname, comment);
    
	_level++;
	if (! realSave(obj, id_rec, nb_assoc == 0)) {
	    _level--;
	    return 0;
	}

	// here, look for unsaved objects in;
	// _obj_id_assoc and "real" save them;

	int still_to_save, next_nb_assoc;
	while (still_to_save = _to_be_done.nelems()) {
	    
	    // get the last element, remove it from the array and save it;

	    void* next_obj = _to_be_done.data(still_to_save - 1);
	_to_be_done.rem();
	    const IdRec *next_id_rec = (IdRec*)
		_obj_id_assoc.get(next_obj, next_nb_assoc);
	    if (! (next_nb_assoc && realSave(next_obj, next_id_rec, GFALSE)))
		return 0;
	}
	_level--;
    }
    return 1;
}




int GoStream::realSave(const void* obj, const void* data, GBool first_obj) {
    const IdRec* id_rec = (const IdRec*) data;

    if (! (id_rec->funcs.saveFunc))
	GWarning("GoStream : no registered save function for class %s",
		id_rec->funcs.cname);
    else if (writeObjId(id_rec->id) && writeObjBegin(first_obj) &&
	     writeString(id_rec->funcs.cname, "class name")  &&
	     (*id_rec->funcs.saveFunc)(obj, this))
	return writeObjEnd();
    return 0;
}


_GId GoStream::newId(const void*) {
    return (++_last_id);
}


GoStream::GoStream(GMedium* medium  /* = NULL */) {
    _level   = 0;
    _last_id = (_GId) NULL;
    _medium  = medium;
}


GoStream::~GoStream() {}


/**
FUNCTION
    GoStream::bind(const char* class_name, int saveProc(), int restoreProc())
DESCRIPTION
    
**/
void GoStream::bind(const char* class_name,
		    int (*saveProc) (const void*, GoStream*),
		    int (*restoreProc) (void**, GoStream*)) {
    if (! _class_func_assoc)
	_class_func_assoc = new GStringAssoc;

    int nb_prev_rec;
    const funcRec* prev_rec = (const funcRec*)
	_class_func_assoc->get(class_name, nb_prev_rec);
    if (nb_prev_rec) {
	// we found an existing binding;
	if (prev_rec->saveFunc != saveProc ||
	    prev_rec->restoreFunc != restoreProc)
	    GWarning("GoStream bind: %s save/restore fncts already defined",
		    class_name);
	// else, we are trying to do same binding... do nothing;
    }
    else {
	// new binding... do it;
	funcRec func_rec;
	func_rec.saveFunc = saveProc;
	func_rec.restoreFunc = restoreProc;
	char* str = (char *) GMALLOC((strlen(class_name) + 1) * sizeof(char));
	strcpy(str, class_name);
	func_rec.cname = str;
	_class_func_assoc->set(class_name, sizeof(funcRec), &func_rec);
    }
}


/**
FUNCTION
    int openRead(GMedium*  = NULL);
DESCRIPTION
    if medium is non NULL it sets _medium to medium;
    if _medium is non NULL it tries to read the medium at position 0.
    it returns 1 on completion  or 0 on error.
**/
int GoStream::openRead(GMedium* medium /* = NULL */)
{
    _mode = (_mode & GOPEN_READ) ;
    if (medium)
	_medium = medium;
    
    if (_medium) {
	_medium->mseek(0);;
	int c = _medium->mgetc();
	if (c != GEOM ) {
	    _medium->mseek(0);
	    return 1;
	}
    }
    return 0;
}

/**
FUNCTION
    int openWrite(GMedium*  = NULL);
DESCRIPTION
    if medium is non NULL it sets _medium to medium;
    if _medium is non NULL it returns 1 else 0
**/
int GoStream::openWrite(GMedium* medium /* = NULL */)
{
    _mode = ( _mode & GOPEN_WRITE );
    if (medium)
	_medium = medium;
    
    if (_medium)
	return 1;
    return 0;
}


int GoStream::writeInt(int value, const char* comment) {
    return writeLong(value, comment);
}

int GoStream::readInt(int* value) {
    long lval = *value;
    int ret   = readLong(&lval);
    *value    = (int) lval;
    return ret;
}

void GoStream::close()					{}
int GoStream::rewind()					{ return 0; }
int GoStream::forward()					{ return 0; }
int GoStream::writeHeader(unsigned long , const char*)	{ return 0; }
int GoStream::readHeader (unsigned long*, char**)       { return 0; }
int GoStream::writeFooter()				{ return 0; }
int GoStream::writeComment(const char*)			{ return 0; }
int GoStream::deleteFooter()				{ return 0; }

int GoStream::writeObj(_GId, const char*, const char*)	{ return 0; }
int GoStream::writeObjId(_GId)				{ return 0; }
int GoStream::writeObjBegin(GBool)			{ return 0; }
int GoStream::writeObjEnd()				{ return 0; }
int GoStream::readObj(_GId*)				{ return 0; }
int GoStream::readObjId(_GId*)				{ return 0; }
int GoStream::readObjBegin()				{ return 0; }
int GoStream::readObjEnd()				{ return 0; }

int GoStream::goToObj(const _GObjRec&)			{ return 0; }
int GoStream::goToNextObj()				{ return 0; }

int GoStream::writeBool(GBool, const char*)		{ return 0; }
int GoStream::readBool(GBool*)				{ return 0; }

int GoStream::writeShort(short, const char*)		{ return 0; }
int GoStream::readShort(short*)				{ return 0; }

int GoStream::writeLong(long, const char*)		{ return 0; }
int GoStream::readLong(long*)				{ return 0; }

int GoStream::writeChar(char, const char*)		{ return 0; }
int GoStream::readChar(char*)				{ return 0; }

int GoStream::writeNumber(GNumber, const char*)		{ return 0; }
int GoStream::readNumber(GNumber*)			{ return 0; }

int GoStream::writeDouble(double, const char*)		{ return 0; }
int GoStream::readDouble(double*)			{ return 0; }

int GoStream::writeVector(const GVector&, const char*)	{ return 0; }
int GoStream::readVector(GVector*)			{ return 0; }

int GoStream::writePoint(const GPoint&, const char*)	{ return 0; }
int GoStream::readPoint(GPoint*)				{ return 0; }

int GoStream::writeBox(const GBox&, const char*)		{ return 0; }
int GoStream::readBox(GBox*)				{ return 0; }

int GoStream::writeRgb(const GRgb&, const char*)		{ return 0; }
int GoStream::readRgb(GRgb*)				{ return 0; }

int GoStream::writeCoordSys(const GCoordSys&,const char*){ return 0; }
int GoStream::readCoordSys(GCoordSys*)			{ return 0; }

int GoStream::writeRegion(const GRegion&, const char*)	{ return 0; }
int GoStream::readRegion(GRegion*)			{ return 0; }

    // **read spec carefully !**;
int GoStream::writeString(const char*, const char*)	{ return 0; }
int GoStream::readString(unsigned long, char**)		{ return 0; }

    // **read spec carefully !**;
int GoStream::writeBuffer(unsigned long, const char*, const char*) { return 0; }
int GoStream::readBuffer(unsigned long*, char**)	{ return 0; }





