#ifdef REV_INFO
static char rcsid[] = "$Id: Alloc.cc,v 1.2 1992/09/25 13:58:09 smadja Exp $";
#endif
/*****************************************************************************
MODULE:  Tools
FILE:    Alloc.cc
AUTHOR:  Philippe Smadja
DATE:    February 1990

DESCRIPTION: 
         Memory management functions

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include <Go/Tools/Alloc.h>
#include <Go/Tools/Assoc.h>
#include <Go/Tools/Error.h>


#ifdef WINDOWS_int_16b
#include <windows.h>

#ifndef __BORLANDC__
GAssoc assocMultiSeg;
#else
GAssoc* p_assocMultiSeg = new GAssoc;
GAssoc& assocMultiSeg = * p_assocMultiSeg;
#endif

char * MultiSegmentAllocation(unsigned long size);
char * MultiSegmentReAlloc(char * p, unsigned long size);
void MultiSegmentFree(char * p);
#endif

/**
FUNCTION
    void* _GMalloc(unsigned long nbytes, GBool fatal = GTRUE) 
DESCRIPTION
    Allocation of nbytes. If there is an error, it returns NULL
    or exits depending on fatal.
**/
void* _GMalloc (unsigned long nbytes, GBool fatal)
{
    if (nbytes)
    {
	void* p;
#ifdef WINDOWS_int_16b
        if(nbytes < 0x10000L)
	    p = malloc((uint)nbytes);
	else
	    p = MultiSegmentAllocation(nbytes);
#else
	p = (void*)malloc((uint)nbytes);
#endif
	if (!p && fatal)
	    GError (" -- Not Enough Memory -- ");
	else return p;
    }
    return NULL;
}

/**
FUNCTION
    void _GFree (void** p)
DESCRIPTION
    This function frees the memory pointed  to by p (if non null);
    p is set to NULL.
**/
void _GFree (void** p) 
{
#ifdef WINDOWS_int_16b
    MultiSegmentFree((char *) *p);
    *p = NULL; 
#else
    if (*p)
    {
	free((char*)*p);
	*p = NULL;
    }	
#endif
}


/**
FUNCTION    _GRealloc (void** p, unsigned long nbytes, GBool fatal = GTRUE)
DESCRIPTION
    If *p is non null "realloc" else "alloc". If there is
    an error, it returns NULL or exits depending on fatal.
    If nbytes is 0, p is freed.
**/
void _GRealloc (void** p, unsigned long nbytes, GBool fatal) 
{
    if (!nbytes) 
    {
        _GFree(p);
	return;
    }

    if (*p == NULL)
    {
       *p = _GMalloc (nbytes, fatal);
       return;
    }
#ifdef WINDOWS_int_16b
    *p = MultiSegmentReAlloc((char *) *p, nbytes);
#else
    *p = (void*)realloc((char*)*p, (uint)nbytes);
#endif

    if (*p == NULL && fatal)
	GError (" -- Not Enough Memory -- ");
}



#ifdef WINDOWS_int_16b


char * MultiSegmentAllocation(unsigned long size)
{
    char huge * lpMem;
    HANDLE hMem;

    if((hMem = GlobalAlloc(GMEM_MOVEABLE | GMEM_DISCARDABLE | GMEM_ZEROINIT, 
		size)) 
       != NULL)
    {
	lpMem = (char huge *) GlobalLock(hMem);
	assocMultiSeg.set((void *)(lpMem), (void *)(0xffffL & hMem));
	return (char*)lpMem;
    }
    else
    {
	return NULL;
    }
}

char * MultiSegmentReAlloc(char * p, unsigned long size)
{
    HANDLE hTempHugeObject;
    HANDLE hHugeMemSaved;
    char huge * lpMemHuge;

    hHugeMemSaved = (HANDLE) assocMultiSeg.get((void*) p); 
    if(hHugeMemSaved == NULL) {
	p = (char*) realloc(p, (uint)size);
	return p;
	}

    if((hTempHugeObject = GlobalReAlloc(hHugeMemSaved, size, GMEM_MOVEABLE |
				       GMEM_DISCARDABLE| GMEM_ZEROINIT)))
    {
	lpMemHuge = (char huge *) GlobalLock(hTempHugeObject);
	if(p == (char*)lpMemHuge)
	{
	    assocMultiSeg.rem((void *) p);
	    assocMultiSeg.set((void*)p, 
			      (void *)(0xffffL & hTempHugeObject));
	    return p;
	}
	else
	{
	    assocMultiSeg.set((void*)lpMemHuge, 
			      (void *)(0xffffL & hTempHugeObject));
	    return (char*)lpMemHuge;
	}
    }
    else
	return p; 		
}

void MultiSegmentFree(char * p)
{
    if(p)
    {
	HANDLE hHugeMemSaved;
	hHugeMemSaved = (HANDLE) assocMultiSeg.get((void *) p);
	if(hHugeMemSaved != NULL)
	{
	    GlobalUnlock(hHugeMemSaved);
	    GlobalFree(hHugeMemSaved);
	    assocMultiSeg.rem((void*) p);
	}
	else	
	    free(p);
    }
}
#endif
