#ifdef REV_INFO
static char rcsid[] = "$Id: ShArray.cc,v 1.3 1992/09/25 14:00:06 smadja Exp $";
#endif
/*****************************************************************************
MODULE:  Tools
FILE:    ShArray.cc
AUTHOR:  Philippe Smadja
DATE:    Dec 90

DESCRIPTION:
  see <Go/Tools/ShArray.h>
  
COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#include <Go/Tools/ShArray.h>
#include <Go/GoStream.h>
#include <Go/Tools/Alloc.h>
#include <assert.h>

/*
 * static of the class
 */
GList*        GShArray::_headOfList;
GFunctionList GShArray::_funcList;


/**
  PROTECTED SH ARRAY METHODS
  **/


int GShArray::save(const GShArray*, GoStream*) {
    return 0;
}

/**/
int GShArray::restore(GShArray**, GoStream*) {
    return 0;
}

/**/
int GShArray::readParams(GShArray*, GoStream*) {
    return 0;
}

/**/
GShArray* _newGShArray (GFunctionList* funcList, ulong mask,
		       const void * values, const char* name)
{
   return new GShArray(funcList, mask,(const GShArrayValues*) values, name);
}

/**/
GShArray::GShArray (GFunctionList* funcList, ulong mask,
		    const GShArrayValues* values,
		    const char* name) : GSharedObject(funcList, mask, name)
{
    assert (mask == GSHARRAY_MASK);
    assert (name == NULL);
    _array = new GArray(values->data,values->nelems, values->elemsize);
}

/**/
GList*& GShArray::_head ()
{
    return _headOfList;
}

/**/
int GShArray::_valueSize ()
{
    return (sizeof(GShArrayValues));
}

/**/
GBool GShArray::_match (ulong mask, const void * values1) const
{
const GShArrayValues* values = (const GShArrayValues*)values1;

    assert ( mask == GSHARRAY_MASK);
    if (values->elemsize != _array->elemsize())
	return GFALSE;
    if (values->nelems !=  _array->nelems())
	return GFALSE;
    return (GBool) (memcmp(_array->data(), values->data,
		  	  _array->nelems()*_array->elemsize()) == 0);
}

/**/
void GShArray::_makeNewValues (ulong mask, const void * values,
			       ulong& newMask, void * newValues) const
{
const GShArrayValues* __values = (const GShArrayValues*) values;
GShArrayValues* __newValues = ( GShArrayValues*) newValues;

    *__newValues = *__values;
    newMask = mask;
}

/**/
ulong GShArray::_update (ulong mask, const void * __values)
{
const GShArrayValues* values =(const GShArrayValues*) __values;

    delete _array;
    _array = new GArray(values->data, values->nelems,
			values->elemsize);
    return mask;
}

/**/
GShArray::~GShArray ()
{
    delete _array;
}

/**
  PUBLIC GShArray METHODS
  **********************
  **/

/* static */
const char* GShArray::classname()
{
    return "GShArray";
}

/* virtual */
const char* GShArray::getClassname() const
{
    return GShArray::classname();
}

/**/
GShArray* GCreateShArray (const GShArrayValues* values)
{
   return (GShArray*)GCREATE_SHAREDOBJECT(GShArray, GSHARRAY_MASK, values, NULL);
}

/**
FUNCTION
    GCreateShArray
DESCRIPTION
    This function  creates an shared array of "nelems"
    of elemsize.
    the data content is put inside the array.
**/
GShArray* GCreateShArray (void* data, int nelems, int elemsize)
{
    GShArrayValues values;
    values.elemsize = elemsize;
    values.nelems = nelems;
    values.data   = data;
    return GCreateShArray(&values);
}

/**/
void GAddElem(GShArray*& sharray, void* elem, int nelems_to_add)
{
    GShArrayValues values;
    values.elemsize = sharray->elemsize();
    values.nelems   = sharray->nelems() + nelems_to_add;
    int size  = values.elemsize* values.nelems;
#ifdef NO_ALLOCA
    values.data = GMALLOC(size);
#else
    values.data = alloca(size);
#endif
    memcpy(values.data, sharray->data(),
	   size = sharray->nelems()*values.elemsize);
    memcpy(((char*)values.data) + size, elem, values.elemsize*nelems_to_add);
    /**
      make the modif
      **/
    sharray->modify((GSharedObject*&)sharray, GSHARRAY_MASK, &values);
#ifdef NO_ALLOCA
    GFREE(values.data);
#endif
}

/**/
void GRemElem (GShArray*& sharray, int index, int nelems_to_free)
{
    assert (index + nelems_to_free <= sharray->nelems());
    if (nelems_to_free == sharray->nelems())
    {
	sharray->destroy();
	sharray = NULL;
	return;
    }

    GShArrayValues values;
    int size = sharray->elemsize() * (sharray->nelems()-nelems_to_free);
#ifdef NO_ALLOCA
    values.data = GMALLOC(size);
#else
    values.data = alloca(size);
#endif
    values.elemsize = sharray->elemsize();
    size = values.elemsize*index;
    memcpy(values.data, sharray->data(), size);

    size = values.elemsize*(sharray->nelems()-(index+nelems_to_free));
    memcpy(((char*)values.data) + values.elemsize*index,
	   ((char*)sharray->data()) + values.elemsize*(index+nelems_to_free),
	   size);

    values.nelems = sharray->nelems()  - nelems_to_free;
    /**
      make the modif
      **/
    sharray->modify((GSharedObject*&)sharray, GSHARRAY_MASK, &values);
#ifdef NO_ALLOCA
    GFREE(values.data);
#endif
}
