#ifdef REV_INFO
static char rcsid[] = "$Id: Atom.cc,v 1.2 1992/05/26 16:18:01 benati Exp $";
#endif
/*****************************************************************************
MODULE:  Tools
FILE:    Atom.cc
AUTHOR:  Philippe Smadja
DATE:    Dec 90

DESCRIPTION:
    Association of string to atom for atom comparison.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include <Go/Tools/StringAssoc.h>
#include <Go/Tools/Atom.h>
#include <assert.h>

static GStringAssoc& Assoc() {
    static GStringAssoc *assoc = NULL;

    if (! assoc)
	assoc = new GStringAssoc();

    return *assoc;
}

struct GStringRef
{
    char* string;
    int   nref;
};

GAtom GCreateAtom(const char* string)
{
    int num;
    GStringRef* sr = (GStringRef*)Assoc().get(string, num);
    if (sr == NULL) {
	GStringRef _sr;
	_sr.string = new char[strlen(string)+1] ;
	strcpy(_sr.string, string);
	_sr.nref = 1;
	Assoc().set(string, sizeof _sr, &_sr);
	return  _sr.string;
    }
    assert (num == 1);
    sr->nref++;
    return  sr->string;
}

void GFreeAtom(GAtom atom)
{
    int num;
    GStringRef* sr = (GStringRef*)Assoc().get((char*)atom, num);
    assert (sr);
    assert (num == 1);
    assert (atom == (void*)sr->string);
    if (--sr->nref == 0) {
	char* string = sr->string;
	/** remove the struct associated to string **/
	Assoc().rem (string);

	/** *sr is not available now **/
	delete string;

    }
}


