#ifdef REV_INFO
static char rcsid[] = "$Id: Bind.cc,v 1.2 1992/12/10 13:55:05 jojic Exp $";
#endif
/*****************************************************************************
MODULE:  Go Tools
FILE:    Bind.cc
AUTHOR:  jojic
DATE:    Jun 29 1992

DESCRIPTION:
  See also the Bind.h header.
  
COPYRIGHT:
  Copyright (c) 1992 Bull S.A.
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull S.A. may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull S.A..
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull S.A. under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull S.A..

  Bull S.A. assumes no responsibility for the use or inability to use
  any of its software. The software is provided "AS IS" without warranty
  of any kind, and Bull S.A. expressly disclaims all implied warranties,
  including but not limited to the implied warranties of merchantability
  and fitness for a particular purpose.

  In no event shall Bull S.A. be liable for any special, incidental,
  indirect or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#include <Go/Tools/Bind.h>
#include <Go/Tools/StringAssoc.h>
#include <Go/Tools/Error.h>



struct BindRec
{
    void* value;
    const char* type;

    BindRec(void* v, const char* t) { value = v; type = t; }
};


/*
 * A static function is used in place of a static variable to make
 * the porting on Windows easier.
 */
static GStringAssoc* bind_assoc ()
{
    static GStringAssoc* assoc;

    if (assoc == NULL)
	assoc = new GStringAssoc;

    return assoc;
}


/*
 *-------------------------------------------------------
 * public functions
 *-------------------------------------------------------
 */


/*
 * This function sets the value ``value'' to the key ``name''.
 * If a previous value is overwritten, `GBind' calls ``GWarning''
 * with a corresponding error message.
 *
 * ``name'' is a string, that means that key comparison is done
 * by comparing all characters of the string (and not the equality
 * of string addresses).
 */
void GBind (const char* name, void* value, const char* type)
{
    GStringAssoc* assoc = bind_assoc ();
    int count;
    BindRec* prev_value = (BindRec*) assoc->get (name, count);

    if (count > 0)
    {
	if (prev_value->value == value || prev_value->type == type)
	    return;
	else
	    GWarning("GBind: setting a value twice for key \"%s\"", name);
    }
    
    BindRec bind_rec(value, type);
    assoc->set (name, sizeof bind_rec, &bind_rec);
}


/*
 * This method returns the value ``value'' which was previously
 * associated to ``name''.
 * If there were no such assocation, `GBind' returns NULL.
 */
GBool GBinded (const char* name, void*& value, const char*& type)
{
    int count;
    BindRec* bind_rec = (BindRec*) bind_assoc()->get(name, count);
    if (count == 0)
	return GFALSE;

    if (&value != NULL)
	value = bind_rec->value;
    if (&type != NULL)
	type = bind_rec->type;

    return GTRUE;
}


void GRemoveBinding (const char* name)
{
    bind_assoc ()->rem (name);
}
