#ifdef REV_INFO
static char rcsid[] = "$Id: Box.cc,v 1.2 1992/05/26 16:29:25 benati Exp $";
#endif
/*****************************************************************************
MODULE:  Tools
FILE:    Box.cc
AUTHOR:  Olivier Jojic
DATE:    29 May 1990

DESCRIPTION:
    See Box.h

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include <Go/Tools/Box.h>

const GBox& GEmptyBox() {
    static GBox* empty_box = NULL;

    if (! empty_box)
	empty_box = new GBox(1, 1, 0, 0);

    return *empty_box;
}

const GBox& GBiggestBox() {
    static GBox* biggest_box = NULL;

    if (! biggest_box)
	biggest_box = new GBox(-MAXFLOAT, -MAXFLOAT, MAXFLOAT, MAXFLOAT);

    return *biggest_box;
}


/*
 * This method may write up to 3 boxes inside "boxes".
 * The result boxes are disjoined and orderred by y axis
 * and then by x axis.
 *
 * The method return the number of boxes written.
 *
 * a and b are supposed not empty.
 */
int GUnion (const GBox& a, const GBox& b, GBox* pbox)
{
    if (b.ymin < a.ymin ||
	b.ymin == a.ymin && b.xmin < a.xmin)
	return GUnion(b, a, pbox);

    // now:
    //      a is before b (y then x axis);

    if (a.ymax < b.ymin || a.xmax < b.xmin || b.xmax < a.xmin)
    {
	pbox[0] = a;
	pbox[1] = b;
	return 2;
    }

    // now:
    // 	    a is before b (y then x axis);
    //      a and b intersect;

    if (a.xmin == b.xmin && a.xmax == b.xmax)
    {
	pbox->ymin = a.ymin;
	pbox->ymax = max(a.ymax, b.ymax);
	pbox->xmin = a.xmin;
	pbox->xmax = a.xmax;
	return 1;
    }

    // now:
    //      a is before b (y then x axis);
    //      a and b intersect;
    //      they are not on the "same column";

    if (a.ymax == b.ymin)
    {
	pbox[0] = a;
	pbox[1] = b;
	return 2;
    }

    int count = 0;

    if (a.ymin != b.ymin) // i.e. less than
    {
	pbox->ymin = a.ymin;
	pbox->ymax = b.ymin;
	pbox->xmin = a.xmin;
	pbox->xmax = a.xmax;
	pbox++;
	count++;
    }

    pbox->ymin = b.ymin;
    pbox->xmin = min(a.xmin, b.xmin);
    pbox->xmax = max(a.xmax, b.xmax);

    if (a.ymax < b.ymax)
    {
	pbox->ymax = a.ymax;
	pbox++;
	count++;

	pbox->ymin = a.ymax;
	pbox->ymax = b.ymax;
	pbox->xmin = b.xmin;
	pbox->xmax = b.xmax;
    }
    else if (b.ymax < a.ymax)
    {
	pbox->ymax = b.ymax;
	pbox++;
	count++;

	pbox->ymin = b.ymax;
	pbox->ymax = a.ymax;
	pbox->xmin = a.xmin;
	pbox->xmax = a.xmax;
    }
    else // a.ymax == b.ymax
	pbox->ymax = a.ymax;

    return count + 1;
}

/*
 * This method may write up to 4 boxes inside "boxes".
 * The result boxes are disjoined and orderred by y axis
 * and then by x axis.
 *
 * The method return the number of boxes written.
 *
 * a and b are supposed not empty.
 */
int GSubtraction (const GBox& a, const GBox& b, GBox* pbox)
{
    /* a mathematical width of 0 in b cannot subtract anything in a */
    if (b.xmax <= b.xmin || b.ymax <= b.ymin ||
	!GIntersect(a, b))
    {
	*pbox = a;
	return 1;
    }

    // now:
    //      a and b intersect;

    int count = 0;
    GNumber firsty;
    GNumber lasty;

    if (a.ymin < b.ymin)
    {
	firsty = b.ymin;
	pbox->ymin = a.ymin;
	pbox->ymax = b.ymin;
	pbox->xmin = a.xmin;
	pbox->xmax = a.xmax;
	pbox++;
	count++;
    }
    else
	firsty = a.ymin;

    lasty = (a.ymax > b.ymax) ? b.ymax : a.ymax;

    if (a.xmin < b.xmin)
    {
	pbox->ymin = firsty;
	pbox->ymax = lasty;
	pbox->xmin = a.xmin;
	pbox->xmax = b.xmin;
	pbox++;
	count++;
    }

    if (a.xmax > b.xmax)
    {
	pbox->ymin = firsty;
	pbox->ymax = lasty;
	pbox->xmin = b.xmax;
	pbox->xmax = a.xmax;
	pbox++;
	count++;
    }

    if (a.ymax > b.ymax)
    {
	pbox->ymin = b.ymax;
	pbox->ymax = a.ymax;
	pbox->xmin = a.xmin;
	pbox->xmax = a.xmax;
	pbox++;
	count++;
    }

    return count;
}

GBox operator + (const GBox& a, const GBox& b)
{
    if (a.empty()) return b;
    if (b.empty()) return a;

    return GBox(min(a.xmin, b.xmin),
	       min(a.ymin, b.ymin),
	       max(a.xmax, b.xmax),
	       max(a.ymax, b.ymax));
}

GBox operator - (const GBox& a, const GBox& b)
{
    if (a.empty() || b.empty())
	return a;

    GBox box(a);
    if (b.xmin <= box.xmin && b.xmax >= box.xmax)
    {
	if (b.ymax >= box.ymin) box.ymin = b.ymax;
	if (b.ymin <= box.ymax) box.ymax = b.ymin;
    }
    if (b.ymin <= box.ymin && b.ymax >= box.ymax)
    {
	if (b.xmax >= box.xmin) box.xmin = b.xmax;
	if (b.xmin <= box.xmax) box.xmax = b.xmin;
    }

    return box;
}

GBox& GBox::operator += (const GBox& box)
{
    if (empty())
	*this = box;
    else if (!box.empty())
    {
	if (box.xmin < xmin) xmin = box.xmin;
	if (box.xmax > xmax) xmax = box.xmax;
	if (box.ymin < ymin) ymin = box.ymin;
	if (box.ymax > ymax) ymax = box.ymax;
    }

    return *this;
}

GBox& GBox::operator &= (const GBox& box)
{
    if (!empty())
    {
	if (box.empty()) { xmin = 1; xmax = 0; }
	else
	    *this = GIntersection(*this, box);
    }

    return *this;
}

GBox::GBox (const GPoint& a, const GPoint& b, const GPoint& c, const GPoint& d)
{
    if (a.x <= b.x)
	if (c.x <= d.x)	{
	    xmin = min(a.x, c.x);
	    xmax = max(b.x, d.x);
	} else {
	    xmin = min(a.x, d.x);
	    xmax = max(b.x, c.x);
        }
    else
	if (c.x <= d.x)	{
	    xmin = min(b.x, c.x);
	    xmax = max(a.x, d.x);
	} else {
	    xmin = min(b.x, d.x);
	    xmax = max(a.x, c.x);
	}

    if (a.y <= b.y)
	if (c.y <= d.y)	{
	    ymin = min(a.y, c.y);
	    ymax = max(b.y, d.y);
	} else {
	    ymin = min(a.y, d.y);
	    ymax = max(b.y, c.y);
	}
    else
	if (c.y <= d.y) {
	    ymin = min(b.y, c.y);
	    ymax = max(a.y, d.y);
	} else {
	    ymin = min(b.y, d.y);
	    ymax = max(a.y, c.y);
	}
}

GBox& GBox::operator -= (const GBox& box)
{
    return *this = *this - box;
}


GBox::GBox (const GPoint& a, const GPoint& b)
{
    if (a.x <= b.x) { xmin = a.x; xmax = b.x; }
    else	    { xmin = b.x; xmax = a.x; }

    if (a.y <= b.y) { ymin = a.y; ymax = b.y; }
    else	    { ymin = b.y; ymax = a.y; }
}


GBool operator == (const GBox& a, const GBox& b)
{
    return GEqual(a, b) ||
	   a.empty() && b.empty();
}

GBool operator != (const GBox& a, const GBox& b)
{
    return GNotEqual(a, b) &&
	   !(a.empty() && b.empty());
}


GBool operator && (const GBox& a, const GBox& b)
{
    return GIntersect(a, b) && !a.empty() && !b.empty();
}


GBox GIntersection (const GBox& a, const GBox& b)
{
    return GBox(max(a.xmin, b.xmin),
	       max(a.ymin, b.ymin),
	       min(a.xmax, b.xmax),
	       min(a.ymax, b.ymax));
}

GBox operator & (const GBox& a, const GBox& b)
{
    if (a.empty()) return a;
    if (b.empty()) return b;

    return GIntersection(a, b);
}

