#ifdef REV_INFO
static char rcsid[] = "$Id: Buffer.cc,v 1.2 1992/07/06 14:25:23 jacomet Exp $";
#endif
/*****************************************************************************
MODULE:  Tools
FILE:    Buffer.cc
AUTHOR:  Aline Jacomet
DATE:    April 1990

DESCRIPTION: 
    Buffer management (see Go/Tools/Buffer.h)
    
COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include <Go/Tools/Buffer.h>
#include <Go/machdef.h>


/**
FUNCTION
   GBuffer::~GBuffer(void)
DESCRIPTION
   destructor of GBuffer class
**/
 GBuffer::~GBuffer(void)
{
    GCharBuf *p = head, *q;
    
    while (p != NULL) {
	q = p;
	if (p->next)
	    p = (GCharBuf*)((char*)p + p->next);
	else
	    p = NULL;
	delete q;
    }
    head = NULL;
    cursor_write = NULL;
    cursor_read = NULL;
    cursor_end = NULL;
    ptr_write = NULL;
    ptr_read = NULL;
    end_write = NULL;
    size = 0;
    total_written = 0;
}


/**
FUNCTION
  GCharBuf* GBuffer::add(void)
DESCRIPTION
  This function adds an instance of GCharBuf at the linked list
**/
GCharBuf * GBuffer::add(void) 
{
    GCharBuf *p = head, *q = head, *r;
    
    while (p != NULL) {
	q = p;
	if (p->next)
	    p = (GCharBuf*)((char*)p + p->next);
	else
	    p = NULL;
    }

    r = new GCharBuf(size,total_written) ;
    if (head)
	q->next = (char*)r - (char*)q;
    else
	head = r;
    
	if (cursor_read == NULL) {
	    cursor_read = r;
	    ptr_read = r->buffer;
	}
    if (cursor_end == NULL) {
		cursor_end = r;
		end_write = r->buffer-1;
	    }
    return r;
}


/**
FUNCTION
   unsigned long GBuffer::write(const char*, unsigned long)
DESCRIPTION
   This function writes in a linked list.
**/
unsigned long GBuffer::write(const char *buf, unsigned long s)
{
    int nb_free;      // GNumber of free bytes in the buffer;
    ulong nb_total = 0; // GNumber of bytes which are written;
    ulong offset_ret = 0;
    ulong offset;
    
    if (ptr_write != NULL)
	offset_ret = ptr_write - cursor_write->buffer +
	    cursor_write->first_offset;
    
    while (s > nb_total) {
	if (cursor_write == NULL) {
	    if ((cursor_write = add()) == NULL) {
		return 0;
	    }
	    ptr_write = cursor_write->buffer;
	    if (offset_ret == 0)
		offset_ret = ptr_write - cursor_write->buffer + 
			    cursor_write->first_offset;
	}
	nb_free = &(cursor_write->buffer[size-1]) - ptr_write + 1; 
	if (nb_free <= (int)(s - nb_total)) {
	    /*
	     * All the buffer will be written;
	     */
	    memcpy(ptr_write,&buf[nb_total],nb_free);
	    nb_total += nb_free;
	    offset = &(cursor_write->buffer[size-1]) - cursor_write->buffer + 
		    cursor_write->first_offset;
	    if (total_written < offset) {
		total_written = offset;
		end_write = &(cursor_write->buffer[size-1]);
		cursor_end = cursor_write;
	    }
	    if (cursor_write->next)
		cursor_write = (GCharBuf*)((char*)cursor_write +
					  cursor_write->next);
	    else
		cursor_write = NULL;
	    
	    if (cursor_write != NULL) {
		ptr_write = cursor_write->buffer;
	    } else {
		ptr_write = NULL;
	    }
	} else {
	    /*
	     * Only a part of the buffer will be written;
	     */
	    memcpy(ptr_write,&buf[nb_total],(int)(s - nb_total));
	    ptr_write = ptr_write + s - nb_total;	
	    offset = ptr_write - 1 - cursor_write->buffer +
		cursor_write->first_offset;
	    if (total_written < offset) {
		total_written = offset;
		end_write = ptr_write - 1;
		cursor_end = cursor_write;
	    }
	    nb_total = s;
	}
    }
    
    return offset_ret;
}


/**
FUNCTION
   unsigned long GCopy(GBuffer*, GBuffer*)
DESCRIPTION
   This function copies a GBuffer in an other GBuffer
**/
unsigned long GCopy(GBuffer *save, GBuffer *buf)
{
    unsigned long ret = 0, sauv = 0;
    int nb_unread;
    int i = 0;
    
    if (buf->cursor_read == NULL)
	return 0;
    
    while (buf->cursor_read != buf->cursor_end) {
	nb_unread = &(buf->cursor_read->buffer[buf->size-1]) - buf->ptr_read +
	    1;
	i++;
	if ((ret = save->write(buf->ptr_read,nb_unread)) == 0)
	    return 0;
	else 
	    if (i == 1)
		sauv = ret;
	if (buf->cursor_read->next)
	    buf->cursor_read = (GCharBuf*)((char*)buf->cursor_read +
		buf->cursor_read->next);
	else
	    buf->cursor_read = NULL;
	buf->ptr_read = buf->cursor_read->buffer;
    }
    
    nb_unread = buf->end_write - buf->ptr_read + 1;
    i++;
    if ((ret = save->write(buf->ptr_read,nb_unread)) == 0)
	return 0;
    if (i == 1)
	sauv = ret;
    if (buf->end_write == &(buf->cursor_read->buffer[buf->size-1])) {
	if (buf->cursor_read->next)
	    buf->cursor_read = (GCharBuf*)((char*)buf->cursor_read +
		buf->cursor_read->next);
	else
	    buf->cursor_read = NULL;
	if (buf->cursor_read != NULL)
	    buf->ptr_read = buf->end_write + 1;
    } else {
	buf->ptr_read = buf->end_write + 1;
    }
    
    return sauv;
}


/**
FUNCTION
   unsigned long GBuffer::read(char*, unsigned long)
DESCRIPTION
   This function reads n bytes in a linked list.
**/
unsigned long GBuffer::read(char *buf, unsigned long nb_byte)
{
    
    unsigned long nb_unread;    // GNumber of unread bytes in the buffer;
    unsigned long nb_total = 0; // GNumber of bytes which are read; 
    GBool end = GFALSE;
    unsigned long offset_ret = 0;
    
    
    if (cursor_read == NULL) {
	return 0;
    }
    
    offset_ret = ptr_read - cursor_read->buffer + cursor_read->first_offset;
    
    if (offset_ret > end_write - cursor_end->buffer + cursor_end->first_offset)
	return 0;
    
    while (nb_byte > nb_total) {
	
	if (cursor_read == NULL || end == GTRUE)
	    /*
	     * All bytes have been read;
	     */
	return offset_ret;
	
	if (cursor_read == cursor_end) {
	    nb_unread = end_write - ptr_read + 1;
	    end = GTRUE;
	} else {
	    nb_unread = &(cursor_read->buffer[size-1]) - ptr_read + 1; 
	}
	
	if (nb_unread == 0)
	    return 0;
	
	if (nb_unread <= (nb_byte - nb_total)) {
	    /*
	     * All the end of the buffer is read;
	     */
	    memcpy(&buf[nb_total],ptr_read,(int)nb_unread);
	    nb_total += nb_unread;
	    if (end == GFALSE || end_write == &(cursor_read->buffer[size-1])) {
		if (cursor_read->next)
		    cursor_read = (GCharBuf*)((char*)cursor_read +
					     cursor_read->next);
		else
		    cursor_read = NULL;
		if (cursor_read != NULL) 
		    ptr_read = cursor_read->buffer;
	    } else {
		ptr_read = end_write + 1;
	    }	
	} else {
	    /*
	     * Only a part of the buffer is read;
	     */
	    memcpy(&buf[nb_total],ptr_read,(int)(nb_byte - nb_total));
	    ptr_read = ptr_read + nb_byte - nb_total;	
	    nb_total = nb_byte;
	}
    }
    return offset_ret;
}	


/** 
FUNCTION
   char* GBuffer::readString()
DESCRIPTION
   This function reads a string in a linked list
**/
char* GBuffer::readString()
{
    
    GBool end_string = GFALSE, end = GFALSE;
    int nb_unread;
    int j, i, nb_buf = 0;
    char *ptr_save = ptr_read;
    GCharBuf *cursor_save = cursor_read;

    char* buf = NULL;
    char** tab_buf = NULL;
    
    tab_buf = (char**)malloc(20*sizeof(char*));
    
    while (end_string == GFALSE) {
	
	if (cursor_read == NULL || end == GTRUE) {
	    cursor_read = cursor_save;
	    ptr_read = ptr_save;
	    if (tab_buf)
		free((char*) tab_buf);
	    return buf;
	}
	
	if (cursor_read == cursor_end) {
	    nb_unread = end_write - ptr_read + 1;
	    end = GTRUE;
	} else {
	    nb_unread = &(cursor_read->buffer[size-1]) - ptr_read + 1; 
	}
	
	if (nb_unread == 0) {
	    cursor_read = cursor_save;
	    ptr_read = ptr_save;
	    if (tab_buf)
		free((char*) tab_buf);
	    return buf;
	}

	if (nb_buf >= 20)
	    tab_buf = (char**)realloc((char*)tab_buf,
				      sizeof(char*) * (20 + ((nb_buf%20)*20)));
	
	tab_buf[nb_buf] = new char[nb_unread+1];
	strncpy(tab_buf[nb_buf],ptr_read,nb_unread);
	tab_buf[nb_buf][nb_unread] = '\0';
	j = 0;
	for (i = 0; i < nb_unread && end_string == GFALSE; i++) {
	    if (tab_buf[nb_buf][i] == '\0') {
		end_string = GTRUE;
		j++;
	    } else 
		j++;
	}
	if (end_string == GFALSE) {
	    if (end == GFALSE || end_write == &(cursor_read->buffer[size-1])) {
		if (cursor_read->next)
		    cursor_read = (GCharBuf*)((char*)cursor_read +
					     cursor_read->next);
		else
		    cursor_read = NULL;
		if (cursor_read != NULL)
		    ptr_read = cursor_read->buffer;
	    } else {
		ptr_read = end_write + 1;
	    }
	} else {
	    if (ptr_read + j > &(cursor_read->buffer[size-1])) {
		if (cursor_read->next)
		    cursor_read = (GCharBuf*)((char*)cursor_read +
					     cursor_read->next);
		else
		    cursor_read = NULL;
		if (cursor_read != NULL)
		    ptr_read = cursor_read->buffer;
	    } else {
		ptr_read = ptr_read + j;
	    }
	}
	nb_buf++;
    }

    int buf_size = 0;
    for (i = 0; i < nb_buf; i++)
	buf_size += strlen(tab_buf[i]);
    buf_size++;
    buf = new char[buf_size];
    buf[0] = '\0';
    for (i = 0; i < nb_buf; i++)
	strcat(buf, tab_buf[i]);
    buf[buf_size - 1] = '\0';

    if (tab_buf)
	free((char*) tab_buf);
    
    return buf;
}


/**
FUNCTION
   unsigned long GBuffer::writeInFile(FILE*)
DESCRIPTION
   This function writes a linked list in a file
**/
unsigned long GBuffer::writeInFile(FILE *file)
{
    int nb_unread;    // GNumber of unread bytes in the buffer;
    unsigned long nb_total = 0; // GNumber of bytes which are read; 
    char *ptr_save = ptr_read;
    GCharBuf *cursor_save = cursor_read;
    
    if (cursor_read == NULL) {
	return 0;
    }
    
    while (cursor_read != cursor_end) {
	nb_unread = &(cursor_read->buffer[size-1]) - ptr_read + 1;
	if (fwrite(ptr_read,sizeof(*ptr_read),nb_unread,file) == 0) {
	    cursor_read = cursor_save;
	    ptr_read = ptr_save;
	    return 0;
	};
	nb_total += nb_unread;
	if (cursor_read->next)
	    cursor_read = (GCharBuf*)((char*)cursor_read + cursor_read->next);
	else
	    cursor_read = NULL;
	ptr_read = cursor_read->buffer;
    }
    
    nb_unread = end_write - ptr_read + 1;
    if (fwrite(ptr_read,sizeof(*ptr_read),nb_unread,file) == 0) {
	cursor_read = cursor_save;
	ptr_read = ptr_save;
	return 0;
    };
    nb_total += nb_unread;
    if (end_write == &(cursor_read->buffer[size-1])) {
	if (cursor_read->next)
	    cursor_read = (GCharBuf*)((char*)cursor_read + cursor_read->next);
	else
	    cursor_read = NULL;
	if (cursor_read != NULL)
	    ptr_read = cursor_read->buffer;
    } else {
	ptr_read = end_write + 1;
    }
    
    return nb_total;
}	


/**
FUNCTION
   unsigned long GBuffer::readFile(FILE*)
DESCRIPTION
   This function reads a file and writes its content in a list.
**/
unsigned long GBuffer::readFile(FILE *file)
{
    unsigned long total_read = 0;
    int nb_read;
    char cont = '1';
    char *buffer;

    buffer = new char[1024];
    while (cont == '1') {
	nb_read = fread(buffer, sizeof(*buffer), 1024, file);
	if (nb_read) {
	    write(buffer, nb_read);
	    total_read += nb_read;
	} else
	    cont = '0';
    }
    delete buffer;
    return total_read;
}


/**
FUNCTION
   int GBuffer::resetWrite(unsigned long)
DESCRIPTION
   This function moves pointers for writing at the offset specified. The datas which are "after" the offset are lost. If the pointers read are "after" the offset, they are moved to the offset.
**/
int GBuffer::resetWrite(unsigned long offset)
{
    int ret;
    
    if (!cursor_write)
	return 0;
    
    if (offset == 1) {
	if (head != NULL) {
	    cursor_write = head;
	    cursor_end = head;
	    ptr_write = head->buffer;
	    end_write = head->buffer - 1;
	    total_written = 0;
	    seekRead(1);
	}
 	ret = 1;
    } else {
	if (offset > total_written + 1 || offset == 0) {
	    ret = 0;
	} else {
	    /*
	     * Move "write" pointers;
	     */
	    cursor_write = head;
	    ulong j = offset / size;
	    if (offset % size == 0)
		j--;
	    for (uint i = 0; i< j; i++) {
		if (cursor_write->next)
		    cursor_write = (GCharBuf*)((char*)cursor_write +
					      cursor_write->next);
		else
		    cursor_write = NULL;
	    }
	    ptr_write = cursor_write->buffer + offset -
		cursor_write->first_offset;
	    /*
	     * Moves "end" pointers;
	     */
	    cursor_end = head;
	    j = (offset - 1) / size;
	    if ((offset - 1) % size == 0)
		j--;
	    for (i = 0; i< j; i++) {
		if (cursor_end->next)
		    cursor_end = (GCharBuf*)((char*)cursor_end +
					    cursor_end->next);
		else
		    cursor_end = NULL;
	    }
	    end_write = cursor_end->buffer + offset - 1 -
		cursor_end->first_offset;
	    if (ptr_read != NULL) {
		int offset_read = (int)(ptr_read - cursor_read->buffer + 
					cursor_read->first_offset);
		if ((ulong)offset_read > offset) {
		    cursor_read = cursor_write;
		    ptr_read = ptr_write;
		}
	    }
	    total_written =  end_write - cursor_end->buffer +
		cursor_end->first_offset;
	    ret = 1;
	}	
    }
    return ret;
}


/**
FUNCTION
   int GBuffer::seekWrite(unsigned long)
DESCRIPTION
   This function moves pointers for writing
**/
int GBuffer::seekWrite(unsigned long offset)
{
    int ret;

    if (!cursor_write)
	return 0;
    
    if (offset > total_written + 1 || offset <= 0) {
	ret = 0;
    } else {
	cursor_write = head;
	ulong j = offset / size;
	if (offset % size == 0)
	    j--;
	for (uint i = 0; i< j; i++) {
	    if (cursor_write->next)
		cursor_write = (GCharBuf*)((char*)cursor_write +
					  cursor_write->next);
	    else
		cursor_write = NULL;
	}
	ptr_write = cursor_write->buffer + offset - cursor_write->first_offset;
	ret = 1;
    }	
    return ret;
}


/** 
FUNCTION
   void GBuffer::appendWrite(void)
DESCRIPTION
   This function moves pointers for writing at the end of linked list.
**/
void GBuffer::appendWrite(void)
{
    if (!cursor_write)
	return;
    
    if (end_write == &(cursor_end->buffer[size-1])) {
	if (cursor_end->next)
	    cursor_write = (GCharBuf*)((char*)cursor_write + cursor_end->next);
	else
	    cursor_write = NULL;
	if (cursor_write != NULL) 
	    ptr_write = cursor_write->buffer;
	else
	    ptr_write = NULL;
    } else {
	cursor_write = cursor_end;
	ptr_write = end_write + 1;
    }
}


/**
FUNCTION
   int GBuffer::seekRead(unsigned long)
DESCRIPTION
   This function moves pointers for reading
**/
int GBuffer::seekRead(unsigned long offset)
{
    int ret;

    if (!cursor_read)
	return 0;
    
    if (offset > total_written + 1 || offset <= 0) {
	ret = 0;
    } else {
	cursor_read = head;
	ulong j = offset / size;
	if (offset % size == 0)
	    j--;
	for (uint i = 0; i< j; i++) {
	    if (cursor_read->next)
		cursor_read = (GCharBuf*)((char*)cursor_read +
					 cursor_read->next);
	    else
		cursor_read = NULL;
	}
	ptr_read = cursor_read->buffer + offset - cursor_read->first_offset;
	ret = 1;
    }	
	return ret;
}


/** 
FUNCTION
   void GBuffer::appendRead(void)
DESCRIPTION
   This function moves pointers for reading at the end of linked list.
**/
void GBuffer::appendRead(void)
{
    if (!cursor_read)
	return;
   
    if (end_write == &(cursor_end->buffer[size-1])) {
	if (cursor_end->next)
	    cursor_read = (GCharBuf*)((char*)cursor_read + cursor_end->next);
	else
	    cursor_read = NULL;
	if (cursor_write != NULL) 
	    ptr_read = cursor_read->buffer;
	else
	    ptr_read = NULL;
    } else {
	cursor_read = cursor_end;
	ptr_read = end_write + 1;
    }
}





