#ifdef REV_INFO
static char rcsid[] = "$Id: Callback.cc,v 1.1.1.1 1992/03/18 10:40:16 uep Exp $";
#endif
/*****************************************************************************
MODULE:  Tools
FILE:    Callback.cc
AUTHOR:  Oliver Jojic, Philippe Smadja
DATE:    28 August 1990

DESCRIPTION:
  See description of "StopPostcondLoopOn" and "GCallCallbacks".
  
COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include <Go/Tools/Callback.h>
#include <Go/Tools/Assoc.h>
#include <Go/Tools/Alloc.h>


/**
FUNCTION
    bool GCallCallbacks()
DESCRIPTION
    Call the callbacks of the callbacks list.
    The callback list is copied before the loop. It allows the callback to
    destroy a postcond.
    We have a function for 1, 2, 3, 4, 5 void* parameter.
**/
void GCallCallbacks(GCallbackRec* cbrec, int count, void* p1)
{
    GCallbackRec* _cbrec = cbrec;
#ifdef NO_ALLOCA
    GCallbackRec* allocated_cbrec = NULL;
#endif

    
    if (count > 1)
    {
	int size = sizeof(GCallbackRec)*count;
#ifdef NO_ALLOCA
	allocated_cbrec = _cbrec = (GCallbackRec*) GMALLOC(size);
#else
	_cbrec = (GCallbackRec*)alloca(size);
#endif
	memcpy(_cbrec, cbrec, size);

    }

    for (int i = 0; i < count; i++, _cbrec++) 
	((void (*)(void*, void*))_cbrec->proc)(_cbrec->closure, p1);

    
#ifdef NO_ALLOCA
    GFREE (allocated_cbrec);
#endif
};

void GCallCallbacks(GCallbackRec* cbrec, int count, void* p1, ulong p2)
{
    GCallbackRec* _cbrec = cbrec;
#ifdef NO_ALLOCA
    GCallbackRec * allocated_cbrec = NULL;
#endif


    
    if (count > 1)
    {
	int size = sizeof(GCallbackRec)*count;
#ifdef NO_ALLOCA
       allocated_cbrec =	_cbrec = (GCallbackRec*)GMALLOC(size);
#else
	_cbrec = (GCallbackRec*)alloca(size);
#endif
	memcpy(_cbrec, cbrec, size);

    }

    for (int i = 0; i < count; i++, _cbrec++) 
	((void (*)(void*, void*, ulong))_cbrec->proc)(_cbrec->closure, p1, p2);

    
#ifdef NO_ALLOCA
    GFREE(allocated_cbrec);
#endif
};

void GCallCallbacks(GCallbackRec* cbrec, int count, void* p1, ulong p2, ulong p3)
{
    GCallbackRec* _cbrec = cbrec;

#ifdef NO_ALLOCA
    GCallbackRec * allocated_cbrec = NULL;
#endif
    
    if (count > 1)
    {
	int size = sizeof(GCallbackRec)*count;
#ifdef NO_ALLOCA
       allocated_cbrec =	_cbrec = (GCallbackRec*)GMALLOC(size);
#else
	_cbrec = (GCallbackRec*)alloca(size);
#endif
	memcpy(_cbrec, cbrec, size);

    }

    for (int i = 0; i < count; i++, _cbrec++) 
	((void (*)(void*, void*, ulong, ulong))_cbrec->proc)(_cbrec->closure, p1,
							     p2, p3);
    
#ifdef NO_ALLOCA
    GFREE(allocated_cbrec);
#endif
};

void GCallCallbacks(GCallbackRec* cbrec, int count, void* p1, ulong p2, ulong p3, ulong p4)
{
    GCallbackRec* _cbrec = cbrec;
#ifdef NO_ALLOCA
    GCallbackRec * allocated_cbrec = NULL;
#endif
    
    if (count > 1)
    {
	int size = sizeof(GCallbackRec)*count;
#ifdef NO_ALLOCA
	allocated_cbrec = _cbrec = (GCallbackRec*)GMALLOC(size);
#else
	_cbrec = (GCallbackRec*)alloca(size);
#endif
	memcpy(_cbrec, cbrec, size);

    }

    for (int i = 0; i < count; i++, _cbrec++) 
	((void (*)(void*, void*, ulong, ulong, ulong))_cbrec->proc)(_cbrec->closure, p1,
								    p2, p3, p4);
    
#ifdef NO_ALLOCA
    GFREE(allocated_cbrec);
#endif
};

void GCallCallbacks(GCallbackRec* cbrec, int count, void* p1, ulong p2,
		   ulong p3, ulong p4, ulong p5)
{
    GCallbackRec* _cbrec = cbrec;
#ifdef NO_ALLOCA
    GCallbackRec * allocated_cbrec = NULL;
#endif

    
    if (count > 1)
    {
	int size = sizeof(GCallbackRec)*count;
#ifdef NO_ALLOCA
	allocated_cbrec = _cbrec = (GCallbackRec*)GMALLOC(size);
#else
	_cbrec = (GCallbackRec*)alloca(size);
#endif
	memcpy(_cbrec, cbrec, size);
    }

    for (int i = 0; i < count; i++, _cbrec++) 
	((void (*)(void*, void*, ulong, ulong, ulong, ulong))_cbrec->proc)
	    (_cbrec->closure, p1, p2, p3, p4, p5);
    
#ifdef NO_ALLOCA
    GFREE(allocated_cbrec);
#endif
};


void GCallCallbacks(GCallbackRec* cbrec, int count, void* p1, ulong p2,
		   ulong p3, ulong p4, ulong p5, ulong p6)
{
    GCallbackRec* _cbrec = cbrec;
#ifdef NO_ALLOCA
    GCallbackRec * allocated_cbrec = NULL;
#endif

    
    if (count > 1)
    {
	int size = sizeof(GCallbackRec)*count;
#ifdef NO_ALLOCA
	allocated_cbrec = _cbrec = (GCallbackRec*)GMALLOC(size);
#else
	_cbrec = (GCallbackRec*)alloca(size);
#endif
	memcpy(_cbrec, cbrec, size);
    }

    for (int i = 0; i < count; i++, _cbrec++) 
	((void (*)(void*, void*,
		   ulong, ulong, ulong, ulong, ulong))_cbrec->proc)
	    (_cbrec->closure, p1, p2, p3, p4, p5, p6);
    
#ifdef NO_ALLOCA
    GFREE(allocated_cbrec);
#endif
};



