#ifdef REV_INFO
static char rcsid[] = "$Id: ShColor.cc,v 1.3 1992/09/25 14:04:01 smadja Exp $";
#endif
/*****************************************************************************
MODULE:  Tools
FILE:    ShColor.cc
AUTHOR:  Jacques Davy
DATE:    August 1990

DESCRIPTION:
    See <Go/ShColor.h>

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  license from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#include <Go/Tools/ShColor.h>
#include <Go/Tools/ShPattern.h>
#include <Go/GoStream.h>
#include <Go/Tools/Alloc.h>
#include <Go/Tools/Error.h>

/*
 * static of the class
 */
GList*        GShColor::_headOfList;
GFunctionList GShColor::_funcList;

/*
 * Protected Methods
 */


int GShColor::save(const GShColor* sh, GoStream* gostream)
{
    const GColorValues* val = sh->values();
    unsigned long mask = sh->mask();
    GBool named = sh->isNamed();
    
    if (!gostream->writeBool(named, "is named"))
	goto failure;
    if (named && !gostream->writeString(sh->name(), "name"))
	goto failure;
    if (!gostream->writeLong(sh->mask(), "mask"))
	goto failure;
    if ((mask & GCOLOR_RGB) &&
	! gostream->writeRgb(((GColorValues*) val)->rgb, "rgb"))
	goto failure;
    if ((mask & GCOLOR_GRAY) && ! gostream->writeChar(val->gray, "gray"))
	goto failure;
    if ((mask & GCOLOR_HALFTONE) &&
	! gostream->saveObj(val->halftone,
			    val->halftone->getClassname(), "halftone"))
	goto failure;
    return GSharedObject::save(sh, gostream);
    
  failure:
    GWarning("GShColor save failure");
    return 0;
}

/**/
int GShColor::restore(GShColor** sh, GoStream* gostream)
{
    long mask;
    GColorValues val;
    char str[256];
    char *name = str;
    const char* class_name;
    GBool named;
    
    if (!gostream->readBool(&named))
	goto failure;
    if (named && !gostream->readString(256, &name))
	goto failure;
    if (!gostream->readLong(&mask))
	goto failure;

    if ((mask & GCOLOR_RGB) && ! gostream->readRgb(&val.rgb))
	goto failure;
    if ((mask & GCOLOR_GRAY) && ! gostream->readChar((char*) &val.gray))
	goto failure;
    if ((mask & GCOLOR_HALFTONE) &&
	! gostream->restoreObj((void**) &(val.halftone), &class_name))
	goto failure;
    
    if (*sh = GCreateShColor(mask, &val, named ? name : NULL))
    {
	if (GSharedObject::readParams(*sh, gostream))
	    return 1;
	else
	{
	    (*sh)->destroy();
	    *sh = NULL;
	}
    }
    
  failure:
    GWarning("restoreGShColor failure");
    return 0;
}

/**/
int GShColor::readParams(GShColor*, GoStream*)
{
    return 0;
}

/**/
GShColor* _newGShColor (GFunctionList* funcList, ulong mask,
		      const void * values, const char* name)
{
    return new GShColor (funcList, mask, (const GColorValues*)values, name);
}

/**/
GShColor::GShColor (GFunctionList* funcList, ulong mask,
		  const GColorValues* values, const char* name)
    : GSharedObject(funcList, mask, name)
{
    if ( mask & GCOLOR_RGB )
        _values.rgb = values->rgb;
    if ( mask & GCOLOR_GRAY )
        _values.gray = values->gray;
    if ( mask & GCOLOR_HALFTONE )
        _values.halftone = values->halftone->dup();
}

    
/**/
GShColor::~GShColor ()
{
    if ( _mask & GCOLOR_HALFTONE )
        _values.halftone->destroy();
}

/**/
GBool GShColor::_match (ulong mask, const void * __values) const
{
const GColorValues* values =(const GColorValues* ) __values;

    ulong inter_mask = mask & _mask;

    // for instance if  RGB is set GRAY is a modifier;
    // else it is a definition;

    if (!inter_mask)     // The most current behavior for SharedObject subclasses
	return GFALSE;    // is here GTRUE;


    if (inter_mask & GCOLOR_RGB)
        if ((_values.rgb.red != values->rgb.red) ||
	    (_values.rgb.green != values->rgb.green) ||
	    (_values.rgb.blue != values->rgb.blue) )
	     return GFALSE;

    if (inter_mask & GCOLOR_GRAY)
	if (_values.gray != values->gray)
            return GFALSE;

    if (inter_mask & GCOLOR_HALFTONE)
        if (_values.halftone != values->halftone)
            return GFALSE;

    return GTRUE;
}

/**/
void GShColor::_makeNewValues (ulong mask, const void * __values,
			    ulong& newMask, void * __newValues)	const
{
const GColorValues* values =(const GColorValues* ) __values;
GColorValues* newValues = (GColorValues*) __newValues;

    newMask = mask | _mask;

/* Values must not be duplicated because SharedObj class does not free them */

    newValues->rgb = (mask & GCOLOR_RGB ? values->rgb : _values.rgb);
    newValues->gray = (mask & GCOLOR_GRAY ? values->gray : _values.gray);
    newValues->halftone = (mask & GCOLOR_HALFTONE ?
			   values->halftone : _values.halftone);
}

/**/
ulong GShColor::_update (ulong mask, const void * __values)
{
const GColorValues* values =(const GColorValues* ) __values;
    ulong update_mask = 0;

#define updateValue(maskBit, field)\
    if ((mask & maskBit) && \
	  (!(_mask & maskBit) || (_values.field != values->field))) \
        {\
            _values.field = values->field;\
	    update_mask |= maskBit;\
	}

    if ((mask & GCOLOR_RGB) &&
	  (!(_mask & GCOLOR_RGB) || (_values.rgb.red != values->rgb.red)
	                        || (_values.rgb.green != values->rgb.green)
	                        || (_values.rgb.blue != values->rgb.blue)))
        {
            _values.rgb = values->rgb;
	    update_mask |= GCOLOR_RGB;
	}

    updateValue(GCOLOR_GRAY, gray);

    if ((mask & GCOLOR_HALFTONE) &&
	  (!(_mask & GCOLOR_HALFTONE) || (_values.halftone != values->halftone)))
    {
        if (_mask & GCOLOR_HALFTONE)
            _values.halftone->destroy();
        _values.halftone = values->halftone->dup();
        update_mask |= GCOLOR_HALFTONE;
    }

    _mask |= mask;
    return update_mask;
}

/*
 * Public Methods
 */

/* static */
const char* GShColor::classname()
{
    return "GShColor";
}

/* virtual */
const char* GShColor::getClassname() const
{
    return GShColor::classname();
}

/**/
void GModify (const char* name, ulong mask, const GColorValues* values)
{
    GMODIFY_NAMED_SHAREDOBJECT (GShColor, mask, values, name);
}


static GShColor* default_color = NULL;

/**/
GShColor* GCreateShColor (ulong mask, const GColorValues* values, const char* name)
{
    if (mask == 0 && name == NULL)
    {
	// take the default one;
	if (default_color == NULL)
	{
	    // make a black;
	    GColorValues v;
	    v.rgb.red = v.rgb.green = v.rgb.blue = 0;
	    default_color = GCREATE_SHAREDOBJECT(GShColor,
						 GCOLOR_RGB, &v, NULL);
	}
	return  default_color->dup();
    }
    return  GCREATE_SHAREDOBJECT(GShColor, mask, values, name);
}

/**/
GShColor* GCreateShColor ()
{
    return GCreateShColor(0, NULL, NULL);
}

/**/
GShColor* GCreateShColor (const char* name)
{
    return GCreateShColor (0, NULL, name);
}

/**/
GShColor* GCreateShColor (const GRgb* rgb, const char* name)
{
    GColorValues values;
    values.rgb.red = rgb->red;
    values.rgb.green = rgb->green;
    values.rgb.blue = rgb->blue;
    return GCREATE_SHAREDOBJECT (GShColor, GCOLOR_RGB, &values, name);
}

/**/
GShColor* GCreateShColor (uchar gray, const char* name)
{
    GColorValues values;
    values.gray = gray;
    return GCREATE_SHAREDOBJECT (GShColor, GCOLOR_GRAY, &values, name);
}

/**/
GShColor* GCreateShColor (const GShPattern* halftone, const char* name)
{
    GColorValues values;
    values.halftone = (GShPattern*)halftone;
    return GCREATE_SHAREDOBJECT (GShColor, GCOLOR_HALFTONE, &values, name);
}



/**
FUNCTION
    makePatternForRgb.
DESCRIPTION
    this private function is used with the ShColor member function
    makePattern().
    It makes a pattern w=8, h=8, depth = 4; colormap is the 8 corner + the gray center.
    of the rgb cube.
    Thank for Jean-Michel Grugeau (Bull Massy France) for the algorithmic
**/

#define CoinBlack   0           /* Cube Corners */
#define CoinRed     1           
#define CoinGreen   2
#define CoinYellow  3
#define CoinBlue    4
#define CoinMagenta 5
#define CoinCyan    6
#define CoinWhite   7
#define CentreGris  8

#define DATASIZE 64           /* datasize a created pixelarray */

static uchar lgray[32][8] =
{
{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01},
{0x00, 0x00, 0x00, 0x10, 0x00, 0x00, 0x00, 0x01},
{0x00, 0x00, 0x00, 0x10, 0x00, 0x00, 0x00, 0x11},
{0x00, 0x00, 0x00, 0x11, 0x00, 0x00, 0x00, 0x11},
{0x00, 0x40, 0x00, 0x11, 0x00, 0x00, 0x00, 0x11},
{0x00, 0x40, 0x00, 0x11, 0x00, 0x04, 0x00, 0x11},
{0x00, 0x44, 0x00, 0x11, 0x00, 0x04, 0x00, 0x11},
{0x00, 0x44, 0x00, 0x11, 0x00, 0x44, 0x00, 0x11},
{0x00, 0x44, 0x00, 0x11, 0x00, 0x44, 0x00, 0x51},
{0x00, 0x44, 0x00, 0x15, 0x00, 0x44, 0x00, 0x51},
{0x00, 0x44, 0x00, 0x15, 0x00, 0x44, 0x00, 0x55},
{0x00, 0x44, 0x00, 0x55, 0x00, 0x44, 0x00, 0x55},
{0x00, 0x45, 0x00, 0x55, 0x00, 0x44, 0x00, 0x55},
{0x00, 0x45, 0x00, 0x55, 0x00, 0x54, 0x00, 0x55},
{0x00, 0x55, 0x00, 0x55, 0x00, 0x54, 0x00, 0x55},
{0x00, 0x55, 0x00, 0x55, 0x00, 0x55, 0x00, 0x55},
{0x80, 0x55, 0x00, 0x55, 0x00, 0x55, 0x00, 0x55},
{0x80, 0x55, 0x00, 0x55, 0x08, 0x55, 0x00, 0x55},
{0x88, 0x55, 0x00, 0x55, 0x08, 0x55, 0x00, 0x55},
{0x88, 0x55, 0x00, 0x55, 0x88, 0x55, 0x00, 0x55},
{0x88, 0x55, 0x20, 0x55, 0x88, 0x55, 0x00, 0x55},
{0x88, 0x55, 0x20, 0x55, 0x88, 0x55, 0x02, 0x55},
{0x88, 0x55, 0x22, 0x55, 0x88, 0x55, 0x02, 0x55},
{0x88, 0x55, 0x22, 0x55, 0x88, 0x55, 0x22, 0x55},
{0xA8, 0x55, 0x22, 0x55, 0x88, 0x55, 0x22, 0x55},
{0xA8, 0x55, 0x22, 0x55, 0x8A, 0x55, 0x22, 0x55},
{0xAA, 0x55, 0x22, 0x55, 0x8A, 0x55, 0x22, 0x55},
{0xAA, 0x55, 0x22, 0x55, 0xAA, 0x55, 0x22, 0x55},
{0xAA, 0x55, 0xA2, 0x55, 0xAA, 0x55, 0x22, 0x55},
{0xAA, 0x55, 0xA2, 0x55, 0xAA, 0x55, 0x2A, 0x55},
{0xAA, 0x55, 0xAA, 0x55, 0xAA, 0x55, 0x2A, 0x55},
{0xAA, 0x55, 0xAA, 0x55, 0xAA, 0x55, 0xAA, 0x55},
};

static GShPattern*  makePatternForRgb(GRgbValue red, GRgbValue green, GRgbValue blue)
{
    short    poidcouleur[CentreGris+1]; /* poid des couleur de coin du cube dans la */
    uchar    ordrepoid[6];
    short    orix, oriy, oriz, larete, i,j;
    short    coulutile, coin, p, k, sk;

    /**
      orix, oriy, oriz give the origin point of a cube of size: larete.
      x, y, z : are coordinates of a point in the cube
      the first loop give the sub cube where is the (x,y,z).
      sub cuble are from 0 to 7.
      **/
    
    for (i=0; i <= CentreGris; i++)
	poidcouleur[i] = 0;

    orix = oriy = oriz = 0;
    larete = 256;
    red >>= 8;		      /* on ramene red, green, blue entre 0 et 255 */
    green >>= 8;
    blue >>= 8;
    for (i=0, coulutile=0; i<7; i++){
	coin = 0;
	larete /= 2;
	if ((red - orix) >= larete) { coin += 1; orix += larete;}
	if ((green - oriy) >= larete) { coin += 2; oriy += larete;}
	if ((blue - oriz) >= larete) { coin += 4; oriz += larete;}
	if ((i < 6) || (coulutile == 1)) {
	    if (!poidcouleur[coin]) {
		ordrepoid[coulutile]= coin;
		coulutile++;
	    }
	    if (i == 6) poidcouleur [coin] += 1;
	    else poidcouleur [coin] += (1 << (5-i));
	}
    }

#ifdef _USE_GRAY_CENTER_
    /**
      Use the gray to magnify the result
      **/
    short    wp[CentreGris+1];    /* representation de couleur a simulee */

    for (i=0; i <= CentreGris; i++)
	wp[i] = poidcouleur[i];
    for (i=0; i <4; i++)
    {
	k = (poidcouleur[i] > poidcouleur[7-i])?poidcouleur[7-i]:poidcouleur[i];
	if (k)
	{
	    poidcouleur[i] -= k;
	    poidcouleur[7-i] -= k;
	    poidcouleur[CentreGris] += (k+k);
	}
    }
    for (i=p=0; i <= CentreGris; i++)
	if (poidcouleur[i] > p) p =poidcouleur[i];

    if (p < 32)
    {
	/**
	  the result will have a less quality: so stop the gray replacement
	  **/
	for (i=0; i <= CentreGris; i++) poidcouleur[i] = wp[i];
    }
    
    if (poidcouleur[CentreGris])
    {
	for (i=0, coulutile = 0; i<=CentreGris; i++)
	    if (poidcouleur[i]) coulutile += 1;

	for (i=0; i<coulutile; i++)
	{
	    for (k = j = 0; j<= CentreGris; j++)
		if ((poidcouleur[j] > k) && (poidcouleur[j] < 100))
		{
		    k = poidcouleur[j]; p = j;
		}
	    ordrepoid[i] = p;
	    poidcouleur[p] += 100;
	}
	for (j=0; j<=CentreGris; j++)
	    if (poidcouleur[j] >= 100) poidcouleur[j] -= 100;
    }
    
#endif
    
    /**
      PATTERN GENERATION
      **/
    uchar* data = (uchar*)GMALLOC(DATASIZE);

    if (coulutile == 1) 
	memset(data, ordrepoid[0], DATASIZE);

    else    /* coulutil == 1 */
    {
	j = 64 - poidcouleur[ordrepoid[0]];

	struct _Point
	{
	    ushort x;
	    ushort y;
	};
	_Point* points;

	uchar* d = lgray[j-1];
	uchar* _data = data;
	for (i=0; i < 8; i++, d++) 
	    for (k=0; k < 8; k++)
	    {
		if (*d & (1<<k))    // LSBFIRST lgray representation:
		    *_data++ =  ordrepoid[1];
		else
		    *_data++ = ordrepoid[0];
	    }
	
	if (coulutile > 2)
	{
	    for (i=2, j=0; i<coulutile; i++)
		j += poidcouleur[ordrepoid[i]];
	    points = (_Point*)GMALLOC (sizeof (_Point) * j);
	    for (coin=2; coin < coulutile; coin++)
	    {
		for (i=coin, j=0; i<coulutile; i++)
		    j += poidcouleur[ordrepoid[i]];
		uchar pixel = ordrepoid[coin];
		p = 0;
		for (k=0; k < 8; k++)
		{
		    uchar z = lgray[j-1][k];
		    if (z) for (sk=0; sk<8; sk++)
		    {
			if ((z & 0x01) && (p < j))
			{
			    points[p].x = sk;
			    points[p].y = k;
			    p++;
			}
			z >>= 1;
		    }
		}
		while (--p >= 0)
		    *(data + points[p].y * 8 + points[p].x) = pixel;
	    }
	    GFREE (points);
	}
    }
    /**
      make the pattern
      **/
    GShColor* _colors[9];
    GColormapValues cmapv;
    cmapv.n_color = 9;
    cmapv.colors = _colors;
    GColorValues cv;

    cv.rgb.red = 0, cv.rgb.green = 0, cv.rgb.blue = 0;
    _colors[CoinBlack] = GCreateShColor(GCOLOR_RGB, &cv);

    cv.rgb.red = GMAX_RGB_VALUE;
    _colors[CoinRed] = GCreateShColor(GCOLOR_RGB, &cv);

    cv.rgb.red = 0, cv.rgb.green = GMAX_RGB_VALUE;
    _colors[CoinGreen] = GCreateShColor(GCOLOR_RGB, &cv);

    cv.rgb.red = GMAX_RGB_VALUE;
    _colors[CoinYellow] = GCreateShColor(GCOLOR_RGB, &cv);

    cv.rgb.red = 0, cv.rgb.green = 0, cv.rgb.blue = GMAX_RGB_VALUE;
    _colors[CoinBlue] = GCreateShColor(GCOLOR_RGB, &cv);

    cv.rgb.red = GMAX_RGB_VALUE;
    _colors[CoinMagenta] = GCreateShColor(GCOLOR_RGB, &cv);

    cv.rgb.red = 0; cv.rgb.green = GMAX_RGB_VALUE; cv.rgb.blue = GMAX_RGB_VALUE;
    _colors[CoinCyan] = GCreateShColor(GCOLOR_RGB, &cv);

    cv.rgb.red = GMAX_RGB_VALUE;
    _colors[CoinWhite] = GCreateShColor(GCOLOR_RGB, &cv);

    cv.rgb.red = cv.rgb.green = cv.rgb.blue = GMAX_RGB_VALUE/2;
    _colors[CentreGris] = GCreateShColor(GCOLOR_RGB, &cv);

    GPatternValues ptrnv;
    ptrnv.colormap = GCreateShColormap(GCOLORMAP_COLORS, &cmapv);
    for (i = 0 ; i < 9; i++)
	_colors[i]->destroy();

    GPixelArrayValues pav;
    pav.width = 8;
    pav.height = 8;
    pav.xoffset = 0;
    pav.bitmap_unit = 8;
    pav.padding = 8;
    pav.bits_per_pixel = 8;
    pav.depth = 4;
    pav.format.bit = GLSB_FIRST;
    pav.format.byte = GLSB_FIRST;
    pav.format.coding = GBITMAP_CODING;
    pav.data = data;
    ptrnv.pixelArray = GCreateShPixelArray(GPXL_VALUE, &pav, NULL, GDATA_GIFT);
    ptrnv.width = ptrnv.height = 0; /** pixel size **/

    GShPattern* pattern =
	GCreateShPattern(GPTRN_PIXEL_ARRAY|GPTRN_COLORMAP|GPTRN_WIDTH|GPTRN_HEIGHT,
			 &ptrnv);
    ptrnv.colormap->destroy();
    ptrnv.pixelArray->destroy();

    return pattern;
}

/**
FUNCTION
    makeHalftone()
DESCRIPTION
    see makePatternForRgb description
    Note that the function does not update the current color with
    the computed pattern.
**/
GShPattern* GShColor::makeHalftone(GBool gray) const
{
    GShPattern* pa = NULL;
    if (_mask & GCOLOR_RGB)
    {
	if (gray)
	{
	    GRgbValue br = ::GBrightness(_values.rgb.red,
					_values.rgb.green,
					_values.rgb.blue);
	    pa  = makePatternForRgb(br,br,br);
	} else
	    pa = makePatternForRgb(_values.rgb.red,
				   _values.rgb.green,
				   _values.rgb.blue);
    }
    else if (_mask & GCOLOR_GRAY)
    {
	GRgbValue br = _values.gray * 256;
	pa =  makePatternForRgb(br, br, br);
    }
    return pa;
}

