#ifdef REV_INFO
static char rcsid[] = "$Id: Colormap.cc,v 1.2 1992/09/25 14:18:43 smadja Exp $";
#endif
/*****************************************************************************
MODULE:  Tools
FILE:    Colormap.cc
AUTHOR:  Jacques Davy
DATE:    November 1990

DESCRIPTION:
    (see <Go/Tools/Colormap.h>

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#include <Go/Tools/Colormap.h>
#include <assert.h>

/*
 * Callback types
 */
static char cmapNmodifyPrecond[1];
static char cmapNmodifyPostcond[1];
static char cmapNdestroyPostcond[1];
static char cmapNaddColorPrecond[1];
static char cmapNaddColorPostcond[1];
static char cmapNremColorPrecond[1];
static char cmapNremColorPostcond[1];
static char cmapNrepColorPrecond[1];
static char cmapNrepColorPostcond[1];

typedef void (*PsProc)(void*, GColormap*, ulong);
typedef void (*PdProc)(void*, const GColormap*);
typedef GBool (*PeProc)(void*, const GColormap*, ulong index_or_shcolor);
typedef GBool (*ModifPeProc)(void*, const GColormap*, ulong&, GColormapValues*);
typedef GBool (*AddColorPeProc)(void*, const GColormap*, const GShColor* shcolor);
typedef GBool (*RepColorPeProc)(void*, const GColormap*, ulong index, const GShColor* shcolor);

struct ModifPrecondRec
{
    ModifPeProc proc;
    void* closure;
};

struct AddColorPrecondRec
{
    AddColorPeProc proc;
    void* closure;
};

struct RepColorPrecondRec
{
    RepColorPeProc proc;
    void* closure;
};

struct PrecondRec
{
    PeProc proc;
    void* closure;
};

struct PostcondRec
{
    PsProc proc;
    void* closure;
};

struct DestroyPostcondRec
{
    PdProc proc;
    void* closure;
};

/*
 * Private methods
 * ---------------
 */

/**
FUNCTION
    _call_call_*_postcond
DESCRIPTION
    These functions are called when the ShColormap is updated
**/

/**/
static void _call_call_modify_postcond (void* This, GShColormap* sh_colormap,
					ulong update_mask)
{
    Call_call_modify_postcond ((GColormap*)This, sh_colormap, update_mask);
}

void Call_call_modify_postcond (GColormap* This, const GShColormap* sh_colormap,
				ulong update_mask)
{
    assert (sh_colormap == This->shColormap());
    This->_call_modify_postcond (update_mask);
}

/**/
static void _call_call_addColor_postcond (void* This, GShColormap* sh_colormap,
					  ulong index)
{
    Call_call_addColor_postcond ((GColormap*)This, sh_colormap, index);
}

void Call_call_addColor_postcond (GColormap* This, const GShColormap* ,ulong index)
{
    This->_call_addColor_postcond (index);
}

/**/
static void _call_call_remColor_postcond (void* This, GShColormap* sh_colormap,
					  ulong index)
{
    Call_call_remColor_postcond ((GColormap*)This, sh_colormap, index);
}

void Call_call_remColor_postcond (GColormap* This, const GShColormap*, ulong index)
{
    This->_call_remColor_postcond (index);
}

/**/
static void _call_call_repColor_postcond (void* This, GShColormap* sh_colormap,
					  ulong index)
{
    Call_call_repColor_postcond ((GColormap*)This, sh_colormap, index);
}

void Call_call_repColor_postcond (GColormap* This, const GShColormap*, ulong index)
{
    This->_call_repColor_postcond (index);
}


/*
 * Protected methods
 * -----------------
 */
/**/
int GColormap::_addCallback (const char* type, GCBProc proc, const void* closure) const
{
    const GAssoc* _cb = &_assoc;
    GAssoc* cb = (GAssoc*)_cb;
    
    GCallbackRec cbrec (proc, closure);
    return cb->add(type, sizeof cbrec, (char*)&cbrec);
}

/**/
int GColormap::_remCallback (const char* type, GCBProc proc, const void* closure) const
{
    const GAssoc* _cb = &_assoc;
    GAssoc* cb = (GAssoc*)_cb;

    GCallbackRec cbrec (proc, closure);
    return cb->rem(type, sizeof cbrec, (char*)&cbrec);
}

/**/
GBool GColormap::_call_modify_precond (ulong& modify_mask, GColormapValues* new_values)
{
    if (_flags._has_modify_precond)
    {
	int count;
	ModifPrecondRec* cbrec = (ModifPrecondRec*)_assoc.get (cmapNmodifyPrecond, count);
	for (int i = 0; i < count; i++, cbrec++)
	    if (!cbrec->proc(cbrec->closure, this, modify_mask, new_values))
                return GFALSE;
    }
    return GTRUE;
}

/**/
void GColormap::_call_modify_postcond (ulong modify_mask)
{
    if (_flags._has_modify_postcond)
    {
	int count;
	GCallbackRec* cbrec = (GCallbackRec*)_assoc.get (cmapNmodifyPostcond, count);
	(void)GCallCallbacks(cbrec, count, this, modify_mask);
    }
}

/**/
void GColormap::_call_destroy_postcond ()
{
    if (_flags._has_destroy_postcond)
    {
	int count;
	DestroyPostcondRec* cbrec = (DestroyPostcondRec*)_assoc.get (cmapNdestroyPostcond, count);
	for (int i = 0; i < count; i++, cbrec++)
	     cbrec->proc(cbrec->closure, this);
    }
}

/**/
GBool GColormap::_call_addColor_precond (const GShColor* color)
{
    if (_flags._has_addColor_precond)
    {
	int count;
	AddColorPrecondRec* cbrec = (AddColorPrecondRec*)_assoc.get (cmapNaddColorPrecond, count);
	for (int i = 0; i < count; i++, cbrec++)
	     if (!cbrec->proc(cbrec->closure, this, color))
                return GFALSE;
    }
    return GTRUE;
}

/**/
void GColormap::_call_addColor_postcond (ulong index)
{
    if (_flags._has_addColor_postcond)
    {
	int count;
	GCallbackRec* cbrec = (GCallbackRec*)_assoc.get (cmapNaddColorPostcond, count);
	(void)GCallCallbacks(cbrec, count, this, index);
    }
}

/**/
GBool GColormap::_call_remColor_precond (ulong index)
{
    if (_flags._has_remColor_precond)
    {
	int count;
	PrecondRec* cbrec = (PrecondRec*)_assoc.get (cmapNremColorPrecond, count);
	for (int i = 0; i < count; i++, cbrec++)
	     if (!cbrec->proc(cbrec->closure, this, index))
                return GFALSE;
    }
    return GTRUE;
}

/**/
void GColormap::_call_remColor_postcond (ulong index)
{
    if (_flags._has_remColor_postcond)
    {
	int count;
	GCallbackRec* cbrec = (GCallbackRec*)_assoc.get (cmapNremColorPostcond, count);
	(void)GCallCallbacks(cbrec, count, this, index);
    }
}

/**/
GBool GColormap::_call_repColor_precond (ulong index, const GShColor* new_color)
{
    if (_flags._has_replaceColor_precond)
    {
	int count;
	RepColorPrecondRec* cbrec = (RepColorPrecondRec*)_assoc.get (cmapNrepColorPrecond, count);
	for (int i = 0; i < count; i++, cbrec++)
	    if (!cbrec->proc(cbrec->closure, this, index, new_color))
                return GFALSE;
    }
    return GTRUE;
}

/**/
void GColormap::_call_repColor_postcond (ulong index)
{
    if (_flags._has_replaceColor_postcond)
    {
	int count;
	GCallbackRec* cbrec = (GCallbackRec*)_assoc.get (cmapNrepColorPostcond, count);
	(void)GCallCallbacks(cbrec, count, this, index);
    }
}

static GShColormap* default_shcolormap = NULL;
/*
 * Public methods
 * --------------
 */
GColormap::GColormap ()
{
    if (default_shcolormap == NULL)
	default_shcolormap = GCreateShColormap(0, NULL);
    assert (default_shcolormap->refcnt() >=1 );
    _sh_colormap = default_shcolormap->dup();
}

GColormap::GColormap(const GColormap* colormap)
{
    _sh_colormap = colormap->_sh_colormap->dup();
}

GColormap::GColormap (const GShColormap* sh_colormap)
{
    _sh_colormap = sh_colormap->dup();
}

GColormap::GColormap (ulong mask, const GColormapValues* values, const char* name)
{
    _sh_colormap = GCreateShColormap (mask, values, name);
}

GColormap::GColormap (const char* name)
{
    _sh_colormap = GCreateShColormap (name);
}

GColormap::~GColormap ()
{
    _call_destroy_postcond ();

    if (_flags._has_modify_postcond)
	_sh_colormap->updatePostcond (_call_call_modify_postcond,
				      this, GREMOVE);
    if (_flags._has_addColor_postcond)
	_sh_colormap->addColorPostcond (_call_call_addColor_postcond,
					this, GREMOVE);
    if (_flags._has_remColor_postcond)
	_sh_colormap->remColorPostcond (_call_call_remColor_postcond,
					this, GREMOVE);
    if (_flags._has_replaceColor_postcond)
	_sh_colormap->replaceColorPostcond (_call_call_repColor_postcond,
					    this, GREMOVE);

    _sh_colormap->destroy();
}

/**
FUNCTION
    GColormap::modify
DESCRIPTION
    Modify the shared colormap and call pre/post condition.
**/

void GColormap::modify (ulong mask, const GColormapValues* values)
{
    GColormapValues Values = *values;
    if (mask && _call_modify_precond (mask, &Values))
    {
	// Remove post cond;
	if (_flags._has_modify_postcond)
	    _sh_colormap->updatePostcond (_call_call_modify_postcond,
					  this, GREMOVE);
	if (_flags._has_addColor_postcond)
	    _sh_colormap->addColorPostcond (_call_call_addColor_postcond,
					    this, GREMOVE);
	if (_flags._has_remColor_postcond)
	    _sh_colormap->remColorPostcond (_call_call_remColor_postcond,
					    this, GREMOVE);
	if (_flags._has_replaceColor_postcond)
	    _sh_colormap->replaceColorPostcond (_call_call_repColor_postcond,
						this, GREMOVE);

	// Modify;
        GModify(_sh_colormap, mask, values);

	// Call postcond;
	_call_modify_postcond(mask);
	
	// Add postcond;
	if (_flags._has_modify_postcond)
	    _sh_colormap->updatePostcond (_call_call_modify_postcond, this, GADD);
	if (_flags._has_addColor_postcond)
	    _sh_colormap->addColorPostcond (_call_call_addColor_postcond,
					    this, GADD);
	if (_flags._has_remColor_postcond)
	    _sh_colormap->remColorPostcond (_call_call_remColor_postcond,
					    this, GADD);
	if (_flags._has_replaceColor_postcond)
	    _sh_colormap->replaceColorPostcond (_call_call_repColor_postcond,
						this, GADD);
    }
}

void GColormap::modify (const char* name)
{
    if (name && _sh_colormap->name() && (strcmp (name, _sh_colormap->name()) == 0))
        return;

    // Remove postcond;

    if (_flags._has_modify_postcond)
	_sh_colormap->updatePostcond (_call_call_modify_postcond, this, GREMOVE);

    if (_flags._has_addColor_postcond)
	_sh_colormap->addColorPostcond (_call_call_addColor_postcond,
					this, GREMOVE);
    if (_flags._has_remColor_postcond)
	_sh_colormap->remColorPostcond (_call_call_remColor_postcond,
					this, GREMOVE);
    if (_flags._has_replaceColor_postcond)
	_sh_colormap->replaceColorPostcond (_call_call_repColor_postcond,
					    this, GREMOVE);

    // Get mask and values
    GShColormap* save = _sh_colormap->dup();

    GRename (save, name);
    
    GColormapValues Values = *save->values();
    ulong mask = save->mask();

    if (mask && _call_modify_precond (mask, &Values))
    {
         _sh_colormap->destroy();
         _sh_colormap = GCreateShColormap (mask, &Values, name);
         _call_modify_postcond (mask);
    }
    save->destroy();

    // Add postcond;

    if (_flags._has_modify_postcond)
	_sh_colormap->updatePostcond (_call_call_modify_postcond, this, GADD);

    if (_flags._has_addColor_postcond)
	_sh_colormap->addColorPostcond (_call_call_addColor_postcond,
					this, GADD);
    if (_flags._has_remColor_postcond)
	_sh_colormap->remColorPostcond (_call_call_remColor_postcond,
					this, GADD);
    if (_flags._has_replaceColor_postcond)
	_sh_colormap->replaceColorPostcond (_call_call_repColor_postcond,
					    this, GADD);
}

void GColormap::modify(const GShColormap* sh_colormap)
{
    if (_sh_colormap == sh_colormap)
	return;

    // top of list;
    sh_colormap->raise(GShColormap::_headOfList);
    
    if (sh_colormap->isNamed())
	modify(sh_colormap->name());
    else {

	GColormapValues Values = *sh_colormap->values();
	ulong mask = sh_colormap->mask();

	// Remove postcond;

	if (_flags._has_modify_postcond)
	    _sh_colormap->updatePostcond (_call_call_modify_postcond, this, GREMOVE);

	if (_flags._has_addColor_postcond)
	    _sh_colormap->addColorPostcond (_call_call_addColor_postcond,
					    this, GREMOVE);
	if (_flags._has_remColor_postcond)
	    _sh_colormap->remColorPostcond (_call_call_remColor_postcond,
					    this, GREMOVE);
	if (_flags._has_replaceColor_postcond)
	    _sh_colormap->replaceColorPostcond (_call_call_repColor_postcond,
						this, GREMOVE);

	// unname the sh_colormap;
	GRename(_sh_colormap, NULL);
		
	if (mask && _call_modify_precond (mask, &Values))
	{
	    // Modify;
	    GModify (_sh_colormap, mask, &Values);

	    // Call postcond;
	    _call_modify_postcond (mask);
	}
	
	// Add postcond;

	if (_flags._has_modify_postcond)
	    _sh_colormap->updatePostcond (_call_call_modify_postcond, this, GADD);

	if (_flags._has_addColor_postcond)
	    _sh_colormap->addColorPostcond (_call_call_addColor_postcond,
					    this, GADD);
	if (_flags._has_remColor_postcond)
	    _sh_colormap->remColorPostcond (_call_call_remColor_postcond,
					    this, GADD);
	if (_flags._has_replaceColor_postcond)
	    _sh_colormap->replaceColorPostcond (_call_call_repColor_postcond,
						this, GADD);
    }
}

/**
FUNCTION
    GColormap::addColor
DESCRIPTION
    This method adds a color in the colormap and returns the color index in
    the colormap. If the color already exists in the colormap, it simply
    returns the color index in the colormap.
    In any case, the added color may be freed by the application.
**/
long GColormap::addColor (const GShColor* color)
{
    if (!_call_addColor_precond (color))
        return -1;

    return  _sh_colormap->addColor(_sh_colormap, color);

    // The postcond proc are called
    // from the sh_colormap
}

/**
FUNCTION
    GColormap::remColor
DESCRIPTION
    These methods remove a color from the colormap and destroy it
**/
void GColormap::remColor (const GShColor* color)
{
    long index = _sh_colormap->getIndex(color);
    if (index == -1)
        return;
    remColor (index);
}

void GColormap::remColor (ulong index)
{
    if (!_call_remColor_precond (index))
        return;

    _sh_colormap->remColor(_sh_colormap, index);

    // The postcond proc are called
    // from the sh_colormap
}

/**
FUNCTION
    GColormap::replaceColor
DESCRIPTION
    These functions replace an existing color by another. If the old
    color does not exist, the new one is inserted in the colormap
    (see addColor). These functions return the index. Old_color and
    new_color may be freed by the application.
**/
long GColormap::replaceColor (const GShColor* old_color, const GShColor* new_color)
{
    long index = _sh_colormap->getIndex(old_color);
    return (index == -1 ? -1 : replaceColor (index, new_color));
}

long GColormap::replaceColor (ulong index, const GShColor* new_color)
{
    if (!_call_repColor_precond (index, new_color))
        return -1;

    return _sh_colormap->replaceColor (_sh_colormap, index, new_color);

    // The postcond proc are called
    // from the sh_colormap
}

/**
FUNCTION
    GColormap::add/rem pre/postconditions
DESCRIPTION
    Add or remove pre/post condition on modification and on destruction.

    GWarning _call_call_modify_postcond
    is stored directly in the shared object and pre/condition
    may be not called!
**/

#define _add_rem_callback(type, flag) \
    (FLAGS->flag =\
         0 < ((add == GADD)\
	      ? _addCallback(type, (GCBProc)proc, closure)\
	      : _remCallback(type, (GCBProc)proc, closure)))

/**/
void GColormap::modifyPrecond (ModifPeProc proc,
			      const void* closure, GAddOrRemove add) const
{
    const GColormapCondFlags* flags = &_flags;
    GColormapCondFlags* FLAGS = (GColormapCondFlags*)flags;
    
   _add_rem_callback(cmapNmodifyPrecond, _has_modify_precond);
}

/**/
void GColormap::modifyPostcond (PsProc proc,
			       const void* closure, GAddOrRemove add) const
{
    ushort flag = _flags._has_modify_postcond;

    const GColormapCondFlags* flags = &_flags;
    GColormapCondFlags* FLAGS = (GColormapCondFlags*)flags;
    
    _add_rem_callback(cmapNmodifyPostcond, _has_modify_postcond);

    // The modifyPostcond is called from the  sh_colormap;


    if (!flag && _flags._has_modify_postcond)
	_sh_colormap->updatePostcond (_call_call_modify_postcond, this, GADD);
    else
	if (flag && !_flags._has_modify_postcond)
	    _sh_colormap->updatePostcond (_call_call_modify_postcond, this, GREMOVE);

}

/**/
void GColormap::destroyPostcond (PdProc proc,
				const void* closure, GAddOrRemove add) const
{
    const GColormapCondFlags* flags = &_flags;
    GColormapCondFlags* FLAGS = (GColormapCondFlags*)flags;
    
   _add_rem_callback(cmapNdestroyPostcond, _has_destroy_postcond);
}

/**/
void GColormap::addColorPrecond (AddColorPeProc proc,
				const void* closure, GAddOrRemove add) const
{
    const GColormapCondFlags* flags = &_flags;
    GColormapCondFlags* FLAGS = (GColormapCondFlags*)flags;
    
   _add_rem_callback(cmapNaddColorPrecond, _has_addColor_precond);
}

/**/
void GColormap::addColorPostcond (PsProc proc,
				 const void* closure, GAddOrRemove add) const
{
    ushort flag = _flags._has_addColor_postcond;

    const GColormapCondFlags* flags = &_flags;
    GColormapCondFlags* FLAGS = (GColormapCondFlags*)flags;
    
    _add_rem_callback(cmapNaddColorPostcond, _has_addColor_postcond);

    // The addColorPostcond is called from the sh_colormap;

    if (!flag && _flags._has_addColor_postcond)
	_sh_colormap->addColorPostcond (_call_call_addColor_postcond, this, GADD);
    else
	if (flag && !_flags._has_addColor_postcond)
	    _sh_colormap->addColorPostcond (_call_call_addColor_postcond,
					    this, GREMOVE);
}

/**/
void GColormap::remColorPrecond (PeProc proc,
				const void* closure, GAddOrRemove add) const
{
    const GColormapCondFlags* flags = &_flags;
    GColormapCondFlags* FLAGS = (GColormapCondFlags*)flags;
    
   _add_rem_callback(cmapNremColorPrecond, _has_remColor_precond);
}

/**/
void GColormap::remColorPostcond (PsProc proc,
				 const void* closure, GAddOrRemove add) const
{
    ushort flag = _flags._has_remColor_postcond;

    const GColormapCondFlags* flags = &_flags;
    GColormapCondFlags* FLAGS = (GColormapCondFlags*)flags;
    
    _add_rem_callback(cmapNremColorPostcond, _has_remColor_postcond);

    // The remColorPostcond is called from the ;
    // sh_colormap;

    if (!flag && _flags._has_remColor_postcond)
	_sh_colormap->remColorPostcond (_call_call_remColor_postcond,
					this, GADD);
    else
	if (flag && !_flags._has_remColor_postcond)
	    _sh_colormap->remColorPostcond (_call_call_remColor_postcond,
					    this, GREMOVE);
}

/**/
void GColormap::replaceColorPrecond (RepColorPeProc proc,
				    const void* closure, GAddOrRemove add) const
{
    const GColormapCondFlags* flags = &_flags;
    GColormapCondFlags* FLAGS = (GColormapCondFlags*)flags;
    
   _add_rem_callback(cmapNrepColorPrecond, _has_replaceColor_precond);
}

/**/
void GColormap::replaceColorPostcond (PsProc proc,
				     const void* closure, GAddOrRemove add) const
{
    ushort flag = _flags._has_replaceColor_postcond;

    const GColormapCondFlags* flags = &_flags;
    GColormapCondFlags* FLAGS = (GColormapCondFlags*)flags;
    
    _add_rem_callback(cmapNrepColorPostcond, _has_replaceColor_postcond);

    // The repColorPostcond is called from the ;
    // sh_colormap;

    if (!flag && _flags._has_replaceColor_postcond)
	_sh_colormap->replaceColorPostcond (_call_call_repColor_postcond,
					    this, GADD);
    else
	if (flag && !_flags._has_replaceColor_postcond)
	    _sh_colormap->replaceColorPostcond (_call_call_repColor_postcond,
						this, GREMOVE);

}

#undef _add_rem_callback
