#ifdef REV_INFO
static char rcsid[] = "$Id: ShColormap.cc,v 1.5 1992/09/25 14:19:33 smadja Exp $";
#endif
/*****************************************************************************
MODULE:  Tools
FILE:    ShColormap.cc
AUTHOR:  Jacques Davy
DATE:    October 1990

DESCRIPTION:
    See <Go/ShColormap.h>

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  license from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#include <Go/Tools/ShColormap.h>
#include <Go/GoStream.h>
#include <Go/Tools/Alloc.h>
#include <Go/Tools/Error.h>
#include <assert.h>

/*
 * static of the class
 */
GList*        GShColormap::_headOfList;
GFunctionList GShColormap::_funcList;

/*
 * GCallback types
 */
static char shcolormapNaddPostcond[1];
static char shcolormapNremPostcond[1];
static char shcolormapNrepPostcond[1];

struct PostcondRec
{
    void (*proc)(void*, const GShColormap*...);
    void* closure;
};




INLINE  void _ColorIsUpdated(void* This, GShColor* color, ulong mask)
{
    ColorIsUpdated((GShColormap*)This, color, mask);
}

void ColorIsUpdated(GShColormap* This, const GShColor* color, ulong mask)
{
    assert (color && mask);
    This->_call_update_postcond(GCOLORMAP_COLORS);
}

/*
 * Private methods
 */
/**/
ulong GShColormap::_arraySize (const GColormapValues* values) const
{
    return values->n_color;
}

/**/
void GShColormap::_resizeArray (ulong new_size)
{
    GShColor** new_array = NULL;
    if (new_size != 0)
    {
        new_array = new GShColor*[new_size];
        memset (new_array, 0, (int)new_size * sizeof (GShColor*));
    }

    if (_array_size != 0)
    {
        memcpy (new_array, _values.colors, (int)_array_size*sizeof(GShColor*));
	delete _values.colors;
    }
    _array_size = new_size;
    _values.colors = new_array;
}

/**/
const GShColor* GShColormap::_getColor (long index) const
{
    if (index >= 0 && ((ulong)index) < _array_size)
        return _values.colors[index];
    return NULL;
}

/**/
void GShColormap::_makeColormapValues (ulong array_size, GColormapValues& values)
{
    values.colors = new GShColor*[array_size];
    memset (values.colors, 0, (int)array_size * sizeof (GShColor*));
    memcpy (values.colors, _values.colors, (int)_array_size * sizeof(GShColor*));
    values.n_color = _values.n_color;
}

/**/
void GShColormap::_call_addColor_postcond (ulong index)
{
    if (_colormap_flags._has_addColor_postcond)
    {
	int count;
	PostcondRec* cbrec = (PostcondRec*)_callbacks.get (shcolormapNaddPostcond, count);
	for (int i = 0; i < count; i++, cbrec++)
	     cbrec->proc(cbrec->closure, this, index);
    }
}

/**/
void GShColormap::_call_remColor_postcond (ulong index)
{
    if (_colormap_flags._has_remColor_postcond)
    {
	int count;
	PostcondRec* cbrec = (PostcondRec*)_callbacks.get (shcolormapNremPostcond, count);
	for (int i = 0; i < count; i++, cbrec++)
	     cbrec->proc(cbrec->closure, this, index);
    }
}

/**/
void GShColormap::_call_replaceColor_postcond (ulong index)
{
    if (_colormap_flags._has_replaceColor_postcond)
    {
	int count;
	PostcondRec* cbrec = (PostcondRec*)_callbacks.get (shcolormapNrepPostcond, count);
	for (int i = 0; i < count; i++, cbrec++)
	     cbrec->proc(cbrec->closure, this, index);
    }
}


/*
 * Protected Methods
 */

int GShColormap::save(const GShColormap* colormap, GoStream* gostream) {

    ulong ncolors = colormap->nColor();
    const GShColor** colors = colormap->colors();
    GBool named = colormap->isNamed();
    ulong mask = colormap->mask();
    uint i = 0;

    if (!gostream->writeBool(named, "is named"))
	goto failure;
    if (named && !gostream->writeString(colormap->name(), "name"))
	goto failure;
    if (!gostream->writeLong(mask, "mask"))
	goto failure;
    if (mask & GCOLORMAP_COLORS) {
	if (! gostream->writeLong(ncolors, "nb colors"))
	    goto failure;

	for (i = 0 ; i < ncolors; i++, colors++) {
	    if (*colors == NULL) 
		gostream->writeLong(0, "undefined");
	    else  {
		ulong mask = (*colors)->mask();
		if ( mask == 0 ||  mask & (GCOLOR_GRAY|GCOLOR_HALFTONE) || (*colors)->isNamed()) {

		    if (!gostream->saveObj(*colors, (*colors)->getClassname()))
			goto failure;

		} else  if (mask & GCOLOR_RGB){
		    if (!gostream->writeRgb(*((*colors)->rgb())))
			goto failure;
		}
	    }
	}
    }
    return GSharedObject::save(colormap, gostream);
    
  failure:
    GWarning("save GShColormap failure");
    return 0;
}

/**/
int GShColormap::restore(GShColormap** sh, GoStream* gostream) {
    long mask;
    GColormapValues val;
    char str[256];
    char *name = str;
    GBool named;
    
    if (!gostream->readBool(&named))
	goto failure;
    if (named && !gostream->readString(256, &name))
	goto failure;

    if (!gostream->readLong(&mask))
	goto failure;

    if (mask & GCOLORMAP_COLORS)
    {
	if (! gostream->readLong((long*) &val.n_color))
	    goto failure;

	if (val.n_color <= 0)
	{
	    val.n_color = 0;
	    val.colors = NULL;
	}
	else
	{
	    val.colors = (GShColor**)GMALLOC(sizeof(GShColor*) * val.n_color);
	    GShColor** colors = val.colors;
	    for (uint i = 0 ; i < val.n_color; i++,  colors++)
	    {
		GRgb rgb;
		if (gostream->readRgb(&rgb))
		{
		    *colors = GCreateShColor(&rgb);
		    /**
		      to make ths creation equivalent to
		      a normal restoreObj function.
		      the following function does not read the stream
		      It just decrements the object refcnt.
		      **/
		  GSharedObject::readParams(*colors, gostream);
		}
		else if (gostream->readLong((long*)colors))
		{
		    if (*colors != NULL)
			goto failure;
		}
		else
		{
		    const char* classname;
		    if (! gostream->restoreObj((void**)colors, &classname))
			goto failure;
		    if (classname == NULL || strcmp (classname, GShColor::classname()) != 0)
			goto failure;
		}
	    }
	}
    }

    
    if (*sh = GCreateShColormap(mask, &val, named ? name : NULL))
    {
        GSharedObject::readParams(*sh, gostream);
	return 1;
    }
    else
    {
	(*sh)->destroy();
	*sh = NULL;
    }
    
  failure:
    GWarning("GShColormap restore failure");
    return 0;
}

/**/
int GShColormap::readParams(GShColormap*, GoStream*) {
    return 0;
}

/**/
GShColormap* _newGShColormap (GFunctionList* funcList, ulong mask,
			      const void * values,
			      const char* name)
{
    return new GShColormap (funcList, mask, (const GColormapValues*)values, name);
}

/**/
GShColormap::GShColormap (GFunctionList* funcList, ulong mask,
			  const GColormapValues* values, const char* name)
    : GSharedObject(funcList, mask, name)
{
    _array_size = 0;

    if (mask & GCOLORMAP_COLORS)
    {
        _resizeArray (_arraySize(values));
        for (ulong i = 0; i < _array_size; i++)
	    if (values->colors[i] != NULL)
	    {
                _values.colors[i] = values->colors[i]->dup();
		if (_values.colors[i]->isNamed())
		    _values.colors[i]->updatePostcond(_ColorIsUpdated, this, GADD);
	    }
        _values.n_color = values->n_color;
    }
    else
    {
        _values.n_color = 0;
	_values.colors = NULL;
    }
}

/**/
GShColormap::~GShColormap ()
{
    if (_array_size != 0)
    {
        for ( ; _values.n_color != 0; _array_size--)
            if (_values.colors[_array_size-1])
	    {
		if (_values.colors[_array_size-1]->isNamed())
		    _values.colors[_array_size-1]->updatePostcond(_ColorIsUpdated,
								  this, GREMOVE);
                _values.colors[_array_size-1]->destroy();
	        _values.n_color--;
            }
        delete _values.colors;
    }
}

/**/
GBool GShColormap::_match (ulong mask, const void * __values) const
{
const GColormapValues* values = (const GColormapValues*)__values;

    ulong inter_mask = mask & _mask;

    if (!inter_mask)
        return GTRUE;

    if (inter_mask & GCOLORMAP_COLORS)
    {
        if (values->n_color != _values.n_color)
            return GFALSE;
        if (memcmp (_values.colors, values->colors,
		    (int)_array_size * sizeof(GShColor*)) != 0)
	    return GFALSE;
    }
    return GTRUE;
}

/**/
void GShColormap::_makeNewValues (ulong mask, const void * __values,
			 ulong& newMask, void * __newValues) const
{
const GColormapValues* values = (const GColormapValues*)__values;
GColormapValues* newValues = ( GColormapValues*)__newValues;

    newMask = mask | _mask;

/* Values must not be duplicated because SharedObj class does not free them */
    if (mask & GCOLORMAP_COLORS)
    {
        newValues->n_color = values->n_color;
        newValues->colors = values->colors;
    } else
    {
        newValues->n_color = _values.n_color;
        newValues->colors = _values.colors;
    }
}

/**/
ulong GShColormap::_update (ulong mask, const void * __values)
{
const GColormapValues* values = (const GColormapValues*)__values;

    ulong update_mask = 0;

    if (mask & GCOLORMAP_COLORS)
    {
        ulong old_size = 0;
        ulong old_n_color = 0;
	GShColor** old_colors = NULL;

	/** Save old values to avoid unusefull destroy's **/
        if (_mask & GCOLORMAP_COLORS)
	{
            old_size = _array_size;
            old_n_color = _values.n_color;
            old_colors = _values.colors;
	}

	/** Init new values **/
        _array_size = 0;
        _resizeArray (_arraySize(values));
        for (ulong i = 0; i < _array_size; i++)
	    if (values->colors[i] != NULL)  {
		if (values->colors[i]->isNamed())
		    values->colors[i]->updatePostcond(_ColorIsUpdated, this, GADD);
                _values.colors[i] = values->colors[i]->dup();
	    }
        _values.n_color = values->n_color;

        // Free old values;
        if (_mask & GCOLORMAP_COLORS)
	{
            for ( ; old_n_color != 0; old_size--)
                if (old_colors[old_size-1])
	        {
		    if (old_colors[old_size-1]->isNamed())
			old_colors[old_size-1]->updatePostcond(_ColorIsUpdated,
							       this, GREMOVE);
                    old_colors[old_size-1]->destroy();
	            old_n_color--;
                }
            delete old_colors;
	}
	update_mask |= GCOLORMAP_COLORS;
}

    _mask |= mask;
    return update_mask;
}

/*
 * Public Methods
 */

/* static */
const char* GShColormap::classname()
{
    return "GShColormap";
}

/* virtual */
const char* GShColormap::getClassname() const
{
    return GShColormap::classname();
}

/**/
void GModify (const char* name, ulong mask, const GColormapValues* values)
{
    GMODIFY_NAMED_SHAREDOBJECT (GShColormap, mask, values, name);
}

static GShColormap* default_shcolormap = NULL;
/**/
GShColormap* GCreateShColormap (ulong mask, const GColormapValues* values, const char* name)
{
    if (mask == 0 && name == NULL)
    {
	if (!default_shcolormap)
	{
	    GColormapValues values;
	    GColorValues black_values, white_values;
	    GShColor* black, * white;
	    black_values.rgb.red =
		black_values.rgb.green =
		    black_values.rgb.blue = 0;
	    black = GCreateShColor (GCOLOR_RGB, &black_values);
	    white_values.rgb.red = 
		white_values.rgb.green =
		    white_values.rgb.blue = GMAX_RGB_VALUE;
	    white = GCreateShColor (GCOLOR_RGB, &white_values);
	    values.n_color = 2;
#ifdef NO_ALLOCA
	    values.colors = (GShColor**)GMALLOC(sizeof(GShColor*)*2);
#else
	    values.colors = (GShColor**)alloca((int)(sizeof(GShColor*) * 2));
#endif
	    values.colors[0] = black;
	    values.colors[1] = white;
	    default_shcolormap = GCreateShColormap (GCOLORMAP_COLORS, &values);
	    black->destroy();
	    white->destroy();
#ifdef NO_ALLOCA
	    GFREE(values.colors);
#endif
	}
	return default_shcolormap->dup();
    }
    
    return GCREATE_SHAREDOBJECT (GShColormap, mask, values, name);
}

/**/
GShColormap* GCreateShColormap (const char* name)
{
    return GCreateShColormap (0, NULL, name);
}

/**/
GShColormap* GCreateShColormap ()
{
    return GCreateShColormap (0, NULL, NULL);
}

/**
FUNCTION
    GShColormap::addColor
DESCRIPTION
    This method adds a color in the colormap and returns the color index in
    the colormap. If the color already exists in the colormap, it simply
    returns the color index in the colormap.
    In any case, the added color may be freed by the application.
**/
ulong GShColormap::addColor (GShColormap*& colormap, const GShColor* color)
{
    long index = getIndex (color);
    if (index >= 0)
	return index;        // color already exists in the color map

    // Create a new GColormapValues structure and call modify()
    GColormapValues newValues;
    ulong new_size = (_array_size == _values.n_color ?
		      _array_size + 1 : _array_size);
    _makeColormapValues (new_size, newValues);

    for (index = 0; newValues.colors[index] != NULL; index++);
    newValues.colors[index] = (GShColor*)color;
    newValues.n_color = _values.n_color + 1;

    modify ((GSharedObject*&)colormap, GCOLORMAP_COLORS, &newValues);
    delete newValues.colors;
    return index;
}

/**
FUNCTION
    GShColormap::remColor
DESCRIPTION
    These methods remove a color from the colormap and destroy it
**/
void GShColormap::remColor (GShColormap*& colormap, const GShColor* color)
{
    long index = getIndex (color);

    if (index >= 0)
	remColor (colormap, index);
}
/**/
void GShColormap::remColor (GShColormap*& colormap, ulong index)
{
    if (index < _array_size)
        if (_values.colors[index])
	{
            GColormapValues newValues;
	    _makeColormapValues (_array_size, newValues);
	    newValues.colors[index] = NULL;
	    newValues.n_color = _values.n_color - 1;

	    modify ((GSharedObject*&)colormap, GCOLORMAP_COLORS, &newValues);
	    delete newValues.colors;
        }
}
/**
FUNCTION
    GShColormap::replaceColor
DESCRIPTION
    These functions replace an existing color by another. If the old
    color does not exist, the new one is inserted in the colormap
    (see addColor). These functions return the index. Old_color and
    new_color may be freed by the application.
**/
ulong GShColormap::replaceColor (GShColormap*& colormap,
			      const GShColor* old_color, const GShColor* new_color)
{
    long index = getIndex (old_color);
    if (index < 0)
        return addColor (colormap, new_color);

    return replaceColor (colormap, index, new_color);
}

/**/
ulong GShColormap::replaceColor (GShColormap*& colormap,
			      ulong index, const GShColor* new_color)
{
    ulong new_size = (index >= _array_size ?
		      index + 1 : _array_size);

    GColormapValues newValues;
    _makeColormapValues (new_size, newValues);
    if (newValues.colors[index] == NULL)
        newValues.n_color = _values.n_color + 1;
    newValues.colors[index] = (GShColor*)new_color;

    modify ((GSharedObject*&)colormap, GCOLORMAP_COLORS, &newValues);
    delete newValues.colors;
    return index;
}


/**
FUNCTION
    GShColormap::getColor (From Something)
DESCRIPTION
    These methods returns a pointer to the found color or NULL
**/
const GShColor* GShColormap::getColor (const char* name) const
{
    return (_getColor(getIndex(name)));
}

/**/
const GShColor* GShColormap::getColor (const GRgb* rgb) const
{
    return (_getColor(getIndex(rgb)));
}

/**/
const GShColor* GShColormap::getColor (uchar gray) const
{
    return (_getColor(getIndex(gray)));
}

/**/
const GShColor* GShColormap::getColor (const GShPattern* halftone) const
{
    return (_getColor(getIndex(halftone)));
}

/**/
const GShColor* GShColormap::getColor (ulong index) const
{
    return (_getColor(index));
}

/**
FUNCTION
    GShColormap::getIndex (From Something)
DESCRIPTION
    These methods returns the color index in the colormap or -1
**/
long GShColormap::getIndex (const char* name) const
{
    if (!name)
        return -1;

    ulong index = 0;
    GShColor* color;

    for ( ; index < _array_size; index++)
        if ((color = _values.colors[index]) &&
            (color->name()) && (strcmp(color->name(), name) == 0))
	        return index;
    return -1;
}

/**/
long GShColormap::getIndex (const GRgb* rgb) const
{
    ulong index = 0;
    GShColor* color;

    for ( ; index < _array_size; index++)
        if ((color = _values.colors[index]) &&
             (color->mask() & GCOLOR_RGB) &&
	      (color->rgb()->red == rgb->red) &&
               (color->rgb()->green == rgb->green) &&
	        (color->rgb()->blue == rgb->blue))
	          return index;
    return -1;
}

/**/
long GShColormap::getIndex (uchar gray) const
{
    ulong index = 0;
    GShColor* color;

    for ( ; index < _array_size; index++)
        if ((color = _values.colors[index]) &&
             (color->mask() & GCOLOR_GRAY) &&
	      (color->gray() == gray))
	          return index;
    return -1;
}

/**/
long GShColormap::getIndex (const GShPattern* halftone) const
{
    ulong index = 0;
    GShColor* color;

    for ( ; index < _array_size; index++)
        if ((color = _values.colors[index]) &&
             (color->mask() & GCOLOR_HALFTONE) &&
	      (color->halftone() == halftone))
	          return (long)index;
    return -1;
}

/**/
long GShColormap::getIndex (const GShColor* color) const
{
    ulong index = 0;
    for ( ; index < _array_size; index++)
        if (_values.colors[index] == color)
	    return (long)index;
    return -1;
}

/**
FUNCTION
    Add/Rem Postconditions
DESCRIPTION
    Allow to add/rem postcondition on adding, removing or replacing color
    in the colormap.
**/
#define _add_rem_callback(type, flag) \
    (FLAGS->flag =\
         0 < ((add == GADD)\
	      ? _addCallback(type, (GCBProc)cb, closure)\
	      : _remCallback(type, (GCBProc)cb, closure)))

/**/
void GShColormap::addColorPostcond (void cb(void*, GShColormap*, ulong),
				   const void* closure, GAddOrRemove add) const
{
    const GShColormapCondFlags* flags = &_colormap_flags;
    GShColormapCondFlags* FLAGS = (GShColormapCondFlags*)flags;
    
   _add_rem_callback(shcolormapNaddPostcond, _has_addColor_postcond);
}

void GShColormap::remColorPostcond (void cb(void*, GShColormap*, ulong),
				   const void* closure, GAddOrRemove add) const
{
    const GShColormapCondFlags* flags = &_colormap_flags;
    GShColormapCondFlags* FLAGS = (GShColormapCondFlags*)flags;
    
   _add_rem_callback(shcolormapNremPostcond, _has_remColor_postcond);
}

void GShColormap::replaceColorPostcond (void cb(void*, GShColormap*, ulong),
				       const void* closure, GAddOrRemove add) const
{
    const GShColormapCondFlags* flags = &_colormap_flags;
    GShColormapCondFlags* FLAGS = (GShColormapCondFlags*)flags;
    
   _add_rem_callback(shcolormapNrepPostcond, _has_replaceColor_postcond);
}
