#ifdef REV_INFO
static char rcsid[] = "$Id: Cons.cc,v 1.1.1.1 1992/03/18 10:41:44 uep Exp $";
#endif
/*****************************************************************************
MODULE:  Tools
FILE:    Cons.cc
AUTHOR:  Olivier Jojic
DATE:    29 May 1990

DESCRIPTION:
    See Cons.h

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#include <Go/Tools/Cons.h>


/**
FUNCTION
    Its the constructor for class GCons.
    
DESCRIPTION
    It returns a new GCons <a, l>.
**/
GCons::GCons (const void* a, const GCons* l)
{
    _car = (void*)a;
    _cdr = (GCons*)l;
}


/**
FUNCTION
    This function allows you to create a list.
    
DESCRIPTION
    the last parameter must be GNIL.
    Nothing is as clear as an example:
        list(a, b, c, GNIL) --> <a, b, c>
**/
GCons* GConsList (void* a, ...)
{
    if (!a) return GNIL;
    
    void** p = &a;
    GCons* head = new GCons(*p++, GNIL);
    GCons* l = head;
    while (*p)
	l = l->_cdr = new GCons(*p++, GNIL);

    return head;
}


/**
FUNCTION
    This function insert an elem at the beginning of the list.
**/
void GNewl (GCons*& l, const void* a)
{
    l = new GCons(a, l);
}


/**
FUNCTION
    This function append an elem at the end of the list.
**/
void GNewr (GCons*& lref, const void* a)
{
    if (lref == GNIL)
	lref = new GCons(a, GNIL);
    else
    {
	for (GCons* l = lref; l->_cdr; l = l->_cdr);
	l->_cdr = new GCons(a, GNIL);
    }
}


/**
FUNCTION
    Concat the list "l2" at the end of the list "l1".
**/
void GNconc (GCons*& lref, const GCons* l2)
{
    if (!lref)
	lref = (GCons*)l2;
    else
    {
	for (GCons* l = lref; l->_cdr != GNIL; l = l->_cdr);
	l->_cdr = (GCons*)l2;
    }
}


/**
FUNCTION
    Concat the list "l2" at the end of a copy of the list "l1".
    Then returns the copy.

DESCRIPTION
    If "l2" is GNIL, this function allows you to make a copy
    of a list.
**/
GCons* GAppend (GCons* l1, const GCons* l2)
{
    if (!l1) return (GCons*)l2;
    GCons* head = new GCons(l1->_car, GNIL);

    for (GCons* l = head; l1 != GNIL; l1 = l1->_cdr)
	l = l->_cdr = new GCons(l1->_car, GNIL);
    l->_cdr = (GCons*)l2;
    return head;
}


/**
FUNCTION
    It's the destructor of the class GCons.
    
DESCRIPTION
    It deletes the entire list (not only the first elem).
    Don't forget to free the elems if necessary.
**/
GCons::~GCons ()
{
    if (_cdr)
	delete _cdr;
}


/**
FUNCTION
    Deletes the elem "a" from the list "lref".
    
DESCRIPTION
    If "a" is not in the list does nothing.
**/
void GRemq (GCons*& lref, const void* a)
{
    GCons*& l = GMemq(lref, a);
    if (l)
    {
	GCons* tmp = l;
	l = l->_cdr;
	tmp->_cdr = GNIL;
	delete tmp;
    }
}


/**
FUNCTION
    Returns the first elem of the list and removes the first elem
    from the list.
**/
void* GNextl (GCons*& l)
{
    GCons* tmp = l;
    l = l->_cdr;
    tmp->_cdr = GNIL;
    void* a = tmp->_car;
    delete tmp;
    return a;
}
    

/**
FUNCTION
    Returns the last GCons of the list.

DESCRIPTION
    An example:
        last(<a b c d>) --> <d>  (and not d)
    If you want the last elem, you can do:
        car(last(<a b c d>)) --> d
**/
GCons* GLast (const GCons* l)
{
    if (l == GNIL) return (GCons*)l;
    for (; l->_cdr != GNIL; l = l->_cdr);
    return (GCons*)l;
}


/**
FUNCTION
    Returns the sub-list starting with the elem.

DESCRIPTION
    Two examples:
        memq(<1 2 3 4>, 3) --> <3 4>
	memq(<1 2 3 4>, 5) --> GNIL
**/
GCons*& GMemq (GCons*& lref, const void* a)
{
    static GCons* ref_to_GNIL = GNIL;

    for (GCons** p = &lref; *p != GNIL; p = &((*p)->_cdr))
	if ((*p)->_car == a)
	    return *p;

    return ref_to_GNIL;
}
    
/**
FUNCTION
    Returns the number of elems in the list.
**/
int GLength (const GCons* l)
{
    for (int i = 0; l != GNIL; l = l->_cdr, i++);
    return i;
}
