#ifdef REV_INFO
static char rcsid[] = "$Id: Dash.cc,v 1.2 1992/09/25 14:20:56 smadja Exp $";
#endif
/*****************************************************************************
MODULE:  Tools
FILE:    Dash.cc
AUTHOR:  Jacques Davy
DATE:    November 1990

DESCRIPTION:
    (see <Go/Dash.h>

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#include <Go/Tools/Dash.h>
#include <assert.h>

/*
 * Callback types
 */
static char dashNmodifyPrecond[1];
static char dashNmodifyPostcond[1];
static char dashNdestroyPostcond[1];

typedef GBool (*PeProc)(void*, const GDash*, ulong&, GDashValues*);
typedef void (*PsProc)(void*, GDash*, ulong);
typedef void (*PdProc)(void*, const GDash*);

struct PrecondRec
{
    PeProc proc;
    void* closure;
};

struct ModifPostcondRec
{
    PsProc proc;
    void* closure;
};

struct DestroyPostcondRec
{
    PdProc proc;
    void* closure;
};

/*
 * Private methods
 * ---------------
 */
/**
FUNCTION
    _call_call_modify_postcond
DESCRIPTION
    This function is called when the NAMED GShDash is updated
**/
void _call_call_modify_postcond (void* This, GShDash* sh_Dash,
				 ulong update_mask)
{
    Call_call_modify_postcond ((GDash*)This, sh_Dash, update_mask);
}

void Call_call_modify_postcond (GDash* This,
				const GShDash* sh_Dash,
				ulong update_mask)
{
    assert (sh_Dash == This->shDash());
    This->_call_modify_postcond (update_mask);
}

/*
 * Protected methods
 * -----------------
 */
/**/
int GDash::_addCallback (const char* type, GCBProc proc, const void* closure) const
{
    const GAssoc* _cb = &_assoc;
    GAssoc* cb = (GAssoc*)_cb;
    
    GCallbackRec cbrec (proc, closure);
    return cb->add(type, sizeof cbrec, (char*)&cbrec);
}

/**/
int GDash::_remCallback (const char* type, GCBProc proc, const void* closure) const
{
    const GAssoc* _cb = &_assoc;
    GAssoc* cb = (GAssoc*)_cb;


    GCallbackRec cbrec (proc, closure);
    return cb->rem(type, sizeof cbrec, (char*)&cbrec);
}

/**/
GBool GDash::_call_modify_precond (ulong& modify_mask, GDashValues* new_values)
{
    if (_flags._has_modify_precond)
    {
	int count;
	PrecondRec* cbrec = (PrecondRec*)_assoc.get (dashNmodifyPrecond, count);
	for (int i = 0; i < count; i++, cbrec++)
	     if (!cbrec->proc(cbrec->closure, this, modify_mask, new_values))
		 return GFALSE;
    }
    return GTRUE;
}

/**/
void GDash::_call_modify_postcond (ulong modify_mask)
{
    if (_flags._has_modify_postcond)
    {
	int count;
	ModifPostcondRec* cbrec = (ModifPostcondRec*)_assoc.get (dashNmodifyPostcond, count);
	for (int i = 0; i < count; i++, cbrec++)
	     cbrec->proc(cbrec->closure, this, modify_mask);
    }
}

/**/
void GDash::_call_destroy_postcond ()
{
    if (_flags._has_destroy_postcond)
    {
	int count;
	DestroyPostcondRec* cbrec = (DestroyPostcondRec*)_assoc.get (dashNdestroyPostcond, count);
	for (int i = 0; i < count; i++, cbrec++)
	     cbrec->proc(cbrec->closure, this);
    }
}

static GShDash* default_shdash = NULL;
/*
 * Public methods
 * --------------
 */
GDash::GDash ()
{
    if (default_shdash == NULL)
	default_shdash = GCreateShDash(0, NULL);
    assert (default_shdash->refcnt() >= 1);
    _sh_dash = default_shdash->dup();
}

GDash::GDash(const GDash* dash)
{
    _sh_dash = dash->_sh_dash->dup();
}

GDash::GDash (const GShDash* sh_dash)
{
    _sh_dash = sh_dash->dup();
}

GDash::GDash (ulong mask, const GDashValues* values, const char* name)
{
    _sh_dash = GCreateShDash (mask, values, name);
}

GDash::GDash (const char* name)
{
    _sh_dash = GCreateShDash (name);
}

GDash::~GDash ()
{
    _call_destroy_postcond ();
    if (_sh_dash->isNamed() && _flags._has_modify_postcond)
        _sh_dash->updatePostcond (_call_call_modify_postcond, this, GREMOVE);
    _sh_dash->destroy();
}

/**
FUNCTION
    GDash::modify
DESCRIPTION
    Modify the shared dash and call pre/post condition.
**/

void GDash::modify (ulong mask, const GDashValues* values)
{
    GDashValues Values = *values;
    if (mask && _call_modify_precond (mask, &Values))
    {
	// Remove postcond;
	if (_sh_dash->isNamed() && _flags._has_modify_postcond)
	    _sh_dash->updatePostcond (_call_call_modify_postcond, this, GREMOVE);

	// Modify;
        GModify (_sh_dash, mask, &Values);

	// Call postcond;
	_call_modify_postcond (mask);

	// Add postcond;
	if (_sh_dash->isNamed() && _flags._has_modify_postcond)
	    _sh_dash->updatePostcond (_call_call_modify_postcond, this, GADD);

    }
}

void GDash::modify (const char* name)
{
    if (name && _sh_dash->name() && (strcmp (name, _sh_dash->name()) == 0))
        return;

    // Remove postcond if the dash is already named.
    if (_sh_dash->isNamed() && _flags._has_modify_postcond)
        _sh_dash->updatePostcond (_call_call_modify_postcond, this, GREMOVE);

    // Get mask and values
    GShDash* save = _sh_dash->dup();

    GRename (save, name);
    
    GDashValues Values = *save->values();
    ulong mask = save->mask();

    if (mask && _call_modify_precond (mask, &Values))
    {
         _sh_dash->destroy();
         _sh_dash = GCreateShDash (mask, &Values, name);
         _call_modify_postcond (mask);
    }
    save->destroy();

    // Add postcond if the dash is named.
    if (name && _flags._has_modify_postcond)
	_sh_dash->updatePostcond (_call_call_modify_postcond, this, GADD);
}

void GDash::modify(const GShDash* sh_dash)
{
    if (sh_dash == _sh_dash)
	return;

    // top of list.
    sh_dash->raise(GShDash::_headOfList);
    
    if (sh_dash->isNamed())
	modify(sh_dash->name());
    else
    {
	GDashValues Values ;
	Values  = *sh_dash->values();
	ulong mask = sh_dash->mask();
	    

	// Remove postcond if the dash is already named;
	if (_sh_dash->name() && _flags._has_modify_postcond)
	    _sh_dash->updatePostcond (_call_call_modify_postcond, this, GREMOVE);

    	// Rename to NULL;
	GRename(_sh_dash, NULL);
		
	if (mask && _call_modify_precond (mask, &Values))
	{
	    // Modify;
	    GModify (_sh_dash, mask, &Values);

	    // Call postcond;
	    _call_modify_postcond (mask);
	}


	// Add postcond;
	if (_sh_dash->isNamed() && _flags._has_modify_postcond)
	    _sh_dash->updatePostcond (_call_call_modify_postcond, this, GADD);
    }
}

/**
FUNCTION
    GDash::field_modification_methods
DESCRIPTION
    Each method modify one parameter of the GDash.
**/

#define _modifyDashValue(bit_mask, field, value) \
    GDashValues values;\
    values.field = value;\
    modify (bit_mask, &values)

#undef _modifyDashValue

/**
FUNCTION
    GDash::add/rem pre/postconditions
DESCRIPTION
    Add or remove pre/post condition on modification and on destruction.

    GWarning, if the GShDash is named, _call_call_modify_postcond
    is stored directly in the shared object and pre/condition
    may be not called!
**/

#define _add_rem_callback(type, flag) \
    (FLAGS->flag =\
         0 < ((add == GADD)\
	      ? _addCallback(type, (GCBProc)proc, closure)\
	      : _remCallback(type, (GCBProc)proc, closure)))

/**/
void GDash::modifyPrecond (PeProc proc,
			  const void* closure, GAddOrRemove add) const
{
    const GDashCondFlags* flags = &_flags;
    GDashCondFlags* FLAGS = (GDashCondFlags*)flags;
    
   _add_rem_callback(dashNmodifyPrecond, _has_modify_precond);
}

/**/
void GDash::modifyPostcond (PsProc proc,
			   const void* closure, GAddOrRemove add) const
{
   ushort flag = _flags._has_modify_postcond;

    const GDashCondFlags* flags = &_flags;
    GDashCondFlags* FLAGS = (GDashCondFlags*)flags;
    
   _add_rem_callback(dashNmodifyPostcond, _has_modify_postcond);

   // If the dash is named, the modifyPostcond is called from the 
   // sh_dash
   if (_sh_dash->isNamed())
   {
       if (!flag && _flags._has_modify_postcond)
           _sh_dash->updatePostcond (_call_call_modify_postcond, this, GADD);
       else
           if (flag && !_flags._has_modify_postcond)
               _sh_dash->updatePostcond (_call_call_modify_postcond, this, GREMOVE);
   }
}

/**/
void GDash::destroyPostcond (PdProc proc,
			    const void* closure, GAddOrRemove add) const
{
    const GDashCondFlags* flags = &_flags;
    GDashCondFlags* FLAGS = (GDashCondFlags*)flags;
    
   _add_rem_callback(dashNdestroyPostcond, _has_destroy_postcond);
}

#undef _add_rem_callback
