#ifdef REV_INFO
static char rcsid[] = "$Id: ShDash.cc,v 1.5 1992/12/14 15:52:52 smadja Exp $";
#endif
/*****************************************************************************
MODULE:  Tools
FILE:    ShDash.cc
AUTHOR:  Jacques Davy
DATE:    October 1990

DESCRIPTION:
    See <Go/Tools/ShDash.h>

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  license from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#include <Go/Tools/ShDash.h>
#include <Go/Tools/Error.h>
#include <Go/GoStream.h>
#include <Go/Tools/Medium.h>

/*
 * static of the class
 */
GList*        GShDash::_headOfList;
GFunctionList GShDash::_funcList;

/*
 * Protected Methods
 */

int GShDash::save(const GShDash* sh, GoStream* gostream)
{
    const GDashValues* val = sh->values();
    unsigned long mask = sh->mask();
    GBool named = sh->isNamed();
    
    if (!gostream->writeBool(named, "is named"))
	goto failure;
    if (named && !gostream->writeString(sh->name(), "name"))
	goto failure;
    if (!gostream->writeLong(sh->mask(), "mask"))
	goto failure;
    if ((mask & GDASH_OFFSET) &&
	! gostream->writeNumber(val->dash_offset, "dash offset"))
	goto failure;
    if (mask & GDASH_LIST)
    {
	if (gostream->writeShort(val->dash_number, "dash number"))
	{
	    if (val->dash_number > 0 &&
		! gostream->writeNumber(val->dash_list[0], "dash list"))
		goto failure;
	    for (int i = 1; i < val->dash_number; i++)
	    {
		if (! gostream->writeNumber(val->dash_list[i], NULL))
		    goto failure;
	    }
	}
	else goto failure;
    }
    return GSharedObject::save(sh, gostream);
    
  failure:
    GWarning("GShDash save failure");
    return 0;
}

/**/
int GShDash::restore(GShDash** sh, GoStream* gostream)
{
    long mask;
    GDashValues val;
    char str[256];
    char *name = str;
    GBool named;
    
    if (!gostream->readBool(&named))
	goto failure;
    if (named && !gostream->readString(256, &name))
	goto failure;

    if (!gostream->readLong(&mask))
	goto failure;
    
    if (mask & GDASH_OFFSET)
    {
	// here I have to handle several versions;
	// For the previous version. dash_offset was a ushort;
	// WHAT a BUG !!!!!;
	ulong save_pos = gostream->medium()->mtell();
	// begin by the normal type;
	if (!gostream->readNumber(&val.dash_offset))
	{
	    // re pos the medium;
	    gostream->medium()->mseek(save_pos, GMSEEK_FROM_BEGINNING);
	    ushort old_dash_offset;
	    if (!gostream->readShort((short*)&old_dash_offset))
		goto failure;
	    val.dash_offset = (GNumber)old_dash_offset;
	}
    }
	    
    if (mask & GDASH_LIST) {
	if (gostream->readShort((short*) &val.dash_number))
	{
	    if (val.dash_number <= 0)
	    {
		val.dash_number = 0;
		val.dash_list = NULL;
	    }
	    else
	    {
		val.dash_list = new GNumber[val.dash_number];
		for (int i = 0; i < val.dash_number; i++)
		{
		    if (!gostream->readNumber(&(val.dash_list[i])))
			goto failure;
		}
	    }
	}
	else goto failure;
    }

    if (*sh = GCreateShDash(mask, &val, named ? name : NULL))
    {
	if (GSharedObject::readParams(*sh, gostream))
	    return 1;
	else
	{
	    (*sh)->destroy();
	    *sh = NULL;
	}
    }
    
  failure:
    GWarning("GShDash restore failure");
    return 0;
}

/**/
int GShDash::readParams(GShDash*, GoStream*)
{
    return 0;
}

/**/
GShDash* _newGShDash (GFunctionList* funcList, ulong mask,
		      const void * values, const char* name)
{
    return new GShDash (funcList, mask, (const GDashValues*)values, name);
}

/**/
GShDash::GShDash (GFunctionList* funcList, ulong mask, const GDashValues* values,
		const char* name) : GSharedObject(funcList, mask, name)
{
    _values.dash_offset = (mask & GDASH_OFFSET ? values->dash_offset : 0);

    if (mask & GDASH_LIST && values->dash_number)
    {
        _values.dash_number = values->dash_number;
	_values.dash_list = new GNumber[_values.dash_number];
        memcpy (_values.dash_list, values->dash_list,
		_values.dash_number*sizeof(GNumber));
    }
    else
    {
	_values.dash_number = 0;
        _values.dash_list = NULL;
	_mask &= ~GDASH_LIST;
    }
}

/**/
GShDash::~GShDash ()
{
    if ( _mask & GDASH_LIST )
        delete _values.dash_list;
}

/**/
GBool GShDash::_match (ulong mask, const void * __values) const
{
const GDashValues* values = (const GDashValues* ) __values;

    ulong inter_mask = mask & _mask;

    if (!inter_mask)
        return GTRUE;

    if (inter_mask & GDASH_LIST)
    {
        if (_values.dash_number != values->dash_number)
	    return GFALSE;
        if (bcmp (_values.dash_list, values->dash_list,
		  _values.dash_number*sizeof(GNumber)) != 0)
	    return GFALSE;
    }

    if (inter_mask & GDASH_OFFSET)
        if (_values.dash_offset != values->dash_offset)
	    return GFALSE;

    return (GTRUE);
}

/**/
void GShDash::_makeNewValues (ulong mask, const void * __values,
			ulong& newMask, void * __newValues) const
{
const GDashValues* values = (const GDashValues* ) __values;
GDashValues* newValues = (GDashValues* ) __newValues;

    newMask = mask | _mask;

    /* Values must not be duplicated because SharedObj class does not free them */

    newValues->dash_offset = (mask & GDASH_OFFSET ?
			      values->dash_offset: _values.dash_offset);
    if (mask & GDASH_LIST)
    {
        newValues->dash_number = values->dash_number;
	newValues->dash_list = values->dash_list;
    }
    else
    {
        newValues->dash_number = _values.dash_number;
	newValues->dash_list = _values.dash_list;
    }
}

/**/
ulong GShDash::_update (ulong mask, const void * __values)
{
const GDashValues* values = (const GDashValues* ) __values;
    ulong update_mask = 0;

    if ((mask & GDASH_OFFSET) &&
	(!(_mask & GDASH_OFFSET) ||
	 (_values.dash_offset != values->dash_offset)))
    {
        _values.dash_offset = values->dash_offset;
	update_mask |= GDASH_OFFSET;
    }

    if ((mask & GDASH_LIST) &&
	(!(_mask & GDASH_LIST) ||
	 (_values.dash_number != values->dash_number) ||
	 (bcmp(_values.dash_list, values->dash_list,
	       values->dash_number*sizeof(GNumber)) != 0)))
    {
        if (_mask & GDASH_LIST)
        {
            delete _values.dash_list;
	    _values.dash_list = NULL;
	}
        _values.dash_number = values->dash_number;
	if (_values.dash_number)
	{
	    _values.dash_list = new GNumber[_values.dash_number];
            memcpy (_values.dash_list, values->dash_list,
		    _values.dash_number*sizeof(GNumber));
	}
        update_mask |= GDASH_LIST;
    }

    _mask |= mask;
    return update_mask;
}

/*
 * Public Methods
 */

/* static */
const char* GShDash::classname()
{
    return "GShDash";
}

/* virtual */
const char* GShDash::getClassname() const
{
    return GShDash::classname();
}

/**/
void GModify (const char* name, ulong mask, const GDashValues* values)
{
    GMODIFY_NAMED_SHAREDOBJECT (GShDash, mask, values, name);
}

static GShDash* default_shdash = NULL;
/**/
GShDash* GCreateShDash (ulong mask, const GDashValues* values, const char* name)
{
    if (mask == 0 && name == NULL)
    {
	if (default_shdash == NULL)
	{
	    GDashValues v;
	    v.dash_offset = 0;
	    v.dash_number = 2;
	    GNumber list[2];
	    list[0] = 1;
	    list[1] = 1;
	    v.dash_list = &list[0];
	    default_shdash = GCREATE_SHAREDOBJECT(GShDash,
						GDASH_LIST|GDASH_OFFSET,
						&v, NULL);
	}
	return default_shdash->dup();
    }
    return GCREATE_SHAREDOBJECT(GShDash, mask, values, name);
}

/**/
GShDash* GCreateShDash ()
{
    return GCreateShDash(0, NULL, NULL);
}
/**/
GShDash* GCreateShDash (const char* name)
{
    return GCreateShDash(0, NULL, name);
}



