#ifdef REV_INFO
static char rcsid[] = "$Id: Error.cc,v 1.4 1992/12/10 13:33:19 jojic Exp $";
#endif
/*****************************************************************************
MODULE:  Tools
FILE:    Error.cc
AUTHOR:  Philippe Smadja
DATE:    05-17-89

DESCRIPTION:
     See Error(3Go)

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include <Go/machdef.h>
#include <Go/Tools/Error.h>
#include <stdio.h>
#include <stdarg.h>
#if defined (__BORLANDC__) || defined (__MSC7__)
#include <io.h>
#include <windows.h>
#endif

int _gTraceLevel  = 1; // keep to be external for unix debugging;
int& GTraceLevel()
{
    return _gTraceLevel;
}
    
extern "C" {
    void GWarning (const char* format, ...);
    void GError (const char* format, ...)  ;
}


/**
FUNCTION
    void GWarning (const char* format, ...)
DESCRIPTION
    Print a Warning message
**/
void GWarning (const char* format, ...)
{
    if (_gTraceLevel ) {
	va_list args;
	va_start(args, format);
#if defined (__BORLANDC__) || defined (__MSC7__)
	char tmp[256];
	vsprintf(tmp, format, args);
	MessageBox(NULL, tmp, "** Go Warning :", MB_ICONEXCLAMATION | MB_OK);
#else
	fprintf(stderr, "** Go Warning: ");
	vfprintf(stderr, format, args);
	fprintf(stderr, "\n");
#endif
	va_end(args);
    }
}

/**
FUNCTION
    void GError (const char* format, ...)
DESCRIPTION
    Print an error message and exit
**/
void GError (const char* format, ...)
{
    if (_gTraceLevel) {
    	va_list args;
   	va_start(args, format);

#if defined (__BORLANDC__) || defined (__MSC7__)
	char tmp[256];
	vsprintf(tmp, format, args);
	MessageBox(NULL, tmp, "** Go Error: ", MB_ICONEXCLAMATION | MB_OK);
#else
	fprintf(stderr, "** Go Error: ");
	vfprintf(stderr, format, args);
	fprintf(stderr, "\n");
#endif
    va_end(args);
#ifndef __MSC7__
    exit(1);
#else
/* MSC7 doesn't have exit for DLLs. A general protection error is
   liberately produced to replace it */
    char * tmpError = NULL;
    *tmpError = '\0';
#endif		
   }
}



