#ifdef REV_INFO
static char rcsid[] = "$Id: ShFont.cc,v 1.3 1992/09/25 14:24:28 smadja Exp $";
#endif
/*****************************************************************************
MODULE:  Tools
FILE:    ShFont.cc
AUTHOR:  Jacques Davy
DATE:    August 1990

DESCRIPTION:
    See <Go/Tools/ShFont.h>

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  license from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#include <Go/Tools/ShFont.h>
#include <Go/Tools/Error.h>
#include <Go/GoStream.h>
#include <ctype.h>


/*
 * static of the class
 */
GList*        GShFont::_headOfList;
GFunctionList GShFont::_funcList;

/**
  some little tools
  for  fontvalues field:
  matching and copying are in lower case;
  **/

static void _copyField(char* dst, const char* src, int len)
{
    for (int i = 0; i < len; i++)
    {
	char c = *src++;
	if (isupper(c))
	    *dst++ = tolower(c);
	else *dst++ = c;
    }
    *dst = '\0';
}

static GBool _sameField(const char* s1, const char* s2)
{
    int len = strlen(s1);
    if (len != strlen(s2))
	return GFALSE;
    for (int i = 0; i < len; i++)
    {
	char c1 = *s1++;
	if (isupper(c1)) c1 = tolower(c1);
	char c2 = *s2++;
	if (isupper(c2)) c2 = tolower(c2);
	if (c1 != c2)
	    return GFALSE;
    }
    return GTRUE;
}

/*
 * Protected Methods
 */


int GShFont::save(const GShFont* sh, GoStream* gostream) {
    const GFontValues* val = sh->values();
    unsigned long mask = sh->mask();
    GBool named = sh->isNamed();
    
    if (!gostream->writeBool(named, "is named"))
	goto failure;
    if (named && !gostream->writeString(sh->name(), "name"))
	goto failure;
    if (!gostream->writeLong(sh->mask(), "mask"))
	goto failure;
    if ((mask & GFONT_REGISTRY) &&
	! gostream->writeString(val->registry, "registry"))
	goto failure;
    if ((mask & GFONT_FAMILY) &&
	! gostream->writeString(val->family, "family"))
	goto failure;
    if ((mask & GFONT_WEIGHT) &&
	! gostream->writeString(val->weight, "weight"))
	goto failure;
    if ((mask & GFONT_SLANT) &&
	! gostream->writeString(val->slant, "slant"))
	goto failure;
    if ((mask & GFONT_WIDTH) &&
	! gostream->writeString(val->width, "width"))
	goto failure;
    if ((mask & GFONT_ADD_STYLE) &&
	! gostream->writeString(val->add_style, "add style"))
	goto failure;
    if ((mask & GFONT_SPACING) &&
	! gostream->writeString(val->spacing, "spacing"))
	goto failure;
    if ((mask & GFONT_CHARSET) &&
	! gostream->writeString(val->charset_registry, "charset"))
	goto failure;
    if ((mask & GFONT_CHARCODE) &&
	! gostream->writeString(val->charset_encoding, "charcode"))
	goto failure;
    return GSharedObject::save(sh, gostream);
    
  failure:
    GWarning("GShFont save failure");
    return 0;
}

/**/
int GShFont::restore(GShFont** sh, GoStream* gostream) {
    long mask;
    GFontValues val;
    char str[256];
    char *name = str;
    GBool named;
    char reg[256], fam[256], wei[256], sla[256], wid[256];
    char add[256], spa[256], set[256], cod[256];

    if (!gostream->readBool(&named))
	goto failure;
    if (named && !gostream->readString(256, &name))
	goto failure;
    if (!gostream->readLong(&mask))
	goto failure;
    
    if (mask & GFONT_REGISTRY) {
	val.registry = reg;
	if (! gostream->readString(256, &val.registry))
	    goto failure;
    }
    if (mask & GFONT_FAMILY) {
	val.family = fam;
	if (! gostream->readString(256, &val.family))
	    goto failure;
    }
    if (mask & GFONT_WEIGHT) {
	val.weight = wei;
	if (! gostream->readString(256, &val.weight))
	    goto failure;
    }
    if (mask & GFONT_SLANT) {
	val.slant = sla;
	if (! gostream->readString(256, &val.slant))
	    goto failure;
    }
    if (mask & GFONT_WIDTH) {
	val.width = wid;
	if (! gostream->readString(256, &val.width))
	    goto failure;
    }
    if (mask & GFONT_ADD_STYLE) {
	val.add_style = add;
	if (! gostream->readString(256, &val.add_style))
	    goto failure;
    }
    if (mask & GFONT_SPACING) {
	val.spacing = spa;
	if (! gostream->readString(256, &val.spacing))
	    goto failure;
    }
    if (mask & GFONT_CHARSET) {
	val.charset_registry = set;
	if (! gostream->readString(256, &val.charset_registry))
	    goto failure;
    }
    if (mask & GFONT_CHARCODE) {
	val.charset_encoding = cod;
	if (! gostream->readString(256, &val.charset_encoding))
	    goto failure;
    }

    if (*sh = GCreateShFont(mask, &val, named ? name : NULL)) {
	if (GSharedObject::readParams(*sh, gostream))
	    return 1;
	else {
	    (*sh)->destroy();
	    *sh = NULL;
	}
    }
    
  failure:
    GWarning("GShFont restore failure");
    return 0;
}

/**/
int GShFont::readParams(GShFont*, GoStream*) {
    return 0;
}

/**/
GShFont* _newGShFont (GFunctionList* funcList, ulong mask,
		  const void * values, const char* name)
{
    return new GShFont (funcList, mask, (const GFontValues*)values, name);
}

/**/
GShFont::GShFont (GFunctionList* funcList, ulong mask, const GFontValues* values,
		const char* name) : GSharedObject(funcList, mask, name)
{
    int len;
    
#define setValue(maskBit, field)\
    if (mask & maskBit)\
    {\
        _values.field = new char[(len=strlen(values->field))+1];\
	_copyField (_values.field, values->field, len);\
        mask &= ~maskBit;\
    } else _values.field = GFONTNAME_UNDEFINED_FIELD;

    setValue (GFONT_REGISTRY, registry);
    setValue (GFONT_FAMILY, family);
    setValue (GFONT_WEIGHT, weight);
    setValue (GFONT_SLANT, slant);
    setValue (GFONT_WIDTH, width);
    setValue (GFONT_ADD_STYLE, add_style);
    setValue (GFONT_SPACING, spacing);
    setValue (GFONT_CHARSET, charset_registry);
    setValue (GFONT_CHARCODE, charset_encoding);
}

GShFont::~GShFont ()
{
#define deleteValue(maskBit, field)\
    if (!_mask) return;\
    if (_mask & maskBit)\
    {\
        delete _values.field;\
        _mask &= ~maskBit;\
    }

    deleteValue (GFONT_REGISTRY, registry);
    deleteValue (GFONT_FAMILY, family);
    deleteValue (GFONT_WEIGHT, weight);
    deleteValue (GFONT_SLANT, slant);
    deleteValue (GFONT_WIDTH, width);
    deleteValue (GFONT_ADD_STYLE, add_style);
    deleteValue (GFONT_SPACING, spacing);
    deleteValue (GFONT_CHARSET, charset_registry);
    deleteValue (GFONT_CHARCODE, charset_encoding);
#undef deleteValue
}


/**/
GBool GShFont::_match (ulong mask, const void * __values) const
{
const GFontValues* values = (const GFontValues* )__values;

    ulong inter_mask = mask & _mask;
    if (!inter_mask)
        return GTRUE;

#define _matchValue(maskBit, field) \
    if (!inter_mask) return GTRUE;\
    if (inter_mask & maskBit)\
    {\
        inter_mask &= ~maskBit;\
        if (!_sameField(_values.field, values->field))\
	    return GFALSE;\
    }

    _matchValue (GFONT_REGISTRY, registry);
    _matchValue (GFONT_FAMILY, family);
    _matchValue (GFONT_WEIGHT, weight);
    _matchValue (GFONT_SLANT, slant);
    _matchValue (GFONT_WIDTH, width);
    _matchValue (GFONT_ADD_STYLE, add_style);
    _matchValue (GFONT_SPACING, spacing);
    _matchValue (GFONT_CHARSET, charset_registry);
    _matchValue (GFONT_CHARCODE, charset_encoding);
#undef _matchValue

    return GTRUE;
}

/**/
void GShFont::_makeNewValues (ulong mask, const void * __values,
			      ulong& newMask, void * __newValues) const
{
const GFontValues* values = (const GFontValues* )__values;
GFontValues* newValues = (GFontValues* )__newValues;

    ulong orMask = mask | _mask;
    newMask = orMask;

#define _makeNewValue(maskBit, field)\
    if (!orMask) return;\
    newValues->field = (mask & maskBit ?\
			values->field : _values.field);\
    orMask &= ~maskBit

    _makeNewValue (GFONT_REGISTRY, registry);
    _makeNewValue (GFONT_FAMILY, family);
    _makeNewValue (GFONT_WEIGHT, weight);
    _makeNewValue (GFONT_SLANT, slant);
    _makeNewValue (GFONT_WIDTH, width);
    _makeNewValue (GFONT_ADD_STYLE, add_style);
    _makeNewValue (GFONT_SPACING, spacing);
    _makeNewValue (GFONT_CHARSET, charset_registry);
    _makeNewValue (GFONT_CHARCODE, charset_encoding);
#undef _makeNewValue
}

/**/
ulong GShFont::_update (ulong mask, const void * __values)
{
const GFontValues* values = (const GFontValues*) __values;

    ulong old_mask = _mask;
    ulong update_mask = 0;
    _mask |= mask;

#define _updateValue(maskBit, field)\
    if (!mask) return update_mask;\
    if (mask & maskBit)\
    {\
        if (old_mask & maskBit)\
	    delete _values.field;\
        _values.field = new char[strlen(values->field)+1];\
	strcpy (_values.field, values->field);\
	update_mask |= maskBit;\
	mask &= ~maskBit;\
    }

    _updateValue (GFONT_REGISTRY, registry);
    _updateValue (GFONT_FAMILY, family);
    _updateValue (GFONT_WEIGHT, weight);
    _updateValue (GFONT_SLANT, slant);
    _updateValue (GFONT_WIDTH, width);
    _updateValue (GFONT_ADD_STYLE, add_style);
    _updateValue (GFONT_SPACING, spacing);
    _updateValue (GFONT_CHARSET, charset_registry);
    _updateValue (GFONT_CHARCODE, charset_encoding);
#undef _updateValue
    return update_mask;	 
}

/*
 * Public Methods
 */

/* static */
const char* GShFont::classname()
{
    return "GShFont";
}

/* virtual */
const char* GShFont::getClassname() const
{
    return GShFont::classname();
}

/**/
void GModify (const char* name, ulong mask, const GFontValues* values)
{
    GMODIFY_NAMED_SHAREDOBJECT (GShFont, mask, values, name);
}

static GShFont* default_shfont = NULL;
/**/
GShFont* GCreateShFont (ulong mask, const GFontValues* values, const char* name)
{
    if (mask == 0 && name == NULL)
    {
	if (!default_shfont)
	{
	    GFontValues _values;
	    ulong mask;
	    _values.family = "helvetica";
	    _values.weight = "medium";
	    _values.slant = "r";
	    _values.width = "normal";
	    _values.spacing = "p";
	    _values.charset_registry = "iso8859";
	    _values.charset_encoding = "1";
	    mask = GFONT_FAMILY | GFONT_WEIGHT | GFONT_SLANT | GFONT_WIDTH |
		GFONT_SPACING | GFONT_CHARSET | GFONT_CHARCODE;
	    default_shfont = GCREATE_SHAREDOBJECT(GShFont, mask, &_values, NULL);
	}
	return default_shfont->dup();
    }
    return GCREATE_SHAREDOBJECT(GShFont, mask, values, name);
}

/**/
GShFont* GCreateShFont (const char* name)
{
    return GCreateShFont (0, NULL, name);
}

/**/
GShFont* GCreateShFont ()
{
    return GCreateShFont (0, NULL, NULL);
}

/**
FUNCTION
    fullName
DESCRIPTION
    Returns the full name, if a field is not defined it is replaced by
    the undified character, or by the default string given by the
    GoInterf
**/
char* GShFont::fullName () const
{
#define _size_full_name\
    strlen(_values.registry) +\
    strlen(_values.family) +\
    strlen(_values.weight) +\
    strlen(_values.slant) +\
    strlen(_values.width) +\
    strlen(_values.add_style) +\
    strlen(_values.spacing) +\
    strlen(_values.charset_registry) +\
    strlen(_values.charset_encoding) + 12

    char* full_name = new char[_size_full_name];
    strcat(strcat(strcat(strcat(strcat(strcat(strcat(strcat(strcat(strcat(
      strcat(strcat(strcat(strcat(strcat(strcat(strcat(strcat(
		strcpy(full_name, GFONTNAME_SEPARATOR),
		       GFONTNAME_SEPARATOR),
		      _values.family), GFONTNAME_SEPARATOR),
		      _values.weight), GFONTNAME_SEPARATOR),
	              _values.slant), GFONTNAME_SEPARATOR),
	              _values.width), GFONTNAME_SEPARATOR),
	              _values.add_style), GFONTNAME_SEPARATOR),
                      _values.spacing), GFONTNAME_SEPARATOR),
	              GFONTNAME_SEPARATOR),
                      _values.charset_registry), GFONTNAME_SEPARATOR),
	              _values.charset_encoding), GFONTNAME_SEPARATOR);
    return full_name;
}

