#ifdef REV_INFO
static char rcsid[] = "$Id: ImgBrush.cc,v 1.2 1992/09/25 14:30:41 smadja Exp $";
#endif
/*****************************************************************************
MODULE:  GoGC
FILE:    ImgBrush.cc
AUTHOR:  Jacques Davy
DATE:    November 1990

DESCRIPTION:
    (see <Go/ImageBrush.h>

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#include <Go/Tools/ImageBrush.h>
#include <Go/Tools/Colormap.h>
#include <Go/Tools/Pattern.h>
#include <assert.h>

/*
 * Callback types
 */
static char ibrushNmodifyPrecond[1];
static char ibrushNmodifyPostcond[1];
static char ibrushNdestroyPostcond[1];

typedef GBool (*PeProc)(void*, const GImageBrush*, ulong&, GImageBrushValues*);
typedef void (*PsProc)(void*, GImageBrush*, ulong);
typedef void (*PdProc)(void*, const GImageBrush*);

struct PrecondRec
{
    PeProc proc;
    void* closure;
};

struct ModifPostcondRec
{
    PsProc proc;
    void* closure;
};

struct DestroyPostcondRec
{
    PdProc proc;
    void* closure;
};

/*
 * Private methods
 * ---------------
 */
/**
FUNCTION
    _call_call_modify_postcond
DESCRIPTION
    This function is called when the NAMED GShImageBrush is updated
**/
void _call_call_modify_postcond (void* This, GShImageBrush* sh_ImageBrush,
				 ulong update_mask)
{
    Call_call_modify_postcond ((GImageBrush*)This, sh_ImageBrush, update_mask);
}

void Call_call_modify_postcond (GImageBrush* This,
				const GShImageBrush* sh_Imagebrush,
				ulong update_mask)
{
    assert (sh_Imagebrush == This->shImageBrush());
    This->_call_modify_postcond (update_mask);
}

/*
 * Protected methods
 * -----------------
 */
/**/
int GImageBrush::_addCallback (const char* type, GCBProc proc, const void* closure) const
{
    const GAssoc* _cb = &_assoc;
    GAssoc* cb = (GAssoc*)_cb;
    
    GCallbackRec cbrec (proc, closure);
    return cb->add(type, sizeof cbrec, (char*)&cbrec);
}

/**/
int GImageBrush::_remCallback (const char* type, GCBProc proc, const void* closure) const
{
    const GAssoc* _cb = &_assoc;
    GAssoc* cb = (GAssoc*)_cb;

    GCallbackRec cbrec (proc, closure);
    return cb->rem(type, sizeof cbrec, (char*)&cbrec);
}

/**/
GBool GImageBrush::_call_modify_precond (ulong& modify_mask, GImageBrushValues* new_values)
{
    if (_flags._has_modify_precond)
    {
	int count;
	PrecondRec* cbrec = (PrecondRec*)_assoc.get (ibrushNmodifyPrecond, count);
	for (int i = 0; i < count; i++, cbrec++)
	    if (!cbrec->proc(cbrec->closure, this, modify_mask, new_values))
                return GFALSE;
    }
    return GTRUE;
}

/**/
void GImageBrush::_call_modify_postcond (ulong modify_mask)
{
    if (_flags._has_modify_postcond)
    {
	int count;
	GCallbackRec* cbrec = (GCallbackRec*)_assoc.get (ibrushNmodifyPostcond, count);
	(void)GCallCallbacks(cbrec, count, this, modify_mask);
    }
}

/**/
void GImageBrush::_call_destroy_postcond ()
{
    if (_flags._has_destroy_postcond)
    {
	int count;
	DestroyPostcondRec* cbrec = (DestroyPostcondRec*)_assoc.get (ibrushNdestroyPostcond, count);
	for (int i = 0; i < count; i++, cbrec++)
	     cbrec->proc(cbrec->closure, this);
    }
}

/*
 * Public methods
 * --------------
 */
GImageBrush::GImageBrush ()
{
    _sh_ibrush = GCreateShImageBrush();  /* the default one */
}

GImageBrush::GImageBrush (const GImageBrush* ibrush)
{
    _sh_ibrush = ibrush->_sh_ibrush->dup();
}
GImageBrush::GImageBrush (const GShImageBrush* sh_ibrush)
{
    _sh_ibrush = sh_ibrush->dup();
}

GImageBrush::GImageBrush (ulong mask, const GImageBrushValues* values, const char* name)
{
    _sh_ibrush = GCreateShImageBrush (mask, values, name);
}

GImageBrush::GImageBrush (const char* name)
{
    _sh_ibrush = GCreateShImageBrush (name);
}

GImageBrush::~GImageBrush ()
{
    _call_destroy_postcond ();
    if (_flags._has_modify_postcond)
        _sh_ibrush->updatePostcond (_call_call_modify_postcond, this, GREMOVE);
    _sh_ibrush->destroy();
}

/**
FUNCTION
    GImageBrush::modify
DESCRIPTION
    Modify the shared ibrush and call pre/post condition.
**/

void GImageBrush::modify (ulong mask, const GImageBrushValues* values)
{
    GImageBrushValues Values = *values;
    if (mask && _call_modify_precond (mask, &Values))
    {
	// Remove postcond;
	if (_flags._has_modify_postcond)
	    _sh_ibrush->updatePostcond (_call_call_modify_postcond, this, GREMOVE);

	// Modify;
        GModify (_sh_ibrush, mask, &Values);

	// Call postcond;
	_call_modify_postcond (mask);

	// Add postcond;
	if (_flags._has_modify_postcond)
	    _sh_ibrush->updatePostcond (_call_call_modify_postcond, this, GADD);

    }
}

void GImageBrush::modify (const char* name)
{
    if (name && _sh_ibrush->name() && (strcmp (name, _sh_ibrush->name()) == 0))
        return;

    // Remove postcond.
    if (_flags._has_modify_postcond)
        _sh_ibrush->updatePostcond (_call_call_modify_postcond, this, GREMOVE);

    // Get mask and values
    GShImageBrush* save = _sh_ibrush->dup();

    GRename (save, name);

    GImageBrushValues Values = *save->values();
    ulong mask = save->mask();

    if (mask && _call_modify_precond (mask, &Values))
    {
         _sh_ibrush->destroy();
         _sh_ibrush = GCreateShImageBrush (mask, &Values, name);
         _call_modify_postcond (mask);
    }
    save->destroy();

    // Add postcond.
    if (_flags._has_modify_postcond)
	_sh_ibrush->updatePostcond (_call_call_modify_postcond, this, GADD);
}

void GImageBrush::modify (const GShImageBrush* sh_ibrush)
{
    if (sh_ibrush == _sh_ibrush)
        return;

    // top of list;
    sh_ibrush->raise(GShImageBrush::_headOfList);
    
    if (sh_ibrush->isNamed())
	modify(sh_ibrush->name());
    else 
    {
	GImageBrushValues Values = *sh_ibrush->values();
	ulong mask = sh_ibrush->mask();
	    
	// Remove postcond;
	if (_flags._has_modify_postcond)
	    _sh_ibrush->updatePostcond (_call_call_modify_postcond, this, GREMOVE);

	// Rename to NULL;
	GRename(_sh_ibrush, NULL);
		
	if (mask && _call_modify_precond (mask, &Values)) 
	{
	    // Modify;
	    GModify (_sh_ibrush, mask, &Values);

	    // Call postcond;
	    _call_modify_postcond (mask);
	}

	// Add postcond;
	if (_flags._has_modify_postcond)
	    _sh_ibrush->updatePostcond (_call_call_modify_postcond, this, GADD);
    }
}

/**
FUNCTION
    GImageBrush::field_modification_methods
DESCRIPTION
    Each method modify one parameter of the GImageBrush.
**/

#define _modifyImageBrushValue(bit_mask, field, value) \
    GImageBrushValues values;\
    values.field = value;\
    modify (bit_mask, &values)

void GImageBrush::function (ulong function)
{
    _modifyImageBrushValue(GFUNCTION, function, function);
}

void GImageBrush::planeMask (ulong plane_mask)
{
    _modifyImageBrushValue(GPLANE_MASK, plane_mask, plane_mask);
}

void GImageBrush::fillStyle (ushort fill_style)
{
    _modifyImageBrushValue(GFILL_STYLE, fill.style, fill_style);
}

void GImageBrush::colormap (const GColormap* colormap)
{
    _modifyImageBrushValue(GCOLORMAP, colormap, (GShColormap*)colormap->shColormap());
}

void GImageBrush::colormap (const GShColormap* colormap)
{
    _modifyImageBrushValue(GCOLORMAP, colormap, (GShColormap*)colormap);
}

void GImageBrush::colormap (const char* name)
{
    GImageBrushValues values;
    values.colormap = GCreateShColormap(name);
    modify (GCOLORMAP, &values);
    values.colormap->destroy();
}


void GImageBrush::stipple (const GPattern* stipple)
{
    _modifyImageBrushValue(GSTIPPLE, stipple, (GShPattern*)stipple->shPattern());
}

void GImageBrush::stipple (const GShPattern* stipple)
{
    _modifyImageBrushValue(GSTIPPLE, stipple, (GShPattern*)stipple);
}

void GImageBrush::stipple (const char* name)
{
    GImageBrushValues values;
    values.stipple = GCreateShPattern(name);
    modify (GSTIPPLE, &values);
    values.stipple->destroy();
}
#undef _modifyImageBrushValue

/**
FUNCTION
    GImageBrush::add/rem pre/postconditions
DESCRIPTION
    Add or remove pre/post condition on modification and on destruction.

    GWarning  _call_call_modify_postcond
    is stored directly in the shared object and pre/condition
    may be not called!
**/

#define _add_rem_callback(type, flag) \
    (FLAGS->flag =\
         0 < ((add == GADD)\
	      ? _addCallback(type, (GCBProc)proc, closure)\
	      : _remCallback(type, (GCBProc)proc, closure)))

/**/
void GImageBrush::modifyPrecond (PeProc proc,
				const void* closure, GAddOrRemove add) const
{
    const GImageBrushCondFlags* flags = &_flags;
    GImageBrushCondFlags* FLAGS = (GImageBrushCondFlags*)flags;
    
   _add_rem_callback(ibrushNmodifyPrecond, _has_modify_precond);
}

/**/
void GImageBrush::modifyPostcond (PsProc proc, const void* closure, GAddOrRemove add) const
{
    ushort flag = _flags._has_modify_postcond;

    const GImageBrushCondFlags* flags = &_flags;
    GImageBrushCondFlags* FLAGS = (GImageBrushCondFlags*)flags;
    
    _add_rem_callback(ibrushNmodifyPostcond, _has_modify_postcond);

    // The modifyPostcond is called from the  sh_imagebrush.

    if (!flag && _flags._has_modify_postcond)
	_sh_ibrush->updatePostcond (_call_call_modify_postcond, this, GADD);
    else
	if (flag && !_flags._has_modify_postcond)
	    _sh_ibrush->updatePostcond (_call_call_modify_postcond, this, GREMOVE);

}

/**/
void GImageBrush::destroyPostcond (PdProc proc, const void* closure, GAddOrRemove add) const
{
    const GImageBrushCondFlags* flags = &_flags;
    GImageBrushCondFlags* FLAGS = (GImageBrushCondFlags*)flags;
    
   _add_rem_callback(ibrushNdestroyPostcond, _has_destroy_postcond);
}

#undef _add_rem_callback


