#ifdef REV_INFO
static char rcsid[] = "$Id: Pen.cc,v 1.2 1992/09/25 14:30:34 smadja Exp $";
#endif
/*****************************************************************************
MODULE:  GoGC
FILE:    Pen.cc
AUTHOR:  Jacques Davy
DATE:    November 1990

DESCRIPTION:
    (see <Go/Tools/Pen.h>

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#include <Go/Tools/Pen.h>
#include <Go/Tools/Color.h>
#include <Go/Tools/Dash.h>
#include <Go/Tools/Pattern.h>
#include <assert.h>

/*
 * Callback types
 */
static char penNmodifyPrecond[1];
static char penNmodifyPostcond[1];
static char penNdestroyPostcond[1];

typedef GBool (*PeProc)(void*, const GPen*, ulong&, GPenValues*);
typedef void (*PsProc)(void*, GPen*, ulong);
typedef void (*PdProc)(void*, const GPen*);

struct PrecondRec
{
    PeProc proc;
    void* closure;
};

struct ModifPostcondRec
{
    PsProc proc;
    void* closure;
};

struct DestroyPostcondRec
{
    PdProc proc;
    void* closure;
};

/*
 * Private methods
 * ---------------
 */
/**
FUNCTION
    _call_call_modify_postcond
DESCRIPTION
    This function is called when the GShPen is updated
**/
void _call_call_modify_postcond (void* This, GShPen* sh_pen,
				 ulong update_mask)
{
    Call_call_modify_postcond ((GPen*)This, sh_pen, update_mask);
}

void Call_call_modify_postcond (GPen* This, const GShPen* sh_pen,
				ulong update_mask)
{
    assert (sh_pen == This->shPen());
    This->_call_modify_postcond (update_mask);
}


/*
 * Protected methods
 * -----------------
 */
/**/
int GPen::_addCallback (const char* type, GCBProc proc, const void* closure) const
{
    const GAssoc* _cb = &_assoc;
    GAssoc* cb = (GAssoc*)_cb;
    
    GCallbackRec cbrec (proc, closure);
    return cb->add(type, sizeof cbrec, (char*)&cbrec);
}

/**/
int GPen::_remCallback (const char* type, GCBProc proc, const void* closure) const
{
    const GAssoc* _cb = &_assoc;
    GAssoc* cb = (GAssoc*)_cb;

    GCallbackRec cbrec (proc, closure);
    return cb->rem(type, sizeof cbrec, (char*)&cbrec);
}

/**/
GBool GPen::_call_modify_precond (ulong& modify_mask, GPenValues* new_values)
{
    if (_flags._has_modify_precond)
    {
	int count;
	PrecondRec* cbrec = (PrecondRec*)_assoc.get (penNmodifyPrecond, count);
	for (int i = 0; i < count; i++, cbrec++)
	    if (!cbrec->proc(cbrec->closure, this, modify_mask, new_values))
                return GFALSE;
    }
    return GTRUE;
}

/**/
void GPen::_call_modify_postcond (ulong modify_mask)
{
    if (_flags._has_modify_postcond)
    {
	int count;
	GCallbackRec* cbrec = (GCallbackRec*)_assoc.get (penNmodifyPostcond, count);
	(void)GCallCallbacks(cbrec, count, this, modify_mask);
    }
}

/**/
void GPen::_call_destroy_postcond ()
{
    if (_flags._has_destroy_postcond)
    {
	int count;
	DestroyPostcondRec* cbrec = (DestroyPostcondRec*)_assoc.get (penNdestroyPostcond, count);
	for (int i = 0; i < count; i++, cbrec++)
	     cbrec->proc(cbrec->closure, this);
    }
}


/*
 * Public methods
 * --------------
 */
GPen::GPen ()
{
    _sh_pen = GCreateShPen();  /* default one */
}
    
GPen::GPen (const GPen* pen)
{
    _sh_pen = pen->_sh_pen->dup();
}

GPen::GPen (const GShPen* sh_pen)
{
    _sh_pen = sh_pen->dup();
}

GPen::GPen (ulong mask, const GPenValues* values, const char* name)
{
    _sh_pen = GCreateShPen (mask, values, name);
}

GPen::GPen (const char* name)
{
    _sh_pen = GCreateShPen (name);
}

GPen::~GPen ()
{
    _call_destroy_postcond ();
    
    if (_flags._has_modify_postcond)
        _sh_pen->updatePostcond (_call_call_modify_postcond, this, GREMOVE);
    _sh_pen->destroy();
}

/**
FUNCTION
    GPen::modify
DESCRIPTION
    Modify the shared pen and call pre/post condition.
**/

void GPen::modify (ulong mask, const GPenValues* values)
{
    GPenValues Values = *values;
    if (mask && _call_modify_precond (mask, &Values))
    {
	// Remove postcond;
	if (_flags._has_modify_postcond)
	    _sh_pen->updatePostcond (_call_call_modify_postcond, this, GREMOVE);

	// Modify;
        GModify (_sh_pen, mask, &Values);

	// Call postcond;
	_call_modify_postcond (mask);

	// Add postcond;
	if (_flags._has_modify_postcond)
	    _sh_pen->updatePostcond (_call_call_modify_postcond, this, GADD);
    }
}


void GPen::modify (const char* name)
{
    if (name && _sh_pen->name() && (strcmp (name, _sh_pen->name()) == 0))
        return;

    // Remove postcond;
    if (_flags._has_modify_postcond)
        _sh_pen->updatePostcond (_call_call_modify_postcond, this, GREMOVE);

    // Get mask and values
    GShPen* save = _sh_pen->dup();

    GRename (save, name);

    GPenValues Values = *save->values();
    ulong mask = save->mask();

    if (mask && _call_modify_precond (mask, &Values))
    {
         _sh_pen->destroy();
         _sh_pen = GCreateShPen (mask, &Values, name);
         _call_modify_postcond (mask);
    }
    save->destroy();

    if (_flags._has_modify_postcond)
	_sh_pen->updatePostcond (_call_call_modify_postcond, this, GADD);
}

void GPen::modify (const GShPen* sh_pen)
{
    if (sh_pen == _sh_pen)
        return;

    // top of list;
    sh_pen->raise(GShPen::_headOfList);
    
    if (sh_pen->isNamed())
	modify(sh_pen->name());
    else 
    {
	GPenValues Values = *sh_pen->values();
	ulong mask = sh_pen->mask();
	    
	// Remove postcond;
	if (_flags._has_modify_postcond)
	    _sh_pen->updatePostcond (_call_call_modify_postcond, this, GREMOVE);

	// Rename to NULL;
	GRename(_sh_pen, NULL);
		
	if (mask && _call_modify_precond (mask, &Values)) 
	{
	    // Modify;
	    GModify (_sh_pen, mask, &Values);

	    // Call postcond;
	    _call_modify_postcond (mask);
	}

	// Add postcond;
	if (_flags._has_modify_postcond)
	    _sh_pen->updatePostcond (_call_call_modify_postcond, this, GADD);
    }
}	    

/**
FUNCTION
    GPen::field_modification_methods
DESCRIPTION
    Each method modify one parameter of the GPen.
**/

#define _modifyPenValue(bit_mask, field, value) \
    GPenValues values;\
    values.field = value;\
    modify (bit_mask, &values)

void GPen::function (ulong function)
{
    _modifyPenValue(GFUNCTION, function, function);
}

void GPen::planeMask (ulong plane_mask)
{
    _modifyPenValue(GPLANE_MASK, plane_mask, plane_mask);
}

/**/
void GPen::color (const GColor* color)
{
    _modifyPenValue(GCOLOR, color, (GShColor*)color->shColor());
}

void GPen::color (const GShColor* color)
{
    _modifyPenValue(GCOLOR, color, (GShColor*)color);
}

void GPen::color (const char* name)
{
    GPenValues values;
    values.color = GCreateShColor (name);
    modify (GCOLOR, &values);
    values.color->destroy();
}

/**/
void GPen::altColor (const GColor* color)
{
    _modifyPenValue(GALT_COLOR, alt_color, (GShColor*)color->shColor());
}

void GPen::altColor (const GShColor* color)
{
    _modifyPenValue(GALT_COLOR, alt_color, (GShColor*)color);
}

void GPen::altColor (const char* name)
{
    GPenValues values;
    values.alt_color = GCreateShColor (name);
    modify (GALT_COLOR, &values);
    values.alt_color->destroy();
}

void GPen::lineWidth (GNumber line_width)
{
    _modifyPenValue(GLINE_WIDTH, line_width, line_width);
}

void GPen::lineStyle (ushort line_style)
{
    _modifyPenValue(GLINE_STYLE, line.style, line_style);
}

void GPen::capStyle (ushort cap_style)
{
    _modifyPenValue(GCAP_STYLE, line.cap, cap_style);
}

void GPen::joinStyle (ushort join_style)
{
    _modifyPenValue(GJOIN_STYLE, line.join, join_style);
}

void GPen::miterLimit (GNumber miter_limit)
{
    _modifyPenValue(GMITER_LIMIT, miter_limit, miter_limit);
}

void GPen::dash (const GDash* dash)
{
    _modifyPenValue(GDASH, dash, (GShDash*)dash->shDash());
}

void GPen::dash (const GShDash* dash)
{
    _modifyPenValue(GDASH, dash, (GShDash*)dash);
}

void GPen::dash (const char* name)
{
    GPenValues values;
    values.dash = GCreateShDash (name);
    modify (GDASH, &values);
    values.dash->destroy();
}

/**/
void GPen::fillStyle (ushort fill_style)
{
    _modifyPenValue(GFILL_STYLE, fill.style, fill_style);
}

void GPen::tile (const GPattern* tile)
{
    _modifyPenValue(GTILE, tile, (GShPattern*)tile->shPattern());
}

void GPen::tile (const GShPattern* tile)
{
    _modifyPenValue(GTILE, tile, (GShPattern*)tile);
}

void GPen::tile (const char* name)
{
    GPenValues values;
    values.tile = GCreateShPattern(name);
    modify (GTILE, &values);
    values.tile->destroy();
}

void GPen::stipple (const GPattern* stipple)
{
    _modifyPenValue(GSTIPPLE, stipple, (GShPattern*)stipple->shPattern());
}

void GPen::stipple (const GShPattern* stipple)
{
    _modifyPenValue(GSTIPPLE, stipple, (GShPattern*)stipple);
}

void GPen::stipple (const char* name)
{
    GPenValues values;
    values.stipple = GCreateShPattern(name);
    modify (GSTIPPLE, &values);
    values.stipple->destroy();
}

void GPen::tsOrigin (GNumber x_origin, GNumber y_origin)
{
    GPenValues values;
    values.ts_x_origin = x_origin;
    values.ts_y_origin = y_origin;
    modify (GTILE_STIP_XORIGIN | GTILE_STIP_YORIGIN, &values);
}
#undef _modifyPenValue

/**
FUNCTION
    GPen::add/rem pre/postconditions
DESCRIPTION
    Add or remove pre/post condition on modification and on destruction.

    GWarning, if the GShPen is named, _call_call_modify_postcond
    is stored directly in the shared object and pre/condition
    may be not called!
**/

#define _add_rem_callback(type, flag) \
    (FLAGS->flag =\
         0 < ((add == GADD)\
	      ? _addCallback(type, (GCBProc)proc, closure)\
	      : _remCallback(type, (GCBProc)proc, closure)))

/**/
void GPen::modifyPrecond (PeProc proc,
			 const void* closure, GAddOrRemove add) const
{
    const GPenCondFlags* flags = &_flags;
    GPenCondFlags* FLAGS = (GPenCondFlags*)flags;
    
   _add_rem_callback(penNmodifyPrecond, _has_modify_precond);
}

/**/
void GPen::modifyPostcond (PsProc proc, const void* closure, GAddOrRemove add) const
{
    ushort flag = _flags._has_modify_postcond;

    const GPenCondFlags* flags = &_flags;
    GPenCondFlags* FLAGS = (GPenCondFlags*)flags;

    _add_rem_callback(penNmodifyPostcond, _has_modify_postcond);

    /* The modifyPostcond is called from the  _sh_pen */

    if (!flag && _flags._has_modify_postcond)
	_sh_pen->updatePostcond (_call_call_modify_postcond, this, GADD);
    else
	if (flag && !_flags._has_modify_postcond)
	    _sh_pen->updatePostcond (_call_call_modify_postcond, this, GREMOVE);
}

/**/
void GPen::destroyPostcond (PdProc proc, const void* closure, GAddOrRemove add) const
{
    const GPenCondFlags* flags = &_flags;
    GPenCondFlags* FLAGS = (GPenCondFlags*)flags;

   _add_rem_callback(penNdestroyPostcond, _has_destroy_postcond);
}

#undef _add_rem_callback
