#ifdef REV_INFO
static char rcsid[] = "$Id: ShBrush.cc,v 1.3 1992/09/25 14:26:24 smadja Exp $";
#endif
/*****************************************************************************
MODULE:  GoGC
FILE:    ShBrush.cc
AUTHOR:  Jacques Davy
DATE:    October 1990

DESCRIPTION:
    (See <Go/ShBrush.h>

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  license from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#include <Go/Tools/Error.h>
#include <Go/Tools/ShBrush.h>
#include <Go/Tools/ShColor.h>
#include <Go/Tools/ShPattern.h>
#include <Go/GoStream.h>
#include <assert.h>

/*
 * static of the class
 */
GList*        GShBrush::_headOfList;
GFunctionList GShBrush::_funcList;

/*
 * Default values for ShBrush constructor
 */

#define DEFAULT_FUNCTION    GCOPY
#define DEFAULT_PLANE_MASK  GALL_PLANES
#define DEFAULT_COLOR       GCreateShColor()
#define DEFAULT_ALT_COLOR   NULL

#define DEFAULT_FILL_STYLE  GFILL_SOLID
#define DEFAULT_FILL_RULE   GEVEN_ODD_RULE
#define DEFAULT_TILE        NULL
#define DEFAULT_STIPPLE     NULL
#define DEFAULT_TS_X_ORIGIN 0.0
#define DEFAULT_TS_Y_ORIGIN 0.0

#define DEFAULT_MASK GFUNCTION   | \
	             GPLANE_MASK  | \
	             GCOLOR      | \
	             GFILL_STYLE  | \
	             GFILL_RULE

/*
 * static post conds
 */
static void _ColorIsUpdated(void* This, GShColor* color, ulong mask)
{
    ColorIsUpdated((GShBrush*)This, color, mask);
}

void ColorIsUpdated(GShBrush* This, const GShColor* color, ulong mask)
{
   const GBrushValues* values = This->values();
   if (color == values->color)
       mask = GCOLOR;
   else if (color == values->alt_color)
       mask = GALT_COLOR;
   else 
       GError("%s %s %s", "GShBrush.cc", "ColorIsUpdated",
	     "values and parameter incoherence");

   assert (color->isNamed());
   This->_call_update_postcond(mask);
}

static   void _PatternIsUpdated(void* This, GShPattern* pattern,
				     ulong mask)
{
    PatternIsUpdated((GShBrush*)This, pattern, mask);
}

void PatternIsUpdated(GShBrush* This, const GShPattern* pattern,
		      ulong mask)
{
    const GBrushValues* values = This->values();
    if (pattern == values->tile)
	mask = GTILE;
    else if (pattern == values->stipple)
	mask = GSTIPPLE;
    else 
	GError("%s %s %s", "GShBrush.cc", "PatternIsUpdated",
	       "values and parameter incoherence");
    
    This->_call_update_postcond(mask);
}


/*
 * Protected Methods
 */

int GShBrush::save(const GShBrush* sh, GoStream* gostream)
{
    const GBrushValues* val = sh->values();
    unsigned long mask = sh->mask();
    GBool named = sh->isNamed();
    
    if (!gostream->writeBool(named, "is named"))
	goto failure;
    if (named && !gostream->writeString(sh->name(), "name"))
	goto failure;
    if (!gostream->writeLong(sh->mask(), "mask"))
	goto failure;
    if ((mask & GFUNCTION) &&
	! gostream->writeLong(val->function, "function"))
	goto failure;
    if ((mask & GPLANE_MASK) &&
	! gostream->writeLong(val->plane_mask, "plane mask"))
	goto failure;
    if ((mask & GCOLOR) &&
	! gostream->saveObj(val->color, val->color->getClassname(), "color"))
	goto failure;
    if ((mask & GALT_COLOR) &&
	! gostream->saveObj(val->alt_color,
			    val->alt_color->getClassname(), "alt color"))
	goto failure;
    if ((mask & GFILL_STYLE) &&
	! gostream->writeChar((char) val->fill.style, "fill style"))
	goto failure;
    if ((mask & GFILL_RULE) &&
	! gostream->writeChar((char) val->fill.rule, "fill rule"))
	goto failure;
    if ((mask & GTILE) &&
	! gostream->saveObj(val->tile, val->tile->getClassname(), "tile"))
	goto failure;
    if ((mask & GSTIPPLE) &&
	! gostream->saveObj(val->stipple, val->stipple->getClassname(), "stipple"))
	goto failure;
    if ((mask & GTILE_STIP_XORIGIN) &&
	! gostream->writeNumber(val->ts_x_origin, "ts x origin"))
	goto failure;
    if ((mask & GTILE_STIP_YORIGIN) &&
	! gostream->writeNumber(val->ts_y_origin, "ts y origin"))
	goto failure;
    return GSharedObject::save(sh, gostream);
    
  failure:
    GWarning("GShBrush save failure");
    return 0;
}

/**/
int GShBrush::restore(GShBrush** sh, GoStream* gostream) {
    long mask;
    GBrushValues val;
    char c, str[256];
    char *name = str;
    const char* class_name;
    GBool named;
    
    if (!gostream->readBool(&named))
	goto failure;
    if (named && !gostream->readString(256, &name))
	goto failure;

    if (!gostream->readLong(&mask))
	goto failure;
    if ((mask & GFUNCTION) && ! gostream->readLong((long*) &val.function))
	goto failure;
    if ((mask & GPLANE_MASK) && ! gostream->readLong((long*) &val.plane_mask))
	goto failure;
    if ((mask & GCOLOR) &&
	! gostream->restoreObj((void**) &(val.color), &class_name))
	goto failure;
    if ((mask & GALT_COLOR) &&
	! gostream->restoreObj((void**) &(val.alt_color), &class_name))
	goto failure;
    if (mask & GFILL_STYLE)
    {
	if (! gostream->readChar(&c))
	    goto failure;
	else val.fill.style = c;
    }
    if (mask & GFILL_RULE)
    {
	if (! gostream->readChar(&c))
	    goto failure;
	else val.fill.rule = c;
    }
    if ((mask & GTILE) &&
	! gostream->restoreObj((void**) &(val.tile), &class_name))
	goto failure;
    if ((mask & GSTIPPLE) &&
	! gostream->restoreObj((void**) &(val.stipple), &class_name))
	goto failure;
    if ((mask & GTILE_STIP_XORIGIN) && ! gostream->readNumber(&val.ts_x_origin))
	goto failure;
    if ((mask & GTILE_STIP_YORIGIN) && ! gostream->readNumber(&val.ts_y_origin))
	goto failure;

    if (*sh = GCreateShBrush(mask, &val, named ? name : NULL))
    {
	if (GSharedObject::readParams(*sh, gostream))
	    return 1;
	else
	{
	    (*sh)->destroy();
	    *sh = NULL;
	}
    }
    
  failure:
    GWarning("GShBrush restore failure");
    return 0;
}

/**/
int GShBrush::readParams(GShBrush*, GoStream*)
{
    return 0;
}

/**/
GShBrush* _newGShBrush (GFunctionList* funcList, ulong mask, const void * values,
			const char* name)
{
    return new GShBrush (funcList, mask, (const GBrushValues*)values, name);
}

/**/
GShBrush::GShBrush (GFunctionList* funcList, ulong mask,
		  const GBrushValues* values, const char* name)
    : GSharedObject(funcList, mask, name)
{
#define _setValue(maskBit, field, default_value) \
    _values.field = (mask & maskBit ? \
		     values->field : default_value)
#define _setSharedValue(maskBit, field, default_value) \
    _values.field = (mask & maskBit ? \
		     values->field->dup() : default_value)

    _setValue (GFUNCTION, function, DEFAULT_FUNCTION);
    _setValue (GPLANE_MASK, plane_mask, DEFAULT_PLANE_MASK);
    _setSharedValue (GCOLOR, color, DEFAULT_COLOR);
    if (_values.color->isNamed())
	_values.color->updatePostcond(_ColorIsUpdated, this, GADD);

    _setSharedValue (GALT_COLOR, alt_color, DEFAULT_ALT_COLOR);
    if (_values.alt_color && _values.alt_color->isNamed())
	_values.alt_color->updatePostcond(_ColorIsUpdated, this, GADD);

    _setValue (GFILL_STYLE, fill.style, DEFAULT_FILL_STYLE);
    _setValue (GFILL_RULE, fill.rule, DEFAULT_FILL_RULE);
    _setSharedValue (GTILE, tile, DEFAULT_TILE);
    if (_values.tile)
	_values.tile->updatePostcond(_PatternIsUpdated, this, GADD);

    _setSharedValue (GSTIPPLE, stipple, DEFAULT_STIPPLE);
    if (_values.stipple)
	_values.stipple->updatePostcond(_PatternIsUpdated, this, GADD);
    _setValue (GTILE_STIP_XORIGIN, ts_x_origin, DEFAULT_TS_X_ORIGIN);
    _setValue (GTILE_STIP_YORIGIN, ts_y_origin, DEFAULT_TS_Y_ORIGIN);

    _mask = mask | DEFAULT_MASK;
#undef _setValue
#undef _setSharedValue
}

/**/
GShBrush::~GShBrush()
{
    if (_values.color->isNamed())
	_values.color->updatePostcond(_ColorIsUpdated, this, GREMOVE);
    _values.color->destroy();

    if (_mask & GALT_COLOR)
    {
	if (_values.alt_color->isNamed())
	    _values.alt_color->updatePostcond(_ColorIsUpdated, this, GREMOVE);
        _values.alt_color->destroy();
    }
    if (_mask & GTILE)
    {
	_values.tile->updatePostcond(_PatternIsUpdated, this, GREMOVE);
        _values.tile->destroy();
    }

    if (_mask & GSTIPPLE)
    {
	_values.stipple->updatePostcond(_PatternIsUpdated, this, GREMOVE);
	_values.stipple->destroy();
    }
}

/**/
GBool GShBrush::_match (ulong mask, const void * __values) const
{
const GBrushValues* values = (const GBrushValues* )__values;

    ulong inter_mask = mask & _mask;

    if (!inter_mask)
        return GTRUE;

#define _matchValue(maskBit, field) \
    if (!inter_mask) return GTRUE; \
    if (inter_mask & maskBit)\
    {\
	inter_mask &= ~maskBit ;\
	if ( values->field != _values.field ) return GFALSE ;\
    }

    _matchValue (GFUNCTION, function);
    _matchValue (GPLANE_MASK, plane_mask);
    _matchValue (GCOLOR, color);
    _matchValue (GFILL_STYLE, fill.style);
    _matchValue (GFILL_RULE, fill.rule);
    _matchValue (GTILE, tile);
    _matchValue (GSTIPPLE, stipple);
    _matchValue (GTILE_STIP_XORIGIN, ts_x_origin);
    _matchValue (GTILE_STIP_YORIGIN, ts_y_origin);
    _matchValue (GALT_COLOR, alt_color);
#undef _matchValue

    return GTRUE;
}

/**/
void GShBrush::_makeNewValues (ulong mask, const void * __values,
			      ulong& newMask, void * __newValues) const
{
const GBrushValues* values = (const GBrushValues* )__values;
GBrushValues* newValues = (GBrushValues* )__newValues;
    ulong orMask = mask | _mask;
    newMask = orMask;

#define _makeNewValue(maskBit, field) \
    if (!orMask) return;\
    newValues->field = (mask & maskBit ?\
			values->field : _values.field);\
    orMask &= ~maskBit

/* GSharedObject field are not "dup()ed" because it is not necessary */
    _makeNewValue (GFUNCTION, function);
    _makeNewValue (GPLANE_MASK, plane_mask);
    _makeNewValue (GCOLOR, color);
    _makeNewValue (GFILL_STYLE, fill.style);
    _makeNewValue (GFILL_RULE, fill.rule);
    _makeNewValue (GTILE, tile);
    _makeNewValue (GSTIPPLE, stipple);
    _makeNewValue (GTILE_STIP_XORIGIN, ts_x_origin);
    _makeNewValue (GTILE_STIP_YORIGIN, ts_y_origin);
    _makeNewValue (GALT_COLOR, alt_color);
#undef _makeNewValue
}

/**/
ulong GShBrush::_update (ulong mask, const void * __values)
{
const GBrushValues* values = (const GBrushValues* )__values;

    ulong old_mask = _mask;
    ulong update_mask = 0;
    _mask |= mask;

#define _updatePrivateValue(maskBit, field)\
    if (!mask) return update_mask;\
    if (mask & maskBit) {\
	if  (!(old_mask & maskBit) || (_values.field != values->field)){\
           update_mask |= maskBit;\
	   _values.field = values->field;\
        }\
        mask &= ~maskBit;\
    }

#define _updateColorSharedValue(maskBit, field)\
    if (!mask) return update_mask;\
    if (mask & maskBit) {\
	if (!(old_mask & maskBit) || (_values.field != values->field)){\
           update_mask |= maskBit;\
           if (old_mask & maskBit) {\
	       if (_values.field->isNamed())\
		   _values.field->updatePostcond(_ColorIsUpdated, this, GREMOVE);\
              _values.field->destroy();\
	      }\
	   _values.field = values->field->dup();\
           if (_values.field->isNamed())\
	       _values.field->updatePostcond(_ColorIsUpdated, this, GADD);\
        }\
        mask &= ~maskBit;\
    }

#define _updatePatternSharedValue(maskBit, field)\
    if (!mask) return update_mask;\
    if (mask & maskBit) {\
	if (!(old_mask & maskBit) || (_values.field != values->field)){\
           update_mask |= maskBit;\
           if (old_mask & maskBit) {\
              _values.field->updatePostcond(_PatternIsUpdated, this, GREMOVE);\
              _values.field->destroy();\
	      }\
	   _values.field = values->field->dup();\
           _values.field->updatePostcond(_PatternIsUpdated, this, GADD);\
        }\
        mask &= ~maskBit;\
    }


    _updateColorSharedValue (GCOLOR, color);
    _updateColorSharedValue (GALT_COLOR, alt_color);
    _updatePrivateValue (GFUNCTION, function);
    _updatePrivateValue (GFILL_STYLE, fill.style);
    _updatePrivateValue (GFILL_RULE, fill.rule);
    _updatePatternSharedValue (GTILE, tile);
    _updatePatternSharedValue (GSTIPPLE, stipple);
    _updatePrivateValue (GTILE_STIP_XORIGIN, ts_x_origin);
    _updatePrivateValue (GTILE_STIP_YORIGIN, ts_y_origin);
    _updatePrivateValue (GPLANE_MASK, plane_mask);

#undef _updatePrivateValue
#undef _updateColorSharedValue
#undef _updatePatternSharedValue
    return update_mask;
}

/*
 * Public Methods
 */

/* static */
const char* GShBrush::classname()
{
    return "GShBrush";
}

/* virtual */
const char* GShBrush::getClassname() const
{
    return GShBrush::classname();
}

/**/
void GModify (const char* name, ulong mask, const GBrushValues* values)
{
    GMODIFY_NAMED_SHAREDOBJECT (GShBrush, mask, values, name);
}

static GShBrush* default_shbrush = NULL;

/**/
GShBrush* GCreateShBrush (ulong mask, const GBrushValues* values, const char* name)
{
    if (mask == 0 && name == NULL)
    {
	if (default_shbrush == NULL)
	{
	    GBrushValues _values;
	    _values.function =    DEFAULT_FUNCTION;
	    _values.plane_mask =  DEFAULT_PLANE_MASK;
	    _values.color =       DEFAULT_COLOR;
	    _values.fill.style  = DEFAULT_FILL_STYLE;
	    _values.fill.rule  =  DEFAULT_FILL_RULE;

	    default_shbrush =  GCREATE_SHAREDOBJECT(GShBrush, DEFAULT_MASK,
						    &_values, NULL);
	    _values.color->destroy();
	}
	return default_shbrush->dup();
    }
    return GCREATE_SHAREDOBJECT (GShBrush, mask, values, name);
}

/**/
GShBrush* GCreateShBrush (const char* name)
{
    
    return GCreateShBrush(0, NULL, name);
}

/**/
GShBrush* GCreateShBrush ()
{
    return GCreateShBrush(0, NULL, NULL);
}


