#ifdef REV_INFO
static char rcsid[] = "$Id: ShImgBrush.cc,v 1.3 1992/09/25 14:26:29 smadja Exp $";
#endif
/*****************************************************************************
MODULE:  GoGC
FILE:    ShImgBrush.cc
AUTHOR:  Jacques Davy
DATE:    October 1990

DESCRIPTION:
    (See <Go/ShImageBrush.h>

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  license from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#include <Go/Tools/ShImageBrush.h>
#include <Go/Tools/ShColor.h>
#include <Go/Tools/ShColormap.h>
#include <Go/Tools/ShPattern.h>
#include <Go/Tools/Error.h>
#include <Go/GoStream.h>
#include <assert.h>


/*
 * static of the class
 */
GList*        GShImageBrush::_headOfList;
GFunctionList GShImageBrush::_funcList;

/*
 * Default values for ShImageBrush constructor
 */

#define DEFAULT_FUNCTION    GCOPY
#define DEFAULT_PLANE_MASK  GALL_PLANES
#define DEFAULT_FILL_STYLE  GFILL_SOLID
#define DEFAULT_COLORMAP    GCreateShColormap()
#define DEFAULT_STIPPLE     NULL

#define DEFAULT_MASK GFUNCTION|GPLANE_MASK|GFILL_STYLE|GCOLORMAP

static void _ColormapIsUpdated(void* This, GShColormap* colormap, ulong mask)
{
    ColormapIsUpdated((GShImageBrush*)This, colormap, mask);
}

void ColormapIsUpdated(GShImageBrush* This, const GShColormap* colormap,
		    ulong mask)
{
   assert(colormap == This->colormap() && mask);
   assert (colormap->isNamed());
   This->_call_update_postcond(GCOLORMAP);
}

static void _StippleIsUpdated(void* This, GShPattern* stipple, ulong mask)
{
    StippleIsUpdated((GShImageBrush*)This, stipple, mask);
}

void StippleIsUpdated(GShImageBrush* This, const GShPattern* stipple,
		      ulong mask)
{
   assert (stipple == This->stipple() && mask);
   This->_call_update_postcond(GSTIPPLE);
}


/*
 * Protected Methods
 */


int GShImageBrush::save(const GShImageBrush* sh, GoStream* gostream)
{
    const GImageBrushValues* val = sh->values();
    unsigned long mask = sh->mask();
    GBool named = sh->isNamed();
    
    if (!gostream->writeBool(named, "is named"))
	goto failure;
    if (named && !gostream->writeString(sh->name(), "name"))
	goto failure;
    if (!gostream->writeLong(sh->mask(), "mask"))
	goto failure;
    if ((mask & GFUNCTION) &&
	! gostream->writeLong(val->function, "function"))
	goto failure;
    if ((mask & GPLANE_MASK) &&
	! gostream->writeLong(val->plane_mask, "plane mask"))
	goto failure;
    if ((mask & GFILL_STYLE) &&
	! gostream->writeChar((char) val->fill.style, "fill style"))
	goto failure;
    if ((mask & GSTIPPLE) &&
	! gostream->saveObj(val->stipple, val->stipple->getClassname(), "stipple"))
	goto failure;
    if ((mask & GCOLORMAP) &&
	! gostream->saveObj(val->colormap, val->colormap->getClassname(), "color map"))
	goto failure;
    return GSharedObject::save(sh, gostream);
    
  failure:
    GWarning("save GShImageBrush failure");
    return 0;
}

/**/
int GShImageBrush::restore(GShImageBrush** sh, GoStream* gostream)
{
    long mask;
    GImageBrushValues val;
    char c, str[256];
    char *name = str;
    const char* class_name;
    GBool named;
    
    if (!gostream->readBool(&named))
	goto failure;
    if (named && !gostream->readString(256, &name))
	goto failure;

    if (!gostream->readLong(&mask))
	goto failure;
    if ((mask & GFUNCTION) && ! gostream->readLong((long*) &val.function))
	goto failure;
    if ((mask & GPLANE_MASK) && ! gostream->readLong((long*) &val.plane_mask))
	goto failure;
    if (mask & GFILL_STYLE)
    {
	if (! gostream->readChar(&c))
	    goto failure;
	else val.fill.style = c;
    }
    if ((mask & GSTIPPLE) &&
	! gostream->restoreObj((void**) &(val.stipple), &class_name))
	goto failure;
    if ((mask & GCOLORMAP) &&
	! gostream->restoreObj((void**) &(val.colormap), &class_name))
	goto failure;

    if (*sh = GCreateShImageBrush(mask, &val, named ? name : NULL))
    {
	if (GSharedObject::readParams(*sh, gostream))
	    return 1;
	else
	{
	    (*sh)->destroy();
	    *sh = NULL;
	}
    }
    
  failure:
    GWarning("GShImageBrush restore failure");
    return 0;
}

/**/
int GShImageBrush::readParams(GShImageBrush*, GoStream*)
{
    return 0;
}

/**/
GShImageBrush* _newGShImageBrush (GFunctionList* funcList, ulong mask,
				  const void * values,
				  const char* name)
{
    return new GShImageBrush (funcList, mask,(const GImageBrushValues*)values, name);
}

/**/
GShImageBrush::GShImageBrush (GFunctionList* funcList, ulong mask,
			      const GImageBrushValues* values, const char* name)
    : GSharedObject(funcList, mask, name)
{
#define _setValue(maskBit, field, default_value) \
    _values.field = (mask & maskBit ? \
		     values->field : default_value)
#define _setSharedValue(maskBit, field, default_value) \
    _values.field = (mask & maskBit ? \
		     values->field->dup() : default_value)

    _setValue (GFUNCTION, function, DEFAULT_FUNCTION);
    _setValue (GPLANE_MASK, plane_mask, DEFAULT_PLANE_MASK);
    _setValue (GFILL_STYLE, fill.style, DEFAULT_FILL_STYLE);

    _setSharedValue (GCOLORMAP, colormap, DEFAULT_COLORMAP);
    if (_values.colormap->isNamed())
	_values.colormap->updatePostcond(_ColormapIsUpdated, this, GADD);

    _setSharedValue (GSTIPPLE, stipple, DEFAULT_STIPPLE);
    if (_values.stipple)
	_values.stipple->updatePostcond(_StippleIsUpdated, this, GADD);

    _mask = mask;
#undef _setValue
#undef _setSharedValue
}

/**/
GShImageBrush::~GShImageBrush()
{
    if (_values.colormap->isNamed())
	_values.colormap->updatePostcond(_ColormapIsUpdated, this, GREMOVE);

    _values.colormap->destroy();

    if (_mask & GSTIPPLE)
    {
	_values.stipple->updatePostcond(_StippleIsUpdated, this, GREMOVE);
	_values.stipple->destroy();
    }

}

/**/
GBool GShImageBrush::_match (ulong mask, const void* __values) const
{
const GImageBrushValues* values = (const GImageBrushValues*)__values;

    ulong inter_mask = mask & _mask;

    if (!inter_mask)
        return GTRUE;

#define _matchValue(maskBit, field) \
    if (!inter_mask) return GTRUE; \
    if (inter_mask & maskBit)\
    {\
	inter_mask &= ~maskBit ;\
	if ( values->field != _values.field ) return GFALSE ;\
    }

    _matchValue (GFUNCTION, function);
    _matchValue (GPLANE_MASK, plane_mask);
    _matchValue (GFILL_STYLE, fill.style);
    _matchValue (GCOLORMAP, colormap);
    _matchValue (GSTIPPLE,  stipple);
#undef _matchValue

    return GTRUE;
}

/**/
void GShImageBrush::_makeNewValues (ulong mask, const void * __values,
				   ulong& newMask, void * __newValues) const
{
const GImageBrushValues* values = (const GImageBrushValues*)__values;
GImageBrushValues* newValues = (GImageBrushValues*)__newValues;

    ulong orMask = mask | _mask;
    newMask = orMask;

#define _makeNewValue(maskBit, field) \
    if (!orMask) return;\
    newValues->field = (mask & maskBit ?\
			values->field : _values.field);\
    orMask &= ~maskBit

/* GSharedObject field are not "dup()ed" because it is not necessary */

    _makeNewValue (GFUNCTION, function);
    _makeNewValue (GPLANE_MASK, plane_mask);
    _makeNewValue (GFILL_STYLE, fill.style);
    _makeNewValue (GCOLORMAP, colormap);
    _makeNewValue (GSTIPPLE, stipple);

#undef _makeNewValue
}

/**/
ulong GShImageBrush::_update (ulong mask, const void * __values)
{
const GImageBrushValues* values = (const GImageBrushValues*)__values;

    ulong old_mask = _mask;
    ulong update_mask = 0;
    _mask |= mask;

#define _updatePrivateValue(maskBit, field)\
    if (!mask) return update_mask;\
    if (mask & maskBit) {\
	if  (!(old_mask & maskBit) || (_values.field != values->field)){\
           update_mask |= maskBit;\
	   _values.field = values->field;\
        }\
        mask &= ~maskBit;\
    }

#define _updateColormapSharedValue(maskBit, field)\
    if (!mask) return update_mask;\
    if (mask & maskBit) {\
	if (!(old_mask & maskBit) || (_values.field != values->field)){\
           update_mask |= maskBit;\
           if (old_mask & maskBit) {\
	       if (_values.field->isNamed()) \
                   _values.field->updatePostcond(_ColormapIsUpdated, this, GREMOVE);\
              _values.field->destroy();\
	      }\
	   _values.field = values->field->dup();\
	   if (_values.field->isNamed()) \
               _values.field->updatePostcond(_ColormapIsUpdated, this, GADD);\
        }\
        mask &= ~maskBit;\
    }

#define _updateStippleSharedValue(maskBit, field)\
    if (!mask) return update_mask;\
    if (mask & maskBit) {\
	if (!(old_mask & maskBit) || (_values.field != values->field)){\
           update_mask |= maskBit;\
           if (old_mask & maskBit) {\
              _values.field->updatePostcond(_StippleIsUpdated, this, GREMOVE);\
              _values.field->destroy();\
	      }\
	   _values.field = values->field->dup();\
           _values.field->updatePostcond(_StippleIsUpdated, this, GADD);\
        }\
        mask &= ~maskBit;\
    }

    _updatePrivateValue (GPLANE_MASK, plane_mask);
    _updatePrivateValue (GFUNCTION, function);
    _updatePrivateValue (GFILL_STYLE, fill.style);
    _updateColormapSharedValue (GCOLORMAP, colormap);
    _updateStippleSharedValue (GSTIPPLE, stipple);


#undef _updatePrivateValue
#undef _updateColormapSharedValue
#undef _updateStippleSharedValue
    return update_mask;
}

/*
 * Public Methods
 */

/* static */
const char* GShImageBrush::classname()
{
    return "GShImageBrush";
}

/* virtual */
const char* GShImageBrush::getClassname() const
{
    return GShImageBrush::classname();
}

/**/
void GModify (const char* name, ulong mask, const GImageBrushValues* values)
{
    GMODIFY_NAMED_SHAREDOBJECT (GShImageBrush, mask, values, name);
}

static GShImageBrush* default_shimgbrush = NULL;

/**/
GShImageBrush* GCreateShImageBrush (ulong mask,
				  const GImageBrushValues* values, const char* name)
{
    if (mask == 0 && name == NULL)
    {
	if (default_shimgbrush == NULL)
	{
	    GImageBrushValues _values;
	    _values.function =   DEFAULT_FUNCTION;
	    _values.plane_mask = DEFAULT_PLANE_MASK;
	    _values.fill.style = DEFAULT_FILL_STYLE;
	    _values.colormap =   DEFAULT_COLORMAP;

	    default_shimgbrush =
		GCREATE_SHAREDOBJECT(GShImageBrush, DEFAULT_MASK, &_values, NULL);
	    _values.colormap->destroy();
	}
	return  default_shimgbrush->dup();
    }
    return GCREATE_SHAREDOBJECT (GShImageBrush, mask, values, name);
}

/**/
GShImageBrush* GCreateShImageBrush (const char* name)
{
    return GCreateShImageBrush(0, NULL, name);
}

/**/
GShImageBrush* GCreateShImageBrush ()
{
    return GCreateShImageBrush(0, NULL, NULL);
}
