#ifdef REV_INFO
static char rcsid[] = "$Id: TxtBrush.cc,v 1.2 1992/09/25 14:30:45 smadja Exp $";
#endif
/*****************************************************************************
MODULE:  GoGC
FILE:    TxtBrush.cc
AUTHOR:  Jacques Davy
DATE:    November 1990

DESCRIPTION:
    (see <Go/TextBrush.h>

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#include <Go/Tools/TextBrush.h>
#include <Go/Tools/Color.h>
#include <Go/Tools/Pattern.h>
#include <Go/Tools/Font.h>
#include <assert.h>

/*
 * Callback types
 */
static char tbrushNmodifyPrecond[1];
static char tbrushNmodifyPostcond[1];
static char tbrushNdestroyPostcond[1];

typedef GBool (*PeProc)(void*, const GTextBrush*, ulong&, GTextBrushValues*);
typedef void (*PsProc)(void*, GTextBrush*, ulong);
typedef void (*PdProc)(void*, const GTextBrush*);

struct PrecondRec
{
    PeProc proc;
    void* closure;
};

struct ModifPostcondRec
{
    PsProc proc;
    void* closure;
};

struct DestroyPostcondRec
{
    PdProc proc;
    void* closure;
};

/*
 * Private methods
 * ---------------
 */
/**
FUNCTION
    _call_call_modify_postcond
DESCRIPTION
    This function is called when the GShTextBrush is updated
**/
void _call_call_modify_postcond (void* This, GShTextBrush* sh_TextBrush,
				 ulong update_mask)
{
    Call_call_modify_postcond ((GTextBrush*)This, sh_TextBrush, update_mask);
}

void Call_call_modify_postcond (GTextBrush* This,
				const GShTextBrush* sh_textbrush,
				ulong update_mask)
{
    assert (sh_textbrush == This->shTextBrush());
    This->_call_modify_postcond (update_mask);
}


/*
 * Protected methods
 * -----------------
 */
/**/
int GTextBrush::_addCallback (const char* type, GCBProc proc, const void* closure) const
{
    const GAssoc* _cb = &_assoc;
    GAssoc* cb = (GAssoc*)_cb;
    
    GCallbackRec cbrec (proc, closure);
    return cb->add(type, sizeof cbrec, (char*)&cbrec);
}

/**/
int GTextBrush::_remCallback (const char* type, GCBProc proc, const void* closure) const
{
    const GAssoc* _cb = &_assoc;
    GAssoc* cb = (GAssoc*)_cb;

    GCallbackRec cbrec (proc, closure);
    return cb->rem(type, sizeof cbrec, (char*)&cbrec);
}

/**/
GBool GTextBrush::_call_modify_precond (ulong& modify_mask, GTextBrushValues* new_values)
{
    if (_flags._has_modify_precond)
    {
	int count;
	PrecondRec* cbrec = (PrecondRec*)_assoc.get (tbrushNmodifyPrecond, count);
	for (int i = 0; i < count; i++, cbrec++)
	    if (!cbrec->proc(cbrec->closure, this, modify_mask, new_values))
                return GFALSE;
    }
    return GTRUE;
}

/**/
void GTextBrush::_call_modify_postcond (ulong modify_mask)
{
    if (_flags._has_modify_postcond)
    {
	int count;
	GCallbackRec* cbrec = (GCallbackRec*)_assoc.get (tbrushNmodifyPostcond, count);
	(void)GCallCallbacks(cbrec, count, this, modify_mask);
    }
}

/**/
void GTextBrush::_call_destroy_postcond ()
{
    if (_flags._has_destroy_postcond)
    {
	int count;
	DestroyPostcondRec* cbrec = (DestroyPostcondRec*)_assoc.get (tbrushNdestroyPostcond, count);
	for (int i = 0; i < count; i++, cbrec++)
	     cbrec->proc(cbrec->closure, this);
    }
}

/*
 * Public methods
 * --------------
 */
GTextBrush::GTextBrush ()
{
    _sh_tbrush = GCreateShTextBrush(); /* the default one */
}

GTextBrush::GTextBrush (const GTextBrush* tbrush)
{
    _sh_tbrush = tbrush->_sh_tbrush->dup();
}
GTextBrush::GTextBrush (const GShTextBrush* sh_tbrush)
{
    _sh_tbrush = sh_tbrush->dup();
}

GTextBrush::GTextBrush (ulong mask, const GTextBrushValues* values, const char* name)
{
    _sh_tbrush = GCreateShTextBrush (mask, values, name);
}

GTextBrush::GTextBrush (const char* name)
{
    _sh_tbrush = GCreateShTextBrush (name);
}

GTextBrush::~GTextBrush ()
{
    _call_destroy_postcond ();
    if (_flags._has_modify_postcond)
        _sh_tbrush->updatePostcond (_call_call_modify_postcond, this, GREMOVE);
    _sh_tbrush->destroy();
}

/**
FUNCTION
    GTextBrush::modify
DESCRIPTION
    Modify the shared tbrush and call pre/post condition.
**/

void GTextBrush::modify (ulong mask, const GTextBrushValues* values)
{
    GTextBrushValues Values = *values;
    if (mask && _call_modify_precond (mask, &Values))
    {
	// Remove postcond;
	if (_flags._has_modify_postcond)
	    _sh_tbrush->updatePostcond (_call_call_modify_postcond, this, GREMOVE);

	// Modify;
        GModify(_sh_tbrush, mask, &Values);

	// Call postcond;
	_call_modify_postcond (mask);

	// Add postcond;
	if (_flags._has_modify_postcond)
	    _sh_tbrush->updatePostcond (_call_call_modify_postcond, this, GADD);
    }
}

void GTextBrush::modify (const char* name)
{
    if (name && _sh_tbrush->name() && (strcmp (name, _sh_tbrush->name()) == 0))
        return;

    // Remove postcond;
    if (_flags._has_modify_postcond)
        _sh_tbrush->updatePostcond (_call_call_modify_postcond, this, GREMOVE);

    // Get mask and values
    GShTextBrush* save = _sh_tbrush->dup();

    GRename (save, name);

    GTextBrushValues Values = *save->values();
    ulong mask = save->mask();

    if (mask && _call_modify_precond (mask, &Values))
    {
         _sh_tbrush->destroy();
         _sh_tbrush = GCreateShTextBrush (mask, &Values, name);
         _call_modify_postcond (mask);
    }
    save->destroy();
    
    // Add postcond.
    if (_flags._has_modify_postcond)
	_sh_tbrush->updatePostcond (_call_call_modify_postcond, this, GADD);
}

void GTextBrush::modify (const GShTextBrush* sh_tbrush)
{
    if (sh_tbrush == _sh_tbrush)
        return;

    // top of list;
    sh_tbrush->raise(GShTextBrush::_headOfList);
    
    if (sh_tbrush->isNamed())
	modify(sh_tbrush->name());
    else 
    {
	GTextBrushValues Values = *sh_tbrush->values();
	ulong mask = sh_tbrush->mask();
	    
	// Remove postcond;
	if (_flags._has_modify_postcond)
	    _sh_tbrush->updatePostcond (_call_call_modify_postcond, this, GREMOVE);

	// Rename to NULL;
	GRename(_sh_tbrush, NULL);
		
	if (mask && _call_modify_precond (mask, &Values)) 
	{
	    // Modify;
	    GModify (_sh_tbrush, mask, &Values);

	    // Call postcond;
	    _call_modify_postcond (mask);
	}

	// Add postcond;
	if (_flags._has_modify_postcond)
	    _sh_tbrush->updatePostcond (_call_call_modify_postcond, this, GADD);
    }
}

/**
FUNCTION
    GTextBrush::field_modification_methods
DESCRIPTION
    Each method modify one parameter of the GTextBrush.
**/

#define _modifyTextBrushValue(bit_mask, field, value) \
    GTextBrushValues values;\
    values.field = value;\
    modify (bit_mask, &values)

void GTextBrush::function (ulong function)
{
    _modifyTextBrushValue(GFUNCTION, function, function);
}

void GTextBrush::planeMask (ulong plane_mask)
{
    _modifyTextBrushValue(GPLANE_MASK, plane_mask, plane_mask);
}

/**/
void GTextBrush::color (const GColor* color)
{
    _modifyTextBrushValue(GCOLOR, color, (GShColor*)color->shColor());
}

void GTextBrush::color (const GShColor* color)
{
    _modifyTextBrushValue(GCOLOR, color, (GShColor*)color);
}

void GTextBrush::color (const char* name)
{
    GTextBrushValues values;
    values.color = GCreateShColor (name);
    modify (GCOLOR, &values);
    values.color->destroy();
}

/**/
void GTextBrush::altColor (const GColor* color)
{
    _modifyTextBrushValue(GALT_COLOR, alt_color, (GShColor*)color->shColor());
}

void GTextBrush::altColor (const GShColor* color)
{
    _modifyTextBrushValue(GALT_COLOR, alt_color, (GShColor*)color);
}

void GTextBrush::altColor (const char* name)
{
    GTextBrushValues values;
    values.alt_color = GCreateShColor (name);
    modify (GALT_COLOR, &values);
    values.alt_color->destroy();
}

/**/
void GTextBrush::fillStyle (ushort fill_style)
{
    _modifyTextBrushValue(GFILL_STYLE, fill.style, fill_style);
}

void GTextBrush::tile (const GPattern* tile)
{
    _modifyTextBrushValue(GTILE, tile, (GShPattern*)tile->shPattern());
}

void GTextBrush::tile (const GShPattern* tile)
{
    _modifyTextBrushValue(GTILE, tile, (GShPattern*)tile);
}

void GTextBrush::tile (const char* name)
{
    GTextBrushValues values;
    values.tile = GCreateShPattern(name);
    modify (GTILE, &values);
    values.tile->destroy();
}

void GTextBrush::stipple (const GPattern* stipple)
{
    _modifyTextBrushValue(GSTIPPLE, stipple, (GShPattern*)stipple->shPattern());
}

void GTextBrush::stipple (const GShPattern* stipple)
{
    _modifyTextBrushValue(GSTIPPLE, stipple, (GShPattern*)stipple);
}

void GTextBrush::stipple (const char* name)
{
    GTextBrushValues values;
    values.stipple = GCreateShPattern(name);
    modify (GSTIPPLE, &values);
    values.stipple->destroy();
}

void GTextBrush::tsOrigin (GNumber x_origin, GNumber y_origin)
{
    GTextBrushValues values;
    values.ts_x_origin = x_origin;
    values.ts_y_origin = y_origin;
    modify (GTILE_STIP_XORIGIN | GTILE_STIP_YORIGIN, &values);
}

/**/
void GTextBrush::font (const GFont* font)
{
    _modifyTextBrushValue(GFONT, font, (GShFont*)font->shFont());
}

void GTextBrush::font (const GShFont* font)
{
    _modifyTextBrushValue(GFONT, font, (GShFont*)font);
}

void GTextBrush::font (const char* name)
{
    GTextBrushValues values;
    values.font = GCreateShFont(name);
    modify (GFONT, &values);
    values.font->destroy();
}

#undef _modifyTextBrushValue

/**
FUNCTION
    GTextBrush::add/rem pre/postconditions
DESCRIPTION
    Add or remove pre/post condition on modification and on destruction.

    GWarning, _call_call_modify_postcond
    is stored directly in the shared object and pre/condition
    may be not called!
**/

#define _add_rem_callback(type, flag) \
    (FLAGS->flag =\
         0 < ((add == GADD)\
	      ? _addCallback(type, (GCBProc)proc, closure)\
	      : _remCallback(type, (GCBProc)proc, closure)))

/**/
void GTextBrush::modifyPrecond (PeProc proc,
			       const void* closure, GAddOrRemove add) const
{
    const GTextBrushCondFlags* flags = &_flags;
    GTextBrushCondFlags* FLAGS = (GTextBrushCondFlags*)flags;
    
   _add_rem_callback(tbrushNmodifyPrecond, _has_modify_precond);
}

/**/
void GTextBrush::modifyPostcond (PsProc proc, const void* closure, GAddOrRemove add) const
{
    ushort flag = _flags._has_modify_postcond;

    const GTextBrushCondFlags* flags = &_flags;
    GTextBrushCondFlags* FLAGS = (GTextBrushCondFlags*)flags;
    
    _add_rem_callback(tbrushNmodifyPostcond, _has_modify_postcond);

    /* The modifyPostcond is called from the  sh_tbrush */

    if (!flag && _flags._has_modify_postcond)
	_sh_tbrush->updatePostcond (_call_call_modify_postcond, this, GADD);
    else
	if (flag && !_flags._has_modify_postcond)
	    _sh_tbrush->updatePostcond (_call_call_modify_postcond, this, GREMOVE);

}

/**/
void GTextBrush::destroyPostcond (PdProc proc, const void* closure, GAddOrRemove add) const
{
    const GTextBrushCondFlags* flags = &_flags;
    GTextBrushCondFlags* FLAGS = (GTextBrushCondFlags*)flags;
    
   _add_rem_callback(tbrushNdestroyPostcond, _has_destroy_postcond);
}

#undef _add_rem_callback
