#ifdef REV_INFO
static char rcsid[] = "$Id: Matrix.cc,v 1.1.1.1 1992/03/18 10:47:02 uep Exp $";
#endif
/*****************************************************************************
MODULE:  Tools
FILE:    Matrix.cc
AUTHOR:  Olivier Jojic
DATE:    Jan 90

DESCRIPTION:
    See the Go/Tools/Matrix.h header.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
*****************************************************************************/

#include <stdio.h>
#include <Go/Tools/Matrix.h>
#include <Go/Tools/Error.h>

GMatrix GMatrix::operator ~ () const
{
    GNumber d = deter();
    if (d == 0)
    {
	d = 0.0001; //???
 	GWarning("GMatrix::operator ~ (): deter == 0\n");
    }

    return GMatrix(GVector(_vy.y / d, - _vx.y / d),
		  GVector(- _vy.x / d, _vx.x / d));
}


GMatrix operator * (const GMatrix& a, const GMatrix& b)
{
    return GMatrix(GVector(a._vx.x * b._vx.x + a._vy.x * b._vx.y,
			 a._vx.y * b._vx.x + a._vy.y * b._vx.y),
		  GVector(a._vx.x * b._vy.x + a._vy.x * b._vy.y,
			 a._vx.y * b._vy.x + a._vy.y * b._vy.y));
}


GVector operator * (const GMatrix& m, const GVector& v)
{
    return GVector(m._vx.x * v.x + m._vy.x * v.y,
		  m._vx.y * v.x + m._vy.y * v.y);
}


GVector& operator *= (GVector& v, const GMatrix& m)
{
    GNumber tmpx = m._vx.x * v.x + m._vy.x * v.y;
    v.y = m._vx.y * v.x + m._vy.y * v.y;
    v.x = tmpx;

    return v;
}


GMatrix GRotationMatrixFromVectors (const GVector& a, const GVector& b)
{
    GNumber cosang = (a * b) / sqrt(a.squareSize() * b.squareSize());
    GNumber sinang = sqrt(1 - cosang * cosang);

    if (a.x * b.y - a.y * b.x < 0)
	sinang = - sinang;

    return GMatrix(GVector(cosang, sinang), GVector(-sinang, cosang));
}
