#ifdef REV_INFO
static char rcsid[] = "$Id: FileMedium.cc,v 1.3 1992/11/25 16:17:55 kuta Exp $";
#endif
/******************************************************************************
MODULE: Tools
FILE:	FileMedium.cc
AUTHOR:	Philippe Smadja
DATE:   Jun 91

DESCRIPTION:
   The Medium CLASS provides an interface to read/write from/to "something"
   called here Medium.  The GFileMedium CLASS is subclass of Medium.
   It provides the input/output virtual methods and specific constructors
   
COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
 
#include <Go/Tools/FileMedium.h>
#include <Go/Tools/Error.h>
#include <errno.h>


/**
FUNCTION
    Medium destructor;
DESCRIPTION
    We need to define the Medium destructor which does nothing
    It is usefull because of virtual definition.
    Normaly  this definition should be in a Medium.cc file.
    Putting it in the FileMedium.cc file oblige the linker to
    link FileMedium.o which is not so big.
**/
GMedium::~GMedium() {
}

int GMedium::readuntil(char stop)
{
    int step;
    do
	step = mgetc();
    while (step != GEOM && step != stop);
    return step;
}


/**
FUNCTION
   Constructors(....)
DESCRIPTION
**/
GFileMedium::GFileMedium()
{
    _opened = GFALSE;
    _fileMustBeClosed  = GFALSE;
    _file = NULL;
}

GFileMedium::GFileMedium(FILE* file)
{
    _opened = GTRUE;
    _fileMustBeClosed  = GFALSE;
    _file = file;
}

int GFileMedium::_commonOpen(const char* filename, const char* type)
{
    _file =  fopen(filename, type);
    
    if (_file == NULL)  {
	_fileMustBeClosed  = GFALSE;
	_opened = GFALSE;
    } else {
	_fileMustBeClosed  = GTRUE;
	_opened = GTRUE;
    }

    return _file == NULL ? 1 : 0;
}

GFileMedium::GFileMedium(const char* filename, const char* type)
{
    (void)_commonOpen(filename, type);
}


/**

FUNCTION
    open a file
DESCRIPTION
    open filename according the open type cf fopen library for the string type.
**/
int GFileMedium::open(const char* filename, const char* type)
{
    return _commonOpen(filename, type);
}

/**
FUNCTION
    close the file
DESCRIPTION
    fclose is called: close a pipe will produce an error.
**/
void GFileMedium::close()
{
    if (_opened) {
	fclose(_file);
	_opened = GFALSE;
    }
}


/**
FUNCTION
    fflush the file
DESCRIPTION
    
**/
void  GFileMedium::flush()
{
    if (_opened)
	fflush(_file);
}

/**
FUNCTION
    Destructor
DESCRIPTION
    flushes the  output , close the file if it was opened the me.
**/
 GFileMedium::~GFileMedium()
{
    if (_fileMustBeClosed)
	close();
    else
	flush();
}

/**
    flush();
FUNCTION
    INPUT methods
DESCRIPTION
    Warning: methods does not check if _file is available.
**/

#if defined(__BORLANDC__) || defined (__MSC7__) 
char huge * hugeArray;
#endif

long GFileMedium::read(char* array, long size)
{
    long ret = 0;
    long SIZE = size;
#if defined( __BORLANDC__) || defined (__MSC7__) 
    hugeArray = array;
    while (size) {
	int sz = (int) min(size, 0x8000L);
	int n = fread(hugeArray, 1, sz, _file);
	ret += n;
	if (ret == SIZE || n != sz)
	    return ret;
	hugeArray += (((long)sz) & 0x0000ffffL);
	size -= (((long)sz) & 0x0000ffffL);

    }
#else
    while (size) {
	int sz = (int)size;
	int n = fread(array, 1, sz, _file);
	ret += n;
	if (ret == SIZE || n != sz)
	    return ret;
	array += sz;
	size -= sz;
    }
#endif
    return ret;
}

int GFileMedium::mgetc()
{
    int ret = getc(_file);
    return (ret == EOF ? GEOM : ret);
}

int GFileMedium::readuntil(char stop)
{
    int step;
    while ((step = getc(_file)) != EOF) {
	if (step == stop)
	    return step;
    }
    return GEOM;
}

char* GFileMedium::mgets(char* s, ulong max_size)
{
    return fgets(s, (int)max_size, _file);
}

/**
FUNCTION
    OUPUT methods
DESCRIPTION
    Warning: methods does not check if _file is available.
**/

long GFileMedium::write(const char* array, long size)
{
    long ret = 0;
    long SIZE = size;
#if defined( __BORLANDC__) || defined (__MSC7__) 
    hugeArray = (char*)array;
    while (size) {
	int sz = (int) min(size, 0x8000L);
	int n = fwrite(array, 1, sz, _file);
	ret += n;
	if (ret == SIZE || n != sz)
	    return ret;
	size -= (((long)sz) & 0x0000ffffL);
	hugeArray += (((long)sz) & 0x0000ffffL);
    }
#else
    while (size) {
	int sz = (int)size;
	int n = fwrite(array, 1, sz, _file);
	ret += n;
	if (ret == SIZE || n != sz)
	    return ret;
	size -= sz;
	array += sz;
    }
#endif
    return ret;
}

int GFileMedium::mputc(char c)
{
    int ret = putc((c == (char)GEOM ? (char)EOF : c), _file);
    return (ret == EOF ? GEOM : ret);  
}

int GFileMedium::mputback(char c)
{
    int ret = ungetc((c == (char)GEOM ? (char)EOF : c), _file);
    return (ret == EOF ? GEOM : ret);
}

int GFileMedium::mputs(const char* s)
{
    int ret = fputs(s, _file);
    return (ret == EOF ? GEOM : ret);  
}


/**
FUNCTION
    POSITION
DESCRIPTION
    Warning: methods does not check if _file is available.
    The SeekType enum have the same value has the C library.
**/

int GFileMedium::mseek(long offset, GMSeekType st)
{
    return fseek(_file, offset, (int)st);
}

long  GFileMedium::mtell()
{
    return ftell(_file);
}
