#ifdef REV_INFO
static char rcsid[] = "$Id: FilePath.cc,v 1.2 1992/09/25 14:35:28 smadja Exp $";
#endif
/*****************************************************************************
MODULE:  Tools
FILE:    FilePath.cc
AUTHOR:  Aline Jacomet
DATE:    November 21th 1991

DESCRIPTION:
    Function GetFilePath() for localizing environment files. First, it
    looks in the current working directory, then it takes in account
    the GoENVIRON Shell variable and then it looks in /usr/local/lib/Go
    as defined in Go/Tools/FilePath.h.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  license from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include <Go/machdef.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>
#include <stdlib.h>
#if defined(__BORLANDC__) || defined (__MSC7__)
#include <windows.h>
extern char* windll_getenv(const char *);
#define getenv windll_getenv
#endif 
#include <Go/Tools/Alloc.h>
#include <Go/Tools/Error.h>
#include <Go/Tools/FilePath.h>


/***
 * on dpx2-300 with ATT C++ compiler NULL is undefined after includes
 ***/
#ifndef NULL
#define NULL 0
#endif



char* GGetFilePath(const char* path, const char* name)
{
    if (!name) {
	GWarning("GGetFilePath: The file name is NULL"); 
	return NULL;
    }

    /*
     * Transform '/' or '\' in character used on the system.
     */
    char* p = NULL;
    char* n = (char*)GMALLOC(strlen(name) + 1);
    strcpy(n, name);
    for (int i = 0; n[i] != '\0'; i++)
	if ((n[i] == '/' && GSLASH[0] == '\\') ||
	    (n[i] == '\\' && GSLASH[0] == '/'))
	    n[i] = GSLASH[0];
    
    struct stat file_status;
    char* dir;
    
    /*
     * Looking for file: ./"name"
     */
    dir = (char*)GMALLOC(strlen(".") + strlen(GSLASH) + strlen(n) + 1);
    strcat(strcat(strcpy(dir, "."), GSLASH), n);
    if (! stat(dir, &file_status)) {
	GFREE(n);
	return dir;
    }
    GFREE(dir);

    if (path) {
	p = (char*)GMALLOC(strlen(path) + 1);
	strcpy(p, path);
	for (i = 0; p[i] != '\0'; i++)
	    if ((p[i] == '/' && GSLASH[0] == '\\') ||
		(p[i] == '\\' && GSLASH[0] == '/'))
		p[i] = GSLASH[0];
	
	/*
	 * Looking for file: "GoENVIRON"/"path"/"name"
	 */
	short add_first_GSLASH = 1;
	if (p[0] == GSLASH[0])
	    add_first_GSLASH = 0;
	
	short add_last_GSLASH = 1;
	if (p[strlen(p) - 1] == GSLASH[0])
	    add_last_GSLASH = 0;
	
	ushort free_path = 0;
	char* path_with_GSLASH;
	if (add_first_GSLASH || add_last_GSLASH) {
	    path_with_GSLASH = (char*)GMALLOC(strlen(p) + add_first_GSLASH +
					      add_last_GSLASH + 1);
	    free_path = 1;
	    if (add_first_GSLASH && add_last_GSLASH)
		strcat(strcat(strcpy(path_with_GSLASH, GSLASH), p), GSLASH);
	    else {
		if (add_first_GSLASH)
		    strcat(strcpy(path_with_GSLASH, GSLASH), p);
		else
		    strcat(strcpy(path_with_GSLASH, p), GSLASH);
	    }
	} else
	    path_with_GSLASH = p;
	
	char* go_path = (char*)getenv("GoENVIRON");

	if (go_path != NULL) {
	    if (go_path[strlen(go_path) - 1] == GSLASH[0])
		go_path[strlen(go_path) - 1] = '\0';
	    dir = (char*)GMALLOC(strlen(go_path) + strlen(path_with_GSLASH) +
				  strlen(n) + 1);
	    strcat(strcat(strcpy(dir, go_path), path_with_GSLASH), n);
	    if (! stat(dir, &file_status)) {
		if (free_path)
		    GFREE(path_with_GSLASH);
		GFREE(n);
		GFREE(p);
		return dir;
	    } else
		GFREE(dir);
	}
	
	
	/*
	 * Looking for file: /usr/local/lib/Go/"path"/"name"
	 */
	dir = (char*)GMALLOC(strlen("usrlocallibGo") +
			      strlen(path_with_GSLASH) + 4*strlen(GSLASH) +
			      strlen(n) + 1);
	strcat(strcat(strcpy(dir, GSLASH), "usr"), GSLASH);
	strcat(strcat(strcat(strcat(dir, "local"), GSLASH), "lib"), GSLASH);
	strcat(strcat(strcat(dir, "Go"), path_with_GSLASH), n);
	if (free_path)
	    GFREE(path_with_GSLASH);
	GFREE(n);
	GFREE(p);
	if (! stat(dir, &file_status))
	    return dir;

	GFREE(dir);
	return NULL;

    } else {
	/*
	 * Looking for file: "GoENVIRON"/"name"
	 */
	char* go_path = (char*)getenv("GoENVIRON");

	if (go_path != NULL) {
	    if (go_path[strlen(go_path) - 1] == GSLASH[0])
		go_path[strlen(go_path) - 1] = '\0';
	    dir = (char*)GMALLOC(strlen(go_path) + strlen(GSLASH) +
				  strlen(n) + 1);
	    strcat(strcat(strcpy(dir, go_path), GSLASH), n);
	    if (! stat(dir, &file_status)) {
		GFREE(n);
		return dir;
	    } else
		GFREE(dir);
	}
	
	
	/*
	 * Looking for file: /usr/local/lib/Go/"name"
	 */
	dir = (char*)GMALLOC(strlen("usrlocallibGo") + 5*strlen(GSLASH) +
			      strlen(n) + 1);
	strcat(strcat(strcpy(dir, GSLASH), "usr"), GSLASH);
	strcat(strcat(strcat(strcat(dir, "local"), GSLASH), "lib"), GSLASH);
	strcat(strcat(strcat(dir, "Go"), GSLASH), n);
	GFREE(n);
	if (! stat(dir, &file_status))
	    return dir;
	
	GFREE(dir);
	return NULL;
    }
}

#if defined( __BORLANDC__) || defined (__MSC7__)

static char go_env_var[64];

extern char ** environ;


char* windll_getenv(const char * env_name)
{
     char ** environTmp = environ;
     
     int i= 0; 
     while((*(environTmp +i) != NULL) || i > 32 ) {
	if(!strnicmp(*(environTmp+i) , env_name, strlen(env_name)-1))	
	     break;
	i++;
     }
     char * go_name_start1 = *(environTmp +i);

     if(go_name_start1 != NULL)	{
	     char * go_name_start2 = strstr(go_name_start1, "=");
	     if(go_name_start2 != NULL) {
		   strnset(go_env_var, '0', 64);
		   strcpy(go_env_var, go_name_start2 + 1);
		   return go_env_var;
	}
     }
     return NULL;		
}
			
#endif						 		
