#ifdef REV_INFO
static char rcsid[] = "$Id: ShPattern.cc,v 1.3 1992/09/25 14:50:59 smadja Exp $";
#endif
/*****************************************************************************
MODULE:  Tools
FILE:    ShPattern.cc
AUTHOR:  Jacques Davy
DATE:    August 1990

DESCRIPTION:
    See <Go/Tools/ShPattern.h>

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  license from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#include <Go/Tools/ShPattern.h>
#include <Go/Tools/Error.h>
#include <Go/GoStream.h>
#include <assert.h>

/*
 * static of the class
 */
GList*        GShPattern::_headOfList;
GFunctionList GShPattern::_funcList;

/*
 * static postcond
 */
static void _ColormapIsUpdated(void* This, GShColormap* colormap, ulong mask)
{
    ColormapIsUpdated((GShPattern*)This, colormap, mask);
}

void ColormapIsUpdated(GShPattern* This, const GShColormap* colormap, ulong mask)
{
   const GPatternValues* values = This->values();
   assert (colormap == values->colormap);
   assert (mask);
   This->_call_update_postcond(GPTRN_COLORMAP);
}

static   void _PixelArrayIsUpdated(void* This, GShPixelArray* pixelArray,
				     ulong mask)
{
    PixelArrayIsUpdated((GShPattern*)This, pixelArray, mask);
}

void PixelArrayIsUpdated(GShPattern* This, const GShPixelArray* pixelArray,
		      ulong mask)
{
    const GPatternValues* values = This->values();
    assert (pixelArray == values->pixelArray);
    assert (mask);
    This->_call_update_postcond(GPTRN_PIXEL_ARRAY);
}

/*
 * Protected Methods
 */

int GShPattern::save(const GShPattern* sh, GoStream* gostream) {
    const GPatternValues* val = sh->values();
    unsigned long mask = sh->mask();
    GBool named = sh->isNamed();
    
    if (!gostream->writeBool(named, "is named"))
	goto failure;
    if (named && !gostream->writeString(sh->name(), "name"))
	goto failure;
    if (!gostream->writeLong(sh->mask(), "mask"))
	goto failure;
    if ((mask & GPTRN_PIXEL_ARRAY) &&
	! gostream->saveObj(val->pixelArray,
			    val->pixelArray->getClassname(), "pixel array"))
	goto failure;
    if ((mask & GPTRN_COLORMAP) &&
	! gostream->saveObj(val->colormap,
			    val->colormap->getClassname(), "color map"))
	goto failure;
    if ((mask & GPTRN_WIDTH) &&
	! gostream->writeNumber(val->width, "width"))
	goto failure;
    if ((mask & GPTRN_HEIGHT) &&
	! gostream->writeNumber(val->height, "height"))
	goto failure;
    return GSharedObject::save(sh, gostream);
    
  failure:
    GWarning("save GShPattern failure");
    return 0;
}

/**/
int GShPattern::restore(GShPattern** sh, GoStream* gostream) {
    long mask;
    GPatternValues val;
    char str[256];
    char *name = str;
    const char* class_name;
    GBool named;
    
    if (!gostream->readBool(&named))
	goto failure;
    if (named && !gostream->readString(256, &name))
	goto failure;

    if (!gostream->readLong(&mask))
	goto failure;
    if ((mask & GPTRN_PIXEL_ARRAY) &&
	! gostream->restoreObj((void**) &(val.pixelArray), &class_name))
	goto failure;
    if ((mask & GPTRN_COLORMAP) &&
	! gostream->restoreObj((void**) &(val.colormap), &class_name))
	goto failure;
    if ((mask & GPTRN_WIDTH) && ! gostream->readNumber(&val.width))
	goto failure;
    if ((mask & GPTRN_HEIGHT) && ! gostream->readNumber(&val.height))
	goto failure;

    if (*sh = GCreateShPattern(mask, &val, named ? name : NULL)) {
	if (GSharedObject::readParams(*sh, gostream))
	    return 1;
	else {
	    (*sh)->destroy();
	    *sh = NULL;
	}
    }
    
  failure:
    GWarning("GShPattern restore failure");
    return 0;
}

/**/
int GShPattern::readParams(GShPattern*, GoStream*) {
    return 0;
}

/**/
GShPattern* _newGShPattern (GFunctionList* funcList, ulong mask,
			    const void * values, const char* name)
{
    return new GShPattern (funcList, mask, (const GPatternValues*)values, name);
}

/**/
GShPattern::GShPattern (GFunctionList* funcList, ulong mask,
		      const GPatternValues* values, const char* name)
    : GSharedObject(funcList, mask, name)
{
    _values.pixelArray = ( mask & GPTRN_PIXEL_ARRAY ? 
			   values->pixelArray->dup() : NULL );
    if (_values.pixelArray && _values.pixelArray->isNamed())
	_values.pixelArray->updatePostcond(_PixelArrayIsUpdated, this, GADD);

    _values.colormap   = ( mask & GPTRN_COLORMAP ?
			   values->colormap->dup() : NULL );
    if (_values.colormap)
	_values.colormap->updatePostcond(_ColormapIsUpdated, this, GADD);
    
    _values.height     = ( mask & GPTRN_HEIGHT ?
			   values->height : 0 );
    _values.width      = ( mask & GPTRN_WIDTH ?
			   values->width : 0 );
}

/**/
GShPattern::~GShPattern ()
{
    if (_values.pixelArray) {
	if (_values.pixelArray->isNamed())
	    _values.pixelArray->updatePostcond(_PixelArrayIsUpdated, this,
					       GREMOVE);
        _values.pixelArray->destroy();
    }
    if (_values.colormap) {
	_values.colormap->updatePostcond(_ColormapIsUpdated, this,
					  GREMOVE);
	_values.colormap->destroy();
    }
}

/**/
GBool GShPattern::_match (ulong mask, const void * __values) const
{
const GPatternValues* values = (const GPatternValues* )__values;

    ulong inter_mask = mask & _mask;

    if (!inter_mask)
	return GTRUE;

#define _matchValue(maskBit, field) \
    if (!inter_mask) return GTRUE; \
    if (inter_mask & maskBit)\
    {\
	inter_mask &= ~maskBit ;\
	if ( values->field != _values.field ) return GFALSE ;\
    }

    _matchValue (GPTRN_PIXEL_ARRAY, pixelArray);
    _matchValue (GPTRN_COLORMAP, colormap);
    _matchValue (GPTRN_HEIGHT, height);
    _matchValue (GPTRN_WIDTH, width);
#undef _matchValue

    return GTRUE;
}

/**/
void GShPattern::_makeNewValues (ulong mask, const void * __values,
				ulong& newMask, void * __newValues) const
{
const GPatternValues* values = (const GPatternValues* )__values;
GPatternValues* newValues = (GPatternValues* )__newValues;

    ulong orMask = mask | _mask;
    newMask = orMask;

#define _makeNewValue(maskBit, field) \
    if (!orMask) return;\
    newValues->field = (mask & maskBit ?\
			values->field : _values.field);\
    orMask &= ~maskBit

/* GSharedObject field are not "dup()ed" because it is not necessary */
    _makeNewValue (GPTRN_PIXEL_ARRAY, pixelArray);
    _makeNewValue (GPTRN_COLORMAP, colormap);
    _makeNewValue (GPTRN_WIDTH, width);
    _makeNewValue (GPTRN_HEIGHT, height);
#undef _makeNewValue
}

/**/
ulong GShPattern::_update (ulong mask, const void * __values)
{
const GPatternValues* values = (const GPatternValues* )__values;

    ulong old_mask = _mask;
    ulong update_mask = 0;
    _mask |= mask;

#define _updatePrivateValue(maskBit, field)\
    if (!mask) return update_mask;\
    if (mask & maskBit) {\
	if  (!(old_mask & maskBit) || (_values.field != values->field)){\
           update_mask |= maskBit;\
	   _values.field = values->field;\
        }\
        mask &= ~maskBit;\
    }

#define _updatePixelArraySharedValue(maskBit, field)\
    if (!mask) return update_mask;\
    if (mask & maskBit) {\
	if (!(old_mask & maskBit) || (_values.field != values->field)){\
           update_mask |= maskBit;\
           if (old_mask & maskBit) {\
	       if (_values.field->isNamed())\
		   _values.field->updatePostcond(_PixelArrayIsUpdated, this, GREMOVE);\
              _values.field->destroy();\
	      }\
	   _values.field = values->field->dup();\
           if (_values.field->isNamed())\
	       _values.field->updatePostcond(_PixelArrayIsUpdated, this, GADD);\
        }\
        mask &= ~maskBit;\
    }

#define _updateColormapSharedValue(maskBit, field)\
    if (!mask) return update_mask;\
    if (mask & maskBit) {\
	if (!(old_mask & maskBit) || (_values.field != values->field)){\
           update_mask |= maskBit;\
           if (old_mask & maskBit) {\
              _values.field->updatePostcond(_ColormapIsUpdated, this, GREMOVE);\
              _values.field->destroy();\
	      }\
	   _values.field = values->field->dup();\
           _values.field->updatePostcond(_ColormapIsUpdated, this, GADD);\
        }\
        mask &= ~maskBit;\
    }

    _updatePixelArraySharedValue (GPTRN_PIXEL_ARRAY, pixelArray);
    _updateColormapSharedValue (GPTRN_COLORMAP, colormap);
    _updatePrivateValue (GPTRN_HEIGHT, height);
    _updatePrivateValue (GPTRN_WIDTH, width);

#undef _updatePrivateValue
#undef _updatePixelArraySharedValue
#undef _updateColormapSharedValue
    return update_mask;
}

/*
 * Public Methods
 */

/* static */
const char* GShPattern::classname()
{
    return "GShPattern";
}

/* virtual */
const char* GShPattern::getClassname() const
{
    return GShPattern::classname();
}

/**/
void GModify (const char* name, ulong mask, const GPatternValues* values)
{
    GMODIFY_NAMED_SHAREDOBJECT (GShPattern, mask, values, name);
}

static GShPattern* default_shpattern = NULL;

/**/
GShPattern* GCreateShPattern (ulong mask, const GPatternValues* values, const char* name)
{
    if (mask == 0 && name == NULL)
    {
	if (! default_shpattern)
	{
	    GPatternValues _values;
	    _values.pixelArray = GCreateShPixelArray();
	    _values.colormap = GCreateShColormap();
	    _values.width = 0;
	    _values.height = 0;

	    default_shpattern =
		GCREATE_SHAREDOBJECT(GShPattern,
				     GPTRN_PIXEL_ARRAY|GPTRN_COLORMAP|
				     GPTRN_WIDTH|GPTRN_HEIGHT,
				     &_values, NULL);
	}
	return default_shpattern->dup();
    }
    return GCREATE_SHAREDOBJECT(GShPattern, mask, values, name);
}

/**/
GShPattern* GCreateShPattern (const GShPixelArray* pixelArray, const GShColormap* cmap,
			      GNumber width, GNumber height, const char* name)
{
    GPatternValues values;
    values.pixelArray = (GShPixelArray*)pixelArray;
    values.colormap = (GShColormap*)cmap;
    values.width = width;
    values.height = height;

    return GCREATE_SHAREDOBJECT(GShPattern,
				GPTRN_PIXEL_ARRAY|GPTRN_COLORMAP|GPTRN_WIDTH|GPTRN_HEIGHT,
				&values, name);
}

/**/
GShPattern* GCreateShPattern (const char* name)
{
    return GCreateShPattern (0, NULL, name);
}


/**/
GShPattern* GCreateShPattern ()
{
    return GCreateShPattern (0, NULL, NULL);
}
