#ifdef REV_INFO
static char rcsid[] = "$Id: ShPxlArray.cc,v 1.5 1992/12/10 13:33:57 jojic Exp $";
#endif
/*****************************************************************************
MODULE:  Tools
FILE:    ShPixelArray.cc
AUTHOR:  Jacques Davy
DATE:    August 1990

DESCRIPTION:
    See <Go/Tools/ShPixelArray.h>

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#include <Go/Tools/ShPixelArray.h>
#include <Go/Tools/Array.h>
#include <Go/Tools/Alloc.h>
#include <Go/Tools/Error.h>
#include <Go/GoStream.h>

/*
 * Private Methods
 */
#define BYTES_PER_LINE(v)\
    ( ((((v).width + (v).xoffset) * (v).bits_per_pixel) + (v).padding - 1) / (v).padding * (v).padding / 8 )

#if  defined (__BORLANDC__) || defined(__MSC7__) 
#define DATA_SIZE(v) (((ulong)_bytes_per_line) & 0x0ffffL) * (((ulong)(v).height) & 0x0ffffL)
#else
#define DATA_SIZE(v) _bytes_per_line * (v).height
#endif

/*
 * static of the class
 */
GList*        GShPixelArray::_headOfList;
GFunctionList GShPixelArray::_funcList;


/*
 * Protected Methods
 */

#if  defined (__BORLANDC__) || defined(__MSC7__) 
static int go_memcmp(const char huge * s1, const  char huge * s2, ulong size)
{
	if(s1 == NULL || s2 == NULL)
		return 1;
	for(ulong i =0; i < size; i++) 
	  if(s1++ != s2++ )
		return 1;
	return 0;
}
#endif




int GShPixelArray::save(const GShPixelArray* sh, GoStream* gostream) {
    const GPixelArrayValues* val = sh->values();
    unsigned long mask = sh->mask();
    GBool named = sh->isNamed();
    
    if (!gostream->writeBool(named, "is named"))
	goto failure;
    if (named && !gostream->writeString(sh->name(), "name"))
	goto failure;
    if (!gostream->writeLong(sh->mask(), "mask"))
	goto failure;
    if (mask & GPXL_VALUE) {
	if (! gostream->writeShort(val->width, "width"))
	    goto failure;
	if (! gostream->writeShort(val->height, "height"))
	    goto failure;
	if (! gostream->writeShort(val->xoffset, "xoffset"))
	    goto failure;
	if (! gostream->writeChar(val->bitmap_unit, "bitmap unit"))
	    goto failure;
	if (! gostream->writeChar(val->padding, "padding"))
	    goto failure;
	if (! gostream->writeChar(val->bits_per_pixel, "bits_per_pixel"))
	    goto failure;
	if (! gostream->writeChar(val->depth, "depth"))
	    goto failure;
	if (! gostream->writeChar(val->format.bit, "bit order"))
	    goto failure;
	if (! gostream->writeChar(val->format.byte, "byte order"))
	    goto failure;
	if (! gostream->writeChar(val->format.coding, "encoding format"))
	    goto failure;
	if (! gostream->writeBuffer(sh->dataSize(), (char*) val->data, "data"))
		 goto failure;
    }

    return GSharedObject::save(sh, gostream);
    
  failure:
    GWarning("GShPixelArray save failure");
    return 0;
}

/**/
int GShPixelArray::restore(GShPixelArray** sh, GoStream* gostream) {
    long mask;
    GPixelArrayValues val;
    char c, str[256];
    char *name = str;
    GBool named;

    val.data = NULL;
    
    if (!gostream->readBool(&named))
	goto failure;
    if (named && !gostream->readString(256, &name))
	goto failure;

    if (!gostream->readLong(&mask))
	goto failure;
    if (mask & GPXL_VALUE) {
	if (! gostream->readShort((short*) &val.width))
	    goto failure;
	if (! gostream->readShort((short*) &val.height))
	    goto failure;
	if (! gostream->readShort((short*) &val.xoffset))
	    goto failure;
	if (! gostream->readChar((char*) &val.bitmap_unit))
	    goto failure;
	if (! gostream->readChar((char*) &val.padding))
	    goto failure;
	if (! gostream->readChar((char*) &val.bits_per_pixel))
	    goto failure;
	if (! gostream->readChar((char*) &val.depth))
	    goto failure;
	if (! gostream->readChar(&c))
	    goto failure;
	else val.format.bit = c;
	if (! gostream->readChar(&c))
	    goto failure;
	else val.format.byte = c;
	if (! gostream->readChar(&c))
	    goto failure;
	else val.format.coding = c;
	val.data = NULL;
	ulong size;
	if (! gostream->readBuffer(&size, (char**) &val.data))
	    goto failure;
    }

    /**
      create a shared pixelarray
      donot copy the data. get only the  data pointer (val.data)
      **/
    if (*sh = GCreateShPixelArray(mask, &val, named ? name : NULL,  GDATA_GIFT)) {
	if ((*sh)->refcnt() > 1 && val.data != (*sh)->values()->data) {
	    /**
	      the pixelarray exists yet
	      so free the new data
	      **/
	    GFREE(val.data);
	    /* val.data = NULL done by GFREE; */
	}

	if (GSharedObject::readParams(*sh, gostream))
	    return 1;
	else {
	    (*sh)->destroy();
	    *sh = NULL;
	}
    }
    
  failure:
    GWarning("GShPixelArray restore failure");
    GFREE(val.data);
    return 0;
}

/**/
int GShPixelArray::readParams(GShPixelArray*, GoStream*) {
    return 0;
}

/**/
GShPixelArray* _newGShPixelArray (GFunctionList* funcList, ulong mask,
				const void * values, const char* name)
{
    return new GShPixelArray (funcList, mask, (const GPixelArrayValues *) values, name);
}

/**/
GShPixelArray::GShPixelArray (GFunctionList* funcList, ulong mask, const GPixelArrayValues* values,
			    const char* name) : GSharedObject(funcList, mask, name)
{
    if (mask & GPXL_VALUE)
    {
        _values = *values;
        _bytes_per_line = BYTES_PER_LINE (_values);

	if (!(mask & (GPXL_SHARED|GPXL_GIVEN)))
	{
	    _values.data = (uchar*) GMALLOC(DATA_SIZE(_values));
	    memcpy (_values.data, values->data, DATA_SIZE(_values));
	}
    }
}

/**/
GShPixelArray::~GShPixelArray ()
{
    if (!(_mask & GPXL_SHARED))
        GFREE  (_values.data);
}

/**/
GBool GShPixelArray::_match (ulong mask, const void * __values) const
{
const GPixelArrayValues* values = (const GPixelArrayValues*) __values;

    ulong inter_mask = mask & _mask;
    if (!inter_mask)
        return GTRUE;

    if (inter_mask & GPXL_VALUE)
    {
#ifdef __GNUC__   /* bug optimization **/
	uchar  coding = values->format.coding;
	uchar  byte  = values->format.byte;
	uchar  bit   = values->format.bit;
	uchar  bitmap_unit = values->bitmap_unit;
	uchar  padding     = values->padding;
	uchar  bits_per_pixel = values->bits_per_pixel;
	ushort width = values->width;
	ushort height = values->height;
	ushort xoffset = values->xoffset;
	uchar  depth = values->depth;
	
        if ((_values.width != width)  ||
	    (_values.height != height) ||
	    (_values.xoffset != xoffset) ||
	    (_values.bitmap_unit != bitmap_unit) ||
	    (_values.padding != padding) ||
	    (_values.bits_per_pixel != bits_per_pixel) ||
	    (_values.depth != depth) ||
	    (_values.format.bit != bit) ||
	    (_values.format.byte != byte) ||
	    (_values.format.coding != coding) ||
	    (memcmp (_values.data, values->data, DATA_SIZE(_values)) != 0))
	    return GFALSE;
#else
        if ((_values.width != values->width)  ||
	    (_values.height != values->height) ||
	    (_values.xoffset != values->xoffset) ||
	    (_values.bitmap_unit != values->bitmap_unit) ||
	    (_values.padding != values->padding) ||
	    (_values.bits_per_pixel != values->bits_per_pixel) ||
	    (_values.depth != values->depth) ||
	    (_values.format.bit != values->format.bit) ||
	    (_values.format.byte != values->format.byte) ||
	    (_values.format.coding != values->format.coding) ||
	    (memcmp (_values.data, values->data, DATA_SIZE(_values)) != 0))
	    return GFALSE;
#endif
    }

    return GTRUE;
}

/**/
void GShPixelArray::_makeNewValues (ulong mask, const void * __values,
				   ulong& newMask, void * __newValues)	const
{
const GPixelArrayValues* values = (const GPixelArrayValues*) __values;
GPixelArrayValues* newValues = (GPixelArrayValues*) __newValues;

    *newValues = (mask & GPXL_VALUE ? *values : _values);
    newMask = (mask & GPXL_VALUE ? mask : _mask);
}

/**/
ulong GShPixelArray::_update (ulong mask, const void * __values)
{
const GPixelArrayValues* values = (const GPixelArrayValues*) __values;

    ulong update_mask = 0;

    if (mask & GPXL_VALUE)
    {
        if (_mask & GPXL_VALUE && !(_mask & GPXL_SHARED))
	    GFREE(_values.data);
        _values = *values;
        _bytes_per_line = BYTES_PER_LINE(_values);
	// data
        if (!(mask & (GPXL_SHARED|GPXL_GIVEN)))
        {
	    _values.data = (uchar*) GMALLOC(DATA_SIZE(_values));
#if defined( __BORLANDC__) ||  defined(__MSC7__)
	   char huge * tmp1 = (char huge *) _values.data;
	   char huge * tmp2 = (char huge *) values->data;
	   for(long ii = 0; ii < DATA_SIZE(_values); ii++)
		*tmp1++ = *tmp2++;
#else
	 memcpy (_values.data, values->data, DATA_SIZE(_values));
#endif
	}
        update_mask |= GPXL_VALUE;
    }

    _mask |= mask;
    return update_mask;
}

/*
 * Public Methods
 */

/* static */
const char* GShPixelArray::classname()
{
    return "GShPixelArray";
}

/* virtual */
const char* GShPixelArray::getClassname() const
{
    return GShPixelArray::classname();
}

/**/
void GModify (const char* name, ulong mask, const GPixelArrayValues* values)
{
    GMODIFY_NAMED_SHAREDOBJECT (GShPixelArray, mask, values, name);
}


/**
FUNCTION
    GShPixelArray::subPixelArray
DESCRIPTION
    This method create a new GPixelArray from the current. The new one
    has NO NAME and its data are SHARED if the current are SHARED and
    COPIED else. It returns the new pixel array or NULL.

    If values are GNumbers, they are expressed in the Coordinate System
    ((0,0),(1,1)).
**/
GShPixelArray* GShPixelArray::subPixelArray (ushort x, ushort y,
					   ushort width, ushort height) const
{
    if (!(_mask & GPXL_VALUE))
        return NULL;

    if (x >= _values.width || y >= _values.height ||
	x + width > _values.width || y + height > _values.height)
        return NULL;

    if (x == 0 && y == 0 && width == _values.width && height == _values.height)
	return dup();

    GPixelArrayValues values;
    subPixelArrayValues (x, y, width, height, values);
    ulong mask = _mask;

    GShPixelArray* new_pixel_array = GCreateShPixelArray (mask, &values, 0);

    if (!(_mask & GPXL_SHARED))
        GFREE( values.data);
    return new_pixel_array;
}

/**/
GShPixelArray* GShPixelArray::subPixelArray (GNumber x, GNumber y,
					   GNumber width, GNumber height) const
{
    return subPixelArray ((ushort)((_values.width -1) * x),
			  (ushort)((_values.height - 1) * y),
			  (ushort)(_values.width * width),
			  (ushort)(_values.height * height));
}

/**
FUNCTION
    GShPixelArray::subPixelArrayValues
DESCRIPTION
    Initialize the GPixelArrayValues except for the name.
**/
void GShPixelArray::subPixelArrayValues (ushort x, ushort y, ushort width, ushort height, GPixelArrayValues& values) const
{
    ushort bit_offset = (_values.xoffset + x) * _values.bits_per_pixel;
    ushort byte_offset = bit_offset / 8;

    values = _values;
    values.width = width;
    values.height = height;
    values.xoffset = bit_offset - (byte_offset * 8);
    ushort bytes_per_line = BYTES_PER_LINE(values);
    values.data = (uchar*) GMALLOC(bytes_per_line * height);

    uchar* source = _values.data + _bytes_per_line * y +  byte_offset;
    uchar* dest = values.data;
    for (ushort i = y; i < height;
	 i++, source += _bytes_per_line + byte_offset, dest += bytes_per_line)
    {
        memcpy (dest, source, bytes_per_line);
    }
}

/**/
GShPixelArray* GCreateShPixelArray (ulong mask, const GPixelArrayValues* values,
				    const char* name, GDataSpace space)
{
    
    if (mask == 0 && values == NULL ) {
	GPixelArrayValues v ;
	bzero(&v , sizeof v);
	v.padding = 8;
	mask = GPXL_VALUE;
	values = &v;
    }
    
    switch (space) {
      case GDATA_GIFT:
	mask |= GPXL_GIVEN;
	break;
      case GDATA_SHARE:
	mask |= GPXL_SHARED;
	break;
      default:
	break;
    }
	
    return GCREATE_SHAREDOBJECT(GShPixelArray, mask, values, name);
}

/**/
GShPixelArray* GCreateShPixelArray (const char* name)
{
    return GCreateShPixelArray(0, NULL, name);
}

GShPixelArray* GCreateShPixelArray ()
{
    return GCreateShPixelArray(0, NULL, NULL);
}

/**
FUNCTION
    deformation()
DESCRIPTION
    Deformation of a pixelarray.
    The implementation is able to handle only 1 bit per pixel
    or 8 bits per pixel.
**/
void GShPixelArray::deformation (GPixelArrayValues* result,
			       ushort*  xOrigin_result, ushort* yOrigin_result,
			       const GCoordSys&        cs,
			       const GArray*           pixel_translation,
			       uchar                  bits_per_pixel_of_result,
			       uchar                  depth_of_result,
			       uchar   /* UNUSED      bit_order_of_result */,
			       uchar   /* UNUSED      byte_order_of_result */,
			       GQuality /* UNUSED      quality */) const
{
    *result = _values;
     result->depth = depth_of_result;
    *xOrigin_result = *yOrigin_result = 0;
			     
    if (_values.bits_per_pixel != 8  && _values.bits_per_pixel != 1) {
	GWarning("Sorry shPixelArray->deformation(): %s",
		"bits_per_pixel != 8,1 not implemented");
	return;
    }
        
    GBox box = cs.to(GBox(0,0,1,1));

    int  srcwidth = (int)_values.width;
    int  srcheight = (int)_values.height;
    int  src_xoffset = (int)_values.xoffset;
    int  dst_xoffset ;

    /**
      compute the new Data
      **/
    GCoordSys CS = GCoordSys(GPoint ((int)-box.xmin, (int)-box.ymin),
			   cs.I / srcwidth, cs.J / srcheight);
    CS.invert();
 
    GPoint src_origin(CS.to(GPoint(0,0)));
    src_origin.x += 0.5;
    src_origin.y += 0.5;      /** for int cast */

    GVector src_dx(CS.to(GVector(1,0)));
    GVector src_dy(CS.to(GVector(0,1)));

    int x,y;
#if defined( __BORLANDC__) || defined(__MSC7__)
    uchar huge * srcdata = _values.data;
#else
    uchar* srcdata = _values.data;
#endif
    int   srcbpl  = (int)bytesPerLine();
    int   width   = (int)box.width();
    int   height  = (int)box.height();
    int    y_begin = 0;
    GPoint  ps;
    GPoint  savePS;

    /**
      first compute the dst_xoffset
      **/
#define INVALID 32000
    ps = src_origin;
    dst_xoffset = INVALID;
    for (y=0; y < height; y++, ps = savePS + src_dy) {
	savePS = ps;
	for (x=0; x < width; x++, ps += src_dx) {
	    int ps_x = (int)ps.x;
	    if (ps_x >= 0 && ps_x <= srcwidth) {
		int ps_y = (int)ps.y;
		if (ps_y >=0 && ps_y <= srcheight)
		    if (dst_xoffset > x) {
			dst_xoffset = x;
			if (dst_xoffset == 0)
			    goto _COMPUTE_DATA;
			break; /* next line */
		    }
	    }
	}
	if (dst_xoffset == INVALID) 
	    /** the current line will not be initialize **/
	    y_begin++;
    }

    if (dst_xoffset == INVALID) {
	/**
	  the result is less than 1 square pixel
	  **/
	result->height = 0;
	result->data = NULL;
	return;
    }

#undef INVALID

    
    _COMPUTE_DATA:
    /**
      give to the caller the new origin
      inside the pixmap.
      **/
    *xOrigin_result = (ushort)-box.xmin;
    *yOrigin_result = (ushort)-box.ymin - y_begin;

    height -= y_begin;
    result->bitmap_unit = 8;

    if (_values.bits_per_pixel == 8) {
	/**
	  ZFORMAT on char
          bits_per_pixels == 8 <==> bitmap_unit = 8;
	  **/
	result->xoffset = dst_xoffset;
	result->width = width;
	result->height = height;
	result->padding = 8;
	result->bits_per_pixel = bits_per_pixel_of_result;

#if defined( __BORLANDC__)  ||  defined(__MSC7__)
        int dst_bytesPerLine = width + dst_xoffset;
        uchar huge * temp = (uchar*)GMALLOC(dst_bytesPerLine*result->height);
        result->data = (uchar*) temp;
        uchar huge * dstdata = temp + dst_xoffset;
#else
        int dst_bytesPerLine = BYTES_PER_LINE (*result);
        result->data = (uchar*)GMALLOC(dst_bytesPerLine*result->height);
        uchar* dstdata = result->data + dst_xoffset;
#endif
	ps = src_origin + src_dy*y_begin;
	
	if (pixel_translation)  {
	    ulong* pixels = (ulong*)pixel_translation->data();
	    switch (bits_per_pixel_of_result) {
	      default:
		GWarning("ShPxlArray: transformation from 8 bits to %d bits not implemented",
			bits_per_pixel_of_result);
		break;
	      case 8:
		for (y=0; y < height; y++,
		     dstdata = result->data + y * dst_bytesPerLine + dst_xoffset,
		     ps = savePS + src_dy) {
		
		    savePS = ps;
		    for (x=0; x < width; x++, ps += src_dx, dstdata++) {

			int ps_x = (int)ps.x;
			if (ps_x >=0 && ps_x <= srcwidth) {
			    int ps_y = (int)ps.y;
			    if (ps_y >=0 && ps_y <= srcheight) {
				if (ps_x == srcwidth) ps_x = srcwidth-1;
				if (ps_y == srcheight) ps_y = srcheight -1;
			    
				*dstdata = (uchar)pixels[*(srcdata + ps_y*srcbpl +
							   ps_x + src_xoffset)];
			    }
			}
		    }
		}
		break;
		
	      case 1:
                result->format.bit = GMSB_FIRST;
		int maxX = dst_bytesPerLine* 8;
		for (y=0; y < height; y++,
		     dstdata = result->data + y * dst_bytesPerLine + dst_xoffset,
		     ps = savePS + src_dy) {
		
		    savePS = ps;
		    for (x=0; x < maxX; x++, ps += src_dx) {

			int ps_x;
			uchar result  = 0;
		    
			/** x0 + 0 **/
			ps_x = (int)ps.x;

			if (ps_x >= 0 && ps_x <= srcwidth) {
			    int ps_y = (int)ps.y;
			    if (ps_y >=0 && ps_y <= srcheight) {
				if (ps_x == srcwidth) ps_x = srcwidth -1 + src_xoffset;
				else ps_x += src_xoffset;
				if (ps_y == srcheight) ps_y = srcheight -1;

				if (pixels[*(srcdata + ps_y*srcbpl + ps_x)])
				    result |= 0x80;
			    }
			}

			/** x0 + 1 */
			x++, ps += src_dx;
			ps_x = (int)ps.x;
		    
			if (ps_x >= 0 && ps_x <= srcwidth) {
			    int ps_y = (int)ps.y;
			    if (ps_y >=0 && ps_y <= srcheight) {
				if (ps_x == srcwidth) ps_x = srcwidth -1 + src_xoffset;
				else ps_x += src_xoffset;
				if (ps_y == srcheight) ps_y = srcheight -1;

				if (pixels[*(srcdata + ps_y*srcbpl + ps_x)])
				    result |= 0x40;
			    }
			}

			/** x0 + 2 */
			x++, ps += src_dx;
			ps_x = (int)ps.x;
		    
			if (ps_x >= 0 && ps_x <= srcwidth) {
			    int ps_y = (int)ps.y;
			    if (ps_y >=0 && ps_y <= srcheight) {
				if (ps_x == srcwidth) ps_x = srcwidth -1 + src_xoffset;
				else ps_x += src_xoffset;
				if (ps_y == srcheight) ps_y = srcheight -1;

				if (pixels[*(srcdata + ps_y*srcbpl + ps_x)])
				    result |= 0x20;
			    }
			}

			/** x0 + 3 */
			x++, ps += src_dx;
			ps_x = (int)ps.x;
		    
			if (ps_x >= 0 && ps_x <= srcwidth) {
			    int ps_y = (int)ps.y;
			    if (ps_y >=0 && ps_y <= srcheight) {
				if (ps_x == srcwidth) ps_x = srcwidth -1 + src_xoffset;
				else ps_x += src_xoffset;
				if (ps_y == srcheight) ps_y = srcheight -1;

				if (pixels[*(srcdata + ps_y*srcbpl + ps_x)])
				    result |= 0x10;

			    }
			}
			/** x0 + 4 */
			x++, ps += src_dx;
			ps_x = (int)ps.x;
		    
			if (ps_x >= 0 && ps_x <= srcwidth) {
			    int ps_y = (int)ps.y;
			    if (ps_y >=0 && ps_y <= srcheight) {
				if (ps_x == srcwidth) ps_x = srcwidth -1 + src_xoffset;
				else ps_x += src_xoffset;
				if (ps_y == srcheight) ps_y = srcheight -1;

				if (pixels[*(srcdata + ps_y*srcbpl + ps_x)])
				    result |= 0x08;
			    }
			}

			/** x + 5 */
			x++, ps += src_dx;
			ps_x = (int)ps.x;
		    
			if (ps_x >= 0 && ps_x <= srcwidth) {
			    int ps_y = (int)ps.y;
			    if (ps_y >=0 && ps_y <= srcheight) {
				if (ps_x == srcwidth) ps_x = srcwidth -1 + src_xoffset;
				else ps_x += src_xoffset;
				if (ps_y == srcheight) ps_y = srcheight -1;

				if (pixels[*(srcdata + ps_y*srcbpl + ps_x)])
				    result |= 0x04;
			    }
			}

			/** x0 + 6 */
			x++, ps += src_dx;
			ps_x = (int)ps.x;
		    
			if (ps_x >= 0 && ps_x <= srcwidth) {
			    int ps_y = (int)ps.y;
			    if (ps_y >=0 && ps_y <= srcheight) {
				if (ps_x == srcwidth) ps_x = srcwidth -1 + src_xoffset;
				else ps_x += src_xoffset;
				if (ps_y == srcheight) ps_y = srcheight -1;

				if (pixels[*(srcdata + ps_y*srcbpl + ps_x)])
				    result |= 0x02;

			    }
			}
			/** x0 + 7 */
			x++, ps += src_dx;
			ps_x = (int)ps.x;
		    
			if (ps_x >= 0 && ps_x <= srcwidth) {
			    int ps_y = (int)ps.y;
			    if (ps_y >=0 && ps_y <= srcheight) {
				if (ps_x == srcwidth) ps_x = srcwidth -1 + src_xoffset;
				else ps_x += src_xoffset;
				if (ps_y == srcheight) ps_y = srcheight -1;

				if (pixels[*(srcdata + ps_y*srcbpl + ps_x)])
				    result |= 0x01;
			    }
			}
		    

			/**
			  set the result for 8 bits
			  **/
			*dstdata++ = result;
		    }
		}
		break;  /* switch bits_per_pixel_of_result */
	    }
	} else 
	    /**
	      NO TRANSLATION
	      different bits_per_pixel are unavailable.
	      **/
	    for (y=0; y < height; y++,
#if defined( __BORLANDC__)  ||	defined(__MSC7__)
		 dstdata = temp + y * dst_bytesPerLine + (((ulong)dst_xoffset) & 0xffffL),
#else
		 dstdata = result->data + y * dst_bytesPerLine + dst_xoffset,
#endif
		 ps = savePS + src_dy) { 

		savePS = ps;
		for (x=0; x < width; x++, ps += src_dx, dstdata++) {
		    int ps_x = (int)ps.x;
		    if (ps_x >=0 && ps_x <= srcwidth) {
			int ps_y = (int)ps.y;
			if (ps_y >=0 && ps_y <= srcheight) {
			    if (ps_x == srcwidth) ps_x = srcwidth-1;
			    if (ps_y == srcheight) ps_y = srcheight -1;
			    
			    *dstdata = *(srcdata + ps_y*srcbpl + ps_x + src_xoffset);
			}
		    }
		}
	    }
    } else  {
	/**
	  ZFORMAT bits_per_pixel = 1  : pixel_translation is unavailable
	  for depth 1 pixelArray.
	  **/
	result->xoffset = dst_xoffset;
	result->width = width;
	result->height = height;
        result->padding = 8;
        int dst_bytesPerLine = (width+dst_xoffset+7)/8;
        int maxX = dst_bytesPerLine * 8;
#if defined( __BORLANDC__)  ||  defined(__MSC7__)
        uchar huge * temp = (uchar*)GMALLOC(dst_bytesPerLine*result->height);
        result->data =(uchar*) temp;
#else
        result->data = (uchar*)GMALLOC(dst_bytesPerLine*result->height);
#endif
	/**
	  set data to zero because
	  depth one may be use for stipple
	  **/
#if defined( __BORLANDC__) ||   defined(__MSC7__)
        uchar huge * temp2 = result->data;
        for(ulong il= 0; il < dst_bytesPerLine * (result->height); il++)
             *temp2++ = (uchar) 0;       
        uchar huge* dstdata = temp + dst_xoffset / 8;
#else   
        memset(result->data, 0, dst_bytesPerLine*result->height);
        uchar* dstdata = result->data + dst_xoffset / 8;
#endif  

        if (_values.format.byte != GMSB_FIRST && _values.bitmap_unit != 8) {
            int save = GTraceLevel();
            GTraceLevel() = 1;
            GWarning("Sorry shPixelArray->deformation(): %s",
                     "bits_per_pixel == 1 and bitmap_unit != 8 and format.byte != GMSB_FIRST not implemented yet");
            GTraceLevel() = save;
            return;
        }
	ps = src_origin + src_dy*y_begin;
	
	switch (_values.format.bit) {
	  case GMSB_FIRST:
	    for (y = 0 ; y < height; y++,
#if defined( __BORLANDC__)  ||	defined(__MSC7__)
		 dstdata = temp + ((ulong)y) * dst_bytesPerLine + dst_xoffset / 8,
#else
		 dstdata = result->data + y * dst_bytesPerLine + dst_xoffset / 8,
#endif
		 ps = savePS + src_dy)  {

		savePS = ps;
		for (x = 0; x < maxX; x++, ps += src_dx) {
		    uchar byte;
		    uchar result = 0;
		    int ps_x;
		    
		    /** x0 + 0 **/
		    ps_x = (int)ps.x;

		    if (ps_x >= 0 && ps_x <= srcwidth) {
			int ps_y = (int)ps.y;
			if (ps_y >=0 && ps_y <= srcheight) {
			    if (ps_x == srcwidth) ps_x = srcwidth -1 + src_xoffset;
			    else ps_x += src_xoffset;
			    if (ps_y == srcheight) ps_y = srcheight -1;

			    switch( byte = *(srcdata + ps_y *  srcbpl + ((uint)ps_x)/8)) {
			      case 0:
				break;
			      case 0xff:
				result |= 0x80;
				break;
			      default:
				if (byte & (0x80 >> ((uint)ps_x)%8))
				    result |= 0x80;
				break;
			    }
			}
		    }

		    /** x0 + 1 */
		    x++, ps += src_dx;
		    ps_x = (int)ps.x;
		    
		    if (ps_x >= 0 && ps_x <= srcwidth) {
			int ps_y = (int)ps.y;
			if (ps_y >=0 && ps_y <= srcheight) {
			    if (ps_x == srcwidth) ps_x = srcwidth -1 + src_xoffset;
			    else ps_x += src_xoffset;
			    if (ps_y == srcheight) ps_y = srcheight -1;

			    switch( byte = *(srcdata + ps_y *  srcbpl + ((uint)ps_x)/8)) {
			      case 0:
				break;
			      case 0xff:
				result |= 0x40;
				break;
			      default:
				if (byte & (0x80 >> ((uint)ps_x)%8))
				    result |= 0x40;
				break;
			    }
			}
		    }

		    /** x0 + 2 */
		    x++, ps += src_dx;
		    ps_x = (int)ps.x;
		    
		    if (ps_x >= 0 && ps_x <= srcwidth) {
			int ps_y = (int)ps.y;
			if (ps_y >=0 && ps_y <= srcheight) {
			    if (ps_x == srcwidth) ps_x = srcwidth -1 + src_xoffset;
			    else ps_x += src_xoffset;
			    if (ps_y == srcheight) ps_y = srcheight -1;

			    switch( byte = *(srcdata + ps_y *  srcbpl + ((uint)ps_x)/8)) {
			      case 0:
				break;
			      case 0xff:
				result |= 0x20;
				break;
			      default:
				if (byte & (0x80 >> ((uint)ps_x)%8))
				    result |= 0x20;
				break;
			    }
			}
		    }

		    /** x0 + 3 */
		    x++, ps += src_dx;
		    ps_x = (int)ps.x;
		    
		    if (ps_x >= 0 && ps_x <= srcwidth) {
			int ps_y = (int)ps.y;
			if (ps_y >=0 && ps_y <= srcheight) {
			    if (ps_x == srcwidth) ps_x = srcwidth -1 + src_xoffset;
			    else ps_x += src_xoffset;
			    if (ps_y == srcheight) ps_y = srcheight -1;

			    switch( byte = *(srcdata + ps_y *  srcbpl + ((uint)ps_x)/8)) {
			      case 0:
				break;
			      case 0xff:
				result |= 0x10;
				break;
			      default:
				if (byte & (0x80 >> ((uint)ps_x)%8))
				    result |= 0x10;
				break;
			    }

			}
		    }
		    /** x0 + 4 */
		    x++, ps += src_dx;
		    ps_x = (int)ps.x;
		    
		    if (ps_x >= 0 && ps_x <= srcwidth) {
			int ps_y = (int)ps.y;
			if (ps_y >=0 && ps_y <= srcheight) {
			    if (ps_x == srcwidth) ps_x = srcwidth -1 + src_xoffset;
			    else ps_x += src_xoffset;
			    if (ps_y == srcheight) ps_y = srcheight -1;

			    switch( byte = *(srcdata + ps_y *  srcbpl + ((uint)ps_x)/8)) {
			      case 0:
				break;
			      case 0xff:
				result |= 0x08;
				break;
			      default:
				if (byte & (0x80 >> ((uint)ps_x)%8))
				    result |= 0x08;
				break;
			    }
			}
		    }

		    /** x + 5 */
		    x++, ps += src_dx;
		    ps_x = (int)ps.x;
		    
		    if (ps_x >= 0 && ps_x <= srcwidth) {
			int ps_y = (int)ps.y;
			if (ps_y >=0 && ps_y <= srcheight) {
			    if (ps_x == srcwidth) ps_x = srcwidth -1 + src_xoffset;
			    else ps_x += src_xoffset;
			    if (ps_y == srcheight) ps_y = srcheight -1;

			    switch( byte = *(srcdata + ps_y *  srcbpl + ((uint)ps_x)/8)) {
			      case 0:
				break;
			      case 0xff:
				result |= 0x04;
				break;
			      default:
				if (byte & (0x80 >> ((uint)ps_x)%8))
				    result |= 0x04;
				break;
			    }
			}
		    }

		    /** x0 + 6 */
		    x++, ps += src_dx;
		    ps_x = (int)ps.x;
		    
		    if (ps_x >= 0 && ps_x <= srcwidth) {
			int ps_y = (int)ps.y;
			if (ps_y >=0 && ps_y <= srcheight) {
			    if (ps_x == srcwidth) ps_x = srcwidth -1 + src_xoffset;
			    else ps_x += src_xoffset;
			    if (ps_y == srcheight) ps_y = srcheight -1;

			    switch( byte = *(srcdata + ps_y *  srcbpl + ((uint)ps_x)/8)) {
			      case 0:
				break;
			      case 0xff:
				result |= 0x02;
				break;
			      default:
				if (byte & (0x80 >> ((uint)ps_x)%8))
				    result |= 0x02;
				break;
			    }

			}
		    }
		    /** x0 + 7 */
		    x++, ps += src_dx;
		    ps_x = (int)ps.x;
		    
		    if (ps_x >= 0 && ps_x <= srcwidth) {
			int ps_y = (int)ps.y;
			if (ps_y >=0 && ps_y <= srcheight) {
			    if (ps_x == srcwidth) ps_x = srcwidth -1 + src_xoffset;
			    else ps_x += src_xoffset;
			    if (ps_y == srcheight) ps_y = srcheight -1;

			    switch( byte = *(srcdata + ps_y *  srcbpl + ((uint)ps_x)/8)) {
			      case 0:
				break;
			      case 0xff:
				result |= 0x01;
				break;
			      default:
				if (byte & (0x80 >> ((uint)ps_x)%8))
				    result |= 0x01;
				break;
			    }
			}
		    }
		    

		    /**
		      set the result for 8 bits
		      **/
		    *dstdata++ = result;
		}
	    }
				    
	    break;
	    
	  case GLSB_FIRST:      /** src bit_order **/
	    for (y = 0 ; y < height; y++,
#if defined( __BORLANDC__) ||	defined(__MSC7__)
		 dstdata = temp + ((ulong)y) * dst_bytesPerLine + ((ulong)dst_xoffset) / 8,
#else
		 dstdata = result->data + y * dst_bytesPerLine + dst_xoffset / 8,
#endif
		 ps = savePS + src_dy)  {

		savePS = ps;
		for (x = 0; x < maxX; x++, ps += src_dx) {
		    uchar byte;
		    uchar result = 0;
		    int ps_x;
		    
		    /** x0 + 0 **/
		    ps_x = (int)ps.x;
		    
		    if (ps_x >= 0 && ps_x <= srcwidth) {
			int ps_y = (int)ps.y;
			if (ps_y >=0 && ps_y <= srcheight) {
			    if (ps_x == srcwidth) ps_x = srcwidth -1 + src_xoffset;
			    else ps_x += src_xoffset;
			    if (ps_y == srcheight) ps_y = srcheight -1;

			    switch( byte = *(srcdata + ps_y *  srcbpl + ((uint)ps_x)/8)) {
			      case 0:
				break;
			      case 0xff:
				result |= 0x01;
				break;
			      default:
				if (byte & (0x01 << ((uint)ps_x)%8))
				    result |= 0x01;
				break;
			    }
			}
		    }

		    /** x0 + 1 */
		    x++, ps += src_dx;
		    ps_x = (int)ps.x;
		    
		    if (ps_x >= 0 && ps_x <= srcwidth) {
			int ps_y = (int)ps.y;
			if (ps_y >=0 && ps_y <= srcheight) {
			    if (ps_x == srcwidth) ps_x = srcwidth -1 + src_xoffset;
			    else ps_x += src_xoffset;
			    if (ps_y == srcheight) ps_y = srcheight -1;

			    switch( byte = *(srcdata + ps_y *  srcbpl + ((uint)ps_x)/8)) {
			      case 0:
				break;
			      case 0xff:
				result |= 0x02;
				break;
			      default:
				if (byte & (1 << ((uint)ps_x)%8))
				    result |= 0x02;
				break;
			    }

			}
		    }
		    
		    /** x0 + 2 */
		    x++, ps += src_dx;
		    ps_x = (int)ps.x;
		    
		    if (ps_x >= 0 && ps_x <= srcwidth) {
			int ps_y = (int)ps.y;
			if (ps_y >=0 && ps_y <= srcheight) {
			    if (ps_x == srcwidth) ps_x = srcwidth -1 + src_xoffset;
			    else ps_x += src_xoffset;
			    if (ps_y == srcheight) ps_y = srcheight -1;

			    switch( byte = *(srcdata + ps_y *  srcbpl + ((uint)ps_x)/8)) {
			      case 0:
				break;
			      case 0xff:
				result |= 0x04;
				break;
			      default:
				if (byte & (1 << ((uint)ps_x)%8))
				    result |= 0x04;
				break;
			    }

			}
		    }
		    /** x0 + 3 */
		    x++, ps += src_dx;
		    ps_x = (int)ps.x;
		    
		    if (ps_x >= 0 && ps_x <= srcwidth) {
			int ps_y = (int)ps.y;
			if (ps_y >=0 && ps_y <= srcheight) {
			    if (ps_x == srcwidth) ps_x = srcwidth -1 + src_xoffset;
			    else ps_x += src_xoffset;
			    if (ps_y == srcheight) ps_y = srcheight -1;

			    switch( byte = *(srcdata + ps_y *  srcbpl + ((uint)ps_x)/8)) {
			      case 0:
				break;
			      case 0xff:
				result |= 0x08;
				break;
			      default:
				if (byte & (1 << ((uint)ps_x)%8))
				    result |= 0x08;
				break;
			    }
			}
		    }

		    /** x0 + 4 */
		    x++, ps += src_dx;
		    ps_x = (int)ps.x;
		    
		    if (ps_x >= 0 && ps_x <= srcwidth) {
			int ps_y = (int)ps.y;
			if (ps_y >=0 && ps_y <= srcheight) {
			    if (ps_x == srcwidth) ps_x = srcwidth -1 + src_xoffset;
			    else ps_x += src_xoffset;
			    if (ps_y == srcheight) ps_y = srcheight -1;

			    switch( byte = *(srcdata + ps_y *  srcbpl + ((uint)ps_x)/8)) {
			      case 0:
				break;
			      case 0xff:
				result |= 0x10;
				break;
			      default:
				if (byte & ( 1 << ((uint)ps_x)%8))
				    result |= 0x10;
				break;
			    }
			}
		    }

		    /** x0 + 5 */
		    x++, ps += src_dx;
		    ps_x = (int)ps.x;
		    
		    if (ps_x >= 0 && ps_x <= srcwidth) {
			int ps_y = (int)ps.y;
			if (ps_y >=0 && ps_y <= srcheight) {
			    if (ps_x == srcwidth) ps_x = srcwidth -1 + src_xoffset;
			    else ps_x += src_xoffset;
			    if (ps_y == srcheight) ps_y = srcheight -1;

			    switch( byte = *(srcdata + ps_y *  srcbpl + ((uint)ps_x)/8)) {
			      case 0:
				break;
			      case 0xff:
				result |= 0x20;
				break;
			      default:
				if (byte & (1 << ((uint)ps_x)%8))
				    result |= 0x20;
				break;
			    }
			}
		    }
		    
		    /** x0 + 6 */
		    x++, ps += src_dx;
		    ps_x = (int)ps.x;
		    
		    if (ps_x >= 0 && ps_x <= srcwidth) {
			int ps_y = (int)ps.y;
			if (ps_y >=0 && ps_y <= srcheight) {
			    if (ps_x == srcwidth) ps_x = srcwidth -1 + src_xoffset;
			    else ps_x += src_xoffset;
			    if (ps_y == srcheight) ps_y = srcheight -1;

			    switch( byte = *(srcdata + ps_y *  srcbpl + ((uint)ps_x)/8)) {
			      case 0:
				break;
			      case 0xff:
				result |= 0x40;
				break;
			      default:
				if (byte & (1 << ((uint)ps_x)%8))
				    result |= 0x40;
				break;
			    }
			}
		    }

		    /** x0 + 7 */
		    x++, ps += src_dx;
		    ps_x = (int)ps.x;
		    

		    if (ps_x >= 0 && ps_x <= srcwidth) {
			int ps_y = (int)ps.y;
			if (ps_y >=0 && ps_y <= srcheight) {
			    if (ps_x == srcwidth) ps_x = srcwidth -1 + src_xoffset;
			    else ps_x += src_xoffset;
			    if (ps_y == srcheight) ps_y = srcheight -1;

			    switch( byte = *(srcdata + ps_y *  srcbpl + ((uint)ps_x)/8)) {
			      case 0:
				break;
			      case 0xff:
				result |= 0x80;
				break;
			      default:
				if (byte & (1 << ((uint)ps_x)%8))
				    result |= 0x80;
				break;
			    }
			}
		    }

		    /**
		      set the result for 8 bits
		      **/
		    *dstdata++ = result;
		}
	    }
	    break;
	}
    }

    /**
      set the xoffset of the destination
      **/
    result->xoffset = dst_xoffset;
}
