#ifdef REV_INFO
static char rcsid[] = "$Id: Queue.cc,v 1.3 1992/12/17 12:22:32 jojic Exp $";
#endif
/*****************************************************************************
MODULE:  Tools
FILE:    Queue.cc
AUTHOR:  Oliver Jojic
DATE:    October 1990

DESCRIPTION:
    Queue object implementation    

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include <Go/Tools/Queue.h>
#include <Go/Tools/Set.h>
    

GBool GQueue::getRef (int& ref, void*& value) const
{
    if (ref < first || ref >= last)
	return GFALSE;

    value = array[ref];
    ref += 1;

    return GTRUE;
}


GBool GQueue::remRef (int& ref) 
{
    if (ref < first || ref >= last)
	return GFALSE;

    bcopy (&array[ref+1], &array[ref], (last - ref - 1) * sizeof array[ref]);
    last -= 1;

    return GTRUE;
}


void GQueue::copyfirst (const GQueue& queue)
{
    int incr = queue.last - queue.first;
    while (first < incr)
	incr_size_for_first();

    first -= incr;
    bcopy (&queue.array[queue.first], &array[first], incr * sizeof (void*));
}

void GQueue::copylast (const GQueue& q)
{
    int incr = q.length();
    while (size - last < incr)
	incr_size_for_last();

    last += incr;
    bcopy (&q.array[q.first], &array[first], incr * sizeof (void*));
}


void GQueue::incr_size_for_last ()
{
    // to avoid too much new/delete:
    if (last < size || first < 8) // we must realloc
    {
	void** tmparray = array;
	size += size;
	array = (void**)malloc(size * sizeof(void*));
	bcopy(&tmparray[first], &array[first], (last - first) * sizeof(void*));
	free((char*) tmparray);
    }
    else
    {
	int oldfirst = first;
	first /= 2;
	last -= oldfirst - first;
	bcopy(&array[oldfirst], &array[first], (last - first) * sizeof(void*));
    }
}

void GQueue::incr_size_for_first ()
{
    // to avoid too much new/delete:

    if (first > 0 || (size - last) < 8) // we must realloc
    {
	void** tmparray = array;
	array = (void**)malloc((size + size) * sizeof(void*));
	bcopy(&tmparray[first], &array[size + first],
	      (last - first) * sizeof(void*));
	free((char*) tmparray);
	first += size;
	last += size;
	size += size;
    }
    else
    {
	int delta = (size - last) / 2;
	bcopy(&array[first], &array[first + delta], (last - first) * sizeof(void*));
	first += delta;
	last += delta;
    }
}


void GQueue::copyTo (GSet& set) const
{
    for (int i = first; i < last; i++)
	set.add (array[i]);
}
