#ifdef REV_INFO
static char rcsid[] = "$Id: Set.cc,v 1.5 1992/09/25 15:05:52 smadja Exp $";
#endif
/*****************************************************************************
MODULE:  Go Tools
FILE:    Set.cc
AUTHOR:  Oliver Jojic
DATE:    December 90

DESCRIPTION:
    See Go/Tools/Set.h

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#include <Go/Tools/Set.h>
#include <Go/Tools/Alloc.h>


GSet::GSet ()
{
    size = count = next_index = 0;
    array = NULL;
}

GSet::~GSet ()
{
    GFREE(array);
}


void GSet::remove (const void* obj)
{
    for (int i = count - 1; i >= 0; --i)
	if (array[i] == obj)
	{
	    bcopy(array + i + 1, (void*)(array + i), (--count - i) * sizeof(void*));
	    break;
	}
}

void GSet::incrArraySize ()
{
    if (size == 0)
	size = 8;
    else
	size += size;

    void** tmp = (void**)array;
    array = (const void**)GMALLOC(size * sizeof(void*));

    bcopy(tmp, (void*)array, count * sizeof(void*));
    GFREE(tmp);
}

GBool GSet::contains (const void* obj) const
{
    for (int i = 0; i < count; i++)
	if (array[i] == obj)
	    return GTRUE;
    return GFALSE;
}


void GSet::add (const void* obj)
{
    if (count >= size)
	incrArraySize();
    array[count++] = (void*)obj;
}


void* GSet::first () const
{
    if (empty())
	return NULL;

    ((GSet*)this)->next_index = 1;
    return (void*)array[0];
}


void* GSet::next () const
{
    return (next_index >= count)
	? NULL
	    : (void*)array[((GSet*)this)->next_index++];
}
