#ifdef REV_INFO
static char rcsid[] = "$Id: ShoFoo.cc,v 1.1.1.1 1992/03/18 10:51:12 uep Exp $";
#endif
/*****************************************************************************
MODULE:  Tools (Shared Object Management)
FILE:    ShoFoo.cc
AUTHOR:  Jacques Davy
DATE:    August 1990

DESCRIPTION:
    Test program for GSharedObject class

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#include "ShoFoo.h"

/*
 * Protected Methods
 */

GShoFoo* _newShoFoo (FunctionList* funcList, ulong mask, const GShoFooValues* values, const char* name)
{
   return new GShoFoo(funcList, mask, values, name);
}

/**/
GShoFoo::GShoFoo (FunctionList* funcList, ulong mask, const GShoFooValues* values, const char* name) : (funcList, mask, name)
{
    _values.a = values->a;
    _values.b = values->b;
    _values.c = values->c;
}

/**/
List*& GShoFoo::_head ()
{
    return _headOfList;
}

/**/
int GShoFoo::_valueSize ()
{
    return (sizeof(GShoFooValues));
}

/**/
bool GShoFoo::_match (ulong mask, const GShoFooValues* values)
{
    ulong inter_mask = mask & _mask;

    if (!inter_mask)
	return TRUE;

    if (inter_mask & A)
        if (values->a != _values.a)
	    return FALSE;

    if (inter_mask & B)
        if (values->b != _values.b)
	    return FALSE;

    if (inter_mask & C)
        if (values->c != _values.c)
	    return FALSE;

    return TRUE;
}

/**/
void GShoFoo::_makeNewValues (ulong mask, const GShoFooValues* values,
			     ulong& newMask, GShoFooValues* newValues)
{
    newMask = mask | _mask;

    if (mask & A)
        newValues->a = values->a;
    else
	if (_mask & A)
            newValues->a = _values.a;

    if (mask & B)
        newValues->b = values->b;
    else
	if (_mask & B)
            newValues->b = _values.b;

    if (mask & C)
        newValues->c = values->c;
    else
	if (_mask & C)
            newValues->c = _values.c;
}

/**/
ulong GShoFoo::_update (ulong mask, const GShoFooValues* values)
{
    ulong update_mask = 0;

#define updateValue(maskBit, field)\
    if ((mask & maskBit) && \
	  (!(_mask & maskBit) || (_values.field != values->field))) \
        {\
            _values.field = values->field;\
	    update_mask |= maskBit;\
	}

    updateValue(A, a);
    updateValue(B, b);
    updateValue(C, c);
#undef updateValue

    _mask |= mask;

    return (update_mask);
}

/**/
GShoFoo::~GShoFoo ()
{
}

/*
 * Public Methods
 */
void modify (GShoFoo*& sho, ulong mask, const GShoFooValues* values)
{
   sho->modify((GSharedObject*&)sho, mask, (const Pointer)values);
}

/**/
void modify (const char* name, ulong mask, const GShoFooValues* values)
{
   ModifyNamedSharedObject (GShoFoo, mask, values, name);
}

/**/
void rename (GShoFoo*& sho, const char* name)
{
   sho->rename((GSharedObject*&)sho, name);
}
/**/
GShoFoo* GShoFoo::dup() const
{
    return (GShoFoo*)((GSharedObject*)this)->dup();
}

/**/
void GShoFoo::updatePostcond (void proc(void*, GShoFoo*, ulong), const void* closure, AddOrRemove add) const
{
    GSharedObject::updatePostcond (
		        (void(*)(void*, GSharedObject*, ulong))proc, closure, add);
}

/**/
void GShoFoo::incrRefPostcond (void proc(void*, GShoFoo*), const void* closure, AddOrRemove add) const
{
    GSharedObject::incrRefPostcond (
		         (void(*)(void*, GSharedObject*))proc, closure, add);
}

/**/
void GShoFoo::decrRefPostcond (void proc(void*, GShoFoo*), const void* closure, AddOrRemove add) const
{
    GSharedObject::decrRefPostcond (
                         (void(*)(void*, GSharedObject*))proc, closure, add);
}

/**/
void GShoFoo::destroyPostcond (void proc(void*, const GShoFoo*), const void* closure, AddOrRemove add) const
{
    GSharedObject::destroyPostcond (
		          (void(*)(void*, const GSharedObject*))proc, closure, add);
};

/**/
GShoFoo* CreateShoFoo (ulong mask, const GShoFooValues* values, const char* name)
{
   return (GShoFoo*)CreateSharedObject(GShoFoo, mask, values, name);
}

