/* $Id: ShoFoo.h,v 1.1.1.1 1992/03/18 10:51:17 uep Exp $ */
/*****************************************************************************
MODULE:  Tools (Shared Object Management)
FILE:    ShoFoo.h
AUTHOR:  Jacques Davy
DATE:    August 1990

DESCRIPTION:
    Test program for SharedObject class

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#ifndef _ShoFoo_h
#define _ShoFoo_h

#include <Go/SharedObj.h>

/*
 * ShoFooValues
 */
struct ShoFooValues {
    int a;
    int b;
    int c;
};

/*
 * ShoFoo mask bits
 */
#define A (1L<<0)
#define B (1L<<1)
#define C (1L<<2)

/*
 * SharedObject subclass ShoFoo
 */
class ShoFoo : public SharedObject {
  private:
    ShoFooValues _values;
    static List* _headOfList;
    static FunctionList _funcList;

  protected:
    List*& _head();
    int _valueSize ();
    bool _match (ulong mask, const ShoFooValues* values);
    void _makeNewValues (ulong mask, const ShoFooValues* values,
			 ulong& newMask, ShoFooValues* newValues);
    ulong _update (ulong mask, const ShoFooValues* values);
    friend ShoFoo* _newShoFoo (FunctionList* funcList,
			       ulong mask, const ShoFooValues* values, const char* name = NULL);

    ShoFoo (FunctionList* funcList, ulong mask, const ShoFooValues* values, const char* name = NULL);
    ~ShoFoo ();
  public:
    friend void modify (ShoFoo*& sho, ulong mask, const ShoFooValues* values);
    friend void modify (const char* name, ulong mask, const ShoFooValues* values);
    friend void rename (ShoFoo*& sho, const char* name);
    ShoFoo* dup() const;
    void updatePostcond (void (*)(void*, ShoFoo*, ulong),
			 const void* closure, AddOrRemove = ADD) const;
    void incrRefPostcond (void (*)(void*, ShoFoo*),
			  const void* closure, AddOrRemove = ADD) const;
    void decrRefPostcond (void (*)(void*, ShoFoo*),
			  const void* closure, AddOrRemove = ADD) const;
    void destroyPostcond (void (*)(void*, const ShoFoo*),
			  const void* closure, AddOrRemove = ADD) const;

    friend ShoFoo* CreateShoFoo (ulong mask, const ShoFooValues* values, const char* name = NULL);
    inline const ShoFooValues* values() const { return &_values; }
};

/*
 * inline functions
 */

#endif /* _ShoFoo_h */
