#ifdef REV_INFO
static char rcsid[] = "$Id: Test.cc,v 1.1.1.1 1992/03/18 10:51:22 uep Exp $";
#endif
/*****************************************************************************
MODULE:  Tools (Shared Object Management)
FILE:    Test.cc
AUTHOR:  Jacques Davy
DATE:    August 1990

DESCRIPTION:
    Test for SharedObject class

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#include <stdio.h>
#include "ShoFoo.h"


void updateCB (void* closure, ShoFoo* foo, ulong mask)
{
    printf ("update %s = %x\n\tupdate_mask = %x\n\tmask = %x a = %d, b = %d, c = %d\n", closure,
	    foo, mask,
	    foo->mask(), foo->values()->a, foo->values()->b, foo->values()->c);
}

void incrCB (void* closure, ShoFoo* foo)
{
   printf ("incrRef %s = %x, refcnt = %d\n", closure, foo, foo->refcnt());
}

void decrCB (void* closure, ShoFoo* foo)
{
   printf ("decrRef %s = %x, refcnt = %d\n", closure, foo, foo->refcnt());
}

void destroyCB (void* closure, const ShoFoo* foo)
{
   printf ("destroy %s = %x\n", closure, foo);
}

void AddCB (ShoFoo* sho, char* closure)
{
    sho->updatePostcond (updateCB, closure, ADD);
    sho->destroyPostcond (destroyCB, closure, ADD);
    sho->incrRefPostcond (incrCB, closure, ADD);
    sho->decrRefPostcond (decrCB, closure, ADD);
}

void RemCB (ShoFoo* sho, char* closure)
{
    sho->updatePostcond (updateCB, closure, REMOVE);
    sho->destroyPostcond (destroyCB, closure, REMOVE);
    sho->incrRefPostcond (incrCB, closure, REMOVE);
    sho->decrRefPostcond (decrCB, closure, REMOVE);
}

main ()
{
    ShoFoo *sh1, *sh2, *sh3, *nsh1, *nsh2, *nsh3;
    ShoFooValues values;
    ulong mask;

    // Named Shared Object
    mask = A;
    values.a = 1;
    nsh1 = CreateShoFoo (mask, &values, "nsh1");
    AddCB (nsh1, "nsh1");

    mask = A|B;
    values.b = 2;
    nsh2 = CreateShoFoo (mask, &values, "nsh2");
    if (nsh1 != nsh2)
        AddCB (nsh2, "nsh2");

    nsh3 = (ShoFoo*)nsh2->dup();

    mask = A|C;
    values.a = 11; values.c = 13;
    modify (nsh2, mask, &values);
    if ( nsh2 != nsh3 )
        AddCB (nsh2, "nsh2");
    
    rename (nsh1, "nsh2");
    rename (nsh2, "nsh22");
    rename (nsh3, "nsh33");
    rename (nsh1, "nsh11");

    mask = A;
    values.a = 1;
    sh1 = CreateShoFoo (mask, &values);
    AddCB (sh1, "sh1");

    mask = A|B;
    values.b = 2;
    sh2 = CreateShoFoo (mask, &values);
    if (sh1 != sh2)
        AddCB (sh2, "sh2");

    sh3 = (ShoFoo*)sh2->dup();

    mask = A|C;
    values.a = 11; values.c = 13;
    modify (sh2, mask, &values);
    if ( sh2 != sh3 )
        AddCB (sh2, "sh2");

    rename (nsh1, 0);

    sh3->destroy();
    sh3 = sh2->dup();


    RemCB (sh1, "sh1");
    nsh1->destroy();
    nsh2->destroy();
    nsh3->destroy();
    sh1->destroy();
    sh2->destroy();
    RemCB (sh3, "sh3"); // Don't remove callbacks because of closure
    sh3->destroy();
}
