#ifdef REV_INFO
static char rcsid[] = "$Id: Grabber.cc,v 1.4 1992/12/21 16:16:09 jojic Exp $";
#endif
/*****************************************************************************
MODULE:  Go Kernel (User Interface)
FILE:    Grabber.cc
AUTHOR:  Oliver Jojic
DATE:    June 92

DESCRIPTION:
  See also the Grabber.h header.

COPYRIGHT:
  Copyright (c) 1990, 1991, 1992 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.

*****************************************************************************/

#include <Go/Grabber.h>
#include <Go/GoUI.h>
#include <Go/GoInterf.h>
#include <Go/Sensor.h>
#include <Go/Event.h>



GGrabberPgm::GGrabberPgm (const char* name)
    : GInteractorPgm (name)
{
}


GEventMask GGrabberPgm::inputMask () const
{
    return
	GBUTTON_PRESS_MASK |
	    GBUTTON_RELEASE_MASK |
		GCHAR_MASK |
		    GFUNCTION_KEY_MASK |
			GCOMPRESS_MOTION_MASK |
			    GEXPOSE_MASK;
}


GGrabber::GGrabber (GGrabberPgm* pgm, const GEvent* event, Go* go)
    : GInteractor(pgm, go)
{
    extra_vector.x = extra_vector.y = 0;
    draw_from_expose = GFALSE;
    first_pos = event ? event->position() : _go->interf()->getPointer();
    _pos = prev_pos = first_pos;
    _goui = NULL;
    _sensor = NULL;
}


GGrabber::~GGrabber ()
{
    if (_goui != NULL)
    {
	GoSensor* sor = sensor();
	if (sor != NULL)
	    _goui->ungrab (sor);
    }
}


void GGrabber::draw ()
{
}


void GGrabber::drawAll ()
{
    draw();
}


void GGrabber::clear ()
{
    draw();
}


void GGrabber::clearAll ()
{
    clear ();
}
    

GBool GGrabber::isDoitEvent (const GEvent* event) const
{
    return
	 (event->type() == GBUTTON_RELEASE_EVENT ||
	  event->type() == GBUTTON_PRESS_EVENT) && event->button() != GBUTTON3
	     ||
		 event->type() == GCHAR_EVENT && event->character() == 13; //CR
}


GBool GGrabber::isCancelEvent (const GEvent* event) const
{
    return
	event->type() == GCHAR_EVENT &&
	    event->character() == 27  //BUG: "ESCAPE" would be better than 27
    ||
	event->type() == GBUTTON_PRESS_EVENT &&
	    event->button() == GBUTTON3;
}


void GGrabber::compute (const GEvent* event)
{
    prev_pos = _pos;
    _pos = event->position() + extra_vector;
}


void GGrabber::Initialize ()
{
    GoSensor* sor = sensor();
    
    if (_goui == NULL)
	_goui = GoUIFrom (_go->root());
    
    if (_goui != NULL && sor != NULL)
    {
	_goui->grab (sor);
    }
    
    drawAll ();
    GInteractor::Initialize ();
}


GBool GGrabber::careAbout (const GEvent* event) const
{
    return
	event->type() == GMOTION_EVENT ||
	    event->type() == GEXPOSE_EVENT ||
		isDoitEvent (event) ||
		    isCancelEvent (event) ||
			event->type() == GFUNCTION_KEY_EVENT &&
			    (event->functionKey() == GKEY_LEFT ||
			     event->functionKey() == GKEY_RIGHT ||
			     event->functionKey() == GKEY_UP ||
			     event->functionKey() == GKEY_DOWN);
}
    

GBool GGrabber::handle (const GEvent* event)
{
    if (state() != GITOR_RUNNING)
    {
	GWarning ("GGrabber::handle(): my state is not GITOR_RUNNING");
	return GFALSE;
    }

    if (isDoitEvent (event))
    {
	doit();
	return GTRUE;
    }

    if (isCancelEvent (event))
    {
	cancel ();
	return GTRUE;
    }

    if (event->type() == GFUNCTION_KEY_EVENT)
    {
	GVector delta (0, 0);
	if (event->functionKey() == GKEY_LEFT)
	    delta.x -= 1;
	else if (event->functionKey() == GKEY_RIGHT)
	    delta.x += 1;
	else if (event->functionKey() == GKEY_UP)
	    delta.y -= 1;
	else if (event->functionKey() == GKEY_DOWN)
	    delta.y += 1;
	
	if (event->modifiers() & GCONTROL)
	    delta *= 0.1;
	else if (event->modifiers() & GALTERNATE)
	    delta *= 10;

	if (delta.x != 0 || delta.y != 0)
	{
	    extra_vector += delta;
	    goto handle_motion;
	}
    }	

    if (event->type() == GMOTION_EVENT)
    {
      handle_motion:
	clear ();
	compute (event);
	draw ();
	return GTRUE;
    }

    if (event->type() == GEXPOSE_EVENT)
    {
	_go->interf ()->setClip (event->region());
	draw_from_expose = GTRUE;
	drawAll ();
	draw_from_expose = GFALSE;
	_go->interf ()->resetClip ();
	return GTRUE;
    }

    return GFALSE;
}


void GGrabber::doit ()
{
    GInteractor::doit ();
    clearAll ();
}


void GGrabber::cancel ()
{
    GInteractor::cancel ();
    clearAll ();
}


GoSensor* GGrabber::sensor () const
{
    if (_sensor == NULL)
    {
	for (const char* actionName = Pgm()->name();
	     *actionName && *actionName != '.';
	     actionName++);
	actionName++;

	for (Go* ch = _go->Child(); ch != NULL; ch = ch->Next())
	    if (ch->getMetaclass()->isSubclass(GoSensor::metaclass()))
	    {
		GoSensor* sor = (GoSensor*) ch;
		if (sor->interactor() == (GInteractor*)this ||
		    sor->interactor() == NULL &&
		    sor->program() != NULL &&
		    !strcmp(sor->program(), actionName))
		{
		    ((GGrabber*) this)->_sensor = sor;
		    break;
		}
	    }
    }
    
    return _sensor;
}

    
