#ifdef REV_INFO
static char rcsid[] = "$Id: Interactor.cc,v 1.1 1992/12/10 14:46:22 jojic Exp $";
#endif
/*****************************************************************************
MODULE:  Go Kernel (User Interface)
FILE:    Interactor.cc
AUTHOR:  Oliver Jojic
DATE:    June 92

DESCRIPTION:
  See also the Interactor.h header.

COPYRIGHT:
  Copyright (c) 1990, 1991, 1992 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.

*****************************************************************************/

#include <Go/ItorPgm.h>
#include <Go/mGo.h>
#include <Go/Go.h>
#include <Go/Event.h>
#include <Go/Tools/Error.h>
#include <Go/Tools/StringAssoc.h>
#include <Go/Tools/Alloc.h>
#include <Go/Tools/Bind.h>
#include <Go/Tools/Cursor.h>
#include <Go/GoInterf.h>
#include <stdio.h>


const char* GInteractorPgm::type()
{
    return "GInteractorPgm";
}


GInteractor* GCreateInteractor (const char* name, const GEvent* event, Go* go)
{
    if (name == NULL || go == NULL)
	return NULL;

    char ia_name[256];
    ia_name[0] = 0;
    GInteractor* itor = NULL;

    GoMetaclass* mclass = go->getMetaclass();
    do
    {
	sprintf(ia_name, "%s.%s", mclass->classname(), name);
	void* ptr;
	const char* type;
	if (GBinded (ia_name, ptr, type) &&
	    type != NULL &&
	    !strcmp(type, GInteractorPgm::type()))
	{
	    GInteractorPgm* ipgm = (GInteractorPgm*) ptr;
	    itor = ipgm->start(event, go);
	}
    }
    while (itor == NULL &&
	   (mclass = mclass->superclass()) != NULL);
	
    if (itor == NULL)
	GWarning("GCreateInteractor(): no such interactor: %s.%s",
		 go->getMetaclass()->classname(), name);

    return itor;
}


GInteractor::~GInteractor ()
{
}


GEventMask GInteractor::inputMask () const
{
    return _pgm->inputMask();
}


GBool GInteractor::careAbout (const GEvent* event) const
{
    return inputMask() & event->type();
}


void GInteractor::doit ()
{
    _state = GITOR_DONE;
    if (_go)
	_go->interf()->popCursor();
}


void GInteractor::cancel ()
{
    _state = GITOR_CANCELLED;
    if (_go)
	_go->interf()->popCursor();
}


GInteractor::GInteractor (const GInteractorPgm* pgm, Go* go)
{
    _pgm = pgm;
    _go = go;
    _state = GITOR_RUNNING;
    return_value = NULL;
}


void GInteractor::Initialize ()
{
    if (_go)
	_go->interf()->pushCursor(pgm()->cursorIdent());
}


GInteractorPgm::GInteractorPgm (const char* name)
{
    if (name)
    {
	_name = (const char*)GMALLOC(strlen(name)+1);
	strcpy((char*)_name, name);
    }
    else _name = NULL;
    
    if (GBinded(_name))
    {
	GWarning("GInteractorPgm::GInteractorPgm(): already binded interactor %s",
		 _name);
	GRemoveBinding(_name);
    }
    GBind(_name, this, GInteractorPgm::type());

    _cursorIdent = GCURSOR_DEFAULT;
}


GInteractorPgm::~GInteractorPgm ()
{
    GRemoveBinding(_name);
    GFREE(_name);
}


GInteractorPgm* GInteractorPgmFromName (const char* name)
{
    void* ptr;
    const char* type;

    if (!GBinded(name, ptr, type) ||
	type == NULL ||
	strcmp(type, GInteractorPgm::type()))
	ptr = NULL;

    return (GInteractorPgm*) ptr;
}
