#ifdef REV_INFO
static char rcsid[] = "$Id: ClipSensor.cc,v 1.4 1993/01/04 14:09:49 jacomet Exp $";
#endif
/*****************************************************************************
MODULE:  Graphic Objects
FILE:    ClipSensor.cc
AUTHOR:  Philippe Smadja
DATE:    Dec 92

DESCRIPTION:
  See also the ClipSensor.h header.
  
COPYRIGHT:
  Copyright (c) 1992 Bull S.A.
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull S.A. may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull S.A..
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull S.A. under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull S.A..

  Bull S.A. assumes no responsibility for the use or inability to use
  any of its software. The software is provided "AS IS" without warranty
  of any kind, and Bull S.A. expressly disclaims all implied warranties,
  including but not limited to the implied warranties of merchantability
  and fitness for a particular purpose.

  In no event shall Bull S.A. be liable for any special, incidental,
  indirect or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#include <Go/ClipSensor.h>
#include <Go/GoStream.h>



/*
 *-------------------------------------------------------
 * pre- and post- conditions.
 *-------------------------------------------------------
 */



/*
 *-------------------------------------------------------
 * GoClipSensor public methods
 *-------------------------------------------------------
 */



/*
 *-------------------------------------------------------
 * GoClipSensor private methods
 *-------------------------------------------------------
 */



/*
 *-------------------------------------------------------
 * GoClipSensorMetaclass methods
 *-------------------------------------------------------
 */

/*
 This global variable is used to store the unique instance
 of the GoClipSensorMetaclass
 */
static GoClipSensorMetaclass* ClipSensor_metaclass = NULL;


/*
 * The GoClipSensorMetaclass constructor.
 */
GoClipSensorMetaclass::GoClipSensorMetaclass ()
{
    /* initialization of metaclass private data: */
}


/*
 * The GoClipSensorMetaclass destructor.
 */
GoClipSensorMetaclass::~GoClipSensorMetaclass ()
{
    /* If we are deleting the unique GoClipSensorMetaclass instance
       (and not some subclass), we must remove "this" from the
       set of subclasses in the superclass, and set NULL to the
       global pointer "ClipSensor_metaclass".
       Quid about GoClipSensor instances ? should we delete them ?
       */
    if (this == ClipSensor_metaclass)
    {
        superclass()->RemoveSubclass(this);
	ClipSensor_metaclass = NULL;
    }
}


/*
 * The virtual method "m_classname" returns the name (string) of
 * this class, i.e. "GoClipSensor".
 */
const char* GoClipSensorMetaclass::m_classname () const
{
    return "GoClipSensor";
}


/*
 * The virtual method "m_superclass" returns the unique instance of
 * the superclass of this GoClipSensorMetaclass class.
 */
GoMetaclass* GoClipSensorMetaclass::m_superclass () const
{
    return GoSensor::metaclass();
}


GoClipSensor* GoClipSensor::create
	(const char* program, Go* parent, const GEvent* startEvent,
	 void (*postProc)(void* cl, GoSensor*), void* closure) 
{
    GoClipSensor* fsor = GoClipSensor::create ();
    if (fsor == NULL)
	return NULL;
    
    fsor->program (program);
    fsor->start_event = startEvent;
    fsor->_postcond = postProc;
    fsor->_closure = closure;
    fsor->reparent (parent); //should be last, because it may start an interactor
    return fsor;
}


static void create_Clip_sensor_action (const char* program, Go* parent, const GEvent* startEvent)
{
    if (program != NULL && parent != NULL)
	(void) GoClipSensor::create (program, parent, startEvent, NULL, NULL);
}
	    

/*
 * The action table of GoClipSensor objects.
 * Functions inside this table will be accessible through
 * their name (i.e. a string). This is useClip to implement some
 * interpreted languages dealing with graphic objects.
 */
static GActionRec ClipSensor_actions[] =
{
    {"create(program, parent, event)",
	 (GActionProc)create_Clip_sensor_action},
    {NULL, NULL} //this must be the last line
};


/*
 * This method returns the action table of GoClipSensor objects.
 */
const GActionRec* GoClipSensorMetaclass::m_actions () const
{
    return ClipSensor_actions;
}


/*
 * This method creates and returns a new GoClipSensor instance.
 * This allows the creation of a new instance of an object, having only
 * the name of the object class.
 */
Go* GoClipSensorMetaclass::m_create () const
{
    return new GoClipSensor;
}



/*
 *-------------------------------------------------------
 * GoClipSensor methods (derived from Go)
 *-------------------------------------------------------
 */


/*
 * This static method returns the unique GoClipSensorMetaclass instance.
 * If this instance does not already exist, it is created. In such a case
 * the static global variable "ClipSensor_metaclass" is set up and this
 * metaclass instance is added to the set of subclass instances of the
 * superclass.
 */
GoClipSensorMetaclass* GoClipSensor::metaclass ()
{
    if (ClipSensor_metaclass == NULL)
    {
	ClipSensor_metaclass = new GoClipSensorMetaclass;
        ClipSensor_metaclass->superclass()->AddSubclass(ClipSensor_metaclass);

	GoSensor::bindActions (ClipSensor_metaclass);
      GoStream::bind(GoClipSensor::metaclass()->classname(),
		     (int (*) (const void*, GoStream*)) GoClipSensor::save,
		     (int (*) (void**, GoStream*)) GoClipSensor::restore);
    }

    return ClipSensor_metaclass;
}


/*
 * This is the GoClipSensor constructor.
 */
GoClipSensor::GoClipSensor ()
{
}


/*
 * This is the GoClipSensor destructor.
 */
GoClipSensor::~GoClipSensor ()
{
}

static GBool _contains_recur(Go* go, const GPoint& p, GNumber maxd)
{
    if (go->contains(p, maxd))
	return GTRUE;
    
    // go on the hierarchy;
    Go* _child = go->child();
    for (Go* _next = _child; _next; _next = _next->next())
	if (_contains_recur(_next, p, maxd))
	    return GTRUE;

    return GFALSE;
}

/*
 * This virtual method returns GTRUE if "p" is inside one of a child()
 * of its parent.
 */
GBool GoClipSensor::m_contains (const GPoint& p, GNumber maxd) const
{
    if (!parent())
	return GFALSE;

    GRegion clip = parent()->clip();
    GBox box =  clip.bigbox();
    box.xmin -= maxd;
    box.ymin -= maxd;
    box.xmax += maxd;
    box.ymax += maxd;
    if (!box.contains(p))
	return GFALSE;

    return _contains_recur(parent(), p, maxd);
}


/*
 * This private virtual method must copy its own class private data
 * from "src" to "this".
 * This method is used by the "dup(go)" method. The dup method returns a
 * copy of the "go" subtree.
 *
 * If dup was called on "this", then this next brother must not be dupped,
 * "donext" is then FALSE.
 * If dup was called on some ancestor of "this", then this next brother
 * must also be dupped. The flag "donext" is TRUE. In that case, IF THIS
 * CLASS stores a "_next" field, it must call the recursive Dup on the
 * next brother.
 */
void GoClipSensor::m_dup
    (const Go* src, Go* parcopy, Go* prevcopy, GBool donext, GBool dochildren)
{
    GoSensor::m_dup(src, parcopy, prevcopy, donext, dochildren);
}
	

/*
 * This private virtual method returns the metaclass of "this".
 */
GoMetaclass* GoClipSensor::m_metaclass () const
{
    return GoClipSensor::metaclass();
}


/*
 * This static method is responsible to save "this" go on "stream".
 * It must write its private data and then call the save of its superclass.
 */
int GoClipSensor::save (const GoClipSensor* go, GoStream* stream)
{
    return GoSensor::save(go, stream);
}


/*
 * This static method is responsible to restore "this" go from a stream.
 * For that purpose, it reads its private data and then calls the
 * readParams of its superclass.
 */
int GoClipSensor::readParams (GoClipSensor* go, GoStream* stream)
{
    return GoSensor::readParams(go, stream);
}

/**
FUNCTION
    int restore();
DESCRIPTION
    
**/
int GoClipSensor::restore(GoClipSensor** sensor, GoStream* gostream) {
    if (*sensor = GoClipSensor::create())
	return GoClipSensor::readParams(*sensor, gostream);
    return GFALSE;
}

