#ifdef REV_INFO
static char rcsid[] = "$Id: FullSensor.cc,v 1.3 1993/01/04 14:09:51 jacomet Exp $";
#endif
/*****************************************************************************
MODULE:  Graphic Objects
FILE:    FullSensor.cc
AUTHOR:  jojic
DATE:    Jul 28 1992

DESCRIPTION:
  See also the FullSensor.h header.
  
COPYRIGHT:
  Copyright (c) 1992 Bull S.A.
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull S.A. may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull S.A..
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull S.A. under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull S.A..

  Bull S.A. assumes no responsibility for the use or inability to use
  any of its software. The software is provided "AS IS" without warranty
  of any kind, and Bull S.A. expressly disclaims all implied warranties,
  including but not limited to the implied warranties of merchantability
  and fitness for a particular purpose.

  In no event shall Bull S.A. be liable for any special, incidental,
  indirect or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#include <Go/FullSensor.h>
#include <Go/GoStream.h>



/*
 *-------------------------------------------------------
 * pre- and post- conditions.
 *-------------------------------------------------------
 */



/*
 *-------------------------------------------------------
 * GoFullSensor public methods
 *-------------------------------------------------------
 */



/*
 *-------------------------------------------------------
 * GoFullSensor private methods
 *-------------------------------------------------------
 */



/*
 *-------------------------------------------------------
 * GoFullSensorMetaclass methods
 *-------------------------------------------------------
 */

/*
 This global variable is used to store the unique instance
 of the GoFullSensorMetaclass
 */
static GoFullSensorMetaclass* FullSensor_metaclass = NULL;


/*
 * The GoFullSensorMetaclass constructor.
 */
GoFullSensorMetaclass::GoFullSensorMetaclass ()
{
    /* initialization of metaclass private data: */
}


/*
 * The GoFullSensorMetaclass destructor.
 */
GoFullSensorMetaclass::~GoFullSensorMetaclass ()
{
    /* If we are deleting the unique GoFullSensorMetaclass instance
       (and not some subclass), we must remove "this" from the
       set of subclasses in the superclass, and set NULL to the
       global pointer "FullSensor_metaclass".
       Quid about GoFullSensor instances ? should we delete them ?
       */
    if (this == FullSensor_metaclass)
    {
        superclass()->RemoveSubclass(this);
	FullSensor_metaclass = NULL;
    }
}


/*
 * The virtual method "m_classname" returns the name (string) of
 * this class, i.e. "GoFullSensor".
 */
const char* GoFullSensorMetaclass::m_classname () const
{
    return "GoFullSensor";
}


/*
 * The virtual method "m_superclass" returns the unique instance of
 * the superclass of this GoFullSensorMetaclass class.
 */
GoMetaclass* GoFullSensorMetaclass::m_superclass () const
{
    return GoSensor::metaclass();
}


GoFullSensor* GoFullSensor::create
	(const char* program, Go* parent, const GEvent* startEvent,
	 void (*postProc)(void* cl, GoSensor*), void* closure) 
{
    GoFullSensor* fsor = GoFullSensor::create ();
    if (fsor == NULL)
	return NULL;
    
    fsor->program (program);
    fsor->start_event = startEvent;
    fsor->_postcond = postProc;
    fsor->_closure = closure;
    fsor->reparent (parent); //should be last, because it may start an interactor
    return fsor;
}


static void create_full_sensor_action (const char* program, Go* parent, const GEvent* startEvent)
{
    if (program != NULL && parent != NULL)
	(void) GoFullSensor::create (program, parent, startEvent, NULL, NULL);
}
	    

/*
 * The action table of GoFullSensor objects.
 * Functions inside this table will be accessible through
 * their name (i.e. a string). This is usefull to implement some
 * interpreted languages dealing with graphic objects.
 */
static GActionRec FullSensor_actions[] =
{
    {"create(program, parent, event)",
	 (GActionProc)create_full_sensor_action},
    {NULL, NULL} //this must be the last line
};


/*
 * This method returns the action table of GoFullSensor objects.
 */
const GActionRec* GoFullSensorMetaclass::m_actions () const
{
    return FullSensor_actions;
}


/*
 * This method creates and returns a new GoFullSensor instance.
 * This allows the creation of a new instance of an object, having only
 * the name of the object class.
 */
Go* GoFullSensorMetaclass::m_create () const
{
    return new GoFullSensor;
}



/*
 *-------------------------------------------------------
 * GoFullSensor methods (derived from Go)
 *-------------------------------------------------------
 */


/*
 * This static method returns the unique GoFullSensorMetaclass instance.
 * If this instance does not already exist, it is created. In such a case
 * the static global variable "FullSensor_metaclass" is set up and this
 * metaclass instance is added to the set of subclass instances of the
 * superclass.
 */
GoFullSensorMetaclass* GoFullSensor::metaclass ()
{
    if (FullSensor_metaclass == NULL)
    {
	FullSensor_metaclass = new GoFullSensorMetaclass;
        FullSensor_metaclass->superclass()->AddSubclass(FullSensor_metaclass);

      GoStream::bind(GoFullSensor::metaclass()->classname(),
		     (int (*) (const void*, GoStream*)) GoFullSensor::save,
		     (int (*) (void**, GoStream*)) GoFullSensor::restore);
	GoSensor::bindActions (FullSensor_metaclass);
    }

    return FullSensor_metaclass;
}


/*
 * This is the GoFullSensor constructor.
 */
GoFullSensor::GoFullSensor ()
{
}


/*
 * This is the GoFullSensor destructor.
 */
GoFullSensor::~GoFullSensor ()
{
}


/*
 * This virtual method returns GTRUE if "p" is inside the drawing of this
 * object. With an accepted distance "maxd".
 */
GBool GoFullSensor::m_contains (const GPoint&, GNumber) const
{
    return GTRUE;
}


/*
 * This private virtual method must copy its own class private data
 * from "src" to "this".
 * This method is used by the "dup(go)" method. The dup method returns a
 * copy of the "go" subtree.
 *
 * If dup was called on "this", then this next brother must not be dupped,
 * "donext" is then FALSE.
 * If dup was called on some ancestor of "this", then this next brother
 * must also be dupped. The flag "donext" is TRUE. In that case, IF THIS
 * CLASS stores a "_next" field, it must call the recursive Dup on the
 * next brother.
 */
void GoFullSensor::m_dup
    (const Go* src, Go* parcopy, Go* prevcopy, GBool donext, GBool dochildren)
{
    GoSensor::m_dup(src, parcopy, prevcopy, donext, dochildren);
}
	

/*
 * This private virtual method returns the metaclass of "this".
 */
GoMetaclass* GoFullSensor::m_metaclass () const
{
    return GoFullSensor::metaclass();
}


/*
 * This static method is responsible to save "this" go on "stream".
 * It must write its private data and then call the save of its superclass.
 */
int GoFullSensor::save (const GoFullSensor* go, GoStream* stream)
{
    return GoSensor::save(go, stream);
}


/*
 * This static method is responsible to restore "this" go from a stream.
 * For that purpose, it reads its private data and then calls the
 * readParams of its superclass.
 */
int GoFullSensor::readParams (GoFullSensor* go, GoStream* stream)
{
    return GoSensor::readParams(go, stream);
}

/**
FUNCTION
    int restore();
DESCRIPTION
    
**/
int GoFullSensor::restore(GoFullSensor** sensor, GoStream* gostream) {
    if (*sensor = GoFullSensor::create())
	return GoFullSensor::readParams(*sensor, gostream);
    return GFALSE;
}

