#ifdef REV_INFO
static char rcsid[] = "$Id: StdSensor.cc,v 1.4 1993/01/04 14:09:54 jacomet Exp $";
#endif
/*****************************************************************************
MODULE:  Graphic Objects
FILE:    StdSensor.cc
AUTHOR:  jojic
DATE:    Jul 2 1992

DESCRIPTION:
  See also the StdSensor.h header.
  
COPYRIGHT:
  Copyright (c) 1992 Bull S.A.
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull S.A. may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull S.A..
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull S.A. under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull S.A..

  Bull S.A. assumes no responsibility for the use or inability to use
  any of its software. The software is provided "AS IS" without warranty
  of any kind, and Bull S.A. expressly disclaims all implied warranties,
  including but not limited to the implied warranties of merchantability
  and fitness for a particular purpose.

  In no event shall Bull S.A. be liable for any special, incidental,
  indirect or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#include <Go/StdSensor.h>
#include <Go/Tools/Medium.h>
#include <Go/GoStream.h>



/*
 *-------------------------------------------------------
 * GoStdSensorMetaclass methods
 *-------------------------------------------------------
 */

/*
 This global variable is used to store the unique instance
 of the GoStdSensorMetaclass
 */
static GoStdSensorMetaclass* StdSensor_metaclass = NULL;


/*
 * The GoStdSensorMetaclass constructor.
 */
GoStdSensorMetaclass::GoStdSensorMetaclass ()
{
    /* initialization of metaclass private data: */
}


/*
 * The GoStdSensorMetaclass destructor.
 */
GoStdSensorMetaclass::~GoStdSensorMetaclass ()
{
    /* If we are deleting the unique GoStdSensorMetaclass instance
       (and not some subclass), we must remove "this" from the
       set of subclasses in the superclass, and set NULL to the
       global pointer "StdSensor_metaclass".
       Quid about GoStdSensor instances ? should we delete them ?
       */
    if (this == StdSensor_metaclass)
    {
        superclass()->RemoveSubclass(this);
	StdSensor_metaclass = NULL;
    }
}


/*
 * The virtual method "m_classname" returns the name (string) of
 * this class, i.e. "GoStdSensor".
 */
const char* GoStdSensorMetaclass::m_classname () const
{
    return "GoStdSensor";
}


/*
 * The virtual method "m_superclass" returns the unique instance of
 * the superclass of this GoStdSensorMetaclass class.
 */
GoMetaclass* GoStdSensorMetaclass::m_superclass () const
{
    return GoSensor::metaclass();
}


/*
 * The action table of GoStdSensor objects.
 * Functions inside this table will be accessible through
 * their name (i.e. a string). This is usefull to implement some
 * interpreted languages dealing with graphic objects.
 */
static GActionRec StdSensor_actions[] = {
    {NULL, NULL} //this must be the last line;
};


/*
 * This method returns the action table of GoStdSensor objects.
 */
const GActionRec* GoStdSensorMetaclass::m_actions () const
{
    return StdSensor_actions;
}


/*
 * This method creates and returns a new GoStdSensor instance.
 * This allows the creation of a new instance of an object, having only
 * the name of the object class.
 */
Go* GoStdSensorMetaclass::m_create () const
{
    return new GoStdSensor;
}



/*
 *-------------------------------------------------------
 * GoStdSensor methods (derived from Go)
 *-------------------------------------------------------
 */


/*
 * This static method returns the unique GoStdSensorMetaclass instance.
 * If this instance does not already exist, it is created. In such a case
 * the static global variable "StdSensor_metaclass" is set up and this
 * metaclass instance is added to the set of subclass instances of the
 * superclass.
 */
GoStdSensorMetaclass* GoStdSensor::metaclass ()
{
    if (StdSensor_metaclass == NULL)
    {
	StdSensor_metaclass = new GoStdSensorMetaclass;
        StdSensor_metaclass->superclass()->AddSubclass(StdSensor_metaclass);
      GoStream::bind(GoStdSensor::metaclass()->classname(),
		     (int (*) (const void*, GoStream*)) GoStdSensor::save,
		     (int (*) (void**, GoStream*)) GoStdSensor::restore);
    }

    return StdSensor_metaclass;
}


/*
 * This is the GoStdSensor constructor.
 */
GoStdSensor::GoStdSensor ()
{
    _child = NULL;

    extern GBool
	gosensor_relcs_precond(void*, const Go*,
			       GCoordSys*, const GCoordSys*, const GCoordSys*),
	gosensor_add_child_precond(void*, const Go*, const Go*, const Go*(*));

    /* a GoStdSensor can have children (versus its GoSensor superclass) */
    addChildPrecond(gosensor_add_child_precond, NULL, GREMOVE);

    /* a GoStdSensor can change its parent relative coordinate system
       (versus its GoSensor superclass) */
    relcsPrecond(gosensor_relcs_precond, NULL, GREMOVE);
}


/*
 * This is the GoStdSensor destructor.
 */
GoStdSensor::~GoStdSensor ()
{
}


/*
 * This private virtual method must copy its own class private data
 * from "src" to "this".
 * This method is used by the "dup(go)" method. The dup method returns a
 * copy of the "go" subtree.
 *
 * If dup was called on "this", then this next brother must not be dupped,
 * "donext" is then FALSE.
 * If dup was called on some ancestor of "this", then this next brother
 * must also be dupped. The flag "donext" is TRUE. In that case, IF THIS
 * CLASS stores a "_next" field, it must call the recursive Dup on the
 * next brother.
 */
void GoStdSensor::m_dup
    (const Go* src, Go* parcopy, Go* prevcopy, GBool donext, GBool dochildren)
{
    GoSensor::m_dup(src, parcopy, prevcopy, donext, dochildren);

    const GoStdSensor* orig = (GoStdSensor*)src;
    _abscs = orig->_abscs;

    if (dochildren)
	for (Go* orig_ch = orig->_child; orig_ch; orig_ch = orig_ch->Next())
	    if (_child = orig_ch->Dup(this, NULL, GTRUE, GTRUE))
		break;
}
	

/*
 * This private virtual method returns the metaclass of "this".
 */
GoMetaclass* GoStdSensor::m_metaclass () const
{
    return GoStdSensor::metaclass();
}


/*
 * This is a private virtual "data storage" method.
 * It returns the first child of "this".
 */
Go* GoStdSensor::d_get_first_child () const
{
    return _child;
}


/*
 * This is a private virtual "data storage" method.
 * It removes the child "go" of the data structure used to
 * store children.
 */
void GoStdSensor::d_remove_child (const Go* go, int)
{
    if (go == _child)
	_child = go->Next();
}


/*
 * This is a private virtual "data storage" method.
 * It adds a new child to the data structure used to store children.
 */
void GoStdSensor::d_add_child (const Go* go, int child_index)
{
    if (child_index == 0)
	_child = (Go*)go;
}


/*
 * This is a private virtual "data storage" method.
 * It returns TRUE if this class stores an absolute coordinate system.
 */
GBool GoStdSensor::d_store_abscs () const
{
    return GTRUE;
}


/*
 * This is a private virtual "data storage" method.
 * It returns TRUE if this class stores a parent relative
 * coordinate system.
 */
GBool GoStdSensor::d_store_relcs () const
{
    return GFALSE;
}


/*
 * This is a private virtual "data storage" method.
 * It returns the absolute coordinate system.
 * The Go kernel will never call this method if "d_store_abscs()"
 * returns FALSE (you do not have to implement it then).
 */
const GCoordSys& GoStdSensor::d_get_abscs
    (const GCoordSys* /*parent_abscs*/) const
{
    return _abscs;
}


/*
 * This is a private virtual "data storage" method.
 * It must set up data used to retrieve the coordinate system.
 */
void GoStdSensor::d_set_coordsys
    (const GCoordSys& /*relcs*/, const GCoordSys& abscs,
     const GCoordSys& /*parent_abscs*/)
{
    _abscs = abscs;
}


/*
 * This static method is responsible to save "this" go on "stream".
 * It must write its private data and then call the save of its superclass.
 */
int GoStdSensor::save (const GoStdSensor* go, GoStream* stream)
{
    extern int save_children(const Go*, GoStream*);

    return
	stream->writeString (go->_program, "program") &&
	    stream->writeCoordSys(go->Abscs(), "CoordSys") &&
		save_children(go->_child, stream) &&
		    stream->writeBool (go->_active, "active") &&
			Go::save (go, stream);
}


/*
 * This static method is responsible to restore "this" go from a stream.
 * For that purpose, it reads its private data and then calls the
 * readParams of its superclass.
 */
int GoStdSensor::readParams (GoStdSensor* go, GoStream* stream)
{
   extern int restore_children(Go*, GoStream*);

   if (!stream->readString (0, (char**) &go->_program) ||
       !stream->readCoordSys (&go->_abscs) ||
       !restore_children (go, stream))
       return GFALSE;

   /* sensors stored in old GoStreams do not have the _active flag: */
   ulong streamPos = stream->medium()->mtell();
   if (!stream->readBool (&go->_active))
       stream->medium()->mseek (streamPos, GMSEEK_FROM_BEGINNING);
       
   return Go::readParams (go, stream);
}

/**
FUNCTION
    int restore();
DESCRIPTION
    
**/
int GoStdSensor::restore(GoStdSensor** sensor, GoStream* gostream) {
    if (*sensor = GoStdSensor::create())
	return GoStdSensor::readParams(*sensor, gostream);
    return GFALSE;
}
